#include "tree_sitter/parser.h"

#if defined(__GNUC__) || defined(__clang__)
#pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#ifdef _MSC_VER
#pragma optimize("", off)
#elif defined(__clang__)
#pragma clang optimize off
#elif defined(__GNUC__)
#pragma GCC optimize ("O0")
#endif

#define LANGUAGE_VERSION 14
#define STATE_COUNT 2021
#define LARGE_STATE_COUNT 496
#define SYMBOL_COUNT 348
#define ALIAS_COUNT 3
#define TOKEN_COUNT 155
#define EXTERNAL_TOKEN_COUNT 0
#define FIELD_COUNT 39
#define MAX_ALIAS_SEQUENCE_LENGTH 9
#define PRODUCTION_ID_COUNT 132

enum ts_symbol_identifiers {
  sym_identifier = 1,
  aux_sym_preproc_include_token1 = 2,
  aux_sym_preproc_include_token2 = 3,
  aux_sym_preproc_def_token1 = 4,
  anon_sym_LPAREN = 5,
  anon_sym_DOT_DOT_DOT = 6,
  anon_sym_COMMA = 7,
  anon_sym_RPAREN = 8,
  aux_sym_preproc_if_token1 = 9,
  anon_sym_LF = 10,
  aux_sym_preproc_if_token2 = 11,
  aux_sym_preproc_ifdef_token1 = 12,
  aux_sym_preproc_ifdef_token2 = 13,
  aux_sym_preproc_else_token1 = 14,
  aux_sym_preproc_elif_token1 = 15,
  aux_sym_preproc_elifdef_token1 = 16,
  aux_sym_preproc_elifdef_token2 = 17,
  sym_preproc_arg = 18,
  sym_preproc_directive = 19,
  anon_sym_LPAREN2 = 20,
  anon_sym_defined = 21,
  anon_sym_BANG = 22,
  anon_sym_TILDE = 23,
  anon_sym_DASH = 24,
  anon_sym_PLUS = 25,
  anon_sym_STAR = 26,
  anon_sym_SLASH = 27,
  anon_sym_PERCENT = 28,
  anon_sym_PIPE_PIPE = 29,
  anon_sym_AMP_AMP = 30,
  anon_sym_PIPE = 31,
  anon_sym_CARET = 32,
  anon_sym_AMP = 33,
  anon_sym_EQ_EQ = 34,
  anon_sym_BANG_EQ = 35,
  anon_sym_GT = 36,
  anon_sym_GT_EQ = 37,
  anon_sym_LT_EQ = 38,
  anon_sym_LT = 39,
  anon_sym_LT_LT = 40,
  anon_sym_GT_GT = 41,
  anon_sym_SEMI = 42,
  anon_sym___extension__ = 43,
  anon_sym_typedef = 44,
  anon_sym_extern = 45,
  anon_sym___attribute__ = 46,
  anon_sym_COLON_COLON = 47,
  anon_sym_LBRACK_LBRACK = 48,
  anon_sym_RBRACK_RBRACK = 49,
  anon_sym___declspec = 50,
  anon_sym___based = 51,
  anon_sym___cdecl = 52,
  anon_sym___clrcall = 53,
  anon_sym___stdcall = 54,
  anon_sym___fastcall = 55,
  anon_sym___thiscall = 56,
  anon_sym___vectorcall = 57,
  sym_ms_restrict_modifier = 58,
  sym_ms_unsigned_ptr_modifier = 59,
  sym_ms_signed_ptr_modifier = 60,
  anon_sym__unaligned = 61,
  anon_sym___unaligned = 62,
  anon_sym_LBRACE = 63,
  anon_sym_RBRACE = 64,
  anon_sym_signed = 65,
  anon_sym_unsigned = 66,
  anon_sym_long = 67,
  anon_sym_short = 68,
  anon_sym_LBRACK = 69,
  anon_sym_RBRACK = 70,
  anon_sym_EQ = 71,
  anon_sym_static = 72,
  anon_sym_auto = 73,
  anon_sym_register = 74,
  anon_sym_inline = 75,
  anon_sym___inline = 76,
  anon_sym___inline__ = 77,
  anon_sym___forceinline = 78,
  anon_sym_thread_local = 79,
  anon_sym___thread = 80,
  anon_sym_const = 81,
  anon_sym_constexpr = 82,
  anon_sym_volatile = 83,
  anon_sym_restrict = 84,
  anon_sym___restrict__ = 85,
  anon_sym__Atomic = 86,
  anon_sym__Noreturn = 87,
  anon_sym_noreturn = 88,
  sym_primitive_type = 89,
  anon_sym_enum = 90,
  anon_sym_COLON = 91,
  anon_sym_struct = 92,
  anon_sym_union = 93,
  anon_sym_if = 94,
  anon_sym_else = 95,
  anon_sym_switch = 96,
  anon_sym_case = 97,
  anon_sym_default = 98,
  anon_sym_while = 99,
  anon_sym_do = 100,
  anon_sym_for = 101,
  anon_sym_return = 102,
  anon_sym_break = 103,
  anon_sym_continue = 104,
  anon_sym_goto = 105,
  anon_sym___try = 106,
  anon_sym___except = 107,
  anon_sym___finally = 108,
  anon_sym___leave = 109,
  anon_sym_QMARK = 110,
  anon_sym_STAR_EQ = 111,
  anon_sym_SLASH_EQ = 112,
  anon_sym_PERCENT_EQ = 113,
  anon_sym_PLUS_EQ = 114,
  anon_sym_DASH_EQ = 115,
  anon_sym_LT_LT_EQ = 116,
  anon_sym_GT_GT_EQ = 117,
  anon_sym_AMP_EQ = 118,
  anon_sym_CARET_EQ = 119,
  anon_sym_PIPE_EQ = 120,
  anon_sym_DASH_DASH = 121,
  anon_sym_PLUS_PLUS = 122,
  anon_sym_sizeof = 123,
  anon_sym___alignof__ = 124,
  anon_sym___alignof = 125,
  anon_sym__alignof = 126,
  anon_sym_alignof = 127,
  anon_sym__Alignof = 128,
  anon_sym_offsetof = 129,
  anon_sym__Generic = 130,
  anon_sym_asm = 131,
  anon_sym___asm__ = 132,
  anon_sym_DOT = 133,
  anon_sym_DASH_GT = 134,
  sym_number_literal = 135,
  anon_sym_L_SQUOTE = 136,
  anon_sym_u_SQUOTE = 137,
  anon_sym_U_SQUOTE = 138,
  anon_sym_u8_SQUOTE = 139,
  anon_sym_SQUOTE = 140,
  aux_sym_char_literal_token1 = 141,
  anon_sym_L_DQUOTE = 142,
  anon_sym_u_DQUOTE = 143,
  anon_sym_U_DQUOTE = 144,
  anon_sym_u8_DQUOTE = 145,
  anon_sym_DQUOTE = 146,
  aux_sym_string_literal_token1 = 147,
  sym_escape_sequence = 148,
  sym_system_lib_string = 149,
  sym_true = 150,
  sym_false = 151,
  anon_sym_NULL = 152,
  anon_sym_nullptr = 153,
  sym_comment = 154,
  sym_translation_unit = 155,
  sym_preproc_include = 156,
  sym_preproc_def = 157,
  sym_preproc_function_def = 158,
  sym_preproc_params = 159,
  sym_preproc_call = 160,
  sym_preproc_if = 161,
  sym_preproc_ifdef = 162,
  sym_preproc_else = 163,
  sym_preproc_elif = 164,
  sym_preproc_elifdef = 165,
  sym_preproc_if_in_field_declaration_list = 166,
  sym_preproc_ifdef_in_field_declaration_list = 167,
  sym_preproc_else_in_field_declaration_list = 168,
  sym_preproc_elif_in_field_declaration_list = 169,
  sym_preproc_elifdef_in_field_declaration_list = 170,
  sym_preproc_if_in_enumerator_list = 171,
  sym_preproc_ifdef_in_enumerator_list = 172,
  sym_preproc_else_in_enumerator_list = 173,
  sym_preproc_elif_in_enumerator_list = 174,
  sym_preproc_elifdef_in_enumerator_list = 175,
  sym_preproc_if_in_enumerator_list_no_comma = 176,
  sym_preproc_ifdef_in_enumerator_list_no_comma = 177,
  sym_preproc_else_in_enumerator_list_no_comma = 178,
  sym_preproc_elif_in_enumerator_list_no_comma = 179,
  sym_preproc_elifdef_in_enumerator_list_no_comma = 180,
  sym__preproc_expression = 181,
  sym_preproc_parenthesized_expression = 182,
  sym_preproc_defined = 183,
  sym_preproc_unary_expression = 184,
  sym_preproc_call_expression = 185,
  sym_preproc_argument_list = 186,
  sym_preproc_binary_expression = 187,
  sym_function_definition = 188,
  sym__old_style_function_definition = 189,
  sym_declaration = 190,
  sym_type_definition = 191,
  sym__type_definition_type = 192,
  sym__type_definition_declarators = 193,
  sym__declaration_modifiers = 194,
  sym__declaration_specifiers = 195,
  sym_linkage_specification = 196,
  sym_attribute_specifier = 197,
  sym_attribute = 198,
  sym_attribute_declaration = 199,
  sym_ms_declspec_modifier = 200,
  sym_ms_based_modifier = 201,
  sym_ms_call_modifier = 202,
  sym_ms_unaligned_ptr_modifier = 203,
  sym_ms_pointer_modifier = 204,
  sym_declaration_list = 205,
  sym__declarator = 206,
  sym__declaration_declarator = 207,
  sym__field_declarator = 208,
  sym__type_declarator = 209,
  sym__abstract_declarator = 210,
  sym_parenthesized_declarator = 211,
  sym_parenthesized_field_declarator = 212,
  sym_parenthesized_type_declarator = 213,
  sym_abstract_parenthesized_declarator = 214,
  sym_attributed_declarator = 215,
  sym_attributed_field_declarator = 216,
  sym_attributed_type_declarator = 217,
  sym_pointer_declarator = 218,
  sym_pointer_field_declarator = 219,
  sym_pointer_type_declarator = 220,
  sym_abstract_pointer_declarator = 221,
  sym_function_declarator = 222,
  sym__function_declaration_declarator = 223,
  sym_function_field_declarator = 224,
  sym_function_type_declarator = 225,
  sym_abstract_function_declarator = 226,
  sym__old_style_function_declarator = 227,
  sym_array_declarator = 228,
  sym_array_field_declarator = 229,
  sym_array_type_declarator = 230,
  sym_abstract_array_declarator = 231,
  sym_init_declarator = 232,
  sym_compound_statement = 233,
  sym_storage_class_specifier = 234,
  sym_type_qualifier = 235,
  sym__type_specifier = 236,
  sym_sized_type_specifier = 237,
  sym_enum_specifier = 238,
  sym_enumerator_list = 239,
  sym_struct_specifier = 240,
  sym_union_specifier = 241,
  sym_field_declaration_list = 242,
  sym__field_declaration_list_item = 243,
  sym_field_declaration = 244,
  sym__field_declaration_declarator = 245,
  sym_bitfield_clause = 246,
  sym_enumerator = 247,
  sym_variadic_parameter = 248,
  sym_parameter_list = 249,
  sym__old_style_parameter_list = 250,
  sym_parameter_declaration = 251,
  sym_attributed_statement = 252,
  sym_labeled_statement = 253,
  sym__top_level_expression_statement = 254,
  sym_expression_statement = 255,
  sym_if_statement = 256,
  sym_else_clause = 257,
  sym_switch_statement = 258,
  sym_case_statement = 259,
  sym_while_statement = 260,
  sym_do_statement = 261,
  sym_for_statement = 262,
  sym__for_statement_body = 263,
  sym_return_statement = 264,
  sym_break_statement = 265,
  sym_continue_statement = 266,
  sym_goto_statement = 267,
  sym_seh_try_statement = 268,
  sym_seh_except_clause = 269,
  sym_seh_finally_clause = 270,
  sym_seh_leave_statement = 271,
  sym__expression = 272,
  sym__expression_not_binary = 273,
  sym__string = 274,
  sym_comma_expression = 275,
  sym_conditional_expression = 276,
  sym_assignment_expression = 277,
  sym_pointer_expression = 278,
  sym_unary_expression = 279,
  sym_binary_expression = 280,
  sym_update_expression = 281,
  sym_cast_expression = 282,
  sym_type_descriptor = 283,
  sym_sizeof_expression = 284,
  sym_alignof_expression = 285,
  sym_offsetof_expression = 286,
  sym_generic_expression = 287,
  sym_subscript_expression = 288,
  sym_call_expression = 289,
  sym_gnu_asm_expression = 290,
  sym_gnu_asm_qualifier = 291,
  sym_gnu_asm_output_operand_list = 292,
  sym_gnu_asm_output_operand = 293,
  sym_gnu_asm_input_operand_list = 294,
  sym_gnu_asm_input_operand = 295,
  sym_gnu_asm_clobber_list = 296,
  sym_gnu_asm_goto_list = 297,
  sym_argument_list = 298,
  sym_field_expression = 299,
  sym_compound_literal_expression = 300,
  sym_parenthesized_expression = 301,
  sym_initializer_list = 302,
  sym_initializer_pair = 303,
  sym_subscript_designator = 304,
  sym_subscript_range_designator = 305,
  sym_field_designator = 306,
  sym_char_literal = 307,
  sym_concatenated_string = 308,
  sym_string_literal = 309,
  sym_null = 310,
  sym__empty_declaration = 311,
  sym_macro_type_specifier = 312,
  aux_sym_translation_unit_repeat1 = 313,
  aux_sym_preproc_params_repeat1 = 314,
  aux_sym_preproc_if_repeat1 = 315,
  aux_sym_preproc_if_in_field_declaration_list_repeat1 = 316,
  aux_sym_preproc_if_in_enumerator_list_repeat1 = 317,
  aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1 = 318,
  aux_sym_preproc_argument_list_repeat1 = 319,
  aux_sym__old_style_function_definition_repeat1 = 320,
  aux_sym_declaration_repeat1 = 321,
  aux_sym_type_definition_repeat1 = 322,
  aux_sym__type_definition_type_repeat1 = 323,
  aux_sym__type_definition_declarators_repeat1 = 324,
  aux_sym__declaration_specifiers_repeat1 = 325,
  aux_sym_attribute_declaration_repeat1 = 326,
  aux_sym_attributed_declarator_repeat1 = 327,
  aux_sym_pointer_declarator_repeat1 = 328,
  aux_sym_function_declarator_repeat1 = 329,
  aux_sym_sized_type_specifier_repeat1 = 330,
  aux_sym_enumerator_list_repeat1 = 331,
  aux_sym__field_declaration_declarator_repeat1 = 332,
  aux_sym_parameter_list_repeat1 = 333,
  aux_sym__old_style_parameter_list_repeat1 = 334,
  aux_sym_case_statement_repeat1 = 335,
  aux_sym_generic_expression_repeat1 = 336,
  aux_sym_gnu_asm_expression_repeat1 = 337,
  aux_sym_gnu_asm_output_operand_list_repeat1 = 338,
  aux_sym_gnu_asm_input_operand_list_repeat1 = 339,
  aux_sym_gnu_asm_clobber_list_repeat1 = 340,
  aux_sym_gnu_asm_goto_list_repeat1 = 341,
  aux_sym_argument_list_repeat1 = 342,
  aux_sym_initializer_list_repeat1 = 343,
  aux_sym_initializer_pair_repeat1 = 344,
  aux_sym_char_literal_repeat1 = 345,
  aux_sym_concatenated_string_repeat1 = 346,
  aux_sym_string_literal_repeat1 = 347,
  alias_sym_field_identifier = 348,
  alias_sym_statement_identifier = 349,
  alias_sym_type_identifier = 350,
};

static const char * const ts_symbol_names[] = {
  [ts_builtin_sym_end] = "end",
  [sym_identifier] = "identifier",
  [aux_sym_preproc_include_token1] = "#include",
  [aux_sym_preproc_include_token2] = "preproc_include_token2",
  [aux_sym_preproc_def_token1] = "#define",
  [anon_sym_LPAREN] = "(",
  [anon_sym_DOT_DOT_DOT] = "...",
  [anon_sym_COMMA] = ",",
  [anon_sym_RPAREN] = ")",
  [aux_sym_preproc_if_token1] = "#if",
  [anon_sym_LF] = "\n",
  [aux_sym_preproc_if_token2] = "#endif",
  [aux_sym_preproc_ifdef_token1] = "#ifdef",
  [aux_sym_preproc_ifdef_token2] = "#ifndef",
  [aux_sym_preproc_else_token1] = "#else",
  [aux_sym_preproc_elif_token1] = "#elif",
  [aux_sym_preproc_elifdef_token1] = "#elifdef",
  [aux_sym_preproc_elifdef_token2] = "#elifndef",
  [sym_preproc_arg] = "preproc_arg",
  [sym_preproc_directive] = "preproc_directive",
  [anon_sym_LPAREN2] = "(",
  [anon_sym_defined] = "defined",
  [anon_sym_BANG] = "!",
  [anon_sym_TILDE] = "~",
  [anon_sym_DASH] = "-",
  [anon_sym_PLUS] = "+",
  [anon_sym_STAR] = "*",
  [anon_sym_SLASH] = "/",
  [anon_sym_PERCENT] = "%",
  [anon_sym_PIPE_PIPE] = "||",
  [anon_sym_AMP_AMP] = "&&",
  [anon_sym_PIPE] = "|",
  [anon_sym_CARET] = "^",
  [anon_sym_AMP] = "&",
  [anon_sym_EQ_EQ] = "==",
  [anon_sym_BANG_EQ] = "!=",
  [anon_sym_GT] = ">",
  [anon_sym_GT_EQ] = ">=",
  [anon_sym_LT_EQ] = "<=",
  [anon_sym_LT] = "<",
  [anon_sym_LT_LT] = "<<",
  [anon_sym_GT_GT] = ">>",
  [anon_sym_SEMI] = ";",
  [anon_sym___extension__] = "__extension__",
  [anon_sym_typedef] = "typedef",
  [anon_sym_extern] = "extern",
  [anon_sym___attribute__] = "__attribute__",
  [anon_sym_COLON_COLON] = "::",
  [anon_sym_LBRACK_LBRACK] = "[[",
  [anon_sym_RBRACK_RBRACK] = "]]",
  [anon_sym___declspec] = "__declspec",
  [anon_sym___based] = "__based",
  [anon_sym___cdecl] = "__cdecl",
  [anon_sym___clrcall] = "__clrcall",
  [anon_sym___stdcall] = "__stdcall",
  [anon_sym___fastcall] = "__fastcall",
  [anon_sym___thiscall] = "__thiscall",
  [anon_sym___vectorcall] = "__vectorcall",
  [sym_ms_restrict_modifier] = "ms_restrict_modifier",
  [sym_ms_unsigned_ptr_modifier] = "ms_unsigned_ptr_modifier",
  [sym_ms_signed_ptr_modifier] = "ms_signed_ptr_modifier",
  [anon_sym__unaligned] = "_unaligned",
  [anon_sym___unaligned] = "__unaligned",
  [anon_sym_LBRACE] = "{",
  [anon_sym_RBRACE] = "}",
  [anon_sym_signed] = "signed",
  [anon_sym_unsigned] = "unsigned",
  [anon_sym_long] = "long",
  [anon_sym_short] = "short",
  [anon_sym_LBRACK] = "[",
  [anon_sym_RBRACK] = "]",
  [anon_sym_EQ] = "=",
  [anon_sym_static] = "static",
  [anon_sym_auto] = "auto",
  [anon_sym_register] = "register",
  [anon_sym_inline] = "inline",
  [anon_sym___inline] = "__inline",
  [anon_sym___inline__] = "__inline__",
  [anon_sym___forceinline] = "__forceinline",
  [anon_sym_thread_local] = "thread_local",
  [anon_sym___thread] = "__thread",
  [anon_sym_const] = "const",
  [anon_sym_constexpr] = "constexpr",
  [anon_sym_volatile] = "volatile",
  [anon_sym_restrict] = "restrict",
  [anon_sym___restrict__] = "__restrict__",
  [anon_sym__Atomic] = "_Atomic",
  [anon_sym__Noreturn] = "_Noreturn",
  [anon_sym_noreturn] = "noreturn",
  [sym_primitive_type] = "primitive_type",
  [anon_sym_enum] = "enum",
  [anon_sym_COLON] = ":",
  [anon_sym_struct] = "struct",
  [anon_sym_union] = "union",
  [anon_sym_if] = "if",
  [anon_sym_else] = "else",
  [anon_sym_switch] = "switch",
  [anon_sym_case] = "case",
  [anon_sym_default] = "default",
  [anon_sym_while] = "while",
  [anon_sym_do] = "do",
  [anon_sym_for] = "for",
  [anon_sym_return] = "return",
  [anon_sym_break] = "break",
  [anon_sym_continue] = "continue",
  [anon_sym_goto] = "goto",
  [anon_sym___try] = "__try",
  [anon_sym___except] = "__except",
  [anon_sym___finally] = "__finally",
  [anon_sym___leave] = "__leave",
  [anon_sym_QMARK] = "\?",
  [anon_sym_STAR_EQ] = "*=",
  [anon_sym_SLASH_EQ] = "/=",
  [anon_sym_PERCENT_EQ] = "%=",
  [anon_sym_PLUS_EQ] = "+=",
  [anon_sym_DASH_EQ] = "-=",
  [anon_sym_LT_LT_EQ] = "<<=",
  [anon_sym_GT_GT_EQ] = ">>=",
  [anon_sym_AMP_EQ] = "&=",
  [anon_sym_CARET_EQ] = "^=",
  [anon_sym_PIPE_EQ] = "|=",
  [anon_sym_DASH_DASH] = "--",
  [anon_sym_PLUS_PLUS] = "++",
  [anon_sym_sizeof] = "sizeof",
  [anon_sym___alignof__] = "__alignof__",
  [anon_sym___alignof] = "__alignof",
  [anon_sym__alignof] = "_alignof",
  [anon_sym_alignof] = "alignof",
  [anon_sym__Alignof] = "_Alignof",
  [anon_sym_offsetof] = "offsetof",
  [anon_sym__Generic] = "_Generic",
  [anon_sym_asm] = "asm",
  [anon_sym___asm__] = "__asm__",
  [anon_sym_DOT] = ".",
  [anon_sym_DASH_GT] = "->",
  [sym_number_literal] = "number_literal",
  [anon_sym_L_SQUOTE] = "L'",
  [anon_sym_u_SQUOTE] = "u'",
  [anon_sym_U_SQUOTE] = "U'",
  [anon_sym_u8_SQUOTE] = "u8'",
  [anon_sym_SQUOTE] = "'",
  [aux_sym_char_literal_token1] = "character",
  [anon_sym_L_DQUOTE] = "L\"",
  [anon_sym_u_DQUOTE] = "u\"",
  [anon_sym_U_DQUOTE] = "U\"",
  [anon_sym_u8_DQUOTE] = "u8\"",
  [anon_sym_DQUOTE] = "\"",
  [aux_sym_string_literal_token1] = "string_content",
  [sym_escape_sequence] = "escape_sequence",
  [sym_system_lib_string] = "system_lib_string",
  [sym_true] = "true",
  [sym_false] = "false",
  [anon_sym_NULL] = "NULL",
  [anon_sym_nullptr] = "nullptr",
  [sym_comment] = "comment",
  [sym_translation_unit] = "translation_unit",
  [sym_preproc_include] = "preproc_include",
  [sym_preproc_def] = "preproc_def",
  [sym_preproc_function_def] = "preproc_function_def",
  [sym_preproc_params] = "preproc_params",
  [sym_preproc_call] = "preproc_call",
  [sym_preproc_if] = "preproc_if",
  [sym_preproc_ifdef] = "preproc_ifdef",
  [sym_preproc_else] = "preproc_else",
  [sym_preproc_elif] = "preproc_elif",
  [sym_preproc_elifdef] = "preproc_elifdef",
  [sym_preproc_if_in_field_declaration_list] = "preproc_if",
  [sym_preproc_ifdef_in_field_declaration_list] = "preproc_ifdef",
  [sym_preproc_else_in_field_declaration_list] = "preproc_else",
  [sym_preproc_elif_in_field_declaration_list] = "preproc_elif",
  [sym_preproc_elifdef_in_field_declaration_list] = "preproc_elifdef",
  [sym_preproc_if_in_enumerator_list] = "preproc_if",
  [sym_preproc_ifdef_in_enumerator_list] = "preproc_ifdef",
  [sym_preproc_else_in_enumerator_list] = "preproc_else",
  [sym_preproc_elif_in_enumerator_list] = "preproc_elif",
  [sym_preproc_elifdef_in_enumerator_list] = "preproc_elifdef",
  [sym_preproc_if_in_enumerator_list_no_comma] = "preproc_if",
  [sym_preproc_ifdef_in_enumerator_list_no_comma] = "preproc_ifdef",
  [sym_preproc_else_in_enumerator_list_no_comma] = "preproc_else",
  [sym_preproc_elif_in_enumerator_list_no_comma] = "preproc_elif",
  [sym_preproc_elifdef_in_enumerator_list_no_comma] = "preproc_elifdef",
  [sym__preproc_expression] = "_preproc_expression",
  [sym_preproc_parenthesized_expression] = "parenthesized_expression",
  [sym_preproc_defined] = "preproc_defined",
  [sym_preproc_unary_expression] = "unary_expression",
  [sym_preproc_call_expression] = "call_expression",
  [sym_preproc_argument_list] = "argument_list",
  [sym_preproc_binary_expression] = "binary_expression",
  [sym_function_definition] = "function_definition",
  [sym__old_style_function_definition] = "function_definition",
  [sym_declaration] = "declaration",
  [sym_type_definition] = "type_definition",
  [sym__type_definition_type] = "_type_definition_type",
  [sym__type_definition_declarators] = "_type_definition_declarators",
  [sym__declaration_modifiers] = "_declaration_modifiers",
  [sym__declaration_specifiers] = "_declaration_specifiers",
  [sym_linkage_specification] = "linkage_specification",
  [sym_attribute_specifier] = "attribute_specifier",
  [sym_attribute] = "attribute",
  [sym_attribute_declaration] = "attribute_declaration",
  [sym_ms_declspec_modifier] = "ms_declspec_modifier",
  [sym_ms_based_modifier] = "ms_based_modifier",
  [sym_ms_call_modifier] = "ms_call_modifier",
  [sym_ms_unaligned_ptr_modifier] = "ms_unaligned_ptr_modifier",
  [sym_ms_pointer_modifier] = "ms_pointer_modifier",
  [sym_declaration_list] = "declaration_list",
  [sym__declarator] = "_declarator",
  [sym__declaration_declarator] = "_declaration_declarator",
  [sym__field_declarator] = "_field_declarator",
  [sym__type_declarator] = "_type_declarator",
  [sym__abstract_declarator] = "_abstract_declarator",
  [sym_parenthesized_declarator] = "parenthesized_declarator",
  [sym_parenthesized_field_declarator] = "parenthesized_declarator",
  [sym_parenthesized_type_declarator] = "parenthesized_declarator",
  [sym_abstract_parenthesized_declarator] = "abstract_parenthesized_declarator",
  [sym_attributed_declarator] = "attributed_declarator",
  [sym_attributed_field_declarator] = "attributed_declarator",
  [sym_attributed_type_declarator] = "attributed_declarator",
  [sym_pointer_declarator] = "pointer_declarator",
  [sym_pointer_field_declarator] = "pointer_declarator",
  [sym_pointer_type_declarator] = "pointer_declarator",
  [sym_abstract_pointer_declarator] = "abstract_pointer_declarator",
  [sym_function_declarator] = "function_declarator",
  [sym__function_declaration_declarator] = "function_declarator",
  [sym_function_field_declarator] = "function_declarator",
  [sym_function_type_declarator] = "function_declarator",
  [sym_abstract_function_declarator] = "abstract_function_declarator",
  [sym__old_style_function_declarator] = "function_declarator",
  [sym_array_declarator] = "array_declarator",
  [sym_array_field_declarator] = "array_declarator",
  [sym_array_type_declarator] = "array_declarator",
  [sym_abstract_array_declarator] = "abstract_array_declarator",
  [sym_init_declarator] = "init_declarator",
  [sym_compound_statement] = "compound_statement",
  [sym_storage_class_specifier] = "storage_class_specifier",
  [sym_type_qualifier] = "type_qualifier",
  [sym__type_specifier] = "_type_specifier",
  [sym_sized_type_specifier] = "sized_type_specifier",
  [sym_enum_specifier] = "enum_specifier",
  [sym_enumerator_list] = "enumerator_list",
  [sym_struct_specifier] = "struct_specifier",
  [sym_union_specifier] = "union_specifier",
  [sym_field_declaration_list] = "field_declaration_list",
  [sym__field_declaration_list_item] = "_field_declaration_list_item",
  [sym_field_declaration] = "field_declaration",
  [sym__field_declaration_declarator] = "_field_declaration_declarator",
  [sym_bitfield_clause] = "bitfield_clause",
  [sym_enumerator] = "enumerator",
  [sym_variadic_parameter] = "variadic_parameter",
  [sym_parameter_list] = "parameter_list",
  [sym__old_style_parameter_list] = "parameter_list",
  [sym_parameter_declaration] = "parameter_declaration",
  [sym_attributed_statement] = "attributed_statement",
  [sym_labeled_statement] = "labeled_statement",
  [sym__top_level_expression_statement] = "expression_statement",
  [sym_expression_statement] = "expression_statement",
  [sym_if_statement] = "if_statement",
  [sym_else_clause] = "else_clause",
  [sym_switch_statement] = "switch_statement",
  [sym_case_statement] = "case_statement",
  [sym_while_statement] = "while_statement",
  [sym_do_statement] = "do_statement",
  [sym_for_statement] = "for_statement",
  [sym__for_statement_body] = "_for_statement_body",
  [sym_return_statement] = "return_statement",
  [sym_break_statement] = "break_statement",
  [sym_continue_statement] = "continue_statement",
  [sym_goto_statement] = "goto_statement",
  [sym_seh_try_statement] = "seh_try_statement",
  [sym_seh_except_clause] = "seh_except_clause",
  [sym_seh_finally_clause] = "seh_finally_clause",
  [sym_seh_leave_statement] = "seh_leave_statement",
  [sym__expression] = "_expression",
  [sym__expression_not_binary] = "_expression_not_binary",
  [sym__string] = "_string",
  [sym_comma_expression] = "comma_expression",
  [sym_conditional_expression] = "conditional_expression",
  [sym_assignment_expression] = "assignment_expression",
  [sym_pointer_expression] = "pointer_expression",
  [sym_unary_expression] = "unary_expression",
  [sym_binary_expression] = "binary_expression",
  [sym_update_expression] = "update_expression",
  [sym_cast_expression] = "cast_expression",
  [sym_type_descriptor] = "type_descriptor",
  [sym_sizeof_expression] = "sizeof_expression",
  [sym_alignof_expression] = "alignof_expression",
  [sym_offsetof_expression] = "offsetof_expression",
  [sym_generic_expression] = "generic_expression",
  [sym_subscript_expression] = "subscript_expression",
  [sym_call_expression] = "call_expression",
  [sym_gnu_asm_expression] = "gnu_asm_expression",
  [sym_gnu_asm_qualifier] = "gnu_asm_qualifier",
  [sym_gnu_asm_output_operand_list] = "gnu_asm_output_operand_list",
  [sym_gnu_asm_output_operand] = "gnu_asm_output_operand",
  [sym_gnu_asm_input_operand_list] = "gnu_asm_input_operand_list",
  [sym_gnu_asm_input_operand] = "gnu_asm_input_operand",
  [sym_gnu_asm_clobber_list] = "gnu_asm_clobber_list",
  [sym_gnu_asm_goto_list] = "gnu_asm_goto_list",
  [sym_argument_list] = "argument_list",
  [sym_field_expression] = "field_expression",
  [sym_compound_literal_expression] = "compound_literal_expression",
  [sym_parenthesized_expression] = "parenthesized_expression",
  [sym_initializer_list] = "initializer_list",
  [sym_initializer_pair] = "initializer_pair",
  [sym_subscript_designator] = "subscript_designator",
  [sym_subscript_range_designator] = "subscript_range_designator",
  [sym_field_designator] = "field_designator",
  [sym_char_literal] = "char_literal",
  [sym_concatenated_string] = "concatenated_string",
  [sym_string_literal] = "string_literal",
  [sym_null] = "null",
  [sym__empty_declaration] = "_empty_declaration",
  [sym_macro_type_specifier] = "macro_type_specifier",
  [aux_sym_translation_unit_repeat1] = "translation_unit_repeat1",
  [aux_sym_preproc_params_repeat1] = "preproc_params_repeat1",
  [aux_sym_preproc_if_repeat1] = "preproc_if_repeat1",
  [aux_sym_preproc_if_in_field_declaration_list_repeat1] = "preproc_if_in_field_declaration_list_repeat1",
  [aux_sym_preproc_if_in_enumerator_list_repeat1] = "preproc_if_in_enumerator_list_repeat1",
  [aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1] = "preproc_if_in_enumerator_list_no_comma_repeat1",
  [aux_sym_preproc_argument_list_repeat1] = "preproc_argument_list_repeat1",
  [aux_sym__old_style_function_definition_repeat1] = "_old_style_function_definition_repeat1",
  [aux_sym_declaration_repeat1] = "declaration_repeat1",
  [aux_sym_type_definition_repeat1] = "type_definition_repeat1",
  [aux_sym__type_definition_type_repeat1] = "_type_definition_type_repeat1",
  [aux_sym__type_definition_declarators_repeat1] = "_type_definition_declarators_repeat1",
  [aux_sym__declaration_specifiers_repeat1] = "_declaration_specifiers_repeat1",
  [aux_sym_attribute_declaration_repeat1] = "attribute_declaration_repeat1",
  [aux_sym_attributed_declarator_repeat1] = "attributed_declarator_repeat1",
  [aux_sym_pointer_declarator_repeat1] = "pointer_declarator_repeat1",
  [aux_sym_function_declarator_repeat1] = "function_declarator_repeat1",
  [aux_sym_sized_type_specifier_repeat1] = "sized_type_specifier_repeat1",
  [aux_sym_enumerator_list_repeat1] = "enumerator_list_repeat1",
  [aux_sym__field_declaration_declarator_repeat1] = "_field_declaration_declarator_repeat1",
  [aux_sym_parameter_list_repeat1] = "parameter_list_repeat1",
  [aux_sym__old_style_parameter_list_repeat1] = "_old_style_parameter_list_repeat1",
  [aux_sym_case_statement_repeat1] = "case_statement_repeat1",
  [aux_sym_generic_expression_repeat1] = "generic_expression_repeat1",
  [aux_sym_gnu_asm_expression_repeat1] = "gnu_asm_expression_repeat1",
  [aux_sym_gnu_asm_output_operand_list_repeat1] = "gnu_asm_output_operand_list_repeat1",
  [aux_sym_gnu_asm_input_operand_list_repeat1] = "gnu_asm_input_operand_list_repeat1",
  [aux_sym_gnu_asm_clobber_list_repeat1] = "gnu_asm_clobber_list_repeat1",
  [aux_sym_gnu_asm_goto_list_repeat1] = "gnu_asm_goto_list_repeat1",
  [aux_sym_argument_list_repeat1] = "argument_list_repeat1",
  [aux_sym_initializer_list_repeat1] = "initializer_list_repeat1",
  [aux_sym_initializer_pair_repeat1] = "initializer_pair_repeat1",
  [aux_sym_char_literal_repeat1] = "char_literal_repeat1",
  [aux_sym_concatenated_string_repeat1] = "concatenated_string_repeat1",
  [aux_sym_string_literal_repeat1] = "string_literal_repeat1",
  [alias_sym_field_identifier] = "field_identifier",
  [alias_sym_statement_identifier] = "statement_identifier",
  [alias_sym_type_identifier] = "type_identifier",
};

static const TSSymbol ts_symbol_map[] = {
  [ts_builtin_sym_end] = ts_builtin_sym_end,
  [sym_identifier] = sym_identifier,
  [aux_sym_preproc_include_token1] = aux_sym_preproc_include_token1,
  [aux_sym_preproc_include_token2] = aux_sym_preproc_include_token2,
  [aux_sym_preproc_def_token1] = aux_sym_preproc_def_token1,
  [anon_sym_LPAREN] = anon_sym_LPAREN,
  [anon_sym_DOT_DOT_DOT] = anon_sym_DOT_DOT_DOT,
  [anon_sym_COMMA] = anon_sym_COMMA,
  [anon_sym_RPAREN] = anon_sym_RPAREN,
  [aux_sym_preproc_if_token1] = aux_sym_preproc_if_token1,
  [anon_sym_LF] = anon_sym_LF,
  [aux_sym_preproc_if_token2] = aux_sym_preproc_if_token2,
  [aux_sym_preproc_ifdef_token1] = aux_sym_preproc_ifdef_token1,
  [aux_sym_preproc_ifdef_token2] = aux_sym_preproc_ifdef_token2,
  [aux_sym_preproc_else_token1] = aux_sym_preproc_else_token1,
  [aux_sym_preproc_elif_token1] = aux_sym_preproc_elif_token1,
  [aux_sym_preproc_elifdef_token1] = aux_sym_preproc_elifdef_token1,
  [aux_sym_preproc_elifdef_token2] = aux_sym_preproc_elifdef_token2,
  [sym_preproc_arg] = sym_preproc_arg,
  [sym_preproc_directive] = sym_preproc_directive,
  [anon_sym_LPAREN2] = anon_sym_LPAREN,
  [anon_sym_defined] = anon_sym_defined,
  [anon_sym_BANG] = anon_sym_BANG,
  [anon_sym_TILDE] = anon_sym_TILDE,
  [anon_sym_DASH] = anon_sym_DASH,
  [anon_sym_PLUS] = anon_sym_PLUS,
  [anon_sym_STAR] = anon_sym_STAR,
  [anon_sym_SLASH] = anon_sym_SLASH,
  [anon_sym_PERCENT] = anon_sym_PERCENT,
  [anon_sym_PIPE_PIPE] = anon_sym_PIPE_PIPE,
  [anon_sym_AMP_AMP] = anon_sym_AMP_AMP,
  [anon_sym_PIPE] = anon_sym_PIPE,
  [anon_sym_CARET] = anon_sym_CARET,
  [anon_sym_AMP] = anon_sym_AMP,
  [anon_sym_EQ_EQ] = anon_sym_EQ_EQ,
  [anon_sym_BANG_EQ] = anon_sym_BANG_EQ,
  [anon_sym_GT] = anon_sym_GT,
  [anon_sym_GT_EQ] = anon_sym_GT_EQ,
  [anon_sym_LT_EQ] = anon_sym_LT_EQ,
  [anon_sym_LT] = anon_sym_LT,
  [anon_sym_LT_LT] = anon_sym_LT_LT,
  [anon_sym_GT_GT] = anon_sym_GT_GT,
  [anon_sym_SEMI] = anon_sym_SEMI,
  [anon_sym___extension__] = anon_sym___extension__,
  [anon_sym_typedef] = anon_sym_typedef,
  [anon_sym_extern] = anon_sym_extern,
  [anon_sym___attribute__] = anon_sym___attribute__,
  [anon_sym_COLON_COLON] = anon_sym_COLON_COLON,
  [anon_sym_LBRACK_LBRACK] = anon_sym_LBRACK_LBRACK,
  [anon_sym_RBRACK_RBRACK] = anon_sym_RBRACK_RBRACK,
  [anon_sym___declspec] = anon_sym___declspec,
  [anon_sym___based] = anon_sym___based,
  [anon_sym___cdecl] = anon_sym___cdecl,
  [anon_sym___clrcall] = anon_sym___clrcall,
  [anon_sym___stdcall] = anon_sym___stdcall,
  [anon_sym___fastcall] = anon_sym___fastcall,
  [anon_sym___thiscall] = anon_sym___thiscall,
  [anon_sym___vectorcall] = anon_sym___vectorcall,
  [sym_ms_restrict_modifier] = sym_ms_restrict_modifier,
  [sym_ms_unsigned_ptr_modifier] = sym_ms_unsigned_ptr_modifier,
  [sym_ms_signed_ptr_modifier] = sym_ms_signed_ptr_modifier,
  [anon_sym__unaligned] = anon_sym__unaligned,
  [anon_sym___unaligned] = anon_sym___unaligned,
  [anon_sym_LBRACE] = anon_sym_LBRACE,
  [anon_sym_RBRACE] = anon_sym_RBRACE,
  [anon_sym_signed] = anon_sym_signed,
  [anon_sym_unsigned] = anon_sym_unsigned,
  [anon_sym_long] = anon_sym_long,
  [anon_sym_short] = anon_sym_short,
  [anon_sym_LBRACK] = anon_sym_LBRACK,
  [anon_sym_RBRACK] = anon_sym_RBRACK,
  [anon_sym_EQ] = anon_sym_EQ,
  [anon_sym_static] = anon_sym_static,
  [anon_sym_auto] = anon_sym_auto,
  [anon_sym_register] = anon_sym_register,
  [anon_sym_inline] = anon_sym_inline,
  [anon_sym___inline] = anon_sym___inline,
  [anon_sym___inline__] = anon_sym___inline__,
  [anon_sym___forceinline] = anon_sym___forceinline,
  [anon_sym_thread_local] = anon_sym_thread_local,
  [anon_sym___thread] = anon_sym___thread,
  [anon_sym_const] = anon_sym_const,
  [anon_sym_constexpr] = anon_sym_constexpr,
  [anon_sym_volatile] = anon_sym_volatile,
  [anon_sym_restrict] = anon_sym_restrict,
  [anon_sym___restrict__] = anon_sym___restrict__,
  [anon_sym__Atomic] = anon_sym__Atomic,
  [anon_sym__Noreturn] = anon_sym__Noreturn,
  [anon_sym_noreturn] = anon_sym_noreturn,
  [sym_primitive_type] = sym_primitive_type,
  [anon_sym_enum] = anon_sym_enum,
  [anon_sym_COLON] = anon_sym_COLON,
  [anon_sym_struct] = anon_sym_struct,
  [anon_sym_union] = anon_sym_union,
  [anon_sym_if] = anon_sym_if,
  [anon_sym_else] = anon_sym_else,
  [anon_sym_switch] = anon_sym_switch,
  [anon_sym_case] = anon_sym_case,
  [anon_sym_default] = anon_sym_default,
  [anon_sym_while] = anon_sym_while,
  [anon_sym_do] = anon_sym_do,
  [anon_sym_for] = anon_sym_for,
  [anon_sym_return] = anon_sym_return,
  [anon_sym_break] = anon_sym_break,
  [anon_sym_continue] = anon_sym_continue,
  [anon_sym_goto] = anon_sym_goto,
  [anon_sym___try] = anon_sym___try,
  [anon_sym___except] = anon_sym___except,
  [anon_sym___finally] = anon_sym___finally,
  [anon_sym___leave] = anon_sym___leave,
  [anon_sym_QMARK] = anon_sym_QMARK,
  [anon_sym_STAR_EQ] = anon_sym_STAR_EQ,
  [anon_sym_SLASH_EQ] = anon_sym_SLASH_EQ,
  [anon_sym_PERCENT_EQ] = anon_sym_PERCENT_EQ,
  [anon_sym_PLUS_EQ] = anon_sym_PLUS_EQ,
  [anon_sym_DASH_EQ] = anon_sym_DASH_EQ,
  [anon_sym_LT_LT_EQ] = anon_sym_LT_LT_EQ,
  [anon_sym_GT_GT_EQ] = anon_sym_GT_GT_EQ,
  [anon_sym_AMP_EQ] = anon_sym_AMP_EQ,
  [anon_sym_CARET_EQ] = anon_sym_CARET_EQ,
  [anon_sym_PIPE_EQ] = anon_sym_PIPE_EQ,
  [anon_sym_DASH_DASH] = anon_sym_DASH_DASH,
  [anon_sym_PLUS_PLUS] = anon_sym_PLUS_PLUS,
  [anon_sym_sizeof] = anon_sym_sizeof,
  [anon_sym___alignof__] = anon_sym___alignof__,
  [anon_sym___alignof] = anon_sym___alignof,
  [anon_sym__alignof] = anon_sym__alignof,
  [anon_sym_alignof] = anon_sym_alignof,
  [anon_sym__Alignof] = anon_sym__Alignof,
  [anon_sym_offsetof] = anon_sym_offsetof,
  [anon_sym__Generic] = anon_sym__Generic,
  [anon_sym_asm] = anon_sym_asm,
  [anon_sym___asm__] = anon_sym___asm__,
  [anon_sym_DOT] = anon_sym_DOT,
  [anon_sym_DASH_GT] = anon_sym_DASH_GT,
  [sym_number_literal] = sym_number_literal,
  [anon_sym_L_SQUOTE] = anon_sym_L_SQUOTE,
  [anon_sym_u_SQUOTE] = anon_sym_u_SQUOTE,
  [anon_sym_U_SQUOTE] = anon_sym_U_SQUOTE,
  [anon_sym_u8_SQUOTE] = anon_sym_u8_SQUOTE,
  [anon_sym_SQUOTE] = anon_sym_SQUOTE,
  [aux_sym_char_literal_token1] = aux_sym_char_literal_token1,
  [anon_sym_L_DQUOTE] = anon_sym_L_DQUOTE,
  [anon_sym_u_DQUOTE] = anon_sym_u_DQUOTE,
  [anon_sym_U_DQUOTE] = anon_sym_U_DQUOTE,
  [anon_sym_u8_DQUOTE] = anon_sym_u8_DQUOTE,
  [anon_sym_DQUOTE] = anon_sym_DQUOTE,
  [aux_sym_string_literal_token1] = aux_sym_string_literal_token1,
  [sym_escape_sequence] = sym_escape_sequence,
  [sym_system_lib_string] = sym_system_lib_string,
  [sym_true] = sym_true,
  [sym_false] = sym_false,
  [anon_sym_NULL] = anon_sym_NULL,
  [anon_sym_nullptr] = anon_sym_nullptr,
  [sym_comment] = sym_comment,
  [sym_translation_unit] = sym_translation_unit,
  [sym_preproc_include] = sym_preproc_include,
  [sym_preproc_def] = sym_preproc_def,
  [sym_preproc_function_def] = sym_preproc_function_def,
  [sym_preproc_params] = sym_preproc_params,
  [sym_preproc_call] = sym_preproc_call,
  [sym_preproc_if] = sym_preproc_if,
  [sym_preproc_ifdef] = sym_preproc_ifdef,
  [sym_preproc_else] = sym_preproc_else,
  [sym_preproc_elif] = sym_preproc_elif,
  [sym_preproc_elifdef] = sym_preproc_elifdef,
  [sym_preproc_if_in_field_declaration_list] = sym_preproc_if,
  [sym_preproc_ifdef_in_field_declaration_list] = sym_preproc_ifdef,
  [sym_preproc_else_in_field_declaration_list] = sym_preproc_else,
  [sym_preproc_elif_in_field_declaration_list] = sym_preproc_elif,
  [sym_preproc_elifdef_in_field_declaration_list] = sym_preproc_elifdef,
  [sym_preproc_if_in_enumerator_list] = sym_preproc_if,
  [sym_preproc_ifdef_in_enumerator_list] = sym_preproc_ifdef,
  [sym_preproc_else_in_enumerator_list] = sym_preproc_else,
  [sym_preproc_elif_in_enumerator_list] = sym_preproc_elif,
  [sym_preproc_elifdef_in_enumerator_list] = sym_preproc_elifdef,
  [sym_preproc_if_in_enumerator_list_no_comma] = sym_preproc_if,
  [sym_preproc_ifdef_in_enumerator_list_no_comma] = sym_preproc_ifdef,
  [sym_preproc_else_in_enumerator_list_no_comma] = sym_preproc_else,
  [sym_preproc_elif_in_enumerator_list_no_comma] = sym_preproc_elif,
  [sym_preproc_elifdef_in_enumerator_list_no_comma] = sym_preproc_elifdef,
  [sym__preproc_expression] = sym__preproc_expression,
  [sym_preproc_parenthesized_expression] = sym_parenthesized_expression,
  [sym_preproc_defined] = sym_preproc_defined,
  [sym_preproc_unary_expression] = sym_unary_expression,
  [sym_preproc_call_expression] = sym_call_expression,
  [sym_preproc_argument_list] = sym_argument_list,
  [sym_preproc_binary_expression] = sym_binary_expression,
  [sym_function_definition] = sym_function_definition,
  [sym__old_style_function_definition] = sym_function_definition,
  [sym_declaration] = sym_declaration,
  [sym_type_definition] = sym_type_definition,
  [sym__type_definition_type] = sym__type_definition_type,
  [sym__type_definition_declarators] = sym__type_definition_declarators,
  [sym__declaration_modifiers] = sym__declaration_modifiers,
  [sym__declaration_specifiers] = sym__declaration_specifiers,
  [sym_linkage_specification] = sym_linkage_specification,
  [sym_attribute_specifier] = sym_attribute_specifier,
  [sym_attribute] = sym_attribute,
  [sym_attribute_declaration] = sym_attribute_declaration,
  [sym_ms_declspec_modifier] = sym_ms_declspec_modifier,
  [sym_ms_based_modifier] = sym_ms_based_modifier,
  [sym_ms_call_modifier] = sym_ms_call_modifier,
  [sym_ms_unaligned_ptr_modifier] = sym_ms_unaligned_ptr_modifier,
  [sym_ms_pointer_modifier] = sym_ms_pointer_modifier,
  [sym_declaration_list] = sym_declaration_list,
  [sym__declarator] = sym__declarator,
  [sym__declaration_declarator] = sym__declaration_declarator,
  [sym__field_declarator] = sym__field_declarator,
  [sym__type_declarator] = sym__type_declarator,
  [sym__abstract_declarator] = sym__abstract_declarator,
  [sym_parenthesized_declarator] = sym_parenthesized_declarator,
  [sym_parenthesized_field_declarator] = sym_parenthesized_declarator,
  [sym_parenthesized_type_declarator] = sym_parenthesized_declarator,
  [sym_abstract_parenthesized_declarator] = sym_abstract_parenthesized_declarator,
  [sym_attributed_declarator] = sym_attributed_declarator,
  [sym_attributed_field_declarator] = sym_attributed_declarator,
  [sym_attributed_type_declarator] = sym_attributed_declarator,
  [sym_pointer_declarator] = sym_pointer_declarator,
  [sym_pointer_field_declarator] = sym_pointer_declarator,
  [sym_pointer_type_declarator] = sym_pointer_declarator,
  [sym_abstract_pointer_declarator] = sym_abstract_pointer_declarator,
  [sym_function_declarator] = sym_function_declarator,
  [sym__function_declaration_declarator] = sym_function_declarator,
  [sym_function_field_declarator] = sym_function_declarator,
  [sym_function_type_declarator] = sym_function_declarator,
  [sym_abstract_function_declarator] = sym_abstract_function_declarator,
  [sym__old_style_function_declarator] = sym_function_declarator,
  [sym_array_declarator] = sym_array_declarator,
  [sym_array_field_declarator] = sym_array_declarator,
  [sym_array_type_declarator] = sym_array_declarator,
  [sym_abstract_array_declarator] = sym_abstract_array_declarator,
  [sym_init_declarator] = sym_init_declarator,
  [sym_compound_statement] = sym_compound_statement,
  [sym_storage_class_specifier] = sym_storage_class_specifier,
  [sym_type_qualifier] = sym_type_qualifier,
  [sym__type_specifier] = sym__type_specifier,
  [sym_sized_type_specifier] = sym_sized_type_specifier,
  [sym_enum_specifier] = sym_enum_specifier,
  [sym_enumerator_list] = sym_enumerator_list,
  [sym_struct_specifier] = sym_struct_specifier,
  [sym_union_specifier] = sym_union_specifier,
  [sym_field_declaration_list] = sym_field_declaration_list,
  [sym__field_declaration_list_item] = sym__field_declaration_list_item,
  [sym_field_declaration] = sym_field_declaration,
  [sym__field_declaration_declarator] = sym__field_declaration_declarator,
  [sym_bitfield_clause] = sym_bitfield_clause,
  [sym_enumerator] = sym_enumerator,
  [sym_variadic_parameter] = sym_variadic_parameter,
  [sym_parameter_list] = sym_parameter_list,
  [sym__old_style_parameter_list] = sym_parameter_list,
  [sym_parameter_declaration] = sym_parameter_declaration,
  [sym_attributed_statement] = sym_attributed_statement,
  [sym_labeled_statement] = sym_labeled_statement,
  [sym__top_level_expression_statement] = sym_expression_statement,
  [sym_expression_statement] = sym_expression_statement,
  [sym_if_statement] = sym_if_statement,
  [sym_else_clause] = sym_else_clause,
  [sym_switch_statement] = sym_switch_statement,
  [sym_case_statement] = sym_case_statement,
  [sym_while_statement] = sym_while_statement,
  [sym_do_statement] = sym_do_statement,
  [sym_for_statement] = sym_for_statement,
  [sym__for_statement_body] = sym__for_statement_body,
  [sym_return_statement] = sym_return_statement,
  [sym_break_statement] = sym_break_statement,
  [sym_continue_statement] = sym_continue_statement,
  [sym_goto_statement] = sym_goto_statement,
  [sym_seh_try_statement] = sym_seh_try_statement,
  [sym_seh_except_clause] = sym_seh_except_clause,
  [sym_seh_finally_clause] = sym_seh_finally_clause,
  [sym_seh_leave_statement] = sym_seh_leave_statement,
  [sym__expression] = sym__expression,
  [sym__expression_not_binary] = sym__expression_not_binary,
  [sym__string] = sym__string,
  [sym_comma_expression] = sym_comma_expression,
  [sym_conditional_expression] = sym_conditional_expression,
  [sym_assignment_expression] = sym_assignment_expression,
  [sym_pointer_expression] = sym_pointer_expression,
  [sym_unary_expression] = sym_unary_expression,
  [sym_binary_expression] = sym_binary_expression,
  [sym_update_expression] = sym_update_expression,
  [sym_cast_expression] = sym_cast_expression,
  [sym_type_descriptor] = sym_type_descriptor,
  [sym_sizeof_expression] = sym_sizeof_expression,
  [sym_alignof_expression] = sym_alignof_expression,
  [sym_offsetof_expression] = sym_offsetof_expression,
  [sym_generic_expression] = sym_generic_expression,
  [sym_subscript_expression] = sym_subscript_expression,
  [sym_call_expression] = sym_call_expression,
  [sym_gnu_asm_expression] = sym_gnu_asm_expression,
  [sym_gnu_asm_qualifier] = sym_gnu_asm_qualifier,
  [sym_gnu_asm_output_operand_list] = sym_gnu_asm_output_operand_list,
  [sym_gnu_asm_output_operand] = sym_gnu_asm_output_operand,
  [sym_gnu_asm_input_operand_list] = sym_gnu_asm_input_operand_list,
  [sym_gnu_asm_input_operand] = sym_gnu_asm_input_operand,
  [sym_gnu_asm_clobber_list] = sym_gnu_asm_clobber_list,
  [sym_gnu_asm_goto_list] = sym_gnu_asm_goto_list,
  [sym_argument_list] = sym_argument_list,
  [sym_field_expression] = sym_field_expression,
  [sym_compound_literal_expression] = sym_compound_literal_expression,
  [sym_parenthesized_expression] = sym_parenthesized_expression,
  [sym_initializer_list] = sym_initializer_list,
  [sym_initializer_pair] = sym_initializer_pair,
  [sym_subscript_designator] = sym_subscript_designator,
  [sym_subscript_range_designator] = sym_subscript_range_designator,
  [sym_field_designator] = sym_field_designator,
  [sym_char_literal] = sym_char_literal,
  [sym_concatenated_string] = sym_concatenated_string,
  [sym_string_literal] = sym_string_literal,
  [sym_null] = sym_null,
  [sym__empty_declaration] = sym__empty_declaration,
  [sym_macro_type_specifier] = sym_macro_type_specifier,
  [aux_sym_translation_unit_repeat1] = aux_sym_translation_unit_repeat1,
  [aux_sym_preproc_params_repeat1] = aux_sym_preproc_params_repeat1,
  [aux_sym_preproc_if_repeat1] = aux_sym_preproc_if_repeat1,
  [aux_sym_preproc_if_in_field_declaration_list_repeat1] = aux_sym_preproc_if_in_field_declaration_list_repeat1,
  [aux_sym_preproc_if_in_enumerator_list_repeat1] = aux_sym_preproc_if_in_enumerator_list_repeat1,
  [aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1] = aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
  [aux_sym_preproc_argument_list_repeat1] = aux_sym_preproc_argument_list_repeat1,
  [aux_sym__old_style_function_definition_repeat1] = aux_sym__old_style_function_definition_repeat1,
  [aux_sym_declaration_repeat1] = aux_sym_declaration_repeat1,
  [aux_sym_type_definition_repeat1] = aux_sym_type_definition_repeat1,
  [aux_sym__type_definition_type_repeat1] = aux_sym__type_definition_type_repeat1,
  [aux_sym__type_definition_declarators_repeat1] = aux_sym__type_definition_declarators_repeat1,
  [aux_sym__declaration_specifiers_repeat1] = aux_sym__declaration_specifiers_repeat1,
  [aux_sym_attribute_declaration_repeat1] = aux_sym_attribute_declaration_repeat1,
  [aux_sym_attributed_declarator_repeat1] = aux_sym_attributed_declarator_repeat1,
  [aux_sym_pointer_declarator_repeat1] = aux_sym_pointer_declarator_repeat1,
  [aux_sym_function_declarator_repeat1] = aux_sym_function_declarator_repeat1,
  [aux_sym_sized_type_specifier_repeat1] = aux_sym_sized_type_specifier_repeat1,
  [aux_sym_enumerator_list_repeat1] = aux_sym_enumerator_list_repeat1,
  [aux_sym__field_declaration_declarator_repeat1] = aux_sym__field_declaration_declarator_repeat1,
  [aux_sym_parameter_list_repeat1] = aux_sym_parameter_list_repeat1,
  [aux_sym__old_style_parameter_list_repeat1] = aux_sym__old_style_parameter_list_repeat1,
  [aux_sym_case_statement_repeat1] = aux_sym_case_statement_repeat1,
  [aux_sym_generic_expression_repeat1] = aux_sym_generic_expression_repeat1,
  [aux_sym_gnu_asm_expression_repeat1] = aux_sym_gnu_asm_expression_repeat1,
  [aux_sym_gnu_asm_output_operand_list_repeat1] = aux_sym_gnu_asm_output_operand_list_repeat1,
  [aux_sym_gnu_asm_input_operand_list_repeat1] = aux_sym_gnu_asm_input_operand_list_repeat1,
  [aux_sym_gnu_asm_clobber_list_repeat1] = aux_sym_gnu_asm_clobber_list_repeat1,
  [aux_sym_gnu_asm_goto_list_repeat1] = aux_sym_gnu_asm_goto_list_repeat1,
  [aux_sym_argument_list_repeat1] = aux_sym_argument_list_repeat1,
  [aux_sym_initializer_list_repeat1] = aux_sym_initializer_list_repeat1,
  [aux_sym_initializer_pair_repeat1] = aux_sym_initializer_pair_repeat1,
  [aux_sym_char_literal_repeat1] = aux_sym_char_literal_repeat1,
  [aux_sym_concatenated_string_repeat1] = aux_sym_concatenated_string_repeat1,
  [aux_sym_string_literal_repeat1] = aux_sym_string_literal_repeat1,
  [alias_sym_field_identifier] = alias_sym_field_identifier,
  [alias_sym_statement_identifier] = alias_sym_statement_identifier,
  [alias_sym_type_identifier] = alias_sym_type_identifier,
};

static const TSSymbolMetadata ts_symbol_metadata[] = {
  [ts_builtin_sym_end] = {
    .visible = false,
    .named = true,
  },
  [sym_identifier] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_preproc_include_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_include_token2] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_def_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LPAREN] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT_DOT_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COMMA] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RPAREN] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_if_token1] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LF] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_if_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_ifdef_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_ifdef_token2] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_else_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_elif_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_elifdef_token1] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_preproc_elifdef_token2] = {
    .visible = true,
    .named = false,
  },
  [sym_preproc_arg] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_directive] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_LPAREN2] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_defined] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_TILDE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_BANG_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SEMI] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___extension__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_typedef] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_extern] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___attribute__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___declspec] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___based] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___cdecl] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___clrcall] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___stdcall] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___fastcall] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___thiscall] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___vectorcall] = {
    .visible = true,
    .named = false,
  },
  [sym_ms_restrict_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_ms_unsigned_ptr_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_ms_signed_ptr_modifier] = {
    .visible = true,
    .named = true,
  },
  [anon_sym__unaligned] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___unaligned] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_signed] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_unsigned] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_long] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_short] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_RBRACK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_static] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_auto] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_register] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_inline] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___inline] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___inline__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___forceinline] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_thread_local] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___thread] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_const] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_constexpr] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_volatile] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_restrict] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___restrict__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym__Atomic] = {
    .visible = true,
    .named = false,
  },
  [anon_sym__Noreturn] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_noreturn] = {
    .visible = true,
    .named = false,
  },
  [sym_primitive_type] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_enum] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_COLON] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_struct] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_union] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_if] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_else] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_switch] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_case] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_default] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_while] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_do] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_for] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_return] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_break] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_continue] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_goto] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___try] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___except] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___finally] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___leave] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_QMARK] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_STAR_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SLASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PERCENT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_LT_LT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_GT_GT_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_AMP_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_CARET_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PIPE_EQ] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_DASH] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_PLUS_PLUS] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_sizeof] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___alignof__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___alignof] = {
    .visible = true,
    .named = false,
  },
  [anon_sym__alignof] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_alignof] = {
    .visible = true,
    .named = false,
  },
  [anon_sym__Alignof] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_offsetof] = {
    .visible = true,
    .named = false,
  },
  [anon_sym__Generic] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_asm] = {
    .visible = true,
    .named = false,
  },
  [anon_sym___asm__] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DOT] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DASH_GT] = {
    .visible = true,
    .named = false,
  },
  [sym_number_literal] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_L_SQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u_SQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_U_SQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u8_SQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_SQUOTE] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_char_literal_token1] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_L_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_U_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_u8_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_DQUOTE] = {
    .visible = true,
    .named = false,
  },
  [aux_sym_string_literal_token1] = {
    .visible = true,
    .named = true,
  },
  [sym_escape_sequence] = {
    .visible = true,
    .named = true,
  },
  [sym_system_lib_string] = {
    .visible = true,
    .named = true,
  },
  [sym_true] = {
    .visible = true,
    .named = true,
  },
  [sym_false] = {
    .visible = true,
    .named = true,
  },
  [anon_sym_NULL] = {
    .visible = true,
    .named = false,
  },
  [anon_sym_nullptr] = {
    .visible = true,
    .named = false,
  },
  [sym_comment] = {
    .visible = true,
    .named = true,
  },
  [sym_translation_unit] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_include] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_def] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_function_def] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_params] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_call] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_ifdef] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_elif] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_elifdef] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_in_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_ifdef_in_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_elif_in_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_elifdef_in_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_in_enumerator_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_ifdef_in_enumerator_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_enumerator_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_elif_in_enumerator_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_elifdef_in_enumerator_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_if_in_enumerator_list_no_comma] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_ifdef_in_enumerator_list_no_comma] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_else_in_enumerator_list_no_comma] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_elif_in_enumerator_list_no_comma] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_elifdef_in_enumerator_list_no_comma] = {
    .visible = true,
    .named = true,
  },
  [sym__preproc_expression] = {
    .visible = false,
    .named = true,
  },
  [sym_preproc_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_defined] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_unary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym_preproc_binary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_function_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__old_style_function_definition] = {
    .visible = true,
    .named = true,
  },
  [sym_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_type_definition] = {
    .visible = true,
    .named = true,
  },
  [sym__type_definition_type] = {
    .visible = false,
    .named = true,
  },
  [sym__type_definition_declarators] = {
    .visible = false,
    .named = true,
  },
  [sym__declaration_modifiers] = {
    .visible = false,
    .named = true,
  },
  [sym__declaration_specifiers] = {
    .visible = false,
    .named = true,
  },
  [sym_linkage_specification] = {
    .visible = true,
    .named = true,
  },
  [sym_attribute_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_attribute] = {
    .visible = true,
    .named = true,
  },
  [sym_attribute_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_ms_declspec_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_ms_based_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_ms_call_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_ms_unaligned_ptr_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_ms_pointer_modifier] = {
    .visible = true,
    .named = true,
  },
  [sym_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym__declarator] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__declaration_declarator] = {
    .visible = false,
    .named = true,
  },
  [sym__field_declarator] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__type_declarator] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__abstract_declarator] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_parenthesized_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_field_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_type_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_abstract_parenthesized_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_attributed_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_attributed_field_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_attributed_type_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_pointer_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_pointer_field_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_pointer_type_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_abstract_pointer_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_function_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym__function_declaration_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_function_field_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_function_type_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_abstract_function_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym__old_style_function_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_array_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_array_field_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_array_type_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_abstract_array_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_init_declarator] = {
    .visible = true,
    .named = true,
  },
  [sym_compound_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_storage_class_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_type_qualifier] = {
    .visible = true,
    .named = true,
  },
  [sym__type_specifier] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym_sized_type_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_enum_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_enumerator_list] = {
    .visible = true,
    .named = true,
  },
  [sym_struct_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_union_specifier] = {
    .visible = true,
    .named = true,
  },
  [sym_field_declaration_list] = {
    .visible = true,
    .named = true,
  },
  [sym__field_declaration_list_item] = {
    .visible = false,
    .named = true,
  },
  [sym_field_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym__field_declaration_declarator] = {
    .visible = false,
    .named = true,
  },
  [sym_bitfield_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_enumerator] = {
    .visible = true,
    .named = true,
  },
  [sym_variadic_parameter] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter_list] = {
    .visible = true,
    .named = true,
  },
  [sym__old_style_parameter_list] = {
    .visible = true,
    .named = true,
  },
  [sym_parameter_declaration] = {
    .visible = true,
    .named = true,
  },
  [sym_attributed_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_labeled_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__top_level_expression_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_expression_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_if_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_else_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_switch_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_case_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_while_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_do_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_for_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__for_statement_body] = {
    .visible = false,
    .named = true,
  },
  [sym_return_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_break_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_continue_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_goto_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_seh_try_statement] = {
    .visible = true,
    .named = true,
  },
  [sym_seh_except_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_seh_finally_clause] = {
    .visible = true,
    .named = true,
  },
  [sym_seh_leave_statement] = {
    .visible = true,
    .named = true,
  },
  [sym__expression] = {
    .visible = false,
    .named = true,
    .supertype = true,
  },
  [sym__expression_not_binary] = {
    .visible = false,
    .named = true,
  },
  [sym__string] = {
    .visible = false,
    .named = true,
  },
  [sym_comma_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_conditional_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_assignment_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_pointer_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_unary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_binary_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_update_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_cast_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_type_descriptor] = {
    .visible = true,
    .named = true,
  },
  [sym_sizeof_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_alignof_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_offsetof_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_generic_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_subscript_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_call_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_gnu_asm_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_gnu_asm_qualifier] = {
    .visible = true,
    .named = true,
  },
  [sym_gnu_asm_output_operand_list] = {
    .visible = true,
    .named = true,
  },
  [sym_gnu_asm_output_operand] = {
    .visible = true,
    .named = true,
  },
  [sym_gnu_asm_input_operand_list] = {
    .visible = true,
    .named = true,
  },
  [sym_gnu_asm_input_operand] = {
    .visible = true,
    .named = true,
  },
  [sym_gnu_asm_clobber_list] = {
    .visible = true,
    .named = true,
  },
  [sym_gnu_asm_goto_list] = {
    .visible = true,
    .named = true,
  },
  [sym_argument_list] = {
    .visible = true,
    .named = true,
  },
  [sym_field_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_compound_literal_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_parenthesized_expression] = {
    .visible = true,
    .named = true,
  },
  [sym_initializer_list] = {
    .visible = true,
    .named = true,
  },
  [sym_initializer_pair] = {
    .visible = true,
    .named = true,
  },
  [sym_subscript_designator] = {
    .visible = true,
    .named = true,
  },
  [sym_subscript_range_designator] = {
    .visible = true,
    .named = true,
  },
  [sym_field_designator] = {
    .visible = true,
    .named = true,
  },
  [sym_char_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_concatenated_string] = {
    .visible = true,
    .named = true,
  },
  [sym_string_literal] = {
    .visible = true,
    .named = true,
  },
  [sym_null] = {
    .visible = true,
    .named = true,
  },
  [sym__empty_declaration] = {
    .visible = false,
    .named = true,
  },
  [sym_macro_type_specifier] = {
    .visible = true,
    .named = true,
  },
  [aux_sym_translation_unit_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_params_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_in_field_declaration_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_in_enumerator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_preproc_argument_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__old_style_function_definition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_type_definition_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__type_definition_type_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__type_definition_declarators_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__declaration_specifiers_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_attribute_declaration_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_attributed_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_pointer_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_function_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_sized_type_specifier_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_enumerator_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__field_declaration_declarator_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_parameter_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym__old_style_parameter_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_case_statement_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_generic_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_gnu_asm_expression_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_gnu_asm_output_operand_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_gnu_asm_input_operand_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_gnu_asm_clobber_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_gnu_asm_goto_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_argument_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_initializer_list_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_initializer_pair_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_char_literal_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_concatenated_string_repeat1] = {
    .visible = false,
    .named = false,
  },
  [aux_sym_string_literal_repeat1] = {
    .visible = false,
    .named = false,
  },
  [alias_sym_field_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_statement_identifier] = {
    .visible = true,
    .named = true,
  },
  [alias_sym_type_identifier] = {
    .visible = true,
    .named = true,
  },
};

enum ts_field_identifiers {
  field_alternative = 1,
  field_argument = 2,
  field_arguments = 3,
  field_assembly_code = 4,
  field_body = 5,
  field_clobbers = 6,
  field_condition = 7,
  field_consequence = 8,
  field_constraint = 9,
  field_declarator = 10,
  field_designator = 11,
  field_directive = 12,
  field_end = 13,
  field_field = 14,
  field_filter = 15,
  field_function = 16,
  field_goto_labels = 17,
  field_index = 18,
  field_initializer = 19,
  field_input_operands = 20,
  field_label = 21,
  field_left = 22,
  field_member = 23,
  field_name = 24,
  field_operand = 25,
  field_operator = 26,
  field_output_operands = 27,
  field_parameters = 28,
  field_path = 29,
  field_prefix = 30,
  field_register = 31,
  field_right = 32,
  field_size = 33,
  field_start = 34,
  field_symbol = 35,
  field_type = 36,
  field_underlying_type = 37,
  field_update = 38,
  field_value = 39,
};

static const char * const ts_field_names[] = {
  [0] = NULL,
  [field_alternative] = "alternative",
  [field_argument] = "argument",
  [field_arguments] = "arguments",
  [field_assembly_code] = "assembly_code",
  [field_body] = "body",
  [field_clobbers] = "clobbers",
  [field_condition] = "condition",
  [field_consequence] = "consequence",
  [field_constraint] = "constraint",
  [field_declarator] = "declarator",
  [field_designator] = "designator",
  [field_directive] = "directive",
  [field_end] = "end",
  [field_field] = "field",
  [field_filter] = "filter",
  [field_function] = "function",
  [field_goto_labels] = "goto_labels",
  [field_index] = "index",
  [field_initializer] = "initializer",
  [field_input_operands] = "input_operands",
  [field_label] = "label",
  [field_left] = "left",
  [field_member] = "member",
  [field_name] = "name",
  [field_operand] = "operand",
  [field_operator] = "operator",
  [field_output_operands] = "output_operands",
  [field_parameters] = "parameters",
  [field_path] = "path",
  [field_prefix] = "prefix",
  [field_register] = "register",
  [field_right] = "right",
  [field_size] = "size",
  [field_start] = "start",
  [field_symbol] = "symbol",
  [field_type] = "type",
  [field_underlying_type] = "underlying_type",
  [field_update] = "update",
  [field_value] = "value",
};

static const TSFieldMapSlice ts_field_map_slices[PRODUCTION_ID_COUNT] = {
  [2] = {.index = 0, .length = 3},
  [3] = {.index = 3, .length = 1},
  [4] = {.index = 4, .length = 1},
  [5] = {.index = 5, .length = 2},
  [6] = {.index = 7, .length = 1},
  [7] = {.index = 8, .length = 1},
  [8] = {.index = 9, .length = 1},
  [9] = {.index = 10, .length = 1},
  [10] = {.index = 3, .length = 1},
  [11] = {.index = 11, .length = 2},
  [12] = {.index = 13, .length = 2},
  [13] = {.index = 15, .length = 2},
  [14] = {.index = 17, .length = 1},
  [15] = {.index = 17, .length = 1},
  [16] = {.index = 18, .length = 1},
  [17] = {.index = 8, .length = 1},
  [18] = {.index = 19, .length = 2},
  [19] = {.index = 21, .length = 2},
  [20] = {.index = 23, .length = 1},
  [22] = {.index = 24, .length = 1},
  [23] = {.index = 25, .length = 2},
  [24] = {.index = 27, .length = 2},
  [25] = {.index = 29, .length = 1},
  [26] = {.index = 30, .length = 1},
  [27] = {.index = 31, .length = 2},
  [28] = {.index = 33, .length = 2},
  [29] = {.index = 35, .length = 1},
  [30] = {.index = 36, .length = 3},
  [31] = {.index = 39, .length = 1},
  [32] = {.index = 40, .length = 1},
  [33] = {.index = 41, .length = 3},
  [34] = {.index = 44, .length = 2},
  [35] = {.index = 46, .length = 2},
  [36] = {.index = 48, .length = 5},
  [37] = {.index = 53, .length = 3},
  [38] = {.index = 56, .length = 2},
  [39] = {.index = 58, .length = 2},
  [40] = {.index = 60, .length = 1},
  [41] = {.index = 61, .length = 2},
  [42] = {.index = 63, .length = 1},
  [43] = {.index = 64, .length = 2},
  [44] = {.index = 66, .length = 2},
  [45] = {.index = 68, .length = 2},
  [46] = {.index = 70, .length = 2},
  [47] = {.index = 72, .length = 2},
  [48] = {.index = 74, .length = 2},
  [49] = {.index = 76, .length = 2},
  [50] = {.index = 78, .length = 2},
  [52] = {.index = 80, .length = 2},
  [53] = {.index = 82, .length = 1},
  [54] = {.index = 83, .length = 1},
  [55] = {.index = 84, .length = 3},
  [56] = {.index = 87, .length = 1},
  [57] = {.index = 88, .length = 1},
  [58] = {.index = 89, .length = 1},
  [59] = {.index = 90, .length = 2},
  [60] = {.index = 92, .length = 1},
  [61] = {.index = 93, .length = 3},
  [62] = {.index = 96, .length = 3},
  [63] = {.index = 99, .length = 2},
  [64] = {.index = 101, .length = 3},
  [65] = {.index = 104, .length = 2},
  [66] = {.index = 106, .length = 5},
  [67] = {.index = 111, .length = 3},
  [68] = {.index = 114, .length = 5},
  [69] = {.index = 119, .length = 2},
  [70] = {.index = 121, .length = 2},
  [71] = {.index = 123, .length = 3},
  [72] = {.index = 126, .length = 2},
  [73] = {.index = 128, .length = 2},
  [74] = {.index = 130, .length = 1},
  [75] = {.index = 131, .length = 2},
  [76] = {.index = 133, .length = 2},
  [77] = {.index = 135, .length = 2},
  [78] = {.index = 137, .length = 3},
  [79] = {.index = 140, .length = 2},
  [80] = {.index = 142, .length = 2},
  [81] = {.index = 144, .length = 2},
  [82] = {.index = 146, .length = 1},
  [83] = {.index = 147, .length = 2},
  [84] = {.index = 149, .length = 2},
  [85] = {.index = 151, .length = 4},
  [86] = {.index = 155, .length = 1},
  [87] = {.index = 156, .length = 2},
  [88] = {.index = 158, .length = 1},
  [89] = {.index = 159, .length = 1},
  [90] = {.index = 160, .length = 4},
  [91] = {.index = 164, .length = 4},
  [92] = {.index = 168, .length = 2},
  [93] = {.index = 170, .length = 2},
  [94] = {.index = 172, .length = 3},
  [95] = {.index = 175, .length = 5},
  [96] = {.index = 180, .length = 3},
  [97] = {.index = 183, .length = 2},
  [98] = {.index = 185, .length = 1},
  [100] = {.index = 186, .length = 2},
  [101] = {.index = 188, .length = 2},
  [102] = {.index = 190, .length = 2},
  [103] = {.index = 192, .length = 3},
  [104] = {.index = 195, .length = 2},
  [105] = {.index = 197, .length = 2},
  [106] = {.index = 199, .length = 2},
  [107] = {.index = 201, .length = 2},
  [108] = {.index = 203, .length = 3},
  [109] = {.index = 206, .length = 2},
  [110] = {.index = 208, .length = 1},
  [111] = {.index = 209, .length = 5},
  [112] = {.index = 214, .length = 2},
  [113] = {.index = 216, .length = 3},
  [114] = {.index = 219, .length = 2},
  [115] = {.index = 219, .length = 2},
  [116] = {.index = 221, .length = 3},
  [117] = {.index = 224, .length = 2},
  [118] = {.index = 226, .length = 1},
  [119] = {.index = 227, .length = 4},
  [120] = {.index = 231, .length = 3},
  [121] = {.index = 234, .length = 2},
  [122] = {.index = 236, .length = 2},
  [123] = {.index = 35, .length = 1},
  [124] = {.index = 238, .length = 5},
  [125] = {.index = 243, .length = 4},
  [126] = {.index = 247, .length = 2},
  [127] = {.index = 249, .length = 2},
  [128] = {.index = 251, .length = 2},
  [129] = {.index = 253, .length = 5},
  [130] = {.index = 258, .length = 2},
  [131] = {.index = 260, .length = 3},
};

static const TSFieldMapEntry ts_field_map_entries[] = {
  [0] =
    {field_body, 0, .inherited = true},
    {field_declarator, 0, .inherited = true},
    {field_type, 0, .inherited = true},
  [3] =
    {field_type, 0},
  [4] =
    {field_directive, 0},
  [5] =
    {field_argument, 1},
    {field_operator, 0},
  [7] =
    {field_name, 0},
  [8] =
    {field_name, 1},
  [9] =
    {field_body, 1},
  [10] =
    {field_value, 1},
  [11] =
    {field_declarator, 0, .inherited = true},
    {field_parameters, 0, .inherited = true},
  [13] =
    {field_argument, 0},
    {field_operator, 1},
  [15] =
    {field_arguments, 1},
    {field_function, 0},
  [17] =
    {field_type, 1},
  [18] =
    {field_path, 1},
  [19] =
    {field_argument, 1},
    {field_directive, 0},
  [21] =
    {field_declarator, 1},
    {field_type, 0},
  [23] =
    {field_parameters, 0},
  [24] =
    {field_declarator, 0},
  [25] =
    {field_body, 2},
    {field_value, 1},
  [27] =
    {field_body, 2},
    {field_name, 1},
  [29] =
    {field_name, 2},
  [30] =
    {field_body, 2},
  [31] =
    {field_condition, 1},
    {field_consequence, 2},
  [33] =
    {field_body, 2},
    {field_condition, 1},
  [35] =
    {field_label, 1},
  [36] =
    {field_left, 0},
    {field_operator, 1},
    {field_right, 2},
  [39] =
    {field_label, 0},
  [40] =
    {field_declarator, 1},
  [41] =
    {field_body, 2},
    {field_declarator, 1},
    {field_type, 0, .inherited = true},
  [44] =
    {field_declarator, 0},
    {field_parameters, 1},
  [46] =
    {field_declarator, 1},
    {field_type, 0, .inherited = true},
  [48] =
    {field_body, 2},
    {field_declarator, 1},
    {field_declarator, 1, .inherited = true},
    {field_parameters, 1, .inherited = true},
    {field_type, 0, .inherited = true},
  [53] =
    {field_argument, 0},
    {field_field, 2},
    {field_operator, 1},
  [56] =
    {field_name, 1},
    {field_value, 2},
  [58] =
    {field_name, 1},
    {field_parameters, 2},
  [60] =
    {field_condition, 1},
  [61] =
    {field_alternative, 2},
    {field_name, 1},
  [63] =
    {field_type, 0, .inherited = true},
  [64] =
    {field_declarator, 2},
    {field_type, 0},
  [66] =
    {field_left, 0},
    {field_right, 2},
  [68] =
    {field_type, 1},
    {field_value, 3},
  [70] =
    {field_declarator, 2},
    {field_type, 1},
  [72] =
    {field_declarator, 2, .inherited = true},
    {field_type, 1, .inherited = true},
  [74] =
    {field_declarator, 0},
    {field_declarator, 1, .inherited = true},
  [76] =
    {field_name, 2},
    {field_prefix, 0},
  [78] =
    {field_name, 1},
    {field_underlying_type, 3},
  [80] =
    {field_body, 3},
    {field_name, 2},
  [82] =
    {field_name, 3},
  [83] =
    {field_body, 3},
  [84] =
    {field_alternative, 3},
    {field_condition, 1},
    {field_consequence, 2},
  [87] =
    {field_initializer, 0},
  [88] =
    {field_type, 2},
  [89] =
    {field_assembly_code, 2},
  [90] =
    {field_name, 0},
    {field_type, 2},
  [92] =
    {field_declarator, 2},
  [93] =
    {field_body, 3},
    {field_declarator, 2},
    {field_type, 0, .inherited = true},
  [96] =
    {field_declarator, 1},
    {field_declarator, 2},
    {field_type, 0, .inherited = true},
  [99] =
    {field_declarator, 0},
    {field_value, 2},
  [101] =
    {field_declarator, 1},
    {field_declarator, 2, .inherited = true},
    {field_type, 0, .inherited = true},
  [104] =
    {field_declarator, 0, .inherited = true},
    {field_declarator, 1, .inherited = true},
  [106] =
    {field_body, 3},
    {field_declarator, 1},
    {field_declarator, 1, .inherited = true},
    {field_parameters, 1, .inherited = true},
    {field_type, 0, .inherited = true},
  [111] =
    {field_body, 3},
    {field_declarator, 2},
    {field_type, 1, .inherited = true},
  [114] =
    {field_body, 3},
    {field_declarator, 2},
    {field_declarator, 2, .inherited = true},
    {field_parameters, 2, .inherited = true},
    {field_type, 1, .inherited = true},
  [119] =
    {field_argument, 0},
    {field_index, 2},
  [121] =
    {field_alternative, 3},
    {field_condition, 0},
  [123] =
    {field_name, 1},
    {field_parameters, 2},
    {field_value, 3},
  [126] =
    {field_alternative, 3},
    {field_condition, 1},
  [128] =
    {field_alternative, 3},
    {field_name, 1},
  [130] =
    {field_size, 1},
  [131] =
    {field_declarator, 3},
    {field_type, 1},
  [133] =
    {field_declarator, 3, .inherited = true},
    {field_type, 2, .inherited = true},
  [135] =
    {field_name, 0},
    {field_value, 2},
  [137] =
    {field_body, 4},
    {field_name, 1},
    {field_underlying_type, 3},
  [140] =
    {field_declarator, 1, .inherited = true},
    {field_type, 0, .inherited = true},
  [142] =
    {field_body, 4},
    {field_name, 3},
  [144] =
    {field_body, 1},
    {field_condition, 3},
  [146] =
    {field_update, 2},
  [147] =
    {field_initializer, 0},
    {field_update, 2},
  [149] =
    {field_condition, 1},
    {field_initializer, 0},
  [151] =
    {field_body, 4},
    {field_condition, 2, .inherited = true},
    {field_initializer, 2, .inherited = true},
    {field_update, 2, .inherited = true},
  [155] =
    {field_operand, 1},
  [156] =
    {field_assembly_code, 2},
    {field_output_operands, 3},
  [158] =
    {field_assembly_code, 3},
  [159] =
    {field_declarator, 3},
  [160] =
    {field_declarator, 1},
    {field_declarator, 2},
    {field_declarator, 3},
    {field_type, 0, .inherited = true},
  [164] =
    {field_declarator, 1},
    {field_declarator, 2},
    {field_declarator, 3, .inherited = true},
    {field_type, 0, .inherited = true},
  [168] =
    {field_declarator, 0},
    {field_size, 2},
  [170] =
    {field_declarator, 1},
    {field_declarator, 2},
  [172] =
    {field_body, 4},
    {field_declarator, 3},
    {field_type, 1, .inherited = true},
  [175] =
    {field_body, 4},
    {field_declarator, 2},
    {field_declarator, 2, .inherited = true},
    {field_parameters, 2, .inherited = true},
    {field_type, 1, .inherited = true},
  [180] =
    {field_alternative, 4},
    {field_condition, 0},
    {field_consequence, 2},
  [183] =
    {field_alternative, 4},
    {field_condition, 1},
  [185] =
    {field_size, 2},
  [186] =
    {field_body, 2},
    {field_filter, 1},
  [188] =
    {field_declarator, 0},
    {field_declarator, 2, .inherited = true},
  [190] =
    {field_condition, 1},
    {field_update, 3},
  [192] =
    {field_condition, 1},
    {field_initializer, 0},
    {field_update, 3},
  [195] =
    {field_initializer, 0},
    {field_update, 3},
  [197] =
    {field_condition, 2},
    {field_initializer, 0},
  [199] =
    {field_member, 4},
    {field_type, 2},
  [201] =
    {field_operand, 1},
    {field_operand, 2, .inherited = true},
  [203] =
    {field_assembly_code, 2},
    {field_input_operands, 4},
    {field_output_operands, 3},
  [206] =
    {field_assembly_code, 3},
    {field_output_operands, 4},
  [208] =
    {field_declarator, 4},
  [209] =
    {field_declarator, 1},
    {field_declarator, 2},
    {field_declarator, 3},
    {field_declarator, 4, .inherited = true},
    {field_type, 0, .inherited = true},
  [214] =
    {field_declarator, 0},
    {field_size, 3},
  [216] =
    {field_declarator, 1},
    {field_declarator, 2},
    {field_declarator, 3},
  [219] =
    {field_designator, 0},
    {field_value, 2},
  [221] =
    {field_condition, 2},
    {field_initializer, 0},
    {field_update, 4},
  [224] =
    {field_operand, 0, .inherited = true},
    {field_operand, 1, .inherited = true},
  [226] =
    {field_register, 1},
  [227] =
    {field_assembly_code, 2},
    {field_clobbers, 5},
    {field_input_operands, 4},
    {field_output_operands, 3},
  [231] =
    {field_assembly_code, 3},
    {field_input_operands, 5},
    {field_output_operands, 4},
  [234] =
    {field_constraint, 0},
    {field_value, 2},
  [236] =
    {field_register, 1},
    {field_register, 2, .inherited = true},
  [238] =
    {field_assembly_code, 2},
    {field_clobbers, 5},
    {field_goto_labels, 6},
    {field_input_operands, 4},
    {field_output_operands, 3},
  [243] =
    {field_assembly_code, 3},
    {field_clobbers, 6},
    {field_input_operands, 5},
    {field_output_operands, 4},
  [247] =
    {field_end, 3},
    {field_start, 1},
  [249] =
    {field_register, 0, .inherited = true},
    {field_register, 1, .inherited = true},
  [251] =
    {field_label, 1},
    {field_label, 2, .inherited = true},
  [253] =
    {field_assembly_code, 3},
    {field_clobbers, 6},
    {field_goto_labels, 7},
    {field_input_operands, 5},
    {field_output_operands, 4},
  [258] =
    {field_label, 0, .inherited = true},
    {field_label, 1, .inherited = true},
  [260] =
    {field_constraint, 3},
    {field_symbol, 1},
    {field_value, 5},
};

static const TSSymbol ts_alias_sequences[PRODUCTION_ID_COUNT][MAX_ALIAS_SEQUENCE_LENGTH] = {
  [0] = {0},
  [1] = {
    [0] = alias_sym_type_identifier,
  },
  [7] = {
    [1] = alias_sym_type_identifier,
  },
  [10] = {
    [0] = alias_sym_type_identifier,
  },
  [15] = {
    [1] = alias_sym_type_identifier,
  },
  [21] = {
    [0] = sym_primitive_type,
  },
  [24] = {
    [1] = alias_sym_type_identifier,
  },
  [25] = {
    [2] = alias_sym_type_identifier,
  },
  [29] = {
    [1] = alias_sym_statement_identifier,
  },
  [31] = {
    [0] = alias_sym_statement_identifier,
  },
  [37] = {
    [2] = alias_sym_field_identifier,
  },
  [50] = {
    [1] = alias_sym_type_identifier,
  },
  [51] = {
    [0] = alias_sym_field_identifier,
  },
  [52] = {
    [2] = alias_sym_type_identifier,
  },
  [53] = {
    [3] = alias_sym_type_identifier,
  },
  [78] = {
    [1] = alias_sym_type_identifier,
  },
  [80] = {
    [3] = alias_sym_type_identifier,
  },
  [99] = {
    [1] = alias_sym_field_identifier,
  },
  [106] = {
    [4] = alias_sym_field_identifier,
  },
  [114] = {
    [0] = alias_sym_field_identifier,
  },
};

static const uint16_t ts_non_terminal_alias_map[] = {
  0,
};

static const TSStateId ts_primary_state_ids[STATE_COUNT] = {
  [0] = 0,
  [1] = 1,
  [2] = 2,
  [3] = 3,
  [4] = 4,
  [5] = 5,
  [6] = 5,
  [7] = 7,
  [8] = 5,
  [9] = 5,
  [10] = 10,
  [11] = 3,
  [12] = 10,
  [13] = 3,
  [14] = 2,
  [15] = 10,
  [16] = 3,
  [17] = 2,
  [18] = 18,
  [19] = 2,
  [20] = 10,
  [21] = 21,
  [22] = 22,
  [23] = 23,
  [24] = 24,
  [25] = 22,
  [26] = 23,
  [27] = 27,
  [28] = 28,
  [29] = 29,
  [30] = 28,
  [31] = 23,
  [32] = 28,
  [33] = 33,
  [34] = 28,
  [35] = 29,
  [36] = 29,
  [37] = 23,
  [38] = 33,
  [39] = 22,
  [40] = 33,
  [41] = 33,
  [42] = 29,
  [43] = 43,
  [44] = 44,
  [45] = 45,
  [46] = 46,
  [47] = 47,
  [48] = 48,
  [49] = 49,
  [50] = 48,
  [51] = 47,
  [52] = 46,
  [53] = 48,
  [54] = 47,
  [55] = 45,
  [56] = 45,
  [57] = 46,
  [58] = 48,
  [59] = 49,
  [60] = 47,
  [61] = 45,
  [62] = 46,
  [63] = 49,
  [64] = 49,
  [65] = 48,
  [66] = 46,
  [67] = 47,
  [68] = 45,
  [69] = 49,
  [70] = 70,
  [71] = 70,
  [72] = 70,
  [73] = 70,
  [74] = 70,
  [75] = 75,
  [76] = 76,
  [77] = 77,
  [78] = 78,
  [79] = 79,
  [80] = 80,
  [81] = 81,
  [82] = 82,
  [83] = 83,
  [84] = 84,
  [85] = 85,
  [86] = 86,
  [87] = 87,
  [88] = 88,
  [89] = 89,
  [90] = 90,
  [91] = 91,
  [92] = 80,
  [93] = 93,
  [94] = 94,
  [95] = 95,
  [96] = 96,
  [97] = 97,
  [98] = 98,
  [99] = 99,
  [100] = 100,
  [101] = 101,
  [102] = 102,
  [103] = 103,
  [104] = 104,
  [105] = 105,
  [106] = 106,
  [107] = 107,
  [108] = 108,
  [109] = 109,
  [110] = 110,
  [111] = 104,
  [112] = 112,
  [113] = 113,
  [114] = 114,
  [115] = 115,
  [116] = 116,
  [117] = 117,
  [118] = 118,
  [119] = 119,
  [120] = 120,
  [121] = 121,
  [122] = 122,
  [123] = 123,
  [124] = 124,
  [125] = 125,
  [126] = 126,
  [127] = 127,
  [128] = 128,
  [129] = 129,
  [130] = 130,
  [131] = 131,
  [132] = 132,
  [133] = 133,
  [134] = 134,
  [135] = 135,
  [136] = 136,
  [137] = 137,
  [138] = 138,
  [139] = 139,
  [140] = 140,
  [141] = 141,
  [142] = 142,
  [143] = 143,
  [144] = 144,
  [145] = 145,
  [146] = 146,
  [147] = 147,
  [148] = 148,
  [149] = 149,
  [150] = 150,
  [151] = 151,
  [152] = 152,
  [153] = 153,
  [154] = 154,
  [155] = 155,
  [156] = 75,
  [157] = 157,
  [158] = 75,
  [159] = 157,
  [160] = 75,
  [161] = 78,
  [162] = 108,
  [163] = 126,
  [164] = 127,
  [165] = 125,
  [166] = 124,
  [167] = 123,
  [168] = 122,
  [169] = 121,
  [170] = 76,
  [171] = 76,
  [172] = 106,
  [173] = 114,
  [174] = 113,
  [175] = 114,
  [176] = 118,
  [177] = 102,
  [178] = 100,
  [179] = 112,
  [180] = 113,
  [181] = 121,
  [182] = 122,
  [183] = 123,
  [184] = 124,
  [185] = 125,
  [186] = 127,
  [187] = 126,
  [188] = 119,
  [189] = 93,
  [190] = 87,
  [191] = 86,
  [192] = 120,
  [193] = 84,
  [194] = 95,
  [195] = 119,
  [196] = 118,
  [197] = 117,
  [198] = 116,
  [199] = 110,
  [200] = 96,
  [201] = 97,
  [202] = 98,
  [203] = 99,
  [204] = 117,
  [205] = 109,
  [206] = 108,
  [207] = 116,
  [208] = 101,
  [209] = 110,
  [210] = 104,
  [211] = 109,
  [212] = 107,
  [213] = 77,
  [214] = 105,
  [215] = 103,
  [216] = 115,
  [217] = 94,
  [218] = 93,
  [219] = 91,
  [220] = 87,
  [221] = 112,
  [222] = 90,
  [223] = 88,
  [224] = 83,
  [225] = 82,
  [226] = 81,
  [227] = 108,
  [228] = 80,
  [229] = 79,
  [230] = 107,
  [231] = 77,
  [232] = 103,
  [233] = 85,
  [234] = 79,
  [235] = 80,
  [236] = 90,
  [237] = 81,
  [238] = 88,
  [239] = 82,
  [240] = 83,
  [241] = 112,
  [242] = 86,
  [243] = 115,
  [244] = 105,
  [245] = 104,
  [246] = 101,
  [247] = 89,
  [248] = 115,
  [249] = 105,
  [250] = 99,
  [251] = 95,
  [252] = 100,
  [253] = 102,
  [254] = 106,
  [255] = 94,
  [256] = 84,
  [257] = 91,
  [258] = 85,
  [259] = 98,
  [260] = 97,
  [261] = 83,
  [262] = 82,
  [263] = 96,
  [264] = 89,
  [265] = 81,
  [266] = 78,
  [267] = 84,
  [268] = 86,
  [269] = 87,
  [270] = 95,
  [271] = 100,
  [272] = 102,
  [273] = 106,
  [274] = 113,
  [275] = 114,
  [276] = 76,
  [277] = 121,
  [278] = 122,
  [279] = 123,
  [280] = 124,
  [281] = 101,
  [282] = 125,
  [283] = 127,
  [284] = 79,
  [285] = 99,
  [286] = 126,
  [287] = 120,
  [288] = 119,
  [289] = 118,
  [290] = 117,
  [291] = 116,
  [292] = 98,
  [293] = 97,
  [294] = 110,
  [295] = 109,
  [296] = 120,
  [297] = 107,
  [298] = 77,
  [299] = 103,
  [300] = 94,
  [301] = 93,
  [302] = 91,
  [303] = 90,
  [304] = 88,
  [305] = 85,
  [306] = 78,
  [307] = 89,
  [308] = 96,
  [309] = 145,
  [310] = 141,
  [311] = 311,
  [312] = 312,
  [313] = 313,
  [314] = 314,
  [315] = 315,
  [316] = 316,
  [317] = 315,
  [318] = 318,
  [319] = 133,
  [320] = 320,
  [321] = 134,
  [322] = 135,
  [323] = 316,
  [324] = 136,
  [325] = 138,
  [326] = 139,
  [327] = 142,
  [328] = 128,
  [329] = 151,
  [330] = 152,
  [331] = 153,
  [332] = 155,
  [333] = 140,
  [334] = 152,
  [335] = 314,
  [336] = 149,
  [337] = 150,
  [338] = 315,
  [339] = 314,
  [340] = 137,
  [341] = 132,
  [342] = 151,
  [343] = 153,
  [344] = 318,
  [345] = 148,
  [346] = 147,
  [347] = 145,
  [348] = 130,
  [349] = 316,
  [350] = 313,
  [351] = 312,
  [352] = 311,
  [353] = 146,
  [354] = 320,
  [355] = 318,
  [356] = 129,
  [357] = 320,
  [358] = 314,
  [359] = 131,
  [360] = 148,
  [361] = 311,
  [362] = 312,
  [363] = 318,
  [364] = 142,
  [365] = 144,
  [366] = 313,
  [367] = 150,
  [368] = 129,
  [369] = 131,
  [370] = 143,
  [371] = 316,
  [372] = 146,
  [373] = 316,
  [374] = 133,
  [375] = 144,
  [376] = 130,
  [377] = 134,
  [378] = 135,
  [379] = 136,
  [380] = 137,
  [381] = 138,
  [382] = 139,
  [383] = 147,
  [384] = 313,
  [385] = 132,
  [386] = 313,
  [387] = 320,
  [388] = 141,
  [389] = 312,
  [390] = 155,
  [391] = 311,
  [392] = 128,
  [393] = 154,
  [394] = 320,
  [395] = 143,
  [396] = 312,
  [397] = 311,
  [398] = 140,
  [399] = 318,
  [400] = 314,
  [401] = 149,
  [402] = 154,
  [403] = 315,
  [404] = 131,
  [405] = 142,
  [406] = 157,
  [407] = 148,
  [408] = 155,
  [409] = 134,
  [410] = 135,
  [411] = 138,
  [412] = 132,
  [413] = 139,
  [414] = 136,
  [415] = 415,
  [416] = 416,
  [417] = 141,
  [418] = 137,
  [419] = 149,
  [420] = 146,
  [421] = 152,
  [422] = 143,
  [423] = 133,
  [424] = 154,
  [425] = 144,
  [426] = 147,
  [427] = 145,
  [428] = 151,
  [429] = 130,
  [430] = 129,
  [431] = 140,
  [432] = 153,
  [433] = 128,
  [434] = 434,
  [435] = 435,
  [436] = 435,
  [437] = 435,
  [438] = 434,
  [439] = 435,
  [440] = 435,
  [441] = 434,
  [442] = 434,
  [443] = 434,
  [444] = 435,
  [445] = 434,
  [446] = 446,
  [447] = 447,
  [448] = 157,
  [449] = 157,
  [450] = 157,
  [451] = 75,
  [452] = 452,
  [453] = 452,
  [454] = 452,
  [455] = 452,
  [456] = 452,
  [457] = 452,
  [458] = 458,
  [459] = 452,
  [460] = 452,
  [461] = 461,
  [462] = 452,
  [463] = 463,
  [464] = 464,
  [465] = 465,
  [466] = 466,
  [467] = 467,
  [468] = 468,
  [469] = 469,
  [470] = 470,
  [471] = 471,
  [472] = 472,
  [473] = 473,
  [474] = 474,
  [475] = 475,
  [476] = 476,
  [477] = 477,
  [478] = 478,
  [479] = 479,
  [480] = 480,
  [481] = 481,
  [482] = 482,
  [483] = 483,
  [484] = 480,
  [485] = 485,
  [486] = 486,
  [487] = 480,
  [488] = 485,
  [489] = 478,
  [490] = 478,
  [491] = 485,
  [492] = 486,
  [493] = 486,
  [494] = 494,
  [495] = 495,
  [496] = 496,
  [497] = 496,
  [498] = 498,
  [499] = 499,
  [500] = 500,
  [501] = 501,
  [502] = 502,
  [503] = 503,
  [504] = 500,
  [505] = 505,
  [506] = 500,
  [507] = 503,
  [508] = 501,
  [509] = 501,
  [510] = 503,
  [511] = 501,
  [512] = 512,
  [513] = 503,
  [514] = 501,
  [515] = 515,
  [516] = 516,
  [517] = 517,
  [518] = 518,
  [519] = 501,
  [520] = 520,
  [521] = 500,
  [522] = 503,
  [523] = 523,
  [524] = 503,
  [525] = 525,
  [526] = 526,
  [527] = 527,
  [528] = 495,
  [529] = 529,
  [530] = 530,
  [531] = 531,
  [532] = 532,
  [533] = 495,
  [534] = 534,
  [535] = 535,
  [536] = 536,
  [537] = 537,
  [538] = 538,
  [539] = 539,
  [540] = 540,
  [541] = 532,
  [542] = 542,
  [543] = 543,
  [544] = 544,
  [545] = 545,
  [546] = 546,
  [547] = 547,
  [548] = 548,
  [549] = 549,
  [550] = 550,
  [551] = 547,
  [552] = 552,
  [553] = 553,
  [554] = 554,
  [555] = 555,
  [556] = 556,
  [557] = 557,
  [558] = 558,
  [559] = 559,
  [560] = 560,
  [561] = 561,
  [562] = 562,
  [563] = 563,
  [564] = 561,
  [565] = 565,
  [566] = 566,
  [567] = 563,
  [568] = 568,
  [569] = 566,
  [570] = 570,
  [571] = 571,
  [572] = 560,
  [573] = 554,
  [574] = 548,
  [575] = 561,
  [576] = 549,
  [577] = 550,
  [578] = 562,
  [579] = 565,
  [580] = 547,
  [581] = 566,
  [582] = 563,
  [583] = 552,
  [584] = 553,
  [585] = 555,
  [586] = 586,
  [587] = 548,
  [588] = 588,
  [589] = 586,
  [590] = 549,
  [591] = 550,
  [592] = 552,
  [593] = 553,
  [594] = 555,
  [595] = 556,
  [596] = 586,
  [597] = 557,
  [598] = 558,
  [599] = 599,
  [600] = 556,
  [601] = 557,
  [602] = 562,
  [603] = 565,
  [604] = 558,
  [605] = 563,
  [606] = 561,
  [607] = 560,
  [608] = 558,
  [609] = 557,
  [610] = 556,
  [611] = 555,
  [612] = 553,
  [613] = 552,
  [614] = 547,
  [615] = 550,
  [616] = 548,
  [617] = 561,
  [618] = 618,
  [619] = 560,
  [620] = 561,
  [621] = 560,
  [622] = 599,
  [623] = 558,
  [624] = 624,
  [625] = 625,
  [626] = 557,
  [627] = 560,
  [628] = 556,
  [629] = 555,
  [630] = 553,
  [631] = 552,
  [632] = 547,
  [633] = 550,
  [634] = 549,
  [635] = 548,
  [636] = 554,
  [637] = 563,
  [638] = 565,
  [639] = 562,
  [640] = 546,
  [641] = 548,
  [642] = 642,
  [643] = 549,
  [644] = 550,
  [645] = 547,
  [646] = 552,
  [647] = 553,
  [648] = 555,
  [649] = 556,
  [650] = 557,
  [651] = 558,
  [652] = 652,
  [653] = 549,
  [654] = 563,
  [655] = 586,
  [656] = 586,
  [657] = 565,
  [658] = 562,
  [659] = 562,
  [660] = 565,
  [661] = 461,
  [662] = 662,
  [663] = 663,
  [664] = 663,
  [665] = 665,
  [666] = 666,
  [667] = 663,
  [668] = 663,
  [669] = 669,
  [670] = 670,
  [671] = 671,
  [672] = 672,
  [673] = 673,
  [674] = 674,
  [675] = 675,
  [676] = 676,
  [677] = 677,
  [678] = 678,
  [679] = 679,
  [680] = 680,
  [681] = 681,
  [682] = 682,
  [683] = 683,
  [684] = 684,
  [685] = 684,
  [686] = 447,
  [687] = 446,
  [688] = 688,
  [689] = 689,
  [690] = 690,
  [691] = 691,
  [692] = 692,
  [693] = 690,
  [694] = 688,
  [695] = 695,
  [696] = 696,
  [697] = 697,
  [698] = 688,
  [699] = 690,
  [700] = 700,
  [701] = 691,
  [702] = 702,
  [703] = 695,
  [704] = 704,
  [705] = 705,
  [706] = 706,
  [707] = 695,
  [708] = 691,
  [709] = 695,
  [710] = 688,
  [711] = 711,
  [712] = 691,
  [713] = 690,
  [714] = 714,
  [715] = 715,
  [716] = 716,
  [717] = 717,
  [718] = 718,
  [719] = 719,
  [720] = 720,
  [721] = 721,
  [722] = 722,
  [723] = 723,
  [724] = 724,
  [725] = 725,
  [726] = 726,
  [727] = 727,
  [728] = 669,
  [729] = 729,
  [730] = 730,
  [731] = 731,
  [732] = 732,
  [733] = 733,
  [734] = 734,
  [735] = 683,
  [736] = 736,
  [737] = 737,
  [738] = 669,
  [739] = 739,
  [740] = 740,
  [741] = 741,
  [742] = 741,
  [743] = 741,
  [744] = 740,
  [745] = 741,
  [746] = 740,
  [747] = 740,
  [748] = 748,
  [749] = 749,
  [750] = 750,
  [751] = 751,
  [752] = 752,
  [753] = 748,
  [754] = 754,
  [755] = 750,
  [756] = 756,
  [757] = 757,
  [758] = 748,
  [759] = 759,
  [760] = 748,
  [761] = 748,
  [762] = 762,
  [763] = 763,
  [764] = 669,
  [765] = 765,
  [766] = 766,
  [767] = 767,
  [768] = 768,
  [769] = 769,
  [770] = 770,
  [771] = 771,
  [772] = 772,
  [773] = 446,
  [774] = 774,
  [775] = 447,
  [776] = 776,
  [777] = 777,
  [778] = 778,
  [779] = 779,
  [780] = 767,
  [781] = 781,
  [782] = 782,
  [783] = 783,
  [784] = 784,
  [785] = 785,
  [786] = 786,
  [787] = 787,
  [788] = 788,
  [789] = 763,
  [790] = 770,
  [791] = 791,
  [792] = 769,
  [793] = 777,
  [794] = 794,
  [795] = 795,
  [796] = 768,
  [797] = 774,
  [798] = 766,
  [799] = 799,
  [800] = 778,
  [801] = 772,
  [802] = 802,
  [803] = 771,
  [804] = 804,
  [805] = 765,
  [806] = 806,
  [807] = 807,
  [808] = 808,
  [809] = 809,
  [810] = 810,
  [811] = 811,
  [812] = 812,
  [813] = 813,
  [814] = 814,
  [815] = 815,
  [816] = 816,
  [817] = 817,
  [818] = 818,
  [819] = 819,
  [820] = 820,
  [821] = 821,
  [822] = 822,
  [823] = 823,
  [824] = 824,
  [825] = 825,
  [826] = 826,
  [827] = 827,
  [828] = 828,
  [829] = 829,
  [830] = 830,
  [831] = 831,
  [832] = 832,
  [833] = 833,
  [834] = 834,
  [835] = 835,
  [836] = 836,
  [837] = 837,
  [838] = 838,
  [839] = 839,
  [840] = 799,
  [841] = 841,
  [842] = 842,
  [843] = 843,
  [844] = 844,
  [845] = 845,
  [846] = 144,
  [847] = 847,
  [848] = 848,
  [849] = 137,
  [850] = 154,
  [851] = 143,
  [852] = 852,
  [853] = 853,
  [854] = 854,
  [855] = 855,
  [856] = 856,
  [857] = 131,
  [858] = 142,
  [859] = 859,
  [860] = 860,
  [861] = 861,
  [862] = 862,
  [863] = 863,
  [864] = 864,
  [865] = 863,
  [866] = 863,
  [867] = 863,
  [868] = 727,
  [869] = 722,
  [870] = 772,
  [871] = 771,
  [872] = 872,
  [873] = 765,
  [874] = 766,
  [875] = 767,
  [876] = 768,
  [877] = 769,
  [878] = 770,
  [879] = 763,
  [880] = 778,
  [881] = 777,
  [882] = 774,
  [883] = 883,
  [884] = 884,
  [885] = 736,
  [886] = 733,
  [887] = 723,
  [888] = 888,
  [889] = 889,
  [890] = 737,
  [891] = 732,
  [892] = 671,
  [893] = 670,
  [894] = 726,
  [895] = 799,
  [896] = 724,
  [897] = 729,
  [898] = 861,
  [899] = 862,
  [900] = 154,
  [901] = 855,
  [902] = 902,
  [903] = 860,
  [904] = 845,
  [905] = 137,
  [906] = 848,
  [907] = 852,
  [908] = 855,
  [909] = 909,
  [910] = 853,
  [911] = 847,
  [912] = 859,
  [913] = 862,
  [914] = 854,
  [915] = 143,
  [916] = 847,
  [917] = 845,
  [918] = 861,
  [919] = 142,
  [920] = 852,
  [921] = 859,
  [922] = 922,
  [923] = 853,
  [924] = 856,
  [925] = 848,
  [926] = 854,
  [927] = 142,
  [928] = 928,
  [929] = 902,
  [930] = 902,
  [931] = 856,
  [932] = 902,
  [933] = 144,
  [934] = 131,
  [935] = 154,
  [936] = 137,
  [937] = 143,
  [938] = 131,
  [939] = 860,
  [940] = 144,
  [941] = 94,
  [942] = 102,
  [943] = 106,
  [944] = 89,
  [945] = 95,
  [946] = 78,
  [947] = 85,
  [948] = 91,
  [949] = 93,
  [950] = 100,
  [951] = 951,
  [952] = 952,
  [953] = 953,
  [954] = 954,
  [955] = 955,
  [956] = 956,
  [957] = 957,
  [958] = 958,
  [959] = 769,
  [960] = 778,
  [961] = 763,
  [962] = 774,
  [963] = 963,
  [964] = 770,
  [965] = 777,
  [966] = 768,
  [967] = 767,
  [968] = 766,
  [969] = 765,
  [970] = 771,
  [971] = 772,
  [972] = 771,
  [973] = 766,
  [974] = 768,
  [975] = 975,
  [976] = 769,
  [977] = 770,
  [978] = 978,
  [979] = 975,
  [980] = 763,
  [981] = 975,
  [982] = 106,
  [983] = 102,
  [984] = 94,
  [985] = 767,
  [986] = 100,
  [987] = 772,
  [988] = 95,
  [989] = 989,
  [990] = 765,
  [991] = 78,
  [992] = 777,
  [993] = 93,
  [994] = 89,
  [995] = 995,
  [996] = 91,
  [997] = 85,
  [998] = 778,
  [999] = 975,
  [1000] = 774,
  [1001] = 1001,
  [1002] = 1002,
  [1003] = 1003,
  [1004] = 1004,
  [1005] = 1005,
  [1006] = 1006,
  [1007] = 1007,
  [1008] = 1008,
  [1009] = 1009,
  [1010] = 1010,
  [1011] = 1011,
  [1012] = 1012,
  [1013] = 1013,
  [1014] = 1014,
  [1015] = 1015,
  [1016] = 1016,
  [1017] = 1017,
  [1018] = 1018,
  [1019] = 1019,
  [1020] = 1020,
  [1021] = 1019,
  [1022] = 1022,
  [1023] = 1022,
  [1024] = 1024,
  [1025] = 1025,
  [1026] = 1026,
  [1027] = 1027,
  [1028] = 1028,
  [1029] = 1029,
  [1030] = 768,
  [1031] = 1031,
  [1032] = 1032,
  [1033] = 1033,
  [1034] = 1034,
  [1035] = 1035,
  [1036] = 767,
  [1037] = 1037,
  [1038] = 1038,
  [1039] = 1039,
  [1040] = 769,
  [1041] = 1034,
  [1042] = 1031,
  [1043] = 1029,
  [1044] = 1044,
  [1045] = 766,
  [1046] = 1031,
  [1047] = 1047,
  [1048] = 765,
  [1049] = 1049,
  [1050] = 774,
  [1051] = 771,
  [1052] = 1035,
  [1053] = 770,
  [1054] = 1031,
  [1055] = 1055,
  [1056] = 1056,
  [1057] = 1057,
  [1058] = 1058,
  [1059] = 777,
  [1060] = 1029,
  [1061] = 763,
  [1062] = 1062,
  [1063] = 772,
  [1064] = 1064,
  [1065] = 1033,
  [1066] = 778,
  [1067] = 1067,
  [1068] = 1068,
  [1069] = 1034,
  [1070] = 1070,
  [1071] = 1035,
  [1072] = 1072,
  [1073] = 1029,
  [1074] = 963,
  [1075] = 1031,
  [1076] = 1076,
  [1077] = 1035,
  [1078] = 1031,
  [1079] = 1079,
  [1080] = 1033,
  [1081] = 1034,
  [1082] = 1082,
  [1083] = 1068,
  [1084] = 1084,
  [1085] = 1033,
  [1086] = 1086,
  [1087] = 1087,
  [1088] = 1088,
  [1089] = 1089,
  [1090] = 1090,
  [1091] = 1091,
  [1092] = 1092,
  [1093] = 1093,
  [1094] = 1094,
  [1095] = 1095,
  [1096] = 1087,
  [1097] = 1087,
  [1098] = 1087,
  [1099] = 1099,
  [1100] = 1100,
  [1101] = 1101,
  [1102] = 1102,
  [1103] = 1103,
  [1104] = 1104,
  [1105] = 1105,
  [1106] = 1106,
  [1107] = 1102,
  [1108] = 1108,
  [1109] = 1109,
  [1110] = 1110,
  [1111] = 1087,
  [1112] = 1112,
  [1113] = 739,
  [1114] = 1114,
  [1115] = 1115,
  [1116] = 864,
  [1117] = 1117,
  [1118] = 883,
  [1119] = 909,
  [1120] = 922,
  [1121] = 928,
  [1122] = 1122,
  [1123] = 1123,
  [1124] = 1124,
  [1125] = 1125,
  [1126] = 1126,
  [1127] = 1127,
  [1128] = 1128,
  [1129] = 1129,
  [1130] = 1130,
  [1131] = 1131,
  [1132] = 1129,
  [1133] = 835,
  [1134] = 1129,
  [1135] = 1129,
  [1136] = 1136,
  [1137] = 1137,
  [1138] = 1138,
  [1139] = 843,
  [1140] = 1140,
  [1141] = 1141,
  [1142] = 1142,
  [1143] = 1143,
  [1144] = 1142,
  [1145] = 1142,
  [1146] = 1146,
  [1147] = 1142,
  [1148] = 1148,
  [1149] = 1149,
  [1150] = 1148,
  [1151] = 1140,
  [1152] = 1146,
  [1153] = 1142,
  [1154] = 1143,
  [1155] = 1143,
  [1156] = 1143,
  [1157] = 1148,
  [1158] = 1142,
  [1159] = 1146,
  [1160] = 1160,
  [1161] = 1161,
  [1162] = 1162,
  [1163] = 1163,
  [1164] = 1164,
  [1165] = 1165,
  [1166] = 1166,
  [1167] = 1167,
  [1168] = 1168,
  [1169] = 1169,
  [1170] = 1170,
  [1171] = 1171,
  [1172] = 1172,
  [1173] = 1173,
  [1174] = 1174,
  [1175] = 1175,
  [1176] = 1176,
  [1177] = 1177,
  [1178] = 1178,
  [1179] = 1179,
  [1180] = 1180,
  [1181] = 1181,
  [1182] = 1182,
  [1183] = 1183,
  [1184] = 1184,
  [1185] = 1185,
  [1186] = 1186,
  [1187] = 1187,
  [1188] = 1188,
  [1189] = 1189,
  [1190] = 1173,
  [1191] = 1191,
  [1192] = 1192,
  [1193] = 1193,
  [1194] = 1179,
  [1195] = 1191,
  [1196] = 1196,
  [1197] = 717,
  [1198] = 1198,
  [1199] = 1199,
  [1200] = 1193,
  [1201] = 1161,
  [1202] = 1193,
  [1203] = 1172,
  [1204] = 1171,
  [1205] = 1205,
  [1206] = 1161,
  [1207] = 1207,
  [1208] = 1186,
  [1209] = 1185,
  [1210] = 1183,
  [1211] = 1211,
  [1212] = 1182,
  [1213] = 1181,
  [1214] = 1178,
  [1215] = 1215,
  [1216] = 1216,
  [1217] = 1177,
  [1218] = 1218,
  [1219] = 1193,
  [1220] = 1211,
  [1221] = 1168,
  [1222] = 717,
  [1223] = 843,
  [1224] = 1008,
  [1225] = 1225,
  [1226] = 1226,
  [1227] = 1175,
  [1228] = 1226,
  [1229] = 1226,
  [1230] = 1230,
  [1231] = 1231,
  [1232] = 1232,
  [1233] = 1233,
  [1234] = 1225,
  [1235] = 841,
  [1236] = 1180,
  [1237] = 1237,
  [1238] = 1162,
  [1239] = 831,
  [1240] = 1232,
  [1241] = 1017,
  [1242] = 1242,
  [1243] = 1243,
  [1244] = 1163,
  [1245] = 1226,
  [1246] = 1207,
  [1247] = 1164,
  [1248] = 1170,
  [1249] = 811,
  [1250] = 1225,
  [1251] = 1012,
  [1252] = 1169,
  [1253] = 812,
  [1254] = 1237,
  [1255] = 1232,
  [1256] = 1256,
  [1257] = 1166,
  [1258] = 813,
  [1259] = 1192,
  [1260] = 834,
  [1261] = 835,
  [1262] = 1225,
  [1263] = 816,
  [1264] = 1165,
  [1265] = 818,
  [1266] = 1174,
  [1267] = 1267,
  [1268] = 1014,
  [1269] = 1269,
  [1270] = 1270,
  [1271] = 1270,
  [1272] = 1270,
  [1273] = 1270,
  [1274] = 1274,
  [1275] = 1275,
  [1276] = 1276,
  [1277] = 1277,
  [1278] = 1276,
  [1279] = 1276,
  [1280] = 1276,
  [1281] = 864,
  [1282] = 1277,
  [1283] = 1277,
  [1284] = 1277,
  [1285] = 1285,
  [1286] = 1286,
  [1287] = 1287,
  [1288] = 1288,
  [1289] = 1289,
  [1290] = 1290,
  [1291] = 1291,
  [1292] = 1292,
  [1293] = 1293,
  [1294] = 1294,
  [1295] = 1289,
  [1296] = 1296,
  [1297] = 1297,
  [1298] = 1298,
  [1299] = 1289,
  [1300] = 1300,
  [1301] = 1301,
  [1302] = 1302,
  [1303] = 1303,
  [1304] = 1304,
  [1305] = 1304,
  [1306] = 1306,
  [1307] = 1307,
  [1308] = 1304,
  [1309] = 1309,
  [1310] = 1310,
  [1311] = 1309,
  [1312] = 1312,
  [1313] = 1304,
  [1314] = 1304,
  [1315] = 1309,
  [1316] = 1316,
  [1317] = 1317,
  [1318] = 1318,
  [1319] = 1309,
  [1320] = 1304,
  [1321] = 1321,
  [1322] = 1322,
  [1323] = 1323,
  [1324] = 1324,
  [1325] = 1325,
  [1326] = 1326,
  [1327] = 1327,
  [1328] = 1328,
  [1329] = 1329,
  [1330] = 1330,
  [1331] = 1331,
  [1332] = 1332,
  [1333] = 1333,
  [1334] = 1334,
  [1335] = 1335,
  [1336] = 1336,
  [1337] = 1337,
  [1338] = 1338,
  [1339] = 1339,
  [1340] = 1340,
  [1341] = 1327,
  [1342] = 1342,
  [1343] = 1343,
  [1344] = 1344,
  [1345] = 1345,
  [1346] = 1346,
  [1347] = 1327,
  [1348] = 1348,
  [1349] = 1349,
  [1350] = 1350,
  [1351] = 1351,
  [1352] = 1352,
  [1353] = 1327,
  [1354] = 1354,
  [1355] = 1355,
  [1356] = 1356,
  [1357] = 1357,
  [1358] = 1357,
  [1359] = 1359,
  [1360] = 1360,
  [1361] = 1361,
  [1362] = 1357,
  [1363] = 1363,
  [1364] = 1364,
  [1365] = 1365,
  [1366] = 1366,
  [1367] = 1367,
  [1368] = 1368,
  [1369] = 1357,
  [1370] = 1370,
  [1371] = 1371,
  [1372] = 1372,
  [1373] = 1373,
  [1374] = 1374,
  [1375] = 1373,
  [1376] = 1376,
  [1377] = 1373,
  [1378] = 1376,
  [1379] = 1373,
  [1380] = 1380,
  [1381] = 1381,
  [1382] = 1382,
  [1383] = 1376,
  [1384] = 1381,
  [1385] = 1385,
  [1386] = 1386,
  [1387] = 1387,
  [1388] = 1376,
  [1389] = 1381,
  [1390] = 1390,
  [1391] = 1381,
  [1392] = 1392,
  [1393] = 1393,
  [1394] = 1394,
  [1395] = 1395,
  [1396] = 1396,
  [1397] = 1397,
  [1398] = 1398,
  [1399] = 1399,
  [1400] = 1400,
  [1401] = 1397,
  [1402] = 1402,
  [1403] = 1396,
  [1404] = 1396,
  [1405] = 1397,
  [1406] = 1402,
  [1407] = 1396,
  [1408] = 1408,
  [1409] = 1409,
  [1410] = 1410,
  [1411] = 1397,
  [1412] = 1412,
  [1413] = 1413,
  [1414] = 1402,
  [1415] = 1415,
  [1416] = 1416,
  [1417] = 1402,
  [1418] = 1418,
  [1419] = 1419,
  [1420] = 1420,
  [1421] = 1421,
  [1422] = 1422,
  [1423] = 1423,
  [1424] = 1424,
  [1425] = 1425,
  [1426] = 1426,
  [1427] = 1427,
  [1428] = 1428,
  [1429] = 1429,
  [1430] = 1430,
  [1431] = 1431,
  [1432] = 1432,
  [1433] = 1433,
  [1434] = 1434,
  [1435] = 1435,
  [1436] = 1436,
  [1437] = 1437,
  [1438] = 1438,
  [1439] = 1439,
  [1440] = 1440,
  [1441] = 1441,
  [1442] = 1442,
  [1443] = 1443,
  [1444] = 1444,
  [1445] = 1445,
  [1446] = 1446,
  [1447] = 1447,
  [1448] = 1448,
  [1449] = 1449,
  [1450] = 1444,
  [1451] = 1451,
  [1452] = 1452,
  [1453] = 1453,
  [1454] = 1454,
  [1455] = 1455,
  [1456] = 1456,
  [1457] = 1457,
  [1458] = 1458,
  [1459] = 1459,
  [1460] = 1459,
  [1461] = 1459,
  [1462] = 1462,
  [1463] = 1463,
  [1464] = 1464,
  [1465] = 1465,
  [1466] = 1444,
  [1467] = 1467,
  [1468] = 1459,
  [1469] = 1469,
  [1470] = 1459,
  [1471] = 1444,
  [1472] = 1444,
  [1473] = 1473,
  [1474] = 1474,
  [1475] = 1475,
  [1476] = 1476,
  [1477] = 1459,
  [1478] = 1478,
  [1479] = 1479,
  [1480] = 1444,
  [1481] = 1481,
  [1482] = 1482,
  [1483] = 1483,
  [1484] = 1484,
  [1485] = 1485,
  [1486] = 1486,
  [1487] = 1487,
  [1488] = 1488,
  [1489] = 1489,
  [1490] = 1490,
  [1491] = 1491,
  [1492] = 1492,
  [1493] = 1493,
  [1494] = 1494,
  [1495] = 1495,
  [1496] = 1496,
  [1497] = 1497,
  [1498] = 1498,
  [1499] = 1499,
  [1500] = 1500,
  [1501] = 1501,
  [1502] = 1502,
  [1503] = 1503,
  [1504] = 1504,
  [1505] = 1505,
  [1506] = 1506,
  [1507] = 1507,
  [1508] = 1508,
  [1509] = 1509,
  [1510] = 1505,
  [1511] = 1504,
  [1512] = 1512,
  [1513] = 1513,
  [1514] = 1514,
  [1515] = 1515,
  [1516] = 1516,
  [1517] = 1514,
  [1518] = 1514,
  [1519] = 1519,
  [1520] = 1520,
  [1521] = 1521,
  [1522] = 1502,
  [1523] = 1523,
  [1524] = 1524,
  [1525] = 1525,
  [1526] = 1514,
  [1527] = 1503,
  [1528] = 1525,
  [1529] = 1529,
  [1530] = 1504,
  [1531] = 1505,
  [1532] = 1532,
  [1533] = 1533,
  [1534] = 1534,
  [1535] = 1503,
  [1536] = 1536,
  [1537] = 1537,
  [1538] = 1514,
  [1539] = 1539,
  [1540] = 1507,
  [1541] = 1507,
  [1542] = 1508,
  [1543] = 1543,
  [1544] = 1502,
  [1545] = 1507,
  [1546] = 1546,
  [1547] = 1508,
  [1548] = 1514,
  [1549] = 1514,
  [1550] = 1508,
  [1551] = 1503,
  [1552] = 1552,
  [1553] = 1553,
  [1554] = 1554,
  [1555] = 1505,
  [1556] = 1515,
  [1557] = 1557,
  [1558] = 1525,
  [1559] = 1504,
  [1560] = 1560,
  [1561] = 1507,
  [1562] = 1562,
  [1563] = 1563,
  [1564] = 1515,
  [1565] = 1565,
  [1566] = 1566,
  [1567] = 1567,
  [1568] = 1568,
  [1569] = 1569,
  [1570] = 1565,
  [1571] = 1566,
  [1572] = 1572,
  [1573] = 1565,
  [1574] = 1574,
  [1575] = 1566,
  [1576] = 1572,
  [1577] = 1577,
  [1578] = 1578,
  [1579] = 1579,
  [1580] = 1580,
  [1581] = 1581,
  [1582] = 1582,
  [1583] = 1583,
  [1584] = 1568,
  [1585] = 1566,
  [1586] = 1586,
  [1587] = 1572,
  [1588] = 1588,
  [1589] = 1589,
  [1590] = 1590,
  [1591] = 1591,
  [1592] = 1578,
  [1593] = 1593,
  [1594] = 1594,
  [1595] = 1595,
  [1596] = 1594,
  [1597] = 1597,
  [1598] = 1598,
  [1599] = 1599,
  [1600] = 1577,
  [1601] = 1572,
  [1602] = 1602,
  [1603] = 1603,
  [1604] = 1580,
  [1605] = 1566,
  [1606] = 1568,
  [1607] = 1594,
  [1608] = 1578,
  [1609] = 1577,
  [1610] = 1413,
  [1611] = 1611,
  [1612] = 1612,
  [1613] = 1613,
  [1614] = 1612,
  [1615] = 1581,
  [1616] = 1616,
  [1617] = 1617,
  [1618] = 1590,
  [1619] = 1565,
  [1620] = 1590,
  [1621] = 1621,
  [1622] = 1594,
  [1623] = 1623,
  [1624] = 1612,
  [1625] = 1572,
  [1626] = 1594,
  [1627] = 1568,
  [1628] = 1578,
  [1629] = 1629,
  [1630] = 1630,
  [1631] = 1594,
  [1632] = 1632,
  [1633] = 1633,
  [1634] = 1634,
  [1635] = 1635,
  [1636] = 1577,
  [1637] = 1637,
  [1638] = 1638,
  [1639] = 1578,
  [1640] = 1640,
  [1641] = 1641,
  [1642] = 1617,
  [1643] = 1581,
  [1644] = 1568,
  [1645] = 1645,
  [1646] = 1590,
  [1647] = 1647,
  [1648] = 1648,
  [1649] = 1649,
  [1650] = 1611,
  [1651] = 1617,
  [1652] = 1572,
  [1653] = 1566,
  [1654] = 1612,
  [1655] = 1568,
  [1656] = 1656,
  [1657] = 1657,
  [1658] = 1658,
  [1659] = 1640,
  [1660] = 1660,
  [1661] = 1565,
  [1662] = 1577,
  [1663] = 1663,
  [1664] = 1613,
  [1665] = 1665,
  [1666] = 1666,
  [1667] = 1590,
  [1668] = 1590,
  [1669] = 1669,
  [1670] = 1670,
  [1671] = 1671,
  [1672] = 1567,
  [1673] = 1595,
  [1674] = 1674,
  [1675] = 1675,
  [1676] = 1676,
  [1677] = 1677,
  [1678] = 1613,
  [1679] = 1583,
  [1680] = 1611,
  [1681] = 1578,
  [1682] = 1577,
  [1683] = 1565,
  [1684] = 1595,
  [1685] = 1685,
  [1686] = 1580,
  [1687] = 1687,
  [1688] = 1688,
  [1689] = 1689,
  [1690] = 1690,
  [1691] = 1691,
  [1692] = 1690,
  [1693] = 1693,
  [1694] = 1687,
  [1695] = 1689,
  [1696] = 1696,
  [1697] = 1697,
  [1698] = 1698,
  [1699] = 1699,
  [1700] = 1700,
  [1701] = 1696,
  [1702] = 1702,
  [1703] = 1688,
  [1704] = 1704,
  [1705] = 1705,
  [1706] = 1687,
  [1707] = 1707,
  [1708] = 1708,
  [1709] = 1689,
  [1710] = 1710,
  [1711] = 1702,
  [1712] = 1712,
  [1713] = 1713,
  [1714] = 1714,
  [1715] = 1715,
  [1716] = 1716,
  [1717] = 1717,
  [1718] = 1699,
  [1719] = 1719,
  [1720] = 1719,
  [1721] = 1696,
  [1722] = 1702,
  [1723] = 1688,
  [1724] = 1724,
  [1725] = 1725,
  [1726] = 1726,
  [1727] = 1699,
  [1728] = 1728,
  [1729] = 1719,
  [1730] = 1702,
  [1731] = 1731,
  [1732] = 1696,
  [1733] = 1691,
  [1734] = 1689,
  [1735] = 1690,
  [1736] = 1710,
  [1737] = 1688,
  [1738] = 1697,
  [1739] = 1739,
  [1740] = 1690,
  [1741] = 1689,
  [1742] = 1717,
  [1743] = 1702,
  [1744] = 1719,
  [1745] = 1716,
  [1746] = 1699,
  [1747] = 1747,
  [1748] = 1748,
  [1749] = 1749,
  [1750] = 1697,
  [1751] = 1751,
  [1752] = 1752,
  [1753] = 1753,
  [1754] = 1690,
  [1755] = 1699,
  [1756] = 1717,
  [1757] = 1702,
  [1758] = 1758,
  [1759] = 1716,
  [1760] = 1696,
  [1761] = 1761,
  [1762] = 1699,
  [1763] = 1699,
  [1764] = 1687,
  [1765] = 1688,
  [1766] = 1766,
  [1767] = 1691,
  [1768] = 1716,
  [1769] = 1702,
  [1770] = 1697,
  [1771] = 1717,
  [1772] = 1717,
  [1773] = 1773,
  [1774] = 1697,
  [1775] = 1775,
  [1776] = 1776,
  [1777] = 1777,
  [1778] = 1778,
  [1779] = 1779,
  [1780] = 1780,
  [1781] = 1781,
  [1782] = 1782,
  [1783] = 1783,
  [1784] = 1784,
  [1785] = 1785,
  [1786] = 1786,
  [1787] = 1787,
  [1788] = 1788,
  [1789] = 1789,
  [1790] = 1790,
  [1791] = 1791,
  [1792] = 1792,
  [1793] = 1793,
  [1794] = 1791,
  [1795] = 1795,
  [1796] = 1796,
  [1797] = 1797,
  [1798] = 1798,
  [1799] = 1799,
  [1800] = 1795,
  [1801] = 1801,
  [1802] = 1782,
  [1803] = 1803,
  [1804] = 1804,
  [1805] = 1805,
  [1806] = 1806,
  [1807] = 1783,
  [1808] = 1808,
  [1809] = 1809,
  [1810] = 1796,
  [1811] = 1811,
  [1812] = 1812,
  [1813] = 1813,
  [1814] = 1814,
  [1815] = 1815,
  [1816] = 1796,
  [1817] = 1817,
  [1818] = 1818,
  [1819] = 1795,
  [1820] = 1775,
  [1821] = 1821,
  [1822] = 1822,
  [1823] = 1823,
  [1824] = 1793,
  [1825] = 1783,
  [1826] = 1826,
  [1827] = 1827,
  [1828] = 1828,
  [1829] = 1829,
  [1830] = 1830,
  [1831] = 1831,
  [1832] = 1796,
  [1833] = 1795,
  [1834] = 1834,
  [1835] = 1783,
  [1836] = 1836,
  [1837] = 1837,
  [1838] = 1838,
  [1839] = 1793,
  [1840] = 1840,
  [1841] = 1787,
  [1842] = 1793,
  [1843] = 1828,
  [1844] = 1844,
  [1845] = 1822,
  [1846] = 1792,
  [1847] = 1781,
  [1848] = 1775,
  [1849] = 1849,
  [1850] = 1850,
  [1851] = 1799,
  [1852] = 1852,
  [1853] = 1809,
  [1854] = 1789,
  [1855] = 1834,
  [1856] = 1784,
  [1857] = 1783,
  [1858] = 1858,
  [1859] = 1859,
  [1860] = 1860,
  [1861] = 1801,
  [1862] = 1008,
  [1863] = 1863,
  [1864] = 1864,
  [1865] = 1865,
  [1866] = 1866,
  [1867] = 1867,
  [1868] = 1868,
  [1869] = 1869,
  [1870] = 1870,
  [1871] = 1871,
  [1872] = 1872,
  [1873] = 1840,
  [1874] = 1874,
  [1875] = 1812,
  [1876] = 1796,
  [1877] = 1877,
  [1878] = 1012,
  [1879] = 1879,
  [1880] = 1782,
  [1881] = 1881,
  [1882] = 1882,
  [1883] = 1883,
  [1884] = 1884,
  [1885] = 1809,
  [1886] = 1882,
  [1887] = 1782,
  [1888] = 1888,
  [1889] = 1889,
  [1890] = 1890,
  [1891] = 1795,
  [1892] = 1860,
  [1893] = 1893,
  [1894] = 1894,
  [1895] = 1879,
  [1896] = 1822,
  [1897] = 1795,
  [1898] = 1805,
  [1899] = 1899,
  [1900] = 1900,
  [1901] = 1901,
  [1902] = 1902,
  [1903] = 1791,
  [1904] = 1850,
  [1905] = 1905,
  [1906] = 1906,
  [1907] = 1907,
  [1908] = 1812,
  [1909] = 1799,
  [1910] = 670,
  [1911] = 1801,
  [1912] = 1789,
  [1913] = 1913,
  [1914] = 1860,
  [1915] = 1915,
  [1916] = 1916,
  [1917] = 1837,
  [1918] = 1838,
  [1919] = 1919,
  [1920] = 1801,
  [1921] = 1799,
  [1922] = 1805,
  [1923] = 1781,
  [1924] = 1806,
  [1925] = 1809,
  [1926] = 1792,
  [1927] = 1796,
  [1928] = 1928,
  [1929] = 1929,
  [1930] = 1930,
  [1931] = 1812,
  [1932] = 1932,
  [1933] = 1933,
  [1934] = 1775,
  [1935] = 1822,
  [1936] = 1936,
  [1937] = 1793,
  [1938] = 1884,
  [1939] = 1939,
  [1940] = 1882,
  [1941] = 1884,
  [1942] = 1796,
  [1943] = 1943,
  [1944] = 1944,
  [1945] = 1883,
  [1946] = 1809,
  [1947] = 1947,
  [1948] = 1837,
  [1949] = 1838,
  [1950] = 1950,
  [1951] = 1882,
  [1952] = 1795,
  [1953] = 1791,
  [1954] = 1792,
  [1955] = 1955,
  [1956] = 1782,
  [1957] = 1957,
  [1958] = 1958,
  [1959] = 1959,
  [1960] = 1960,
  [1961] = 1961,
  [1962] = 1962,
  [1963] = 1963,
  [1964] = 1964,
  [1965] = 1965,
  [1966] = 1966,
  [1967] = 1936,
  [1968] = 1968,
  [1969] = 1837,
  [1970] = 1970,
  [1971] = 1971,
  [1972] = 1882,
  [1973] = 1837,
  [1974] = 1837,
  [1975] = 671,
  [1976] = 1976,
  [1977] = 1782,
  [1978] = 1830,
  [1979] = 1979,
  [1980] = 1778,
  [1981] = 1829,
  [1982] = 1828,
  [1983] = 1829,
  [1984] = 1830,
  [1985] = 1809,
  [1986] = 1834,
  [1987] = 1840,
  [1988] = 1787,
  [1989] = 1894,
  [1990] = 1990,
  [1991] = 1991,
  [1992] = 1992,
  [1993] = 1787,
  [1994] = 1994,
  [1995] = 1995,
  [1996] = 1860,
  [1997] = 1860,
  [1998] = 1781,
  [1999] = 1778,
  [2000] = 1806,
  [2001] = 1850,
  [2002] = 1894,
  [2003] = 1805,
  [2004] = 1017,
  [2005] = 1014,
  [2006] = 2006,
  [2007] = 1806,
  [2008] = 1789,
  [2009] = 1783,
  [2010] = 1894,
  [2011] = 1784,
  [2012] = 2012,
  [2013] = 1783,
  [2014] = 1838,
  [2015] = 1778,
  [2016] = 1860,
  [2017] = 2012,
  [2018] = 1778,
  [2019] = 1784,
  [2020] = 1837,
};

static TSCharacterRange sym_number_literal_character_set_13[] = {
  {'0', '9'}, {'B', 'B'}, {'D', 'D'}, {'F', 'F'}, {'L', 'L'}, {'U', 'U'}, {'W', 'W'}, {'b', 'b'},
  {'d', 'd'}, {'f', 'f'}, {'l', 'l'}, {'u', 'u'}, {'w', 'w'},
};

static TSCharacterRange sym_identifier_character_set_1[] = {
  {'$', '$'}, {'A', 'Z'}, {'\\', '\\'}, {'_', '_'}, {'a', 'z'}, {0xaa, 0xaa}, {0xb5, 0xb5}, {0xba, 0xba},
  {0xc0, 0xd6}, {0xd8, 0xf6}, {0xf8, 0x2c1}, {0x2c6, 0x2d1}, {0x2e0, 0x2e4}, {0x2ec, 0x2ec}, {0x2ee, 0x2ee}, {0x370, 0x374},
  {0x376, 0x377}, {0x37b, 0x37d}, {0x37f, 0x37f}, {0x386, 0x386}, {0x388, 0x38a}, {0x38c, 0x38c}, {0x38e, 0x3a1}, {0x3a3, 0x3f5},
  {0x3f7, 0x481}, {0x48a, 0x52f}, {0x531, 0x556}, {0x559, 0x559}, {0x560, 0x588}, {0x5d0, 0x5ea}, {0x5ef, 0x5f2}, {0x620, 0x64a},
  {0x66e, 0x66f}, {0x671, 0x6d3}, {0x6d5, 0x6d5}, {0x6e5, 0x6e6}, {0x6ee, 0x6ef}, {0x6fa, 0x6fc}, {0x6ff, 0x6ff}, {0x710, 0x710},
  {0x712, 0x72f}, {0x74d, 0x7a5}, {0x7b1, 0x7b1}, {0x7ca, 0x7ea}, {0x7f4, 0x7f5}, {0x7fa, 0x7fa}, {0x800, 0x815}, {0x81a, 0x81a},
  {0x824, 0x824}, {0x828, 0x828}, {0x840, 0x858}, {0x860, 0x86a}, {0x870, 0x887}, {0x889, 0x88e}, {0x8a0, 0x8c9}, {0x904, 0x939},
  {0x93d, 0x93d}, {0x950, 0x950}, {0x958, 0x961}, {0x971, 0x980}, {0x985, 0x98c}, {0x98f, 0x990}, {0x993, 0x9a8}, {0x9aa, 0x9b0},
  {0x9b2, 0x9b2}, {0x9b6, 0x9b9}, {0x9bd, 0x9bd}, {0x9ce, 0x9ce}, {0x9dc, 0x9dd}, {0x9df, 0x9e1}, {0x9f0, 0x9f1}, {0x9fc, 0x9fc},
  {0xa05, 0xa0a}, {0xa0f, 0xa10}, {0xa13, 0xa28}, {0xa2a, 0xa30}, {0xa32, 0xa33}, {0xa35, 0xa36}, {0xa38, 0xa39}, {0xa59, 0xa5c},
  {0xa5e, 0xa5e}, {0xa72, 0xa74}, {0xa85, 0xa8d}, {0xa8f, 0xa91}, {0xa93, 0xaa8}, {0xaaa, 0xab0}, {0xab2, 0xab3}, {0xab5, 0xab9},
  {0xabd, 0xabd}, {0xad0, 0xad0}, {0xae0, 0xae1}, {0xaf9, 0xaf9}, {0xb05, 0xb0c}, {0xb0f, 0xb10}, {0xb13, 0xb28}, {0xb2a, 0xb30},
  {0xb32, 0xb33}, {0xb35, 0xb39}, {0xb3d, 0xb3d}, {0xb5c, 0xb5d}, {0xb5f, 0xb61}, {0xb71, 0xb71}, {0xb83, 0xb83}, {0xb85, 0xb8a},
  {0xb8e, 0xb90}, {0xb92, 0xb95}, {0xb99, 0xb9a}, {0xb9c, 0xb9c}, {0xb9e, 0xb9f}, {0xba3, 0xba4}, {0xba8, 0xbaa}, {0xbae, 0xbb9},
  {0xbd0, 0xbd0}, {0xc05, 0xc0c}, {0xc0e, 0xc10}, {0xc12, 0xc28}, {0xc2a, 0xc39}, {0xc3d, 0xc3d}, {0xc58, 0xc5a}, {0xc5d, 0xc5d},
  {0xc60, 0xc61}, {0xc80, 0xc80}, {0xc85, 0xc8c}, {0xc8e, 0xc90}, {0xc92, 0xca8}, {0xcaa, 0xcb3}, {0xcb5, 0xcb9}, {0xcbd, 0xcbd},
  {0xcdd, 0xcde}, {0xce0, 0xce1}, {0xcf1, 0xcf2}, {0xd04, 0xd0c}, {0xd0e, 0xd10}, {0xd12, 0xd3a}, {0xd3d, 0xd3d}, {0xd4e, 0xd4e},
  {0xd54, 0xd56}, {0xd5f, 0xd61}, {0xd7a, 0xd7f}, {0xd85, 0xd96}, {0xd9a, 0xdb1}, {0xdb3, 0xdbb}, {0xdbd, 0xdbd}, {0xdc0, 0xdc6},
  {0xe01, 0xe30}, {0xe32, 0xe32}, {0xe40, 0xe46}, {0xe81, 0xe82}, {0xe84, 0xe84}, {0xe86, 0xe8a}, {0xe8c, 0xea3}, {0xea5, 0xea5},
  {0xea7, 0xeb0}, {0xeb2, 0xeb2}, {0xebd, 0xebd}, {0xec0, 0xec4}, {0xec6, 0xec6}, {0xedc, 0xedf}, {0xf00, 0xf00}, {0xf40, 0xf47},
  {0xf49, 0xf6c}, {0xf88, 0xf8c}, {0x1000, 0x102a}, {0x103f, 0x103f}, {0x1050, 0x1055}, {0x105a, 0x105d}, {0x1061, 0x1061}, {0x1065, 0x1066},
  {0x106e, 0x1070}, {0x1075, 0x1081}, {0x108e, 0x108e}, {0x10a0, 0x10c5}, {0x10c7, 0x10c7}, {0x10cd, 0x10cd}, {0x10d0, 0x10fa}, {0x10fc, 0x1248},
  {0x124a, 0x124d}, {0x1250, 0x1256}, {0x1258, 0x1258}, {0x125a, 0x125d}, {0x1260, 0x1288}, {0x128a, 0x128d}, {0x1290, 0x12b0}, {0x12b2, 0x12b5},
  {0x12b8, 0x12be}, {0x12c0, 0x12c0}, {0x12c2, 0x12c5}, {0x12c8, 0x12d6}, {0x12d8, 0x1310}, {0x1312, 0x1315}, {0x1318, 0x135a}, {0x1380, 0x138f},
  {0x13a0, 0x13f5}, {0x13f8, 0x13fd}, {0x1401, 0x166c}, {0x166f, 0x167f}, {0x1681, 0x169a}, {0x16a0, 0x16ea}, {0x16ee, 0x16f8}, {0x1700, 0x1711},
  {0x171f, 0x1731}, {0x1740, 0x1751}, {0x1760, 0x176c}, {0x176e, 0x1770}, {0x1780, 0x17b3}, {0x17d7, 0x17d7}, {0x17dc, 0x17dc}, {0x1820, 0x1878},
  {0x1880, 0x18a8}, {0x18aa, 0x18aa}, {0x18b0, 0x18f5}, {0x1900, 0x191e}, {0x1950, 0x196d}, {0x1970, 0x1974}, {0x1980, 0x19ab}, {0x19b0, 0x19c9},
  {0x1a00, 0x1a16}, {0x1a20, 0x1a54}, {0x1aa7, 0x1aa7}, {0x1b05, 0x1b33}, {0x1b45, 0x1b4c}, {0x1b83, 0x1ba0}, {0x1bae, 0x1baf}, {0x1bba, 0x1be5},
  {0x1c00, 0x1c23}, {0x1c4d, 0x1c4f}, {0x1c5a, 0x1c7d}, {0x1c80, 0x1c88}, {0x1c90, 0x1cba}, {0x1cbd, 0x1cbf}, {0x1ce9, 0x1cec}, {0x1cee, 0x1cf3},
  {0x1cf5, 0x1cf6}, {0x1cfa, 0x1cfa}, {0x1d00, 0x1dbf}, {0x1e00, 0x1f15}, {0x1f18, 0x1f1d}, {0x1f20, 0x1f45}, {0x1f48, 0x1f4d}, {0x1f50, 0x1f57},
  {0x1f59, 0x1f59}, {0x1f5b, 0x1f5b}, {0x1f5d, 0x1f5d}, {0x1f5f, 0x1f7d}, {0x1f80, 0x1fb4}, {0x1fb6, 0x1fbc}, {0x1fbe, 0x1fbe}, {0x1fc2, 0x1fc4},
  {0x1fc6, 0x1fcc}, {0x1fd0, 0x1fd3}, {0x1fd6, 0x1fdb}, {0x1fe0, 0x1fec}, {0x1ff2, 0x1ff4}, {0x1ff6, 0x1ffc}, {0x2071, 0x2071}, {0x207f, 0x207f},
  {0x2090, 0x209c}, {0x2102, 0x2102}, {0x2107, 0x2107}, {0x210a, 0x2113}, {0x2115, 0x2115}, {0x2118, 0x211d}, {0x2124, 0x2124}, {0x2126, 0x2126},
  {0x2128, 0x2128}, {0x212a, 0x2139}, {0x213c, 0x213f}, {0x2145, 0x2149}, {0x214e, 0x214e}, {0x2160, 0x2188}, {0x2c00, 0x2ce4}, {0x2ceb, 0x2cee},
  {0x2cf2, 0x2cf3}, {0x2d00, 0x2d25}, {0x2d27, 0x2d27}, {0x2d2d, 0x2d2d}, {0x2d30, 0x2d67}, {0x2d6f, 0x2d6f}, {0x2d80, 0x2d96}, {0x2da0, 0x2da6},
  {0x2da8, 0x2dae}, {0x2db0, 0x2db6}, {0x2db8, 0x2dbe}, {0x2dc0, 0x2dc6}, {0x2dc8, 0x2dce}, {0x2dd0, 0x2dd6}, {0x2dd8, 0x2dde}, {0x3005, 0x3007},
  {0x3021, 0x3029}, {0x3031, 0x3035}, {0x3038, 0x303c}, {0x3041, 0x3096}, {0x309d, 0x309f}, {0x30a1, 0x30fa}, {0x30fc, 0x30ff}, {0x3105, 0x312f},
  {0x3131, 0x318e}, {0x31a0, 0x31bf}, {0x31f0, 0x31ff}, {0x3400, 0x4dbf}, {0x4e00, 0xa48c}, {0xa4d0, 0xa4fd}, {0xa500, 0xa60c}, {0xa610, 0xa61f},
  {0xa62a, 0xa62b}, {0xa640, 0xa66e}, {0xa67f, 0xa69d}, {0xa6a0, 0xa6ef}, {0xa717, 0xa71f}, {0xa722, 0xa788}, {0xa78b, 0xa7ca}, {0xa7d0, 0xa7d1},
  {0xa7d3, 0xa7d3}, {0xa7d5, 0xa7d9}, {0xa7f2, 0xa801}, {0xa803, 0xa805}, {0xa807, 0xa80a}, {0xa80c, 0xa822}, {0xa840, 0xa873}, {0xa882, 0xa8b3},
  {0xa8f2, 0xa8f7}, {0xa8fb, 0xa8fb}, {0xa8fd, 0xa8fe}, {0xa90a, 0xa925}, {0xa930, 0xa946}, {0xa960, 0xa97c}, {0xa984, 0xa9b2}, {0xa9cf, 0xa9cf},
  {0xa9e0, 0xa9e4}, {0xa9e6, 0xa9ef}, {0xa9fa, 0xa9fe}, {0xaa00, 0xaa28}, {0xaa40, 0xaa42}, {0xaa44, 0xaa4b}, {0xaa60, 0xaa76}, {0xaa7a, 0xaa7a},
  {0xaa7e, 0xaaaf}, {0xaab1, 0xaab1}, {0xaab5, 0xaab6}, {0xaab9, 0xaabd}, {0xaac0, 0xaac0}, {0xaac2, 0xaac2}, {0xaadb, 0xaadd}, {0xaae0, 0xaaea},
  {0xaaf2, 0xaaf4}, {0xab01, 0xab06}, {0xab09, 0xab0e}, {0xab11, 0xab16}, {0xab20, 0xab26}, {0xab28, 0xab2e}, {0xab30, 0xab5a}, {0xab5c, 0xab69},
  {0xab70, 0xabe2}, {0xac00, 0xd7a3}, {0xd7b0, 0xd7c6}, {0xd7cb, 0xd7fb}, {0xf900, 0xfa6d}, {0xfa70, 0xfad9}, {0xfb00, 0xfb06}, {0xfb13, 0xfb17},
  {0xfb1d, 0xfb1d}, {0xfb1f, 0xfb28}, {0xfb2a, 0xfb36}, {0xfb38, 0xfb3c}, {0xfb3e, 0xfb3e}, {0xfb40, 0xfb41}, {0xfb43, 0xfb44}, {0xfb46, 0xfbb1},
  {0xfbd3, 0xfc5d}, {0xfc64, 0xfd3d}, {0xfd50, 0xfd8f}, {0xfd92, 0xfdc7}, {0xfdf0, 0xfdf9}, {0xfe71, 0xfe71}, {0xfe73, 0xfe73}, {0xfe77, 0xfe77},
  {0xfe79, 0xfe79}, {0xfe7b, 0xfe7b}, {0xfe7d, 0xfe7d}, {0xfe7f, 0xfefc}, {0xff21, 0xff3a}, {0xff41, 0xff5a}, {0xff66, 0xff9d}, {0xffa0, 0xffbe},
  {0xffc2, 0xffc7}, {0xffca, 0xffcf}, {0xffd2, 0xffd7}, {0xffda, 0xffdc}, {0x10000, 0x1000b}, {0x1000d, 0x10026}, {0x10028, 0x1003a}, {0x1003c, 0x1003d},
  {0x1003f, 0x1004d}, {0x10050, 0x1005d}, {0x10080, 0x100fa}, {0x10140, 0x10174}, {0x10280, 0x1029c}, {0x102a0, 0x102d0}, {0x10300, 0x1031f}, {0x1032d, 0x1034a},
  {0x10350, 0x10375}, {0x10380, 0x1039d}, {0x103a0, 0x103c3}, {0x103c8, 0x103cf}, {0x103d1, 0x103d5}, {0x10400, 0x1049d}, {0x104b0, 0x104d3}, {0x104d8, 0x104fb},
  {0x10500, 0x10527}, {0x10530, 0x10563}, {0x10570, 0x1057a}, {0x1057c, 0x1058a}, {0x1058c, 0x10592}, {0x10594, 0x10595}, {0x10597, 0x105a1}, {0x105a3, 0x105b1},
  {0x105b3, 0x105b9}, {0x105bb, 0x105bc}, {0x10600, 0x10736}, {0x10740, 0x10755}, {0x10760, 0x10767}, {0x10780, 0x10785}, {0x10787, 0x107b0}, {0x107b2, 0x107ba},
  {0x10800, 0x10805}, {0x10808, 0x10808}, {0x1080a, 0x10835}, {0x10837, 0x10838}, {0x1083c, 0x1083c}, {0x1083f, 0x10855}, {0x10860, 0x10876}, {0x10880, 0x1089e},
  {0x108e0, 0x108f2}, {0x108f4, 0x108f5}, {0x10900, 0x10915}, {0x10920, 0x10939}, {0x10980, 0x109b7}, {0x109be, 0x109bf}, {0x10a00, 0x10a00}, {0x10a10, 0x10a13},
  {0x10a15, 0x10a17}, {0x10a19, 0x10a35}, {0x10a60, 0x10a7c}, {0x10a80, 0x10a9c}, {0x10ac0, 0x10ac7}, {0x10ac9, 0x10ae4}, {0x10b00, 0x10b35}, {0x10b40, 0x10b55},
  {0x10b60, 0x10b72}, {0x10b80, 0x10b91}, {0x10c00, 0x10c48}, {0x10c80, 0x10cb2}, {0x10cc0, 0x10cf2}, {0x10d00, 0x10d23}, {0x10e80, 0x10ea9}, {0x10eb0, 0x10eb1},
  {0x10f00, 0x10f1c}, {0x10f27, 0x10f27}, {0x10f30, 0x10f45}, {0x10f70, 0x10f81}, {0x10fb0, 0x10fc4}, {0x10fe0, 0x10ff6}, {0x11003, 0x11037}, {0x11071, 0x11072},
  {0x11075, 0x11075}, {0x11083, 0x110af}, {0x110d0, 0x110e8}, {0x11103, 0x11126}, {0x11144, 0x11144}, {0x11147, 0x11147}, {0x11150, 0x11172}, {0x11176, 0x11176},
  {0x11183, 0x111b2}, {0x111c1, 0x111c4}, {0x111da, 0x111da}, {0x111dc, 0x111dc}, {0x11200, 0x11211}, {0x11213, 0x1122b}, {0x1123f, 0x11240}, {0x11280, 0x11286},
  {0x11288, 0x11288}, {0x1128a, 0x1128d}, {0x1128f, 0x1129d}, {0x1129f, 0x112a8}, {0x112b0, 0x112de}, {0x11305, 0x1130c}, {0x1130f, 0x11310}, {0x11313, 0x11328},
  {0x1132a, 0x11330}, {0x11332, 0x11333}, {0x11335, 0x11339}, {0x1133d, 0x1133d}, {0x11350, 0x11350}, {0x1135d, 0x11361}, {0x11400, 0x11434}, {0x11447, 0x1144a},
  {0x1145f, 0x11461}, {0x11480, 0x114af}, {0x114c4, 0x114c5}, {0x114c7, 0x114c7}, {0x11580, 0x115ae}, {0x115d8, 0x115db}, {0x11600, 0x1162f}, {0x11644, 0x11644},
  {0x11680, 0x116aa}, {0x116b8, 0x116b8}, {0x11700, 0x1171a}, {0x11740, 0x11746}, {0x11800, 0x1182b}, {0x118a0, 0x118df}, {0x118ff, 0x11906}, {0x11909, 0x11909},
  {0x1190c, 0x11913}, {0x11915, 0x11916}, {0x11918, 0x1192f}, {0x1193f, 0x1193f}, {0x11941, 0x11941}, {0x119a0, 0x119a7}, {0x119aa, 0x119d0}, {0x119e1, 0x119e1},
  {0x119e3, 0x119e3}, {0x11a00, 0x11a00}, {0x11a0b, 0x11a32}, {0x11a3a, 0x11a3a}, {0x11a50, 0x11a50}, {0x11a5c, 0x11a89}, {0x11a9d, 0x11a9d}, {0x11ab0, 0x11af8},
  {0x11c00, 0x11c08}, {0x11c0a, 0x11c2e}, {0x11c40, 0x11c40}, {0x11c72, 0x11c8f}, {0x11d00, 0x11d06}, {0x11d08, 0x11d09}, {0x11d0b, 0x11d30}, {0x11d46, 0x11d46},
  {0x11d60, 0x11d65}, {0x11d67, 0x11d68}, {0x11d6a, 0x11d89}, {0x11d98, 0x11d98}, {0x11ee0, 0x11ef2}, {0x11f02, 0x11f02}, {0x11f04, 0x11f10}, {0x11f12, 0x11f33},
  {0x11fb0, 0x11fb0}, {0x12000, 0x12399}, {0x12400, 0x1246e}, {0x12480, 0x12543}, {0x12f90, 0x12ff0}, {0x13000, 0x1342f}, {0x13441, 0x13446}, {0x14400, 0x14646},
  {0x16800, 0x16a38}, {0x16a40, 0x16a5e}, {0x16a70, 0x16abe}, {0x16ad0, 0x16aed}, {0x16b00, 0x16b2f}, {0x16b40, 0x16b43}, {0x16b63, 0x16b77}, {0x16b7d, 0x16b8f},
  {0x16e40, 0x16e7f}, {0x16f00, 0x16f4a}, {0x16f50, 0x16f50}, {0x16f93, 0x16f9f}, {0x16fe0, 0x16fe1}, {0x16fe3, 0x16fe3}, {0x17000, 0x187f7}, {0x18800, 0x18cd5},
  {0x18d00, 0x18d08}, {0x1aff0, 0x1aff3}, {0x1aff5, 0x1affb}, {0x1affd, 0x1affe}, {0x1b000, 0x1b122}, {0x1b132, 0x1b132}, {0x1b150, 0x1b152}, {0x1b155, 0x1b155},
  {0x1b164, 0x1b167}, {0x1b170, 0x1b2fb}, {0x1bc00, 0x1bc6a}, {0x1bc70, 0x1bc7c}, {0x1bc80, 0x1bc88}, {0x1bc90, 0x1bc99}, {0x1d400, 0x1d454}, {0x1d456, 0x1d49c},
  {0x1d49e, 0x1d49f}, {0x1d4a2, 0x1d4a2}, {0x1d4a5, 0x1d4a6}, {0x1d4a9, 0x1d4ac}, {0x1d4ae, 0x1d4b9}, {0x1d4bb, 0x1d4bb}, {0x1d4bd, 0x1d4c3}, {0x1d4c5, 0x1d505},
  {0x1d507, 0x1d50a}, {0x1d50d, 0x1d514}, {0x1d516, 0x1d51c}, {0x1d51e, 0x1d539}, {0x1d53b, 0x1d53e}, {0x1d540, 0x1d544}, {0x1d546, 0x1d546}, {0x1d54a, 0x1d550},
  {0x1d552, 0x1d6a5}, {0x1d6a8, 0x1d6c0}, {0x1d6c2, 0x1d6da}, {0x1d6dc, 0x1d6fa}, {0x1d6fc, 0x1d714}, {0x1d716, 0x1d734}, {0x1d736, 0x1d74e}, {0x1d750, 0x1d76e},
  {0x1d770, 0x1d788}, {0x1d78a, 0x1d7a8}, {0x1d7aa, 0x1d7c2}, {0x1d7c4, 0x1d7cb}, {0x1df00, 0x1df1e}, {0x1df25, 0x1df2a}, {0x1e030, 0x1e06d}, {0x1e100, 0x1e12c},
  {0x1e137, 0x1e13d}, {0x1e14e, 0x1e14e}, {0x1e290, 0x1e2ad}, {0x1e2c0, 0x1e2eb}, {0x1e4d0, 0x1e4eb}, {0x1e7e0, 0x1e7e6}, {0x1e7e8, 0x1e7eb}, {0x1e7ed, 0x1e7ee},
  {0x1e7f0, 0x1e7fe}, {0x1e800, 0x1e8c4}, {0x1e900, 0x1e943}, {0x1e94b, 0x1e94b}, {0x1ee00, 0x1ee03}, {0x1ee05, 0x1ee1f}, {0x1ee21, 0x1ee22}, {0x1ee24, 0x1ee24},
  {0x1ee27, 0x1ee27}, {0x1ee29, 0x1ee32}, {0x1ee34, 0x1ee37}, {0x1ee39, 0x1ee39}, {0x1ee3b, 0x1ee3b}, {0x1ee42, 0x1ee42}, {0x1ee47, 0x1ee47}, {0x1ee49, 0x1ee49},
  {0x1ee4b, 0x1ee4b}, {0x1ee4d, 0x1ee4f}, {0x1ee51, 0x1ee52}, {0x1ee54, 0x1ee54}, {0x1ee57, 0x1ee57}, {0x1ee59, 0x1ee59}, {0x1ee5b, 0x1ee5b}, {0x1ee5d, 0x1ee5d},
  {0x1ee5f, 0x1ee5f}, {0x1ee61, 0x1ee62}, {0x1ee64, 0x1ee64}, {0x1ee67, 0x1ee6a}, {0x1ee6c, 0x1ee72}, {0x1ee74, 0x1ee77}, {0x1ee79, 0x1ee7c}, {0x1ee7e, 0x1ee7e},
  {0x1ee80, 0x1ee89}, {0x1ee8b, 0x1ee9b}, {0x1eea1, 0x1eea3}, {0x1eea5, 0x1eea9}, {0x1eeab, 0x1eebb}, {0x20000, 0x2a6df}, {0x2a700, 0x2b739}, {0x2b740, 0x2b81d},
  {0x2b820, 0x2cea1}, {0x2ceb0, 0x2ebe0}, {0x2ebf0, 0x2ee5d}, {0x2f800, 0x2fa1d}, {0x30000, 0x3134a}, {0x31350, 0x323af},
};

static TSCharacterRange sym_identifier_character_set_2[] = {
  {'$', '$'}, {'0', '9'}, {'A', 'Z'}, {'\\', '\\'}, {'_', '_'}, {'a', 'z'}, {0xaa, 0xaa}, {0xb5, 0xb5},
  {0xb7, 0xb7}, {0xba, 0xba}, {0xc0, 0xd6}, {0xd8, 0xf6}, {0xf8, 0x2c1}, {0x2c6, 0x2d1}, {0x2e0, 0x2e4}, {0x2ec, 0x2ec},
  {0x2ee, 0x2ee}, {0x300, 0x374}, {0x376, 0x377}, {0x37b, 0x37d}, {0x37f, 0x37f}, {0x386, 0x38a}, {0x38c, 0x38c}, {0x38e, 0x3a1},
  {0x3a3, 0x3f5}, {0x3f7, 0x481}, {0x483, 0x487}, {0x48a, 0x52f}, {0x531, 0x556}, {0x559, 0x559}, {0x560, 0x588}, {0x591, 0x5bd},
  {0x5bf, 0x5bf}, {0x5c1, 0x5c2}, {0x5c4, 0x5c5}, {0x5c7, 0x5c7}, {0x5d0, 0x5ea}, {0x5ef, 0x5f2}, {0x610, 0x61a}, {0x620, 0x669},
  {0x66e, 0x6d3}, {0x6d5, 0x6dc}, {0x6df, 0x6e8}, {0x6ea, 0x6fc}, {0x6ff, 0x6ff}, {0x710, 0x74a}, {0x74d, 0x7b1}, {0x7c0, 0x7f5},
  {0x7fa, 0x7fa}, {0x7fd, 0x7fd}, {0x800, 0x82d}, {0x840, 0x85b}, {0x860, 0x86a}, {0x870, 0x887}, {0x889, 0x88e}, {0x898, 0x8e1},
  {0x8e3, 0x963}, {0x966, 0x96f}, {0x971, 0x983}, {0x985, 0x98c}, {0x98f, 0x990}, {0x993, 0x9a8}, {0x9aa, 0x9b0}, {0x9b2, 0x9b2},
  {0x9b6, 0x9b9}, {0x9bc, 0x9c4}, {0x9c7, 0x9c8}, {0x9cb, 0x9ce}, {0x9d7, 0x9d7}, {0x9dc, 0x9dd}, {0x9df, 0x9e3}, {0x9e6, 0x9f1},
  {0x9fc, 0x9fc}, {0x9fe, 0x9fe}, {0xa01, 0xa03}, {0xa05, 0xa0a}, {0xa0f, 0xa10}, {0xa13, 0xa28}, {0xa2a, 0xa30}, {0xa32, 0xa33},
  {0xa35, 0xa36}, {0xa38, 0xa39}, {0xa3c, 0xa3c}, {0xa3e, 0xa42}, {0xa47, 0xa48}, {0xa4b, 0xa4d}, {0xa51, 0xa51}, {0xa59, 0xa5c},
  {0xa5e, 0xa5e}, {0xa66, 0xa75}, {0xa81, 0xa83}, {0xa85, 0xa8d}, {0xa8f, 0xa91}, {0xa93, 0xaa8}, {0xaaa, 0xab0}, {0xab2, 0xab3},
  {0xab5, 0xab9}, {0xabc, 0xac5}, {0xac7, 0xac9}, {0xacb, 0xacd}, {0xad0, 0xad0}, {0xae0, 0xae3}, {0xae6, 0xaef}, {0xaf9, 0xaff},
  {0xb01, 0xb03}, {0xb05, 0xb0c}, {0xb0f, 0xb10}, {0xb13, 0xb28}, {0xb2a, 0xb30}, {0xb32, 0xb33}, {0xb35, 0xb39}, {0xb3c, 0xb44},
  {0xb47, 0xb48}, {0xb4b, 0xb4d}, {0xb55, 0xb57}, {0xb5c, 0xb5d}, {0xb5f, 0xb63}, {0xb66, 0xb6f}, {0xb71, 0xb71}, {0xb82, 0xb83},
  {0xb85, 0xb8a}, {0xb8e, 0xb90}, {0xb92, 0xb95}, {0xb99, 0xb9a}, {0xb9c, 0xb9c}, {0xb9e, 0xb9f}, {0xba3, 0xba4}, {0xba8, 0xbaa},
  {0xbae, 0xbb9}, {0xbbe, 0xbc2}, {0xbc6, 0xbc8}, {0xbca, 0xbcd}, {0xbd0, 0xbd0}, {0xbd7, 0xbd7}, {0xbe6, 0xbef}, {0xc00, 0xc0c},
  {0xc0e, 0xc10}, {0xc12, 0xc28}, {0xc2a, 0xc39}, {0xc3c, 0xc44}, {0xc46, 0xc48}, {0xc4a, 0xc4d}, {0xc55, 0xc56}, {0xc58, 0xc5a},
  {0xc5d, 0xc5d}, {0xc60, 0xc63}, {0xc66, 0xc6f}, {0xc80, 0xc83}, {0xc85, 0xc8c}, {0xc8e, 0xc90}, {0xc92, 0xca8}, {0xcaa, 0xcb3},
  {0xcb5, 0xcb9}, {0xcbc, 0xcc4}, {0xcc6, 0xcc8}, {0xcca, 0xccd}, {0xcd5, 0xcd6}, {0xcdd, 0xcde}, {0xce0, 0xce3}, {0xce6, 0xcef},
  {0xcf1, 0xcf3}, {0xd00, 0xd0c}, {0xd0e, 0xd10}, {0xd12, 0xd44}, {0xd46, 0xd48}, {0xd4a, 0xd4e}, {0xd54, 0xd57}, {0xd5f, 0xd63},
  {0xd66, 0xd6f}, {0xd7a, 0xd7f}, {0xd81, 0xd83}, {0xd85, 0xd96}, {0xd9a, 0xdb1}, {0xdb3, 0xdbb}, {0xdbd, 0xdbd}, {0xdc0, 0xdc6},
  {0xdca, 0xdca}, {0xdcf, 0xdd4}, {0xdd6, 0xdd6}, {0xdd8, 0xddf}, {0xde6, 0xdef}, {0xdf2, 0xdf3}, {0xe01, 0xe3a}, {0xe40, 0xe4e},
  {0xe50, 0xe59}, {0xe81, 0xe82}, {0xe84, 0xe84}, {0xe86, 0xe8a}, {0xe8c, 0xea3}, {0xea5, 0xea5}, {0xea7, 0xebd}, {0xec0, 0xec4},
  {0xec6, 0xec6}, {0xec8, 0xece}, {0xed0, 0xed9}, {0xedc, 0xedf}, {0xf00, 0xf00}, {0xf18, 0xf19}, {0xf20, 0xf29}, {0xf35, 0xf35},
  {0xf37, 0xf37}, {0xf39, 0xf39}, {0xf3e, 0xf47}, {0xf49, 0xf6c}, {0xf71, 0xf84}, {0xf86, 0xf97}, {0xf99, 0xfbc}, {0xfc6, 0xfc6},
  {0x1000, 0x1049}, {0x1050, 0x109d}, {0x10a0, 0x10c5}, {0x10c7, 0x10c7}, {0x10cd, 0x10cd}, {0x10d0, 0x10fa}, {0x10fc, 0x1248}, {0x124a, 0x124d},
  {0x1250, 0x1256}, {0x1258, 0x1258}, {0x125a, 0x125d}, {0x1260, 0x1288}, {0x128a, 0x128d}, {0x1290, 0x12b0}, {0x12b2, 0x12b5}, {0x12b8, 0x12be},
  {0x12c0, 0x12c0}, {0x12c2, 0x12c5}, {0x12c8, 0x12d6}, {0x12d8, 0x1310}, {0x1312, 0x1315}, {0x1318, 0x135a}, {0x135d, 0x135f}, {0x1369, 0x1371},
  {0x1380, 0x138f}, {0x13a0, 0x13f5}, {0x13f8, 0x13fd}, {0x1401, 0x166c}, {0x166f, 0x167f}, {0x1681, 0x169a}, {0x16a0, 0x16ea}, {0x16ee, 0x16f8},
  {0x1700, 0x1715}, {0x171f, 0x1734}, {0x1740, 0x1753}, {0x1760, 0x176c}, {0x176e, 0x1770}, {0x1772, 0x1773}, {0x1780, 0x17d3}, {0x17d7, 0x17d7},
  {0x17dc, 0x17dd}, {0x17e0, 0x17e9}, {0x180b, 0x180d}, {0x180f, 0x1819}, {0x1820, 0x1878}, {0x1880, 0x18aa}, {0x18b0, 0x18f5}, {0x1900, 0x191e},
  {0x1920, 0x192b}, {0x1930, 0x193b}, {0x1946, 0x196d}, {0x1970, 0x1974}, {0x1980, 0x19ab}, {0x19b0, 0x19c9}, {0x19d0, 0x19da}, {0x1a00, 0x1a1b},
  {0x1a20, 0x1a5e}, {0x1a60, 0x1a7c}, {0x1a7f, 0x1a89}, {0x1a90, 0x1a99}, {0x1aa7, 0x1aa7}, {0x1ab0, 0x1abd}, {0x1abf, 0x1ace}, {0x1b00, 0x1b4c},
  {0x1b50, 0x1b59}, {0x1b6b, 0x1b73}, {0x1b80, 0x1bf3}, {0x1c00, 0x1c37}, {0x1c40, 0x1c49}, {0x1c4d, 0x1c7d}, {0x1c80, 0x1c88}, {0x1c90, 0x1cba},
  {0x1cbd, 0x1cbf}, {0x1cd0, 0x1cd2}, {0x1cd4, 0x1cfa}, {0x1d00, 0x1f15}, {0x1f18, 0x1f1d}, {0x1f20, 0x1f45}, {0x1f48, 0x1f4d}, {0x1f50, 0x1f57},
  {0x1f59, 0x1f59}, {0x1f5b, 0x1f5b}, {0x1f5d, 0x1f5d}, {0x1f5f, 0x1f7d}, {0x1f80, 0x1fb4}, {0x1fb6, 0x1fbc}, {0x1fbe, 0x1fbe}, {0x1fc2, 0x1fc4},
  {0x1fc6, 0x1fcc}, {0x1fd0, 0x1fd3}, {0x1fd6, 0x1fdb}, {0x1fe0, 0x1fec}, {0x1ff2, 0x1ff4}, {0x1ff6, 0x1ffc}, {0x200c, 0x200d}, {0x203f, 0x2040},
  {0x2054, 0x2054}, {0x2071, 0x2071}, {0x207f, 0x207f}, {0x2090, 0x209c}, {0x20d0, 0x20dc}, {0x20e1, 0x20e1}, {0x20e5, 0x20f0}, {0x2102, 0x2102},
  {0x2107, 0x2107}, {0x210a, 0x2113}, {0x2115, 0x2115}, {0x2118, 0x211d}, {0x2124, 0x2124}, {0x2126, 0x2126}, {0x2128, 0x2128}, {0x212a, 0x2139},
  {0x213c, 0x213f}, {0x2145, 0x2149}, {0x214e, 0x214e}, {0x2160, 0x2188}, {0x2c00, 0x2ce4}, {0x2ceb, 0x2cf3}, {0x2d00, 0x2d25}, {0x2d27, 0x2d27},
  {0x2d2d, 0x2d2d}, {0x2d30, 0x2d67}, {0x2d6f, 0x2d6f}, {0x2d7f, 0x2d96}, {0x2da0, 0x2da6}, {0x2da8, 0x2dae}, {0x2db0, 0x2db6}, {0x2db8, 0x2dbe},
  {0x2dc0, 0x2dc6}, {0x2dc8, 0x2dce}, {0x2dd0, 0x2dd6}, {0x2dd8, 0x2dde}, {0x2de0, 0x2dff}, {0x3005, 0x3007}, {0x3021, 0x302f}, {0x3031, 0x3035},
  {0x3038, 0x303c}, {0x3041, 0x3096}, {0x3099, 0x309a}, {0x309d, 0x309f}, {0x30a1, 0x30ff}, {0x3105, 0x312f}, {0x3131, 0x318e}, {0x31a0, 0x31bf},
  {0x31f0, 0x31ff}, {0x3400, 0x4dbf}, {0x4e00, 0xa48c}, {0xa4d0, 0xa4fd}, {0xa500, 0xa60c}, {0xa610, 0xa62b}, {0xa640, 0xa66f}, {0xa674, 0xa67d},
  {0xa67f, 0xa6f1}, {0xa717, 0xa71f}, {0xa722, 0xa788}, {0xa78b, 0xa7ca}, {0xa7d0, 0xa7d1}, {0xa7d3, 0xa7d3}, {0xa7d5, 0xa7d9}, {0xa7f2, 0xa827},
  {0xa82c, 0xa82c}, {0xa840, 0xa873}, {0xa880, 0xa8c5}, {0xa8d0, 0xa8d9}, {0xa8e0, 0xa8f7}, {0xa8fb, 0xa8fb}, {0xa8fd, 0xa92d}, {0xa930, 0xa953},
  {0xa960, 0xa97c}, {0xa980, 0xa9c0}, {0xa9cf, 0xa9d9}, {0xa9e0, 0xa9fe}, {0xaa00, 0xaa36}, {0xaa40, 0xaa4d}, {0xaa50, 0xaa59}, {0xaa60, 0xaa76},
  {0xaa7a, 0xaac2}, {0xaadb, 0xaadd}, {0xaae0, 0xaaef}, {0xaaf2, 0xaaf6}, {0xab01, 0xab06}, {0xab09, 0xab0e}, {0xab11, 0xab16}, {0xab20, 0xab26},
  {0xab28, 0xab2e}, {0xab30, 0xab5a}, {0xab5c, 0xab69}, {0xab70, 0xabea}, {0xabec, 0xabed}, {0xabf0, 0xabf9}, {0xac00, 0xd7a3}, {0xd7b0, 0xd7c6},
  {0xd7cb, 0xd7fb}, {0xf900, 0xfa6d}, {0xfa70, 0xfad9}, {0xfb00, 0xfb06}, {0xfb13, 0xfb17}, {0xfb1d, 0xfb28}, {0xfb2a, 0xfb36}, {0xfb38, 0xfb3c},
  {0xfb3e, 0xfb3e}, {0xfb40, 0xfb41}, {0xfb43, 0xfb44}, {0xfb46, 0xfbb1}, {0xfbd3, 0xfc5d}, {0xfc64, 0xfd3d}, {0xfd50, 0xfd8f}, {0xfd92, 0xfdc7},
  {0xfdf0, 0xfdf9}, {0xfe00, 0xfe0f}, {0xfe20, 0xfe2f}, {0xfe33, 0xfe34}, {0xfe4d, 0xfe4f}, {0xfe71, 0xfe71}, {0xfe73, 0xfe73}, {0xfe77, 0xfe77},
  {0xfe79, 0xfe79}, {0xfe7b, 0xfe7b}, {0xfe7d, 0xfe7d}, {0xfe7f, 0xfefc}, {0xff10, 0xff19}, {0xff21, 0xff3a}, {0xff3f, 0xff3f}, {0xff41, 0xff5a},
  {0xff65, 0xffbe}, {0xffc2, 0xffc7}, {0xffca, 0xffcf}, {0xffd2, 0xffd7}, {0xffda, 0xffdc}, {0x10000, 0x1000b}, {0x1000d, 0x10026}, {0x10028, 0x1003a},
  {0x1003c, 0x1003d}, {0x1003f, 0x1004d}, {0x10050, 0x1005d}, {0x10080, 0x100fa}, {0x10140, 0x10174}, {0x101fd, 0x101fd}, {0x10280, 0x1029c}, {0x102a0, 0x102d0},
  {0x102e0, 0x102e0}, {0x10300, 0x1031f}, {0x1032d, 0x1034a}, {0x10350, 0x1037a}, {0x10380, 0x1039d}, {0x103a0, 0x103c3}, {0x103c8, 0x103cf}, {0x103d1, 0x103d5},
  {0x10400, 0x1049d}, {0x104a0, 0x104a9}, {0x104b0, 0x104d3}, {0x104d8, 0x104fb}, {0x10500, 0x10527}, {0x10530, 0x10563}, {0x10570, 0x1057a}, {0x1057c, 0x1058a},
  {0x1058c, 0x10592}, {0x10594, 0x10595}, {0x10597, 0x105a1}, {0x105a3, 0x105b1}, {0x105b3, 0x105b9}, {0x105bb, 0x105bc}, {0x10600, 0x10736}, {0x10740, 0x10755},
  {0x10760, 0x10767}, {0x10780, 0x10785}, {0x10787, 0x107b0}, {0x107b2, 0x107ba}, {0x10800, 0x10805}, {0x10808, 0x10808}, {0x1080a, 0x10835}, {0x10837, 0x10838},
  {0x1083c, 0x1083c}, {0x1083f, 0x10855}, {0x10860, 0x10876}, {0x10880, 0x1089e}, {0x108e0, 0x108f2}, {0x108f4, 0x108f5}, {0x10900, 0x10915}, {0x10920, 0x10939},
  {0x10980, 0x109b7}, {0x109be, 0x109bf}, {0x10a00, 0x10a03}, {0x10a05, 0x10a06}, {0x10a0c, 0x10a13}, {0x10a15, 0x10a17}, {0x10a19, 0x10a35}, {0x10a38, 0x10a3a},
  {0x10a3f, 0x10a3f}, {0x10a60, 0x10a7c}, {0x10a80, 0x10a9c}, {0x10ac0, 0x10ac7}, {0x10ac9, 0x10ae6}, {0x10b00, 0x10b35}, {0x10b40, 0x10b55}, {0x10b60, 0x10b72},
  {0x10b80, 0x10b91}, {0x10c00, 0x10c48}, {0x10c80, 0x10cb2}, {0x10cc0, 0x10cf2}, {0x10d00, 0x10d27}, {0x10d30, 0x10d39}, {0x10e80, 0x10ea9}, {0x10eab, 0x10eac},
  {0x10eb0, 0x10eb1}, {0x10efd, 0x10f1c}, {0x10f27, 0x10f27}, {0x10f30, 0x10f50}, {0x10f70, 0x10f85}, {0x10fb0, 0x10fc4}, {0x10fe0, 0x10ff6}, {0x11000, 0x11046},
  {0x11066, 0x11075}, {0x1107f, 0x110ba}, {0x110c2, 0x110c2}, {0x110d0, 0x110e8}, {0x110f0, 0x110f9}, {0x11100, 0x11134}, {0x11136, 0x1113f}, {0x11144, 0x11147},
  {0x11150, 0x11173}, {0x11176, 0x11176}, {0x11180, 0x111c4}, {0x111c9, 0x111cc}, {0x111ce, 0x111da}, {0x111dc, 0x111dc}, {0x11200, 0x11211}, {0x11213, 0x11237},
  {0x1123e, 0x11241}, {0x11280, 0x11286}, {0x11288, 0x11288}, {0x1128a, 0x1128d}, {0x1128f, 0x1129d}, {0x1129f, 0x112a8}, {0x112b0, 0x112ea}, {0x112f0, 0x112f9},
  {0x11300, 0x11303}, {0x11305, 0x1130c}, {0x1130f, 0x11310}, {0x11313, 0x11328}, {0x1132a, 0x11330}, {0x11332, 0x11333}, {0x11335, 0x11339}, {0x1133b, 0x11344},
  {0x11347, 0x11348}, {0x1134b, 0x1134d}, {0x11350, 0x11350}, {0x11357, 0x11357}, {0x1135d, 0x11363}, {0x11366, 0x1136c}, {0x11370, 0x11374}, {0x11400, 0x1144a},
  {0x11450, 0x11459}, {0x1145e, 0x11461}, {0x11480, 0x114c5}, {0x114c7, 0x114c7}, {0x114d0, 0x114d9}, {0x11580, 0x115b5}, {0x115b8, 0x115c0}, {0x115d8, 0x115dd},
  {0x11600, 0x11640}, {0x11644, 0x11644}, {0x11650, 0x11659}, {0x11680, 0x116b8}, {0x116c0, 0x116c9}, {0x11700, 0x1171a}, {0x1171d, 0x1172b}, {0x11730, 0x11739},
  {0x11740, 0x11746}, {0x11800, 0x1183a}, {0x118a0, 0x118e9}, {0x118ff, 0x11906}, {0x11909, 0x11909}, {0x1190c, 0x11913}, {0x11915, 0x11916}, {0x11918, 0x11935},
  {0x11937, 0x11938}, {0x1193b, 0x11943}, {0x11950, 0x11959}, {0x119a0, 0x119a7}, {0x119aa, 0x119d7}, {0x119da, 0x119e1}, {0x119e3, 0x119e4}, {0x11a00, 0x11a3e},
  {0x11a47, 0x11a47}, {0x11a50, 0x11a99}, {0x11a9d, 0x11a9d}, {0x11ab0, 0x11af8}, {0x11c00, 0x11c08}, {0x11c0a, 0x11c36}, {0x11c38, 0x11c40}, {0x11c50, 0x11c59},
  {0x11c72, 0x11c8f}, {0x11c92, 0x11ca7}, {0x11ca9, 0x11cb6}, {0x11d00, 0x11d06}, {0x11d08, 0x11d09}, {0x11d0b, 0x11d36}, {0x11d3a, 0x11d3a}, {0x11d3c, 0x11d3d},
  {0x11d3f, 0x11d47}, {0x11d50, 0x11d59}, {0x11d60, 0x11d65}, {0x11d67, 0x11d68}, {0x11d6a, 0x11d8e}, {0x11d90, 0x11d91}, {0x11d93, 0x11d98}, {0x11da0, 0x11da9},
  {0x11ee0, 0x11ef6}, {0x11f00, 0x11f10}, {0x11f12, 0x11f3a}, {0x11f3e, 0x11f42}, {0x11f50, 0x11f59}, {0x11fb0, 0x11fb0}, {0x12000, 0x12399}, {0x12400, 0x1246e},
  {0x12480, 0x12543}, {0x12f90, 0x12ff0}, {0x13000, 0x1342f}, {0x13440, 0x13455}, {0x14400, 0x14646}, {0x16800, 0x16a38}, {0x16a40, 0x16a5e}, {0x16a60, 0x16a69},
  {0x16a70, 0x16abe}, {0x16ac0, 0x16ac9}, {0x16ad0, 0x16aed}, {0x16af0, 0x16af4}, {0x16b00, 0x16b36}, {0x16b40, 0x16b43}, {0x16b50, 0x16b59}, {0x16b63, 0x16b77},
  {0x16b7d, 0x16b8f}, {0x16e40, 0x16e7f}, {0x16f00, 0x16f4a}, {0x16f4f, 0x16f87}, {0x16f8f, 0x16f9f}, {0x16fe0, 0x16fe1}, {0x16fe3, 0x16fe4}, {0x16ff0, 0x16ff1},
  {0x17000, 0x187f7}, {0x18800, 0x18cd5}, {0x18d00, 0x18d08}, {0x1aff0, 0x1aff3}, {0x1aff5, 0x1affb}, {0x1affd, 0x1affe}, {0x1b000, 0x1b122}, {0x1b132, 0x1b132},
  {0x1b150, 0x1b152}, {0x1b155, 0x1b155}, {0x1b164, 0x1b167}, {0x1b170, 0x1b2fb}, {0x1bc00, 0x1bc6a}, {0x1bc70, 0x1bc7c}, {0x1bc80, 0x1bc88}, {0x1bc90, 0x1bc99},
  {0x1bc9d, 0x1bc9e}, {0x1cf00, 0x1cf2d}, {0x1cf30, 0x1cf46}, {0x1d165, 0x1d169}, {0x1d16d, 0x1d172}, {0x1d17b, 0x1d182}, {0x1d185, 0x1d18b}, {0x1d1aa, 0x1d1ad},
  {0x1d242, 0x1d244}, {0x1d400, 0x1d454}, {0x1d456, 0x1d49c}, {0x1d49e, 0x1d49f}, {0x1d4a2, 0x1d4a2}, {0x1d4a5, 0x1d4a6}, {0x1d4a9, 0x1d4ac}, {0x1d4ae, 0x1d4b9},
  {0x1d4bb, 0x1d4bb}, {0x1d4bd, 0x1d4c3}, {0x1d4c5, 0x1d505}, {0x1d507, 0x1d50a}, {0x1d50d, 0x1d514}, {0x1d516, 0x1d51c}, {0x1d51e, 0x1d539}, {0x1d53b, 0x1d53e},
  {0x1d540, 0x1d544}, {0x1d546, 0x1d546}, {0x1d54a, 0x1d550}, {0x1d552, 0x1d6a5}, {0x1d6a8, 0x1d6c0}, {0x1d6c2, 0x1d6da}, {0x1d6dc, 0x1d6fa}, {0x1d6fc, 0x1d714},
  {0x1d716, 0x1d734}, {0x1d736, 0x1d74e}, {0x1d750, 0x1d76e}, {0x1d770, 0x1d788}, {0x1d78a, 0x1d7a8}, {0x1d7aa, 0x1d7c2}, {0x1d7c4, 0x1d7cb}, {0x1d7ce, 0x1d7ff},
  {0x1da00, 0x1da36}, {0x1da3b, 0x1da6c}, {0x1da75, 0x1da75}, {0x1da84, 0x1da84}, {0x1da9b, 0x1da9f}, {0x1daa1, 0x1daaf}, {0x1df00, 0x1df1e}, {0x1df25, 0x1df2a},
  {0x1e000, 0x1e006}, {0x1e008, 0x1e018}, {0x1e01b, 0x1e021}, {0x1e023, 0x1e024}, {0x1e026, 0x1e02a}, {0x1e030, 0x1e06d}, {0x1e08f, 0x1e08f}, {0x1e100, 0x1e12c},
  {0x1e130, 0x1e13d}, {0x1e140, 0x1e149}, {0x1e14e, 0x1e14e}, {0x1e290, 0x1e2ae}, {0x1e2c0, 0x1e2f9}, {0x1e4d0, 0x1e4f9}, {0x1e7e0, 0x1e7e6}, {0x1e7e8, 0x1e7eb},
  {0x1e7ed, 0x1e7ee}, {0x1e7f0, 0x1e7fe}, {0x1e800, 0x1e8c4}, {0x1e8d0, 0x1e8d6}, {0x1e900, 0x1e94b}, {0x1e950, 0x1e959}, {0x1ee00, 0x1ee03}, {0x1ee05, 0x1ee1f},
  {0x1ee21, 0x1ee22}, {0x1ee24, 0x1ee24}, {0x1ee27, 0x1ee27}, {0x1ee29, 0x1ee32}, {0x1ee34, 0x1ee37}, {0x1ee39, 0x1ee39}, {0x1ee3b, 0x1ee3b}, {0x1ee42, 0x1ee42},
  {0x1ee47, 0x1ee47}, {0x1ee49, 0x1ee49}, {0x1ee4b, 0x1ee4b}, {0x1ee4d, 0x1ee4f}, {0x1ee51, 0x1ee52}, {0x1ee54, 0x1ee54}, {0x1ee57, 0x1ee57}, {0x1ee59, 0x1ee59},
  {0x1ee5b, 0x1ee5b}, {0x1ee5d, 0x1ee5d}, {0x1ee5f, 0x1ee5f}, {0x1ee61, 0x1ee62}, {0x1ee64, 0x1ee64}, {0x1ee67, 0x1ee6a}, {0x1ee6c, 0x1ee72}, {0x1ee74, 0x1ee77},
  {0x1ee79, 0x1ee7c}, {0x1ee7e, 0x1ee7e}, {0x1ee80, 0x1ee89}, {0x1ee8b, 0x1ee9b}, {0x1eea1, 0x1eea3}, {0x1eea5, 0x1eea9}, {0x1eeab, 0x1eebb}, {0x1fbf0, 0x1fbf9},
  {0x20000, 0x2a6df}, {0x2a700, 0x2b739}, {0x2b740, 0x2b81d}, {0x2b820, 0x2cea1}, {0x2ceb0, 0x2ebe0}, {0x2ebf0, 0x2ee5d}, {0x2f800, 0x2fa1d}, {0x30000, 0x3134a},
  {0x31350, 0x323af}, {0xe0100, 0xe01ef},
};

static bool ts_lex(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (eof) ADVANCE(120);
      ADVANCE_MAP(
        '!', 187,
        '"', 286,
        '#', 75,
        '%', 204,
        '&', 213,
        '\'', 277,
        '(', 124,
        ')', 127,
        '*', 200,
        '+', 195,
        ',', 126,
        '-', 190,
        '.', 253,
        '/', 202,
        '0', 259,
        ':', 237,
        ';', 226,
        '<', 220,
        '=', 236,
        '>', 216,
        '?', 238,
        'L', 298,
        'U', 300,
        '[', 233,
        '\\', 2,
        ']', 234,
        '^', 210,
        'u', 302,
        '{', 230,
        '|', 207,
        '}', 231,
        '~', 188,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(118);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 1:
      if (lookahead == '\n') SKIP(43);
      END_STATE();
    case 2:
      if (lookahead == '\n') SKIP(43);
      if (lookahead == '\r') SKIP(1);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 3:
      if (lookahead == '\n') SKIP(46);
      END_STATE();
    case 4:
      if (lookahead == '\n') SKIP(46);
      if (lookahead == '\r') SKIP(3);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 5:
      if (lookahead == '\n') SKIP(45);
      END_STATE();
    case 6:
      if (lookahead == '\n') SKIP(45);
      if (lookahead == '\r') SKIP(5);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 7:
      if (lookahead == '\n') SKIP(47);
      END_STATE();
    case 8:
      if (lookahead == '\n') SKIP(47);
      if (lookahead == '\r') SKIP(7);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 9:
      if (lookahead == '\n') SKIP(49);
      END_STATE();
    case 10:
      if (lookahead == '\n') SKIP(49);
      if (lookahead == '\r') SKIP(9);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 11:
      if (lookahead == '\n') SKIP(52);
      END_STATE();
    case 12:
      if (lookahead == '\n') SKIP(52);
      if (lookahead == '\r') SKIP(11);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 13:
      if (lookahead == '\n') SKIP(53);
      END_STATE();
    case 14:
      if (lookahead == '\n') SKIP(53);
      if (lookahead == '\r') SKIP(13);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 15:
      if (lookahead == '\n') SKIP(57);
      END_STATE();
    case 16:
      if (lookahead == '\n') SKIP(57);
      if (lookahead == '\r') SKIP(15);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 17:
      if (lookahead == '\n') SKIP(50);
      END_STATE();
    case 18:
      if (lookahead == '\n') SKIP(50);
      if (lookahead == '\r') SKIP(17);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 19:
      if (lookahead == '\n') SKIP(51);
      END_STATE();
    case 20:
      if (lookahead == '\n') SKIP(51);
      if (lookahead == '\r') SKIP(19);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 21:
      if (lookahead == '\n') SKIP(48);
      END_STATE();
    case 22:
      if (lookahead == '\n') SKIP(48);
      if (lookahead == '\r') SKIP(21);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 23:
      if (lookahead == '\n') SKIP(25);
      END_STATE();
    case 24:
      if (lookahead == '\n') SKIP(25);
      if (lookahead == '\r') SKIP(23);
      END_STATE();
    case 25:
      ADVANCE_MAP(
        '\n', 129,
        '!', 68,
        '%', 203,
        '&', 212,
        '(', 185,
        '*', 199,
        '+', 194,
        '-', 189,
        '/', 201,
        '<', 221,
        '=', 69,
        '>', 217,
      );
      if (lookahead == '\\') SKIP(24);
      if (lookahead == '^') ADVANCE(209);
      if (lookahead == '|') ADVANCE(208);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(25);
      END_STATE();
    case 26:
      if (lookahead == '\n') SKIP(56);
      END_STATE();
    case 27:
      if (lookahead == '\n') SKIP(56);
      if (lookahead == '\r') SKIP(26);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 28:
      if (lookahead == '\n') SKIP(54);
      END_STATE();
    case 29:
      if (lookahead == '\n') SKIP(54);
      if (lookahead == '\r') SKIP(28);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 30:
      if (lookahead == '\n') SKIP(58);
      if (lookahead == '\'') ADVANCE(277);
      if (lookahead == '/') ADVANCE(280);
      if (lookahead == '\\') ADVANCE(279);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(281);
      if (lookahead != 0) ADVANCE(278);
      END_STATE();
    case 31:
      if (lookahead == '\n') ADVANCE(293);
      if (lookahead == '\r') ADVANCE(292);
      if (lookahead == 'U') ADVANCE(116);
      if (lookahead == 'u') ADVANCE(108);
      if (lookahead == 'x') ADVANCE(104);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(295);
      if (lookahead != 0) ADVANCE(292);
      END_STATE();
    case 32:
      if (lookahead == '\n') ADVANCE(122);
      if (lookahead == '\r') ADVANCE(36);
      if (lookahead == '(') ADVANCE(124);
      if (lookahead == '/') ADVANCE(150);
      if (lookahead == '\\') ADVANCE(145);
      if (('\t' <= lookahead && lookahead <= '\f') ||
          lookahead == ' ') SKIP(66);
      if (lookahead != 0) ADVANCE(152);
      END_STATE();
    case 33:
      if (lookahead == '\n') ADVANCE(122);
      if (lookahead == '\r') ADVANCE(36);
      if (lookahead == '/') ADVANCE(150);
      if (lookahead == '\\') ADVANCE(145);
      if (('\t' <= lookahead && lookahead <= '\f') ||
          lookahead == ' ') SKIP(66);
      if (lookahead != 0) ADVANCE(152);
      END_STATE();
    case 34:
      if (lookahead == '\n') ADVANCE(122);
      if (lookahead == '\r') ADVANCE(35);
      if (lookahead == '(') ADVANCE(185);
      if (lookahead == '/') ADVANCE(60);
      if (lookahead == '\\') SKIP(39);
      if (('\t' <= lookahead && lookahead <= '\f') ||
          lookahead == ' ') SKIP(59);
      END_STATE();
    case 35:
      if (lookahead == '\n') ADVANCE(122);
      if (lookahead == '(') ADVANCE(185);
      if (lookahead == '/') ADVANCE(60);
      if (lookahead == '\\') SKIP(39);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(59);
      END_STATE();
    case 36:
      if (lookahead == '\n') ADVANCE(122);
      if (lookahead == '/') ADVANCE(150);
      if (lookahead == '\\') ADVANCE(145);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(66);
      if (lookahead != 0) ADVANCE(152);
      END_STATE();
    case 37:
      if (lookahead == '\n') SKIP(55);
      if (lookahead == '"') ADVANCE(286);
      if (lookahead == '/') ADVANCE(287);
      if (lookahead == '\\') ADVANCE(31);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(290);
      if (lookahead != 0) ADVANCE(291);
      END_STATE();
    case 38:
      if (lookahead == '\n') SKIP(59);
      END_STATE();
    case 39:
      if (lookahead == '\n') SKIP(59);
      if (lookahead == '\r') SKIP(38);
      END_STATE();
    case 40:
      if (lookahead == '\n') SKIP(44);
      END_STATE();
    case 41:
      if (lookahead == '\n') SKIP(44);
      if (lookahead == '\r') SKIP(40);
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 42:
      if (lookahead == '\r') ADVANCE(319);
      if (lookahead == '\\') ADVANCE(313);
      if (lookahead != 0) ADVANCE(318);
      END_STATE();
    case 43:
      ADVANCE_MAP(
        '!', 187,
        '"', 286,
        '#', 75,
        '%', 204,
        '&', 213,
        '\'', 277,
        '(', 185,
        ')', 127,
        '*', 200,
        '+', 195,
        ',', 126,
        '-', 190,
        '.', 253,
        '/', 202,
        '0', 259,
        ':', 237,
        ';', 226,
        '<', 220,
        '=', 236,
        '>', 216,
        '?', 238,
        'L', 298,
        'U', 300,
        '[', 233,
        '\\', 2,
        ']', 234,
        '^', 210,
        'u', 302,
        '{', 230,
        '|', 207,
        '}', 231,
        '~', 188,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(43);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 44:
      ADVANCE_MAP(
        '!', 187,
        '"', 286,
        '#', 83,
        '%', 204,
        '&', 213,
        '\'', 277,
        '(', 185,
        ')', 127,
        '*', 200,
        '+', 195,
        ',', 126,
        '-', 190,
        '.', 253,
        '/', 202,
        '0', 259,
        ':', 237,
        ';', 226,
        '<', 220,
        '=', 236,
        '>', 216,
        '?', 238,
        'L', 298,
        'U', 300,
        '[', 232,
        '\\', 41,
        ']', 234,
        '^', 210,
        'u', 302,
        '{', 230,
        '|', 207,
        '}', 231,
        '~', 188,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(44);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 45:
      ADVANCE_MAP(
        '!', 186,
        '"', 286,
        '#', 75,
        '&', 211,
        '\'', 277,
        '(', 185,
        '*', 199,
        '+', 196,
        ',', 126,
        '-', 191,
        '.', 96,
        '/', 60,
        '0', 259,
        ':', 67,
        ';', 226,
        'L', 298,
        'U', 300,
        '[', 73,
        '\\', 6,
        ']', 74,
        'u', 302,
        '{', 230,
        '~', 188,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(45);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 46:
      ADVANCE_MAP(
        '!', 186,
        '"', 286,
        '#', 79,
        '&', 211,
        '\'', 277,
        '(', 185,
        ')', 127,
        '*', 199,
        '+', 196,
        ',', 126,
        '-', 191,
        '.', 254,
        '/', 60,
        '0', 259,
        ':', 237,
        ';', 226,
        '=', 235,
        'L', 298,
        'U', 300,
        '[', 233,
        '\\', 4,
        ']', 234,
        'u', 302,
        '{', 230,
        '}', 231,
        '~', 188,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(46);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 47:
      ADVANCE_MAP(
        '!', 186,
        '"', 286,
        '#', 77,
        '&', 211,
        '\'', 277,
        '(', 185,
        '*', 199,
        '+', 196,
        '-', 191,
        '.', 96,
        '/', 60,
        '0', 259,
        ';', 226,
        'L', 298,
        'U', 300,
        '[', 73,
        '\\', 8,
        'u', 302,
        '{', 230,
        '~', 188,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(47);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 48:
      ADVANCE_MAP(
        '!', 186,
        '\'', 277,
        '(', 185,
        ')', 127,
        '+', 198,
        '-', 193,
        '.', 96,
        '/', 60,
        '0', 259,
        'L', 306,
        'U', 307,
        '\\', 22,
        'u', 308,
        '~', 188,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(48);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 49:
      ADVANCE_MAP(
        '!', 68,
        '"', 286,
        '#', 83,
        '%', 204,
        '&', 213,
        '(', 185,
        ')', 127,
        '*', 200,
        '+', 197,
        ',', 126,
        '-', 192,
        '.', 252,
        '/', 202,
        ':', 237,
        ';', 226,
        '<', 220,
        '=', 236,
        '>', 216,
        '?', 238,
        'L', 299,
        'U', 301,
        '[', 233,
        '\\', 10,
        ']', 234,
        '^', 210,
        'u', 303,
        '|', 207,
        '}', 231,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(49);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 50:
      ADVANCE_MAP(
        '!', 68,
        '#', 83,
        '%', 204,
        '&', 213,
        '(', 185,
        ')', 127,
        '*', 200,
        '+', 197,
        ',', 126,
        '-', 192,
        '.', 252,
        '/', 202,
        ':', 237,
        ';', 226,
        '<', 220,
        '=', 236,
        '>', 216,
        '?', 238,
        '[', 233,
        '\\', 18,
        ']', 234,
        '^', 210,
        '{', 230,
        '|', 207,
        '}', 231,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(50);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 51:
      ADVANCE_MAP(
        '!', 68,
        '#', 83,
        '%', 204,
        '&', 213,
        '(', 185,
        ')', 127,
        '*', 200,
        '+', 197,
        ',', 126,
        '-', 192,
        '.', 251,
        '/', 202,
        ':', 237,
        ';', 226,
        '<', 220,
        '=', 236,
        '>', 216,
        '?', 238,
        '[', 232,
        '\\', 20,
        ']', 74,
        '^', 210,
        '|', 207,
        '}', 231,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(51);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 52:
      ADVANCE_MAP(
        '!', 68,
        '#', 76,
        '%', 204,
        '&', 213,
        '(', 185,
        ')', 127,
        '*', 200,
        '+', 197,
        ',', 126,
        '-', 192,
        '.', 252,
        '/', 202,
        ':', 237,
        ';', 226,
        '<', 220,
        '=', 236,
        '>', 216,
        '?', 238,
        '[', 233,
        '\\', 12,
        '^', 210,
        '{', 230,
        '|', 207,
        '}', 231,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(52);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 53:
      ADVANCE_MAP(
        '!', 68,
        '#', 78,
        '%', 203,
        '&', 212,
        '(', 185,
        ')', 127,
        '*', 199,
        '+', 194,
        ',', 126,
        '-', 189,
        '/', 201,
        ';', 226,
        '<', 221,
        '=', 236,
        '>', 217,
        '[', 233,
        '\\', 14,
        '^', 209,
        '{', 230,
        '|', 208,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(53);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 54:
      if (lookahead == '"') ADVANCE(286);
      if (lookahead == '/') ADVANCE(60);
      if (lookahead == '<') ADVANCE(70);
      if (lookahead == 'L') ADVANCE(299);
      if (lookahead == 'U') ADVANCE(301);
      if (lookahead == '\\') ADVANCE(29);
      if (lookahead == 'u') ADVANCE(303);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(54);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 55:
      if (lookahead == '"') ADVANCE(286);
      if (lookahead == '/') ADVANCE(60);
      if (lookahead == '\\') ADVANCE(31);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(55);
      END_STATE();
    case 56:
      if (lookahead == '#') ADVANCE(93);
      if (lookahead == '/') ADVANCE(60);
      if (lookahead == '\\') ADVANCE(27);
      if (lookahead == '}') ADVANCE(231);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(56);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 57:
      if (lookahead == '#') ADVANCE(80);
      if (lookahead == '/') ADVANCE(60);
      if (lookahead == '[') ADVANCE(73);
      if (lookahead == '\\') ADVANCE(16);
      if (lookahead == '}') ADVANCE(231);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(57);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 58:
      if (lookahead == '\'') ADVANCE(277);
      if (lookahead == '/') ADVANCE(60);
      if (lookahead == '\\') ADVANCE(31);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(58);
      END_STATE();
    case 59:
      if (lookahead == '(') ADVANCE(185);
      if (lookahead == '/') ADVANCE(60);
      if (lookahead == '\\') SKIP(39);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(59);
      END_STATE();
    case 60:
      if (lookahead == '*') ADVANCE(62);
      if (lookahead == '/') ADVANCE(318);
      END_STATE();
    case 61:
      if (lookahead == '*') ADVANCE(61);
      if (lookahead == '/') ADVANCE(311);
      if (lookahead != 0) ADVANCE(62);
      END_STATE();
    case 62:
      if (lookahead == '*') ADVANCE(61);
      if (lookahead != 0) ADVANCE(62);
      END_STATE();
    case 63:
      if (lookahead == '*') ADVANCE(61);
      if (lookahead != 0) ADVANCE(143);
      END_STATE();
    case 64:
      if (lookahead == '.') ADVANCE(96);
      if (lookahead == '0') ADVANCE(257);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(258);
      if (('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(267);
      END_STATE();
    case 65:
      if (lookahead == '.') ADVANCE(125);
      END_STATE();
    case 66:
      if (lookahead == '/') ADVANCE(150);
      if (lookahead == '\\') ADVANCE(145);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(66);
      if (lookahead != 0) ADVANCE(152);
      END_STATE();
    case 67:
      if (lookahead == ':') ADVANCE(227);
      END_STATE();
    case 68:
      if (lookahead == '=') ADVANCE(215);
      END_STATE();
    case 69:
      if (lookahead == '=') ADVANCE(214);
      END_STATE();
    case 70:
      if (lookahead == '>') ADVANCE(296);
      if (lookahead == '\\') ADVANCE(71);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(70);
      END_STATE();
    case 71:
      if (lookahead == '>') ADVANCE(297);
      if (lookahead == '\\') ADVANCE(71);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(70);
      END_STATE();
    case 72:
      if (lookahead == 'U') ADVANCE(115);
      if (lookahead == 'u') ADVANCE(107);
      END_STATE();
    case 73:
      if (lookahead == '[') ADVANCE(228);
      END_STATE();
    case 74:
      if (lookahead == ']') ADVANCE(229);
      END_STATE();
    case 75:
      if (lookahead == 'd') ADVANCE(159);
      if (lookahead == 'e') ADVANCE(179);
      if (lookahead == 'i') ADVANCE(167);
      if (lookahead == '\t' ||
          lookahead == ' ') ADVANCE(75);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 76:
      if (lookahead == 'd') ADVANCE(159);
      if (lookahead == 'e') ADVANCE(179);
      if (lookahead == 'i') ADVANCE(168);
      if (lookahead == '\t' ||
          lookahead == ' ') ADVANCE(76);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 77:
      if (lookahead == 'd') ADVANCE(159);
      if (lookahead == 'e') ADVANCE(181);
      if (lookahead == 'i') ADVANCE(167);
      if (lookahead == '\t' ||
          lookahead == ' ') ADVANCE(77);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 78:
      if (lookahead == 'd') ADVANCE(159);
      if (lookahead == 'e') ADVANCE(181);
      if (lookahead == 'i') ADVANCE(168);
      if (lookahead == '\t' ||
          lookahead == ' ') ADVANCE(78);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 79:
      if (lookahead == 'd') ADVANCE(159);
      if (lookahead == 'i') ADVANCE(167);
      if (lookahead == '\t' ||
          lookahead == ' ') ADVANCE(79);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 80:
      if (lookahead == 'd') ADVANCE(159);
      if (lookahead == 'i') ADVANCE(168);
      if (lookahead == '\t' ||
          lookahead == ' ') ADVANCE(80);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 81:
      if (lookahead == 'd') ADVANCE(92);
      END_STATE();
    case 82:
      if (lookahead == 'd') ADVANCE(86);
      END_STATE();
    case 83:
      if (lookahead == 'e') ADVANCE(94);
      if (lookahead == '\t' ||
          lookahead == ' ') ADVANCE(83);
      END_STATE();
    case 84:
      if (lookahead == 'e') ADVANCE(134);
      END_STATE();
    case 85:
      if (lookahead == 'e') ADVANCE(89);
      END_STATE();
    case 86:
      if (lookahead == 'e') ADVANCE(90);
      END_STATE();
    case 87:
      if (lookahead == 'f') ADVANCE(136);
      END_STATE();
    case 88:
      if (lookahead == 'f') ADVANCE(130);
      END_STATE();
    case 89:
      if (lookahead == 'f') ADVANCE(138);
      END_STATE();
    case 90:
      if (lookahead == 'f') ADVANCE(140);
      END_STATE();
    case 91:
      if (lookahead == 'i') ADVANCE(87);
      if (lookahead == 's') ADVANCE(84);
      END_STATE();
    case 92:
      if (lookahead == 'i') ADVANCE(88);
      END_STATE();
    case 93:
      if (lookahead == 'i') ADVANCE(168);
      if (lookahead == '\t' ||
          lookahead == ' ') ADVANCE(93);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 94:
      if (lookahead == 'l') ADVANCE(91);
      if (lookahead == 'n') ADVANCE(81);
      END_STATE();
    case 95:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(261);
      END_STATE();
    case 96:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(256);
      END_STATE();
    case 97:
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(258);
      if (('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(267);
      END_STATE();
    case 98:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(310);
      END_STATE();
    case 99:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(264);
      END_STATE();
    case 100:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(267);
      END_STATE();
    case 101:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(262);
      END_STATE();
    case 102:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(292);
      END_STATE();
    case 103:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(98);
      END_STATE();
    case 104:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(102);
      END_STATE();
    case 105:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(103);
      END_STATE();
    case 106:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(104);
      END_STATE();
    case 107:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(105);
      END_STATE();
    case 108:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(106);
      END_STATE();
    case 109:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(107);
      END_STATE();
    case 110:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(108);
      END_STATE();
    case 111:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(109);
      END_STATE();
    case 112:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(110);
      END_STATE();
    case 113:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(111);
      END_STATE();
    case 114:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(112);
      END_STATE();
    case 115:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(113);
      END_STATE();
    case 116:
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'F') ||
          ('a' <= lookahead && lookahead <= 'f')) ADVANCE(114);
      END_STATE();
    case 117:
      if (lookahead != 0 &&
          lookahead != '*') ADVANCE(152);
      END_STATE();
    case 118:
      if (eof) ADVANCE(120);
      ADVANCE_MAP(
        '!', 187,
        '"', 286,
        '#', 75,
        '%', 204,
        '&', 213,
        '\'', 277,
        '(', 185,
        ')', 127,
        '*', 200,
        '+', 195,
        ',', 126,
        '-', 190,
        '.', 253,
        '/', 202,
        '0', 259,
        ':', 237,
        ';', 226,
        '<', 220,
        '=', 236,
        '>', 216,
        '?', 238,
        'L', 298,
        'U', 300,
        '[', 233,
        '\\', 2,
        ']', 234,
        '^', 210,
        'u', 302,
        '{', 230,
        '|', 207,
        '}', 231,
        '~', 188,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(118);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 119:
      if (eof) ADVANCE(120);
      ADVANCE_MAP(
        '!', 186,
        '"', 286,
        '#', 79,
        '&', 211,
        '\'', 277,
        '(', 185,
        ')', 127,
        '*', 199,
        '+', 196,
        ',', 126,
        '-', 191,
        '.', 254,
        '/', 60,
        '0', 259,
        ':', 237,
        ';', 226,
        '=', 235,
        'L', 298,
        'U', 300,
        '[', 233,
        '\\', 4,
        ']', 234,
        'u', 302,
        '{', 230,
        '}', 231,
        '~', 188,
      );
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(119);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_identifier_character_set_1, 670, lookahead)) ADVANCE(310);
      END_STATE();
    case 120:
      ACCEPT_TOKEN(ts_builtin_sym_end);
      END_STATE();
    case 121:
      ACCEPT_TOKEN(aux_sym_preproc_include_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 122:
      ACCEPT_TOKEN(aux_sym_preproc_include_token2);
      END_STATE();
    case 123:
      ACCEPT_TOKEN(aux_sym_preproc_def_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 124:
      ACCEPT_TOKEN(anon_sym_LPAREN);
      END_STATE();
    case 125:
      ACCEPT_TOKEN(anon_sym_DOT_DOT_DOT);
      END_STATE();
    case 126:
      ACCEPT_TOKEN(anon_sym_COMMA);
      END_STATE();
    case 127:
      ACCEPT_TOKEN(anon_sym_RPAREN);
      END_STATE();
    case 128:
      ACCEPT_TOKEN(aux_sym_preproc_if_token1);
      if (lookahead == 'd') ADVANCE(163);
      if (lookahead == 'n') ADVANCE(157);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 129:
      ACCEPT_TOKEN(anon_sym_LF);
      if (lookahead == '\n') ADVANCE(129);
      END_STATE();
    case 130:
      ACCEPT_TOKEN(aux_sym_preproc_if_token2);
      END_STATE();
    case 131:
      ACCEPT_TOKEN(aux_sym_preproc_if_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 132:
      ACCEPT_TOKEN(aux_sym_preproc_ifdef_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 133:
      ACCEPT_TOKEN(aux_sym_preproc_ifdef_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 134:
      ACCEPT_TOKEN(aux_sym_preproc_else_token1);
      END_STATE();
    case 135:
      ACCEPT_TOKEN(aux_sym_preproc_else_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 136:
      ACCEPT_TOKEN(aux_sym_preproc_elif_token1);
      if (lookahead == 'd') ADVANCE(85);
      if (lookahead == 'n') ADVANCE(82);
      END_STATE();
    case 137:
      ACCEPT_TOKEN(aux_sym_preproc_elif_token1);
      if (lookahead == 'd') ADVANCE(165);
      if (lookahead == 'n') ADVANCE(158);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 138:
      ACCEPT_TOKEN(aux_sym_preproc_elifdef_token1);
      END_STATE();
    case 139:
      ACCEPT_TOKEN(aux_sym_preproc_elifdef_token1);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 140:
      ACCEPT_TOKEN(aux_sym_preproc_elifdef_token2);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(aux_sym_preproc_elifdef_token2);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 142:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') ADVANCE(62);
      if (lookahead == '*') ADVANCE(142);
      if (lookahead == '/') ADVANCE(311);
      if (lookahead == '\\') ADVANCE(148);
      if (lookahead != 0) ADVANCE(143);
      END_STATE();
    case 143:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') ADVANCE(62);
      if (lookahead == '*') ADVANCE(142);
      if (lookahead == '/') ADVANCE(63);
      if (lookahead == '\\') ADVANCE(148);
      if (lookahead != 0) ADVANCE(143);
      END_STATE();
    case 144:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') ADVANCE(318);
      if (lookahead == '\r') ADVANCE(312);
      if (lookahead == '/') ADVANCE(315);
      if (lookahead == '\\') ADVANCE(314);
      if (lookahead != 0) ADVANCE(316);
      END_STATE();
    case 145:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') SKIP(66);
      if (lookahead == '\r') ADVANCE(146);
      if (lookahead == '/') ADVANCE(117);
      if (lookahead == '\\') ADVANCE(147);
      if (lookahead != 0) ADVANCE(152);
      END_STATE();
    case 146:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\n') SKIP(66);
      if (lookahead == '/') ADVANCE(117);
      if (lookahead == '\\') ADVANCE(147);
      if (lookahead != 0) ADVANCE(152);
      END_STATE();
    case 147:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\r') ADVANCE(153);
      if (lookahead == '/') ADVANCE(117);
      if (lookahead == '\\') ADVANCE(147);
      if (lookahead != 0) ADVANCE(152);
      END_STATE();
    case 148:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\r') ADVANCE(151);
      if (lookahead == '*') ADVANCE(142);
      if (lookahead == '/') ADVANCE(63);
      if (lookahead == '\\') ADVANCE(148);
      if (lookahead != 0) ADVANCE(143);
      END_STATE();
    case 149:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '\r') ADVANCE(317);
      if (lookahead == '/') ADVANCE(315);
      if (lookahead == '\\') ADVANCE(314);
      if (lookahead != 0) ADVANCE(316);
      END_STATE();
    case 150:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '*') ADVANCE(143);
      if (lookahead == '/') ADVANCE(315);
      if (lookahead == '\\') ADVANCE(147);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(152);
      END_STATE();
    case 151:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '*') ADVANCE(142);
      if (lookahead == '/') ADVANCE(63);
      if (lookahead == '\\') ADVANCE(148);
      if (lookahead != 0) ADVANCE(143);
      END_STATE();
    case 152:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(117);
      if (lookahead == '\\') ADVANCE(147);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(152);
      END_STATE();
    case 153:
      ACCEPT_TOKEN(sym_preproc_arg);
      if (lookahead == '/') ADVANCE(117);
      if (lookahead == '\\') ADVANCE(147);
      if (lookahead != 0) ADVANCE(152);
      END_STATE();
    case 154:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'c') ADVANCE(180);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 155:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'd') ADVANCE(178);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 156:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'd') ADVANCE(162);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 157:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'd') ADVANCE(164);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 158:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'd') ADVANCE(166);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 159:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'e') ADVANCE(169);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 160:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'e') ADVANCE(135);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 161:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'e') ADVANCE(123);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 162:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'e') ADVANCE(121);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 163:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'e') ADVANCE(172);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 164:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'e') ADVANCE(173);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 165:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'e') ADVANCE(174);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 166:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'e') ADVANCE(175);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 167:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'f') ADVANCE(128);
      if (lookahead == 'n') ADVANCE(154);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 168:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'f') ADVANCE(128);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 169:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'f') ADVANCE(176);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 170:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'f') ADVANCE(137);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 171:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'f') ADVANCE(131);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 172:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'f') ADVANCE(132);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 173:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'f') ADVANCE(133);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 174:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'f') ADVANCE(139);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 175:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'f') ADVANCE(141);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 176:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'i') ADVANCE(182);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 177:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'i') ADVANCE(170);
      if (lookahead == 's') ADVANCE(160);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 178:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'i') ADVANCE(171);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 179:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'l') ADVANCE(177);
      if (lookahead == 'n') ADVANCE(155);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 180:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'l') ADVANCE(183);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 181:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'n') ADVANCE(155);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 182:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'n') ADVANCE(161);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 183:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (lookahead == 'u') ADVANCE(156);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 184:
      ACCEPT_TOKEN(sym_preproc_directive);
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'Z') ||
          lookahead == '_' ||
          ('a' <= lookahead && lookahead <= 'z')) ADVANCE(184);
      END_STATE();
    case 185:
      ACCEPT_TOKEN(anon_sym_LPAREN2);
      END_STATE();
    case 186:
      ACCEPT_TOKEN(anon_sym_BANG);
      END_STATE();
    case 187:
      ACCEPT_TOKEN(anon_sym_BANG);
      if (lookahead == '=') ADVANCE(215);
      END_STATE();
    case 188:
      ACCEPT_TOKEN(anon_sym_TILDE);
      END_STATE();
    case 189:
      ACCEPT_TOKEN(anon_sym_DASH);
      END_STATE();
    case 190:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '-') ADVANCE(249);
      if (lookahead == '.') ADVANCE(96);
      if (lookahead == '0') ADVANCE(259);
      if (lookahead == '=') ADVANCE(243);
      if (lookahead == '>') ADVANCE(255);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      END_STATE();
    case 191:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '-') ADVANCE(249);
      if (lookahead == '.') ADVANCE(96);
      if (lookahead == '0') ADVANCE(259);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      END_STATE();
    case 192:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '-') ADVANCE(249);
      if (lookahead == '=') ADVANCE(243);
      if (lookahead == '>') ADVANCE(255);
      END_STATE();
    case 193:
      ACCEPT_TOKEN(anon_sym_DASH);
      if (lookahead == '.') ADVANCE(96);
      if (lookahead == '0') ADVANCE(259);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      END_STATE();
    case 194:
      ACCEPT_TOKEN(anon_sym_PLUS);
      END_STATE();
    case 195:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '+') ADVANCE(250);
      if (lookahead == '.') ADVANCE(96);
      if (lookahead == '0') ADVANCE(259);
      if (lookahead == '=') ADVANCE(242);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      END_STATE();
    case 196:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '+') ADVANCE(250);
      if (lookahead == '.') ADVANCE(96);
      if (lookahead == '0') ADVANCE(259);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      END_STATE();
    case 197:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '+') ADVANCE(250);
      if (lookahead == '=') ADVANCE(242);
      END_STATE();
    case 198:
      ACCEPT_TOKEN(anon_sym_PLUS);
      if (lookahead == '.') ADVANCE(96);
      if (lookahead == '0') ADVANCE(259);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      END_STATE();
    case 199:
      ACCEPT_TOKEN(anon_sym_STAR);
      END_STATE();
    case 200:
      ACCEPT_TOKEN(anon_sym_STAR);
      if (lookahead == '=') ADVANCE(239);
      END_STATE();
    case 201:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*') ADVANCE(62);
      if (lookahead == '/') ADVANCE(318);
      END_STATE();
    case 202:
      ACCEPT_TOKEN(anon_sym_SLASH);
      if (lookahead == '*') ADVANCE(62);
      if (lookahead == '/') ADVANCE(318);
      if (lookahead == '=') ADVANCE(240);
      END_STATE();
    case 203:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      END_STATE();
    case 204:
      ACCEPT_TOKEN(anon_sym_PERCENT);
      if (lookahead == '=') ADVANCE(241);
      END_STATE();
    case 205:
      ACCEPT_TOKEN(anon_sym_PIPE_PIPE);
      END_STATE();
    case 206:
      ACCEPT_TOKEN(anon_sym_AMP_AMP);
      END_STATE();
    case 207:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '=') ADVANCE(248);
      if (lookahead == '|') ADVANCE(205);
      END_STATE();
    case 208:
      ACCEPT_TOKEN(anon_sym_PIPE);
      if (lookahead == '|') ADVANCE(205);
      END_STATE();
    case 209:
      ACCEPT_TOKEN(anon_sym_CARET);
      END_STATE();
    case 210:
      ACCEPT_TOKEN(anon_sym_CARET);
      if (lookahead == '=') ADVANCE(247);
      END_STATE();
    case 211:
      ACCEPT_TOKEN(anon_sym_AMP);
      END_STATE();
    case 212:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(206);
      END_STATE();
    case 213:
      ACCEPT_TOKEN(anon_sym_AMP);
      if (lookahead == '&') ADVANCE(206);
      if (lookahead == '=') ADVANCE(246);
      END_STATE();
    case 214:
      ACCEPT_TOKEN(anon_sym_EQ_EQ);
      END_STATE();
    case 215:
      ACCEPT_TOKEN(anon_sym_BANG_EQ);
      END_STATE();
    case 216:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(218);
      if (lookahead == '>') ADVANCE(225);
      END_STATE();
    case 217:
      ACCEPT_TOKEN(anon_sym_GT);
      if (lookahead == '=') ADVANCE(218);
      if (lookahead == '>') ADVANCE(224);
      END_STATE();
    case 218:
      ACCEPT_TOKEN(anon_sym_GT_EQ);
      END_STATE();
    case 219:
      ACCEPT_TOKEN(anon_sym_LT_EQ);
      END_STATE();
    case 220:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(223);
      if (lookahead == '=') ADVANCE(219);
      END_STATE();
    case 221:
      ACCEPT_TOKEN(anon_sym_LT);
      if (lookahead == '<') ADVANCE(222);
      if (lookahead == '=') ADVANCE(219);
      END_STATE();
    case 222:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      END_STATE();
    case 223:
      ACCEPT_TOKEN(anon_sym_LT_LT);
      if (lookahead == '=') ADVANCE(244);
      END_STATE();
    case 224:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      END_STATE();
    case 225:
      ACCEPT_TOKEN(anon_sym_GT_GT);
      if (lookahead == '=') ADVANCE(245);
      END_STATE();
    case 226:
      ACCEPT_TOKEN(anon_sym_SEMI);
      END_STATE();
    case 227:
      ACCEPT_TOKEN(anon_sym_COLON_COLON);
      END_STATE();
    case 228:
      ACCEPT_TOKEN(anon_sym_LBRACK_LBRACK);
      END_STATE();
    case 229:
      ACCEPT_TOKEN(anon_sym_RBRACK_RBRACK);
      END_STATE();
    case 230:
      ACCEPT_TOKEN(anon_sym_LBRACE);
      END_STATE();
    case 231:
      ACCEPT_TOKEN(anon_sym_RBRACE);
      END_STATE();
    case 232:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      END_STATE();
    case 233:
      ACCEPT_TOKEN(anon_sym_LBRACK);
      if (lookahead == '[') ADVANCE(228);
      END_STATE();
    case 234:
      ACCEPT_TOKEN(anon_sym_RBRACK);
      END_STATE();
    case 235:
      ACCEPT_TOKEN(anon_sym_EQ);
      END_STATE();
    case 236:
      ACCEPT_TOKEN(anon_sym_EQ);
      if (lookahead == '=') ADVANCE(214);
      END_STATE();
    case 237:
      ACCEPT_TOKEN(anon_sym_COLON);
      END_STATE();
    case 238:
      ACCEPT_TOKEN(anon_sym_QMARK);
      END_STATE();
    case 239:
      ACCEPT_TOKEN(anon_sym_STAR_EQ);
      END_STATE();
    case 240:
      ACCEPT_TOKEN(anon_sym_SLASH_EQ);
      END_STATE();
    case 241:
      ACCEPT_TOKEN(anon_sym_PERCENT_EQ);
      END_STATE();
    case 242:
      ACCEPT_TOKEN(anon_sym_PLUS_EQ);
      END_STATE();
    case 243:
      ACCEPT_TOKEN(anon_sym_DASH_EQ);
      END_STATE();
    case 244:
      ACCEPT_TOKEN(anon_sym_LT_LT_EQ);
      END_STATE();
    case 245:
      ACCEPT_TOKEN(anon_sym_GT_GT_EQ);
      END_STATE();
    case 246:
      ACCEPT_TOKEN(anon_sym_AMP_EQ);
      END_STATE();
    case 247:
      ACCEPT_TOKEN(anon_sym_CARET_EQ);
      END_STATE();
    case 248:
      ACCEPT_TOKEN(anon_sym_PIPE_EQ);
      END_STATE();
    case 249:
      ACCEPT_TOKEN(anon_sym_DASH_DASH);
      END_STATE();
    case 250:
      ACCEPT_TOKEN(anon_sym_PLUS_PLUS);
      END_STATE();
    case 251:
      ACCEPT_TOKEN(anon_sym_DOT);
      END_STATE();
    case 252:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(65);
      END_STATE();
    case 253:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (lookahead == '.') ADVANCE(65);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(256);
      END_STATE();
    case 254:
      ACCEPT_TOKEN(anon_sym_DOT);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(256);
      END_STATE();
    case 255:
      ACCEPT_TOKEN(anon_sym_DASH_GT);
      END_STATE();
    case 256:
      ACCEPT_TOKEN(sym_number_literal);
      if (lookahead == '\'') ADVANCE(96);
      if (lookahead == 'E' ||
          lookahead == 'P' ||
          lookahead == 'e' ||
          lookahead == 'p') ADVANCE(269);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(256);
      if (set_contains(sym_number_literal_character_set_13, 13, lookahead)) ADVANCE(272);
      END_STATE();
    case 257:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 97,
        '.', 270,
        'B', 266,
        'b', 266,
        'E', 265,
        'e', 265,
        'P', 269,
        'p', 269,
        'X', 100,
        'x', 100,
        'A', 267,
        'C', 267,
        'a', 267,
        'c', 267,
        'D', 267,
        'F', 267,
        'd', 267,
        'f', 267,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(258);
      END_STATE();
    case 258:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 97,
        '.', 270,
        'E', 265,
        'e', 265,
        'P', 269,
        'p', 269,
        'A', 267,
        'C', 267,
        'a', 267,
        'c', 267,
        'B', 267,
        'D', 267,
        'F', 267,
        'b', 267,
        'd', 267,
        'f', 267,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(258);
      END_STATE();
    case 259:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 95,
        '.', 270,
        'B', 268,
        'b', 268,
        'X', 64,
        'x', 64,
        'E', 269,
        'P', 269,
        'e', 269,
        'p', 269,
        'D', 272,
        'F', 272,
        'L', 272,
        'U', 272,
        'W', 272,
        'd', 272,
        'f', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(261);
      END_STATE();
    case 260:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 95,
        '.', 270,
        'B', 271,
        'b', 271,
        'X', 100,
        'x', 100,
        'E', 269,
        'P', 269,
        'e', 269,
        'p', 269,
        'D', 272,
        'F', 272,
        'L', 272,
        'U', 272,
        'W', 272,
        'd', 272,
        'f', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(261);
      END_STATE();
    case 261:
      ACCEPT_TOKEN(sym_number_literal);
      if (lookahead == '\'') ADVANCE(95);
      if (lookahead == '.') ADVANCE(270);
      if (lookahead == 'E' ||
          lookahead == 'P' ||
          lookahead == 'e' ||
          lookahead == 'p') ADVANCE(269);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_number_literal_character_set_13, 13, lookahead)) ADVANCE(272);
      END_STATE();
    case 262:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 101,
        'B', 262,
        'D', 262,
        'F', 262,
        'b', 262,
        'd', 262,
        'f', 262,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'E') ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(262);
      END_STATE();
    case 263:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 99,
        '+', 101,
        '-', 101,
        'E', 263,
        'e', 263,
        'P', 269,
        'p', 269,
        'B', 264,
        'D', 264,
        'F', 264,
        'b', 264,
        'd', 264,
        'f', 264,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'C') ||
          ('a' <= lookahead && lookahead <= 'c')) ADVANCE(264);
      END_STATE();
    case 264:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 99,
        'E', 263,
        'e', 263,
        'P', 269,
        'p', 269,
        'B', 264,
        'D', 264,
        'F', 264,
        'b', 264,
        'd', 264,
        'f', 264,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'C') ||
          ('a' <= lookahead && lookahead <= 'c')) ADVANCE(264);
      END_STATE();
    case 265:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 100,
        '.', 270,
        '+', 101,
        '-', 101,
        'E', 265,
        'e', 265,
        'P', 269,
        'p', 269,
        'B', 267,
        'D', 267,
        'F', 267,
        'b', 267,
        'd', 267,
        'f', 267,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'C') ||
          ('a' <= lookahead && lookahead <= 'c')) ADVANCE(267);
      END_STATE();
    case 266:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 100,
        '.', 270,
        'E', 265,
        'e', 265,
        'P', 269,
        'p', 269,
        'A', 267,
        'C', 267,
        'a', 267,
        'c', 267,
        'B', 267,
        'D', 267,
        'F', 267,
        'b', 267,
        'd', 267,
        'f', 267,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(258);
      END_STATE();
    case 267:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '\'', 100,
        '.', 270,
        'E', 265,
        'e', 265,
        'P', 269,
        'p', 269,
        'B', 267,
        'D', 267,
        'F', 267,
        'b', 267,
        'd', 267,
        'f', 267,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'C') ||
          ('a' <= lookahead && lookahead <= 'c')) ADVANCE(267);
      END_STATE();
    case 268:
      ACCEPT_TOKEN(sym_number_literal);
      if (lookahead == '.') ADVANCE(96);
      if (lookahead == '0') ADVANCE(260);
      if (('1' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_number_literal_character_set_13, 13, lookahead)) ADVANCE(272);
      END_STATE();
    case 269:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        '+', 101,
        '-', 101,
        'B', 262,
        'D', 262,
        'F', 262,
        'b', 262,
        'd', 262,
        'f', 262,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'E') ||
          ('a' <= lookahead && lookahead <= 'e')) ADVANCE(262);
      END_STATE();
    case 270:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        'E', 263,
        'e', 263,
        'P', 269,
        'p', 269,
        'B', 264,
        'D', 264,
        'F', 264,
        'b', 264,
        'd', 264,
        'f', 264,
        'L', 272,
        'U', 272,
        'W', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      if (('0' <= lookahead && lookahead <= '9') ||
          ('A' <= lookahead && lookahead <= 'C') ||
          ('a' <= lookahead && lookahead <= 'c')) ADVANCE(264);
      END_STATE();
    case 271:
      ACCEPT_TOKEN(sym_number_literal);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(261);
      if (set_contains(sym_number_literal_character_set_13, 13, lookahead)) ADVANCE(272);
      END_STATE();
    case 272:
      ACCEPT_TOKEN(sym_number_literal);
      ADVANCE_MAP(
        'B', 272,
        'D', 272,
        'F', 272,
        'L', 272,
        'U', 272,
        'W', 272,
        'b', 272,
        'd', 272,
        'f', 272,
        'l', 272,
        'u', 272,
        'w', 272,
      );
      END_STATE();
    case 273:
      ACCEPT_TOKEN(anon_sym_L_SQUOTE);
      END_STATE();
    case 274:
      ACCEPT_TOKEN(anon_sym_u_SQUOTE);
      END_STATE();
    case 275:
      ACCEPT_TOKEN(anon_sym_U_SQUOTE);
      END_STATE();
    case 276:
      ACCEPT_TOKEN(anon_sym_u8_SQUOTE);
      END_STATE();
    case 277:
      ACCEPT_TOKEN(anon_sym_SQUOTE);
      END_STATE();
    case 278:
      ACCEPT_TOKEN(aux_sym_char_literal_token1);
      END_STATE();
    case 279:
      ACCEPT_TOKEN(aux_sym_char_literal_token1);
      if (lookahead == '\n') ADVANCE(293);
      if (lookahead == '\r') ADVANCE(292);
      if (lookahead == 'U') ADVANCE(116);
      if (lookahead == 'u') ADVANCE(108);
      if (lookahead == 'x') ADVANCE(104);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(295);
      if (lookahead != 0) ADVANCE(292);
      END_STATE();
    case 280:
      ACCEPT_TOKEN(aux_sym_char_literal_token1);
      if (lookahead == '*') ADVANCE(62);
      if (lookahead == '/') ADVANCE(318);
      END_STATE();
    case 281:
      ACCEPT_TOKEN(aux_sym_char_literal_token1);
      if (lookahead == '\\') ADVANCE(31);
      END_STATE();
    case 282:
      ACCEPT_TOKEN(anon_sym_L_DQUOTE);
      END_STATE();
    case 283:
      ACCEPT_TOKEN(anon_sym_u_DQUOTE);
      END_STATE();
    case 284:
      ACCEPT_TOKEN(anon_sym_U_DQUOTE);
      END_STATE();
    case 285:
      ACCEPT_TOKEN(anon_sym_u8_DQUOTE);
      END_STATE();
    case 286:
      ACCEPT_TOKEN(anon_sym_DQUOTE);
      END_STATE();
    case 287:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '*') ADVANCE(289);
      if (lookahead == '/') ADVANCE(291);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(291);
      END_STATE();
    case 288:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '*') ADVANCE(288);
      if (lookahead == '/') ADVANCE(291);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(289);
      END_STATE();
    case 289:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '*') ADVANCE(288);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(289);
      END_STATE();
    case 290:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead == '/') ADVANCE(287);
      if (lookahead == '\t' ||
          (0x0b <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') ADVANCE(290);
      if (lookahead != 0 &&
          (lookahead < '\t' || '\r' < lookahead) &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(291);
      END_STATE();
    case 291:
      ACCEPT_TOKEN(aux_sym_string_literal_token1);
      if (lookahead != 0 &&
          lookahead != '\n' &&
          lookahead != '"' &&
          lookahead != '\\') ADVANCE(291);
      END_STATE();
    case 292:
      ACCEPT_TOKEN(sym_escape_sequence);
      END_STATE();
    case 293:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (lookahead == '\\') ADVANCE(31);
      END_STATE();
    case 294:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(292);
      END_STATE();
    case 295:
      ACCEPT_TOKEN(sym_escape_sequence);
      if (('0' <= lookahead && lookahead <= '9')) ADVANCE(294);
      END_STATE();
    case 296:
      ACCEPT_TOKEN(sym_system_lib_string);
      END_STATE();
    case 297:
      ACCEPT_TOKEN(sym_system_lib_string);
      if (lookahead == '>') ADVANCE(296);
      if (lookahead == '\\') ADVANCE(71);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(70);
      END_STATE();
    case 298:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(282);
      if (lookahead == '\'') ADVANCE(273);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 299:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(282);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 300:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(284);
      if (lookahead == '\'') ADVANCE(275);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 301:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(284);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 302:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(283);
      if (lookahead == '\'') ADVANCE(274);
      if (lookahead == '8') ADVANCE(304);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 303:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(283);
      if (lookahead == '8') ADVANCE(305);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 304:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(285);
      if (lookahead == '\'') ADVANCE(276);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 305:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '"') ADVANCE(285);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 306:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '\'') ADVANCE(273);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 307:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '\'') ADVANCE(275);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 308:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '\'') ADVANCE(274);
      if (lookahead == '8') ADVANCE(309);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 309:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '\'') ADVANCE(276);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 310:
      ACCEPT_TOKEN(sym_identifier);
      if (lookahead == '\\') ADVANCE(72);
      if (set_contains(sym_identifier_character_set_2, 778, lookahead)) ADVANCE(310);
      END_STATE();
    case 311:
      ACCEPT_TOKEN(sym_comment);
      END_STATE();
    case 312:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\n') ADVANCE(318);
      if (lookahead == '/') ADVANCE(315);
      if (lookahead == '\\') ADVANCE(149);
      if (lookahead != 0) ADVANCE(316);
      END_STATE();
    case 313:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\r') ADVANCE(319);
      if (lookahead == '\\') ADVANCE(313);
      if (lookahead != 0) ADVANCE(318);
      END_STATE();
    case 314:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\r') ADVANCE(317);
      if (lookahead == '/') ADVANCE(315);
      if (lookahead == '\\') ADVANCE(314);
      if (lookahead != 0) ADVANCE(316);
      END_STATE();
    case 315:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '*') ADVANCE(318);
      if (lookahead == '\\') ADVANCE(144);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(316);
      END_STATE();
    case 316:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '/') ADVANCE(315);
      if (lookahead == '\\') ADVANCE(149);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(316);
      END_STATE();
    case 317:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '/') ADVANCE(315);
      if (lookahead == '\\') ADVANCE(149);
      if (lookahead != 0) ADVANCE(316);
      END_STATE();
    case 318:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\\') ADVANCE(42);
      if (lookahead != 0 &&
          lookahead != '\n') ADVANCE(318);
      END_STATE();
    case 319:
      ACCEPT_TOKEN(sym_comment);
      if (lookahead == '\\') ADVANCE(42);
      if (lookahead != 0) ADVANCE(318);
      END_STATE();
    default:
      return false;
  }
}

static bool ts_lex_keywords(TSLexer *lexer, TSStateId state) {
  START_LEXER();
  eof = lexer->eof(lexer);
  switch (state) {
    case 0:
      if (lookahead == 'F') ADVANCE(1);
      if (lookahead == 'N') ADVANCE(2);
      if (lookahead == 'T') ADVANCE(3);
      if (lookahead == '\\') SKIP(4);
      if (lookahead == '_') ADVANCE(5);
      if (lookahead == 'a') ADVANCE(6);
      if (lookahead == 'b') ADVANCE(7);
      if (lookahead == 'c') ADVANCE(8);
      if (lookahead == 'd') ADVANCE(9);
      if (lookahead == 'e') ADVANCE(10);
      if (lookahead == 'f') ADVANCE(11);
      if (lookahead == 'g') ADVANCE(12);
      if (lookahead == 'i') ADVANCE(13);
      if (lookahead == 'l') ADVANCE(14);
      if (lookahead == 'm') ADVANCE(15);
      if (lookahead == 'n') ADVANCE(16);
      if (lookahead == 'o') ADVANCE(17);
      if (lookahead == 'p') ADVANCE(18);
      if (lookahead == 'r') ADVANCE(19);
      if (lookahead == 's') ADVANCE(20);
      if (lookahead == 't') ADVANCE(21);
      if (lookahead == 'u') ADVANCE(22);
      if (lookahead == 'v') ADVANCE(23);
      if (lookahead == 'w') ADVANCE(24);
      if (('\t' <= lookahead && lookahead <= '\r') ||
          lookahead == ' ') SKIP(0);
      END_STATE();
    case 1:
      if (lookahead == 'A') ADVANCE(25);
      END_STATE();
    case 2:
      if (lookahead == 'U') ADVANCE(26);
      END_STATE();
    case 3:
      if (lookahead == 'R') ADVANCE(27);
      END_STATE();
    case 4:
      if (lookahead == '\n') SKIP(0);
      if (lookahead == '\r') SKIP(28);
      END_STATE();
    case 5:
      if (lookahead == 'A') ADVANCE(29);
      if (lookahead == 'G') ADVANCE(30);
      if (lookahead == 'N') ADVANCE(31);
      if (lookahead == '_') ADVANCE(32);
      if (lookahead == 'a') ADVANCE(33);
      if (lookahead == 'u') ADVANCE(34);
      END_STATE();
    case 6:
      if (lookahead == 'l') ADVANCE(35);
      if (lookahead == 's') ADVANCE(36);
      if (lookahead == 'u') ADVANCE(37);
      END_STATE();
    case 7:
      if (lookahead == 'o') ADVANCE(38);
      if (lookahead == 'r') ADVANCE(39);
      END_STATE();
    case 8:
      if (lookahead == 'a') ADVANCE(40);
      if (lookahead == 'h') ADVANCE(41);
      if (lookahead == 'o') ADVANCE(42);
      END_STATE();
    case 9:
      if (lookahead == 'e') ADVANCE(43);
      if (lookahead == 'o') ADVANCE(44);
      END_STATE();
    case 10:
      if (lookahead == 'l') ADVANCE(45);
      if (lookahead == 'n') ADVANCE(46);
      if (lookahead == 'x') ADVANCE(47);
      END_STATE();
    case 11:
      if (lookahead == 'a') ADVANCE(48);
      if (lookahead == 'l') ADVANCE(49);
      if (lookahead == 'o') ADVANCE(50);
      END_STATE();
    case 12:
      if (lookahead == 'o') ADVANCE(51);
      END_STATE();
    case 13:
      if (lookahead == 'f') ADVANCE(52);
      if (lookahead == 'n') ADVANCE(53);
      END_STATE();
    case 14:
      if (lookahead == 'o') ADVANCE(54);
      END_STATE();
    case 15:
      if (lookahead == 'a') ADVANCE(55);
      END_STATE();
    case 16:
      if (lookahead == 'o') ADVANCE(56);
      if (lookahead == 'u') ADVANCE(57);
      END_STATE();
    case 17:
      if (lookahead == 'f') ADVANCE(58);
      END_STATE();
    case 18:
      if (lookahead == 't') ADVANCE(59);
      END_STATE();
    case 19:
      if (lookahead == 'e') ADVANCE(60);
      END_STATE();
    case 20:
      if (lookahead == 'h') ADVANCE(61);
      if (lookahead == 'i') ADVANCE(62);
      if (lookahead == 's') ADVANCE(63);
      if (lookahead == 't') ADVANCE(64);
      if (lookahead == 'w') ADVANCE(65);
      END_STATE();
    case 21:
      if (lookahead == 'h') ADVANCE(66);
      if (lookahead == 'r') ADVANCE(67);
      if (lookahead == 'y') ADVANCE(68);
      END_STATE();
    case 22:
      if (lookahead == 'i') ADVANCE(69);
      if (lookahead == 'n') ADVANCE(70);
      END_STATE();
    case 23:
      if (lookahead == 'o') ADVANCE(71);
      END_STATE();
    case 24:
      if (lookahead == 'h') ADVANCE(72);
      END_STATE();
    case 25:
      if (lookahead == 'L') ADVANCE(73);
      END_STATE();
    case 26:
      if (lookahead == 'L') ADVANCE(74);
      END_STATE();
    case 27:
      if (lookahead == 'U') ADVANCE(75);
      END_STATE();
    case 28:
      if (lookahead == '\n') SKIP(0);
      END_STATE();
    case 29:
      if (lookahead == 'l') ADVANCE(76);
      if (lookahead == 't') ADVANCE(77);
      END_STATE();
    case 30:
      if (lookahead == 'e') ADVANCE(78);
      END_STATE();
    case 31:
      if (lookahead == 'o') ADVANCE(79);
      END_STATE();
    case 32:
      ADVANCE_MAP(
        'a', 80,
        'b', 81,
        'c', 82,
        'd', 83,
        'e', 84,
        'f', 85,
        'i', 86,
        'l', 87,
        'r', 88,
        's', 89,
        't', 90,
        'u', 91,
        'v', 92,
      );
      END_STATE();
    case 33:
      if (lookahead == 'l') ADVANCE(93);
      END_STATE();
    case 34:
      if (lookahead == 'n') ADVANCE(94);
      END_STATE();
    case 35:
      if (lookahead == 'i') ADVANCE(95);
      END_STATE();
    case 36:
      if (lookahead == 'm') ADVANCE(96);
      END_STATE();
    case 37:
      if (lookahead == 't') ADVANCE(97);
      END_STATE();
    case 38:
      if (lookahead == 'o') ADVANCE(98);
      END_STATE();
    case 39:
      if (lookahead == 'e') ADVANCE(99);
      END_STATE();
    case 40:
      if (lookahead == 's') ADVANCE(100);
      END_STATE();
    case 41:
      if (lookahead == 'a') ADVANCE(101);
      END_STATE();
    case 42:
      if (lookahead == 'n') ADVANCE(102);
      END_STATE();
    case 43:
      if (lookahead == 'f') ADVANCE(103);
      END_STATE();
    case 44:
      ACCEPT_TOKEN(anon_sym_do);
      if (lookahead == 'u') ADVANCE(104);
      END_STATE();
    case 45:
      if (lookahead == 's') ADVANCE(105);
      END_STATE();
    case 46:
      if (lookahead == 'u') ADVANCE(106);
      END_STATE();
    case 47:
      if (lookahead == 't') ADVANCE(107);
      END_STATE();
    case 48:
      if (lookahead == 'l') ADVANCE(108);
      END_STATE();
    case 49:
      if (lookahead == 'o') ADVANCE(109);
      END_STATE();
    case 50:
      if (lookahead == 'r') ADVANCE(110);
      END_STATE();
    case 51:
      if (lookahead == 't') ADVANCE(111);
      END_STATE();
    case 52:
      ACCEPT_TOKEN(anon_sym_if);
      END_STATE();
    case 53:
      if (lookahead == 'l') ADVANCE(112);
      if (lookahead == 't') ADVANCE(113);
      END_STATE();
    case 54:
      if (lookahead == 'n') ADVANCE(114);
      END_STATE();
    case 55:
      if (lookahead == 'x') ADVANCE(115);
      END_STATE();
    case 56:
      if (lookahead == 'r') ADVANCE(116);
      END_STATE();
    case 57:
      if (lookahead == 'l') ADVANCE(117);
      END_STATE();
    case 58:
      if (lookahead == 'f') ADVANCE(118);
      END_STATE();
    case 59:
      if (lookahead == 'r') ADVANCE(119);
      END_STATE();
    case 60:
      if (lookahead == 'g') ADVANCE(120);
      if (lookahead == 's') ADVANCE(121);
      if (lookahead == 't') ADVANCE(122);
      END_STATE();
    case 61:
      if (lookahead == 'o') ADVANCE(123);
      END_STATE();
    case 62:
      if (lookahead == 'g') ADVANCE(124);
      if (lookahead == 'z') ADVANCE(125);
      END_STATE();
    case 63:
      if (lookahead == 'i') ADVANCE(126);
      END_STATE();
    case 64:
      if (lookahead == 'a') ADVANCE(127);
      if (lookahead == 'r') ADVANCE(128);
      END_STATE();
    case 65:
      if (lookahead == 'i') ADVANCE(129);
      END_STATE();
    case 66:
      if (lookahead == 'r') ADVANCE(130);
      END_STATE();
    case 67:
      if (lookahead == 'u') ADVANCE(131);
      END_STATE();
    case 68:
      if (lookahead == 'p') ADVANCE(132);
      END_STATE();
    case 69:
      if (lookahead == 'n') ADVANCE(133);
      END_STATE();
    case 70:
      if (lookahead == 'i') ADVANCE(134);
      if (lookahead == 's') ADVANCE(135);
      END_STATE();
    case 71:
      if (lookahead == 'i') ADVANCE(136);
      if (lookahead == 'l') ADVANCE(137);
      END_STATE();
    case 72:
      if (lookahead == 'i') ADVANCE(138);
      END_STATE();
    case 73:
      if (lookahead == 'S') ADVANCE(139);
      END_STATE();
    case 74:
      if (lookahead == 'L') ADVANCE(140);
      END_STATE();
    case 75:
      if (lookahead == 'E') ADVANCE(141);
      END_STATE();
    case 76:
      if (lookahead == 'i') ADVANCE(142);
      END_STATE();
    case 77:
      if (lookahead == 'o') ADVANCE(143);
      END_STATE();
    case 78:
      if (lookahead == 'n') ADVANCE(144);
      END_STATE();
    case 79:
      if (lookahead == 'r') ADVANCE(145);
      END_STATE();
    case 80:
      if (lookahead == 'l') ADVANCE(146);
      if (lookahead == 's') ADVANCE(147);
      if (lookahead == 't') ADVANCE(148);
      END_STATE();
    case 81:
      if (lookahead == 'a') ADVANCE(149);
      END_STATE();
    case 82:
      if (lookahead == 'd') ADVANCE(150);
      if (lookahead == 'l') ADVANCE(151);
      END_STATE();
    case 83:
      if (lookahead == 'e') ADVANCE(152);
      END_STATE();
    case 84:
      if (lookahead == 'x') ADVANCE(153);
      END_STATE();
    case 85:
      if (lookahead == 'a') ADVANCE(154);
      if (lookahead == 'i') ADVANCE(155);
      if (lookahead == 'o') ADVANCE(156);
      END_STATE();
    case 86:
      if (lookahead == 'n') ADVANCE(157);
      END_STATE();
    case 87:
      if (lookahead == 'e') ADVANCE(158);
      END_STATE();
    case 88:
      if (lookahead == 'e') ADVANCE(159);
      END_STATE();
    case 89:
      if (lookahead == 'p') ADVANCE(160);
      if (lookahead == 't') ADVANCE(161);
      END_STATE();
    case 90:
      if (lookahead == 'h') ADVANCE(162);
      if (lookahead == 'r') ADVANCE(163);
      END_STATE();
    case 91:
      if (lookahead == 'n') ADVANCE(164);
      if (lookahead == 'p') ADVANCE(165);
      END_STATE();
    case 92:
      if (lookahead == 'e') ADVANCE(166);
      END_STATE();
    case 93:
      if (lookahead == 'i') ADVANCE(167);
      END_STATE();
    case 94:
      if (lookahead == 'a') ADVANCE(168);
      END_STATE();
    case 95:
      if (lookahead == 'g') ADVANCE(169);
      END_STATE();
    case 96:
      ACCEPT_TOKEN(anon_sym_asm);
      END_STATE();
    case 97:
      if (lookahead == 'o') ADVANCE(170);
      END_STATE();
    case 98:
      if (lookahead == 'l') ADVANCE(171);
      END_STATE();
    case 99:
      if (lookahead == 'a') ADVANCE(172);
      END_STATE();
    case 100:
      if (lookahead == 'e') ADVANCE(173);
      END_STATE();
    case 101:
      if (lookahead == 'r') ADVANCE(174);
      END_STATE();
    case 102:
      if (lookahead == 's') ADVANCE(175);
      if (lookahead == 't') ADVANCE(176);
      END_STATE();
    case 103:
      if (lookahead == 'a') ADVANCE(177);
      if (lookahead == 'i') ADVANCE(178);
      END_STATE();
    case 104:
      if (lookahead == 'b') ADVANCE(179);
      END_STATE();
    case 105:
      if (lookahead == 'e') ADVANCE(180);
      END_STATE();
    case 106:
      if (lookahead == 'm') ADVANCE(181);
      END_STATE();
    case 107:
      if (lookahead == 'e') ADVANCE(182);
      END_STATE();
    case 108:
      if (lookahead == 's') ADVANCE(183);
      END_STATE();
    case 109:
      if (lookahead == 'a') ADVANCE(184);
      END_STATE();
    case 110:
      ACCEPT_TOKEN(anon_sym_for);
      END_STATE();
    case 111:
      if (lookahead == 'o') ADVANCE(185);
      END_STATE();
    case 112:
      if (lookahead == 'i') ADVANCE(186);
      END_STATE();
    case 113:
      ACCEPT_TOKEN(sym_primitive_type);
      if (lookahead == '1') ADVANCE(187);
      if (lookahead == '3') ADVANCE(188);
      if (lookahead == '6') ADVANCE(189);
      if (lookahead == '8') ADVANCE(190);
      if (lookahead == 'p') ADVANCE(191);
      END_STATE();
    case 114:
      if (lookahead == 'g') ADVANCE(192);
      END_STATE();
    case 115:
      if (lookahead == '_') ADVANCE(193);
      END_STATE();
    case 116:
      if (lookahead == 'e') ADVANCE(194);
      END_STATE();
    case 117:
      if (lookahead == 'l') ADVANCE(195);
      END_STATE();
    case 118:
      if (lookahead == 's') ADVANCE(196);
      END_STATE();
    case 119:
      if (lookahead == 'd') ADVANCE(197);
      END_STATE();
    case 120:
      if (lookahead == 'i') ADVANCE(198);
      END_STATE();
    case 121:
      if (lookahead == 't') ADVANCE(199);
      END_STATE();
    case 122:
      if (lookahead == 'u') ADVANCE(200);
      END_STATE();
    case 123:
      if (lookahead == 'r') ADVANCE(201);
      END_STATE();
    case 124:
      if (lookahead == 'n') ADVANCE(202);
      END_STATE();
    case 125:
      if (lookahead == 'e') ADVANCE(203);
      END_STATE();
    case 126:
      if (lookahead == 'z') ADVANCE(204);
      END_STATE();
    case 127:
      if (lookahead == 't') ADVANCE(205);
      END_STATE();
    case 128:
      if (lookahead == 'u') ADVANCE(206);
      END_STATE();
    case 129:
      if (lookahead == 't') ADVANCE(207);
      END_STATE();
    case 130:
      if (lookahead == 'e') ADVANCE(208);
      END_STATE();
    case 131:
      if (lookahead == 'e') ADVANCE(141);
      END_STATE();
    case 132:
      if (lookahead == 'e') ADVANCE(209);
      END_STATE();
    case 133:
      if (lookahead == 't') ADVANCE(210);
      END_STATE();
    case 134:
      if (lookahead == 'o') ADVANCE(211);
      END_STATE();
    case 135:
      if (lookahead == 'i') ADVANCE(212);
      END_STATE();
    case 136:
      if (lookahead == 'd') ADVANCE(171);
      END_STATE();
    case 137:
      if (lookahead == 'a') ADVANCE(213);
      END_STATE();
    case 138:
      if (lookahead == 'l') ADVANCE(214);
      END_STATE();
    case 139:
      if (lookahead == 'E') ADVANCE(215);
      END_STATE();
    case 140:
      ACCEPT_TOKEN(anon_sym_NULL);
      END_STATE();
    case 141:
      ACCEPT_TOKEN(sym_true);
      END_STATE();
    case 142:
      if (lookahead == 'g') ADVANCE(216);
      END_STATE();
    case 143:
      if (lookahead == 'm') ADVANCE(217);
      END_STATE();
    case 144:
      if (lookahead == 'e') ADVANCE(218);
      END_STATE();
    case 145:
      if (lookahead == 'e') ADVANCE(219);
      END_STATE();
    case 146:
      if (lookahead == 'i') ADVANCE(220);
      END_STATE();
    case 147:
      if (lookahead == 'm') ADVANCE(221);
      END_STATE();
    case 148:
      if (lookahead == 't') ADVANCE(222);
      END_STATE();
    case 149:
      if (lookahead == 's') ADVANCE(223);
      END_STATE();
    case 150:
      if (lookahead == 'e') ADVANCE(224);
      END_STATE();
    case 151:
      if (lookahead == 'r') ADVANCE(225);
      END_STATE();
    case 152:
      if (lookahead == 'c') ADVANCE(226);
      END_STATE();
    case 153:
      if (lookahead == 'c') ADVANCE(227);
      if (lookahead == 't') ADVANCE(228);
      END_STATE();
    case 154:
      if (lookahead == 's') ADVANCE(229);
      END_STATE();
    case 155:
      if (lookahead == 'n') ADVANCE(230);
      END_STATE();
    case 156:
      if (lookahead == 'r') ADVANCE(231);
      END_STATE();
    case 157:
      if (lookahead == 'l') ADVANCE(232);
      END_STATE();
    case 158:
      if (lookahead == 'a') ADVANCE(233);
      END_STATE();
    case 159:
      if (lookahead == 's') ADVANCE(234);
      END_STATE();
    case 160:
      if (lookahead == 't') ADVANCE(235);
      END_STATE();
    case 161:
      if (lookahead == 'd') ADVANCE(236);
      END_STATE();
    case 162:
      if (lookahead == 'i') ADVANCE(237);
      if (lookahead == 'r') ADVANCE(238);
      END_STATE();
    case 163:
      if (lookahead == 'y') ADVANCE(239);
      END_STATE();
    case 164:
      if (lookahead == 'a') ADVANCE(240);
      END_STATE();
    case 165:
      if (lookahead == 't') ADVANCE(241);
      END_STATE();
    case 166:
      if (lookahead == 'c') ADVANCE(242);
      END_STATE();
    case 167:
      if (lookahead == 'g') ADVANCE(243);
      END_STATE();
    case 168:
      if (lookahead == 'l') ADVANCE(244);
      END_STATE();
    case 169:
      if (lookahead == 'n') ADVANCE(245);
      END_STATE();
    case 170:
      ACCEPT_TOKEN(anon_sym_auto);
      END_STATE();
    case 171:
      ACCEPT_TOKEN(sym_primitive_type);
      END_STATE();
    case 172:
      if (lookahead == 'k') ADVANCE(246);
      END_STATE();
    case 173:
      ACCEPT_TOKEN(anon_sym_case);
      END_STATE();
    case 174:
      ACCEPT_TOKEN(sym_primitive_type);
      if (lookahead == '1') ADVANCE(247);
      if (lookahead == '3') ADVANCE(248);
      if (lookahead == '6') ADVANCE(249);
      if (lookahead == '8') ADVANCE(250);
      if (lookahead == 'p') ADVANCE(251);
      END_STATE();
    case 175:
      if (lookahead == 't') ADVANCE(252);
      END_STATE();
    case 176:
      if (lookahead == 'i') ADVANCE(253);
      END_STATE();
    case 177:
      if (lookahead == 'u') ADVANCE(254);
      END_STATE();
    case 178:
      if (lookahead == 'n') ADVANCE(255);
      END_STATE();
    case 179:
      if (lookahead == 'l') ADVANCE(256);
      END_STATE();
    case 180:
      ACCEPT_TOKEN(anon_sym_else);
      END_STATE();
    case 181:
      ACCEPT_TOKEN(anon_sym_enum);
      END_STATE();
    case 182:
      if (lookahead == 'r') ADVANCE(257);
      END_STATE();
    case 183:
      if (lookahead == 'e') ADVANCE(215);
      END_STATE();
    case 184:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 185:
      ACCEPT_TOKEN(anon_sym_goto);
      END_STATE();
    case 186:
      if (lookahead == 'n') ADVANCE(258);
      END_STATE();
    case 187:
      if (lookahead == '6') ADVANCE(259);
      END_STATE();
    case 188:
      if (lookahead == '2') ADVANCE(260);
      END_STATE();
    case 189:
      if (lookahead == '4') ADVANCE(261);
      END_STATE();
    case 190:
      if (lookahead == '_') ADVANCE(262);
      END_STATE();
    case 191:
      if (lookahead == 't') ADVANCE(263);
      END_STATE();
    case 192:
      ACCEPT_TOKEN(anon_sym_long);
      END_STATE();
    case 193:
      if (lookahead == 'a') ADVANCE(264);
      END_STATE();
    case 194:
      if (lookahead == 't') ADVANCE(265);
      END_STATE();
    case 195:
      if (lookahead == 'p') ADVANCE(266);
      END_STATE();
    case 196:
      if (lookahead == 'e') ADVANCE(267);
      END_STATE();
    case 197:
      if (lookahead == 'i') ADVANCE(268);
      END_STATE();
    case 198:
      if (lookahead == 's') ADVANCE(269);
      END_STATE();
    case 199:
      if (lookahead == 'r') ADVANCE(270);
      END_STATE();
    case 200:
      if (lookahead == 'r') ADVANCE(271);
      END_STATE();
    case 201:
      if (lookahead == 't') ADVANCE(272);
      END_STATE();
    case 202:
      if (lookahead == 'e') ADVANCE(273);
      END_STATE();
    case 203:
      if (lookahead == '_') ADVANCE(274);
      if (lookahead == 'o') ADVANCE(275);
      END_STATE();
    case 204:
      if (lookahead == 'e') ADVANCE(276);
      END_STATE();
    case 205:
      if (lookahead == 'i') ADVANCE(277);
      END_STATE();
    case 206:
      if (lookahead == 'c') ADVANCE(278);
      END_STATE();
    case 207:
      if (lookahead == 'c') ADVANCE(279);
      END_STATE();
    case 208:
      if (lookahead == 'a') ADVANCE(280);
      END_STATE();
    case 209:
      if (lookahead == 'd') ADVANCE(281);
      END_STATE();
    case 210:
      if (lookahead == '1') ADVANCE(282);
      if (lookahead == '3') ADVANCE(283);
      if (lookahead == '6') ADVANCE(284);
      if (lookahead == '8') ADVANCE(285);
      if (lookahead == 'p') ADVANCE(286);
      END_STATE();
    case 211:
      if (lookahead == 'n') ADVANCE(287);
      END_STATE();
    case 212:
      if (lookahead == 'g') ADVANCE(288);
      END_STATE();
    case 213:
      if (lookahead == 't') ADVANCE(289);
      END_STATE();
    case 214:
      if (lookahead == 'e') ADVANCE(290);
      END_STATE();
    case 215:
      ACCEPT_TOKEN(sym_false);
      END_STATE();
    case 216:
      if (lookahead == 'n') ADVANCE(291);
      END_STATE();
    case 217:
      if (lookahead == 'i') ADVANCE(292);
      END_STATE();
    case 218:
      if (lookahead == 'r') ADVANCE(293);
      END_STATE();
    case 219:
      if (lookahead == 't') ADVANCE(294);
      END_STATE();
    case 220:
      if (lookahead == 'g') ADVANCE(295);
      END_STATE();
    case 221:
      if (lookahead == '_') ADVANCE(296);
      END_STATE();
    case 222:
      if (lookahead == 'r') ADVANCE(297);
      END_STATE();
    case 223:
      if (lookahead == 'e') ADVANCE(298);
      END_STATE();
    case 224:
      if (lookahead == 'c') ADVANCE(299);
      END_STATE();
    case 225:
      if (lookahead == 'c') ADVANCE(300);
      END_STATE();
    case 226:
      if (lookahead == 'l') ADVANCE(301);
      END_STATE();
    case 227:
      if (lookahead == 'e') ADVANCE(302);
      END_STATE();
    case 228:
      if (lookahead == 'e') ADVANCE(303);
      END_STATE();
    case 229:
      if (lookahead == 't') ADVANCE(304);
      END_STATE();
    case 230:
      if (lookahead == 'a') ADVANCE(305);
      END_STATE();
    case 231:
      if (lookahead == 'c') ADVANCE(306);
      END_STATE();
    case 232:
      if (lookahead == 'i') ADVANCE(307);
      END_STATE();
    case 233:
      if (lookahead == 'v') ADVANCE(308);
      END_STATE();
    case 234:
      if (lookahead == 't') ADVANCE(309);
      END_STATE();
    case 235:
      if (lookahead == 'r') ADVANCE(310);
      END_STATE();
    case 236:
      if (lookahead == 'c') ADVANCE(311);
      END_STATE();
    case 237:
      if (lookahead == 's') ADVANCE(312);
      END_STATE();
    case 238:
      if (lookahead == 'e') ADVANCE(313);
      END_STATE();
    case 239:
      ACCEPT_TOKEN(anon_sym___try);
      END_STATE();
    case 240:
      if (lookahead == 'l') ADVANCE(314);
      END_STATE();
    case 241:
      if (lookahead == 'r') ADVANCE(315);
      END_STATE();
    case 242:
      if (lookahead == 't') ADVANCE(316);
      END_STATE();
    case 243:
      if (lookahead == 'n') ADVANCE(317);
      END_STATE();
    case 244:
      if (lookahead == 'i') ADVANCE(318);
      END_STATE();
    case 245:
      if (lookahead == 'o') ADVANCE(319);
      END_STATE();
    case 246:
      ACCEPT_TOKEN(anon_sym_break);
      END_STATE();
    case 247:
      if (lookahead == '6') ADVANCE(320);
      END_STATE();
    case 248:
      if (lookahead == '2') ADVANCE(321);
      END_STATE();
    case 249:
      if (lookahead == '4') ADVANCE(322);
      END_STATE();
    case 250:
      if (lookahead == '_') ADVANCE(323);
      END_STATE();
    case 251:
      if (lookahead == 't') ADVANCE(324);
      END_STATE();
    case 252:
      ACCEPT_TOKEN(anon_sym_const);
      if (lookahead == 'e') ADVANCE(325);
      END_STATE();
    case 253:
      if (lookahead == 'n') ADVANCE(326);
      END_STATE();
    case 254:
      if (lookahead == 'l') ADVANCE(327);
      END_STATE();
    case 255:
      if (lookahead == 'e') ADVANCE(328);
      END_STATE();
    case 256:
      if (lookahead == 'e') ADVANCE(171);
      END_STATE();
    case 257:
      if (lookahead == 'n') ADVANCE(329);
      END_STATE();
    case 258:
      if (lookahead == 'e') ADVANCE(330);
      END_STATE();
    case 259:
      if (lookahead == '_') ADVANCE(331);
      END_STATE();
    case 260:
      if (lookahead == '_') ADVANCE(332);
      END_STATE();
    case 261:
      if (lookahead == '_') ADVANCE(333);
      END_STATE();
    case 262:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 263:
      if (lookahead == 'r') ADVANCE(334);
      END_STATE();
    case 264:
      if (lookahead == 'l') ADVANCE(335);
      END_STATE();
    case 265:
      if (lookahead == 'u') ADVANCE(336);
      END_STATE();
    case 266:
      if (lookahead == 't') ADVANCE(337);
      END_STATE();
    case 267:
      if (lookahead == 't') ADVANCE(338);
      END_STATE();
    case 268:
      if (lookahead == 'f') ADVANCE(339);
      END_STATE();
    case 269:
      if (lookahead == 't') ADVANCE(340);
      END_STATE();
    case 270:
      if (lookahead == 'i') ADVANCE(341);
      END_STATE();
    case 271:
      if (lookahead == 'n') ADVANCE(342);
      END_STATE();
    case 272:
      ACCEPT_TOKEN(anon_sym_short);
      END_STATE();
    case 273:
      if (lookahead == 'd') ADVANCE(343);
      END_STATE();
    case 274:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 275:
      if (lookahead == 'f') ADVANCE(344);
      END_STATE();
    case 276:
      if (lookahead == '_') ADVANCE(345);
      END_STATE();
    case 277:
      if (lookahead == 'c') ADVANCE(346);
      END_STATE();
    case 278:
      if (lookahead == 't') ADVANCE(347);
      END_STATE();
    case 279:
      if (lookahead == 'h') ADVANCE(348);
      END_STATE();
    case 280:
      if (lookahead == 'd') ADVANCE(349);
      END_STATE();
    case 281:
      if (lookahead == 'e') ADVANCE(350);
      END_STATE();
    case 282:
      if (lookahead == '6') ADVANCE(351);
      END_STATE();
    case 283:
      if (lookahead == '2') ADVANCE(352);
      END_STATE();
    case 284:
      if (lookahead == '4') ADVANCE(353);
      END_STATE();
    case 285:
      if (lookahead == '_') ADVANCE(354);
      END_STATE();
    case 286:
      if (lookahead == 't') ADVANCE(355);
      END_STATE();
    case 287:
      ACCEPT_TOKEN(anon_sym_union);
      END_STATE();
    case 288:
      if (lookahead == 'n') ADVANCE(356);
      END_STATE();
    case 289:
      if (lookahead == 'i') ADVANCE(357);
      END_STATE();
    case 290:
      ACCEPT_TOKEN(anon_sym_while);
      END_STATE();
    case 291:
      if (lookahead == 'o') ADVANCE(358);
      END_STATE();
    case 292:
      if (lookahead == 'c') ADVANCE(359);
      END_STATE();
    case 293:
      if (lookahead == 'i') ADVANCE(360);
      END_STATE();
    case 294:
      if (lookahead == 'u') ADVANCE(361);
      END_STATE();
    case 295:
      if (lookahead == 'n') ADVANCE(362);
      END_STATE();
    case 296:
      if (lookahead == '_') ADVANCE(363);
      END_STATE();
    case 297:
      if (lookahead == 'i') ADVANCE(364);
      END_STATE();
    case 298:
      if (lookahead == 'd') ADVANCE(365);
      END_STATE();
    case 299:
      if (lookahead == 'l') ADVANCE(366);
      END_STATE();
    case 300:
      if (lookahead == 'a') ADVANCE(367);
      END_STATE();
    case 301:
      if (lookahead == 's') ADVANCE(368);
      END_STATE();
    case 302:
      if (lookahead == 'p') ADVANCE(369);
      END_STATE();
    case 303:
      if (lookahead == 'n') ADVANCE(370);
      END_STATE();
    case 304:
      if (lookahead == 'c') ADVANCE(371);
      END_STATE();
    case 305:
      if (lookahead == 'l') ADVANCE(372);
      END_STATE();
    case 306:
      if (lookahead == 'e') ADVANCE(373);
      END_STATE();
    case 307:
      if (lookahead == 'n') ADVANCE(374);
      END_STATE();
    case 308:
      if (lookahead == 'e') ADVANCE(375);
      END_STATE();
    case 309:
      if (lookahead == 'r') ADVANCE(376);
      END_STATE();
    case 310:
      ACCEPT_TOKEN(sym_ms_signed_ptr_modifier);
      END_STATE();
    case 311:
      if (lookahead == 'a') ADVANCE(377);
      END_STATE();
    case 312:
      if (lookahead == 'c') ADVANCE(378);
      END_STATE();
    case 313:
      if (lookahead == 'a') ADVANCE(379);
      END_STATE();
    case 314:
      if (lookahead == 'i') ADVANCE(380);
      END_STATE();
    case 315:
      ACCEPT_TOKEN(sym_ms_unsigned_ptr_modifier);
      END_STATE();
    case 316:
      if (lookahead == 'o') ADVANCE(381);
      END_STATE();
    case 317:
      if (lookahead == 'o') ADVANCE(382);
      END_STATE();
    case 318:
      if (lookahead == 'g') ADVANCE(383);
      END_STATE();
    case 319:
      if (lookahead == 'f') ADVANCE(384);
      END_STATE();
    case 320:
      if (lookahead == '_') ADVANCE(385);
      END_STATE();
    case 321:
      if (lookahead == '_') ADVANCE(386);
      END_STATE();
    case 322:
      if (lookahead == '_') ADVANCE(387);
      END_STATE();
    case 323:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 324:
      if (lookahead == 'r') ADVANCE(388);
      END_STATE();
    case 325:
      if (lookahead == 'x') ADVANCE(389);
      END_STATE();
    case 326:
      if (lookahead == 'u') ADVANCE(390);
      END_STATE();
    case 327:
      if (lookahead == 't') ADVANCE(391);
      END_STATE();
    case 328:
      if (lookahead == 'd') ADVANCE(392);
      END_STATE();
    case 329:
      ACCEPT_TOKEN(anon_sym_extern);
      END_STATE();
    case 330:
      ACCEPT_TOKEN(anon_sym_inline);
      END_STATE();
    case 331:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 332:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 333:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 334:
      if (lookahead == '_') ADVANCE(393);
      END_STATE();
    case 335:
      if (lookahead == 'i') ADVANCE(394);
      END_STATE();
    case 336:
      if (lookahead == 'r') ADVANCE(395);
      END_STATE();
    case 337:
      if (lookahead == 'r') ADVANCE(396);
      END_STATE();
    case 338:
      if (lookahead == 'o') ADVANCE(397);
      END_STATE();
    case 339:
      if (lookahead == 'f') ADVANCE(398);
      END_STATE();
    case 340:
      if (lookahead == 'e') ADVANCE(399);
      END_STATE();
    case 341:
      if (lookahead == 'c') ADVANCE(400);
      END_STATE();
    case 342:
      ACCEPT_TOKEN(anon_sym_return);
      END_STATE();
    case 343:
      ACCEPT_TOKEN(anon_sym_signed);
      END_STATE();
    case 344:
      ACCEPT_TOKEN(anon_sym_sizeof);
      END_STATE();
    case 345:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 346:
      ACCEPT_TOKEN(anon_sym_static);
      END_STATE();
    case 347:
      ACCEPT_TOKEN(anon_sym_struct);
      END_STATE();
    case 348:
      ACCEPT_TOKEN(anon_sym_switch);
      END_STATE();
    case 349:
      if (lookahead == '_') ADVANCE(401);
      END_STATE();
    case 350:
      if (lookahead == 'f') ADVANCE(402);
      END_STATE();
    case 351:
      if (lookahead == '_') ADVANCE(403);
      END_STATE();
    case 352:
      if (lookahead == '_') ADVANCE(404);
      END_STATE();
    case 353:
      if (lookahead == '_') ADVANCE(405);
      END_STATE();
    case 354:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 355:
      if (lookahead == 'r') ADVANCE(406);
      END_STATE();
    case 356:
      if (lookahead == 'e') ADVANCE(407);
      END_STATE();
    case 357:
      if (lookahead == 'l') ADVANCE(408);
      END_STATE();
    case 358:
      if (lookahead == 'f') ADVANCE(409);
      END_STATE();
    case 359:
      ACCEPT_TOKEN(anon_sym__Atomic);
      END_STATE();
    case 360:
      if (lookahead == 'c') ADVANCE(410);
      END_STATE();
    case 361:
      if (lookahead == 'r') ADVANCE(411);
      END_STATE();
    case 362:
      if (lookahead == 'o') ADVANCE(412);
      END_STATE();
    case 363:
      ACCEPT_TOKEN(anon_sym___asm__);
      END_STATE();
    case 364:
      if (lookahead == 'b') ADVANCE(413);
      END_STATE();
    case 365:
      ACCEPT_TOKEN(anon_sym___based);
      END_STATE();
    case 366:
      ACCEPT_TOKEN(anon_sym___cdecl);
      END_STATE();
    case 367:
      if (lookahead == 'l') ADVANCE(414);
      END_STATE();
    case 368:
      if (lookahead == 'p') ADVANCE(415);
      END_STATE();
    case 369:
      if (lookahead == 't') ADVANCE(416);
      END_STATE();
    case 370:
      if (lookahead == 's') ADVANCE(417);
      END_STATE();
    case 371:
      if (lookahead == 'a') ADVANCE(418);
      END_STATE();
    case 372:
      if (lookahead == 'l') ADVANCE(419);
      END_STATE();
    case 373:
      if (lookahead == 'i') ADVANCE(420);
      END_STATE();
    case 374:
      if (lookahead == 'e') ADVANCE(421);
      END_STATE();
    case 375:
      ACCEPT_TOKEN(anon_sym___leave);
      END_STATE();
    case 376:
      if (lookahead == 'i') ADVANCE(422);
      END_STATE();
    case 377:
      if (lookahead == 'l') ADVANCE(423);
      END_STATE();
    case 378:
      if (lookahead == 'a') ADVANCE(424);
      END_STATE();
    case 379:
      if (lookahead == 'd') ADVANCE(425);
      END_STATE();
    case 380:
      if (lookahead == 'g') ADVANCE(426);
      END_STATE();
    case 381:
      if (lookahead == 'r') ADVANCE(427);
      END_STATE();
    case 382:
      if (lookahead == 'f') ADVANCE(428);
      END_STATE();
    case 383:
      if (lookahead == 'n') ADVANCE(429);
      END_STATE();
    case 384:
      ACCEPT_TOKEN(anon_sym_alignof);
      END_STATE();
    case 385:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 386:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 387:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 388:
      if (lookahead == '_') ADVANCE(430);
      END_STATE();
    case 389:
      if (lookahead == 'p') ADVANCE(431);
      END_STATE();
    case 390:
      if (lookahead == 'e') ADVANCE(432);
      END_STATE();
    case 391:
      ACCEPT_TOKEN(anon_sym_default);
      END_STATE();
    case 392:
      ACCEPT_TOKEN(anon_sym_defined);
      END_STATE();
    case 393:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 394:
      if (lookahead == 'g') ADVANCE(433);
      END_STATE();
    case 395:
      if (lookahead == 'n') ADVANCE(434);
      END_STATE();
    case 396:
      ACCEPT_TOKEN(anon_sym_nullptr);
      if (lookahead == '_') ADVANCE(435);
      END_STATE();
    case 397:
      if (lookahead == 'f') ADVANCE(436);
      END_STATE();
    case 398:
      if (lookahead == '_') ADVANCE(437);
      END_STATE();
    case 399:
      if (lookahead == 'r') ADVANCE(438);
      END_STATE();
    case 400:
      if (lookahead == 't') ADVANCE(439);
      END_STATE();
    case 401:
      if (lookahead == 'l') ADVANCE(440);
      END_STATE();
    case 402:
      ACCEPT_TOKEN(anon_sym_typedef);
      END_STATE();
    case 403:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 404:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 405:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 406:
      if (lookahead == '_') ADVANCE(441);
      END_STATE();
    case 407:
      if (lookahead == 'd') ADVANCE(442);
      END_STATE();
    case 408:
      if (lookahead == 'e') ADVANCE(443);
      END_STATE();
    case 409:
      ACCEPT_TOKEN(anon_sym__Alignof);
      END_STATE();
    case 410:
      ACCEPT_TOKEN(anon_sym__Generic);
      END_STATE();
    case 411:
      if (lookahead == 'n') ADVANCE(444);
      END_STATE();
    case 412:
      if (lookahead == 'f') ADVANCE(445);
      END_STATE();
    case 413:
      if (lookahead == 'u') ADVANCE(446);
      END_STATE();
    case 414:
      if (lookahead == 'l') ADVANCE(447);
      END_STATE();
    case 415:
      if (lookahead == 'e') ADVANCE(448);
      END_STATE();
    case 416:
      ACCEPT_TOKEN(anon_sym___except);
      END_STATE();
    case 417:
      if (lookahead == 'i') ADVANCE(449);
      END_STATE();
    case 418:
      if (lookahead == 'l') ADVANCE(450);
      END_STATE();
    case 419:
      if (lookahead == 'y') ADVANCE(451);
      END_STATE();
    case 420:
      if (lookahead == 'n') ADVANCE(452);
      END_STATE();
    case 421:
      ACCEPT_TOKEN(anon_sym___inline);
      if (lookahead == '_') ADVANCE(453);
      END_STATE();
    case 422:
      if (lookahead == 'c') ADVANCE(454);
      END_STATE();
    case 423:
      if (lookahead == 'l') ADVANCE(455);
      END_STATE();
    case 424:
      if (lookahead == 'l') ADVANCE(456);
      END_STATE();
    case 425:
      ACCEPT_TOKEN(anon_sym___thread);
      END_STATE();
    case 426:
      if (lookahead == 'n') ADVANCE(457);
      END_STATE();
    case 427:
      if (lookahead == 'c') ADVANCE(458);
      END_STATE();
    case 428:
      ACCEPT_TOKEN(anon_sym__alignof);
      END_STATE();
    case 429:
      if (lookahead == 'e') ADVANCE(459);
      END_STATE();
    case 430:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 431:
      if (lookahead == 'r') ADVANCE(460);
      END_STATE();
    case 432:
      ACCEPT_TOKEN(anon_sym_continue);
      END_STATE();
    case 433:
      if (lookahead == 'n') ADVANCE(461);
      END_STATE();
    case 434:
      ACCEPT_TOKEN(anon_sym_noreturn);
      END_STATE();
    case 435:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 436:
      ACCEPT_TOKEN(anon_sym_offsetof);
      END_STATE();
    case 437:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 438:
      ACCEPT_TOKEN(anon_sym_register);
      END_STATE();
    case 439:
      ACCEPT_TOKEN(anon_sym_restrict);
      END_STATE();
    case 440:
      if (lookahead == 'o') ADVANCE(462);
      END_STATE();
    case 441:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 442:
      ACCEPT_TOKEN(anon_sym_unsigned);
      END_STATE();
    case 443:
      ACCEPT_TOKEN(anon_sym_volatile);
      END_STATE();
    case 444:
      ACCEPT_TOKEN(anon_sym__Noreturn);
      END_STATE();
    case 445:
      ACCEPT_TOKEN(anon_sym___alignof);
      if (lookahead == '_') ADVANCE(463);
      END_STATE();
    case 446:
      if (lookahead == 't') ADVANCE(464);
      END_STATE();
    case 447:
      ACCEPT_TOKEN(anon_sym___clrcall);
      END_STATE();
    case 448:
      if (lookahead == 'c') ADVANCE(465);
      END_STATE();
    case 449:
      if (lookahead == 'o') ADVANCE(466);
      END_STATE();
    case 450:
      if (lookahead == 'l') ADVANCE(467);
      END_STATE();
    case 451:
      ACCEPT_TOKEN(anon_sym___finally);
      END_STATE();
    case 452:
      if (lookahead == 'l') ADVANCE(468);
      END_STATE();
    case 453:
      if (lookahead == '_') ADVANCE(469);
      END_STATE();
    case 454:
      if (lookahead == 't') ADVANCE(470);
      END_STATE();
    case 455:
      ACCEPT_TOKEN(anon_sym___stdcall);
      END_STATE();
    case 456:
      if (lookahead == 'l') ADVANCE(471);
      END_STATE();
    case 457:
      if (lookahead == 'e') ADVANCE(472);
      END_STATE();
    case 458:
      if (lookahead == 'a') ADVANCE(473);
      END_STATE();
    case 459:
      if (lookahead == 'd') ADVANCE(474);
      END_STATE();
    case 460:
      ACCEPT_TOKEN(anon_sym_constexpr);
      END_STATE();
    case 461:
      if (lookahead == '_') ADVANCE(475);
      END_STATE();
    case 462:
      if (lookahead == 'c') ADVANCE(476);
      END_STATE();
    case 463:
      if (lookahead == '_') ADVANCE(477);
      END_STATE();
    case 464:
      if (lookahead == 'e') ADVANCE(478);
      END_STATE();
    case 465:
      ACCEPT_TOKEN(anon_sym___declspec);
      END_STATE();
    case 466:
      if (lookahead == 'n') ADVANCE(479);
      END_STATE();
    case 467:
      ACCEPT_TOKEN(anon_sym___fastcall);
      END_STATE();
    case 468:
      if (lookahead == 'i') ADVANCE(480);
      END_STATE();
    case 469:
      ACCEPT_TOKEN(anon_sym___inline__);
      END_STATE();
    case 470:
      ACCEPT_TOKEN(sym_ms_restrict_modifier);
      if (lookahead == '_') ADVANCE(481);
      END_STATE();
    case 471:
      ACCEPT_TOKEN(anon_sym___thiscall);
      END_STATE();
    case 472:
      if (lookahead == 'd') ADVANCE(482);
      END_STATE();
    case 473:
      if (lookahead == 'l') ADVANCE(483);
      END_STATE();
    case 474:
      ACCEPT_TOKEN(anon_sym__unaligned);
      END_STATE();
    case 475:
      if (lookahead == 't') ADVANCE(171);
      END_STATE();
    case 476:
      if (lookahead == 'a') ADVANCE(484);
      END_STATE();
    case 477:
      ACCEPT_TOKEN(anon_sym___alignof__);
      END_STATE();
    case 478:
      if (lookahead == '_') ADVANCE(485);
      END_STATE();
    case 479:
      if (lookahead == '_') ADVANCE(486);
      END_STATE();
    case 480:
      if (lookahead == 'n') ADVANCE(487);
      END_STATE();
    case 481:
      if (lookahead == '_') ADVANCE(488);
      END_STATE();
    case 482:
      ACCEPT_TOKEN(anon_sym___unaligned);
      END_STATE();
    case 483:
      if (lookahead == 'l') ADVANCE(489);
      END_STATE();
    case 484:
      if (lookahead == 'l') ADVANCE(490);
      END_STATE();
    case 485:
      if (lookahead == '_') ADVANCE(491);
      END_STATE();
    case 486:
      if (lookahead == '_') ADVANCE(492);
      END_STATE();
    case 487:
      if (lookahead == 'e') ADVANCE(493);
      END_STATE();
    case 488:
      ACCEPT_TOKEN(anon_sym___restrict__);
      END_STATE();
    case 489:
      ACCEPT_TOKEN(anon_sym___vectorcall);
      END_STATE();
    case 490:
      ACCEPT_TOKEN(anon_sym_thread_local);
      END_STATE();
    case 491:
      ACCEPT_TOKEN(anon_sym___attribute__);
      END_STATE();
    case 492:
      ACCEPT_TOKEN(anon_sym___extension__);
      END_STATE();
    case 493:
      ACCEPT_TOKEN(anon_sym___forceinline);
      END_STATE();
    default:
      return false;
  }
}

static const TSLexMode ts_lex_modes[STATE_COUNT] = {
  [0] = {.lex_state = 0},
  [1] = {.lex_state = 119},
  [2] = {.lex_state = 45},
  [3] = {.lex_state = 45},
  [4] = {.lex_state = 45},
  [5] = {.lex_state = 45},
  [6] = {.lex_state = 45},
  [7] = {.lex_state = 45},
  [8] = {.lex_state = 45},
  [9] = {.lex_state = 45},
  [10] = {.lex_state = 45},
  [11] = {.lex_state = 45},
  [12] = {.lex_state = 45},
  [13] = {.lex_state = 45},
  [14] = {.lex_state = 45},
  [15] = {.lex_state = 45},
  [16] = {.lex_state = 45},
  [17] = {.lex_state = 45},
  [18] = {.lex_state = 45},
  [19] = {.lex_state = 45},
  [20] = {.lex_state = 45},
  [21] = {.lex_state = 45},
  [22] = {.lex_state = 45},
  [23] = {.lex_state = 119},
  [24] = {.lex_state = 47},
  [25] = {.lex_state = 47},
  [26] = {.lex_state = 119},
  [27] = {.lex_state = 47},
  [28] = {.lex_state = 119},
  [29] = {.lex_state = 119},
  [30] = {.lex_state = 119},
  [31] = {.lex_state = 119},
  [32] = {.lex_state = 119},
  [33] = {.lex_state = 119},
  [34] = {.lex_state = 119},
  [35] = {.lex_state = 119},
  [36] = {.lex_state = 119},
  [37] = {.lex_state = 119},
  [38] = {.lex_state = 119},
  [39] = {.lex_state = 119},
  [40] = {.lex_state = 119},
  [41] = {.lex_state = 119},
  [42] = {.lex_state = 119},
  [43] = {.lex_state = 119},
  [44] = {.lex_state = 119},
  [45] = {.lex_state = 45},
  [46] = {.lex_state = 45},
  [47] = {.lex_state = 45},
  [48] = {.lex_state = 45},
  [49] = {.lex_state = 45},
  [50] = {.lex_state = 119},
  [51] = {.lex_state = 119},
  [52] = {.lex_state = 119},
  [53] = {.lex_state = 119},
  [54] = {.lex_state = 47},
  [55] = {.lex_state = 47},
  [56] = {.lex_state = 119},
  [57] = {.lex_state = 47},
  [58] = {.lex_state = 47},
  [59] = {.lex_state = 119},
  [60] = {.lex_state = 119},
  [61] = {.lex_state = 119},
  [62] = {.lex_state = 119},
  [63] = {.lex_state = 47},
  [64] = {.lex_state = 119},
  [65] = {.lex_state = 119},
  [66] = {.lex_state = 119},
  [67] = {.lex_state = 119},
  [68] = {.lex_state = 119},
  [69] = {.lex_state = 119},
  [70] = {.lex_state = 119},
  [71] = {.lex_state = 119},
  [72] = {.lex_state = 119},
  [73] = {.lex_state = 119},
  [74] = {.lex_state = 119},
  [75] = {.lex_state = 45},
  [76] = {.lex_state = 45},
  [77] = {.lex_state = 45},
  [78] = {.lex_state = 45},
  [79] = {.lex_state = 45},
  [80] = {.lex_state = 45},
  [81] = {.lex_state = 45},
  [82] = {.lex_state = 45},
  [83] = {.lex_state = 45},
  [84] = {.lex_state = 45},
  [85] = {.lex_state = 45},
  [86] = {.lex_state = 45},
  [87] = {.lex_state = 45},
  [88] = {.lex_state = 45},
  [89] = {.lex_state = 45},
  [90] = {.lex_state = 45},
  [91] = {.lex_state = 45},
  [92] = {.lex_state = 119},
  [93] = {.lex_state = 45},
  [94] = {.lex_state = 45},
  [95] = {.lex_state = 45},
  [96] = {.lex_state = 45},
  [97] = {.lex_state = 45},
  [98] = {.lex_state = 45},
  [99] = {.lex_state = 45},
  [100] = {.lex_state = 45},
  [101] = {.lex_state = 45},
  [102] = {.lex_state = 45},
  [103] = {.lex_state = 45},
  [104] = {.lex_state = 45},
  [105] = {.lex_state = 45},
  [106] = {.lex_state = 45},
  [107] = {.lex_state = 45},
  [108] = {.lex_state = 45},
  [109] = {.lex_state = 45},
  [110] = {.lex_state = 45},
  [111] = {.lex_state = 119},
  [112] = {.lex_state = 45},
  [113] = {.lex_state = 45},
  [114] = {.lex_state = 45},
  [115] = {.lex_state = 45},
  [116] = {.lex_state = 45},
  [117] = {.lex_state = 45},
  [118] = {.lex_state = 45},
  [119] = {.lex_state = 45},
  [120] = {.lex_state = 45},
  [121] = {.lex_state = 45},
  [122] = {.lex_state = 45},
  [123] = {.lex_state = 45},
  [124] = {.lex_state = 45},
  [125] = {.lex_state = 45},
  [126] = {.lex_state = 45},
  [127] = {.lex_state = 45},
  [128] = {.lex_state = 45},
  [129] = {.lex_state = 45},
  [130] = {.lex_state = 45},
  [131] = {.lex_state = 45},
  [132] = {.lex_state = 45},
  [133] = {.lex_state = 45},
  [134] = {.lex_state = 45},
  [135] = {.lex_state = 45},
  [136] = {.lex_state = 45},
  [137] = {.lex_state = 45},
  [138] = {.lex_state = 45},
  [139] = {.lex_state = 45},
  [140] = {.lex_state = 45},
  [141] = {.lex_state = 45},
  [142] = {.lex_state = 45},
  [143] = {.lex_state = 45},
  [144] = {.lex_state = 45},
  [145] = {.lex_state = 45},
  [146] = {.lex_state = 45},
  [147] = {.lex_state = 45},
  [148] = {.lex_state = 45},
  [149] = {.lex_state = 45},
  [150] = {.lex_state = 45},
  [151] = {.lex_state = 45},
  [152] = {.lex_state = 45},
  [153] = {.lex_state = 45},
  [154] = {.lex_state = 45},
  [155] = {.lex_state = 45},
  [156] = {.lex_state = 47},
  [157] = {.lex_state = 44},
  [158] = {.lex_state = 119},
  [159] = {.lex_state = 44},
  [160] = {.lex_state = 119},
  [161] = {.lex_state = 47},
  [162] = {.lex_state = 119},
  [163] = {.lex_state = 47},
  [164] = {.lex_state = 47},
  [165] = {.lex_state = 47},
  [166] = {.lex_state = 47},
  [167] = {.lex_state = 47},
  [168] = {.lex_state = 47},
  [169] = {.lex_state = 47},
  [170] = {.lex_state = 47},
  [171] = {.lex_state = 119},
  [172] = {.lex_state = 119},
  [173] = {.lex_state = 47},
  [174] = {.lex_state = 47},
  [175] = {.lex_state = 119},
  [176] = {.lex_state = 47},
  [177] = {.lex_state = 119},
  [178] = {.lex_state = 119},
  [179] = {.lex_state = 119},
  [180] = {.lex_state = 119},
  [181] = {.lex_state = 119},
  [182] = {.lex_state = 119},
  [183] = {.lex_state = 119},
  [184] = {.lex_state = 119},
  [185] = {.lex_state = 119},
  [186] = {.lex_state = 119},
  [187] = {.lex_state = 119},
  [188] = {.lex_state = 47},
  [189] = {.lex_state = 47},
  [190] = {.lex_state = 47},
  [191] = {.lex_state = 47},
  [192] = {.lex_state = 119},
  [193] = {.lex_state = 47},
  [194] = {.lex_state = 119},
  [195] = {.lex_state = 119},
  [196] = {.lex_state = 119},
  [197] = {.lex_state = 119},
  [198] = {.lex_state = 119},
  [199] = {.lex_state = 119},
  [200] = {.lex_state = 47},
  [201] = {.lex_state = 47},
  [202] = {.lex_state = 47},
  [203] = {.lex_state = 47},
  [204] = {.lex_state = 47},
  [205] = {.lex_state = 119},
  [206] = {.lex_state = 119},
  [207] = {.lex_state = 47},
  [208] = {.lex_state = 47},
  [209] = {.lex_state = 47},
  [210] = {.lex_state = 47},
  [211] = {.lex_state = 47},
  [212] = {.lex_state = 119},
  [213] = {.lex_state = 119},
  [214] = {.lex_state = 47},
  [215] = {.lex_state = 119},
  [216] = {.lex_state = 47},
  [217] = {.lex_state = 119},
  [218] = {.lex_state = 119},
  [219] = {.lex_state = 119},
  [220] = {.lex_state = 119},
  [221] = {.lex_state = 47},
  [222] = {.lex_state = 119},
  [223] = {.lex_state = 119},
  [224] = {.lex_state = 47},
  [225] = {.lex_state = 47},
  [226] = {.lex_state = 47},
  [227] = {.lex_state = 47},
  [228] = {.lex_state = 47},
  [229] = {.lex_state = 47},
  [230] = {.lex_state = 47},
  [231] = {.lex_state = 47},
  [232] = {.lex_state = 47},
  [233] = {.lex_state = 119},
  [234] = {.lex_state = 119},
  [235] = {.lex_state = 119},
  [236] = {.lex_state = 47},
  [237] = {.lex_state = 119},
  [238] = {.lex_state = 47},
  [239] = {.lex_state = 119},
  [240] = {.lex_state = 119},
  [241] = {.lex_state = 119},
  [242] = {.lex_state = 119},
  [243] = {.lex_state = 119},
  [244] = {.lex_state = 119},
  [245] = {.lex_state = 119},
  [246] = {.lex_state = 119},
  [247] = {.lex_state = 47},
  [248] = {.lex_state = 119},
  [249] = {.lex_state = 119},
  [250] = {.lex_state = 119},
  [251] = {.lex_state = 47},
  [252] = {.lex_state = 47},
  [253] = {.lex_state = 47},
  [254] = {.lex_state = 47},
  [255] = {.lex_state = 47},
  [256] = {.lex_state = 119},
  [257] = {.lex_state = 47},
  [258] = {.lex_state = 47},
  [259] = {.lex_state = 119},
  [260] = {.lex_state = 119},
  [261] = {.lex_state = 119},
  [262] = {.lex_state = 119},
  [263] = {.lex_state = 119},
  [264] = {.lex_state = 119},
  [265] = {.lex_state = 119},
  [266] = {.lex_state = 119},
  [267] = {.lex_state = 119},
  [268] = {.lex_state = 119},
  [269] = {.lex_state = 119},
  [270] = {.lex_state = 119},
  [271] = {.lex_state = 119},
  [272] = {.lex_state = 119},
  [273] = {.lex_state = 119},
  [274] = {.lex_state = 119},
  [275] = {.lex_state = 119},
  [276] = {.lex_state = 119},
  [277] = {.lex_state = 119},
  [278] = {.lex_state = 119},
  [279] = {.lex_state = 119},
  [280] = {.lex_state = 119},
  [281] = {.lex_state = 119},
  [282] = {.lex_state = 119},
  [283] = {.lex_state = 119},
  [284] = {.lex_state = 119},
  [285] = {.lex_state = 119},
  [286] = {.lex_state = 119},
  [287] = {.lex_state = 119},
  [288] = {.lex_state = 119},
  [289] = {.lex_state = 119},
  [290] = {.lex_state = 119},
  [291] = {.lex_state = 119},
  [292] = {.lex_state = 119},
  [293] = {.lex_state = 119},
  [294] = {.lex_state = 119},
  [295] = {.lex_state = 119},
  [296] = {.lex_state = 47},
  [297] = {.lex_state = 119},
  [298] = {.lex_state = 119},
  [299] = {.lex_state = 119},
  [300] = {.lex_state = 119},
  [301] = {.lex_state = 119},
  [302] = {.lex_state = 119},
  [303] = {.lex_state = 119},
  [304] = {.lex_state = 119},
  [305] = {.lex_state = 119},
  [306] = {.lex_state = 119},
  [307] = {.lex_state = 119},
  [308] = {.lex_state = 119},
  [309] = {.lex_state = 47},
  [310] = {.lex_state = 47},
  [311] = {.lex_state = 119},
  [312] = {.lex_state = 119},
  [313] = {.lex_state = 119},
  [314] = {.lex_state = 119},
  [315] = {.lex_state = 119},
  [316] = {.lex_state = 119},
  [317] = {.lex_state = 119},
  [318] = {.lex_state = 119},
  [319] = {.lex_state = 47},
  [320] = {.lex_state = 119},
  [321] = {.lex_state = 47},
  [322] = {.lex_state = 47},
  [323] = {.lex_state = 119},
  [324] = {.lex_state = 47},
  [325] = {.lex_state = 47},
  [326] = {.lex_state = 47},
  [327] = {.lex_state = 119},
  [328] = {.lex_state = 47},
  [329] = {.lex_state = 47},
  [330] = {.lex_state = 47},
  [331] = {.lex_state = 47},
  [332] = {.lex_state = 47},
  [333] = {.lex_state = 47},
  [334] = {.lex_state = 119},
  [335] = {.lex_state = 119},
  [336] = {.lex_state = 47},
  [337] = {.lex_state = 47},
  [338] = {.lex_state = 119},
  [339] = {.lex_state = 119},
  [340] = {.lex_state = 47},
  [341] = {.lex_state = 47},
  [342] = {.lex_state = 119},
  [343] = {.lex_state = 119},
  [344] = {.lex_state = 119},
  [345] = {.lex_state = 119},
  [346] = {.lex_state = 119},
  [347] = {.lex_state = 119},
  [348] = {.lex_state = 47},
  [349] = {.lex_state = 119},
  [350] = {.lex_state = 119},
  [351] = {.lex_state = 119},
  [352] = {.lex_state = 119},
  [353] = {.lex_state = 47},
  [354] = {.lex_state = 119},
  [355] = {.lex_state = 119},
  [356] = {.lex_state = 47},
  [357] = {.lex_state = 119},
  [358] = {.lex_state = 119},
  [359] = {.lex_state = 119},
  [360] = {.lex_state = 47},
  [361] = {.lex_state = 119},
  [362] = {.lex_state = 119},
  [363] = {.lex_state = 119},
  [364] = {.lex_state = 47},
  [365] = {.lex_state = 47},
  [366] = {.lex_state = 119},
  [367] = {.lex_state = 119},
  [368] = {.lex_state = 119},
  [369] = {.lex_state = 47},
  [370] = {.lex_state = 47},
  [371] = {.lex_state = 119},
  [372] = {.lex_state = 119},
  [373] = {.lex_state = 119},
  [374] = {.lex_state = 119},
  [375] = {.lex_state = 119},
  [376] = {.lex_state = 119},
  [377] = {.lex_state = 119},
  [378] = {.lex_state = 119},
  [379] = {.lex_state = 119},
  [380] = {.lex_state = 119},
  [381] = {.lex_state = 119},
  [382] = {.lex_state = 119},
  [383] = {.lex_state = 47},
  [384] = {.lex_state = 119},
  [385] = {.lex_state = 119},
  [386] = {.lex_state = 119},
  [387] = {.lex_state = 119},
  [388] = {.lex_state = 119},
  [389] = {.lex_state = 119},
  [390] = {.lex_state = 119},
  [391] = {.lex_state = 119},
  [392] = {.lex_state = 119},
  [393] = {.lex_state = 119},
  [394] = {.lex_state = 119},
  [395] = {.lex_state = 119},
  [396] = {.lex_state = 119},
  [397] = {.lex_state = 119},
  [398] = {.lex_state = 119},
  [399] = {.lex_state = 119},
  [400] = {.lex_state = 119},
  [401] = {.lex_state = 119},
  [402] = {.lex_state = 47},
  [403] = {.lex_state = 119},
  [404] = {.lex_state = 119},
  [405] = {.lex_state = 119},
  [406] = {.lex_state = 44},
  [407] = {.lex_state = 119},
  [408] = {.lex_state = 119},
  [409] = {.lex_state = 119},
  [410] = {.lex_state = 119},
  [411] = {.lex_state = 119},
  [412] = {.lex_state = 119},
  [413] = {.lex_state = 119},
  [414] = {.lex_state = 119},
  [415] = {.lex_state = 119},
  [416] = {.lex_state = 119},
  [417] = {.lex_state = 119},
  [418] = {.lex_state = 119},
  [419] = {.lex_state = 119},
  [420] = {.lex_state = 119},
  [421] = {.lex_state = 119},
  [422] = {.lex_state = 119},
  [423] = {.lex_state = 119},
  [424] = {.lex_state = 119},
  [425] = {.lex_state = 119},
  [426] = {.lex_state = 119},
  [427] = {.lex_state = 119},
  [428] = {.lex_state = 119},
  [429] = {.lex_state = 119},
  [430] = {.lex_state = 119},
  [431] = {.lex_state = 119},
  [432] = {.lex_state = 119},
  [433] = {.lex_state = 119},
  [434] = {.lex_state = 119},
  [435] = {.lex_state = 119},
  [436] = {.lex_state = 119},
  [437] = {.lex_state = 119},
  [438] = {.lex_state = 119},
  [439] = {.lex_state = 119},
  [440] = {.lex_state = 119},
  [441] = {.lex_state = 119},
  [442] = {.lex_state = 119},
  [443] = {.lex_state = 119},
  [444] = {.lex_state = 119},
  [445] = {.lex_state = 119},
  [446] = {.lex_state = 119},
  [447] = {.lex_state = 119},
  [448] = {.lex_state = 44},
  [449] = {.lex_state = 44},
  [450] = {.lex_state = 44},
  [451] = {.lex_state = 119},
  [452] = {.lex_state = 49},
  [453] = {.lex_state = 49},
  [454] = {.lex_state = 49},
  [455] = {.lex_state = 49},
  [456] = {.lex_state = 49},
  [457] = {.lex_state = 49},
  [458] = {.lex_state = 119},
  [459] = {.lex_state = 49},
  [460] = {.lex_state = 49},
  [461] = {.lex_state = 49},
  [462] = {.lex_state = 49},
  [463] = {.lex_state = 52},
  [464] = {.lex_state = 119},
  [465] = {.lex_state = 119},
  [466] = {.lex_state = 119},
  [467] = {.lex_state = 119},
  [468] = {.lex_state = 119},
  [469] = {.lex_state = 119},
  [470] = {.lex_state = 119},
  [471] = {.lex_state = 119},
  [472] = {.lex_state = 119},
  [473] = {.lex_state = 119},
  [474] = {.lex_state = 119},
  [475] = {.lex_state = 119},
  [476] = {.lex_state = 119},
  [477] = {.lex_state = 119},
  [478] = {.lex_state = 52},
  [479] = {.lex_state = 52},
  [480] = {.lex_state = 52},
  [481] = {.lex_state = 52},
  [482] = {.lex_state = 52},
  [483] = {.lex_state = 52},
  [484] = {.lex_state = 52},
  [485] = {.lex_state = 52},
  [486] = {.lex_state = 52},
  [487] = {.lex_state = 52},
  [488] = {.lex_state = 52},
  [489] = {.lex_state = 52},
  [490] = {.lex_state = 52},
  [491] = {.lex_state = 52},
  [492] = {.lex_state = 52},
  [493] = {.lex_state = 52},
  [494] = {.lex_state = 52},
  [495] = {.lex_state = 52},
  [496] = {.lex_state = 119},
  [497] = {.lex_state = 119},
  [498] = {.lex_state = 119},
  [499] = {.lex_state = 119},
  [500] = {.lex_state = 119},
  [501] = {.lex_state = 119},
  [502] = {.lex_state = 119},
  [503] = {.lex_state = 119},
  [504] = {.lex_state = 119},
  [505] = {.lex_state = 119},
  [506] = {.lex_state = 119},
  [507] = {.lex_state = 119},
  [508] = {.lex_state = 119},
  [509] = {.lex_state = 119},
  [510] = {.lex_state = 119},
  [511] = {.lex_state = 119},
  [512] = {.lex_state = 119},
  [513] = {.lex_state = 119},
  [514] = {.lex_state = 119},
  [515] = {.lex_state = 119},
  [516] = {.lex_state = 119},
  [517] = {.lex_state = 119},
  [518] = {.lex_state = 119},
  [519] = {.lex_state = 119},
  [520] = {.lex_state = 119},
  [521] = {.lex_state = 119},
  [522] = {.lex_state = 119},
  [523] = {.lex_state = 119},
  [524] = {.lex_state = 119},
  [525] = {.lex_state = 119},
  [526] = {.lex_state = 119},
  [527] = {.lex_state = 53},
  [528] = {.lex_state = 53},
  [529] = {.lex_state = 119},
  [530] = {.lex_state = 119},
  [531] = {.lex_state = 119},
  [532] = {.lex_state = 119},
  [533] = {.lex_state = 57},
  [534] = {.lex_state = 57},
  [535] = {.lex_state = 119},
  [536] = {.lex_state = 119},
  [537] = {.lex_state = 119},
  [538] = {.lex_state = 119},
  [539] = {.lex_state = 53},
  [540] = {.lex_state = 119},
  [541] = {.lex_state = 119},
  [542] = {.lex_state = 119},
  [543] = {.lex_state = 57},
  [544] = {.lex_state = 119},
  [545] = {.lex_state = 119},
  [546] = {.lex_state = 119},
  [547] = {.lex_state = 119},
  [548] = {.lex_state = 119},
  [549] = {.lex_state = 119},
  [550] = {.lex_state = 119},
  [551] = {.lex_state = 119},
  [552] = {.lex_state = 119},
  [553] = {.lex_state = 119},
  [554] = {.lex_state = 119},
  [555] = {.lex_state = 119},
  [556] = {.lex_state = 119},
  [557] = {.lex_state = 119},
  [558] = {.lex_state = 119},
  [559] = {.lex_state = 119},
  [560] = {.lex_state = 119},
  [561] = {.lex_state = 119},
  [562] = {.lex_state = 119},
  [563] = {.lex_state = 119},
  [564] = {.lex_state = 119},
  [565] = {.lex_state = 119},
  [566] = {.lex_state = 119},
  [567] = {.lex_state = 119},
  [568] = {.lex_state = 119},
  [569] = {.lex_state = 119},
  [570] = {.lex_state = 119},
  [571] = {.lex_state = 119},
  [572] = {.lex_state = 119},
  [573] = {.lex_state = 119},
  [574] = {.lex_state = 119},
  [575] = {.lex_state = 119},
  [576] = {.lex_state = 119},
  [577] = {.lex_state = 119},
  [578] = {.lex_state = 119},
  [579] = {.lex_state = 119},
  [580] = {.lex_state = 119},
  [581] = {.lex_state = 119},
  [582] = {.lex_state = 119},
  [583] = {.lex_state = 119},
  [584] = {.lex_state = 119},
  [585] = {.lex_state = 119},
  [586] = {.lex_state = 119},
  [587] = {.lex_state = 119},
  [588] = {.lex_state = 119},
  [589] = {.lex_state = 119},
  [590] = {.lex_state = 119},
  [591] = {.lex_state = 119},
  [592] = {.lex_state = 119},
  [593] = {.lex_state = 119},
  [594] = {.lex_state = 119},
  [595] = {.lex_state = 119},
  [596] = {.lex_state = 119},
  [597] = {.lex_state = 119},
  [598] = {.lex_state = 119},
  [599] = {.lex_state = 119},
  [600] = {.lex_state = 119},
  [601] = {.lex_state = 119},
  [602] = {.lex_state = 119},
  [603] = {.lex_state = 119},
  [604] = {.lex_state = 119},
  [605] = {.lex_state = 119},
  [606] = {.lex_state = 119},
  [607] = {.lex_state = 119},
  [608] = {.lex_state = 119},
  [609] = {.lex_state = 119},
  [610] = {.lex_state = 119},
  [611] = {.lex_state = 119},
  [612] = {.lex_state = 119},
  [613] = {.lex_state = 119},
  [614] = {.lex_state = 119},
  [615] = {.lex_state = 119},
  [616] = {.lex_state = 119},
  [617] = {.lex_state = 119},
  [618] = {.lex_state = 119},
  [619] = {.lex_state = 119},
  [620] = {.lex_state = 119},
  [621] = {.lex_state = 119},
  [622] = {.lex_state = 119},
  [623] = {.lex_state = 119},
  [624] = {.lex_state = 119},
  [625] = {.lex_state = 119},
  [626] = {.lex_state = 119},
  [627] = {.lex_state = 119},
  [628] = {.lex_state = 119},
  [629] = {.lex_state = 119},
  [630] = {.lex_state = 119},
  [631] = {.lex_state = 119},
  [632] = {.lex_state = 119},
  [633] = {.lex_state = 119},
  [634] = {.lex_state = 119},
  [635] = {.lex_state = 119},
  [636] = {.lex_state = 119},
  [637] = {.lex_state = 119},
  [638] = {.lex_state = 119},
  [639] = {.lex_state = 119},
  [640] = {.lex_state = 119},
  [641] = {.lex_state = 119},
  [642] = {.lex_state = 119},
  [643] = {.lex_state = 119},
  [644] = {.lex_state = 119},
  [645] = {.lex_state = 119},
  [646] = {.lex_state = 119},
  [647] = {.lex_state = 119},
  [648] = {.lex_state = 119},
  [649] = {.lex_state = 119},
  [650] = {.lex_state = 119},
  [651] = {.lex_state = 119},
  [652] = {.lex_state = 119},
  [653] = {.lex_state = 119},
  [654] = {.lex_state = 119},
  [655] = {.lex_state = 119},
  [656] = {.lex_state = 119},
  [657] = {.lex_state = 119},
  [658] = {.lex_state = 119},
  [659] = {.lex_state = 119},
  [660] = {.lex_state = 119},
  [661] = {.lex_state = 49},
  [662] = {.lex_state = 49},
  [663] = {.lex_state = 52},
  [664] = {.lex_state = 52},
  [665] = {.lex_state = 49},
  [666] = {.lex_state = 49},
  [667] = {.lex_state = 52},
  [668] = {.lex_state = 52},
  [669] = {.lex_state = 49},
  [670] = {.lex_state = 49},
  [671] = {.lex_state = 49},
  [672] = {.lex_state = 50},
  [673] = {.lex_state = 50},
  [674] = {.lex_state = 50},
  [675] = {.lex_state = 50},
  [676] = {.lex_state = 50},
  [677] = {.lex_state = 50},
  [678] = {.lex_state = 50},
  [679] = {.lex_state = 50},
  [680] = {.lex_state = 50},
  [681] = {.lex_state = 50},
  [682] = {.lex_state = 52},
  [683] = {.lex_state = 49},
  [684] = {.lex_state = 50},
  [685] = {.lex_state = 119},
  [686] = {.lex_state = 119},
  [687] = {.lex_state = 119},
  [688] = {.lex_state = 52},
  [689] = {.lex_state = 52},
  [690] = {.lex_state = 52},
  [691] = {.lex_state = 52},
  [692] = {.lex_state = 50},
  [693] = {.lex_state = 52},
  [694] = {.lex_state = 52},
  [695] = {.lex_state = 52},
  [696] = {.lex_state = 50},
  [697] = {.lex_state = 50},
  [698] = {.lex_state = 52},
  [699] = {.lex_state = 52},
  [700] = {.lex_state = 50},
  [701] = {.lex_state = 52},
  [702] = {.lex_state = 50},
  [703] = {.lex_state = 52},
  [704] = {.lex_state = 50},
  [705] = {.lex_state = 50},
  [706] = {.lex_state = 52},
  [707] = {.lex_state = 52},
  [708] = {.lex_state = 52},
  [709] = {.lex_state = 52},
  [710] = {.lex_state = 52},
  [711] = {.lex_state = 50},
  [712] = {.lex_state = 52},
  [713] = {.lex_state = 52},
  [714] = {.lex_state = 50},
  [715] = {.lex_state = 50},
  [716] = {.lex_state = 50},
  [717] = {.lex_state = 50},
  [718] = {.lex_state = 50},
  [719] = {.lex_state = 50},
  [720] = {.lex_state = 50},
  [721] = {.lex_state = 50},
  [722] = {.lex_state = 50},
  [723] = {.lex_state = 50},
  [724] = {.lex_state = 51},
  [725] = {.lex_state = 52},
  [726] = {.lex_state = 51},
  [727] = {.lex_state = 50},
  [728] = {.lex_state = 49},
  [729] = {.lex_state = 51},
  [730] = {.lex_state = 52},
  [731] = {.lex_state = 52},
  [732] = {.lex_state = 51},
  [733] = {.lex_state = 50},
  [734] = {.lex_state = 52},
  [735] = {.lex_state = 49},
  [736] = {.lex_state = 50},
  [737] = {.lex_state = 50},
  [738] = {.lex_state = 49},
  [739] = {.lex_state = 119},
  [740] = {.lex_state = 52},
  [741] = {.lex_state = 52},
  [742] = {.lex_state = 52},
  [743] = {.lex_state = 52},
  [744] = {.lex_state = 52},
  [745] = {.lex_state = 52},
  [746] = {.lex_state = 52},
  [747] = {.lex_state = 52},
  [748] = {.lex_state = 49},
  [749] = {.lex_state = 52},
  [750] = {.lex_state = 119},
  [751] = {.lex_state = 52},
  [752] = {.lex_state = 52},
  [753] = {.lex_state = 49},
  [754] = {.lex_state = 52},
  [755] = {.lex_state = 52},
  [756] = {.lex_state = 52},
  [757] = {.lex_state = 52},
  [758] = {.lex_state = 49},
  [759] = {.lex_state = 49},
  [760] = {.lex_state = 49},
  [761] = {.lex_state = 49},
  [762] = {.lex_state = 52},
  [763] = {.lex_state = 50},
  [764] = {.lex_state = 49},
  [765] = {.lex_state = 50},
  [766] = {.lex_state = 50},
  [767] = {.lex_state = 50},
  [768] = {.lex_state = 50},
  [769] = {.lex_state = 50},
  [770] = {.lex_state = 50},
  [771] = {.lex_state = 50},
  [772] = {.lex_state = 50},
  [773] = {.lex_state = 52},
  [774] = {.lex_state = 50},
  [775] = {.lex_state = 52},
  [776] = {.lex_state = 52},
  [777] = {.lex_state = 50},
  [778] = {.lex_state = 50},
  [779] = {.lex_state = 52},
  [780] = {.lex_state = 52},
  [781] = {.lex_state = 52},
  [782] = {.lex_state = 52},
  [783] = {.lex_state = 52},
  [784] = {.lex_state = 52},
  [785] = {.lex_state = 52},
  [786] = {.lex_state = 52},
  [787] = {.lex_state = 52},
  [788] = {.lex_state = 52},
  [789] = {.lex_state = 52},
  [790] = {.lex_state = 52},
  [791] = {.lex_state = 52},
  [792] = {.lex_state = 52},
  [793] = {.lex_state = 52},
  [794] = {.lex_state = 52},
  [795] = {.lex_state = 52},
  [796] = {.lex_state = 52},
  [797] = {.lex_state = 52},
  [798] = {.lex_state = 52},
  [799] = {.lex_state = 50},
  [800] = {.lex_state = 52},
  [801] = {.lex_state = 52},
  [802] = {.lex_state = 52},
  [803] = {.lex_state = 52},
  [804] = {.lex_state = 52},
  [805] = {.lex_state = 52},
  [806] = {.lex_state = 52},
  [807] = {.lex_state = 52},
  [808] = {.lex_state = 52},
  [809] = {.lex_state = 52},
  [810] = {.lex_state = 52},
  [811] = {.lex_state = 52},
  [812] = {.lex_state = 52},
  [813] = {.lex_state = 52},
  [814] = {.lex_state = 52},
  [815] = {.lex_state = 52},
  [816] = {.lex_state = 52},
  [817] = {.lex_state = 52},
  [818] = {.lex_state = 52},
  [819] = {.lex_state = 52},
  [820] = {.lex_state = 52},
  [821] = {.lex_state = 52},
  [822] = {.lex_state = 52},
  [823] = {.lex_state = 52},
  [824] = {.lex_state = 52},
  [825] = {.lex_state = 52},
  [826] = {.lex_state = 52},
  [827] = {.lex_state = 52},
  [828] = {.lex_state = 52},
  [829] = {.lex_state = 52},
  [830] = {.lex_state = 52},
  [831] = {.lex_state = 52},
  [832] = {.lex_state = 52},
  [833] = {.lex_state = 52},
  [834] = {.lex_state = 52},
  [835] = {.lex_state = 52},
  [836] = {.lex_state = 52},
  [837] = {.lex_state = 52},
  [838] = {.lex_state = 52},
  [839] = {.lex_state = 52},
  [840] = {.lex_state = 52},
  [841] = {.lex_state = 52},
  [842] = {.lex_state = 52},
  [843] = {.lex_state = 52},
  [844] = {.lex_state = 52},
  [845] = {.lex_state = 52},
  [846] = {.lex_state = 52},
  [847] = {.lex_state = 52},
  [848] = {.lex_state = 52},
  [849] = {.lex_state = 52},
  [850] = {.lex_state = 52},
  [851] = {.lex_state = 52},
  [852] = {.lex_state = 52},
  [853] = {.lex_state = 52},
  [854] = {.lex_state = 52},
  [855] = {.lex_state = 52},
  [856] = {.lex_state = 52},
  [857] = {.lex_state = 52},
  [858] = {.lex_state = 52},
  [859] = {.lex_state = 52},
  [860] = {.lex_state = 52},
  [861] = {.lex_state = 52},
  [862] = {.lex_state = 52},
  [863] = {.lex_state = 52},
  [864] = {.lex_state = 52},
  [865] = {.lex_state = 52},
  [866] = {.lex_state = 52},
  [867] = {.lex_state = 52},
  [868] = {.lex_state = 49},
  [869] = {.lex_state = 49},
  [870] = {.lex_state = 49},
  [871] = {.lex_state = 49},
  [872] = {.lex_state = 52},
  [873] = {.lex_state = 49},
  [874] = {.lex_state = 49},
  [875] = {.lex_state = 49},
  [876] = {.lex_state = 49},
  [877] = {.lex_state = 49},
  [878] = {.lex_state = 49},
  [879] = {.lex_state = 49},
  [880] = {.lex_state = 49},
  [881] = {.lex_state = 49},
  [882] = {.lex_state = 49},
  [883] = {.lex_state = 52},
  [884] = {.lex_state = 52},
  [885] = {.lex_state = 49},
  [886] = {.lex_state = 49},
  [887] = {.lex_state = 49},
  [888] = {.lex_state = 52},
  [889] = {.lex_state = 52},
  [890] = {.lex_state = 49},
  [891] = {.lex_state = 49},
  [892] = {.lex_state = 52},
  [893] = {.lex_state = 52},
  [894] = {.lex_state = 49},
  [895] = {.lex_state = 49},
  [896] = {.lex_state = 49},
  [897] = {.lex_state = 49},
  [898] = {.lex_state = 53},
  [899] = {.lex_state = 57},
  [900] = {.lex_state = 53},
  [901] = {.lex_state = 57},
  [902] = {.lex_state = 49},
  [903] = {.lex_state = 53},
  [904] = {.lex_state = 57},
  [905] = {.lex_state = 53},
  [906] = {.lex_state = 53},
  [907] = {.lex_state = 53},
  [908] = {.lex_state = 53},
  [909] = {.lex_state = 52},
  [910] = {.lex_state = 53},
  [911] = {.lex_state = 57},
  [912] = {.lex_state = 53},
  [913] = {.lex_state = 53},
  [914] = {.lex_state = 53},
  [915] = {.lex_state = 53},
  [916] = {.lex_state = 53},
  [917] = {.lex_state = 53},
  [918] = {.lex_state = 57},
  [919] = {.lex_state = 53},
  [920] = {.lex_state = 57},
  [921] = {.lex_state = 57},
  [922] = {.lex_state = 52},
  [923] = {.lex_state = 57},
  [924] = {.lex_state = 53},
  [925] = {.lex_state = 57},
  [926] = {.lex_state = 57},
  [927] = {.lex_state = 57},
  [928] = {.lex_state = 52},
  [929] = {.lex_state = 49},
  [930] = {.lex_state = 49},
  [931] = {.lex_state = 57},
  [932] = {.lex_state = 49},
  [933] = {.lex_state = 53},
  [934] = {.lex_state = 53},
  [935] = {.lex_state = 57},
  [936] = {.lex_state = 57},
  [937] = {.lex_state = 57},
  [938] = {.lex_state = 57},
  [939] = {.lex_state = 57},
  [940] = {.lex_state = 57},
  [941] = {.lex_state = 119},
  [942] = {.lex_state = 119},
  [943] = {.lex_state = 119},
  [944] = {.lex_state = 119},
  [945] = {.lex_state = 119},
  [946] = {.lex_state = 119},
  [947] = {.lex_state = 119},
  [948] = {.lex_state = 119},
  [949] = {.lex_state = 119},
  [950] = {.lex_state = 119},
  [951] = {.lex_state = 52},
  [952] = {.lex_state = 52},
  [953] = {.lex_state = 52},
  [954] = {.lex_state = 52},
  [955] = {.lex_state = 52},
  [956] = {.lex_state = 52},
  [957] = {.lex_state = 52},
  [958] = {.lex_state = 52},
  [959] = {.lex_state = 50},
  [960] = {.lex_state = 50},
  [961] = {.lex_state = 50},
  [962] = {.lex_state = 50},
  [963] = {.lex_state = 50},
  [964] = {.lex_state = 50},
  [965] = {.lex_state = 50},
  [966] = {.lex_state = 50},
  [967] = {.lex_state = 50},
  [968] = {.lex_state = 50},
  [969] = {.lex_state = 50},
  [970] = {.lex_state = 50},
  [971] = {.lex_state = 50},
  [972] = {.lex_state = 52},
  [973] = {.lex_state = 52},
  [974] = {.lex_state = 52},
  [975] = {.lex_state = 52},
  [976] = {.lex_state = 52},
  [977] = {.lex_state = 52},
  [978] = {.lex_state = 52},
  [979] = {.lex_state = 52},
  [980] = {.lex_state = 52},
  [981] = {.lex_state = 52},
  [982] = {.lex_state = 52},
  [983] = {.lex_state = 52},
  [984] = {.lex_state = 52},
  [985] = {.lex_state = 52},
  [986] = {.lex_state = 52},
  [987] = {.lex_state = 52},
  [988] = {.lex_state = 52},
  [989] = {.lex_state = 52},
  [990] = {.lex_state = 52},
  [991] = {.lex_state = 52},
  [992] = {.lex_state = 52},
  [993] = {.lex_state = 52},
  [994] = {.lex_state = 52},
  [995] = {.lex_state = 52},
  [996] = {.lex_state = 52},
  [997] = {.lex_state = 52},
  [998] = {.lex_state = 52},
  [999] = {.lex_state = 52},
  [1000] = {.lex_state = 52},
  [1001] = {.lex_state = 52},
  [1002] = {.lex_state = 52},
  [1003] = {.lex_state = 52},
  [1004] = {.lex_state = 52},
  [1005] = {.lex_state = 52},
  [1006] = {.lex_state = 52},
  [1007] = {.lex_state = 52},
  [1008] = {.lex_state = 53},
  [1009] = {.lex_state = 52},
  [1010] = {.lex_state = 52},
  [1011] = {.lex_state = 52},
  [1012] = {.lex_state = 53},
  [1013] = {.lex_state = 52},
  [1014] = {.lex_state = 53},
  [1015] = {.lex_state = 52},
  [1016] = {.lex_state = 52},
  [1017] = {.lex_state = 53},
  [1018] = {.lex_state = 49},
  [1019] = {.lex_state = 49},
  [1020] = {.lex_state = 52},
  [1021] = {.lex_state = 49},
  [1022] = {.lex_state = 49},
  [1023] = {.lex_state = 49},
  [1024] = {.lex_state = 49},
  [1025] = {.lex_state = 49},
  [1026] = {.lex_state = 49},
  [1027] = {.lex_state = 49},
  [1028] = {.lex_state = 52},
  [1029] = {.lex_state = 52},
  [1030] = {.lex_state = 49},
  [1031] = {.lex_state = 49},
  [1032] = {.lex_state = 49},
  [1033] = {.lex_state = 49},
  [1034] = {.lex_state = 49},
  [1035] = {.lex_state = 52},
  [1036] = {.lex_state = 49},
  [1037] = {.lex_state = 52},
  [1038] = {.lex_state = 52},
  [1039] = {.lex_state = 49},
  [1040] = {.lex_state = 49},
  [1041] = {.lex_state = 49},
  [1042] = {.lex_state = 49},
  [1043] = {.lex_state = 52},
  [1044] = {.lex_state = 49},
  [1045] = {.lex_state = 49},
  [1046] = {.lex_state = 49},
  [1047] = {.lex_state = 49},
  [1048] = {.lex_state = 49},
  [1049] = {.lex_state = 49},
  [1050] = {.lex_state = 49},
  [1051] = {.lex_state = 49},
  [1052] = {.lex_state = 52},
  [1053] = {.lex_state = 49},
  [1054] = {.lex_state = 49},
  [1055] = {.lex_state = 49},
  [1056] = {.lex_state = 49},
  [1057] = {.lex_state = 49},
  [1058] = {.lex_state = 49},
  [1059] = {.lex_state = 49},
  [1060] = {.lex_state = 52},
  [1061] = {.lex_state = 49},
  [1062] = {.lex_state = 52},
  [1063] = {.lex_state = 49},
  [1064] = {.lex_state = 49},
  [1065] = {.lex_state = 49},
  [1066] = {.lex_state = 49},
  [1067] = {.lex_state = 49},
  [1068] = {.lex_state = 49},
  [1069] = {.lex_state = 49},
  [1070] = {.lex_state = 49},
  [1071] = {.lex_state = 52},
  [1072] = {.lex_state = 52},
  [1073] = {.lex_state = 52},
  [1074] = {.lex_state = 49},
  [1075] = {.lex_state = 49},
  [1076] = {.lex_state = 49},
  [1077] = {.lex_state = 52},
  [1078] = {.lex_state = 49},
  [1079] = {.lex_state = 49},
  [1080] = {.lex_state = 49},
  [1081] = {.lex_state = 49},
  [1082] = {.lex_state = 49},
  [1083] = {.lex_state = 49},
  [1084] = {.lex_state = 49},
  [1085] = {.lex_state = 49},
  [1086] = {.lex_state = 49},
  [1087] = {.lex_state = 49},
  [1088] = {.lex_state = 49},
  [1089] = {.lex_state = 52},
  [1090] = {.lex_state = 49},
  [1091] = {.lex_state = 49},
  [1092] = {.lex_state = 49},
  [1093] = {.lex_state = 49},
  [1094] = {.lex_state = 49},
  [1095] = {.lex_state = 49},
  [1096] = {.lex_state = 49},
  [1097] = {.lex_state = 49},
  [1098] = {.lex_state = 49},
  [1099] = {.lex_state = 52},
  [1100] = {.lex_state = 49},
  [1101] = {.lex_state = 49},
  [1102] = {.lex_state = 52},
  [1103] = {.lex_state = 49},
  [1104] = {.lex_state = 49},
  [1105] = {.lex_state = 52},
  [1106] = {.lex_state = 49},
  [1107] = {.lex_state = 52},
  [1108] = {.lex_state = 52},
  [1109] = {.lex_state = 52},
  [1110] = {.lex_state = 52},
  [1111] = {.lex_state = 49},
  [1112] = {.lex_state = 49},
  [1113] = {.lex_state = 52},
  [1114] = {.lex_state = 49},
  [1115] = {.lex_state = 49},
  [1116] = {.lex_state = 52},
  [1117] = {.lex_state = 49},
  [1118] = {.lex_state = 52},
  [1119] = {.lex_state = 52},
  [1120] = {.lex_state = 52},
  [1121] = {.lex_state = 52},
  [1122] = {.lex_state = 52},
  [1123] = {.lex_state = 52},
  [1124] = {.lex_state = 52},
  [1125] = {.lex_state = 52},
  [1126] = {.lex_state = 52},
  [1127] = {.lex_state = 52},
  [1128] = {.lex_state = 52},
  [1129] = {.lex_state = 52},
  [1130] = {.lex_state = 52},
  [1131] = {.lex_state = 52},
  [1132] = {.lex_state = 52},
  [1133] = {.lex_state = 52},
  [1134] = {.lex_state = 52},
  [1135] = {.lex_state = 52},
  [1136] = {.lex_state = 52},
  [1137] = {.lex_state = 52},
  [1138] = {.lex_state = 52},
  [1139] = {.lex_state = 52},
  [1140] = {.lex_state = 53},
  [1141] = {.lex_state = 52},
  [1142] = {.lex_state = 52},
  [1143] = {.lex_state = 52},
  [1144] = {.lex_state = 52},
  [1145] = {.lex_state = 52},
  [1146] = {.lex_state = 48},
  [1147] = {.lex_state = 52},
  [1148] = {.lex_state = 53},
  [1149] = {.lex_state = 52},
  [1150] = {.lex_state = 53},
  [1151] = {.lex_state = 25},
  [1152] = {.lex_state = 48},
  [1153] = {.lex_state = 52},
  [1154] = {.lex_state = 52},
  [1155] = {.lex_state = 52},
  [1156] = {.lex_state = 52},
  [1157] = {.lex_state = 53},
  [1158] = {.lex_state = 52},
  [1159] = {.lex_state = 48},
  [1160] = {.lex_state = 53},
  [1161] = {.lex_state = 48},
  [1162] = {.lex_state = 53},
  [1163] = {.lex_state = 53},
  [1164] = {.lex_state = 53},
  [1165] = {.lex_state = 53},
  [1166] = {.lex_state = 53},
  [1167] = {.lex_state = 48},
  [1168] = {.lex_state = 53},
  [1169] = {.lex_state = 53},
  [1170] = {.lex_state = 53},
  [1171] = {.lex_state = 48},
  [1172] = {.lex_state = 48},
  [1173] = {.lex_state = 48},
  [1174] = {.lex_state = 53},
  [1175] = {.lex_state = 53},
  [1176] = {.lex_state = 52},
  [1177] = {.lex_state = 48},
  [1178] = {.lex_state = 48},
  [1179] = {.lex_state = 48},
  [1180] = {.lex_state = 53},
  [1181] = {.lex_state = 48},
  [1182] = {.lex_state = 48},
  [1183] = {.lex_state = 48},
  [1184] = {.lex_state = 52},
  [1185] = {.lex_state = 48},
  [1186] = {.lex_state = 48},
  [1187] = {.lex_state = 48},
  [1188] = {.lex_state = 48},
  [1189] = {.lex_state = 48},
  [1190] = {.lex_state = 48},
  [1191] = {.lex_state = 48},
  [1192] = {.lex_state = 53},
  [1193] = {.lex_state = 48},
  [1194] = {.lex_state = 48},
  [1195] = {.lex_state = 48},
  [1196] = {.lex_state = 52},
  [1197] = {.lex_state = 53},
  [1198] = {.lex_state = 52},
  [1199] = {.lex_state = 48},
  [1200] = {.lex_state = 48},
  [1201] = {.lex_state = 48},
  [1202] = {.lex_state = 48},
  [1203] = {.lex_state = 48},
  [1204] = {.lex_state = 48},
  [1205] = {.lex_state = 48},
  [1206] = {.lex_state = 48},
  [1207] = {.lex_state = 53},
  [1208] = {.lex_state = 48},
  [1209] = {.lex_state = 48},
  [1210] = {.lex_state = 48},
  [1211] = {.lex_state = 53},
  [1212] = {.lex_state = 48},
  [1213] = {.lex_state = 48},
  [1214] = {.lex_state = 48},
  [1215] = {.lex_state = 48},
  [1216] = {.lex_state = 48},
  [1217] = {.lex_state = 48},
  [1218] = {.lex_state = 52},
  [1219] = {.lex_state = 48},
  [1220] = {.lex_state = 25},
  [1221] = {.lex_state = 25},
  [1222] = {.lex_state = 25},
  [1223] = {.lex_state = 52},
  [1224] = {.lex_state = 25},
  [1225] = {.lex_state = 25},
  [1226] = {.lex_state = 52},
  [1227] = {.lex_state = 25},
  [1228] = {.lex_state = 52},
  [1229] = {.lex_state = 52},
  [1230] = {.lex_state = 25},
  [1231] = {.lex_state = 25},
  [1232] = {.lex_state = 25},
  [1233] = {.lex_state = 25},
  [1234] = {.lex_state = 25},
  [1235] = {.lex_state = 52},
  [1236] = {.lex_state = 25},
  [1237] = {.lex_state = 53},
  [1238] = {.lex_state = 25},
  [1239] = {.lex_state = 52},
  [1240] = {.lex_state = 25},
  [1241] = {.lex_state = 25},
  [1242] = {.lex_state = 25},
  [1243] = {.lex_state = 25},
  [1244] = {.lex_state = 25},
  [1245] = {.lex_state = 52},
  [1246] = {.lex_state = 25},
  [1247] = {.lex_state = 25},
  [1248] = {.lex_state = 25},
  [1249] = {.lex_state = 52},
  [1250] = {.lex_state = 25},
  [1251] = {.lex_state = 25},
  [1252] = {.lex_state = 25},
  [1253] = {.lex_state = 52},
  [1254] = {.lex_state = 53},
  [1255] = {.lex_state = 25},
  [1256] = {.lex_state = 25},
  [1257] = {.lex_state = 25},
  [1258] = {.lex_state = 52},
  [1259] = {.lex_state = 25},
  [1260] = {.lex_state = 52},
  [1261] = {.lex_state = 52},
  [1262] = {.lex_state = 25},
  [1263] = {.lex_state = 52},
  [1264] = {.lex_state = 25},
  [1265] = {.lex_state = 52},
  [1266] = {.lex_state = 25},
  [1267] = {.lex_state = 25},
  [1268] = {.lex_state = 25},
  [1269] = {.lex_state = 52},
  [1270] = {.lex_state = 52},
  [1271] = {.lex_state = 52},
  [1272] = {.lex_state = 52},
  [1273] = {.lex_state = 52},
  [1274] = {.lex_state = 52},
  [1275] = {.lex_state = 52},
  [1276] = {.lex_state = 52},
  [1277] = {.lex_state = 52},
  [1278] = {.lex_state = 52},
  [1279] = {.lex_state = 52},
  [1280] = {.lex_state = 52},
  [1281] = {.lex_state = 52},
  [1282] = {.lex_state = 52},
  [1283] = {.lex_state = 52},
  [1284] = {.lex_state = 52},
  [1285] = {.lex_state = 52},
  [1286] = {.lex_state = 52},
  [1287] = {.lex_state = 52},
  [1288] = {.lex_state = 52},
  [1289] = {.lex_state = 52},
  [1290] = {.lex_state = 50},
  [1291] = {.lex_state = 52},
  [1292] = {.lex_state = 52},
  [1293] = {.lex_state = 52},
  [1294] = {.lex_state = 50},
  [1295] = {.lex_state = 52},
  [1296] = {.lex_state = 52},
  [1297] = {.lex_state = 52},
  [1298] = {.lex_state = 50},
  [1299] = {.lex_state = 52},
  [1300] = {.lex_state = 50},
  [1301] = {.lex_state = 52},
  [1302] = {.lex_state = 52},
  [1303] = {.lex_state = 52},
  [1304] = {.lex_state = 52},
  [1305] = {.lex_state = 52},
  [1306] = {.lex_state = 52},
  [1307] = {.lex_state = 52},
  [1308] = {.lex_state = 52},
  [1309] = {.lex_state = 52},
  [1310] = {.lex_state = 56},
  [1311] = {.lex_state = 52},
  [1312] = {.lex_state = 52},
  [1313] = {.lex_state = 52},
  [1314] = {.lex_state = 52},
  [1315] = {.lex_state = 52},
  [1316] = {.lex_state = 52},
  [1317] = {.lex_state = 56},
  [1318] = {.lex_state = 52},
  [1319] = {.lex_state = 52},
  [1320] = {.lex_state = 52},
  [1321] = {.lex_state = 52},
  [1322] = {.lex_state = 52},
  [1323] = {.lex_state = 52},
  [1324] = {.lex_state = 52},
  [1325] = {.lex_state = 50},
  [1326] = {.lex_state = 50},
  [1327] = {.lex_state = 52},
  [1328] = {.lex_state = 56},
  [1329] = {.lex_state = 50},
  [1330] = {.lex_state = 50},
  [1331] = {.lex_state = 50},
  [1332] = {.lex_state = 52},
  [1333] = {.lex_state = 52},
  [1334] = {.lex_state = 52},
  [1335] = {.lex_state = 52},
  [1336] = {.lex_state = 50},
  [1337] = {.lex_state = 52},
  [1338] = {.lex_state = 50},
  [1339] = {.lex_state = 50},
  [1340] = {.lex_state = 52},
  [1341] = {.lex_state = 52},
  [1342] = {.lex_state = 52},
  [1343] = {.lex_state = 49},
  [1344] = {.lex_state = 52},
  [1345] = {.lex_state = 50},
  [1346] = {.lex_state = 50},
  [1347] = {.lex_state = 52},
  [1348] = {.lex_state = 50},
  [1349] = {.lex_state = 50},
  [1350] = {.lex_state = 52},
  [1351] = {.lex_state = 50},
  [1352] = {.lex_state = 50},
  [1353] = {.lex_state = 52},
  [1354] = {.lex_state = 52},
  [1355] = {.lex_state = 52},
  [1356] = {.lex_state = 52},
  [1357] = {.lex_state = 119},
  [1358] = {.lex_state = 119},
  [1359] = {.lex_state = 52},
  [1360] = {.lex_state = 52},
  [1361] = {.lex_state = 52},
  [1362] = {.lex_state = 119},
  [1363] = {.lex_state = 52},
  [1364] = {.lex_state = 52},
  [1365] = {.lex_state = 0},
  [1366] = {.lex_state = 52},
  [1367] = {.lex_state = 52},
  [1368] = {.lex_state = 52},
  [1369] = {.lex_state = 119},
  [1370] = {.lex_state = 0},
  [1371] = {.lex_state = 52},
  [1372] = {.lex_state = 52},
  [1373] = {.lex_state = 119},
  [1374] = {.lex_state = 52},
  [1375] = {.lex_state = 119},
  [1376] = {.lex_state = 119},
  [1377] = {.lex_state = 119},
  [1378] = {.lex_state = 119},
  [1379] = {.lex_state = 119},
  [1380] = {.lex_state = 49},
  [1381] = {.lex_state = 54},
  [1382] = {.lex_state = 49},
  [1383] = {.lex_state = 119},
  [1384] = {.lex_state = 54},
  [1385] = {.lex_state = 52},
  [1386] = {.lex_state = 52},
  [1387] = {.lex_state = 49},
  [1388] = {.lex_state = 119},
  [1389] = {.lex_state = 54},
  [1390] = {.lex_state = 119},
  [1391] = {.lex_state = 54},
  [1392] = {.lex_state = 52},
  [1393] = {.lex_state = 52},
  [1394] = {.lex_state = 119},
  [1395] = {.lex_state = 52},
  [1396] = {.lex_state = 119},
  [1397] = {.lex_state = 119},
  [1398] = {.lex_state = 52},
  [1399] = {.lex_state = 52},
  [1400] = {.lex_state = 52},
  [1401] = {.lex_state = 119},
  [1402] = {.lex_state = 119},
  [1403] = {.lex_state = 119},
  [1404] = {.lex_state = 119},
  [1405] = {.lex_state = 119},
  [1406] = {.lex_state = 119},
  [1407] = {.lex_state = 119},
  [1408] = {.lex_state = 0},
  [1409] = {.lex_state = 50},
  [1410] = {.lex_state = 50},
  [1411] = {.lex_state = 119},
  [1412] = {.lex_state = 52},
  [1413] = {.lex_state = 50},
  [1414] = {.lex_state = 119},
  [1415] = {.lex_state = 52},
  [1416] = {.lex_state = 52},
  [1417] = {.lex_state = 119},
  [1418] = {.lex_state = 0},
  [1419] = {.lex_state = 52},
  [1420] = {.lex_state = 50},
  [1421] = {.lex_state = 119},
  [1422] = {.lex_state = 52},
  [1423] = {.lex_state = 52},
  [1424] = {.lex_state = 52},
  [1425] = {.lex_state = 119},
  [1426] = {.lex_state = 52},
  [1427] = {.lex_state = 119},
  [1428] = {.lex_state = 52},
  [1429] = {.lex_state = 119},
  [1430] = {.lex_state = 0},
  [1431] = {.lex_state = 119},
  [1432] = {.lex_state = 52},
  [1433] = {.lex_state = 52},
  [1434] = {.lex_state = 52},
  [1435] = {.lex_state = 52},
  [1436] = {.lex_state = 52},
  [1437] = {.lex_state = 119},
  [1438] = {.lex_state = 52},
  [1439] = {.lex_state = 119},
  [1440] = {.lex_state = 0},
  [1441] = {.lex_state = 52},
  [1442] = {.lex_state = 52},
  [1443] = {.lex_state = 52},
  [1444] = {.lex_state = 52},
  [1445] = {.lex_state = 52},
  [1446] = {.lex_state = 119},
  [1447] = {.lex_state = 119},
  [1448] = {.lex_state = 56},
  [1449] = {.lex_state = 119},
  [1450] = {.lex_state = 52},
  [1451] = {.lex_state = 119},
  [1452] = {.lex_state = 0},
  [1453] = {.lex_state = 56},
  [1454] = {.lex_state = 56},
  [1455] = {.lex_state = 56},
  [1456] = {.lex_state = 56},
  [1457] = {.lex_state = 0},
  [1458] = {.lex_state = 0},
  [1459] = {.lex_state = 52},
  [1460] = {.lex_state = 52},
  [1461] = {.lex_state = 52},
  [1462] = {.lex_state = 56},
  [1463] = {.lex_state = 52},
  [1464] = {.lex_state = 56},
  [1465] = {.lex_state = 50},
  [1466] = {.lex_state = 52},
  [1467] = {.lex_state = 56},
  [1468] = {.lex_state = 52},
  [1469] = {.lex_state = 119},
  [1470] = {.lex_state = 52},
  [1471] = {.lex_state = 52},
  [1472] = {.lex_state = 52},
  [1473] = {.lex_state = 119},
  [1474] = {.lex_state = 119},
  [1475] = {.lex_state = 119},
  [1476] = {.lex_state = 56},
  [1477] = {.lex_state = 52},
  [1478] = {.lex_state = 56},
  [1479] = {.lex_state = 56},
  [1480] = {.lex_state = 52},
  [1481] = {.lex_state = 119},
  [1482] = {.lex_state = 119},
  [1483] = {.lex_state = 52},
  [1484] = {.lex_state = 119},
  [1485] = {.lex_state = 45},
  [1486] = {.lex_state = 119},
  [1487] = {.lex_state = 119},
  [1488] = {.lex_state = 119},
  [1489] = {.lex_state = 119},
  [1490] = {.lex_state = 119},
  [1491] = {.lex_state = 119},
  [1492] = {.lex_state = 119},
  [1493] = {.lex_state = 50},
  [1494] = {.lex_state = 52},
  [1495] = {.lex_state = 119},
  [1496] = {.lex_state = 119},
  [1497] = {.lex_state = 119},
  [1498] = {.lex_state = 52},
  [1499] = {.lex_state = 52},
  [1500] = {.lex_state = 119},
  [1501] = {.lex_state = 52},
  [1502] = {.lex_state = 30},
  [1503] = {.lex_state = 52},
  [1504] = {.lex_state = 52},
  [1505] = {.lex_state = 52},
  [1506] = {.lex_state = 0},
  [1507] = {.lex_state = 52},
  [1508] = {.lex_state = 52},
  [1509] = {.lex_state = 0},
  [1510] = {.lex_state = 52},
  [1511] = {.lex_state = 52},
  [1512] = {.lex_state = 52},
  [1513] = {.lex_state = 0},
  [1514] = {.lex_state = 32},
  [1515] = {.lex_state = 37},
  [1516] = {.lex_state = 52},
  [1517] = {.lex_state = 32},
  [1518] = {.lex_state = 32},
  [1519] = {.lex_state = 0},
  [1520] = {.lex_state = 0},
  [1521] = {.lex_state = 0},
  [1522] = {.lex_state = 30},
  [1523] = {.lex_state = 119},
  [1524] = {.lex_state = 52},
  [1525] = {.lex_state = 37},
  [1526] = {.lex_state = 32},
  [1527] = {.lex_state = 52},
  [1528] = {.lex_state = 37},
  [1529] = {.lex_state = 0},
  [1530] = {.lex_state = 52},
  [1531] = {.lex_state = 52},
  [1532] = {.lex_state = 0},
  [1533] = {.lex_state = 0},
  [1534] = {.lex_state = 119},
  [1535] = {.lex_state = 52},
  [1536] = {.lex_state = 52},
  [1537] = {.lex_state = 52},
  [1538] = {.lex_state = 32},
  [1539] = {.lex_state = 0},
  [1540] = {.lex_state = 52},
  [1541] = {.lex_state = 52},
  [1542] = {.lex_state = 52},
  [1543] = {.lex_state = 52},
  [1544] = {.lex_state = 30},
  [1545] = {.lex_state = 52},
  [1546] = {.lex_state = 50},
  [1547] = {.lex_state = 52},
  [1548] = {.lex_state = 32},
  [1549] = {.lex_state = 32},
  [1550] = {.lex_state = 52},
  [1551] = {.lex_state = 52},
  [1552] = {.lex_state = 52},
  [1553] = {.lex_state = 50},
  [1554] = {.lex_state = 50},
  [1555] = {.lex_state = 52},
  [1556] = {.lex_state = 37},
  [1557] = {.lex_state = 37},
  [1558] = {.lex_state = 37},
  [1559] = {.lex_state = 52},
  [1560] = {.lex_state = 30},
  [1561] = {.lex_state = 52},
  [1562] = {.lex_state = 50},
  [1563] = {.lex_state = 45},
  [1564] = {.lex_state = 37},
  [1565] = {.lex_state = 0},
  [1566] = {.lex_state = 0},
  [1567] = {.lex_state = 0},
  [1568] = {.lex_state = 0},
  [1569] = {.lex_state = 0},
  [1570] = {.lex_state = 0},
  [1571] = {.lex_state = 0},
  [1572] = {.lex_state = 0},
  [1573] = {.lex_state = 0},
  [1574] = {.lex_state = 0},
  [1575] = {.lex_state = 0},
  [1576] = {.lex_state = 0},
  [1577] = {.lex_state = 0},
  [1578] = {.lex_state = 0},
  [1579] = {.lex_state = 0},
  [1580] = {.lex_state = 52},
  [1581] = {.lex_state = 45},
  [1582] = {.lex_state = 0},
  [1583] = {.lex_state = 0},
  [1584] = {.lex_state = 0},
  [1585] = {.lex_state = 0},
  [1586] = {.lex_state = 0},
  [1587] = {.lex_state = 0},
  [1588] = {.lex_state = 52},
  [1589] = {.lex_state = 0},
  [1590] = {.lex_state = 0},
  [1591] = {.lex_state = 0},
  [1592] = {.lex_state = 0},
  [1593] = {.lex_state = 45},
  [1594] = {.lex_state = 0},
  [1595] = {.lex_state = 0},
  [1596] = {.lex_state = 0},
  [1597] = {.lex_state = 0},
  [1598] = {.lex_state = 0},
  [1599] = {.lex_state = 52},
  [1600] = {.lex_state = 0},
  [1601] = {.lex_state = 0},
  [1602] = {.lex_state = 0},
  [1603] = {.lex_state = 52},
  [1604] = {.lex_state = 52},
  [1605] = {.lex_state = 0},
  [1606] = {.lex_state = 0},
  [1607] = {.lex_state = 0},
  [1608] = {.lex_state = 0},
  [1609] = {.lex_state = 0},
  [1610] = {.lex_state = 0},
  [1611] = {.lex_state = 30},
  [1612] = {.lex_state = 34},
  [1613] = {.lex_state = 52},
  [1614] = {.lex_state = 34},
  [1615] = {.lex_state = 45},
  [1616] = {.lex_state = 0},
  [1617] = {.lex_state = 45},
  [1618] = {.lex_state = 0},
  [1619] = {.lex_state = 0},
  [1620] = {.lex_state = 0},
  [1621] = {.lex_state = 0},
  [1622] = {.lex_state = 0},
  [1623] = {.lex_state = 0},
  [1624] = {.lex_state = 34},
  [1625] = {.lex_state = 0},
  [1626] = {.lex_state = 0},
  [1627] = {.lex_state = 0},
  [1628] = {.lex_state = 0},
  [1629] = {.lex_state = 0},
  [1630] = {.lex_state = 0},
  [1631] = {.lex_state = 0},
  [1632] = {.lex_state = 0},
  [1633] = {.lex_state = 0},
  [1634] = {.lex_state = 0},
  [1635] = {.lex_state = 0},
  [1636] = {.lex_state = 0},
  [1637] = {.lex_state = 0},
  [1638] = {.lex_state = 52},
  [1639] = {.lex_state = 0},
  [1640] = {.lex_state = 0},
  [1641] = {.lex_state = 52},
  [1642] = {.lex_state = 45},
  [1643] = {.lex_state = 45},
  [1644] = {.lex_state = 0},
  [1645] = {.lex_state = 0},
  [1646] = {.lex_state = 0},
  [1647] = {.lex_state = 0},
  [1648] = {.lex_state = 52},
  [1649] = {.lex_state = 0},
  [1650] = {.lex_state = 30},
  [1651] = {.lex_state = 45},
  [1652] = {.lex_state = 0},
  [1653] = {.lex_state = 0},
  [1654] = {.lex_state = 34},
  [1655] = {.lex_state = 0},
  [1656] = {.lex_state = 0},
  [1657] = {.lex_state = 0},
  [1658] = {.lex_state = 0},
  [1659] = {.lex_state = 0},
  [1660] = {.lex_state = 0},
  [1661] = {.lex_state = 0},
  [1662] = {.lex_state = 0},
  [1663] = {.lex_state = 0},
  [1664] = {.lex_state = 52},
  [1665] = {.lex_state = 0},
  [1666] = {.lex_state = 0},
  [1667] = {.lex_state = 0},
  [1668] = {.lex_state = 0},
  [1669] = {.lex_state = 0},
  [1670] = {.lex_state = 0},
  [1671] = {.lex_state = 0},
  [1672] = {.lex_state = 0},
  [1673] = {.lex_state = 0},
  [1674] = {.lex_state = 0},
  [1675] = {.lex_state = 0},
  [1676] = {.lex_state = 0},
  [1677] = {.lex_state = 0},
  [1678] = {.lex_state = 52},
  [1679] = {.lex_state = 0},
  [1680] = {.lex_state = 30},
  [1681] = {.lex_state = 0},
  [1682] = {.lex_state = 0},
  [1683] = {.lex_state = 0},
  [1684] = {.lex_state = 0},
  [1685] = {.lex_state = 0},
  [1686] = {.lex_state = 52},
  [1687] = {.lex_state = 119},
  [1688] = {.lex_state = 0},
  [1689] = {.lex_state = 119},
  [1690] = {.lex_state = 119},
  [1691] = {.lex_state = 52},
  [1692] = {.lex_state = 119},
  [1693] = {.lex_state = 0},
  [1694] = {.lex_state = 119},
  [1695] = {.lex_state = 119},
  [1696] = {.lex_state = 0},
  [1697] = {.lex_state = 0},
  [1698] = {.lex_state = 0},
  [1699] = {.lex_state = 33},
  [1700] = {.lex_state = 119},
  [1701] = {.lex_state = 0},
  [1702] = {.lex_state = 33},
  [1703] = {.lex_state = 0},
  [1704] = {.lex_state = 0},
  [1705] = {.lex_state = 119},
  [1706] = {.lex_state = 119},
  [1707] = {.lex_state = 0},
  [1708] = {.lex_state = 33},
  [1709] = {.lex_state = 119},
  [1710] = {.lex_state = 52},
  [1711] = {.lex_state = 33},
  [1712] = {.lex_state = 0},
  [1713] = {.lex_state = 0},
  [1714] = {.lex_state = 0},
  [1715] = {.lex_state = 0},
  [1716] = {.lex_state = 119},
  [1717] = {.lex_state = 119},
  [1718] = {.lex_state = 33},
  [1719] = {.lex_state = 0},
  [1720] = {.lex_state = 0},
  [1721] = {.lex_state = 0},
  [1722] = {.lex_state = 33},
  [1723] = {.lex_state = 0},
  [1724] = {.lex_state = 0},
  [1725] = {.lex_state = 45},
  [1726] = {.lex_state = 0},
  [1727] = {.lex_state = 33},
  [1728] = {.lex_state = 45},
  [1729] = {.lex_state = 0},
  [1730] = {.lex_state = 33},
  [1731] = {.lex_state = 52},
  [1732] = {.lex_state = 0},
  [1733] = {.lex_state = 52},
  [1734] = {.lex_state = 119},
  [1735] = {.lex_state = 119},
  [1736] = {.lex_state = 52},
  [1737] = {.lex_state = 0},
  [1738] = {.lex_state = 0},
  [1739] = {.lex_state = 0},
  [1740] = {.lex_state = 119},
  [1741] = {.lex_state = 119},
  [1742] = {.lex_state = 119},
  [1743] = {.lex_state = 33},
  [1744] = {.lex_state = 0},
  [1745] = {.lex_state = 119},
  [1746] = {.lex_state = 33},
  [1747] = {.lex_state = 33},
  [1748] = {.lex_state = 0},
  [1749] = {.lex_state = 52},
  [1750] = {.lex_state = 0},
  [1751] = {.lex_state = 0},
  [1752] = {.lex_state = 52},
  [1753] = {.lex_state = 0},
  [1754] = {.lex_state = 119},
  [1755] = {.lex_state = 33},
  [1756] = {.lex_state = 119},
  [1757] = {.lex_state = 33},
  [1758] = {.lex_state = 45},
  [1759] = {.lex_state = 119},
  [1760] = {.lex_state = 0},
  [1761] = {.lex_state = 33},
  [1762] = {.lex_state = 33},
  [1763] = {.lex_state = 33},
  [1764] = {.lex_state = 119},
  [1765] = {.lex_state = 0},
  [1766] = {.lex_state = 0},
  [1767] = {.lex_state = 52},
  [1768] = {.lex_state = 119},
  [1769] = {.lex_state = 33},
  [1770] = {.lex_state = 0},
  [1771] = {.lex_state = 119},
  [1772] = {.lex_state = 119},
  [1773] = {.lex_state = 0},
  [1774] = {.lex_state = 0},
  [1775] = {.lex_state = 0},
  [1776] = {.lex_state = 44},
  [1777] = {.lex_state = 0},
  [1778] = {.lex_state = 119},
  [1779] = {.lex_state = 44},
  [1780] = {.lex_state = 44},
  [1781] = {.lex_state = 0},
  [1782] = {.lex_state = 0},
  [1783] = {.lex_state = 34},
  [1784] = {.lex_state = 44},
  [1785] = {.lex_state = 44},
  [1786] = {.lex_state = 0},
  [1787] = {.lex_state = 44},
  [1788] = {.lex_state = 0},
  [1789] = {.lex_state = 44},
  [1790] = {.lex_state = 0},
  [1791] = {.lex_state = 44},
  [1792] = {.lex_state = 52},
  [1793] = {.lex_state = 0},
  [1794] = {.lex_state = 44},
  [1795] = {.lex_state = 34},
  [1796] = {.lex_state = 34},
  [1797] = {.lex_state = 0},
  [1798] = {.lex_state = 44},
  [1799] = {.lex_state = 0},
  [1800] = {.lex_state = 34},
  [1801] = {.lex_state = 0},
  [1802] = {.lex_state = 0},
  [1803] = {.lex_state = 0},
  [1804] = {.lex_state = 0},
  [1805] = {.lex_state = 0},
  [1806] = {.lex_state = 0},
  [1807] = {.lex_state = 34},
  [1808] = {.lex_state = 0},
  [1809] = {.lex_state = 0},
  [1810] = {.lex_state = 34},
  [1811] = {.lex_state = 119},
  [1812] = {.lex_state = 34},
  [1813] = {.lex_state = 119},
  [1814] = {.lex_state = 0},
  [1815] = {.lex_state = 0},
  [1816] = {.lex_state = 34},
  [1817] = {.lex_state = 0},
  [1818] = {.lex_state = 52},
  [1819] = {.lex_state = 34},
  [1820] = {.lex_state = 0},
  [1821] = {.lex_state = 0},
  [1822] = {.lex_state = 0},
  [1823] = {.lex_state = 119},
  [1824] = {.lex_state = 0},
  [1825] = {.lex_state = 34},
  [1826] = {.lex_state = 119},
  [1827] = {.lex_state = 0},
  [1828] = {.lex_state = 44},
  [1829] = {.lex_state = 44},
  [1830] = {.lex_state = 44},
  [1831] = {.lex_state = 44},
  [1832] = {.lex_state = 34},
  [1833] = {.lex_state = 34},
  [1834] = {.lex_state = 0},
  [1835] = {.lex_state = 34},
  [1836] = {.lex_state = 0},
  [1837] = {.lex_state = 52},
  [1838] = {.lex_state = 52},
  [1839] = {.lex_state = 0},
  [1840] = {.lex_state = 44},
  [1841] = {.lex_state = 44},
  [1842] = {.lex_state = 0},
  [1843] = {.lex_state = 44},
  [1844] = {.lex_state = 0},
  [1845] = {.lex_state = 0},
  [1846] = {.lex_state = 52},
  [1847] = {.lex_state = 0},
  [1848] = {.lex_state = 0},
  [1849] = {.lex_state = 119},
  [1850] = {.lex_state = 0},
  [1851] = {.lex_state = 0},
  [1852] = {.lex_state = 52},
  [1853] = {.lex_state = 0},
  [1854] = {.lex_state = 44},
  [1855] = {.lex_state = 0},
  [1856] = {.lex_state = 44},
  [1857] = {.lex_state = 34},
  [1858] = {.lex_state = 0},
  [1859] = {.lex_state = 0},
  [1860] = {.lex_state = 0},
  [1861] = {.lex_state = 0},
  [1862] = {.lex_state = 34},
  [1863] = {.lex_state = 0},
  [1864] = {.lex_state = 44},
  [1865] = {.lex_state = 44},
  [1866] = {.lex_state = 44},
  [1867] = {.lex_state = 52},
  [1868] = {.lex_state = 0},
  [1869] = {.lex_state = 44},
  [1870] = {.lex_state = 52},
  [1871] = {.lex_state = 119},
  [1872] = {.lex_state = 44},
  [1873] = {.lex_state = 44},
  [1874] = {.lex_state = 52},
  [1875] = {.lex_state = 34},
  [1876] = {.lex_state = 34},
  [1877] = {.lex_state = 52},
  [1878] = {.lex_state = 34},
  [1879] = {.lex_state = 52},
  [1880] = {.lex_state = 0},
  [1881] = {.lex_state = 0},
  [1882] = {.lex_state = 0},
  [1883] = {.lex_state = 0},
  [1884] = {.lex_state = 52},
  [1885] = {.lex_state = 0},
  [1886] = {.lex_state = 0},
  [1887] = {.lex_state = 0},
  [1888] = {.lex_state = 0},
  [1889] = {.lex_state = 0},
  [1890] = {.lex_state = 52},
  [1891] = {.lex_state = 34},
  [1892] = {.lex_state = 0},
  [1893] = {.lex_state = 0},
  [1894] = {.lex_state = 52},
  [1895] = {.lex_state = 52},
  [1896] = {.lex_state = 0},
  [1897] = {.lex_state = 34},
  [1898] = {.lex_state = 0},
  [1899] = {.lex_state = 44},
  [1900] = {.lex_state = 44},
  [1901] = {.lex_state = 52},
  [1902] = {.lex_state = 52},
  [1903] = {.lex_state = 44},
  [1904] = {.lex_state = 0},
  [1905] = {.lex_state = 0},
  [1906] = {.lex_state = 0},
  [1907] = {.lex_state = 44},
  [1908] = {.lex_state = 34},
  [1909] = {.lex_state = 0},
  [1910] = {.lex_state = 34},
  [1911] = {.lex_state = 0},
  [1912] = {.lex_state = 44},
  [1913] = {.lex_state = 0},
  [1914] = {.lex_state = 0},
  [1915] = {.lex_state = 0},
  [1916] = {.lex_state = 52},
  [1917] = {.lex_state = 52},
  [1918] = {.lex_state = 52},
  [1919] = {.lex_state = 44},
  [1920] = {.lex_state = 0},
  [1921] = {.lex_state = 0},
  [1922] = {.lex_state = 0},
  [1923] = {.lex_state = 0},
  [1924] = {.lex_state = 0},
  [1925] = {.lex_state = 0},
  [1926] = {.lex_state = 52},
  [1927] = {.lex_state = 34},
  [1928] = {.lex_state = 0},
  [1929] = {.lex_state = 44},
  [1930] = {.lex_state = 52},
  [1931] = {.lex_state = 34},
  [1932] = {.lex_state = 44},
  [1933] = {.lex_state = 0},
  [1934] = {.lex_state = 0},
  [1935] = {.lex_state = 0},
  [1936] = {.lex_state = 52},
  [1937] = {.lex_state = 0},
  [1938] = {.lex_state = 52},
  [1939] = {.lex_state = 44},
  [1940] = {.lex_state = 0},
  [1941] = {.lex_state = 52},
  [1942] = {.lex_state = 34},
  [1943] = {.lex_state = 0},
  [1944] = {.lex_state = 44},
  [1945] = {.lex_state = 0},
  [1946] = {.lex_state = 0},
  [1947] = {.lex_state = 44},
  [1948] = {.lex_state = 52},
  [1949] = {.lex_state = 52},
  [1950] = {.lex_state = 119},
  [1951] = {.lex_state = 0},
  [1952] = {.lex_state = 34},
  [1953] = {.lex_state = 44},
  [1954] = {.lex_state = 52},
  [1955] = {.lex_state = 0},
  [1956] = {.lex_state = 0},
  [1957] = {.lex_state = 0},
  [1958] = {.lex_state = 0},
  [1959] = {.lex_state = 44},
  [1960] = {.lex_state = 0},
  [1961] = {.lex_state = 52},
  [1962] = {.lex_state = 44},
  [1963] = {.lex_state = 0},
  [1964] = {.lex_state = 0},
  [1965] = {.lex_state = 52},
  [1966] = {.lex_state = 44},
  [1967] = {.lex_state = 52},
  [1968] = {.lex_state = 0},
  [1969] = {.lex_state = 52},
  [1970] = {.lex_state = 52},
  [1971] = {.lex_state = 119},
  [1972] = {.lex_state = 0},
  [1973] = {.lex_state = 52},
  [1974] = {.lex_state = 52},
  [1975] = {.lex_state = 34},
  [1976] = {.lex_state = 0},
  [1977] = {.lex_state = 0},
  [1978] = {.lex_state = 44},
  [1979] = {.lex_state = 0},
  [1980] = {.lex_state = 119},
  [1981] = {.lex_state = 44},
  [1982] = {.lex_state = 44},
  [1983] = {.lex_state = 44},
  [1984] = {.lex_state = 44},
  [1985] = {.lex_state = 0},
  [1986] = {.lex_state = 0},
  [1987] = {.lex_state = 44},
  [1988] = {.lex_state = 44},
  [1989] = {.lex_state = 52},
  [1990] = {.lex_state = 119},
  [1991] = {.lex_state = 52},
  [1992] = {.lex_state = 44},
  [1993] = {.lex_state = 44},
  [1994] = {.lex_state = 52},
  [1995] = {.lex_state = 52},
  [1996] = {.lex_state = 0},
  [1997] = {.lex_state = 0},
  [1998] = {.lex_state = 0},
  [1999] = {.lex_state = 119},
  [2000] = {.lex_state = 0},
  [2001] = {.lex_state = 0},
  [2002] = {.lex_state = 52},
  [2003] = {.lex_state = 0},
  [2004] = {.lex_state = 34},
  [2005] = {.lex_state = 34},
  [2006] = {.lex_state = 0},
  [2007] = {.lex_state = 0},
  [2008] = {.lex_state = 44},
  [2009] = {.lex_state = 34},
  [2010] = {.lex_state = 52},
  [2011] = {.lex_state = 44},
  [2012] = {.lex_state = 0},
  [2013] = {.lex_state = 34},
  [2014] = {.lex_state = 52},
  [2015] = {.lex_state = 119},
  [2016] = {.lex_state = 0},
  [2017] = {.lex_state = 0},
  [2018] = {.lex_state = 119},
  [2019] = {.lex_state = 44},
  [2020] = {.lex_state = 52},
};

static const uint16_t ts_parse_table[LARGE_STATE_COUNT][SYMBOL_COUNT] = {
  [0] = {
    [ts_builtin_sym_end] = ACTIONS(1),
    [sym_identifier] = ACTIONS(1),
    [aux_sym_preproc_include_token1] = ACTIONS(1),
    [aux_sym_preproc_def_token1] = ACTIONS(1),
    [anon_sym_LPAREN] = ACTIONS(1),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1),
    [anon_sym_COMMA] = ACTIONS(1),
    [anon_sym_RPAREN] = ACTIONS(1),
    [aux_sym_preproc_if_token1] = ACTIONS(1),
    [aux_sym_preproc_if_token2] = ACTIONS(1),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1),
    [aux_sym_preproc_else_token1] = ACTIONS(1),
    [aux_sym_preproc_elif_token1] = ACTIONS(1),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1),
    [sym_preproc_directive] = ACTIONS(1),
    [anon_sym_LPAREN2] = ACTIONS(1),
    [anon_sym_defined] = ACTIONS(1),
    [anon_sym_BANG] = ACTIONS(1),
    [anon_sym_TILDE] = ACTIONS(1),
    [anon_sym_DASH] = ACTIONS(1),
    [anon_sym_PLUS] = ACTIONS(1),
    [anon_sym_STAR] = ACTIONS(1),
    [anon_sym_SLASH] = ACTIONS(1),
    [anon_sym_PERCENT] = ACTIONS(1),
    [anon_sym_PIPE_PIPE] = ACTIONS(1),
    [anon_sym_AMP_AMP] = ACTIONS(1),
    [anon_sym_PIPE] = ACTIONS(1),
    [anon_sym_CARET] = ACTIONS(1),
    [anon_sym_AMP] = ACTIONS(1),
    [anon_sym_EQ_EQ] = ACTIONS(1),
    [anon_sym_BANG_EQ] = ACTIONS(1),
    [anon_sym_GT] = ACTIONS(1),
    [anon_sym_GT_EQ] = ACTIONS(1),
    [anon_sym_LT_EQ] = ACTIONS(1),
    [anon_sym_LT] = ACTIONS(1),
    [anon_sym_LT_LT] = ACTIONS(1),
    [anon_sym_GT_GT] = ACTIONS(1),
    [anon_sym_SEMI] = ACTIONS(1),
    [anon_sym___extension__] = ACTIONS(1),
    [anon_sym_typedef] = ACTIONS(1),
    [anon_sym_extern] = ACTIONS(1),
    [anon_sym___attribute__] = ACTIONS(1),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1),
    [anon_sym___declspec] = ACTIONS(1),
    [anon_sym___based] = ACTIONS(1),
    [anon_sym___cdecl] = ACTIONS(1),
    [anon_sym___clrcall] = ACTIONS(1),
    [anon_sym___stdcall] = ACTIONS(1),
    [anon_sym___fastcall] = ACTIONS(1),
    [anon_sym___thiscall] = ACTIONS(1),
    [anon_sym___vectorcall] = ACTIONS(1),
    [sym_ms_restrict_modifier] = ACTIONS(1),
    [sym_ms_unsigned_ptr_modifier] = ACTIONS(1),
    [sym_ms_signed_ptr_modifier] = ACTIONS(1),
    [anon_sym__unaligned] = ACTIONS(1),
    [anon_sym___unaligned] = ACTIONS(1),
    [anon_sym_LBRACE] = ACTIONS(1),
    [anon_sym_RBRACE] = ACTIONS(1),
    [anon_sym_signed] = ACTIONS(1),
    [anon_sym_unsigned] = ACTIONS(1),
    [anon_sym_long] = ACTIONS(1),
    [anon_sym_short] = ACTIONS(1),
    [anon_sym_LBRACK] = ACTIONS(1),
    [anon_sym_RBRACK] = ACTIONS(1),
    [anon_sym_EQ] = ACTIONS(1),
    [anon_sym_static] = ACTIONS(1),
    [anon_sym_auto] = ACTIONS(1),
    [anon_sym_register] = ACTIONS(1),
    [anon_sym_inline] = ACTIONS(1),
    [anon_sym___inline] = ACTIONS(1),
    [anon_sym___inline__] = ACTIONS(1),
    [anon_sym___forceinline] = ACTIONS(1),
    [anon_sym_thread_local] = ACTIONS(1),
    [anon_sym___thread] = ACTIONS(1),
    [anon_sym_const] = ACTIONS(1),
    [anon_sym_constexpr] = ACTIONS(1),
    [anon_sym_volatile] = ACTIONS(1),
    [anon_sym_restrict] = ACTIONS(1),
    [anon_sym___restrict__] = ACTIONS(1),
    [anon_sym__Atomic] = ACTIONS(1),
    [anon_sym__Noreturn] = ACTIONS(1),
    [anon_sym_noreturn] = ACTIONS(1),
    [sym_primitive_type] = ACTIONS(1),
    [anon_sym_enum] = ACTIONS(1),
    [anon_sym_COLON] = ACTIONS(1),
    [anon_sym_struct] = ACTIONS(1),
    [anon_sym_union] = ACTIONS(1),
    [anon_sym_if] = ACTIONS(1),
    [anon_sym_else] = ACTIONS(1),
    [anon_sym_switch] = ACTIONS(1),
    [anon_sym_case] = ACTIONS(1),
    [anon_sym_default] = ACTIONS(1),
    [anon_sym_while] = ACTIONS(1),
    [anon_sym_do] = ACTIONS(1),
    [anon_sym_for] = ACTIONS(1),
    [anon_sym_return] = ACTIONS(1),
    [anon_sym_break] = ACTIONS(1),
    [anon_sym_continue] = ACTIONS(1),
    [anon_sym_goto] = ACTIONS(1),
    [anon_sym___try] = ACTIONS(1),
    [anon_sym___except] = ACTIONS(1),
    [anon_sym___finally] = ACTIONS(1),
    [anon_sym___leave] = ACTIONS(1),
    [anon_sym_QMARK] = ACTIONS(1),
    [anon_sym_STAR_EQ] = ACTIONS(1),
    [anon_sym_SLASH_EQ] = ACTIONS(1),
    [anon_sym_PERCENT_EQ] = ACTIONS(1),
    [anon_sym_PLUS_EQ] = ACTIONS(1),
    [anon_sym_DASH_EQ] = ACTIONS(1),
    [anon_sym_LT_LT_EQ] = ACTIONS(1),
    [anon_sym_GT_GT_EQ] = ACTIONS(1),
    [anon_sym_AMP_EQ] = ACTIONS(1),
    [anon_sym_CARET_EQ] = ACTIONS(1),
    [anon_sym_PIPE_EQ] = ACTIONS(1),
    [anon_sym_DASH_DASH] = ACTIONS(1),
    [anon_sym_PLUS_PLUS] = ACTIONS(1),
    [anon_sym_sizeof] = ACTIONS(1),
    [anon_sym___alignof__] = ACTIONS(1),
    [anon_sym___alignof] = ACTIONS(1),
    [anon_sym__alignof] = ACTIONS(1),
    [anon_sym_alignof] = ACTIONS(1),
    [anon_sym__Alignof] = ACTIONS(1),
    [anon_sym_offsetof] = ACTIONS(1),
    [anon_sym__Generic] = ACTIONS(1),
    [anon_sym_asm] = ACTIONS(1),
    [anon_sym___asm__] = ACTIONS(1),
    [anon_sym_DOT] = ACTIONS(1),
    [anon_sym_DASH_GT] = ACTIONS(1),
    [sym_number_literal] = ACTIONS(1),
    [anon_sym_L_SQUOTE] = ACTIONS(1),
    [anon_sym_u_SQUOTE] = ACTIONS(1),
    [anon_sym_U_SQUOTE] = ACTIONS(1),
    [anon_sym_u8_SQUOTE] = ACTIONS(1),
    [anon_sym_SQUOTE] = ACTIONS(1),
    [anon_sym_L_DQUOTE] = ACTIONS(1),
    [anon_sym_u_DQUOTE] = ACTIONS(1),
    [anon_sym_U_DQUOTE] = ACTIONS(1),
    [anon_sym_u8_DQUOTE] = ACTIONS(1),
    [anon_sym_DQUOTE] = ACTIONS(1),
    [sym_true] = ACTIONS(1),
    [sym_false] = ACTIONS(1),
    [anon_sym_NULL] = ACTIONS(1),
    [anon_sym_nullptr] = ACTIONS(1),
    [sym_comment] = ACTIONS(3),
  },
  [1] = {
    [sym_translation_unit] = STATE(1836),
    [sym_preproc_include] = STATE(43),
    [sym_preproc_def] = STATE(43),
    [sym_preproc_function_def] = STATE(43),
    [sym_preproc_call] = STATE(43),
    [sym_preproc_if] = STATE(43),
    [sym_preproc_ifdef] = STATE(43),
    [sym_function_definition] = STATE(43),
    [sym__old_style_function_definition] = STATE(415),
    [sym_declaration] = STATE(43),
    [sym_type_definition] = STATE(43),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1129),
    [sym_linkage_specification] = STATE(43),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(747),
    [sym_compound_statement] = STATE(43),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(866),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(43),
    [sym_labeled_statement] = STATE(43),
    [sym__top_level_expression_statement] = STATE(43),
    [sym_if_statement] = STATE(43),
    [sym_switch_statement] = STATE(43),
    [sym_case_statement] = STATE(43),
    [sym_while_statement] = STATE(43),
    [sym_do_statement] = STATE(43),
    [sym_for_statement] = STATE(43),
    [sym_return_statement] = STATE(43),
    [sym_break_statement] = STATE(43),
    [sym_continue_statement] = STATE(43),
    [sym_goto_statement] = STATE(43),
    [sym__expression] = STATE(1117),
    [sym__expression_not_binary] = STATE(1115),
    [sym__string] = STATE(1115),
    [sym_conditional_expression] = STATE(1115),
    [sym_assignment_expression] = STATE(1115),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(1115),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(1115),
    [sym_cast_expression] = STATE(1115),
    [sym_sizeof_expression] = STATE(1115),
    [sym_alignof_expression] = STATE(1115),
    [sym_offsetof_expression] = STATE(1115),
    [sym_generic_expression] = STATE(1115),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(1115),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(1115),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(1115),
    [sym_concatenated_string] = STATE(1115),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(1115),
    [sym__empty_declaration] = STATE(43),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_translation_unit_repeat1] = STATE(43),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [ts_builtin_sym_end] = ACTIONS(5),
    [sym_identifier] = ACTIONS(7),
    [aux_sym_preproc_include_token1] = ACTIONS(9),
    [aux_sym_preproc_def_token1] = ACTIONS(11),
    [aux_sym_preproc_if_token1] = ACTIONS(13),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(15),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(15),
    [sym_preproc_directive] = ACTIONS(17),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(31),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(61),
    [anon_sym_default] = ACTIONS(63),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(91),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(97),
    [sym_false] = ACTIONS(97),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [2] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(1787),
    [sym_preproc_elif] = STATE(1787),
    [sym_preproc_elifdef] = STATE(1787),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(109),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [3] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(1789),
    [sym_preproc_elif] = STATE(1789),
    [sym_preproc_elifdef] = STATE(1789),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(161),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [4] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(1959),
    [sym_preproc_elif] = STATE(1959),
    [sym_preproc_elifdef] = STATE(1959),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(163),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [5] = {
    [sym_preproc_include] = STATE(11),
    [sym_preproc_def] = STATE(11),
    [sym_preproc_function_def] = STATE(11),
    [sym_preproc_call] = STATE(11),
    [sym_preproc_if] = STATE(11),
    [sym_preproc_ifdef] = STATE(11),
    [sym_preproc_else] = STATE(1903),
    [sym_preproc_elif] = STATE(1903),
    [sym_preproc_elifdef] = STATE(1903),
    [sym_function_definition] = STATE(11),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(11),
    [sym_type_definition] = STATE(11),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(11),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(11),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(11),
    [sym_labeled_statement] = STATE(11),
    [sym_expression_statement] = STATE(11),
    [sym_if_statement] = STATE(11),
    [sym_switch_statement] = STATE(11),
    [sym_case_statement] = STATE(11),
    [sym_while_statement] = STATE(11),
    [sym_do_statement] = STATE(11),
    [sym_for_statement] = STATE(11),
    [sym_return_statement] = STATE(11),
    [sym_break_statement] = STATE(11),
    [sym_continue_statement] = STATE(11),
    [sym_goto_statement] = STATE(11),
    [sym_seh_try_statement] = STATE(11),
    [sym_seh_leave_statement] = STATE(11),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(11),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(11),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(165),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [6] = {
    [sym_preproc_include] = STATE(13),
    [sym_preproc_def] = STATE(13),
    [sym_preproc_function_def] = STATE(13),
    [sym_preproc_call] = STATE(13),
    [sym_preproc_if] = STATE(13),
    [sym_preproc_ifdef] = STATE(13),
    [sym_preproc_else] = STATE(1953),
    [sym_preproc_elif] = STATE(1953),
    [sym_preproc_elifdef] = STATE(1953),
    [sym_function_definition] = STATE(13),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(13),
    [sym_type_definition] = STATE(13),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(13),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(13),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(13),
    [sym_labeled_statement] = STATE(13),
    [sym_expression_statement] = STATE(13),
    [sym_if_statement] = STATE(13),
    [sym_switch_statement] = STATE(13),
    [sym_case_statement] = STATE(13),
    [sym_while_statement] = STATE(13),
    [sym_do_statement] = STATE(13),
    [sym_for_statement] = STATE(13),
    [sym_return_statement] = STATE(13),
    [sym_break_statement] = STATE(13),
    [sym_continue_statement] = STATE(13),
    [sym_goto_statement] = STATE(13),
    [sym_seh_try_statement] = STATE(13),
    [sym_seh_leave_statement] = STATE(13),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(13),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(13),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(167),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [7] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(1872),
    [sym_preproc_elif] = STATE(1872),
    [sym_preproc_elifdef] = STATE(1872),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(169),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [8] = {
    [sym_preproc_include] = STATE(16),
    [sym_preproc_def] = STATE(16),
    [sym_preproc_function_def] = STATE(16),
    [sym_preproc_call] = STATE(16),
    [sym_preproc_if] = STATE(16),
    [sym_preproc_ifdef] = STATE(16),
    [sym_preproc_else] = STATE(1791),
    [sym_preproc_elif] = STATE(1791),
    [sym_preproc_elifdef] = STATE(1791),
    [sym_function_definition] = STATE(16),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(16),
    [sym_type_definition] = STATE(16),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(16),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(16),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(16),
    [sym_labeled_statement] = STATE(16),
    [sym_expression_statement] = STATE(16),
    [sym_if_statement] = STATE(16),
    [sym_switch_statement] = STATE(16),
    [sym_case_statement] = STATE(16),
    [sym_while_statement] = STATE(16),
    [sym_do_statement] = STATE(16),
    [sym_for_statement] = STATE(16),
    [sym_return_statement] = STATE(16),
    [sym_break_statement] = STATE(16),
    [sym_continue_statement] = STATE(16),
    [sym_goto_statement] = STATE(16),
    [sym_seh_try_statement] = STATE(16),
    [sym_seh_leave_statement] = STATE(16),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(16),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(16),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(171),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [9] = {
    [sym_preproc_include] = STATE(3),
    [sym_preproc_def] = STATE(3),
    [sym_preproc_function_def] = STATE(3),
    [sym_preproc_call] = STATE(3),
    [sym_preproc_if] = STATE(3),
    [sym_preproc_ifdef] = STATE(3),
    [sym_preproc_else] = STATE(1794),
    [sym_preproc_elif] = STATE(1794),
    [sym_preproc_elifdef] = STATE(1794),
    [sym_function_definition] = STATE(3),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(3),
    [sym_type_definition] = STATE(3),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(3),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(3),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(3),
    [sym_labeled_statement] = STATE(3),
    [sym_expression_statement] = STATE(3),
    [sym_if_statement] = STATE(3),
    [sym_switch_statement] = STATE(3),
    [sym_case_statement] = STATE(3),
    [sym_while_statement] = STATE(3),
    [sym_do_statement] = STATE(3),
    [sym_for_statement] = STATE(3),
    [sym_return_statement] = STATE(3),
    [sym_break_statement] = STATE(3),
    [sym_continue_statement] = STATE(3),
    [sym_goto_statement] = STATE(3),
    [sym_seh_try_statement] = STATE(3),
    [sym_seh_leave_statement] = STATE(3),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(3),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(3),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(173),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [10] = {
    [sym_preproc_include] = STATE(2),
    [sym_preproc_def] = STATE(2),
    [sym_preproc_function_def] = STATE(2),
    [sym_preproc_call] = STATE(2),
    [sym_preproc_if] = STATE(2),
    [sym_preproc_ifdef] = STATE(2),
    [sym_preproc_else] = STATE(1784),
    [sym_preproc_elif] = STATE(1784),
    [sym_preproc_elifdef] = STATE(1784),
    [sym_function_definition] = STATE(2),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(2),
    [sym_type_definition] = STATE(2),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(2),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(2),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(2),
    [sym_labeled_statement] = STATE(2),
    [sym_expression_statement] = STATE(2),
    [sym_if_statement] = STATE(2),
    [sym_switch_statement] = STATE(2),
    [sym_case_statement] = STATE(2),
    [sym_while_statement] = STATE(2),
    [sym_do_statement] = STATE(2),
    [sym_for_statement] = STATE(2),
    [sym_return_statement] = STATE(2),
    [sym_break_statement] = STATE(2),
    [sym_continue_statement] = STATE(2),
    [sym_goto_statement] = STATE(2),
    [sym_seh_try_statement] = STATE(2),
    [sym_seh_leave_statement] = STATE(2),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(2),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(2),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(175),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [11] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(1912),
    [sym_preproc_elif] = STATE(1912),
    [sym_preproc_elifdef] = STATE(1912),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(177),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [12] = {
    [sym_preproc_include] = STATE(14),
    [sym_preproc_def] = STATE(14),
    [sym_preproc_function_def] = STATE(14),
    [sym_preproc_call] = STATE(14),
    [sym_preproc_if] = STATE(14),
    [sym_preproc_ifdef] = STATE(14),
    [sym_preproc_else] = STATE(2011),
    [sym_preproc_elif] = STATE(2011),
    [sym_preproc_elifdef] = STATE(2011),
    [sym_function_definition] = STATE(14),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(14),
    [sym_type_definition] = STATE(14),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(14),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(14),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(14),
    [sym_labeled_statement] = STATE(14),
    [sym_expression_statement] = STATE(14),
    [sym_if_statement] = STATE(14),
    [sym_switch_statement] = STATE(14),
    [sym_case_statement] = STATE(14),
    [sym_while_statement] = STATE(14),
    [sym_do_statement] = STATE(14),
    [sym_for_statement] = STATE(14),
    [sym_return_statement] = STATE(14),
    [sym_break_statement] = STATE(14),
    [sym_continue_statement] = STATE(14),
    [sym_goto_statement] = STATE(14),
    [sym_seh_try_statement] = STATE(14),
    [sym_seh_leave_statement] = STATE(14),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(14),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(14),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(179),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [13] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(2008),
    [sym_preproc_elif] = STATE(2008),
    [sym_preproc_elifdef] = STATE(2008),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(181),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [14] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(1988),
    [sym_preproc_elif] = STATE(1988),
    [sym_preproc_elifdef] = STATE(1988),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(183),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [15] = {
    [sym_preproc_include] = STATE(19),
    [sym_preproc_def] = STATE(19),
    [sym_preproc_function_def] = STATE(19),
    [sym_preproc_call] = STATE(19),
    [sym_preproc_if] = STATE(19),
    [sym_preproc_ifdef] = STATE(19),
    [sym_preproc_else] = STATE(1856),
    [sym_preproc_elif] = STATE(1856),
    [sym_preproc_elifdef] = STATE(1856),
    [sym_function_definition] = STATE(19),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(19),
    [sym_type_definition] = STATE(19),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(19),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(19),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(19),
    [sym_labeled_statement] = STATE(19),
    [sym_expression_statement] = STATE(19),
    [sym_if_statement] = STATE(19),
    [sym_switch_statement] = STATE(19),
    [sym_case_statement] = STATE(19),
    [sym_while_statement] = STATE(19),
    [sym_do_statement] = STATE(19),
    [sym_for_statement] = STATE(19),
    [sym_return_statement] = STATE(19),
    [sym_break_statement] = STATE(19),
    [sym_continue_statement] = STATE(19),
    [sym_goto_statement] = STATE(19),
    [sym_seh_try_statement] = STATE(19),
    [sym_seh_leave_statement] = STATE(19),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(19),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(19),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(185),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [16] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(1854),
    [sym_preproc_elif] = STATE(1854),
    [sym_preproc_elifdef] = STATE(1854),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(187),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [17] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(1993),
    [sym_preproc_elif] = STATE(1993),
    [sym_preproc_elifdef] = STATE(1993),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(189),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [18] = {
    [sym_preproc_include] = STATE(4),
    [sym_preproc_def] = STATE(4),
    [sym_preproc_function_def] = STATE(4),
    [sym_preproc_call] = STATE(4),
    [sym_preproc_if] = STATE(4),
    [sym_preproc_ifdef] = STATE(4),
    [sym_preproc_else] = STATE(1869),
    [sym_preproc_elif] = STATE(1869),
    [sym_preproc_elifdef] = STATE(1869),
    [sym_function_definition] = STATE(4),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(4),
    [sym_type_definition] = STATE(4),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(4),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(4),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(4),
    [sym_labeled_statement] = STATE(4),
    [sym_expression_statement] = STATE(4),
    [sym_if_statement] = STATE(4),
    [sym_switch_statement] = STATE(4),
    [sym_case_statement] = STATE(4),
    [sym_while_statement] = STATE(4),
    [sym_do_statement] = STATE(4),
    [sym_for_statement] = STATE(4),
    [sym_return_statement] = STATE(4),
    [sym_break_statement] = STATE(4),
    [sym_continue_statement] = STATE(4),
    [sym_goto_statement] = STATE(4),
    [sym_seh_try_statement] = STATE(4),
    [sym_seh_leave_statement] = STATE(4),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(4),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(4),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(191),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [19] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_preproc_else] = STATE(1841),
    [sym_preproc_elif] = STATE(1841),
    [sym_preproc_elifdef] = STATE(1841),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(193),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [20] = {
    [sym_preproc_include] = STATE(17),
    [sym_preproc_def] = STATE(17),
    [sym_preproc_function_def] = STATE(17),
    [sym_preproc_call] = STATE(17),
    [sym_preproc_if] = STATE(17),
    [sym_preproc_ifdef] = STATE(17),
    [sym_preproc_else] = STATE(2019),
    [sym_preproc_elif] = STATE(2019),
    [sym_preproc_elifdef] = STATE(2019),
    [sym_function_definition] = STATE(17),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(17),
    [sym_type_definition] = STATE(17),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(17),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(17),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(17),
    [sym_labeled_statement] = STATE(17),
    [sym_expression_statement] = STATE(17),
    [sym_if_statement] = STATE(17),
    [sym_switch_statement] = STATE(17),
    [sym_case_statement] = STATE(17),
    [sym_while_statement] = STATE(17),
    [sym_do_statement] = STATE(17),
    [sym_for_statement] = STATE(17),
    [sym_return_statement] = STATE(17),
    [sym_break_statement] = STATE(17),
    [sym_continue_statement] = STATE(17),
    [sym_goto_statement] = STATE(17),
    [sym_seh_try_statement] = STATE(17),
    [sym_seh_leave_statement] = STATE(17),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(17),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(17),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(195),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [21] = {
    [sym_preproc_include] = STATE(7),
    [sym_preproc_def] = STATE(7),
    [sym_preproc_function_def] = STATE(7),
    [sym_preproc_call] = STATE(7),
    [sym_preproc_if] = STATE(7),
    [sym_preproc_ifdef] = STATE(7),
    [sym_preproc_else] = STATE(1992),
    [sym_preproc_elif] = STATE(1992),
    [sym_preproc_elifdef] = STATE(1992),
    [sym_function_definition] = STATE(7),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(7),
    [sym_type_definition] = STATE(7),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(7),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(7),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(7),
    [sym_labeled_statement] = STATE(7),
    [sym_expression_statement] = STATE(7),
    [sym_if_statement] = STATE(7),
    [sym_switch_statement] = STATE(7),
    [sym_case_statement] = STATE(7),
    [sym_while_statement] = STATE(7),
    [sym_do_statement] = STATE(7),
    [sym_for_statement] = STATE(7),
    [sym_return_statement] = STATE(7),
    [sym_break_statement] = STATE(7),
    [sym_continue_statement] = STATE(7),
    [sym_goto_statement] = STATE(7),
    [sym_seh_try_statement] = STATE(7),
    [sym_seh_leave_statement] = STATE(7),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(7),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(7),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(101),
    [aux_sym_preproc_include_token1] = ACTIONS(103),
    [aux_sym_preproc_def_token1] = ACTIONS(105),
    [aux_sym_preproc_if_token1] = ACTIONS(107),
    [aux_sym_preproc_if_token2] = ACTIONS(197),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(111),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(111),
    [aux_sym_preproc_else_token1] = ACTIONS(113),
    [aux_sym_preproc_elif_token1] = ACTIONS(115),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(117),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(117),
    [sym_preproc_directive] = ACTIONS(119),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(127),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [22] = {
    [sym_preproc_include] = STATE(22),
    [sym_preproc_def] = STATE(22),
    [sym_preproc_function_def] = STATE(22),
    [sym_preproc_call] = STATE(22),
    [sym_preproc_if] = STATE(22),
    [sym_preproc_ifdef] = STATE(22),
    [sym_function_definition] = STATE(22),
    [sym__old_style_function_definition] = STATE(150),
    [sym_declaration] = STATE(22),
    [sym_type_definition] = STATE(22),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1132),
    [sym_linkage_specification] = STATE(22),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(740),
    [sym_compound_statement] = STATE(22),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(865),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(22),
    [sym_labeled_statement] = STATE(22),
    [sym_expression_statement] = STATE(22),
    [sym_if_statement] = STATE(22),
    [sym_switch_statement] = STATE(22),
    [sym_case_statement] = STATE(22),
    [sym_while_statement] = STATE(22),
    [sym_do_statement] = STATE(22),
    [sym_for_statement] = STATE(22),
    [sym_return_statement] = STATE(22),
    [sym_break_statement] = STATE(22),
    [sym_continue_statement] = STATE(22),
    [sym_goto_statement] = STATE(22),
    [sym_seh_try_statement] = STATE(22),
    [sym_seh_leave_statement] = STATE(22),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(22),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(22),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(199),
    [aux_sym_preproc_include_token1] = ACTIONS(202),
    [aux_sym_preproc_def_token1] = ACTIONS(205),
    [aux_sym_preproc_if_token1] = ACTIONS(208),
    [aux_sym_preproc_if_token2] = ACTIONS(211),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(213),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(213),
    [aux_sym_preproc_else_token1] = ACTIONS(211),
    [aux_sym_preproc_elif_token1] = ACTIONS(211),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(211),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(211),
    [sym_preproc_directive] = ACTIONS(216),
    [anon_sym_LPAREN2] = ACTIONS(219),
    [anon_sym_BANG] = ACTIONS(222),
    [anon_sym_TILDE] = ACTIONS(222),
    [anon_sym_DASH] = ACTIONS(225),
    [anon_sym_PLUS] = ACTIONS(225),
    [anon_sym_STAR] = ACTIONS(228),
    [anon_sym_AMP] = ACTIONS(228),
    [anon_sym_SEMI] = ACTIONS(231),
    [anon_sym___extension__] = ACTIONS(234),
    [anon_sym_typedef] = ACTIONS(237),
    [anon_sym_extern] = ACTIONS(240),
    [anon_sym___attribute__] = ACTIONS(243),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(246),
    [anon_sym___declspec] = ACTIONS(249),
    [anon_sym___cdecl] = ACTIONS(252),
    [anon_sym___clrcall] = ACTIONS(252),
    [anon_sym___stdcall] = ACTIONS(252),
    [anon_sym___fastcall] = ACTIONS(252),
    [anon_sym___thiscall] = ACTIONS(252),
    [anon_sym___vectorcall] = ACTIONS(252),
    [anon_sym_LBRACE] = ACTIONS(255),
    [anon_sym_signed] = ACTIONS(258),
    [anon_sym_unsigned] = ACTIONS(258),
    [anon_sym_long] = ACTIONS(258),
    [anon_sym_short] = ACTIONS(258),
    [anon_sym_static] = ACTIONS(261),
    [anon_sym_auto] = ACTIONS(261),
    [anon_sym_register] = ACTIONS(261),
    [anon_sym_inline] = ACTIONS(261),
    [anon_sym___inline] = ACTIONS(261),
    [anon_sym___inline__] = ACTIONS(261),
    [anon_sym___forceinline] = ACTIONS(261),
    [anon_sym_thread_local] = ACTIONS(261),
    [anon_sym___thread] = ACTIONS(261),
    [anon_sym_const] = ACTIONS(264),
    [anon_sym_constexpr] = ACTIONS(264),
    [anon_sym_volatile] = ACTIONS(264),
    [anon_sym_restrict] = ACTIONS(264),
    [anon_sym___restrict__] = ACTIONS(264),
    [anon_sym__Atomic] = ACTIONS(264),
    [anon_sym__Noreturn] = ACTIONS(264),
    [anon_sym_noreturn] = ACTIONS(264),
    [sym_primitive_type] = ACTIONS(267),
    [anon_sym_enum] = ACTIONS(270),
    [anon_sym_struct] = ACTIONS(273),
    [anon_sym_union] = ACTIONS(276),
    [anon_sym_if] = ACTIONS(279),
    [anon_sym_switch] = ACTIONS(282),
    [anon_sym_case] = ACTIONS(285),
    [anon_sym_default] = ACTIONS(288),
    [anon_sym_while] = ACTIONS(291),
    [anon_sym_do] = ACTIONS(294),
    [anon_sym_for] = ACTIONS(297),
    [anon_sym_return] = ACTIONS(300),
    [anon_sym_break] = ACTIONS(303),
    [anon_sym_continue] = ACTIONS(306),
    [anon_sym_goto] = ACTIONS(309),
    [anon_sym___try] = ACTIONS(312),
    [anon_sym___leave] = ACTIONS(315),
    [anon_sym_DASH_DASH] = ACTIONS(318),
    [anon_sym_PLUS_PLUS] = ACTIONS(318),
    [anon_sym_sizeof] = ACTIONS(321),
    [anon_sym___alignof__] = ACTIONS(324),
    [anon_sym___alignof] = ACTIONS(324),
    [anon_sym__alignof] = ACTIONS(324),
    [anon_sym_alignof] = ACTIONS(324),
    [anon_sym__Alignof] = ACTIONS(324),
    [anon_sym_offsetof] = ACTIONS(327),
    [anon_sym__Generic] = ACTIONS(330),
    [anon_sym_asm] = ACTIONS(333),
    [anon_sym___asm__] = ACTIONS(333),
    [sym_number_literal] = ACTIONS(336),
    [anon_sym_L_SQUOTE] = ACTIONS(339),
    [anon_sym_u_SQUOTE] = ACTIONS(339),
    [anon_sym_U_SQUOTE] = ACTIONS(339),
    [anon_sym_u8_SQUOTE] = ACTIONS(339),
    [anon_sym_SQUOTE] = ACTIONS(339),
    [anon_sym_L_DQUOTE] = ACTIONS(342),
    [anon_sym_u_DQUOTE] = ACTIONS(342),
    [anon_sym_U_DQUOTE] = ACTIONS(342),
    [anon_sym_u8_DQUOTE] = ACTIONS(342),
    [anon_sym_DQUOTE] = ACTIONS(342),
    [sym_true] = ACTIONS(345),
    [sym_false] = ACTIONS(345),
    [anon_sym_NULL] = ACTIONS(348),
    [anon_sym_nullptr] = ACTIONS(348),
    [sym_comment] = ACTIONS(3),
  },
  [23] = {
    [sym_preproc_include] = STATE(28),
    [sym_preproc_def] = STATE(28),
    [sym_preproc_function_def] = STATE(28),
    [sym_preproc_call] = STATE(28),
    [sym_preproc_if] = STATE(28),
    [sym_preproc_ifdef] = STATE(28),
    [sym_function_definition] = STATE(28),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(28),
    [sym_type_definition] = STATE(28),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(28),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(28),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(28),
    [sym_labeled_statement] = STATE(28),
    [sym_expression_statement] = STATE(28),
    [sym_if_statement] = STATE(28),
    [sym_switch_statement] = STATE(28),
    [sym_case_statement] = STATE(28),
    [sym_while_statement] = STATE(28),
    [sym_do_statement] = STATE(28),
    [sym_for_statement] = STATE(28),
    [sym_return_statement] = STATE(28),
    [sym_break_statement] = STATE(28),
    [sym_continue_statement] = STATE(28),
    [sym_goto_statement] = STATE(28),
    [sym_seh_try_statement] = STATE(28),
    [sym_seh_leave_statement] = STATE(28),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(28),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(28),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(373),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [24] = {
    [sym_preproc_include] = STATE(25),
    [sym_preproc_def] = STATE(25),
    [sym_preproc_function_def] = STATE(25),
    [sym_preproc_call] = STATE(25),
    [sym_preproc_if] = STATE(25),
    [sym_preproc_ifdef] = STATE(25),
    [sym_function_definition] = STATE(25),
    [sym__old_style_function_definition] = STATE(337),
    [sym_declaration] = STATE(25),
    [sym_type_definition] = STATE(25),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1135),
    [sym_linkage_specification] = STATE(25),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(744),
    [sym_compound_statement] = STATE(25),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(863),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(25),
    [sym_labeled_statement] = STATE(25),
    [sym_expression_statement] = STATE(25),
    [sym_if_statement] = STATE(25),
    [sym_switch_statement] = STATE(25),
    [sym_case_statement] = STATE(25),
    [sym_while_statement] = STATE(25),
    [sym_do_statement] = STATE(25),
    [sym_for_statement] = STATE(25),
    [sym_return_statement] = STATE(25),
    [sym_break_statement] = STATE(25),
    [sym_continue_statement] = STATE(25),
    [sym_goto_statement] = STATE(25),
    [sym_seh_try_statement] = STATE(25),
    [sym_seh_leave_statement] = STATE(25),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(25),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(25),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(401),
    [aux_sym_preproc_include_token1] = ACTIONS(403),
    [aux_sym_preproc_def_token1] = ACTIONS(405),
    [aux_sym_preproc_if_token1] = ACTIONS(407),
    [aux_sym_preproc_if_token2] = ACTIONS(409),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(411),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(411),
    [sym_preproc_directive] = ACTIONS(413),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym___extension__] = ACTIONS(417),
    [anon_sym_typedef] = ACTIONS(419),
    [anon_sym_extern] = ACTIONS(421),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(429),
    [anon_sym_default] = ACTIONS(431),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [25] = {
    [sym_preproc_include] = STATE(25),
    [sym_preproc_def] = STATE(25),
    [sym_preproc_function_def] = STATE(25),
    [sym_preproc_call] = STATE(25),
    [sym_preproc_if] = STATE(25),
    [sym_preproc_ifdef] = STATE(25),
    [sym_function_definition] = STATE(25),
    [sym__old_style_function_definition] = STATE(337),
    [sym_declaration] = STATE(25),
    [sym_type_definition] = STATE(25),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1135),
    [sym_linkage_specification] = STATE(25),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(744),
    [sym_compound_statement] = STATE(25),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(863),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(25),
    [sym_labeled_statement] = STATE(25),
    [sym_expression_statement] = STATE(25),
    [sym_if_statement] = STATE(25),
    [sym_switch_statement] = STATE(25),
    [sym_case_statement] = STATE(25),
    [sym_while_statement] = STATE(25),
    [sym_do_statement] = STATE(25),
    [sym_for_statement] = STATE(25),
    [sym_return_statement] = STATE(25),
    [sym_break_statement] = STATE(25),
    [sym_continue_statement] = STATE(25),
    [sym_goto_statement] = STATE(25),
    [sym_seh_try_statement] = STATE(25),
    [sym_seh_leave_statement] = STATE(25),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(25),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(25),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(451),
    [aux_sym_preproc_include_token1] = ACTIONS(454),
    [aux_sym_preproc_def_token1] = ACTIONS(457),
    [aux_sym_preproc_if_token1] = ACTIONS(460),
    [aux_sym_preproc_if_token2] = ACTIONS(211),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(463),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(463),
    [sym_preproc_directive] = ACTIONS(466),
    [anon_sym_LPAREN2] = ACTIONS(219),
    [anon_sym_BANG] = ACTIONS(222),
    [anon_sym_TILDE] = ACTIONS(222),
    [anon_sym_DASH] = ACTIONS(225),
    [anon_sym_PLUS] = ACTIONS(225),
    [anon_sym_STAR] = ACTIONS(228),
    [anon_sym_AMP] = ACTIONS(228),
    [anon_sym_SEMI] = ACTIONS(469),
    [anon_sym___extension__] = ACTIONS(472),
    [anon_sym_typedef] = ACTIONS(475),
    [anon_sym_extern] = ACTIONS(478),
    [anon_sym___attribute__] = ACTIONS(243),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(246),
    [anon_sym___declspec] = ACTIONS(249),
    [anon_sym___cdecl] = ACTIONS(252),
    [anon_sym___clrcall] = ACTIONS(252),
    [anon_sym___stdcall] = ACTIONS(252),
    [anon_sym___fastcall] = ACTIONS(252),
    [anon_sym___thiscall] = ACTIONS(252),
    [anon_sym___vectorcall] = ACTIONS(252),
    [anon_sym_LBRACE] = ACTIONS(481),
    [anon_sym_signed] = ACTIONS(258),
    [anon_sym_unsigned] = ACTIONS(258),
    [anon_sym_long] = ACTIONS(258),
    [anon_sym_short] = ACTIONS(258),
    [anon_sym_static] = ACTIONS(261),
    [anon_sym_auto] = ACTIONS(261),
    [anon_sym_register] = ACTIONS(261),
    [anon_sym_inline] = ACTIONS(261),
    [anon_sym___inline] = ACTIONS(261),
    [anon_sym___inline__] = ACTIONS(261),
    [anon_sym___forceinline] = ACTIONS(261),
    [anon_sym_thread_local] = ACTIONS(261),
    [anon_sym___thread] = ACTIONS(261),
    [anon_sym_const] = ACTIONS(264),
    [anon_sym_constexpr] = ACTIONS(264),
    [anon_sym_volatile] = ACTIONS(264),
    [anon_sym_restrict] = ACTIONS(264),
    [anon_sym___restrict__] = ACTIONS(264),
    [anon_sym__Atomic] = ACTIONS(264),
    [anon_sym__Noreturn] = ACTIONS(264),
    [anon_sym_noreturn] = ACTIONS(264),
    [sym_primitive_type] = ACTIONS(267),
    [anon_sym_enum] = ACTIONS(270),
    [anon_sym_struct] = ACTIONS(273),
    [anon_sym_union] = ACTIONS(276),
    [anon_sym_if] = ACTIONS(484),
    [anon_sym_switch] = ACTIONS(487),
    [anon_sym_case] = ACTIONS(490),
    [anon_sym_default] = ACTIONS(493),
    [anon_sym_while] = ACTIONS(496),
    [anon_sym_do] = ACTIONS(499),
    [anon_sym_for] = ACTIONS(502),
    [anon_sym_return] = ACTIONS(505),
    [anon_sym_break] = ACTIONS(508),
    [anon_sym_continue] = ACTIONS(511),
    [anon_sym_goto] = ACTIONS(514),
    [anon_sym___try] = ACTIONS(517),
    [anon_sym___leave] = ACTIONS(520),
    [anon_sym_DASH_DASH] = ACTIONS(318),
    [anon_sym_PLUS_PLUS] = ACTIONS(318),
    [anon_sym_sizeof] = ACTIONS(321),
    [anon_sym___alignof__] = ACTIONS(324),
    [anon_sym___alignof] = ACTIONS(324),
    [anon_sym__alignof] = ACTIONS(324),
    [anon_sym_alignof] = ACTIONS(324),
    [anon_sym__Alignof] = ACTIONS(324),
    [anon_sym_offsetof] = ACTIONS(327),
    [anon_sym__Generic] = ACTIONS(330),
    [anon_sym_asm] = ACTIONS(333),
    [anon_sym___asm__] = ACTIONS(333),
    [sym_number_literal] = ACTIONS(336),
    [anon_sym_L_SQUOTE] = ACTIONS(339),
    [anon_sym_u_SQUOTE] = ACTIONS(339),
    [anon_sym_U_SQUOTE] = ACTIONS(339),
    [anon_sym_u8_SQUOTE] = ACTIONS(339),
    [anon_sym_SQUOTE] = ACTIONS(339),
    [anon_sym_L_DQUOTE] = ACTIONS(342),
    [anon_sym_u_DQUOTE] = ACTIONS(342),
    [anon_sym_U_DQUOTE] = ACTIONS(342),
    [anon_sym_u8_DQUOTE] = ACTIONS(342),
    [anon_sym_DQUOTE] = ACTIONS(342),
    [sym_true] = ACTIONS(345),
    [sym_false] = ACTIONS(345),
    [anon_sym_NULL] = ACTIONS(348),
    [anon_sym_nullptr] = ACTIONS(348),
    [sym_comment] = ACTIONS(3),
  },
  [26] = {
    [sym_preproc_include] = STATE(34),
    [sym_preproc_def] = STATE(34),
    [sym_preproc_function_def] = STATE(34),
    [sym_preproc_call] = STATE(34),
    [sym_preproc_if] = STATE(34),
    [sym_preproc_ifdef] = STATE(34),
    [sym_function_definition] = STATE(34),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(34),
    [sym_type_definition] = STATE(34),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(34),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(34),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(34),
    [sym_labeled_statement] = STATE(34),
    [sym_expression_statement] = STATE(34),
    [sym_if_statement] = STATE(34),
    [sym_switch_statement] = STATE(34),
    [sym_case_statement] = STATE(34),
    [sym_while_statement] = STATE(34),
    [sym_do_statement] = STATE(34),
    [sym_for_statement] = STATE(34),
    [sym_return_statement] = STATE(34),
    [sym_break_statement] = STATE(34),
    [sym_continue_statement] = STATE(34),
    [sym_goto_statement] = STATE(34),
    [sym_seh_try_statement] = STATE(34),
    [sym_seh_leave_statement] = STATE(34),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(34),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(34),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(523),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [27] = {
    [sym_preproc_include] = STATE(24),
    [sym_preproc_def] = STATE(24),
    [sym_preproc_function_def] = STATE(24),
    [sym_preproc_call] = STATE(24),
    [sym_preproc_if] = STATE(24),
    [sym_preproc_ifdef] = STATE(24),
    [sym_function_definition] = STATE(24),
    [sym__old_style_function_definition] = STATE(337),
    [sym_declaration] = STATE(24),
    [sym_type_definition] = STATE(24),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1135),
    [sym_linkage_specification] = STATE(24),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(744),
    [sym_compound_statement] = STATE(24),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(863),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(24),
    [sym_labeled_statement] = STATE(24),
    [sym_expression_statement] = STATE(24),
    [sym_if_statement] = STATE(24),
    [sym_switch_statement] = STATE(24),
    [sym_case_statement] = STATE(24),
    [sym_while_statement] = STATE(24),
    [sym_do_statement] = STATE(24),
    [sym_for_statement] = STATE(24),
    [sym_return_statement] = STATE(24),
    [sym_break_statement] = STATE(24),
    [sym_continue_statement] = STATE(24),
    [sym_goto_statement] = STATE(24),
    [sym_seh_try_statement] = STATE(24),
    [sym_seh_leave_statement] = STATE(24),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(24),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(24),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(401),
    [aux_sym_preproc_include_token1] = ACTIONS(403),
    [aux_sym_preproc_def_token1] = ACTIONS(405),
    [aux_sym_preproc_if_token1] = ACTIONS(407),
    [aux_sym_preproc_if_token2] = ACTIONS(525),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(411),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(411),
    [sym_preproc_directive] = ACTIONS(413),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym___extension__] = ACTIONS(417),
    [anon_sym_typedef] = ACTIONS(419),
    [anon_sym_extern] = ACTIONS(421),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(429),
    [anon_sym_default] = ACTIONS(431),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [28] = {
    [sym_preproc_include] = STATE(39),
    [sym_preproc_def] = STATE(39),
    [sym_preproc_function_def] = STATE(39),
    [sym_preproc_call] = STATE(39),
    [sym_preproc_if] = STATE(39),
    [sym_preproc_ifdef] = STATE(39),
    [sym_function_definition] = STATE(39),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(39),
    [sym_type_definition] = STATE(39),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(39),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(39),
    [sym_labeled_statement] = STATE(39),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_case_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_do_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_goto_statement] = STATE(39),
    [sym_seh_try_statement] = STATE(39),
    [sym_seh_leave_statement] = STATE(39),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(39),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(39),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(527),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [29] = {
    [sym_preproc_include] = STATE(39),
    [sym_preproc_def] = STATE(39),
    [sym_preproc_function_def] = STATE(39),
    [sym_preproc_call] = STATE(39),
    [sym_preproc_if] = STATE(39),
    [sym_preproc_ifdef] = STATE(39),
    [sym_function_definition] = STATE(39),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(39),
    [sym_type_definition] = STATE(39),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(39),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(39),
    [sym_labeled_statement] = STATE(39),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_case_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_do_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_goto_statement] = STATE(39),
    [sym_seh_try_statement] = STATE(39),
    [sym_seh_leave_statement] = STATE(39),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(39),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(39),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(529),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [30] = {
    [sym_preproc_include] = STATE(39),
    [sym_preproc_def] = STATE(39),
    [sym_preproc_function_def] = STATE(39),
    [sym_preproc_call] = STATE(39),
    [sym_preproc_if] = STATE(39),
    [sym_preproc_ifdef] = STATE(39),
    [sym_function_definition] = STATE(39),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(39),
    [sym_type_definition] = STATE(39),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(39),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(39),
    [sym_labeled_statement] = STATE(39),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_case_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_do_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_goto_statement] = STATE(39),
    [sym_seh_try_statement] = STATE(39),
    [sym_seh_leave_statement] = STATE(39),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(39),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(39),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(531),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [31] = {
    [sym_preproc_include] = STATE(30),
    [sym_preproc_def] = STATE(30),
    [sym_preproc_function_def] = STATE(30),
    [sym_preproc_call] = STATE(30),
    [sym_preproc_if] = STATE(30),
    [sym_preproc_ifdef] = STATE(30),
    [sym_function_definition] = STATE(30),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(30),
    [sym_type_definition] = STATE(30),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(30),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(30),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(30),
    [sym_labeled_statement] = STATE(30),
    [sym_expression_statement] = STATE(30),
    [sym_if_statement] = STATE(30),
    [sym_switch_statement] = STATE(30),
    [sym_case_statement] = STATE(30),
    [sym_while_statement] = STATE(30),
    [sym_do_statement] = STATE(30),
    [sym_for_statement] = STATE(30),
    [sym_return_statement] = STATE(30),
    [sym_break_statement] = STATE(30),
    [sym_continue_statement] = STATE(30),
    [sym_goto_statement] = STATE(30),
    [sym_seh_try_statement] = STATE(30),
    [sym_seh_leave_statement] = STATE(30),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(30),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(30),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(533),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [32] = {
    [sym_preproc_include] = STATE(39),
    [sym_preproc_def] = STATE(39),
    [sym_preproc_function_def] = STATE(39),
    [sym_preproc_call] = STATE(39),
    [sym_preproc_if] = STATE(39),
    [sym_preproc_ifdef] = STATE(39),
    [sym_function_definition] = STATE(39),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(39),
    [sym_type_definition] = STATE(39),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(39),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(39),
    [sym_labeled_statement] = STATE(39),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_case_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_do_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_goto_statement] = STATE(39),
    [sym_seh_try_statement] = STATE(39),
    [sym_seh_leave_statement] = STATE(39),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(39),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(39),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(535),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [33] = {
    [sym_preproc_include] = STATE(29),
    [sym_preproc_def] = STATE(29),
    [sym_preproc_function_def] = STATE(29),
    [sym_preproc_call] = STATE(29),
    [sym_preproc_if] = STATE(29),
    [sym_preproc_ifdef] = STATE(29),
    [sym_function_definition] = STATE(29),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(29),
    [sym_type_definition] = STATE(29),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(29),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(29),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(29),
    [sym_labeled_statement] = STATE(29),
    [sym_expression_statement] = STATE(29),
    [sym_if_statement] = STATE(29),
    [sym_switch_statement] = STATE(29),
    [sym_case_statement] = STATE(29),
    [sym_while_statement] = STATE(29),
    [sym_do_statement] = STATE(29),
    [sym_for_statement] = STATE(29),
    [sym_return_statement] = STATE(29),
    [sym_break_statement] = STATE(29),
    [sym_continue_statement] = STATE(29),
    [sym_goto_statement] = STATE(29),
    [sym_seh_try_statement] = STATE(29),
    [sym_seh_leave_statement] = STATE(29),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(29),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(29),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(537),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [34] = {
    [sym_preproc_include] = STATE(39),
    [sym_preproc_def] = STATE(39),
    [sym_preproc_function_def] = STATE(39),
    [sym_preproc_call] = STATE(39),
    [sym_preproc_if] = STATE(39),
    [sym_preproc_ifdef] = STATE(39),
    [sym_function_definition] = STATE(39),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(39),
    [sym_type_definition] = STATE(39),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(39),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(39),
    [sym_labeled_statement] = STATE(39),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_case_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_do_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_goto_statement] = STATE(39),
    [sym_seh_try_statement] = STATE(39),
    [sym_seh_leave_statement] = STATE(39),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(39),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(39),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(539),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [35] = {
    [sym_preproc_include] = STATE(39),
    [sym_preproc_def] = STATE(39),
    [sym_preproc_function_def] = STATE(39),
    [sym_preproc_call] = STATE(39),
    [sym_preproc_if] = STATE(39),
    [sym_preproc_ifdef] = STATE(39),
    [sym_function_definition] = STATE(39),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(39),
    [sym_type_definition] = STATE(39),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(39),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(39),
    [sym_labeled_statement] = STATE(39),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_case_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_do_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_goto_statement] = STATE(39),
    [sym_seh_try_statement] = STATE(39),
    [sym_seh_leave_statement] = STATE(39),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(39),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(39),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(541),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [36] = {
    [sym_preproc_include] = STATE(39),
    [sym_preproc_def] = STATE(39),
    [sym_preproc_function_def] = STATE(39),
    [sym_preproc_call] = STATE(39),
    [sym_preproc_if] = STATE(39),
    [sym_preproc_ifdef] = STATE(39),
    [sym_function_definition] = STATE(39),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(39),
    [sym_type_definition] = STATE(39),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(39),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(39),
    [sym_labeled_statement] = STATE(39),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_case_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_do_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_goto_statement] = STATE(39),
    [sym_seh_try_statement] = STATE(39),
    [sym_seh_leave_statement] = STATE(39),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(39),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(39),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(543),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [37] = {
    [sym_preproc_include] = STATE(32),
    [sym_preproc_def] = STATE(32),
    [sym_preproc_function_def] = STATE(32),
    [sym_preproc_call] = STATE(32),
    [sym_preproc_if] = STATE(32),
    [sym_preproc_ifdef] = STATE(32),
    [sym_function_definition] = STATE(32),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(32),
    [sym_type_definition] = STATE(32),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(32),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(32),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(32),
    [sym_labeled_statement] = STATE(32),
    [sym_expression_statement] = STATE(32),
    [sym_if_statement] = STATE(32),
    [sym_switch_statement] = STATE(32),
    [sym_case_statement] = STATE(32),
    [sym_while_statement] = STATE(32),
    [sym_do_statement] = STATE(32),
    [sym_for_statement] = STATE(32),
    [sym_return_statement] = STATE(32),
    [sym_break_statement] = STATE(32),
    [sym_continue_statement] = STATE(32),
    [sym_goto_statement] = STATE(32),
    [sym_seh_try_statement] = STATE(32),
    [sym_seh_leave_statement] = STATE(32),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(32),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(32),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(545),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [38] = {
    [sym_preproc_include] = STATE(35),
    [sym_preproc_def] = STATE(35),
    [sym_preproc_function_def] = STATE(35),
    [sym_preproc_call] = STATE(35),
    [sym_preproc_if] = STATE(35),
    [sym_preproc_ifdef] = STATE(35),
    [sym_function_definition] = STATE(35),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(35),
    [sym_type_definition] = STATE(35),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(35),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(35),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(35),
    [sym_labeled_statement] = STATE(35),
    [sym_expression_statement] = STATE(35),
    [sym_if_statement] = STATE(35),
    [sym_switch_statement] = STATE(35),
    [sym_case_statement] = STATE(35),
    [sym_while_statement] = STATE(35),
    [sym_do_statement] = STATE(35),
    [sym_for_statement] = STATE(35),
    [sym_return_statement] = STATE(35),
    [sym_break_statement] = STATE(35),
    [sym_continue_statement] = STATE(35),
    [sym_goto_statement] = STATE(35),
    [sym_seh_try_statement] = STATE(35),
    [sym_seh_leave_statement] = STATE(35),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(35),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(35),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(547),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [39] = {
    [sym_preproc_include] = STATE(39),
    [sym_preproc_def] = STATE(39),
    [sym_preproc_function_def] = STATE(39),
    [sym_preproc_call] = STATE(39),
    [sym_preproc_if] = STATE(39),
    [sym_preproc_ifdef] = STATE(39),
    [sym_function_definition] = STATE(39),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(39),
    [sym_type_definition] = STATE(39),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(39),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(39),
    [sym_labeled_statement] = STATE(39),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_case_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_do_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_goto_statement] = STATE(39),
    [sym_seh_try_statement] = STATE(39),
    [sym_seh_leave_statement] = STATE(39),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(39),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(39),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(549),
    [aux_sym_preproc_include_token1] = ACTIONS(552),
    [aux_sym_preproc_def_token1] = ACTIONS(555),
    [aux_sym_preproc_if_token1] = ACTIONS(558),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(561),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(561),
    [sym_preproc_directive] = ACTIONS(564),
    [anon_sym_LPAREN2] = ACTIONS(219),
    [anon_sym_BANG] = ACTIONS(222),
    [anon_sym_TILDE] = ACTIONS(222),
    [anon_sym_DASH] = ACTIONS(225),
    [anon_sym_PLUS] = ACTIONS(225),
    [anon_sym_STAR] = ACTIONS(228),
    [anon_sym_AMP] = ACTIONS(228),
    [anon_sym_SEMI] = ACTIONS(567),
    [anon_sym___extension__] = ACTIONS(570),
    [anon_sym_typedef] = ACTIONS(573),
    [anon_sym_extern] = ACTIONS(576),
    [anon_sym___attribute__] = ACTIONS(243),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(246),
    [anon_sym___declspec] = ACTIONS(249),
    [anon_sym___cdecl] = ACTIONS(252),
    [anon_sym___clrcall] = ACTIONS(252),
    [anon_sym___stdcall] = ACTIONS(252),
    [anon_sym___fastcall] = ACTIONS(252),
    [anon_sym___thiscall] = ACTIONS(252),
    [anon_sym___vectorcall] = ACTIONS(252),
    [anon_sym_LBRACE] = ACTIONS(579),
    [anon_sym_RBRACE] = ACTIONS(582),
    [anon_sym_signed] = ACTIONS(258),
    [anon_sym_unsigned] = ACTIONS(258),
    [anon_sym_long] = ACTIONS(258),
    [anon_sym_short] = ACTIONS(258),
    [anon_sym_static] = ACTIONS(261),
    [anon_sym_auto] = ACTIONS(261),
    [anon_sym_register] = ACTIONS(261),
    [anon_sym_inline] = ACTIONS(261),
    [anon_sym___inline] = ACTIONS(261),
    [anon_sym___inline__] = ACTIONS(261),
    [anon_sym___forceinline] = ACTIONS(261),
    [anon_sym_thread_local] = ACTIONS(261),
    [anon_sym___thread] = ACTIONS(261),
    [anon_sym_const] = ACTIONS(264),
    [anon_sym_constexpr] = ACTIONS(264),
    [anon_sym_volatile] = ACTIONS(264),
    [anon_sym_restrict] = ACTIONS(264),
    [anon_sym___restrict__] = ACTIONS(264),
    [anon_sym__Atomic] = ACTIONS(264),
    [anon_sym__Noreturn] = ACTIONS(264),
    [anon_sym_noreturn] = ACTIONS(264),
    [sym_primitive_type] = ACTIONS(267),
    [anon_sym_enum] = ACTIONS(270),
    [anon_sym_struct] = ACTIONS(273),
    [anon_sym_union] = ACTIONS(276),
    [anon_sym_if] = ACTIONS(584),
    [anon_sym_switch] = ACTIONS(587),
    [anon_sym_case] = ACTIONS(590),
    [anon_sym_default] = ACTIONS(593),
    [anon_sym_while] = ACTIONS(596),
    [anon_sym_do] = ACTIONS(599),
    [anon_sym_for] = ACTIONS(602),
    [anon_sym_return] = ACTIONS(605),
    [anon_sym_break] = ACTIONS(608),
    [anon_sym_continue] = ACTIONS(611),
    [anon_sym_goto] = ACTIONS(614),
    [anon_sym___try] = ACTIONS(617),
    [anon_sym___leave] = ACTIONS(620),
    [anon_sym_DASH_DASH] = ACTIONS(318),
    [anon_sym_PLUS_PLUS] = ACTIONS(318),
    [anon_sym_sizeof] = ACTIONS(321),
    [anon_sym___alignof__] = ACTIONS(324),
    [anon_sym___alignof] = ACTIONS(324),
    [anon_sym__alignof] = ACTIONS(324),
    [anon_sym_alignof] = ACTIONS(324),
    [anon_sym__Alignof] = ACTIONS(324),
    [anon_sym_offsetof] = ACTIONS(327),
    [anon_sym__Generic] = ACTIONS(330),
    [anon_sym_asm] = ACTIONS(333),
    [anon_sym___asm__] = ACTIONS(333),
    [sym_number_literal] = ACTIONS(336),
    [anon_sym_L_SQUOTE] = ACTIONS(339),
    [anon_sym_u_SQUOTE] = ACTIONS(339),
    [anon_sym_U_SQUOTE] = ACTIONS(339),
    [anon_sym_u8_SQUOTE] = ACTIONS(339),
    [anon_sym_SQUOTE] = ACTIONS(339),
    [anon_sym_L_DQUOTE] = ACTIONS(342),
    [anon_sym_u_DQUOTE] = ACTIONS(342),
    [anon_sym_U_DQUOTE] = ACTIONS(342),
    [anon_sym_u8_DQUOTE] = ACTIONS(342),
    [anon_sym_DQUOTE] = ACTIONS(342),
    [sym_true] = ACTIONS(345),
    [sym_false] = ACTIONS(345),
    [anon_sym_NULL] = ACTIONS(348),
    [anon_sym_nullptr] = ACTIONS(348),
    [sym_comment] = ACTIONS(3),
  },
  [40] = {
    [sym_preproc_include] = STATE(42),
    [sym_preproc_def] = STATE(42),
    [sym_preproc_function_def] = STATE(42),
    [sym_preproc_call] = STATE(42),
    [sym_preproc_if] = STATE(42),
    [sym_preproc_ifdef] = STATE(42),
    [sym_function_definition] = STATE(42),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(42),
    [sym_type_definition] = STATE(42),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(42),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(42),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(42),
    [sym_labeled_statement] = STATE(42),
    [sym_expression_statement] = STATE(42),
    [sym_if_statement] = STATE(42),
    [sym_switch_statement] = STATE(42),
    [sym_case_statement] = STATE(42),
    [sym_while_statement] = STATE(42),
    [sym_do_statement] = STATE(42),
    [sym_for_statement] = STATE(42),
    [sym_return_statement] = STATE(42),
    [sym_break_statement] = STATE(42),
    [sym_continue_statement] = STATE(42),
    [sym_goto_statement] = STATE(42),
    [sym_seh_try_statement] = STATE(42),
    [sym_seh_leave_statement] = STATE(42),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(42),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(42),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(623),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [41] = {
    [sym_preproc_include] = STATE(36),
    [sym_preproc_def] = STATE(36),
    [sym_preproc_function_def] = STATE(36),
    [sym_preproc_call] = STATE(36),
    [sym_preproc_if] = STATE(36),
    [sym_preproc_ifdef] = STATE(36),
    [sym_function_definition] = STATE(36),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(36),
    [sym_type_definition] = STATE(36),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(36),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(36),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(36),
    [sym_labeled_statement] = STATE(36),
    [sym_expression_statement] = STATE(36),
    [sym_if_statement] = STATE(36),
    [sym_switch_statement] = STATE(36),
    [sym_case_statement] = STATE(36),
    [sym_while_statement] = STATE(36),
    [sym_do_statement] = STATE(36),
    [sym_for_statement] = STATE(36),
    [sym_return_statement] = STATE(36),
    [sym_break_statement] = STATE(36),
    [sym_continue_statement] = STATE(36),
    [sym_goto_statement] = STATE(36),
    [sym_seh_try_statement] = STATE(36),
    [sym_seh_leave_statement] = STATE(36),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(36),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(36),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(625),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [42] = {
    [sym_preproc_include] = STATE(39),
    [sym_preproc_def] = STATE(39),
    [sym_preproc_function_def] = STATE(39),
    [sym_preproc_call] = STATE(39),
    [sym_preproc_if] = STATE(39),
    [sym_preproc_ifdef] = STATE(39),
    [sym_function_definition] = STATE(39),
    [sym__old_style_function_definition] = STATE(367),
    [sym_declaration] = STATE(39),
    [sym_type_definition] = STATE(39),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1134),
    [sym_linkage_specification] = STATE(39),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(746),
    [sym_compound_statement] = STATE(39),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(867),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(39),
    [sym_labeled_statement] = STATE(39),
    [sym_expression_statement] = STATE(39),
    [sym_if_statement] = STATE(39),
    [sym_switch_statement] = STATE(39),
    [sym_case_statement] = STATE(39),
    [sym_while_statement] = STATE(39),
    [sym_do_statement] = STATE(39),
    [sym_for_statement] = STATE(39),
    [sym_return_statement] = STATE(39),
    [sym_break_statement] = STATE(39),
    [sym_continue_statement] = STATE(39),
    [sym_goto_statement] = STATE(39),
    [sym_seh_try_statement] = STATE(39),
    [sym_seh_leave_statement] = STATE(39),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym__empty_declaration] = STATE(39),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_repeat1] = STATE(39),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(351),
    [aux_sym_preproc_include_token1] = ACTIONS(353),
    [aux_sym_preproc_def_token1] = ACTIONS(355),
    [aux_sym_preproc_if_token1] = ACTIONS(357),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(359),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(359),
    [sym_preproc_directive] = ACTIONS(361),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(369),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(627),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [43] = {
    [sym_preproc_include] = STATE(44),
    [sym_preproc_def] = STATE(44),
    [sym_preproc_function_def] = STATE(44),
    [sym_preproc_call] = STATE(44),
    [sym_preproc_if] = STATE(44),
    [sym_preproc_ifdef] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym__old_style_function_definition] = STATE(415),
    [sym_declaration] = STATE(44),
    [sym_type_definition] = STATE(44),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1129),
    [sym_linkage_specification] = STATE(44),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(747),
    [sym_compound_statement] = STATE(44),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(866),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(44),
    [sym_labeled_statement] = STATE(44),
    [sym__top_level_expression_statement] = STATE(44),
    [sym_if_statement] = STATE(44),
    [sym_switch_statement] = STATE(44),
    [sym_case_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_do_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_return_statement] = STATE(44),
    [sym_break_statement] = STATE(44),
    [sym_continue_statement] = STATE(44),
    [sym_goto_statement] = STATE(44),
    [sym__expression] = STATE(1117),
    [sym__expression_not_binary] = STATE(1115),
    [sym__string] = STATE(1115),
    [sym_conditional_expression] = STATE(1115),
    [sym_assignment_expression] = STATE(1115),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(1115),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(1115),
    [sym_cast_expression] = STATE(1115),
    [sym_sizeof_expression] = STATE(1115),
    [sym_alignof_expression] = STATE(1115),
    [sym_offsetof_expression] = STATE(1115),
    [sym_generic_expression] = STATE(1115),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(1115),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(1115),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(1115),
    [sym_concatenated_string] = STATE(1115),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(1115),
    [sym__empty_declaration] = STATE(44),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_translation_unit_repeat1] = STATE(44),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [ts_builtin_sym_end] = ACTIONS(629),
    [sym_identifier] = ACTIONS(7),
    [aux_sym_preproc_include_token1] = ACTIONS(9),
    [aux_sym_preproc_def_token1] = ACTIONS(11),
    [aux_sym_preproc_if_token1] = ACTIONS(13),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(15),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(15),
    [sym_preproc_directive] = ACTIONS(17),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(31),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(61),
    [anon_sym_default] = ACTIONS(63),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(91),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(97),
    [sym_false] = ACTIONS(97),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [44] = {
    [sym_preproc_include] = STATE(44),
    [sym_preproc_def] = STATE(44),
    [sym_preproc_function_def] = STATE(44),
    [sym_preproc_call] = STATE(44),
    [sym_preproc_if] = STATE(44),
    [sym_preproc_ifdef] = STATE(44),
    [sym_function_definition] = STATE(44),
    [sym__old_style_function_definition] = STATE(415),
    [sym_declaration] = STATE(44),
    [sym_type_definition] = STATE(44),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1129),
    [sym_linkage_specification] = STATE(44),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(747),
    [sym_compound_statement] = STATE(44),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(866),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(44),
    [sym_labeled_statement] = STATE(44),
    [sym__top_level_expression_statement] = STATE(44),
    [sym_if_statement] = STATE(44),
    [sym_switch_statement] = STATE(44),
    [sym_case_statement] = STATE(44),
    [sym_while_statement] = STATE(44),
    [sym_do_statement] = STATE(44),
    [sym_for_statement] = STATE(44),
    [sym_return_statement] = STATE(44),
    [sym_break_statement] = STATE(44),
    [sym_continue_statement] = STATE(44),
    [sym_goto_statement] = STATE(44),
    [sym__expression] = STATE(1117),
    [sym__expression_not_binary] = STATE(1115),
    [sym__string] = STATE(1115),
    [sym_conditional_expression] = STATE(1115),
    [sym_assignment_expression] = STATE(1115),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(1115),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(1115),
    [sym_cast_expression] = STATE(1115),
    [sym_sizeof_expression] = STATE(1115),
    [sym_alignof_expression] = STATE(1115),
    [sym_offsetof_expression] = STATE(1115),
    [sym_generic_expression] = STATE(1115),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(1115),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(1115),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(1115),
    [sym_concatenated_string] = STATE(1115),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(1115),
    [sym__empty_declaration] = STATE(44),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_translation_unit_repeat1] = STATE(44),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [ts_builtin_sym_end] = ACTIONS(631),
    [sym_identifier] = ACTIONS(633),
    [aux_sym_preproc_include_token1] = ACTIONS(636),
    [aux_sym_preproc_def_token1] = ACTIONS(639),
    [aux_sym_preproc_if_token1] = ACTIONS(642),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(645),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(645),
    [sym_preproc_directive] = ACTIONS(648),
    [anon_sym_LPAREN2] = ACTIONS(651),
    [anon_sym_BANG] = ACTIONS(654),
    [anon_sym_TILDE] = ACTIONS(654),
    [anon_sym_DASH] = ACTIONS(657),
    [anon_sym_PLUS] = ACTIONS(657),
    [anon_sym_STAR] = ACTIONS(660),
    [anon_sym_AMP] = ACTIONS(660),
    [anon_sym___extension__] = ACTIONS(663),
    [anon_sym_typedef] = ACTIONS(666),
    [anon_sym_extern] = ACTIONS(669),
    [anon_sym___attribute__] = ACTIONS(672),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(675),
    [anon_sym___declspec] = ACTIONS(678),
    [anon_sym___cdecl] = ACTIONS(681),
    [anon_sym___clrcall] = ACTIONS(681),
    [anon_sym___stdcall] = ACTIONS(681),
    [anon_sym___fastcall] = ACTIONS(681),
    [anon_sym___thiscall] = ACTIONS(681),
    [anon_sym___vectorcall] = ACTIONS(681),
    [anon_sym_LBRACE] = ACTIONS(684),
    [anon_sym_signed] = ACTIONS(687),
    [anon_sym_unsigned] = ACTIONS(687),
    [anon_sym_long] = ACTIONS(687),
    [anon_sym_short] = ACTIONS(687),
    [anon_sym_static] = ACTIONS(690),
    [anon_sym_auto] = ACTIONS(690),
    [anon_sym_register] = ACTIONS(690),
    [anon_sym_inline] = ACTIONS(690),
    [anon_sym___inline] = ACTIONS(690),
    [anon_sym___inline__] = ACTIONS(690),
    [anon_sym___forceinline] = ACTIONS(690),
    [anon_sym_thread_local] = ACTIONS(690),
    [anon_sym___thread] = ACTIONS(690),
    [anon_sym_const] = ACTIONS(693),
    [anon_sym_constexpr] = ACTIONS(693),
    [anon_sym_volatile] = ACTIONS(693),
    [anon_sym_restrict] = ACTIONS(693),
    [anon_sym___restrict__] = ACTIONS(693),
    [anon_sym__Atomic] = ACTIONS(693),
    [anon_sym__Noreturn] = ACTIONS(693),
    [anon_sym_noreturn] = ACTIONS(693),
    [sym_primitive_type] = ACTIONS(696),
    [anon_sym_enum] = ACTIONS(699),
    [anon_sym_struct] = ACTIONS(702),
    [anon_sym_union] = ACTIONS(705),
    [anon_sym_if] = ACTIONS(708),
    [anon_sym_switch] = ACTIONS(711),
    [anon_sym_case] = ACTIONS(714),
    [anon_sym_default] = ACTIONS(717),
    [anon_sym_while] = ACTIONS(720),
    [anon_sym_do] = ACTIONS(723),
    [anon_sym_for] = ACTIONS(726),
    [anon_sym_return] = ACTIONS(729),
    [anon_sym_break] = ACTIONS(732),
    [anon_sym_continue] = ACTIONS(735),
    [anon_sym_goto] = ACTIONS(738),
    [anon_sym_DASH_DASH] = ACTIONS(741),
    [anon_sym_PLUS_PLUS] = ACTIONS(741),
    [anon_sym_sizeof] = ACTIONS(744),
    [anon_sym___alignof__] = ACTIONS(747),
    [anon_sym___alignof] = ACTIONS(747),
    [anon_sym__alignof] = ACTIONS(747),
    [anon_sym_alignof] = ACTIONS(747),
    [anon_sym__Alignof] = ACTIONS(747),
    [anon_sym_offsetof] = ACTIONS(750),
    [anon_sym__Generic] = ACTIONS(753),
    [anon_sym_asm] = ACTIONS(756),
    [anon_sym___asm__] = ACTIONS(756),
    [sym_number_literal] = ACTIONS(759),
    [anon_sym_L_SQUOTE] = ACTIONS(762),
    [anon_sym_u_SQUOTE] = ACTIONS(762),
    [anon_sym_U_SQUOTE] = ACTIONS(762),
    [anon_sym_u8_SQUOTE] = ACTIONS(762),
    [anon_sym_SQUOTE] = ACTIONS(762),
    [anon_sym_L_DQUOTE] = ACTIONS(765),
    [anon_sym_u_DQUOTE] = ACTIONS(765),
    [anon_sym_U_DQUOTE] = ACTIONS(765),
    [anon_sym_u8_DQUOTE] = ACTIONS(765),
    [anon_sym_DQUOTE] = ACTIONS(765),
    [sym_true] = ACTIONS(768),
    [sym_false] = ACTIONS(768),
    [anon_sym_NULL] = ACTIONS(771),
    [anon_sym_nullptr] = ACTIONS(771),
    [sym_comment] = ACTIONS(3),
  },
  [45] = {
    [sym_declaration] = STATE(46),
    [sym_type_definition] = STATE(46),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1153),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(46),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(46),
    [sym_labeled_statement] = STATE(46),
    [sym_expression_statement] = STATE(46),
    [sym_if_statement] = STATE(46),
    [sym_switch_statement] = STATE(46),
    [sym_while_statement] = STATE(46),
    [sym_do_statement] = STATE(46),
    [sym_for_statement] = STATE(46),
    [sym_return_statement] = STATE(46),
    [sym_break_statement] = STATE(46),
    [sym_continue_statement] = STATE(46),
    [sym_goto_statement] = STATE(46),
    [sym_seh_try_statement] = STATE(46),
    [sym_seh_leave_statement] = STATE(46),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(46),
    [sym_identifier] = ACTIONS(774),
    [aux_sym_preproc_include_token1] = ACTIONS(776),
    [aux_sym_preproc_def_token1] = ACTIONS(776),
    [aux_sym_preproc_if_token1] = ACTIONS(776),
    [aux_sym_preproc_if_token2] = ACTIONS(776),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(776),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(776),
    [aux_sym_preproc_else_token1] = ACTIONS(776),
    [aux_sym_preproc_elif_token1] = ACTIONS(776),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(776),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(776),
    [sym_preproc_directive] = ACTIONS(776),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(776),
    [anon_sym___clrcall] = ACTIONS(776),
    [anon_sym___stdcall] = ACTIONS(776),
    [anon_sym___fastcall] = ACTIONS(776),
    [anon_sym___thiscall] = ACTIONS(776),
    [anon_sym___vectorcall] = ACTIONS(776),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_else] = ACTIONS(776),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(776),
    [anon_sym_default] = ACTIONS(776),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [46] = {
    [sym_declaration] = STATE(47),
    [sym_type_definition] = STATE(47),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1153),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(47),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(47),
    [sym_labeled_statement] = STATE(47),
    [sym_expression_statement] = STATE(47),
    [sym_if_statement] = STATE(47),
    [sym_switch_statement] = STATE(47),
    [sym_while_statement] = STATE(47),
    [sym_do_statement] = STATE(47),
    [sym_for_statement] = STATE(47),
    [sym_return_statement] = STATE(47),
    [sym_break_statement] = STATE(47),
    [sym_continue_statement] = STATE(47),
    [sym_goto_statement] = STATE(47),
    [sym_seh_try_statement] = STATE(47),
    [sym_seh_leave_statement] = STATE(47),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(47),
    [sym_identifier] = ACTIONS(774),
    [aux_sym_preproc_include_token1] = ACTIONS(778),
    [aux_sym_preproc_def_token1] = ACTIONS(778),
    [aux_sym_preproc_if_token1] = ACTIONS(778),
    [aux_sym_preproc_if_token2] = ACTIONS(778),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(778),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(778),
    [aux_sym_preproc_else_token1] = ACTIONS(778),
    [aux_sym_preproc_elif_token1] = ACTIONS(778),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(778),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(778),
    [sym_preproc_directive] = ACTIONS(778),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(778),
    [anon_sym___clrcall] = ACTIONS(778),
    [anon_sym___stdcall] = ACTIONS(778),
    [anon_sym___fastcall] = ACTIONS(778),
    [anon_sym___thiscall] = ACTIONS(778),
    [anon_sym___vectorcall] = ACTIONS(778),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_else] = ACTIONS(778),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(778),
    [anon_sym_default] = ACTIONS(778),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [47] = {
    [sym_declaration] = STATE(47),
    [sym_type_definition] = STATE(47),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1153),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(47),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(47),
    [sym_labeled_statement] = STATE(47),
    [sym_expression_statement] = STATE(47),
    [sym_if_statement] = STATE(47),
    [sym_switch_statement] = STATE(47),
    [sym_while_statement] = STATE(47),
    [sym_do_statement] = STATE(47),
    [sym_for_statement] = STATE(47),
    [sym_return_statement] = STATE(47),
    [sym_break_statement] = STATE(47),
    [sym_continue_statement] = STATE(47),
    [sym_goto_statement] = STATE(47),
    [sym_seh_try_statement] = STATE(47),
    [sym_seh_leave_statement] = STATE(47),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(47),
    [sym_identifier] = ACTIONS(780),
    [aux_sym_preproc_include_token1] = ACTIONS(783),
    [aux_sym_preproc_def_token1] = ACTIONS(783),
    [aux_sym_preproc_if_token1] = ACTIONS(783),
    [aux_sym_preproc_if_token2] = ACTIONS(783),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(783),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(783),
    [aux_sym_preproc_else_token1] = ACTIONS(783),
    [aux_sym_preproc_elif_token1] = ACTIONS(783),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(783),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(783),
    [sym_preproc_directive] = ACTIONS(783),
    [anon_sym_LPAREN2] = ACTIONS(785),
    [anon_sym_BANG] = ACTIONS(788),
    [anon_sym_TILDE] = ACTIONS(788),
    [anon_sym_DASH] = ACTIONS(791),
    [anon_sym_PLUS] = ACTIONS(791),
    [anon_sym_STAR] = ACTIONS(794),
    [anon_sym_AMP] = ACTIONS(794),
    [anon_sym_SEMI] = ACTIONS(797),
    [anon_sym___extension__] = ACTIONS(800),
    [anon_sym_typedef] = ACTIONS(803),
    [anon_sym_extern] = ACTIONS(806),
    [anon_sym___attribute__] = ACTIONS(809),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(812),
    [anon_sym___declspec] = ACTIONS(815),
    [anon_sym___cdecl] = ACTIONS(783),
    [anon_sym___clrcall] = ACTIONS(783),
    [anon_sym___stdcall] = ACTIONS(783),
    [anon_sym___fastcall] = ACTIONS(783),
    [anon_sym___thiscall] = ACTIONS(783),
    [anon_sym___vectorcall] = ACTIONS(783),
    [anon_sym_LBRACE] = ACTIONS(818),
    [anon_sym_signed] = ACTIONS(821),
    [anon_sym_unsigned] = ACTIONS(821),
    [anon_sym_long] = ACTIONS(821),
    [anon_sym_short] = ACTIONS(821),
    [anon_sym_static] = ACTIONS(806),
    [anon_sym_auto] = ACTIONS(806),
    [anon_sym_register] = ACTIONS(806),
    [anon_sym_inline] = ACTIONS(806),
    [anon_sym___inline] = ACTIONS(806),
    [anon_sym___inline__] = ACTIONS(806),
    [anon_sym___forceinline] = ACTIONS(806),
    [anon_sym_thread_local] = ACTIONS(806),
    [anon_sym___thread] = ACTIONS(806),
    [anon_sym_const] = ACTIONS(824),
    [anon_sym_constexpr] = ACTIONS(824),
    [anon_sym_volatile] = ACTIONS(824),
    [anon_sym_restrict] = ACTIONS(824),
    [anon_sym___restrict__] = ACTIONS(824),
    [anon_sym__Atomic] = ACTIONS(824),
    [anon_sym__Noreturn] = ACTIONS(824),
    [anon_sym_noreturn] = ACTIONS(824),
    [sym_primitive_type] = ACTIONS(827),
    [anon_sym_enum] = ACTIONS(830),
    [anon_sym_struct] = ACTIONS(833),
    [anon_sym_union] = ACTIONS(836),
    [anon_sym_if] = ACTIONS(839),
    [anon_sym_else] = ACTIONS(783),
    [anon_sym_switch] = ACTIONS(842),
    [anon_sym_case] = ACTIONS(783),
    [anon_sym_default] = ACTIONS(783),
    [anon_sym_while] = ACTIONS(845),
    [anon_sym_do] = ACTIONS(848),
    [anon_sym_for] = ACTIONS(851),
    [anon_sym_return] = ACTIONS(854),
    [anon_sym_break] = ACTIONS(857),
    [anon_sym_continue] = ACTIONS(860),
    [anon_sym_goto] = ACTIONS(863),
    [anon_sym___try] = ACTIONS(866),
    [anon_sym___leave] = ACTIONS(869),
    [anon_sym_DASH_DASH] = ACTIONS(872),
    [anon_sym_PLUS_PLUS] = ACTIONS(872),
    [anon_sym_sizeof] = ACTIONS(875),
    [anon_sym___alignof__] = ACTIONS(878),
    [anon_sym___alignof] = ACTIONS(878),
    [anon_sym__alignof] = ACTIONS(878),
    [anon_sym_alignof] = ACTIONS(878),
    [anon_sym__Alignof] = ACTIONS(878),
    [anon_sym_offsetof] = ACTIONS(881),
    [anon_sym__Generic] = ACTIONS(884),
    [anon_sym_asm] = ACTIONS(887),
    [anon_sym___asm__] = ACTIONS(887),
    [sym_number_literal] = ACTIONS(890),
    [anon_sym_L_SQUOTE] = ACTIONS(893),
    [anon_sym_u_SQUOTE] = ACTIONS(893),
    [anon_sym_U_SQUOTE] = ACTIONS(893),
    [anon_sym_u8_SQUOTE] = ACTIONS(893),
    [anon_sym_SQUOTE] = ACTIONS(893),
    [anon_sym_L_DQUOTE] = ACTIONS(896),
    [anon_sym_u_DQUOTE] = ACTIONS(896),
    [anon_sym_U_DQUOTE] = ACTIONS(896),
    [anon_sym_u8_DQUOTE] = ACTIONS(896),
    [anon_sym_DQUOTE] = ACTIONS(896),
    [sym_true] = ACTIONS(899),
    [sym_false] = ACTIONS(899),
    [anon_sym_NULL] = ACTIONS(902),
    [anon_sym_nullptr] = ACTIONS(902),
    [sym_comment] = ACTIONS(3),
  },
  [48] = {
    [sym_declaration] = STATE(47),
    [sym_type_definition] = STATE(47),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1153),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(47),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(47),
    [sym_labeled_statement] = STATE(47),
    [sym_expression_statement] = STATE(47),
    [sym_if_statement] = STATE(47),
    [sym_switch_statement] = STATE(47),
    [sym_while_statement] = STATE(47),
    [sym_do_statement] = STATE(47),
    [sym_for_statement] = STATE(47),
    [sym_return_statement] = STATE(47),
    [sym_break_statement] = STATE(47),
    [sym_continue_statement] = STATE(47),
    [sym_goto_statement] = STATE(47),
    [sym_seh_try_statement] = STATE(47),
    [sym_seh_leave_statement] = STATE(47),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(47),
    [sym_identifier] = ACTIONS(774),
    [aux_sym_preproc_include_token1] = ACTIONS(905),
    [aux_sym_preproc_def_token1] = ACTIONS(905),
    [aux_sym_preproc_if_token1] = ACTIONS(905),
    [aux_sym_preproc_if_token2] = ACTIONS(905),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(905),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(905),
    [aux_sym_preproc_else_token1] = ACTIONS(905),
    [aux_sym_preproc_elif_token1] = ACTIONS(905),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(905),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(905),
    [sym_preproc_directive] = ACTIONS(905),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(905),
    [anon_sym___clrcall] = ACTIONS(905),
    [anon_sym___stdcall] = ACTIONS(905),
    [anon_sym___fastcall] = ACTIONS(905),
    [anon_sym___thiscall] = ACTIONS(905),
    [anon_sym___vectorcall] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(905),
    [anon_sym_default] = ACTIONS(905),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [49] = {
    [sym_declaration] = STATE(48),
    [sym_type_definition] = STATE(48),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1153),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(48),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(48),
    [sym_labeled_statement] = STATE(48),
    [sym_expression_statement] = STATE(48),
    [sym_if_statement] = STATE(48),
    [sym_switch_statement] = STATE(48),
    [sym_while_statement] = STATE(48),
    [sym_do_statement] = STATE(48),
    [sym_for_statement] = STATE(48),
    [sym_return_statement] = STATE(48),
    [sym_break_statement] = STATE(48),
    [sym_continue_statement] = STATE(48),
    [sym_goto_statement] = STATE(48),
    [sym_seh_try_statement] = STATE(48),
    [sym_seh_leave_statement] = STATE(48),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(48),
    [sym_identifier] = ACTIONS(774),
    [aux_sym_preproc_include_token1] = ACTIONS(907),
    [aux_sym_preproc_def_token1] = ACTIONS(907),
    [aux_sym_preproc_if_token1] = ACTIONS(907),
    [aux_sym_preproc_if_token2] = ACTIONS(907),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(907),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(907),
    [aux_sym_preproc_else_token1] = ACTIONS(907),
    [aux_sym_preproc_elif_token1] = ACTIONS(907),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(907),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(907),
    [sym_preproc_directive] = ACTIONS(907),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym___extension__] = ACTIONS(123),
    [anon_sym_typedef] = ACTIONS(125),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(907),
    [anon_sym___clrcall] = ACTIONS(907),
    [anon_sym___stdcall] = ACTIONS(907),
    [anon_sym___fastcall] = ACTIONS(907),
    [anon_sym___thiscall] = ACTIONS(907),
    [anon_sym___vectorcall] = ACTIONS(907),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_else] = ACTIONS(907),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(907),
    [anon_sym_default] = ACTIONS(907),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [50] = {
    [sym_declaration] = STATE(60),
    [sym_type_definition] = STATE(60),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(60),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(60),
    [sym_labeled_statement] = STATE(60),
    [sym_expression_statement] = STATE(60),
    [sym_if_statement] = STATE(60),
    [sym_switch_statement] = STATE(60),
    [sym_while_statement] = STATE(60),
    [sym_do_statement] = STATE(60),
    [sym_for_statement] = STATE(60),
    [sym_return_statement] = STATE(60),
    [sym_break_statement] = STATE(60),
    [sym_continue_statement] = STATE(60),
    [sym_goto_statement] = STATE(60),
    [sym_seh_try_statement] = STATE(60),
    [sym_seh_leave_statement] = STATE(60),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(60),
    [ts_builtin_sym_end] = ACTIONS(909),
    [sym_identifier] = ACTIONS(911),
    [aux_sym_preproc_include_token1] = ACTIONS(905),
    [aux_sym_preproc_def_token1] = ACTIONS(905),
    [aux_sym_preproc_if_token1] = ACTIONS(905),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(905),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(905),
    [sym_preproc_directive] = ACTIONS(905),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(905),
    [anon_sym___clrcall] = ACTIONS(905),
    [anon_sym___stdcall] = ACTIONS(905),
    [anon_sym___fastcall] = ACTIONS(905),
    [anon_sym___thiscall] = ACTIONS(905),
    [anon_sym___vectorcall] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(905),
    [anon_sym_default] = ACTIONS(905),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [51] = {
    [sym_declaration] = STATE(51),
    [sym_type_definition] = STATE(51),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1144),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(51),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(51),
    [sym_labeled_statement] = STATE(51),
    [sym_expression_statement] = STATE(51),
    [sym_if_statement] = STATE(51),
    [sym_switch_statement] = STATE(51),
    [sym_while_statement] = STATE(51),
    [sym_do_statement] = STATE(51),
    [sym_for_statement] = STATE(51),
    [sym_return_statement] = STATE(51),
    [sym_break_statement] = STATE(51),
    [sym_continue_statement] = STATE(51),
    [sym_goto_statement] = STATE(51),
    [sym_seh_try_statement] = STATE(51),
    [sym_seh_leave_statement] = STATE(51),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(51),
    [sym_identifier] = ACTIONS(919),
    [aux_sym_preproc_include_token1] = ACTIONS(783),
    [aux_sym_preproc_def_token1] = ACTIONS(783),
    [aux_sym_preproc_if_token1] = ACTIONS(783),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(783),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(783),
    [sym_preproc_directive] = ACTIONS(783),
    [anon_sym_LPAREN2] = ACTIONS(785),
    [anon_sym_BANG] = ACTIONS(788),
    [anon_sym_TILDE] = ACTIONS(788),
    [anon_sym_DASH] = ACTIONS(791),
    [anon_sym_PLUS] = ACTIONS(791),
    [anon_sym_STAR] = ACTIONS(794),
    [anon_sym_AMP] = ACTIONS(794),
    [anon_sym_SEMI] = ACTIONS(922),
    [anon_sym___extension__] = ACTIONS(925),
    [anon_sym_typedef] = ACTIONS(928),
    [anon_sym_extern] = ACTIONS(806),
    [anon_sym___attribute__] = ACTIONS(809),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(812),
    [anon_sym___declspec] = ACTIONS(815),
    [anon_sym___cdecl] = ACTIONS(783),
    [anon_sym___clrcall] = ACTIONS(783),
    [anon_sym___stdcall] = ACTIONS(783),
    [anon_sym___fastcall] = ACTIONS(783),
    [anon_sym___thiscall] = ACTIONS(783),
    [anon_sym___vectorcall] = ACTIONS(783),
    [anon_sym_LBRACE] = ACTIONS(931),
    [anon_sym_RBRACE] = ACTIONS(934),
    [anon_sym_signed] = ACTIONS(821),
    [anon_sym_unsigned] = ACTIONS(821),
    [anon_sym_long] = ACTIONS(821),
    [anon_sym_short] = ACTIONS(821),
    [anon_sym_static] = ACTIONS(806),
    [anon_sym_auto] = ACTIONS(806),
    [anon_sym_register] = ACTIONS(806),
    [anon_sym_inline] = ACTIONS(806),
    [anon_sym___inline] = ACTIONS(806),
    [anon_sym___inline__] = ACTIONS(806),
    [anon_sym___forceinline] = ACTIONS(806),
    [anon_sym_thread_local] = ACTIONS(806),
    [anon_sym___thread] = ACTIONS(806),
    [anon_sym_const] = ACTIONS(824),
    [anon_sym_constexpr] = ACTIONS(824),
    [anon_sym_volatile] = ACTIONS(824),
    [anon_sym_restrict] = ACTIONS(824),
    [anon_sym___restrict__] = ACTIONS(824),
    [anon_sym__Atomic] = ACTIONS(824),
    [anon_sym__Noreturn] = ACTIONS(824),
    [anon_sym_noreturn] = ACTIONS(824),
    [sym_primitive_type] = ACTIONS(827),
    [anon_sym_enum] = ACTIONS(830),
    [anon_sym_struct] = ACTIONS(833),
    [anon_sym_union] = ACTIONS(836),
    [anon_sym_if] = ACTIONS(936),
    [anon_sym_else] = ACTIONS(783),
    [anon_sym_switch] = ACTIONS(939),
    [anon_sym_case] = ACTIONS(783),
    [anon_sym_default] = ACTIONS(783),
    [anon_sym_while] = ACTIONS(942),
    [anon_sym_do] = ACTIONS(945),
    [anon_sym_for] = ACTIONS(948),
    [anon_sym_return] = ACTIONS(951),
    [anon_sym_break] = ACTIONS(954),
    [anon_sym_continue] = ACTIONS(957),
    [anon_sym_goto] = ACTIONS(960),
    [anon_sym___try] = ACTIONS(963),
    [anon_sym___leave] = ACTIONS(966),
    [anon_sym_DASH_DASH] = ACTIONS(872),
    [anon_sym_PLUS_PLUS] = ACTIONS(872),
    [anon_sym_sizeof] = ACTIONS(875),
    [anon_sym___alignof__] = ACTIONS(878),
    [anon_sym___alignof] = ACTIONS(878),
    [anon_sym__alignof] = ACTIONS(878),
    [anon_sym_alignof] = ACTIONS(878),
    [anon_sym__Alignof] = ACTIONS(878),
    [anon_sym_offsetof] = ACTIONS(881),
    [anon_sym__Generic] = ACTIONS(884),
    [anon_sym_asm] = ACTIONS(887),
    [anon_sym___asm__] = ACTIONS(887),
    [sym_number_literal] = ACTIONS(890),
    [anon_sym_L_SQUOTE] = ACTIONS(893),
    [anon_sym_u_SQUOTE] = ACTIONS(893),
    [anon_sym_U_SQUOTE] = ACTIONS(893),
    [anon_sym_u8_SQUOTE] = ACTIONS(893),
    [anon_sym_SQUOTE] = ACTIONS(893),
    [anon_sym_L_DQUOTE] = ACTIONS(896),
    [anon_sym_u_DQUOTE] = ACTIONS(896),
    [anon_sym_U_DQUOTE] = ACTIONS(896),
    [anon_sym_u8_DQUOTE] = ACTIONS(896),
    [anon_sym_DQUOTE] = ACTIONS(896),
    [sym_true] = ACTIONS(899),
    [sym_false] = ACTIONS(899),
    [anon_sym_NULL] = ACTIONS(902),
    [anon_sym_nullptr] = ACTIONS(902),
    [sym_comment] = ACTIONS(3),
  },
  [52] = {
    [sym_declaration] = STATE(51),
    [sym_type_definition] = STATE(51),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1144),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(51),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(51),
    [sym_labeled_statement] = STATE(51),
    [sym_expression_statement] = STATE(51),
    [sym_if_statement] = STATE(51),
    [sym_switch_statement] = STATE(51),
    [sym_while_statement] = STATE(51),
    [sym_do_statement] = STATE(51),
    [sym_for_statement] = STATE(51),
    [sym_return_statement] = STATE(51),
    [sym_break_statement] = STATE(51),
    [sym_continue_statement] = STATE(51),
    [sym_goto_statement] = STATE(51),
    [sym_seh_try_statement] = STATE(51),
    [sym_seh_leave_statement] = STATE(51),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(51),
    [sym_identifier] = ACTIONS(969),
    [aux_sym_preproc_include_token1] = ACTIONS(778),
    [aux_sym_preproc_def_token1] = ACTIONS(778),
    [aux_sym_preproc_if_token1] = ACTIONS(778),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(778),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(778),
    [sym_preproc_directive] = ACTIONS(778),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(778),
    [anon_sym___clrcall] = ACTIONS(778),
    [anon_sym___stdcall] = ACTIONS(778),
    [anon_sym___fastcall] = ACTIONS(778),
    [anon_sym___thiscall] = ACTIONS(778),
    [anon_sym___vectorcall] = ACTIONS(778),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(971),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_else] = ACTIONS(778),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(778),
    [anon_sym_default] = ACTIONS(778),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [53] = {
    [sym_declaration] = STATE(51),
    [sym_type_definition] = STATE(51),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1144),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(51),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(51),
    [sym_labeled_statement] = STATE(51),
    [sym_expression_statement] = STATE(51),
    [sym_if_statement] = STATE(51),
    [sym_switch_statement] = STATE(51),
    [sym_while_statement] = STATE(51),
    [sym_do_statement] = STATE(51),
    [sym_for_statement] = STATE(51),
    [sym_return_statement] = STATE(51),
    [sym_break_statement] = STATE(51),
    [sym_continue_statement] = STATE(51),
    [sym_goto_statement] = STATE(51),
    [sym_seh_try_statement] = STATE(51),
    [sym_seh_leave_statement] = STATE(51),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(51),
    [sym_identifier] = ACTIONS(969),
    [aux_sym_preproc_include_token1] = ACTIONS(905),
    [aux_sym_preproc_def_token1] = ACTIONS(905),
    [aux_sym_preproc_if_token1] = ACTIONS(905),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(905),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(905),
    [sym_preproc_directive] = ACTIONS(905),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(905),
    [anon_sym___clrcall] = ACTIONS(905),
    [anon_sym___stdcall] = ACTIONS(905),
    [anon_sym___fastcall] = ACTIONS(905),
    [anon_sym___thiscall] = ACTIONS(905),
    [anon_sym___vectorcall] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(909),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(905),
    [anon_sym_default] = ACTIONS(905),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [54] = {
    [sym_declaration] = STATE(54),
    [sym_type_definition] = STATE(54),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1158),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(54),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(54),
    [sym_labeled_statement] = STATE(54),
    [sym_expression_statement] = STATE(54),
    [sym_if_statement] = STATE(54),
    [sym_switch_statement] = STATE(54),
    [sym_while_statement] = STATE(54),
    [sym_do_statement] = STATE(54),
    [sym_for_statement] = STATE(54),
    [sym_return_statement] = STATE(54),
    [sym_break_statement] = STATE(54),
    [sym_continue_statement] = STATE(54),
    [sym_goto_statement] = STATE(54),
    [sym_seh_try_statement] = STATE(54),
    [sym_seh_leave_statement] = STATE(54),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(54),
    [sym_identifier] = ACTIONS(973),
    [aux_sym_preproc_include_token1] = ACTIONS(783),
    [aux_sym_preproc_def_token1] = ACTIONS(783),
    [aux_sym_preproc_if_token1] = ACTIONS(783),
    [aux_sym_preproc_if_token2] = ACTIONS(783),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(783),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(783),
    [sym_preproc_directive] = ACTIONS(783),
    [anon_sym_LPAREN2] = ACTIONS(785),
    [anon_sym_BANG] = ACTIONS(788),
    [anon_sym_TILDE] = ACTIONS(788),
    [anon_sym_DASH] = ACTIONS(791),
    [anon_sym_PLUS] = ACTIONS(791),
    [anon_sym_STAR] = ACTIONS(794),
    [anon_sym_AMP] = ACTIONS(794),
    [anon_sym_SEMI] = ACTIONS(976),
    [anon_sym___extension__] = ACTIONS(979),
    [anon_sym_typedef] = ACTIONS(982),
    [anon_sym_extern] = ACTIONS(806),
    [anon_sym___attribute__] = ACTIONS(809),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(812),
    [anon_sym___declspec] = ACTIONS(815),
    [anon_sym___cdecl] = ACTIONS(783),
    [anon_sym___clrcall] = ACTIONS(783),
    [anon_sym___stdcall] = ACTIONS(783),
    [anon_sym___fastcall] = ACTIONS(783),
    [anon_sym___thiscall] = ACTIONS(783),
    [anon_sym___vectorcall] = ACTIONS(783),
    [anon_sym_LBRACE] = ACTIONS(985),
    [anon_sym_signed] = ACTIONS(821),
    [anon_sym_unsigned] = ACTIONS(821),
    [anon_sym_long] = ACTIONS(821),
    [anon_sym_short] = ACTIONS(821),
    [anon_sym_static] = ACTIONS(806),
    [anon_sym_auto] = ACTIONS(806),
    [anon_sym_register] = ACTIONS(806),
    [anon_sym_inline] = ACTIONS(806),
    [anon_sym___inline] = ACTIONS(806),
    [anon_sym___inline__] = ACTIONS(806),
    [anon_sym___forceinline] = ACTIONS(806),
    [anon_sym_thread_local] = ACTIONS(806),
    [anon_sym___thread] = ACTIONS(806),
    [anon_sym_const] = ACTIONS(824),
    [anon_sym_constexpr] = ACTIONS(824),
    [anon_sym_volatile] = ACTIONS(824),
    [anon_sym_restrict] = ACTIONS(824),
    [anon_sym___restrict__] = ACTIONS(824),
    [anon_sym__Atomic] = ACTIONS(824),
    [anon_sym__Noreturn] = ACTIONS(824),
    [anon_sym_noreturn] = ACTIONS(824),
    [sym_primitive_type] = ACTIONS(827),
    [anon_sym_enum] = ACTIONS(830),
    [anon_sym_struct] = ACTIONS(833),
    [anon_sym_union] = ACTIONS(836),
    [anon_sym_if] = ACTIONS(988),
    [anon_sym_else] = ACTIONS(783),
    [anon_sym_switch] = ACTIONS(991),
    [anon_sym_case] = ACTIONS(783),
    [anon_sym_default] = ACTIONS(783),
    [anon_sym_while] = ACTIONS(994),
    [anon_sym_do] = ACTIONS(997),
    [anon_sym_for] = ACTIONS(1000),
    [anon_sym_return] = ACTIONS(1003),
    [anon_sym_break] = ACTIONS(1006),
    [anon_sym_continue] = ACTIONS(1009),
    [anon_sym_goto] = ACTIONS(1012),
    [anon_sym___try] = ACTIONS(1015),
    [anon_sym___leave] = ACTIONS(1018),
    [anon_sym_DASH_DASH] = ACTIONS(872),
    [anon_sym_PLUS_PLUS] = ACTIONS(872),
    [anon_sym_sizeof] = ACTIONS(875),
    [anon_sym___alignof__] = ACTIONS(878),
    [anon_sym___alignof] = ACTIONS(878),
    [anon_sym__alignof] = ACTIONS(878),
    [anon_sym_alignof] = ACTIONS(878),
    [anon_sym__Alignof] = ACTIONS(878),
    [anon_sym_offsetof] = ACTIONS(881),
    [anon_sym__Generic] = ACTIONS(884),
    [anon_sym_asm] = ACTIONS(887),
    [anon_sym___asm__] = ACTIONS(887),
    [sym_number_literal] = ACTIONS(890),
    [anon_sym_L_SQUOTE] = ACTIONS(893),
    [anon_sym_u_SQUOTE] = ACTIONS(893),
    [anon_sym_U_SQUOTE] = ACTIONS(893),
    [anon_sym_u8_SQUOTE] = ACTIONS(893),
    [anon_sym_SQUOTE] = ACTIONS(893),
    [anon_sym_L_DQUOTE] = ACTIONS(896),
    [anon_sym_u_DQUOTE] = ACTIONS(896),
    [anon_sym_U_DQUOTE] = ACTIONS(896),
    [anon_sym_u8_DQUOTE] = ACTIONS(896),
    [anon_sym_DQUOTE] = ACTIONS(896),
    [sym_true] = ACTIONS(899),
    [sym_false] = ACTIONS(899),
    [anon_sym_NULL] = ACTIONS(902),
    [anon_sym_nullptr] = ACTIONS(902),
    [sym_comment] = ACTIONS(3),
  },
  [55] = {
    [sym_declaration] = STATE(57),
    [sym_type_definition] = STATE(57),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1158),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(57),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(57),
    [sym_labeled_statement] = STATE(57),
    [sym_expression_statement] = STATE(57),
    [sym_if_statement] = STATE(57),
    [sym_switch_statement] = STATE(57),
    [sym_while_statement] = STATE(57),
    [sym_do_statement] = STATE(57),
    [sym_for_statement] = STATE(57),
    [sym_return_statement] = STATE(57),
    [sym_break_statement] = STATE(57),
    [sym_continue_statement] = STATE(57),
    [sym_goto_statement] = STATE(57),
    [sym_seh_try_statement] = STATE(57),
    [sym_seh_leave_statement] = STATE(57),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(57),
    [sym_identifier] = ACTIONS(1021),
    [aux_sym_preproc_include_token1] = ACTIONS(776),
    [aux_sym_preproc_def_token1] = ACTIONS(776),
    [aux_sym_preproc_if_token1] = ACTIONS(776),
    [aux_sym_preproc_if_token2] = ACTIONS(776),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(776),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(776),
    [sym_preproc_directive] = ACTIONS(776),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym___extension__] = ACTIONS(417),
    [anon_sym_typedef] = ACTIONS(419),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(776),
    [anon_sym___clrcall] = ACTIONS(776),
    [anon_sym___stdcall] = ACTIONS(776),
    [anon_sym___fastcall] = ACTIONS(776),
    [anon_sym___thiscall] = ACTIONS(776),
    [anon_sym___vectorcall] = ACTIONS(776),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_else] = ACTIONS(776),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(776),
    [anon_sym_default] = ACTIONS(776),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [56] = {
    [sym_declaration] = STATE(52),
    [sym_type_definition] = STATE(52),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1144),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(52),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(52),
    [sym_labeled_statement] = STATE(52),
    [sym_expression_statement] = STATE(52),
    [sym_if_statement] = STATE(52),
    [sym_switch_statement] = STATE(52),
    [sym_while_statement] = STATE(52),
    [sym_do_statement] = STATE(52),
    [sym_for_statement] = STATE(52),
    [sym_return_statement] = STATE(52),
    [sym_break_statement] = STATE(52),
    [sym_continue_statement] = STATE(52),
    [sym_goto_statement] = STATE(52),
    [sym_seh_try_statement] = STATE(52),
    [sym_seh_leave_statement] = STATE(52),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(52),
    [sym_identifier] = ACTIONS(969),
    [aux_sym_preproc_include_token1] = ACTIONS(776),
    [aux_sym_preproc_def_token1] = ACTIONS(776),
    [aux_sym_preproc_if_token1] = ACTIONS(776),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(776),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(776),
    [sym_preproc_directive] = ACTIONS(776),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(776),
    [anon_sym___clrcall] = ACTIONS(776),
    [anon_sym___stdcall] = ACTIONS(776),
    [anon_sym___fastcall] = ACTIONS(776),
    [anon_sym___thiscall] = ACTIONS(776),
    [anon_sym___vectorcall] = ACTIONS(776),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(1023),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_else] = ACTIONS(776),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(776),
    [anon_sym_default] = ACTIONS(776),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [57] = {
    [sym_declaration] = STATE(54),
    [sym_type_definition] = STATE(54),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1158),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(54),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(54),
    [sym_labeled_statement] = STATE(54),
    [sym_expression_statement] = STATE(54),
    [sym_if_statement] = STATE(54),
    [sym_switch_statement] = STATE(54),
    [sym_while_statement] = STATE(54),
    [sym_do_statement] = STATE(54),
    [sym_for_statement] = STATE(54),
    [sym_return_statement] = STATE(54),
    [sym_break_statement] = STATE(54),
    [sym_continue_statement] = STATE(54),
    [sym_goto_statement] = STATE(54),
    [sym_seh_try_statement] = STATE(54),
    [sym_seh_leave_statement] = STATE(54),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(54),
    [sym_identifier] = ACTIONS(1021),
    [aux_sym_preproc_include_token1] = ACTIONS(778),
    [aux_sym_preproc_def_token1] = ACTIONS(778),
    [aux_sym_preproc_if_token1] = ACTIONS(778),
    [aux_sym_preproc_if_token2] = ACTIONS(778),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(778),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(778),
    [sym_preproc_directive] = ACTIONS(778),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym___extension__] = ACTIONS(417),
    [anon_sym_typedef] = ACTIONS(419),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(778),
    [anon_sym___clrcall] = ACTIONS(778),
    [anon_sym___stdcall] = ACTIONS(778),
    [anon_sym___fastcall] = ACTIONS(778),
    [anon_sym___thiscall] = ACTIONS(778),
    [anon_sym___vectorcall] = ACTIONS(778),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_else] = ACTIONS(778),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(778),
    [anon_sym_default] = ACTIONS(778),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [58] = {
    [sym_declaration] = STATE(54),
    [sym_type_definition] = STATE(54),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1158),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(54),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(54),
    [sym_labeled_statement] = STATE(54),
    [sym_expression_statement] = STATE(54),
    [sym_if_statement] = STATE(54),
    [sym_switch_statement] = STATE(54),
    [sym_while_statement] = STATE(54),
    [sym_do_statement] = STATE(54),
    [sym_for_statement] = STATE(54),
    [sym_return_statement] = STATE(54),
    [sym_break_statement] = STATE(54),
    [sym_continue_statement] = STATE(54),
    [sym_goto_statement] = STATE(54),
    [sym_seh_try_statement] = STATE(54),
    [sym_seh_leave_statement] = STATE(54),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(54),
    [sym_identifier] = ACTIONS(1021),
    [aux_sym_preproc_include_token1] = ACTIONS(905),
    [aux_sym_preproc_def_token1] = ACTIONS(905),
    [aux_sym_preproc_if_token1] = ACTIONS(905),
    [aux_sym_preproc_if_token2] = ACTIONS(905),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(905),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(905),
    [sym_preproc_directive] = ACTIONS(905),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym___extension__] = ACTIONS(417),
    [anon_sym_typedef] = ACTIONS(419),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(905),
    [anon_sym___clrcall] = ACTIONS(905),
    [anon_sym___stdcall] = ACTIONS(905),
    [anon_sym___fastcall] = ACTIONS(905),
    [anon_sym___thiscall] = ACTIONS(905),
    [anon_sym___vectorcall] = ACTIONS(905),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(905),
    [anon_sym_default] = ACTIONS(905),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [59] = {
    [sym_declaration] = STATE(53),
    [sym_type_definition] = STATE(53),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1144),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(53),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(53),
    [sym_labeled_statement] = STATE(53),
    [sym_expression_statement] = STATE(53),
    [sym_if_statement] = STATE(53),
    [sym_switch_statement] = STATE(53),
    [sym_while_statement] = STATE(53),
    [sym_do_statement] = STATE(53),
    [sym_for_statement] = STATE(53),
    [sym_return_statement] = STATE(53),
    [sym_break_statement] = STATE(53),
    [sym_continue_statement] = STATE(53),
    [sym_goto_statement] = STATE(53),
    [sym_seh_try_statement] = STATE(53),
    [sym_seh_leave_statement] = STATE(53),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(53),
    [sym_identifier] = ACTIONS(969),
    [aux_sym_preproc_include_token1] = ACTIONS(907),
    [aux_sym_preproc_def_token1] = ACTIONS(907),
    [aux_sym_preproc_if_token1] = ACTIONS(907),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(907),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(907),
    [sym_preproc_directive] = ACTIONS(907),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(365),
    [anon_sym_typedef] = ACTIONS(367),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(907),
    [anon_sym___clrcall] = ACTIONS(907),
    [anon_sym___stdcall] = ACTIONS(907),
    [anon_sym___fastcall] = ACTIONS(907),
    [anon_sym___thiscall] = ACTIONS(907),
    [anon_sym___vectorcall] = ACTIONS(907),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_RBRACE] = ACTIONS(1025),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_else] = ACTIONS(907),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(907),
    [anon_sym_default] = ACTIONS(907),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [60] = {
    [sym_declaration] = STATE(60),
    [sym_type_definition] = STATE(60),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(60),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(60),
    [sym_labeled_statement] = STATE(60),
    [sym_expression_statement] = STATE(60),
    [sym_if_statement] = STATE(60),
    [sym_switch_statement] = STATE(60),
    [sym_while_statement] = STATE(60),
    [sym_do_statement] = STATE(60),
    [sym_for_statement] = STATE(60),
    [sym_return_statement] = STATE(60),
    [sym_break_statement] = STATE(60),
    [sym_continue_statement] = STATE(60),
    [sym_goto_statement] = STATE(60),
    [sym_seh_try_statement] = STATE(60),
    [sym_seh_leave_statement] = STATE(60),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(60),
    [ts_builtin_sym_end] = ACTIONS(934),
    [sym_identifier] = ACTIONS(1027),
    [aux_sym_preproc_include_token1] = ACTIONS(783),
    [aux_sym_preproc_def_token1] = ACTIONS(783),
    [aux_sym_preproc_if_token1] = ACTIONS(783),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(783),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(783),
    [sym_preproc_directive] = ACTIONS(783),
    [anon_sym_LPAREN2] = ACTIONS(785),
    [anon_sym_BANG] = ACTIONS(788),
    [anon_sym_TILDE] = ACTIONS(788),
    [anon_sym_DASH] = ACTIONS(791),
    [anon_sym_PLUS] = ACTIONS(791),
    [anon_sym_STAR] = ACTIONS(794),
    [anon_sym_AMP] = ACTIONS(794),
    [anon_sym_SEMI] = ACTIONS(1030),
    [anon_sym___extension__] = ACTIONS(1033),
    [anon_sym_typedef] = ACTIONS(1036),
    [anon_sym_extern] = ACTIONS(806),
    [anon_sym___attribute__] = ACTIONS(809),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(812),
    [anon_sym___declspec] = ACTIONS(815),
    [anon_sym___cdecl] = ACTIONS(783),
    [anon_sym___clrcall] = ACTIONS(783),
    [anon_sym___stdcall] = ACTIONS(783),
    [anon_sym___fastcall] = ACTIONS(783),
    [anon_sym___thiscall] = ACTIONS(783),
    [anon_sym___vectorcall] = ACTIONS(783),
    [anon_sym_LBRACE] = ACTIONS(1039),
    [anon_sym_signed] = ACTIONS(821),
    [anon_sym_unsigned] = ACTIONS(821),
    [anon_sym_long] = ACTIONS(821),
    [anon_sym_short] = ACTIONS(821),
    [anon_sym_static] = ACTIONS(806),
    [anon_sym_auto] = ACTIONS(806),
    [anon_sym_register] = ACTIONS(806),
    [anon_sym_inline] = ACTIONS(806),
    [anon_sym___inline] = ACTIONS(806),
    [anon_sym___inline__] = ACTIONS(806),
    [anon_sym___forceinline] = ACTIONS(806),
    [anon_sym_thread_local] = ACTIONS(806),
    [anon_sym___thread] = ACTIONS(806),
    [anon_sym_const] = ACTIONS(824),
    [anon_sym_constexpr] = ACTIONS(824),
    [anon_sym_volatile] = ACTIONS(824),
    [anon_sym_restrict] = ACTIONS(824),
    [anon_sym___restrict__] = ACTIONS(824),
    [anon_sym__Atomic] = ACTIONS(824),
    [anon_sym__Noreturn] = ACTIONS(824),
    [anon_sym_noreturn] = ACTIONS(824),
    [sym_primitive_type] = ACTIONS(827),
    [anon_sym_enum] = ACTIONS(830),
    [anon_sym_struct] = ACTIONS(833),
    [anon_sym_union] = ACTIONS(836),
    [anon_sym_if] = ACTIONS(1042),
    [anon_sym_else] = ACTIONS(783),
    [anon_sym_switch] = ACTIONS(1045),
    [anon_sym_case] = ACTIONS(783),
    [anon_sym_default] = ACTIONS(783),
    [anon_sym_while] = ACTIONS(1048),
    [anon_sym_do] = ACTIONS(1051),
    [anon_sym_for] = ACTIONS(1054),
    [anon_sym_return] = ACTIONS(1057),
    [anon_sym_break] = ACTIONS(1060),
    [anon_sym_continue] = ACTIONS(1063),
    [anon_sym_goto] = ACTIONS(1066),
    [anon_sym___try] = ACTIONS(1069),
    [anon_sym___leave] = ACTIONS(1072),
    [anon_sym_DASH_DASH] = ACTIONS(872),
    [anon_sym_PLUS_PLUS] = ACTIONS(872),
    [anon_sym_sizeof] = ACTIONS(875),
    [anon_sym___alignof__] = ACTIONS(878),
    [anon_sym___alignof] = ACTIONS(878),
    [anon_sym__alignof] = ACTIONS(878),
    [anon_sym_alignof] = ACTIONS(878),
    [anon_sym__Alignof] = ACTIONS(878),
    [anon_sym_offsetof] = ACTIONS(881),
    [anon_sym__Generic] = ACTIONS(884),
    [anon_sym_asm] = ACTIONS(887),
    [anon_sym___asm__] = ACTIONS(887),
    [sym_number_literal] = ACTIONS(890),
    [anon_sym_L_SQUOTE] = ACTIONS(893),
    [anon_sym_u_SQUOTE] = ACTIONS(893),
    [anon_sym_U_SQUOTE] = ACTIONS(893),
    [anon_sym_u8_SQUOTE] = ACTIONS(893),
    [anon_sym_SQUOTE] = ACTIONS(893),
    [anon_sym_L_DQUOTE] = ACTIONS(896),
    [anon_sym_u_DQUOTE] = ACTIONS(896),
    [anon_sym_U_DQUOTE] = ACTIONS(896),
    [anon_sym_u8_DQUOTE] = ACTIONS(896),
    [anon_sym_DQUOTE] = ACTIONS(896),
    [sym_true] = ACTIONS(899),
    [sym_false] = ACTIONS(899),
    [anon_sym_NULL] = ACTIONS(902),
    [anon_sym_nullptr] = ACTIONS(902),
    [sym_comment] = ACTIONS(3),
  },
  [61] = {
    [sym_declaration] = STATE(62),
    [sym_type_definition] = STATE(62),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(62),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(62),
    [sym_labeled_statement] = STATE(62),
    [sym_expression_statement] = STATE(62),
    [sym_if_statement] = STATE(62),
    [sym_switch_statement] = STATE(62),
    [sym_while_statement] = STATE(62),
    [sym_do_statement] = STATE(62),
    [sym_for_statement] = STATE(62),
    [sym_return_statement] = STATE(62),
    [sym_break_statement] = STATE(62),
    [sym_continue_statement] = STATE(62),
    [sym_goto_statement] = STATE(62),
    [sym_seh_try_statement] = STATE(62),
    [sym_seh_leave_statement] = STATE(62),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(62),
    [ts_builtin_sym_end] = ACTIONS(1023),
    [sym_identifier] = ACTIONS(911),
    [aux_sym_preproc_include_token1] = ACTIONS(776),
    [aux_sym_preproc_def_token1] = ACTIONS(776),
    [aux_sym_preproc_if_token1] = ACTIONS(776),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(776),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(776),
    [sym_preproc_directive] = ACTIONS(776),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(776),
    [anon_sym___clrcall] = ACTIONS(776),
    [anon_sym___stdcall] = ACTIONS(776),
    [anon_sym___fastcall] = ACTIONS(776),
    [anon_sym___thiscall] = ACTIONS(776),
    [anon_sym___vectorcall] = ACTIONS(776),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_else] = ACTIONS(776),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(776),
    [anon_sym_default] = ACTIONS(776),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [62] = {
    [sym_declaration] = STATE(60),
    [sym_type_definition] = STATE(60),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(60),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(60),
    [sym_labeled_statement] = STATE(60),
    [sym_expression_statement] = STATE(60),
    [sym_if_statement] = STATE(60),
    [sym_switch_statement] = STATE(60),
    [sym_while_statement] = STATE(60),
    [sym_do_statement] = STATE(60),
    [sym_for_statement] = STATE(60),
    [sym_return_statement] = STATE(60),
    [sym_break_statement] = STATE(60),
    [sym_continue_statement] = STATE(60),
    [sym_goto_statement] = STATE(60),
    [sym_seh_try_statement] = STATE(60),
    [sym_seh_leave_statement] = STATE(60),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(60),
    [ts_builtin_sym_end] = ACTIONS(971),
    [sym_identifier] = ACTIONS(911),
    [aux_sym_preproc_include_token1] = ACTIONS(778),
    [aux_sym_preproc_def_token1] = ACTIONS(778),
    [aux_sym_preproc_if_token1] = ACTIONS(778),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(778),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(778),
    [sym_preproc_directive] = ACTIONS(778),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(778),
    [anon_sym___clrcall] = ACTIONS(778),
    [anon_sym___stdcall] = ACTIONS(778),
    [anon_sym___fastcall] = ACTIONS(778),
    [anon_sym___thiscall] = ACTIONS(778),
    [anon_sym___vectorcall] = ACTIONS(778),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_else] = ACTIONS(778),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(778),
    [anon_sym_default] = ACTIONS(778),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [63] = {
    [sym_declaration] = STATE(58),
    [sym_type_definition] = STATE(58),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1158),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(58),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(58),
    [sym_labeled_statement] = STATE(58),
    [sym_expression_statement] = STATE(58),
    [sym_if_statement] = STATE(58),
    [sym_switch_statement] = STATE(58),
    [sym_while_statement] = STATE(58),
    [sym_do_statement] = STATE(58),
    [sym_for_statement] = STATE(58),
    [sym_return_statement] = STATE(58),
    [sym_break_statement] = STATE(58),
    [sym_continue_statement] = STATE(58),
    [sym_goto_statement] = STATE(58),
    [sym_seh_try_statement] = STATE(58),
    [sym_seh_leave_statement] = STATE(58),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(58),
    [sym_identifier] = ACTIONS(1021),
    [aux_sym_preproc_include_token1] = ACTIONS(907),
    [aux_sym_preproc_def_token1] = ACTIONS(907),
    [aux_sym_preproc_if_token1] = ACTIONS(907),
    [aux_sym_preproc_if_token2] = ACTIONS(907),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(907),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(907),
    [sym_preproc_directive] = ACTIONS(907),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym___extension__] = ACTIONS(417),
    [anon_sym_typedef] = ACTIONS(419),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(907),
    [anon_sym___clrcall] = ACTIONS(907),
    [anon_sym___stdcall] = ACTIONS(907),
    [anon_sym___fastcall] = ACTIONS(907),
    [anon_sym___thiscall] = ACTIONS(907),
    [anon_sym___vectorcall] = ACTIONS(907),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_else] = ACTIONS(907),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(907),
    [anon_sym_default] = ACTIONS(907),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [64] = {
    [sym_declaration] = STATE(50),
    [sym_type_definition] = STATE(50),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(50),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(50),
    [sym_labeled_statement] = STATE(50),
    [sym_expression_statement] = STATE(50),
    [sym_if_statement] = STATE(50),
    [sym_switch_statement] = STATE(50),
    [sym_while_statement] = STATE(50),
    [sym_do_statement] = STATE(50),
    [sym_for_statement] = STATE(50),
    [sym_return_statement] = STATE(50),
    [sym_break_statement] = STATE(50),
    [sym_continue_statement] = STATE(50),
    [sym_goto_statement] = STATE(50),
    [sym_seh_try_statement] = STATE(50),
    [sym_seh_leave_statement] = STATE(50),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(50),
    [ts_builtin_sym_end] = ACTIONS(1025),
    [sym_identifier] = ACTIONS(911),
    [aux_sym_preproc_include_token1] = ACTIONS(907),
    [aux_sym_preproc_def_token1] = ACTIONS(907),
    [aux_sym_preproc_if_token1] = ACTIONS(907),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(907),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(907),
    [sym_preproc_directive] = ACTIONS(907),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(907),
    [anon_sym___clrcall] = ACTIONS(907),
    [anon_sym___stdcall] = ACTIONS(907),
    [anon_sym___fastcall] = ACTIONS(907),
    [anon_sym___thiscall] = ACTIONS(907),
    [anon_sym___vectorcall] = ACTIONS(907),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_else] = ACTIONS(907),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(907),
    [anon_sym_default] = ACTIONS(907),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [65] = {
    [sym_declaration] = STATE(67),
    [sym_type_definition] = STATE(67),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(67),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(67),
    [sym_labeled_statement] = STATE(67),
    [sym_expression_statement] = STATE(67),
    [sym_if_statement] = STATE(67),
    [sym_switch_statement] = STATE(67),
    [sym_while_statement] = STATE(67),
    [sym_do_statement] = STATE(67),
    [sym_for_statement] = STATE(67),
    [sym_return_statement] = STATE(67),
    [sym_break_statement] = STATE(67),
    [sym_continue_statement] = STATE(67),
    [sym_goto_statement] = STATE(67),
    [sym_seh_try_statement] = STATE(67),
    [sym_seh_leave_statement] = STATE(67),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(67),
    [sym_identifier] = ACTIONS(1075),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_else] = ACTIONS(905),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [66] = {
    [sym_declaration] = STATE(67),
    [sym_type_definition] = STATE(67),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(67),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(67),
    [sym_labeled_statement] = STATE(67),
    [sym_expression_statement] = STATE(67),
    [sym_if_statement] = STATE(67),
    [sym_switch_statement] = STATE(67),
    [sym_while_statement] = STATE(67),
    [sym_do_statement] = STATE(67),
    [sym_for_statement] = STATE(67),
    [sym_return_statement] = STATE(67),
    [sym_break_statement] = STATE(67),
    [sym_continue_statement] = STATE(67),
    [sym_goto_statement] = STATE(67),
    [sym_seh_try_statement] = STATE(67),
    [sym_seh_leave_statement] = STATE(67),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(67),
    [sym_identifier] = ACTIONS(1075),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_else] = ACTIONS(778),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [67] = {
    [sym_declaration] = STATE(67),
    [sym_type_definition] = STATE(67),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(67),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(67),
    [sym_labeled_statement] = STATE(67),
    [sym_expression_statement] = STATE(67),
    [sym_if_statement] = STATE(67),
    [sym_switch_statement] = STATE(67),
    [sym_while_statement] = STATE(67),
    [sym_do_statement] = STATE(67),
    [sym_for_statement] = STATE(67),
    [sym_return_statement] = STATE(67),
    [sym_break_statement] = STATE(67),
    [sym_continue_statement] = STATE(67),
    [sym_goto_statement] = STATE(67),
    [sym_seh_try_statement] = STATE(67),
    [sym_seh_leave_statement] = STATE(67),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(67),
    [sym_identifier] = ACTIONS(1085),
    [anon_sym_LPAREN2] = ACTIONS(785),
    [anon_sym_BANG] = ACTIONS(788),
    [anon_sym_TILDE] = ACTIONS(788),
    [anon_sym_DASH] = ACTIONS(791),
    [anon_sym_PLUS] = ACTIONS(791),
    [anon_sym_STAR] = ACTIONS(794),
    [anon_sym_AMP] = ACTIONS(794),
    [anon_sym_SEMI] = ACTIONS(922),
    [anon_sym___extension__] = ACTIONS(1033),
    [anon_sym_typedef] = ACTIONS(1036),
    [anon_sym_extern] = ACTIONS(806),
    [anon_sym___attribute__] = ACTIONS(809),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(812),
    [anon_sym___declspec] = ACTIONS(815),
    [anon_sym_LBRACE] = ACTIONS(1039),
    [anon_sym_signed] = ACTIONS(821),
    [anon_sym_unsigned] = ACTIONS(821),
    [anon_sym_long] = ACTIONS(821),
    [anon_sym_short] = ACTIONS(821),
    [anon_sym_static] = ACTIONS(806),
    [anon_sym_auto] = ACTIONS(806),
    [anon_sym_register] = ACTIONS(806),
    [anon_sym_inline] = ACTIONS(806),
    [anon_sym___inline] = ACTIONS(806),
    [anon_sym___inline__] = ACTIONS(806),
    [anon_sym___forceinline] = ACTIONS(806),
    [anon_sym_thread_local] = ACTIONS(806),
    [anon_sym___thread] = ACTIONS(806),
    [anon_sym_const] = ACTIONS(824),
    [anon_sym_constexpr] = ACTIONS(824),
    [anon_sym_volatile] = ACTIONS(824),
    [anon_sym_restrict] = ACTIONS(824),
    [anon_sym___restrict__] = ACTIONS(824),
    [anon_sym__Atomic] = ACTIONS(824),
    [anon_sym__Noreturn] = ACTIONS(824),
    [anon_sym_noreturn] = ACTIONS(824),
    [sym_primitive_type] = ACTIONS(827),
    [anon_sym_enum] = ACTIONS(830),
    [anon_sym_struct] = ACTIONS(833),
    [anon_sym_union] = ACTIONS(836),
    [anon_sym_if] = ACTIONS(1088),
    [anon_sym_else] = ACTIONS(783),
    [anon_sym_switch] = ACTIONS(1045),
    [anon_sym_while] = ACTIONS(1091),
    [anon_sym_do] = ACTIONS(1051),
    [anon_sym_for] = ACTIONS(1094),
    [anon_sym_return] = ACTIONS(1057),
    [anon_sym_break] = ACTIONS(1060),
    [anon_sym_continue] = ACTIONS(1063),
    [anon_sym_goto] = ACTIONS(1066),
    [anon_sym___try] = ACTIONS(1097),
    [anon_sym___leave] = ACTIONS(966),
    [anon_sym_DASH_DASH] = ACTIONS(872),
    [anon_sym_PLUS_PLUS] = ACTIONS(872),
    [anon_sym_sizeof] = ACTIONS(875),
    [anon_sym___alignof__] = ACTIONS(878),
    [anon_sym___alignof] = ACTIONS(878),
    [anon_sym__alignof] = ACTIONS(878),
    [anon_sym_alignof] = ACTIONS(878),
    [anon_sym__Alignof] = ACTIONS(878),
    [anon_sym_offsetof] = ACTIONS(881),
    [anon_sym__Generic] = ACTIONS(884),
    [anon_sym_asm] = ACTIONS(887),
    [anon_sym___asm__] = ACTIONS(887),
    [sym_number_literal] = ACTIONS(890),
    [anon_sym_L_SQUOTE] = ACTIONS(893),
    [anon_sym_u_SQUOTE] = ACTIONS(893),
    [anon_sym_U_SQUOTE] = ACTIONS(893),
    [anon_sym_u8_SQUOTE] = ACTIONS(893),
    [anon_sym_SQUOTE] = ACTIONS(893),
    [anon_sym_L_DQUOTE] = ACTIONS(896),
    [anon_sym_u_DQUOTE] = ACTIONS(896),
    [anon_sym_U_DQUOTE] = ACTIONS(896),
    [anon_sym_u8_DQUOTE] = ACTIONS(896),
    [anon_sym_DQUOTE] = ACTIONS(896),
    [sym_true] = ACTIONS(899),
    [sym_false] = ACTIONS(899),
    [anon_sym_NULL] = ACTIONS(902),
    [anon_sym_nullptr] = ACTIONS(902),
    [sym_comment] = ACTIONS(3),
  },
  [68] = {
    [sym_declaration] = STATE(66),
    [sym_type_definition] = STATE(66),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(66),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(66),
    [sym_labeled_statement] = STATE(66),
    [sym_expression_statement] = STATE(66),
    [sym_if_statement] = STATE(66),
    [sym_switch_statement] = STATE(66),
    [sym_while_statement] = STATE(66),
    [sym_do_statement] = STATE(66),
    [sym_for_statement] = STATE(66),
    [sym_return_statement] = STATE(66),
    [sym_break_statement] = STATE(66),
    [sym_continue_statement] = STATE(66),
    [sym_goto_statement] = STATE(66),
    [sym_seh_try_statement] = STATE(66),
    [sym_seh_leave_statement] = STATE(66),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(66),
    [sym_identifier] = ACTIONS(1075),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_else] = ACTIONS(776),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [69] = {
    [sym_declaration] = STATE(65),
    [sym_type_definition] = STATE(65),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1145),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(458),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_compound_statement] = STATE(65),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_attributed_statement] = STATE(65),
    [sym_labeled_statement] = STATE(65),
    [sym_expression_statement] = STATE(65),
    [sym_if_statement] = STATE(65),
    [sym_switch_statement] = STATE(65),
    [sym_while_statement] = STATE(65),
    [sym_do_statement] = STATE(65),
    [sym_for_statement] = STATE(65),
    [sym_return_statement] = STATE(65),
    [sym_break_statement] = STATE(65),
    [sym_continue_statement] = STATE(65),
    [sym_goto_statement] = STATE(65),
    [sym_seh_try_statement] = STATE(65),
    [sym_seh_leave_statement] = STATE(65),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [aux_sym_case_statement_repeat1] = STATE(65),
    [sym_identifier] = ACTIONS(1075),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym___extension__] = ACTIONS(27),
    [anon_sym_typedef] = ACTIONS(29),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(35),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_else] = ACTIONS(907),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [70] = {
    [sym_declaration] = STATE(505),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1147),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__for_statement_body] = STATE(1886),
    [sym__expression] = STATE(1039),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1955),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1100),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(1102),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [71] = {
    [sym_declaration] = STATE(505),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1147),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__for_statement_body] = STATE(1940),
    [sym__expression] = STATE(1039),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1955),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1100),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(1102),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [72] = {
    [sym_declaration] = STATE(505),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1147),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__for_statement_body] = STATE(1951),
    [sym__expression] = STATE(1039),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1955),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1100),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(1102),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [73] = {
    [sym_declaration] = STATE(505),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1147),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__for_statement_body] = STATE(1972),
    [sym__expression] = STATE(1039),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1955),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1100),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(1102),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [74] = {
    [sym_declaration] = STATE(505),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1147),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__for_statement_body] = STATE(1882),
    [sym__expression] = STATE(1039),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1955),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1100),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(1102),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [75] = {
    [sym_else_clause] = STATE(87),
    [sym_identifier] = ACTIONS(1106),
    [aux_sym_preproc_include_token1] = ACTIONS(1106),
    [aux_sym_preproc_def_token1] = ACTIONS(1106),
    [aux_sym_preproc_if_token1] = ACTIONS(1106),
    [aux_sym_preproc_if_token2] = ACTIONS(1106),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1106),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1106),
    [aux_sym_preproc_else_token1] = ACTIONS(1106),
    [aux_sym_preproc_elif_token1] = ACTIONS(1106),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1106),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1106),
    [sym_preproc_directive] = ACTIONS(1106),
    [anon_sym_LPAREN2] = ACTIONS(1108),
    [anon_sym_BANG] = ACTIONS(1108),
    [anon_sym_TILDE] = ACTIONS(1108),
    [anon_sym_DASH] = ACTIONS(1106),
    [anon_sym_PLUS] = ACTIONS(1106),
    [anon_sym_STAR] = ACTIONS(1108),
    [anon_sym_AMP] = ACTIONS(1108),
    [anon_sym_SEMI] = ACTIONS(1108),
    [anon_sym___extension__] = ACTIONS(1106),
    [anon_sym_typedef] = ACTIONS(1106),
    [anon_sym_extern] = ACTIONS(1106),
    [anon_sym___attribute__] = ACTIONS(1106),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1108),
    [anon_sym___declspec] = ACTIONS(1106),
    [anon_sym___cdecl] = ACTIONS(1106),
    [anon_sym___clrcall] = ACTIONS(1106),
    [anon_sym___stdcall] = ACTIONS(1106),
    [anon_sym___fastcall] = ACTIONS(1106),
    [anon_sym___thiscall] = ACTIONS(1106),
    [anon_sym___vectorcall] = ACTIONS(1106),
    [anon_sym_LBRACE] = ACTIONS(1108),
    [anon_sym_signed] = ACTIONS(1106),
    [anon_sym_unsigned] = ACTIONS(1106),
    [anon_sym_long] = ACTIONS(1106),
    [anon_sym_short] = ACTIONS(1106),
    [anon_sym_static] = ACTIONS(1106),
    [anon_sym_auto] = ACTIONS(1106),
    [anon_sym_register] = ACTIONS(1106),
    [anon_sym_inline] = ACTIONS(1106),
    [anon_sym___inline] = ACTIONS(1106),
    [anon_sym___inline__] = ACTIONS(1106),
    [anon_sym___forceinline] = ACTIONS(1106),
    [anon_sym_thread_local] = ACTIONS(1106),
    [anon_sym___thread] = ACTIONS(1106),
    [anon_sym_const] = ACTIONS(1106),
    [anon_sym_constexpr] = ACTIONS(1106),
    [anon_sym_volatile] = ACTIONS(1106),
    [anon_sym_restrict] = ACTIONS(1106),
    [anon_sym___restrict__] = ACTIONS(1106),
    [anon_sym__Atomic] = ACTIONS(1106),
    [anon_sym__Noreturn] = ACTIONS(1106),
    [anon_sym_noreturn] = ACTIONS(1106),
    [sym_primitive_type] = ACTIONS(1106),
    [anon_sym_enum] = ACTIONS(1106),
    [anon_sym_struct] = ACTIONS(1106),
    [anon_sym_union] = ACTIONS(1106),
    [anon_sym_if] = ACTIONS(1106),
    [anon_sym_else] = ACTIONS(1110),
    [anon_sym_switch] = ACTIONS(1106),
    [anon_sym_case] = ACTIONS(1106),
    [anon_sym_default] = ACTIONS(1106),
    [anon_sym_while] = ACTIONS(1106),
    [anon_sym_do] = ACTIONS(1106),
    [anon_sym_for] = ACTIONS(1106),
    [anon_sym_return] = ACTIONS(1106),
    [anon_sym_break] = ACTIONS(1106),
    [anon_sym_continue] = ACTIONS(1106),
    [anon_sym_goto] = ACTIONS(1106),
    [anon_sym___try] = ACTIONS(1106),
    [anon_sym___leave] = ACTIONS(1106),
    [anon_sym_DASH_DASH] = ACTIONS(1108),
    [anon_sym_PLUS_PLUS] = ACTIONS(1108),
    [anon_sym_sizeof] = ACTIONS(1106),
    [anon_sym___alignof__] = ACTIONS(1106),
    [anon_sym___alignof] = ACTIONS(1106),
    [anon_sym__alignof] = ACTIONS(1106),
    [anon_sym_alignof] = ACTIONS(1106),
    [anon_sym__Alignof] = ACTIONS(1106),
    [anon_sym_offsetof] = ACTIONS(1106),
    [anon_sym__Generic] = ACTIONS(1106),
    [anon_sym_asm] = ACTIONS(1106),
    [anon_sym___asm__] = ACTIONS(1106),
    [sym_number_literal] = ACTIONS(1108),
    [anon_sym_L_SQUOTE] = ACTIONS(1108),
    [anon_sym_u_SQUOTE] = ACTIONS(1108),
    [anon_sym_U_SQUOTE] = ACTIONS(1108),
    [anon_sym_u8_SQUOTE] = ACTIONS(1108),
    [anon_sym_SQUOTE] = ACTIONS(1108),
    [anon_sym_L_DQUOTE] = ACTIONS(1108),
    [anon_sym_u_DQUOTE] = ACTIONS(1108),
    [anon_sym_U_DQUOTE] = ACTIONS(1108),
    [anon_sym_u8_DQUOTE] = ACTIONS(1108),
    [anon_sym_DQUOTE] = ACTIONS(1108),
    [sym_true] = ACTIONS(1106),
    [sym_false] = ACTIONS(1106),
    [anon_sym_NULL] = ACTIONS(1106),
    [anon_sym_nullptr] = ACTIONS(1106),
    [sym_comment] = ACTIONS(3),
  },
  [76] = {
    [sym_identifier] = ACTIONS(1112),
    [aux_sym_preproc_include_token1] = ACTIONS(1112),
    [aux_sym_preproc_def_token1] = ACTIONS(1112),
    [aux_sym_preproc_if_token1] = ACTIONS(1112),
    [aux_sym_preproc_if_token2] = ACTIONS(1112),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1112),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1112),
    [aux_sym_preproc_else_token1] = ACTIONS(1112),
    [aux_sym_preproc_elif_token1] = ACTIONS(1112),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1112),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1112),
    [sym_preproc_directive] = ACTIONS(1112),
    [anon_sym_LPAREN2] = ACTIONS(1114),
    [anon_sym_BANG] = ACTIONS(1114),
    [anon_sym_TILDE] = ACTIONS(1114),
    [anon_sym_DASH] = ACTIONS(1112),
    [anon_sym_PLUS] = ACTIONS(1112),
    [anon_sym_STAR] = ACTIONS(1114),
    [anon_sym_AMP] = ACTIONS(1114),
    [anon_sym_SEMI] = ACTIONS(1114),
    [anon_sym___extension__] = ACTIONS(1112),
    [anon_sym_typedef] = ACTIONS(1112),
    [anon_sym_extern] = ACTIONS(1112),
    [anon_sym___attribute__] = ACTIONS(1112),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1114),
    [anon_sym___declspec] = ACTIONS(1112),
    [anon_sym___cdecl] = ACTIONS(1112),
    [anon_sym___clrcall] = ACTIONS(1112),
    [anon_sym___stdcall] = ACTIONS(1112),
    [anon_sym___fastcall] = ACTIONS(1112),
    [anon_sym___thiscall] = ACTIONS(1112),
    [anon_sym___vectorcall] = ACTIONS(1112),
    [anon_sym_LBRACE] = ACTIONS(1114),
    [anon_sym_signed] = ACTIONS(1112),
    [anon_sym_unsigned] = ACTIONS(1112),
    [anon_sym_long] = ACTIONS(1112),
    [anon_sym_short] = ACTIONS(1112),
    [anon_sym_static] = ACTIONS(1112),
    [anon_sym_auto] = ACTIONS(1112),
    [anon_sym_register] = ACTIONS(1112),
    [anon_sym_inline] = ACTIONS(1112),
    [anon_sym___inline] = ACTIONS(1112),
    [anon_sym___inline__] = ACTIONS(1112),
    [anon_sym___forceinline] = ACTIONS(1112),
    [anon_sym_thread_local] = ACTIONS(1112),
    [anon_sym___thread] = ACTIONS(1112),
    [anon_sym_const] = ACTIONS(1112),
    [anon_sym_constexpr] = ACTIONS(1112),
    [anon_sym_volatile] = ACTIONS(1112),
    [anon_sym_restrict] = ACTIONS(1112),
    [anon_sym___restrict__] = ACTIONS(1112),
    [anon_sym__Atomic] = ACTIONS(1112),
    [anon_sym__Noreturn] = ACTIONS(1112),
    [anon_sym_noreturn] = ACTIONS(1112),
    [sym_primitive_type] = ACTIONS(1112),
    [anon_sym_enum] = ACTIONS(1112),
    [anon_sym_struct] = ACTIONS(1112),
    [anon_sym_union] = ACTIONS(1112),
    [anon_sym_if] = ACTIONS(1112),
    [anon_sym_else] = ACTIONS(1112),
    [anon_sym_switch] = ACTIONS(1112),
    [anon_sym_case] = ACTIONS(1112),
    [anon_sym_default] = ACTIONS(1112),
    [anon_sym_while] = ACTIONS(1112),
    [anon_sym_do] = ACTIONS(1112),
    [anon_sym_for] = ACTIONS(1112),
    [anon_sym_return] = ACTIONS(1112),
    [anon_sym_break] = ACTIONS(1112),
    [anon_sym_continue] = ACTIONS(1112),
    [anon_sym_goto] = ACTIONS(1112),
    [anon_sym___try] = ACTIONS(1112),
    [anon_sym___leave] = ACTIONS(1112),
    [anon_sym_DASH_DASH] = ACTIONS(1114),
    [anon_sym_PLUS_PLUS] = ACTIONS(1114),
    [anon_sym_sizeof] = ACTIONS(1112),
    [anon_sym___alignof__] = ACTIONS(1112),
    [anon_sym___alignof] = ACTIONS(1112),
    [anon_sym__alignof] = ACTIONS(1112),
    [anon_sym_alignof] = ACTIONS(1112),
    [anon_sym__Alignof] = ACTIONS(1112),
    [anon_sym_offsetof] = ACTIONS(1112),
    [anon_sym__Generic] = ACTIONS(1112),
    [anon_sym_asm] = ACTIONS(1112),
    [anon_sym___asm__] = ACTIONS(1112),
    [sym_number_literal] = ACTIONS(1114),
    [anon_sym_L_SQUOTE] = ACTIONS(1114),
    [anon_sym_u_SQUOTE] = ACTIONS(1114),
    [anon_sym_U_SQUOTE] = ACTIONS(1114),
    [anon_sym_u8_SQUOTE] = ACTIONS(1114),
    [anon_sym_SQUOTE] = ACTIONS(1114),
    [anon_sym_L_DQUOTE] = ACTIONS(1114),
    [anon_sym_u_DQUOTE] = ACTIONS(1114),
    [anon_sym_U_DQUOTE] = ACTIONS(1114),
    [anon_sym_u8_DQUOTE] = ACTIONS(1114),
    [anon_sym_DQUOTE] = ACTIONS(1114),
    [sym_true] = ACTIONS(1112),
    [sym_false] = ACTIONS(1112),
    [anon_sym_NULL] = ACTIONS(1112),
    [anon_sym_nullptr] = ACTIONS(1112),
    [sym_comment] = ACTIONS(3),
  },
  [77] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [78] = {
    [sym_identifier] = ACTIONS(1120),
    [aux_sym_preproc_include_token1] = ACTIONS(1120),
    [aux_sym_preproc_def_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token2] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1120),
    [aux_sym_preproc_else_token1] = ACTIONS(1120),
    [aux_sym_preproc_elif_token1] = ACTIONS(1120),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1120),
    [sym_preproc_directive] = ACTIONS(1120),
    [anon_sym_LPAREN2] = ACTIONS(1122),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_DASH] = ACTIONS(1120),
    [anon_sym_PLUS] = ACTIONS(1120),
    [anon_sym_STAR] = ACTIONS(1122),
    [anon_sym_AMP] = ACTIONS(1122),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym___extension__] = ACTIONS(1120),
    [anon_sym_typedef] = ACTIONS(1120),
    [anon_sym_extern] = ACTIONS(1120),
    [anon_sym___attribute__] = ACTIONS(1120),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1122),
    [anon_sym___declspec] = ACTIONS(1120),
    [anon_sym___cdecl] = ACTIONS(1120),
    [anon_sym___clrcall] = ACTIONS(1120),
    [anon_sym___stdcall] = ACTIONS(1120),
    [anon_sym___fastcall] = ACTIONS(1120),
    [anon_sym___thiscall] = ACTIONS(1120),
    [anon_sym___vectorcall] = ACTIONS(1120),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_signed] = ACTIONS(1120),
    [anon_sym_unsigned] = ACTIONS(1120),
    [anon_sym_long] = ACTIONS(1120),
    [anon_sym_short] = ACTIONS(1120),
    [anon_sym_static] = ACTIONS(1120),
    [anon_sym_auto] = ACTIONS(1120),
    [anon_sym_register] = ACTIONS(1120),
    [anon_sym_inline] = ACTIONS(1120),
    [anon_sym___inline] = ACTIONS(1120),
    [anon_sym___inline__] = ACTIONS(1120),
    [anon_sym___forceinline] = ACTIONS(1120),
    [anon_sym_thread_local] = ACTIONS(1120),
    [anon_sym___thread] = ACTIONS(1120),
    [anon_sym_const] = ACTIONS(1120),
    [anon_sym_constexpr] = ACTIONS(1120),
    [anon_sym_volatile] = ACTIONS(1120),
    [anon_sym_restrict] = ACTIONS(1120),
    [anon_sym___restrict__] = ACTIONS(1120),
    [anon_sym__Atomic] = ACTIONS(1120),
    [anon_sym__Noreturn] = ACTIONS(1120),
    [anon_sym_noreturn] = ACTIONS(1120),
    [sym_primitive_type] = ACTIONS(1120),
    [anon_sym_enum] = ACTIONS(1120),
    [anon_sym_struct] = ACTIONS(1120),
    [anon_sym_union] = ACTIONS(1120),
    [anon_sym_if] = ACTIONS(1120),
    [anon_sym_else] = ACTIONS(1120),
    [anon_sym_switch] = ACTIONS(1120),
    [anon_sym_case] = ACTIONS(1120),
    [anon_sym_default] = ACTIONS(1120),
    [anon_sym_while] = ACTIONS(1120),
    [anon_sym_do] = ACTIONS(1120),
    [anon_sym_for] = ACTIONS(1120),
    [anon_sym_return] = ACTIONS(1120),
    [anon_sym_break] = ACTIONS(1120),
    [anon_sym_continue] = ACTIONS(1120),
    [anon_sym_goto] = ACTIONS(1120),
    [anon_sym___try] = ACTIONS(1120),
    [anon_sym___leave] = ACTIONS(1120),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_sizeof] = ACTIONS(1120),
    [anon_sym___alignof__] = ACTIONS(1120),
    [anon_sym___alignof] = ACTIONS(1120),
    [anon_sym__alignof] = ACTIONS(1120),
    [anon_sym_alignof] = ACTIONS(1120),
    [anon_sym__Alignof] = ACTIONS(1120),
    [anon_sym_offsetof] = ACTIONS(1120),
    [anon_sym__Generic] = ACTIONS(1120),
    [anon_sym_asm] = ACTIONS(1120),
    [anon_sym___asm__] = ACTIONS(1120),
    [sym_number_literal] = ACTIONS(1122),
    [anon_sym_L_SQUOTE] = ACTIONS(1122),
    [anon_sym_u_SQUOTE] = ACTIONS(1122),
    [anon_sym_U_SQUOTE] = ACTIONS(1122),
    [anon_sym_u8_SQUOTE] = ACTIONS(1122),
    [anon_sym_SQUOTE] = ACTIONS(1122),
    [anon_sym_L_DQUOTE] = ACTIONS(1122),
    [anon_sym_u_DQUOTE] = ACTIONS(1122),
    [anon_sym_U_DQUOTE] = ACTIONS(1122),
    [anon_sym_u8_DQUOTE] = ACTIONS(1122),
    [anon_sym_DQUOTE] = ACTIONS(1122),
    [sym_true] = ACTIONS(1120),
    [sym_false] = ACTIONS(1120),
    [anon_sym_NULL] = ACTIONS(1120),
    [anon_sym_nullptr] = ACTIONS(1120),
    [sym_comment] = ACTIONS(3),
  },
  [79] = {
    [sym_identifier] = ACTIONS(1124),
    [aux_sym_preproc_include_token1] = ACTIONS(1124),
    [aux_sym_preproc_def_token1] = ACTIONS(1124),
    [aux_sym_preproc_if_token1] = ACTIONS(1124),
    [aux_sym_preproc_if_token2] = ACTIONS(1124),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1124),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1124),
    [aux_sym_preproc_else_token1] = ACTIONS(1124),
    [aux_sym_preproc_elif_token1] = ACTIONS(1124),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1124),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1124),
    [sym_preproc_directive] = ACTIONS(1124),
    [anon_sym_LPAREN2] = ACTIONS(1126),
    [anon_sym_BANG] = ACTIONS(1126),
    [anon_sym_TILDE] = ACTIONS(1126),
    [anon_sym_DASH] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1124),
    [anon_sym_STAR] = ACTIONS(1126),
    [anon_sym_AMP] = ACTIONS(1126),
    [anon_sym_SEMI] = ACTIONS(1126),
    [anon_sym___extension__] = ACTIONS(1124),
    [anon_sym_typedef] = ACTIONS(1124),
    [anon_sym_extern] = ACTIONS(1124),
    [anon_sym___attribute__] = ACTIONS(1124),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1126),
    [anon_sym___declspec] = ACTIONS(1124),
    [anon_sym___cdecl] = ACTIONS(1124),
    [anon_sym___clrcall] = ACTIONS(1124),
    [anon_sym___stdcall] = ACTIONS(1124),
    [anon_sym___fastcall] = ACTIONS(1124),
    [anon_sym___thiscall] = ACTIONS(1124),
    [anon_sym___vectorcall] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1126),
    [anon_sym_signed] = ACTIONS(1124),
    [anon_sym_unsigned] = ACTIONS(1124),
    [anon_sym_long] = ACTIONS(1124),
    [anon_sym_short] = ACTIONS(1124),
    [anon_sym_static] = ACTIONS(1124),
    [anon_sym_auto] = ACTIONS(1124),
    [anon_sym_register] = ACTIONS(1124),
    [anon_sym_inline] = ACTIONS(1124),
    [anon_sym___inline] = ACTIONS(1124),
    [anon_sym___inline__] = ACTIONS(1124),
    [anon_sym___forceinline] = ACTIONS(1124),
    [anon_sym_thread_local] = ACTIONS(1124),
    [anon_sym___thread] = ACTIONS(1124),
    [anon_sym_const] = ACTIONS(1124),
    [anon_sym_constexpr] = ACTIONS(1124),
    [anon_sym_volatile] = ACTIONS(1124),
    [anon_sym_restrict] = ACTIONS(1124),
    [anon_sym___restrict__] = ACTIONS(1124),
    [anon_sym__Atomic] = ACTIONS(1124),
    [anon_sym__Noreturn] = ACTIONS(1124),
    [anon_sym_noreturn] = ACTIONS(1124),
    [sym_primitive_type] = ACTIONS(1124),
    [anon_sym_enum] = ACTIONS(1124),
    [anon_sym_struct] = ACTIONS(1124),
    [anon_sym_union] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_else] = ACTIONS(1124),
    [anon_sym_switch] = ACTIONS(1124),
    [anon_sym_case] = ACTIONS(1124),
    [anon_sym_default] = ACTIONS(1124),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_do] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_continue] = ACTIONS(1124),
    [anon_sym_goto] = ACTIONS(1124),
    [anon_sym___try] = ACTIONS(1124),
    [anon_sym___leave] = ACTIONS(1124),
    [anon_sym_DASH_DASH] = ACTIONS(1126),
    [anon_sym_PLUS_PLUS] = ACTIONS(1126),
    [anon_sym_sizeof] = ACTIONS(1124),
    [anon_sym___alignof__] = ACTIONS(1124),
    [anon_sym___alignof] = ACTIONS(1124),
    [anon_sym__alignof] = ACTIONS(1124),
    [anon_sym_alignof] = ACTIONS(1124),
    [anon_sym__Alignof] = ACTIONS(1124),
    [anon_sym_offsetof] = ACTIONS(1124),
    [anon_sym__Generic] = ACTIONS(1124),
    [anon_sym_asm] = ACTIONS(1124),
    [anon_sym___asm__] = ACTIONS(1124),
    [sym_number_literal] = ACTIONS(1126),
    [anon_sym_L_SQUOTE] = ACTIONS(1126),
    [anon_sym_u_SQUOTE] = ACTIONS(1126),
    [anon_sym_U_SQUOTE] = ACTIONS(1126),
    [anon_sym_u8_SQUOTE] = ACTIONS(1126),
    [anon_sym_SQUOTE] = ACTIONS(1126),
    [anon_sym_L_DQUOTE] = ACTIONS(1126),
    [anon_sym_u_DQUOTE] = ACTIONS(1126),
    [anon_sym_U_DQUOTE] = ACTIONS(1126),
    [anon_sym_u8_DQUOTE] = ACTIONS(1126),
    [anon_sym_DQUOTE] = ACTIONS(1126),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [anon_sym_NULL] = ACTIONS(1124),
    [anon_sym_nullptr] = ACTIONS(1124),
    [sym_comment] = ACTIONS(3),
  },
  [80] = {
    [sym_identifier] = ACTIONS(1128),
    [aux_sym_preproc_include_token1] = ACTIONS(1128),
    [aux_sym_preproc_def_token1] = ACTIONS(1128),
    [aux_sym_preproc_if_token1] = ACTIONS(1128),
    [aux_sym_preproc_if_token2] = ACTIONS(1128),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1128),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1128),
    [aux_sym_preproc_else_token1] = ACTIONS(1128),
    [aux_sym_preproc_elif_token1] = ACTIONS(1128),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1128),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1128),
    [sym_preproc_directive] = ACTIONS(1128),
    [anon_sym_LPAREN2] = ACTIONS(1130),
    [anon_sym_BANG] = ACTIONS(1130),
    [anon_sym_TILDE] = ACTIONS(1130),
    [anon_sym_DASH] = ACTIONS(1128),
    [anon_sym_PLUS] = ACTIONS(1128),
    [anon_sym_STAR] = ACTIONS(1130),
    [anon_sym_AMP] = ACTIONS(1130),
    [anon_sym_SEMI] = ACTIONS(1130),
    [anon_sym___extension__] = ACTIONS(1128),
    [anon_sym_typedef] = ACTIONS(1128),
    [anon_sym_extern] = ACTIONS(1128),
    [anon_sym___attribute__] = ACTIONS(1128),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1130),
    [anon_sym___declspec] = ACTIONS(1128),
    [anon_sym___cdecl] = ACTIONS(1128),
    [anon_sym___clrcall] = ACTIONS(1128),
    [anon_sym___stdcall] = ACTIONS(1128),
    [anon_sym___fastcall] = ACTIONS(1128),
    [anon_sym___thiscall] = ACTIONS(1128),
    [anon_sym___vectorcall] = ACTIONS(1128),
    [anon_sym_LBRACE] = ACTIONS(1130),
    [anon_sym_signed] = ACTIONS(1128),
    [anon_sym_unsigned] = ACTIONS(1128),
    [anon_sym_long] = ACTIONS(1128),
    [anon_sym_short] = ACTIONS(1128),
    [anon_sym_static] = ACTIONS(1128),
    [anon_sym_auto] = ACTIONS(1128),
    [anon_sym_register] = ACTIONS(1128),
    [anon_sym_inline] = ACTIONS(1128),
    [anon_sym___inline] = ACTIONS(1128),
    [anon_sym___inline__] = ACTIONS(1128),
    [anon_sym___forceinline] = ACTIONS(1128),
    [anon_sym_thread_local] = ACTIONS(1128),
    [anon_sym___thread] = ACTIONS(1128),
    [anon_sym_const] = ACTIONS(1128),
    [anon_sym_constexpr] = ACTIONS(1128),
    [anon_sym_volatile] = ACTIONS(1128),
    [anon_sym_restrict] = ACTIONS(1128),
    [anon_sym___restrict__] = ACTIONS(1128),
    [anon_sym__Atomic] = ACTIONS(1128),
    [anon_sym__Noreturn] = ACTIONS(1128),
    [anon_sym_noreturn] = ACTIONS(1128),
    [sym_primitive_type] = ACTIONS(1128),
    [anon_sym_enum] = ACTIONS(1128),
    [anon_sym_struct] = ACTIONS(1128),
    [anon_sym_union] = ACTIONS(1128),
    [anon_sym_if] = ACTIONS(1128),
    [anon_sym_else] = ACTIONS(1128),
    [anon_sym_switch] = ACTIONS(1128),
    [anon_sym_case] = ACTIONS(1128),
    [anon_sym_default] = ACTIONS(1128),
    [anon_sym_while] = ACTIONS(1128),
    [anon_sym_do] = ACTIONS(1128),
    [anon_sym_for] = ACTIONS(1128),
    [anon_sym_return] = ACTIONS(1128),
    [anon_sym_break] = ACTIONS(1128),
    [anon_sym_continue] = ACTIONS(1128),
    [anon_sym_goto] = ACTIONS(1128),
    [anon_sym___try] = ACTIONS(1128),
    [anon_sym___leave] = ACTIONS(1128),
    [anon_sym_DASH_DASH] = ACTIONS(1130),
    [anon_sym_PLUS_PLUS] = ACTIONS(1130),
    [anon_sym_sizeof] = ACTIONS(1128),
    [anon_sym___alignof__] = ACTIONS(1128),
    [anon_sym___alignof] = ACTIONS(1128),
    [anon_sym__alignof] = ACTIONS(1128),
    [anon_sym_alignof] = ACTIONS(1128),
    [anon_sym__Alignof] = ACTIONS(1128),
    [anon_sym_offsetof] = ACTIONS(1128),
    [anon_sym__Generic] = ACTIONS(1128),
    [anon_sym_asm] = ACTIONS(1128),
    [anon_sym___asm__] = ACTIONS(1128),
    [sym_number_literal] = ACTIONS(1130),
    [anon_sym_L_SQUOTE] = ACTIONS(1130),
    [anon_sym_u_SQUOTE] = ACTIONS(1130),
    [anon_sym_U_SQUOTE] = ACTIONS(1130),
    [anon_sym_u8_SQUOTE] = ACTIONS(1130),
    [anon_sym_SQUOTE] = ACTIONS(1130),
    [anon_sym_L_DQUOTE] = ACTIONS(1130),
    [anon_sym_u_DQUOTE] = ACTIONS(1130),
    [anon_sym_U_DQUOTE] = ACTIONS(1130),
    [anon_sym_u8_DQUOTE] = ACTIONS(1130),
    [anon_sym_DQUOTE] = ACTIONS(1130),
    [sym_true] = ACTIONS(1128),
    [sym_false] = ACTIONS(1128),
    [anon_sym_NULL] = ACTIONS(1128),
    [anon_sym_nullptr] = ACTIONS(1128),
    [sym_comment] = ACTIONS(3),
  },
  [81] = {
    [sym_identifier] = ACTIONS(1132),
    [aux_sym_preproc_include_token1] = ACTIONS(1132),
    [aux_sym_preproc_def_token1] = ACTIONS(1132),
    [aux_sym_preproc_if_token1] = ACTIONS(1132),
    [aux_sym_preproc_if_token2] = ACTIONS(1132),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1132),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1132),
    [aux_sym_preproc_else_token1] = ACTIONS(1132),
    [aux_sym_preproc_elif_token1] = ACTIONS(1132),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1132),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1132),
    [sym_preproc_directive] = ACTIONS(1132),
    [anon_sym_LPAREN2] = ACTIONS(1134),
    [anon_sym_BANG] = ACTIONS(1134),
    [anon_sym_TILDE] = ACTIONS(1134),
    [anon_sym_DASH] = ACTIONS(1132),
    [anon_sym_PLUS] = ACTIONS(1132),
    [anon_sym_STAR] = ACTIONS(1134),
    [anon_sym_AMP] = ACTIONS(1134),
    [anon_sym_SEMI] = ACTIONS(1134),
    [anon_sym___extension__] = ACTIONS(1132),
    [anon_sym_typedef] = ACTIONS(1132),
    [anon_sym_extern] = ACTIONS(1132),
    [anon_sym___attribute__] = ACTIONS(1132),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1134),
    [anon_sym___declspec] = ACTIONS(1132),
    [anon_sym___cdecl] = ACTIONS(1132),
    [anon_sym___clrcall] = ACTIONS(1132),
    [anon_sym___stdcall] = ACTIONS(1132),
    [anon_sym___fastcall] = ACTIONS(1132),
    [anon_sym___thiscall] = ACTIONS(1132),
    [anon_sym___vectorcall] = ACTIONS(1132),
    [anon_sym_LBRACE] = ACTIONS(1134),
    [anon_sym_signed] = ACTIONS(1132),
    [anon_sym_unsigned] = ACTIONS(1132),
    [anon_sym_long] = ACTIONS(1132),
    [anon_sym_short] = ACTIONS(1132),
    [anon_sym_static] = ACTIONS(1132),
    [anon_sym_auto] = ACTIONS(1132),
    [anon_sym_register] = ACTIONS(1132),
    [anon_sym_inline] = ACTIONS(1132),
    [anon_sym___inline] = ACTIONS(1132),
    [anon_sym___inline__] = ACTIONS(1132),
    [anon_sym___forceinline] = ACTIONS(1132),
    [anon_sym_thread_local] = ACTIONS(1132),
    [anon_sym___thread] = ACTIONS(1132),
    [anon_sym_const] = ACTIONS(1132),
    [anon_sym_constexpr] = ACTIONS(1132),
    [anon_sym_volatile] = ACTIONS(1132),
    [anon_sym_restrict] = ACTIONS(1132),
    [anon_sym___restrict__] = ACTIONS(1132),
    [anon_sym__Atomic] = ACTIONS(1132),
    [anon_sym__Noreturn] = ACTIONS(1132),
    [anon_sym_noreturn] = ACTIONS(1132),
    [sym_primitive_type] = ACTIONS(1132),
    [anon_sym_enum] = ACTIONS(1132),
    [anon_sym_struct] = ACTIONS(1132),
    [anon_sym_union] = ACTIONS(1132),
    [anon_sym_if] = ACTIONS(1132),
    [anon_sym_else] = ACTIONS(1132),
    [anon_sym_switch] = ACTIONS(1132),
    [anon_sym_case] = ACTIONS(1132),
    [anon_sym_default] = ACTIONS(1132),
    [anon_sym_while] = ACTIONS(1132),
    [anon_sym_do] = ACTIONS(1132),
    [anon_sym_for] = ACTIONS(1132),
    [anon_sym_return] = ACTIONS(1132),
    [anon_sym_break] = ACTIONS(1132),
    [anon_sym_continue] = ACTIONS(1132),
    [anon_sym_goto] = ACTIONS(1132),
    [anon_sym___try] = ACTIONS(1132),
    [anon_sym___leave] = ACTIONS(1132),
    [anon_sym_DASH_DASH] = ACTIONS(1134),
    [anon_sym_PLUS_PLUS] = ACTIONS(1134),
    [anon_sym_sizeof] = ACTIONS(1132),
    [anon_sym___alignof__] = ACTIONS(1132),
    [anon_sym___alignof] = ACTIONS(1132),
    [anon_sym__alignof] = ACTIONS(1132),
    [anon_sym_alignof] = ACTIONS(1132),
    [anon_sym__Alignof] = ACTIONS(1132),
    [anon_sym_offsetof] = ACTIONS(1132),
    [anon_sym__Generic] = ACTIONS(1132),
    [anon_sym_asm] = ACTIONS(1132),
    [anon_sym___asm__] = ACTIONS(1132),
    [sym_number_literal] = ACTIONS(1134),
    [anon_sym_L_SQUOTE] = ACTIONS(1134),
    [anon_sym_u_SQUOTE] = ACTIONS(1134),
    [anon_sym_U_SQUOTE] = ACTIONS(1134),
    [anon_sym_u8_SQUOTE] = ACTIONS(1134),
    [anon_sym_SQUOTE] = ACTIONS(1134),
    [anon_sym_L_DQUOTE] = ACTIONS(1134),
    [anon_sym_u_DQUOTE] = ACTIONS(1134),
    [anon_sym_U_DQUOTE] = ACTIONS(1134),
    [anon_sym_u8_DQUOTE] = ACTIONS(1134),
    [anon_sym_DQUOTE] = ACTIONS(1134),
    [sym_true] = ACTIONS(1132),
    [sym_false] = ACTIONS(1132),
    [anon_sym_NULL] = ACTIONS(1132),
    [anon_sym_nullptr] = ACTIONS(1132),
    [sym_comment] = ACTIONS(3),
  },
  [82] = {
    [sym_identifier] = ACTIONS(1136),
    [aux_sym_preproc_include_token1] = ACTIONS(1136),
    [aux_sym_preproc_def_token1] = ACTIONS(1136),
    [aux_sym_preproc_if_token1] = ACTIONS(1136),
    [aux_sym_preproc_if_token2] = ACTIONS(1136),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1136),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1136),
    [aux_sym_preproc_else_token1] = ACTIONS(1136),
    [aux_sym_preproc_elif_token1] = ACTIONS(1136),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1136),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1136),
    [sym_preproc_directive] = ACTIONS(1136),
    [anon_sym_LPAREN2] = ACTIONS(1138),
    [anon_sym_BANG] = ACTIONS(1138),
    [anon_sym_TILDE] = ACTIONS(1138),
    [anon_sym_DASH] = ACTIONS(1136),
    [anon_sym_PLUS] = ACTIONS(1136),
    [anon_sym_STAR] = ACTIONS(1138),
    [anon_sym_AMP] = ACTIONS(1138),
    [anon_sym_SEMI] = ACTIONS(1138),
    [anon_sym___extension__] = ACTIONS(1136),
    [anon_sym_typedef] = ACTIONS(1136),
    [anon_sym_extern] = ACTIONS(1136),
    [anon_sym___attribute__] = ACTIONS(1136),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1138),
    [anon_sym___declspec] = ACTIONS(1136),
    [anon_sym___cdecl] = ACTIONS(1136),
    [anon_sym___clrcall] = ACTIONS(1136),
    [anon_sym___stdcall] = ACTIONS(1136),
    [anon_sym___fastcall] = ACTIONS(1136),
    [anon_sym___thiscall] = ACTIONS(1136),
    [anon_sym___vectorcall] = ACTIONS(1136),
    [anon_sym_LBRACE] = ACTIONS(1138),
    [anon_sym_signed] = ACTIONS(1136),
    [anon_sym_unsigned] = ACTIONS(1136),
    [anon_sym_long] = ACTIONS(1136),
    [anon_sym_short] = ACTIONS(1136),
    [anon_sym_static] = ACTIONS(1136),
    [anon_sym_auto] = ACTIONS(1136),
    [anon_sym_register] = ACTIONS(1136),
    [anon_sym_inline] = ACTIONS(1136),
    [anon_sym___inline] = ACTIONS(1136),
    [anon_sym___inline__] = ACTIONS(1136),
    [anon_sym___forceinline] = ACTIONS(1136),
    [anon_sym_thread_local] = ACTIONS(1136),
    [anon_sym___thread] = ACTIONS(1136),
    [anon_sym_const] = ACTIONS(1136),
    [anon_sym_constexpr] = ACTIONS(1136),
    [anon_sym_volatile] = ACTIONS(1136),
    [anon_sym_restrict] = ACTIONS(1136),
    [anon_sym___restrict__] = ACTIONS(1136),
    [anon_sym__Atomic] = ACTIONS(1136),
    [anon_sym__Noreturn] = ACTIONS(1136),
    [anon_sym_noreturn] = ACTIONS(1136),
    [sym_primitive_type] = ACTIONS(1136),
    [anon_sym_enum] = ACTIONS(1136),
    [anon_sym_struct] = ACTIONS(1136),
    [anon_sym_union] = ACTIONS(1136),
    [anon_sym_if] = ACTIONS(1136),
    [anon_sym_else] = ACTIONS(1136),
    [anon_sym_switch] = ACTIONS(1136),
    [anon_sym_case] = ACTIONS(1136),
    [anon_sym_default] = ACTIONS(1136),
    [anon_sym_while] = ACTIONS(1136),
    [anon_sym_do] = ACTIONS(1136),
    [anon_sym_for] = ACTIONS(1136),
    [anon_sym_return] = ACTIONS(1136),
    [anon_sym_break] = ACTIONS(1136),
    [anon_sym_continue] = ACTIONS(1136),
    [anon_sym_goto] = ACTIONS(1136),
    [anon_sym___try] = ACTIONS(1136),
    [anon_sym___leave] = ACTIONS(1136),
    [anon_sym_DASH_DASH] = ACTIONS(1138),
    [anon_sym_PLUS_PLUS] = ACTIONS(1138),
    [anon_sym_sizeof] = ACTIONS(1136),
    [anon_sym___alignof__] = ACTIONS(1136),
    [anon_sym___alignof] = ACTIONS(1136),
    [anon_sym__alignof] = ACTIONS(1136),
    [anon_sym_alignof] = ACTIONS(1136),
    [anon_sym__Alignof] = ACTIONS(1136),
    [anon_sym_offsetof] = ACTIONS(1136),
    [anon_sym__Generic] = ACTIONS(1136),
    [anon_sym_asm] = ACTIONS(1136),
    [anon_sym___asm__] = ACTIONS(1136),
    [sym_number_literal] = ACTIONS(1138),
    [anon_sym_L_SQUOTE] = ACTIONS(1138),
    [anon_sym_u_SQUOTE] = ACTIONS(1138),
    [anon_sym_U_SQUOTE] = ACTIONS(1138),
    [anon_sym_u8_SQUOTE] = ACTIONS(1138),
    [anon_sym_SQUOTE] = ACTIONS(1138),
    [anon_sym_L_DQUOTE] = ACTIONS(1138),
    [anon_sym_u_DQUOTE] = ACTIONS(1138),
    [anon_sym_U_DQUOTE] = ACTIONS(1138),
    [anon_sym_u8_DQUOTE] = ACTIONS(1138),
    [anon_sym_DQUOTE] = ACTIONS(1138),
    [sym_true] = ACTIONS(1136),
    [sym_false] = ACTIONS(1136),
    [anon_sym_NULL] = ACTIONS(1136),
    [anon_sym_nullptr] = ACTIONS(1136),
    [sym_comment] = ACTIONS(3),
  },
  [83] = {
    [sym_identifier] = ACTIONS(1140),
    [aux_sym_preproc_include_token1] = ACTIONS(1140),
    [aux_sym_preproc_def_token1] = ACTIONS(1140),
    [aux_sym_preproc_if_token1] = ACTIONS(1140),
    [aux_sym_preproc_if_token2] = ACTIONS(1140),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1140),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1140),
    [aux_sym_preproc_else_token1] = ACTIONS(1140),
    [aux_sym_preproc_elif_token1] = ACTIONS(1140),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1140),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1140),
    [sym_preproc_directive] = ACTIONS(1140),
    [anon_sym_LPAREN2] = ACTIONS(1142),
    [anon_sym_BANG] = ACTIONS(1142),
    [anon_sym_TILDE] = ACTIONS(1142),
    [anon_sym_DASH] = ACTIONS(1140),
    [anon_sym_PLUS] = ACTIONS(1140),
    [anon_sym_STAR] = ACTIONS(1142),
    [anon_sym_AMP] = ACTIONS(1142),
    [anon_sym_SEMI] = ACTIONS(1142),
    [anon_sym___extension__] = ACTIONS(1140),
    [anon_sym_typedef] = ACTIONS(1140),
    [anon_sym_extern] = ACTIONS(1140),
    [anon_sym___attribute__] = ACTIONS(1140),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1142),
    [anon_sym___declspec] = ACTIONS(1140),
    [anon_sym___cdecl] = ACTIONS(1140),
    [anon_sym___clrcall] = ACTIONS(1140),
    [anon_sym___stdcall] = ACTIONS(1140),
    [anon_sym___fastcall] = ACTIONS(1140),
    [anon_sym___thiscall] = ACTIONS(1140),
    [anon_sym___vectorcall] = ACTIONS(1140),
    [anon_sym_LBRACE] = ACTIONS(1142),
    [anon_sym_signed] = ACTIONS(1140),
    [anon_sym_unsigned] = ACTIONS(1140),
    [anon_sym_long] = ACTIONS(1140),
    [anon_sym_short] = ACTIONS(1140),
    [anon_sym_static] = ACTIONS(1140),
    [anon_sym_auto] = ACTIONS(1140),
    [anon_sym_register] = ACTIONS(1140),
    [anon_sym_inline] = ACTIONS(1140),
    [anon_sym___inline] = ACTIONS(1140),
    [anon_sym___inline__] = ACTIONS(1140),
    [anon_sym___forceinline] = ACTIONS(1140),
    [anon_sym_thread_local] = ACTIONS(1140),
    [anon_sym___thread] = ACTIONS(1140),
    [anon_sym_const] = ACTIONS(1140),
    [anon_sym_constexpr] = ACTIONS(1140),
    [anon_sym_volatile] = ACTIONS(1140),
    [anon_sym_restrict] = ACTIONS(1140),
    [anon_sym___restrict__] = ACTIONS(1140),
    [anon_sym__Atomic] = ACTIONS(1140),
    [anon_sym__Noreturn] = ACTIONS(1140),
    [anon_sym_noreturn] = ACTIONS(1140),
    [sym_primitive_type] = ACTIONS(1140),
    [anon_sym_enum] = ACTIONS(1140),
    [anon_sym_struct] = ACTIONS(1140),
    [anon_sym_union] = ACTIONS(1140),
    [anon_sym_if] = ACTIONS(1140),
    [anon_sym_else] = ACTIONS(1140),
    [anon_sym_switch] = ACTIONS(1140),
    [anon_sym_case] = ACTIONS(1140),
    [anon_sym_default] = ACTIONS(1140),
    [anon_sym_while] = ACTIONS(1140),
    [anon_sym_do] = ACTIONS(1140),
    [anon_sym_for] = ACTIONS(1140),
    [anon_sym_return] = ACTIONS(1140),
    [anon_sym_break] = ACTIONS(1140),
    [anon_sym_continue] = ACTIONS(1140),
    [anon_sym_goto] = ACTIONS(1140),
    [anon_sym___try] = ACTIONS(1140),
    [anon_sym___leave] = ACTIONS(1140),
    [anon_sym_DASH_DASH] = ACTIONS(1142),
    [anon_sym_PLUS_PLUS] = ACTIONS(1142),
    [anon_sym_sizeof] = ACTIONS(1140),
    [anon_sym___alignof__] = ACTIONS(1140),
    [anon_sym___alignof] = ACTIONS(1140),
    [anon_sym__alignof] = ACTIONS(1140),
    [anon_sym_alignof] = ACTIONS(1140),
    [anon_sym__Alignof] = ACTIONS(1140),
    [anon_sym_offsetof] = ACTIONS(1140),
    [anon_sym__Generic] = ACTIONS(1140),
    [anon_sym_asm] = ACTIONS(1140),
    [anon_sym___asm__] = ACTIONS(1140),
    [sym_number_literal] = ACTIONS(1142),
    [anon_sym_L_SQUOTE] = ACTIONS(1142),
    [anon_sym_u_SQUOTE] = ACTIONS(1142),
    [anon_sym_U_SQUOTE] = ACTIONS(1142),
    [anon_sym_u8_SQUOTE] = ACTIONS(1142),
    [anon_sym_SQUOTE] = ACTIONS(1142),
    [anon_sym_L_DQUOTE] = ACTIONS(1142),
    [anon_sym_u_DQUOTE] = ACTIONS(1142),
    [anon_sym_U_DQUOTE] = ACTIONS(1142),
    [anon_sym_u8_DQUOTE] = ACTIONS(1142),
    [anon_sym_DQUOTE] = ACTIONS(1142),
    [sym_true] = ACTIONS(1140),
    [sym_false] = ACTIONS(1140),
    [anon_sym_NULL] = ACTIONS(1140),
    [anon_sym_nullptr] = ACTIONS(1140),
    [sym_comment] = ACTIONS(3),
  },
  [84] = {
    [sym_identifier] = ACTIONS(1144),
    [aux_sym_preproc_include_token1] = ACTIONS(1144),
    [aux_sym_preproc_def_token1] = ACTIONS(1144),
    [aux_sym_preproc_if_token1] = ACTIONS(1144),
    [aux_sym_preproc_if_token2] = ACTIONS(1144),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1144),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1144),
    [aux_sym_preproc_else_token1] = ACTIONS(1144),
    [aux_sym_preproc_elif_token1] = ACTIONS(1144),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1144),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1144),
    [sym_preproc_directive] = ACTIONS(1144),
    [anon_sym_LPAREN2] = ACTIONS(1146),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_STAR] = ACTIONS(1146),
    [anon_sym_AMP] = ACTIONS(1146),
    [anon_sym_SEMI] = ACTIONS(1146),
    [anon_sym___extension__] = ACTIONS(1144),
    [anon_sym_typedef] = ACTIONS(1144),
    [anon_sym_extern] = ACTIONS(1144),
    [anon_sym___attribute__] = ACTIONS(1144),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1146),
    [anon_sym___declspec] = ACTIONS(1144),
    [anon_sym___cdecl] = ACTIONS(1144),
    [anon_sym___clrcall] = ACTIONS(1144),
    [anon_sym___stdcall] = ACTIONS(1144),
    [anon_sym___fastcall] = ACTIONS(1144),
    [anon_sym___thiscall] = ACTIONS(1144),
    [anon_sym___vectorcall] = ACTIONS(1144),
    [anon_sym_LBRACE] = ACTIONS(1146),
    [anon_sym_signed] = ACTIONS(1144),
    [anon_sym_unsigned] = ACTIONS(1144),
    [anon_sym_long] = ACTIONS(1144),
    [anon_sym_short] = ACTIONS(1144),
    [anon_sym_static] = ACTIONS(1144),
    [anon_sym_auto] = ACTIONS(1144),
    [anon_sym_register] = ACTIONS(1144),
    [anon_sym_inline] = ACTIONS(1144),
    [anon_sym___inline] = ACTIONS(1144),
    [anon_sym___inline__] = ACTIONS(1144),
    [anon_sym___forceinline] = ACTIONS(1144),
    [anon_sym_thread_local] = ACTIONS(1144),
    [anon_sym___thread] = ACTIONS(1144),
    [anon_sym_const] = ACTIONS(1144),
    [anon_sym_constexpr] = ACTIONS(1144),
    [anon_sym_volatile] = ACTIONS(1144),
    [anon_sym_restrict] = ACTIONS(1144),
    [anon_sym___restrict__] = ACTIONS(1144),
    [anon_sym__Atomic] = ACTIONS(1144),
    [anon_sym__Noreturn] = ACTIONS(1144),
    [anon_sym_noreturn] = ACTIONS(1144),
    [sym_primitive_type] = ACTIONS(1144),
    [anon_sym_enum] = ACTIONS(1144),
    [anon_sym_struct] = ACTIONS(1144),
    [anon_sym_union] = ACTIONS(1144),
    [anon_sym_if] = ACTIONS(1144),
    [anon_sym_else] = ACTIONS(1144),
    [anon_sym_switch] = ACTIONS(1144),
    [anon_sym_case] = ACTIONS(1144),
    [anon_sym_default] = ACTIONS(1144),
    [anon_sym_while] = ACTIONS(1144),
    [anon_sym_do] = ACTIONS(1144),
    [anon_sym_for] = ACTIONS(1144),
    [anon_sym_return] = ACTIONS(1144),
    [anon_sym_break] = ACTIONS(1144),
    [anon_sym_continue] = ACTIONS(1144),
    [anon_sym_goto] = ACTIONS(1144),
    [anon_sym___try] = ACTIONS(1144),
    [anon_sym___leave] = ACTIONS(1144),
    [anon_sym_DASH_DASH] = ACTIONS(1146),
    [anon_sym_PLUS_PLUS] = ACTIONS(1146),
    [anon_sym_sizeof] = ACTIONS(1144),
    [anon_sym___alignof__] = ACTIONS(1144),
    [anon_sym___alignof] = ACTIONS(1144),
    [anon_sym__alignof] = ACTIONS(1144),
    [anon_sym_alignof] = ACTIONS(1144),
    [anon_sym__Alignof] = ACTIONS(1144),
    [anon_sym_offsetof] = ACTIONS(1144),
    [anon_sym__Generic] = ACTIONS(1144),
    [anon_sym_asm] = ACTIONS(1144),
    [anon_sym___asm__] = ACTIONS(1144),
    [sym_number_literal] = ACTIONS(1146),
    [anon_sym_L_SQUOTE] = ACTIONS(1146),
    [anon_sym_u_SQUOTE] = ACTIONS(1146),
    [anon_sym_U_SQUOTE] = ACTIONS(1146),
    [anon_sym_u8_SQUOTE] = ACTIONS(1146),
    [anon_sym_SQUOTE] = ACTIONS(1146),
    [anon_sym_L_DQUOTE] = ACTIONS(1146),
    [anon_sym_u_DQUOTE] = ACTIONS(1146),
    [anon_sym_U_DQUOTE] = ACTIONS(1146),
    [anon_sym_u8_DQUOTE] = ACTIONS(1146),
    [anon_sym_DQUOTE] = ACTIONS(1146),
    [sym_true] = ACTIONS(1144),
    [sym_false] = ACTIONS(1144),
    [anon_sym_NULL] = ACTIONS(1144),
    [anon_sym_nullptr] = ACTIONS(1144),
    [sym_comment] = ACTIONS(3),
  },
  [85] = {
    [sym_identifier] = ACTIONS(1148),
    [aux_sym_preproc_include_token1] = ACTIONS(1148),
    [aux_sym_preproc_def_token1] = ACTIONS(1148),
    [aux_sym_preproc_if_token1] = ACTIONS(1148),
    [aux_sym_preproc_if_token2] = ACTIONS(1148),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1148),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1148),
    [aux_sym_preproc_else_token1] = ACTIONS(1148),
    [aux_sym_preproc_elif_token1] = ACTIONS(1148),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1148),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1148),
    [sym_preproc_directive] = ACTIONS(1148),
    [anon_sym_LPAREN2] = ACTIONS(1150),
    [anon_sym_BANG] = ACTIONS(1150),
    [anon_sym_TILDE] = ACTIONS(1150),
    [anon_sym_DASH] = ACTIONS(1148),
    [anon_sym_PLUS] = ACTIONS(1148),
    [anon_sym_STAR] = ACTIONS(1150),
    [anon_sym_AMP] = ACTIONS(1150),
    [anon_sym_SEMI] = ACTIONS(1150),
    [anon_sym___extension__] = ACTIONS(1148),
    [anon_sym_typedef] = ACTIONS(1148),
    [anon_sym_extern] = ACTIONS(1148),
    [anon_sym___attribute__] = ACTIONS(1148),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1150),
    [anon_sym___declspec] = ACTIONS(1148),
    [anon_sym___cdecl] = ACTIONS(1148),
    [anon_sym___clrcall] = ACTIONS(1148),
    [anon_sym___stdcall] = ACTIONS(1148),
    [anon_sym___fastcall] = ACTIONS(1148),
    [anon_sym___thiscall] = ACTIONS(1148),
    [anon_sym___vectorcall] = ACTIONS(1148),
    [anon_sym_LBRACE] = ACTIONS(1150),
    [anon_sym_signed] = ACTIONS(1148),
    [anon_sym_unsigned] = ACTIONS(1148),
    [anon_sym_long] = ACTIONS(1148),
    [anon_sym_short] = ACTIONS(1148),
    [anon_sym_static] = ACTIONS(1148),
    [anon_sym_auto] = ACTIONS(1148),
    [anon_sym_register] = ACTIONS(1148),
    [anon_sym_inline] = ACTIONS(1148),
    [anon_sym___inline] = ACTIONS(1148),
    [anon_sym___inline__] = ACTIONS(1148),
    [anon_sym___forceinline] = ACTIONS(1148),
    [anon_sym_thread_local] = ACTIONS(1148),
    [anon_sym___thread] = ACTIONS(1148),
    [anon_sym_const] = ACTIONS(1148),
    [anon_sym_constexpr] = ACTIONS(1148),
    [anon_sym_volatile] = ACTIONS(1148),
    [anon_sym_restrict] = ACTIONS(1148),
    [anon_sym___restrict__] = ACTIONS(1148),
    [anon_sym__Atomic] = ACTIONS(1148),
    [anon_sym__Noreturn] = ACTIONS(1148),
    [anon_sym_noreturn] = ACTIONS(1148),
    [sym_primitive_type] = ACTIONS(1148),
    [anon_sym_enum] = ACTIONS(1148),
    [anon_sym_struct] = ACTIONS(1148),
    [anon_sym_union] = ACTIONS(1148),
    [anon_sym_if] = ACTIONS(1148),
    [anon_sym_else] = ACTIONS(1148),
    [anon_sym_switch] = ACTIONS(1148),
    [anon_sym_case] = ACTIONS(1148),
    [anon_sym_default] = ACTIONS(1148),
    [anon_sym_while] = ACTIONS(1148),
    [anon_sym_do] = ACTIONS(1148),
    [anon_sym_for] = ACTIONS(1148),
    [anon_sym_return] = ACTIONS(1148),
    [anon_sym_break] = ACTIONS(1148),
    [anon_sym_continue] = ACTIONS(1148),
    [anon_sym_goto] = ACTIONS(1148),
    [anon_sym___try] = ACTIONS(1148),
    [anon_sym___leave] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1150),
    [anon_sym_PLUS_PLUS] = ACTIONS(1150),
    [anon_sym_sizeof] = ACTIONS(1148),
    [anon_sym___alignof__] = ACTIONS(1148),
    [anon_sym___alignof] = ACTIONS(1148),
    [anon_sym__alignof] = ACTIONS(1148),
    [anon_sym_alignof] = ACTIONS(1148),
    [anon_sym__Alignof] = ACTIONS(1148),
    [anon_sym_offsetof] = ACTIONS(1148),
    [anon_sym__Generic] = ACTIONS(1148),
    [anon_sym_asm] = ACTIONS(1148),
    [anon_sym___asm__] = ACTIONS(1148),
    [sym_number_literal] = ACTIONS(1150),
    [anon_sym_L_SQUOTE] = ACTIONS(1150),
    [anon_sym_u_SQUOTE] = ACTIONS(1150),
    [anon_sym_U_SQUOTE] = ACTIONS(1150),
    [anon_sym_u8_SQUOTE] = ACTIONS(1150),
    [anon_sym_SQUOTE] = ACTIONS(1150),
    [anon_sym_L_DQUOTE] = ACTIONS(1150),
    [anon_sym_u_DQUOTE] = ACTIONS(1150),
    [anon_sym_U_DQUOTE] = ACTIONS(1150),
    [anon_sym_u8_DQUOTE] = ACTIONS(1150),
    [anon_sym_DQUOTE] = ACTIONS(1150),
    [sym_true] = ACTIONS(1148),
    [sym_false] = ACTIONS(1148),
    [anon_sym_NULL] = ACTIONS(1148),
    [anon_sym_nullptr] = ACTIONS(1148),
    [sym_comment] = ACTIONS(3),
  },
  [86] = {
    [sym_identifier] = ACTIONS(1152),
    [aux_sym_preproc_include_token1] = ACTIONS(1152),
    [aux_sym_preproc_def_token1] = ACTIONS(1152),
    [aux_sym_preproc_if_token1] = ACTIONS(1152),
    [aux_sym_preproc_if_token2] = ACTIONS(1152),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1152),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1152),
    [aux_sym_preproc_else_token1] = ACTIONS(1152),
    [aux_sym_preproc_elif_token1] = ACTIONS(1152),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1152),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1152),
    [sym_preproc_directive] = ACTIONS(1152),
    [anon_sym_LPAREN2] = ACTIONS(1154),
    [anon_sym_BANG] = ACTIONS(1154),
    [anon_sym_TILDE] = ACTIONS(1154),
    [anon_sym_DASH] = ACTIONS(1152),
    [anon_sym_PLUS] = ACTIONS(1152),
    [anon_sym_STAR] = ACTIONS(1154),
    [anon_sym_AMP] = ACTIONS(1154),
    [anon_sym_SEMI] = ACTIONS(1154),
    [anon_sym___extension__] = ACTIONS(1152),
    [anon_sym_typedef] = ACTIONS(1152),
    [anon_sym_extern] = ACTIONS(1152),
    [anon_sym___attribute__] = ACTIONS(1152),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1154),
    [anon_sym___declspec] = ACTIONS(1152),
    [anon_sym___cdecl] = ACTIONS(1152),
    [anon_sym___clrcall] = ACTIONS(1152),
    [anon_sym___stdcall] = ACTIONS(1152),
    [anon_sym___fastcall] = ACTIONS(1152),
    [anon_sym___thiscall] = ACTIONS(1152),
    [anon_sym___vectorcall] = ACTIONS(1152),
    [anon_sym_LBRACE] = ACTIONS(1154),
    [anon_sym_signed] = ACTIONS(1152),
    [anon_sym_unsigned] = ACTIONS(1152),
    [anon_sym_long] = ACTIONS(1152),
    [anon_sym_short] = ACTIONS(1152),
    [anon_sym_static] = ACTIONS(1152),
    [anon_sym_auto] = ACTIONS(1152),
    [anon_sym_register] = ACTIONS(1152),
    [anon_sym_inline] = ACTIONS(1152),
    [anon_sym___inline] = ACTIONS(1152),
    [anon_sym___inline__] = ACTIONS(1152),
    [anon_sym___forceinline] = ACTIONS(1152),
    [anon_sym_thread_local] = ACTIONS(1152),
    [anon_sym___thread] = ACTIONS(1152),
    [anon_sym_const] = ACTIONS(1152),
    [anon_sym_constexpr] = ACTIONS(1152),
    [anon_sym_volatile] = ACTIONS(1152),
    [anon_sym_restrict] = ACTIONS(1152),
    [anon_sym___restrict__] = ACTIONS(1152),
    [anon_sym__Atomic] = ACTIONS(1152),
    [anon_sym__Noreturn] = ACTIONS(1152),
    [anon_sym_noreturn] = ACTIONS(1152),
    [sym_primitive_type] = ACTIONS(1152),
    [anon_sym_enum] = ACTIONS(1152),
    [anon_sym_struct] = ACTIONS(1152),
    [anon_sym_union] = ACTIONS(1152),
    [anon_sym_if] = ACTIONS(1152),
    [anon_sym_else] = ACTIONS(1152),
    [anon_sym_switch] = ACTIONS(1152),
    [anon_sym_case] = ACTIONS(1152),
    [anon_sym_default] = ACTIONS(1152),
    [anon_sym_while] = ACTIONS(1152),
    [anon_sym_do] = ACTIONS(1152),
    [anon_sym_for] = ACTIONS(1152),
    [anon_sym_return] = ACTIONS(1152),
    [anon_sym_break] = ACTIONS(1152),
    [anon_sym_continue] = ACTIONS(1152),
    [anon_sym_goto] = ACTIONS(1152),
    [anon_sym___try] = ACTIONS(1152),
    [anon_sym___leave] = ACTIONS(1152),
    [anon_sym_DASH_DASH] = ACTIONS(1154),
    [anon_sym_PLUS_PLUS] = ACTIONS(1154),
    [anon_sym_sizeof] = ACTIONS(1152),
    [anon_sym___alignof__] = ACTIONS(1152),
    [anon_sym___alignof] = ACTIONS(1152),
    [anon_sym__alignof] = ACTIONS(1152),
    [anon_sym_alignof] = ACTIONS(1152),
    [anon_sym__Alignof] = ACTIONS(1152),
    [anon_sym_offsetof] = ACTIONS(1152),
    [anon_sym__Generic] = ACTIONS(1152),
    [anon_sym_asm] = ACTIONS(1152),
    [anon_sym___asm__] = ACTIONS(1152),
    [sym_number_literal] = ACTIONS(1154),
    [anon_sym_L_SQUOTE] = ACTIONS(1154),
    [anon_sym_u_SQUOTE] = ACTIONS(1154),
    [anon_sym_U_SQUOTE] = ACTIONS(1154),
    [anon_sym_u8_SQUOTE] = ACTIONS(1154),
    [anon_sym_SQUOTE] = ACTIONS(1154),
    [anon_sym_L_DQUOTE] = ACTIONS(1154),
    [anon_sym_u_DQUOTE] = ACTIONS(1154),
    [anon_sym_U_DQUOTE] = ACTIONS(1154),
    [anon_sym_u8_DQUOTE] = ACTIONS(1154),
    [anon_sym_DQUOTE] = ACTIONS(1154),
    [sym_true] = ACTIONS(1152),
    [sym_false] = ACTIONS(1152),
    [anon_sym_NULL] = ACTIONS(1152),
    [anon_sym_nullptr] = ACTIONS(1152),
    [sym_comment] = ACTIONS(3),
  },
  [87] = {
    [sym_identifier] = ACTIONS(1156),
    [aux_sym_preproc_include_token1] = ACTIONS(1156),
    [aux_sym_preproc_def_token1] = ACTIONS(1156),
    [aux_sym_preproc_if_token1] = ACTIONS(1156),
    [aux_sym_preproc_if_token2] = ACTIONS(1156),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1156),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1156),
    [aux_sym_preproc_else_token1] = ACTIONS(1156),
    [aux_sym_preproc_elif_token1] = ACTIONS(1156),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1156),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1156),
    [sym_preproc_directive] = ACTIONS(1156),
    [anon_sym_LPAREN2] = ACTIONS(1158),
    [anon_sym_BANG] = ACTIONS(1158),
    [anon_sym_TILDE] = ACTIONS(1158),
    [anon_sym_DASH] = ACTIONS(1156),
    [anon_sym_PLUS] = ACTIONS(1156),
    [anon_sym_STAR] = ACTIONS(1158),
    [anon_sym_AMP] = ACTIONS(1158),
    [anon_sym_SEMI] = ACTIONS(1158),
    [anon_sym___extension__] = ACTIONS(1156),
    [anon_sym_typedef] = ACTIONS(1156),
    [anon_sym_extern] = ACTIONS(1156),
    [anon_sym___attribute__] = ACTIONS(1156),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1158),
    [anon_sym___declspec] = ACTIONS(1156),
    [anon_sym___cdecl] = ACTIONS(1156),
    [anon_sym___clrcall] = ACTIONS(1156),
    [anon_sym___stdcall] = ACTIONS(1156),
    [anon_sym___fastcall] = ACTIONS(1156),
    [anon_sym___thiscall] = ACTIONS(1156),
    [anon_sym___vectorcall] = ACTIONS(1156),
    [anon_sym_LBRACE] = ACTIONS(1158),
    [anon_sym_signed] = ACTIONS(1156),
    [anon_sym_unsigned] = ACTIONS(1156),
    [anon_sym_long] = ACTIONS(1156),
    [anon_sym_short] = ACTIONS(1156),
    [anon_sym_static] = ACTIONS(1156),
    [anon_sym_auto] = ACTIONS(1156),
    [anon_sym_register] = ACTIONS(1156),
    [anon_sym_inline] = ACTIONS(1156),
    [anon_sym___inline] = ACTIONS(1156),
    [anon_sym___inline__] = ACTIONS(1156),
    [anon_sym___forceinline] = ACTIONS(1156),
    [anon_sym_thread_local] = ACTIONS(1156),
    [anon_sym___thread] = ACTIONS(1156),
    [anon_sym_const] = ACTIONS(1156),
    [anon_sym_constexpr] = ACTIONS(1156),
    [anon_sym_volatile] = ACTIONS(1156),
    [anon_sym_restrict] = ACTIONS(1156),
    [anon_sym___restrict__] = ACTIONS(1156),
    [anon_sym__Atomic] = ACTIONS(1156),
    [anon_sym__Noreturn] = ACTIONS(1156),
    [anon_sym_noreturn] = ACTIONS(1156),
    [sym_primitive_type] = ACTIONS(1156),
    [anon_sym_enum] = ACTIONS(1156),
    [anon_sym_struct] = ACTIONS(1156),
    [anon_sym_union] = ACTIONS(1156),
    [anon_sym_if] = ACTIONS(1156),
    [anon_sym_else] = ACTIONS(1156),
    [anon_sym_switch] = ACTIONS(1156),
    [anon_sym_case] = ACTIONS(1156),
    [anon_sym_default] = ACTIONS(1156),
    [anon_sym_while] = ACTIONS(1156),
    [anon_sym_do] = ACTIONS(1156),
    [anon_sym_for] = ACTIONS(1156),
    [anon_sym_return] = ACTIONS(1156),
    [anon_sym_break] = ACTIONS(1156),
    [anon_sym_continue] = ACTIONS(1156),
    [anon_sym_goto] = ACTIONS(1156),
    [anon_sym___try] = ACTIONS(1156),
    [anon_sym___leave] = ACTIONS(1156),
    [anon_sym_DASH_DASH] = ACTIONS(1158),
    [anon_sym_PLUS_PLUS] = ACTIONS(1158),
    [anon_sym_sizeof] = ACTIONS(1156),
    [anon_sym___alignof__] = ACTIONS(1156),
    [anon_sym___alignof] = ACTIONS(1156),
    [anon_sym__alignof] = ACTIONS(1156),
    [anon_sym_alignof] = ACTIONS(1156),
    [anon_sym__Alignof] = ACTIONS(1156),
    [anon_sym_offsetof] = ACTIONS(1156),
    [anon_sym__Generic] = ACTIONS(1156),
    [anon_sym_asm] = ACTIONS(1156),
    [anon_sym___asm__] = ACTIONS(1156),
    [sym_number_literal] = ACTIONS(1158),
    [anon_sym_L_SQUOTE] = ACTIONS(1158),
    [anon_sym_u_SQUOTE] = ACTIONS(1158),
    [anon_sym_U_SQUOTE] = ACTIONS(1158),
    [anon_sym_u8_SQUOTE] = ACTIONS(1158),
    [anon_sym_SQUOTE] = ACTIONS(1158),
    [anon_sym_L_DQUOTE] = ACTIONS(1158),
    [anon_sym_u_DQUOTE] = ACTIONS(1158),
    [anon_sym_U_DQUOTE] = ACTIONS(1158),
    [anon_sym_u8_DQUOTE] = ACTIONS(1158),
    [anon_sym_DQUOTE] = ACTIONS(1158),
    [sym_true] = ACTIONS(1156),
    [sym_false] = ACTIONS(1156),
    [anon_sym_NULL] = ACTIONS(1156),
    [anon_sym_nullptr] = ACTIONS(1156),
    [sym_comment] = ACTIONS(3),
  },
  [88] = {
    [sym_identifier] = ACTIONS(1160),
    [aux_sym_preproc_include_token1] = ACTIONS(1160),
    [aux_sym_preproc_def_token1] = ACTIONS(1160),
    [aux_sym_preproc_if_token1] = ACTIONS(1160),
    [aux_sym_preproc_if_token2] = ACTIONS(1160),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1160),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1160),
    [aux_sym_preproc_else_token1] = ACTIONS(1160),
    [aux_sym_preproc_elif_token1] = ACTIONS(1160),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1160),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1160),
    [sym_preproc_directive] = ACTIONS(1160),
    [anon_sym_LPAREN2] = ACTIONS(1162),
    [anon_sym_BANG] = ACTIONS(1162),
    [anon_sym_TILDE] = ACTIONS(1162),
    [anon_sym_DASH] = ACTIONS(1160),
    [anon_sym_PLUS] = ACTIONS(1160),
    [anon_sym_STAR] = ACTIONS(1162),
    [anon_sym_AMP] = ACTIONS(1162),
    [anon_sym_SEMI] = ACTIONS(1162),
    [anon_sym___extension__] = ACTIONS(1160),
    [anon_sym_typedef] = ACTIONS(1160),
    [anon_sym_extern] = ACTIONS(1160),
    [anon_sym___attribute__] = ACTIONS(1160),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1162),
    [anon_sym___declspec] = ACTIONS(1160),
    [anon_sym___cdecl] = ACTIONS(1160),
    [anon_sym___clrcall] = ACTIONS(1160),
    [anon_sym___stdcall] = ACTIONS(1160),
    [anon_sym___fastcall] = ACTIONS(1160),
    [anon_sym___thiscall] = ACTIONS(1160),
    [anon_sym___vectorcall] = ACTIONS(1160),
    [anon_sym_LBRACE] = ACTIONS(1162),
    [anon_sym_signed] = ACTIONS(1160),
    [anon_sym_unsigned] = ACTIONS(1160),
    [anon_sym_long] = ACTIONS(1160),
    [anon_sym_short] = ACTIONS(1160),
    [anon_sym_static] = ACTIONS(1160),
    [anon_sym_auto] = ACTIONS(1160),
    [anon_sym_register] = ACTIONS(1160),
    [anon_sym_inline] = ACTIONS(1160),
    [anon_sym___inline] = ACTIONS(1160),
    [anon_sym___inline__] = ACTIONS(1160),
    [anon_sym___forceinline] = ACTIONS(1160),
    [anon_sym_thread_local] = ACTIONS(1160),
    [anon_sym___thread] = ACTIONS(1160),
    [anon_sym_const] = ACTIONS(1160),
    [anon_sym_constexpr] = ACTIONS(1160),
    [anon_sym_volatile] = ACTIONS(1160),
    [anon_sym_restrict] = ACTIONS(1160),
    [anon_sym___restrict__] = ACTIONS(1160),
    [anon_sym__Atomic] = ACTIONS(1160),
    [anon_sym__Noreturn] = ACTIONS(1160),
    [anon_sym_noreturn] = ACTIONS(1160),
    [sym_primitive_type] = ACTIONS(1160),
    [anon_sym_enum] = ACTIONS(1160),
    [anon_sym_struct] = ACTIONS(1160),
    [anon_sym_union] = ACTIONS(1160),
    [anon_sym_if] = ACTIONS(1160),
    [anon_sym_else] = ACTIONS(1160),
    [anon_sym_switch] = ACTIONS(1160),
    [anon_sym_case] = ACTIONS(1160),
    [anon_sym_default] = ACTIONS(1160),
    [anon_sym_while] = ACTIONS(1160),
    [anon_sym_do] = ACTIONS(1160),
    [anon_sym_for] = ACTIONS(1160),
    [anon_sym_return] = ACTIONS(1160),
    [anon_sym_break] = ACTIONS(1160),
    [anon_sym_continue] = ACTIONS(1160),
    [anon_sym_goto] = ACTIONS(1160),
    [anon_sym___try] = ACTIONS(1160),
    [anon_sym___leave] = ACTIONS(1160),
    [anon_sym_DASH_DASH] = ACTIONS(1162),
    [anon_sym_PLUS_PLUS] = ACTIONS(1162),
    [anon_sym_sizeof] = ACTIONS(1160),
    [anon_sym___alignof__] = ACTIONS(1160),
    [anon_sym___alignof] = ACTIONS(1160),
    [anon_sym__alignof] = ACTIONS(1160),
    [anon_sym_alignof] = ACTIONS(1160),
    [anon_sym__Alignof] = ACTIONS(1160),
    [anon_sym_offsetof] = ACTIONS(1160),
    [anon_sym__Generic] = ACTIONS(1160),
    [anon_sym_asm] = ACTIONS(1160),
    [anon_sym___asm__] = ACTIONS(1160),
    [sym_number_literal] = ACTIONS(1162),
    [anon_sym_L_SQUOTE] = ACTIONS(1162),
    [anon_sym_u_SQUOTE] = ACTIONS(1162),
    [anon_sym_U_SQUOTE] = ACTIONS(1162),
    [anon_sym_u8_SQUOTE] = ACTIONS(1162),
    [anon_sym_SQUOTE] = ACTIONS(1162),
    [anon_sym_L_DQUOTE] = ACTIONS(1162),
    [anon_sym_u_DQUOTE] = ACTIONS(1162),
    [anon_sym_U_DQUOTE] = ACTIONS(1162),
    [anon_sym_u8_DQUOTE] = ACTIONS(1162),
    [anon_sym_DQUOTE] = ACTIONS(1162),
    [sym_true] = ACTIONS(1160),
    [sym_false] = ACTIONS(1160),
    [anon_sym_NULL] = ACTIONS(1160),
    [anon_sym_nullptr] = ACTIONS(1160),
    [sym_comment] = ACTIONS(3),
  },
  [89] = {
    [sym_identifier] = ACTIONS(1120),
    [aux_sym_preproc_include_token1] = ACTIONS(1120),
    [aux_sym_preproc_def_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token2] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1120),
    [aux_sym_preproc_else_token1] = ACTIONS(1120),
    [aux_sym_preproc_elif_token1] = ACTIONS(1120),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1120),
    [sym_preproc_directive] = ACTIONS(1120),
    [anon_sym_LPAREN2] = ACTIONS(1122),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_DASH] = ACTIONS(1120),
    [anon_sym_PLUS] = ACTIONS(1120),
    [anon_sym_STAR] = ACTIONS(1122),
    [anon_sym_AMP] = ACTIONS(1122),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym___extension__] = ACTIONS(1120),
    [anon_sym_typedef] = ACTIONS(1120),
    [anon_sym_extern] = ACTIONS(1120),
    [anon_sym___attribute__] = ACTIONS(1120),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1122),
    [anon_sym___declspec] = ACTIONS(1120),
    [anon_sym___cdecl] = ACTIONS(1120),
    [anon_sym___clrcall] = ACTIONS(1120),
    [anon_sym___stdcall] = ACTIONS(1120),
    [anon_sym___fastcall] = ACTIONS(1120),
    [anon_sym___thiscall] = ACTIONS(1120),
    [anon_sym___vectorcall] = ACTIONS(1120),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_signed] = ACTIONS(1120),
    [anon_sym_unsigned] = ACTIONS(1120),
    [anon_sym_long] = ACTIONS(1120),
    [anon_sym_short] = ACTIONS(1120),
    [anon_sym_static] = ACTIONS(1120),
    [anon_sym_auto] = ACTIONS(1120),
    [anon_sym_register] = ACTIONS(1120),
    [anon_sym_inline] = ACTIONS(1120),
    [anon_sym___inline] = ACTIONS(1120),
    [anon_sym___inline__] = ACTIONS(1120),
    [anon_sym___forceinline] = ACTIONS(1120),
    [anon_sym_thread_local] = ACTIONS(1120),
    [anon_sym___thread] = ACTIONS(1120),
    [anon_sym_const] = ACTIONS(1120),
    [anon_sym_constexpr] = ACTIONS(1120),
    [anon_sym_volatile] = ACTIONS(1120),
    [anon_sym_restrict] = ACTIONS(1120),
    [anon_sym___restrict__] = ACTIONS(1120),
    [anon_sym__Atomic] = ACTIONS(1120),
    [anon_sym__Noreturn] = ACTIONS(1120),
    [anon_sym_noreturn] = ACTIONS(1120),
    [sym_primitive_type] = ACTIONS(1120),
    [anon_sym_enum] = ACTIONS(1120),
    [anon_sym_struct] = ACTIONS(1120),
    [anon_sym_union] = ACTIONS(1120),
    [anon_sym_if] = ACTIONS(1120),
    [anon_sym_else] = ACTIONS(1120),
    [anon_sym_switch] = ACTIONS(1120),
    [anon_sym_case] = ACTIONS(1120),
    [anon_sym_default] = ACTIONS(1120),
    [anon_sym_while] = ACTIONS(1120),
    [anon_sym_do] = ACTIONS(1120),
    [anon_sym_for] = ACTIONS(1120),
    [anon_sym_return] = ACTIONS(1120),
    [anon_sym_break] = ACTIONS(1120),
    [anon_sym_continue] = ACTIONS(1120),
    [anon_sym_goto] = ACTIONS(1120),
    [anon_sym___try] = ACTIONS(1120),
    [anon_sym___leave] = ACTIONS(1120),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_sizeof] = ACTIONS(1120),
    [anon_sym___alignof__] = ACTIONS(1120),
    [anon_sym___alignof] = ACTIONS(1120),
    [anon_sym__alignof] = ACTIONS(1120),
    [anon_sym_alignof] = ACTIONS(1120),
    [anon_sym__Alignof] = ACTIONS(1120),
    [anon_sym_offsetof] = ACTIONS(1120),
    [anon_sym__Generic] = ACTIONS(1120),
    [anon_sym_asm] = ACTIONS(1120),
    [anon_sym___asm__] = ACTIONS(1120),
    [sym_number_literal] = ACTIONS(1122),
    [anon_sym_L_SQUOTE] = ACTIONS(1122),
    [anon_sym_u_SQUOTE] = ACTIONS(1122),
    [anon_sym_U_SQUOTE] = ACTIONS(1122),
    [anon_sym_u8_SQUOTE] = ACTIONS(1122),
    [anon_sym_SQUOTE] = ACTIONS(1122),
    [anon_sym_L_DQUOTE] = ACTIONS(1122),
    [anon_sym_u_DQUOTE] = ACTIONS(1122),
    [anon_sym_U_DQUOTE] = ACTIONS(1122),
    [anon_sym_u8_DQUOTE] = ACTIONS(1122),
    [anon_sym_DQUOTE] = ACTIONS(1122),
    [sym_true] = ACTIONS(1120),
    [sym_false] = ACTIONS(1120),
    [anon_sym_NULL] = ACTIONS(1120),
    [anon_sym_nullptr] = ACTIONS(1120),
    [sym_comment] = ACTIONS(3),
  },
  [90] = {
    [sym_identifier] = ACTIONS(1164),
    [aux_sym_preproc_include_token1] = ACTIONS(1164),
    [aux_sym_preproc_def_token1] = ACTIONS(1164),
    [aux_sym_preproc_if_token1] = ACTIONS(1164),
    [aux_sym_preproc_if_token2] = ACTIONS(1164),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1164),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1164),
    [aux_sym_preproc_else_token1] = ACTIONS(1164),
    [aux_sym_preproc_elif_token1] = ACTIONS(1164),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1164),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1164),
    [sym_preproc_directive] = ACTIONS(1164),
    [anon_sym_LPAREN2] = ACTIONS(1166),
    [anon_sym_BANG] = ACTIONS(1166),
    [anon_sym_TILDE] = ACTIONS(1166),
    [anon_sym_DASH] = ACTIONS(1164),
    [anon_sym_PLUS] = ACTIONS(1164),
    [anon_sym_STAR] = ACTIONS(1166),
    [anon_sym_AMP] = ACTIONS(1166),
    [anon_sym_SEMI] = ACTIONS(1166),
    [anon_sym___extension__] = ACTIONS(1164),
    [anon_sym_typedef] = ACTIONS(1164),
    [anon_sym_extern] = ACTIONS(1164),
    [anon_sym___attribute__] = ACTIONS(1164),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1166),
    [anon_sym___declspec] = ACTIONS(1164),
    [anon_sym___cdecl] = ACTIONS(1164),
    [anon_sym___clrcall] = ACTIONS(1164),
    [anon_sym___stdcall] = ACTIONS(1164),
    [anon_sym___fastcall] = ACTIONS(1164),
    [anon_sym___thiscall] = ACTIONS(1164),
    [anon_sym___vectorcall] = ACTIONS(1164),
    [anon_sym_LBRACE] = ACTIONS(1166),
    [anon_sym_signed] = ACTIONS(1164),
    [anon_sym_unsigned] = ACTIONS(1164),
    [anon_sym_long] = ACTIONS(1164),
    [anon_sym_short] = ACTIONS(1164),
    [anon_sym_static] = ACTIONS(1164),
    [anon_sym_auto] = ACTIONS(1164),
    [anon_sym_register] = ACTIONS(1164),
    [anon_sym_inline] = ACTIONS(1164),
    [anon_sym___inline] = ACTIONS(1164),
    [anon_sym___inline__] = ACTIONS(1164),
    [anon_sym___forceinline] = ACTIONS(1164),
    [anon_sym_thread_local] = ACTIONS(1164),
    [anon_sym___thread] = ACTIONS(1164),
    [anon_sym_const] = ACTIONS(1164),
    [anon_sym_constexpr] = ACTIONS(1164),
    [anon_sym_volatile] = ACTIONS(1164),
    [anon_sym_restrict] = ACTIONS(1164),
    [anon_sym___restrict__] = ACTIONS(1164),
    [anon_sym__Atomic] = ACTIONS(1164),
    [anon_sym__Noreturn] = ACTIONS(1164),
    [anon_sym_noreturn] = ACTIONS(1164),
    [sym_primitive_type] = ACTIONS(1164),
    [anon_sym_enum] = ACTIONS(1164),
    [anon_sym_struct] = ACTIONS(1164),
    [anon_sym_union] = ACTIONS(1164),
    [anon_sym_if] = ACTIONS(1164),
    [anon_sym_else] = ACTIONS(1164),
    [anon_sym_switch] = ACTIONS(1164),
    [anon_sym_case] = ACTIONS(1164),
    [anon_sym_default] = ACTIONS(1164),
    [anon_sym_while] = ACTIONS(1164),
    [anon_sym_do] = ACTIONS(1164),
    [anon_sym_for] = ACTIONS(1164),
    [anon_sym_return] = ACTIONS(1164),
    [anon_sym_break] = ACTIONS(1164),
    [anon_sym_continue] = ACTIONS(1164),
    [anon_sym_goto] = ACTIONS(1164),
    [anon_sym___try] = ACTIONS(1164),
    [anon_sym___leave] = ACTIONS(1164),
    [anon_sym_DASH_DASH] = ACTIONS(1166),
    [anon_sym_PLUS_PLUS] = ACTIONS(1166),
    [anon_sym_sizeof] = ACTIONS(1164),
    [anon_sym___alignof__] = ACTIONS(1164),
    [anon_sym___alignof] = ACTIONS(1164),
    [anon_sym__alignof] = ACTIONS(1164),
    [anon_sym_alignof] = ACTIONS(1164),
    [anon_sym__Alignof] = ACTIONS(1164),
    [anon_sym_offsetof] = ACTIONS(1164),
    [anon_sym__Generic] = ACTIONS(1164),
    [anon_sym_asm] = ACTIONS(1164),
    [anon_sym___asm__] = ACTIONS(1164),
    [sym_number_literal] = ACTIONS(1166),
    [anon_sym_L_SQUOTE] = ACTIONS(1166),
    [anon_sym_u_SQUOTE] = ACTIONS(1166),
    [anon_sym_U_SQUOTE] = ACTIONS(1166),
    [anon_sym_u8_SQUOTE] = ACTIONS(1166),
    [anon_sym_SQUOTE] = ACTIONS(1166),
    [anon_sym_L_DQUOTE] = ACTIONS(1166),
    [anon_sym_u_DQUOTE] = ACTIONS(1166),
    [anon_sym_U_DQUOTE] = ACTIONS(1166),
    [anon_sym_u8_DQUOTE] = ACTIONS(1166),
    [anon_sym_DQUOTE] = ACTIONS(1166),
    [sym_true] = ACTIONS(1164),
    [sym_false] = ACTIONS(1164),
    [anon_sym_NULL] = ACTIONS(1164),
    [anon_sym_nullptr] = ACTIONS(1164),
    [sym_comment] = ACTIONS(3),
  },
  [91] = {
    [sym_identifier] = ACTIONS(1168),
    [aux_sym_preproc_include_token1] = ACTIONS(1168),
    [aux_sym_preproc_def_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token2] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1168),
    [aux_sym_preproc_else_token1] = ACTIONS(1168),
    [aux_sym_preproc_elif_token1] = ACTIONS(1168),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1168),
    [sym_preproc_directive] = ACTIONS(1168),
    [anon_sym_LPAREN2] = ACTIONS(1170),
    [anon_sym_BANG] = ACTIONS(1170),
    [anon_sym_TILDE] = ACTIONS(1170),
    [anon_sym_DASH] = ACTIONS(1168),
    [anon_sym_PLUS] = ACTIONS(1168),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1170),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym___extension__] = ACTIONS(1168),
    [anon_sym_typedef] = ACTIONS(1168),
    [anon_sym_extern] = ACTIONS(1168),
    [anon_sym___attribute__] = ACTIONS(1168),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1170),
    [anon_sym___declspec] = ACTIONS(1168),
    [anon_sym___cdecl] = ACTIONS(1168),
    [anon_sym___clrcall] = ACTIONS(1168),
    [anon_sym___stdcall] = ACTIONS(1168),
    [anon_sym___fastcall] = ACTIONS(1168),
    [anon_sym___thiscall] = ACTIONS(1168),
    [anon_sym___vectorcall] = ACTIONS(1168),
    [anon_sym_LBRACE] = ACTIONS(1170),
    [anon_sym_signed] = ACTIONS(1168),
    [anon_sym_unsigned] = ACTIONS(1168),
    [anon_sym_long] = ACTIONS(1168),
    [anon_sym_short] = ACTIONS(1168),
    [anon_sym_static] = ACTIONS(1168),
    [anon_sym_auto] = ACTIONS(1168),
    [anon_sym_register] = ACTIONS(1168),
    [anon_sym_inline] = ACTIONS(1168),
    [anon_sym___inline] = ACTIONS(1168),
    [anon_sym___inline__] = ACTIONS(1168),
    [anon_sym___forceinline] = ACTIONS(1168),
    [anon_sym_thread_local] = ACTIONS(1168),
    [anon_sym___thread] = ACTIONS(1168),
    [anon_sym_const] = ACTIONS(1168),
    [anon_sym_constexpr] = ACTIONS(1168),
    [anon_sym_volatile] = ACTIONS(1168),
    [anon_sym_restrict] = ACTIONS(1168),
    [anon_sym___restrict__] = ACTIONS(1168),
    [anon_sym__Atomic] = ACTIONS(1168),
    [anon_sym__Noreturn] = ACTIONS(1168),
    [anon_sym_noreturn] = ACTIONS(1168),
    [sym_primitive_type] = ACTIONS(1168),
    [anon_sym_enum] = ACTIONS(1168),
    [anon_sym_struct] = ACTIONS(1168),
    [anon_sym_union] = ACTIONS(1168),
    [anon_sym_if] = ACTIONS(1168),
    [anon_sym_else] = ACTIONS(1168),
    [anon_sym_switch] = ACTIONS(1168),
    [anon_sym_case] = ACTIONS(1168),
    [anon_sym_default] = ACTIONS(1168),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_do] = ACTIONS(1168),
    [anon_sym_for] = ACTIONS(1168),
    [anon_sym_return] = ACTIONS(1168),
    [anon_sym_break] = ACTIONS(1168),
    [anon_sym_continue] = ACTIONS(1168),
    [anon_sym_goto] = ACTIONS(1168),
    [anon_sym___try] = ACTIONS(1168),
    [anon_sym___leave] = ACTIONS(1168),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_sizeof] = ACTIONS(1168),
    [anon_sym___alignof__] = ACTIONS(1168),
    [anon_sym___alignof] = ACTIONS(1168),
    [anon_sym__alignof] = ACTIONS(1168),
    [anon_sym_alignof] = ACTIONS(1168),
    [anon_sym__Alignof] = ACTIONS(1168),
    [anon_sym_offsetof] = ACTIONS(1168),
    [anon_sym__Generic] = ACTIONS(1168),
    [anon_sym_asm] = ACTIONS(1168),
    [anon_sym___asm__] = ACTIONS(1168),
    [sym_number_literal] = ACTIONS(1170),
    [anon_sym_L_SQUOTE] = ACTIONS(1170),
    [anon_sym_u_SQUOTE] = ACTIONS(1170),
    [anon_sym_U_SQUOTE] = ACTIONS(1170),
    [anon_sym_u8_SQUOTE] = ACTIONS(1170),
    [anon_sym_SQUOTE] = ACTIONS(1170),
    [anon_sym_L_DQUOTE] = ACTIONS(1170),
    [anon_sym_u_DQUOTE] = ACTIONS(1170),
    [anon_sym_U_DQUOTE] = ACTIONS(1170),
    [anon_sym_u8_DQUOTE] = ACTIONS(1170),
    [anon_sym_DQUOTE] = ACTIONS(1170),
    [sym_true] = ACTIONS(1168),
    [sym_false] = ACTIONS(1168),
    [anon_sym_NULL] = ACTIONS(1168),
    [anon_sym_nullptr] = ACTIONS(1168),
    [sym_comment] = ACTIONS(3),
  },
  [92] = {
    [ts_builtin_sym_end] = ACTIONS(1130),
    [sym_identifier] = ACTIONS(1128),
    [aux_sym_preproc_include_token1] = ACTIONS(1128),
    [aux_sym_preproc_def_token1] = ACTIONS(1128),
    [anon_sym_COMMA] = ACTIONS(1130),
    [anon_sym_RPAREN] = ACTIONS(1130),
    [aux_sym_preproc_if_token1] = ACTIONS(1128),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1128),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1128),
    [sym_preproc_directive] = ACTIONS(1128),
    [anon_sym_LPAREN2] = ACTIONS(1130),
    [anon_sym_BANG] = ACTIONS(1130),
    [anon_sym_TILDE] = ACTIONS(1130),
    [anon_sym_DASH] = ACTIONS(1128),
    [anon_sym_PLUS] = ACTIONS(1128),
    [anon_sym_STAR] = ACTIONS(1130),
    [anon_sym_AMP] = ACTIONS(1130),
    [anon_sym_SEMI] = ACTIONS(1130),
    [anon_sym___extension__] = ACTIONS(1128),
    [anon_sym_typedef] = ACTIONS(1128),
    [anon_sym_extern] = ACTIONS(1128),
    [anon_sym___attribute__] = ACTIONS(1128),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1130),
    [anon_sym___declspec] = ACTIONS(1128),
    [anon_sym___cdecl] = ACTIONS(1128),
    [anon_sym___clrcall] = ACTIONS(1128),
    [anon_sym___stdcall] = ACTIONS(1128),
    [anon_sym___fastcall] = ACTIONS(1128),
    [anon_sym___thiscall] = ACTIONS(1128),
    [anon_sym___vectorcall] = ACTIONS(1128),
    [anon_sym_LBRACE] = ACTIONS(1130),
    [anon_sym_signed] = ACTIONS(1128),
    [anon_sym_unsigned] = ACTIONS(1128),
    [anon_sym_long] = ACTIONS(1128),
    [anon_sym_short] = ACTIONS(1128),
    [anon_sym_static] = ACTIONS(1128),
    [anon_sym_auto] = ACTIONS(1128),
    [anon_sym_register] = ACTIONS(1128),
    [anon_sym_inline] = ACTIONS(1128),
    [anon_sym___inline] = ACTIONS(1128),
    [anon_sym___inline__] = ACTIONS(1128),
    [anon_sym___forceinline] = ACTIONS(1128),
    [anon_sym_thread_local] = ACTIONS(1128),
    [anon_sym___thread] = ACTIONS(1128),
    [anon_sym_const] = ACTIONS(1128),
    [anon_sym_constexpr] = ACTIONS(1128),
    [anon_sym_volatile] = ACTIONS(1128),
    [anon_sym_restrict] = ACTIONS(1128),
    [anon_sym___restrict__] = ACTIONS(1128),
    [anon_sym__Atomic] = ACTIONS(1128),
    [anon_sym__Noreturn] = ACTIONS(1128),
    [anon_sym_noreturn] = ACTIONS(1128),
    [sym_primitive_type] = ACTIONS(1128),
    [anon_sym_enum] = ACTIONS(1128),
    [anon_sym_struct] = ACTIONS(1128),
    [anon_sym_union] = ACTIONS(1128),
    [anon_sym_if] = ACTIONS(1128),
    [anon_sym_else] = ACTIONS(1128),
    [anon_sym_switch] = ACTIONS(1128),
    [anon_sym_case] = ACTIONS(1128),
    [anon_sym_default] = ACTIONS(1128),
    [anon_sym_while] = ACTIONS(1128),
    [anon_sym_do] = ACTIONS(1128),
    [anon_sym_for] = ACTIONS(1128),
    [anon_sym_return] = ACTIONS(1128),
    [anon_sym_break] = ACTIONS(1128),
    [anon_sym_continue] = ACTIONS(1128),
    [anon_sym_goto] = ACTIONS(1128),
    [anon_sym___try] = ACTIONS(1128),
    [anon_sym___except] = ACTIONS(1128),
    [anon_sym___finally] = ACTIONS(1128),
    [anon_sym___leave] = ACTIONS(1128),
    [anon_sym_DASH_DASH] = ACTIONS(1130),
    [anon_sym_PLUS_PLUS] = ACTIONS(1130),
    [anon_sym_sizeof] = ACTIONS(1128),
    [anon_sym___alignof__] = ACTIONS(1128),
    [anon_sym___alignof] = ACTIONS(1128),
    [anon_sym__alignof] = ACTIONS(1128),
    [anon_sym_alignof] = ACTIONS(1128),
    [anon_sym__Alignof] = ACTIONS(1128),
    [anon_sym_offsetof] = ACTIONS(1128),
    [anon_sym__Generic] = ACTIONS(1128),
    [anon_sym_asm] = ACTIONS(1128),
    [anon_sym___asm__] = ACTIONS(1128),
    [sym_number_literal] = ACTIONS(1130),
    [anon_sym_L_SQUOTE] = ACTIONS(1130),
    [anon_sym_u_SQUOTE] = ACTIONS(1130),
    [anon_sym_U_SQUOTE] = ACTIONS(1130),
    [anon_sym_u8_SQUOTE] = ACTIONS(1130),
    [anon_sym_SQUOTE] = ACTIONS(1130),
    [anon_sym_L_DQUOTE] = ACTIONS(1130),
    [anon_sym_u_DQUOTE] = ACTIONS(1130),
    [anon_sym_U_DQUOTE] = ACTIONS(1130),
    [anon_sym_u8_DQUOTE] = ACTIONS(1130),
    [anon_sym_DQUOTE] = ACTIONS(1130),
    [sym_true] = ACTIONS(1128),
    [sym_false] = ACTIONS(1128),
    [anon_sym_NULL] = ACTIONS(1128),
    [anon_sym_nullptr] = ACTIONS(1128),
    [sym_comment] = ACTIONS(3),
  },
  [93] = {
    [sym_identifier] = ACTIONS(1168),
    [aux_sym_preproc_include_token1] = ACTIONS(1168),
    [aux_sym_preproc_def_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token2] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1168),
    [aux_sym_preproc_else_token1] = ACTIONS(1168),
    [aux_sym_preproc_elif_token1] = ACTIONS(1168),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1168),
    [sym_preproc_directive] = ACTIONS(1168),
    [anon_sym_LPAREN2] = ACTIONS(1170),
    [anon_sym_BANG] = ACTIONS(1170),
    [anon_sym_TILDE] = ACTIONS(1170),
    [anon_sym_DASH] = ACTIONS(1168),
    [anon_sym_PLUS] = ACTIONS(1168),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1170),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym___extension__] = ACTIONS(1168),
    [anon_sym_typedef] = ACTIONS(1168),
    [anon_sym_extern] = ACTIONS(1168),
    [anon_sym___attribute__] = ACTIONS(1168),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1170),
    [anon_sym___declspec] = ACTIONS(1168),
    [anon_sym___cdecl] = ACTIONS(1168),
    [anon_sym___clrcall] = ACTIONS(1168),
    [anon_sym___stdcall] = ACTIONS(1168),
    [anon_sym___fastcall] = ACTIONS(1168),
    [anon_sym___thiscall] = ACTIONS(1168),
    [anon_sym___vectorcall] = ACTIONS(1168),
    [anon_sym_LBRACE] = ACTIONS(1170),
    [anon_sym_signed] = ACTIONS(1168),
    [anon_sym_unsigned] = ACTIONS(1168),
    [anon_sym_long] = ACTIONS(1168),
    [anon_sym_short] = ACTIONS(1168),
    [anon_sym_static] = ACTIONS(1168),
    [anon_sym_auto] = ACTIONS(1168),
    [anon_sym_register] = ACTIONS(1168),
    [anon_sym_inline] = ACTIONS(1168),
    [anon_sym___inline] = ACTIONS(1168),
    [anon_sym___inline__] = ACTIONS(1168),
    [anon_sym___forceinline] = ACTIONS(1168),
    [anon_sym_thread_local] = ACTIONS(1168),
    [anon_sym___thread] = ACTIONS(1168),
    [anon_sym_const] = ACTIONS(1168),
    [anon_sym_constexpr] = ACTIONS(1168),
    [anon_sym_volatile] = ACTIONS(1168),
    [anon_sym_restrict] = ACTIONS(1168),
    [anon_sym___restrict__] = ACTIONS(1168),
    [anon_sym__Atomic] = ACTIONS(1168),
    [anon_sym__Noreturn] = ACTIONS(1168),
    [anon_sym_noreturn] = ACTIONS(1168),
    [sym_primitive_type] = ACTIONS(1168),
    [anon_sym_enum] = ACTIONS(1168),
    [anon_sym_struct] = ACTIONS(1168),
    [anon_sym_union] = ACTIONS(1168),
    [anon_sym_if] = ACTIONS(1168),
    [anon_sym_else] = ACTIONS(1168),
    [anon_sym_switch] = ACTIONS(1168),
    [anon_sym_case] = ACTIONS(1168),
    [anon_sym_default] = ACTIONS(1168),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_do] = ACTIONS(1168),
    [anon_sym_for] = ACTIONS(1168),
    [anon_sym_return] = ACTIONS(1168),
    [anon_sym_break] = ACTIONS(1168),
    [anon_sym_continue] = ACTIONS(1168),
    [anon_sym_goto] = ACTIONS(1168),
    [anon_sym___try] = ACTIONS(1168),
    [anon_sym___leave] = ACTIONS(1168),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_sizeof] = ACTIONS(1168),
    [anon_sym___alignof__] = ACTIONS(1168),
    [anon_sym___alignof] = ACTIONS(1168),
    [anon_sym__alignof] = ACTIONS(1168),
    [anon_sym_alignof] = ACTIONS(1168),
    [anon_sym__Alignof] = ACTIONS(1168),
    [anon_sym_offsetof] = ACTIONS(1168),
    [anon_sym__Generic] = ACTIONS(1168),
    [anon_sym_asm] = ACTIONS(1168),
    [anon_sym___asm__] = ACTIONS(1168),
    [sym_number_literal] = ACTIONS(1170),
    [anon_sym_L_SQUOTE] = ACTIONS(1170),
    [anon_sym_u_SQUOTE] = ACTIONS(1170),
    [anon_sym_U_SQUOTE] = ACTIONS(1170),
    [anon_sym_u8_SQUOTE] = ACTIONS(1170),
    [anon_sym_SQUOTE] = ACTIONS(1170),
    [anon_sym_L_DQUOTE] = ACTIONS(1170),
    [anon_sym_u_DQUOTE] = ACTIONS(1170),
    [anon_sym_U_DQUOTE] = ACTIONS(1170),
    [anon_sym_u8_DQUOTE] = ACTIONS(1170),
    [anon_sym_DQUOTE] = ACTIONS(1170),
    [sym_true] = ACTIONS(1168),
    [sym_false] = ACTIONS(1168),
    [anon_sym_NULL] = ACTIONS(1168),
    [anon_sym_nullptr] = ACTIONS(1168),
    [sym_comment] = ACTIONS(3),
  },
  [94] = {
    [sym_identifier] = ACTIONS(1172),
    [aux_sym_preproc_include_token1] = ACTIONS(1172),
    [aux_sym_preproc_def_token1] = ACTIONS(1172),
    [aux_sym_preproc_if_token1] = ACTIONS(1172),
    [aux_sym_preproc_if_token2] = ACTIONS(1172),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1172),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1172),
    [aux_sym_preproc_else_token1] = ACTIONS(1172),
    [aux_sym_preproc_elif_token1] = ACTIONS(1172),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1172),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1172),
    [sym_preproc_directive] = ACTIONS(1172),
    [anon_sym_LPAREN2] = ACTIONS(1174),
    [anon_sym_BANG] = ACTIONS(1174),
    [anon_sym_TILDE] = ACTIONS(1174),
    [anon_sym_DASH] = ACTIONS(1172),
    [anon_sym_PLUS] = ACTIONS(1172),
    [anon_sym_STAR] = ACTIONS(1174),
    [anon_sym_AMP] = ACTIONS(1174),
    [anon_sym_SEMI] = ACTIONS(1174),
    [anon_sym___extension__] = ACTIONS(1172),
    [anon_sym_typedef] = ACTIONS(1172),
    [anon_sym_extern] = ACTIONS(1172),
    [anon_sym___attribute__] = ACTIONS(1172),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1174),
    [anon_sym___declspec] = ACTIONS(1172),
    [anon_sym___cdecl] = ACTIONS(1172),
    [anon_sym___clrcall] = ACTIONS(1172),
    [anon_sym___stdcall] = ACTIONS(1172),
    [anon_sym___fastcall] = ACTIONS(1172),
    [anon_sym___thiscall] = ACTIONS(1172),
    [anon_sym___vectorcall] = ACTIONS(1172),
    [anon_sym_LBRACE] = ACTIONS(1174),
    [anon_sym_signed] = ACTIONS(1172),
    [anon_sym_unsigned] = ACTIONS(1172),
    [anon_sym_long] = ACTIONS(1172),
    [anon_sym_short] = ACTIONS(1172),
    [anon_sym_static] = ACTIONS(1172),
    [anon_sym_auto] = ACTIONS(1172),
    [anon_sym_register] = ACTIONS(1172),
    [anon_sym_inline] = ACTIONS(1172),
    [anon_sym___inline] = ACTIONS(1172),
    [anon_sym___inline__] = ACTIONS(1172),
    [anon_sym___forceinline] = ACTIONS(1172),
    [anon_sym_thread_local] = ACTIONS(1172),
    [anon_sym___thread] = ACTIONS(1172),
    [anon_sym_const] = ACTIONS(1172),
    [anon_sym_constexpr] = ACTIONS(1172),
    [anon_sym_volatile] = ACTIONS(1172),
    [anon_sym_restrict] = ACTIONS(1172),
    [anon_sym___restrict__] = ACTIONS(1172),
    [anon_sym__Atomic] = ACTIONS(1172),
    [anon_sym__Noreturn] = ACTIONS(1172),
    [anon_sym_noreturn] = ACTIONS(1172),
    [sym_primitive_type] = ACTIONS(1172),
    [anon_sym_enum] = ACTIONS(1172),
    [anon_sym_struct] = ACTIONS(1172),
    [anon_sym_union] = ACTIONS(1172),
    [anon_sym_if] = ACTIONS(1172),
    [anon_sym_else] = ACTIONS(1172),
    [anon_sym_switch] = ACTIONS(1172),
    [anon_sym_case] = ACTIONS(1172),
    [anon_sym_default] = ACTIONS(1172),
    [anon_sym_while] = ACTIONS(1172),
    [anon_sym_do] = ACTIONS(1172),
    [anon_sym_for] = ACTIONS(1172),
    [anon_sym_return] = ACTIONS(1172),
    [anon_sym_break] = ACTIONS(1172),
    [anon_sym_continue] = ACTIONS(1172),
    [anon_sym_goto] = ACTIONS(1172),
    [anon_sym___try] = ACTIONS(1172),
    [anon_sym___leave] = ACTIONS(1172),
    [anon_sym_DASH_DASH] = ACTIONS(1174),
    [anon_sym_PLUS_PLUS] = ACTIONS(1174),
    [anon_sym_sizeof] = ACTIONS(1172),
    [anon_sym___alignof__] = ACTIONS(1172),
    [anon_sym___alignof] = ACTIONS(1172),
    [anon_sym__alignof] = ACTIONS(1172),
    [anon_sym_alignof] = ACTIONS(1172),
    [anon_sym__Alignof] = ACTIONS(1172),
    [anon_sym_offsetof] = ACTIONS(1172),
    [anon_sym__Generic] = ACTIONS(1172),
    [anon_sym_asm] = ACTIONS(1172),
    [anon_sym___asm__] = ACTIONS(1172),
    [sym_number_literal] = ACTIONS(1174),
    [anon_sym_L_SQUOTE] = ACTIONS(1174),
    [anon_sym_u_SQUOTE] = ACTIONS(1174),
    [anon_sym_U_SQUOTE] = ACTIONS(1174),
    [anon_sym_u8_SQUOTE] = ACTIONS(1174),
    [anon_sym_SQUOTE] = ACTIONS(1174),
    [anon_sym_L_DQUOTE] = ACTIONS(1174),
    [anon_sym_u_DQUOTE] = ACTIONS(1174),
    [anon_sym_U_DQUOTE] = ACTIONS(1174),
    [anon_sym_u8_DQUOTE] = ACTIONS(1174),
    [anon_sym_DQUOTE] = ACTIONS(1174),
    [sym_true] = ACTIONS(1172),
    [sym_false] = ACTIONS(1172),
    [anon_sym_NULL] = ACTIONS(1172),
    [anon_sym_nullptr] = ACTIONS(1172),
    [sym_comment] = ACTIONS(3),
  },
  [95] = {
    [sym_identifier] = ACTIONS(1176),
    [aux_sym_preproc_include_token1] = ACTIONS(1176),
    [aux_sym_preproc_def_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token2] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1176),
    [aux_sym_preproc_else_token1] = ACTIONS(1176),
    [aux_sym_preproc_elif_token1] = ACTIONS(1176),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1176),
    [sym_preproc_directive] = ACTIONS(1176),
    [anon_sym_LPAREN2] = ACTIONS(1178),
    [anon_sym_BANG] = ACTIONS(1178),
    [anon_sym_TILDE] = ACTIONS(1178),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_STAR] = ACTIONS(1178),
    [anon_sym_AMP] = ACTIONS(1178),
    [anon_sym_SEMI] = ACTIONS(1178),
    [anon_sym___extension__] = ACTIONS(1176),
    [anon_sym_typedef] = ACTIONS(1176),
    [anon_sym_extern] = ACTIONS(1176),
    [anon_sym___attribute__] = ACTIONS(1176),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1178),
    [anon_sym___declspec] = ACTIONS(1176),
    [anon_sym___cdecl] = ACTIONS(1176),
    [anon_sym___clrcall] = ACTIONS(1176),
    [anon_sym___stdcall] = ACTIONS(1176),
    [anon_sym___fastcall] = ACTIONS(1176),
    [anon_sym___thiscall] = ACTIONS(1176),
    [anon_sym___vectorcall] = ACTIONS(1176),
    [anon_sym_LBRACE] = ACTIONS(1178),
    [anon_sym_signed] = ACTIONS(1176),
    [anon_sym_unsigned] = ACTIONS(1176),
    [anon_sym_long] = ACTIONS(1176),
    [anon_sym_short] = ACTIONS(1176),
    [anon_sym_static] = ACTIONS(1176),
    [anon_sym_auto] = ACTIONS(1176),
    [anon_sym_register] = ACTIONS(1176),
    [anon_sym_inline] = ACTIONS(1176),
    [anon_sym___inline] = ACTIONS(1176),
    [anon_sym___inline__] = ACTIONS(1176),
    [anon_sym___forceinline] = ACTIONS(1176),
    [anon_sym_thread_local] = ACTIONS(1176),
    [anon_sym___thread] = ACTIONS(1176),
    [anon_sym_const] = ACTIONS(1176),
    [anon_sym_constexpr] = ACTIONS(1176),
    [anon_sym_volatile] = ACTIONS(1176),
    [anon_sym_restrict] = ACTIONS(1176),
    [anon_sym___restrict__] = ACTIONS(1176),
    [anon_sym__Atomic] = ACTIONS(1176),
    [anon_sym__Noreturn] = ACTIONS(1176),
    [anon_sym_noreturn] = ACTIONS(1176),
    [sym_primitive_type] = ACTIONS(1176),
    [anon_sym_enum] = ACTIONS(1176),
    [anon_sym_struct] = ACTIONS(1176),
    [anon_sym_union] = ACTIONS(1176),
    [anon_sym_if] = ACTIONS(1176),
    [anon_sym_else] = ACTIONS(1176),
    [anon_sym_switch] = ACTIONS(1176),
    [anon_sym_case] = ACTIONS(1176),
    [anon_sym_default] = ACTIONS(1176),
    [anon_sym_while] = ACTIONS(1176),
    [anon_sym_do] = ACTIONS(1176),
    [anon_sym_for] = ACTIONS(1176),
    [anon_sym_return] = ACTIONS(1176),
    [anon_sym_break] = ACTIONS(1176),
    [anon_sym_continue] = ACTIONS(1176),
    [anon_sym_goto] = ACTIONS(1176),
    [anon_sym___try] = ACTIONS(1176),
    [anon_sym___leave] = ACTIONS(1176),
    [anon_sym_DASH_DASH] = ACTIONS(1178),
    [anon_sym_PLUS_PLUS] = ACTIONS(1178),
    [anon_sym_sizeof] = ACTIONS(1176),
    [anon_sym___alignof__] = ACTIONS(1176),
    [anon_sym___alignof] = ACTIONS(1176),
    [anon_sym__alignof] = ACTIONS(1176),
    [anon_sym_alignof] = ACTIONS(1176),
    [anon_sym__Alignof] = ACTIONS(1176),
    [anon_sym_offsetof] = ACTIONS(1176),
    [anon_sym__Generic] = ACTIONS(1176),
    [anon_sym_asm] = ACTIONS(1176),
    [anon_sym___asm__] = ACTIONS(1176),
    [sym_number_literal] = ACTIONS(1178),
    [anon_sym_L_SQUOTE] = ACTIONS(1178),
    [anon_sym_u_SQUOTE] = ACTIONS(1178),
    [anon_sym_U_SQUOTE] = ACTIONS(1178),
    [anon_sym_u8_SQUOTE] = ACTIONS(1178),
    [anon_sym_SQUOTE] = ACTIONS(1178),
    [anon_sym_L_DQUOTE] = ACTIONS(1178),
    [anon_sym_u_DQUOTE] = ACTIONS(1178),
    [anon_sym_U_DQUOTE] = ACTIONS(1178),
    [anon_sym_u8_DQUOTE] = ACTIONS(1178),
    [anon_sym_DQUOTE] = ACTIONS(1178),
    [sym_true] = ACTIONS(1176),
    [sym_false] = ACTIONS(1176),
    [anon_sym_NULL] = ACTIONS(1176),
    [anon_sym_nullptr] = ACTIONS(1176),
    [sym_comment] = ACTIONS(3),
  },
  [96] = {
    [sym_identifier] = ACTIONS(1180),
    [aux_sym_preproc_include_token1] = ACTIONS(1180),
    [aux_sym_preproc_def_token1] = ACTIONS(1180),
    [aux_sym_preproc_if_token1] = ACTIONS(1180),
    [aux_sym_preproc_if_token2] = ACTIONS(1180),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1180),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1180),
    [aux_sym_preproc_else_token1] = ACTIONS(1180),
    [aux_sym_preproc_elif_token1] = ACTIONS(1180),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1180),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1180),
    [sym_preproc_directive] = ACTIONS(1180),
    [anon_sym_LPAREN2] = ACTIONS(1182),
    [anon_sym_BANG] = ACTIONS(1182),
    [anon_sym_TILDE] = ACTIONS(1182),
    [anon_sym_DASH] = ACTIONS(1180),
    [anon_sym_PLUS] = ACTIONS(1180),
    [anon_sym_STAR] = ACTIONS(1182),
    [anon_sym_AMP] = ACTIONS(1182),
    [anon_sym_SEMI] = ACTIONS(1182),
    [anon_sym___extension__] = ACTIONS(1180),
    [anon_sym_typedef] = ACTIONS(1180),
    [anon_sym_extern] = ACTIONS(1180),
    [anon_sym___attribute__] = ACTIONS(1180),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1182),
    [anon_sym___declspec] = ACTIONS(1180),
    [anon_sym___cdecl] = ACTIONS(1180),
    [anon_sym___clrcall] = ACTIONS(1180),
    [anon_sym___stdcall] = ACTIONS(1180),
    [anon_sym___fastcall] = ACTIONS(1180),
    [anon_sym___thiscall] = ACTIONS(1180),
    [anon_sym___vectorcall] = ACTIONS(1180),
    [anon_sym_LBRACE] = ACTIONS(1182),
    [anon_sym_signed] = ACTIONS(1180),
    [anon_sym_unsigned] = ACTIONS(1180),
    [anon_sym_long] = ACTIONS(1180),
    [anon_sym_short] = ACTIONS(1180),
    [anon_sym_static] = ACTIONS(1180),
    [anon_sym_auto] = ACTIONS(1180),
    [anon_sym_register] = ACTIONS(1180),
    [anon_sym_inline] = ACTIONS(1180),
    [anon_sym___inline] = ACTIONS(1180),
    [anon_sym___inline__] = ACTIONS(1180),
    [anon_sym___forceinline] = ACTIONS(1180),
    [anon_sym_thread_local] = ACTIONS(1180),
    [anon_sym___thread] = ACTIONS(1180),
    [anon_sym_const] = ACTIONS(1180),
    [anon_sym_constexpr] = ACTIONS(1180),
    [anon_sym_volatile] = ACTIONS(1180),
    [anon_sym_restrict] = ACTIONS(1180),
    [anon_sym___restrict__] = ACTIONS(1180),
    [anon_sym__Atomic] = ACTIONS(1180),
    [anon_sym__Noreturn] = ACTIONS(1180),
    [anon_sym_noreturn] = ACTIONS(1180),
    [sym_primitive_type] = ACTIONS(1180),
    [anon_sym_enum] = ACTIONS(1180),
    [anon_sym_struct] = ACTIONS(1180),
    [anon_sym_union] = ACTIONS(1180),
    [anon_sym_if] = ACTIONS(1180),
    [anon_sym_else] = ACTIONS(1180),
    [anon_sym_switch] = ACTIONS(1180),
    [anon_sym_case] = ACTIONS(1180),
    [anon_sym_default] = ACTIONS(1180),
    [anon_sym_while] = ACTIONS(1180),
    [anon_sym_do] = ACTIONS(1180),
    [anon_sym_for] = ACTIONS(1180),
    [anon_sym_return] = ACTIONS(1180),
    [anon_sym_break] = ACTIONS(1180),
    [anon_sym_continue] = ACTIONS(1180),
    [anon_sym_goto] = ACTIONS(1180),
    [anon_sym___try] = ACTIONS(1180),
    [anon_sym___leave] = ACTIONS(1180),
    [anon_sym_DASH_DASH] = ACTIONS(1182),
    [anon_sym_PLUS_PLUS] = ACTIONS(1182),
    [anon_sym_sizeof] = ACTIONS(1180),
    [anon_sym___alignof__] = ACTIONS(1180),
    [anon_sym___alignof] = ACTIONS(1180),
    [anon_sym__alignof] = ACTIONS(1180),
    [anon_sym_alignof] = ACTIONS(1180),
    [anon_sym__Alignof] = ACTIONS(1180),
    [anon_sym_offsetof] = ACTIONS(1180),
    [anon_sym__Generic] = ACTIONS(1180),
    [anon_sym_asm] = ACTIONS(1180),
    [anon_sym___asm__] = ACTIONS(1180),
    [sym_number_literal] = ACTIONS(1182),
    [anon_sym_L_SQUOTE] = ACTIONS(1182),
    [anon_sym_u_SQUOTE] = ACTIONS(1182),
    [anon_sym_U_SQUOTE] = ACTIONS(1182),
    [anon_sym_u8_SQUOTE] = ACTIONS(1182),
    [anon_sym_SQUOTE] = ACTIONS(1182),
    [anon_sym_L_DQUOTE] = ACTIONS(1182),
    [anon_sym_u_DQUOTE] = ACTIONS(1182),
    [anon_sym_U_DQUOTE] = ACTIONS(1182),
    [anon_sym_u8_DQUOTE] = ACTIONS(1182),
    [anon_sym_DQUOTE] = ACTIONS(1182),
    [sym_true] = ACTIONS(1180),
    [sym_false] = ACTIONS(1180),
    [anon_sym_NULL] = ACTIONS(1180),
    [anon_sym_nullptr] = ACTIONS(1180),
    [sym_comment] = ACTIONS(3),
  },
  [97] = {
    [sym_identifier] = ACTIONS(1184),
    [aux_sym_preproc_include_token1] = ACTIONS(1184),
    [aux_sym_preproc_def_token1] = ACTIONS(1184),
    [aux_sym_preproc_if_token1] = ACTIONS(1184),
    [aux_sym_preproc_if_token2] = ACTIONS(1184),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1184),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1184),
    [aux_sym_preproc_else_token1] = ACTIONS(1184),
    [aux_sym_preproc_elif_token1] = ACTIONS(1184),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1184),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1184),
    [sym_preproc_directive] = ACTIONS(1184),
    [anon_sym_LPAREN2] = ACTIONS(1186),
    [anon_sym_BANG] = ACTIONS(1186),
    [anon_sym_TILDE] = ACTIONS(1186),
    [anon_sym_DASH] = ACTIONS(1184),
    [anon_sym_PLUS] = ACTIONS(1184),
    [anon_sym_STAR] = ACTIONS(1186),
    [anon_sym_AMP] = ACTIONS(1186),
    [anon_sym_SEMI] = ACTIONS(1186),
    [anon_sym___extension__] = ACTIONS(1184),
    [anon_sym_typedef] = ACTIONS(1184),
    [anon_sym_extern] = ACTIONS(1184),
    [anon_sym___attribute__] = ACTIONS(1184),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1186),
    [anon_sym___declspec] = ACTIONS(1184),
    [anon_sym___cdecl] = ACTIONS(1184),
    [anon_sym___clrcall] = ACTIONS(1184),
    [anon_sym___stdcall] = ACTIONS(1184),
    [anon_sym___fastcall] = ACTIONS(1184),
    [anon_sym___thiscall] = ACTIONS(1184),
    [anon_sym___vectorcall] = ACTIONS(1184),
    [anon_sym_LBRACE] = ACTIONS(1186),
    [anon_sym_signed] = ACTIONS(1184),
    [anon_sym_unsigned] = ACTIONS(1184),
    [anon_sym_long] = ACTIONS(1184),
    [anon_sym_short] = ACTIONS(1184),
    [anon_sym_static] = ACTIONS(1184),
    [anon_sym_auto] = ACTIONS(1184),
    [anon_sym_register] = ACTIONS(1184),
    [anon_sym_inline] = ACTIONS(1184),
    [anon_sym___inline] = ACTIONS(1184),
    [anon_sym___inline__] = ACTIONS(1184),
    [anon_sym___forceinline] = ACTIONS(1184),
    [anon_sym_thread_local] = ACTIONS(1184),
    [anon_sym___thread] = ACTIONS(1184),
    [anon_sym_const] = ACTIONS(1184),
    [anon_sym_constexpr] = ACTIONS(1184),
    [anon_sym_volatile] = ACTIONS(1184),
    [anon_sym_restrict] = ACTIONS(1184),
    [anon_sym___restrict__] = ACTIONS(1184),
    [anon_sym__Atomic] = ACTIONS(1184),
    [anon_sym__Noreturn] = ACTIONS(1184),
    [anon_sym_noreturn] = ACTIONS(1184),
    [sym_primitive_type] = ACTIONS(1184),
    [anon_sym_enum] = ACTIONS(1184),
    [anon_sym_struct] = ACTIONS(1184),
    [anon_sym_union] = ACTIONS(1184),
    [anon_sym_if] = ACTIONS(1184),
    [anon_sym_else] = ACTIONS(1184),
    [anon_sym_switch] = ACTIONS(1184),
    [anon_sym_case] = ACTIONS(1184),
    [anon_sym_default] = ACTIONS(1184),
    [anon_sym_while] = ACTIONS(1184),
    [anon_sym_do] = ACTIONS(1184),
    [anon_sym_for] = ACTIONS(1184),
    [anon_sym_return] = ACTIONS(1184),
    [anon_sym_break] = ACTIONS(1184),
    [anon_sym_continue] = ACTIONS(1184),
    [anon_sym_goto] = ACTIONS(1184),
    [anon_sym___try] = ACTIONS(1184),
    [anon_sym___leave] = ACTIONS(1184),
    [anon_sym_DASH_DASH] = ACTIONS(1186),
    [anon_sym_PLUS_PLUS] = ACTIONS(1186),
    [anon_sym_sizeof] = ACTIONS(1184),
    [anon_sym___alignof__] = ACTIONS(1184),
    [anon_sym___alignof] = ACTIONS(1184),
    [anon_sym__alignof] = ACTIONS(1184),
    [anon_sym_alignof] = ACTIONS(1184),
    [anon_sym__Alignof] = ACTIONS(1184),
    [anon_sym_offsetof] = ACTIONS(1184),
    [anon_sym__Generic] = ACTIONS(1184),
    [anon_sym_asm] = ACTIONS(1184),
    [anon_sym___asm__] = ACTIONS(1184),
    [sym_number_literal] = ACTIONS(1186),
    [anon_sym_L_SQUOTE] = ACTIONS(1186),
    [anon_sym_u_SQUOTE] = ACTIONS(1186),
    [anon_sym_U_SQUOTE] = ACTIONS(1186),
    [anon_sym_u8_SQUOTE] = ACTIONS(1186),
    [anon_sym_SQUOTE] = ACTIONS(1186),
    [anon_sym_L_DQUOTE] = ACTIONS(1186),
    [anon_sym_u_DQUOTE] = ACTIONS(1186),
    [anon_sym_U_DQUOTE] = ACTIONS(1186),
    [anon_sym_u8_DQUOTE] = ACTIONS(1186),
    [anon_sym_DQUOTE] = ACTIONS(1186),
    [sym_true] = ACTIONS(1184),
    [sym_false] = ACTIONS(1184),
    [anon_sym_NULL] = ACTIONS(1184),
    [anon_sym_nullptr] = ACTIONS(1184),
    [sym_comment] = ACTIONS(3),
  },
  [98] = {
    [sym_identifier] = ACTIONS(1188),
    [aux_sym_preproc_include_token1] = ACTIONS(1188),
    [aux_sym_preproc_def_token1] = ACTIONS(1188),
    [aux_sym_preproc_if_token1] = ACTIONS(1188),
    [aux_sym_preproc_if_token2] = ACTIONS(1188),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1188),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1188),
    [aux_sym_preproc_else_token1] = ACTIONS(1188),
    [aux_sym_preproc_elif_token1] = ACTIONS(1188),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1188),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1188),
    [sym_preproc_directive] = ACTIONS(1188),
    [anon_sym_LPAREN2] = ACTIONS(1190),
    [anon_sym_BANG] = ACTIONS(1190),
    [anon_sym_TILDE] = ACTIONS(1190),
    [anon_sym_DASH] = ACTIONS(1188),
    [anon_sym_PLUS] = ACTIONS(1188),
    [anon_sym_STAR] = ACTIONS(1190),
    [anon_sym_AMP] = ACTIONS(1190),
    [anon_sym_SEMI] = ACTIONS(1190),
    [anon_sym___extension__] = ACTIONS(1188),
    [anon_sym_typedef] = ACTIONS(1188),
    [anon_sym_extern] = ACTIONS(1188),
    [anon_sym___attribute__] = ACTIONS(1188),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1190),
    [anon_sym___declspec] = ACTIONS(1188),
    [anon_sym___cdecl] = ACTIONS(1188),
    [anon_sym___clrcall] = ACTIONS(1188),
    [anon_sym___stdcall] = ACTIONS(1188),
    [anon_sym___fastcall] = ACTIONS(1188),
    [anon_sym___thiscall] = ACTIONS(1188),
    [anon_sym___vectorcall] = ACTIONS(1188),
    [anon_sym_LBRACE] = ACTIONS(1190),
    [anon_sym_signed] = ACTIONS(1188),
    [anon_sym_unsigned] = ACTIONS(1188),
    [anon_sym_long] = ACTIONS(1188),
    [anon_sym_short] = ACTIONS(1188),
    [anon_sym_static] = ACTIONS(1188),
    [anon_sym_auto] = ACTIONS(1188),
    [anon_sym_register] = ACTIONS(1188),
    [anon_sym_inline] = ACTIONS(1188),
    [anon_sym___inline] = ACTIONS(1188),
    [anon_sym___inline__] = ACTIONS(1188),
    [anon_sym___forceinline] = ACTIONS(1188),
    [anon_sym_thread_local] = ACTIONS(1188),
    [anon_sym___thread] = ACTIONS(1188),
    [anon_sym_const] = ACTIONS(1188),
    [anon_sym_constexpr] = ACTIONS(1188),
    [anon_sym_volatile] = ACTIONS(1188),
    [anon_sym_restrict] = ACTIONS(1188),
    [anon_sym___restrict__] = ACTIONS(1188),
    [anon_sym__Atomic] = ACTIONS(1188),
    [anon_sym__Noreturn] = ACTIONS(1188),
    [anon_sym_noreturn] = ACTIONS(1188),
    [sym_primitive_type] = ACTIONS(1188),
    [anon_sym_enum] = ACTIONS(1188),
    [anon_sym_struct] = ACTIONS(1188),
    [anon_sym_union] = ACTIONS(1188),
    [anon_sym_if] = ACTIONS(1188),
    [anon_sym_else] = ACTIONS(1188),
    [anon_sym_switch] = ACTIONS(1188),
    [anon_sym_case] = ACTIONS(1188),
    [anon_sym_default] = ACTIONS(1188),
    [anon_sym_while] = ACTIONS(1188),
    [anon_sym_do] = ACTIONS(1188),
    [anon_sym_for] = ACTIONS(1188),
    [anon_sym_return] = ACTIONS(1188),
    [anon_sym_break] = ACTIONS(1188),
    [anon_sym_continue] = ACTIONS(1188),
    [anon_sym_goto] = ACTIONS(1188),
    [anon_sym___try] = ACTIONS(1188),
    [anon_sym___leave] = ACTIONS(1188),
    [anon_sym_DASH_DASH] = ACTIONS(1190),
    [anon_sym_PLUS_PLUS] = ACTIONS(1190),
    [anon_sym_sizeof] = ACTIONS(1188),
    [anon_sym___alignof__] = ACTIONS(1188),
    [anon_sym___alignof] = ACTIONS(1188),
    [anon_sym__alignof] = ACTIONS(1188),
    [anon_sym_alignof] = ACTIONS(1188),
    [anon_sym__Alignof] = ACTIONS(1188),
    [anon_sym_offsetof] = ACTIONS(1188),
    [anon_sym__Generic] = ACTIONS(1188),
    [anon_sym_asm] = ACTIONS(1188),
    [anon_sym___asm__] = ACTIONS(1188),
    [sym_number_literal] = ACTIONS(1190),
    [anon_sym_L_SQUOTE] = ACTIONS(1190),
    [anon_sym_u_SQUOTE] = ACTIONS(1190),
    [anon_sym_U_SQUOTE] = ACTIONS(1190),
    [anon_sym_u8_SQUOTE] = ACTIONS(1190),
    [anon_sym_SQUOTE] = ACTIONS(1190),
    [anon_sym_L_DQUOTE] = ACTIONS(1190),
    [anon_sym_u_DQUOTE] = ACTIONS(1190),
    [anon_sym_U_DQUOTE] = ACTIONS(1190),
    [anon_sym_u8_DQUOTE] = ACTIONS(1190),
    [anon_sym_DQUOTE] = ACTIONS(1190),
    [sym_true] = ACTIONS(1188),
    [sym_false] = ACTIONS(1188),
    [anon_sym_NULL] = ACTIONS(1188),
    [anon_sym_nullptr] = ACTIONS(1188),
    [sym_comment] = ACTIONS(3),
  },
  [99] = {
    [sym_identifier] = ACTIONS(1192),
    [aux_sym_preproc_include_token1] = ACTIONS(1192),
    [aux_sym_preproc_def_token1] = ACTIONS(1192),
    [aux_sym_preproc_if_token1] = ACTIONS(1192),
    [aux_sym_preproc_if_token2] = ACTIONS(1192),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1192),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1192),
    [aux_sym_preproc_else_token1] = ACTIONS(1192),
    [aux_sym_preproc_elif_token1] = ACTIONS(1192),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1192),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1192),
    [sym_preproc_directive] = ACTIONS(1192),
    [anon_sym_LPAREN2] = ACTIONS(1194),
    [anon_sym_BANG] = ACTIONS(1194),
    [anon_sym_TILDE] = ACTIONS(1194),
    [anon_sym_DASH] = ACTIONS(1192),
    [anon_sym_PLUS] = ACTIONS(1192),
    [anon_sym_STAR] = ACTIONS(1194),
    [anon_sym_AMP] = ACTIONS(1194),
    [anon_sym_SEMI] = ACTIONS(1194),
    [anon_sym___extension__] = ACTIONS(1192),
    [anon_sym_typedef] = ACTIONS(1192),
    [anon_sym_extern] = ACTIONS(1192),
    [anon_sym___attribute__] = ACTIONS(1192),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1194),
    [anon_sym___declspec] = ACTIONS(1192),
    [anon_sym___cdecl] = ACTIONS(1192),
    [anon_sym___clrcall] = ACTIONS(1192),
    [anon_sym___stdcall] = ACTIONS(1192),
    [anon_sym___fastcall] = ACTIONS(1192),
    [anon_sym___thiscall] = ACTIONS(1192),
    [anon_sym___vectorcall] = ACTIONS(1192),
    [anon_sym_LBRACE] = ACTIONS(1194),
    [anon_sym_signed] = ACTIONS(1192),
    [anon_sym_unsigned] = ACTIONS(1192),
    [anon_sym_long] = ACTIONS(1192),
    [anon_sym_short] = ACTIONS(1192),
    [anon_sym_static] = ACTIONS(1192),
    [anon_sym_auto] = ACTIONS(1192),
    [anon_sym_register] = ACTIONS(1192),
    [anon_sym_inline] = ACTIONS(1192),
    [anon_sym___inline] = ACTIONS(1192),
    [anon_sym___inline__] = ACTIONS(1192),
    [anon_sym___forceinline] = ACTIONS(1192),
    [anon_sym_thread_local] = ACTIONS(1192),
    [anon_sym___thread] = ACTIONS(1192),
    [anon_sym_const] = ACTIONS(1192),
    [anon_sym_constexpr] = ACTIONS(1192),
    [anon_sym_volatile] = ACTIONS(1192),
    [anon_sym_restrict] = ACTIONS(1192),
    [anon_sym___restrict__] = ACTIONS(1192),
    [anon_sym__Atomic] = ACTIONS(1192),
    [anon_sym__Noreturn] = ACTIONS(1192),
    [anon_sym_noreturn] = ACTIONS(1192),
    [sym_primitive_type] = ACTIONS(1192),
    [anon_sym_enum] = ACTIONS(1192),
    [anon_sym_struct] = ACTIONS(1192),
    [anon_sym_union] = ACTIONS(1192),
    [anon_sym_if] = ACTIONS(1192),
    [anon_sym_else] = ACTIONS(1192),
    [anon_sym_switch] = ACTIONS(1192),
    [anon_sym_case] = ACTIONS(1192),
    [anon_sym_default] = ACTIONS(1192),
    [anon_sym_while] = ACTIONS(1192),
    [anon_sym_do] = ACTIONS(1192),
    [anon_sym_for] = ACTIONS(1192),
    [anon_sym_return] = ACTIONS(1192),
    [anon_sym_break] = ACTIONS(1192),
    [anon_sym_continue] = ACTIONS(1192),
    [anon_sym_goto] = ACTIONS(1192),
    [anon_sym___try] = ACTIONS(1192),
    [anon_sym___leave] = ACTIONS(1192),
    [anon_sym_DASH_DASH] = ACTIONS(1194),
    [anon_sym_PLUS_PLUS] = ACTIONS(1194),
    [anon_sym_sizeof] = ACTIONS(1192),
    [anon_sym___alignof__] = ACTIONS(1192),
    [anon_sym___alignof] = ACTIONS(1192),
    [anon_sym__alignof] = ACTIONS(1192),
    [anon_sym_alignof] = ACTIONS(1192),
    [anon_sym__Alignof] = ACTIONS(1192),
    [anon_sym_offsetof] = ACTIONS(1192),
    [anon_sym__Generic] = ACTIONS(1192),
    [anon_sym_asm] = ACTIONS(1192),
    [anon_sym___asm__] = ACTIONS(1192),
    [sym_number_literal] = ACTIONS(1194),
    [anon_sym_L_SQUOTE] = ACTIONS(1194),
    [anon_sym_u_SQUOTE] = ACTIONS(1194),
    [anon_sym_U_SQUOTE] = ACTIONS(1194),
    [anon_sym_u8_SQUOTE] = ACTIONS(1194),
    [anon_sym_SQUOTE] = ACTIONS(1194),
    [anon_sym_L_DQUOTE] = ACTIONS(1194),
    [anon_sym_u_DQUOTE] = ACTIONS(1194),
    [anon_sym_U_DQUOTE] = ACTIONS(1194),
    [anon_sym_u8_DQUOTE] = ACTIONS(1194),
    [anon_sym_DQUOTE] = ACTIONS(1194),
    [sym_true] = ACTIONS(1192),
    [sym_false] = ACTIONS(1192),
    [anon_sym_NULL] = ACTIONS(1192),
    [anon_sym_nullptr] = ACTIONS(1192),
    [sym_comment] = ACTIONS(3),
  },
  [100] = {
    [sym_identifier] = ACTIONS(1176),
    [aux_sym_preproc_include_token1] = ACTIONS(1176),
    [aux_sym_preproc_def_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token2] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1176),
    [aux_sym_preproc_else_token1] = ACTIONS(1176),
    [aux_sym_preproc_elif_token1] = ACTIONS(1176),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1176),
    [sym_preproc_directive] = ACTIONS(1176),
    [anon_sym_LPAREN2] = ACTIONS(1178),
    [anon_sym_BANG] = ACTIONS(1178),
    [anon_sym_TILDE] = ACTIONS(1178),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_STAR] = ACTIONS(1178),
    [anon_sym_AMP] = ACTIONS(1178),
    [anon_sym_SEMI] = ACTIONS(1178),
    [anon_sym___extension__] = ACTIONS(1176),
    [anon_sym_typedef] = ACTIONS(1176),
    [anon_sym_extern] = ACTIONS(1176),
    [anon_sym___attribute__] = ACTIONS(1176),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1178),
    [anon_sym___declspec] = ACTIONS(1176),
    [anon_sym___cdecl] = ACTIONS(1176),
    [anon_sym___clrcall] = ACTIONS(1176),
    [anon_sym___stdcall] = ACTIONS(1176),
    [anon_sym___fastcall] = ACTIONS(1176),
    [anon_sym___thiscall] = ACTIONS(1176),
    [anon_sym___vectorcall] = ACTIONS(1176),
    [anon_sym_LBRACE] = ACTIONS(1178),
    [anon_sym_signed] = ACTIONS(1176),
    [anon_sym_unsigned] = ACTIONS(1176),
    [anon_sym_long] = ACTIONS(1176),
    [anon_sym_short] = ACTIONS(1176),
    [anon_sym_static] = ACTIONS(1176),
    [anon_sym_auto] = ACTIONS(1176),
    [anon_sym_register] = ACTIONS(1176),
    [anon_sym_inline] = ACTIONS(1176),
    [anon_sym___inline] = ACTIONS(1176),
    [anon_sym___inline__] = ACTIONS(1176),
    [anon_sym___forceinline] = ACTIONS(1176),
    [anon_sym_thread_local] = ACTIONS(1176),
    [anon_sym___thread] = ACTIONS(1176),
    [anon_sym_const] = ACTIONS(1176),
    [anon_sym_constexpr] = ACTIONS(1176),
    [anon_sym_volatile] = ACTIONS(1176),
    [anon_sym_restrict] = ACTIONS(1176),
    [anon_sym___restrict__] = ACTIONS(1176),
    [anon_sym__Atomic] = ACTIONS(1176),
    [anon_sym__Noreturn] = ACTIONS(1176),
    [anon_sym_noreturn] = ACTIONS(1176),
    [sym_primitive_type] = ACTIONS(1176),
    [anon_sym_enum] = ACTIONS(1176),
    [anon_sym_struct] = ACTIONS(1176),
    [anon_sym_union] = ACTIONS(1176),
    [anon_sym_if] = ACTIONS(1176),
    [anon_sym_else] = ACTIONS(1176),
    [anon_sym_switch] = ACTIONS(1176),
    [anon_sym_case] = ACTIONS(1176),
    [anon_sym_default] = ACTIONS(1176),
    [anon_sym_while] = ACTIONS(1176),
    [anon_sym_do] = ACTIONS(1176),
    [anon_sym_for] = ACTIONS(1176),
    [anon_sym_return] = ACTIONS(1176),
    [anon_sym_break] = ACTIONS(1176),
    [anon_sym_continue] = ACTIONS(1176),
    [anon_sym_goto] = ACTIONS(1176),
    [anon_sym___try] = ACTIONS(1176),
    [anon_sym___leave] = ACTIONS(1176),
    [anon_sym_DASH_DASH] = ACTIONS(1178),
    [anon_sym_PLUS_PLUS] = ACTIONS(1178),
    [anon_sym_sizeof] = ACTIONS(1176),
    [anon_sym___alignof__] = ACTIONS(1176),
    [anon_sym___alignof] = ACTIONS(1176),
    [anon_sym__alignof] = ACTIONS(1176),
    [anon_sym_alignof] = ACTIONS(1176),
    [anon_sym__Alignof] = ACTIONS(1176),
    [anon_sym_offsetof] = ACTIONS(1176),
    [anon_sym__Generic] = ACTIONS(1176),
    [anon_sym_asm] = ACTIONS(1176),
    [anon_sym___asm__] = ACTIONS(1176),
    [sym_number_literal] = ACTIONS(1178),
    [anon_sym_L_SQUOTE] = ACTIONS(1178),
    [anon_sym_u_SQUOTE] = ACTIONS(1178),
    [anon_sym_U_SQUOTE] = ACTIONS(1178),
    [anon_sym_u8_SQUOTE] = ACTIONS(1178),
    [anon_sym_SQUOTE] = ACTIONS(1178),
    [anon_sym_L_DQUOTE] = ACTIONS(1178),
    [anon_sym_u_DQUOTE] = ACTIONS(1178),
    [anon_sym_U_DQUOTE] = ACTIONS(1178),
    [anon_sym_u8_DQUOTE] = ACTIONS(1178),
    [anon_sym_DQUOTE] = ACTIONS(1178),
    [sym_true] = ACTIONS(1176),
    [sym_false] = ACTIONS(1176),
    [anon_sym_NULL] = ACTIONS(1176),
    [anon_sym_nullptr] = ACTIONS(1176),
    [sym_comment] = ACTIONS(3),
  },
  [101] = {
    [sym_identifier] = ACTIONS(1196),
    [aux_sym_preproc_include_token1] = ACTIONS(1196),
    [aux_sym_preproc_def_token1] = ACTIONS(1196),
    [aux_sym_preproc_if_token1] = ACTIONS(1196),
    [aux_sym_preproc_if_token2] = ACTIONS(1196),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1196),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1196),
    [aux_sym_preproc_else_token1] = ACTIONS(1196),
    [aux_sym_preproc_elif_token1] = ACTIONS(1196),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1196),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1196),
    [sym_preproc_directive] = ACTIONS(1196),
    [anon_sym_LPAREN2] = ACTIONS(1198),
    [anon_sym_BANG] = ACTIONS(1198),
    [anon_sym_TILDE] = ACTIONS(1198),
    [anon_sym_DASH] = ACTIONS(1196),
    [anon_sym_PLUS] = ACTIONS(1196),
    [anon_sym_STAR] = ACTIONS(1198),
    [anon_sym_AMP] = ACTIONS(1198),
    [anon_sym_SEMI] = ACTIONS(1198),
    [anon_sym___extension__] = ACTIONS(1196),
    [anon_sym_typedef] = ACTIONS(1196),
    [anon_sym_extern] = ACTIONS(1196),
    [anon_sym___attribute__] = ACTIONS(1196),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1198),
    [anon_sym___declspec] = ACTIONS(1196),
    [anon_sym___cdecl] = ACTIONS(1196),
    [anon_sym___clrcall] = ACTIONS(1196),
    [anon_sym___stdcall] = ACTIONS(1196),
    [anon_sym___fastcall] = ACTIONS(1196),
    [anon_sym___thiscall] = ACTIONS(1196),
    [anon_sym___vectorcall] = ACTIONS(1196),
    [anon_sym_LBRACE] = ACTIONS(1198),
    [anon_sym_signed] = ACTIONS(1196),
    [anon_sym_unsigned] = ACTIONS(1196),
    [anon_sym_long] = ACTIONS(1196),
    [anon_sym_short] = ACTIONS(1196),
    [anon_sym_static] = ACTIONS(1196),
    [anon_sym_auto] = ACTIONS(1196),
    [anon_sym_register] = ACTIONS(1196),
    [anon_sym_inline] = ACTIONS(1196),
    [anon_sym___inline] = ACTIONS(1196),
    [anon_sym___inline__] = ACTIONS(1196),
    [anon_sym___forceinline] = ACTIONS(1196),
    [anon_sym_thread_local] = ACTIONS(1196),
    [anon_sym___thread] = ACTIONS(1196),
    [anon_sym_const] = ACTIONS(1196),
    [anon_sym_constexpr] = ACTIONS(1196),
    [anon_sym_volatile] = ACTIONS(1196),
    [anon_sym_restrict] = ACTIONS(1196),
    [anon_sym___restrict__] = ACTIONS(1196),
    [anon_sym__Atomic] = ACTIONS(1196),
    [anon_sym__Noreturn] = ACTIONS(1196),
    [anon_sym_noreturn] = ACTIONS(1196),
    [sym_primitive_type] = ACTIONS(1196),
    [anon_sym_enum] = ACTIONS(1196),
    [anon_sym_struct] = ACTIONS(1196),
    [anon_sym_union] = ACTIONS(1196),
    [anon_sym_if] = ACTIONS(1196),
    [anon_sym_else] = ACTIONS(1196),
    [anon_sym_switch] = ACTIONS(1196),
    [anon_sym_case] = ACTIONS(1196),
    [anon_sym_default] = ACTIONS(1196),
    [anon_sym_while] = ACTIONS(1196),
    [anon_sym_do] = ACTIONS(1196),
    [anon_sym_for] = ACTIONS(1196),
    [anon_sym_return] = ACTIONS(1196),
    [anon_sym_break] = ACTIONS(1196),
    [anon_sym_continue] = ACTIONS(1196),
    [anon_sym_goto] = ACTIONS(1196),
    [anon_sym___try] = ACTIONS(1196),
    [anon_sym___leave] = ACTIONS(1196),
    [anon_sym_DASH_DASH] = ACTIONS(1198),
    [anon_sym_PLUS_PLUS] = ACTIONS(1198),
    [anon_sym_sizeof] = ACTIONS(1196),
    [anon_sym___alignof__] = ACTIONS(1196),
    [anon_sym___alignof] = ACTIONS(1196),
    [anon_sym__alignof] = ACTIONS(1196),
    [anon_sym_alignof] = ACTIONS(1196),
    [anon_sym__Alignof] = ACTIONS(1196),
    [anon_sym_offsetof] = ACTIONS(1196),
    [anon_sym__Generic] = ACTIONS(1196),
    [anon_sym_asm] = ACTIONS(1196),
    [anon_sym___asm__] = ACTIONS(1196),
    [sym_number_literal] = ACTIONS(1198),
    [anon_sym_L_SQUOTE] = ACTIONS(1198),
    [anon_sym_u_SQUOTE] = ACTIONS(1198),
    [anon_sym_U_SQUOTE] = ACTIONS(1198),
    [anon_sym_u8_SQUOTE] = ACTIONS(1198),
    [anon_sym_SQUOTE] = ACTIONS(1198),
    [anon_sym_L_DQUOTE] = ACTIONS(1198),
    [anon_sym_u_DQUOTE] = ACTIONS(1198),
    [anon_sym_U_DQUOTE] = ACTIONS(1198),
    [anon_sym_u8_DQUOTE] = ACTIONS(1198),
    [anon_sym_DQUOTE] = ACTIONS(1198),
    [sym_true] = ACTIONS(1196),
    [sym_false] = ACTIONS(1196),
    [anon_sym_NULL] = ACTIONS(1196),
    [anon_sym_nullptr] = ACTIONS(1196),
    [sym_comment] = ACTIONS(3),
  },
  [102] = {
    [sym_identifier] = ACTIONS(1200),
    [aux_sym_preproc_include_token1] = ACTIONS(1200),
    [aux_sym_preproc_def_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token2] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1200),
    [aux_sym_preproc_else_token1] = ACTIONS(1200),
    [aux_sym_preproc_elif_token1] = ACTIONS(1200),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1200),
    [sym_preproc_directive] = ACTIONS(1200),
    [anon_sym_LPAREN2] = ACTIONS(1202),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_DASH] = ACTIONS(1200),
    [anon_sym_PLUS] = ACTIONS(1200),
    [anon_sym_STAR] = ACTIONS(1202),
    [anon_sym_AMP] = ACTIONS(1202),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym___extension__] = ACTIONS(1200),
    [anon_sym_typedef] = ACTIONS(1200),
    [anon_sym_extern] = ACTIONS(1200),
    [anon_sym___attribute__] = ACTIONS(1200),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1202),
    [anon_sym___declspec] = ACTIONS(1200),
    [anon_sym___cdecl] = ACTIONS(1200),
    [anon_sym___clrcall] = ACTIONS(1200),
    [anon_sym___stdcall] = ACTIONS(1200),
    [anon_sym___fastcall] = ACTIONS(1200),
    [anon_sym___thiscall] = ACTIONS(1200),
    [anon_sym___vectorcall] = ACTIONS(1200),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_signed] = ACTIONS(1200),
    [anon_sym_unsigned] = ACTIONS(1200),
    [anon_sym_long] = ACTIONS(1200),
    [anon_sym_short] = ACTIONS(1200),
    [anon_sym_static] = ACTIONS(1200),
    [anon_sym_auto] = ACTIONS(1200),
    [anon_sym_register] = ACTIONS(1200),
    [anon_sym_inline] = ACTIONS(1200),
    [anon_sym___inline] = ACTIONS(1200),
    [anon_sym___inline__] = ACTIONS(1200),
    [anon_sym___forceinline] = ACTIONS(1200),
    [anon_sym_thread_local] = ACTIONS(1200),
    [anon_sym___thread] = ACTIONS(1200),
    [anon_sym_const] = ACTIONS(1200),
    [anon_sym_constexpr] = ACTIONS(1200),
    [anon_sym_volatile] = ACTIONS(1200),
    [anon_sym_restrict] = ACTIONS(1200),
    [anon_sym___restrict__] = ACTIONS(1200),
    [anon_sym__Atomic] = ACTIONS(1200),
    [anon_sym__Noreturn] = ACTIONS(1200),
    [anon_sym_noreturn] = ACTIONS(1200),
    [sym_primitive_type] = ACTIONS(1200),
    [anon_sym_enum] = ACTIONS(1200),
    [anon_sym_struct] = ACTIONS(1200),
    [anon_sym_union] = ACTIONS(1200),
    [anon_sym_if] = ACTIONS(1200),
    [anon_sym_else] = ACTIONS(1200),
    [anon_sym_switch] = ACTIONS(1200),
    [anon_sym_case] = ACTIONS(1200),
    [anon_sym_default] = ACTIONS(1200),
    [anon_sym_while] = ACTIONS(1200),
    [anon_sym_do] = ACTIONS(1200),
    [anon_sym_for] = ACTIONS(1200),
    [anon_sym_return] = ACTIONS(1200),
    [anon_sym_break] = ACTIONS(1200),
    [anon_sym_continue] = ACTIONS(1200),
    [anon_sym_goto] = ACTIONS(1200),
    [anon_sym___try] = ACTIONS(1200),
    [anon_sym___leave] = ACTIONS(1200),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_sizeof] = ACTIONS(1200),
    [anon_sym___alignof__] = ACTIONS(1200),
    [anon_sym___alignof] = ACTIONS(1200),
    [anon_sym__alignof] = ACTIONS(1200),
    [anon_sym_alignof] = ACTIONS(1200),
    [anon_sym__Alignof] = ACTIONS(1200),
    [anon_sym_offsetof] = ACTIONS(1200),
    [anon_sym__Generic] = ACTIONS(1200),
    [anon_sym_asm] = ACTIONS(1200),
    [anon_sym___asm__] = ACTIONS(1200),
    [sym_number_literal] = ACTIONS(1202),
    [anon_sym_L_SQUOTE] = ACTIONS(1202),
    [anon_sym_u_SQUOTE] = ACTIONS(1202),
    [anon_sym_U_SQUOTE] = ACTIONS(1202),
    [anon_sym_u8_SQUOTE] = ACTIONS(1202),
    [anon_sym_SQUOTE] = ACTIONS(1202),
    [anon_sym_L_DQUOTE] = ACTIONS(1202),
    [anon_sym_u_DQUOTE] = ACTIONS(1202),
    [anon_sym_U_DQUOTE] = ACTIONS(1202),
    [anon_sym_u8_DQUOTE] = ACTIONS(1202),
    [anon_sym_DQUOTE] = ACTIONS(1202),
    [sym_true] = ACTIONS(1200),
    [sym_false] = ACTIONS(1200),
    [anon_sym_NULL] = ACTIONS(1200),
    [anon_sym_nullptr] = ACTIONS(1200),
    [sym_comment] = ACTIONS(3),
  },
  [103] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [104] = {
    [sym_identifier] = ACTIONS(1204),
    [aux_sym_preproc_include_token1] = ACTIONS(1204),
    [aux_sym_preproc_def_token1] = ACTIONS(1204),
    [aux_sym_preproc_if_token1] = ACTIONS(1204),
    [aux_sym_preproc_if_token2] = ACTIONS(1204),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1204),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1204),
    [aux_sym_preproc_else_token1] = ACTIONS(1204),
    [aux_sym_preproc_elif_token1] = ACTIONS(1204),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1204),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1204),
    [sym_preproc_directive] = ACTIONS(1204),
    [anon_sym_LPAREN2] = ACTIONS(1206),
    [anon_sym_BANG] = ACTIONS(1206),
    [anon_sym_TILDE] = ACTIONS(1206),
    [anon_sym_DASH] = ACTIONS(1204),
    [anon_sym_PLUS] = ACTIONS(1204),
    [anon_sym_STAR] = ACTIONS(1206),
    [anon_sym_AMP] = ACTIONS(1206),
    [anon_sym_SEMI] = ACTIONS(1206),
    [anon_sym___extension__] = ACTIONS(1204),
    [anon_sym_typedef] = ACTIONS(1204),
    [anon_sym_extern] = ACTIONS(1204),
    [anon_sym___attribute__] = ACTIONS(1204),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1206),
    [anon_sym___declspec] = ACTIONS(1204),
    [anon_sym___cdecl] = ACTIONS(1204),
    [anon_sym___clrcall] = ACTIONS(1204),
    [anon_sym___stdcall] = ACTIONS(1204),
    [anon_sym___fastcall] = ACTIONS(1204),
    [anon_sym___thiscall] = ACTIONS(1204),
    [anon_sym___vectorcall] = ACTIONS(1204),
    [anon_sym_LBRACE] = ACTIONS(1206),
    [anon_sym_signed] = ACTIONS(1204),
    [anon_sym_unsigned] = ACTIONS(1204),
    [anon_sym_long] = ACTIONS(1204),
    [anon_sym_short] = ACTIONS(1204),
    [anon_sym_static] = ACTIONS(1204),
    [anon_sym_auto] = ACTIONS(1204),
    [anon_sym_register] = ACTIONS(1204),
    [anon_sym_inline] = ACTIONS(1204),
    [anon_sym___inline] = ACTIONS(1204),
    [anon_sym___inline__] = ACTIONS(1204),
    [anon_sym___forceinline] = ACTIONS(1204),
    [anon_sym_thread_local] = ACTIONS(1204),
    [anon_sym___thread] = ACTIONS(1204),
    [anon_sym_const] = ACTIONS(1204),
    [anon_sym_constexpr] = ACTIONS(1204),
    [anon_sym_volatile] = ACTIONS(1204),
    [anon_sym_restrict] = ACTIONS(1204),
    [anon_sym___restrict__] = ACTIONS(1204),
    [anon_sym__Atomic] = ACTIONS(1204),
    [anon_sym__Noreturn] = ACTIONS(1204),
    [anon_sym_noreturn] = ACTIONS(1204),
    [sym_primitive_type] = ACTIONS(1204),
    [anon_sym_enum] = ACTIONS(1204),
    [anon_sym_struct] = ACTIONS(1204),
    [anon_sym_union] = ACTIONS(1204),
    [anon_sym_if] = ACTIONS(1204),
    [anon_sym_else] = ACTIONS(1204),
    [anon_sym_switch] = ACTIONS(1204),
    [anon_sym_case] = ACTIONS(1204),
    [anon_sym_default] = ACTIONS(1204),
    [anon_sym_while] = ACTIONS(1204),
    [anon_sym_do] = ACTIONS(1204),
    [anon_sym_for] = ACTIONS(1204),
    [anon_sym_return] = ACTIONS(1204),
    [anon_sym_break] = ACTIONS(1204),
    [anon_sym_continue] = ACTIONS(1204),
    [anon_sym_goto] = ACTIONS(1204),
    [anon_sym___try] = ACTIONS(1204),
    [anon_sym___leave] = ACTIONS(1204),
    [anon_sym_DASH_DASH] = ACTIONS(1206),
    [anon_sym_PLUS_PLUS] = ACTIONS(1206),
    [anon_sym_sizeof] = ACTIONS(1204),
    [anon_sym___alignof__] = ACTIONS(1204),
    [anon_sym___alignof] = ACTIONS(1204),
    [anon_sym__alignof] = ACTIONS(1204),
    [anon_sym_alignof] = ACTIONS(1204),
    [anon_sym__Alignof] = ACTIONS(1204),
    [anon_sym_offsetof] = ACTIONS(1204),
    [anon_sym__Generic] = ACTIONS(1204),
    [anon_sym_asm] = ACTIONS(1204),
    [anon_sym___asm__] = ACTIONS(1204),
    [sym_number_literal] = ACTIONS(1206),
    [anon_sym_L_SQUOTE] = ACTIONS(1206),
    [anon_sym_u_SQUOTE] = ACTIONS(1206),
    [anon_sym_U_SQUOTE] = ACTIONS(1206),
    [anon_sym_u8_SQUOTE] = ACTIONS(1206),
    [anon_sym_SQUOTE] = ACTIONS(1206),
    [anon_sym_L_DQUOTE] = ACTIONS(1206),
    [anon_sym_u_DQUOTE] = ACTIONS(1206),
    [anon_sym_U_DQUOTE] = ACTIONS(1206),
    [anon_sym_u8_DQUOTE] = ACTIONS(1206),
    [anon_sym_DQUOTE] = ACTIONS(1206),
    [sym_true] = ACTIONS(1204),
    [sym_false] = ACTIONS(1204),
    [anon_sym_NULL] = ACTIONS(1204),
    [anon_sym_nullptr] = ACTIONS(1204),
    [sym_comment] = ACTIONS(3),
  },
  [105] = {
    [sym_identifier] = ACTIONS(1208),
    [aux_sym_preproc_include_token1] = ACTIONS(1208),
    [aux_sym_preproc_def_token1] = ACTIONS(1208),
    [aux_sym_preproc_if_token1] = ACTIONS(1208),
    [aux_sym_preproc_if_token2] = ACTIONS(1208),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1208),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1208),
    [aux_sym_preproc_else_token1] = ACTIONS(1208),
    [aux_sym_preproc_elif_token1] = ACTIONS(1208),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1208),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1208),
    [sym_preproc_directive] = ACTIONS(1208),
    [anon_sym_LPAREN2] = ACTIONS(1210),
    [anon_sym_BANG] = ACTIONS(1210),
    [anon_sym_TILDE] = ACTIONS(1210),
    [anon_sym_DASH] = ACTIONS(1208),
    [anon_sym_PLUS] = ACTIONS(1208),
    [anon_sym_STAR] = ACTIONS(1210),
    [anon_sym_AMP] = ACTIONS(1210),
    [anon_sym_SEMI] = ACTIONS(1210),
    [anon_sym___extension__] = ACTIONS(1208),
    [anon_sym_typedef] = ACTIONS(1208),
    [anon_sym_extern] = ACTIONS(1208),
    [anon_sym___attribute__] = ACTIONS(1208),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1210),
    [anon_sym___declspec] = ACTIONS(1208),
    [anon_sym___cdecl] = ACTIONS(1208),
    [anon_sym___clrcall] = ACTIONS(1208),
    [anon_sym___stdcall] = ACTIONS(1208),
    [anon_sym___fastcall] = ACTIONS(1208),
    [anon_sym___thiscall] = ACTIONS(1208),
    [anon_sym___vectorcall] = ACTIONS(1208),
    [anon_sym_LBRACE] = ACTIONS(1210),
    [anon_sym_signed] = ACTIONS(1208),
    [anon_sym_unsigned] = ACTIONS(1208),
    [anon_sym_long] = ACTIONS(1208),
    [anon_sym_short] = ACTIONS(1208),
    [anon_sym_static] = ACTIONS(1208),
    [anon_sym_auto] = ACTIONS(1208),
    [anon_sym_register] = ACTIONS(1208),
    [anon_sym_inline] = ACTIONS(1208),
    [anon_sym___inline] = ACTIONS(1208),
    [anon_sym___inline__] = ACTIONS(1208),
    [anon_sym___forceinline] = ACTIONS(1208),
    [anon_sym_thread_local] = ACTIONS(1208),
    [anon_sym___thread] = ACTIONS(1208),
    [anon_sym_const] = ACTIONS(1208),
    [anon_sym_constexpr] = ACTIONS(1208),
    [anon_sym_volatile] = ACTIONS(1208),
    [anon_sym_restrict] = ACTIONS(1208),
    [anon_sym___restrict__] = ACTIONS(1208),
    [anon_sym__Atomic] = ACTIONS(1208),
    [anon_sym__Noreturn] = ACTIONS(1208),
    [anon_sym_noreturn] = ACTIONS(1208),
    [sym_primitive_type] = ACTIONS(1208),
    [anon_sym_enum] = ACTIONS(1208),
    [anon_sym_struct] = ACTIONS(1208),
    [anon_sym_union] = ACTIONS(1208),
    [anon_sym_if] = ACTIONS(1208),
    [anon_sym_else] = ACTIONS(1208),
    [anon_sym_switch] = ACTIONS(1208),
    [anon_sym_case] = ACTIONS(1208),
    [anon_sym_default] = ACTIONS(1208),
    [anon_sym_while] = ACTIONS(1208),
    [anon_sym_do] = ACTIONS(1208),
    [anon_sym_for] = ACTIONS(1208),
    [anon_sym_return] = ACTIONS(1208),
    [anon_sym_break] = ACTIONS(1208),
    [anon_sym_continue] = ACTIONS(1208),
    [anon_sym_goto] = ACTIONS(1208),
    [anon_sym___try] = ACTIONS(1208),
    [anon_sym___leave] = ACTIONS(1208),
    [anon_sym_DASH_DASH] = ACTIONS(1210),
    [anon_sym_PLUS_PLUS] = ACTIONS(1210),
    [anon_sym_sizeof] = ACTIONS(1208),
    [anon_sym___alignof__] = ACTIONS(1208),
    [anon_sym___alignof] = ACTIONS(1208),
    [anon_sym__alignof] = ACTIONS(1208),
    [anon_sym_alignof] = ACTIONS(1208),
    [anon_sym__Alignof] = ACTIONS(1208),
    [anon_sym_offsetof] = ACTIONS(1208),
    [anon_sym__Generic] = ACTIONS(1208),
    [anon_sym_asm] = ACTIONS(1208),
    [anon_sym___asm__] = ACTIONS(1208),
    [sym_number_literal] = ACTIONS(1210),
    [anon_sym_L_SQUOTE] = ACTIONS(1210),
    [anon_sym_u_SQUOTE] = ACTIONS(1210),
    [anon_sym_U_SQUOTE] = ACTIONS(1210),
    [anon_sym_u8_SQUOTE] = ACTIONS(1210),
    [anon_sym_SQUOTE] = ACTIONS(1210),
    [anon_sym_L_DQUOTE] = ACTIONS(1210),
    [anon_sym_u_DQUOTE] = ACTIONS(1210),
    [anon_sym_U_DQUOTE] = ACTIONS(1210),
    [anon_sym_u8_DQUOTE] = ACTIONS(1210),
    [anon_sym_DQUOTE] = ACTIONS(1210),
    [sym_true] = ACTIONS(1208),
    [sym_false] = ACTIONS(1208),
    [anon_sym_NULL] = ACTIONS(1208),
    [anon_sym_nullptr] = ACTIONS(1208),
    [sym_comment] = ACTIONS(3),
  },
  [106] = {
    [sym_identifier] = ACTIONS(1200),
    [aux_sym_preproc_include_token1] = ACTIONS(1200),
    [aux_sym_preproc_def_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token2] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1200),
    [aux_sym_preproc_else_token1] = ACTIONS(1200),
    [aux_sym_preproc_elif_token1] = ACTIONS(1200),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1200),
    [sym_preproc_directive] = ACTIONS(1200),
    [anon_sym_LPAREN2] = ACTIONS(1202),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_DASH] = ACTIONS(1200),
    [anon_sym_PLUS] = ACTIONS(1200),
    [anon_sym_STAR] = ACTIONS(1202),
    [anon_sym_AMP] = ACTIONS(1202),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym___extension__] = ACTIONS(1200),
    [anon_sym_typedef] = ACTIONS(1200),
    [anon_sym_extern] = ACTIONS(1200),
    [anon_sym___attribute__] = ACTIONS(1200),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1202),
    [anon_sym___declspec] = ACTIONS(1200),
    [anon_sym___cdecl] = ACTIONS(1200),
    [anon_sym___clrcall] = ACTIONS(1200),
    [anon_sym___stdcall] = ACTIONS(1200),
    [anon_sym___fastcall] = ACTIONS(1200),
    [anon_sym___thiscall] = ACTIONS(1200),
    [anon_sym___vectorcall] = ACTIONS(1200),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_signed] = ACTIONS(1200),
    [anon_sym_unsigned] = ACTIONS(1200),
    [anon_sym_long] = ACTIONS(1200),
    [anon_sym_short] = ACTIONS(1200),
    [anon_sym_static] = ACTIONS(1200),
    [anon_sym_auto] = ACTIONS(1200),
    [anon_sym_register] = ACTIONS(1200),
    [anon_sym_inline] = ACTIONS(1200),
    [anon_sym___inline] = ACTIONS(1200),
    [anon_sym___inline__] = ACTIONS(1200),
    [anon_sym___forceinline] = ACTIONS(1200),
    [anon_sym_thread_local] = ACTIONS(1200),
    [anon_sym___thread] = ACTIONS(1200),
    [anon_sym_const] = ACTIONS(1200),
    [anon_sym_constexpr] = ACTIONS(1200),
    [anon_sym_volatile] = ACTIONS(1200),
    [anon_sym_restrict] = ACTIONS(1200),
    [anon_sym___restrict__] = ACTIONS(1200),
    [anon_sym__Atomic] = ACTIONS(1200),
    [anon_sym__Noreturn] = ACTIONS(1200),
    [anon_sym_noreturn] = ACTIONS(1200),
    [sym_primitive_type] = ACTIONS(1200),
    [anon_sym_enum] = ACTIONS(1200),
    [anon_sym_struct] = ACTIONS(1200),
    [anon_sym_union] = ACTIONS(1200),
    [anon_sym_if] = ACTIONS(1200),
    [anon_sym_else] = ACTIONS(1200),
    [anon_sym_switch] = ACTIONS(1200),
    [anon_sym_case] = ACTIONS(1200),
    [anon_sym_default] = ACTIONS(1200),
    [anon_sym_while] = ACTIONS(1200),
    [anon_sym_do] = ACTIONS(1200),
    [anon_sym_for] = ACTIONS(1200),
    [anon_sym_return] = ACTIONS(1200),
    [anon_sym_break] = ACTIONS(1200),
    [anon_sym_continue] = ACTIONS(1200),
    [anon_sym_goto] = ACTIONS(1200),
    [anon_sym___try] = ACTIONS(1200),
    [anon_sym___leave] = ACTIONS(1200),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_sizeof] = ACTIONS(1200),
    [anon_sym___alignof__] = ACTIONS(1200),
    [anon_sym___alignof] = ACTIONS(1200),
    [anon_sym__alignof] = ACTIONS(1200),
    [anon_sym_alignof] = ACTIONS(1200),
    [anon_sym__Alignof] = ACTIONS(1200),
    [anon_sym_offsetof] = ACTIONS(1200),
    [anon_sym__Generic] = ACTIONS(1200),
    [anon_sym_asm] = ACTIONS(1200),
    [anon_sym___asm__] = ACTIONS(1200),
    [sym_number_literal] = ACTIONS(1202),
    [anon_sym_L_SQUOTE] = ACTIONS(1202),
    [anon_sym_u_SQUOTE] = ACTIONS(1202),
    [anon_sym_U_SQUOTE] = ACTIONS(1202),
    [anon_sym_u8_SQUOTE] = ACTIONS(1202),
    [anon_sym_SQUOTE] = ACTIONS(1202),
    [anon_sym_L_DQUOTE] = ACTIONS(1202),
    [anon_sym_u_DQUOTE] = ACTIONS(1202),
    [anon_sym_U_DQUOTE] = ACTIONS(1202),
    [anon_sym_u8_DQUOTE] = ACTIONS(1202),
    [anon_sym_DQUOTE] = ACTIONS(1202),
    [sym_true] = ACTIONS(1200),
    [sym_false] = ACTIONS(1200),
    [anon_sym_NULL] = ACTIONS(1200),
    [anon_sym_nullptr] = ACTIONS(1200),
    [sym_comment] = ACTIONS(3),
  },
  [107] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [108] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [109] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [110] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [111] = {
    [ts_builtin_sym_end] = ACTIONS(1206),
    [sym_identifier] = ACTIONS(1204),
    [aux_sym_preproc_include_token1] = ACTIONS(1204),
    [aux_sym_preproc_def_token1] = ACTIONS(1204),
    [anon_sym_COMMA] = ACTIONS(1206),
    [anon_sym_RPAREN] = ACTIONS(1206),
    [aux_sym_preproc_if_token1] = ACTIONS(1204),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1204),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1204),
    [sym_preproc_directive] = ACTIONS(1204),
    [anon_sym_LPAREN2] = ACTIONS(1206),
    [anon_sym_BANG] = ACTIONS(1206),
    [anon_sym_TILDE] = ACTIONS(1206),
    [anon_sym_DASH] = ACTIONS(1204),
    [anon_sym_PLUS] = ACTIONS(1204),
    [anon_sym_STAR] = ACTIONS(1206),
    [anon_sym_AMP] = ACTIONS(1206),
    [anon_sym_SEMI] = ACTIONS(1206),
    [anon_sym___extension__] = ACTIONS(1204),
    [anon_sym_typedef] = ACTIONS(1204),
    [anon_sym_extern] = ACTIONS(1204),
    [anon_sym___attribute__] = ACTIONS(1204),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1206),
    [anon_sym___declspec] = ACTIONS(1204),
    [anon_sym___cdecl] = ACTIONS(1204),
    [anon_sym___clrcall] = ACTIONS(1204),
    [anon_sym___stdcall] = ACTIONS(1204),
    [anon_sym___fastcall] = ACTIONS(1204),
    [anon_sym___thiscall] = ACTIONS(1204),
    [anon_sym___vectorcall] = ACTIONS(1204),
    [anon_sym_LBRACE] = ACTIONS(1206),
    [anon_sym_signed] = ACTIONS(1204),
    [anon_sym_unsigned] = ACTIONS(1204),
    [anon_sym_long] = ACTIONS(1204),
    [anon_sym_short] = ACTIONS(1204),
    [anon_sym_static] = ACTIONS(1204),
    [anon_sym_auto] = ACTIONS(1204),
    [anon_sym_register] = ACTIONS(1204),
    [anon_sym_inline] = ACTIONS(1204),
    [anon_sym___inline] = ACTIONS(1204),
    [anon_sym___inline__] = ACTIONS(1204),
    [anon_sym___forceinline] = ACTIONS(1204),
    [anon_sym_thread_local] = ACTIONS(1204),
    [anon_sym___thread] = ACTIONS(1204),
    [anon_sym_const] = ACTIONS(1204),
    [anon_sym_constexpr] = ACTIONS(1204),
    [anon_sym_volatile] = ACTIONS(1204),
    [anon_sym_restrict] = ACTIONS(1204),
    [anon_sym___restrict__] = ACTIONS(1204),
    [anon_sym__Atomic] = ACTIONS(1204),
    [anon_sym__Noreturn] = ACTIONS(1204),
    [anon_sym_noreturn] = ACTIONS(1204),
    [sym_primitive_type] = ACTIONS(1204),
    [anon_sym_enum] = ACTIONS(1204),
    [anon_sym_struct] = ACTIONS(1204),
    [anon_sym_union] = ACTIONS(1204),
    [anon_sym_if] = ACTIONS(1204),
    [anon_sym_else] = ACTIONS(1204),
    [anon_sym_switch] = ACTIONS(1204),
    [anon_sym_case] = ACTIONS(1204),
    [anon_sym_default] = ACTIONS(1204),
    [anon_sym_while] = ACTIONS(1204),
    [anon_sym_do] = ACTIONS(1204),
    [anon_sym_for] = ACTIONS(1204),
    [anon_sym_return] = ACTIONS(1204),
    [anon_sym_break] = ACTIONS(1204),
    [anon_sym_continue] = ACTIONS(1204),
    [anon_sym_goto] = ACTIONS(1204),
    [anon_sym___try] = ACTIONS(1204),
    [anon_sym___except] = ACTIONS(1204),
    [anon_sym___finally] = ACTIONS(1204),
    [anon_sym___leave] = ACTIONS(1204),
    [anon_sym_DASH_DASH] = ACTIONS(1206),
    [anon_sym_PLUS_PLUS] = ACTIONS(1206),
    [anon_sym_sizeof] = ACTIONS(1204),
    [anon_sym___alignof__] = ACTIONS(1204),
    [anon_sym___alignof] = ACTIONS(1204),
    [anon_sym__alignof] = ACTIONS(1204),
    [anon_sym_alignof] = ACTIONS(1204),
    [anon_sym__Alignof] = ACTIONS(1204),
    [anon_sym_offsetof] = ACTIONS(1204),
    [anon_sym__Generic] = ACTIONS(1204),
    [anon_sym_asm] = ACTIONS(1204),
    [anon_sym___asm__] = ACTIONS(1204),
    [sym_number_literal] = ACTIONS(1206),
    [anon_sym_L_SQUOTE] = ACTIONS(1206),
    [anon_sym_u_SQUOTE] = ACTIONS(1206),
    [anon_sym_U_SQUOTE] = ACTIONS(1206),
    [anon_sym_u8_SQUOTE] = ACTIONS(1206),
    [anon_sym_SQUOTE] = ACTIONS(1206),
    [anon_sym_L_DQUOTE] = ACTIONS(1206),
    [anon_sym_u_DQUOTE] = ACTIONS(1206),
    [anon_sym_U_DQUOTE] = ACTIONS(1206),
    [anon_sym_u8_DQUOTE] = ACTIONS(1206),
    [anon_sym_DQUOTE] = ACTIONS(1206),
    [sym_true] = ACTIONS(1204),
    [sym_false] = ACTIONS(1204),
    [anon_sym_NULL] = ACTIONS(1204),
    [anon_sym_nullptr] = ACTIONS(1204),
    [sym_comment] = ACTIONS(3),
  },
  [112] = {
    [sym_identifier] = ACTIONS(1212),
    [aux_sym_preproc_include_token1] = ACTIONS(1212),
    [aux_sym_preproc_def_token1] = ACTIONS(1212),
    [aux_sym_preproc_if_token1] = ACTIONS(1212),
    [aux_sym_preproc_if_token2] = ACTIONS(1212),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1212),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1212),
    [aux_sym_preproc_else_token1] = ACTIONS(1212),
    [aux_sym_preproc_elif_token1] = ACTIONS(1212),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1212),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1212),
    [sym_preproc_directive] = ACTIONS(1212),
    [anon_sym_LPAREN2] = ACTIONS(1214),
    [anon_sym_BANG] = ACTIONS(1214),
    [anon_sym_TILDE] = ACTIONS(1214),
    [anon_sym_DASH] = ACTIONS(1212),
    [anon_sym_PLUS] = ACTIONS(1212),
    [anon_sym_STAR] = ACTIONS(1214),
    [anon_sym_AMP] = ACTIONS(1214),
    [anon_sym_SEMI] = ACTIONS(1214),
    [anon_sym___extension__] = ACTIONS(1212),
    [anon_sym_typedef] = ACTIONS(1212),
    [anon_sym_extern] = ACTIONS(1212),
    [anon_sym___attribute__] = ACTIONS(1212),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1214),
    [anon_sym___declspec] = ACTIONS(1212),
    [anon_sym___cdecl] = ACTIONS(1212),
    [anon_sym___clrcall] = ACTIONS(1212),
    [anon_sym___stdcall] = ACTIONS(1212),
    [anon_sym___fastcall] = ACTIONS(1212),
    [anon_sym___thiscall] = ACTIONS(1212),
    [anon_sym___vectorcall] = ACTIONS(1212),
    [anon_sym_LBRACE] = ACTIONS(1214),
    [anon_sym_signed] = ACTIONS(1212),
    [anon_sym_unsigned] = ACTIONS(1212),
    [anon_sym_long] = ACTIONS(1212),
    [anon_sym_short] = ACTIONS(1212),
    [anon_sym_static] = ACTIONS(1212),
    [anon_sym_auto] = ACTIONS(1212),
    [anon_sym_register] = ACTIONS(1212),
    [anon_sym_inline] = ACTIONS(1212),
    [anon_sym___inline] = ACTIONS(1212),
    [anon_sym___inline__] = ACTIONS(1212),
    [anon_sym___forceinline] = ACTIONS(1212),
    [anon_sym_thread_local] = ACTIONS(1212),
    [anon_sym___thread] = ACTIONS(1212),
    [anon_sym_const] = ACTIONS(1212),
    [anon_sym_constexpr] = ACTIONS(1212),
    [anon_sym_volatile] = ACTIONS(1212),
    [anon_sym_restrict] = ACTIONS(1212),
    [anon_sym___restrict__] = ACTIONS(1212),
    [anon_sym__Atomic] = ACTIONS(1212),
    [anon_sym__Noreturn] = ACTIONS(1212),
    [anon_sym_noreturn] = ACTIONS(1212),
    [sym_primitive_type] = ACTIONS(1212),
    [anon_sym_enum] = ACTIONS(1212),
    [anon_sym_struct] = ACTIONS(1212),
    [anon_sym_union] = ACTIONS(1212),
    [anon_sym_if] = ACTIONS(1212),
    [anon_sym_else] = ACTIONS(1212),
    [anon_sym_switch] = ACTIONS(1212),
    [anon_sym_case] = ACTIONS(1212),
    [anon_sym_default] = ACTIONS(1212),
    [anon_sym_while] = ACTIONS(1212),
    [anon_sym_do] = ACTIONS(1212),
    [anon_sym_for] = ACTIONS(1212),
    [anon_sym_return] = ACTIONS(1212),
    [anon_sym_break] = ACTIONS(1212),
    [anon_sym_continue] = ACTIONS(1212),
    [anon_sym_goto] = ACTIONS(1212),
    [anon_sym___try] = ACTIONS(1212),
    [anon_sym___leave] = ACTIONS(1212),
    [anon_sym_DASH_DASH] = ACTIONS(1214),
    [anon_sym_PLUS_PLUS] = ACTIONS(1214),
    [anon_sym_sizeof] = ACTIONS(1212),
    [anon_sym___alignof__] = ACTIONS(1212),
    [anon_sym___alignof] = ACTIONS(1212),
    [anon_sym__alignof] = ACTIONS(1212),
    [anon_sym_alignof] = ACTIONS(1212),
    [anon_sym__Alignof] = ACTIONS(1212),
    [anon_sym_offsetof] = ACTIONS(1212),
    [anon_sym__Generic] = ACTIONS(1212),
    [anon_sym_asm] = ACTIONS(1212),
    [anon_sym___asm__] = ACTIONS(1212),
    [sym_number_literal] = ACTIONS(1214),
    [anon_sym_L_SQUOTE] = ACTIONS(1214),
    [anon_sym_u_SQUOTE] = ACTIONS(1214),
    [anon_sym_U_SQUOTE] = ACTIONS(1214),
    [anon_sym_u8_SQUOTE] = ACTIONS(1214),
    [anon_sym_SQUOTE] = ACTIONS(1214),
    [anon_sym_L_DQUOTE] = ACTIONS(1214),
    [anon_sym_u_DQUOTE] = ACTIONS(1214),
    [anon_sym_U_DQUOTE] = ACTIONS(1214),
    [anon_sym_u8_DQUOTE] = ACTIONS(1214),
    [anon_sym_DQUOTE] = ACTIONS(1214),
    [sym_true] = ACTIONS(1212),
    [sym_false] = ACTIONS(1212),
    [anon_sym_NULL] = ACTIONS(1212),
    [anon_sym_nullptr] = ACTIONS(1212),
    [sym_comment] = ACTIONS(3),
  },
  [113] = {
    [sym_identifier] = ACTIONS(1216),
    [aux_sym_preproc_include_token1] = ACTIONS(1216),
    [aux_sym_preproc_def_token1] = ACTIONS(1216),
    [aux_sym_preproc_if_token1] = ACTIONS(1216),
    [aux_sym_preproc_if_token2] = ACTIONS(1216),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1216),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1216),
    [aux_sym_preproc_else_token1] = ACTIONS(1216),
    [aux_sym_preproc_elif_token1] = ACTIONS(1216),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1216),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1216),
    [sym_preproc_directive] = ACTIONS(1216),
    [anon_sym_LPAREN2] = ACTIONS(1218),
    [anon_sym_BANG] = ACTIONS(1218),
    [anon_sym_TILDE] = ACTIONS(1218),
    [anon_sym_DASH] = ACTIONS(1216),
    [anon_sym_PLUS] = ACTIONS(1216),
    [anon_sym_STAR] = ACTIONS(1218),
    [anon_sym_AMP] = ACTIONS(1218),
    [anon_sym_SEMI] = ACTIONS(1218),
    [anon_sym___extension__] = ACTIONS(1216),
    [anon_sym_typedef] = ACTIONS(1216),
    [anon_sym_extern] = ACTIONS(1216),
    [anon_sym___attribute__] = ACTIONS(1216),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1218),
    [anon_sym___declspec] = ACTIONS(1216),
    [anon_sym___cdecl] = ACTIONS(1216),
    [anon_sym___clrcall] = ACTIONS(1216),
    [anon_sym___stdcall] = ACTIONS(1216),
    [anon_sym___fastcall] = ACTIONS(1216),
    [anon_sym___thiscall] = ACTIONS(1216),
    [anon_sym___vectorcall] = ACTIONS(1216),
    [anon_sym_LBRACE] = ACTIONS(1218),
    [anon_sym_signed] = ACTIONS(1216),
    [anon_sym_unsigned] = ACTIONS(1216),
    [anon_sym_long] = ACTIONS(1216),
    [anon_sym_short] = ACTIONS(1216),
    [anon_sym_static] = ACTIONS(1216),
    [anon_sym_auto] = ACTIONS(1216),
    [anon_sym_register] = ACTIONS(1216),
    [anon_sym_inline] = ACTIONS(1216),
    [anon_sym___inline] = ACTIONS(1216),
    [anon_sym___inline__] = ACTIONS(1216),
    [anon_sym___forceinline] = ACTIONS(1216),
    [anon_sym_thread_local] = ACTIONS(1216),
    [anon_sym___thread] = ACTIONS(1216),
    [anon_sym_const] = ACTIONS(1216),
    [anon_sym_constexpr] = ACTIONS(1216),
    [anon_sym_volatile] = ACTIONS(1216),
    [anon_sym_restrict] = ACTIONS(1216),
    [anon_sym___restrict__] = ACTIONS(1216),
    [anon_sym__Atomic] = ACTIONS(1216),
    [anon_sym__Noreturn] = ACTIONS(1216),
    [anon_sym_noreturn] = ACTIONS(1216),
    [sym_primitive_type] = ACTIONS(1216),
    [anon_sym_enum] = ACTIONS(1216),
    [anon_sym_struct] = ACTIONS(1216),
    [anon_sym_union] = ACTIONS(1216),
    [anon_sym_if] = ACTIONS(1216),
    [anon_sym_else] = ACTIONS(1216),
    [anon_sym_switch] = ACTIONS(1216),
    [anon_sym_case] = ACTIONS(1216),
    [anon_sym_default] = ACTIONS(1216),
    [anon_sym_while] = ACTIONS(1216),
    [anon_sym_do] = ACTIONS(1216),
    [anon_sym_for] = ACTIONS(1216),
    [anon_sym_return] = ACTIONS(1216),
    [anon_sym_break] = ACTIONS(1216),
    [anon_sym_continue] = ACTIONS(1216),
    [anon_sym_goto] = ACTIONS(1216),
    [anon_sym___try] = ACTIONS(1216),
    [anon_sym___leave] = ACTIONS(1216),
    [anon_sym_DASH_DASH] = ACTIONS(1218),
    [anon_sym_PLUS_PLUS] = ACTIONS(1218),
    [anon_sym_sizeof] = ACTIONS(1216),
    [anon_sym___alignof__] = ACTIONS(1216),
    [anon_sym___alignof] = ACTIONS(1216),
    [anon_sym__alignof] = ACTIONS(1216),
    [anon_sym_alignof] = ACTIONS(1216),
    [anon_sym__Alignof] = ACTIONS(1216),
    [anon_sym_offsetof] = ACTIONS(1216),
    [anon_sym__Generic] = ACTIONS(1216),
    [anon_sym_asm] = ACTIONS(1216),
    [anon_sym___asm__] = ACTIONS(1216),
    [sym_number_literal] = ACTIONS(1218),
    [anon_sym_L_SQUOTE] = ACTIONS(1218),
    [anon_sym_u_SQUOTE] = ACTIONS(1218),
    [anon_sym_U_SQUOTE] = ACTIONS(1218),
    [anon_sym_u8_SQUOTE] = ACTIONS(1218),
    [anon_sym_SQUOTE] = ACTIONS(1218),
    [anon_sym_L_DQUOTE] = ACTIONS(1218),
    [anon_sym_u_DQUOTE] = ACTIONS(1218),
    [anon_sym_U_DQUOTE] = ACTIONS(1218),
    [anon_sym_u8_DQUOTE] = ACTIONS(1218),
    [anon_sym_DQUOTE] = ACTIONS(1218),
    [sym_true] = ACTIONS(1216),
    [sym_false] = ACTIONS(1216),
    [anon_sym_NULL] = ACTIONS(1216),
    [anon_sym_nullptr] = ACTIONS(1216),
    [sym_comment] = ACTIONS(3),
  },
  [114] = {
    [sym_identifier] = ACTIONS(1220),
    [aux_sym_preproc_include_token1] = ACTIONS(1220),
    [aux_sym_preproc_def_token1] = ACTIONS(1220),
    [aux_sym_preproc_if_token1] = ACTIONS(1220),
    [aux_sym_preproc_if_token2] = ACTIONS(1220),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1220),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1220),
    [aux_sym_preproc_else_token1] = ACTIONS(1220),
    [aux_sym_preproc_elif_token1] = ACTIONS(1220),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1220),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1220),
    [sym_preproc_directive] = ACTIONS(1220),
    [anon_sym_LPAREN2] = ACTIONS(1222),
    [anon_sym_BANG] = ACTIONS(1222),
    [anon_sym_TILDE] = ACTIONS(1222),
    [anon_sym_DASH] = ACTIONS(1220),
    [anon_sym_PLUS] = ACTIONS(1220),
    [anon_sym_STAR] = ACTIONS(1222),
    [anon_sym_AMP] = ACTIONS(1222),
    [anon_sym_SEMI] = ACTIONS(1222),
    [anon_sym___extension__] = ACTIONS(1220),
    [anon_sym_typedef] = ACTIONS(1220),
    [anon_sym_extern] = ACTIONS(1220),
    [anon_sym___attribute__] = ACTIONS(1220),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1222),
    [anon_sym___declspec] = ACTIONS(1220),
    [anon_sym___cdecl] = ACTIONS(1220),
    [anon_sym___clrcall] = ACTIONS(1220),
    [anon_sym___stdcall] = ACTIONS(1220),
    [anon_sym___fastcall] = ACTIONS(1220),
    [anon_sym___thiscall] = ACTIONS(1220),
    [anon_sym___vectorcall] = ACTIONS(1220),
    [anon_sym_LBRACE] = ACTIONS(1222),
    [anon_sym_signed] = ACTIONS(1220),
    [anon_sym_unsigned] = ACTIONS(1220),
    [anon_sym_long] = ACTIONS(1220),
    [anon_sym_short] = ACTIONS(1220),
    [anon_sym_static] = ACTIONS(1220),
    [anon_sym_auto] = ACTIONS(1220),
    [anon_sym_register] = ACTIONS(1220),
    [anon_sym_inline] = ACTIONS(1220),
    [anon_sym___inline] = ACTIONS(1220),
    [anon_sym___inline__] = ACTIONS(1220),
    [anon_sym___forceinline] = ACTIONS(1220),
    [anon_sym_thread_local] = ACTIONS(1220),
    [anon_sym___thread] = ACTIONS(1220),
    [anon_sym_const] = ACTIONS(1220),
    [anon_sym_constexpr] = ACTIONS(1220),
    [anon_sym_volatile] = ACTIONS(1220),
    [anon_sym_restrict] = ACTIONS(1220),
    [anon_sym___restrict__] = ACTIONS(1220),
    [anon_sym__Atomic] = ACTIONS(1220),
    [anon_sym__Noreturn] = ACTIONS(1220),
    [anon_sym_noreturn] = ACTIONS(1220),
    [sym_primitive_type] = ACTIONS(1220),
    [anon_sym_enum] = ACTIONS(1220),
    [anon_sym_struct] = ACTIONS(1220),
    [anon_sym_union] = ACTIONS(1220),
    [anon_sym_if] = ACTIONS(1220),
    [anon_sym_else] = ACTIONS(1220),
    [anon_sym_switch] = ACTIONS(1220),
    [anon_sym_case] = ACTIONS(1220),
    [anon_sym_default] = ACTIONS(1220),
    [anon_sym_while] = ACTIONS(1220),
    [anon_sym_do] = ACTIONS(1220),
    [anon_sym_for] = ACTIONS(1220),
    [anon_sym_return] = ACTIONS(1220),
    [anon_sym_break] = ACTIONS(1220),
    [anon_sym_continue] = ACTIONS(1220),
    [anon_sym_goto] = ACTIONS(1220),
    [anon_sym___try] = ACTIONS(1220),
    [anon_sym___leave] = ACTIONS(1220),
    [anon_sym_DASH_DASH] = ACTIONS(1222),
    [anon_sym_PLUS_PLUS] = ACTIONS(1222),
    [anon_sym_sizeof] = ACTIONS(1220),
    [anon_sym___alignof__] = ACTIONS(1220),
    [anon_sym___alignof] = ACTIONS(1220),
    [anon_sym__alignof] = ACTIONS(1220),
    [anon_sym_alignof] = ACTIONS(1220),
    [anon_sym__Alignof] = ACTIONS(1220),
    [anon_sym_offsetof] = ACTIONS(1220),
    [anon_sym__Generic] = ACTIONS(1220),
    [anon_sym_asm] = ACTIONS(1220),
    [anon_sym___asm__] = ACTIONS(1220),
    [sym_number_literal] = ACTIONS(1222),
    [anon_sym_L_SQUOTE] = ACTIONS(1222),
    [anon_sym_u_SQUOTE] = ACTIONS(1222),
    [anon_sym_U_SQUOTE] = ACTIONS(1222),
    [anon_sym_u8_SQUOTE] = ACTIONS(1222),
    [anon_sym_SQUOTE] = ACTIONS(1222),
    [anon_sym_L_DQUOTE] = ACTIONS(1222),
    [anon_sym_u_DQUOTE] = ACTIONS(1222),
    [anon_sym_U_DQUOTE] = ACTIONS(1222),
    [anon_sym_u8_DQUOTE] = ACTIONS(1222),
    [anon_sym_DQUOTE] = ACTIONS(1222),
    [sym_true] = ACTIONS(1220),
    [sym_false] = ACTIONS(1220),
    [anon_sym_NULL] = ACTIONS(1220),
    [anon_sym_nullptr] = ACTIONS(1220),
    [sym_comment] = ACTIONS(3),
  },
  [115] = {
    [sym_identifier] = ACTIONS(1224),
    [aux_sym_preproc_include_token1] = ACTIONS(1224),
    [aux_sym_preproc_def_token1] = ACTIONS(1224),
    [aux_sym_preproc_if_token1] = ACTIONS(1224),
    [aux_sym_preproc_if_token2] = ACTIONS(1224),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1224),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1224),
    [aux_sym_preproc_else_token1] = ACTIONS(1224),
    [aux_sym_preproc_elif_token1] = ACTIONS(1224),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1224),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1224),
    [sym_preproc_directive] = ACTIONS(1224),
    [anon_sym_LPAREN2] = ACTIONS(1226),
    [anon_sym_BANG] = ACTIONS(1226),
    [anon_sym_TILDE] = ACTIONS(1226),
    [anon_sym_DASH] = ACTIONS(1224),
    [anon_sym_PLUS] = ACTIONS(1224),
    [anon_sym_STAR] = ACTIONS(1226),
    [anon_sym_AMP] = ACTIONS(1226),
    [anon_sym_SEMI] = ACTIONS(1226),
    [anon_sym___extension__] = ACTIONS(1224),
    [anon_sym_typedef] = ACTIONS(1224),
    [anon_sym_extern] = ACTIONS(1224),
    [anon_sym___attribute__] = ACTIONS(1224),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1226),
    [anon_sym___declspec] = ACTIONS(1224),
    [anon_sym___cdecl] = ACTIONS(1224),
    [anon_sym___clrcall] = ACTIONS(1224),
    [anon_sym___stdcall] = ACTIONS(1224),
    [anon_sym___fastcall] = ACTIONS(1224),
    [anon_sym___thiscall] = ACTIONS(1224),
    [anon_sym___vectorcall] = ACTIONS(1224),
    [anon_sym_LBRACE] = ACTIONS(1226),
    [anon_sym_signed] = ACTIONS(1224),
    [anon_sym_unsigned] = ACTIONS(1224),
    [anon_sym_long] = ACTIONS(1224),
    [anon_sym_short] = ACTIONS(1224),
    [anon_sym_static] = ACTIONS(1224),
    [anon_sym_auto] = ACTIONS(1224),
    [anon_sym_register] = ACTIONS(1224),
    [anon_sym_inline] = ACTIONS(1224),
    [anon_sym___inline] = ACTIONS(1224),
    [anon_sym___inline__] = ACTIONS(1224),
    [anon_sym___forceinline] = ACTIONS(1224),
    [anon_sym_thread_local] = ACTIONS(1224),
    [anon_sym___thread] = ACTIONS(1224),
    [anon_sym_const] = ACTIONS(1224),
    [anon_sym_constexpr] = ACTIONS(1224),
    [anon_sym_volatile] = ACTIONS(1224),
    [anon_sym_restrict] = ACTIONS(1224),
    [anon_sym___restrict__] = ACTIONS(1224),
    [anon_sym__Atomic] = ACTIONS(1224),
    [anon_sym__Noreturn] = ACTIONS(1224),
    [anon_sym_noreturn] = ACTIONS(1224),
    [sym_primitive_type] = ACTIONS(1224),
    [anon_sym_enum] = ACTIONS(1224),
    [anon_sym_struct] = ACTIONS(1224),
    [anon_sym_union] = ACTIONS(1224),
    [anon_sym_if] = ACTIONS(1224),
    [anon_sym_else] = ACTIONS(1224),
    [anon_sym_switch] = ACTIONS(1224),
    [anon_sym_case] = ACTIONS(1224),
    [anon_sym_default] = ACTIONS(1224),
    [anon_sym_while] = ACTIONS(1224),
    [anon_sym_do] = ACTIONS(1224),
    [anon_sym_for] = ACTIONS(1224),
    [anon_sym_return] = ACTIONS(1224),
    [anon_sym_break] = ACTIONS(1224),
    [anon_sym_continue] = ACTIONS(1224),
    [anon_sym_goto] = ACTIONS(1224),
    [anon_sym___try] = ACTIONS(1224),
    [anon_sym___leave] = ACTIONS(1224),
    [anon_sym_DASH_DASH] = ACTIONS(1226),
    [anon_sym_PLUS_PLUS] = ACTIONS(1226),
    [anon_sym_sizeof] = ACTIONS(1224),
    [anon_sym___alignof__] = ACTIONS(1224),
    [anon_sym___alignof] = ACTIONS(1224),
    [anon_sym__alignof] = ACTIONS(1224),
    [anon_sym_alignof] = ACTIONS(1224),
    [anon_sym__Alignof] = ACTIONS(1224),
    [anon_sym_offsetof] = ACTIONS(1224),
    [anon_sym__Generic] = ACTIONS(1224),
    [anon_sym_asm] = ACTIONS(1224),
    [anon_sym___asm__] = ACTIONS(1224),
    [sym_number_literal] = ACTIONS(1226),
    [anon_sym_L_SQUOTE] = ACTIONS(1226),
    [anon_sym_u_SQUOTE] = ACTIONS(1226),
    [anon_sym_U_SQUOTE] = ACTIONS(1226),
    [anon_sym_u8_SQUOTE] = ACTIONS(1226),
    [anon_sym_SQUOTE] = ACTIONS(1226),
    [anon_sym_L_DQUOTE] = ACTIONS(1226),
    [anon_sym_u_DQUOTE] = ACTIONS(1226),
    [anon_sym_U_DQUOTE] = ACTIONS(1226),
    [anon_sym_u8_DQUOTE] = ACTIONS(1226),
    [anon_sym_DQUOTE] = ACTIONS(1226),
    [sym_true] = ACTIONS(1224),
    [sym_false] = ACTIONS(1224),
    [anon_sym_NULL] = ACTIONS(1224),
    [anon_sym_nullptr] = ACTIONS(1224),
    [sym_comment] = ACTIONS(3),
  },
  [116] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [117] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [118] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [119] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [120] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [121] = {
    [sym_identifier] = ACTIONS(1228),
    [aux_sym_preproc_include_token1] = ACTIONS(1228),
    [aux_sym_preproc_def_token1] = ACTIONS(1228),
    [aux_sym_preproc_if_token1] = ACTIONS(1228),
    [aux_sym_preproc_if_token2] = ACTIONS(1228),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1228),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1228),
    [aux_sym_preproc_else_token1] = ACTIONS(1228),
    [aux_sym_preproc_elif_token1] = ACTIONS(1228),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1228),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1228),
    [sym_preproc_directive] = ACTIONS(1228),
    [anon_sym_LPAREN2] = ACTIONS(1230),
    [anon_sym_BANG] = ACTIONS(1230),
    [anon_sym_TILDE] = ACTIONS(1230),
    [anon_sym_DASH] = ACTIONS(1228),
    [anon_sym_PLUS] = ACTIONS(1228),
    [anon_sym_STAR] = ACTIONS(1230),
    [anon_sym_AMP] = ACTIONS(1230),
    [anon_sym_SEMI] = ACTIONS(1230),
    [anon_sym___extension__] = ACTIONS(1228),
    [anon_sym_typedef] = ACTIONS(1228),
    [anon_sym_extern] = ACTIONS(1228),
    [anon_sym___attribute__] = ACTIONS(1228),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1230),
    [anon_sym___declspec] = ACTIONS(1228),
    [anon_sym___cdecl] = ACTIONS(1228),
    [anon_sym___clrcall] = ACTIONS(1228),
    [anon_sym___stdcall] = ACTIONS(1228),
    [anon_sym___fastcall] = ACTIONS(1228),
    [anon_sym___thiscall] = ACTIONS(1228),
    [anon_sym___vectorcall] = ACTIONS(1228),
    [anon_sym_LBRACE] = ACTIONS(1230),
    [anon_sym_signed] = ACTIONS(1228),
    [anon_sym_unsigned] = ACTIONS(1228),
    [anon_sym_long] = ACTIONS(1228),
    [anon_sym_short] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_register] = ACTIONS(1228),
    [anon_sym_inline] = ACTIONS(1228),
    [anon_sym___inline] = ACTIONS(1228),
    [anon_sym___inline__] = ACTIONS(1228),
    [anon_sym___forceinline] = ACTIONS(1228),
    [anon_sym_thread_local] = ACTIONS(1228),
    [anon_sym___thread] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_constexpr] = ACTIONS(1228),
    [anon_sym_volatile] = ACTIONS(1228),
    [anon_sym_restrict] = ACTIONS(1228),
    [anon_sym___restrict__] = ACTIONS(1228),
    [anon_sym__Atomic] = ACTIONS(1228),
    [anon_sym__Noreturn] = ACTIONS(1228),
    [anon_sym_noreturn] = ACTIONS(1228),
    [sym_primitive_type] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_else] = ACTIONS(1228),
    [anon_sym_switch] = ACTIONS(1228),
    [anon_sym_case] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [anon_sym_do] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_goto] = ACTIONS(1228),
    [anon_sym___try] = ACTIONS(1228),
    [anon_sym___leave] = ACTIONS(1228),
    [anon_sym_DASH_DASH] = ACTIONS(1230),
    [anon_sym_PLUS_PLUS] = ACTIONS(1230),
    [anon_sym_sizeof] = ACTIONS(1228),
    [anon_sym___alignof__] = ACTIONS(1228),
    [anon_sym___alignof] = ACTIONS(1228),
    [anon_sym__alignof] = ACTIONS(1228),
    [anon_sym_alignof] = ACTIONS(1228),
    [anon_sym__Alignof] = ACTIONS(1228),
    [anon_sym_offsetof] = ACTIONS(1228),
    [anon_sym__Generic] = ACTIONS(1228),
    [anon_sym_asm] = ACTIONS(1228),
    [anon_sym___asm__] = ACTIONS(1228),
    [sym_number_literal] = ACTIONS(1230),
    [anon_sym_L_SQUOTE] = ACTIONS(1230),
    [anon_sym_u_SQUOTE] = ACTIONS(1230),
    [anon_sym_U_SQUOTE] = ACTIONS(1230),
    [anon_sym_u8_SQUOTE] = ACTIONS(1230),
    [anon_sym_SQUOTE] = ACTIONS(1230),
    [anon_sym_L_DQUOTE] = ACTIONS(1230),
    [anon_sym_u_DQUOTE] = ACTIONS(1230),
    [anon_sym_U_DQUOTE] = ACTIONS(1230),
    [anon_sym_u8_DQUOTE] = ACTIONS(1230),
    [anon_sym_DQUOTE] = ACTIONS(1230),
    [sym_true] = ACTIONS(1228),
    [sym_false] = ACTIONS(1228),
    [anon_sym_NULL] = ACTIONS(1228),
    [anon_sym_nullptr] = ACTIONS(1228),
    [sym_comment] = ACTIONS(3),
  },
  [122] = {
    [sym_identifier] = ACTIONS(1232),
    [aux_sym_preproc_include_token1] = ACTIONS(1232),
    [aux_sym_preproc_def_token1] = ACTIONS(1232),
    [aux_sym_preproc_if_token1] = ACTIONS(1232),
    [aux_sym_preproc_if_token2] = ACTIONS(1232),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1232),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1232),
    [aux_sym_preproc_else_token1] = ACTIONS(1232),
    [aux_sym_preproc_elif_token1] = ACTIONS(1232),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1232),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1232),
    [sym_preproc_directive] = ACTIONS(1232),
    [anon_sym_LPAREN2] = ACTIONS(1234),
    [anon_sym_BANG] = ACTIONS(1234),
    [anon_sym_TILDE] = ACTIONS(1234),
    [anon_sym_DASH] = ACTIONS(1232),
    [anon_sym_PLUS] = ACTIONS(1232),
    [anon_sym_STAR] = ACTIONS(1234),
    [anon_sym_AMP] = ACTIONS(1234),
    [anon_sym_SEMI] = ACTIONS(1234),
    [anon_sym___extension__] = ACTIONS(1232),
    [anon_sym_typedef] = ACTIONS(1232),
    [anon_sym_extern] = ACTIONS(1232),
    [anon_sym___attribute__] = ACTIONS(1232),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1234),
    [anon_sym___declspec] = ACTIONS(1232),
    [anon_sym___cdecl] = ACTIONS(1232),
    [anon_sym___clrcall] = ACTIONS(1232),
    [anon_sym___stdcall] = ACTIONS(1232),
    [anon_sym___fastcall] = ACTIONS(1232),
    [anon_sym___thiscall] = ACTIONS(1232),
    [anon_sym___vectorcall] = ACTIONS(1232),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_signed] = ACTIONS(1232),
    [anon_sym_unsigned] = ACTIONS(1232),
    [anon_sym_long] = ACTIONS(1232),
    [anon_sym_short] = ACTIONS(1232),
    [anon_sym_static] = ACTIONS(1232),
    [anon_sym_auto] = ACTIONS(1232),
    [anon_sym_register] = ACTIONS(1232),
    [anon_sym_inline] = ACTIONS(1232),
    [anon_sym___inline] = ACTIONS(1232),
    [anon_sym___inline__] = ACTIONS(1232),
    [anon_sym___forceinline] = ACTIONS(1232),
    [anon_sym_thread_local] = ACTIONS(1232),
    [anon_sym___thread] = ACTIONS(1232),
    [anon_sym_const] = ACTIONS(1232),
    [anon_sym_constexpr] = ACTIONS(1232),
    [anon_sym_volatile] = ACTIONS(1232),
    [anon_sym_restrict] = ACTIONS(1232),
    [anon_sym___restrict__] = ACTIONS(1232),
    [anon_sym__Atomic] = ACTIONS(1232),
    [anon_sym__Noreturn] = ACTIONS(1232),
    [anon_sym_noreturn] = ACTIONS(1232),
    [sym_primitive_type] = ACTIONS(1232),
    [anon_sym_enum] = ACTIONS(1232),
    [anon_sym_struct] = ACTIONS(1232),
    [anon_sym_union] = ACTIONS(1232),
    [anon_sym_if] = ACTIONS(1232),
    [anon_sym_else] = ACTIONS(1232),
    [anon_sym_switch] = ACTIONS(1232),
    [anon_sym_case] = ACTIONS(1232),
    [anon_sym_default] = ACTIONS(1232),
    [anon_sym_while] = ACTIONS(1232),
    [anon_sym_do] = ACTIONS(1232),
    [anon_sym_for] = ACTIONS(1232),
    [anon_sym_return] = ACTIONS(1232),
    [anon_sym_break] = ACTIONS(1232),
    [anon_sym_continue] = ACTIONS(1232),
    [anon_sym_goto] = ACTIONS(1232),
    [anon_sym___try] = ACTIONS(1232),
    [anon_sym___leave] = ACTIONS(1232),
    [anon_sym_DASH_DASH] = ACTIONS(1234),
    [anon_sym_PLUS_PLUS] = ACTIONS(1234),
    [anon_sym_sizeof] = ACTIONS(1232),
    [anon_sym___alignof__] = ACTIONS(1232),
    [anon_sym___alignof] = ACTIONS(1232),
    [anon_sym__alignof] = ACTIONS(1232),
    [anon_sym_alignof] = ACTIONS(1232),
    [anon_sym__Alignof] = ACTIONS(1232),
    [anon_sym_offsetof] = ACTIONS(1232),
    [anon_sym__Generic] = ACTIONS(1232),
    [anon_sym_asm] = ACTIONS(1232),
    [anon_sym___asm__] = ACTIONS(1232),
    [sym_number_literal] = ACTIONS(1234),
    [anon_sym_L_SQUOTE] = ACTIONS(1234),
    [anon_sym_u_SQUOTE] = ACTIONS(1234),
    [anon_sym_U_SQUOTE] = ACTIONS(1234),
    [anon_sym_u8_SQUOTE] = ACTIONS(1234),
    [anon_sym_SQUOTE] = ACTIONS(1234),
    [anon_sym_L_DQUOTE] = ACTIONS(1234),
    [anon_sym_u_DQUOTE] = ACTIONS(1234),
    [anon_sym_U_DQUOTE] = ACTIONS(1234),
    [anon_sym_u8_DQUOTE] = ACTIONS(1234),
    [anon_sym_DQUOTE] = ACTIONS(1234),
    [sym_true] = ACTIONS(1232),
    [sym_false] = ACTIONS(1232),
    [anon_sym_NULL] = ACTIONS(1232),
    [anon_sym_nullptr] = ACTIONS(1232),
    [sym_comment] = ACTIONS(3),
  },
  [123] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [124] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [125] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [126] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [127] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [aux_sym_preproc_else_token1] = ACTIONS(1116),
    [aux_sym_preproc_elif_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [128] = {
    [sym_identifier] = ACTIONS(1236),
    [aux_sym_preproc_include_token1] = ACTIONS(1236),
    [aux_sym_preproc_def_token1] = ACTIONS(1236),
    [aux_sym_preproc_if_token1] = ACTIONS(1236),
    [aux_sym_preproc_if_token2] = ACTIONS(1236),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1236),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1236),
    [aux_sym_preproc_else_token1] = ACTIONS(1236),
    [aux_sym_preproc_elif_token1] = ACTIONS(1236),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1236),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1236),
    [sym_preproc_directive] = ACTIONS(1236),
    [anon_sym_LPAREN2] = ACTIONS(1238),
    [anon_sym_BANG] = ACTIONS(1238),
    [anon_sym_TILDE] = ACTIONS(1238),
    [anon_sym_DASH] = ACTIONS(1236),
    [anon_sym_PLUS] = ACTIONS(1236),
    [anon_sym_STAR] = ACTIONS(1238),
    [anon_sym_AMP] = ACTIONS(1238),
    [anon_sym_SEMI] = ACTIONS(1238),
    [anon_sym___extension__] = ACTIONS(1236),
    [anon_sym_typedef] = ACTIONS(1236),
    [anon_sym_extern] = ACTIONS(1236),
    [anon_sym___attribute__] = ACTIONS(1236),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1238),
    [anon_sym___declspec] = ACTIONS(1236),
    [anon_sym___cdecl] = ACTIONS(1236),
    [anon_sym___clrcall] = ACTIONS(1236),
    [anon_sym___stdcall] = ACTIONS(1236),
    [anon_sym___fastcall] = ACTIONS(1236),
    [anon_sym___thiscall] = ACTIONS(1236),
    [anon_sym___vectorcall] = ACTIONS(1236),
    [anon_sym_LBRACE] = ACTIONS(1238),
    [anon_sym_signed] = ACTIONS(1236),
    [anon_sym_unsigned] = ACTIONS(1236),
    [anon_sym_long] = ACTIONS(1236),
    [anon_sym_short] = ACTIONS(1236),
    [anon_sym_static] = ACTIONS(1236),
    [anon_sym_auto] = ACTIONS(1236),
    [anon_sym_register] = ACTIONS(1236),
    [anon_sym_inline] = ACTIONS(1236),
    [anon_sym___inline] = ACTIONS(1236),
    [anon_sym___inline__] = ACTIONS(1236),
    [anon_sym___forceinline] = ACTIONS(1236),
    [anon_sym_thread_local] = ACTIONS(1236),
    [anon_sym___thread] = ACTIONS(1236),
    [anon_sym_const] = ACTIONS(1236),
    [anon_sym_constexpr] = ACTIONS(1236),
    [anon_sym_volatile] = ACTIONS(1236),
    [anon_sym_restrict] = ACTIONS(1236),
    [anon_sym___restrict__] = ACTIONS(1236),
    [anon_sym__Atomic] = ACTIONS(1236),
    [anon_sym__Noreturn] = ACTIONS(1236),
    [anon_sym_noreturn] = ACTIONS(1236),
    [sym_primitive_type] = ACTIONS(1236),
    [anon_sym_enum] = ACTIONS(1236),
    [anon_sym_struct] = ACTIONS(1236),
    [anon_sym_union] = ACTIONS(1236),
    [anon_sym_if] = ACTIONS(1236),
    [anon_sym_switch] = ACTIONS(1236),
    [anon_sym_case] = ACTIONS(1236),
    [anon_sym_default] = ACTIONS(1236),
    [anon_sym_while] = ACTIONS(1236),
    [anon_sym_do] = ACTIONS(1236),
    [anon_sym_for] = ACTIONS(1236),
    [anon_sym_return] = ACTIONS(1236),
    [anon_sym_break] = ACTIONS(1236),
    [anon_sym_continue] = ACTIONS(1236),
    [anon_sym_goto] = ACTIONS(1236),
    [anon_sym___try] = ACTIONS(1236),
    [anon_sym___leave] = ACTIONS(1236),
    [anon_sym_DASH_DASH] = ACTIONS(1238),
    [anon_sym_PLUS_PLUS] = ACTIONS(1238),
    [anon_sym_sizeof] = ACTIONS(1236),
    [anon_sym___alignof__] = ACTIONS(1236),
    [anon_sym___alignof] = ACTIONS(1236),
    [anon_sym__alignof] = ACTIONS(1236),
    [anon_sym_alignof] = ACTIONS(1236),
    [anon_sym__Alignof] = ACTIONS(1236),
    [anon_sym_offsetof] = ACTIONS(1236),
    [anon_sym__Generic] = ACTIONS(1236),
    [anon_sym_asm] = ACTIONS(1236),
    [anon_sym___asm__] = ACTIONS(1236),
    [sym_number_literal] = ACTIONS(1238),
    [anon_sym_L_SQUOTE] = ACTIONS(1238),
    [anon_sym_u_SQUOTE] = ACTIONS(1238),
    [anon_sym_U_SQUOTE] = ACTIONS(1238),
    [anon_sym_u8_SQUOTE] = ACTIONS(1238),
    [anon_sym_SQUOTE] = ACTIONS(1238),
    [anon_sym_L_DQUOTE] = ACTIONS(1238),
    [anon_sym_u_DQUOTE] = ACTIONS(1238),
    [anon_sym_U_DQUOTE] = ACTIONS(1238),
    [anon_sym_u8_DQUOTE] = ACTIONS(1238),
    [anon_sym_DQUOTE] = ACTIONS(1238),
    [sym_true] = ACTIONS(1236),
    [sym_false] = ACTIONS(1236),
    [anon_sym_NULL] = ACTIONS(1236),
    [anon_sym_nullptr] = ACTIONS(1236),
    [sym_comment] = ACTIONS(3),
  },
  [129] = {
    [sym_identifier] = ACTIONS(1240),
    [aux_sym_preproc_include_token1] = ACTIONS(1240),
    [aux_sym_preproc_def_token1] = ACTIONS(1240),
    [aux_sym_preproc_if_token1] = ACTIONS(1240),
    [aux_sym_preproc_if_token2] = ACTIONS(1240),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1240),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1240),
    [aux_sym_preproc_else_token1] = ACTIONS(1240),
    [aux_sym_preproc_elif_token1] = ACTIONS(1240),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1240),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1240),
    [sym_preproc_directive] = ACTIONS(1240),
    [anon_sym_LPAREN2] = ACTIONS(1242),
    [anon_sym_BANG] = ACTIONS(1242),
    [anon_sym_TILDE] = ACTIONS(1242),
    [anon_sym_DASH] = ACTIONS(1240),
    [anon_sym_PLUS] = ACTIONS(1240),
    [anon_sym_STAR] = ACTIONS(1242),
    [anon_sym_AMP] = ACTIONS(1242),
    [anon_sym_SEMI] = ACTIONS(1242),
    [anon_sym___extension__] = ACTIONS(1240),
    [anon_sym_typedef] = ACTIONS(1240),
    [anon_sym_extern] = ACTIONS(1240),
    [anon_sym___attribute__] = ACTIONS(1240),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1242),
    [anon_sym___declspec] = ACTIONS(1240),
    [anon_sym___cdecl] = ACTIONS(1240),
    [anon_sym___clrcall] = ACTIONS(1240),
    [anon_sym___stdcall] = ACTIONS(1240),
    [anon_sym___fastcall] = ACTIONS(1240),
    [anon_sym___thiscall] = ACTIONS(1240),
    [anon_sym___vectorcall] = ACTIONS(1240),
    [anon_sym_LBRACE] = ACTIONS(1242),
    [anon_sym_signed] = ACTIONS(1240),
    [anon_sym_unsigned] = ACTIONS(1240),
    [anon_sym_long] = ACTIONS(1240),
    [anon_sym_short] = ACTIONS(1240),
    [anon_sym_static] = ACTIONS(1240),
    [anon_sym_auto] = ACTIONS(1240),
    [anon_sym_register] = ACTIONS(1240),
    [anon_sym_inline] = ACTIONS(1240),
    [anon_sym___inline] = ACTIONS(1240),
    [anon_sym___inline__] = ACTIONS(1240),
    [anon_sym___forceinline] = ACTIONS(1240),
    [anon_sym_thread_local] = ACTIONS(1240),
    [anon_sym___thread] = ACTIONS(1240),
    [anon_sym_const] = ACTIONS(1240),
    [anon_sym_constexpr] = ACTIONS(1240),
    [anon_sym_volatile] = ACTIONS(1240),
    [anon_sym_restrict] = ACTIONS(1240),
    [anon_sym___restrict__] = ACTIONS(1240),
    [anon_sym__Atomic] = ACTIONS(1240),
    [anon_sym__Noreturn] = ACTIONS(1240),
    [anon_sym_noreturn] = ACTIONS(1240),
    [sym_primitive_type] = ACTIONS(1240),
    [anon_sym_enum] = ACTIONS(1240),
    [anon_sym_struct] = ACTIONS(1240),
    [anon_sym_union] = ACTIONS(1240),
    [anon_sym_if] = ACTIONS(1240),
    [anon_sym_switch] = ACTIONS(1240),
    [anon_sym_case] = ACTIONS(1240),
    [anon_sym_default] = ACTIONS(1240),
    [anon_sym_while] = ACTIONS(1240),
    [anon_sym_do] = ACTIONS(1240),
    [anon_sym_for] = ACTIONS(1240),
    [anon_sym_return] = ACTIONS(1240),
    [anon_sym_break] = ACTIONS(1240),
    [anon_sym_continue] = ACTIONS(1240),
    [anon_sym_goto] = ACTIONS(1240),
    [anon_sym___try] = ACTIONS(1240),
    [anon_sym___leave] = ACTIONS(1240),
    [anon_sym_DASH_DASH] = ACTIONS(1242),
    [anon_sym_PLUS_PLUS] = ACTIONS(1242),
    [anon_sym_sizeof] = ACTIONS(1240),
    [anon_sym___alignof__] = ACTIONS(1240),
    [anon_sym___alignof] = ACTIONS(1240),
    [anon_sym__alignof] = ACTIONS(1240),
    [anon_sym_alignof] = ACTIONS(1240),
    [anon_sym__Alignof] = ACTIONS(1240),
    [anon_sym_offsetof] = ACTIONS(1240),
    [anon_sym__Generic] = ACTIONS(1240),
    [anon_sym_asm] = ACTIONS(1240),
    [anon_sym___asm__] = ACTIONS(1240),
    [sym_number_literal] = ACTIONS(1242),
    [anon_sym_L_SQUOTE] = ACTIONS(1242),
    [anon_sym_u_SQUOTE] = ACTIONS(1242),
    [anon_sym_U_SQUOTE] = ACTIONS(1242),
    [anon_sym_u8_SQUOTE] = ACTIONS(1242),
    [anon_sym_SQUOTE] = ACTIONS(1242),
    [anon_sym_L_DQUOTE] = ACTIONS(1242),
    [anon_sym_u_DQUOTE] = ACTIONS(1242),
    [anon_sym_U_DQUOTE] = ACTIONS(1242),
    [anon_sym_u8_DQUOTE] = ACTIONS(1242),
    [anon_sym_DQUOTE] = ACTIONS(1242),
    [sym_true] = ACTIONS(1240),
    [sym_false] = ACTIONS(1240),
    [anon_sym_NULL] = ACTIONS(1240),
    [anon_sym_nullptr] = ACTIONS(1240),
    [sym_comment] = ACTIONS(3),
  },
  [130] = {
    [sym_identifier] = ACTIONS(1244),
    [aux_sym_preproc_include_token1] = ACTIONS(1244),
    [aux_sym_preproc_def_token1] = ACTIONS(1244),
    [aux_sym_preproc_if_token1] = ACTIONS(1244),
    [aux_sym_preproc_if_token2] = ACTIONS(1244),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1244),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1244),
    [aux_sym_preproc_else_token1] = ACTIONS(1244),
    [aux_sym_preproc_elif_token1] = ACTIONS(1244),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1244),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1244),
    [sym_preproc_directive] = ACTIONS(1244),
    [anon_sym_LPAREN2] = ACTIONS(1246),
    [anon_sym_BANG] = ACTIONS(1246),
    [anon_sym_TILDE] = ACTIONS(1246),
    [anon_sym_DASH] = ACTIONS(1244),
    [anon_sym_PLUS] = ACTIONS(1244),
    [anon_sym_STAR] = ACTIONS(1246),
    [anon_sym_AMP] = ACTIONS(1246),
    [anon_sym_SEMI] = ACTIONS(1246),
    [anon_sym___extension__] = ACTIONS(1244),
    [anon_sym_typedef] = ACTIONS(1244),
    [anon_sym_extern] = ACTIONS(1244),
    [anon_sym___attribute__] = ACTIONS(1244),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1246),
    [anon_sym___declspec] = ACTIONS(1244),
    [anon_sym___cdecl] = ACTIONS(1244),
    [anon_sym___clrcall] = ACTIONS(1244),
    [anon_sym___stdcall] = ACTIONS(1244),
    [anon_sym___fastcall] = ACTIONS(1244),
    [anon_sym___thiscall] = ACTIONS(1244),
    [anon_sym___vectorcall] = ACTIONS(1244),
    [anon_sym_LBRACE] = ACTIONS(1246),
    [anon_sym_signed] = ACTIONS(1244),
    [anon_sym_unsigned] = ACTIONS(1244),
    [anon_sym_long] = ACTIONS(1244),
    [anon_sym_short] = ACTIONS(1244),
    [anon_sym_static] = ACTIONS(1244),
    [anon_sym_auto] = ACTIONS(1244),
    [anon_sym_register] = ACTIONS(1244),
    [anon_sym_inline] = ACTIONS(1244),
    [anon_sym___inline] = ACTIONS(1244),
    [anon_sym___inline__] = ACTIONS(1244),
    [anon_sym___forceinline] = ACTIONS(1244),
    [anon_sym_thread_local] = ACTIONS(1244),
    [anon_sym___thread] = ACTIONS(1244),
    [anon_sym_const] = ACTIONS(1244),
    [anon_sym_constexpr] = ACTIONS(1244),
    [anon_sym_volatile] = ACTIONS(1244),
    [anon_sym_restrict] = ACTIONS(1244),
    [anon_sym___restrict__] = ACTIONS(1244),
    [anon_sym__Atomic] = ACTIONS(1244),
    [anon_sym__Noreturn] = ACTIONS(1244),
    [anon_sym_noreturn] = ACTIONS(1244),
    [sym_primitive_type] = ACTIONS(1244),
    [anon_sym_enum] = ACTIONS(1244),
    [anon_sym_struct] = ACTIONS(1244),
    [anon_sym_union] = ACTIONS(1244),
    [anon_sym_if] = ACTIONS(1244),
    [anon_sym_switch] = ACTIONS(1244),
    [anon_sym_case] = ACTIONS(1244),
    [anon_sym_default] = ACTIONS(1244),
    [anon_sym_while] = ACTIONS(1244),
    [anon_sym_do] = ACTIONS(1244),
    [anon_sym_for] = ACTIONS(1244),
    [anon_sym_return] = ACTIONS(1244),
    [anon_sym_break] = ACTIONS(1244),
    [anon_sym_continue] = ACTIONS(1244),
    [anon_sym_goto] = ACTIONS(1244),
    [anon_sym___try] = ACTIONS(1244),
    [anon_sym___leave] = ACTIONS(1244),
    [anon_sym_DASH_DASH] = ACTIONS(1246),
    [anon_sym_PLUS_PLUS] = ACTIONS(1246),
    [anon_sym_sizeof] = ACTIONS(1244),
    [anon_sym___alignof__] = ACTIONS(1244),
    [anon_sym___alignof] = ACTIONS(1244),
    [anon_sym__alignof] = ACTIONS(1244),
    [anon_sym_alignof] = ACTIONS(1244),
    [anon_sym__Alignof] = ACTIONS(1244),
    [anon_sym_offsetof] = ACTIONS(1244),
    [anon_sym__Generic] = ACTIONS(1244),
    [anon_sym_asm] = ACTIONS(1244),
    [anon_sym___asm__] = ACTIONS(1244),
    [sym_number_literal] = ACTIONS(1246),
    [anon_sym_L_SQUOTE] = ACTIONS(1246),
    [anon_sym_u_SQUOTE] = ACTIONS(1246),
    [anon_sym_U_SQUOTE] = ACTIONS(1246),
    [anon_sym_u8_SQUOTE] = ACTIONS(1246),
    [anon_sym_SQUOTE] = ACTIONS(1246),
    [anon_sym_L_DQUOTE] = ACTIONS(1246),
    [anon_sym_u_DQUOTE] = ACTIONS(1246),
    [anon_sym_U_DQUOTE] = ACTIONS(1246),
    [anon_sym_u8_DQUOTE] = ACTIONS(1246),
    [anon_sym_DQUOTE] = ACTIONS(1246),
    [sym_true] = ACTIONS(1244),
    [sym_false] = ACTIONS(1244),
    [anon_sym_NULL] = ACTIONS(1244),
    [anon_sym_nullptr] = ACTIONS(1244),
    [sym_comment] = ACTIONS(3),
  },
  [131] = {
    [sym_identifier] = ACTIONS(1248),
    [aux_sym_preproc_include_token1] = ACTIONS(1248),
    [aux_sym_preproc_def_token1] = ACTIONS(1248),
    [aux_sym_preproc_if_token1] = ACTIONS(1248),
    [aux_sym_preproc_if_token2] = ACTIONS(1248),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1248),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1248),
    [aux_sym_preproc_else_token1] = ACTIONS(1248),
    [aux_sym_preproc_elif_token1] = ACTIONS(1248),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1248),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1248),
    [sym_preproc_directive] = ACTIONS(1248),
    [anon_sym_LPAREN2] = ACTIONS(1250),
    [anon_sym_BANG] = ACTIONS(1250),
    [anon_sym_TILDE] = ACTIONS(1250),
    [anon_sym_DASH] = ACTIONS(1248),
    [anon_sym_PLUS] = ACTIONS(1248),
    [anon_sym_STAR] = ACTIONS(1250),
    [anon_sym_AMP] = ACTIONS(1250),
    [anon_sym_SEMI] = ACTIONS(1250),
    [anon_sym___extension__] = ACTIONS(1248),
    [anon_sym_typedef] = ACTIONS(1248),
    [anon_sym_extern] = ACTIONS(1248),
    [anon_sym___attribute__] = ACTIONS(1248),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1250),
    [anon_sym___declspec] = ACTIONS(1248),
    [anon_sym___cdecl] = ACTIONS(1248),
    [anon_sym___clrcall] = ACTIONS(1248),
    [anon_sym___stdcall] = ACTIONS(1248),
    [anon_sym___fastcall] = ACTIONS(1248),
    [anon_sym___thiscall] = ACTIONS(1248),
    [anon_sym___vectorcall] = ACTIONS(1248),
    [anon_sym_LBRACE] = ACTIONS(1250),
    [anon_sym_signed] = ACTIONS(1248),
    [anon_sym_unsigned] = ACTIONS(1248),
    [anon_sym_long] = ACTIONS(1248),
    [anon_sym_short] = ACTIONS(1248),
    [anon_sym_static] = ACTIONS(1248),
    [anon_sym_auto] = ACTIONS(1248),
    [anon_sym_register] = ACTIONS(1248),
    [anon_sym_inline] = ACTIONS(1248),
    [anon_sym___inline] = ACTIONS(1248),
    [anon_sym___inline__] = ACTIONS(1248),
    [anon_sym___forceinline] = ACTIONS(1248),
    [anon_sym_thread_local] = ACTIONS(1248),
    [anon_sym___thread] = ACTIONS(1248),
    [anon_sym_const] = ACTIONS(1248),
    [anon_sym_constexpr] = ACTIONS(1248),
    [anon_sym_volatile] = ACTIONS(1248),
    [anon_sym_restrict] = ACTIONS(1248),
    [anon_sym___restrict__] = ACTIONS(1248),
    [anon_sym__Atomic] = ACTIONS(1248),
    [anon_sym__Noreturn] = ACTIONS(1248),
    [anon_sym_noreturn] = ACTIONS(1248),
    [sym_primitive_type] = ACTIONS(1248),
    [anon_sym_enum] = ACTIONS(1248),
    [anon_sym_struct] = ACTIONS(1248),
    [anon_sym_union] = ACTIONS(1248),
    [anon_sym_if] = ACTIONS(1248),
    [anon_sym_switch] = ACTIONS(1248),
    [anon_sym_case] = ACTIONS(1248),
    [anon_sym_default] = ACTIONS(1248),
    [anon_sym_while] = ACTIONS(1248),
    [anon_sym_do] = ACTIONS(1248),
    [anon_sym_for] = ACTIONS(1248),
    [anon_sym_return] = ACTIONS(1248),
    [anon_sym_break] = ACTIONS(1248),
    [anon_sym_continue] = ACTIONS(1248),
    [anon_sym_goto] = ACTIONS(1248),
    [anon_sym___try] = ACTIONS(1248),
    [anon_sym___leave] = ACTIONS(1248),
    [anon_sym_DASH_DASH] = ACTIONS(1250),
    [anon_sym_PLUS_PLUS] = ACTIONS(1250),
    [anon_sym_sizeof] = ACTIONS(1248),
    [anon_sym___alignof__] = ACTIONS(1248),
    [anon_sym___alignof] = ACTIONS(1248),
    [anon_sym__alignof] = ACTIONS(1248),
    [anon_sym_alignof] = ACTIONS(1248),
    [anon_sym__Alignof] = ACTIONS(1248),
    [anon_sym_offsetof] = ACTIONS(1248),
    [anon_sym__Generic] = ACTIONS(1248),
    [anon_sym_asm] = ACTIONS(1248),
    [anon_sym___asm__] = ACTIONS(1248),
    [sym_number_literal] = ACTIONS(1250),
    [anon_sym_L_SQUOTE] = ACTIONS(1250),
    [anon_sym_u_SQUOTE] = ACTIONS(1250),
    [anon_sym_U_SQUOTE] = ACTIONS(1250),
    [anon_sym_u8_SQUOTE] = ACTIONS(1250),
    [anon_sym_SQUOTE] = ACTIONS(1250),
    [anon_sym_L_DQUOTE] = ACTIONS(1250),
    [anon_sym_u_DQUOTE] = ACTIONS(1250),
    [anon_sym_U_DQUOTE] = ACTIONS(1250),
    [anon_sym_u8_DQUOTE] = ACTIONS(1250),
    [anon_sym_DQUOTE] = ACTIONS(1250),
    [sym_true] = ACTIONS(1248),
    [sym_false] = ACTIONS(1248),
    [anon_sym_NULL] = ACTIONS(1248),
    [anon_sym_nullptr] = ACTIONS(1248),
    [sym_comment] = ACTIONS(3),
  },
  [132] = {
    [sym_identifier] = ACTIONS(1252),
    [aux_sym_preproc_include_token1] = ACTIONS(1252),
    [aux_sym_preproc_def_token1] = ACTIONS(1252),
    [aux_sym_preproc_if_token1] = ACTIONS(1252),
    [aux_sym_preproc_if_token2] = ACTIONS(1252),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1252),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1252),
    [aux_sym_preproc_else_token1] = ACTIONS(1252),
    [aux_sym_preproc_elif_token1] = ACTIONS(1252),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1252),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1252),
    [sym_preproc_directive] = ACTIONS(1252),
    [anon_sym_LPAREN2] = ACTIONS(1254),
    [anon_sym_BANG] = ACTIONS(1254),
    [anon_sym_TILDE] = ACTIONS(1254),
    [anon_sym_DASH] = ACTIONS(1252),
    [anon_sym_PLUS] = ACTIONS(1252),
    [anon_sym_STAR] = ACTIONS(1254),
    [anon_sym_AMP] = ACTIONS(1254),
    [anon_sym_SEMI] = ACTIONS(1254),
    [anon_sym___extension__] = ACTIONS(1252),
    [anon_sym_typedef] = ACTIONS(1252),
    [anon_sym_extern] = ACTIONS(1252),
    [anon_sym___attribute__] = ACTIONS(1252),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1254),
    [anon_sym___declspec] = ACTIONS(1252),
    [anon_sym___cdecl] = ACTIONS(1252),
    [anon_sym___clrcall] = ACTIONS(1252),
    [anon_sym___stdcall] = ACTIONS(1252),
    [anon_sym___fastcall] = ACTIONS(1252),
    [anon_sym___thiscall] = ACTIONS(1252),
    [anon_sym___vectorcall] = ACTIONS(1252),
    [anon_sym_LBRACE] = ACTIONS(1254),
    [anon_sym_signed] = ACTIONS(1252),
    [anon_sym_unsigned] = ACTIONS(1252),
    [anon_sym_long] = ACTIONS(1252),
    [anon_sym_short] = ACTIONS(1252),
    [anon_sym_static] = ACTIONS(1252),
    [anon_sym_auto] = ACTIONS(1252),
    [anon_sym_register] = ACTIONS(1252),
    [anon_sym_inline] = ACTIONS(1252),
    [anon_sym___inline] = ACTIONS(1252),
    [anon_sym___inline__] = ACTIONS(1252),
    [anon_sym___forceinline] = ACTIONS(1252),
    [anon_sym_thread_local] = ACTIONS(1252),
    [anon_sym___thread] = ACTIONS(1252),
    [anon_sym_const] = ACTIONS(1252),
    [anon_sym_constexpr] = ACTIONS(1252),
    [anon_sym_volatile] = ACTIONS(1252),
    [anon_sym_restrict] = ACTIONS(1252),
    [anon_sym___restrict__] = ACTIONS(1252),
    [anon_sym__Atomic] = ACTIONS(1252),
    [anon_sym__Noreturn] = ACTIONS(1252),
    [anon_sym_noreturn] = ACTIONS(1252),
    [sym_primitive_type] = ACTIONS(1252),
    [anon_sym_enum] = ACTIONS(1252),
    [anon_sym_struct] = ACTIONS(1252),
    [anon_sym_union] = ACTIONS(1252),
    [anon_sym_if] = ACTIONS(1252),
    [anon_sym_switch] = ACTIONS(1252),
    [anon_sym_case] = ACTIONS(1252),
    [anon_sym_default] = ACTIONS(1252),
    [anon_sym_while] = ACTIONS(1252),
    [anon_sym_do] = ACTIONS(1252),
    [anon_sym_for] = ACTIONS(1252),
    [anon_sym_return] = ACTIONS(1252),
    [anon_sym_break] = ACTIONS(1252),
    [anon_sym_continue] = ACTIONS(1252),
    [anon_sym_goto] = ACTIONS(1252),
    [anon_sym___try] = ACTIONS(1252),
    [anon_sym___leave] = ACTIONS(1252),
    [anon_sym_DASH_DASH] = ACTIONS(1254),
    [anon_sym_PLUS_PLUS] = ACTIONS(1254),
    [anon_sym_sizeof] = ACTIONS(1252),
    [anon_sym___alignof__] = ACTIONS(1252),
    [anon_sym___alignof] = ACTIONS(1252),
    [anon_sym__alignof] = ACTIONS(1252),
    [anon_sym_alignof] = ACTIONS(1252),
    [anon_sym__Alignof] = ACTIONS(1252),
    [anon_sym_offsetof] = ACTIONS(1252),
    [anon_sym__Generic] = ACTIONS(1252),
    [anon_sym_asm] = ACTIONS(1252),
    [anon_sym___asm__] = ACTIONS(1252),
    [sym_number_literal] = ACTIONS(1254),
    [anon_sym_L_SQUOTE] = ACTIONS(1254),
    [anon_sym_u_SQUOTE] = ACTIONS(1254),
    [anon_sym_U_SQUOTE] = ACTIONS(1254),
    [anon_sym_u8_SQUOTE] = ACTIONS(1254),
    [anon_sym_SQUOTE] = ACTIONS(1254),
    [anon_sym_L_DQUOTE] = ACTIONS(1254),
    [anon_sym_u_DQUOTE] = ACTIONS(1254),
    [anon_sym_U_DQUOTE] = ACTIONS(1254),
    [anon_sym_u8_DQUOTE] = ACTIONS(1254),
    [anon_sym_DQUOTE] = ACTIONS(1254),
    [sym_true] = ACTIONS(1252),
    [sym_false] = ACTIONS(1252),
    [anon_sym_NULL] = ACTIONS(1252),
    [anon_sym_nullptr] = ACTIONS(1252),
    [sym_comment] = ACTIONS(3),
  },
  [133] = {
    [sym_identifier] = ACTIONS(1256),
    [aux_sym_preproc_include_token1] = ACTIONS(1256),
    [aux_sym_preproc_def_token1] = ACTIONS(1256),
    [aux_sym_preproc_if_token1] = ACTIONS(1256),
    [aux_sym_preproc_if_token2] = ACTIONS(1256),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1256),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1256),
    [aux_sym_preproc_else_token1] = ACTIONS(1256),
    [aux_sym_preproc_elif_token1] = ACTIONS(1256),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1256),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1256),
    [sym_preproc_directive] = ACTIONS(1256),
    [anon_sym_LPAREN2] = ACTIONS(1258),
    [anon_sym_BANG] = ACTIONS(1258),
    [anon_sym_TILDE] = ACTIONS(1258),
    [anon_sym_DASH] = ACTIONS(1256),
    [anon_sym_PLUS] = ACTIONS(1256),
    [anon_sym_STAR] = ACTIONS(1258),
    [anon_sym_AMP] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1258),
    [anon_sym___extension__] = ACTIONS(1256),
    [anon_sym_typedef] = ACTIONS(1256),
    [anon_sym_extern] = ACTIONS(1256),
    [anon_sym___attribute__] = ACTIONS(1256),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1258),
    [anon_sym___declspec] = ACTIONS(1256),
    [anon_sym___cdecl] = ACTIONS(1256),
    [anon_sym___clrcall] = ACTIONS(1256),
    [anon_sym___stdcall] = ACTIONS(1256),
    [anon_sym___fastcall] = ACTIONS(1256),
    [anon_sym___thiscall] = ACTIONS(1256),
    [anon_sym___vectorcall] = ACTIONS(1256),
    [anon_sym_LBRACE] = ACTIONS(1258),
    [anon_sym_signed] = ACTIONS(1256),
    [anon_sym_unsigned] = ACTIONS(1256),
    [anon_sym_long] = ACTIONS(1256),
    [anon_sym_short] = ACTIONS(1256),
    [anon_sym_static] = ACTIONS(1256),
    [anon_sym_auto] = ACTIONS(1256),
    [anon_sym_register] = ACTIONS(1256),
    [anon_sym_inline] = ACTIONS(1256),
    [anon_sym___inline] = ACTIONS(1256),
    [anon_sym___inline__] = ACTIONS(1256),
    [anon_sym___forceinline] = ACTIONS(1256),
    [anon_sym_thread_local] = ACTIONS(1256),
    [anon_sym___thread] = ACTIONS(1256),
    [anon_sym_const] = ACTIONS(1256),
    [anon_sym_constexpr] = ACTIONS(1256),
    [anon_sym_volatile] = ACTIONS(1256),
    [anon_sym_restrict] = ACTIONS(1256),
    [anon_sym___restrict__] = ACTIONS(1256),
    [anon_sym__Atomic] = ACTIONS(1256),
    [anon_sym__Noreturn] = ACTIONS(1256),
    [anon_sym_noreturn] = ACTIONS(1256),
    [sym_primitive_type] = ACTIONS(1256),
    [anon_sym_enum] = ACTIONS(1256),
    [anon_sym_struct] = ACTIONS(1256),
    [anon_sym_union] = ACTIONS(1256),
    [anon_sym_if] = ACTIONS(1256),
    [anon_sym_switch] = ACTIONS(1256),
    [anon_sym_case] = ACTIONS(1256),
    [anon_sym_default] = ACTIONS(1256),
    [anon_sym_while] = ACTIONS(1256),
    [anon_sym_do] = ACTIONS(1256),
    [anon_sym_for] = ACTIONS(1256),
    [anon_sym_return] = ACTIONS(1256),
    [anon_sym_break] = ACTIONS(1256),
    [anon_sym_continue] = ACTIONS(1256),
    [anon_sym_goto] = ACTIONS(1256),
    [anon_sym___try] = ACTIONS(1256),
    [anon_sym___leave] = ACTIONS(1256),
    [anon_sym_DASH_DASH] = ACTIONS(1258),
    [anon_sym_PLUS_PLUS] = ACTIONS(1258),
    [anon_sym_sizeof] = ACTIONS(1256),
    [anon_sym___alignof__] = ACTIONS(1256),
    [anon_sym___alignof] = ACTIONS(1256),
    [anon_sym__alignof] = ACTIONS(1256),
    [anon_sym_alignof] = ACTIONS(1256),
    [anon_sym__Alignof] = ACTIONS(1256),
    [anon_sym_offsetof] = ACTIONS(1256),
    [anon_sym__Generic] = ACTIONS(1256),
    [anon_sym_asm] = ACTIONS(1256),
    [anon_sym___asm__] = ACTIONS(1256),
    [sym_number_literal] = ACTIONS(1258),
    [anon_sym_L_SQUOTE] = ACTIONS(1258),
    [anon_sym_u_SQUOTE] = ACTIONS(1258),
    [anon_sym_U_SQUOTE] = ACTIONS(1258),
    [anon_sym_u8_SQUOTE] = ACTIONS(1258),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_L_DQUOTE] = ACTIONS(1258),
    [anon_sym_u_DQUOTE] = ACTIONS(1258),
    [anon_sym_U_DQUOTE] = ACTIONS(1258),
    [anon_sym_u8_DQUOTE] = ACTIONS(1258),
    [anon_sym_DQUOTE] = ACTIONS(1258),
    [sym_true] = ACTIONS(1256),
    [sym_false] = ACTIONS(1256),
    [anon_sym_NULL] = ACTIONS(1256),
    [anon_sym_nullptr] = ACTIONS(1256),
    [sym_comment] = ACTIONS(3),
  },
  [134] = {
    [sym_identifier] = ACTIONS(1260),
    [aux_sym_preproc_include_token1] = ACTIONS(1260),
    [aux_sym_preproc_def_token1] = ACTIONS(1260),
    [aux_sym_preproc_if_token1] = ACTIONS(1260),
    [aux_sym_preproc_if_token2] = ACTIONS(1260),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1260),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1260),
    [aux_sym_preproc_else_token1] = ACTIONS(1260),
    [aux_sym_preproc_elif_token1] = ACTIONS(1260),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1260),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1260),
    [sym_preproc_directive] = ACTIONS(1260),
    [anon_sym_LPAREN2] = ACTIONS(1262),
    [anon_sym_BANG] = ACTIONS(1262),
    [anon_sym_TILDE] = ACTIONS(1262),
    [anon_sym_DASH] = ACTIONS(1260),
    [anon_sym_PLUS] = ACTIONS(1260),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1262),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym___extension__] = ACTIONS(1260),
    [anon_sym_typedef] = ACTIONS(1260),
    [anon_sym_extern] = ACTIONS(1260),
    [anon_sym___attribute__] = ACTIONS(1260),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1262),
    [anon_sym___declspec] = ACTIONS(1260),
    [anon_sym___cdecl] = ACTIONS(1260),
    [anon_sym___clrcall] = ACTIONS(1260),
    [anon_sym___stdcall] = ACTIONS(1260),
    [anon_sym___fastcall] = ACTIONS(1260),
    [anon_sym___thiscall] = ACTIONS(1260),
    [anon_sym___vectorcall] = ACTIONS(1260),
    [anon_sym_LBRACE] = ACTIONS(1262),
    [anon_sym_signed] = ACTIONS(1260),
    [anon_sym_unsigned] = ACTIONS(1260),
    [anon_sym_long] = ACTIONS(1260),
    [anon_sym_short] = ACTIONS(1260),
    [anon_sym_static] = ACTIONS(1260),
    [anon_sym_auto] = ACTIONS(1260),
    [anon_sym_register] = ACTIONS(1260),
    [anon_sym_inline] = ACTIONS(1260),
    [anon_sym___inline] = ACTIONS(1260),
    [anon_sym___inline__] = ACTIONS(1260),
    [anon_sym___forceinline] = ACTIONS(1260),
    [anon_sym_thread_local] = ACTIONS(1260),
    [anon_sym___thread] = ACTIONS(1260),
    [anon_sym_const] = ACTIONS(1260),
    [anon_sym_constexpr] = ACTIONS(1260),
    [anon_sym_volatile] = ACTIONS(1260),
    [anon_sym_restrict] = ACTIONS(1260),
    [anon_sym___restrict__] = ACTIONS(1260),
    [anon_sym__Atomic] = ACTIONS(1260),
    [anon_sym__Noreturn] = ACTIONS(1260),
    [anon_sym_noreturn] = ACTIONS(1260),
    [sym_primitive_type] = ACTIONS(1260),
    [anon_sym_enum] = ACTIONS(1260),
    [anon_sym_struct] = ACTIONS(1260),
    [anon_sym_union] = ACTIONS(1260),
    [anon_sym_if] = ACTIONS(1260),
    [anon_sym_switch] = ACTIONS(1260),
    [anon_sym_case] = ACTIONS(1260),
    [anon_sym_default] = ACTIONS(1260),
    [anon_sym_while] = ACTIONS(1260),
    [anon_sym_do] = ACTIONS(1260),
    [anon_sym_for] = ACTIONS(1260),
    [anon_sym_return] = ACTIONS(1260),
    [anon_sym_break] = ACTIONS(1260),
    [anon_sym_continue] = ACTIONS(1260),
    [anon_sym_goto] = ACTIONS(1260),
    [anon_sym___try] = ACTIONS(1260),
    [anon_sym___leave] = ACTIONS(1260),
    [anon_sym_DASH_DASH] = ACTIONS(1262),
    [anon_sym_PLUS_PLUS] = ACTIONS(1262),
    [anon_sym_sizeof] = ACTIONS(1260),
    [anon_sym___alignof__] = ACTIONS(1260),
    [anon_sym___alignof] = ACTIONS(1260),
    [anon_sym__alignof] = ACTIONS(1260),
    [anon_sym_alignof] = ACTIONS(1260),
    [anon_sym__Alignof] = ACTIONS(1260),
    [anon_sym_offsetof] = ACTIONS(1260),
    [anon_sym__Generic] = ACTIONS(1260),
    [anon_sym_asm] = ACTIONS(1260),
    [anon_sym___asm__] = ACTIONS(1260),
    [sym_number_literal] = ACTIONS(1262),
    [anon_sym_L_SQUOTE] = ACTIONS(1262),
    [anon_sym_u_SQUOTE] = ACTIONS(1262),
    [anon_sym_U_SQUOTE] = ACTIONS(1262),
    [anon_sym_u8_SQUOTE] = ACTIONS(1262),
    [anon_sym_SQUOTE] = ACTIONS(1262),
    [anon_sym_L_DQUOTE] = ACTIONS(1262),
    [anon_sym_u_DQUOTE] = ACTIONS(1262),
    [anon_sym_U_DQUOTE] = ACTIONS(1262),
    [anon_sym_u8_DQUOTE] = ACTIONS(1262),
    [anon_sym_DQUOTE] = ACTIONS(1262),
    [sym_true] = ACTIONS(1260),
    [sym_false] = ACTIONS(1260),
    [anon_sym_NULL] = ACTIONS(1260),
    [anon_sym_nullptr] = ACTIONS(1260),
    [sym_comment] = ACTIONS(3),
  },
  [135] = {
    [sym_identifier] = ACTIONS(1264),
    [aux_sym_preproc_include_token1] = ACTIONS(1264),
    [aux_sym_preproc_def_token1] = ACTIONS(1264),
    [aux_sym_preproc_if_token1] = ACTIONS(1264),
    [aux_sym_preproc_if_token2] = ACTIONS(1264),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1264),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1264),
    [aux_sym_preproc_else_token1] = ACTIONS(1264),
    [aux_sym_preproc_elif_token1] = ACTIONS(1264),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1264),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1264),
    [sym_preproc_directive] = ACTIONS(1264),
    [anon_sym_LPAREN2] = ACTIONS(1266),
    [anon_sym_BANG] = ACTIONS(1266),
    [anon_sym_TILDE] = ACTIONS(1266),
    [anon_sym_DASH] = ACTIONS(1264),
    [anon_sym_PLUS] = ACTIONS(1264),
    [anon_sym_STAR] = ACTIONS(1266),
    [anon_sym_AMP] = ACTIONS(1266),
    [anon_sym_SEMI] = ACTIONS(1266),
    [anon_sym___extension__] = ACTIONS(1264),
    [anon_sym_typedef] = ACTIONS(1264),
    [anon_sym_extern] = ACTIONS(1264),
    [anon_sym___attribute__] = ACTIONS(1264),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1266),
    [anon_sym___declspec] = ACTIONS(1264),
    [anon_sym___cdecl] = ACTIONS(1264),
    [anon_sym___clrcall] = ACTIONS(1264),
    [anon_sym___stdcall] = ACTIONS(1264),
    [anon_sym___fastcall] = ACTIONS(1264),
    [anon_sym___thiscall] = ACTIONS(1264),
    [anon_sym___vectorcall] = ACTIONS(1264),
    [anon_sym_LBRACE] = ACTIONS(1266),
    [anon_sym_signed] = ACTIONS(1264),
    [anon_sym_unsigned] = ACTIONS(1264),
    [anon_sym_long] = ACTIONS(1264),
    [anon_sym_short] = ACTIONS(1264),
    [anon_sym_static] = ACTIONS(1264),
    [anon_sym_auto] = ACTIONS(1264),
    [anon_sym_register] = ACTIONS(1264),
    [anon_sym_inline] = ACTIONS(1264),
    [anon_sym___inline] = ACTIONS(1264),
    [anon_sym___inline__] = ACTIONS(1264),
    [anon_sym___forceinline] = ACTIONS(1264),
    [anon_sym_thread_local] = ACTIONS(1264),
    [anon_sym___thread] = ACTIONS(1264),
    [anon_sym_const] = ACTIONS(1264),
    [anon_sym_constexpr] = ACTIONS(1264),
    [anon_sym_volatile] = ACTIONS(1264),
    [anon_sym_restrict] = ACTIONS(1264),
    [anon_sym___restrict__] = ACTIONS(1264),
    [anon_sym__Atomic] = ACTIONS(1264),
    [anon_sym__Noreturn] = ACTIONS(1264),
    [anon_sym_noreturn] = ACTIONS(1264),
    [sym_primitive_type] = ACTIONS(1264),
    [anon_sym_enum] = ACTIONS(1264),
    [anon_sym_struct] = ACTIONS(1264),
    [anon_sym_union] = ACTIONS(1264),
    [anon_sym_if] = ACTIONS(1264),
    [anon_sym_switch] = ACTIONS(1264),
    [anon_sym_case] = ACTIONS(1264),
    [anon_sym_default] = ACTIONS(1264),
    [anon_sym_while] = ACTIONS(1264),
    [anon_sym_do] = ACTIONS(1264),
    [anon_sym_for] = ACTIONS(1264),
    [anon_sym_return] = ACTIONS(1264),
    [anon_sym_break] = ACTIONS(1264),
    [anon_sym_continue] = ACTIONS(1264),
    [anon_sym_goto] = ACTIONS(1264),
    [anon_sym___try] = ACTIONS(1264),
    [anon_sym___leave] = ACTIONS(1264),
    [anon_sym_DASH_DASH] = ACTIONS(1266),
    [anon_sym_PLUS_PLUS] = ACTIONS(1266),
    [anon_sym_sizeof] = ACTIONS(1264),
    [anon_sym___alignof__] = ACTIONS(1264),
    [anon_sym___alignof] = ACTIONS(1264),
    [anon_sym__alignof] = ACTIONS(1264),
    [anon_sym_alignof] = ACTIONS(1264),
    [anon_sym__Alignof] = ACTIONS(1264),
    [anon_sym_offsetof] = ACTIONS(1264),
    [anon_sym__Generic] = ACTIONS(1264),
    [anon_sym_asm] = ACTIONS(1264),
    [anon_sym___asm__] = ACTIONS(1264),
    [sym_number_literal] = ACTIONS(1266),
    [anon_sym_L_SQUOTE] = ACTIONS(1266),
    [anon_sym_u_SQUOTE] = ACTIONS(1266),
    [anon_sym_U_SQUOTE] = ACTIONS(1266),
    [anon_sym_u8_SQUOTE] = ACTIONS(1266),
    [anon_sym_SQUOTE] = ACTIONS(1266),
    [anon_sym_L_DQUOTE] = ACTIONS(1266),
    [anon_sym_u_DQUOTE] = ACTIONS(1266),
    [anon_sym_U_DQUOTE] = ACTIONS(1266),
    [anon_sym_u8_DQUOTE] = ACTIONS(1266),
    [anon_sym_DQUOTE] = ACTIONS(1266),
    [sym_true] = ACTIONS(1264),
    [sym_false] = ACTIONS(1264),
    [anon_sym_NULL] = ACTIONS(1264),
    [anon_sym_nullptr] = ACTIONS(1264),
    [sym_comment] = ACTIONS(3),
  },
  [136] = {
    [sym_identifier] = ACTIONS(1268),
    [aux_sym_preproc_include_token1] = ACTIONS(1268),
    [aux_sym_preproc_def_token1] = ACTIONS(1268),
    [aux_sym_preproc_if_token1] = ACTIONS(1268),
    [aux_sym_preproc_if_token2] = ACTIONS(1268),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1268),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1268),
    [aux_sym_preproc_else_token1] = ACTIONS(1268),
    [aux_sym_preproc_elif_token1] = ACTIONS(1268),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1268),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1268),
    [sym_preproc_directive] = ACTIONS(1268),
    [anon_sym_LPAREN2] = ACTIONS(1270),
    [anon_sym_BANG] = ACTIONS(1270),
    [anon_sym_TILDE] = ACTIONS(1270),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1270),
    [anon_sym_AMP] = ACTIONS(1270),
    [anon_sym_SEMI] = ACTIONS(1270),
    [anon_sym___extension__] = ACTIONS(1268),
    [anon_sym_typedef] = ACTIONS(1268),
    [anon_sym_extern] = ACTIONS(1268),
    [anon_sym___attribute__] = ACTIONS(1268),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1270),
    [anon_sym___declspec] = ACTIONS(1268),
    [anon_sym___cdecl] = ACTIONS(1268),
    [anon_sym___clrcall] = ACTIONS(1268),
    [anon_sym___stdcall] = ACTIONS(1268),
    [anon_sym___fastcall] = ACTIONS(1268),
    [anon_sym___thiscall] = ACTIONS(1268),
    [anon_sym___vectorcall] = ACTIONS(1268),
    [anon_sym_LBRACE] = ACTIONS(1270),
    [anon_sym_signed] = ACTIONS(1268),
    [anon_sym_unsigned] = ACTIONS(1268),
    [anon_sym_long] = ACTIONS(1268),
    [anon_sym_short] = ACTIONS(1268),
    [anon_sym_static] = ACTIONS(1268),
    [anon_sym_auto] = ACTIONS(1268),
    [anon_sym_register] = ACTIONS(1268),
    [anon_sym_inline] = ACTIONS(1268),
    [anon_sym___inline] = ACTIONS(1268),
    [anon_sym___inline__] = ACTIONS(1268),
    [anon_sym___forceinline] = ACTIONS(1268),
    [anon_sym_thread_local] = ACTIONS(1268),
    [anon_sym___thread] = ACTIONS(1268),
    [anon_sym_const] = ACTIONS(1268),
    [anon_sym_constexpr] = ACTIONS(1268),
    [anon_sym_volatile] = ACTIONS(1268),
    [anon_sym_restrict] = ACTIONS(1268),
    [anon_sym___restrict__] = ACTIONS(1268),
    [anon_sym__Atomic] = ACTIONS(1268),
    [anon_sym__Noreturn] = ACTIONS(1268),
    [anon_sym_noreturn] = ACTIONS(1268),
    [sym_primitive_type] = ACTIONS(1268),
    [anon_sym_enum] = ACTIONS(1268),
    [anon_sym_struct] = ACTIONS(1268),
    [anon_sym_union] = ACTIONS(1268),
    [anon_sym_if] = ACTIONS(1268),
    [anon_sym_switch] = ACTIONS(1268),
    [anon_sym_case] = ACTIONS(1268),
    [anon_sym_default] = ACTIONS(1268),
    [anon_sym_while] = ACTIONS(1268),
    [anon_sym_do] = ACTIONS(1268),
    [anon_sym_for] = ACTIONS(1268),
    [anon_sym_return] = ACTIONS(1268),
    [anon_sym_break] = ACTIONS(1268),
    [anon_sym_continue] = ACTIONS(1268),
    [anon_sym_goto] = ACTIONS(1268),
    [anon_sym___try] = ACTIONS(1268),
    [anon_sym___leave] = ACTIONS(1268),
    [anon_sym_DASH_DASH] = ACTIONS(1270),
    [anon_sym_PLUS_PLUS] = ACTIONS(1270),
    [anon_sym_sizeof] = ACTIONS(1268),
    [anon_sym___alignof__] = ACTIONS(1268),
    [anon_sym___alignof] = ACTIONS(1268),
    [anon_sym__alignof] = ACTIONS(1268),
    [anon_sym_alignof] = ACTIONS(1268),
    [anon_sym__Alignof] = ACTIONS(1268),
    [anon_sym_offsetof] = ACTIONS(1268),
    [anon_sym__Generic] = ACTIONS(1268),
    [anon_sym_asm] = ACTIONS(1268),
    [anon_sym___asm__] = ACTIONS(1268),
    [sym_number_literal] = ACTIONS(1270),
    [anon_sym_L_SQUOTE] = ACTIONS(1270),
    [anon_sym_u_SQUOTE] = ACTIONS(1270),
    [anon_sym_U_SQUOTE] = ACTIONS(1270),
    [anon_sym_u8_SQUOTE] = ACTIONS(1270),
    [anon_sym_SQUOTE] = ACTIONS(1270),
    [anon_sym_L_DQUOTE] = ACTIONS(1270),
    [anon_sym_u_DQUOTE] = ACTIONS(1270),
    [anon_sym_U_DQUOTE] = ACTIONS(1270),
    [anon_sym_u8_DQUOTE] = ACTIONS(1270),
    [anon_sym_DQUOTE] = ACTIONS(1270),
    [sym_true] = ACTIONS(1268),
    [sym_false] = ACTIONS(1268),
    [anon_sym_NULL] = ACTIONS(1268),
    [anon_sym_nullptr] = ACTIONS(1268),
    [sym_comment] = ACTIONS(3),
  },
  [137] = {
    [sym_identifier] = ACTIONS(1272),
    [aux_sym_preproc_include_token1] = ACTIONS(1272),
    [aux_sym_preproc_def_token1] = ACTIONS(1272),
    [aux_sym_preproc_if_token1] = ACTIONS(1272),
    [aux_sym_preproc_if_token2] = ACTIONS(1272),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1272),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1272),
    [aux_sym_preproc_else_token1] = ACTIONS(1272),
    [aux_sym_preproc_elif_token1] = ACTIONS(1272),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1272),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1272),
    [sym_preproc_directive] = ACTIONS(1272),
    [anon_sym_LPAREN2] = ACTIONS(1274),
    [anon_sym_BANG] = ACTIONS(1274),
    [anon_sym_TILDE] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1272),
    [anon_sym_STAR] = ACTIONS(1274),
    [anon_sym_AMP] = ACTIONS(1274),
    [anon_sym_SEMI] = ACTIONS(1274),
    [anon_sym___extension__] = ACTIONS(1272),
    [anon_sym_typedef] = ACTIONS(1272),
    [anon_sym_extern] = ACTIONS(1272),
    [anon_sym___attribute__] = ACTIONS(1272),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1274),
    [anon_sym___declspec] = ACTIONS(1272),
    [anon_sym___cdecl] = ACTIONS(1272),
    [anon_sym___clrcall] = ACTIONS(1272),
    [anon_sym___stdcall] = ACTIONS(1272),
    [anon_sym___fastcall] = ACTIONS(1272),
    [anon_sym___thiscall] = ACTIONS(1272),
    [anon_sym___vectorcall] = ACTIONS(1272),
    [anon_sym_LBRACE] = ACTIONS(1274),
    [anon_sym_signed] = ACTIONS(1272),
    [anon_sym_unsigned] = ACTIONS(1272),
    [anon_sym_long] = ACTIONS(1272),
    [anon_sym_short] = ACTIONS(1272),
    [anon_sym_static] = ACTIONS(1272),
    [anon_sym_auto] = ACTIONS(1272),
    [anon_sym_register] = ACTIONS(1272),
    [anon_sym_inline] = ACTIONS(1272),
    [anon_sym___inline] = ACTIONS(1272),
    [anon_sym___inline__] = ACTIONS(1272),
    [anon_sym___forceinline] = ACTIONS(1272),
    [anon_sym_thread_local] = ACTIONS(1272),
    [anon_sym___thread] = ACTIONS(1272),
    [anon_sym_const] = ACTIONS(1272),
    [anon_sym_constexpr] = ACTIONS(1272),
    [anon_sym_volatile] = ACTIONS(1272),
    [anon_sym_restrict] = ACTIONS(1272),
    [anon_sym___restrict__] = ACTIONS(1272),
    [anon_sym__Atomic] = ACTIONS(1272),
    [anon_sym__Noreturn] = ACTIONS(1272),
    [anon_sym_noreturn] = ACTIONS(1272),
    [sym_primitive_type] = ACTIONS(1272),
    [anon_sym_enum] = ACTIONS(1272),
    [anon_sym_struct] = ACTIONS(1272),
    [anon_sym_union] = ACTIONS(1272),
    [anon_sym_if] = ACTIONS(1272),
    [anon_sym_switch] = ACTIONS(1272),
    [anon_sym_case] = ACTIONS(1272),
    [anon_sym_default] = ACTIONS(1272),
    [anon_sym_while] = ACTIONS(1272),
    [anon_sym_do] = ACTIONS(1272),
    [anon_sym_for] = ACTIONS(1272),
    [anon_sym_return] = ACTIONS(1272),
    [anon_sym_break] = ACTIONS(1272),
    [anon_sym_continue] = ACTIONS(1272),
    [anon_sym_goto] = ACTIONS(1272),
    [anon_sym___try] = ACTIONS(1272),
    [anon_sym___leave] = ACTIONS(1272),
    [anon_sym_DASH_DASH] = ACTIONS(1274),
    [anon_sym_PLUS_PLUS] = ACTIONS(1274),
    [anon_sym_sizeof] = ACTIONS(1272),
    [anon_sym___alignof__] = ACTIONS(1272),
    [anon_sym___alignof] = ACTIONS(1272),
    [anon_sym__alignof] = ACTIONS(1272),
    [anon_sym_alignof] = ACTIONS(1272),
    [anon_sym__Alignof] = ACTIONS(1272),
    [anon_sym_offsetof] = ACTIONS(1272),
    [anon_sym__Generic] = ACTIONS(1272),
    [anon_sym_asm] = ACTIONS(1272),
    [anon_sym___asm__] = ACTIONS(1272),
    [sym_number_literal] = ACTIONS(1274),
    [anon_sym_L_SQUOTE] = ACTIONS(1274),
    [anon_sym_u_SQUOTE] = ACTIONS(1274),
    [anon_sym_U_SQUOTE] = ACTIONS(1274),
    [anon_sym_u8_SQUOTE] = ACTIONS(1274),
    [anon_sym_SQUOTE] = ACTIONS(1274),
    [anon_sym_L_DQUOTE] = ACTIONS(1274),
    [anon_sym_u_DQUOTE] = ACTIONS(1274),
    [anon_sym_U_DQUOTE] = ACTIONS(1274),
    [anon_sym_u8_DQUOTE] = ACTIONS(1274),
    [anon_sym_DQUOTE] = ACTIONS(1274),
    [sym_true] = ACTIONS(1272),
    [sym_false] = ACTIONS(1272),
    [anon_sym_NULL] = ACTIONS(1272),
    [anon_sym_nullptr] = ACTIONS(1272),
    [sym_comment] = ACTIONS(3),
  },
  [138] = {
    [sym_identifier] = ACTIONS(1276),
    [aux_sym_preproc_include_token1] = ACTIONS(1276),
    [aux_sym_preproc_def_token1] = ACTIONS(1276),
    [aux_sym_preproc_if_token1] = ACTIONS(1276),
    [aux_sym_preproc_if_token2] = ACTIONS(1276),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1276),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1276),
    [aux_sym_preproc_else_token1] = ACTIONS(1276),
    [aux_sym_preproc_elif_token1] = ACTIONS(1276),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1276),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1276),
    [sym_preproc_directive] = ACTIONS(1276),
    [anon_sym_LPAREN2] = ACTIONS(1278),
    [anon_sym_BANG] = ACTIONS(1278),
    [anon_sym_TILDE] = ACTIONS(1278),
    [anon_sym_DASH] = ACTIONS(1276),
    [anon_sym_PLUS] = ACTIONS(1276),
    [anon_sym_STAR] = ACTIONS(1278),
    [anon_sym_AMP] = ACTIONS(1278),
    [anon_sym_SEMI] = ACTIONS(1278),
    [anon_sym___extension__] = ACTIONS(1276),
    [anon_sym_typedef] = ACTIONS(1276),
    [anon_sym_extern] = ACTIONS(1276),
    [anon_sym___attribute__] = ACTIONS(1276),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1278),
    [anon_sym___declspec] = ACTIONS(1276),
    [anon_sym___cdecl] = ACTIONS(1276),
    [anon_sym___clrcall] = ACTIONS(1276),
    [anon_sym___stdcall] = ACTIONS(1276),
    [anon_sym___fastcall] = ACTIONS(1276),
    [anon_sym___thiscall] = ACTIONS(1276),
    [anon_sym___vectorcall] = ACTIONS(1276),
    [anon_sym_LBRACE] = ACTIONS(1278),
    [anon_sym_signed] = ACTIONS(1276),
    [anon_sym_unsigned] = ACTIONS(1276),
    [anon_sym_long] = ACTIONS(1276),
    [anon_sym_short] = ACTIONS(1276),
    [anon_sym_static] = ACTIONS(1276),
    [anon_sym_auto] = ACTIONS(1276),
    [anon_sym_register] = ACTIONS(1276),
    [anon_sym_inline] = ACTIONS(1276),
    [anon_sym___inline] = ACTIONS(1276),
    [anon_sym___inline__] = ACTIONS(1276),
    [anon_sym___forceinline] = ACTIONS(1276),
    [anon_sym_thread_local] = ACTIONS(1276),
    [anon_sym___thread] = ACTIONS(1276),
    [anon_sym_const] = ACTIONS(1276),
    [anon_sym_constexpr] = ACTIONS(1276),
    [anon_sym_volatile] = ACTIONS(1276),
    [anon_sym_restrict] = ACTIONS(1276),
    [anon_sym___restrict__] = ACTIONS(1276),
    [anon_sym__Atomic] = ACTIONS(1276),
    [anon_sym__Noreturn] = ACTIONS(1276),
    [anon_sym_noreturn] = ACTIONS(1276),
    [sym_primitive_type] = ACTIONS(1276),
    [anon_sym_enum] = ACTIONS(1276),
    [anon_sym_struct] = ACTIONS(1276),
    [anon_sym_union] = ACTIONS(1276),
    [anon_sym_if] = ACTIONS(1276),
    [anon_sym_switch] = ACTIONS(1276),
    [anon_sym_case] = ACTIONS(1276),
    [anon_sym_default] = ACTIONS(1276),
    [anon_sym_while] = ACTIONS(1276),
    [anon_sym_do] = ACTIONS(1276),
    [anon_sym_for] = ACTIONS(1276),
    [anon_sym_return] = ACTIONS(1276),
    [anon_sym_break] = ACTIONS(1276),
    [anon_sym_continue] = ACTIONS(1276),
    [anon_sym_goto] = ACTIONS(1276),
    [anon_sym___try] = ACTIONS(1276),
    [anon_sym___leave] = ACTIONS(1276),
    [anon_sym_DASH_DASH] = ACTIONS(1278),
    [anon_sym_PLUS_PLUS] = ACTIONS(1278),
    [anon_sym_sizeof] = ACTIONS(1276),
    [anon_sym___alignof__] = ACTIONS(1276),
    [anon_sym___alignof] = ACTIONS(1276),
    [anon_sym__alignof] = ACTIONS(1276),
    [anon_sym_alignof] = ACTIONS(1276),
    [anon_sym__Alignof] = ACTIONS(1276),
    [anon_sym_offsetof] = ACTIONS(1276),
    [anon_sym__Generic] = ACTIONS(1276),
    [anon_sym_asm] = ACTIONS(1276),
    [anon_sym___asm__] = ACTIONS(1276),
    [sym_number_literal] = ACTIONS(1278),
    [anon_sym_L_SQUOTE] = ACTIONS(1278),
    [anon_sym_u_SQUOTE] = ACTIONS(1278),
    [anon_sym_U_SQUOTE] = ACTIONS(1278),
    [anon_sym_u8_SQUOTE] = ACTIONS(1278),
    [anon_sym_SQUOTE] = ACTIONS(1278),
    [anon_sym_L_DQUOTE] = ACTIONS(1278),
    [anon_sym_u_DQUOTE] = ACTIONS(1278),
    [anon_sym_U_DQUOTE] = ACTIONS(1278),
    [anon_sym_u8_DQUOTE] = ACTIONS(1278),
    [anon_sym_DQUOTE] = ACTIONS(1278),
    [sym_true] = ACTIONS(1276),
    [sym_false] = ACTIONS(1276),
    [anon_sym_NULL] = ACTIONS(1276),
    [anon_sym_nullptr] = ACTIONS(1276),
    [sym_comment] = ACTIONS(3),
  },
  [139] = {
    [sym_identifier] = ACTIONS(1280),
    [aux_sym_preproc_include_token1] = ACTIONS(1280),
    [aux_sym_preproc_def_token1] = ACTIONS(1280),
    [aux_sym_preproc_if_token1] = ACTIONS(1280),
    [aux_sym_preproc_if_token2] = ACTIONS(1280),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1280),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1280),
    [aux_sym_preproc_else_token1] = ACTIONS(1280),
    [aux_sym_preproc_elif_token1] = ACTIONS(1280),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1280),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1280),
    [sym_preproc_directive] = ACTIONS(1280),
    [anon_sym_LPAREN2] = ACTIONS(1282),
    [anon_sym_BANG] = ACTIONS(1282),
    [anon_sym_TILDE] = ACTIONS(1282),
    [anon_sym_DASH] = ACTIONS(1280),
    [anon_sym_PLUS] = ACTIONS(1280),
    [anon_sym_STAR] = ACTIONS(1282),
    [anon_sym_AMP] = ACTIONS(1282),
    [anon_sym_SEMI] = ACTIONS(1282),
    [anon_sym___extension__] = ACTIONS(1280),
    [anon_sym_typedef] = ACTIONS(1280),
    [anon_sym_extern] = ACTIONS(1280),
    [anon_sym___attribute__] = ACTIONS(1280),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1282),
    [anon_sym___declspec] = ACTIONS(1280),
    [anon_sym___cdecl] = ACTIONS(1280),
    [anon_sym___clrcall] = ACTIONS(1280),
    [anon_sym___stdcall] = ACTIONS(1280),
    [anon_sym___fastcall] = ACTIONS(1280),
    [anon_sym___thiscall] = ACTIONS(1280),
    [anon_sym___vectorcall] = ACTIONS(1280),
    [anon_sym_LBRACE] = ACTIONS(1282),
    [anon_sym_signed] = ACTIONS(1280),
    [anon_sym_unsigned] = ACTIONS(1280),
    [anon_sym_long] = ACTIONS(1280),
    [anon_sym_short] = ACTIONS(1280),
    [anon_sym_static] = ACTIONS(1280),
    [anon_sym_auto] = ACTIONS(1280),
    [anon_sym_register] = ACTIONS(1280),
    [anon_sym_inline] = ACTIONS(1280),
    [anon_sym___inline] = ACTIONS(1280),
    [anon_sym___inline__] = ACTIONS(1280),
    [anon_sym___forceinline] = ACTIONS(1280),
    [anon_sym_thread_local] = ACTIONS(1280),
    [anon_sym___thread] = ACTIONS(1280),
    [anon_sym_const] = ACTIONS(1280),
    [anon_sym_constexpr] = ACTIONS(1280),
    [anon_sym_volatile] = ACTIONS(1280),
    [anon_sym_restrict] = ACTIONS(1280),
    [anon_sym___restrict__] = ACTIONS(1280),
    [anon_sym__Atomic] = ACTIONS(1280),
    [anon_sym__Noreturn] = ACTIONS(1280),
    [anon_sym_noreturn] = ACTIONS(1280),
    [sym_primitive_type] = ACTIONS(1280),
    [anon_sym_enum] = ACTIONS(1280),
    [anon_sym_struct] = ACTIONS(1280),
    [anon_sym_union] = ACTIONS(1280),
    [anon_sym_if] = ACTIONS(1280),
    [anon_sym_switch] = ACTIONS(1280),
    [anon_sym_case] = ACTIONS(1280),
    [anon_sym_default] = ACTIONS(1280),
    [anon_sym_while] = ACTIONS(1280),
    [anon_sym_do] = ACTIONS(1280),
    [anon_sym_for] = ACTIONS(1280),
    [anon_sym_return] = ACTIONS(1280),
    [anon_sym_break] = ACTIONS(1280),
    [anon_sym_continue] = ACTIONS(1280),
    [anon_sym_goto] = ACTIONS(1280),
    [anon_sym___try] = ACTIONS(1280),
    [anon_sym___leave] = ACTIONS(1280),
    [anon_sym_DASH_DASH] = ACTIONS(1282),
    [anon_sym_PLUS_PLUS] = ACTIONS(1282),
    [anon_sym_sizeof] = ACTIONS(1280),
    [anon_sym___alignof__] = ACTIONS(1280),
    [anon_sym___alignof] = ACTIONS(1280),
    [anon_sym__alignof] = ACTIONS(1280),
    [anon_sym_alignof] = ACTIONS(1280),
    [anon_sym__Alignof] = ACTIONS(1280),
    [anon_sym_offsetof] = ACTIONS(1280),
    [anon_sym__Generic] = ACTIONS(1280),
    [anon_sym_asm] = ACTIONS(1280),
    [anon_sym___asm__] = ACTIONS(1280),
    [sym_number_literal] = ACTIONS(1282),
    [anon_sym_L_SQUOTE] = ACTIONS(1282),
    [anon_sym_u_SQUOTE] = ACTIONS(1282),
    [anon_sym_U_SQUOTE] = ACTIONS(1282),
    [anon_sym_u8_SQUOTE] = ACTIONS(1282),
    [anon_sym_SQUOTE] = ACTIONS(1282),
    [anon_sym_L_DQUOTE] = ACTIONS(1282),
    [anon_sym_u_DQUOTE] = ACTIONS(1282),
    [anon_sym_U_DQUOTE] = ACTIONS(1282),
    [anon_sym_u8_DQUOTE] = ACTIONS(1282),
    [anon_sym_DQUOTE] = ACTIONS(1282),
    [sym_true] = ACTIONS(1280),
    [sym_false] = ACTIONS(1280),
    [anon_sym_NULL] = ACTIONS(1280),
    [anon_sym_nullptr] = ACTIONS(1280),
    [sym_comment] = ACTIONS(3),
  },
  [140] = {
    [sym_identifier] = ACTIONS(1284),
    [aux_sym_preproc_include_token1] = ACTIONS(1284),
    [aux_sym_preproc_def_token1] = ACTIONS(1284),
    [aux_sym_preproc_if_token1] = ACTIONS(1284),
    [aux_sym_preproc_if_token2] = ACTIONS(1284),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1284),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1284),
    [aux_sym_preproc_else_token1] = ACTIONS(1284),
    [aux_sym_preproc_elif_token1] = ACTIONS(1284),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1284),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1284),
    [sym_preproc_directive] = ACTIONS(1284),
    [anon_sym_LPAREN2] = ACTIONS(1286),
    [anon_sym_BANG] = ACTIONS(1286),
    [anon_sym_TILDE] = ACTIONS(1286),
    [anon_sym_DASH] = ACTIONS(1284),
    [anon_sym_PLUS] = ACTIONS(1284),
    [anon_sym_STAR] = ACTIONS(1286),
    [anon_sym_AMP] = ACTIONS(1286),
    [anon_sym_SEMI] = ACTIONS(1286),
    [anon_sym___extension__] = ACTIONS(1284),
    [anon_sym_typedef] = ACTIONS(1284),
    [anon_sym_extern] = ACTIONS(1284),
    [anon_sym___attribute__] = ACTIONS(1284),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1286),
    [anon_sym___declspec] = ACTIONS(1284),
    [anon_sym___cdecl] = ACTIONS(1284),
    [anon_sym___clrcall] = ACTIONS(1284),
    [anon_sym___stdcall] = ACTIONS(1284),
    [anon_sym___fastcall] = ACTIONS(1284),
    [anon_sym___thiscall] = ACTIONS(1284),
    [anon_sym___vectorcall] = ACTIONS(1284),
    [anon_sym_LBRACE] = ACTIONS(1286),
    [anon_sym_signed] = ACTIONS(1284),
    [anon_sym_unsigned] = ACTIONS(1284),
    [anon_sym_long] = ACTIONS(1284),
    [anon_sym_short] = ACTIONS(1284),
    [anon_sym_static] = ACTIONS(1284),
    [anon_sym_auto] = ACTIONS(1284),
    [anon_sym_register] = ACTIONS(1284),
    [anon_sym_inline] = ACTIONS(1284),
    [anon_sym___inline] = ACTIONS(1284),
    [anon_sym___inline__] = ACTIONS(1284),
    [anon_sym___forceinline] = ACTIONS(1284),
    [anon_sym_thread_local] = ACTIONS(1284),
    [anon_sym___thread] = ACTIONS(1284),
    [anon_sym_const] = ACTIONS(1284),
    [anon_sym_constexpr] = ACTIONS(1284),
    [anon_sym_volatile] = ACTIONS(1284),
    [anon_sym_restrict] = ACTIONS(1284),
    [anon_sym___restrict__] = ACTIONS(1284),
    [anon_sym__Atomic] = ACTIONS(1284),
    [anon_sym__Noreturn] = ACTIONS(1284),
    [anon_sym_noreturn] = ACTIONS(1284),
    [sym_primitive_type] = ACTIONS(1284),
    [anon_sym_enum] = ACTIONS(1284),
    [anon_sym_struct] = ACTIONS(1284),
    [anon_sym_union] = ACTIONS(1284),
    [anon_sym_if] = ACTIONS(1284),
    [anon_sym_switch] = ACTIONS(1284),
    [anon_sym_case] = ACTIONS(1284),
    [anon_sym_default] = ACTIONS(1284),
    [anon_sym_while] = ACTIONS(1284),
    [anon_sym_do] = ACTIONS(1284),
    [anon_sym_for] = ACTIONS(1284),
    [anon_sym_return] = ACTIONS(1284),
    [anon_sym_break] = ACTIONS(1284),
    [anon_sym_continue] = ACTIONS(1284),
    [anon_sym_goto] = ACTIONS(1284),
    [anon_sym___try] = ACTIONS(1284),
    [anon_sym___leave] = ACTIONS(1284),
    [anon_sym_DASH_DASH] = ACTIONS(1286),
    [anon_sym_PLUS_PLUS] = ACTIONS(1286),
    [anon_sym_sizeof] = ACTIONS(1284),
    [anon_sym___alignof__] = ACTIONS(1284),
    [anon_sym___alignof] = ACTIONS(1284),
    [anon_sym__alignof] = ACTIONS(1284),
    [anon_sym_alignof] = ACTIONS(1284),
    [anon_sym__Alignof] = ACTIONS(1284),
    [anon_sym_offsetof] = ACTIONS(1284),
    [anon_sym__Generic] = ACTIONS(1284),
    [anon_sym_asm] = ACTIONS(1284),
    [anon_sym___asm__] = ACTIONS(1284),
    [sym_number_literal] = ACTIONS(1286),
    [anon_sym_L_SQUOTE] = ACTIONS(1286),
    [anon_sym_u_SQUOTE] = ACTIONS(1286),
    [anon_sym_U_SQUOTE] = ACTIONS(1286),
    [anon_sym_u8_SQUOTE] = ACTIONS(1286),
    [anon_sym_SQUOTE] = ACTIONS(1286),
    [anon_sym_L_DQUOTE] = ACTIONS(1286),
    [anon_sym_u_DQUOTE] = ACTIONS(1286),
    [anon_sym_U_DQUOTE] = ACTIONS(1286),
    [anon_sym_u8_DQUOTE] = ACTIONS(1286),
    [anon_sym_DQUOTE] = ACTIONS(1286),
    [sym_true] = ACTIONS(1284),
    [sym_false] = ACTIONS(1284),
    [anon_sym_NULL] = ACTIONS(1284),
    [anon_sym_nullptr] = ACTIONS(1284),
    [sym_comment] = ACTIONS(3),
  },
  [141] = {
    [sym_identifier] = ACTIONS(1288),
    [aux_sym_preproc_include_token1] = ACTIONS(1288),
    [aux_sym_preproc_def_token1] = ACTIONS(1288),
    [aux_sym_preproc_if_token1] = ACTIONS(1288),
    [aux_sym_preproc_if_token2] = ACTIONS(1288),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1288),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1288),
    [aux_sym_preproc_else_token1] = ACTIONS(1288),
    [aux_sym_preproc_elif_token1] = ACTIONS(1288),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1288),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1288),
    [sym_preproc_directive] = ACTIONS(1288),
    [anon_sym_LPAREN2] = ACTIONS(1290),
    [anon_sym_BANG] = ACTIONS(1290),
    [anon_sym_TILDE] = ACTIONS(1290),
    [anon_sym_DASH] = ACTIONS(1288),
    [anon_sym_PLUS] = ACTIONS(1288),
    [anon_sym_STAR] = ACTIONS(1290),
    [anon_sym_AMP] = ACTIONS(1290),
    [anon_sym_SEMI] = ACTIONS(1290),
    [anon_sym___extension__] = ACTIONS(1288),
    [anon_sym_typedef] = ACTIONS(1288),
    [anon_sym_extern] = ACTIONS(1288),
    [anon_sym___attribute__] = ACTIONS(1288),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1290),
    [anon_sym___declspec] = ACTIONS(1288),
    [anon_sym___cdecl] = ACTIONS(1288),
    [anon_sym___clrcall] = ACTIONS(1288),
    [anon_sym___stdcall] = ACTIONS(1288),
    [anon_sym___fastcall] = ACTIONS(1288),
    [anon_sym___thiscall] = ACTIONS(1288),
    [anon_sym___vectorcall] = ACTIONS(1288),
    [anon_sym_LBRACE] = ACTIONS(1290),
    [anon_sym_signed] = ACTIONS(1288),
    [anon_sym_unsigned] = ACTIONS(1288),
    [anon_sym_long] = ACTIONS(1288),
    [anon_sym_short] = ACTIONS(1288),
    [anon_sym_static] = ACTIONS(1288),
    [anon_sym_auto] = ACTIONS(1288),
    [anon_sym_register] = ACTIONS(1288),
    [anon_sym_inline] = ACTIONS(1288),
    [anon_sym___inline] = ACTIONS(1288),
    [anon_sym___inline__] = ACTIONS(1288),
    [anon_sym___forceinline] = ACTIONS(1288),
    [anon_sym_thread_local] = ACTIONS(1288),
    [anon_sym___thread] = ACTIONS(1288),
    [anon_sym_const] = ACTIONS(1288),
    [anon_sym_constexpr] = ACTIONS(1288),
    [anon_sym_volatile] = ACTIONS(1288),
    [anon_sym_restrict] = ACTIONS(1288),
    [anon_sym___restrict__] = ACTIONS(1288),
    [anon_sym__Atomic] = ACTIONS(1288),
    [anon_sym__Noreturn] = ACTIONS(1288),
    [anon_sym_noreturn] = ACTIONS(1288),
    [sym_primitive_type] = ACTIONS(1288),
    [anon_sym_enum] = ACTIONS(1288),
    [anon_sym_struct] = ACTIONS(1288),
    [anon_sym_union] = ACTIONS(1288),
    [anon_sym_if] = ACTIONS(1288),
    [anon_sym_switch] = ACTIONS(1288),
    [anon_sym_case] = ACTIONS(1288),
    [anon_sym_default] = ACTIONS(1288),
    [anon_sym_while] = ACTIONS(1288),
    [anon_sym_do] = ACTIONS(1288),
    [anon_sym_for] = ACTIONS(1288),
    [anon_sym_return] = ACTIONS(1288),
    [anon_sym_break] = ACTIONS(1288),
    [anon_sym_continue] = ACTIONS(1288),
    [anon_sym_goto] = ACTIONS(1288),
    [anon_sym___try] = ACTIONS(1288),
    [anon_sym___leave] = ACTIONS(1288),
    [anon_sym_DASH_DASH] = ACTIONS(1290),
    [anon_sym_PLUS_PLUS] = ACTIONS(1290),
    [anon_sym_sizeof] = ACTIONS(1288),
    [anon_sym___alignof__] = ACTIONS(1288),
    [anon_sym___alignof] = ACTIONS(1288),
    [anon_sym__alignof] = ACTIONS(1288),
    [anon_sym_alignof] = ACTIONS(1288),
    [anon_sym__Alignof] = ACTIONS(1288),
    [anon_sym_offsetof] = ACTIONS(1288),
    [anon_sym__Generic] = ACTIONS(1288),
    [anon_sym_asm] = ACTIONS(1288),
    [anon_sym___asm__] = ACTIONS(1288),
    [sym_number_literal] = ACTIONS(1290),
    [anon_sym_L_SQUOTE] = ACTIONS(1290),
    [anon_sym_u_SQUOTE] = ACTIONS(1290),
    [anon_sym_U_SQUOTE] = ACTIONS(1290),
    [anon_sym_u8_SQUOTE] = ACTIONS(1290),
    [anon_sym_SQUOTE] = ACTIONS(1290),
    [anon_sym_L_DQUOTE] = ACTIONS(1290),
    [anon_sym_u_DQUOTE] = ACTIONS(1290),
    [anon_sym_U_DQUOTE] = ACTIONS(1290),
    [anon_sym_u8_DQUOTE] = ACTIONS(1290),
    [anon_sym_DQUOTE] = ACTIONS(1290),
    [sym_true] = ACTIONS(1288),
    [sym_false] = ACTIONS(1288),
    [anon_sym_NULL] = ACTIONS(1288),
    [anon_sym_nullptr] = ACTIONS(1288),
    [sym_comment] = ACTIONS(3),
  },
  [142] = {
    [sym_identifier] = ACTIONS(1292),
    [aux_sym_preproc_include_token1] = ACTIONS(1292),
    [aux_sym_preproc_def_token1] = ACTIONS(1292),
    [aux_sym_preproc_if_token1] = ACTIONS(1292),
    [aux_sym_preproc_if_token2] = ACTIONS(1292),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1292),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1292),
    [aux_sym_preproc_else_token1] = ACTIONS(1292),
    [aux_sym_preproc_elif_token1] = ACTIONS(1292),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1292),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1292),
    [sym_preproc_directive] = ACTIONS(1292),
    [anon_sym_LPAREN2] = ACTIONS(1294),
    [anon_sym_BANG] = ACTIONS(1294),
    [anon_sym_TILDE] = ACTIONS(1294),
    [anon_sym_DASH] = ACTIONS(1292),
    [anon_sym_PLUS] = ACTIONS(1292),
    [anon_sym_STAR] = ACTIONS(1294),
    [anon_sym_AMP] = ACTIONS(1294),
    [anon_sym_SEMI] = ACTIONS(1294),
    [anon_sym___extension__] = ACTIONS(1292),
    [anon_sym_typedef] = ACTIONS(1292),
    [anon_sym_extern] = ACTIONS(1292),
    [anon_sym___attribute__] = ACTIONS(1292),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1294),
    [anon_sym___declspec] = ACTIONS(1292),
    [anon_sym___cdecl] = ACTIONS(1292),
    [anon_sym___clrcall] = ACTIONS(1292),
    [anon_sym___stdcall] = ACTIONS(1292),
    [anon_sym___fastcall] = ACTIONS(1292),
    [anon_sym___thiscall] = ACTIONS(1292),
    [anon_sym___vectorcall] = ACTIONS(1292),
    [anon_sym_LBRACE] = ACTIONS(1294),
    [anon_sym_signed] = ACTIONS(1292),
    [anon_sym_unsigned] = ACTIONS(1292),
    [anon_sym_long] = ACTIONS(1292),
    [anon_sym_short] = ACTIONS(1292),
    [anon_sym_static] = ACTIONS(1292),
    [anon_sym_auto] = ACTIONS(1292),
    [anon_sym_register] = ACTIONS(1292),
    [anon_sym_inline] = ACTIONS(1292),
    [anon_sym___inline] = ACTIONS(1292),
    [anon_sym___inline__] = ACTIONS(1292),
    [anon_sym___forceinline] = ACTIONS(1292),
    [anon_sym_thread_local] = ACTIONS(1292),
    [anon_sym___thread] = ACTIONS(1292),
    [anon_sym_const] = ACTIONS(1292),
    [anon_sym_constexpr] = ACTIONS(1292),
    [anon_sym_volatile] = ACTIONS(1292),
    [anon_sym_restrict] = ACTIONS(1292),
    [anon_sym___restrict__] = ACTIONS(1292),
    [anon_sym__Atomic] = ACTIONS(1292),
    [anon_sym__Noreturn] = ACTIONS(1292),
    [anon_sym_noreturn] = ACTIONS(1292),
    [sym_primitive_type] = ACTIONS(1292),
    [anon_sym_enum] = ACTIONS(1292),
    [anon_sym_struct] = ACTIONS(1292),
    [anon_sym_union] = ACTIONS(1292),
    [anon_sym_if] = ACTIONS(1292),
    [anon_sym_switch] = ACTIONS(1292),
    [anon_sym_case] = ACTIONS(1292),
    [anon_sym_default] = ACTIONS(1292),
    [anon_sym_while] = ACTIONS(1292),
    [anon_sym_do] = ACTIONS(1292),
    [anon_sym_for] = ACTIONS(1292),
    [anon_sym_return] = ACTIONS(1292),
    [anon_sym_break] = ACTIONS(1292),
    [anon_sym_continue] = ACTIONS(1292),
    [anon_sym_goto] = ACTIONS(1292),
    [anon_sym___try] = ACTIONS(1292),
    [anon_sym___leave] = ACTIONS(1292),
    [anon_sym_DASH_DASH] = ACTIONS(1294),
    [anon_sym_PLUS_PLUS] = ACTIONS(1294),
    [anon_sym_sizeof] = ACTIONS(1292),
    [anon_sym___alignof__] = ACTIONS(1292),
    [anon_sym___alignof] = ACTIONS(1292),
    [anon_sym__alignof] = ACTIONS(1292),
    [anon_sym_alignof] = ACTIONS(1292),
    [anon_sym__Alignof] = ACTIONS(1292),
    [anon_sym_offsetof] = ACTIONS(1292),
    [anon_sym__Generic] = ACTIONS(1292),
    [anon_sym_asm] = ACTIONS(1292),
    [anon_sym___asm__] = ACTIONS(1292),
    [sym_number_literal] = ACTIONS(1294),
    [anon_sym_L_SQUOTE] = ACTIONS(1294),
    [anon_sym_u_SQUOTE] = ACTIONS(1294),
    [anon_sym_U_SQUOTE] = ACTIONS(1294),
    [anon_sym_u8_SQUOTE] = ACTIONS(1294),
    [anon_sym_SQUOTE] = ACTIONS(1294),
    [anon_sym_L_DQUOTE] = ACTIONS(1294),
    [anon_sym_u_DQUOTE] = ACTIONS(1294),
    [anon_sym_U_DQUOTE] = ACTIONS(1294),
    [anon_sym_u8_DQUOTE] = ACTIONS(1294),
    [anon_sym_DQUOTE] = ACTIONS(1294),
    [sym_true] = ACTIONS(1292),
    [sym_false] = ACTIONS(1292),
    [anon_sym_NULL] = ACTIONS(1292),
    [anon_sym_nullptr] = ACTIONS(1292),
    [sym_comment] = ACTIONS(3),
  },
  [143] = {
    [sym_identifier] = ACTIONS(1296),
    [aux_sym_preproc_include_token1] = ACTIONS(1296),
    [aux_sym_preproc_def_token1] = ACTIONS(1296),
    [aux_sym_preproc_if_token1] = ACTIONS(1296),
    [aux_sym_preproc_if_token2] = ACTIONS(1296),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1296),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1296),
    [aux_sym_preproc_else_token1] = ACTIONS(1296),
    [aux_sym_preproc_elif_token1] = ACTIONS(1296),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1296),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1296),
    [sym_preproc_directive] = ACTIONS(1296),
    [anon_sym_LPAREN2] = ACTIONS(1298),
    [anon_sym_BANG] = ACTIONS(1298),
    [anon_sym_TILDE] = ACTIONS(1298),
    [anon_sym_DASH] = ACTIONS(1296),
    [anon_sym_PLUS] = ACTIONS(1296),
    [anon_sym_STAR] = ACTIONS(1298),
    [anon_sym_AMP] = ACTIONS(1298),
    [anon_sym_SEMI] = ACTIONS(1298),
    [anon_sym___extension__] = ACTIONS(1296),
    [anon_sym_typedef] = ACTIONS(1296),
    [anon_sym_extern] = ACTIONS(1296),
    [anon_sym___attribute__] = ACTIONS(1296),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1298),
    [anon_sym___declspec] = ACTIONS(1296),
    [anon_sym___cdecl] = ACTIONS(1296),
    [anon_sym___clrcall] = ACTIONS(1296),
    [anon_sym___stdcall] = ACTIONS(1296),
    [anon_sym___fastcall] = ACTIONS(1296),
    [anon_sym___thiscall] = ACTIONS(1296),
    [anon_sym___vectorcall] = ACTIONS(1296),
    [anon_sym_LBRACE] = ACTIONS(1298),
    [anon_sym_signed] = ACTIONS(1296),
    [anon_sym_unsigned] = ACTIONS(1296),
    [anon_sym_long] = ACTIONS(1296),
    [anon_sym_short] = ACTIONS(1296),
    [anon_sym_static] = ACTIONS(1296),
    [anon_sym_auto] = ACTIONS(1296),
    [anon_sym_register] = ACTIONS(1296),
    [anon_sym_inline] = ACTIONS(1296),
    [anon_sym___inline] = ACTIONS(1296),
    [anon_sym___inline__] = ACTIONS(1296),
    [anon_sym___forceinline] = ACTIONS(1296),
    [anon_sym_thread_local] = ACTIONS(1296),
    [anon_sym___thread] = ACTIONS(1296),
    [anon_sym_const] = ACTIONS(1296),
    [anon_sym_constexpr] = ACTIONS(1296),
    [anon_sym_volatile] = ACTIONS(1296),
    [anon_sym_restrict] = ACTIONS(1296),
    [anon_sym___restrict__] = ACTIONS(1296),
    [anon_sym__Atomic] = ACTIONS(1296),
    [anon_sym__Noreturn] = ACTIONS(1296),
    [anon_sym_noreturn] = ACTIONS(1296),
    [sym_primitive_type] = ACTIONS(1296),
    [anon_sym_enum] = ACTIONS(1296),
    [anon_sym_struct] = ACTIONS(1296),
    [anon_sym_union] = ACTIONS(1296),
    [anon_sym_if] = ACTIONS(1296),
    [anon_sym_switch] = ACTIONS(1296),
    [anon_sym_case] = ACTIONS(1296),
    [anon_sym_default] = ACTIONS(1296),
    [anon_sym_while] = ACTIONS(1296),
    [anon_sym_do] = ACTIONS(1296),
    [anon_sym_for] = ACTIONS(1296),
    [anon_sym_return] = ACTIONS(1296),
    [anon_sym_break] = ACTIONS(1296),
    [anon_sym_continue] = ACTIONS(1296),
    [anon_sym_goto] = ACTIONS(1296),
    [anon_sym___try] = ACTIONS(1296),
    [anon_sym___leave] = ACTIONS(1296),
    [anon_sym_DASH_DASH] = ACTIONS(1298),
    [anon_sym_PLUS_PLUS] = ACTIONS(1298),
    [anon_sym_sizeof] = ACTIONS(1296),
    [anon_sym___alignof__] = ACTIONS(1296),
    [anon_sym___alignof] = ACTIONS(1296),
    [anon_sym__alignof] = ACTIONS(1296),
    [anon_sym_alignof] = ACTIONS(1296),
    [anon_sym__Alignof] = ACTIONS(1296),
    [anon_sym_offsetof] = ACTIONS(1296),
    [anon_sym__Generic] = ACTIONS(1296),
    [anon_sym_asm] = ACTIONS(1296),
    [anon_sym___asm__] = ACTIONS(1296),
    [sym_number_literal] = ACTIONS(1298),
    [anon_sym_L_SQUOTE] = ACTIONS(1298),
    [anon_sym_u_SQUOTE] = ACTIONS(1298),
    [anon_sym_U_SQUOTE] = ACTIONS(1298),
    [anon_sym_u8_SQUOTE] = ACTIONS(1298),
    [anon_sym_SQUOTE] = ACTIONS(1298),
    [anon_sym_L_DQUOTE] = ACTIONS(1298),
    [anon_sym_u_DQUOTE] = ACTIONS(1298),
    [anon_sym_U_DQUOTE] = ACTIONS(1298),
    [anon_sym_u8_DQUOTE] = ACTIONS(1298),
    [anon_sym_DQUOTE] = ACTIONS(1298),
    [sym_true] = ACTIONS(1296),
    [sym_false] = ACTIONS(1296),
    [anon_sym_NULL] = ACTIONS(1296),
    [anon_sym_nullptr] = ACTIONS(1296),
    [sym_comment] = ACTIONS(3),
  },
  [144] = {
    [sym_identifier] = ACTIONS(1300),
    [aux_sym_preproc_include_token1] = ACTIONS(1300),
    [aux_sym_preproc_def_token1] = ACTIONS(1300),
    [aux_sym_preproc_if_token1] = ACTIONS(1300),
    [aux_sym_preproc_if_token2] = ACTIONS(1300),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1300),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1300),
    [aux_sym_preproc_else_token1] = ACTIONS(1300),
    [aux_sym_preproc_elif_token1] = ACTIONS(1300),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1300),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1300),
    [sym_preproc_directive] = ACTIONS(1300),
    [anon_sym_LPAREN2] = ACTIONS(1302),
    [anon_sym_BANG] = ACTIONS(1302),
    [anon_sym_TILDE] = ACTIONS(1302),
    [anon_sym_DASH] = ACTIONS(1300),
    [anon_sym_PLUS] = ACTIONS(1300),
    [anon_sym_STAR] = ACTIONS(1302),
    [anon_sym_AMP] = ACTIONS(1302),
    [anon_sym_SEMI] = ACTIONS(1302),
    [anon_sym___extension__] = ACTIONS(1300),
    [anon_sym_typedef] = ACTIONS(1300),
    [anon_sym_extern] = ACTIONS(1300),
    [anon_sym___attribute__] = ACTIONS(1300),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1302),
    [anon_sym___declspec] = ACTIONS(1300),
    [anon_sym___cdecl] = ACTIONS(1300),
    [anon_sym___clrcall] = ACTIONS(1300),
    [anon_sym___stdcall] = ACTIONS(1300),
    [anon_sym___fastcall] = ACTIONS(1300),
    [anon_sym___thiscall] = ACTIONS(1300),
    [anon_sym___vectorcall] = ACTIONS(1300),
    [anon_sym_LBRACE] = ACTIONS(1302),
    [anon_sym_signed] = ACTIONS(1300),
    [anon_sym_unsigned] = ACTIONS(1300),
    [anon_sym_long] = ACTIONS(1300),
    [anon_sym_short] = ACTIONS(1300),
    [anon_sym_static] = ACTIONS(1300),
    [anon_sym_auto] = ACTIONS(1300),
    [anon_sym_register] = ACTIONS(1300),
    [anon_sym_inline] = ACTIONS(1300),
    [anon_sym___inline] = ACTIONS(1300),
    [anon_sym___inline__] = ACTIONS(1300),
    [anon_sym___forceinline] = ACTIONS(1300),
    [anon_sym_thread_local] = ACTIONS(1300),
    [anon_sym___thread] = ACTIONS(1300),
    [anon_sym_const] = ACTIONS(1300),
    [anon_sym_constexpr] = ACTIONS(1300),
    [anon_sym_volatile] = ACTIONS(1300),
    [anon_sym_restrict] = ACTIONS(1300),
    [anon_sym___restrict__] = ACTIONS(1300),
    [anon_sym__Atomic] = ACTIONS(1300),
    [anon_sym__Noreturn] = ACTIONS(1300),
    [anon_sym_noreturn] = ACTIONS(1300),
    [sym_primitive_type] = ACTIONS(1300),
    [anon_sym_enum] = ACTIONS(1300),
    [anon_sym_struct] = ACTIONS(1300),
    [anon_sym_union] = ACTIONS(1300),
    [anon_sym_if] = ACTIONS(1300),
    [anon_sym_switch] = ACTIONS(1300),
    [anon_sym_case] = ACTIONS(1300),
    [anon_sym_default] = ACTIONS(1300),
    [anon_sym_while] = ACTIONS(1300),
    [anon_sym_do] = ACTIONS(1300),
    [anon_sym_for] = ACTIONS(1300),
    [anon_sym_return] = ACTIONS(1300),
    [anon_sym_break] = ACTIONS(1300),
    [anon_sym_continue] = ACTIONS(1300),
    [anon_sym_goto] = ACTIONS(1300),
    [anon_sym___try] = ACTIONS(1300),
    [anon_sym___leave] = ACTIONS(1300),
    [anon_sym_DASH_DASH] = ACTIONS(1302),
    [anon_sym_PLUS_PLUS] = ACTIONS(1302),
    [anon_sym_sizeof] = ACTIONS(1300),
    [anon_sym___alignof__] = ACTIONS(1300),
    [anon_sym___alignof] = ACTIONS(1300),
    [anon_sym__alignof] = ACTIONS(1300),
    [anon_sym_alignof] = ACTIONS(1300),
    [anon_sym__Alignof] = ACTIONS(1300),
    [anon_sym_offsetof] = ACTIONS(1300),
    [anon_sym__Generic] = ACTIONS(1300),
    [anon_sym_asm] = ACTIONS(1300),
    [anon_sym___asm__] = ACTIONS(1300),
    [sym_number_literal] = ACTIONS(1302),
    [anon_sym_L_SQUOTE] = ACTIONS(1302),
    [anon_sym_u_SQUOTE] = ACTIONS(1302),
    [anon_sym_U_SQUOTE] = ACTIONS(1302),
    [anon_sym_u8_SQUOTE] = ACTIONS(1302),
    [anon_sym_SQUOTE] = ACTIONS(1302),
    [anon_sym_L_DQUOTE] = ACTIONS(1302),
    [anon_sym_u_DQUOTE] = ACTIONS(1302),
    [anon_sym_U_DQUOTE] = ACTIONS(1302),
    [anon_sym_u8_DQUOTE] = ACTIONS(1302),
    [anon_sym_DQUOTE] = ACTIONS(1302),
    [sym_true] = ACTIONS(1300),
    [sym_false] = ACTIONS(1300),
    [anon_sym_NULL] = ACTIONS(1300),
    [anon_sym_nullptr] = ACTIONS(1300),
    [sym_comment] = ACTIONS(3),
  },
  [145] = {
    [sym_identifier] = ACTIONS(1304),
    [aux_sym_preproc_include_token1] = ACTIONS(1304),
    [aux_sym_preproc_def_token1] = ACTIONS(1304),
    [aux_sym_preproc_if_token1] = ACTIONS(1304),
    [aux_sym_preproc_if_token2] = ACTIONS(1304),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1304),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1304),
    [aux_sym_preproc_else_token1] = ACTIONS(1304),
    [aux_sym_preproc_elif_token1] = ACTIONS(1304),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1304),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1304),
    [sym_preproc_directive] = ACTIONS(1304),
    [anon_sym_LPAREN2] = ACTIONS(1306),
    [anon_sym_BANG] = ACTIONS(1306),
    [anon_sym_TILDE] = ACTIONS(1306),
    [anon_sym_DASH] = ACTIONS(1304),
    [anon_sym_PLUS] = ACTIONS(1304),
    [anon_sym_STAR] = ACTIONS(1306),
    [anon_sym_AMP] = ACTIONS(1306),
    [anon_sym_SEMI] = ACTIONS(1306),
    [anon_sym___extension__] = ACTIONS(1304),
    [anon_sym_typedef] = ACTIONS(1304),
    [anon_sym_extern] = ACTIONS(1304),
    [anon_sym___attribute__] = ACTIONS(1304),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1306),
    [anon_sym___declspec] = ACTIONS(1304),
    [anon_sym___cdecl] = ACTIONS(1304),
    [anon_sym___clrcall] = ACTIONS(1304),
    [anon_sym___stdcall] = ACTIONS(1304),
    [anon_sym___fastcall] = ACTIONS(1304),
    [anon_sym___thiscall] = ACTIONS(1304),
    [anon_sym___vectorcall] = ACTIONS(1304),
    [anon_sym_LBRACE] = ACTIONS(1306),
    [anon_sym_signed] = ACTIONS(1304),
    [anon_sym_unsigned] = ACTIONS(1304),
    [anon_sym_long] = ACTIONS(1304),
    [anon_sym_short] = ACTIONS(1304),
    [anon_sym_static] = ACTIONS(1304),
    [anon_sym_auto] = ACTIONS(1304),
    [anon_sym_register] = ACTIONS(1304),
    [anon_sym_inline] = ACTIONS(1304),
    [anon_sym___inline] = ACTIONS(1304),
    [anon_sym___inline__] = ACTIONS(1304),
    [anon_sym___forceinline] = ACTIONS(1304),
    [anon_sym_thread_local] = ACTIONS(1304),
    [anon_sym___thread] = ACTIONS(1304),
    [anon_sym_const] = ACTIONS(1304),
    [anon_sym_constexpr] = ACTIONS(1304),
    [anon_sym_volatile] = ACTIONS(1304),
    [anon_sym_restrict] = ACTIONS(1304),
    [anon_sym___restrict__] = ACTIONS(1304),
    [anon_sym__Atomic] = ACTIONS(1304),
    [anon_sym__Noreturn] = ACTIONS(1304),
    [anon_sym_noreturn] = ACTIONS(1304),
    [sym_primitive_type] = ACTIONS(1304),
    [anon_sym_enum] = ACTIONS(1304),
    [anon_sym_struct] = ACTIONS(1304),
    [anon_sym_union] = ACTIONS(1304),
    [anon_sym_if] = ACTIONS(1304),
    [anon_sym_switch] = ACTIONS(1304),
    [anon_sym_case] = ACTIONS(1304),
    [anon_sym_default] = ACTIONS(1304),
    [anon_sym_while] = ACTIONS(1304),
    [anon_sym_do] = ACTIONS(1304),
    [anon_sym_for] = ACTIONS(1304),
    [anon_sym_return] = ACTIONS(1304),
    [anon_sym_break] = ACTIONS(1304),
    [anon_sym_continue] = ACTIONS(1304),
    [anon_sym_goto] = ACTIONS(1304),
    [anon_sym___try] = ACTIONS(1304),
    [anon_sym___leave] = ACTIONS(1304),
    [anon_sym_DASH_DASH] = ACTIONS(1306),
    [anon_sym_PLUS_PLUS] = ACTIONS(1306),
    [anon_sym_sizeof] = ACTIONS(1304),
    [anon_sym___alignof__] = ACTIONS(1304),
    [anon_sym___alignof] = ACTIONS(1304),
    [anon_sym__alignof] = ACTIONS(1304),
    [anon_sym_alignof] = ACTIONS(1304),
    [anon_sym__Alignof] = ACTIONS(1304),
    [anon_sym_offsetof] = ACTIONS(1304),
    [anon_sym__Generic] = ACTIONS(1304),
    [anon_sym_asm] = ACTIONS(1304),
    [anon_sym___asm__] = ACTIONS(1304),
    [sym_number_literal] = ACTIONS(1306),
    [anon_sym_L_SQUOTE] = ACTIONS(1306),
    [anon_sym_u_SQUOTE] = ACTIONS(1306),
    [anon_sym_U_SQUOTE] = ACTIONS(1306),
    [anon_sym_u8_SQUOTE] = ACTIONS(1306),
    [anon_sym_SQUOTE] = ACTIONS(1306),
    [anon_sym_L_DQUOTE] = ACTIONS(1306),
    [anon_sym_u_DQUOTE] = ACTIONS(1306),
    [anon_sym_U_DQUOTE] = ACTIONS(1306),
    [anon_sym_u8_DQUOTE] = ACTIONS(1306),
    [anon_sym_DQUOTE] = ACTIONS(1306),
    [sym_true] = ACTIONS(1304),
    [sym_false] = ACTIONS(1304),
    [anon_sym_NULL] = ACTIONS(1304),
    [anon_sym_nullptr] = ACTIONS(1304),
    [sym_comment] = ACTIONS(3),
  },
  [146] = {
    [sym_identifier] = ACTIONS(1308),
    [aux_sym_preproc_include_token1] = ACTIONS(1308),
    [aux_sym_preproc_def_token1] = ACTIONS(1308),
    [aux_sym_preproc_if_token1] = ACTIONS(1308),
    [aux_sym_preproc_if_token2] = ACTIONS(1308),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1308),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1308),
    [aux_sym_preproc_else_token1] = ACTIONS(1308),
    [aux_sym_preproc_elif_token1] = ACTIONS(1308),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1308),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1308),
    [sym_preproc_directive] = ACTIONS(1308),
    [anon_sym_LPAREN2] = ACTIONS(1310),
    [anon_sym_BANG] = ACTIONS(1310),
    [anon_sym_TILDE] = ACTIONS(1310),
    [anon_sym_DASH] = ACTIONS(1308),
    [anon_sym_PLUS] = ACTIONS(1308),
    [anon_sym_STAR] = ACTIONS(1310),
    [anon_sym_AMP] = ACTIONS(1310),
    [anon_sym_SEMI] = ACTIONS(1310),
    [anon_sym___extension__] = ACTIONS(1308),
    [anon_sym_typedef] = ACTIONS(1308),
    [anon_sym_extern] = ACTIONS(1308),
    [anon_sym___attribute__] = ACTIONS(1308),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1310),
    [anon_sym___declspec] = ACTIONS(1308),
    [anon_sym___cdecl] = ACTIONS(1308),
    [anon_sym___clrcall] = ACTIONS(1308),
    [anon_sym___stdcall] = ACTIONS(1308),
    [anon_sym___fastcall] = ACTIONS(1308),
    [anon_sym___thiscall] = ACTIONS(1308),
    [anon_sym___vectorcall] = ACTIONS(1308),
    [anon_sym_LBRACE] = ACTIONS(1310),
    [anon_sym_signed] = ACTIONS(1308),
    [anon_sym_unsigned] = ACTIONS(1308),
    [anon_sym_long] = ACTIONS(1308),
    [anon_sym_short] = ACTIONS(1308),
    [anon_sym_static] = ACTIONS(1308),
    [anon_sym_auto] = ACTIONS(1308),
    [anon_sym_register] = ACTIONS(1308),
    [anon_sym_inline] = ACTIONS(1308),
    [anon_sym___inline] = ACTIONS(1308),
    [anon_sym___inline__] = ACTIONS(1308),
    [anon_sym___forceinline] = ACTIONS(1308),
    [anon_sym_thread_local] = ACTIONS(1308),
    [anon_sym___thread] = ACTIONS(1308),
    [anon_sym_const] = ACTIONS(1308),
    [anon_sym_constexpr] = ACTIONS(1308),
    [anon_sym_volatile] = ACTIONS(1308),
    [anon_sym_restrict] = ACTIONS(1308),
    [anon_sym___restrict__] = ACTIONS(1308),
    [anon_sym__Atomic] = ACTIONS(1308),
    [anon_sym__Noreturn] = ACTIONS(1308),
    [anon_sym_noreturn] = ACTIONS(1308),
    [sym_primitive_type] = ACTIONS(1308),
    [anon_sym_enum] = ACTIONS(1308),
    [anon_sym_struct] = ACTIONS(1308),
    [anon_sym_union] = ACTIONS(1308),
    [anon_sym_if] = ACTIONS(1308),
    [anon_sym_switch] = ACTIONS(1308),
    [anon_sym_case] = ACTIONS(1308),
    [anon_sym_default] = ACTIONS(1308),
    [anon_sym_while] = ACTIONS(1308),
    [anon_sym_do] = ACTIONS(1308),
    [anon_sym_for] = ACTIONS(1308),
    [anon_sym_return] = ACTIONS(1308),
    [anon_sym_break] = ACTIONS(1308),
    [anon_sym_continue] = ACTIONS(1308),
    [anon_sym_goto] = ACTIONS(1308),
    [anon_sym___try] = ACTIONS(1308),
    [anon_sym___leave] = ACTIONS(1308),
    [anon_sym_DASH_DASH] = ACTIONS(1310),
    [anon_sym_PLUS_PLUS] = ACTIONS(1310),
    [anon_sym_sizeof] = ACTIONS(1308),
    [anon_sym___alignof__] = ACTIONS(1308),
    [anon_sym___alignof] = ACTIONS(1308),
    [anon_sym__alignof] = ACTIONS(1308),
    [anon_sym_alignof] = ACTIONS(1308),
    [anon_sym__Alignof] = ACTIONS(1308),
    [anon_sym_offsetof] = ACTIONS(1308),
    [anon_sym__Generic] = ACTIONS(1308),
    [anon_sym_asm] = ACTIONS(1308),
    [anon_sym___asm__] = ACTIONS(1308),
    [sym_number_literal] = ACTIONS(1310),
    [anon_sym_L_SQUOTE] = ACTIONS(1310),
    [anon_sym_u_SQUOTE] = ACTIONS(1310),
    [anon_sym_U_SQUOTE] = ACTIONS(1310),
    [anon_sym_u8_SQUOTE] = ACTIONS(1310),
    [anon_sym_SQUOTE] = ACTIONS(1310),
    [anon_sym_L_DQUOTE] = ACTIONS(1310),
    [anon_sym_u_DQUOTE] = ACTIONS(1310),
    [anon_sym_U_DQUOTE] = ACTIONS(1310),
    [anon_sym_u8_DQUOTE] = ACTIONS(1310),
    [anon_sym_DQUOTE] = ACTIONS(1310),
    [sym_true] = ACTIONS(1308),
    [sym_false] = ACTIONS(1308),
    [anon_sym_NULL] = ACTIONS(1308),
    [anon_sym_nullptr] = ACTIONS(1308),
    [sym_comment] = ACTIONS(3),
  },
  [147] = {
    [sym_identifier] = ACTIONS(1312),
    [aux_sym_preproc_include_token1] = ACTIONS(1312),
    [aux_sym_preproc_def_token1] = ACTIONS(1312),
    [aux_sym_preproc_if_token1] = ACTIONS(1312),
    [aux_sym_preproc_if_token2] = ACTIONS(1312),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1312),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1312),
    [aux_sym_preproc_else_token1] = ACTIONS(1312),
    [aux_sym_preproc_elif_token1] = ACTIONS(1312),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1312),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1312),
    [sym_preproc_directive] = ACTIONS(1312),
    [anon_sym_LPAREN2] = ACTIONS(1314),
    [anon_sym_BANG] = ACTIONS(1314),
    [anon_sym_TILDE] = ACTIONS(1314),
    [anon_sym_DASH] = ACTIONS(1312),
    [anon_sym_PLUS] = ACTIONS(1312),
    [anon_sym_STAR] = ACTIONS(1314),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_SEMI] = ACTIONS(1314),
    [anon_sym___extension__] = ACTIONS(1312),
    [anon_sym_typedef] = ACTIONS(1312),
    [anon_sym_extern] = ACTIONS(1312),
    [anon_sym___attribute__] = ACTIONS(1312),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1314),
    [anon_sym___declspec] = ACTIONS(1312),
    [anon_sym___cdecl] = ACTIONS(1312),
    [anon_sym___clrcall] = ACTIONS(1312),
    [anon_sym___stdcall] = ACTIONS(1312),
    [anon_sym___fastcall] = ACTIONS(1312),
    [anon_sym___thiscall] = ACTIONS(1312),
    [anon_sym___vectorcall] = ACTIONS(1312),
    [anon_sym_LBRACE] = ACTIONS(1314),
    [anon_sym_signed] = ACTIONS(1312),
    [anon_sym_unsigned] = ACTIONS(1312),
    [anon_sym_long] = ACTIONS(1312),
    [anon_sym_short] = ACTIONS(1312),
    [anon_sym_static] = ACTIONS(1312),
    [anon_sym_auto] = ACTIONS(1312),
    [anon_sym_register] = ACTIONS(1312),
    [anon_sym_inline] = ACTIONS(1312),
    [anon_sym___inline] = ACTIONS(1312),
    [anon_sym___inline__] = ACTIONS(1312),
    [anon_sym___forceinline] = ACTIONS(1312),
    [anon_sym_thread_local] = ACTIONS(1312),
    [anon_sym___thread] = ACTIONS(1312),
    [anon_sym_const] = ACTIONS(1312),
    [anon_sym_constexpr] = ACTIONS(1312),
    [anon_sym_volatile] = ACTIONS(1312),
    [anon_sym_restrict] = ACTIONS(1312),
    [anon_sym___restrict__] = ACTIONS(1312),
    [anon_sym__Atomic] = ACTIONS(1312),
    [anon_sym__Noreturn] = ACTIONS(1312),
    [anon_sym_noreturn] = ACTIONS(1312),
    [sym_primitive_type] = ACTIONS(1312),
    [anon_sym_enum] = ACTIONS(1312),
    [anon_sym_struct] = ACTIONS(1312),
    [anon_sym_union] = ACTIONS(1312),
    [anon_sym_if] = ACTIONS(1312),
    [anon_sym_switch] = ACTIONS(1312),
    [anon_sym_case] = ACTIONS(1312),
    [anon_sym_default] = ACTIONS(1312),
    [anon_sym_while] = ACTIONS(1312),
    [anon_sym_do] = ACTIONS(1312),
    [anon_sym_for] = ACTIONS(1312),
    [anon_sym_return] = ACTIONS(1312),
    [anon_sym_break] = ACTIONS(1312),
    [anon_sym_continue] = ACTIONS(1312),
    [anon_sym_goto] = ACTIONS(1312),
    [anon_sym___try] = ACTIONS(1312),
    [anon_sym___leave] = ACTIONS(1312),
    [anon_sym_DASH_DASH] = ACTIONS(1314),
    [anon_sym_PLUS_PLUS] = ACTIONS(1314),
    [anon_sym_sizeof] = ACTIONS(1312),
    [anon_sym___alignof__] = ACTIONS(1312),
    [anon_sym___alignof] = ACTIONS(1312),
    [anon_sym__alignof] = ACTIONS(1312),
    [anon_sym_alignof] = ACTIONS(1312),
    [anon_sym__Alignof] = ACTIONS(1312),
    [anon_sym_offsetof] = ACTIONS(1312),
    [anon_sym__Generic] = ACTIONS(1312),
    [anon_sym_asm] = ACTIONS(1312),
    [anon_sym___asm__] = ACTIONS(1312),
    [sym_number_literal] = ACTIONS(1314),
    [anon_sym_L_SQUOTE] = ACTIONS(1314),
    [anon_sym_u_SQUOTE] = ACTIONS(1314),
    [anon_sym_U_SQUOTE] = ACTIONS(1314),
    [anon_sym_u8_SQUOTE] = ACTIONS(1314),
    [anon_sym_SQUOTE] = ACTIONS(1314),
    [anon_sym_L_DQUOTE] = ACTIONS(1314),
    [anon_sym_u_DQUOTE] = ACTIONS(1314),
    [anon_sym_U_DQUOTE] = ACTIONS(1314),
    [anon_sym_u8_DQUOTE] = ACTIONS(1314),
    [anon_sym_DQUOTE] = ACTIONS(1314),
    [sym_true] = ACTIONS(1312),
    [sym_false] = ACTIONS(1312),
    [anon_sym_NULL] = ACTIONS(1312),
    [anon_sym_nullptr] = ACTIONS(1312),
    [sym_comment] = ACTIONS(3),
  },
  [148] = {
    [sym_identifier] = ACTIONS(1316),
    [aux_sym_preproc_include_token1] = ACTIONS(1316),
    [aux_sym_preproc_def_token1] = ACTIONS(1316),
    [aux_sym_preproc_if_token1] = ACTIONS(1316),
    [aux_sym_preproc_if_token2] = ACTIONS(1316),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1316),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1316),
    [aux_sym_preproc_else_token1] = ACTIONS(1316),
    [aux_sym_preproc_elif_token1] = ACTIONS(1316),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1316),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1316),
    [sym_preproc_directive] = ACTIONS(1316),
    [anon_sym_LPAREN2] = ACTIONS(1318),
    [anon_sym_BANG] = ACTIONS(1318),
    [anon_sym_TILDE] = ACTIONS(1318),
    [anon_sym_DASH] = ACTIONS(1316),
    [anon_sym_PLUS] = ACTIONS(1316),
    [anon_sym_STAR] = ACTIONS(1318),
    [anon_sym_AMP] = ACTIONS(1318),
    [anon_sym_SEMI] = ACTIONS(1318),
    [anon_sym___extension__] = ACTIONS(1316),
    [anon_sym_typedef] = ACTIONS(1316),
    [anon_sym_extern] = ACTIONS(1316),
    [anon_sym___attribute__] = ACTIONS(1316),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1318),
    [anon_sym___declspec] = ACTIONS(1316),
    [anon_sym___cdecl] = ACTIONS(1316),
    [anon_sym___clrcall] = ACTIONS(1316),
    [anon_sym___stdcall] = ACTIONS(1316),
    [anon_sym___fastcall] = ACTIONS(1316),
    [anon_sym___thiscall] = ACTIONS(1316),
    [anon_sym___vectorcall] = ACTIONS(1316),
    [anon_sym_LBRACE] = ACTIONS(1318),
    [anon_sym_signed] = ACTIONS(1316),
    [anon_sym_unsigned] = ACTIONS(1316),
    [anon_sym_long] = ACTIONS(1316),
    [anon_sym_short] = ACTIONS(1316),
    [anon_sym_static] = ACTIONS(1316),
    [anon_sym_auto] = ACTIONS(1316),
    [anon_sym_register] = ACTIONS(1316),
    [anon_sym_inline] = ACTIONS(1316),
    [anon_sym___inline] = ACTIONS(1316),
    [anon_sym___inline__] = ACTIONS(1316),
    [anon_sym___forceinline] = ACTIONS(1316),
    [anon_sym_thread_local] = ACTIONS(1316),
    [anon_sym___thread] = ACTIONS(1316),
    [anon_sym_const] = ACTIONS(1316),
    [anon_sym_constexpr] = ACTIONS(1316),
    [anon_sym_volatile] = ACTIONS(1316),
    [anon_sym_restrict] = ACTIONS(1316),
    [anon_sym___restrict__] = ACTIONS(1316),
    [anon_sym__Atomic] = ACTIONS(1316),
    [anon_sym__Noreturn] = ACTIONS(1316),
    [anon_sym_noreturn] = ACTIONS(1316),
    [sym_primitive_type] = ACTIONS(1316),
    [anon_sym_enum] = ACTIONS(1316),
    [anon_sym_struct] = ACTIONS(1316),
    [anon_sym_union] = ACTIONS(1316),
    [anon_sym_if] = ACTIONS(1316),
    [anon_sym_switch] = ACTIONS(1316),
    [anon_sym_case] = ACTIONS(1316),
    [anon_sym_default] = ACTIONS(1316),
    [anon_sym_while] = ACTIONS(1316),
    [anon_sym_do] = ACTIONS(1316),
    [anon_sym_for] = ACTIONS(1316),
    [anon_sym_return] = ACTIONS(1316),
    [anon_sym_break] = ACTIONS(1316),
    [anon_sym_continue] = ACTIONS(1316),
    [anon_sym_goto] = ACTIONS(1316),
    [anon_sym___try] = ACTIONS(1316),
    [anon_sym___leave] = ACTIONS(1316),
    [anon_sym_DASH_DASH] = ACTIONS(1318),
    [anon_sym_PLUS_PLUS] = ACTIONS(1318),
    [anon_sym_sizeof] = ACTIONS(1316),
    [anon_sym___alignof__] = ACTIONS(1316),
    [anon_sym___alignof] = ACTIONS(1316),
    [anon_sym__alignof] = ACTIONS(1316),
    [anon_sym_alignof] = ACTIONS(1316),
    [anon_sym__Alignof] = ACTIONS(1316),
    [anon_sym_offsetof] = ACTIONS(1316),
    [anon_sym__Generic] = ACTIONS(1316),
    [anon_sym_asm] = ACTIONS(1316),
    [anon_sym___asm__] = ACTIONS(1316),
    [sym_number_literal] = ACTIONS(1318),
    [anon_sym_L_SQUOTE] = ACTIONS(1318),
    [anon_sym_u_SQUOTE] = ACTIONS(1318),
    [anon_sym_U_SQUOTE] = ACTIONS(1318),
    [anon_sym_u8_SQUOTE] = ACTIONS(1318),
    [anon_sym_SQUOTE] = ACTIONS(1318),
    [anon_sym_L_DQUOTE] = ACTIONS(1318),
    [anon_sym_u_DQUOTE] = ACTIONS(1318),
    [anon_sym_U_DQUOTE] = ACTIONS(1318),
    [anon_sym_u8_DQUOTE] = ACTIONS(1318),
    [anon_sym_DQUOTE] = ACTIONS(1318),
    [sym_true] = ACTIONS(1316),
    [sym_false] = ACTIONS(1316),
    [anon_sym_NULL] = ACTIONS(1316),
    [anon_sym_nullptr] = ACTIONS(1316),
    [sym_comment] = ACTIONS(3),
  },
  [149] = {
    [sym_identifier] = ACTIONS(1320),
    [aux_sym_preproc_include_token1] = ACTIONS(1320),
    [aux_sym_preproc_def_token1] = ACTIONS(1320),
    [aux_sym_preproc_if_token1] = ACTIONS(1320),
    [aux_sym_preproc_if_token2] = ACTIONS(1320),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1320),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1320),
    [aux_sym_preproc_else_token1] = ACTIONS(1320),
    [aux_sym_preproc_elif_token1] = ACTIONS(1320),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1320),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1320),
    [sym_preproc_directive] = ACTIONS(1320),
    [anon_sym_LPAREN2] = ACTIONS(1322),
    [anon_sym_BANG] = ACTIONS(1322),
    [anon_sym_TILDE] = ACTIONS(1322),
    [anon_sym_DASH] = ACTIONS(1320),
    [anon_sym_PLUS] = ACTIONS(1320),
    [anon_sym_STAR] = ACTIONS(1322),
    [anon_sym_AMP] = ACTIONS(1322),
    [anon_sym_SEMI] = ACTIONS(1322),
    [anon_sym___extension__] = ACTIONS(1320),
    [anon_sym_typedef] = ACTIONS(1320),
    [anon_sym_extern] = ACTIONS(1320),
    [anon_sym___attribute__] = ACTIONS(1320),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1322),
    [anon_sym___declspec] = ACTIONS(1320),
    [anon_sym___cdecl] = ACTIONS(1320),
    [anon_sym___clrcall] = ACTIONS(1320),
    [anon_sym___stdcall] = ACTIONS(1320),
    [anon_sym___fastcall] = ACTIONS(1320),
    [anon_sym___thiscall] = ACTIONS(1320),
    [anon_sym___vectorcall] = ACTIONS(1320),
    [anon_sym_LBRACE] = ACTIONS(1322),
    [anon_sym_signed] = ACTIONS(1320),
    [anon_sym_unsigned] = ACTIONS(1320),
    [anon_sym_long] = ACTIONS(1320),
    [anon_sym_short] = ACTIONS(1320),
    [anon_sym_static] = ACTIONS(1320),
    [anon_sym_auto] = ACTIONS(1320),
    [anon_sym_register] = ACTIONS(1320),
    [anon_sym_inline] = ACTIONS(1320),
    [anon_sym___inline] = ACTIONS(1320),
    [anon_sym___inline__] = ACTIONS(1320),
    [anon_sym___forceinline] = ACTIONS(1320),
    [anon_sym_thread_local] = ACTIONS(1320),
    [anon_sym___thread] = ACTIONS(1320),
    [anon_sym_const] = ACTIONS(1320),
    [anon_sym_constexpr] = ACTIONS(1320),
    [anon_sym_volatile] = ACTIONS(1320),
    [anon_sym_restrict] = ACTIONS(1320),
    [anon_sym___restrict__] = ACTIONS(1320),
    [anon_sym__Atomic] = ACTIONS(1320),
    [anon_sym__Noreturn] = ACTIONS(1320),
    [anon_sym_noreturn] = ACTIONS(1320),
    [sym_primitive_type] = ACTIONS(1320),
    [anon_sym_enum] = ACTIONS(1320),
    [anon_sym_struct] = ACTIONS(1320),
    [anon_sym_union] = ACTIONS(1320),
    [anon_sym_if] = ACTIONS(1320),
    [anon_sym_switch] = ACTIONS(1320),
    [anon_sym_case] = ACTIONS(1320),
    [anon_sym_default] = ACTIONS(1320),
    [anon_sym_while] = ACTIONS(1320),
    [anon_sym_do] = ACTIONS(1320),
    [anon_sym_for] = ACTIONS(1320),
    [anon_sym_return] = ACTIONS(1320),
    [anon_sym_break] = ACTIONS(1320),
    [anon_sym_continue] = ACTIONS(1320),
    [anon_sym_goto] = ACTIONS(1320),
    [anon_sym___try] = ACTIONS(1320),
    [anon_sym___leave] = ACTIONS(1320),
    [anon_sym_DASH_DASH] = ACTIONS(1322),
    [anon_sym_PLUS_PLUS] = ACTIONS(1322),
    [anon_sym_sizeof] = ACTIONS(1320),
    [anon_sym___alignof__] = ACTIONS(1320),
    [anon_sym___alignof] = ACTIONS(1320),
    [anon_sym__alignof] = ACTIONS(1320),
    [anon_sym_alignof] = ACTIONS(1320),
    [anon_sym__Alignof] = ACTIONS(1320),
    [anon_sym_offsetof] = ACTIONS(1320),
    [anon_sym__Generic] = ACTIONS(1320),
    [anon_sym_asm] = ACTIONS(1320),
    [anon_sym___asm__] = ACTIONS(1320),
    [sym_number_literal] = ACTIONS(1322),
    [anon_sym_L_SQUOTE] = ACTIONS(1322),
    [anon_sym_u_SQUOTE] = ACTIONS(1322),
    [anon_sym_U_SQUOTE] = ACTIONS(1322),
    [anon_sym_u8_SQUOTE] = ACTIONS(1322),
    [anon_sym_SQUOTE] = ACTIONS(1322),
    [anon_sym_L_DQUOTE] = ACTIONS(1322),
    [anon_sym_u_DQUOTE] = ACTIONS(1322),
    [anon_sym_U_DQUOTE] = ACTIONS(1322),
    [anon_sym_u8_DQUOTE] = ACTIONS(1322),
    [anon_sym_DQUOTE] = ACTIONS(1322),
    [sym_true] = ACTIONS(1320),
    [sym_false] = ACTIONS(1320),
    [anon_sym_NULL] = ACTIONS(1320),
    [anon_sym_nullptr] = ACTIONS(1320),
    [sym_comment] = ACTIONS(3),
  },
  [150] = {
    [sym_identifier] = ACTIONS(1324),
    [aux_sym_preproc_include_token1] = ACTIONS(1324),
    [aux_sym_preproc_def_token1] = ACTIONS(1324),
    [aux_sym_preproc_if_token1] = ACTIONS(1324),
    [aux_sym_preproc_if_token2] = ACTIONS(1324),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1324),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1324),
    [aux_sym_preproc_else_token1] = ACTIONS(1324),
    [aux_sym_preproc_elif_token1] = ACTIONS(1324),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1324),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1324),
    [sym_preproc_directive] = ACTIONS(1324),
    [anon_sym_LPAREN2] = ACTIONS(1326),
    [anon_sym_BANG] = ACTIONS(1326),
    [anon_sym_TILDE] = ACTIONS(1326),
    [anon_sym_DASH] = ACTIONS(1324),
    [anon_sym_PLUS] = ACTIONS(1324),
    [anon_sym_STAR] = ACTIONS(1326),
    [anon_sym_AMP] = ACTIONS(1326),
    [anon_sym_SEMI] = ACTIONS(1326),
    [anon_sym___extension__] = ACTIONS(1324),
    [anon_sym_typedef] = ACTIONS(1324),
    [anon_sym_extern] = ACTIONS(1324),
    [anon_sym___attribute__] = ACTIONS(1324),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1326),
    [anon_sym___declspec] = ACTIONS(1324),
    [anon_sym___cdecl] = ACTIONS(1324),
    [anon_sym___clrcall] = ACTIONS(1324),
    [anon_sym___stdcall] = ACTIONS(1324),
    [anon_sym___fastcall] = ACTIONS(1324),
    [anon_sym___thiscall] = ACTIONS(1324),
    [anon_sym___vectorcall] = ACTIONS(1324),
    [anon_sym_LBRACE] = ACTIONS(1326),
    [anon_sym_signed] = ACTIONS(1324),
    [anon_sym_unsigned] = ACTIONS(1324),
    [anon_sym_long] = ACTIONS(1324),
    [anon_sym_short] = ACTIONS(1324),
    [anon_sym_static] = ACTIONS(1324),
    [anon_sym_auto] = ACTIONS(1324),
    [anon_sym_register] = ACTIONS(1324),
    [anon_sym_inline] = ACTIONS(1324),
    [anon_sym___inline] = ACTIONS(1324),
    [anon_sym___inline__] = ACTIONS(1324),
    [anon_sym___forceinline] = ACTIONS(1324),
    [anon_sym_thread_local] = ACTIONS(1324),
    [anon_sym___thread] = ACTIONS(1324),
    [anon_sym_const] = ACTIONS(1324),
    [anon_sym_constexpr] = ACTIONS(1324),
    [anon_sym_volatile] = ACTIONS(1324),
    [anon_sym_restrict] = ACTIONS(1324),
    [anon_sym___restrict__] = ACTIONS(1324),
    [anon_sym__Atomic] = ACTIONS(1324),
    [anon_sym__Noreturn] = ACTIONS(1324),
    [anon_sym_noreturn] = ACTIONS(1324),
    [sym_primitive_type] = ACTIONS(1324),
    [anon_sym_enum] = ACTIONS(1324),
    [anon_sym_struct] = ACTIONS(1324),
    [anon_sym_union] = ACTIONS(1324),
    [anon_sym_if] = ACTIONS(1324),
    [anon_sym_switch] = ACTIONS(1324),
    [anon_sym_case] = ACTIONS(1324),
    [anon_sym_default] = ACTIONS(1324),
    [anon_sym_while] = ACTIONS(1324),
    [anon_sym_do] = ACTIONS(1324),
    [anon_sym_for] = ACTIONS(1324),
    [anon_sym_return] = ACTIONS(1324),
    [anon_sym_break] = ACTIONS(1324),
    [anon_sym_continue] = ACTIONS(1324),
    [anon_sym_goto] = ACTIONS(1324),
    [anon_sym___try] = ACTIONS(1324),
    [anon_sym___leave] = ACTIONS(1324),
    [anon_sym_DASH_DASH] = ACTIONS(1326),
    [anon_sym_PLUS_PLUS] = ACTIONS(1326),
    [anon_sym_sizeof] = ACTIONS(1324),
    [anon_sym___alignof__] = ACTIONS(1324),
    [anon_sym___alignof] = ACTIONS(1324),
    [anon_sym__alignof] = ACTIONS(1324),
    [anon_sym_alignof] = ACTIONS(1324),
    [anon_sym__Alignof] = ACTIONS(1324),
    [anon_sym_offsetof] = ACTIONS(1324),
    [anon_sym__Generic] = ACTIONS(1324),
    [anon_sym_asm] = ACTIONS(1324),
    [anon_sym___asm__] = ACTIONS(1324),
    [sym_number_literal] = ACTIONS(1326),
    [anon_sym_L_SQUOTE] = ACTIONS(1326),
    [anon_sym_u_SQUOTE] = ACTIONS(1326),
    [anon_sym_U_SQUOTE] = ACTIONS(1326),
    [anon_sym_u8_SQUOTE] = ACTIONS(1326),
    [anon_sym_SQUOTE] = ACTIONS(1326),
    [anon_sym_L_DQUOTE] = ACTIONS(1326),
    [anon_sym_u_DQUOTE] = ACTIONS(1326),
    [anon_sym_U_DQUOTE] = ACTIONS(1326),
    [anon_sym_u8_DQUOTE] = ACTIONS(1326),
    [anon_sym_DQUOTE] = ACTIONS(1326),
    [sym_true] = ACTIONS(1324),
    [sym_false] = ACTIONS(1324),
    [anon_sym_NULL] = ACTIONS(1324),
    [anon_sym_nullptr] = ACTIONS(1324),
    [sym_comment] = ACTIONS(3),
  },
  [151] = {
    [sym_identifier] = ACTIONS(1328),
    [aux_sym_preproc_include_token1] = ACTIONS(1328),
    [aux_sym_preproc_def_token1] = ACTIONS(1328),
    [aux_sym_preproc_if_token1] = ACTIONS(1328),
    [aux_sym_preproc_if_token2] = ACTIONS(1328),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1328),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1328),
    [aux_sym_preproc_else_token1] = ACTIONS(1328),
    [aux_sym_preproc_elif_token1] = ACTIONS(1328),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1328),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1328),
    [sym_preproc_directive] = ACTIONS(1328),
    [anon_sym_LPAREN2] = ACTIONS(1330),
    [anon_sym_BANG] = ACTIONS(1330),
    [anon_sym_TILDE] = ACTIONS(1330),
    [anon_sym_DASH] = ACTIONS(1328),
    [anon_sym_PLUS] = ACTIONS(1328),
    [anon_sym_STAR] = ACTIONS(1330),
    [anon_sym_AMP] = ACTIONS(1330),
    [anon_sym_SEMI] = ACTIONS(1330),
    [anon_sym___extension__] = ACTIONS(1328),
    [anon_sym_typedef] = ACTIONS(1328),
    [anon_sym_extern] = ACTIONS(1328),
    [anon_sym___attribute__] = ACTIONS(1328),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1330),
    [anon_sym___declspec] = ACTIONS(1328),
    [anon_sym___cdecl] = ACTIONS(1328),
    [anon_sym___clrcall] = ACTIONS(1328),
    [anon_sym___stdcall] = ACTIONS(1328),
    [anon_sym___fastcall] = ACTIONS(1328),
    [anon_sym___thiscall] = ACTIONS(1328),
    [anon_sym___vectorcall] = ACTIONS(1328),
    [anon_sym_LBRACE] = ACTIONS(1330),
    [anon_sym_signed] = ACTIONS(1328),
    [anon_sym_unsigned] = ACTIONS(1328),
    [anon_sym_long] = ACTIONS(1328),
    [anon_sym_short] = ACTIONS(1328),
    [anon_sym_static] = ACTIONS(1328),
    [anon_sym_auto] = ACTIONS(1328),
    [anon_sym_register] = ACTIONS(1328),
    [anon_sym_inline] = ACTIONS(1328),
    [anon_sym___inline] = ACTIONS(1328),
    [anon_sym___inline__] = ACTIONS(1328),
    [anon_sym___forceinline] = ACTIONS(1328),
    [anon_sym_thread_local] = ACTIONS(1328),
    [anon_sym___thread] = ACTIONS(1328),
    [anon_sym_const] = ACTIONS(1328),
    [anon_sym_constexpr] = ACTIONS(1328),
    [anon_sym_volatile] = ACTIONS(1328),
    [anon_sym_restrict] = ACTIONS(1328),
    [anon_sym___restrict__] = ACTIONS(1328),
    [anon_sym__Atomic] = ACTIONS(1328),
    [anon_sym__Noreturn] = ACTIONS(1328),
    [anon_sym_noreturn] = ACTIONS(1328),
    [sym_primitive_type] = ACTIONS(1328),
    [anon_sym_enum] = ACTIONS(1328),
    [anon_sym_struct] = ACTIONS(1328),
    [anon_sym_union] = ACTIONS(1328),
    [anon_sym_if] = ACTIONS(1328),
    [anon_sym_switch] = ACTIONS(1328),
    [anon_sym_case] = ACTIONS(1328),
    [anon_sym_default] = ACTIONS(1328),
    [anon_sym_while] = ACTIONS(1328),
    [anon_sym_do] = ACTIONS(1328),
    [anon_sym_for] = ACTIONS(1328),
    [anon_sym_return] = ACTIONS(1328),
    [anon_sym_break] = ACTIONS(1328),
    [anon_sym_continue] = ACTIONS(1328),
    [anon_sym_goto] = ACTIONS(1328),
    [anon_sym___try] = ACTIONS(1328),
    [anon_sym___leave] = ACTIONS(1328),
    [anon_sym_DASH_DASH] = ACTIONS(1330),
    [anon_sym_PLUS_PLUS] = ACTIONS(1330),
    [anon_sym_sizeof] = ACTIONS(1328),
    [anon_sym___alignof__] = ACTIONS(1328),
    [anon_sym___alignof] = ACTIONS(1328),
    [anon_sym__alignof] = ACTIONS(1328),
    [anon_sym_alignof] = ACTIONS(1328),
    [anon_sym__Alignof] = ACTIONS(1328),
    [anon_sym_offsetof] = ACTIONS(1328),
    [anon_sym__Generic] = ACTIONS(1328),
    [anon_sym_asm] = ACTIONS(1328),
    [anon_sym___asm__] = ACTIONS(1328),
    [sym_number_literal] = ACTIONS(1330),
    [anon_sym_L_SQUOTE] = ACTIONS(1330),
    [anon_sym_u_SQUOTE] = ACTIONS(1330),
    [anon_sym_U_SQUOTE] = ACTIONS(1330),
    [anon_sym_u8_SQUOTE] = ACTIONS(1330),
    [anon_sym_SQUOTE] = ACTIONS(1330),
    [anon_sym_L_DQUOTE] = ACTIONS(1330),
    [anon_sym_u_DQUOTE] = ACTIONS(1330),
    [anon_sym_U_DQUOTE] = ACTIONS(1330),
    [anon_sym_u8_DQUOTE] = ACTIONS(1330),
    [anon_sym_DQUOTE] = ACTIONS(1330),
    [sym_true] = ACTIONS(1328),
    [sym_false] = ACTIONS(1328),
    [anon_sym_NULL] = ACTIONS(1328),
    [anon_sym_nullptr] = ACTIONS(1328),
    [sym_comment] = ACTIONS(3),
  },
  [152] = {
    [sym_identifier] = ACTIONS(1332),
    [aux_sym_preproc_include_token1] = ACTIONS(1332),
    [aux_sym_preproc_def_token1] = ACTIONS(1332),
    [aux_sym_preproc_if_token1] = ACTIONS(1332),
    [aux_sym_preproc_if_token2] = ACTIONS(1332),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1332),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1332),
    [aux_sym_preproc_else_token1] = ACTIONS(1332),
    [aux_sym_preproc_elif_token1] = ACTIONS(1332),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1332),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1332),
    [sym_preproc_directive] = ACTIONS(1332),
    [anon_sym_LPAREN2] = ACTIONS(1334),
    [anon_sym_BANG] = ACTIONS(1334),
    [anon_sym_TILDE] = ACTIONS(1334),
    [anon_sym_DASH] = ACTIONS(1332),
    [anon_sym_PLUS] = ACTIONS(1332),
    [anon_sym_STAR] = ACTIONS(1334),
    [anon_sym_AMP] = ACTIONS(1334),
    [anon_sym_SEMI] = ACTIONS(1334),
    [anon_sym___extension__] = ACTIONS(1332),
    [anon_sym_typedef] = ACTIONS(1332),
    [anon_sym_extern] = ACTIONS(1332),
    [anon_sym___attribute__] = ACTIONS(1332),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1334),
    [anon_sym___declspec] = ACTIONS(1332),
    [anon_sym___cdecl] = ACTIONS(1332),
    [anon_sym___clrcall] = ACTIONS(1332),
    [anon_sym___stdcall] = ACTIONS(1332),
    [anon_sym___fastcall] = ACTIONS(1332),
    [anon_sym___thiscall] = ACTIONS(1332),
    [anon_sym___vectorcall] = ACTIONS(1332),
    [anon_sym_LBRACE] = ACTIONS(1334),
    [anon_sym_signed] = ACTIONS(1332),
    [anon_sym_unsigned] = ACTIONS(1332),
    [anon_sym_long] = ACTIONS(1332),
    [anon_sym_short] = ACTIONS(1332),
    [anon_sym_static] = ACTIONS(1332),
    [anon_sym_auto] = ACTIONS(1332),
    [anon_sym_register] = ACTIONS(1332),
    [anon_sym_inline] = ACTIONS(1332),
    [anon_sym___inline] = ACTIONS(1332),
    [anon_sym___inline__] = ACTIONS(1332),
    [anon_sym___forceinline] = ACTIONS(1332),
    [anon_sym_thread_local] = ACTIONS(1332),
    [anon_sym___thread] = ACTIONS(1332),
    [anon_sym_const] = ACTIONS(1332),
    [anon_sym_constexpr] = ACTIONS(1332),
    [anon_sym_volatile] = ACTIONS(1332),
    [anon_sym_restrict] = ACTIONS(1332),
    [anon_sym___restrict__] = ACTIONS(1332),
    [anon_sym__Atomic] = ACTIONS(1332),
    [anon_sym__Noreturn] = ACTIONS(1332),
    [anon_sym_noreturn] = ACTIONS(1332),
    [sym_primitive_type] = ACTIONS(1332),
    [anon_sym_enum] = ACTIONS(1332),
    [anon_sym_struct] = ACTIONS(1332),
    [anon_sym_union] = ACTIONS(1332),
    [anon_sym_if] = ACTIONS(1332),
    [anon_sym_switch] = ACTIONS(1332),
    [anon_sym_case] = ACTIONS(1332),
    [anon_sym_default] = ACTIONS(1332),
    [anon_sym_while] = ACTIONS(1332),
    [anon_sym_do] = ACTIONS(1332),
    [anon_sym_for] = ACTIONS(1332),
    [anon_sym_return] = ACTIONS(1332),
    [anon_sym_break] = ACTIONS(1332),
    [anon_sym_continue] = ACTIONS(1332),
    [anon_sym_goto] = ACTIONS(1332),
    [anon_sym___try] = ACTIONS(1332),
    [anon_sym___leave] = ACTIONS(1332),
    [anon_sym_DASH_DASH] = ACTIONS(1334),
    [anon_sym_PLUS_PLUS] = ACTIONS(1334),
    [anon_sym_sizeof] = ACTIONS(1332),
    [anon_sym___alignof__] = ACTIONS(1332),
    [anon_sym___alignof] = ACTIONS(1332),
    [anon_sym__alignof] = ACTIONS(1332),
    [anon_sym_alignof] = ACTIONS(1332),
    [anon_sym__Alignof] = ACTIONS(1332),
    [anon_sym_offsetof] = ACTIONS(1332),
    [anon_sym__Generic] = ACTIONS(1332),
    [anon_sym_asm] = ACTIONS(1332),
    [anon_sym___asm__] = ACTIONS(1332),
    [sym_number_literal] = ACTIONS(1334),
    [anon_sym_L_SQUOTE] = ACTIONS(1334),
    [anon_sym_u_SQUOTE] = ACTIONS(1334),
    [anon_sym_U_SQUOTE] = ACTIONS(1334),
    [anon_sym_u8_SQUOTE] = ACTIONS(1334),
    [anon_sym_SQUOTE] = ACTIONS(1334),
    [anon_sym_L_DQUOTE] = ACTIONS(1334),
    [anon_sym_u_DQUOTE] = ACTIONS(1334),
    [anon_sym_U_DQUOTE] = ACTIONS(1334),
    [anon_sym_u8_DQUOTE] = ACTIONS(1334),
    [anon_sym_DQUOTE] = ACTIONS(1334),
    [sym_true] = ACTIONS(1332),
    [sym_false] = ACTIONS(1332),
    [anon_sym_NULL] = ACTIONS(1332),
    [anon_sym_nullptr] = ACTIONS(1332),
    [sym_comment] = ACTIONS(3),
  },
  [153] = {
    [sym_identifier] = ACTIONS(1336),
    [aux_sym_preproc_include_token1] = ACTIONS(1336),
    [aux_sym_preproc_def_token1] = ACTIONS(1336),
    [aux_sym_preproc_if_token1] = ACTIONS(1336),
    [aux_sym_preproc_if_token2] = ACTIONS(1336),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1336),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1336),
    [aux_sym_preproc_else_token1] = ACTIONS(1336),
    [aux_sym_preproc_elif_token1] = ACTIONS(1336),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1336),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1336),
    [sym_preproc_directive] = ACTIONS(1336),
    [anon_sym_LPAREN2] = ACTIONS(1338),
    [anon_sym_BANG] = ACTIONS(1338),
    [anon_sym_TILDE] = ACTIONS(1338),
    [anon_sym_DASH] = ACTIONS(1336),
    [anon_sym_PLUS] = ACTIONS(1336),
    [anon_sym_STAR] = ACTIONS(1338),
    [anon_sym_AMP] = ACTIONS(1338),
    [anon_sym_SEMI] = ACTIONS(1338),
    [anon_sym___extension__] = ACTIONS(1336),
    [anon_sym_typedef] = ACTIONS(1336),
    [anon_sym_extern] = ACTIONS(1336),
    [anon_sym___attribute__] = ACTIONS(1336),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1338),
    [anon_sym___declspec] = ACTIONS(1336),
    [anon_sym___cdecl] = ACTIONS(1336),
    [anon_sym___clrcall] = ACTIONS(1336),
    [anon_sym___stdcall] = ACTIONS(1336),
    [anon_sym___fastcall] = ACTIONS(1336),
    [anon_sym___thiscall] = ACTIONS(1336),
    [anon_sym___vectorcall] = ACTIONS(1336),
    [anon_sym_LBRACE] = ACTIONS(1338),
    [anon_sym_signed] = ACTIONS(1336),
    [anon_sym_unsigned] = ACTIONS(1336),
    [anon_sym_long] = ACTIONS(1336),
    [anon_sym_short] = ACTIONS(1336),
    [anon_sym_static] = ACTIONS(1336),
    [anon_sym_auto] = ACTIONS(1336),
    [anon_sym_register] = ACTIONS(1336),
    [anon_sym_inline] = ACTIONS(1336),
    [anon_sym___inline] = ACTIONS(1336),
    [anon_sym___inline__] = ACTIONS(1336),
    [anon_sym___forceinline] = ACTIONS(1336),
    [anon_sym_thread_local] = ACTIONS(1336),
    [anon_sym___thread] = ACTIONS(1336),
    [anon_sym_const] = ACTIONS(1336),
    [anon_sym_constexpr] = ACTIONS(1336),
    [anon_sym_volatile] = ACTIONS(1336),
    [anon_sym_restrict] = ACTIONS(1336),
    [anon_sym___restrict__] = ACTIONS(1336),
    [anon_sym__Atomic] = ACTIONS(1336),
    [anon_sym__Noreturn] = ACTIONS(1336),
    [anon_sym_noreturn] = ACTIONS(1336),
    [sym_primitive_type] = ACTIONS(1336),
    [anon_sym_enum] = ACTIONS(1336),
    [anon_sym_struct] = ACTIONS(1336),
    [anon_sym_union] = ACTIONS(1336),
    [anon_sym_if] = ACTIONS(1336),
    [anon_sym_switch] = ACTIONS(1336),
    [anon_sym_case] = ACTIONS(1336),
    [anon_sym_default] = ACTIONS(1336),
    [anon_sym_while] = ACTIONS(1336),
    [anon_sym_do] = ACTIONS(1336),
    [anon_sym_for] = ACTIONS(1336),
    [anon_sym_return] = ACTIONS(1336),
    [anon_sym_break] = ACTIONS(1336),
    [anon_sym_continue] = ACTIONS(1336),
    [anon_sym_goto] = ACTIONS(1336),
    [anon_sym___try] = ACTIONS(1336),
    [anon_sym___leave] = ACTIONS(1336),
    [anon_sym_DASH_DASH] = ACTIONS(1338),
    [anon_sym_PLUS_PLUS] = ACTIONS(1338),
    [anon_sym_sizeof] = ACTIONS(1336),
    [anon_sym___alignof__] = ACTIONS(1336),
    [anon_sym___alignof] = ACTIONS(1336),
    [anon_sym__alignof] = ACTIONS(1336),
    [anon_sym_alignof] = ACTIONS(1336),
    [anon_sym__Alignof] = ACTIONS(1336),
    [anon_sym_offsetof] = ACTIONS(1336),
    [anon_sym__Generic] = ACTIONS(1336),
    [anon_sym_asm] = ACTIONS(1336),
    [anon_sym___asm__] = ACTIONS(1336),
    [sym_number_literal] = ACTIONS(1338),
    [anon_sym_L_SQUOTE] = ACTIONS(1338),
    [anon_sym_u_SQUOTE] = ACTIONS(1338),
    [anon_sym_U_SQUOTE] = ACTIONS(1338),
    [anon_sym_u8_SQUOTE] = ACTIONS(1338),
    [anon_sym_SQUOTE] = ACTIONS(1338),
    [anon_sym_L_DQUOTE] = ACTIONS(1338),
    [anon_sym_u_DQUOTE] = ACTIONS(1338),
    [anon_sym_U_DQUOTE] = ACTIONS(1338),
    [anon_sym_u8_DQUOTE] = ACTIONS(1338),
    [anon_sym_DQUOTE] = ACTIONS(1338),
    [sym_true] = ACTIONS(1336),
    [sym_false] = ACTIONS(1336),
    [anon_sym_NULL] = ACTIONS(1336),
    [anon_sym_nullptr] = ACTIONS(1336),
    [sym_comment] = ACTIONS(3),
  },
  [154] = {
    [sym_identifier] = ACTIONS(1340),
    [aux_sym_preproc_include_token1] = ACTIONS(1340),
    [aux_sym_preproc_def_token1] = ACTIONS(1340),
    [aux_sym_preproc_if_token1] = ACTIONS(1340),
    [aux_sym_preproc_if_token2] = ACTIONS(1340),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1340),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1340),
    [aux_sym_preproc_else_token1] = ACTIONS(1340),
    [aux_sym_preproc_elif_token1] = ACTIONS(1340),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1340),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1340),
    [sym_preproc_directive] = ACTIONS(1340),
    [anon_sym_LPAREN2] = ACTIONS(1342),
    [anon_sym_BANG] = ACTIONS(1342),
    [anon_sym_TILDE] = ACTIONS(1342),
    [anon_sym_DASH] = ACTIONS(1340),
    [anon_sym_PLUS] = ACTIONS(1340),
    [anon_sym_STAR] = ACTIONS(1342),
    [anon_sym_AMP] = ACTIONS(1342),
    [anon_sym_SEMI] = ACTIONS(1342),
    [anon_sym___extension__] = ACTIONS(1340),
    [anon_sym_typedef] = ACTIONS(1340),
    [anon_sym_extern] = ACTIONS(1340),
    [anon_sym___attribute__] = ACTIONS(1340),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1342),
    [anon_sym___declspec] = ACTIONS(1340),
    [anon_sym___cdecl] = ACTIONS(1340),
    [anon_sym___clrcall] = ACTIONS(1340),
    [anon_sym___stdcall] = ACTIONS(1340),
    [anon_sym___fastcall] = ACTIONS(1340),
    [anon_sym___thiscall] = ACTIONS(1340),
    [anon_sym___vectorcall] = ACTIONS(1340),
    [anon_sym_LBRACE] = ACTIONS(1342),
    [anon_sym_signed] = ACTIONS(1340),
    [anon_sym_unsigned] = ACTIONS(1340),
    [anon_sym_long] = ACTIONS(1340),
    [anon_sym_short] = ACTIONS(1340),
    [anon_sym_static] = ACTIONS(1340),
    [anon_sym_auto] = ACTIONS(1340),
    [anon_sym_register] = ACTIONS(1340),
    [anon_sym_inline] = ACTIONS(1340),
    [anon_sym___inline] = ACTIONS(1340),
    [anon_sym___inline__] = ACTIONS(1340),
    [anon_sym___forceinline] = ACTIONS(1340),
    [anon_sym_thread_local] = ACTIONS(1340),
    [anon_sym___thread] = ACTIONS(1340),
    [anon_sym_const] = ACTIONS(1340),
    [anon_sym_constexpr] = ACTIONS(1340),
    [anon_sym_volatile] = ACTIONS(1340),
    [anon_sym_restrict] = ACTIONS(1340),
    [anon_sym___restrict__] = ACTIONS(1340),
    [anon_sym__Atomic] = ACTIONS(1340),
    [anon_sym__Noreturn] = ACTIONS(1340),
    [anon_sym_noreturn] = ACTIONS(1340),
    [sym_primitive_type] = ACTIONS(1340),
    [anon_sym_enum] = ACTIONS(1340),
    [anon_sym_struct] = ACTIONS(1340),
    [anon_sym_union] = ACTIONS(1340),
    [anon_sym_if] = ACTIONS(1340),
    [anon_sym_switch] = ACTIONS(1340),
    [anon_sym_case] = ACTIONS(1340),
    [anon_sym_default] = ACTIONS(1340),
    [anon_sym_while] = ACTIONS(1340),
    [anon_sym_do] = ACTIONS(1340),
    [anon_sym_for] = ACTIONS(1340),
    [anon_sym_return] = ACTIONS(1340),
    [anon_sym_break] = ACTIONS(1340),
    [anon_sym_continue] = ACTIONS(1340),
    [anon_sym_goto] = ACTIONS(1340),
    [anon_sym___try] = ACTIONS(1340),
    [anon_sym___leave] = ACTIONS(1340),
    [anon_sym_DASH_DASH] = ACTIONS(1342),
    [anon_sym_PLUS_PLUS] = ACTIONS(1342),
    [anon_sym_sizeof] = ACTIONS(1340),
    [anon_sym___alignof__] = ACTIONS(1340),
    [anon_sym___alignof] = ACTIONS(1340),
    [anon_sym__alignof] = ACTIONS(1340),
    [anon_sym_alignof] = ACTIONS(1340),
    [anon_sym__Alignof] = ACTIONS(1340),
    [anon_sym_offsetof] = ACTIONS(1340),
    [anon_sym__Generic] = ACTIONS(1340),
    [anon_sym_asm] = ACTIONS(1340),
    [anon_sym___asm__] = ACTIONS(1340),
    [sym_number_literal] = ACTIONS(1342),
    [anon_sym_L_SQUOTE] = ACTIONS(1342),
    [anon_sym_u_SQUOTE] = ACTIONS(1342),
    [anon_sym_U_SQUOTE] = ACTIONS(1342),
    [anon_sym_u8_SQUOTE] = ACTIONS(1342),
    [anon_sym_SQUOTE] = ACTIONS(1342),
    [anon_sym_L_DQUOTE] = ACTIONS(1342),
    [anon_sym_u_DQUOTE] = ACTIONS(1342),
    [anon_sym_U_DQUOTE] = ACTIONS(1342),
    [anon_sym_u8_DQUOTE] = ACTIONS(1342),
    [anon_sym_DQUOTE] = ACTIONS(1342),
    [sym_true] = ACTIONS(1340),
    [sym_false] = ACTIONS(1340),
    [anon_sym_NULL] = ACTIONS(1340),
    [anon_sym_nullptr] = ACTIONS(1340),
    [sym_comment] = ACTIONS(3),
  },
  [155] = {
    [sym_identifier] = ACTIONS(1344),
    [aux_sym_preproc_include_token1] = ACTIONS(1344),
    [aux_sym_preproc_def_token1] = ACTIONS(1344),
    [aux_sym_preproc_if_token1] = ACTIONS(1344),
    [aux_sym_preproc_if_token2] = ACTIONS(1344),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1344),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1344),
    [aux_sym_preproc_else_token1] = ACTIONS(1344),
    [aux_sym_preproc_elif_token1] = ACTIONS(1344),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1344),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1344),
    [sym_preproc_directive] = ACTIONS(1344),
    [anon_sym_LPAREN2] = ACTIONS(1346),
    [anon_sym_BANG] = ACTIONS(1346),
    [anon_sym_TILDE] = ACTIONS(1346),
    [anon_sym_DASH] = ACTIONS(1344),
    [anon_sym_PLUS] = ACTIONS(1344),
    [anon_sym_STAR] = ACTIONS(1346),
    [anon_sym_AMP] = ACTIONS(1346),
    [anon_sym_SEMI] = ACTIONS(1346),
    [anon_sym___extension__] = ACTIONS(1344),
    [anon_sym_typedef] = ACTIONS(1344),
    [anon_sym_extern] = ACTIONS(1344),
    [anon_sym___attribute__] = ACTIONS(1344),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1346),
    [anon_sym___declspec] = ACTIONS(1344),
    [anon_sym___cdecl] = ACTIONS(1344),
    [anon_sym___clrcall] = ACTIONS(1344),
    [anon_sym___stdcall] = ACTIONS(1344),
    [anon_sym___fastcall] = ACTIONS(1344),
    [anon_sym___thiscall] = ACTIONS(1344),
    [anon_sym___vectorcall] = ACTIONS(1344),
    [anon_sym_LBRACE] = ACTIONS(1346),
    [anon_sym_signed] = ACTIONS(1344),
    [anon_sym_unsigned] = ACTIONS(1344),
    [anon_sym_long] = ACTIONS(1344),
    [anon_sym_short] = ACTIONS(1344),
    [anon_sym_static] = ACTIONS(1344),
    [anon_sym_auto] = ACTIONS(1344),
    [anon_sym_register] = ACTIONS(1344),
    [anon_sym_inline] = ACTIONS(1344),
    [anon_sym___inline] = ACTIONS(1344),
    [anon_sym___inline__] = ACTIONS(1344),
    [anon_sym___forceinline] = ACTIONS(1344),
    [anon_sym_thread_local] = ACTIONS(1344),
    [anon_sym___thread] = ACTIONS(1344),
    [anon_sym_const] = ACTIONS(1344),
    [anon_sym_constexpr] = ACTIONS(1344),
    [anon_sym_volatile] = ACTIONS(1344),
    [anon_sym_restrict] = ACTIONS(1344),
    [anon_sym___restrict__] = ACTIONS(1344),
    [anon_sym__Atomic] = ACTIONS(1344),
    [anon_sym__Noreturn] = ACTIONS(1344),
    [anon_sym_noreturn] = ACTIONS(1344),
    [sym_primitive_type] = ACTIONS(1344),
    [anon_sym_enum] = ACTIONS(1344),
    [anon_sym_struct] = ACTIONS(1344),
    [anon_sym_union] = ACTIONS(1344),
    [anon_sym_if] = ACTIONS(1344),
    [anon_sym_switch] = ACTIONS(1344),
    [anon_sym_case] = ACTIONS(1344),
    [anon_sym_default] = ACTIONS(1344),
    [anon_sym_while] = ACTIONS(1344),
    [anon_sym_do] = ACTIONS(1344),
    [anon_sym_for] = ACTIONS(1344),
    [anon_sym_return] = ACTIONS(1344),
    [anon_sym_break] = ACTIONS(1344),
    [anon_sym_continue] = ACTIONS(1344),
    [anon_sym_goto] = ACTIONS(1344),
    [anon_sym___try] = ACTIONS(1344),
    [anon_sym___leave] = ACTIONS(1344),
    [anon_sym_DASH_DASH] = ACTIONS(1346),
    [anon_sym_PLUS_PLUS] = ACTIONS(1346),
    [anon_sym_sizeof] = ACTIONS(1344),
    [anon_sym___alignof__] = ACTIONS(1344),
    [anon_sym___alignof] = ACTIONS(1344),
    [anon_sym__alignof] = ACTIONS(1344),
    [anon_sym_alignof] = ACTIONS(1344),
    [anon_sym__Alignof] = ACTIONS(1344),
    [anon_sym_offsetof] = ACTIONS(1344),
    [anon_sym__Generic] = ACTIONS(1344),
    [anon_sym_asm] = ACTIONS(1344),
    [anon_sym___asm__] = ACTIONS(1344),
    [sym_number_literal] = ACTIONS(1346),
    [anon_sym_L_SQUOTE] = ACTIONS(1346),
    [anon_sym_u_SQUOTE] = ACTIONS(1346),
    [anon_sym_U_SQUOTE] = ACTIONS(1346),
    [anon_sym_u8_SQUOTE] = ACTIONS(1346),
    [anon_sym_SQUOTE] = ACTIONS(1346),
    [anon_sym_L_DQUOTE] = ACTIONS(1346),
    [anon_sym_u_DQUOTE] = ACTIONS(1346),
    [anon_sym_U_DQUOTE] = ACTIONS(1346),
    [anon_sym_u8_DQUOTE] = ACTIONS(1346),
    [anon_sym_DQUOTE] = ACTIONS(1346),
    [sym_true] = ACTIONS(1344),
    [sym_false] = ACTIONS(1344),
    [anon_sym_NULL] = ACTIONS(1344),
    [anon_sym_nullptr] = ACTIONS(1344),
    [sym_comment] = ACTIONS(3),
  },
  [156] = {
    [sym_else_clause] = STATE(190),
    [sym_identifier] = ACTIONS(1106),
    [aux_sym_preproc_include_token1] = ACTIONS(1106),
    [aux_sym_preproc_def_token1] = ACTIONS(1106),
    [aux_sym_preproc_if_token1] = ACTIONS(1106),
    [aux_sym_preproc_if_token2] = ACTIONS(1106),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1106),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1106),
    [sym_preproc_directive] = ACTIONS(1106),
    [anon_sym_LPAREN2] = ACTIONS(1108),
    [anon_sym_BANG] = ACTIONS(1108),
    [anon_sym_TILDE] = ACTIONS(1108),
    [anon_sym_DASH] = ACTIONS(1106),
    [anon_sym_PLUS] = ACTIONS(1106),
    [anon_sym_STAR] = ACTIONS(1108),
    [anon_sym_AMP] = ACTIONS(1108),
    [anon_sym_SEMI] = ACTIONS(1108),
    [anon_sym___extension__] = ACTIONS(1106),
    [anon_sym_typedef] = ACTIONS(1106),
    [anon_sym_extern] = ACTIONS(1106),
    [anon_sym___attribute__] = ACTIONS(1106),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1108),
    [anon_sym___declspec] = ACTIONS(1106),
    [anon_sym___cdecl] = ACTIONS(1106),
    [anon_sym___clrcall] = ACTIONS(1106),
    [anon_sym___stdcall] = ACTIONS(1106),
    [anon_sym___fastcall] = ACTIONS(1106),
    [anon_sym___thiscall] = ACTIONS(1106),
    [anon_sym___vectorcall] = ACTIONS(1106),
    [anon_sym_LBRACE] = ACTIONS(1108),
    [anon_sym_signed] = ACTIONS(1106),
    [anon_sym_unsigned] = ACTIONS(1106),
    [anon_sym_long] = ACTIONS(1106),
    [anon_sym_short] = ACTIONS(1106),
    [anon_sym_static] = ACTIONS(1106),
    [anon_sym_auto] = ACTIONS(1106),
    [anon_sym_register] = ACTIONS(1106),
    [anon_sym_inline] = ACTIONS(1106),
    [anon_sym___inline] = ACTIONS(1106),
    [anon_sym___inline__] = ACTIONS(1106),
    [anon_sym___forceinline] = ACTIONS(1106),
    [anon_sym_thread_local] = ACTIONS(1106),
    [anon_sym___thread] = ACTIONS(1106),
    [anon_sym_const] = ACTIONS(1106),
    [anon_sym_constexpr] = ACTIONS(1106),
    [anon_sym_volatile] = ACTIONS(1106),
    [anon_sym_restrict] = ACTIONS(1106),
    [anon_sym___restrict__] = ACTIONS(1106),
    [anon_sym__Atomic] = ACTIONS(1106),
    [anon_sym__Noreturn] = ACTIONS(1106),
    [anon_sym_noreturn] = ACTIONS(1106),
    [sym_primitive_type] = ACTIONS(1106),
    [anon_sym_enum] = ACTIONS(1106),
    [anon_sym_struct] = ACTIONS(1106),
    [anon_sym_union] = ACTIONS(1106),
    [anon_sym_if] = ACTIONS(1106),
    [anon_sym_else] = ACTIONS(1348),
    [anon_sym_switch] = ACTIONS(1106),
    [anon_sym_case] = ACTIONS(1106),
    [anon_sym_default] = ACTIONS(1106),
    [anon_sym_while] = ACTIONS(1106),
    [anon_sym_do] = ACTIONS(1106),
    [anon_sym_for] = ACTIONS(1106),
    [anon_sym_return] = ACTIONS(1106),
    [anon_sym_break] = ACTIONS(1106),
    [anon_sym_continue] = ACTIONS(1106),
    [anon_sym_goto] = ACTIONS(1106),
    [anon_sym___try] = ACTIONS(1106),
    [anon_sym___leave] = ACTIONS(1106),
    [anon_sym_DASH_DASH] = ACTIONS(1108),
    [anon_sym_PLUS_PLUS] = ACTIONS(1108),
    [anon_sym_sizeof] = ACTIONS(1106),
    [anon_sym___alignof__] = ACTIONS(1106),
    [anon_sym___alignof] = ACTIONS(1106),
    [anon_sym__alignof] = ACTIONS(1106),
    [anon_sym_alignof] = ACTIONS(1106),
    [anon_sym__Alignof] = ACTIONS(1106),
    [anon_sym_offsetof] = ACTIONS(1106),
    [anon_sym__Generic] = ACTIONS(1106),
    [anon_sym_asm] = ACTIONS(1106),
    [anon_sym___asm__] = ACTIONS(1106),
    [sym_number_literal] = ACTIONS(1108),
    [anon_sym_L_SQUOTE] = ACTIONS(1108),
    [anon_sym_u_SQUOTE] = ACTIONS(1108),
    [anon_sym_U_SQUOTE] = ACTIONS(1108),
    [anon_sym_u8_SQUOTE] = ACTIONS(1108),
    [anon_sym_SQUOTE] = ACTIONS(1108),
    [anon_sym_L_DQUOTE] = ACTIONS(1108),
    [anon_sym_u_DQUOTE] = ACTIONS(1108),
    [anon_sym_U_DQUOTE] = ACTIONS(1108),
    [anon_sym_u8_DQUOTE] = ACTIONS(1108),
    [anon_sym_DQUOTE] = ACTIONS(1108),
    [sym_true] = ACTIONS(1106),
    [sym_false] = ACTIONS(1106),
    [anon_sym_NULL] = ACTIONS(1106),
    [anon_sym_nullptr] = ACTIONS(1106),
    [sym_comment] = ACTIONS(3),
  },
  [157] = {
    [sym__expression] = STATE(722),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(700),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(700),
    [sym_call_expression] = STATE(700),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(700),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(700),
    [sym_initializer_list] = STATE(714),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(735),
    [sym_null] = STATE(700),
    [sym_identifier] = ACTIONS(1350),
    [anon_sym_COMMA] = ACTIONS(1352),
    [aux_sym_preproc_if_token2] = ACTIONS(1352),
    [aux_sym_preproc_else_token1] = ACTIONS(1352),
    [aux_sym_preproc_elif_token1] = ACTIONS(1350),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1352),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1352),
    [anon_sym_LPAREN2] = ACTIONS(1352),
    [anon_sym_BANG] = ACTIONS(1354),
    [anon_sym_TILDE] = ACTIONS(1356),
    [anon_sym_DASH] = ACTIONS(1350),
    [anon_sym_PLUS] = ACTIONS(1350),
    [anon_sym_STAR] = ACTIONS(1350),
    [anon_sym_SLASH] = ACTIONS(1350),
    [anon_sym_PERCENT] = ACTIONS(1350),
    [anon_sym_PIPE_PIPE] = ACTIONS(1352),
    [anon_sym_AMP_AMP] = ACTIONS(1352),
    [anon_sym_PIPE] = ACTIONS(1350),
    [anon_sym_CARET] = ACTIONS(1350),
    [anon_sym_AMP] = ACTIONS(1350),
    [anon_sym_EQ_EQ] = ACTIONS(1352),
    [anon_sym_BANG_EQ] = ACTIONS(1352),
    [anon_sym_GT] = ACTIONS(1350),
    [anon_sym_GT_EQ] = ACTIONS(1352),
    [anon_sym_LT_EQ] = ACTIONS(1352),
    [anon_sym_LT] = ACTIONS(1350),
    [anon_sym_LT_LT] = ACTIONS(1350),
    [anon_sym_GT_GT] = ACTIONS(1350),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_LBRACK] = ACTIONS(1352),
    [anon_sym_EQ] = ACTIONS(1350),
    [anon_sym_QMARK] = ACTIONS(1352),
    [anon_sym_STAR_EQ] = ACTIONS(1352),
    [anon_sym_SLASH_EQ] = ACTIONS(1352),
    [anon_sym_PERCENT_EQ] = ACTIONS(1352),
    [anon_sym_PLUS_EQ] = ACTIONS(1352),
    [anon_sym_DASH_EQ] = ACTIONS(1352),
    [anon_sym_LT_LT_EQ] = ACTIONS(1352),
    [anon_sym_GT_GT_EQ] = ACTIONS(1352),
    [anon_sym_AMP_EQ] = ACTIONS(1352),
    [anon_sym_CARET_EQ] = ACTIONS(1352),
    [anon_sym_PIPE_EQ] = ACTIONS(1352),
    [anon_sym_DASH_DASH] = ACTIONS(1352),
    [anon_sym_PLUS_PLUS] = ACTIONS(1352),
    [anon_sym_sizeof] = ACTIONS(1360),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1350),
    [anon_sym_DASH_GT] = ACTIONS(1352),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [158] = {
    [sym_else_clause] = STATE(220),
    [ts_builtin_sym_end] = ACTIONS(1108),
    [sym_identifier] = ACTIONS(1106),
    [aux_sym_preproc_include_token1] = ACTIONS(1106),
    [aux_sym_preproc_def_token1] = ACTIONS(1106),
    [aux_sym_preproc_if_token1] = ACTIONS(1106),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1106),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1106),
    [sym_preproc_directive] = ACTIONS(1106),
    [anon_sym_LPAREN2] = ACTIONS(1108),
    [anon_sym_BANG] = ACTIONS(1108),
    [anon_sym_TILDE] = ACTIONS(1108),
    [anon_sym_DASH] = ACTIONS(1106),
    [anon_sym_PLUS] = ACTIONS(1106),
    [anon_sym_STAR] = ACTIONS(1108),
    [anon_sym_AMP] = ACTIONS(1108),
    [anon_sym_SEMI] = ACTIONS(1108),
    [anon_sym___extension__] = ACTIONS(1106),
    [anon_sym_typedef] = ACTIONS(1106),
    [anon_sym_extern] = ACTIONS(1106),
    [anon_sym___attribute__] = ACTIONS(1106),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1108),
    [anon_sym___declspec] = ACTIONS(1106),
    [anon_sym___cdecl] = ACTIONS(1106),
    [anon_sym___clrcall] = ACTIONS(1106),
    [anon_sym___stdcall] = ACTIONS(1106),
    [anon_sym___fastcall] = ACTIONS(1106),
    [anon_sym___thiscall] = ACTIONS(1106),
    [anon_sym___vectorcall] = ACTIONS(1106),
    [anon_sym_LBRACE] = ACTIONS(1108),
    [anon_sym_signed] = ACTIONS(1106),
    [anon_sym_unsigned] = ACTIONS(1106),
    [anon_sym_long] = ACTIONS(1106),
    [anon_sym_short] = ACTIONS(1106),
    [anon_sym_static] = ACTIONS(1106),
    [anon_sym_auto] = ACTIONS(1106),
    [anon_sym_register] = ACTIONS(1106),
    [anon_sym_inline] = ACTIONS(1106),
    [anon_sym___inline] = ACTIONS(1106),
    [anon_sym___inline__] = ACTIONS(1106),
    [anon_sym___forceinline] = ACTIONS(1106),
    [anon_sym_thread_local] = ACTIONS(1106),
    [anon_sym___thread] = ACTIONS(1106),
    [anon_sym_const] = ACTIONS(1106),
    [anon_sym_constexpr] = ACTIONS(1106),
    [anon_sym_volatile] = ACTIONS(1106),
    [anon_sym_restrict] = ACTIONS(1106),
    [anon_sym___restrict__] = ACTIONS(1106),
    [anon_sym__Atomic] = ACTIONS(1106),
    [anon_sym__Noreturn] = ACTIONS(1106),
    [anon_sym_noreturn] = ACTIONS(1106),
    [sym_primitive_type] = ACTIONS(1106),
    [anon_sym_enum] = ACTIONS(1106),
    [anon_sym_struct] = ACTIONS(1106),
    [anon_sym_union] = ACTIONS(1106),
    [anon_sym_if] = ACTIONS(1106),
    [anon_sym_else] = ACTIONS(1362),
    [anon_sym_switch] = ACTIONS(1106),
    [anon_sym_case] = ACTIONS(1106),
    [anon_sym_default] = ACTIONS(1106),
    [anon_sym_while] = ACTIONS(1106),
    [anon_sym_do] = ACTIONS(1106),
    [anon_sym_for] = ACTIONS(1106),
    [anon_sym_return] = ACTIONS(1106),
    [anon_sym_break] = ACTIONS(1106),
    [anon_sym_continue] = ACTIONS(1106),
    [anon_sym_goto] = ACTIONS(1106),
    [anon_sym___try] = ACTIONS(1106),
    [anon_sym___leave] = ACTIONS(1106),
    [anon_sym_DASH_DASH] = ACTIONS(1108),
    [anon_sym_PLUS_PLUS] = ACTIONS(1108),
    [anon_sym_sizeof] = ACTIONS(1106),
    [anon_sym___alignof__] = ACTIONS(1106),
    [anon_sym___alignof] = ACTIONS(1106),
    [anon_sym__alignof] = ACTIONS(1106),
    [anon_sym_alignof] = ACTIONS(1106),
    [anon_sym__Alignof] = ACTIONS(1106),
    [anon_sym_offsetof] = ACTIONS(1106),
    [anon_sym__Generic] = ACTIONS(1106),
    [anon_sym_asm] = ACTIONS(1106),
    [anon_sym___asm__] = ACTIONS(1106),
    [sym_number_literal] = ACTIONS(1108),
    [anon_sym_L_SQUOTE] = ACTIONS(1108),
    [anon_sym_u_SQUOTE] = ACTIONS(1108),
    [anon_sym_U_SQUOTE] = ACTIONS(1108),
    [anon_sym_u8_SQUOTE] = ACTIONS(1108),
    [anon_sym_SQUOTE] = ACTIONS(1108),
    [anon_sym_L_DQUOTE] = ACTIONS(1108),
    [anon_sym_u_DQUOTE] = ACTIONS(1108),
    [anon_sym_U_DQUOTE] = ACTIONS(1108),
    [anon_sym_u8_DQUOTE] = ACTIONS(1108),
    [anon_sym_DQUOTE] = ACTIONS(1108),
    [sym_true] = ACTIONS(1106),
    [sym_false] = ACTIONS(1106),
    [anon_sym_NULL] = ACTIONS(1106),
    [anon_sym_nullptr] = ACTIONS(1106),
    [sym_comment] = ACTIONS(3),
  },
  [159] = {
    [sym__expression] = STATE(722),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(700),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(700),
    [sym_call_expression] = STATE(700),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(700),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(700),
    [sym_initializer_list] = STATE(714),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_identifier] = ACTIONS(1364),
    [anon_sym_COMMA] = ACTIONS(1352),
    [anon_sym_RPAREN] = ACTIONS(1352),
    [anon_sym_LPAREN2] = ACTIONS(1352),
    [anon_sym_BANG] = ACTIONS(1366),
    [anon_sym_TILDE] = ACTIONS(1368),
    [anon_sym_DASH] = ACTIONS(1350),
    [anon_sym_PLUS] = ACTIONS(1350),
    [anon_sym_STAR] = ACTIONS(1350),
    [anon_sym_SLASH] = ACTIONS(1350),
    [anon_sym_PERCENT] = ACTIONS(1350),
    [anon_sym_PIPE_PIPE] = ACTIONS(1352),
    [anon_sym_AMP_AMP] = ACTIONS(1352),
    [anon_sym_PIPE] = ACTIONS(1350),
    [anon_sym_CARET] = ACTIONS(1350),
    [anon_sym_AMP] = ACTIONS(1350),
    [anon_sym_EQ_EQ] = ACTIONS(1352),
    [anon_sym_BANG_EQ] = ACTIONS(1352),
    [anon_sym_GT] = ACTIONS(1350),
    [anon_sym_GT_EQ] = ACTIONS(1352),
    [anon_sym_LT_EQ] = ACTIONS(1352),
    [anon_sym_LT] = ACTIONS(1350),
    [anon_sym_LT_LT] = ACTIONS(1350),
    [anon_sym_GT_GT] = ACTIONS(1350),
    [anon_sym_SEMI] = ACTIONS(1352),
    [anon_sym___attribute__] = ACTIONS(1350),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_RBRACE] = ACTIONS(1352),
    [anon_sym_LBRACK] = ACTIONS(1352),
    [anon_sym_EQ] = ACTIONS(1350),
    [anon_sym_COLON] = ACTIONS(1352),
    [anon_sym_QMARK] = ACTIONS(1352),
    [anon_sym_STAR_EQ] = ACTIONS(1352),
    [anon_sym_SLASH_EQ] = ACTIONS(1352),
    [anon_sym_PERCENT_EQ] = ACTIONS(1352),
    [anon_sym_PLUS_EQ] = ACTIONS(1352),
    [anon_sym_DASH_EQ] = ACTIONS(1352),
    [anon_sym_LT_LT_EQ] = ACTIONS(1352),
    [anon_sym_GT_GT_EQ] = ACTIONS(1352),
    [anon_sym_AMP_EQ] = ACTIONS(1352),
    [anon_sym_CARET_EQ] = ACTIONS(1352),
    [anon_sym_PIPE_EQ] = ACTIONS(1352),
    [anon_sym_DASH_DASH] = ACTIONS(1352),
    [anon_sym_PLUS_PLUS] = ACTIONS(1352),
    [anon_sym_sizeof] = ACTIONS(1370),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1350),
    [anon_sym_DASH_GT] = ACTIONS(1352),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [160] = {
    [sym_else_clause] = STATE(269),
    [sym_identifier] = ACTIONS(1106),
    [aux_sym_preproc_include_token1] = ACTIONS(1106),
    [aux_sym_preproc_def_token1] = ACTIONS(1106),
    [aux_sym_preproc_if_token1] = ACTIONS(1106),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1106),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1106),
    [sym_preproc_directive] = ACTIONS(1106),
    [anon_sym_LPAREN2] = ACTIONS(1108),
    [anon_sym_BANG] = ACTIONS(1108),
    [anon_sym_TILDE] = ACTIONS(1108),
    [anon_sym_DASH] = ACTIONS(1106),
    [anon_sym_PLUS] = ACTIONS(1106),
    [anon_sym_STAR] = ACTIONS(1108),
    [anon_sym_AMP] = ACTIONS(1108),
    [anon_sym_SEMI] = ACTIONS(1108),
    [anon_sym___extension__] = ACTIONS(1106),
    [anon_sym_typedef] = ACTIONS(1106),
    [anon_sym_extern] = ACTIONS(1106),
    [anon_sym___attribute__] = ACTIONS(1106),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1108),
    [anon_sym___declspec] = ACTIONS(1106),
    [anon_sym___cdecl] = ACTIONS(1106),
    [anon_sym___clrcall] = ACTIONS(1106),
    [anon_sym___stdcall] = ACTIONS(1106),
    [anon_sym___fastcall] = ACTIONS(1106),
    [anon_sym___thiscall] = ACTIONS(1106),
    [anon_sym___vectorcall] = ACTIONS(1106),
    [anon_sym_LBRACE] = ACTIONS(1108),
    [anon_sym_RBRACE] = ACTIONS(1108),
    [anon_sym_signed] = ACTIONS(1106),
    [anon_sym_unsigned] = ACTIONS(1106),
    [anon_sym_long] = ACTIONS(1106),
    [anon_sym_short] = ACTIONS(1106),
    [anon_sym_static] = ACTIONS(1106),
    [anon_sym_auto] = ACTIONS(1106),
    [anon_sym_register] = ACTIONS(1106),
    [anon_sym_inline] = ACTIONS(1106),
    [anon_sym___inline] = ACTIONS(1106),
    [anon_sym___inline__] = ACTIONS(1106),
    [anon_sym___forceinline] = ACTIONS(1106),
    [anon_sym_thread_local] = ACTIONS(1106),
    [anon_sym___thread] = ACTIONS(1106),
    [anon_sym_const] = ACTIONS(1106),
    [anon_sym_constexpr] = ACTIONS(1106),
    [anon_sym_volatile] = ACTIONS(1106),
    [anon_sym_restrict] = ACTIONS(1106),
    [anon_sym___restrict__] = ACTIONS(1106),
    [anon_sym__Atomic] = ACTIONS(1106),
    [anon_sym__Noreturn] = ACTIONS(1106),
    [anon_sym_noreturn] = ACTIONS(1106),
    [sym_primitive_type] = ACTIONS(1106),
    [anon_sym_enum] = ACTIONS(1106),
    [anon_sym_struct] = ACTIONS(1106),
    [anon_sym_union] = ACTIONS(1106),
    [anon_sym_if] = ACTIONS(1106),
    [anon_sym_else] = ACTIONS(1372),
    [anon_sym_switch] = ACTIONS(1106),
    [anon_sym_case] = ACTIONS(1106),
    [anon_sym_default] = ACTIONS(1106),
    [anon_sym_while] = ACTIONS(1106),
    [anon_sym_do] = ACTIONS(1106),
    [anon_sym_for] = ACTIONS(1106),
    [anon_sym_return] = ACTIONS(1106),
    [anon_sym_break] = ACTIONS(1106),
    [anon_sym_continue] = ACTIONS(1106),
    [anon_sym_goto] = ACTIONS(1106),
    [anon_sym___try] = ACTIONS(1106),
    [anon_sym___leave] = ACTIONS(1106),
    [anon_sym_DASH_DASH] = ACTIONS(1108),
    [anon_sym_PLUS_PLUS] = ACTIONS(1108),
    [anon_sym_sizeof] = ACTIONS(1106),
    [anon_sym___alignof__] = ACTIONS(1106),
    [anon_sym___alignof] = ACTIONS(1106),
    [anon_sym__alignof] = ACTIONS(1106),
    [anon_sym_alignof] = ACTIONS(1106),
    [anon_sym__Alignof] = ACTIONS(1106),
    [anon_sym_offsetof] = ACTIONS(1106),
    [anon_sym__Generic] = ACTIONS(1106),
    [anon_sym_asm] = ACTIONS(1106),
    [anon_sym___asm__] = ACTIONS(1106),
    [sym_number_literal] = ACTIONS(1108),
    [anon_sym_L_SQUOTE] = ACTIONS(1108),
    [anon_sym_u_SQUOTE] = ACTIONS(1108),
    [anon_sym_U_SQUOTE] = ACTIONS(1108),
    [anon_sym_u8_SQUOTE] = ACTIONS(1108),
    [anon_sym_SQUOTE] = ACTIONS(1108),
    [anon_sym_L_DQUOTE] = ACTIONS(1108),
    [anon_sym_u_DQUOTE] = ACTIONS(1108),
    [anon_sym_U_DQUOTE] = ACTIONS(1108),
    [anon_sym_u8_DQUOTE] = ACTIONS(1108),
    [anon_sym_DQUOTE] = ACTIONS(1108),
    [sym_true] = ACTIONS(1106),
    [sym_false] = ACTIONS(1106),
    [anon_sym_NULL] = ACTIONS(1106),
    [anon_sym_nullptr] = ACTIONS(1106),
    [sym_comment] = ACTIONS(3),
  },
  [161] = {
    [sym_identifier] = ACTIONS(1120),
    [aux_sym_preproc_include_token1] = ACTIONS(1120),
    [aux_sym_preproc_def_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token2] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1120),
    [sym_preproc_directive] = ACTIONS(1120),
    [anon_sym_LPAREN2] = ACTIONS(1122),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_DASH] = ACTIONS(1120),
    [anon_sym_PLUS] = ACTIONS(1120),
    [anon_sym_STAR] = ACTIONS(1122),
    [anon_sym_AMP] = ACTIONS(1122),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym___extension__] = ACTIONS(1120),
    [anon_sym_typedef] = ACTIONS(1120),
    [anon_sym_extern] = ACTIONS(1120),
    [anon_sym___attribute__] = ACTIONS(1120),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1122),
    [anon_sym___declspec] = ACTIONS(1120),
    [anon_sym___cdecl] = ACTIONS(1120),
    [anon_sym___clrcall] = ACTIONS(1120),
    [anon_sym___stdcall] = ACTIONS(1120),
    [anon_sym___fastcall] = ACTIONS(1120),
    [anon_sym___thiscall] = ACTIONS(1120),
    [anon_sym___vectorcall] = ACTIONS(1120),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_signed] = ACTIONS(1120),
    [anon_sym_unsigned] = ACTIONS(1120),
    [anon_sym_long] = ACTIONS(1120),
    [anon_sym_short] = ACTIONS(1120),
    [anon_sym_static] = ACTIONS(1120),
    [anon_sym_auto] = ACTIONS(1120),
    [anon_sym_register] = ACTIONS(1120),
    [anon_sym_inline] = ACTIONS(1120),
    [anon_sym___inline] = ACTIONS(1120),
    [anon_sym___inline__] = ACTIONS(1120),
    [anon_sym___forceinline] = ACTIONS(1120),
    [anon_sym_thread_local] = ACTIONS(1120),
    [anon_sym___thread] = ACTIONS(1120),
    [anon_sym_const] = ACTIONS(1120),
    [anon_sym_constexpr] = ACTIONS(1120),
    [anon_sym_volatile] = ACTIONS(1120),
    [anon_sym_restrict] = ACTIONS(1120),
    [anon_sym___restrict__] = ACTIONS(1120),
    [anon_sym__Atomic] = ACTIONS(1120),
    [anon_sym__Noreturn] = ACTIONS(1120),
    [anon_sym_noreturn] = ACTIONS(1120),
    [sym_primitive_type] = ACTIONS(1120),
    [anon_sym_enum] = ACTIONS(1120),
    [anon_sym_struct] = ACTIONS(1120),
    [anon_sym_union] = ACTIONS(1120),
    [anon_sym_if] = ACTIONS(1120),
    [anon_sym_else] = ACTIONS(1120),
    [anon_sym_switch] = ACTIONS(1120),
    [anon_sym_case] = ACTIONS(1120),
    [anon_sym_default] = ACTIONS(1120),
    [anon_sym_while] = ACTIONS(1120),
    [anon_sym_do] = ACTIONS(1120),
    [anon_sym_for] = ACTIONS(1120),
    [anon_sym_return] = ACTIONS(1120),
    [anon_sym_break] = ACTIONS(1120),
    [anon_sym_continue] = ACTIONS(1120),
    [anon_sym_goto] = ACTIONS(1120),
    [anon_sym___try] = ACTIONS(1120),
    [anon_sym___leave] = ACTIONS(1120),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_sizeof] = ACTIONS(1120),
    [anon_sym___alignof__] = ACTIONS(1120),
    [anon_sym___alignof] = ACTIONS(1120),
    [anon_sym__alignof] = ACTIONS(1120),
    [anon_sym_alignof] = ACTIONS(1120),
    [anon_sym__Alignof] = ACTIONS(1120),
    [anon_sym_offsetof] = ACTIONS(1120),
    [anon_sym__Generic] = ACTIONS(1120),
    [anon_sym_asm] = ACTIONS(1120),
    [anon_sym___asm__] = ACTIONS(1120),
    [sym_number_literal] = ACTIONS(1122),
    [anon_sym_L_SQUOTE] = ACTIONS(1122),
    [anon_sym_u_SQUOTE] = ACTIONS(1122),
    [anon_sym_U_SQUOTE] = ACTIONS(1122),
    [anon_sym_u8_SQUOTE] = ACTIONS(1122),
    [anon_sym_SQUOTE] = ACTIONS(1122),
    [anon_sym_L_DQUOTE] = ACTIONS(1122),
    [anon_sym_u_DQUOTE] = ACTIONS(1122),
    [anon_sym_U_DQUOTE] = ACTIONS(1122),
    [anon_sym_u8_DQUOTE] = ACTIONS(1122),
    [anon_sym_DQUOTE] = ACTIONS(1122),
    [sym_true] = ACTIONS(1120),
    [sym_false] = ACTIONS(1120),
    [anon_sym_NULL] = ACTIONS(1120),
    [anon_sym_nullptr] = ACTIONS(1120),
    [sym_comment] = ACTIONS(3),
  },
  [162] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [163] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [164] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [165] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [166] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [167] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [168] = {
    [sym_identifier] = ACTIONS(1232),
    [aux_sym_preproc_include_token1] = ACTIONS(1232),
    [aux_sym_preproc_def_token1] = ACTIONS(1232),
    [aux_sym_preproc_if_token1] = ACTIONS(1232),
    [aux_sym_preproc_if_token2] = ACTIONS(1232),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1232),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1232),
    [sym_preproc_directive] = ACTIONS(1232),
    [anon_sym_LPAREN2] = ACTIONS(1234),
    [anon_sym_BANG] = ACTIONS(1234),
    [anon_sym_TILDE] = ACTIONS(1234),
    [anon_sym_DASH] = ACTIONS(1232),
    [anon_sym_PLUS] = ACTIONS(1232),
    [anon_sym_STAR] = ACTIONS(1234),
    [anon_sym_AMP] = ACTIONS(1234),
    [anon_sym_SEMI] = ACTIONS(1234),
    [anon_sym___extension__] = ACTIONS(1232),
    [anon_sym_typedef] = ACTIONS(1232),
    [anon_sym_extern] = ACTIONS(1232),
    [anon_sym___attribute__] = ACTIONS(1232),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1234),
    [anon_sym___declspec] = ACTIONS(1232),
    [anon_sym___cdecl] = ACTIONS(1232),
    [anon_sym___clrcall] = ACTIONS(1232),
    [anon_sym___stdcall] = ACTIONS(1232),
    [anon_sym___fastcall] = ACTIONS(1232),
    [anon_sym___thiscall] = ACTIONS(1232),
    [anon_sym___vectorcall] = ACTIONS(1232),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_signed] = ACTIONS(1232),
    [anon_sym_unsigned] = ACTIONS(1232),
    [anon_sym_long] = ACTIONS(1232),
    [anon_sym_short] = ACTIONS(1232),
    [anon_sym_static] = ACTIONS(1232),
    [anon_sym_auto] = ACTIONS(1232),
    [anon_sym_register] = ACTIONS(1232),
    [anon_sym_inline] = ACTIONS(1232),
    [anon_sym___inline] = ACTIONS(1232),
    [anon_sym___inline__] = ACTIONS(1232),
    [anon_sym___forceinline] = ACTIONS(1232),
    [anon_sym_thread_local] = ACTIONS(1232),
    [anon_sym___thread] = ACTIONS(1232),
    [anon_sym_const] = ACTIONS(1232),
    [anon_sym_constexpr] = ACTIONS(1232),
    [anon_sym_volatile] = ACTIONS(1232),
    [anon_sym_restrict] = ACTIONS(1232),
    [anon_sym___restrict__] = ACTIONS(1232),
    [anon_sym__Atomic] = ACTIONS(1232),
    [anon_sym__Noreturn] = ACTIONS(1232),
    [anon_sym_noreturn] = ACTIONS(1232),
    [sym_primitive_type] = ACTIONS(1232),
    [anon_sym_enum] = ACTIONS(1232),
    [anon_sym_struct] = ACTIONS(1232),
    [anon_sym_union] = ACTIONS(1232),
    [anon_sym_if] = ACTIONS(1232),
    [anon_sym_else] = ACTIONS(1232),
    [anon_sym_switch] = ACTIONS(1232),
    [anon_sym_case] = ACTIONS(1232),
    [anon_sym_default] = ACTIONS(1232),
    [anon_sym_while] = ACTIONS(1232),
    [anon_sym_do] = ACTIONS(1232),
    [anon_sym_for] = ACTIONS(1232),
    [anon_sym_return] = ACTIONS(1232),
    [anon_sym_break] = ACTIONS(1232),
    [anon_sym_continue] = ACTIONS(1232),
    [anon_sym_goto] = ACTIONS(1232),
    [anon_sym___try] = ACTIONS(1232),
    [anon_sym___leave] = ACTIONS(1232),
    [anon_sym_DASH_DASH] = ACTIONS(1234),
    [anon_sym_PLUS_PLUS] = ACTIONS(1234),
    [anon_sym_sizeof] = ACTIONS(1232),
    [anon_sym___alignof__] = ACTIONS(1232),
    [anon_sym___alignof] = ACTIONS(1232),
    [anon_sym__alignof] = ACTIONS(1232),
    [anon_sym_alignof] = ACTIONS(1232),
    [anon_sym__Alignof] = ACTIONS(1232),
    [anon_sym_offsetof] = ACTIONS(1232),
    [anon_sym__Generic] = ACTIONS(1232),
    [anon_sym_asm] = ACTIONS(1232),
    [anon_sym___asm__] = ACTIONS(1232),
    [sym_number_literal] = ACTIONS(1234),
    [anon_sym_L_SQUOTE] = ACTIONS(1234),
    [anon_sym_u_SQUOTE] = ACTIONS(1234),
    [anon_sym_U_SQUOTE] = ACTIONS(1234),
    [anon_sym_u8_SQUOTE] = ACTIONS(1234),
    [anon_sym_SQUOTE] = ACTIONS(1234),
    [anon_sym_L_DQUOTE] = ACTIONS(1234),
    [anon_sym_u_DQUOTE] = ACTIONS(1234),
    [anon_sym_U_DQUOTE] = ACTIONS(1234),
    [anon_sym_u8_DQUOTE] = ACTIONS(1234),
    [anon_sym_DQUOTE] = ACTIONS(1234),
    [sym_true] = ACTIONS(1232),
    [sym_false] = ACTIONS(1232),
    [anon_sym_NULL] = ACTIONS(1232),
    [anon_sym_nullptr] = ACTIONS(1232),
    [sym_comment] = ACTIONS(3),
  },
  [169] = {
    [sym_identifier] = ACTIONS(1228),
    [aux_sym_preproc_include_token1] = ACTIONS(1228),
    [aux_sym_preproc_def_token1] = ACTIONS(1228),
    [aux_sym_preproc_if_token1] = ACTIONS(1228),
    [aux_sym_preproc_if_token2] = ACTIONS(1228),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1228),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1228),
    [sym_preproc_directive] = ACTIONS(1228),
    [anon_sym_LPAREN2] = ACTIONS(1230),
    [anon_sym_BANG] = ACTIONS(1230),
    [anon_sym_TILDE] = ACTIONS(1230),
    [anon_sym_DASH] = ACTIONS(1228),
    [anon_sym_PLUS] = ACTIONS(1228),
    [anon_sym_STAR] = ACTIONS(1230),
    [anon_sym_AMP] = ACTIONS(1230),
    [anon_sym_SEMI] = ACTIONS(1230),
    [anon_sym___extension__] = ACTIONS(1228),
    [anon_sym_typedef] = ACTIONS(1228),
    [anon_sym_extern] = ACTIONS(1228),
    [anon_sym___attribute__] = ACTIONS(1228),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1230),
    [anon_sym___declspec] = ACTIONS(1228),
    [anon_sym___cdecl] = ACTIONS(1228),
    [anon_sym___clrcall] = ACTIONS(1228),
    [anon_sym___stdcall] = ACTIONS(1228),
    [anon_sym___fastcall] = ACTIONS(1228),
    [anon_sym___thiscall] = ACTIONS(1228),
    [anon_sym___vectorcall] = ACTIONS(1228),
    [anon_sym_LBRACE] = ACTIONS(1230),
    [anon_sym_signed] = ACTIONS(1228),
    [anon_sym_unsigned] = ACTIONS(1228),
    [anon_sym_long] = ACTIONS(1228),
    [anon_sym_short] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_register] = ACTIONS(1228),
    [anon_sym_inline] = ACTIONS(1228),
    [anon_sym___inline] = ACTIONS(1228),
    [anon_sym___inline__] = ACTIONS(1228),
    [anon_sym___forceinline] = ACTIONS(1228),
    [anon_sym_thread_local] = ACTIONS(1228),
    [anon_sym___thread] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_constexpr] = ACTIONS(1228),
    [anon_sym_volatile] = ACTIONS(1228),
    [anon_sym_restrict] = ACTIONS(1228),
    [anon_sym___restrict__] = ACTIONS(1228),
    [anon_sym__Atomic] = ACTIONS(1228),
    [anon_sym__Noreturn] = ACTIONS(1228),
    [anon_sym_noreturn] = ACTIONS(1228),
    [sym_primitive_type] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_else] = ACTIONS(1228),
    [anon_sym_switch] = ACTIONS(1228),
    [anon_sym_case] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [anon_sym_do] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_goto] = ACTIONS(1228),
    [anon_sym___try] = ACTIONS(1228),
    [anon_sym___leave] = ACTIONS(1228),
    [anon_sym_DASH_DASH] = ACTIONS(1230),
    [anon_sym_PLUS_PLUS] = ACTIONS(1230),
    [anon_sym_sizeof] = ACTIONS(1228),
    [anon_sym___alignof__] = ACTIONS(1228),
    [anon_sym___alignof] = ACTIONS(1228),
    [anon_sym__alignof] = ACTIONS(1228),
    [anon_sym_alignof] = ACTIONS(1228),
    [anon_sym__Alignof] = ACTIONS(1228),
    [anon_sym_offsetof] = ACTIONS(1228),
    [anon_sym__Generic] = ACTIONS(1228),
    [anon_sym_asm] = ACTIONS(1228),
    [anon_sym___asm__] = ACTIONS(1228),
    [sym_number_literal] = ACTIONS(1230),
    [anon_sym_L_SQUOTE] = ACTIONS(1230),
    [anon_sym_u_SQUOTE] = ACTIONS(1230),
    [anon_sym_U_SQUOTE] = ACTIONS(1230),
    [anon_sym_u8_SQUOTE] = ACTIONS(1230),
    [anon_sym_SQUOTE] = ACTIONS(1230),
    [anon_sym_L_DQUOTE] = ACTIONS(1230),
    [anon_sym_u_DQUOTE] = ACTIONS(1230),
    [anon_sym_U_DQUOTE] = ACTIONS(1230),
    [anon_sym_u8_DQUOTE] = ACTIONS(1230),
    [anon_sym_DQUOTE] = ACTIONS(1230),
    [sym_true] = ACTIONS(1228),
    [sym_false] = ACTIONS(1228),
    [anon_sym_NULL] = ACTIONS(1228),
    [anon_sym_nullptr] = ACTIONS(1228),
    [sym_comment] = ACTIONS(3),
  },
  [170] = {
    [sym_identifier] = ACTIONS(1112),
    [aux_sym_preproc_include_token1] = ACTIONS(1112),
    [aux_sym_preproc_def_token1] = ACTIONS(1112),
    [aux_sym_preproc_if_token1] = ACTIONS(1112),
    [aux_sym_preproc_if_token2] = ACTIONS(1112),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1112),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1112),
    [sym_preproc_directive] = ACTIONS(1112),
    [anon_sym_LPAREN2] = ACTIONS(1114),
    [anon_sym_BANG] = ACTIONS(1114),
    [anon_sym_TILDE] = ACTIONS(1114),
    [anon_sym_DASH] = ACTIONS(1112),
    [anon_sym_PLUS] = ACTIONS(1112),
    [anon_sym_STAR] = ACTIONS(1114),
    [anon_sym_AMP] = ACTIONS(1114),
    [anon_sym_SEMI] = ACTIONS(1114),
    [anon_sym___extension__] = ACTIONS(1112),
    [anon_sym_typedef] = ACTIONS(1112),
    [anon_sym_extern] = ACTIONS(1112),
    [anon_sym___attribute__] = ACTIONS(1112),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1114),
    [anon_sym___declspec] = ACTIONS(1112),
    [anon_sym___cdecl] = ACTIONS(1112),
    [anon_sym___clrcall] = ACTIONS(1112),
    [anon_sym___stdcall] = ACTIONS(1112),
    [anon_sym___fastcall] = ACTIONS(1112),
    [anon_sym___thiscall] = ACTIONS(1112),
    [anon_sym___vectorcall] = ACTIONS(1112),
    [anon_sym_LBRACE] = ACTIONS(1114),
    [anon_sym_signed] = ACTIONS(1112),
    [anon_sym_unsigned] = ACTIONS(1112),
    [anon_sym_long] = ACTIONS(1112),
    [anon_sym_short] = ACTIONS(1112),
    [anon_sym_static] = ACTIONS(1112),
    [anon_sym_auto] = ACTIONS(1112),
    [anon_sym_register] = ACTIONS(1112),
    [anon_sym_inline] = ACTIONS(1112),
    [anon_sym___inline] = ACTIONS(1112),
    [anon_sym___inline__] = ACTIONS(1112),
    [anon_sym___forceinline] = ACTIONS(1112),
    [anon_sym_thread_local] = ACTIONS(1112),
    [anon_sym___thread] = ACTIONS(1112),
    [anon_sym_const] = ACTIONS(1112),
    [anon_sym_constexpr] = ACTIONS(1112),
    [anon_sym_volatile] = ACTIONS(1112),
    [anon_sym_restrict] = ACTIONS(1112),
    [anon_sym___restrict__] = ACTIONS(1112),
    [anon_sym__Atomic] = ACTIONS(1112),
    [anon_sym__Noreturn] = ACTIONS(1112),
    [anon_sym_noreturn] = ACTIONS(1112),
    [sym_primitive_type] = ACTIONS(1112),
    [anon_sym_enum] = ACTIONS(1112),
    [anon_sym_struct] = ACTIONS(1112),
    [anon_sym_union] = ACTIONS(1112),
    [anon_sym_if] = ACTIONS(1112),
    [anon_sym_else] = ACTIONS(1112),
    [anon_sym_switch] = ACTIONS(1112),
    [anon_sym_case] = ACTIONS(1112),
    [anon_sym_default] = ACTIONS(1112),
    [anon_sym_while] = ACTIONS(1112),
    [anon_sym_do] = ACTIONS(1112),
    [anon_sym_for] = ACTIONS(1112),
    [anon_sym_return] = ACTIONS(1112),
    [anon_sym_break] = ACTIONS(1112),
    [anon_sym_continue] = ACTIONS(1112),
    [anon_sym_goto] = ACTIONS(1112),
    [anon_sym___try] = ACTIONS(1112),
    [anon_sym___leave] = ACTIONS(1112),
    [anon_sym_DASH_DASH] = ACTIONS(1114),
    [anon_sym_PLUS_PLUS] = ACTIONS(1114),
    [anon_sym_sizeof] = ACTIONS(1112),
    [anon_sym___alignof__] = ACTIONS(1112),
    [anon_sym___alignof] = ACTIONS(1112),
    [anon_sym__alignof] = ACTIONS(1112),
    [anon_sym_alignof] = ACTIONS(1112),
    [anon_sym__Alignof] = ACTIONS(1112),
    [anon_sym_offsetof] = ACTIONS(1112),
    [anon_sym__Generic] = ACTIONS(1112),
    [anon_sym_asm] = ACTIONS(1112),
    [anon_sym___asm__] = ACTIONS(1112),
    [sym_number_literal] = ACTIONS(1114),
    [anon_sym_L_SQUOTE] = ACTIONS(1114),
    [anon_sym_u_SQUOTE] = ACTIONS(1114),
    [anon_sym_U_SQUOTE] = ACTIONS(1114),
    [anon_sym_u8_SQUOTE] = ACTIONS(1114),
    [anon_sym_SQUOTE] = ACTIONS(1114),
    [anon_sym_L_DQUOTE] = ACTIONS(1114),
    [anon_sym_u_DQUOTE] = ACTIONS(1114),
    [anon_sym_U_DQUOTE] = ACTIONS(1114),
    [anon_sym_u8_DQUOTE] = ACTIONS(1114),
    [anon_sym_DQUOTE] = ACTIONS(1114),
    [sym_true] = ACTIONS(1112),
    [sym_false] = ACTIONS(1112),
    [anon_sym_NULL] = ACTIONS(1112),
    [anon_sym_nullptr] = ACTIONS(1112),
    [sym_comment] = ACTIONS(3),
  },
  [171] = {
    [sym_identifier] = ACTIONS(1112),
    [aux_sym_preproc_include_token1] = ACTIONS(1112),
    [aux_sym_preproc_def_token1] = ACTIONS(1112),
    [aux_sym_preproc_if_token1] = ACTIONS(1112),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1112),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1112),
    [sym_preproc_directive] = ACTIONS(1112),
    [anon_sym_LPAREN2] = ACTIONS(1114),
    [anon_sym_BANG] = ACTIONS(1114),
    [anon_sym_TILDE] = ACTIONS(1114),
    [anon_sym_DASH] = ACTIONS(1112),
    [anon_sym_PLUS] = ACTIONS(1112),
    [anon_sym_STAR] = ACTIONS(1114),
    [anon_sym_AMP] = ACTIONS(1114),
    [anon_sym_SEMI] = ACTIONS(1114),
    [anon_sym___extension__] = ACTIONS(1112),
    [anon_sym_typedef] = ACTIONS(1112),
    [anon_sym_extern] = ACTIONS(1112),
    [anon_sym___attribute__] = ACTIONS(1112),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1114),
    [anon_sym___declspec] = ACTIONS(1112),
    [anon_sym___cdecl] = ACTIONS(1112),
    [anon_sym___clrcall] = ACTIONS(1112),
    [anon_sym___stdcall] = ACTIONS(1112),
    [anon_sym___fastcall] = ACTIONS(1112),
    [anon_sym___thiscall] = ACTIONS(1112),
    [anon_sym___vectorcall] = ACTIONS(1112),
    [anon_sym_LBRACE] = ACTIONS(1114),
    [anon_sym_RBRACE] = ACTIONS(1114),
    [anon_sym_signed] = ACTIONS(1112),
    [anon_sym_unsigned] = ACTIONS(1112),
    [anon_sym_long] = ACTIONS(1112),
    [anon_sym_short] = ACTIONS(1112),
    [anon_sym_static] = ACTIONS(1112),
    [anon_sym_auto] = ACTIONS(1112),
    [anon_sym_register] = ACTIONS(1112),
    [anon_sym_inline] = ACTIONS(1112),
    [anon_sym___inline] = ACTIONS(1112),
    [anon_sym___inline__] = ACTIONS(1112),
    [anon_sym___forceinline] = ACTIONS(1112),
    [anon_sym_thread_local] = ACTIONS(1112),
    [anon_sym___thread] = ACTIONS(1112),
    [anon_sym_const] = ACTIONS(1112),
    [anon_sym_constexpr] = ACTIONS(1112),
    [anon_sym_volatile] = ACTIONS(1112),
    [anon_sym_restrict] = ACTIONS(1112),
    [anon_sym___restrict__] = ACTIONS(1112),
    [anon_sym__Atomic] = ACTIONS(1112),
    [anon_sym__Noreturn] = ACTIONS(1112),
    [anon_sym_noreturn] = ACTIONS(1112),
    [sym_primitive_type] = ACTIONS(1112),
    [anon_sym_enum] = ACTIONS(1112),
    [anon_sym_struct] = ACTIONS(1112),
    [anon_sym_union] = ACTIONS(1112),
    [anon_sym_if] = ACTIONS(1112),
    [anon_sym_else] = ACTIONS(1112),
    [anon_sym_switch] = ACTIONS(1112),
    [anon_sym_case] = ACTIONS(1112),
    [anon_sym_default] = ACTIONS(1112),
    [anon_sym_while] = ACTIONS(1112),
    [anon_sym_do] = ACTIONS(1112),
    [anon_sym_for] = ACTIONS(1112),
    [anon_sym_return] = ACTIONS(1112),
    [anon_sym_break] = ACTIONS(1112),
    [anon_sym_continue] = ACTIONS(1112),
    [anon_sym_goto] = ACTIONS(1112),
    [anon_sym___try] = ACTIONS(1112),
    [anon_sym___leave] = ACTIONS(1112),
    [anon_sym_DASH_DASH] = ACTIONS(1114),
    [anon_sym_PLUS_PLUS] = ACTIONS(1114),
    [anon_sym_sizeof] = ACTIONS(1112),
    [anon_sym___alignof__] = ACTIONS(1112),
    [anon_sym___alignof] = ACTIONS(1112),
    [anon_sym__alignof] = ACTIONS(1112),
    [anon_sym_alignof] = ACTIONS(1112),
    [anon_sym__Alignof] = ACTIONS(1112),
    [anon_sym_offsetof] = ACTIONS(1112),
    [anon_sym__Generic] = ACTIONS(1112),
    [anon_sym_asm] = ACTIONS(1112),
    [anon_sym___asm__] = ACTIONS(1112),
    [sym_number_literal] = ACTIONS(1114),
    [anon_sym_L_SQUOTE] = ACTIONS(1114),
    [anon_sym_u_SQUOTE] = ACTIONS(1114),
    [anon_sym_U_SQUOTE] = ACTIONS(1114),
    [anon_sym_u8_SQUOTE] = ACTIONS(1114),
    [anon_sym_SQUOTE] = ACTIONS(1114),
    [anon_sym_L_DQUOTE] = ACTIONS(1114),
    [anon_sym_u_DQUOTE] = ACTIONS(1114),
    [anon_sym_U_DQUOTE] = ACTIONS(1114),
    [anon_sym_u8_DQUOTE] = ACTIONS(1114),
    [anon_sym_DQUOTE] = ACTIONS(1114),
    [sym_true] = ACTIONS(1112),
    [sym_false] = ACTIONS(1112),
    [anon_sym_NULL] = ACTIONS(1112),
    [anon_sym_nullptr] = ACTIONS(1112),
    [sym_comment] = ACTIONS(3),
  },
  [172] = {
    [ts_builtin_sym_end] = ACTIONS(1202),
    [sym_identifier] = ACTIONS(1200),
    [aux_sym_preproc_include_token1] = ACTIONS(1200),
    [aux_sym_preproc_def_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1200),
    [sym_preproc_directive] = ACTIONS(1200),
    [anon_sym_LPAREN2] = ACTIONS(1202),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_DASH] = ACTIONS(1200),
    [anon_sym_PLUS] = ACTIONS(1200),
    [anon_sym_STAR] = ACTIONS(1202),
    [anon_sym_AMP] = ACTIONS(1202),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym___extension__] = ACTIONS(1200),
    [anon_sym_typedef] = ACTIONS(1200),
    [anon_sym_extern] = ACTIONS(1200),
    [anon_sym___attribute__] = ACTIONS(1200),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1202),
    [anon_sym___declspec] = ACTIONS(1200),
    [anon_sym___cdecl] = ACTIONS(1200),
    [anon_sym___clrcall] = ACTIONS(1200),
    [anon_sym___stdcall] = ACTIONS(1200),
    [anon_sym___fastcall] = ACTIONS(1200),
    [anon_sym___thiscall] = ACTIONS(1200),
    [anon_sym___vectorcall] = ACTIONS(1200),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_signed] = ACTIONS(1200),
    [anon_sym_unsigned] = ACTIONS(1200),
    [anon_sym_long] = ACTIONS(1200),
    [anon_sym_short] = ACTIONS(1200),
    [anon_sym_static] = ACTIONS(1200),
    [anon_sym_auto] = ACTIONS(1200),
    [anon_sym_register] = ACTIONS(1200),
    [anon_sym_inline] = ACTIONS(1200),
    [anon_sym___inline] = ACTIONS(1200),
    [anon_sym___inline__] = ACTIONS(1200),
    [anon_sym___forceinline] = ACTIONS(1200),
    [anon_sym_thread_local] = ACTIONS(1200),
    [anon_sym___thread] = ACTIONS(1200),
    [anon_sym_const] = ACTIONS(1200),
    [anon_sym_constexpr] = ACTIONS(1200),
    [anon_sym_volatile] = ACTIONS(1200),
    [anon_sym_restrict] = ACTIONS(1200),
    [anon_sym___restrict__] = ACTIONS(1200),
    [anon_sym__Atomic] = ACTIONS(1200),
    [anon_sym__Noreturn] = ACTIONS(1200),
    [anon_sym_noreturn] = ACTIONS(1200),
    [sym_primitive_type] = ACTIONS(1200),
    [anon_sym_enum] = ACTIONS(1200),
    [anon_sym_struct] = ACTIONS(1200),
    [anon_sym_union] = ACTIONS(1200),
    [anon_sym_if] = ACTIONS(1200),
    [anon_sym_else] = ACTIONS(1200),
    [anon_sym_switch] = ACTIONS(1200),
    [anon_sym_case] = ACTIONS(1200),
    [anon_sym_default] = ACTIONS(1200),
    [anon_sym_while] = ACTIONS(1200),
    [anon_sym_do] = ACTIONS(1200),
    [anon_sym_for] = ACTIONS(1200),
    [anon_sym_return] = ACTIONS(1200),
    [anon_sym_break] = ACTIONS(1200),
    [anon_sym_continue] = ACTIONS(1200),
    [anon_sym_goto] = ACTIONS(1200),
    [anon_sym___try] = ACTIONS(1200),
    [anon_sym___leave] = ACTIONS(1200),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_sizeof] = ACTIONS(1200),
    [anon_sym___alignof__] = ACTIONS(1200),
    [anon_sym___alignof] = ACTIONS(1200),
    [anon_sym__alignof] = ACTIONS(1200),
    [anon_sym_alignof] = ACTIONS(1200),
    [anon_sym__Alignof] = ACTIONS(1200),
    [anon_sym_offsetof] = ACTIONS(1200),
    [anon_sym__Generic] = ACTIONS(1200),
    [anon_sym_asm] = ACTIONS(1200),
    [anon_sym___asm__] = ACTIONS(1200),
    [sym_number_literal] = ACTIONS(1202),
    [anon_sym_L_SQUOTE] = ACTIONS(1202),
    [anon_sym_u_SQUOTE] = ACTIONS(1202),
    [anon_sym_U_SQUOTE] = ACTIONS(1202),
    [anon_sym_u8_SQUOTE] = ACTIONS(1202),
    [anon_sym_SQUOTE] = ACTIONS(1202),
    [anon_sym_L_DQUOTE] = ACTIONS(1202),
    [anon_sym_u_DQUOTE] = ACTIONS(1202),
    [anon_sym_U_DQUOTE] = ACTIONS(1202),
    [anon_sym_u8_DQUOTE] = ACTIONS(1202),
    [anon_sym_DQUOTE] = ACTIONS(1202),
    [sym_true] = ACTIONS(1200),
    [sym_false] = ACTIONS(1200),
    [anon_sym_NULL] = ACTIONS(1200),
    [anon_sym_nullptr] = ACTIONS(1200),
    [sym_comment] = ACTIONS(3),
  },
  [173] = {
    [sym_identifier] = ACTIONS(1220),
    [aux_sym_preproc_include_token1] = ACTIONS(1220),
    [aux_sym_preproc_def_token1] = ACTIONS(1220),
    [aux_sym_preproc_if_token1] = ACTIONS(1220),
    [aux_sym_preproc_if_token2] = ACTIONS(1220),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1220),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1220),
    [sym_preproc_directive] = ACTIONS(1220),
    [anon_sym_LPAREN2] = ACTIONS(1222),
    [anon_sym_BANG] = ACTIONS(1222),
    [anon_sym_TILDE] = ACTIONS(1222),
    [anon_sym_DASH] = ACTIONS(1220),
    [anon_sym_PLUS] = ACTIONS(1220),
    [anon_sym_STAR] = ACTIONS(1222),
    [anon_sym_AMP] = ACTIONS(1222),
    [anon_sym_SEMI] = ACTIONS(1222),
    [anon_sym___extension__] = ACTIONS(1220),
    [anon_sym_typedef] = ACTIONS(1220),
    [anon_sym_extern] = ACTIONS(1220),
    [anon_sym___attribute__] = ACTIONS(1220),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1222),
    [anon_sym___declspec] = ACTIONS(1220),
    [anon_sym___cdecl] = ACTIONS(1220),
    [anon_sym___clrcall] = ACTIONS(1220),
    [anon_sym___stdcall] = ACTIONS(1220),
    [anon_sym___fastcall] = ACTIONS(1220),
    [anon_sym___thiscall] = ACTIONS(1220),
    [anon_sym___vectorcall] = ACTIONS(1220),
    [anon_sym_LBRACE] = ACTIONS(1222),
    [anon_sym_signed] = ACTIONS(1220),
    [anon_sym_unsigned] = ACTIONS(1220),
    [anon_sym_long] = ACTIONS(1220),
    [anon_sym_short] = ACTIONS(1220),
    [anon_sym_static] = ACTIONS(1220),
    [anon_sym_auto] = ACTIONS(1220),
    [anon_sym_register] = ACTIONS(1220),
    [anon_sym_inline] = ACTIONS(1220),
    [anon_sym___inline] = ACTIONS(1220),
    [anon_sym___inline__] = ACTIONS(1220),
    [anon_sym___forceinline] = ACTIONS(1220),
    [anon_sym_thread_local] = ACTIONS(1220),
    [anon_sym___thread] = ACTIONS(1220),
    [anon_sym_const] = ACTIONS(1220),
    [anon_sym_constexpr] = ACTIONS(1220),
    [anon_sym_volatile] = ACTIONS(1220),
    [anon_sym_restrict] = ACTIONS(1220),
    [anon_sym___restrict__] = ACTIONS(1220),
    [anon_sym__Atomic] = ACTIONS(1220),
    [anon_sym__Noreturn] = ACTIONS(1220),
    [anon_sym_noreturn] = ACTIONS(1220),
    [sym_primitive_type] = ACTIONS(1220),
    [anon_sym_enum] = ACTIONS(1220),
    [anon_sym_struct] = ACTIONS(1220),
    [anon_sym_union] = ACTIONS(1220),
    [anon_sym_if] = ACTIONS(1220),
    [anon_sym_else] = ACTIONS(1220),
    [anon_sym_switch] = ACTIONS(1220),
    [anon_sym_case] = ACTIONS(1220),
    [anon_sym_default] = ACTIONS(1220),
    [anon_sym_while] = ACTIONS(1220),
    [anon_sym_do] = ACTIONS(1220),
    [anon_sym_for] = ACTIONS(1220),
    [anon_sym_return] = ACTIONS(1220),
    [anon_sym_break] = ACTIONS(1220),
    [anon_sym_continue] = ACTIONS(1220),
    [anon_sym_goto] = ACTIONS(1220),
    [anon_sym___try] = ACTIONS(1220),
    [anon_sym___leave] = ACTIONS(1220),
    [anon_sym_DASH_DASH] = ACTIONS(1222),
    [anon_sym_PLUS_PLUS] = ACTIONS(1222),
    [anon_sym_sizeof] = ACTIONS(1220),
    [anon_sym___alignof__] = ACTIONS(1220),
    [anon_sym___alignof] = ACTIONS(1220),
    [anon_sym__alignof] = ACTIONS(1220),
    [anon_sym_alignof] = ACTIONS(1220),
    [anon_sym__Alignof] = ACTIONS(1220),
    [anon_sym_offsetof] = ACTIONS(1220),
    [anon_sym__Generic] = ACTIONS(1220),
    [anon_sym_asm] = ACTIONS(1220),
    [anon_sym___asm__] = ACTIONS(1220),
    [sym_number_literal] = ACTIONS(1222),
    [anon_sym_L_SQUOTE] = ACTIONS(1222),
    [anon_sym_u_SQUOTE] = ACTIONS(1222),
    [anon_sym_U_SQUOTE] = ACTIONS(1222),
    [anon_sym_u8_SQUOTE] = ACTIONS(1222),
    [anon_sym_SQUOTE] = ACTIONS(1222),
    [anon_sym_L_DQUOTE] = ACTIONS(1222),
    [anon_sym_u_DQUOTE] = ACTIONS(1222),
    [anon_sym_U_DQUOTE] = ACTIONS(1222),
    [anon_sym_u8_DQUOTE] = ACTIONS(1222),
    [anon_sym_DQUOTE] = ACTIONS(1222),
    [sym_true] = ACTIONS(1220),
    [sym_false] = ACTIONS(1220),
    [anon_sym_NULL] = ACTIONS(1220),
    [anon_sym_nullptr] = ACTIONS(1220),
    [sym_comment] = ACTIONS(3),
  },
  [174] = {
    [sym_identifier] = ACTIONS(1216),
    [aux_sym_preproc_include_token1] = ACTIONS(1216),
    [aux_sym_preproc_def_token1] = ACTIONS(1216),
    [aux_sym_preproc_if_token1] = ACTIONS(1216),
    [aux_sym_preproc_if_token2] = ACTIONS(1216),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1216),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1216),
    [sym_preproc_directive] = ACTIONS(1216),
    [anon_sym_LPAREN2] = ACTIONS(1218),
    [anon_sym_BANG] = ACTIONS(1218),
    [anon_sym_TILDE] = ACTIONS(1218),
    [anon_sym_DASH] = ACTIONS(1216),
    [anon_sym_PLUS] = ACTIONS(1216),
    [anon_sym_STAR] = ACTIONS(1218),
    [anon_sym_AMP] = ACTIONS(1218),
    [anon_sym_SEMI] = ACTIONS(1218),
    [anon_sym___extension__] = ACTIONS(1216),
    [anon_sym_typedef] = ACTIONS(1216),
    [anon_sym_extern] = ACTIONS(1216),
    [anon_sym___attribute__] = ACTIONS(1216),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1218),
    [anon_sym___declspec] = ACTIONS(1216),
    [anon_sym___cdecl] = ACTIONS(1216),
    [anon_sym___clrcall] = ACTIONS(1216),
    [anon_sym___stdcall] = ACTIONS(1216),
    [anon_sym___fastcall] = ACTIONS(1216),
    [anon_sym___thiscall] = ACTIONS(1216),
    [anon_sym___vectorcall] = ACTIONS(1216),
    [anon_sym_LBRACE] = ACTIONS(1218),
    [anon_sym_signed] = ACTIONS(1216),
    [anon_sym_unsigned] = ACTIONS(1216),
    [anon_sym_long] = ACTIONS(1216),
    [anon_sym_short] = ACTIONS(1216),
    [anon_sym_static] = ACTIONS(1216),
    [anon_sym_auto] = ACTIONS(1216),
    [anon_sym_register] = ACTIONS(1216),
    [anon_sym_inline] = ACTIONS(1216),
    [anon_sym___inline] = ACTIONS(1216),
    [anon_sym___inline__] = ACTIONS(1216),
    [anon_sym___forceinline] = ACTIONS(1216),
    [anon_sym_thread_local] = ACTIONS(1216),
    [anon_sym___thread] = ACTIONS(1216),
    [anon_sym_const] = ACTIONS(1216),
    [anon_sym_constexpr] = ACTIONS(1216),
    [anon_sym_volatile] = ACTIONS(1216),
    [anon_sym_restrict] = ACTIONS(1216),
    [anon_sym___restrict__] = ACTIONS(1216),
    [anon_sym__Atomic] = ACTIONS(1216),
    [anon_sym__Noreturn] = ACTIONS(1216),
    [anon_sym_noreturn] = ACTIONS(1216),
    [sym_primitive_type] = ACTIONS(1216),
    [anon_sym_enum] = ACTIONS(1216),
    [anon_sym_struct] = ACTIONS(1216),
    [anon_sym_union] = ACTIONS(1216),
    [anon_sym_if] = ACTIONS(1216),
    [anon_sym_else] = ACTIONS(1216),
    [anon_sym_switch] = ACTIONS(1216),
    [anon_sym_case] = ACTIONS(1216),
    [anon_sym_default] = ACTIONS(1216),
    [anon_sym_while] = ACTIONS(1216),
    [anon_sym_do] = ACTIONS(1216),
    [anon_sym_for] = ACTIONS(1216),
    [anon_sym_return] = ACTIONS(1216),
    [anon_sym_break] = ACTIONS(1216),
    [anon_sym_continue] = ACTIONS(1216),
    [anon_sym_goto] = ACTIONS(1216),
    [anon_sym___try] = ACTIONS(1216),
    [anon_sym___leave] = ACTIONS(1216),
    [anon_sym_DASH_DASH] = ACTIONS(1218),
    [anon_sym_PLUS_PLUS] = ACTIONS(1218),
    [anon_sym_sizeof] = ACTIONS(1216),
    [anon_sym___alignof__] = ACTIONS(1216),
    [anon_sym___alignof] = ACTIONS(1216),
    [anon_sym__alignof] = ACTIONS(1216),
    [anon_sym_alignof] = ACTIONS(1216),
    [anon_sym__Alignof] = ACTIONS(1216),
    [anon_sym_offsetof] = ACTIONS(1216),
    [anon_sym__Generic] = ACTIONS(1216),
    [anon_sym_asm] = ACTIONS(1216),
    [anon_sym___asm__] = ACTIONS(1216),
    [sym_number_literal] = ACTIONS(1218),
    [anon_sym_L_SQUOTE] = ACTIONS(1218),
    [anon_sym_u_SQUOTE] = ACTIONS(1218),
    [anon_sym_U_SQUOTE] = ACTIONS(1218),
    [anon_sym_u8_SQUOTE] = ACTIONS(1218),
    [anon_sym_SQUOTE] = ACTIONS(1218),
    [anon_sym_L_DQUOTE] = ACTIONS(1218),
    [anon_sym_u_DQUOTE] = ACTIONS(1218),
    [anon_sym_U_DQUOTE] = ACTIONS(1218),
    [anon_sym_u8_DQUOTE] = ACTIONS(1218),
    [anon_sym_DQUOTE] = ACTIONS(1218),
    [sym_true] = ACTIONS(1216),
    [sym_false] = ACTIONS(1216),
    [anon_sym_NULL] = ACTIONS(1216),
    [anon_sym_nullptr] = ACTIONS(1216),
    [sym_comment] = ACTIONS(3),
  },
  [175] = {
    [ts_builtin_sym_end] = ACTIONS(1222),
    [sym_identifier] = ACTIONS(1220),
    [aux_sym_preproc_include_token1] = ACTIONS(1220),
    [aux_sym_preproc_def_token1] = ACTIONS(1220),
    [aux_sym_preproc_if_token1] = ACTIONS(1220),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1220),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1220),
    [sym_preproc_directive] = ACTIONS(1220),
    [anon_sym_LPAREN2] = ACTIONS(1222),
    [anon_sym_BANG] = ACTIONS(1222),
    [anon_sym_TILDE] = ACTIONS(1222),
    [anon_sym_DASH] = ACTIONS(1220),
    [anon_sym_PLUS] = ACTIONS(1220),
    [anon_sym_STAR] = ACTIONS(1222),
    [anon_sym_AMP] = ACTIONS(1222),
    [anon_sym_SEMI] = ACTIONS(1222),
    [anon_sym___extension__] = ACTIONS(1220),
    [anon_sym_typedef] = ACTIONS(1220),
    [anon_sym_extern] = ACTIONS(1220),
    [anon_sym___attribute__] = ACTIONS(1220),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1222),
    [anon_sym___declspec] = ACTIONS(1220),
    [anon_sym___cdecl] = ACTIONS(1220),
    [anon_sym___clrcall] = ACTIONS(1220),
    [anon_sym___stdcall] = ACTIONS(1220),
    [anon_sym___fastcall] = ACTIONS(1220),
    [anon_sym___thiscall] = ACTIONS(1220),
    [anon_sym___vectorcall] = ACTIONS(1220),
    [anon_sym_LBRACE] = ACTIONS(1222),
    [anon_sym_signed] = ACTIONS(1220),
    [anon_sym_unsigned] = ACTIONS(1220),
    [anon_sym_long] = ACTIONS(1220),
    [anon_sym_short] = ACTIONS(1220),
    [anon_sym_static] = ACTIONS(1220),
    [anon_sym_auto] = ACTIONS(1220),
    [anon_sym_register] = ACTIONS(1220),
    [anon_sym_inline] = ACTIONS(1220),
    [anon_sym___inline] = ACTIONS(1220),
    [anon_sym___inline__] = ACTIONS(1220),
    [anon_sym___forceinline] = ACTIONS(1220),
    [anon_sym_thread_local] = ACTIONS(1220),
    [anon_sym___thread] = ACTIONS(1220),
    [anon_sym_const] = ACTIONS(1220),
    [anon_sym_constexpr] = ACTIONS(1220),
    [anon_sym_volatile] = ACTIONS(1220),
    [anon_sym_restrict] = ACTIONS(1220),
    [anon_sym___restrict__] = ACTIONS(1220),
    [anon_sym__Atomic] = ACTIONS(1220),
    [anon_sym__Noreturn] = ACTIONS(1220),
    [anon_sym_noreturn] = ACTIONS(1220),
    [sym_primitive_type] = ACTIONS(1220),
    [anon_sym_enum] = ACTIONS(1220),
    [anon_sym_struct] = ACTIONS(1220),
    [anon_sym_union] = ACTIONS(1220),
    [anon_sym_if] = ACTIONS(1220),
    [anon_sym_else] = ACTIONS(1220),
    [anon_sym_switch] = ACTIONS(1220),
    [anon_sym_case] = ACTIONS(1220),
    [anon_sym_default] = ACTIONS(1220),
    [anon_sym_while] = ACTIONS(1220),
    [anon_sym_do] = ACTIONS(1220),
    [anon_sym_for] = ACTIONS(1220),
    [anon_sym_return] = ACTIONS(1220),
    [anon_sym_break] = ACTIONS(1220),
    [anon_sym_continue] = ACTIONS(1220),
    [anon_sym_goto] = ACTIONS(1220),
    [anon_sym___try] = ACTIONS(1220),
    [anon_sym___leave] = ACTIONS(1220),
    [anon_sym_DASH_DASH] = ACTIONS(1222),
    [anon_sym_PLUS_PLUS] = ACTIONS(1222),
    [anon_sym_sizeof] = ACTIONS(1220),
    [anon_sym___alignof__] = ACTIONS(1220),
    [anon_sym___alignof] = ACTIONS(1220),
    [anon_sym__alignof] = ACTIONS(1220),
    [anon_sym_alignof] = ACTIONS(1220),
    [anon_sym__Alignof] = ACTIONS(1220),
    [anon_sym_offsetof] = ACTIONS(1220),
    [anon_sym__Generic] = ACTIONS(1220),
    [anon_sym_asm] = ACTIONS(1220),
    [anon_sym___asm__] = ACTIONS(1220),
    [sym_number_literal] = ACTIONS(1222),
    [anon_sym_L_SQUOTE] = ACTIONS(1222),
    [anon_sym_u_SQUOTE] = ACTIONS(1222),
    [anon_sym_U_SQUOTE] = ACTIONS(1222),
    [anon_sym_u8_SQUOTE] = ACTIONS(1222),
    [anon_sym_SQUOTE] = ACTIONS(1222),
    [anon_sym_L_DQUOTE] = ACTIONS(1222),
    [anon_sym_u_DQUOTE] = ACTIONS(1222),
    [anon_sym_U_DQUOTE] = ACTIONS(1222),
    [anon_sym_u8_DQUOTE] = ACTIONS(1222),
    [anon_sym_DQUOTE] = ACTIONS(1222),
    [sym_true] = ACTIONS(1220),
    [sym_false] = ACTIONS(1220),
    [anon_sym_NULL] = ACTIONS(1220),
    [anon_sym_nullptr] = ACTIONS(1220),
    [sym_comment] = ACTIONS(3),
  },
  [176] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [177] = {
    [ts_builtin_sym_end] = ACTIONS(1202),
    [sym_identifier] = ACTIONS(1200),
    [aux_sym_preproc_include_token1] = ACTIONS(1200),
    [aux_sym_preproc_def_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1200),
    [sym_preproc_directive] = ACTIONS(1200),
    [anon_sym_LPAREN2] = ACTIONS(1202),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_DASH] = ACTIONS(1200),
    [anon_sym_PLUS] = ACTIONS(1200),
    [anon_sym_STAR] = ACTIONS(1202),
    [anon_sym_AMP] = ACTIONS(1202),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym___extension__] = ACTIONS(1200),
    [anon_sym_typedef] = ACTIONS(1200),
    [anon_sym_extern] = ACTIONS(1200),
    [anon_sym___attribute__] = ACTIONS(1200),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1202),
    [anon_sym___declspec] = ACTIONS(1200),
    [anon_sym___cdecl] = ACTIONS(1200),
    [anon_sym___clrcall] = ACTIONS(1200),
    [anon_sym___stdcall] = ACTIONS(1200),
    [anon_sym___fastcall] = ACTIONS(1200),
    [anon_sym___thiscall] = ACTIONS(1200),
    [anon_sym___vectorcall] = ACTIONS(1200),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_signed] = ACTIONS(1200),
    [anon_sym_unsigned] = ACTIONS(1200),
    [anon_sym_long] = ACTIONS(1200),
    [anon_sym_short] = ACTIONS(1200),
    [anon_sym_static] = ACTIONS(1200),
    [anon_sym_auto] = ACTIONS(1200),
    [anon_sym_register] = ACTIONS(1200),
    [anon_sym_inline] = ACTIONS(1200),
    [anon_sym___inline] = ACTIONS(1200),
    [anon_sym___inline__] = ACTIONS(1200),
    [anon_sym___forceinline] = ACTIONS(1200),
    [anon_sym_thread_local] = ACTIONS(1200),
    [anon_sym___thread] = ACTIONS(1200),
    [anon_sym_const] = ACTIONS(1200),
    [anon_sym_constexpr] = ACTIONS(1200),
    [anon_sym_volatile] = ACTIONS(1200),
    [anon_sym_restrict] = ACTIONS(1200),
    [anon_sym___restrict__] = ACTIONS(1200),
    [anon_sym__Atomic] = ACTIONS(1200),
    [anon_sym__Noreturn] = ACTIONS(1200),
    [anon_sym_noreturn] = ACTIONS(1200),
    [sym_primitive_type] = ACTIONS(1200),
    [anon_sym_enum] = ACTIONS(1200),
    [anon_sym_struct] = ACTIONS(1200),
    [anon_sym_union] = ACTIONS(1200),
    [anon_sym_if] = ACTIONS(1200),
    [anon_sym_else] = ACTIONS(1200),
    [anon_sym_switch] = ACTIONS(1200),
    [anon_sym_case] = ACTIONS(1200),
    [anon_sym_default] = ACTIONS(1200),
    [anon_sym_while] = ACTIONS(1200),
    [anon_sym_do] = ACTIONS(1200),
    [anon_sym_for] = ACTIONS(1200),
    [anon_sym_return] = ACTIONS(1200),
    [anon_sym_break] = ACTIONS(1200),
    [anon_sym_continue] = ACTIONS(1200),
    [anon_sym_goto] = ACTIONS(1200),
    [anon_sym___try] = ACTIONS(1200),
    [anon_sym___leave] = ACTIONS(1200),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_sizeof] = ACTIONS(1200),
    [anon_sym___alignof__] = ACTIONS(1200),
    [anon_sym___alignof] = ACTIONS(1200),
    [anon_sym__alignof] = ACTIONS(1200),
    [anon_sym_alignof] = ACTIONS(1200),
    [anon_sym__Alignof] = ACTIONS(1200),
    [anon_sym_offsetof] = ACTIONS(1200),
    [anon_sym__Generic] = ACTIONS(1200),
    [anon_sym_asm] = ACTIONS(1200),
    [anon_sym___asm__] = ACTIONS(1200),
    [sym_number_literal] = ACTIONS(1202),
    [anon_sym_L_SQUOTE] = ACTIONS(1202),
    [anon_sym_u_SQUOTE] = ACTIONS(1202),
    [anon_sym_U_SQUOTE] = ACTIONS(1202),
    [anon_sym_u8_SQUOTE] = ACTIONS(1202),
    [anon_sym_SQUOTE] = ACTIONS(1202),
    [anon_sym_L_DQUOTE] = ACTIONS(1202),
    [anon_sym_u_DQUOTE] = ACTIONS(1202),
    [anon_sym_U_DQUOTE] = ACTIONS(1202),
    [anon_sym_u8_DQUOTE] = ACTIONS(1202),
    [anon_sym_DQUOTE] = ACTIONS(1202),
    [sym_true] = ACTIONS(1200),
    [sym_false] = ACTIONS(1200),
    [anon_sym_NULL] = ACTIONS(1200),
    [anon_sym_nullptr] = ACTIONS(1200),
    [sym_comment] = ACTIONS(3),
  },
  [178] = {
    [ts_builtin_sym_end] = ACTIONS(1178),
    [sym_identifier] = ACTIONS(1176),
    [aux_sym_preproc_include_token1] = ACTIONS(1176),
    [aux_sym_preproc_def_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1176),
    [sym_preproc_directive] = ACTIONS(1176),
    [anon_sym_LPAREN2] = ACTIONS(1178),
    [anon_sym_BANG] = ACTIONS(1178),
    [anon_sym_TILDE] = ACTIONS(1178),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_STAR] = ACTIONS(1178),
    [anon_sym_AMP] = ACTIONS(1178),
    [anon_sym_SEMI] = ACTIONS(1178),
    [anon_sym___extension__] = ACTIONS(1176),
    [anon_sym_typedef] = ACTIONS(1176),
    [anon_sym_extern] = ACTIONS(1176),
    [anon_sym___attribute__] = ACTIONS(1176),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1178),
    [anon_sym___declspec] = ACTIONS(1176),
    [anon_sym___cdecl] = ACTIONS(1176),
    [anon_sym___clrcall] = ACTIONS(1176),
    [anon_sym___stdcall] = ACTIONS(1176),
    [anon_sym___fastcall] = ACTIONS(1176),
    [anon_sym___thiscall] = ACTIONS(1176),
    [anon_sym___vectorcall] = ACTIONS(1176),
    [anon_sym_LBRACE] = ACTIONS(1178),
    [anon_sym_signed] = ACTIONS(1176),
    [anon_sym_unsigned] = ACTIONS(1176),
    [anon_sym_long] = ACTIONS(1176),
    [anon_sym_short] = ACTIONS(1176),
    [anon_sym_static] = ACTIONS(1176),
    [anon_sym_auto] = ACTIONS(1176),
    [anon_sym_register] = ACTIONS(1176),
    [anon_sym_inline] = ACTIONS(1176),
    [anon_sym___inline] = ACTIONS(1176),
    [anon_sym___inline__] = ACTIONS(1176),
    [anon_sym___forceinline] = ACTIONS(1176),
    [anon_sym_thread_local] = ACTIONS(1176),
    [anon_sym___thread] = ACTIONS(1176),
    [anon_sym_const] = ACTIONS(1176),
    [anon_sym_constexpr] = ACTIONS(1176),
    [anon_sym_volatile] = ACTIONS(1176),
    [anon_sym_restrict] = ACTIONS(1176),
    [anon_sym___restrict__] = ACTIONS(1176),
    [anon_sym__Atomic] = ACTIONS(1176),
    [anon_sym__Noreturn] = ACTIONS(1176),
    [anon_sym_noreturn] = ACTIONS(1176),
    [sym_primitive_type] = ACTIONS(1176),
    [anon_sym_enum] = ACTIONS(1176),
    [anon_sym_struct] = ACTIONS(1176),
    [anon_sym_union] = ACTIONS(1176),
    [anon_sym_if] = ACTIONS(1176),
    [anon_sym_else] = ACTIONS(1176),
    [anon_sym_switch] = ACTIONS(1176),
    [anon_sym_case] = ACTIONS(1176),
    [anon_sym_default] = ACTIONS(1176),
    [anon_sym_while] = ACTIONS(1176),
    [anon_sym_do] = ACTIONS(1176),
    [anon_sym_for] = ACTIONS(1176),
    [anon_sym_return] = ACTIONS(1176),
    [anon_sym_break] = ACTIONS(1176),
    [anon_sym_continue] = ACTIONS(1176),
    [anon_sym_goto] = ACTIONS(1176),
    [anon_sym___try] = ACTIONS(1176),
    [anon_sym___leave] = ACTIONS(1176),
    [anon_sym_DASH_DASH] = ACTIONS(1178),
    [anon_sym_PLUS_PLUS] = ACTIONS(1178),
    [anon_sym_sizeof] = ACTIONS(1176),
    [anon_sym___alignof__] = ACTIONS(1176),
    [anon_sym___alignof] = ACTIONS(1176),
    [anon_sym__alignof] = ACTIONS(1176),
    [anon_sym_alignof] = ACTIONS(1176),
    [anon_sym__Alignof] = ACTIONS(1176),
    [anon_sym_offsetof] = ACTIONS(1176),
    [anon_sym__Generic] = ACTIONS(1176),
    [anon_sym_asm] = ACTIONS(1176),
    [anon_sym___asm__] = ACTIONS(1176),
    [sym_number_literal] = ACTIONS(1178),
    [anon_sym_L_SQUOTE] = ACTIONS(1178),
    [anon_sym_u_SQUOTE] = ACTIONS(1178),
    [anon_sym_U_SQUOTE] = ACTIONS(1178),
    [anon_sym_u8_SQUOTE] = ACTIONS(1178),
    [anon_sym_SQUOTE] = ACTIONS(1178),
    [anon_sym_L_DQUOTE] = ACTIONS(1178),
    [anon_sym_u_DQUOTE] = ACTIONS(1178),
    [anon_sym_U_DQUOTE] = ACTIONS(1178),
    [anon_sym_u8_DQUOTE] = ACTIONS(1178),
    [anon_sym_DQUOTE] = ACTIONS(1178),
    [sym_true] = ACTIONS(1176),
    [sym_false] = ACTIONS(1176),
    [anon_sym_NULL] = ACTIONS(1176),
    [anon_sym_nullptr] = ACTIONS(1176),
    [sym_comment] = ACTIONS(3),
  },
  [179] = {
    [ts_builtin_sym_end] = ACTIONS(1214),
    [sym_identifier] = ACTIONS(1212),
    [aux_sym_preproc_include_token1] = ACTIONS(1212),
    [aux_sym_preproc_def_token1] = ACTIONS(1212),
    [aux_sym_preproc_if_token1] = ACTIONS(1212),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1212),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1212),
    [sym_preproc_directive] = ACTIONS(1212),
    [anon_sym_LPAREN2] = ACTIONS(1214),
    [anon_sym_BANG] = ACTIONS(1214),
    [anon_sym_TILDE] = ACTIONS(1214),
    [anon_sym_DASH] = ACTIONS(1212),
    [anon_sym_PLUS] = ACTIONS(1212),
    [anon_sym_STAR] = ACTIONS(1214),
    [anon_sym_AMP] = ACTIONS(1214),
    [anon_sym_SEMI] = ACTIONS(1214),
    [anon_sym___extension__] = ACTIONS(1212),
    [anon_sym_typedef] = ACTIONS(1212),
    [anon_sym_extern] = ACTIONS(1212),
    [anon_sym___attribute__] = ACTIONS(1212),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1214),
    [anon_sym___declspec] = ACTIONS(1212),
    [anon_sym___cdecl] = ACTIONS(1212),
    [anon_sym___clrcall] = ACTIONS(1212),
    [anon_sym___stdcall] = ACTIONS(1212),
    [anon_sym___fastcall] = ACTIONS(1212),
    [anon_sym___thiscall] = ACTIONS(1212),
    [anon_sym___vectorcall] = ACTIONS(1212),
    [anon_sym_LBRACE] = ACTIONS(1214),
    [anon_sym_signed] = ACTIONS(1212),
    [anon_sym_unsigned] = ACTIONS(1212),
    [anon_sym_long] = ACTIONS(1212),
    [anon_sym_short] = ACTIONS(1212),
    [anon_sym_static] = ACTIONS(1212),
    [anon_sym_auto] = ACTIONS(1212),
    [anon_sym_register] = ACTIONS(1212),
    [anon_sym_inline] = ACTIONS(1212),
    [anon_sym___inline] = ACTIONS(1212),
    [anon_sym___inline__] = ACTIONS(1212),
    [anon_sym___forceinline] = ACTIONS(1212),
    [anon_sym_thread_local] = ACTIONS(1212),
    [anon_sym___thread] = ACTIONS(1212),
    [anon_sym_const] = ACTIONS(1212),
    [anon_sym_constexpr] = ACTIONS(1212),
    [anon_sym_volatile] = ACTIONS(1212),
    [anon_sym_restrict] = ACTIONS(1212),
    [anon_sym___restrict__] = ACTIONS(1212),
    [anon_sym__Atomic] = ACTIONS(1212),
    [anon_sym__Noreturn] = ACTIONS(1212),
    [anon_sym_noreturn] = ACTIONS(1212),
    [sym_primitive_type] = ACTIONS(1212),
    [anon_sym_enum] = ACTIONS(1212),
    [anon_sym_struct] = ACTIONS(1212),
    [anon_sym_union] = ACTIONS(1212),
    [anon_sym_if] = ACTIONS(1212),
    [anon_sym_else] = ACTIONS(1212),
    [anon_sym_switch] = ACTIONS(1212),
    [anon_sym_case] = ACTIONS(1212),
    [anon_sym_default] = ACTIONS(1212),
    [anon_sym_while] = ACTIONS(1212),
    [anon_sym_do] = ACTIONS(1212),
    [anon_sym_for] = ACTIONS(1212),
    [anon_sym_return] = ACTIONS(1212),
    [anon_sym_break] = ACTIONS(1212),
    [anon_sym_continue] = ACTIONS(1212),
    [anon_sym_goto] = ACTIONS(1212),
    [anon_sym___try] = ACTIONS(1212),
    [anon_sym___leave] = ACTIONS(1212),
    [anon_sym_DASH_DASH] = ACTIONS(1214),
    [anon_sym_PLUS_PLUS] = ACTIONS(1214),
    [anon_sym_sizeof] = ACTIONS(1212),
    [anon_sym___alignof__] = ACTIONS(1212),
    [anon_sym___alignof] = ACTIONS(1212),
    [anon_sym__alignof] = ACTIONS(1212),
    [anon_sym_alignof] = ACTIONS(1212),
    [anon_sym__Alignof] = ACTIONS(1212),
    [anon_sym_offsetof] = ACTIONS(1212),
    [anon_sym__Generic] = ACTIONS(1212),
    [anon_sym_asm] = ACTIONS(1212),
    [anon_sym___asm__] = ACTIONS(1212),
    [sym_number_literal] = ACTIONS(1214),
    [anon_sym_L_SQUOTE] = ACTIONS(1214),
    [anon_sym_u_SQUOTE] = ACTIONS(1214),
    [anon_sym_U_SQUOTE] = ACTIONS(1214),
    [anon_sym_u8_SQUOTE] = ACTIONS(1214),
    [anon_sym_SQUOTE] = ACTIONS(1214),
    [anon_sym_L_DQUOTE] = ACTIONS(1214),
    [anon_sym_u_DQUOTE] = ACTIONS(1214),
    [anon_sym_U_DQUOTE] = ACTIONS(1214),
    [anon_sym_u8_DQUOTE] = ACTIONS(1214),
    [anon_sym_DQUOTE] = ACTIONS(1214),
    [sym_true] = ACTIONS(1212),
    [sym_false] = ACTIONS(1212),
    [anon_sym_NULL] = ACTIONS(1212),
    [anon_sym_nullptr] = ACTIONS(1212),
    [sym_comment] = ACTIONS(3),
  },
  [180] = {
    [ts_builtin_sym_end] = ACTIONS(1218),
    [sym_identifier] = ACTIONS(1216),
    [aux_sym_preproc_include_token1] = ACTIONS(1216),
    [aux_sym_preproc_def_token1] = ACTIONS(1216),
    [aux_sym_preproc_if_token1] = ACTIONS(1216),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1216),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1216),
    [sym_preproc_directive] = ACTIONS(1216),
    [anon_sym_LPAREN2] = ACTIONS(1218),
    [anon_sym_BANG] = ACTIONS(1218),
    [anon_sym_TILDE] = ACTIONS(1218),
    [anon_sym_DASH] = ACTIONS(1216),
    [anon_sym_PLUS] = ACTIONS(1216),
    [anon_sym_STAR] = ACTIONS(1218),
    [anon_sym_AMP] = ACTIONS(1218),
    [anon_sym_SEMI] = ACTIONS(1218),
    [anon_sym___extension__] = ACTIONS(1216),
    [anon_sym_typedef] = ACTIONS(1216),
    [anon_sym_extern] = ACTIONS(1216),
    [anon_sym___attribute__] = ACTIONS(1216),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1218),
    [anon_sym___declspec] = ACTIONS(1216),
    [anon_sym___cdecl] = ACTIONS(1216),
    [anon_sym___clrcall] = ACTIONS(1216),
    [anon_sym___stdcall] = ACTIONS(1216),
    [anon_sym___fastcall] = ACTIONS(1216),
    [anon_sym___thiscall] = ACTIONS(1216),
    [anon_sym___vectorcall] = ACTIONS(1216),
    [anon_sym_LBRACE] = ACTIONS(1218),
    [anon_sym_signed] = ACTIONS(1216),
    [anon_sym_unsigned] = ACTIONS(1216),
    [anon_sym_long] = ACTIONS(1216),
    [anon_sym_short] = ACTIONS(1216),
    [anon_sym_static] = ACTIONS(1216),
    [anon_sym_auto] = ACTIONS(1216),
    [anon_sym_register] = ACTIONS(1216),
    [anon_sym_inline] = ACTIONS(1216),
    [anon_sym___inline] = ACTIONS(1216),
    [anon_sym___inline__] = ACTIONS(1216),
    [anon_sym___forceinline] = ACTIONS(1216),
    [anon_sym_thread_local] = ACTIONS(1216),
    [anon_sym___thread] = ACTIONS(1216),
    [anon_sym_const] = ACTIONS(1216),
    [anon_sym_constexpr] = ACTIONS(1216),
    [anon_sym_volatile] = ACTIONS(1216),
    [anon_sym_restrict] = ACTIONS(1216),
    [anon_sym___restrict__] = ACTIONS(1216),
    [anon_sym__Atomic] = ACTIONS(1216),
    [anon_sym__Noreturn] = ACTIONS(1216),
    [anon_sym_noreturn] = ACTIONS(1216),
    [sym_primitive_type] = ACTIONS(1216),
    [anon_sym_enum] = ACTIONS(1216),
    [anon_sym_struct] = ACTIONS(1216),
    [anon_sym_union] = ACTIONS(1216),
    [anon_sym_if] = ACTIONS(1216),
    [anon_sym_else] = ACTIONS(1216),
    [anon_sym_switch] = ACTIONS(1216),
    [anon_sym_case] = ACTIONS(1216),
    [anon_sym_default] = ACTIONS(1216),
    [anon_sym_while] = ACTIONS(1216),
    [anon_sym_do] = ACTIONS(1216),
    [anon_sym_for] = ACTIONS(1216),
    [anon_sym_return] = ACTIONS(1216),
    [anon_sym_break] = ACTIONS(1216),
    [anon_sym_continue] = ACTIONS(1216),
    [anon_sym_goto] = ACTIONS(1216),
    [anon_sym___try] = ACTIONS(1216),
    [anon_sym___leave] = ACTIONS(1216),
    [anon_sym_DASH_DASH] = ACTIONS(1218),
    [anon_sym_PLUS_PLUS] = ACTIONS(1218),
    [anon_sym_sizeof] = ACTIONS(1216),
    [anon_sym___alignof__] = ACTIONS(1216),
    [anon_sym___alignof] = ACTIONS(1216),
    [anon_sym__alignof] = ACTIONS(1216),
    [anon_sym_alignof] = ACTIONS(1216),
    [anon_sym__Alignof] = ACTIONS(1216),
    [anon_sym_offsetof] = ACTIONS(1216),
    [anon_sym__Generic] = ACTIONS(1216),
    [anon_sym_asm] = ACTIONS(1216),
    [anon_sym___asm__] = ACTIONS(1216),
    [sym_number_literal] = ACTIONS(1218),
    [anon_sym_L_SQUOTE] = ACTIONS(1218),
    [anon_sym_u_SQUOTE] = ACTIONS(1218),
    [anon_sym_U_SQUOTE] = ACTIONS(1218),
    [anon_sym_u8_SQUOTE] = ACTIONS(1218),
    [anon_sym_SQUOTE] = ACTIONS(1218),
    [anon_sym_L_DQUOTE] = ACTIONS(1218),
    [anon_sym_u_DQUOTE] = ACTIONS(1218),
    [anon_sym_U_DQUOTE] = ACTIONS(1218),
    [anon_sym_u8_DQUOTE] = ACTIONS(1218),
    [anon_sym_DQUOTE] = ACTIONS(1218),
    [sym_true] = ACTIONS(1216),
    [sym_false] = ACTIONS(1216),
    [anon_sym_NULL] = ACTIONS(1216),
    [anon_sym_nullptr] = ACTIONS(1216),
    [sym_comment] = ACTIONS(3),
  },
  [181] = {
    [ts_builtin_sym_end] = ACTIONS(1230),
    [sym_identifier] = ACTIONS(1228),
    [aux_sym_preproc_include_token1] = ACTIONS(1228),
    [aux_sym_preproc_def_token1] = ACTIONS(1228),
    [aux_sym_preproc_if_token1] = ACTIONS(1228),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1228),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1228),
    [sym_preproc_directive] = ACTIONS(1228),
    [anon_sym_LPAREN2] = ACTIONS(1230),
    [anon_sym_BANG] = ACTIONS(1230),
    [anon_sym_TILDE] = ACTIONS(1230),
    [anon_sym_DASH] = ACTIONS(1228),
    [anon_sym_PLUS] = ACTIONS(1228),
    [anon_sym_STAR] = ACTIONS(1230),
    [anon_sym_AMP] = ACTIONS(1230),
    [anon_sym_SEMI] = ACTIONS(1230),
    [anon_sym___extension__] = ACTIONS(1228),
    [anon_sym_typedef] = ACTIONS(1228),
    [anon_sym_extern] = ACTIONS(1228),
    [anon_sym___attribute__] = ACTIONS(1228),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1230),
    [anon_sym___declspec] = ACTIONS(1228),
    [anon_sym___cdecl] = ACTIONS(1228),
    [anon_sym___clrcall] = ACTIONS(1228),
    [anon_sym___stdcall] = ACTIONS(1228),
    [anon_sym___fastcall] = ACTIONS(1228),
    [anon_sym___thiscall] = ACTIONS(1228),
    [anon_sym___vectorcall] = ACTIONS(1228),
    [anon_sym_LBRACE] = ACTIONS(1230),
    [anon_sym_signed] = ACTIONS(1228),
    [anon_sym_unsigned] = ACTIONS(1228),
    [anon_sym_long] = ACTIONS(1228),
    [anon_sym_short] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_register] = ACTIONS(1228),
    [anon_sym_inline] = ACTIONS(1228),
    [anon_sym___inline] = ACTIONS(1228),
    [anon_sym___inline__] = ACTIONS(1228),
    [anon_sym___forceinline] = ACTIONS(1228),
    [anon_sym_thread_local] = ACTIONS(1228),
    [anon_sym___thread] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_constexpr] = ACTIONS(1228),
    [anon_sym_volatile] = ACTIONS(1228),
    [anon_sym_restrict] = ACTIONS(1228),
    [anon_sym___restrict__] = ACTIONS(1228),
    [anon_sym__Atomic] = ACTIONS(1228),
    [anon_sym__Noreturn] = ACTIONS(1228),
    [anon_sym_noreturn] = ACTIONS(1228),
    [sym_primitive_type] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_else] = ACTIONS(1228),
    [anon_sym_switch] = ACTIONS(1228),
    [anon_sym_case] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [anon_sym_do] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_goto] = ACTIONS(1228),
    [anon_sym___try] = ACTIONS(1228),
    [anon_sym___leave] = ACTIONS(1228),
    [anon_sym_DASH_DASH] = ACTIONS(1230),
    [anon_sym_PLUS_PLUS] = ACTIONS(1230),
    [anon_sym_sizeof] = ACTIONS(1228),
    [anon_sym___alignof__] = ACTIONS(1228),
    [anon_sym___alignof] = ACTIONS(1228),
    [anon_sym__alignof] = ACTIONS(1228),
    [anon_sym_alignof] = ACTIONS(1228),
    [anon_sym__Alignof] = ACTIONS(1228),
    [anon_sym_offsetof] = ACTIONS(1228),
    [anon_sym__Generic] = ACTIONS(1228),
    [anon_sym_asm] = ACTIONS(1228),
    [anon_sym___asm__] = ACTIONS(1228),
    [sym_number_literal] = ACTIONS(1230),
    [anon_sym_L_SQUOTE] = ACTIONS(1230),
    [anon_sym_u_SQUOTE] = ACTIONS(1230),
    [anon_sym_U_SQUOTE] = ACTIONS(1230),
    [anon_sym_u8_SQUOTE] = ACTIONS(1230),
    [anon_sym_SQUOTE] = ACTIONS(1230),
    [anon_sym_L_DQUOTE] = ACTIONS(1230),
    [anon_sym_u_DQUOTE] = ACTIONS(1230),
    [anon_sym_U_DQUOTE] = ACTIONS(1230),
    [anon_sym_u8_DQUOTE] = ACTIONS(1230),
    [anon_sym_DQUOTE] = ACTIONS(1230),
    [sym_true] = ACTIONS(1228),
    [sym_false] = ACTIONS(1228),
    [anon_sym_NULL] = ACTIONS(1228),
    [anon_sym_nullptr] = ACTIONS(1228),
    [sym_comment] = ACTIONS(3),
  },
  [182] = {
    [ts_builtin_sym_end] = ACTIONS(1234),
    [sym_identifier] = ACTIONS(1232),
    [aux_sym_preproc_include_token1] = ACTIONS(1232),
    [aux_sym_preproc_def_token1] = ACTIONS(1232),
    [aux_sym_preproc_if_token1] = ACTIONS(1232),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1232),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1232),
    [sym_preproc_directive] = ACTIONS(1232),
    [anon_sym_LPAREN2] = ACTIONS(1234),
    [anon_sym_BANG] = ACTIONS(1234),
    [anon_sym_TILDE] = ACTIONS(1234),
    [anon_sym_DASH] = ACTIONS(1232),
    [anon_sym_PLUS] = ACTIONS(1232),
    [anon_sym_STAR] = ACTIONS(1234),
    [anon_sym_AMP] = ACTIONS(1234),
    [anon_sym_SEMI] = ACTIONS(1234),
    [anon_sym___extension__] = ACTIONS(1232),
    [anon_sym_typedef] = ACTIONS(1232),
    [anon_sym_extern] = ACTIONS(1232),
    [anon_sym___attribute__] = ACTIONS(1232),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1234),
    [anon_sym___declspec] = ACTIONS(1232),
    [anon_sym___cdecl] = ACTIONS(1232),
    [anon_sym___clrcall] = ACTIONS(1232),
    [anon_sym___stdcall] = ACTIONS(1232),
    [anon_sym___fastcall] = ACTIONS(1232),
    [anon_sym___thiscall] = ACTIONS(1232),
    [anon_sym___vectorcall] = ACTIONS(1232),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_signed] = ACTIONS(1232),
    [anon_sym_unsigned] = ACTIONS(1232),
    [anon_sym_long] = ACTIONS(1232),
    [anon_sym_short] = ACTIONS(1232),
    [anon_sym_static] = ACTIONS(1232),
    [anon_sym_auto] = ACTIONS(1232),
    [anon_sym_register] = ACTIONS(1232),
    [anon_sym_inline] = ACTIONS(1232),
    [anon_sym___inline] = ACTIONS(1232),
    [anon_sym___inline__] = ACTIONS(1232),
    [anon_sym___forceinline] = ACTIONS(1232),
    [anon_sym_thread_local] = ACTIONS(1232),
    [anon_sym___thread] = ACTIONS(1232),
    [anon_sym_const] = ACTIONS(1232),
    [anon_sym_constexpr] = ACTIONS(1232),
    [anon_sym_volatile] = ACTIONS(1232),
    [anon_sym_restrict] = ACTIONS(1232),
    [anon_sym___restrict__] = ACTIONS(1232),
    [anon_sym__Atomic] = ACTIONS(1232),
    [anon_sym__Noreturn] = ACTIONS(1232),
    [anon_sym_noreturn] = ACTIONS(1232),
    [sym_primitive_type] = ACTIONS(1232),
    [anon_sym_enum] = ACTIONS(1232),
    [anon_sym_struct] = ACTIONS(1232),
    [anon_sym_union] = ACTIONS(1232),
    [anon_sym_if] = ACTIONS(1232),
    [anon_sym_else] = ACTIONS(1232),
    [anon_sym_switch] = ACTIONS(1232),
    [anon_sym_case] = ACTIONS(1232),
    [anon_sym_default] = ACTIONS(1232),
    [anon_sym_while] = ACTIONS(1232),
    [anon_sym_do] = ACTIONS(1232),
    [anon_sym_for] = ACTIONS(1232),
    [anon_sym_return] = ACTIONS(1232),
    [anon_sym_break] = ACTIONS(1232),
    [anon_sym_continue] = ACTIONS(1232),
    [anon_sym_goto] = ACTIONS(1232),
    [anon_sym___try] = ACTIONS(1232),
    [anon_sym___leave] = ACTIONS(1232),
    [anon_sym_DASH_DASH] = ACTIONS(1234),
    [anon_sym_PLUS_PLUS] = ACTIONS(1234),
    [anon_sym_sizeof] = ACTIONS(1232),
    [anon_sym___alignof__] = ACTIONS(1232),
    [anon_sym___alignof] = ACTIONS(1232),
    [anon_sym__alignof] = ACTIONS(1232),
    [anon_sym_alignof] = ACTIONS(1232),
    [anon_sym__Alignof] = ACTIONS(1232),
    [anon_sym_offsetof] = ACTIONS(1232),
    [anon_sym__Generic] = ACTIONS(1232),
    [anon_sym_asm] = ACTIONS(1232),
    [anon_sym___asm__] = ACTIONS(1232),
    [sym_number_literal] = ACTIONS(1234),
    [anon_sym_L_SQUOTE] = ACTIONS(1234),
    [anon_sym_u_SQUOTE] = ACTIONS(1234),
    [anon_sym_U_SQUOTE] = ACTIONS(1234),
    [anon_sym_u8_SQUOTE] = ACTIONS(1234),
    [anon_sym_SQUOTE] = ACTIONS(1234),
    [anon_sym_L_DQUOTE] = ACTIONS(1234),
    [anon_sym_u_DQUOTE] = ACTIONS(1234),
    [anon_sym_U_DQUOTE] = ACTIONS(1234),
    [anon_sym_u8_DQUOTE] = ACTIONS(1234),
    [anon_sym_DQUOTE] = ACTIONS(1234),
    [sym_true] = ACTIONS(1232),
    [sym_false] = ACTIONS(1232),
    [anon_sym_NULL] = ACTIONS(1232),
    [anon_sym_nullptr] = ACTIONS(1232),
    [sym_comment] = ACTIONS(3),
  },
  [183] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [184] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [185] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [186] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [187] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [188] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [189] = {
    [sym_identifier] = ACTIONS(1168),
    [aux_sym_preproc_include_token1] = ACTIONS(1168),
    [aux_sym_preproc_def_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token2] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1168),
    [sym_preproc_directive] = ACTIONS(1168),
    [anon_sym_LPAREN2] = ACTIONS(1170),
    [anon_sym_BANG] = ACTIONS(1170),
    [anon_sym_TILDE] = ACTIONS(1170),
    [anon_sym_DASH] = ACTIONS(1168),
    [anon_sym_PLUS] = ACTIONS(1168),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1170),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym___extension__] = ACTIONS(1168),
    [anon_sym_typedef] = ACTIONS(1168),
    [anon_sym_extern] = ACTIONS(1168),
    [anon_sym___attribute__] = ACTIONS(1168),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1170),
    [anon_sym___declspec] = ACTIONS(1168),
    [anon_sym___cdecl] = ACTIONS(1168),
    [anon_sym___clrcall] = ACTIONS(1168),
    [anon_sym___stdcall] = ACTIONS(1168),
    [anon_sym___fastcall] = ACTIONS(1168),
    [anon_sym___thiscall] = ACTIONS(1168),
    [anon_sym___vectorcall] = ACTIONS(1168),
    [anon_sym_LBRACE] = ACTIONS(1170),
    [anon_sym_signed] = ACTIONS(1168),
    [anon_sym_unsigned] = ACTIONS(1168),
    [anon_sym_long] = ACTIONS(1168),
    [anon_sym_short] = ACTIONS(1168),
    [anon_sym_static] = ACTIONS(1168),
    [anon_sym_auto] = ACTIONS(1168),
    [anon_sym_register] = ACTIONS(1168),
    [anon_sym_inline] = ACTIONS(1168),
    [anon_sym___inline] = ACTIONS(1168),
    [anon_sym___inline__] = ACTIONS(1168),
    [anon_sym___forceinline] = ACTIONS(1168),
    [anon_sym_thread_local] = ACTIONS(1168),
    [anon_sym___thread] = ACTIONS(1168),
    [anon_sym_const] = ACTIONS(1168),
    [anon_sym_constexpr] = ACTIONS(1168),
    [anon_sym_volatile] = ACTIONS(1168),
    [anon_sym_restrict] = ACTIONS(1168),
    [anon_sym___restrict__] = ACTIONS(1168),
    [anon_sym__Atomic] = ACTIONS(1168),
    [anon_sym__Noreturn] = ACTIONS(1168),
    [anon_sym_noreturn] = ACTIONS(1168),
    [sym_primitive_type] = ACTIONS(1168),
    [anon_sym_enum] = ACTIONS(1168),
    [anon_sym_struct] = ACTIONS(1168),
    [anon_sym_union] = ACTIONS(1168),
    [anon_sym_if] = ACTIONS(1168),
    [anon_sym_else] = ACTIONS(1168),
    [anon_sym_switch] = ACTIONS(1168),
    [anon_sym_case] = ACTIONS(1168),
    [anon_sym_default] = ACTIONS(1168),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_do] = ACTIONS(1168),
    [anon_sym_for] = ACTIONS(1168),
    [anon_sym_return] = ACTIONS(1168),
    [anon_sym_break] = ACTIONS(1168),
    [anon_sym_continue] = ACTIONS(1168),
    [anon_sym_goto] = ACTIONS(1168),
    [anon_sym___try] = ACTIONS(1168),
    [anon_sym___leave] = ACTIONS(1168),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_sizeof] = ACTIONS(1168),
    [anon_sym___alignof__] = ACTIONS(1168),
    [anon_sym___alignof] = ACTIONS(1168),
    [anon_sym__alignof] = ACTIONS(1168),
    [anon_sym_alignof] = ACTIONS(1168),
    [anon_sym__Alignof] = ACTIONS(1168),
    [anon_sym_offsetof] = ACTIONS(1168),
    [anon_sym__Generic] = ACTIONS(1168),
    [anon_sym_asm] = ACTIONS(1168),
    [anon_sym___asm__] = ACTIONS(1168),
    [sym_number_literal] = ACTIONS(1170),
    [anon_sym_L_SQUOTE] = ACTIONS(1170),
    [anon_sym_u_SQUOTE] = ACTIONS(1170),
    [anon_sym_U_SQUOTE] = ACTIONS(1170),
    [anon_sym_u8_SQUOTE] = ACTIONS(1170),
    [anon_sym_SQUOTE] = ACTIONS(1170),
    [anon_sym_L_DQUOTE] = ACTIONS(1170),
    [anon_sym_u_DQUOTE] = ACTIONS(1170),
    [anon_sym_U_DQUOTE] = ACTIONS(1170),
    [anon_sym_u8_DQUOTE] = ACTIONS(1170),
    [anon_sym_DQUOTE] = ACTIONS(1170),
    [sym_true] = ACTIONS(1168),
    [sym_false] = ACTIONS(1168),
    [anon_sym_NULL] = ACTIONS(1168),
    [anon_sym_nullptr] = ACTIONS(1168),
    [sym_comment] = ACTIONS(3),
  },
  [190] = {
    [sym_identifier] = ACTIONS(1156),
    [aux_sym_preproc_include_token1] = ACTIONS(1156),
    [aux_sym_preproc_def_token1] = ACTIONS(1156),
    [aux_sym_preproc_if_token1] = ACTIONS(1156),
    [aux_sym_preproc_if_token2] = ACTIONS(1156),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1156),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1156),
    [sym_preproc_directive] = ACTIONS(1156),
    [anon_sym_LPAREN2] = ACTIONS(1158),
    [anon_sym_BANG] = ACTIONS(1158),
    [anon_sym_TILDE] = ACTIONS(1158),
    [anon_sym_DASH] = ACTIONS(1156),
    [anon_sym_PLUS] = ACTIONS(1156),
    [anon_sym_STAR] = ACTIONS(1158),
    [anon_sym_AMP] = ACTIONS(1158),
    [anon_sym_SEMI] = ACTIONS(1158),
    [anon_sym___extension__] = ACTIONS(1156),
    [anon_sym_typedef] = ACTIONS(1156),
    [anon_sym_extern] = ACTIONS(1156),
    [anon_sym___attribute__] = ACTIONS(1156),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1158),
    [anon_sym___declspec] = ACTIONS(1156),
    [anon_sym___cdecl] = ACTIONS(1156),
    [anon_sym___clrcall] = ACTIONS(1156),
    [anon_sym___stdcall] = ACTIONS(1156),
    [anon_sym___fastcall] = ACTIONS(1156),
    [anon_sym___thiscall] = ACTIONS(1156),
    [anon_sym___vectorcall] = ACTIONS(1156),
    [anon_sym_LBRACE] = ACTIONS(1158),
    [anon_sym_signed] = ACTIONS(1156),
    [anon_sym_unsigned] = ACTIONS(1156),
    [anon_sym_long] = ACTIONS(1156),
    [anon_sym_short] = ACTIONS(1156),
    [anon_sym_static] = ACTIONS(1156),
    [anon_sym_auto] = ACTIONS(1156),
    [anon_sym_register] = ACTIONS(1156),
    [anon_sym_inline] = ACTIONS(1156),
    [anon_sym___inline] = ACTIONS(1156),
    [anon_sym___inline__] = ACTIONS(1156),
    [anon_sym___forceinline] = ACTIONS(1156),
    [anon_sym_thread_local] = ACTIONS(1156),
    [anon_sym___thread] = ACTIONS(1156),
    [anon_sym_const] = ACTIONS(1156),
    [anon_sym_constexpr] = ACTIONS(1156),
    [anon_sym_volatile] = ACTIONS(1156),
    [anon_sym_restrict] = ACTIONS(1156),
    [anon_sym___restrict__] = ACTIONS(1156),
    [anon_sym__Atomic] = ACTIONS(1156),
    [anon_sym__Noreturn] = ACTIONS(1156),
    [anon_sym_noreturn] = ACTIONS(1156),
    [sym_primitive_type] = ACTIONS(1156),
    [anon_sym_enum] = ACTIONS(1156),
    [anon_sym_struct] = ACTIONS(1156),
    [anon_sym_union] = ACTIONS(1156),
    [anon_sym_if] = ACTIONS(1156),
    [anon_sym_else] = ACTIONS(1156),
    [anon_sym_switch] = ACTIONS(1156),
    [anon_sym_case] = ACTIONS(1156),
    [anon_sym_default] = ACTIONS(1156),
    [anon_sym_while] = ACTIONS(1156),
    [anon_sym_do] = ACTIONS(1156),
    [anon_sym_for] = ACTIONS(1156),
    [anon_sym_return] = ACTIONS(1156),
    [anon_sym_break] = ACTIONS(1156),
    [anon_sym_continue] = ACTIONS(1156),
    [anon_sym_goto] = ACTIONS(1156),
    [anon_sym___try] = ACTIONS(1156),
    [anon_sym___leave] = ACTIONS(1156),
    [anon_sym_DASH_DASH] = ACTIONS(1158),
    [anon_sym_PLUS_PLUS] = ACTIONS(1158),
    [anon_sym_sizeof] = ACTIONS(1156),
    [anon_sym___alignof__] = ACTIONS(1156),
    [anon_sym___alignof] = ACTIONS(1156),
    [anon_sym__alignof] = ACTIONS(1156),
    [anon_sym_alignof] = ACTIONS(1156),
    [anon_sym__Alignof] = ACTIONS(1156),
    [anon_sym_offsetof] = ACTIONS(1156),
    [anon_sym__Generic] = ACTIONS(1156),
    [anon_sym_asm] = ACTIONS(1156),
    [anon_sym___asm__] = ACTIONS(1156),
    [sym_number_literal] = ACTIONS(1158),
    [anon_sym_L_SQUOTE] = ACTIONS(1158),
    [anon_sym_u_SQUOTE] = ACTIONS(1158),
    [anon_sym_U_SQUOTE] = ACTIONS(1158),
    [anon_sym_u8_SQUOTE] = ACTIONS(1158),
    [anon_sym_SQUOTE] = ACTIONS(1158),
    [anon_sym_L_DQUOTE] = ACTIONS(1158),
    [anon_sym_u_DQUOTE] = ACTIONS(1158),
    [anon_sym_U_DQUOTE] = ACTIONS(1158),
    [anon_sym_u8_DQUOTE] = ACTIONS(1158),
    [anon_sym_DQUOTE] = ACTIONS(1158),
    [sym_true] = ACTIONS(1156),
    [sym_false] = ACTIONS(1156),
    [anon_sym_NULL] = ACTIONS(1156),
    [anon_sym_nullptr] = ACTIONS(1156),
    [sym_comment] = ACTIONS(3),
  },
  [191] = {
    [sym_identifier] = ACTIONS(1152),
    [aux_sym_preproc_include_token1] = ACTIONS(1152),
    [aux_sym_preproc_def_token1] = ACTIONS(1152),
    [aux_sym_preproc_if_token1] = ACTIONS(1152),
    [aux_sym_preproc_if_token2] = ACTIONS(1152),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1152),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1152),
    [sym_preproc_directive] = ACTIONS(1152),
    [anon_sym_LPAREN2] = ACTIONS(1154),
    [anon_sym_BANG] = ACTIONS(1154),
    [anon_sym_TILDE] = ACTIONS(1154),
    [anon_sym_DASH] = ACTIONS(1152),
    [anon_sym_PLUS] = ACTIONS(1152),
    [anon_sym_STAR] = ACTIONS(1154),
    [anon_sym_AMP] = ACTIONS(1154),
    [anon_sym_SEMI] = ACTIONS(1154),
    [anon_sym___extension__] = ACTIONS(1152),
    [anon_sym_typedef] = ACTIONS(1152),
    [anon_sym_extern] = ACTIONS(1152),
    [anon_sym___attribute__] = ACTIONS(1152),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1154),
    [anon_sym___declspec] = ACTIONS(1152),
    [anon_sym___cdecl] = ACTIONS(1152),
    [anon_sym___clrcall] = ACTIONS(1152),
    [anon_sym___stdcall] = ACTIONS(1152),
    [anon_sym___fastcall] = ACTIONS(1152),
    [anon_sym___thiscall] = ACTIONS(1152),
    [anon_sym___vectorcall] = ACTIONS(1152),
    [anon_sym_LBRACE] = ACTIONS(1154),
    [anon_sym_signed] = ACTIONS(1152),
    [anon_sym_unsigned] = ACTIONS(1152),
    [anon_sym_long] = ACTIONS(1152),
    [anon_sym_short] = ACTIONS(1152),
    [anon_sym_static] = ACTIONS(1152),
    [anon_sym_auto] = ACTIONS(1152),
    [anon_sym_register] = ACTIONS(1152),
    [anon_sym_inline] = ACTIONS(1152),
    [anon_sym___inline] = ACTIONS(1152),
    [anon_sym___inline__] = ACTIONS(1152),
    [anon_sym___forceinline] = ACTIONS(1152),
    [anon_sym_thread_local] = ACTIONS(1152),
    [anon_sym___thread] = ACTIONS(1152),
    [anon_sym_const] = ACTIONS(1152),
    [anon_sym_constexpr] = ACTIONS(1152),
    [anon_sym_volatile] = ACTIONS(1152),
    [anon_sym_restrict] = ACTIONS(1152),
    [anon_sym___restrict__] = ACTIONS(1152),
    [anon_sym__Atomic] = ACTIONS(1152),
    [anon_sym__Noreturn] = ACTIONS(1152),
    [anon_sym_noreturn] = ACTIONS(1152),
    [sym_primitive_type] = ACTIONS(1152),
    [anon_sym_enum] = ACTIONS(1152),
    [anon_sym_struct] = ACTIONS(1152),
    [anon_sym_union] = ACTIONS(1152),
    [anon_sym_if] = ACTIONS(1152),
    [anon_sym_else] = ACTIONS(1152),
    [anon_sym_switch] = ACTIONS(1152),
    [anon_sym_case] = ACTIONS(1152),
    [anon_sym_default] = ACTIONS(1152),
    [anon_sym_while] = ACTIONS(1152),
    [anon_sym_do] = ACTIONS(1152),
    [anon_sym_for] = ACTIONS(1152),
    [anon_sym_return] = ACTIONS(1152),
    [anon_sym_break] = ACTIONS(1152),
    [anon_sym_continue] = ACTIONS(1152),
    [anon_sym_goto] = ACTIONS(1152),
    [anon_sym___try] = ACTIONS(1152),
    [anon_sym___leave] = ACTIONS(1152),
    [anon_sym_DASH_DASH] = ACTIONS(1154),
    [anon_sym_PLUS_PLUS] = ACTIONS(1154),
    [anon_sym_sizeof] = ACTIONS(1152),
    [anon_sym___alignof__] = ACTIONS(1152),
    [anon_sym___alignof] = ACTIONS(1152),
    [anon_sym__alignof] = ACTIONS(1152),
    [anon_sym_alignof] = ACTIONS(1152),
    [anon_sym__Alignof] = ACTIONS(1152),
    [anon_sym_offsetof] = ACTIONS(1152),
    [anon_sym__Generic] = ACTIONS(1152),
    [anon_sym_asm] = ACTIONS(1152),
    [anon_sym___asm__] = ACTIONS(1152),
    [sym_number_literal] = ACTIONS(1154),
    [anon_sym_L_SQUOTE] = ACTIONS(1154),
    [anon_sym_u_SQUOTE] = ACTIONS(1154),
    [anon_sym_U_SQUOTE] = ACTIONS(1154),
    [anon_sym_u8_SQUOTE] = ACTIONS(1154),
    [anon_sym_SQUOTE] = ACTIONS(1154),
    [anon_sym_L_DQUOTE] = ACTIONS(1154),
    [anon_sym_u_DQUOTE] = ACTIONS(1154),
    [anon_sym_U_DQUOTE] = ACTIONS(1154),
    [anon_sym_u8_DQUOTE] = ACTIONS(1154),
    [anon_sym_DQUOTE] = ACTIONS(1154),
    [sym_true] = ACTIONS(1152),
    [sym_false] = ACTIONS(1152),
    [anon_sym_NULL] = ACTIONS(1152),
    [anon_sym_nullptr] = ACTIONS(1152),
    [sym_comment] = ACTIONS(3),
  },
  [192] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [193] = {
    [sym_identifier] = ACTIONS(1144),
    [aux_sym_preproc_include_token1] = ACTIONS(1144),
    [aux_sym_preproc_def_token1] = ACTIONS(1144),
    [aux_sym_preproc_if_token1] = ACTIONS(1144),
    [aux_sym_preproc_if_token2] = ACTIONS(1144),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1144),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1144),
    [sym_preproc_directive] = ACTIONS(1144),
    [anon_sym_LPAREN2] = ACTIONS(1146),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_STAR] = ACTIONS(1146),
    [anon_sym_AMP] = ACTIONS(1146),
    [anon_sym_SEMI] = ACTIONS(1146),
    [anon_sym___extension__] = ACTIONS(1144),
    [anon_sym_typedef] = ACTIONS(1144),
    [anon_sym_extern] = ACTIONS(1144),
    [anon_sym___attribute__] = ACTIONS(1144),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1146),
    [anon_sym___declspec] = ACTIONS(1144),
    [anon_sym___cdecl] = ACTIONS(1144),
    [anon_sym___clrcall] = ACTIONS(1144),
    [anon_sym___stdcall] = ACTIONS(1144),
    [anon_sym___fastcall] = ACTIONS(1144),
    [anon_sym___thiscall] = ACTIONS(1144),
    [anon_sym___vectorcall] = ACTIONS(1144),
    [anon_sym_LBRACE] = ACTIONS(1146),
    [anon_sym_signed] = ACTIONS(1144),
    [anon_sym_unsigned] = ACTIONS(1144),
    [anon_sym_long] = ACTIONS(1144),
    [anon_sym_short] = ACTIONS(1144),
    [anon_sym_static] = ACTIONS(1144),
    [anon_sym_auto] = ACTIONS(1144),
    [anon_sym_register] = ACTIONS(1144),
    [anon_sym_inline] = ACTIONS(1144),
    [anon_sym___inline] = ACTIONS(1144),
    [anon_sym___inline__] = ACTIONS(1144),
    [anon_sym___forceinline] = ACTIONS(1144),
    [anon_sym_thread_local] = ACTIONS(1144),
    [anon_sym___thread] = ACTIONS(1144),
    [anon_sym_const] = ACTIONS(1144),
    [anon_sym_constexpr] = ACTIONS(1144),
    [anon_sym_volatile] = ACTIONS(1144),
    [anon_sym_restrict] = ACTIONS(1144),
    [anon_sym___restrict__] = ACTIONS(1144),
    [anon_sym__Atomic] = ACTIONS(1144),
    [anon_sym__Noreturn] = ACTIONS(1144),
    [anon_sym_noreturn] = ACTIONS(1144),
    [sym_primitive_type] = ACTIONS(1144),
    [anon_sym_enum] = ACTIONS(1144),
    [anon_sym_struct] = ACTIONS(1144),
    [anon_sym_union] = ACTIONS(1144),
    [anon_sym_if] = ACTIONS(1144),
    [anon_sym_else] = ACTIONS(1144),
    [anon_sym_switch] = ACTIONS(1144),
    [anon_sym_case] = ACTIONS(1144),
    [anon_sym_default] = ACTIONS(1144),
    [anon_sym_while] = ACTIONS(1144),
    [anon_sym_do] = ACTIONS(1144),
    [anon_sym_for] = ACTIONS(1144),
    [anon_sym_return] = ACTIONS(1144),
    [anon_sym_break] = ACTIONS(1144),
    [anon_sym_continue] = ACTIONS(1144),
    [anon_sym_goto] = ACTIONS(1144),
    [anon_sym___try] = ACTIONS(1144),
    [anon_sym___leave] = ACTIONS(1144),
    [anon_sym_DASH_DASH] = ACTIONS(1146),
    [anon_sym_PLUS_PLUS] = ACTIONS(1146),
    [anon_sym_sizeof] = ACTIONS(1144),
    [anon_sym___alignof__] = ACTIONS(1144),
    [anon_sym___alignof] = ACTIONS(1144),
    [anon_sym__alignof] = ACTIONS(1144),
    [anon_sym_alignof] = ACTIONS(1144),
    [anon_sym__Alignof] = ACTIONS(1144),
    [anon_sym_offsetof] = ACTIONS(1144),
    [anon_sym__Generic] = ACTIONS(1144),
    [anon_sym_asm] = ACTIONS(1144),
    [anon_sym___asm__] = ACTIONS(1144),
    [sym_number_literal] = ACTIONS(1146),
    [anon_sym_L_SQUOTE] = ACTIONS(1146),
    [anon_sym_u_SQUOTE] = ACTIONS(1146),
    [anon_sym_U_SQUOTE] = ACTIONS(1146),
    [anon_sym_u8_SQUOTE] = ACTIONS(1146),
    [anon_sym_SQUOTE] = ACTIONS(1146),
    [anon_sym_L_DQUOTE] = ACTIONS(1146),
    [anon_sym_u_DQUOTE] = ACTIONS(1146),
    [anon_sym_U_DQUOTE] = ACTIONS(1146),
    [anon_sym_u8_DQUOTE] = ACTIONS(1146),
    [anon_sym_DQUOTE] = ACTIONS(1146),
    [sym_true] = ACTIONS(1144),
    [sym_false] = ACTIONS(1144),
    [anon_sym_NULL] = ACTIONS(1144),
    [anon_sym_nullptr] = ACTIONS(1144),
    [sym_comment] = ACTIONS(3),
  },
  [194] = {
    [ts_builtin_sym_end] = ACTIONS(1178),
    [sym_identifier] = ACTIONS(1176),
    [aux_sym_preproc_include_token1] = ACTIONS(1176),
    [aux_sym_preproc_def_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1176),
    [sym_preproc_directive] = ACTIONS(1176),
    [anon_sym_LPAREN2] = ACTIONS(1178),
    [anon_sym_BANG] = ACTIONS(1178),
    [anon_sym_TILDE] = ACTIONS(1178),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_STAR] = ACTIONS(1178),
    [anon_sym_AMP] = ACTIONS(1178),
    [anon_sym_SEMI] = ACTIONS(1178),
    [anon_sym___extension__] = ACTIONS(1176),
    [anon_sym_typedef] = ACTIONS(1176),
    [anon_sym_extern] = ACTIONS(1176),
    [anon_sym___attribute__] = ACTIONS(1176),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1178),
    [anon_sym___declspec] = ACTIONS(1176),
    [anon_sym___cdecl] = ACTIONS(1176),
    [anon_sym___clrcall] = ACTIONS(1176),
    [anon_sym___stdcall] = ACTIONS(1176),
    [anon_sym___fastcall] = ACTIONS(1176),
    [anon_sym___thiscall] = ACTIONS(1176),
    [anon_sym___vectorcall] = ACTIONS(1176),
    [anon_sym_LBRACE] = ACTIONS(1178),
    [anon_sym_signed] = ACTIONS(1176),
    [anon_sym_unsigned] = ACTIONS(1176),
    [anon_sym_long] = ACTIONS(1176),
    [anon_sym_short] = ACTIONS(1176),
    [anon_sym_static] = ACTIONS(1176),
    [anon_sym_auto] = ACTIONS(1176),
    [anon_sym_register] = ACTIONS(1176),
    [anon_sym_inline] = ACTIONS(1176),
    [anon_sym___inline] = ACTIONS(1176),
    [anon_sym___inline__] = ACTIONS(1176),
    [anon_sym___forceinline] = ACTIONS(1176),
    [anon_sym_thread_local] = ACTIONS(1176),
    [anon_sym___thread] = ACTIONS(1176),
    [anon_sym_const] = ACTIONS(1176),
    [anon_sym_constexpr] = ACTIONS(1176),
    [anon_sym_volatile] = ACTIONS(1176),
    [anon_sym_restrict] = ACTIONS(1176),
    [anon_sym___restrict__] = ACTIONS(1176),
    [anon_sym__Atomic] = ACTIONS(1176),
    [anon_sym__Noreturn] = ACTIONS(1176),
    [anon_sym_noreturn] = ACTIONS(1176),
    [sym_primitive_type] = ACTIONS(1176),
    [anon_sym_enum] = ACTIONS(1176),
    [anon_sym_struct] = ACTIONS(1176),
    [anon_sym_union] = ACTIONS(1176),
    [anon_sym_if] = ACTIONS(1176),
    [anon_sym_else] = ACTIONS(1176),
    [anon_sym_switch] = ACTIONS(1176),
    [anon_sym_case] = ACTIONS(1176),
    [anon_sym_default] = ACTIONS(1176),
    [anon_sym_while] = ACTIONS(1176),
    [anon_sym_do] = ACTIONS(1176),
    [anon_sym_for] = ACTIONS(1176),
    [anon_sym_return] = ACTIONS(1176),
    [anon_sym_break] = ACTIONS(1176),
    [anon_sym_continue] = ACTIONS(1176),
    [anon_sym_goto] = ACTIONS(1176),
    [anon_sym___try] = ACTIONS(1176),
    [anon_sym___leave] = ACTIONS(1176),
    [anon_sym_DASH_DASH] = ACTIONS(1178),
    [anon_sym_PLUS_PLUS] = ACTIONS(1178),
    [anon_sym_sizeof] = ACTIONS(1176),
    [anon_sym___alignof__] = ACTIONS(1176),
    [anon_sym___alignof] = ACTIONS(1176),
    [anon_sym__alignof] = ACTIONS(1176),
    [anon_sym_alignof] = ACTIONS(1176),
    [anon_sym__Alignof] = ACTIONS(1176),
    [anon_sym_offsetof] = ACTIONS(1176),
    [anon_sym__Generic] = ACTIONS(1176),
    [anon_sym_asm] = ACTIONS(1176),
    [anon_sym___asm__] = ACTIONS(1176),
    [sym_number_literal] = ACTIONS(1178),
    [anon_sym_L_SQUOTE] = ACTIONS(1178),
    [anon_sym_u_SQUOTE] = ACTIONS(1178),
    [anon_sym_U_SQUOTE] = ACTIONS(1178),
    [anon_sym_u8_SQUOTE] = ACTIONS(1178),
    [anon_sym_SQUOTE] = ACTIONS(1178),
    [anon_sym_L_DQUOTE] = ACTIONS(1178),
    [anon_sym_u_DQUOTE] = ACTIONS(1178),
    [anon_sym_U_DQUOTE] = ACTIONS(1178),
    [anon_sym_u8_DQUOTE] = ACTIONS(1178),
    [anon_sym_DQUOTE] = ACTIONS(1178),
    [sym_true] = ACTIONS(1176),
    [sym_false] = ACTIONS(1176),
    [anon_sym_NULL] = ACTIONS(1176),
    [anon_sym_nullptr] = ACTIONS(1176),
    [sym_comment] = ACTIONS(3),
  },
  [195] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [196] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [197] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [198] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [199] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [200] = {
    [sym_identifier] = ACTIONS(1180),
    [aux_sym_preproc_include_token1] = ACTIONS(1180),
    [aux_sym_preproc_def_token1] = ACTIONS(1180),
    [aux_sym_preproc_if_token1] = ACTIONS(1180),
    [aux_sym_preproc_if_token2] = ACTIONS(1180),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1180),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1180),
    [sym_preproc_directive] = ACTIONS(1180),
    [anon_sym_LPAREN2] = ACTIONS(1182),
    [anon_sym_BANG] = ACTIONS(1182),
    [anon_sym_TILDE] = ACTIONS(1182),
    [anon_sym_DASH] = ACTIONS(1180),
    [anon_sym_PLUS] = ACTIONS(1180),
    [anon_sym_STAR] = ACTIONS(1182),
    [anon_sym_AMP] = ACTIONS(1182),
    [anon_sym_SEMI] = ACTIONS(1182),
    [anon_sym___extension__] = ACTIONS(1180),
    [anon_sym_typedef] = ACTIONS(1180),
    [anon_sym_extern] = ACTIONS(1180),
    [anon_sym___attribute__] = ACTIONS(1180),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1182),
    [anon_sym___declspec] = ACTIONS(1180),
    [anon_sym___cdecl] = ACTIONS(1180),
    [anon_sym___clrcall] = ACTIONS(1180),
    [anon_sym___stdcall] = ACTIONS(1180),
    [anon_sym___fastcall] = ACTIONS(1180),
    [anon_sym___thiscall] = ACTIONS(1180),
    [anon_sym___vectorcall] = ACTIONS(1180),
    [anon_sym_LBRACE] = ACTIONS(1182),
    [anon_sym_signed] = ACTIONS(1180),
    [anon_sym_unsigned] = ACTIONS(1180),
    [anon_sym_long] = ACTIONS(1180),
    [anon_sym_short] = ACTIONS(1180),
    [anon_sym_static] = ACTIONS(1180),
    [anon_sym_auto] = ACTIONS(1180),
    [anon_sym_register] = ACTIONS(1180),
    [anon_sym_inline] = ACTIONS(1180),
    [anon_sym___inline] = ACTIONS(1180),
    [anon_sym___inline__] = ACTIONS(1180),
    [anon_sym___forceinline] = ACTIONS(1180),
    [anon_sym_thread_local] = ACTIONS(1180),
    [anon_sym___thread] = ACTIONS(1180),
    [anon_sym_const] = ACTIONS(1180),
    [anon_sym_constexpr] = ACTIONS(1180),
    [anon_sym_volatile] = ACTIONS(1180),
    [anon_sym_restrict] = ACTIONS(1180),
    [anon_sym___restrict__] = ACTIONS(1180),
    [anon_sym__Atomic] = ACTIONS(1180),
    [anon_sym__Noreturn] = ACTIONS(1180),
    [anon_sym_noreturn] = ACTIONS(1180),
    [sym_primitive_type] = ACTIONS(1180),
    [anon_sym_enum] = ACTIONS(1180),
    [anon_sym_struct] = ACTIONS(1180),
    [anon_sym_union] = ACTIONS(1180),
    [anon_sym_if] = ACTIONS(1180),
    [anon_sym_else] = ACTIONS(1180),
    [anon_sym_switch] = ACTIONS(1180),
    [anon_sym_case] = ACTIONS(1180),
    [anon_sym_default] = ACTIONS(1180),
    [anon_sym_while] = ACTIONS(1180),
    [anon_sym_do] = ACTIONS(1180),
    [anon_sym_for] = ACTIONS(1180),
    [anon_sym_return] = ACTIONS(1180),
    [anon_sym_break] = ACTIONS(1180),
    [anon_sym_continue] = ACTIONS(1180),
    [anon_sym_goto] = ACTIONS(1180),
    [anon_sym___try] = ACTIONS(1180),
    [anon_sym___leave] = ACTIONS(1180),
    [anon_sym_DASH_DASH] = ACTIONS(1182),
    [anon_sym_PLUS_PLUS] = ACTIONS(1182),
    [anon_sym_sizeof] = ACTIONS(1180),
    [anon_sym___alignof__] = ACTIONS(1180),
    [anon_sym___alignof] = ACTIONS(1180),
    [anon_sym__alignof] = ACTIONS(1180),
    [anon_sym_alignof] = ACTIONS(1180),
    [anon_sym__Alignof] = ACTIONS(1180),
    [anon_sym_offsetof] = ACTIONS(1180),
    [anon_sym__Generic] = ACTIONS(1180),
    [anon_sym_asm] = ACTIONS(1180),
    [anon_sym___asm__] = ACTIONS(1180),
    [sym_number_literal] = ACTIONS(1182),
    [anon_sym_L_SQUOTE] = ACTIONS(1182),
    [anon_sym_u_SQUOTE] = ACTIONS(1182),
    [anon_sym_U_SQUOTE] = ACTIONS(1182),
    [anon_sym_u8_SQUOTE] = ACTIONS(1182),
    [anon_sym_SQUOTE] = ACTIONS(1182),
    [anon_sym_L_DQUOTE] = ACTIONS(1182),
    [anon_sym_u_DQUOTE] = ACTIONS(1182),
    [anon_sym_U_DQUOTE] = ACTIONS(1182),
    [anon_sym_u8_DQUOTE] = ACTIONS(1182),
    [anon_sym_DQUOTE] = ACTIONS(1182),
    [sym_true] = ACTIONS(1180),
    [sym_false] = ACTIONS(1180),
    [anon_sym_NULL] = ACTIONS(1180),
    [anon_sym_nullptr] = ACTIONS(1180),
    [sym_comment] = ACTIONS(3),
  },
  [201] = {
    [sym_identifier] = ACTIONS(1184),
    [aux_sym_preproc_include_token1] = ACTIONS(1184),
    [aux_sym_preproc_def_token1] = ACTIONS(1184),
    [aux_sym_preproc_if_token1] = ACTIONS(1184),
    [aux_sym_preproc_if_token2] = ACTIONS(1184),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1184),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1184),
    [sym_preproc_directive] = ACTIONS(1184),
    [anon_sym_LPAREN2] = ACTIONS(1186),
    [anon_sym_BANG] = ACTIONS(1186),
    [anon_sym_TILDE] = ACTIONS(1186),
    [anon_sym_DASH] = ACTIONS(1184),
    [anon_sym_PLUS] = ACTIONS(1184),
    [anon_sym_STAR] = ACTIONS(1186),
    [anon_sym_AMP] = ACTIONS(1186),
    [anon_sym_SEMI] = ACTIONS(1186),
    [anon_sym___extension__] = ACTIONS(1184),
    [anon_sym_typedef] = ACTIONS(1184),
    [anon_sym_extern] = ACTIONS(1184),
    [anon_sym___attribute__] = ACTIONS(1184),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1186),
    [anon_sym___declspec] = ACTIONS(1184),
    [anon_sym___cdecl] = ACTIONS(1184),
    [anon_sym___clrcall] = ACTIONS(1184),
    [anon_sym___stdcall] = ACTIONS(1184),
    [anon_sym___fastcall] = ACTIONS(1184),
    [anon_sym___thiscall] = ACTIONS(1184),
    [anon_sym___vectorcall] = ACTIONS(1184),
    [anon_sym_LBRACE] = ACTIONS(1186),
    [anon_sym_signed] = ACTIONS(1184),
    [anon_sym_unsigned] = ACTIONS(1184),
    [anon_sym_long] = ACTIONS(1184),
    [anon_sym_short] = ACTIONS(1184),
    [anon_sym_static] = ACTIONS(1184),
    [anon_sym_auto] = ACTIONS(1184),
    [anon_sym_register] = ACTIONS(1184),
    [anon_sym_inline] = ACTIONS(1184),
    [anon_sym___inline] = ACTIONS(1184),
    [anon_sym___inline__] = ACTIONS(1184),
    [anon_sym___forceinline] = ACTIONS(1184),
    [anon_sym_thread_local] = ACTIONS(1184),
    [anon_sym___thread] = ACTIONS(1184),
    [anon_sym_const] = ACTIONS(1184),
    [anon_sym_constexpr] = ACTIONS(1184),
    [anon_sym_volatile] = ACTIONS(1184),
    [anon_sym_restrict] = ACTIONS(1184),
    [anon_sym___restrict__] = ACTIONS(1184),
    [anon_sym__Atomic] = ACTIONS(1184),
    [anon_sym__Noreturn] = ACTIONS(1184),
    [anon_sym_noreturn] = ACTIONS(1184),
    [sym_primitive_type] = ACTIONS(1184),
    [anon_sym_enum] = ACTIONS(1184),
    [anon_sym_struct] = ACTIONS(1184),
    [anon_sym_union] = ACTIONS(1184),
    [anon_sym_if] = ACTIONS(1184),
    [anon_sym_else] = ACTIONS(1184),
    [anon_sym_switch] = ACTIONS(1184),
    [anon_sym_case] = ACTIONS(1184),
    [anon_sym_default] = ACTIONS(1184),
    [anon_sym_while] = ACTIONS(1184),
    [anon_sym_do] = ACTIONS(1184),
    [anon_sym_for] = ACTIONS(1184),
    [anon_sym_return] = ACTIONS(1184),
    [anon_sym_break] = ACTIONS(1184),
    [anon_sym_continue] = ACTIONS(1184),
    [anon_sym_goto] = ACTIONS(1184),
    [anon_sym___try] = ACTIONS(1184),
    [anon_sym___leave] = ACTIONS(1184),
    [anon_sym_DASH_DASH] = ACTIONS(1186),
    [anon_sym_PLUS_PLUS] = ACTIONS(1186),
    [anon_sym_sizeof] = ACTIONS(1184),
    [anon_sym___alignof__] = ACTIONS(1184),
    [anon_sym___alignof] = ACTIONS(1184),
    [anon_sym__alignof] = ACTIONS(1184),
    [anon_sym_alignof] = ACTIONS(1184),
    [anon_sym__Alignof] = ACTIONS(1184),
    [anon_sym_offsetof] = ACTIONS(1184),
    [anon_sym__Generic] = ACTIONS(1184),
    [anon_sym_asm] = ACTIONS(1184),
    [anon_sym___asm__] = ACTIONS(1184),
    [sym_number_literal] = ACTIONS(1186),
    [anon_sym_L_SQUOTE] = ACTIONS(1186),
    [anon_sym_u_SQUOTE] = ACTIONS(1186),
    [anon_sym_U_SQUOTE] = ACTIONS(1186),
    [anon_sym_u8_SQUOTE] = ACTIONS(1186),
    [anon_sym_SQUOTE] = ACTIONS(1186),
    [anon_sym_L_DQUOTE] = ACTIONS(1186),
    [anon_sym_u_DQUOTE] = ACTIONS(1186),
    [anon_sym_U_DQUOTE] = ACTIONS(1186),
    [anon_sym_u8_DQUOTE] = ACTIONS(1186),
    [anon_sym_DQUOTE] = ACTIONS(1186),
    [sym_true] = ACTIONS(1184),
    [sym_false] = ACTIONS(1184),
    [anon_sym_NULL] = ACTIONS(1184),
    [anon_sym_nullptr] = ACTIONS(1184),
    [sym_comment] = ACTIONS(3),
  },
  [202] = {
    [sym_identifier] = ACTIONS(1188),
    [aux_sym_preproc_include_token1] = ACTIONS(1188),
    [aux_sym_preproc_def_token1] = ACTIONS(1188),
    [aux_sym_preproc_if_token1] = ACTIONS(1188),
    [aux_sym_preproc_if_token2] = ACTIONS(1188),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1188),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1188),
    [sym_preproc_directive] = ACTIONS(1188),
    [anon_sym_LPAREN2] = ACTIONS(1190),
    [anon_sym_BANG] = ACTIONS(1190),
    [anon_sym_TILDE] = ACTIONS(1190),
    [anon_sym_DASH] = ACTIONS(1188),
    [anon_sym_PLUS] = ACTIONS(1188),
    [anon_sym_STAR] = ACTIONS(1190),
    [anon_sym_AMP] = ACTIONS(1190),
    [anon_sym_SEMI] = ACTIONS(1190),
    [anon_sym___extension__] = ACTIONS(1188),
    [anon_sym_typedef] = ACTIONS(1188),
    [anon_sym_extern] = ACTIONS(1188),
    [anon_sym___attribute__] = ACTIONS(1188),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1190),
    [anon_sym___declspec] = ACTIONS(1188),
    [anon_sym___cdecl] = ACTIONS(1188),
    [anon_sym___clrcall] = ACTIONS(1188),
    [anon_sym___stdcall] = ACTIONS(1188),
    [anon_sym___fastcall] = ACTIONS(1188),
    [anon_sym___thiscall] = ACTIONS(1188),
    [anon_sym___vectorcall] = ACTIONS(1188),
    [anon_sym_LBRACE] = ACTIONS(1190),
    [anon_sym_signed] = ACTIONS(1188),
    [anon_sym_unsigned] = ACTIONS(1188),
    [anon_sym_long] = ACTIONS(1188),
    [anon_sym_short] = ACTIONS(1188),
    [anon_sym_static] = ACTIONS(1188),
    [anon_sym_auto] = ACTIONS(1188),
    [anon_sym_register] = ACTIONS(1188),
    [anon_sym_inline] = ACTIONS(1188),
    [anon_sym___inline] = ACTIONS(1188),
    [anon_sym___inline__] = ACTIONS(1188),
    [anon_sym___forceinline] = ACTIONS(1188),
    [anon_sym_thread_local] = ACTIONS(1188),
    [anon_sym___thread] = ACTIONS(1188),
    [anon_sym_const] = ACTIONS(1188),
    [anon_sym_constexpr] = ACTIONS(1188),
    [anon_sym_volatile] = ACTIONS(1188),
    [anon_sym_restrict] = ACTIONS(1188),
    [anon_sym___restrict__] = ACTIONS(1188),
    [anon_sym__Atomic] = ACTIONS(1188),
    [anon_sym__Noreturn] = ACTIONS(1188),
    [anon_sym_noreturn] = ACTIONS(1188),
    [sym_primitive_type] = ACTIONS(1188),
    [anon_sym_enum] = ACTIONS(1188),
    [anon_sym_struct] = ACTIONS(1188),
    [anon_sym_union] = ACTIONS(1188),
    [anon_sym_if] = ACTIONS(1188),
    [anon_sym_else] = ACTIONS(1188),
    [anon_sym_switch] = ACTIONS(1188),
    [anon_sym_case] = ACTIONS(1188),
    [anon_sym_default] = ACTIONS(1188),
    [anon_sym_while] = ACTIONS(1188),
    [anon_sym_do] = ACTIONS(1188),
    [anon_sym_for] = ACTIONS(1188),
    [anon_sym_return] = ACTIONS(1188),
    [anon_sym_break] = ACTIONS(1188),
    [anon_sym_continue] = ACTIONS(1188),
    [anon_sym_goto] = ACTIONS(1188),
    [anon_sym___try] = ACTIONS(1188),
    [anon_sym___leave] = ACTIONS(1188),
    [anon_sym_DASH_DASH] = ACTIONS(1190),
    [anon_sym_PLUS_PLUS] = ACTIONS(1190),
    [anon_sym_sizeof] = ACTIONS(1188),
    [anon_sym___alignof__] = ACTIONS(1188),
    [anon_sym___alignof] = ACTIONS(1188),
    [anon_sym__alignof] = ACTIONS(1188),
    [anon_sym_alignof] = ACTIONS(1188),
    [anon_sym__Alignof] = ACTIONS(1188),
    [anon_sym_offsetof] = ACTIONS(1188),
    [anon_sym__Generic] = ACTIONS(1188),
    [anon_sym_asm] = ACTIONS(1188),
    [anon_sym___asm__] = ACTIONS(1188),
    [sym_number_literal] = ACTIONS(1190),
    [anon_sym_L_SQUOTE] = ACTIONS(1190),
    [anon_sym_u_SQUOTE] = ACTIONS(1190),
    [anon_sym_U_SQUOTE] = ACTIONS(1190),
    [anon_sym_u8_SQUOTE] = ACTIONS(1190),
    [anon_sym_SQUOTE] = ACTIONS(1190),
    [anon_sym_L_DQUOTE] = ACTIONS(1190),
    [anon_sym_u_DQUOTE] = ACTIONS(1190),
    [anon_sym_U_DQUOTE] = ACTIONS(1190),
    [anon_sym_u8_DQUOTE] = ACTIONS(1190),
    [anon_sym_DQUOTE] = ACTIONS(1190),
    [sym_true] = ACTIONS(1188),
    [sym_false] = ACTIONS(1188),
    [anon_sym_NULL] = ACTIONS(1188),
    [anon_sym_nullptr] = ACTIONS(1188),
    [sym_comment] = ACTIONS(3),
  },
  [203] = {
    [sym_identifier] = ACTIONS(1192),
    [aux_sym_preproc_include_token1] = ACTIONS(1192),
    [aux_sym_preproc_def_token1] = ACTIONS(1192),
    [aux_sym_preproc_if_token1] = ACTIONS(1192),
    [aux_sym_preproc_if_token2] = ACTIONS(1192),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1192),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1192),
    [sym_preproc_directive] = ACTIONS(1192),
    [anon_sym_LPAREN2] = ACTIONS(1194),
    [anon_sym_BANG] = ACTIONS(1194),
    [anon_sym_TILDE] = ACTIONS(1194),
    [anon_sym_DASH] = ACTIONS(1192),
    [anon_sym_PLUS] = ACTIONS(1192),
    [anon_sym_STAR] = ACTIONS(1194),
    [anon_sym_AMP] = ACTIONS(1194),
    [anon_sym_SEMI] = ACTIONS(1194),
    [anon_sym___extension__] = ACTIONS(1192),
    [anon_sym_typedef] = ACTIONS(1192),
    [anon_sym_extern] = ACTIONS(1192),
    [anon_sym___attribute__] = ACTIONS(1192),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1194),
    [anon_sym___declspec] = ACTIONS(1192),
    [anon_sym___cdecl] = ACTIONS(1192),
    [anon_sym___clrcall] = ACTIONS(1192),
    [anon_sym___stdcall] = ACTIONS(1192),
    [anon_sym___fastcall] = ACTIONS(1192),
    [anon_sym___thiscall] = ACTIONS(1192),
    [anon_sym___vectorcall] = ACTIONS(1192),
    [anon_sym_LBRACE] = ACTIONS(1194),
    [anon_sym_signed] = ACTIONS(1192),
    [anon_sym_unsigned] = ACTIONS(1192),
    [anon_sym_long] = ACTIONS(1192),
    [anon_sym_short] = ACTIONS(1192),
    [anon_sym_static] = ACTIONS(1192),
    [anon_sym_auto] = ACTIONS(1192),
    [anon_sym_register] = ACTIONS(1192),
    [anon_sym_inline] = ACTIONS(1192),
    [anon_sym___inline] = ACTIONS(1192),
    [anon_sym___inline__] = ACTIONS(1192),
    [anon_sym___forceinline] = ACTIONS(1192),
    [anon_sym_thread_local] = ACTIONS(1192),
    [anon_sym___thread] = ACTIONS(1192),
    [anon_sym_const] = ACTIONS(1192),
    [anon_sym_constexpr] = ACTIONS(1192),
    [anon_sym_volatile] = ACTIONS(1192),
    [anon_sym_restrict] = ACTIONS(1192),
    [anon_sym___restrict__] = ACTIONS(1192),
    [anon_sym__Atomic] = ACTIONS(1192),
    [anon_sym__Noreturn] = ACTIONS(1192),
    [anon_sym_noreturn] = ACTIONS(1192),
    [sym_primitive_type] = ACTIONS(1192),
    [anon_sym_enum] = ACTIONS(1192),
    [anon_sym_struct] = ACTIONS(1192),
    [anon_sym_union] = ACTIONS(1192),
    [anon_sym_if] = ACTIONS(1192),
    [anon_sym_else] = ACTIONS(1192),
    [anon_sym_switch] = ACTIONS(1192),
    [anon_sym_case] = ACTIONS(1192),
    [anon_sym_default] = ACTIONS(1192),
    [anon_sym_while] = ACTIONS(1192),
    [anon_sym_do] = ACTIONS(1192),
    [anon_sym_for] = ACTIONS(1192),
    [anon_sym_return] = ACTIONS(1192),
    [anon_sym_break] = ACTIONS(1192),
    [anon_sym_continue] = ACTIONS(1192),
    [anon_sym_goto] = ACTIONS(1192),
    [anon_sym___try] = ACTIONS(1192),
    [anon_sym___leave] = ACTIONS(1192),
    [anon_sym_DASH_DASH] = ACTIONS(1194),
    [anon_sym_PLUS_PLUS] = ACTIONS(1194),
    [anon_sym_sizeof] = ACTIONS(1192),
    [anon_sym___alignof__] = ACTIONS(1192),
    [anon_sym___alignof] = ACTIONS(1192),
    [anon_sym__alignof] = ACTIONS(1192),
    [anon_sym_alignof] = ACTIONS(1192),
    [anon_sym__Alignof] = ACTIONS(1192),
    [anon_sym_offsetof] = ACTIONS(1192),
    [anon_sym__Generic] = ACTIONS(1192),
    [anon_sym_asm] = ACTIONS(1192),
    [anon_sym___asm__] = ACTIONS(1192),
    [sym_number_literal] = ACTIONS(1194),
    [anon_sym_L_SQUOTE] = ACTIONS(1194),
    [anon_sym_u_SQUOTE] = ACTIONS(1194),
    [anon_sym_U_SQUOTE] = ACTIONS(1194),
    [anon_sym_u8_SQUOTE] = ACTIONS(1194),
    [anon_sym_SQUOTE] = ACTIONS(1194),
    [anon_sym_L_DQUOTE] = ACTIONS(1194),
    [anon_sym_u_DQUOTE] = ACTIONS(1194),
    [anon_sym_U_DQUOTE] = ACTIONS(1194),
    [anon_sym_u8_DQUOTE] = ACTIONS(1194),
    [anon_sym_DQUOTE] = ACTIONS(1194),
    [sym_true] = ACTIONS(1192),
    [sym_false] = ACTIONS(1192),
    [anon_sym_NULL] = ACTIONS(1192),
    [anon_sym_nullptr] = ACTIONS(1192),
    [sym_comment] = ACTIONS(3),
  },
  [204] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [205] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [206] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [207] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [208] = {
    [sym_identifier] = ACTIONS(1196),
    [aux_sym_preproc_include_token1] = ACTIONS(1196),
    [aux_sym_preproc_def_token1] = ACTIONS(1196),
    [aux_sym_preproc_if_token1] = ACTIONS(1196),
    [aux_sym_preproc_if_token2] = ACTIONS(1196),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1196),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1196),
    [sym_preproc_directive] = ACTIONS(1196),
    [anon_sym_LPAREN2] = ACTIONS(1198),
    [anon_sym_BANG] = ACTIONS(1198),
    [anon_sym_TILDE] = ACTIONS(1198),
    [anon_sym_DASH] = ACTIONS(1196),
    [anon_sym_PLUS] = ACTIONS(1196),
    [anon_sym_STAR] = ACTIONS(1198),
    [anon_sym_AMP] = ACTIONS(1198),
    [anon_sym_SEMI] = ACTIONS(1198),
    [anon_sym___extension__] = ACTIONS(1196),
    [anon_sym_typedef] = ACTIONS(1196),
    [anon_sym_extern] = ACTIONS(1196),
    [anon_sym___attribute__] = ACTIONS(1196),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1198),
    [anon_sym___declspec] = ACTIONS(1196),
    [anon_sym___cdecl] = ACTIONS(1196),
    [anon_sym___clrcall] = ACTIONS(1196),
    [anon_sym___stdcall] = ACTIONS(1196),
    [anon_sym___fastcall] = ACTIONS(1196),
    [anon_sym___thiscall] = ACTIONS(1196),
    [anon_sym___vectorcall] = ACTIONS(1196),
    [anon_sym_LBRACE] = ACTIONS(1198),
    [anon_sym_signed] = ACTIONS(1196),
    [anon_sym_unsigned] = ACTIONS(1196),
    [anon_sym_long] = ACTIONS(1196),
    [anon_sym_short] = ACTIONS(1196),
    [anon_sym_static] = ACTIONS(1196),
    [anon_sym_auto] = ACTIONS(1196),
    [anon_sym_register] = ACTIONS(1196),
    [anon_sym_inline] = ACTIONS(1196),
    [anon_sym___inline] = ACTIONS(1196),
    [anon_sym___inline__] = ACTIONS(1196),
    [anon_sym___forceinline] = ACTIONS(1196),
    [anon_sym_thread_local] = ACTIONS(1196),
    [anon_sym___thread] = ACTIONS(1196),
    [anon_sym_const] = ACTIONS(1196),
    [anon_sym_constexpr] = ACTIONS(1196),
    [anon_sym_volatile] = ACTIONS(1196),
    [anon_sym_restrict] = ACTIONS(1196),
    [anon_sym___restrict__] = ACTIONS(1196),
    [anon_sym__Atomic] = ACTIONS(1196),
    [anon_sym__Noreturn] = ACTIONS(1196),
    [anon_sym_noreturn] = ACTIONS(1196),
    [sym_primitive_type] = ACTIONS(1196),
    [anon_sym_enum] = ACTIONS(1196),
    [anon_sym_struct] = ACTIONS(1196),
    [anon_sym_union] = ACTIONS(1196),
    [anon_sym_if] = ACTIONS(1196),
    [anon_sym_else] = ACTIONS(1196),
    [anon_sym_switch] = ACTIONS(1196),
    [anon_sym_case] = ACTIONS(1196),
    [anon_sym_default] = ACTIONS(1196),
    [anon_sym_while] = ACTIONS(1196),
    [anon_sym_do] = ACTIONS(1196),
    [anon_sym_for] = ACTIONS(1196),
    [anon_sym_return] = ACTIONS(1196),
    [anon_sym_break] = ACTIONS(1196),
    [anon_sym_continue] = ACTIONS(1196),
    [anon_sym_goto] = ACTIONS(1196),
    [anon_sym___try] = ACTIONS(1196),
    [anon_sym___leave] = ACTIONS(1196),
    [anon_sym_DASH_DASH] = ACTIONS(1198),
    [anon_sym_PLUS_PLUS] = ACTIONS(1198),
    [anon_sym_sizeof] = ACTIONS(1196),
    [anon_sym___alignof__] = ACTIONS(1196),
    [anon_sym___alignof] = ACTIONS(1196),
    [anon_sym__alignof] = ACTIONS(1196),
    [anon_sym_alignof] = ACTIONS(1196),
    [anon_sym__Alignof] = ACTIONS(1196),
    [anon_sym_offsetof] = ACTIONS(1196),
    [anon_sym__Generic] = ACTIONS(1196),
    [anon_sym_asm] = ACTIONS(1196),
    [anon_sym___asm__] = ACTIONS(1196),
    [sym_number_literal] = ACTIONS(1198),
    [anon_sym_L_SQUOTE] = ACTIONS(1198),
    [anon_sym_u_SQUOTE] = ACTIONS(1198),
    [anon_sym_U_SQUOTE] = ACTIONS(1198),
    [anon_sym_u8_SQUOTE] = ACTIONS(1198),
    [anon_sym_SQUOTE] = ACTIONS(1198),
    [anon_sym_L_DQUOTE] = ACTIONS(1198),
    [anon_sym_u_DQUOTE] = ACTIONS(1198),
    [anon_sym_U_DQUOTE] = ACTIONS(1198),
    [anon_sym_u8_DQUOTE] = ACTIONS(1198),
    [anon_sym_DQUOTE] = ACTIONS(1198),
    [sym_true] = ACTIONS(1196),
    [sym_false] = ACTIONS(1196),
    [anon_sym_NULL] = ACTIONS(1196),
    [anon_sym_nullptr] = ACTIONS(1196),
    [sym_comment] = ACTIONS(3),
  },
  [209] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [210] = {
    [sym_identifier] = ACTIONS(1204),
    [aux_sym_preproc_include_token1] = ACTIONS(1204),
    [aux_sym_preproc_def_token1] = ACTIONS(1204),
    [aux_sym_preproc_if_token1] = ACTIONS(1204),
    [aux_sym_preproc_if_token2] = ACTIONS(1204),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1204),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1204),
    [sym_preproc_directive] = ACTIONS(1204),
    [anon_sym_LPAREN2] = ACTIONS(1206),
    [anon_sym_BANG] = ACTIONS(1206),
    [anon_sym_TILDE] = ACTIONS(1206),
    [anon_sym_DASH] = ACTIONS(1204),
    [anon_sym_PLUS] = ACTIONS(1204),
    [anon_sym_STAR] = ACTIONS(1206),
    [anon_sym_AMP] = ACTIONS(1206),
    [anon_sym_SEMI] = ACTIONS(1206),
    [anon_sym___extension__] = ACTIONS(1204),
    [anon_sym_typedef] = ACTIONS(1204),
    [anon_sym_extern] = ACTIONS(1204),
    [anon_sym___attribute__] = ACTIONS(1204),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1206),
    [anon_sym___declspec] = ACTIONS(1204),
    [anon_sym___cdecl] = ACTIONS(1204),
    [anon_sym___clrcall] = ACTIONS(1204),
    [anon_sym___stdcall] = ACTIONS(1204),
    [anon_sym___fastcall] = ACTIONS(1204),
    [anon_sym___thiscall] = ACTIONS(1204),
    [anon_sym___vectorcall] = ACTIONS(1204),
    [anon_sym_LBRACE] = ACTIONS(1206),
    [anon_sym_signed] = ACTIONS(1204),
    [anon_sym_unsigned] = ACTIONS(1204),
    [anon_sym_long] = ACTIONS(1204),
    [anon_sym_short] = ACTIONS(1204),
    [anon_sym_static] = ACTIONS(1204),
    [anon_sym_auto] = ACTIONS(1204),
    [anon_sym_register] = ACTIONS(1204),
    [anon_sym_inline] = ACTIONS(1204),
    [anon_sym___inline] = ACTIONS(1204),
    [anon_sym___inline__] = ACTIONS(1204),
    [anon_sym___forceinline] = ACTIONS(1204),
    [anon_sym_thread_local] = ACTIONS(1204),
    [anon_sym___thread] = ACTIONS(1204),
    [anon_sym_const] = ACTIONS(1204),
    [anon_sym_constexpr] = ACTIONS(1204),
    [anon_sym_volatile] = ACTIONS(1204),
    [anon_sym_restrict] = ACTIONS(1204),
    [anon_sym___restrict__] = ACTIONS(1204),
    [anon_sym__Atomic] = ACTIONS(1204),
    [anon_sym__Noreturn] = ACTIONS(1204),
    [anon_sym_noreturn] = ACTIONS(1204),
    [sym_primitive_type] = ACTIONS(1204),
    [anon_sym_enum] = ACTIONS(1204),
    [anon_sym_struct] = ACTIONS(1204),
    [anon_sym_union] = ACTIONS(1204),
    [anon_sym_if] = ACTIONS(1204),
    [anon_sym_else] = ACTIONS(1204),
    [anon_sym_switch] = ACTIONS(1204),
    [anon_sym_case] = ACTIONS(1204),
    [anon_sym_default] = ACTIONS(1204),
    [anon_sym_while] = ACTIONS(1204),
    [anon_sym_do] = ACTIONS(1204),
    [anon_sym_for] = ACTIONS(1204),
    [anon_sym_return] = ACTIONS(1204),
    [anon_sym_break] = ACTIONS(1204),
    [anon_sym_continue] = ACTIONS(1204),
    [anon_sym_goto] = ACTIONS(1204),
    [anon_sym___try] = ACTIONS(1204),
    [anon_sym___leave] = ACTIONS(1204),
    [anon_sym_DASH_DASH] = ACTIONS(1206),
    [anon_sym_PLUS_PLUS] = ACTIONS(1206),
    [anon_sym_sizeof] = ACTIONS(1204),
    [anon_sym___alignof__] = ACTIONS(1204),
    [anon_sym___alignof] = ACTIONS(1204),
    [anon_sym__alignof] = ACTIONS(1204),
    [anon_sym_alignof] = ACTIONS(1204),
    [anon_sym__Alignof] = ACTIONS(1204),
    [anon_sym_offsetof] = ACTIONS(1204),
    [anon_sym__Generic] = ACTIONS(1204),
    [anon_sym_asm] = ACTIONS(1204),
    [anon_sym___asm__] = ACTIONS(1204),
    [sym_number_literal] = ACTIONS(1206),
    [anon_sym_L_SQUOTE] = ACTIONS(1206),
    [anon_sym_u_SQUOTE] = ACTIONS(1206),
    [anon_sym_U_SQUOTE] = ACTIONS(1206),
    [anon_sym_u8_SQUOTE] = ACTIONS(1206),
    [anon_sym_SQUOTE] = ACTIONS(1206),
    [anon_sym_L_DQUOTE] = ACTIONS(1206),
    [anon_sym_u_DQUOTE] = ACTIONS(1206),
    [anon_sym_U_DQUOTE] = ACTIONS(1206),
    [anon_sym_u8_DQUOTE] = ACTIONS(1206),
    [anon_sym_DQUOTE] = ACTIONS(1206),
    [sym_true] = ACTIONS(1204),
    [sym_false] = ACTIONS(1204),
    [anon_sym_NULL] = ACTIONS(1204),
    [anon_sym_nullptr] = ACTIONS(1204),
    [sym_comment] = ACTIONS(3),
  },
  [211] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [212] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [213] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [214] = {
    [sym_identifier] = ACTIONS(1208),
    [aux_sym_preproc_include_token1] = ACTIONS(1208),
    [aux_sym_preproc_def_token1] = ACTIONS(1208),
    [aux_sym_preproc_if_token1] = ACTIONS(1208),
    [aux_sym_preproc_if_token2] = ACTIONS(1208),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1208),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1208),
    [sym_preproc_directive] = ACTIONS(1208),
    [anon_sym_LPAREN2] = ACTIONS(1210),
    [anon_sym_BANG] = ACTIONS(1210),
    [anon_sym_TILDE] = ACTIONS(1210),
    [anon_sym_DASH] = ACTIONS(1208),
    [anon_sym_PLUS] = ACTIONS(1208),
    [anon_sym_STAR] = ACTIONS(1210),
    [anon_sym_AMP] = ACTIONS(1210),
    [anon_sym_SEMI] = ACTIONS(1210),
    [anon_sym___extension__] = ACTIONS(1208),
    [anon_sym_typedef] = ACTIONS(1208),
    [anon_sym_extern] = ACTIONS(1208),
    [anon_sym___attribute__] = ACTIONS(1208),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1210),
    [anon_sym___declspec] = ACTIONS(1208),
    [anon_sym___cdecl] = ACTIONS(1208),
    [anon_sym___clrcall] = ACTIONS(1208),
    [anon_sym___stdcall] = ACTIONS(1208),
    [anon_sym___fastcall] = ACTIONS(1208),
    [anon_sym___thiscall] = ACTIONS(1208),
    [anon_sym___vectorcall] = ACTIONS(1208),
    [anon_sym_LBRACE] = ACTIONS(1210),
    [anon_sym_signed] = ACTIONS(1208),
    [anon_sym_unsigned] = ACTIONS(1208),
    [anon_sym_long] = ACTIONS(1208),
    [anon_sym_short] = ACTIONS(1208),
    [anon_sym_static] = ACTIONS(1208),
    [anon_sym_auto] = ACTIONS(1208),
    [anon_sym_register] = ACTIONS(1208),
    [anon_sym_inline] = ACTIONS(1208),
    [anon_sym___inline] = ACTIONS(1208),
    [anon_sym___inline__] = ACTIONS(1208),
    [anon_sym___forceinline] = ACTIONS(1208),
    [anon_sym_thread_local] = ACTIONS(1208),
    [anon_sym___thread] = ACTIONS(1208),
    [anon_sym_const] = ACTIONS(1208),
    [anon_sym_constexpr] = ACTIONS(1208),
    [anon_sym_volatile] = ACTIONS(1208),
    [anon_sym_restrict] = ACTIONS(1208),
    [anon_sym___restrict__] = ACTIONS(1208),
    [anon_sym__Atomic] = ACTIONS(1208),
    [anon_sym__Noreturn] = ACTIONS(1208),
    [anon_sym_noreturn] = ACTIONS(1208),
    [sym_primitive_type] = ACTIONS(1208),
    [anon_sym_enum] = ACTIONS(1208),
    [anon_sym_struct] = ACTIONS(1208),
    [anon_sym_union] = ACTIONS(1208),
    [anon_sym_if] = ACTIONS(1208),
    [anon_sym_else] = ACTIONS(1208),
    [anon_sym_switch] = ACTIONS(1208),
    [anon_sym_case] = ACTIONS(1208),
    [anon_sym_default] = ACTIONS(1208),
    [anon_sym_while] = ACTIONS(1208),
    [anon_sym_do] = ACTIONS(1208),
    [anon_sym_for] = ACTIONS(1208),
    [anon_sym_return] = ACTIONS(1208),
    [anon_sym_break] = ACTIONS(1208),
    [anon_sym_continue] = ACTIONS(1208),
    [anon_sym_goto] = ACTIONS(1208),
    [anon_sym___try] = ACTIONS(1208),
    [anon_sym___leave] = ACTIONS(1208),
    [anon_sym_DASH_DASH] = ACTIONS(1210),
    [anon_sym_PLUS_PLUS] = ACTIONS(1210),
    [anon_sym_sizeof] = ACTIONS(1208),
    [anon_sym___alignof__] = ACTIONS(1208),
    [anon_sym___alignof] = ACTIONS(1208),
    [anon_sym__alignof] = ACTIONS(1208),
    [anon_sym_alignof] = ACTIONS(1208),
    [anon_sym__Alignof] = ACTIONS(1208),
    [anon_sym_offsetof] = ACTIONS(1208),
    [anon_sym__Generic] = ACTIONS(1208),
    [anon_sym_asm] = ACTIONS(1208),
    [anon_sym___asm__] = ACTIONS(1208),
    [sym_number_literal] = ACTIONS(1210),
    [anon_sym_L_SQUOTE] = ACTIONS(1210),
    [anon_sym_u_SQUOTE] = ACTIONS(1210),
    [anon_sym_U_SQUOTE] = ACTIONS(1210),
    [anon_sym_u8_SQUOTE] = ACTIONS(1210),
    [anon_sym_SQUOTE] = ACTIONS(1210),
    [anon_sym_L_DQUOTE] = ACTIONS(1210),
    [anon_sym_u_DQUOTE] = ACTIONS(1210),
    [anon_sym_U_DQUOTE] = ACTIONS(1210),
    [anon_sym_u8_DQUOTE] = ACTIONS(1210),
    [anon_sym_DQUOTE] = ACTIONS(1210),
    [sym_true] = ACTIONS(1208),
    [sym_false] = ACTIONS(1208),
    [anon_sym_NULL] = ACTIONS(1208),
    [anon_sym_nullptr] = ACTIONS(1208),
    [sym_comment] = ACTIONS(3),
  },
  [215] = {
    [ts_builtin_sym_end] = ACTIONS(1118),
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [216] = {
    [sym_identifier] = ACTIONS(1224),
    [aux_sym_preproc_include_token1] = ACTIONS(1224),
    [aux_sym_preproc_def_token1] = ACTIONS(1224),
    [aux_sym_preproc_if_token1] = ACTIONS(1224),
    [aux_sym_preproc_if_token2] = ACTIONS(1224),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1224),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1224),
    [sym_preproc_directive] = ACTIONS(1224),
    [anon_sym_LPAREN2] = ACTIONS(1226),
    [anon_sym_BANG] = ACTIONS(1226),
    [anon_sym_TILDE] = ACTIONS(1226),
    [anon_sym_DASH] = ACTIONS(1224),
    [anon_sym_PLUS] = ACTIONS(1224),
    [anon_sym_STAR] = ACTIONS(1226),
    [anon_sym_AMP] = ACTIONS(1226),
    [anon_sym_SEMI] = ACTIONS(1226),
    [anon_sym___extension__] = ACTIONS(1224),
    [anon_sym_typedef] = ACTIONS(1224),
    [anon_sym_extern] = ACTIONS(1224),
    [anon_sym___attribute__] = ACTIONS(1224),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1226),
    [anon_sym___declspec] = ACTIONS(1224),
    [anon_sym___cdecl] = ACTIONS(1224),
    [anon_sym___clrcall] = ACTIONS(1224),
    [anon_sym___stdcall] = ACTIONS(1224),
    [anon_sym___fastcall] = ACTIONS(1224),
    [anon_sym___thiscall] = ACTIONS(1224),
    [anon_sym___vectorcall] = ACTIONS(1224),
    [anon_sym_LBRACE] = ACTIONS(1226),
    [anon_sym_signed] = ACTIONS(1224),
    [anon_sym_unsigned] = ACTIONS(1224),
    [anon_sym_long] = ACTIONS(1224),
    [anon_sym_short] = ACTIONS(1224),
    [anon_sym_static] = ACTIONS(1224),
    [anon_sym_auto] = ACTIONS(1224),
    [anon_sym_register] = ACTIONS(1224),
    [anon_sym_inline] = ACTIONS(1224),
    [anon_sym___inline] = ACTIONS(1224),
    [anon_sym___inline__] = ACTIONS(1224),
    [anon_sym___forceinline] = ACTIONS(1224),
    [anon_sym_thread_local] = ACTIONS(1224),
    [anon_sym___thread] = ACTIONS(1224),
    [anon_sym_const] = ACTIONS(1224),
    [anon_sym_constexpr] = ACTIONS(1224),
    [anon_sym_volatile] = ACTIONS(1224),
    [anon_sym_restrict] = ACTIONS(1224),
    [anon_sym___restrict__] = ACTIONS(1224),
    [anon_sym__Atomic] = ACTIONS(1224),
    [anon_sym__Noreturn] = ACTIONS(1224),
    [anon_sym_noreturn] = ACTIONS(1224),
    [sym_primitive_type] = ACTIONS(1224),
    [anon_sym_enum] = ACTIONS(1224),
    [anon_sym_struct] = ACTIONS(1224),
    [anon_sym_union] = ACTIONS(1224),
    [anon_sym_if] = ACTIONS(1224),
    [anon_sym_else] = ACTIONS(1224),
    [anon_sym_switch] = ACTIONS(1224),
    [anon_sym_case] = ACTIONS(1224),
    [anon_sym_default] = ACTIONS(1224),
    [anon_sym_while] = ACTIONS(1224),
    [anon_sym_do] = ACTIONS(1224),
    [anon_sym_for] = ACTIONS(1224),
    [anon_sym_return] = ACTIONS(1224),
    [anon_sym_break] = ACTIONS(1224),
    [anon_sym_continue] = ACTIONS(1224),
    [anon_sym_goto] = ACTIONS(1224),
    [anon_sym___try] = ACTIONS(1224),
    [anon_sym___leave] = ACTIONS(1224),
    [anon_sym_DASH_DASH] = ACTIONS(1226),
    [anon_sym_PLUS_PLUS] = ACTIONS(1226),
    [anon_sym_sizeof] = ACTIONS(1224),
    [anon_sym___alignof__] = ACTIONS(1224),
    [anon_sym___alignof] = ACTIONS(1224),
    [anon_sym__alignof] = ACTIONS(1224),
    [anon_sym_alignof] = ACTIONS(1224),
    [anon_sym__Alignof] = ACTIONS(1224),
    [anon_sym_offsetof] = ACTIONS(1224),
    [anon_sym__Generic] = ACTIONS(1224),
    [anon_sym_asm] = ACTIONS(1224),
    [anon_sym___asm__] = ACTIONS(1224),
    [sym_number_literal] = ACTIONS(1226),
    [anon_sym_L_SQUOTE] = ACTIONS(1226),
    [anon_sym_u_SQUOTE] = ACTIONS(1226),
    [anon_sym_U_SQUOTE] = ACTIONS(1226),
    [anon_sym_u8_SQUOTE] = ACTIONS(1226),
    [anon_sym_SQUOTE] = ACTIONS(1226),
    [anon_sym_L_DQUOTE] = ACTIONS(1226),
    [anon_sym_u_DQUOTE] = ACTIONS(1226),
    [anon_sym_U_DQUOTE] = ACTIONS(1226),
    [anon_sym_u8_DQUOTE] = ACTIONS(1226),
    [anon_sym_DQUOTE] = ACTIONS(1226),
    [sym_true] = ACTIONS(1224),
    [sym_false] = ACTIONS(1224),
    [anon_sym_NULL] = ACTIONS(1224),
    [anon_sym_nullptr] = ACTIONS(1224),
    [sym_comment] = ACTIONS(3),
  },
  [217] = {
    [ts_builtin_sym_end] = ACTIONS(1174),
    [sym_identifier] = ACTIONS(1172),
    [aux_sym_preproc_include_token1] = ACTIONS(1172),
    [aux_sym_preproc_def_token1] = ACTIONS(1172),
    [aux_sym_preproc_if_token1] = ACTIONS(1172),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1172),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1172),
    [sym_preproc_directive] = ACTIONS(1172),
    [anon_sym_LPAREN2] = ACTIONS(1174),
    [anon_sym_BANG] = ACTIONS(1174),
    [anon_sym_TILDE] = ACTIONS(1174),
    [anon_sym_DASH] = ACTIONS(1172),
    [anon_sym_PLUS] = ACTIONS(1172),
    [anon_sym_STAR] = ACTIONS(1174),
    [anon_sym_AMP] = ACTIONS(1174),
    [anon_sym_SEMI] = ACTIONS(1174),
    [anon_sym___extension__] = ACTIONS(1172),
    [anon_sym_typedef] = ACTIONS(1172),
    [anon_sym_extern] = ACTIONS(1172),
    [anon_sym___attribute__] = ACTIONS(1172),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1174),
    [anon_sym___declspec] = ACTIONS(1172),
    [anon_sym___cdecl] = ACTIONS(1172),
    [anon_sym___clrcall] = ACTIONS(1172),
    [anon_sym___stdcall] = ACTIONS(1172),
    [anon_sym___fastcall] = ACTIONS(1172),
    [anon_sym___thiscall] = ACTIONS(1172),
    [anon_sym___vectorcall] = ACTIONS(1172),
    [anon_sym_LBRACE] = ACTIONS(1174),
    [anon_sym_signed] = ACTIONS(1172),
    [anon_sym_unsigned] = ACTIONS(1172),
    [anon_sym_long] = ACTIONS(1172),
    [anon_sym_short] = ACTIONS(1172),
    [anon_sym_static] = ACTIONS(1172),
    [anon_sym_auto] = ACTIONS(1172),
    [anon_sym_register] = ACTIONS(1172),
    [anon_sym_inline] = ACTIONS(1172),
    [anon_sym___inline] = ACTIONS(1172),
    [anon_sym___inline__] = ACTIONS(1172),
    [anon_sym___forceinline] = ACTIONS(1172),
    [anon_sym_thread_local] = ACTIONS(1172),
    [anon_sym___thread] = ACTIONS(1172),
    [anon_sym_const] = ACTIONS(1172),
    [anon_sym_constexpr] = ACTIONS(1172),
    [anon_sym_volatile] = ACTIONS(1172),
    [anon_sym_restrict] = ACTIONS(1172),
    [anon_sym___restrict__] = ACTIONS(1172),
    [anon_sym__Atomic] = ACTIONS(1172),
    [anon_sym__Noreturn] = ACTIONS(1172),
    [anon_sym_noreturn] = ACTIONS(1172),
    [sym_primitive_type] = ACTIONS(1172),
    [anon_sym_enum] = ACTIONS(1172),
    [anon_sym_struct] = ACTIONS(1172),
    [anon_sym_union] = ACTIONS(1172),
    [anon_sym_if] = ACTIONS(1172),
    [anon_sym_else] = ACTIONS(1172),
    [anon_sym_switch] = ACTIONS(1172),
    [anon_sym_case] = ACTIONS(1172),
    [anon_sym_default] = ACTIONS(1172),
    [anon_sym_while] = ACTIONS(1172),
    [anon_sym_do] = ACTIONS(1172),
    [anon_sym_for] = ACTIONS(1172),
    [anon_sym_return] = ACTIONS(1172),
    [anon_sym_break] = ACTIONS(1172),
    [anon_sym_continue] = ACTIONS(1172),
    [anon_sym_goto] = ACTIONS(1172),
    [anon_sym___try] = ACTIONS(1172),
    [anon_sym___leave] = ACTIONS(1172),
    [anon_sym_DASH_DASH] = ACTIONS(1174),
    [anon_sym_PLUS_PLUS] = ACTIONS(1174),
    [anon_sym_sizeof] = ACTIONS(1172),
    [anon_sym___alignof__] = ACTIONS(1172),
    [anon_sym___alignof] = ACTIONS(1172),
    [anon_sym__alignof] = ACTIONS(1172),
    [anon_sym_alignof] = ACTIONS(1172),
    [anon_sym__Alignof] = ACTIONS(1172),
    [anon_sym_offsetof] = ACTIONS(1172),
    [anon_sym__Generic] = ACTIONS(1172),
    [anon_sym_asm] = ACTIONS(1172),
    [anon_sym___asm__] = ACTIONS(1172),
    [sym_number_literal] = ACTIONS(1174),
    [anon_sym_L_SQUOTE] = ACTIONS(1174),
    [anon_sym_u_SQUOTE] = ACTIONS(1174),
    [anon_sym_U_SQUOTE] = ACTIONS(1174),
    [anon_sym_u8_SQUOTE] = ACTIONS(1174),
    [anon_sym_SQUOTE] = ACTIONS(1174),
    [anon_sym_L_DQUOTE] = ACTIONS(1174),
    [anon_sym_u_DQUOTE] = ACTIONS(1174),
    [anon_sym_U_DQUOTE] = ACTIONS(1174),
    [anon_sym_u8_DQUOTE] = ACTIONS(1174),
    [anon_sym_DQUOTE] = ACTIONS(1174),
    [sym_true] = ACTIONS(1172),
    [sym_false] = ACTIONS(1172),
    [anon_sym_NULL] = ACTIONS(1172),
    [anon_sym_nullptr] = ACTIONS(1172),
    [sym_comment] = ACTIONS(3),
  },
  [218] = {
    [ts_builtin_sym_end] = ACTIONS(1170),
    [sym_identifier] = ACTIONS(1168),
    [aux_sym_preproc_include_token1] = ACTIONS(1168),
    [aux_sym_preproc_def_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1168),
    [sym_preproc_directive] = ACTIONS(1168),
    [anon_sym_LPAREN2] = ACTIONS(1170),
    [anon_sym_BANG] = ACTIONS(1170),
    [anon_sym_TILDE] = ACTIONS(1170),
    [anon_sym_DASH] = ACTIONS(1168),
    [anon_sym_PLUS] = ACTIONS(1168),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1170),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym___extension__] = ACTIONS(1168),
    [anon_sym_typedef] = ACTIONS(1168),
    [anon_sym_extern] = ACTIONS(1168),
    [anon_sym___attribute__] = ACTIONS(1168),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1170),
    [anon_sym___declspec] = ACTIONS(1168),
    [anon_sym___cdecl] = ACTIONS(1168),
    [anon_sym___clrcall] = ACTIONS(1168),
    [anon_sym___stdcall] = ACTIONS(1168),
    [anon_sym___fastcall] = ACTIONS(1168),
    [anon_sym___thiscall] = ACTIONS(1168),
    [anon_sym___vectorcall] = ACTIONS(1168),
    [anon_sym_LBRACE] = ACTIONS(1170),
    [anon_sym_signed] = ACTIONS(1168),
    [anon_sym_unsigned] = ACTIONS(1168),
    [anon_sym_long] = ACTIONS(1168),
    [anon_sym_short] = ACTIONS(1168),
    [anon_sym_static] = ACTIONS(1168),
    [anon_sym_auto] = ACTIONS(1168),
    [anon_sym_register] = ACTIONS(1168),
    [anon_sym_inline] = ACTIONS(1168),
    [anon_sym___inline] = ACTIONS(1168),
    [anon_sym___inline__] = ACTIONS(1168),
    [anon_sym___forceinline] = ACTIONS(1168),
    [anon_sym_thread_local] = ACTIONS(1168),
    [anon_sym___thread] = ACTIONS(1168),
    [anon_sym_const] = ACTIONS(1168),
    [anon_sym_constexpr] = ACTIONS(1168),
    [anon_sym_volatile] = ACTIONS(1168),
    [anon_sym_restrict] = ACTIONS(1168),
    [anon_sym___restrict__] = ACTIONS(1168),
    [anon_sym__Atomic] = ACTIONS(1168),
    [anon_sym__Noreturn] = ACTIONS(1168),
    [anon_sym_noreturn] = ACTIONS(1168),
    [sym_primitive_type] = ACTIONS(1168),
    [anon_sym_enum] = ACTIONS(1168),
    [anon_sym_struct] = ACTIONS(1168),
    [anon_sym_union] = ACTIONS(1168),
    [anon_sym_if] = ACTIONS(1168),
    [anon_sym_else] = ACTIONS(1168),
    [anon_sym_switch] = ACTIONS(1168),
    [anon_sym_case] = ACTIONS(1168),
    [anon_sym_default] = ACTIONS(1168),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_do] = ACTIONS(1168),
    [anon_sym_for] = ACTIONS(1168),
    [anon_sym_return] = ACTIONS(1168),
    [anon_sym_break] = ACTIONS(1168),
    [anon_sym_continue] = ACTIONS(1168),
    [anon_sym_goto] = ACTIONS(1168),
    [anon_sym___try] = ACTIONS(1168),
    [anon_sym___leave] = ACTIONS(1168),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_sizeof] = ACTIONS(1168),
    [anon_sym___alignof__] = ACTIONS(1168),
    [anon_sym___alignof] = ACTIONS(1168),
    [anon_sym__alignof] = ACTIONS(1168),
    [anon_sym_alignof] = ACTIONS(1168),
    [anon_sym__Alignof] = ACTIONS(1168),
    [anon_sym_offsetof] = ACTIONS(1168),
    [anon_sym__Generic] = ACTIONS(1168),
    [anon_sym_asm] = ACTIONS(1168),
    [anon_sym___asm__] = ACTIONS(1168),
    [sym_number_literal] = ACTIONS(1170),
    [anon_sym_L_SQUOTE] = ACTIONS(1170),
    [anon_sym_u_SQUOTE] = ACTIONS(1170),
    [anon_sym_U_SQUOTE] = ACTIONS(1170),
    [anon_sym_u8_SQUOTE] = ACTIONS(1170),
    [anon_sym_SQUOTE] = ACTIONS(1170),
    [anon_sym_L_DQUOTE] = ACTIONS(1170),
    [anon_sym_u_DQUOTE] = ACTIONS(1170),
    [anon_sym_U_DQUOTE] = ACTIONS(1170),
    [anon_sym_u8_DQUOTE] = ACTIONS(1170),
    [anon_sym_DQUOTE] = ACTIONS(1170),
    [sym_true] = ACTIONS(1168),
    [sym_false] = ACTIONS(1168),
    [anon_sym_NULL] = ACTIONS(1168),
    [anon_sym_nullptr] = ACTIONS(1168),
    [sym_comment] = ACTIONS(3),
  },
  [219] = {
    [ts_builtin_sym_end] = ACTIONS(1170),
    [sym_identifier] = ACTIONS(1168),
    [aux_sym_preproc_include_token1] = ACTIONS(1168),
    [aux_sym_preproc_def_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1168),
    [sym_preproc_directive] = ACTIONS(1168),
    [anon_sym_LPAREN2] = ACTIONS(1170),
    [anon_sym_BANG] = ACTIONS(1170),
    [anon_sym_TILDE] = ACTIONS(1170),
    [anon_sym_DASH] = ACTIONS(1168),
    [anon_sym_PLUS] = ACTIONS(1168),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1170),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym___extension__] = ACTIONS(1168),
    [anon_sym_typedef] = ACTIONS(1168),
    [anon_sym_extern] = ACTIONS(1168),
    [anon_sym___attribute__] = ACTIONS(1168),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1170),
    [anon_sym___declspec] = ACTIONS(1168),
    [anon_sym___cdecl] = ACTIONS(1168),
    [anon_sym___clrcall] = ACTIONS(1168),
    [anon_sym___stdcall] = ACTIONS(1168),
    [anon_sym___fastcall] = ACTIONS(1168),
    [anon_sym___thiscall] = ACTIONS(1168),
    [anon_sym___vectorcall] = ACTIONS(1168),
    [anon_sym_LBRACE] = ACTIONS(1170),
    [anon_sym_signed] = ACTIONS(1168),
    [anon_sym_unsigned] = ACTIONS(1168),
    [anon_sym_long] = ACTIONS(1168),
    [anon_sym_short] = ACTIONS(1168),
    [anon_sym_static] = ACTIONS(1168),
    [anon_sym_auto] = ACTIONS(1168),
    [anon_sym_register] = ACTIONS(1168),
    [anon_sym_inline] = ACTIONS(1168),
    [anon_sym___inline] = ACTIONS(1168),
    [anon_sym___inline__] = ACTIONS(1168),
    [anon_sym___forceinline] = ACTIONS(1168),
    [anon_sym_thread_local] = ACTIONS(1168),
    [anon_sym___thread] = ACTIONS(1168),
    [anon_sym_const] = ACTIONS(1168),
    [anon_sym_constexpr] = ACTIONS(1168),
    [anon_sym_volatile] = ACTIONS(1168),
    [anon_sym_restrict] = ACTIONS(1168),
    [anon_sym___restrict__] = ACTIONS(1168),
    [anon_sym__Atomic] = ACTIONS(1168),
    [anon_sym__Noreturn] = ACTIONS(1168),
    [anon_sym_noreturn] = ACTIONS(1168),
    [sym_primitive_type] = ACTIONS(1168),
    [anon_sym_enum] = ACTIONS(1168),
    [anon_sym_struct] = ACTIONS(1168),
    [anon_sym_union] = ACTIONS(1168),
    [anon_sym_if] = ACTIONS(1168),
    [anon_sym_else] = ACTIONS(1168),
    [anon_sym_switch] = ACTIONS(1168),
    [anon_sym_case] = ACTIONS(1168),
    [anon_sym_default] = ACTIONS(1168),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_do] = ACTIONS(1168),
    [anon_sym_for] = ACTIONS(1168),
    [anon_sym_return] = ACTIONS(1168),
    [anon_sym_break] = ACTIONS(1168),
    [anon_sym_continue] = ACTIONS(1168),
    [anon_sym_goto] = ACTIONS(1168),
    [anon_sym___try] = ACTIONS(1168),
    [anon_sym___leave] = ACTIONS(1168),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_sizeof] = ACTIONS(1168),
    [anon_sym___alignof__] = ACTIONS(1168),
    [anon_sym___alignof] = ACTIONS(1168),
    [anon_sym__alignof] = ACTIONS(1168),
    [anon_sym_alignof] = ACTIONS(1168),
    [anon_sym__Alignof] = ACTIONS(1168),
    [anon_sym_offsetof] = ACTIONS(1168),
    [anon_sym__Generic] = ACTIONS(1168),
    [anon_sym_asm] = ACTIONS(1168),
    [anon_sym___asm__] = ACTIONS(1168),
    [sym_number_literal] = ACTIONS(1170),
    [anon_sym_L_SQUOTE] = ACTIONS(1170),
    [anon_sym_u_SQUOTE] = ACTIONS(1170),
    [anon_sym_U_SQUOTE] = ACTIONS(1170),
    [anon_sym_u8_SQUOTE] = ACTIONS(1170),
    [anon_sym_SQUOTE] = ACTIONS(1170),
    [anon_sym_L_DQUOTE] = ACTIONS(1170),
    [anon_sym_u_DQUOTE] = ACTIONS(1170),
    [anon_sym_U_DQUOTE] = ACTIONS(1170),
    [anon_sym_u8_DQUOTE] = ACTIONS(1170),
    [anon_sym_DQUOTE] = ACTIONS(1170),
    [sym_true] = ACTIONS(1168),
    [sym_false] = ACTIONS(1168),
    [anon_sym_NULL] = ACTIONS(1168),
    [anon_sym_nullptr] = ACTIONS(1168),
    [sym_comment] = ACTIONS(3),
  },
  [220] = {
    [ts_builtin_sym_end] = ACTIONS(1158),
    [sym_identifier] = ACTIONS(1156),
    [aux_sym_preproc_include_token1] = ACTIONS(1156),
    [aux_sym_preproc_def_token1] = ACTIONS(1156),
    [aux_sym_preproc_if_token1] = ACTIONS(1156),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1156),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1156),
    [sym_preproc_directive] = ACTIONS(1156),
    [anon_sym_LPAREN2] = ACTIONS(1158),
    [anon_sym_BANG] = ACTIONS(1158),
    [anon_sym_TILDE] = ACTIONS(1158),
    [anon_sym_DASH] = ACTIONS(1156),
    [anon_sym_PLUS] = ACTIONS(1156),
    [anon_sym_STAR] = ACTIONS(1158),
    [anon_sym_AMP] = ACTIONS(1158),
    [anon_sym_SEMI] = ACTIONS(1158),
    [anon_sym___extension__] = ACTIONS(1156),
    [anon_sym_typedef] = ACTIONS(1156),
    [anon_sym_extern] = ACTIONS(1156),
    [anon_sym___attribute__] = ACTIONS(1156),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1158),
    [anon_sym___declspec] = ACTIONS(1156),
    [anon_sym___cdecl] = ACTIONS(1156),
    [anon_sym___clrcall] = ACTIONS(1156),
    [anon_sym___stdcall] = ACTIONS(1156),
    [anon_sym___fastcall] = ACTIONS(1156),
    [anon_sym___thiscall] = ACTIONS(1156),
    [anon_sym___vectorcall] = ACTIONS(1156),
    [anon_sym_LBRACE] = ACTIONS(1158),
    [anon_sym_signed] = ACTIONS(1156),
    [anon_sym_unsigned] = ACTIONS(1156),
    [anon_sym_long] = ACTIONS(1156),
    [anon_sym_short] = ACTIONS(1156),
    [anon_sym_static] = ACTIONS(1156),
    [anon_sym_auto] = ACTIONS(1156),
    [anon_sym_register] = ACTIONS(1156),
    [anon_sym_inline] = ACTIONS(1156),
    [anon_sym___inline] = ACTIONS(1156),
    [anon_sym___inline__] = ACTIONS(1156),
    [anon_sym___forceinline] = ACTIONS(1156),
    [anon_sym_thread_local] = ACTIONS(1156),
    [anon_sym___thread] = ACTIONS(1156),
    [anon_sym_const] = ACTIONS(1156),
    [anon_sym_constexpr] = ACTIONS(1156),
    [anon_sym_volatile] = ACTIONS(1156),
    [anon_sym_restrict] = ACTIONS(1156),
    [anon_sym___restrict__] = ACTIONS(1156),
    [anon_sym__Atomic] = ACTIONS(1156),
    [anon_sym__Noreturn] = ACTIONS(1156),
    [anon_sym_noreturn] = ACTIONS(1156),
    [sym_primitive_type] = ACTIONS(1156),
    [anon_sym_enum] = ACTIONS(1156),
    [anon_sym_struct] = ACTIONS(1156),
    [anon_sym_union] = ACTIONS(1156),
    [anon_sym_if] = ACTIONS(1156),
    [anon_sym_else] = ACTIONS(1156),
    [anon_sym_switch] = ACTIONS(1156),
    [anon_sym_case] = ACTIONS(1156),
    [anon_sym_default] = ACTIONS(1156),
    [anon_sym_while] = ACTIONS(1156),
    [anon_sym_do] = ACTIONS(1156),
    [anon_sym_for] = ACTIONS(1156),
    [anon_sym_return] = ACTIONS(1156),
    [anon_sym_break] = ACTIONS(1156),
    [anon_sym_continue] = ACTIONS(1156),
    [anon_sym_goto] = ACTIONS(1156),
    [anon_sym___try] = ACTIONS(1156),
    [anon_sym___leave] = ACTIONS(1156),
    [anon_sym_DASH_DASH] = ACTIONS(1158),
    [anon_sym_PLUS_PLUS] = ACTIONS(1158),
    [anon_sym_sizeof] = ACTIONS(1156),
    [anon_sym___alignof__] = ACTIONS(1156),
    [anon_sym___alignof] = ACTIONS(1156),
    [anon_sym__alignof] = ACTIONS(1156),
    [anon_sym_alignof] = ACTIONS(1156),
    [anon_sym__Alignof] = ACTIONS(1156),
    [anon_sym_offsetof] = ACTIONS(1156),
    [anon_sym__Generic] = ACTIONS(1156),
    [anon_sym_asm] = ACTIONS(1156),
    [anon_sym___asm__] = ACTIONS(1156),
    [sym_number_literal] = ACTIONS(1158),
    [anon_sym_L_SQUOTE] = ACTIONS(1158),
    [anon_sym_u_SQUOTE] = ACTIONS(1158),
    [anon_sym_U_SQUOTE] = ACTIONS(1158),
    [anon_sym_u8_SQUOTE] = ACTIONS(1158),
    [anon_sym_SQUOTE] = ACTIONS(1158),
    [anon_sym_L_DQUOTE] = ACTIONS(1158),
    [anon_sym_u_DQUOTE] = ACTIONS(1158),
    [anon_sym_U_DQUOTE] = ACTIONS(1158),
    [anon_sym_u8_DQUOTE] = ACTIONS(1158),
    [anon_sym_DQUOTE] = ACTIONS(1158),
    [sym_true] = ACTIONS(1156),
    [sym_false] = ACTIONS(1156),
    [anon_sym_NULL] = ACTIONS(1156),
    [anon_sym_nullptr] = ACTIONS(1156),
    [sym_comment] = ACTIONS(3),
  },
  [221] = {
    [sym_identifier] = ACTIONS(1212),
    [aux_sym_preproc_include_token1] = ACTIONS(1212),
    [aux_sym_preproc_def_token1] = ACTIONS(1212),
    [aux_sym_preproc_if_token1] = ACTIONS(1212),
    [aux_sym_preproc_if_token2] = ACTIONS(1212),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1212),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1212),
    [sym_preproc_directive] = ACTIONS(1212),
    [anon_sym_LPAREN2] = ACTIONS(1214),
    [anon_sym_BANG] = ACTIONS(1214),
    [anon_sym_TILDE] = ACTIONS(1214),
    [anon_sym_DASH] = ACTIONS(1212),
    [anon_sym_PLUS] = ACTIONS(1212),
    [anon_sym_STAR] = ACTIONS(1214),
    [anon_sym_AMP] = ACTIONS(1214),
    [anon_sym_SEMI] = ACTIONS(1214),
    [anon_sym___extension__] = ACTIONS(1212),
    [anon_sym_typedef] = ACTIONS(1212),
    [anon_sym_extern] = ACTIONS(1212),
    [anon_sym___attribute__] = ACTIONS(1212),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1214),
    [anon_sym___declspec] = ACTIONS(1212),
    [anon_sym___cdecl] = ACTIONS(1212),
    [anon_sym___clrcall] = ACTIONS(1212),
    [anon_sym___stdcall] = ACTIONS(1212),
    [anon_sym___fastcall] = ACTIONS(1212),
    [anon_sym___thiscall] = ACTIONS(1212),
    [anon_sym___vectorcall] = ACTIONS(1212),
    [anon_sym_LBRACE] = ACTIONS(1214),
    [anon_sym_signed] = ACTIONS(1212),
    [anon_sym_unsigned] = ACTIONS(1212),
    [anon_sym_long] = ACTIONS(1212),
    [anon_sym_short] = ACTIONS(1212),
    [anon_sym_static] = ACTIONS(1212),
    [anon_sym_auto] = ACTIONS(1212),
    [anon_sym_register] = ACTIONS(1212),
    [anon_sym_inline] = ACTIONS(1212),
    [anon_sym___inline] = ACTIONS(1212),
    [anon_sym___inline__] = ACTIONS(1212),
    [anon_sym___forceinline] = ACTIONS(1212),
    [anon_sym_thread_local] = ACTIONS(1212),
    [anon_sym___thread] = ACTIONS(1212),
    [anon_sym_const] = ACTIONS(1212),
    [anon_sym_constexpr] = ACTIONS(1212),
    [anon_sym_volatile] = ACTIONS(1212),
    [anon_sym_restrict] = ACTIONS(1212),
    [anon_sym___restrict__] = ACTIONS(1212),
    [anon_sym__Atomic] = ACTIONS(1212),
    [anon_sym__Noreturn] = ACTIONS(1212),
    [anon_sym_noreturn] = ACTIONS(1212),
    [sym_primitive_type] = ACTIONS(1212),
    [anon_sym_enum] = ACTIONS(1212),
    [anon_sym_struct] = ACTIONS(1212),
    [anon_sym_union] = ACTIONS(1212),
    [anon_sym_if] = ACTIONS(1212),
    [anon_sym_else] = ACTIONS(1212),
    [anon_sym_switch] = ACTIONS(1212),
    [anon_sym_case] = ACTIONS(1212),
    [anon_sym_default] = ACTIONS(1212),
    [anon_sym_while] = ACTIONS(1212),
    [anon_sym_do] = ACTIONS(1212),
    [anon_sym_for] = ACTIONS(1212),
    [anon_sym_return] = ACTIONS(1212),
    [anon_sym_break] = ACTIONS(1212),
    [anon_sym_continue] = ACTIONS(1212),
    [anon_sym_goto] = ACTIONS(1212),
    [anon_sym___try] = ACTIONS(1212),
    [anon_sym___leave] = ACTIONS(1212),
    [anon_sym_DASH_DASH] = ACTIONS(1214),
    [anon_sym_PLUS_PLUS] = ACTIONS(1214),
    [anon_sym_sizeof] = ACTIONS(1212),
    [anon_sym___alignof__] = ACTIONS(1212),
    [anon_sym___alignof] = ACTIONS(1212),
    [anon_sym__alignof] = ACTIONS(1212),
    [anon_sym_alignof] = ACTIONS(1212),
    [anon_sym__Alignof] = ACTIONS(1212),
    [anon_sym_offsetof] = ACTIONS(1212),
    [anon_sym__Generic] = ACTIONS(1212),
    [anon_sym_asm] = ACTIONS(1212),
    [anon_sym___asm__] = ACTIONS(1212),
    [sym_number_literal] = ACTIONS(1214),
    [anon_sym_L_SQUOTE] = ACTIONS(1214),
    [anon_sym_u_SQUOTE] = ACTIONS(1214),
    [anon_sym_U_SQUOTE] = ACTIONS(1214),
    [anon_sym_u8_SQUOTE] = ACTIONS(1214),
    [anon_sym_SQUOTE] = ACTIONS(1214),
    [anon_sym_L_DQUOTE] = ACTIONS(1214),
    [anon_sym_u_DQUOTE] = ACTIONS(1214),
    [anon_sym_U_DQUOTE] = ACTIONS(1214),
    [anon_sym_u8_DQUOTE] = ACTIONS(1214),
    [anon_sym_DQUOTE] = ACTIONS(1214),
    [sym_true] = ACTIONS(1212),
    [sym_false] = ACTIONS(1212),
    [anon_sym_NULL] = ACTIONS(1212),
    [anon_sym_nullptr] = ACTIONS(1212),
    [sym_comment] = ACTIONS(3),
  },
  [222] = {
    [ts_builtin_sym_end] = ACTIONS(1166),
    [sym_identifier] = ACTIONS(1164),
    [aux_sym_preproc_include_token1] = ACTIONS(1164),
    [aux_sym_preproc_def_token1] = ACTIONS(1164),
    [aux_sym_preproc_if_token1] = ACTIONS(1164),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1164),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1164),
    [sym_preproc_directive] = ACTIONS(1164),
    [anon_sym_LPAREN2] = ACTIONS(1166),
    [anon_sym_BANG] = ACTIONS(1166),
    [anon_sym_TILDE] = ACTIONS(1166),
    [anon_sym_DASH] = ACTIONS(1164),
    [anon_sym_PLUS] = ACTIONS(1164),
    [anon_sym_STAR] = ACTIONS(1166),
    [anon_sym_AMP] = ACTIONS(1166),
    [anon_sym_SEMI] = ACTIONS(1166),
    [anon_sym___extension__] = ACTIONS(1164),
    [anon_sym_typedef] = ACTIONS(1164),
    [anon_sym_extern] = ACTIONS(1164),
    [anon_sym___attribute__] = ACTIONS(1164),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1166),
    [anon_sym___declspec] = ACTIONS(1164),
    [anon_sym___cdecl] = ACTIONS(1164),
    [anon_sym___clrcall] = ACTIONS(1164),
    [anon_sym___stdcall] = ACTIONS(1164),
    [anon_sym___fastcall] = ACTIONS(1164),
    [anon_sym___thiscall] = ACTIONS(1164),
    [anon_sym___vectorcall] = ACTIONS(1164),
    [anon_sym_LBRACE] = ACTIONS(1166),
    [anon_sym_signed] = ACTIONS(1164),
    [anon_sym_unsigned] = ACTIONS(1164),
    [anon_sym_long] = ACTIONS(1164),
    [anon_sym_short] = ACTIONS(1164),
    [anon_sym_static] = ACTIONS(1164),
    [anon_sym_auto] = ACTIONS(1164),
    [anon_sym_register] = ACTIONS(1164),
    [anon_sym_inline] = ACTIONS(1164),
    [anon_sym___inline] = ACTIONS(1164),
    [anon_sym___inline__] = ACTIONS(1164),
    [anon_sym___forceinline] = ACTIONS(1164),
    [anon_sym_thread_local] = ACTIONS(1164),
    [anon_sym___thread] = ACTIONS(1164),
    [anon_sym_const] = ACTIONS(1164),
    [anon_sym_constexpr] = ACTIONS(1164),
    [anon_sym_volatile] = ACTIONS(1164),
    [anon_sym_restrict] = ACTIONS(1164),
    [anon_sym___restrict__] = ACTIONS(1164),
    [anon_sym__Atomic] = ACTIONS(1164),
    [anon_sym__Noreturn] = ACTIONS(1164),
    [anon_sym_noreturn] = ACTIONS(1164),
    [sym_primitive_type] = ACTIONS(1164),
    [anon_sym_enum] = ACTIONS(1164),
    [anon_sym_struct] = ACTIONS(1164),
    [anon_sym_union] = ACTIONS(1164),
    [anon_sym_if] = ACTIONS(1164),
    [anon_sym_else] = ACTIONS(1164),
    [anon_sym_switch] = ACTIONS(1164),
    [anon_sym_case] = ACTIONS(1164),
    [anon_sym_default] = ACTIONS(1164),
    [anon_sym_while] = ACTIONS(1164),
    [anon_sym_do] = ACTIONS(1164),
    [anon_sym_for] = ACTIONS(1164),
    [anon_sym_return] = ACTIONS(1164),
    [anon_sym_break] = ACTIONS(1164),
    [anon_sym_continue] = ACTIONS(1164),
    [anon_sym_goto] = ACTIONS(1164),
    [anon_sym___try] = ACTIONS(1164),
    [anon_sym___leave] = ACTIONS(1164),
    [anon_sym_DASH_DASH] = ACTIONS(1166),
    [anon_sym_PLUS_PLUS] = ACTIONS(1166),
    [anon_sym_sizeof] = ACTIONS(1164),
    [anon_sym___alignof__] = ACTIONS(1164),
    [anon_sym___alignof] = ACTIONS(1164),
    [anon_sym__alignof] = ACTIONS(1164),
    [anon_sym_alignof] = ACTIONS(1164),
    [anon_sym__Alignof] = ACTIONS(1164),
    [anon_sym_offsetof] = ACTIONS(1164),
    [anon_sym__Generic] = ACTIONS(1164),
    [anon_sym_asm] = ACTIONS(1164),
    [anon_sym___asm__] = ACTIONS(1164),
    [sym_number_literal] = ACTIONS(1166),
    [anon_sym_L_SQUOTE] = ACTIONS(1166),
    [anon_sym_u_SQUOTE] = ACTIONS(1166),
    [anon_sym_U_SQUOTE] = ACTIONS(1166),
    [anon_sym_u8_SQUOTE] = ACTIONS(1166),
    [anon_sym_SQUOTE] = ACTIONS(1166),
    [anon_sym_L_DQUOTE] = ACTIONS(1166),
    [anon_sym_u_DQUOTE] = ACTIONS(1166),
    [anon_sym_U_DQUOTE] = ACTIONS(1166),
    [anon_sym_u8_DQUOTE] = ACTIONS(1166),
    [anon_sym_DQUOTE] = ACTIONS(1166),
    [sym_true] = ACTIONS(1164),
    [sym_false] = ACTIONS(1164),
    [anon_sym_NULL] = ACTIONS(1164),
    [anon_sym_nullptr] = ACTIONS(1164),
    [sym_comment] = ACTIONS(3),
  },
  [223] = {
    [sym_identifier] = ACTIONS(1160),
    [aux_sym_preproc_include_token1] = ACTIONS(1160),
    [aux_sym_preproc_def_token1] = ACTIONS(1160),
    [aux_sym_preproc_if_token1] = ACTIONS(1160),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1160),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1160),
    [sym_preproc_directive] = ACTIONS(1160),
    [anon_sym_LPAREN2] = ACTIONS(1162),
    [anon_sym_BANG] = ACTIONS(1162),
    [anon_sym_TILDE] = ACTIONS(1162),
    [anon_sym_DASH] = ACTIONS(1160),
    [anon_sym_PLUS] = ACTIONS(1160),
    [anon_sym_STAR] = ACTIONS(1162),
    [anon_sym_AMP] = ACTIONS(1162),
    [anon_sym_SEMI] = ACTIONS(1162),
    [anon_sym___extension__] = ACTIONS(1160),
    [anon_sym_typedef] = ACTIONS(1160),
    [anon_sym_extern] = ACTIONS(1160),
    [anon_sym___attribute__] = ACTIONS(1160),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1162),
    [anon_sym___declspec] = ACTIONS(1160),
    [anon_sym___cdecl] = ACTIONS(1160),
    [anon_sym___clrcall] = ACTIONS(1160),
    [anon_sym___stdcall] = ACTIONS(1160),
    [anon_sym___fastcall] = ACTIONS(1160),
    [anon_sym___thiscall] = ACTIONS(1160),
    [anon_sym___vectorcall] = ACTIONS(1160),
    [anon_sym_LBRACE] = ACTIONS(1162),
    [anon_sym_RBRACE] = ACTIONS(1162),
    [anon_sym_signed] = ACTIONS(1160),
    [anon_sym_unsigned] = ACTIONS(1160),
    [anon_sym_long] = ACTIONS(1160),
    [anon_sym_short] = ACTIONS(1160),
    [anon_sym_static] = ACTIONS(1160),
    [anon_sym_auto] = ACTIONS(1160),
    [anon_sym_register] = ACTIONS(1160),
    [anon_sym_inline] = ACTIONS(1160),
    [anon_sym___inline] = ACTIONS(1160),
    [anon_sym___inline__] = ACTIONS(1160),
    [anon_sym___forceinline] = ACTIONS(1160),
    [anon_sym_thread_local] = ACTIONS(1160),
    [anon_sym___thread] = ACTIONS(1160),
    [anon_sym_const] = ACTIONS(1160),
    [anon_sym_constexpr] = ACTIONS(1160),
    [anon_sym_volatile] = ACTIONS(1160),
    [anon_sym_restrict] = ACTIONS(1160),
    [anon_sym___restrict__] = ACTIONS(1160),
    [anon_sym__Atomic] = ACTIONS(1160),
    [anon_sym__Noreturn] = ACTIONS(1160),
    [anon_sym_noreturn] = ACTIONS(1160),
    [sym_primitive_type] = ACTIONS(1160),
    [anon_sym_enum] = ACTIONS(1160),
    [anon_sym_struct] = ACTIONS(1160),
    [anon_sym_union] = ACTIONS(1160),
    [anon_sym_if] = ACTIONS(1160),
    [anon_sym_else] = ACTIONS(1160),
    [anon_sym_switch] = ACTIONS(1160),
    [anon_sym_case] = ACTIONS(1160),
    [anon_sym_default] = ACTIONS(1160),
    [anon_sym_while] = ACTIONS(1160),
    [anon_sym_do] = ACTIONS(1160),
    [anon_sym_for] = ACTIONS(1160),
    [anon_sym_return] = ACTIONS(1160),
    [anon_sym_break] = ACTIONS(1160),
    [anon_sym_continue] = ACTIONS(1160),
    [anon_sym_goto] = ACTIONS(1160),
    [anon_sym___try] = ACTIONS(1160),
    [anon_sym___leave] = ACTIONS(1160),
    [anon_sym_DASH_DASH] = ACTIONS(1162),
    [anon_sym_PLUS_PLUS] = ACTIONS(1162),
    [anon_sym_sizeof] = ACTIONS(1160),
    [anon_sym___alignof__] = ACTIONS(1160),
    [anon_sym___alignof] = ACTIONS(1160),
    [anon_sym__alignof] = ACTIONS(1160),
    [anon_sym_alignof] = ACTIONS(1160),
    [anon_sym__Alignof] = ACTIONS(1160),
    [anon_sym_offsetof] = ACTIONS(1160),
    [anon_sym__Generic] = ACTIONS(1160),
    [anon_sym_asm] = ACTIONS(1160),
    [anon_sym___asm__] = ACTIONS(1160),
    [sym_number_literal] = ACTIONS(1162),
    [anon_sym_L_SQUOTE] = ACTIONS(1162),
    [anon_sym_u_SQUOTE] = ACTIONS(1162),
    [anon_sym_U_SQUOTE] = ACTIONS(1162),
    [anon_sym_u8_SQUOTE] = ACTIONS(1162),
    [anon_sym_SQUOTE] = ACTIONS(1162),
    [anon_sym_L_DQUOTE] = ACTIONS(1162),
    [anon_sym_u_DQUOTE] = ACTIONS(1162),
    [anon_sym_U_DQUOTE] = ACTIONS(1162),
    [anon_sym_u8_DQUOTE] = ACTIONS(1162),
    [anon_sym_DQUOTE] = ACTIONS(1162),
    [sym_true] = ACTIONS(1160),
    [sym_false] = ACTIONS(1160),
    [anon_sym_NULL] = ACTIONS(1160),
    [anon_sym_nullptr] = ACTIONS(1160),
    [sym_comment] = ACTIONS(3),
  },
  [224] = {
    [sym_identifier] = ACTIONS(1140),
    [aux_sym_preproc_include_token1] = ACTIONS(1140),
    [aux_sym_preproc_def_token1] = ACTIONS(1140),
    [aux_sym_preproc_if_token1] = ACTIONS(1140),
    [aux_sym_preproc_if_token2] = ACTIONS(1140),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1140),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1140),
    [sym_preproc_directive] = ACTIONS(1140),
    [anon_sym_LPAREN2] = ACTIONS(1142),
    [anon_sym_BANG] = ACTIONS(1142),
    [anon_sym_TILDE] = ACTIONS(1142),
    [anon_sym_DASH] = ACTIONS(1140),
    [anon_sym_PLUS] = ACTIONS(1140),
    [anon_sym_STAR] = ACTIONS(1142),
    [anon_sym_AMP] = ACTIONS(1142),
    [anon_sym_SEMI] = ACTIONS(1142),
    [anon_sym___extension__] = ACTIONS(1140),
    [anon_sym_typedef] = ACTIONS(1140),
    [anon_sym_extern] = ACTIONS(1140),
    [anon_sym___attribute__] = ACTIONS(1140),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1142),
    [anon_sym___declspec] = ACTIONS(1140),
    [anon_sym___cdecl] = ACTIONS(1140),
    [anon_sym___clrcall] = ACTIONS(1140),
    [anon_sym___stdcall] = ACTIONS(1140),
    [anon_sym___fastcall] = ACTIONS(1140),
    [anon_sym___thiscall] = ACTIONS(1140),
    [anon_sym___vectorcall] = ACTIONS(1140),
    [anon_sym_LBRACE] = ACTIONS(1142),
    [anon_sym_signed] = ACTIONS(1140),
    [anon_sym_unsigned] = ACTIONS(1140),
    [anon_sym_long] = ACTIONS(1140),
    [anon_sym_short] = ACTIONS(1140),
    [anon_sym_static] = ACTIONS(1140),
    [anon_sym_auto] = ACTIONS(1140),
    [anon_sym_register] = ACTIONS(1140),
    [anon_sym_inline] = ACTIONS(1140),
    [anon_sym___inline] = ACTIONS(1140),
    [anon_sym___inline__] = ACTIONS(1140),
    [anon_sym___forceinline] = ACTIONS(1140),
    [anon_sym_thread_local] = ACTIONS(1140),
    [anon_sym___thread] = ACTIONS(1140),
    [anon_sym_const] = ACTIONS(1140),
    [anon_sym_constexpr] = ACTIONS(1140),
    [anon_sym_volatile] = ACTIONS(1140),
    [anon_sym_restrict] = ACTIONS(1140),
    [anon_sym___restrict__] = ACTIONS(1140),
    [anon_sym__Atomic] = ACTIONS(1140),
    [anon_sym__Noreturn] = ACTIONS(1140),
    [anon_sym_noreturn] = ACTIONS(1140),
    [sym_primitive_type] = ACTIONS(1140),
    [anon_sym_enum] = ACTIONS(1140),
    [anon_sym_struct] = ACTIONS(1140),
    [anon_sym_union] = ACTIONS(1140),
    [anon_sym_if] = ACTIONS(1140),
    [anon_sym_else] = ACTIONS(1140),
    [anon_sym_switch] = ACTIONS(1140),
    [anon_sym_case] = ACTIONS(1140),
    [anon_sym_default] = ACTIONS(1140),
    [anon_sym_while] = ACTIONS(1140),
    [anon_sym_do] = ACTIONS(1140),
    [anon_sym_for] = ACTIONS(1140),
    [anon_sym_return] = ACTIONS(1140),
    [anon_sym_break] = ACTIONS(1140),
    [anon_sym_continue] = ACTIONS(1140),
    [anon_sym_goto] = ACTIONS(1140),
    [anon_sym___try] = ACTIONS(1140),
    [anon_sym___leave] = ACTIONS(1140),
    [anon_sym_DASH_DASH] = ACTIONS(1142),
    [anon_sym_PLUS_PLUS] = ACTIONS(1142),
    [anon_sym_sizeof] = ACTIONS(1140),
    [anon_sym___alignof__] = ACTIONS(1140),
    [anon_sym___alignof] = ACTIONS(1140),
    [anon_sym__alignof] = ACTIONS(1140),
    [anon_sym_alignof] = ACTIONS(1140),
    [anon_sym__Alignof] = ACTIONS(1140),
    [anon_sym_offsetof] = ACTIONS(1140),
    [anon_sym__Generic] = ACTIONS(1140),
    [anon_sym_asm] = ACTIONS(1140),
    [anon_sym___asm__] = ACTIONS(1140),
    [sym_number_literal] = ACTIONS(1142),
    [anon_sym_L_SQUOTE] = ACTIONS(1142),
    [anon_sym_u_SQUOTE] = ACTIONS(1142),
    [anon_sym_U_SQUOTE] = ACTIONS(1142),
    [anon_sym_u8_SQUOTE] = ACTIONS(1142),
    [anon_sym_SQUOTE] = ACTIONS(1142),
    [anon_sym_L_DQUOTE] = ACTIONS(1142),
    [anon_sym_u_DQUOTE] = ACTIONS(1142),
    [anon_sym_U_DQUOTE] = ACTIONS(1142),
    [anon_sym_u8_DQUOTE] = ACTIONS(1142),
    [anon_sym_DQUOTE] = ACTIONS(1142),
    [sym_true] = ACTIONS(1140),
    [sym_false] = ACTIONS(1140),
    [anon_sym_NULL] = ACTIONS(1140),
    [anon_sym_nullptr] = ACTIONS(1140),
    [sym_comment] = ACTIONS(3),
  },
  [225] = {
    [sym_identifier] = ACTIONS(1136),
    [aux_sym_preproc_include_token1] = ACTIONS(1136),
    [aux_sym_preproc_def_token1] = ACTIONS(1136),
    [aux_sym_preproc_if_token1] = ACTIONS(1136),
    [aux_sym_preproc_if_token2] = ACTIONS(1136),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1136),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1136),
    [sym_preproc_directive] = ACTIONS(1136),
    [anon_sym_LPAREN2] = ACTIONS(1138),
    [anon_sym_BANG] = ACTIONS(1138),
    [anon_sym_TILDE] = ACTIONS(1138),
    [anon_sym_DASH] = ACTIONS(1136),
    [anon_sym_PLUS] = ACTIONS(1136),
    [anon_sym_STAR] = ACTIONS(1138),
    [anon_sym_AMP] = ACTIONS(1138),
    [anon_sym_SEMI] = ACTIONS(1138),
    [anon_sym___extension__] = ACTIONS(1136),
    [anon_sym_typedef] = ACTIONS(1136),
    [anon_sym_extern] = ACTIONS(1136),
    [anon_sym___attribute__] = ACTIONS(1136),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1138),
    [anon_sym___declspec] = ACTIONS(1136),
    [anon_sym___cdecl] = ACTIONS(1136),
    [anon_sym___clrcall] = ACTIONS(1136),
    [anon_sym___stdcall] = ACTIONS(1136),
    [anon_sym___fastcall] = ACTIONS(1136),
    [anon_sym___thiscall] = ACTIONS(1136),
    [anon_sym___vectorcall] = ACTIONS(1136),
    [anon_sym_LBRACE] = ACTIONS(1138),
    [anon_sym_signed] = ACTIONS(1136),
    [anon_sym_unsigned] = ACTIONS(1136),
    [anon_sym_long] = ACTIONS(1136),
    [anon_sym_short] = ACTIONS(1136),
    [anon_sym_static] = ACTIONS(1136),
    [anon_sym_auto] = ACTIONS(1136),
    [anon_sym_register] = ACTIONS(1136),
    [anon_sym_inline] = ACTIONS(1136),
    [anon_sym___inline] = ACTIONS(1136),
    [anon_sym___inline__] = ACTIONS(1136),
    [anon_sym___forceinline] = ACTIONS(1136),
    [anon_sym_thread_local] = ACTIONS(1136),
    [anon_sym___thread] = ACTIONS(1136),
    [anon_sym_const] = ACTIONS(1136),
    [anon_sym_constexpr] = ACTIONS(1136),
    [anon_sym_volatile] = ACTIONS(1136),
    [anon_sym_restrict] = ACTIONS(1136),
    [anon_sym___restrict__] = ACTIONS(1136),
    [anon_sym__Atomic] = ACTIONS(1136),
    [anon_sym__Noreturn] = ACTIONS(1136),
    [anon_sym_noreturn] = ACTIONS(1136),
    [sym_primitive_type] = ACTIONS(1136),
    [anon_sym_enum] = ACTIONS(1136),
    [anon_sym_struct] = ACTIONS(1136),
    [anon_sym_union] = ACTIONS(1136),
    [anon_sym_if] = ACTIONS(1136),
    [anon_sym_else] = ACTIONS(1136),
    [anon_sym_switch] = ACTIONS(1136),
    [anon_sym_case] = ACTIONS(1136),
    [anon_sym_default] = ACTIONS(1136),
    [anon_sym_while] = ACTIONS(1136),
    [anon_sym_do] = ACTIONS(1136),
    [anon_sym_for] = ACTIONS(1136),
    [anon_sym_return] = ACTIONS(1136),
    [anon_sym_break] = ACTIONS(1136),
    [anon_sym_continue] = ACTIONS(1136),
    [anon_sym_goto] = ACTIONS(1136),
    [anon_sym___try] = ACTIONS(1136),
    [anon_sym___leave] = ACTIONS(1136),
    [anon_sym_DASH_DASH] = ACTIONS(1138),
    [anon_sym_PLUS_PLUS] = ACTIONS(1138),
    [anon_sym_sizeof] = ACTIONS(1136),
    [anon_sym___alignof__] = ACTIONS(1136),
    [anon_sym___alignof] = ACTIONS(1136),
    [anon_sym__alignof] = ACTIONS(1136),
    [anon_sym_alignof] = ACTIONS(1136),
    [anon_sym__Alignof] = ACTIONS(1136),
    [anon_sym_offsetof] = ACTIONS(1136),
    [anon_sym__Generic] = ACTIONS(1136),
    [anon_sym_asm] = ACTIONS(1136),
    [anon_sym___asm__] = ACTIONS(1136),
    [sym_number_literal] = ACTIONS(1138),
    [anon_sym_L_SQUOTE] = ACTIONS(1138),
    [anon_sym_u_SQUOTE] = ACTIONS(1138),
    [anon_sym_U_SQUOTE] = ACTIONS(1138),
    [anon_sym_u8_SQUOTE] = ACTIONS(1138),
    [anon_sym_SQUOTE] = ACTIONS(1138),
    [anon_sym_L_DQUOTE] = ACTIONS(1138),
    [anon_sym_u_DQUOTE] = ACTIONS(1138),
    [anon_sym_U_DQUOTE] = ACTIONS(1138),
    [anon_sym_u8_DQUOTE] = ACTIONS(1138),
    [anon_sym_DQUOTE] = ACTIONS(1138),
    [sym_true] = ACTIONS(1136),
    [sym_false] = ACTIONS(1136),
    [anon_sym_NULL] = ACTIONS(1136),
    [anon_sym_nullptr] = ACTIONS(1136),
    [sym_comment] = ACTIONS(3),
  },
  [226] = {
    [sym_identifier] = ACTIONS(1132),
    [aux_sym_preproc_include_token1] = ACTIONS(1132),
    [aux_sym_preproc_def_token1] = ACTIONS(1132),
    [aux_sym_preproc_if_token1] = ACTIONS(1132),
    [aux_sym_preproc_if_token2] = ACTIONS(1132),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1132),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1132),
    [sym_preproc_directive] = ACTIONS(1132),
    [anon_sym_LPAREN2] = ACTIONS(1134),
    [anon_sym_BANG] = ACTIONS(1134),
    [anon_sym_TILDE] = ACTIONS(1134),
    [anon_sym_DASH] = ACTIONS(1132),
    [anon_sym_PLUS] = ACTIONS(1132),
    [anon_sym_STAR] = ACTIONS(1134),
    [anon_sym_AMP] = ACTIONS(1134),
    [anon_sym_SEMI] = ACTIONS(1134),
    [anon_sym___extension__] = ACTIONS(1132),
    [anon_sym_typedef] = ACTIONS(1132),
    [anon_sym_extern] = ACTIONS(1132),
    [anon_sym___attribute__] = ACTIONS(1132),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1134),
    [anon_sym___declspec] = ACTIONS(1132),
    [anon_sym___cdecl] = ACTIONS(1132),
    [anon_sym___clrcall] = ACTIONS(1132),
    [anon_sym___stdcall] = ACTIONS(1132),
    [anon_sym___fastcall] = ACTIONS(1132),
    [anon_sym___thiscall] = ACTIONS(1132),
    [anon_sym___vectorcall] = ACTIONS(1132),
    [anon_sym_LBRACE] = ACTIONS(1134),
    [anon_sym_signed] = ACTIONS(1132),
    [anon_sym_unsigned] = ACTIONS(1132),
    [anon_sym_long] = ACTIONS(1132),
    [anon_sym_short] = ACTIONS(1132),
    [anon_sym_static] = ACTIONS(1132),
    [anon_sym_auto] = ACTIONS(1132),
    [anon_sym_register] = ACTIONS(1132),
    [anon_sym_inline] = ACTIONS(1132),
    [anon_sym___inline] = ACTIONS(1132),
    [anon_sym___inline__] = ACTIONS(1132),
    [anon_sym___forceinline] = ACTIONS(1132),
    [anon_sym_thread_local] = ACTIONS(1132),
    [anon_sym___thread] = ACTIONS(1132),
    [anon_sym_const] = ACTIONS(1132),
    [anon_sym_constexpr] = ACTIONS(1132),
    [anon_sym_volatile] = ACTIONS(1132),
    [anon_sym_restrict] = ACTIONS(1132),
    [anon_sym___restrict__] = ACTIONS(1132),
    [anon_sym__Atomic] = ACTIONS(1132),
    [anon_sym__Noreturn] = ACTIONS(1132),
    [anon_sym_noreturn] = ACTIONS(1132),
    [sym_primitive_type] = ACTIONS(1132),
    [anon_sym_enum] = ACTIONS(1132),
    [anon_sym_struct] = ACTIONS(1132),
    [anon_sym_union] = ACTIONS(1132),
    [anon_sym_if] = ACTIONS(1132),
    [anon_sym_else] = ACTIONS(1132),
    [anon_sym_switch] = ACTIONS(1132),
    [anon_sym_case] = ACTIONS(1132),
    [anon_sym_default] = ACTIONS(1132),
    [anon_sym_while] = ACTIONS(1132),
    [anon_sym_do] = ACTIONS(1132),
    [anon_sym_for] = ACTIONS(1132),
    [anon_sym_return] = ACTIONS(1132),
    [anon_sym_break] = ACTIONS(1132),
    [anon_sym_continue] = ACTIONS(1132),
    [anon_sym_goto] = ACTIONS(1132),
    [anon_sym___try] = ACTIONS(1132),
    [anon_sym___leave] = ACTIONS(1132),
    [anon_sym_DASH_DASH] = ACTIONS(1134),
    [anon_sym_PLUS_PLUS] = ACTIONS(1134),
    [anon_sym_sizeof] = ACTIONS(1132),
    [anon_sym___alignof__] = ACTIONS(1132),
    [anon_sym___alignof] = ACTIONS(1132),
    [anon_sym__alignof] = ACTIONS(1132),
    [anon_sym_alignof] = ACTIONS(1132),
    [anon_sym__Alignof] = ACTIONS(1132),
    [anon_sym_offsetof] = ACTIONS(1132),
    [anon_sym__Generic] = ACTIONS(1132),
    [anon_sym_asm] = ACTIONS(1132),
    [anon_sym___asm__] = ACTIONS(1132),
    [sym_number_literal] = ACTIONS(1134),
    [anon_sym_L_SQUOTE] = ACTIONS(1134),
    [anon_sym_u_SQUOTE] = ACTIONS(1134),
    [anon_sym_U_SQUOTE] = ACTIONS(1134),
    [anon_sym_u8_SQUOTE] = ACTIONS(1134),
    [anon_sym_SQUOTE] = ACTIONS(1134),
    [anon_sym_L_DQUOTE] = ACTIONS(1134),
    [anon_sym_u_DQUOTE] = ACTIONS(1134),
    [anon_sym_U_DQUOTE] = ACTIONS(1134),
    [anon_sym_u8_DQUOTE] = ACTIONS(1134),
    [anon_sym_DQUOTE] = ACTIONS(1134),
    [sym_true] = ACTIONS(1132),
    [sym_false] = ACTIONS(1132),
    [anon_sym_NULL] = ACTIONS(1132),
    [anon_sym_nullptr] = ACTIONS(1132),
    [sym_comment] = ACTIONS(3),
  },
  [227] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [228] = {
    [sym_identifier] = ACTIONS(1128),
    [aux_sym_preproc_include_token1] = ACTIONS(1128),
    [aux_sym_preproc_def_token1] = ACTIONS(1128),
    [aux_sym_preproc_if_token1] = ACTIONS(1128),
    [aux_sym_preproc_if_token2] = ACTIONS(1128),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1128),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1128),
    [sym_preproc_directive] = ACTIONS(1128),
    [anon_sym_LPAREN2] = ACTIONS(1130),
    [anon_sym_BANG] = ACTIONS(1130),
    [anon_sym_TILDE] = ACTIONS(1130),
    [anon_sym_DASH] = ACTIONS(1128),
    [anon_sym_PLUS] = ACTIONS(1128),
    [anon_sym_STAR] = ACTIONS(1130),
    [anon_sym_AMP] = ACTIONS(1130),
    [anon_sym_SEMI] = ACTIONS(1130),
    [anon_sym___extension__] = ACTIONS(1128),
    [anon_sym_typedef] = ACTIONS(1128),
    [anon_sym_extern] = ACTIONS(1128),
    [anon_sym___attribute__] = ACTIONS(1128),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1130),
    [anon_sym___declspec] = ACTIONS(1128),
    [anon_sym___cdecl] = ACTIONS(1128),
    [anon_sym___clrcall] = ACTIONS(1128),
    [anon_sym___stdcall] = ACTIONS(1128),
    [anon_sym___fastcall] = ACTIONS(1128),
    [anon_sym___thiscall] = ACTIONS(1128),
    [anon_sym___vectorcall] = ACTIONS(1128),
    [anon_sym_LBRACE] = ACTIONS(1130),
    [anon_sym_signed] = ACTIONS(1128),
    [anon_sym_unsigned] = ACTIONS(1128),
    [anon_sym_long] = ACTIONS(1128),
    [anon_sym_short] = ACTIONS(1128),
    [anon_sym_static] = ACTIONS(1128),
    [anon_sym_auto] = ACTIONS(1128),
    [anon_sym_register] = ACTIONS(1128),
    [anon_sym_inline] = ACTIONS(1128),
    [anon_sym___inline] = ACTIONS(1128),
    [anon_sym___inline__] = ACTIONS(1128),
    [anon_sym___forceinline] = ACTIONS(1128),
    [anon_sym_thread_local] = ACTIONS(1128),
    [anon_sym___thread] = ACTIONS(1128),
    [anon_sym_const] = ACTIONS(1128),
    [anon_sym_constexpr] = ACTIONS(1128),
    [anon_sym_volatile] = ACTIONS(1128),
    [anon_sym_restrict] = ACTIONS(1128),
    [anon_sym___restrict__] = ACTIONS(1128),
    [anon_sym__Atomic] = ACTIONS(1128),
    [anon_sym__Noreturn] = ACTIONS(1128),
    [anon_sym_noreturn] = ACTIONS(1128),
    [sym_primitive_type] = ACTIONS(1128),
    [anon_sym_enum] = ACTIONS(1128),
    [anon_sym_struct] = ACTIONS(1128),
    [anon_sym_union] = ACTIONS(1128),
    [anon_sym_if] = ACTIONS(1128),
    [anon_sym_else] = ACTIONS(1128),
    [anon_sym_switch] = ACTIONS(1128),
    [anon_sym_case] = ACTIONS(1128),
    [anon_sym_default] = ACTIONS(1128),
    [anon_sym_while] = ACTIONS(1128),
    [anon_sym_do] = ACTIONS(1128),
    [anon_sym_for] = ACTIONS(1128),
    [anon_sym_return] = ACTIONS(1128),
    [anon_sym_break] = ACTIONS(1128),
    [anon_sym_continue] = ACTIONS(1128),
    [anon_sym_goto] = ACTIONS(1128),
    [anon_sym___try] = ACTIONS(1128),
    [anon_sym___leave] = ACTIONS(1128),
    [anon_sym_DASH_DASH] = ACTIONS(1130),
    [anon_sym_PLUS_PLUS] = ACTIONS(1130),
    [anon_sym_sizeof] = ACTIONS(1128),
    [anon_sym___alignof__] = ACTIONS(1128),
    [anon_sym___alignof] = ACTIONS(1128),
    [anon_sym__alignof] = ACTIONS(1128),
    [anon_sym_alignof] = ACTIONS(1128),
    [anon_sym__Alignof] = ACTIONS(1128),
    [anon_sym_offsetof] = ACTIONS(1128),
    [anon_sym__Generic] = ACTIONS(1128),
    [anon_sym_asm] = ACTIONS(1128),
    [anon_sym___asm__] = ACTIONS(1128),
    [sym_number_literal] = ACTIONS(1130),
    [anon_sym_L_SQUOTE] = ACTIONS(1130),
    [anon_sym_u_SQUOTE] = ACTIONS(1130),
    [anon_sym_U_SQUOTE] = ACTIONS(1130),
    [anon_sym_u8_SQUOTE] = ACTIONS(1130),
    [anon_sym_SQUOTE] = ACTIONS(1130),
    [anon_sym_L_DQUOTE] = ACTIONS(1130),
    [anon_sym_u_DQUOTE] = ACTIONS(1130),
    [anon_sym_U_DQUOTE] = ACTIONS(1130),
    [anon_sym_u8_DQUOTE] = ACTIONS(1130),
    [anon_sym_DQUOTE] = ACTIONS(1130),
    [sym_true] = ACTIONS(1128),
    [sym_false] = ACTIONS(1128),
    [anon_sym_NULL] = ACTIONS(1128),
    [anon_sym_nullptr] = ACTIONS(1128),
    [sym_comment] = ACTIONS(3),
  },
  [229] = {
    [sym_identifier] = ACTIONS(1124),
    [aux_sym_preproc_include_token1] = ACTIONS(1124),
    [aux_sym_preproc_def_token1] = ACTIONS(1124),
    [aux_sym_preproc_if_token1] = ACTIONS(1124),
    [aux_sym_preproc_if_token2] = ACTIONS(1124),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1124),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1124),
    [sym_preproc_directive] = ACTIONS(1124),
    [anon_sym_LPAREN2] = ACTIONS(1126),
    [anon_sym_BANG] = ACTIONS(1126),
    [anon_sym_TILDE] = ACTIONS(1126),
    [anon_sym_DASH] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1124),
    [anon_sym_STAR] = ACTIONS(1126),
    [anon_sym_AMP] = ACTIONS(1126),
    [anon_sym_SEMI] = ACTIONS(1126),
    [anon_sym___extension__] = ACTIONS(1124),
    [anon_sym_typedef] = ACTIONS(1124),
    [anon_sym_extern] = ACTIONS(1124),
    [anon_sym___attribute__] = ACTIONS(1124),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1126),
    [anon_sym___declspec] = ACTIONS(1124),
    [anon_sym___cdecl] = ACTIONS(1124),
    [anon_sym___clrcall] = ACTIONS(1124),
    [anon_sym___stdcall] = ACTIONS(1124),
    [anon_sym___fastcall] = ACTIONS(1124),
    [anon_sym___thiscall] = ACTIONS(1124),
    [anon_sym___vectorcall] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1126),
    [anon_sym_signed] = ACTIONS(1124),
    [anon_sym_unsigned] = ACTIONS(1124),
    [anon_sym_long] = ACTIONS(1124),
    [anon_sym_short] = ACTIONS(1124),
    [anon_sym_static] = ACTIONS(1124),
    [anon_sym_auto] = ACTIONS(1124),
    [anon_sym_register] = ACTIONS(1124),
    [anon_sym_inline] = ACTIONS(1124),
    [anon_sym___inline] = ACTIONS(1124),
    [anon_sym___inline__] = ACTIONS(1124),
    [anon_sym___forceinline] = ACTIONS(1124),
    [anon_sym_thread_local] = ACTIONS(1124),
    [anon_sym___thread] = ACTIONS(1124),
    [anon_sym_const] = ACTIONS(1124),
    [anon_sym_constexpr] = ACTIONS(1124),
    [anon_sym_volatile] = ACTIONS(1124),
    [anon_sym_restrict] = ACTIONS(1124),
    [anon_sym___restrict__] = ACTIONS(1124),
    [anon_sym__Atomic] = ACTIONS(1124),
    [anon_sym__Noreturn] = ACTIONS(1124),
    [anon_sym_noreturn] = ACTIONS(1124),
    [sym_primitive_type] = ACTIONS(1124),
    [anon_sym_enum] = ACTIONS(1124),
    [anon_sym_struct] = ACTIONS(1124),
    [anon_sym_union] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_else] = ACTIONS(1124),
    [anon_sym_switch] = ACTIONS(1124),
    [anon_sym_case] = ACTIONS(1124),
    [anon_sym_default] = ACTIONS(1124),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_do] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_continue] = ACTIONS(1124),
    [anon_sym_goto] = ACTIONS(1124),
    [anon_sym___try] = ACTIONS(1124),
    [anon_sym___leave] = ACTIONS(1124),
    [anon_sym_DASH_DASH] = ACTIONS(1126),
    [anon_sym_PLUS_PLUS] = ACTIONS(1126),
    [anon_sym_sizeof] = ACTIONS(1124),
    [anon_sym___alignof__] = ACTIONS(1124),
    [anon_sym___alignof] = ACTIONS(1124),
    [anon_sym__alignof] = ACTIONS(1124),
    [anon_sym_alignof] = ACTIONS(1124),
    [anon_sym__Alignof] = ACTIONS(1124),
    [anon_sym_offsetof] = ACTIONS(1124),
    [anon_sym__Generic] = ACTIONS(1124),
    [anon_sym_asm] = ACTIONS(1124),
    [anon_sym___asm__] = ACTIONS(1124),
    [sym_number_literal] = ACTIONS(1126),
    [anon_sym_L_SQUOTE] = ACTIONS(1126),
    [anon_sym_u_SQUOTE] = ACTIONS(1126),
    [anon_sym_U_SQUOTE] = ACTIONS(1126),
    [anon_sym_u8_SQUOTE] = ACTIONS(1126),
    [anon_sym_SQUOTE] = ACTIONS(1126),
    [anon_sym_L_DQUOTE] = ACTIONS(1126),
    [anon_sym_u_DQUOTE] = ACTIONS(1126),
    [anon_sym_U_DQUOTE] = ACTIONS(1126),
    [anon_sym_u8_DQUOTE] = ACTIONS(1126),
    [anon_sym_DQUOTE] = ACTIONS(1126),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [anon_sym_NULL] = ACTIONS(1124),
    [anon_sym_nullptr] = ACTIONS(1124),
    [sym_comment] = ACTIONS(3),
  },
  [230] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [231] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [232] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [233] = {
    [ts_builtin_sym_end] = ACTIONS(1150),
    [sym_identifier] = ACTIONS(1148),
    [aux_sym_preproc_include_token1] = ACTIONS(1148),
    [aux_sym_preproc_def_token1] = ACTIONS(1148),
    [aux_sym_preproc_if_token1] = ACTIONS(1148),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1148),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1148),
    [sym_preproc_directive] = ACTIONS(1148),
    [anon_sym_LPAREN2] = ACTIONS(1150),
    [anon_sym_BANG] = ACTIONS(1150),
    [anon_sym_TILDE] = ACTIONS(1150),
    [anon_sym_DASH] = ACTIONS(1148),
    [anon_sym_PLUS] = ACTIONS(1148),
    [anon_sym_STAR] = ACTIONS(1150),
    [anon_sym_AMP] = ACTIONS(1150),
    [anon_sym_SEMI] = ACTIONS(1150),
    [anon_sym___extension__] = ACTIONS(1148),
    [anon_sym_typedef] = ACTIONS(1148),
    [anon_sym_extern] = ACTIONS(1148),
    [anon_sym___attribute__] = ACTIONS(1148),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1150),
    [anon_sym___declspec] = ACTIONS(1148),
    [anon_sym___cdecl] = ACTIONS(1148),
    [anon_sym___clrcall] = ACTIONS(1148),
    [anon_sym___stdcall] = ACTIONS(1148),
    [anon_sym___fastcall] = ACTIONS(1148),
    [anon_sym___thiscall] = ACTIONS(1148),
    [anon_sym___vectorcall] = ACTIONS(1148),
    [anon_sym_LBRACE] = ACTIONS(1150),
    [anon_sym_signed] = ACTIONS(1148),
    [anon_sym_unsigned] = ACTIONS(1148),
    [anon_sym_long] = ACTIONS(1148),
    [anon_sym_short] = ACTIONS(1148),
    [anon_sym_static] = ACTIONS(1148),
    [anon_sym_auto] = ACTIONS(1148),
    [anon_sym_register] = ACTIONS(1148),
    [anon_sym_inline] = ACTIONS(1148),
    [anon_sym___inline] = ACTIONS(1148),
    [anon_sym___inline__] = ACTIONS(1148),
    [anon_sym___forceinline] = ACTIONS(1148),
    [anon_sym_thread_local] = ACTIONS(1148),
    [anon_sym___thread] = ACTIONS(1148),
    [anon_sym_const] = ACTIONS(1148),
    [anon_sym_constexpr] = ACTIONS(1148),
    [anon_sym_volatile] = ACTIONS(1148),
    [anon_sym_restrict] = ACTIONS(1148),
    [anon_sym___restrict__] = ACTIONS(1148),
    [anon_sym__Atomic] = ACTIONS(1148),
    [anon_sym__Noreturn] = ACTIONS(1148),
    [anon_sym_noreturn] = ACTIONS(1148),
    [sym_primitive_type] = ACTIONS(1148),
    [anon_sym_enum] = ACTIONS(1148),
    [anon_sym_struct] = ACTIONS(1148),
    [anon_sym_union] = ACTIONS(1148),
    [anon_sym_if] = ACTIONS(1148),
    [anon_sym_else] = ACTIONS(1148),
    [anon_sym_switch] = ACTIONS(1148),
    [anon_sym_case] = ACTIONS(1148),
    [anon_sym_default] = ACTIONS(1148),
    [anon_sym_while] = ACTIONS(1148),
    [anon_sym_do] = ACTIONS(1148),
    [anon_sym_for] = ACTIONS(1148),
    [anon_sym_return] = ACTIONS(1148),
    [anon_sym_break] = ACTIONS(1148),
    [anon_sym_continue] = ACTIONS(1148),
    [anon_sym_goto] = ACTIONS(1148),
    [anon_sym___try] = ACTIONS(1148),
    [anon_sym___leave] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1150),
    [anon_sym_PLUS_PLUS] = ACTIONS(1150),
    [anon_sym_sizeof] = ACTIONS(1148),
    [anon_sym___alignof__] = ACTIONS(1148),
    [anon_sym___alignof] = ACTIONS(1148),
    [anon_sym__alignof] = ACTIONS(1148),
    [anon_sym_alignof] = ACTIONS(1148),
    [anon_sym__Alignof] = ACTIONS(1148),
    [anon_sym_offsetof] = ACTIONS(1148),
    [anon_sym__Generic] = ACTIONS(1148),
    [anon_sym_asm] = ACTIONS(1148),
    [anon_sym___asm__] = ACTIONS(1148),
    [sym_number_literal] = ACTIONS(1150),
    [anon_sym_L_SQUOTE] = ACTIONS(1150),
    [anon_sym_u_SQUOTE] = ACTIONS(1150),
    [anon_sym_U_SQUOTE] = ACTIONS(1150),
    [anon_sym_u8_SQUOTE] = ACTIONS(1150),
    [anon_sym_SQUOTE] = ACTIONS(1150),
    [anon_sym_L_DQUOTE] = ACTIONS(1150),
    [anon_sym_u_DQUOTE] = ACTIONS(1150),
    [anon_sym_U_DQUOTE] = ACTIONS(1150),
    [anon_sym_u8_DQUOTE] = ACTIONS(1150),
    [anon_sym_DQUOTE] = ACTIONS(1150),
    [sym_true] = ACTIONS(1148),
    [sym_false] = ACTIONS(1148),
    [anon_sym_NULL] = ACTIONS(1148),
    [anon_sym_nullptr] = ACTIONS(1148),
    [sym_comment] = ACTIONS(3),
  },
  [234] = {
    [ts_builtin_sym_end] = ACTIONS(1126),
    [sym_identifier] = ACTIONS(1124),
    [aux_sym_preproc_include_token1] = ACTIONS(1124),
    [aux_sym_preproc_def_token1] = ACTIONS(1124),
    [aux_sym_preproc_if_token1] = ACTIONS(1124),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1124),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1124),
    [sym_preproc_directive] = ACTIONS(1124),
    [anon_sym_LPAREN2] = ACTIONS(1126),
    [anon_sym_BANG] = ACTIONS(1126),
    [anon_sym_TILDE] = ACTIONS(1126),
    [anon_sym_DASH] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1124),
    [anon_sym_STAR] = ACTIONS(1126),
    [anon_sym_AMP] = ACTIONS(1126),
    [anon_sym_SEMI] = ACTIONS(1126),
    [anon_sym___extension__] = ACTIONS(1124),
    [anon_sym_typedef] = ACTIONS(1124),
    [anon_sym_extern] = ACTIONS(1124),
    [anon_sym___attribute__] = ACTIONS(1124),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1126),
    [anon_sym___declspec] = ACTIONS(1124),
    [anon_sym___cdecl] = ACTIONS(1124),
    [anon_sym___clrcall] = ACTIONS(1124),
    [anon_sym___stdcall] = ACTIONS(1124),
    [anon_sym___fastcall] = ACTIONS(1124),
    [anon_sym___thiscall] = ACTIONS(1124),
    [anon_sym___vectorcall] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1126),
    [anon_sym_signed] = ACTIONS(1124),
    [anon_sym_unsigned] = ACTIONS(1124),
    [anon_sym_long] = ACTIONS(1124),
    [anon_sym_short] = ACTIONS(1124),
    [anon_sym_static] = ACTIONS(1124),
    [anon_sym_auto] = ACTIONS(1124),
    [anon_sym_register] = ACTIONS(1124),
    [anon_sym_inline] = ACTIONS(1124),
    [anon_sym___inline] = ACTIONS(1124),
    [anon_sym___inline__] = ACTIONS(1124),
    [anon_sym___forceinline] = ACTIONS(1124),
    [anon_sym_thread_local] = ACTIONS(1124),
    [anon_sym___thread] = ACTIONS(1124),
    [anon_sym_const] = ACTIONS(1124),
    [anon_sym_constexpr] = ACTIONS(1124),
    [anon_sym_volatile] = ACTIONS(1124),
    [anon_sym_restrict] = ACTIONS(1124),
    [anon_sym___restrict__] = ACTIONS(1124),
    [anon_sym__Atomic] = ACTIONS(1124),
    [anon_sym__Noreturn] = ACTIONS(1124),
    [anon_sym_noreturn] = ACTIONS(1124),
    [sym_primitive_type] = ACTIONS(1124),
    [anon_sym_enum] = ACTIONS(1124),
    [anon_sym_struct] = ACTIONS(1124),
    [anon_sym_union] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_else] = ACTIONS(1124),
    [anon_sym_switch] = ACTIONS(1124),
    [anon_sym_case] = ACTIONS(1124),
    [anon_sym_default] = ACTIONS(1124),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_do] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_continue] = ACTIONS(1124),
    [anon_sym_goto] = ACTIONS(1124),
    [anon_sym___try] = ACTIONS(1124),
    [anon_sym___leave] = ACTIONS(1124),
    [anon_sym_DASH_DASH] = ACTIONS(1126),
    [anon_sym_PLUS_PLUS] = ACTIONS(1126),
    [anon_sym_sizeof] = ACTIONS(1124),
    [anon_sym___alignof__] = ACTIONS(1124),
    [anon_sym___alignof] = ACTIONS(1124),
    [anon_sym__alignof] = ACTIONS(1124),
    [anon_sym_alignof] = ACTIONS(1124),
    [anon_sym__Alignof] = ACTIONS(1124),
    [anon_sym_offsetof] = ACTIONS(1124),
    [anon_sym__Generic] = ACTIONS(1124),
    [anon_sym_asm] = ACTIONS(1124),
    [anon_sym___asm__] = ACTIONS(1124),
    [sym_number_literal] = ACTIONS(1126),
    [anon_sym_L_SQUOTE] = ACTIONS(1126),
    [anon_sym_u_SQUOTE] = ACTIONS(1126),
    [anon_sym_U_SQUOTE] = ACTIONS(1126),
    [anon_sym_u8_SQUOTE] = ACTIONS(1126),
    [anon_sym_SQUOTE] = ACTIONS(1126),
    [anon_sym_L_DQUOTE] = ACTIONS(1126),
    [anon_sym_u_DQUOTE] = ACTIONS(1126),
    [anon_sym_U_DQUOTE] = ACTIONS(1126),
    [anon_sym_u8_DQUOTE] = ACTIONS(1126),
    [anon_sym_DQUOTE] = ACTIONS(1126),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [anon_sym_NULL] = ACTIONS(1124),
    [anon_sym_nullptr] = ACTIONS(1124),
    [sym_comment] = ACTIONS(3),
  },
  [235] = {
    [sym_identifier] = ACTIONS(1128),
    [aux_sym_preproc_include_token1] = ACTIONS(1128),
    [aux_sym_preproc_def_token1] = ACTIONS(1128),
    [aux_sym_preproc_if_token1] = ACTIONS(1128),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1128),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1128),
    [sym_preproc_directive] = ACTIONS(1128),
    [anon_sym_LPAREN2] = ACTIONS(1130),
    [anon_sym_BANG] = ACTIONS(1130),
    [anon_sym_TILDE] = ACTIONS(1130),
    [anon_sym_DASH] = ACTIONS(1128),
    [anon_sym_PLUS] = ACTIONS(1128),
    [anon_sym_STAR] = ACTIONS(1130),
    [anon_sym_AMP] = ACTIONS(1130),
    [anon_sym_SEMI] = ACTIONS(1130),
    [anon_sym___extension__] = ACTIONS(1128),
    [anon_sym_typedef] = ACTIONS(1128),
    [anon_sym_extern] = ACTIONS(1128),
    [anon_sym___attribute__] = ACTIONS(1128),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1130),
    [anon_sym___declspec] = ACTIONS(1128),
    [anon_sym___cdecl] = ACTIONS(1128),
    [anon_sym___clrcall] = ACTIONS(1128),
    [anon_sym___stdcall] = ACTIONS(1128),
    [anon_sym___fastcall] = ACTIONS(1128),
    [anon_sym___thiscall] = ACTIONS(1128),
    [anon_sym___vectorcall] = ACTIONS(1128),
    [anon_sym_LBRACE] = ACTIONS(1130),
    [anon_sym_RBRACE] = ACTIONS(1130),
    [anon_sym_signed] = ACTIONS(1128),
    [anon_sym_unsigned] = ACTIONS(1128),
    [anon_sym_long] = ACTIONS(1128),
    [anon_sym_short] = ACTIONS(1128),
    [anon_sym_static] = ACTIONS(1128),
    [anon_sym_auto] = ACTIONS(1128),
    [anon_sym_register] = ACTIONS(1128),
    [anon_sym_inline] = ACTIONS(1128),
    [anon_sym___inline] = ACTIONS(1128),
    [anon_sym___inline__] = ACTIONS(1128),
    [anon_sym___forceinline] = ACTIONS(1128),
    [anon_sym_thread_local] = ACTIONS(1128),
    [anon_sym___thread] = ACTIONS(1128),
    [anon_sym_const] = ACTIONS(1128),
    [anon_sym_constexpr] = ACTIONS(1128),
    [anon_sym_volatile] = ACTIONS(1128),
    [anon_sym_restrict] = ACTIONS(1128),
    [anon_sym___restrict__] = ACTIONS(1128),
    [anon_sym__Atomic] = ACTIONS(1128),
    [anon_sym__Noreturn] = ACTIONS(1128),
    [anon_sym_noreturn] = ACTIONS(1128),
    [sym_primitive_type] = ACTIONS(1128),
    [anon_sym_enum] = ACTIONS(1128),
    [anon_sym_struct] = ACTIONS(1128),
    [anon_sym_union] = ACTIONS(1128),
    [anon_sym_if] = ACTIONS(1128),
    [anon_sym_else] = ACTIONS(1128),
    [anon_sym_switch] = ACTIONS(1128),
    [anon_sym_case] = ACTIONS(1128),
    [anon_sym_default] = ACTIONS(1128),
    [anon_sym_while] = ACTIONS(1128),
    [anon_sym_do] = ACTIONS(1128),
    [anon_sym_for] = ACTIONS(1128),
    [anon_sym_return] = ACTIONS(1128),
    [anon_sym_break] = ACTIONS(1128),
    [anon_sym_continue] = ACTIONS(1128),
    [anon_sym_goto] = ACTIONS(1128),
    [anon_sym___try] = ACTIONS(1128),
    [anon_sym___leave] = ACTIONS(1128),
    [anon_sym_DASH_DASH] = ACTIONS(1130),
    [anon_sym_PLUS_PLUS] = ACTIONS(1130),
    [anon_sym_sizeof] = ACTIONS(1128),
    [anon_sym___alignof__] = ACTIONS(1128),
    [anon_sym___alignof] = ACTIONS(1128),
    [anon_sym__alignof] = ACTIONS(1128),
    [anon_sym_alignof] = ACTIONS(1128),
    [anon_sym__Alignof] = ACTIONS(1128),
    [anon_sym_offsetof] = ACTIONS(1128),
    [anon_sym__Generic] = ACTIONS(1128),
    [anon_sym_asm] = ACTIONS(1128),
    [anon_sym___asm__] = ACTIONS(1128),
    [sym_number_literal] = ACTIONS(1130),
    [anon_sym_L_SQUOTE] = ACTIONS(1130),
    [anon_sym_u_SQUOTE] = ACTIONS(1130),
    [anon_sym_U_SQUOTE] = ACTIONS(1130),
    [anon_sym_u8_SQUOTE] = ACTIONS(1130),
    [anon_sym_SQUOTE] = ACTIONS(1130),
    [anon_sym_L_DQUOTE] = ACTIONS(1130),
    [anon_sym_u_DQUOTE] = ACTIONS(1130),
    [anon_sym_U_DQUOTE] = ACTIONS(1130),
    [anon_sym_u8_DQUOTE] = ACTIONS(1130),
    [anon_sym_DQUOTE] = ACTIONS(1130),
    [sym_true] = ACTIONS(1128),
    [sym_false] = ACTIONS(1128),
    [anon_sym_NULL] = ACTIONS(1128),
    [anon_sym_nullptr] = ACTIONS(1128),
    [sym_comment] = ACTIONS(3),
  },
  [236] = {
    [sym_identifier] = ACTIONS(1164),
    [aux_sym_preproc_include_token1] = ACTIONS(1164),
    [aux_sym_preproc_def_token1] = ACTIONS(1164),
    [aux_sym_preproc_if_token1] = ACTIONS(1164),
    [aux_sym_preproc_if_token2] = ACTIONS(1164),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1164),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1164),
    [sym_preproc_directive] = ACTIONS(1164),
    [anon_sym_LPAREN2] = ACTIONS(1166),
    [anon_sym_BANG] = ACTIONS(1166),
    [anon_sym_TILDE] = ACTIONS(1166),
    [anon_sym_DASH] = ACTIONS(1164),
    [anon_sym_PLUS] = ACTIONS(1164),
    [anon_sym_STAR] = ACTIONS(1166),
    [anon_sym_AMP] = ACTIONS(1166),
    [anon_sym_SEMI] = ACTIONS(1166),
    [anon_sym___extension__] = ACTIONS(1164),
    [anon_sym_typedef] = ACTIONS(1164),
    [anon_sym_extern] = ACTIONS(1164),
    [anon_sym___attribute__] = ACTIONS(1164),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1166),
    [anon_sym___declspec] = ACTIONS(1164),
    [anon_sym___cdecl] = ACTIONS(1164),
    [anon_sym___clrcall] = ACTIONS(1164),
    [anon_sym___stdcall] = ACTIONS(1164),
    [anon_sym___fastcall] = ACTIONS(1164),
    [anon_sym___thiscall] = ACTIONS(1164),
    [anon_sym___vectorcall] = ACTIONS(1164),
    [anon_sym_LBRACE] = ACTIONS(1166),
    [anon_sym_signed] = ACTIONS(1164),
    [anon_sym_unsigned] = ACTIONS(1164),
    [anon_sym_long] = ACTIONS(1164),
    [anon_sym_short] = ACTIONS(1164),
    [anon_sym_static] = ACTIONS(1164),
    [anon_sym_auto] = ACTIONS(1164),
    [anon_sym_register] = ACTIONS(1164),
    [anon_sym_inline] = ACTIONS(1164),
    [anon_sym___inline] = ACTIONS(1164),
    [anon_sym___inline__] = ACTIONS(1164),
    [anon_sym___forceinline] = ACTIONS(1164),
    [anon_sym_thread_local] = ACTIONS(1164),
    [anon_sym___thread] = ACTIONS(1164),
    [anon_sym_const] = ACTIONS(1164),
    [anon_sym_constexpr] = ACTIONS(1164),
    [anon_sym_volatile] = ACTIONS(1164),
    [anon_sym_restrict] = ACTIONS(1164),
    [anon_sym___restrict__] = ACTIONS(1164),
    [anon_sym__Atomic] = ACTIONS(1164),
    [anon_sym__Noreturn] = ACTIONS(1164),
    [anon_sym_noreturn] = ACTIONS(1164),
    [sym_primitive_type] = ACTIONS(1164),
    [anon_sym_enum] = ACTIONS(1164),
    [anon_sym_struct] = ACTIONS(1164),
    [anon_sym_union] = ACTIONS(1164),
    [anon_sym_if] = ACTIONS(1164),
    [anon_sym_else] = ACTIONS(1164),
    [anon_sym_switch] = ACTIONS(1164),
    [anon_sym_case] = ACTIONS(1164),
    [anon_sym_default] = ACTIONS(1164),
    [anon_sym_while] = ACTIONS(1164),
    [anon_sym_do] = ACTIONS(1164),
    [anon_sym_for] = ACTIONS(1164),
    [anon_sym_return] = ACTIONS(1164),
    [anon_sym_break] = ACTIONS(1164),
    [anon_sym_continue] = ACTIONS(1164),
    [anon_sym_goto] = ACTIONS(1164),
    [anon_sym___try] = ACTIONS(1164),
    [anon_sym___leave] = ACTIONS(1164),
    [anon_sym_DASH_DASH] = ACTIONS(1166),
    [anon_sym_PLUS_PLUS] = ACTIONS(1166),
    [anon_sym_sizeof] = ACTIONS(1164),
    [anon_sym___alignof__] = ACTIONS(1164),
    [anon_sym___alignof] = ACTIONS(1164),
    [anon_sym__alignof] = ACTIONS(1164),
    [anon_sym_alignof] = ACTIONS(1164),
    [anon_sym__Alignof] = ACTIONS(1164),
    [anon_sym_offsetof] = ACTIONS(1164),
    [anon_sym__Generic] = ACTIONS(1164),
    [anon_sym_asm] = ACTIONS(1164),
    [anon_sym___asm__] = ACTIONS(1164),
    [sym_number_literal] = ACTIONS(1166),
    [anon_sym_L_SQUOTE] = ACTIONS(1166),
    [anon_sym_u_SQUOTE] = ACTIONS(1166),
    [anon_sym_U_SQUOTE] = ACTIONS(1166),
    [anon_sym_u8_SQUOTE] = ACTIONS(1166),
    [anon_sym_SQUOTE] = ACTIONS(1166),
    [anon_sym_L_DQUOTE] = ACTIONS(1166),
    [anon_sym_u_DQUOTE] = ACTIONS(1166),
    [anon_sym_U_DQUOTE] = ACTIONS(1166),
    [anon_sym_u8_DQUOTE] = ACTIONS(1166),
    [anon_sym_DQUOTE] = ACTIONS(1166),
    [sym_true] = ACTIONS(1164),
    [sym_false] = ACTIONS(1164),
    [anon_sym_NULL] = ACTIONS(1164),
    [anon_sym_nullptr] = ACTIONS(1164),
    [sym_comment] = ACTIONS(3),
  },
  [237] = {
    [sym_identifier] = ACTIONS(1132),
    [aux_sym_preproc_include_token1] = ACTIONS(1132),
    [aux_sym_preproc_def_token1] = ACTIONS(1132),
    [aux_sym_preproc_if_token1] = ACTIONS(1132),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1132),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1132),
    [sym_preproc_directive] = ACTIONS(1132),
    [anon_sym_LPAREN2] = ACTIONS(1134),
    [anon_sym_BANG] = ACTIONS(1134),
    [anon_sym_TILDE] = ACTIONS(1134),
    [anon_sym_DASH] = ACTIONS(1132),
    [anon_sym_PLUS] = ACTIONS(1132),
    [anon_sym_STAR] = ACTIONS(1134),
    [anon_sym_AMP] = ACTIONS(1134),
    [anon_sym_SEMI] = ACTIONS(1134),
    [anon_sym___extension__] = ACTIONS(1132),
    [anon_sym_typedef] = ACTIONS(1132),
    [anon_sym_extern] = ACTIONS(1132),
    [anon_sym___attribute__] = ACTIONS(1132),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1134),
    [anon_sym___declspec] = ACTIONS(1132),
    [anon_sym___cdecl] = ACTIONS(1132),
    [anon_sym___clrcall] = ACTIONS(1132),
    [anon_sym___stdcall] = ACTIONS(1132),
    [anon_sym___fastcall] = ACTIONS(1132),
    [anon_sym___thiscall] = ACTIONS(1132),
    [anon_sym___vectorcall] = ACTIONS(1132),
    [anon_sym_LBRACE] = ACTIONS(1134),
    [anon_sym_RBRACE] = ACTIONS(1134),
    [anon_sym_signed] = ACTIONS(1132),
    [anon_sym_unsigned] = ACTIONS(1132),
    [anon_sym_long] = ACTIONS(1132),
    [anon_sym_short] = ACTIONS(1132),
    [anon_sym_static] = ACTIONS(1132),
    [anon_sym_auto] = ACTIONS(1132),
    [anon_sym_register] = ACTIONS(1132),
    [anon_sym_inline] = ACTIONS(1132),
    [anon_sym___inline] = ACTIONS(1132),
    [anon_sym___inline__] = ACTIONS(1132),
    [anon_sym___forceinline] = ACTIONS(1132),
    [anon_sym_thread_local] = ACTIONS(1132),
    [anon_sym___thread] = ACTIONS(1132),
    [anon_sym_const] = ACTIONS(1132),
    [anon_sym_constexpr] = ACTIONS(1132),
    [anon_sym_volatile] = ACTIONS(1132),
    [anon_sym_restrict] = ACTIONS(1132),
    [anon_sym___restrict__] = ACTIONS(1132),
    [anon_sym__Atomic] = ACTIONS(1132),
    [anon_sym__Noreturn] = ACTIONS(1132),
    [anon_sym_noreturn] = ACTIONS(1132),
    [sym_primitive_type] = ACTIONS(1132),
    [anon_sym_enum] = ACTIONS(1132),
    [anon_sym_struct] = ACTIONS(1132),
    [anon_sym_union] = ACTIONS(1132),
    [anon_sym_if] = ACTIONS(1132),
    [anon_sym_else] = ACTIONS(1132),
    [anon_sym_switch] = ACTIONS(1132),
    [anon_sym_case] = ACTIONS(1132),
    [anon_sym_default] = ACTIONS(1132),
    [anon_sym_while] = ACTIONS(1132),
    [anon_sym_do] = ACTIONS(1132),
    [anon_sym_for] = ACTIONS(1132),
    [anon_sym_return] = ACTIONS(1132),
    [anon_sym_break] = ACTIONS(1132),
    [anon_sym_continue] = ACTIONS(1132),
    [anon_sym_goto] = ACTIONS(1132),
    [anon_sym___try] = ACTIONS(1132),
    [anon_sym___leave] = ACTIONS(1132),
    [anon_sym_DASH_DASH] = ACTIONS(1134),
    [anon_sym_PLUS_PLUS] = ACTIONS(1134),
    [anon_sym_sizeof] = ACTIONS(1132),
    [anon_sym___alignof__] = ACTIONS(1132),
    [anon_sym___alignof] = ACTIONS(1132),
    [anon_sym__alignof] = ACTIONS(1132),
    [anon_sym_alignof] = ACTIONS(1132),
    [anon_sym__Alignof] = ACTIONS(1132),
    [anon_sym_offsetof] = ACTIONS(1132),
    [anon_sym__Generic] = ACTIONS(1132),
    [anon_sym_asm] = ACTIONS(1132),
    [anon_sym___asm__] = ACTIONS(1132),
    [sym_number_literal] = ACTIONS(1134),
    [anon_sym_L_SQUOTE] = ACTIONS(1134),
    [anon_sym_u_SQUOTE] = ACTIONS(1134),
    [anon_sym_U_SQUOTE] = ACTIONS(1134),
    [anon_sym_u8_SQUOTE] = ACTIONS(1134),
    [anon_sym_SQUOTE] = ACTIONS(1134),
    [anon_sym_L_DQUOTE] = ACTIONS(1134),
    [anon_sym_u_DQUOTE] = ACTIONS(1134),
    [anon_sym_U_DQUOTE] = ACTIONS(1134),
    [anon_sym_u8_DQUOTE] = ACTIONS(1134),
    [anon_sym_DQUOTE] = ACTIONS(1134),
    [sym_true] = ACTIONS(1132),
    [sym_false] = ACTIONS(1132),
    [anon_sym_NULL] = ACTIONS(1132),
    [anon_sym_nullptr] = ACTIONS(1132),
    [sym_comment] = ACTIONS(3),
  },
  [238] = {
    [sym_identifier] = ACTIONS(1160),
    [aux_sym_preproc_include_token1] = ACTIONS(1160),
    [aux_sym_preproc_def_token1] = ACTIONS(1160),
    [aux_sym_preproc_if_token1] = ACTIONS(1160),
    [aux_sym_preproc_if_token2] = ACTIONS(1160),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1160),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1160),
    [sym_preproc_directive] = ACTIONS(1160),
    [anon_sym_LPAREN2] = ACTIONS(1162),
    [anon_sym_BANG] = ACTIONS(1162),
    [anon_sym_TILDE] = ACTIONS(1162),
    [anon_sym_DASH] = ACTIONS(1160),
    [anon_sym_PLUS] = ACTIONS(1160),
    [anon_sym_STAR] = ACTIONS(1162),
    [anon_sym_AMP] = ACTIONS(1162),
    [anon_sym_SEMI] = ACTIONS(1162),
    [anon_sym___extension__] = ACTIONS(1160),
    [anon_sym_typedef] = ACTIONS(1160),
    [anon_sym_extern] = ACTIONS(1160),
    [anon_sym___attribute__] = ACTIONS(1160),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1162),
    [anon_sym___declspec] = ACTIONS(1160),
    [anon_sym___cdecl] = ACTIONS(1160),
    [anon_sym___clrcall] = ACTIONS(1160),
    [anon_sym___stdcall] = ACTIONS(1160),
    [anon_sym___fastcall] = ACTIONS(1160),
    [anon_sym___thiscall] = ACTIONS(1160),
    [anon_sym___vectorcall] = ACTIONS(1160),
    [anon_sym_LBRACE] = ACTIONS(1162),
    [anon_sym_signed] = ACTIONS(1160),
    [anon_sym_unsigned] = ACTIONS(1160),
    [anon_sym_long] = ACTIONS(1160),
    [anon_sym_short] = ACTIONS(1160),
    [anon_sym_static] = ACTIONS(1160),
    [anon_sym_auto] = ACTIONS(1160),
    [anon_sym_register] = ACTIONS(1160),
    [anon_sym_inline] = ACTIONS(1160),
    [anon_sym___inline] = ACTIONS(1160),
    [anon_sym___inline__] = ACTIONS(1160),
    [anon_sym___forceinline] = ACTIONS(1160),
    [anon_sym_thread_local] = ACTIONS(1160),
    [anon_sym___thread] = ACTIONS(1160),
    [anon_sym_const] = ACTIONS(1160),
    [anon_sym_constexpr] = ACTIONS(1160),
    [anon_sym_volatile] = ACTIONS(1160),
    [anon_sym_restrict] = ACTIONS(1160),
    [anon_sym___restrict__] = ACTIONS(1160),
    [anon_sym__Atomic] = ACTIONS(1160),
    [anon_sym__Noreturn] = ACTIONS(1160),
    [anon_sym_noreturn] = ACTIONS(1160),
    [sym_primitive_type] = ACTIONS(1160),
    [anon_sym_enum] = ACTIONS(1160),
    [anon_sym_struct] = ACTIONS(1160),
    [anon_sym_union] = ACTIONS(1160),
    [anon_sym_if] = ACTIONS(1160),
    [anon_sym_else] = ACTIONS(1160),
    [anon_sym_switch] = ACTIONS(1160),
    [anon_sym_case] = ACTIONS(1160),
    [anon_sym_default] = ACTIONS(1160),
    [anon_sym_while] = ACTIONS(1160),
    [anon_sym_do] = ACTIONS(1160),
    [anon_sym_for] = ACTIONS(1160),
    [anon_sym_return] = ACTIONS(1160),
    [anon_sym_break] = ACTIONS(1160),
    [anon_sym_continue] = ACTIONS(1160),
    [anon_sym_goto] = ACTIONS(1160),
    [anon_sym___try] = ACTIONS(1160),
    [anon_sym___leave] = ACTIONS(1160),
    [anon_sym_DASH_DASH] = ACTIONS(1162),
    [anon_sym_PLUS_PLUS] = ACTIONS(1162),
    [anon_sym_sizeof] = ACTIONS(1160),
    [anon_sym___alignof__] = ACTIONS(1160),
    [anon_sym___alignof] = ACTIONS(1160),
    [anon_sym__alignof] = ACTIONS(1160),
    [anon_sym_alignof] = ACTIONS(1160),
    [anon_sym__Alignof] = ACTIONS(1160),
    [anon_sym_offsetof] = ACTIONS(1160),
    [anon_sym__Generic] = ACTIONS(1160),
    [anon_sym_asm] = ACTIONS(1160),
    [anon_sym___asm__] = ACTIONS(1160),
    [sym_number_literal] = ACTIONS(1162),
    [anon_sym_L_SQUOTE] = ACTIONS(1162),
    [anon_sym_u_SQUOTE] = ACTIONS(1162),
    [anon_sym_U_SQUOTE] = ACTIONS(1162),
    [anon_sym_u8_SQUOTE] = ACTIONS(1162),
    [anon_sym_SQUOTE] = ACTIONS(1162),
    [anon_sym_L_DQUOTE] = ACTIONS(1162),
    [anon_sym_u_DQUOTE] = ACTIONS(1162),
    [anon_sym_U_DQUOTE] = ACTIONS(1162),
    [anon_sym_u8_DQUOTE] = ACTIONS(1162),
    [anon_sym_DQUOTE] = ACTIONS(1162),
    [sym_true] = ACTIONS(1160),
    [sym_false] = ACTIONS(1160),
    [anon_sym_NULL] = ACTIONS(1160),
    [anon_sym_nullptr] = ACTIONS(1160),
    [sym_comment] = ACTIONS(3),
  },
  [239] = {
    [sym_identifier] = ACTIONS(1136),
    [aux_sym_preproc_include_token1] = ACTIONS(1136),
    [aux_sym_preproc_def_token1] = ACTIONS(1136),
    [aux_sym_preproc_if_token1] = ACTIONS(1136),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1136),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1136),
    [sym_preproc_directive] = ACTIONS(1136),
    [anon_sym_LPAREN2] = ACTIONS(1138),
    [anon_sym_BANG] = ACTIONS(1138),
    [anon_sym_TILDE] = ACTIONS(1138),
    [anon_sym_DASH] = ACTIONS(1136),
    [anon_sym_PLUS] = ACTIONS(1136),
    [anon_sym_STAR] = ACTIONS(1138),
    [anon_sym_AMP] = ACTIONS(1138),
    [anon_sym_SEMI] = ACTIONS(1138),
    [anon_sym___extension__] = ACTIONS(1136),
    [anon_sym_typedef] = ACTIONS(1136),
    [anon_sym_extern] = ACTIONS(1136),
    [anon_sym___attribute__] = ACTIONS(1136),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1138),
    [anon_sym___declspec] = ACTIONS(1136),
    [anon_sym___cdecl] = ACTIONS(1136),
    [anon_sym___clrcall] = ACTIONS(1136),
    [anon_sym___stdcall] = ACTIONS(1136),
    [anon_sym___fastcall] = ACTIONS(1136),
    [anon_sym___thiscall] = ACTIONS(1136),
    [anon_sym___vectorcall] = ACTIONS(1136),
    [anon_sym_LBRACE] = ACTIONS(1138),
    [anon_sym_RBRACE] = ACTIONS(1138),
    [anon_sym_signed] = ACTIONS(1136),
    [anon_sym_unsigned] = ACTIONS(1136),
    [anon_sym_long] = ACTIONS(1136),
    [anon_sym_short] = ACTIONS(1136),
    [anon_sym_static] = ACTIONS(1136),
    [anon_sym_auto] = ACTIONS(1136),
    [anon_sym_register] = ACTIONS(1136),
    [anon_sym_inline] = ACTIONS(1136),
    [anon_sym___inline] = ACTIONS(1136),
    [anon_sym___inline__] = ACTIONS(1136),
    [anon_sym___forceinline] = ACTIONS(1136),
    [anon_sym_thread_local] = ACTIONS(1136),
    [anon_sym___thread] = ACTIONS(1136),
    [anon_sym_const] = ACTIONS(1136),
    [anon_sym_constexpr] = ACTIONS(1136),
    [anon_sym_volatile] = ACTIONS(1136),
    [anon_sym_restrict] = ACTIONS(1136),
    [anon_sym___restrict__] = ACTIONS(1136),
    [anon_sym__Atomic] = ACTIONS(1136),
    [anon_sym__Noreturn] = ACTIONS(1136),
    [anon_sym_noreturn] = ACTIONS(1136),
    [sym_primitive_type] = ACTIONS(1136),
    [anon_sym_enum] = ACTIONS(1136),
    [anon_sym_struct] = ACTIONS(1136),
    [anon_sym_union] = ACTIONS(1136),
    [anon_sym_if] = ACTIONS(1136),
    [anon_sym_else] = ACTIONS(1136),
    [anon_sym_switch] = ACTIONS(1136),
    [anon_sym_case] = ACTIONS(1136),
    [anon_sym_default] = ACTIONS(1136),
    [anon_sym_while] = ACTIONS(1136),
    [anon_sym_do] = ACTIONS(1136),
    [anon_sym_for] = ACTIONS(1136),
    [anon_sym_return] = ACTIONS(1136),
    [anon_sym_break] = ACTIONS(1136),
    [anon_sym_continue] = ACTIONS(1136),
    [anon_sym_goto] = ACTIONS(1136),
    [anon_sym___try] = ACTIONS(1136),
    [anon_sym___leave] = ACTIONS(1136),
    [anon_sym_DASH_DASH] = ACTIONS(1138),
    [anon_sym_PLUS_PLUS] = ACTIONS(1138),
    [anon_sym_sizeof] = ACTIONS(1136),
    [anon_sym___alignof__] = ACTIONS(1136),
    [anon_sym___alignof] = ACTIONS(1136),
    [anon_sym__alignof] = ACTIONS(1136),
    [anon_sym_alignof] = ACTIONS(1136),
    [anon_sym__Alignof] = ACTIONS(1136),
    [anon_sym_offsetof] = ACTIONS(1136),
    [anon_sym__Generic] = ACTIONS(1136),
    [anon_sym_asm] = ACTIONS(1136),
    [anon_sym___asm__] = ACTIONS(1136),
    [sym_number_literal] = ACTIONS(1138),
    [anon_sym_L_SQUOTE] = ACTIONS(1138),
    [anon_sym_u_SQUOTE] = ACTIONS(1138),
    [anon_sym_U_SQUOTE] = ACTIONS(1138),
    [anon_sym_u8_SQUOTE] = ACTIONS(1138),
    [anon_sym_SQUOTE] = ACTIONS(1138),
    [anon_sym_L_DQUOTE] = ACTIONS(1138),
    [anon_sym_u_DQUOTE] = ACTIONS(1138),
    [anon_sym_U_DQUOTE] = ACTIONS(1138),
    [anon_sym_u8_DQUOTE] = ACTIONS(1138),
    [anon_sym_DQUOTE] = ACTIONS(1138),
    [sym_true] = ACTIONS(1136),
    [sym_false] = ACTIONS(1136),
    [anon_sym_NULL] = ACTIONS(1136),
    [anon_sym_nullptr] = ACTIONS(1136),
    [sym_comment] = ACTIONS(3),
  },
  [240] = {
    [sym_identifier] = ACTIONS(1140),
    [aux_sym_preproc_include_token1] = ACTIONS(1140),
    [aux_sym_preproc_def_token1] = ACTIONS(1140),
    [aux_sym_preproc_if_token1] = ACTIONS(1140),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1140),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1140),
    [sym_preproc_directive] = ACTIONS(1140),
    [anon_sym_LPAREN2] = ACTIONS(1142),
    [anon_sym_BANG] = ACTIONS(1142),
    [anon_sym_TILDE] = ACTIONS(1142),
    [anon_sym_DASH] = ACTIONS(1140),
    [anon_sym_PLUS] = ACTIONS(1140),
    [anon_sym_STAR] = ACTIONS(1142),
    [anon_sym_AMP] = ACTIONS(1142),
    [anon_sym_SEMI] = ACTIONS(1142),
    [anon_sym___extension__] = ACTIONS(1140),
    [anon_sym_typedef] = ACTIONS(1140),
    [anon_sym_extern] = ACTIONS(1140),
    [anon_sym___attribute__] = ACTIONS(1140),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1142),
    [anon_sym___declspec] = ACTIONS(1140),
    [anon_sym___cdecl] = ACTIONS(1140),
    [anon_sym___clrcall] = ACTIONS(1140),
    [anon_sym___stdcall] = ACTIONS(1140),
    [anon_sym___fastcall] = ACTIONS(1140),
    [anon_sym___thiscall] = ACTIONS(1140),
    [anon_sym___vectorcall] = ACTIONS(1140),
    [anon_sym_LBRACE] = ACTIONS(1142),
    [anon_sym_RBRACE] = ACTIONS(1142),
    [anon_sym_signed] = ACTIONS(1140),
    [anon_sym_unsigned] = ACTIONS(1140),
    [anon_sym_long] = ACTIONS(1140),
    [anon_sym_short] = ACTIONS(1140),
    [anon_sym_static] = ACTIONS(1140),
    [anon_sym_auto] = ACTIONS(1140),
    [anon_sym_register] = ACTIONS(1140),
    [anon_sym_inline] = ACTIONS(1140),
    [anon_sym___inline] = ACTIONS(1140),
    [anon_sym___inline__] = ACTIONS(1140),
    [anon_sym___forceinline] = ACTIONS(1140),
    [anon_sym_thread_local] = ACTIONS(1140),
    [anon_sym___thread] = ACTIONS(1140),
    [anon_sym_const] = ACTIONS(1140),
    [anon_sym_constexpr] = ACTIONS(1140),
    [anon_sym_volatile] = ACTIONS(1140),
    [anon_sym_restrict] = ACTIONS(1140),
    [anon_sym___restrict__] = ACTIONS(1140),
    [anon_sym__Atomic] = ACTIONS(1140),
    [anon_sym__Noreturn] = ACTIONS(1140),
    [anon_sym_noreturn] = ACTIONS(1140),
    [sym_primitive_type] = ACTIONS(1140),
    [anon_sym_enum] = ACTIONS(1140),
    [anon_sym_struct] = ACTIONS(1140),
    [anon_sym_union] = ACTIONS(1140),
    [anon_sym_if] = ACTIONS(1140),
    [anon_sym_else] = ACTIONS(1140),
    [anon_sym_switch] = ACTIONS(1140),
    [anon_sym_case] = ACTIONS(1140),
    [anon_sym_default] = ACTIONS(1140),
    [anon_sym_while] = ACTIONS(1140),
    [anon_sym_do] = ACTIONS(1140),
    [anon_sym_for] = ACTIONS(1140),
    [anon_sym_return] = ACTIONS(1140),
    [anon_sym_break] = ACTIONS(1140),
    [anon_sym_continue] = ACTIONS(1140),
    [anon_sym_goto] = ACTIONS(1140),
    [anon_sym___try] = ACTIONS(1140),
    [anon_sym___leave] = ACTIONS(1140),
    [anon_sym_DASH_DASH] = ACTIONS(1142),
    [anon_sym_PLUS_PLUS] = ACTIONS(1142),
    [anon_sym_sizeof] = ACTIONS(1140),
    [anon_sym___alignof__] = ACTIONS(1140),
    [anon_sym___alignof] = ACTIONS(1140),
    [anon_sym__alignof] = ACTIONS(1140),
    [anon_sym_alignof] = ACTIONS(1140),
    [anon_sym__Alignof] = ACTIONS(1140),
    [anon_sym_offsetof] = ACTIONS(1140),
    [anon_sym__Generic] = ACTIONS(1140),
    [anon_sym_asm] = ACTIONS(1140),
    [anon_sym___asm__] = ACTIONS(1140),
    [sym_number_literal] = ACTIONS(1142),
    [anon_sym_L_SQUOTE] = ACTIONS(1142),
    [anon_sym_u_SQUOTE] = ACTIONS(1142),
    [anon_sym_U_SQUOTE] = ACTIONS(1142),
    [anon_sym_u8_SQUOTE] = ACTIONS(1142),
    [anon_sym_SQUOTE] = ACTIONS(1142),
    [anon_sym_L_DQUOTE] = ACTIONS(1142),
    [anon_sym_u_DQUOTE] = ACTIONS(1142),
    [anon_sym_U_DQUOTE] = ACTIONS(1142),
    [anon_sym_u8_DQUOTE] = ACTIONS(1142),
    [anon_sym_DQUOTE] = ACTIONS(1142),
    [sym_true] = ACTIONS(1140),
    [sym_false] = ACTIONS(1140),
    [anon_sym_NULL] = ACTIONS(1140),
    [anon_sym_nullptr] = ACTIONS(1140),
    [sym_comment] = ACTIONS(3),
  },
  [241] = {
    [sym_identifier] = ACTIONS(1212),
    [aux_sym_preproc_include_token1] = ACTIONS(1212),
    [aux_sym_preproc_def_token1] = ACTIONS(1212),
    [aux_sym_preproc_if_token1] = ACTIONS(1212),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1212),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1212),
    [sym_preproc_directive] = ACTIONS(1212),
    [anon_sym_LPAREN2] = ACTIONS(1214),
    [anon_sym_BANG] = ACTIONS(1214),
    [anon_sym_TILDE] = ACTIONS(1214),
    [anon_sym_DASH] = ACTIONS(1212),
    [anon_sym_PLUS] = ACTIONS(1212),
    [anon_sym_STAR] = ACTIONS(1214),
    [anon_sym_AMP] = ACTIONS(1214),
    [anon_sym_SEMI] = ACTIONS(1214),
    [anon_sym___extension__] = ACTIONS(1212),
    [anon_sym_typedef] = ACTIONS(1212),
    [anon_sym_extern] = ACTIONS(1212),
    [anon_sym___attribute__] = ACTIONS(1212),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1214),
    [anon_sym___declspec] = ACTIONS(1212),
    [anon_sym___cdecl] = ACTIONS(1212),
    [anon_sym___clrcall] = ACTIONS(1212),
    [anon_sym___stdcall] = ACTIONS(1212),
    [anon_sym___fastcall] = ACTIONS(1212),
    [anon_sym___thiscall] = ACTIONS(1212),
    [anon_sym___vectorcall] = ACTIONS(1212),
    [anon_sym_LBRACE] = ACTIONS(1214),
    [anon_sym_RBRACE] = ACTIONS(1214),
    [anon_sym_signed] = ACTIONS(1212),
    [anon_sym_unsigned] = ACTIONS(1212),
    [anon_sym_long] = ACTIONS(1212),
    [anon_sym_short] = ACTIONS(1212),
    [anon_sym_static] = ACTIONS(1212),
    [anon_sym_auto] = ACTIONS(1212),
    [anon_sym_register] = ACTIONS(1212),
    [anon_sym_inline] = ACTIONS(1212),
    [anon_sym___inline] = ACTIONS(1212),
    [anon_sym___inline__] = ACTIONS(1212),
    [anon_sym___forceinline] = ACTIONS(1212),
    [anon_sym_thread_local] = ACTIONS(1212),
    [anon_sym___thread] = ACTIONS(1212),
    [anon_sym_const] = ACTIONS(1212),
    [anon_sym_constexpr] = ACTIONS(1212),
    [anon_sym_volatile] = ACTIONS(1212),
    [anon_sym_restrict] = ACTIONS(1212),
    [anon_sym___restrict__] = ACTIONS(1212),
    [anon_sym__Atomic] = ACTIONS(1212),
    [anon_sym__Noreturn] = ACTIONS(1212),
    [anon_sym_noreturn] = ACTIONS(1212),
    [sym_primitive_type] = ACTIONS(1212),
    [anon_sym_enum] = ACTIONS(1212),
    [anon_sym_struct] = ACTIONS(1212),
    [anon_sym_union] = ACTIONS(1212),
    [anon_sym_if] = ACTIONS(1212),
    [anon_sym_else] = ACTIONS(1212),
    [anon_sym_switch] = ACTIONS(1212),
    [anon_sym_case] = ACTIONS(1212),
    [anon_sym_default] = ACTIONS(1212),
    [anon_sym_while] = ACTIONS(1212),
    [anon_sym_do] = ACTIONS(1212),
    [anon_sym_for] = ACTIONS(1212),
    [anon_sym_return] = ACTIONS(1212),
    [anon_sym_break] = ACTIONS(1212),
    [anon_sym_continue] = ACTIONS(1212),
    [anon_sym_goto] = ACTIONS(1212),
    [anon_sym___try] = ACTIONS(1212),
    [anon_sym___leave] = ACTIONS(1212),
    [anon_sym_DASH_DASH] = ACTIONS(1214),
    [anon_sym_PLUS_PLUS] = ACTIONS(1214),
    [anon_sym_sizeof] = ACTIONS(1212),
    [anon_sym___alignof__] = ACTIONS(1212),
    [anon_sym___alignof] = ACTIONS(1212),
    [anon_sym__alignof] = ACTIONS(1212),
    [anon_sym_alignof] = ACTIONS(1212),
    [anon_sym__Alignof] = ACTIONS(1212),
    [anon_sym_offsetof] = ACTIONS(1212),
    [anon_sym__Generic] = ACTIONS(1212),
    [anon_sym_asm] = ACTIONS(1212),
    [anon_sym___asm__] = ACTIONS(1212),
    [sym_number_literal] = ACTIONS(1214),
    [anon_sym_L_SQUOTE] = ACTIONS(1214),
    [anon_sym_u_SQUOTE] = ACTIONS(1214),
    [anon_sym_U_SQUOTE] = ACTIONS(1214),
    [anon_sym_u8_SQUOTE] = ACTIONS(1214),
    [anon_sym_SQUOTE] = ACTIONS(1214),
    [anon_sym_L_DQUOTE] = ACTIONS(1214),
    [anon_sym_u_DQUOTE] = ACTIONS(1214),
    [anon_sym_U_DQUOTE] = ACTIONS(1214),
    [anon_sym_u8_DQUOTE] = ACTIONS(1214),
    [anon_sym_DQUOTE] = ACTIONS(1214),
    [sym_true] = ACTIONS(1212),
    [sym_false] = ACTIONS(1212),
    [anon_sym_NULL] = ACTIONS(1212),
    [anon_sym_nullptr] = ACTIONS(1212),
    [sym_comment] = ACTIONS(3),
  },
  [242] = {
    [sym_identifier] = ACTIONS(1152),
    [aux_sym_preproc_include_token1] = ACTIONS(1152),
    [aux_sym_preproc_def_token1] = ACTIONS(1152),
    [aux_sym_preproc_if_token1] = ACTIONS(1152),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1152),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1152),
    [sym_preproc_directive] = ACTIONS(1152),
    [anon_sym_LPAREN2] = ACTIONS(1154),
    [anon_sym_BANG] = ACTIONS(1154),
    [anon_sym_TILDE] = ACTIONS(1154),
    [anon_sym_DASH] = ACTIONS(1152),
    [anon_sym_PLUS] = ACTIONS(1152),
    [anon_sym_STAR] = ACTIONS(1154),
    [anon_sym_AMP] = ACTIONS(1154),
    [anon_sym_SEMI] = ACTIONS(1154),
    [anon_sym___extension__] = ACTIONS(1152),
    [anon_sym_typedef] = ACTIONS(1152),
    [anon_sym_extern] = ACTIONS(1152),
    [anon_sym___attribute__] = ACTIONS(1152),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1154),
    [anon_sym___declspec] = ACTIONS(1152),
    [anon_sym___cdecl] = ACTIONS(1152),
    [anon_sym___clrcall] = ACTIONS(1152),
    [anon_sym___stdcall] = ACTIONS(1152),
    [anon_sym___fastcall] = ACTIONS(1152),
    [anon_sym___thiscall] = ACTIONS(1152),
    [anon_sym___vectorcall] = ACTIONS(1152),
    [anon_sym_LBRACE] = ACTIONS(1154),
    [anon_sym_RBRACE] = ACTIONS(1154),
    [anon_sym_signed] = ACTIONS(1152),
    [anon_sym_unsigned] = ACTIONS(1152),
    [anon_sym_long] = ACTIONS(1152),
    [anon_sym_short] = ACTIONS(1152),
    [anon_sym_static] = ACTIONS(1152),
    [anon_sym_auto] = ACTIONS(1152),
    [anon_sym_register] = ACTIONS(1152),
    [anon_sym_inline] = ACTIONS(1152),
    [anon_sym___inline] = ACTIONS(1152),
    [anon_sym___inline__] = ACTIONS(1152),
    [anon_sym___forceinline] = ACTIONS(1152),
    [anon_sym_thread_local] = ACTIONS(1152),
    [anon_sym___thread] = ACTIONS(1152),
    [anon_sym_const] = ACTIONS(1152),
    [anon_sym_constexpr] = ACTIONS(1152),
    [anon_sym_volatile] = ACTIONS(1152),
    [anon_sym_restrict] = ACTIONS(1152),
    [anon_sym___restrict__] = ACTIONS(1152),
    [anon_sym__Atomic] = ACTIONS(1152),
    [anon_sym__Noreturn] = ACTIONS(1152),
    [anon_sym_noreturn] = ACTIONS(1152),
    [sym_primitive_type] = ACTIONS(1152),
    [anon_sym_enum] = ACTIONS(1152),
    [anon_sym_struct] = ACTIONS(1152),
    [anon_sym_union] = ACTIONS(1152),
    [anon_sym_if] = ACTIONS(1152),
    [anon_sym_else] = ACTIONS(1152),
    [anon_sym_switch] = ACTIONS(1152),
    [anon_sym_case] = ACTIONS(1152),
    [anon_sym_default] = ACTIONS(1152),
    [anon_sym_while] = ACTIONS(1152),
    [anon_sym_do] = ACTIONS(1152),
    [anon_sym_for] = ACTIONS(1152),
    [anon_sym_return] = ACTIONS(1152),
    [anon_sym_break] = ACTIONS(1152),
    [anon_sym_continue] = ACTIONS(1152),
    [anon_sym_goto] = ACTIONS(1152),
    [anon_sym___try] = ACTIONS(1152),
    [anon_sym___leave] = ACTIONS(1152),
    [anon_sym_DASH_DASH] = ACTIONS(1154),
    [anon_sym_PLUS_PLUS] = ACTIONS(1154),
    [anon_sym_sizeof] = ACTIONS(1152),
    [anon_sym___alignof__] = ACTIONS(1152),
    [anon_sym___alignof] = ACTIONS(1152),
    [anon_sym__alignof] = ACTIONS(1152),
    [anon_sym_alignof] = ACTIONS(1152),
    [anon_sym__Alignof] = ACTIONS(1152),
    [anon_sym_offsetof] = ACTIONS(1152),
    [anon_sym__Generic] = ACTIONS(1152),
    [anon_sym_asm] = ACTIONS(1152),
    [anon_sym___asm__] = ACTIONS(1152),
    [sym_number_literal] = ACTIONS(1154),
    [anon_sym_L_SQUOTE] = ACTIONS(1154),
    [anon_sym_u_SQUOTE] = ACTIONS(1154),
    [anon_sym_U_SQUOTE] = ACTIONS(1154),
    [anon_sym_u8_SQUOTE] = ACTIONS(1154),
    [anon_sym_SQUOTE] = ACTIONS(1154),
    [anon_sym_L_DQUOTE] = ACTIONS(1154),
    [anon_sym_u_DQUOTE] = ACTIONS(1154),
    [anon_sym_U_DQUOTE] = ACTIONS(1154),
    [anon_sym_u8_DQUOTE] = ACTIONS(1154),
    [anon_sym_DQUOTE] = ACTIONS(1154),
    [sym_true] = ACTIONS(1152),
    [sym_false] = ACTIONS(1152),
    [anon_sym_NULL] = ACTIONS(1152),
    [anon_sym_nullptr] = ACTIONS(1152),
    [sym_comment] = ACTIONS(3),
  },
  [243] = {
    [ts_builtin_sym_end] = ACTIONS(1226),
    [sym_identifier] = ACTIONS(1224),
    [aux_sym_preproc_include_token1] = ACTIONS(1224),
    [aux_sym_preproc_def_token1] = ACTIONS(1224),
    [aux_sym_preproc_if_token1] = ACTIONS(1224),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1224),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1224),
    [sym_preproc_directive] = ACTIONS(1224),
    [anon_sym_LPAREN2] = ACTIONS(1226),
    [anon_sym_BANG] = ACTIONS(1226),
    [anon_sym_TILDE] = ACTIONS(1226),
    [anon_sym_DASH] = ACTIONS(1224),
    [anon_sym_PLUS] = ACTIONS(1224),
    [anon_sym_STAR] = ACTIONS(1226),
    [anon_sym_AMP] = ACTIONS(1226),
    [anon_sym_SEMI] = ACTIONS(1226),
    [anon_sym___extension__] = ACTIONS(1224),
    [anon_sym_typedef] = ACTIONS(1224),
    [anon_sym_extern] = ACTIONS(1224),
    [anon_sym___attribute__] = ACTIONS(1224),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1226),
    [anon_sym___declspec] = ACTIONS(1224),
    [anon_sym___cdecl] = ACTIONS(1224),
    [anon_sym___clrcall] = ACTIONS(1224),
    [anon_sym___stdcall] = ACTIONS(1224),
    [anon_sym___fastcall] = ACTIONS(1224),
    [anon_sym___thiscall] = ACTIONS(1224),
    [anon_sym___vectorcall] = ACTIONS(1224),
    [anon_sym_LBRACE] = ACTIONS(1226),
    [anon_sym_signed] = ACTIONS(1224),
    [anon_sym_unsigned] = ACTIONS(1224),
    [anon_sym_long] = ACTIONS(1224),
    [anon_sym_short] = ACTIONS(1224),
    [anon_sym_static] = ACTIONS(1224),
    [anon_sym_auto] = ACTIONS(1224),
    [anon_sym_register] = ACTIONS(1224),
    [anon_sym_inline] = ACTIONS(1224),
    [anon_sym___inline] = ACTIONS(1224),
    [anon_sym___inline__] = ACTIONS(1224),
    [anon_sym___forceinline] = ACTIONS(1224),
    [anon_sym_thread_local] = ACTIONS(1224),
    [anon_sym___thread] = ACTIONS(1224),
    [anon_sym_const] = ACTIONS(1224),
    [anon_sym_constexpr] = ACTIONS(1224),
    [anon_sym_volatile] = ACTIONS(1224),
    [anon_sym_restrict] = ACTIONS(1224),
    [anon_sym___restrict__] = ACTIONS(1224),
    [anon_sym__Atomic] = ACTIONS(1224),
    [anon_sym__Noreturn] = ACTIONS(1224),
    [anon_sym_noreturn] = ACTIONS(1224),
    [sym_primitive_type] = ACTIONS(1224),
    [anon_sym_enum] = ACTIONS(1224),
    [anon_sym_struct] = ACTIONS(1224),
    [anon_sym_union] = ACTIONS(1224),
    [anon_sym_if] = ACTIONS(1224),
    [anon_sym_else] = ACTIONS(1224),
    [anon_sym_switch] = ACTIONS(1224),
    [anon_sym_case] = ACTIONS(1224),
    [anon_sym_default] = ACTIONS(1224),
    [anon_sym_while] = ACTIONS(1224),
    [anon_sym_do] = ACTIONS(1224),
    [anon_sym_for] = ACTIONS(1224),
    [anon_sym_return] = ACTIONS(1224),
    [anon_sym_break] = ACTIONS(1224),
    [anon_sym_continue] = ACTIONS(1224),
    [anon_sym_goto] = ACTIONS(1224),
    [anon_sym___try] = ACTIONS(1224),
    [anon_sym___leave] = ACTIONS(1224),
    [anon_sym_DASH_DASH] = ACTIONS(1226),
    [anon_sym_PLUS_PLUS] = ACTIONS(1226),
    [anon_sym_sizeof] = ACTIONS(1224),
    [anon_sym___alignof__] = ACTIONS(1224),
    [anon_sym___alignof] = ACTIONS(1224),
    [anon_sym__alignof] = ACTIONS(1224),
    [anon_sym_alignof] = ACTIONS(1224),
    [anon_sym__Alignof] = ACTIONS(1224),
    [anon_sym_offsetof] = ACTIONS(1224),
    [anon_sym__Generic] = ACTIONS(1224),
    [anon_sym_asm] = ACTIONS(1224),
    [anon_sym___asm__] = ACTIONS(1224),
    [sym_number_literal] = ACTIONS(1226),
    [anon_sym_L_SQUOTE] = ACTIONS(1226),
    [anon_sym_u_SQUOTE] = ACTIONS(1226),
    [anon_sym_U_SQUOTE] = ACTIONS(1226),
    [anon_sym_u8_SQUOTE] = ACTIONS(1226),
    [anon_sym_SQUOTE] = ACTIONS(1226),
    [anon_sym_L_DQUOTE] = ACTIONS(1226),
    [anon_sym_u_DQUOTE] = ACTIONS(1226),
    [anon_sym_U_DQUOTE] = ACTIONS(1226),
    [anon_sym_u8_DQUOTE] = ACTIONS(1226),
    [anon_sym_DQUOTE] = ACTIONS(1226),
    [sym_true] = ACTIONS(1224),
    [sym_false] = ACTIONS(1224),
    [anon_sym_NULL] = ACTIONS(1224),
    [anon_sym_nullptr] = ACTIONS(1224),
    [sym_comment] = ACTIONS(3),
  },
  [244] = {
    [ts_builtin_sym_end] = ACTIONS(1210),
    [sym_identifier] = ACTIONS(1208),
    [aux_sym_preproc_include_token1] = ACTIONS(1208),
    [aux_sym_preproc_def_token1] = ACTIONS(1208),
    [aux_sym_preproc_if_token1] = ACTIONS(1208),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1208),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1208),
    [sym_preproc_directive] = ACTIONS(1208),
    [anon_sym_LPAREN2] = ACTIONS(1210),
    [anon_sym_BANG] = ACTIONS(1210),
    [anon_sym_TILDE] = ACTIONS(1210),
    [anon_sym_DASH] = ACTIONS(1208),
    [anon_sym_PLUS] = ACTIONS(1208),
    [anon_sym_STAR] = ACTIONS(1210),
    [anon_sym_AMP] = ACTIONS(1210),
    [anon_sym_SEMI] = ACTIONS(1210),
    [anon_sym___extension__] = ACTIONS(1208),
    [anon_sym_typedef] = ACTIONS(1208),
    [anon_sym_extern] = ACTIONS(1208),
    [anon_sym___attribute__] = ACTIONS(1208),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1210),
    [anon_sym___declspec] = ACTIONS(1208),
    [anon_sym___cdecl] = ACTIONS(1208),
    [anon_sym___clrcall] = ACTIONS(1208),
    [anon_sym___stdcall] = ACTIONS(1208),
    [anon_sym___fastcall] = ACTIONS(1208),
    [anon_sym___thiscall] = ACTIONS(1208),
    [anon_sym___vectorcall] = ACTIONS(1208),
    [anon_sym_LBRACE] = ACTIONS(1210),
    [anon_sym_signed] = ACTIONS(1208),
    [anon_sym_unsigned] = ACTIONS(1208),
    [anon_sym_long] = ACTIONS(1208),
    [anon_sym_short] = ACTIONS(1208),
    [anon_sym_static] = ACTIONS(1208),
    [anon_sym_auto] = ACTIONS(1208),
    [anon_sym_register] = ACTIONS(1208),
    [anon_sym_inline] = ACTIONS(1208),
    [anon_sym___inline] = ACTIONS(1208),
    [anon_sym___inline__] = ACTIONS(1208),
    [anon_sym___forceinline] = ACTIONS(1208),
    [anon_sym_thread_local] = ACTIONS(1208),
    [anon_sym___thread] = ACTIONS(1208),
    [anon_sym_const] = ACTIONS(1208),
    [anon_sym_constexpr] = ACTIONS(1208),
    [anon_sym_volatile] = ACTIONS(1208),
    [anon_sym_restrict] = ACTIONS(1208),
    [anon_sym___restrict__] = ACTIONS(1208),
    [anon_sym__Atomic] = ACTIONS(1208),
    [anon_sym__Noreturn] = ACTIONS(1208),
    [anon_sym_noreturn] = ACTIONS(1208),
    [sym_primitive_type] = ACTIONS(1208),
    [anon_sym_enum] = ACTIONS(1208),
    [anon_sym_struct] = ACTIONS(1208),
    [anon_sym_union] = ACTIONS(1208),
    [anon_sym_if] = ACTIONS(1208),
    [anon_sym_else] = ACTIONS(1208),
    [anon_sym_switch] = ACTIONS(1208),
    [anon_sym_case] = ACTIONS(1208),
    [anon_sym_default] = ACTIONS(1208),
    [anon_sym_while] = ACTIONS(1208),
    [anon_sym_do] = ACTIONS(1208),
    [anon_sym_for] = ACTIONS(1208),
    [anon_sym_return] = ACTIONS(1208),
    [anon_sym_break] = ACTIONS(1208),
    [anon_sym_continue] = ACTIONS(1208),
    [anon_sym_goto] = ACTIONS(1208),
    [anon_sym___try] = ACTIONS(1208),
    [anon_sym___leave] = ACTIONS(1208),
    [anon_sym_DASH_DASH] = ACTIONS(1210),
    [anon_sym_PLUS_PLUS] = ACTIONS(1210),
    [anon_sym_sizeof] = ACTIONS(1208),
    [anon_sym___alignof__] = ACTIONS(1208),
    [anon_sym___alignof] = ACTIONS(1208),
    [anon_sym__alignof] = ACTIONS(1208),
    [anon_sym_alignof] = ACTIONS(1208),
    [anon_sym__Alignof] = ACTIONS(1208),
    [anon_sym_offsetof] = ACTIONS(1208),
    [anon_sym__Generic] = ACTIONS(1208),
    [anon_sym_asm] = ACTIONS(1208),
    [anon_sym___asm__] = ACTIONS(1208),
    [sym_number_literal] = ACTIONS(1210),
    [anon_sym_L_SQUOTE] = ACTIONS(1210),
    [anon_sym_u_SQUOTE] = ACTIONS(1210),
    [anon_sym_U_SQUOTE] = ACTIONS(1210),
    [anon_sym_u8_SQUOTE] = ACTIONS(1210),
    [anon_sym_SQUOTE] = ACTIONS(1210),
    [anon_sym_L_DQUOTE] = ACTIONS(1210),
    [anon_sym_u_DQUOTE] = ACTIONS(1210),
    [anon_sym_U_DQUOTE] = ACTIONS(1210),
    [anon_sym_u8_DQUOTE] = ACTIONS(1210),
    [anon_sym_DQUOTE] = ACTIONS(1210),
    [sym_true] = ACTIONS(1208),
    [sym_false] = ACTIONS(1208),
    [anon_sym_NULL] = ACTIONS(1208),
    [anon_sym_nullptr] = ACTIONS(1208),
    [sym_comment] = ACTIONS(3),
  },
  [245] = {
    [sym_identifier] = ACTIONS(1204),
    [aux_sym_preproc_include_token1] = ACTIONS(1204),
    [aux_sym_preproc_def_token1] = ACTIONS(1204),
    [aux_sym_preproc_if_token1] = ACTIONS(1204),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1204),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1204),
    [sym_preproc_directive] = ACTIONS(1204),
    [anon_sym_LPAREN2] = ACTIONS(1206),
    [anon_sym_BANG] = ACTIONS(1206),
    [anon_sym_TILDE] = ACTIONS(1206),
    [anon_sym_DASH] = ACTIONS(1204),
    [anon_sym_PLUS] = ACTIONS(1204),
    [anon_sym_STAR] = ACTIONS(1206),
    [anon_sym_AMP] = ACTIONS(1206),
    [anon_sym_SEMI] = ACTIONS(1206),
    [anon_sym___extension__] = ACTIONS(1204),
    [anon_sym_typedef] = ACTIONS(1204),
    [anon_sym_extern] = ACTIONS(1204),
    [anon_sym___attribute__] = ACTIONS(1204),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1206),
    [anon_sym___declspec] = ACTIONS(1204),
    [anon_sym___cdecl] = ACTIONS(1204),
    [anon_sym___clrcall] = ACTIONS(1204),
    [anon_sym___stdcall] = ACTIONS(1204),
    [anon_sym___fastcall] = ACTIONS(1204),
    [anon_sym___thiscall] = ACTIONS(1204),
    [anon_sym___vectorcall] = ACTIONS(1204),
    [anon_sym_LBRACE] = ACTIONS(1206),
    [anon_sym_RBRACE] = ACTIONS(1206),
    [anon_sym_signed] = ACTIONS(1204),
    [anon_sym_unsigned] = ACTIONS(1204),
    [anon_sym_long] = ACTIONS(1204),
    [anon_sym_short] = ACTIONS(1204),
    [anon_sym_static] = ACTIONS(1204),
    [anon_sym_auto] = ACTIONS(1204),
    [anon_sym_register] = ACTIONS(1204),
    [anon_sym_inline] = ACTIONS(1204),
    [anon_sym___inline] = ACTIONS(1204),
    [anon_sym___inline__] = ACTIONS(1204),
    [anon_sym___forceinline] = ACTIONS(1204),
    [anon_sym_thread_local] = ACTIONS(1204),
    [anon_sym___thread] = ACTIONS(1204),
    [anon_sym_const] = ACTIONS(1204),
    [anon_sym_constexpr] = ACTIONS(1204),
    [anon_sym_volatile] = ACTIONS(1204),
    [anon_sym_restrict] = ACTIONS(1204),
    [anon_sym___restrict__] = ACTIONS(1204),
    [anon_sym__Atomic] = ACTIONS(1204),
    [anon_sym__Noreturn] = ACTIONS(1204),
    [anon_sym_noreturn] = ACTIONS(1204),
    [sym_primitive_type] = ACTIONS(1204),
    [anon_sym_enum] = ACTIONS(1204),
    [anon_sym_struct] = ACTIONS(1204),
    [anon_sym_union] = ACTIONS(1204),
    [anon_sym_if] = ACTIONS(1204),
    [anon_sym_else] = ACTIONS(1204),
    [anon_sym_switch] = ACTIONS(1204),
    [anon_sym_case] = ACTIONS(1204),
    [anon_sym_default] = ACTIONS(1204),
    [anon_sym_while] = ACTIONS(1204),
    [anon_sym_do] = ACTIONS(1204),
    [anon_sym_for] = ACTIONS(1204),
    [anon_sym_return] = ACTIONS(1204),
    [anon_sym_break] = ACTIONS(1204),
    [anon_sym_continue] = ACTIONS(1204),
    [anon_sym_goto] = ACTIONS(1204),
    [anon_sym___try] = ACTIONS(1204),
    [anon_sym___leave] = ACTIONS(1204),
    [anon_sym_DASH_DASH] = ACTIONS(1206),
    [anon_sym_PLUS_PLUS] = ACTIONS(1206),
    [anon_sym_sizeof] = ACTIONS(1204),
    [anon_sym___alignof__] = ACTIONS(1204),
    [anon_sym___alignof] = ACTIONS(1204),
    [anon_sym__alignof] = ACTIONS(1204),
    [anon_sym_alignof] = ACTIONS(1204),
    [anon_sym__Alignof] = ACTIONS(1204),
    [anon_sym_offsetof] = ACTIONS(1204),
    [anon_sym__Generic] = ACTIONS(1204),
    [anon_sym_asm] = ACTIONS(1204),
    [anon_sym___asm__] = ACTIONS(1204),
    [sym_number_literal] = ACTIONS(1206),
    [anon_sym_L_SQUOTE] = ACTIONS(1206),
    [anon_sym_u_SQUOTE] = ACTIONS(1206),
    [anon_sym_U_SQUOTE] = ACTIONS(1206),
    [anon_sym_u8_SQUOTE] = ACTIONS(1206),
    [anon_sym_SQUOTE] = ACTIONS(1206),
    [anon_sym_L_DQUOTE] = ACTIONS(1206),
    [anon_sym_u_DQUOTE] = ACTIONS(1206),
    [anon_sym_U_DQUOTE] = ACTIONS(1206),
    [anon_sym_u8_DQUOTE] = ACTIONS(1206),
    [anon_sym_DQUOTE] = ACTIONS(1206),
    [sym_true] = ACTIONS(1204),
    [sym_false] = ACTIONS(1204),
    [anon_sym_NULL] = ACTIONS(1204),
    [anon_sym_nullptr] = ACTIONS(1204),
    [sym_comment] = ACTIONS(3),
  },
  [246] = {
    [sym_identifier] = ACTIONS(1196),
    [aux_sym_preproc_include_token1] = ACTIONS(1196),
    [aux_sym_preproc_def_token1] = ACTIONS(1196),
    [aux_sym_preproc_if_token1] = ACTIONS(1196),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1196),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1196),
    [sym_preproc_directive] = ACTIONS(1196),
    [anon_sym_LPAREN2] = ACTIONS(1198),
    [anon_sym_BANG] = ACTIONS(1198),
    [anon_sym_TILDE] = ACTIONS(1198),
    [anon_sym_DASH] = ACTIONS(1196),
    [anon_sym_PLUS] = ACTIONS(1196),
    [anon_sym_STAR] = ACTIONS(1198),
    [anon_sym_AMP] = ACTIONS(1198),
    [anon_sym_SEMI] = ACTIONS(1198),
    [anon_sym___extension__] = ACTIONS(1196),
    [anon_sym_typedef] = ACTIONS(1196),
    [anon_sym_extern] = ACTIONS(1196),
    [anon_sym___attribute__] = ACTIONS(1196),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1198),
    [anon_sym___declspec] = ACTIONS(1196),
    [anon_sym___cdecl] = ACTIONS(1196),
    [anon_sym___clrcall] = ACTIONS(1196),
    [anon_sym___stdcall] = ACTIONS(1196),
    [anon_sym___fastcall] = ACTIONS(1196),
    [anon_sym___thiscall] = ACTIONS(1196),
    [anon_sym___vectorcall] = ACTIONS(1196),
    [anon_sym_LBRACE] = ACTIONS(1198),
    [anon_sym_RBRACE] = ACTIONS(1198),
    [anon_sym_signed] = ACTIONS(1196),
    [anon_sym_unsigned] = ACTIONS(1196),
    [anon_sym_long] = ACTIONS(1196),
    [anon_sym_short] = ACTIONS(1196),
    [anon_sym_static] = ACTIONS(1196),
    [anon_sym_auto] = ACTIONS(1196),
    [anon_sym_register] = ACTIONS(1196),
    [anon_sym_inline] = ACTIONS(1196),
    [anon_sym___inline] = ACTIONS(1196),
    [anon_sym___inline__] = ACTIONS(1196),
    [anon_sym___forceinline] = ACTIONS(1196),
    [anon_sym_thread_local] = ACTIONS(1196),
    [anon_sym___thread] = ACTIONS(1196),
    [anon_sym_const] = ACTIONS(1196),
    [anon_sym_constexpr] = ACTIONS(1196),
    [anon_sym_volatile] = ACTIONS(1196),
    [anon_sym_restrict] = ACTIONS(1196),
    [anon_sym___restrict__] = ACTIONS(1196),
    [anon_sym__Atomic] = ACTIONS(1196),
    [anon_sym__Noreturn] = ACTIONS(1196),
    [anon_sym_noreturn] = ACTIONS(1196),
    [sym_primitive_type] = ACTIONS(1196),
    [anon_sym_enum] = ACTIONS(1196),
    [anon_sym_struct] = ACTIONS(1196),
    [anon_sym_union] = ACTIONS(1196),
    [anon_sym_if] = ACTIONS(1196),
    [anon_sym_else] = ACTIONS(1196),
    [anon_sym_switch] = ACTIONS(1196),
    [anon_sym_case] = ACTIONS(1196),
    [anon_sym_default] = ACTIONS(1196),
    [anon_sym_while] = ACTIONS(1196),
    [anon_sym_do] = ACTIONS(1196),
    [anon_sym_for] = ACTIONS(1196),
    [anon_sym_return] = ACTIONS(1196),
    [anon_sym_break] = ACTIONS(1196),
    [anon_sym_continue] = ACTIONS(1196),
    [anon_sym_goto] = ACTIONS(1196),
    [anon_sym___try] = ACTIONS(1196),
    [anon_sym___leave] = ACTIONS(1196),
    [anon_sym_DASH_DASH] = ACTIONS(1198),
    [anon_sym_PLUS_PLUS] = ACTIONS(1198),
    [anon_sym_sizeof] = ACTIONS(1196),
    [anon_sym___alignof__] = ACTIONS(1196),
    [anon_sym___alignof] = ACTIONS(1196),
    [anon_sym__alignof] = ACTIONS(1196),
    [anon_sym_alignof] = ACTIONS(1196),
    [anon_sym__Alignof] = ACTIONS(1196),
    [anon_sym_offsetof] = ACTIONS(1196),
    [anon_sym__Generic] = ACTIONS(1196),
    [anon_sym_asm] = ACTIONS(1196),
    [anon_sym___asm__] = ACTIONS(1196),
    [sym_number_literal] = ACTIONS(1198),
    [anon_sym_L_SQUOTE] = ACTIONS(1198),
    [anon_sym_u_SQUOTE] = ACTIONS(1198),
    [anon_sym_U_SQUOTE] = ACTIONS(1198),
    [anon_sym_u8_SQUOTE] = ACTIONS(1198),
    [anon_sym_SQUOTE] = ACTIONS(1198),
    [anon_sym_L_DQUOTE] = ACTIONS(1198),
    [anon_sym_u_DQUOTE] = ACTIONS(1198),
    [anon_sym_U_DQUOTE] = ACTIONS(1198),
    [anon_sym_u8_DQUOTE] = ACTIONS(1198),
    [anon_sym_DQUOTE] = ACTIONS(1198),
    [sym_true] = ACTIONS(1196),
    [sym_false] = ACTIONS(1196),
    [anon_sym_NULL] = ACTIONS(1196),
    [anon_sym_nullptr] = ACTIONS(1196),
    [sym_comment] = ACTIONS(3),
  },
  [247] = {
    [sym_identifier] = ACTIONS(1120),
    [aux_sym_preproc_include_token1] = ACTIONS(1120),
    [aux_sym_preproc_def_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token2] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1120),
    [sym_preproc_directive] = ACTIONS(1120),
    [anon_sym_LPAREN2] = ACTIONS(1122),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_DASH] = ACTIONS(1120),
    [anon_sym_PLUS] = ACTIONS(1120),
    [anon_sym_STAR] = ACTIONS(1122),
    [anon_sym_AMP] = ACTIONS(1122),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym___extension__] = ACTIONS(1120),
    [anon_sym_typedef] = ACTIONS(1120),
    [anon_sym_extern] = ACTIONS(1120),
    [anon_sym___attribute__] = ACTIONS(1120),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1122),
    [anon_sym___declspec] = ACTIONS(1120),
    [anon_sym___cdecl] = ACTIONS(1120),
    [anon_sym___clrcall] = ACTIONS(1120),
    [anon_sym___stdcall] = ACTIONS(1120),
    [anon_sym___fastcall] = ACTIONS(1120),
    [anon_sym___thiscall] = ACTIONS(1120),
    [anon_sym___vectorcall] = ACTIONS(1120),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_signed] = ACTIONS(1120),
    [anon_sym_unsigned] = ACTIONS(1120),
    [anon_sym_long] = ACTIONS(1120),
    [anon_sym_short] = ACTIONS(1120),
    [anon_sym_static] = ACTIONS(1120),
    [anon_sym_auto] = ACTIONS(1120),
    [anon_sym_register] = ACTIONS(1120),
    [anon_sym_inline] = ACTIONS(1120),
    [anon_sym___inline] = ACTIONS(1120),
    [anon_sym___inline__] = ACTIONS(1120),
    [anon_sym___forceinline] = ACTIONS(1120),
    [anon_sym_thread_local] = ACTIONS(1120),
    [anon_sym___thread] = ACTIONS(1120),
    [anon_sym_const] = ACTIONS(1120),
    [anon_sym_constexpr] = ACTIONS(1120),
    [anon_sym_volatile] = ACTIONS(1120),
    [anon_sym_restrict] = ACTIONS(1120),
    [anon_sym___restrict__] = ACTIONS(1120),
    [anon_sym__Atomic] = ACTIONS(1120),
    [anon_sym__Noreturn] = ACTIONS(1120),
    [anon_sym_noreturn] = ACTIONS(1120),
    [sym_primitive_type] = ACTIONS(1120),
    [anon_sym_enum] = ACTIONS(1120),
    [anon_sym_struct] = ACTIONS(1120),
    [anon_sym_union] = ACTIONS(1120),
    [anon_sym_if] = ACTIONS(1120),
    [anon_sym_else] = ACTIONS(1120),
    [anon_sym_switch] = ACTIONS(1120),
    [anon_sym_case] = ACTIONS(1120),
    [anon_sym_default] = ACTIONS(1120),
    [anon_sym_while] = ACTIONS(1120),
    [anon_sym_do] = ACTIONS(1120),
    [anon_sym_for] = ACTIONS(1120),
    [anon_sym_return] = ACTIONS(1120),
    [anon_sym_break] = ACTIONS(1120),
    [anon_sym_continue] = ACTIONS(1120),
    [anon_sym_goto] = ACTIONS(1120),
    [anon_sym___try] = ACTIONS(1120),
    [anon_sym___leave] = ACTIONS(1120),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_sizeof] = ACTIONS(1120),
    [anon_sym___alignof__] = ACTIONS(1120),
    [anon_sym___alignof] = ACTIONS(1120),
    [anon_sym__alignof] = ACTIONS(1120),
    [anon_sym_alignof] = ACTIONS(1120),
    [anon_sym__Alignof] = ACTIONS(1120),
    [anon_sym_offsetof] = ACTIONS(1120),
    [anon_sym__Generic] = ACTIONS(1120),
    [anon_sym_asm] = ACTIONS(1120),
    [anon_sym___asm__] = ACTIONS(1120),
    [sym_number_literal] = ACTIONS(1122),
    [anon_sym_L_SQUOTE] = ACTIONS(1122),
    [anon_sym_u_SQUOTE] = ACTIONS(1122),
    [anon_sym_U_SQUOTE] = ACTIONS(1122),
    [anon_sym_u8_SQUOTE] = ACTIONS(1122),
    [anon_sym_SQUOTE] = ACTIONS(1122),
    [anon_sym_L_DQUOTE] = ACTIONS(1122),
    [anon_sym_u_DQUOTE] = ACTIONS(1122),
    [anon_sym_U_DQUOTE] = ACTIONS(1122),
    [anon_sym_u8_DQUOTE] = ACTIONS(1122),
    [anon_sym_DQUOTE] = ACTIONS(1122),
    [sym_true] = ACTIONS(1120),
    [sym_false] = ACTIONS(1120),
    [anon_sym_NULL] = ACTIONS(1120),
    [anon_sym_nullptr] = ACTIONS(1120),
    [sym_comment] = ACTIONS(3),
  },
  [248] = {
    [sym_identifier] = ACTIONS(1224),
    [aux_sym_preproc_include_token1] = ACTIONS(1224),
    [aux_sym_preproc_def_token1] = ACTIONS(1224),
    [aux_sym_preproc_if_token1] = ACTIONS(1224),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1224),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1224),
    [sym_preproc_directive] = ACTIONS(1224),
    [anon_sym_LPAREN2] = ACTIONS(1226),
    [anon_sym_BANG] = ACTIONS(1226),
    [anon_sym_TILDE] = ACTIONS(1226),
    [anon_sym_DASH] = ACTIONS(1224),
    [anon_sym_PLUS] = ACTIONS(1224),
    [anon_sym_STAR] = ACTIONS(1226),
    [anon_sym_AMP] = ACTIONS(1226),
    [anon_sym_SEMI] = ACTIONS(1226),
    [anon_sym___extension__] = ACTIONS(1224),
    [anon_sym_typedef] = ACTIONS(1224),
    [anon_sym_extern] = ACTIONS(1224),
    [anon_sym___attribute__] = ACTIONS(1224),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1226),
    [anon_sym___declspec] = ACTIONS(1224),
    [anon_sym___cdecl] = ACTIONS(1224),
    [anon_sym___clrcall] = ACTIONS(1224),
    [anon_sym___stdcall] = ACTIONS(1224),
    [anon_sym___fastcall] = ACTIONS(1224),
    [anon_sym___thiscall] = ACTIONS(1224),
    [anon_sym___vectorcall] = ACTIONS(1224),
    [anon_sym_LBRACE] = ACTIONS(1226),
    [anon_sym_RBRACE] = ACTIONS(1226),
    [anon_sym_signed] = ACTIONS(1224),
    [anon_sym_unsigned] = ACTIONS(1224),
    [anon_sym_long] = ACTIONS(1224),
    [anon_sym_short] = ACTIONS(1224),
    [anon_sym_static] = ACTIONS(1224),
    [anon_sym_auto] = ACTIONS(1224),
    [anon_sym_register] = ACTIONS(1224),
    [anon_sym_inline] = ACTIONS(1224),
    [anon_sym___inline] = ACTIONS(1224),
    [anon_sym___inline__] = ACTIONS(1224),
    [anon_sym___forceinline] = ACTIONS(1224),
    [anon_sym_thread_local] = ACTIONS(1224),
    [anon_sym___thread] = ACTIONS(1224),
    [anon_sym_const] = ACTIONS(1224),
    [anon_sym_constexpr] = ACTIONS(1224),
    [anon_sym_volatile] = ACTIONS(1224),
    [anon_sym_restrict] = ACTIONS(1224),
    [anon_sym___restrict__] = ACTIONS(1224),
    [anon_sym__Atomic] = ACTIONS(1224),
    [anon_sym__Noreturn] = ACTIONS(1224),
    [anon_sym_noreturn] = ACTIONS(1224),
    [sym_primitive_type] = ACTIONS(1224),
    [anon_sym_enum] = ACTIONS(1224),
    [anon_sym_struct] = ACTIONS(1224),
    [anon_sym_union] = ACTIONS(1224),
    [anon_sym_if] = ACTIONS(1224),
    [anon_sym_else] = ACTIONS(1224),
    [anon_sym_switch] = ACTIONS(1224),
    [anon_sym_case] = ACTIONS(1224),
    [anon_sym_default] = ACTIONS(1224),
    [anon_sym_while] = ACTIONS(1224),
    [anon_sym_do] = ACTIONS(1224),
    [anon_sym_for] = ACTIONS(1224),
    [anon_sym_return] = ACTIONS(1224),
    [anon_sym_break] = ACTIONS(1224),
    [anon_sym_continue] = ACTIONS(1224),
    [anon_sym_goto] = ACTIONS(1224),
    [anon_sym___try] = ACTIONS(1224),
    [anon_sym___leave] = ACTIONS(1224),
    [anon_sym_DASH_DASH] = ACTIONS(1226),
    [anon_sym_PLUS_PLUS] = ACTIONS(1226),
    [anon_sym_sizeof] = ACTIONS(1224),
    [anon_sym___alignof__] = ACTIONS(1224),
    [anon_sym___alignof] = ACTIONS(1224),
    [anon_sym__alignof] = ACTIONS(1224),
    [anon_sym_alignof] = ACTIONS(1224),
    [anon_sym__Alignof] = ACTIONS(1224),
    [anon_sym_offsetof] = ACTIONS(1224),
    [anon_sym__Generic] = ACTIONS(1224),
    [anon_sym_asm] = ACTIONS(1224),
    [anon_sym___asm__] = ACTIONS(1224),
    [sym_number_literal] = ACTIONS(1226),
    [anon_sym_L_SQUOTE] = ACTIONS(1226),
    [anon_sym_u_SQUOTE] = ACTIONS(1226),
    [anon_sym_U_SQUOTE] = ACTIONS(1226),
    [anon_sym_u8_SQUOTE] = ACTIONS(1226),
    [anon_sym_SQUOTE] = ACTIONS(1226),
    [anon_sym_L_DQUOTE] = ACTIONS(1226),
    [anon_sym_u_DQUOTE] = ACTIONS(1226),
    [anon_sym_U_DQUOTE] = ACTIONS(1226),
    [anon_sym_u8_DQUOTE] = ACTIONS(1226),
    [anon_sym_DQUOTE] = ACTIONS(1226),
    [sym_true] = ACTIONS(1224),
    [sym_false] = ACTIONS(1224),
    [anon_sym_NULL] = ACTIONS(1224),
    [anon_sym_nullptr] = ACTIONS(1224),
    [sym_comment] = ACTIONS(3),
  },
  [249] = {
    [sym_identifier] = ACTIONS(1208),
    [aux_sym_preproc_include_token1] = ACTIONS(1208),
    [aux_sym_preproc_def_token1] = ACTIONS(1208),
    [aux_sym_preproc_if_token1] = ACTIONS(1208),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1208),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1208),
    [sym_preproc_directive] = ACTIONS(1208),
    [anon_sym_LPAREN2] = ACTIONS(1210),
    [anon_sym_BANG] = ACTIONS(1210),
    [anon_sym_TILDE] = ACTIONS(1210),
    [anon_sym_DASH] = ACTIONS(1208),
    [anon_sym_PLUS] = ACTIONS(1208),
    [anon_sym_STAR] = ACTIONS(1210),
    [anon_sym_AMP] = ACTIONS(1210),
    [anon_sym_SEMI] = ACTIONS(1210),
    [anon_sym___extension__] = ACTIONS(1208),
    [anon_sym_typedef] = ACTIONS(1208),
    [anon_sym_extern] = ACTIONS(1208),
    [anon_sym___attribute__] = ACTIONS(1208),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1210),
    [anon_sym___declspec] = ACTIONS(1208),
    [anon_sym___cdecl] = ACTIONS(1208),
    [anon_sym___clrcall] = ACTIONS(1208),
    [anon_sym___stdcall] = ACTIONS(1208),
    [anon_sym___fastcall] = ACTIONS(1208),
    [anon_sym___thiscall] = ACTIONS(1208),
    [anon_sym___vectorcall] = ACTIONS(1208),
    [anon_sym_LBRACE] = ACTIONS(1210),
    [anon_sym_RBRACE] = ACTIONS(1210),
    [anon_sym_signed] = ACTIONS(1208),
    [anon_sym_unsigned] = ACTIONS(1208),
    [anon_sym_long] = ACTIONS(1208),
    [anon_sym_short] = ACTIONS(1208),
    [anon_sym_static] = ACTIONS(1208),
    [anon_sym_auto] = ACTIONS(1208),
    [anon_sym_register] = ACTIONS(1208),
    [anon_sym_inline] = ACTIONS(1208),
    [anon_sym___inline] = ACTIONS(1208),
    [anon_sym___inline__] = ACTIONS(1208),
    [anon_sym___forceinline] = ACTIONS(1208),
    [anon_sym_thread_local] = ACTIONS(1208),
    [anon_sym___thread] = ACTIONS(1208),
    [anon_sym_const] = ACTIONS(1208),
    [anon_sym_constexpr] = ACTIONS(1208),
    [anon_sym_volatile] = ACTIONS(1208),
    [anon_sym_restrict] = ACTIONS(1208),
    [anon_sym___restrict__] = ACTIONS(1208),
    [anon_sym__Atomic] = ACTIONS(1208),
    [anon_sym__Noreturn] = ACTIONS(1208),
    [anon_sym_noreturn] = ACTIONS(1208),
    [sym_primitive_type] = ACTIONS(1208),
    [anon_sym_enum] = ACTIONS(1208),
    [anon_sym_struct] = ACTIONS(1208),
    [anon_sym_union] = ACTIONS(1208),
    [anon_sym_if] = ACTIONS(1208),
    [anon_sym_else] = ACTIONS(1208),
    [anon_sym_switch] = ACTIONS(1208),
    [anon_sym_case] = ACTIONS(1208),
    [anon_sym_default] = ACTIONS(1208),
    [anon_sym_while] = ACTIONS(1208),
    [anon_sym_do] = ACTIONS(1208),
    [anon_sym_for] = ACTIONS(1208),
    [anon_sym_return] = ACTIONS(1208),
    [anon_sym_break] = ACTIONS(1208),
    [anon_sym_continue] = ACTIONS(1208),
    [anon_sym_goto] = ACTIONS(1208),
    [anon_sym___try] = ACTIONS(1208),
    [anon_sym___leave] = ACTIONS(1208),
    [anon_sym_DASH_DASH] = ACTIONS(1210),
    [anon_sym_PLUS_PLUS] = ACTIONS(1210),
    [anon_sym_sizeof] = ACTIONS(1208),
    [anon_sym___alignof__] = ACTIONS(1208),
    [anon_sym___alignof] = ACTIONS(1208),
    [anon_sym__alignof] = ACTIONS(1208),
    [anon_sym_alignof] = ACTIONS(1208),
    [anon_sym__Alignof] = ACTIONS(1208),
    [anon_sym_offsetof] = ACTIONS(1208),
    [anon_sym__Generic] = ACTIONS(1208),
    [anon_sym_asm] = ACTIONS(1208),
    [anon_sym___asm__] = ACTIONS(1208),
    [sym_number_literal] = ACTIONS(1210),
    [anon_sym_L_SQUOTE] = ACTIONS(1210),
    [anon_sym_u_SQUOTE] = ACTIONS(1210),
    [anon_sym_U_SQUOTE] = ACTIONS(1210),
    [anon_sym_u8_SQUOTE] = ACTIONS(1210),
    [anon_sym_SQUOTE] = ACTIONS(1210),
    [anon_sym_L_DQUOTE] = ACTIONS(1210),
    [anon_sym_u_DQUOTE] = ACTIONS(1210),
    [anon_sym_U_DQUOTE] = ACTIONS(1210),
    [anon_sym_u8_DQUOTE] = ACTIONS(1210),
    [anon_sym_DQUOTE] = ACTIONS(1210),
    [sym_true] = ACTIONS(1208),
    [sym_false] = ACTIONS(1208),
    [anon_sym_NULL] = ACTIONS(1208),
    [anon_sym_nullptr] = ACTIONS(1208),
    [sym_comment] = ACTIONS(3),
  },
  [250] = {
    [sym_identifier] = ACTIONS(1192),
    [aux_sym_preproc_include_token1] = ACTIONS(1192),
    [aux_sym_preproc_def_token1] = ACTIONS(1192),
    [aux_sym_preproc_if_token1] = ACTIONS(1192),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1192),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1192),
    [sym_preproc_directive] = ACTIONS(1192),
    [anon_sym_LPAREN2] = ACTIONS(1194),
    [anon_sym_BANG] = ACTIONS(1194),
    [anon_sym_TILDE] = ACTIONS(1194),
    [anon_sym_DASH] = ACTIONS(1192),
    [anon_sym_PLUS] = ACTIONS(1192),
    [anon_sym_STAR] = ACTIONS(1194),
    [anon_sym_AMP] = ACTIONS(1194),
    [anon_sym_SEMI] = ACTIONS(1194),
    [anon_sym___extension__] = ACTIONS(1192),
    [anon_sym_typedef] = ACTIONS(1192),
    [anon_sym_extern] = ACTIONS(1192),
    [anon_sym___attribute__] = ACTIONS(1192),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1194),
    [anon_sym___declspec] = ACTIONS(1192),
    [anon_sym___cdecl] = ACTIONS(1192),
    [anon_sym___clrcall] = ACTIONS(1192),
    [anon_sym___stdcall] = ACTIONS(1192),
    [anon_sym___fastcall] = ACTIONS(1192),
    [anon_sym___thiscall] = ACTIONS(1192),
    [anon_sym___vectorcall] = ACTIONS(1192),
    [anon_sym_LBRACE] = ACTIONS(1194),
    [anon_sym_RBRACE] = ACTIONS(1194),
    [anon_sym_signed] = ACTIONS(1192),
    [anon_sym_unsigned] = ACTIONS(1192),
    [anon_sym_long] = ACTIONS(1192),
    [anon_sym_short] = ACTIONS(1192),
    [anon_sym_static] = ACTIONS(1192),
    [anon_sym_auto] = ACTIONS(1192),
    [anon_sym_register] = ACTIONS(1192),
    [anon_sym_inline] = ACTIONS(1192),
    [anon_sym___inline] = ACTIONS(1192),
    [anon_sym___inline__] = ACTIONS(1192),
    [anon_sym___forceinline] = ACTIONS(1192),
    [anon_sym_thread_local] = ACTIONS(1192),
    [anon_sym___thread] = ACTIONS(1192),
    [anon_sym_const] = ACTIONS(1192),
    [anon_sym_constexpr] = ACTIONS(1192),
    [anon_sym_volatile] = ACTIONS(1192),
    [anon_sym_restrict] = ACTIONS(1192),
    [anon_sym___restrict__] = ACTIONS(1192),
    [anon_sym__Atomic] = ACTIONS(1192),
    [anon_sym__Noreturn] = ACTIONS(1192),
    [anon_sym_noreturn] = ACTIONS(1192),
    [sym_primitive_type] = ACTIONS(1192),
    [anon_sym_enum] = ACTIONS(1192),
    [anon_sym_struct] = ACTIONS(1192),
    [anon_sym_union] = ACTIONS(1192),
    [anon_sym_if] = ACTIONS(1192),
    [anon_sym_else] = ACTIONS(1192),
    [anon_sym_switch] = ACTIONS(1192),
    [anon_sym_case] = ACTIONS(1192),
    [anon_sym_default] = ACTIONS(1192),
    [anon_sym_while] = ACTIONS(1192),
    [anon_sym_do] = ACTIONS(1192),
    [anon_sym_for] = ACTIONS(1192),
    [anon_sym_return] = ACTIONS(1192),
    [anon_sym_break] = ACTIONS(1192),
    [anon_sym_continue] = ACTIONS(1192),
    [anon_sym_goto] = ACTIONS(1192),
    [anon_sym___try] = ACTIONS(1192),
    [anon_sym___leave] = ACTIONS(1192),
    [anon_sym_DASH_DASH] = ACTIONS(1194),
    [anon_sym_PLUS_PLUS] = ACTIONS(1194),
    [anon_sym_sizeof] = ACTIONS(1192),
    [anon_sym___alignof__] = ACTIONS(1192),
    [anon_sym___alignof] = ACTIONS(1192),
    [anon_sym__alignof] = ACTIONS(1192),
    [anon_sym_alignof] = ACTIONS(1192),
    [anon_sym__Alignof] = ACTIONS(1192),
    [anon_sym_offsetof] = ACTIONS(1192),
    [anon_sym__Generic] = ACTIONS(1192),
    [anon_sym_asm] = ACTIONS(1192),
    [anon_sym___asm__] = ACTIONS(1192),
    [sym_number_literal] = ACTIONS(1194),
    [anon_sym_L_SQUOTE] = ACTIONS(1194),
    [anon_sym_u_SQUOTE] = ACTIONS(1194),
    [anon_sym_U_SQUOTE] = ACTIONS(1194),
    [anon_sym_u8_SQUOTE] = ACTIONS(1194),
    [anon_sym_SQUOTE] = ACTIONS(1194),
    [anon_sym_L_DQUOTE] = ACTIONS(1194),
    [anon_sym_u_DQUOTE] = ACTIONS(1194),
    [anon_sym_U_DQUOTE] = ACTIONS(1194),
    [anon_sym_u8_DQUOTE] = ACTIONS(1194),
    [anon_sym_DQUOTE] = ACTIONS(1194),
    [sym_true] = ACTIONS(1192),
    [sym_false] = ACTIONS(1192),
    [anon_sym_NULL] = ACTIONS(1192),
    [anon_sym_nullptr] = ACTIONS(1192),
    [sym_comment] = ACTIONS(3),
  },
  [251] = {
    [sym_identifier] = ACTIONS(1176),
    [aux_sym_preproc_include_token1] = ACTIONS(1176),
    [aux_sym_preproc_def_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token2] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1176),
    [sym_preproc_directive] = ACTIONS(1176),
    [anon_sym_LPAREN2] = ACTIONS(1178),
    [anon_sym_BANG] = ACTIONS(1178),
    [anon_sym_TILDE] = ACTIONS(1178),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_STAR] = ACTIONS(1178),
    [anon_sym_AMP] = ACTIONS(1178),
    [anon_sym_SEMI] = ACTIONS(1178),
    [anon_sym___extension__] = ACTIONS(1176),
    [anon_sym_typedef] = ACTIONS(1176),
    [anon_sym_extern] = ACTIONS(1176),
    [anon_sym___attribute__] = ACTIONS(1176),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1178),
    [anon_sym___declspec] = ACTIONS(1176),
    [anon_sym___cdecl] = ACTIONS(1176),
    [anon_sym___clrcall] = ACTIONS(1176),
    [anon_sym___stdcall] = ACTIONS(1176),
    [anon_sym___fastcall] = ACTIONS(1176),
    [anon_sym___thiscall] = ACTIONS(1176),
    [anon_sym___vectorcall] = ACTIONS(1176),
    [anon_sym_LBRACE] = ACTIONS(1178),
    [anon_sym_signed] = ACTIONS(1176),
    [anon_sym_unsigned] = ACTIONS(1176),
    [anon_sym_long] = ACTIONS(1176),
    [anon_sym_short] = ACTIONS(1176),
    [anon_sym_static] = ACTIONS(1176),
    [anon_sym_auto] = ACTIONS(1176),
    [anon_sym_register] = ACTIONS(1176),
    [anon_sym_inline] = ACTIONS(1176),
    [anon_sym___inline] = ACTIONS(1176),
    [anon_sym___inline__] = ACTIONS(1176),
    [anon_sym___forceinline] = ACTIONS(1176),
    [anon_sym_thread_local] = ACTIONS(1176),
    [anon_sym___thread] = ACTIONS(1176),
    [anon_sym_const] = ACTIONS(1176),
    [anon_sym_constexpr] = ACTIONS(1176),
    [anon_sym_volatile] = ACTIONS(1176),
    [anon_sym_restrict] = ACTIONS(1176),
    [anon_sym___restrict__] = ACTIONS(1176),
    [anon_sym__Atomic] = ACTIONS(1176),
    [anon_sym__Noreturn] = ACTIONS(1176),
    [anon_sym_noreturn] = ACTIONS(1176),
    [sym_primitive_type] = ACTIONS(1176),
    [anon_sym_enum] = ACTIONS(1176),
    [anon_sym_struct] = ACTIONS(1176),
    [anon_sym_union] = ACTIONS(1176),
    [anon_sym_if] = ACTIONS(1176),
    [anon_sym_else] = ACTIONS(1176),
    [anon_sym_switch] = ACTIONS(1176),
    [anon_sym_case] = ACTIONS(1176),
    [anon_sym_default] = ACTIONS(1176),
    [anon_sym_while] = ACTIONS(1176),
    [anon_sym_do] = ACTIONS(1176),
    [anon_sym_for] = ACTIONS(1176),
    [anon_sym_return] = ACTIONS(1176),
    [anon_sym_break] = ACTIONS(1176),
    [anon_sym_continue] = ACTIONS(1176),
    [anon_sym_goto] = ACTIONS(1176),
    [anon_sym___try] = ACTIONS(1176),
    [anon_sym___leave] = ACTIONS(1176),
    [anon_sym_DASH_DASH] = ACTIONS(1178),
    [anon_sym_PLUS_PLUS] = ACTIONS(1178),
    [anon_sym_sizeof] = ACTIONS(1176),
    [anon_sym___alignof__] = ACTIONS(1176),
    [anon_sym___alignof] = ACTIONS(1176),
    [anon_sym__alignof] = ACTIONS(1176),
    [anon_sym_alignof] = ACTIONS(1176),
    [anon_sym__Alignof] = ACTIONS(1176),
    [anon_sym_offsetof] = ACTIONS(1176),
    [anon_sym__Generic] = ACTIONS(1176),
    [anon_sym_asm] = ACTIONS(1176),
    [anon_sym___asm__] = ACTIONS(1176),
    [sym_number_literal] = ACTIONS(1178),
    [anon_sym_L_SQUOTE] = ACTIONS(1178),
    [anon_sym_u_SQUOTE] = ACTIONS(1178),
    [anon_sym_U_SQUOTE] = ACTIONS(1178),
    [anon_sym_u8_SQUOTE] = ACTIONS(1178),
    [anon_sym_SQUOTE] = ACTIONS(1178),
    [anon_sym_L_DQUOTE] = ACTIONS(1178),
    [anon_sym_u_DQUOTE] = ACTIONS(1178),
    [anon_sym_U_DQUOTE] = ACTIONS(1178),
    [anon_sym_u8_DQUOTE] = ACTIONS(1178),
    [anon_sym_DQUOTE] = ACTIONS(1178),
    [sym_true] = ACTIONS(1176),
    [sym_false] = ACTIONS(1176),
    [anon_sym_NULL] = ACTIONS(1176),
    [anon_sym_nullptr] = ACTIONS(1176),
    [sym_comment] = ACTIONS(3),
  },
  [252] = {
    [sym_identifier] = ACTIONS(1176),
    [aux_sym_preproc_include_token1] = ACTIONS(1176),
    [aux_sym_preproc_def_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token2] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1176),
    [sym_preproc_directive] = ACTIONS(1176),
    [anon_sym_LPAREN2] = ACTIONS(1178),
    [anon_sym_BANG] = ACTIONS(1178),
    [anon_sym_TILDE] = ACTIONS(1178),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_STAR] = ACTIONS(1178),
    [anon_sym_AMP] = ACTIONS(1178),
    [anon_sym_SEMI] = ACTIONS(1178),
    [anon_sym___extension__] = ACTIONS(1176),
    [anon_sym_typedef] = ACTIONS(1176),
    [anon_sym_extern] = ACTIONS(1176),
    [anon_sym___attribute__] = ACTIONS(1176),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1178),
    [anon_sym___declspec] = ACTIONS(1176),
    [anon_sym___cdecl] = ACTIONS(1176),
    [anon_sym___clrcall] = ACTIONS(1176),
    [anon_sym___stdcall] = ACTIONS(1176),
    [anon_sym___fastcall] = ACTIONS(1176),
    [anon_sym___thiscall] = ACTIONS(1176),
    [anon_sym___vectorcall] = ACTIONS(1176),
    [anon_sym_LBRACE] = ACTIONS(1178),
    [anon_sym_signed] = ACTIONS(1176),
    [anon_sym_unsigned] = ACTIONS(1176),
    [anon_sym_long] = ACTIONS(1176),
    [anon_sym_short] = ACTIONS(1176),
    [anon_sym_static] = ACTIONS(1176),
    [anon_sym_auto] = ACTIONS(1176),
    [anon_sym_register] = ACTIONS(1176),
    [anon_sym_inline] = ACTIONS(1176),
    [anon_sym___inline] = ACTIONS(1176),
    [anon_sym___inline__] = ACTIONS(1176),
    [anon_sym___forceinline] = ACTIONS(1176),
    [anon_sym_thread_local] = ACTIONS(1176),
    [anon_sym___thread] = ACTIONS(1176),
    [anon_sym_const] = ACTIONS(1176),
    [anon_sym_constexpr] = ACTIONS(1176),
    [anon_sym_volatile] = ACTIONS(1176),
    [anon_sym_restrict] = ACTIONS(1176),
    [anon_sym___restrict__] = ACTIONS(1176),
    [anon_sym__Atomic] = ACTIONS(1176),
    [anon_sym__Noreturn] = ACTIONS(1176),
    [anon_sym_noreturn] = ACTIONS(1176),
    [sym_primitive_type] = ACTIONS(1176),
    [anon_sym_enum] = ACTIONS(1176),
    [anon_sym_struct] = ACTIONS(1176),
    [anon_sym_union] = ACTIONS(1176),
    [anon_sym_if] = ACTIONS(1176),
    [anon_sym_else] = ACTIONS(1176),
    [anon_sym_switch] = ACTIONS(1176),
    [anon_sym_case] = ACTIONS(1176),
    [anon_sym_default] = ACTIONS(1176),
    [anon_sym_while] = ACTIONS(1176),
    [anon_sym_do] = ACTIONS(1176),
    [anon_sym_for] = ACTIONS(1176),
    [anon_sym_return] = ACTIONS(1176),
    [anon_sym_break] = ACTIONS(1176),
    [anon_sym_continue] = ACTIONS(1176),
    [anon_sym_goto] = ACTIONS(1176),
    [anon_sym___try] = ACTIONS(1176),
    [anon_sym___leave] = ACTIONS(1176),
    [anon_sym_DASH_DASH] = ACTIONS(1178),
    [anon_sym_PLUS_PLUS] = ACTIONS(1178),
    [anon_sym_sizeof] = ACTIONS(1176),
    [anon_sym___alignof__] = ACTIONS(1176),
    [anon_sym___alignof] = ACTIONS(1176),
    [anon_sym__alignof] = ACTIONS(1176),
    [anon_sym_alignof] = ACTIONS(1176),
    [anon_sym__Alignof] = ACTIONS(1176),
    [anon_sym_offsetof] = ACTIONS(1176),
    [anon_sym__Generic] = ACTIONS(1176),
    [anon_sym_asm] = ACTIONS(1176),
    [anon_sym___asm__] = ACTIONS(1176),
    [sym_number_literal] = ACTIONS(1178),
    [anon_sym_L_SQUOTE] = ACTIONS(1178),
    [anon_sym_u_SQUOTE] = ACTIONS(1178),
    [anon_sym_U_SQUOTE] = ACTIONS(1178),
    [anon_sym_u8_SQUOTE] = ACTIONS(1178),
    [anon_sym_SQUOTE] = ACTIONS(1178),
    [anon_sym_L_DQUOTE] = ACTIONS(1178),
    [anon_sym_u_DQUOTE] = ACTIONS(1178),
    [anon_sym_U_DQUOTE] = ACTIONS(1178),
    [anon_sym_u8_DQUOTE] = ACTIONS(1178),
    [anon_sym_DQUOTE] = ACTIONS(1178),
    [sym_true] = ACTIONS(1176),
    [sym_false] = ACTIONS(1176),
    [anon_sym_NULL] = ACTIONS(1176),
    [anon_sym_nullptr] = ACTIONS(1176),
    [sym_comment] = ACTIONS(3),
  },
  [253] = {
    [sym_identifier] = ACTIONS(1200),
    [aux_sym_preproc_include_token1] = ACTIONS(1200),
    [aux_sym_preproc_def_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token2] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1200),
    [sym_preproc_directive] = ACTIONS(1200),
    [anon_sym_LPAREN2] = ACTIONS(1202),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_DASH] = ACTIONS(1200),
    [anon_sym_PLUS] = ACTIONS(1200),
    [anon_sym_STAR] = ACTIONS(1202),
    [anon_sym_AMP] = ACTIONS(1202),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym___extension__] = ACTIONS(1200),
    [anon_sym_typedef] = ACTIONS(1200),
    [anon_sym_extern] = ACTIONS(1200),
    [anon_sym___attribute__] = ACTIONS(1200),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1202),
    [anon_sym___declspec] = ACTIONS(1200),
    [anon_sym___cdecl] = ACTIONS(1200),
    [anon_sym___clrcall] = ACTIONS(1200),
    [anon_sym___stdcall] = ACTIONS(1200),
    [anon_sym___fastcall] = ACTIONS(1200),
    [anon_sym___thiscall] = ACTIONS(1200),
    [anon_sym___vectorcall] = ACTIONS(1200),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_signed] = ACTIONS(1200),
    [anon_sym_unsigned] = ACTIONS(1200),
    [anon_sym_long] = ACTIONS(1200),
    [anon_sym_short] = ACTIONS(1200),
    [anon_sym_static] = ACTIONS(1200),
    [anon_sym_auto] = ACTIONS(1200),
    [anon_sym_register] = ACTIONS(1200),
    [anon_sym_inline] = ACTIONS(1200),
    [anon_sym___inline] = ACTIONS(1200),
    [anon_sym___inline__] = ACTIONS(1200),
    [anon_sym___forceinline] = ACTIONS(1200),
    [anon_sym_thread_local] = ACTIONS(1200),
    [anon_sym___thread] = ACTIONS(1200),
    [anon_sym_const] = ACTIONS(1200),
    [anon_sym_constexpr] = ACTIONS(1200),
    [anon_sym_volatile] = ACTIONS(1200),
    [anon_sym_restrict] = ACTIONS(1200),
    [anon_sym___restrict__] = ACTIONS(1200),
    [anon_sym__Atomic] = ACTIONS(1200),
    [anon_sym__Noreturn] = ACTIONS(1200),
    [anon_sym_noreturn] = ACTIONS(1200),
    [sym_primitive_type] = ACTIONS(1200),
    [anon_sym_enum] = ACTIONS(1200),
    [anon_sym_struct] = ACTIONS(1200),
    [anon_sym_union] = ACTIONS(1200),
    [anon_sym_if] = ACTIONS(1200),
    [anon_sym_else] = ACTIONS(1200),
    [anon_sym_switch] = ACTIONS(1200),
    [anon_sym_case] = ACTIONS(1200),
    [anon_sym_default] = ACTIONS(1200),
    [anon_sym_while] = ACTIONS(1200),
    [anon_sym_do] = ACTIONS(1200),
    [anon_sym_for] = ACTIONS(1200),
    [anon_sym_return] = ACTIONS(1200),
    [anon_sym_break] = ACTIONS(1200),
    [anon_sym_continue] = ACTIONS(1200),
    [anon_sym_goto] = ACTIONS(1200),
    [anon_sym___try] = ACTIONS(1200),
    [anon_sym___leave] = ACTIONS(1200),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_sizeof] = ACTIONS(1200),
    [anon_sym___alignof__] = ACTIONS(1200),
    [anon_sym___alignof] = ACTIONS(1200),
    [anon_sym__alignof] = ACTIONS(1200),
    [anon_sym_alignof] = ACTIONS(1200),
    [anon_sym__Alignof] = ACTIONS(1200),
    [anon_sym_offsetof] = ACTIONS(1200),
    [anon_sym__Generic] = ACTIONS(1200),
    [anon_sym_asm] = ACTIONS(1200),
    [anon_sym___asm__] = ACTIONS(1200),
    [sym_number_literal] = ACTIONS(1202),
    [anon_sym_L_SQUOTE] = ACTIONS(1202),
    [anon_sym_u_SQUOTE] = ACTIONS(1202),
    [anon_sym_U_SQUOTE] = ACTIONS(1202),
    [anon_sym_u8_SQUOTE] = ACTIONS(1202),
    [anon_sym_SQUOTE] = ACTIONS(1202),
    [anon_sym_L_DQUOTE] = ACTIONS(1202),
    [anon_sym_u_DQUOTE] = ACTIONS(1202),
    [anon_sym_U_DQUOTE] = ACTIONS(1202),
    [anon_sym_u8_DQUOTE] = ACTIONS(1202),
    [anon_sym_DQUOTE] = ACTIONS(1202),
    [sym_true] = ACTIONS(1200),
    [sym_false] = ACTIONS(1200),
    [anon_sym_NULL] = ACTIONS(1200),
    [anon_sym_nullptr] = ACTIONS(1200),
    [sym_comment] = ACTIONS(3),
  },
  [254] = {
    [sym_identifier] = ACTIONS(1200),
    [aux_sym_preproc_include_token1] = ACTIONS(1200),
    [aux_sym_preproc_def_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token2] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1200),
    [sym_preproc_directive] = ACTIONS(1200),
    [anon_sym_LPAREN2] = ACTIONS(1202),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_DASH] = ACTIONS(1200),
    [anon_sym_PLUS] = ACTIONS(1200),
    [anon_sym_STAR] = ACTIONS(1202),
    [anon_sym_AMP] = ACTIONS(1202),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym___extension__] = ACTIONS(1200),
    [anon_sym_typedef] = ACTIONS(1200),
    [anon_sym_extern] = ACTIONS(1200),
    [anon_sym___attribute__] = ACTIONS(1200),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1202),
    [anon_sym___declspec] = ACTIONS(1200),
    [anon_sym___cdecl] = ACTIONS(1200),
    [anon_sym___clrcall] = ACTIONS(1200),
    [anon_sym___stdcall] = ACTIONS(1200),
    [anon_sym___fastcall] = ACTIONS(1200),
    [anon_sym___thiscall] = ACTIONS(1200),
    [anon_sym___vectorcall] = ACTIONS(1200),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_signed] = ACTIONS(1200),
    [anon_sym_unsigned] = ACTIONS(1200),
    [anon_sym_long] = ACTIONS(1200),
    [anon_sym_short] = ACTIONS(1200),
    [anon_sym_static] = ACTIONS(1200),
    [anon_sym_auto] = ACTIONS(1200),
    [anon_sym_register] = ACTIONS(1200),
    [anon_sym_inline] = ACTIONS(1200),
    [anon_sym___inline] = ACTIONS(1200),
    [anon_sym___inline__] = ACTIONS(1200),
    [anon_sym___forceinline] = ACTIONS(1200),
    [anon_sym_thread_local] = ACTIONS(1200),
    [anon_sym___thread] = ACTIONS(1200),
    [anon_sym_const] = ACTIONS(1200),
    [anon_sym_constexpr] = ACTIONS(1200),
    [anon_sym_volatile] = ACTIONS(1200),
    [anon_sym_restrict] = ACTIONS(1200),
    [anon_sym___restrict__] = ACTIONS(1200),
    [anon_sym__Atomic] = ACTIONS(1200),
    [anon_sym__Noreturn] = ACTIONS(1200),
    [anon_sym_noreturn] = ACTIONS(1200),
    [sym_primitive_type] = ACTIONS(1200),
    [anon_sym_enum] = ACTIONS(1200),
    [anon_sym_struct] = ACTIONS(1200),
    [anon_sym_union] = ACTIONS(1200),
    [anon_sym_if] = ACTIONS(1200),
    [anon_sym_else] = ACTIONS(1200),
    [anon_sym_switch] = ACTIONS(1200),
    [anon_sym_case] = ACTIONS(1200),
    [anon_sym_default] = ACTIONS(1200),
    [anon_sym_while] = ACTIONS(1200),
    [anon_sym_do] = ACTIONS(1200),
    [anon_sym_for] = ACTIONS(1200),
    [anon_sym_return] = ACTIONS(1200),
    [anon_sym_break] = ACTIONS(1200),
    [anon_sym_continue] = ACTIONS(1200),
    [anon_sym_goto] = ACTIONS(1200),
    [anon_sym___try] = ACTIONS(1200),
    [anon_sym___leave] = ACTIONS(1200),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_sizeof] = ACTIONS(1200),
    [anon_sym___alignof__] = ACTIONS(1200),
    [anon_sym___alignof] = ACTIONS(1200),
    [anon_sym__alignof] = ACTIONS(1200),
    [anon_sym_alignof] = ACTIONS(1200),
    [anon_sym__Alignof] = ACTIONS(1200),
    [anon_sym_offsetof] = ACTIONS(1200),
    [anon_sym__Generic] = ACTIONS(1200),
    [anon_sym_asm] = ACTIONS(1200),
    [anon_sym___asm__] = ACTIONS(1200),
    [sym_number_literal] = ACTIONS(1202),
    [anon_sym_L_SQUOTE] = ACTIONS(1202),
    [anon_sym_u_SQUOTE] = ACTIONS(1202),
    [anon_sym_U_SQUOTE] = ACTIONS(1202),
    [anon_sym_u8_SQUOTE] = ACTIONS(1202),
    [anon_sym_SQUOTE] = ACTIONS(1202),
    [anon_sym_L_DQUOTE] = ACTIONS(1202),
    [anon_sym_u_DQUOTE] = ACTIONS(1202),
    [anon_sym_U_DQUOTE] = ACTIONS(1202),
    [anon_sym_u8_DQUOTE] = ACTIONS(1202),
    [anon_sym_DQUOTE] = ACTIONS(1202),
    [sym_true] = ACTIONS(1200),
    [sym_false] = ACTIONS(1200),
    [anon_sym_NULL] = ACTIONS(1200),
    [anon_sym_nullptr] = ACTIONS(1200),
    [sym_comment] = ACTIONS(3),
  },
  [255] = {
    [sym_identifier] = ACTIONS(1172),
    [aux_sym_preproc_include_token1] = ACTIONS(1172),
    [aux_sym_preproc_def_token1] = ACTIONS(1172),
    [aux_sym_preproc_if_token1] = ACTIONS(1172),
    [aux_sym_preproc_if_token2] = ACTIONS(1172),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1172),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1172),
    [sym_preproc_directive] = ACTIONS(1172),
    [anon_sym_LPAREN2] = ACTIONS(1174),
    [anon_sym_BANG] = ACTIONS(1174),
    [anon_sym_TILDE] = ACTIONS(1174),
    [anon_sym_DASH] = ACTIONS(1172),
    [anon_sym_PLUS] = ACTIONS(1172),
    [anon_sym_STAR] = ACTIONS(1174),
    [anon_sym_AMP] = ACTIONS(1174),
    [anon_sym_SEMI] = ACTIONS(1174),
    [anon_sym___extension__] = ACTIONS(1172),
    [anon_sym_typedef] = ACTIONS(1172),
    [anon_sym_extern] = ACTIONS(1172),
    [anon_sym___attribute__] = ACTIONS(1172),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1174),
    [anon_sym___declspec] = ACTIONS(1172),
    [anon_sym___cdecl] = ACTIONS(1172),
    [anon_sym___clrcall] = ACTIONS(1172),
    [anon_sym___stdcall] = ACTIONS(1172),
    [anon_sym___fastcall] = ACTIONS(1172),
    [anon_sym___thiscall] = ACTIONS(1172),
    [anon_sym___vectorcall] = ACTIONS(1172),
    [anon_sym_LBRACE] = ACTIONS(1174),
    [anon_sym_signed] = ACTIONS(1172),
    [anon_sym_unsigned] = ACTIONS(1172),
    [anon_sym_long] = ACTIONS(1172),
    [anon_sym_short] = ACTIONS(1172),
    [anon_sym_static] = ACTIONS(1172),
    [anon_sym_auto] = ACTIONS(1172),
    [anon_sym_register] = ACTIONS(1172),
    [anon_sym_inline] = ACTIONS(1172),
    [anon_sym___inline] = ACTIONS(1172),
    [anon_sym___inline__] = ACTIONS(1172),
    [anon_sym___forceinline] = ACTIONS(1172),
    [anon_sym_thread_local] = ACTIONS(1172),
    [anon_sym___thread] = ACTIONS(1172),
    [anon_sym_const] = ACTIONS(1172),
    [anon_sym_constexpr] = ACTIONS(1172),
    [anon_sym_volatile] = ACTIONS(1172),
    [anon_sym_restrict] = ACTIONS(1172),
    [anon_sym___restrict__] = ACTIONS(1172),
    [anon_sym__Atomic] = ACTIONS(1172),
    [anon_sym__Noreturn] = ACTIONS(1172),
    [anon_sym_noreturn] = ACTIONS(1172),
    [sym_primitive_type] = ACTIONS(1172),
    [anon_sym_enum] = ACTIONS(1172),
    [anon_sym_struct] = ACTIONS(1172),
    [anon_sym_union] = ACTIONS(1172),
    [anon_sym_if] = ACTIONS(1172),
    [anon_sym_else] = ACTIONS(1172),
    [anon_sym_switch] = ACTIONS(1172),
    [anon_sym_case] = ACTIONS(1172),
    [anon_sym_default] = ACTIONS(1172),
    [anon_sym_while] = ACTIONS(1172),
    [anon_sym_do] = ACTIONS(1172),
    [anon_sym_for] = ACTIONS(1172),
    [anon_sym_return] = ACTIONS(1172),
    [anon_sym_break] = ACTIONS(1172),
    [anon_sym_continue] = ACTIONS(1172),
    [anon_sym_goto] = ACTIONS(1172),
    [anon_sym___try] = ACTIONS(1172),
    [anon_sym___leave] = ACTIONS(1172),
    [anon_sym_DASH_DASH] = ACTIONS(1174),
    [anon_sym_PLUS_PLUS] = ACTIONS(1174),
    [anon_sym_sizeof] = ACTIONS(1172),
    [anon_sym___alignof__] = ACTIONS(1172),
    [anon_sym___alignof] = ACTIONS(1172),
    [anon_sym__alignof] = ACTIONS(1172),
    [anon_sym_alignof] = ACTIONS(1172),
    [anon_sym__Alignof] = ACTIONS(1172),
    [anon_sym_offsetof] = ACTIONS(1172),
    [anon_sym__Generic] = ACTIONS(1172),
    [anon_sym_asm] = ACTIONS(1172),
    [anon_sym___asm__] = ACTIONS(1172),
    [sym_number_literal] = ACTIONS(1174),
    [anon_sym_L_SQUOTE] = ACTIONS(1174),
    [anon_sym_u_SQUOTE] = ACTIONS(1174),
    [anon_sym_U_SQUOTE] = ACTIONS(1174),
    [anon_sym_u8_SQUOTE] = ACTIONS(1174),
    [anon_sym_SQUOTE] = ACTIONS(1174),
    [anon_sym_L_DQUOTE] = ACTIONS(1174),
    [anon_sym_u_DQUOTE] = ACTIONS(1174),
    [anon_sym_U_DQUOTE] = ACTIONS(1174),
    [anon_sym_u8_DQUOTE] = ACTIONS(1174),
    [anon_sym_DQUOTE] = ACTIONS(1174),
    [sym_true] = ACTIONS(1172),
    [sym_false] = ACTIONS(1172),
    [anon_sym_NULL] = ACTIONS(1172),
    [anon_sym_nullptr] = ACTIONS(1172),
    [sym_comment] = ACTIONS(3),
  },
  [256] = {
    [ts_builtin_sym_end] = ACTIONS(1146),
    [sym_identifier] = ACTIONS(1144),
    [aux_sym_preproc_include_token1] = ACTIONS(1144),
    [aux_sym_preproc_def_token1] = ACTIONS(1144),
    [aux_sym_preproc_if_token1] = ACTIONS(1144),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1144),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1144),
    [sym_preproc_directive] = ACTIONS(1144),
    [anon_sym_LPAREN2] = ACTIONS(1146),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_STAR] = ACTIONS(1146),
    [anon_sym_AMP] = ACTIONS(1146),
    [anon_sym_SEMI] = ACTIONS(1146),
    [anon_sym___extension__] = ACTIONS(1144),
    [anon_sym_typedef] = ACTIONS(1144),
    [anon_sym_extern] = ACTIONS(1144),
    [anon_sym___attribute__] = ACTIONS(1144),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1146),
    [anon_sym___declspec] = ACTIONS(1144),
    [anon_sym___cdecl] = ACTIONS(1144),
    [anon_sym___clrcall] = ACTIONS(1144),
    [anon_sym___stdcall] = ACTIONS(1144),
    [anon_sym___fastcall] = ACTIONS(1144),
    [anon_sym___thiscall] = ACTIONS(1144),
    [anon_sym___vectorcall] = ACTIONS(1144),
    [anon_sym_LBRACE] = ACTIONS(1146),
    [anon_sym_signed] = ACTIONS(1144),
    [anon_sym_unsigned] = ACTIONS(1144),
    [anon_sym_long] = ACTIONS(1144),
    [anon_sym_short] = ACTIONS(1144),
    [anon_sym_static] = ACTIONS(1144),
    [anon_sym_auto] = ACTIONS(1144),
    [anon_sym_register] = ACTIONS(1144),
    [anon_sym_inline] = ACTIONS(1144),
    [anon_sym___inline] = ACTIONS(1144),
    [anon_sym___inline__] = ACTIONS(1144),
    [anon_sym___forceinline] = ACTIONS(1144),
    [anon_sym_thread_local] = ACTIONS(1144),
    [anon_sym___thread] = ACTIONS(1144),
    [anon_sym_const] = ACTIONS(1144),
    [anon_sym_constexpr] = ACTIONS(1144),
    [anon_sym_volatile] = ACTIONS(1144),
    [anon_sym_restrict] = ACTIONS(1144),
    [anon_sym___restrict__] = ACTIONS(1144),
    [anon_sym__Atomic] = ACTIONS(1144),
    [anon_sym__Noreturn] = ACTIONS(1144),
    [anon_sym_noreturn] = ACTIONS(1144),
    [sym_primitive_type] = ACTIONS(1144),
    [anon_sym_enum] = ACTIONS(1144),
    [anon_sym_struct] = ACTIONS(1144),
    [anon_sym_union] = ACTIONS(1144),
    [anon_sym_if] = ACTIONS(1144),
    [anon_sym_else] = ACTIONS(1144),
    [anon_sym_switch] = ACTIONS(1144),
    [anon_sym_case] = ACTIONS(1144),
    [anon_sym_default] = ACTIONS(1144),
    [anon_sym_while] = ACTIONS(1144),
    [anon_sym_do] = ACTIONS(1144),
    [anon_sym_for] = ACTIONS(1144),
    [anon_sym_return] = ACTIONS(1144),
    [anon_sym_break] = ACTIONS(1144),
    [anon_sym_continue] = ACTIONS(1144),
    [anon_sym_goto] = ACTIONS(1144),
    [anon_sym___try] = ACTIONS(1144),
    [anon_sym___leave] = ACTIONS(1144),
    [anon_sym_DASH_DASH] = ACTIONS(1146),
    [anon_sym_PLUS_PLUS] = ACTIONS(1146),
    [anon_sym_sizeof] = ACTIONS(1144),
    [anon_sym___alignof__] = ACTIONS(1144),
    [anon_sym___alignof] = ACTIONS(1144),
    [anon_sym__alignof] = ACTIONS(1144),
    [anon_sym_alignof] = ACTIONS(1144),
    [anon_sym__Alignof] = ACTIONS(1144),
    [anon_sym_offsetof] = ACTIONS(1144),
    [anon_sym__Generic] = ACTIONS(1144),
    [anon_sym_asm] = ACTIONS(1144),
    [anon_sym___asm__] = ACTIONS(1144),
    [sym_number_literal] = ACTIONS(1146),
    [anon_sym_L_SQUOTE] = ACTIONS(1146),
    [anon_sym_u_SQUOTE] = ACTIONS(1146),
    [anon_sym_U_SQUOTE] = ACTIONS(1146),
    [anon_sym_u8_SQUOTE] = ACTIONS(1146),
    [anon_sym_SQUOTE] = ACTIONS(1146),
    [anon_sym_L_DQUOTE] = ACTIONS(1146),
    [anon_sym_u_DQUOTE] = ACTIONS(1146),
    [anon_sym_U_DQUOTE] = ACTIONS(1146),
    [anon_sym_u8_DQUOTE] = ACTIONS(1146),
    [anon_sym_DQUOTE] = ACTIONS(1146),
    [sym_true] = ACTIONS(1144),
    [sym_false] = ACTIONS(1144),
    [anon_sym_NULL] = ACTIONS(1144),
    [anon_sym_nullptr] = ACTIONS(1144),
    [sym_comment] = ACTIONS(3),
  },
  [257] = {
    [sym_identifier] = ACTIONS(1168),
    [aux_sym_preproc_include_token1] = ACTIONS(1168),
    [aux_sym_preproc_def_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token2] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1168),
    [sym_preproc_directive] = ACTIONS(1168),
    [anon_sym_LPAREN2] = ACTIONS(1170),
    [anon_sym_BANG] = ACTIONS(1170),
    [anon_sym_TILDE] = ACTIONS(1170),
    [anon_sym_DASH] = ACTIONS(1168),
    [anon_sym_PLUS] = ACTIONS(1168),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1170),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym___extension__] = ACTIONS(1168),
    [anon_sym_typedef] = ACTIONS(1168),
    [anon_sym_extern] = ACTIONS(1168),
    [anon_sym___attribute__] = ACTIONS(1168),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1170),
    [anon_sym___declspec] = ACTIONS(1168),
    [anon_sym___cdecl] = ACTIONS(1168),
    [anon_sym___clrcall] = ACTIONS(1168),
    [anon_sym___stdcall] = ACTIONS(1168),
    [anon_sym___fastcall] = ACTIONS(1168),
    [anon_sym___thiscall] = ACTIONS(1168),
    [anon_sym___vectorcall] = ACTIONS(1168),
    [anon_sym_LBRACE] = ACTIONS(1170),
    [anon_sym_signed] = ACTIONS(1168),
    [anon_sym_unsigned] = ACTIONS(1168),
    [anon_sym_long] = ACTIONS(1168),
    [anon_sym_short] = ACTIONS(1168),
    [anon_sym_static] = ACTIONS(1168),
    [anon_sym_auto] = ACTIONS(1168),
    [anon_sym_register] = ACTIONS(1168),
    [anon_sym_inline] = ACTIONS(1168),
    [anon_sym___inline] = ACTIONS(1168),
    [anon_sym___inline__] = ACTIONS(1168),
    [anon_sym___forceinline] = ACTIONS(1168),
    [anon_sym_thread_local] = ACTIONS(1168),
    [anon_sym___thread] = ACTIONS(1168),
    [anon_sym_const] = ACTIONS(1168),
    [anon_sym_constexpr] = ACTIONS(1168),
    [anon_sym_volatile] = ACTIONS(1168),
    [anon_sym_restrict] = ACTIONS(1168),
    [anon_sym___restrict__] = ACTIONS(1168),
    [anon_sym__Atomic] = ACTIONS(1168),
    [anon_sym__Noreturn] = ACTIONS(1168),
    [anon_sym_noreturn] = ACTIONS(1168),
    [sym_primitive_type] = ACTIONS(1168),
    [anon_sym_enum] = ACTIONS(1168),
    [anon_sym_struct] = ACTIONS(1168),
    [anon_sym_union] = ACTIONS(1168),
    [anon_sym_if] = ACTIONS(1168),
    [anon_sym_else] = ACTIONS(1168),
    [anon_sym_switch] = ACTIONS(1168),
    [anon_sym_case] = ACTIONS(1168),
    [anon_sym_default] = ACTIONS(1168),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_do] = ACTIONS(1168),
    [anon_sym_for] = ACTIONS(1168),
    [anon_sym_return] = ACTIONS(1168),
    [anon_sym_break] = ACTIONS(1168),
    [anon_sym_continue] = ACTIONS(1168),
    [anon_sym_goto] = ACTIONS(1168),
    [anon_sym___try] = ACTIONS(1168),
    [anon_sym___leave] = ACTIONS(1168),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_sizeof] = ACTIONS(1168),
    [anon_sym___alignof__] = ACTIONS(1168),
    [anon_sym___alignof] = ACTIONS(1168),
    [anon_sym__alignof] = ACTIONS(1168),
    [anon_sym_alignof] = ACTIONS(1168),
    [anon_sym__Alignof] = ACTIONS(1168),
    [anon_sym_offsetof] = ACTIONS(1168),
    [anon_sym__Generic] = ACTIONS(1168),
    [anon_sym_asm] = ACTIONS(1168),
    [anon_sym___asm__] = ACTIONS(1168),
    [sym_number_literal] = ACTIONS(1170),
    [anon_sym_L_SQUOTE] = ACTIONS(1170),
    [anon_sym_u_SQUOTE] = ACTIONS(1170),
    [anon_sym_U_SQUOTE] = ACTIONS(1170),
    [anon_sym_u8_SQUOTE] = ACTIONS(1170),
    [anon_sym_SQUOTE] = ACTIONS(1170),
    [anon_sym_L_DQUOTE] = ACTIONS(1170),
    [anon_sym_u_DQUOTE] = ACTIONS(1170),
    [anon_sym_U_DQUOTE] = ACTIONS(1170),
    [anon_sym_u8_DQUOTE] = ACTIONS(1170),
    [anon_sym_DQUOTE] = ACTIONS(1170),
    [sym_true] = ACTIONS(1168),
    [sym_false] = ACTIONS(1168),
    [anon_sym_NULL] = ACTIONS(1168),
    [anon_sym_nullptr] = ACTIONS(1168),
    [sym_comment] = ACTIONS(3),
  },
  [258] = {
    [sym_identifier] = ACTIONS(1148),
    [aux_sym_preproc_include_token1] = ACTIONS(1148),
    [aux_sym_preproc_def_token1] = ACTIONS(1148),
    [aux_sym_preproc_if_token1] = ACTIONS(1148),
    [aux_sym_preproc_if_token2] = ACTIONS(1148),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1148),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1148),
    [sym_preproc_directive] = ACTIONS(1148),
    [anon_sym_LPAREN2] = ACTIONS(1150),
    [anon_sym_BANG] = ACTIONS(1150),
    [anon_sym_TILDE] = ACTIONS(1150),
    [anon_sym_DASH] = ACTIONS(1148),
    [anon_sym_PLUS] = ACTIONS(1148),
    [anon_sym_STAR] = ACTIONS(1150),
    [anon_sym_AMP] = ACTIONS(1150),
    [anon_sym_SEMI] = ACTIONS(1150),
    [anon_sym___extension__] = ACTIONS(1148),
    [anon_sym_typedef] = ACTIONS(1148),
    [anon_sym_extern] = ACTIONS(1148),
    [anon_sym___attribute__] = ACTIONS(1148),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1150),
    [anon_sym___declspec] = ACTIONS(1148),
    [anon_sym___cdecl] = ACTIONS(1148),
    [anon_sym___clrcall] = ACTIONS(1148),
    [anon_sym___stdcall] = ACTIONS(1148),
    [anon_sym___fastcall] = ACTIONS(1148),
    [anon_sym___thiscall] = ACTIONS(1148),
    [anon_sym___vectorcall] = ACTIONS(1148),
    [anon_sym_LBRACE] = ACTIONS(1150),
    [anon_sym_signed] = ACTIONS(1148),
    [anon_sym_unsigned] = ACTIONS(1148),
    [anon_sym_long] = ACTIONS(1148),
    [anon_sym_short] = ACTIONS(1148),
    [anon_sym_static] = ACTIONS(1148),
    [anon_sym_auto] = ACTIONS(1148),
    [anon_sym_register] = ACTIONS(1148),
    [anon_sym_inline] = ACTIONS(1148),
    [anon_sym___inline] = ACTIONS(1148),
    [anon_sym___inline__] = ACTIONS(1148),
    [anon_sym___forceinline] = ACTIONS(1148),
    [anon_sym_thread_local] = ACTIONS(1148),
    [anon_sym___thread] = ACTIONS(1148),
    [anon_sym_const] = ACTIONS(1148),
    [anon_sym_constexpr] = ACTIONS(1148),
    [anon_sym_volatile] = ACTIONS(1148),
    [anon_sym_restrict] = ACTIONS(1148),
    [anon_sym___restrict__] = ACTIONS(1148),
    [anon_sym__Atomic] = ACTIONS(1148),
    [anon_sym__Noreturn] = ACTIONS(1148),
    [anon_sym_noreturn] = ACTIONS(1148),
    [sym_primitive_type] = ACTIONS(1148),
    [anon_sym_enum] = ACTIONS(1148),
    [anon_sym_struct] = ACTIONS(1148),
    [anon_sym_union] = ACTIONS(1148),
    [anon_sym_if] = ACTIONS(1148),
    [anon_sym_else] = ACTIONS(1148),
    [anon_sym_switch] = ACTIONS(1148),
    [anon_sym_case] = ACTIONS(1148),
    [anon_sym_default] = ACTIONS(1148),
    [anon_sym_while] = ACTIONS(1148),
    [anon_sym_do] = ACTIONS(1148),
    [anon_sym_for] = ACTIONS(1148),
    [anon_sym_return] = ACTIONS(1148),
    [anon_sym_break] = ACTIONS(1148),
    [anon_sym_continue] = ACTIONS(1148),
    [anon_sym_goto] = ACTIONS(1148),
    [anon_sym___try] = ACTIONS(1148),
    [anon_sym___leave] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1150),
    [anon_sym_PLUS_PLUS] = ACTIONS(1150),
    [anon_sym_sizeof] = ACTIONS(1148),
    [anon_sym___alignof__] = ACTIONS(1148),
    [anon_sym___alignof] = ACTIONS(1148),
    [anon_sym__alignof] = ACTIONS(1148),
    [anon_sym_alignof] = ACTIONS(1148),
    [anon_sym__Alignof] = ACTIONS(1148),
    [anon_sym_offsetof] = ACTIONS(1148),
    [anon_sym__Generic] = ACTIONS(1148),
    [anon_sym_asm] = ACTIONS(1148),
    [anon_sym___asm__] = ACTIONS(1148),
    [sym_number_literal] = ACTIONS(1150),
    [anon_sym_L_SQUOTE] = ACTIONS(1150),
    [anon_sym_u_SQUOTE] = ACTIONS(1150),
    [anon_sym_U_SQUOTE] = ACTIONS(1150),
    [anon_sym_u8_SQUOTE] = ACTIONS(1150),
    [anon_sym_SQUOTE] = ACTIONS(1150),
    [anon_sym_L_DQUOTE] = ACTIONS(1150),
    [anon_sym_u_DQUOTE] = ACTIONS(1150),
    [anon_sym_U_DQUOTE] = ACTIONS(1150),
    [anon_sym_u8_DQUOTE] = ACTIONS(1150),
    [anon_sym_DQUOTE] = ACTIONS(1150),
    [sym_true] = ACTIONS(1148),
    [sym_false] = ACTIONS(1148),
    [anon_sym_NULL] = ACTIONS(1148),
    [anon_sym_nullptr] = ACTIONS(1148),
    [sym_comment] = ACTIONS(3),
  },
  [259] = {
    [sym_identifier] = ACTIONS(1188),
    [aux_sym_preproc_include_token1] = ACTIONS(1188),
    [aux_sym_preproc_def_token1] = ACTIONS(1188),
    [aux_sym_preproc_if_token1] = ACTIONS(1188),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1188),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1188),
    [sym_preproc_directive] = ACTIONS(1188),
    [anon_sym_LPAREN2] = ACTIONS(1190),
    [anon_sym_BANG] = ACTIONS(1190),
    [anon_sym_TILDE] = ACTIONS(1190),
    [anon_sym_DASH] = ACTIONS(1188),
    [anon_sym_PLUS] = ACTIONS(1188),
    [anon_sym_STAR] = ACTIONS(1190),
    [anon_sym_AMP] = ACTIONS(1190),
    [anon_sym_SEMI] = ACTIONS(1190),
    [anon_sym___extension__] = ACTIONS(1188),
    [anon_sym_typedef] = ACTIONS(1188),
    [anon_sym_extern] = ACTIONS(1188),
    [anon_sym___attribute__] = ACTIONS(1188),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1190),
    [anon_sym___declspec] = ACTIONS(1188),
    [anon_sym___cdecl] = ACTIONS(1188),
    [anon_sym___clrcall] = ACTIONS(1188),
    [anon_sym___stdcall] = ACTIONS(1188),
    [anon_sym___fastcall] = ACTIONS(1188),
    [anon_sym___thiscall] = ACTIONS(1188),
    [anon_sym___vectorcall] = ACTIONS(1188),
    [anon_sym_LBRACE] = ACTIONS(1190),
    [anon_sym_RBRACE] = ACTIONS(1190),
    [anon_sym_signed] = ACTIONS(1188),
    [anon_sym_unsigned] = ACTIONS(1188),
    [anon_sym_long] = ACTIONS(1188),
    [anon_sym_short] = ACTIONS(1188),
    [anon_sym_static] = ACTIONS(1188),
    [anon_sym_auto] = ACTIONS(1188),
    [anon_sym_register] = ACTIONS(1188),
    [anon_sym_inline] = ACTIONS(1188),
    [anon_sym___inline] = ACTIONS(1188),
    [anon_sym___inline__] = ACTIONS(1188),
    [anon_sym___forceinline] = ACTIONS(1188),
    [anon_sym_thread_local] = ACTIONS(1188),
    [anon_sym___thread] = ACTIONS(1188),
    [anon_sym_const] = ACTIONS(1188),
    [anon_sym_constexpr] = ACTIONS(1188),
    [anon_sym_volatile] = ACTIONS(1188),
    [anon_sym_restrict] = ACTIONS(1188),
    [anon_sym___restrict__] = ACTIONS(1188),
    [anon_sym__Atomic] = ACTIONS(1188),
    [anon_sym__Noreturn] = ACTIONS(1188),
    [anon_sym_noreturn] = ACTIONS(1188),
    [sym_primitive_type] = ACTIONS(1188),
    [anon_sym_enum] = ACTIONS(1188),
    [anon_sym_struct] = ACTIONS(1188),
    [anon_sym_union] = ACTIONS(1188),
    [anon_sym_if] = ACTIONS(1188),
    [anon_sym_else] = ACTIONS(1188),
    [anon_sym_switch] = ACTIONS(1188),
    [anon_sym_case] = ACTIONS(1188),
    [anon_sym_default] = ACTIONS(1188),
    [anon_sym_while] = ACTIONS(1188),
    [anon_sym_do] = ACTIONS(1188),
    [anon_sym_for] = ACTIONS(1188),
    [anon_sym_return] = ACTIONS(1188),
    [anon_sym_break] = ACTIONS(1188),
    [anon_sym_continue] = ACTIONS(1188),
    [anon_sym_goto] = ACTIONS(1188),
    [anon_sym___try] = ACTIONS(1188),
    [anon_sym___leave] = ACTIONS(1188),
    [anon_sym_DASH_DASH] = ACTIONS(1190),
    [anon_sym_PLUS_PLUS] = ACTIONS(1190),
    [anon_sym_sizeof] = ACTIONS(1188),
    [anon_sym___alignof__] = ACTIONS(1188),
    [anon_sym___alignof] = ACTIONS(1188),
    [anon_sym__alignof] = ACTIONS(1188),
    [anon_sym_alignof] = ACTIONS(1188),
    [anon_sym__Alignof] = ACTIONS(1188),
    [anon_sym_offsetof] = ACTIONS(1188),
    [anon_sym__Generic] = ACTIONS(1188),
    [anon_sym_asm] = ACTIONS(1188),
    [anon_sym___asm__] = ACTIONS(1188),
    [sym_number_literal] = ACTIONS(1190),
    [anon_sym_L_SQUOTE] = ACTIONS(1190),
    [anon_sym_u_SQUOTE] = ACTIONS(1190),
    [anon_sym_U_SQUOTE] = ACTIONS(1190),
    [anon_sym_u8_SQUOTE] = ACTIONS(1190),
    [anon_sym_SQUOTE] = ACTIONS(1190),
    [anon_sym_L_DQUOTE] = ACTIONS(1190),
    [anon_sym_u_DQUOTE] = ACTIONS(1190),
    [anon_sym_U_DQUOTE] = ACTIONS(1190),
    [anon_sym_u8_DQUOTE] = ACTIONS(1190),
    [anon_sym_DQUOTE] = ACTIONS(1190),
    [sym_true] = ACTIONS(1188),
    [sym_false] = ACTIONS(1188),
    [anon_sym_NULL] = ACTIONS(1188),
    [anon_sym_nullptr] = ACTIONS(1188),
    [sym_comment] = ACTIONS(3),
  },
  [260] = {
    [sym_identifier] = ACTIONS(1184),
    [aux_sym_preproc_include_token1] = ACTIONS(1184),
    [aux_sym_preproc_def_token1] = ACTIONS(1184),
    [aux_sym_preproc_if_token1] = ACTIONS(1184),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1184),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1184),
    [sym_preproc_directive] = ACTIONS(1184),
    [anon_sym_LPAREN2] = ACTIONS(1186),
    [anon_sym_BANG] = ACTIONS(1186),
    [anon_sym_TILDE] = ACTIONS(1186),
    [anon_sym_DASH] = ACTIONS(1184),
    [anon_sym_PLUS] = ACTIONS(1184),
    [anon_sym_STAR] = ACTIONS(1186),
    [anon_sym_AMP] = ACTIONS(1186),
    [anon_sym_SEMI] = ACTIONS(1186),
    [anon_sym___extension__] = ACTIONS(1184),
    [anon_sym_typedef] = ACTIONS(1184),
    [anon_sym_extern] = ACTIONS(1184),
    [anon_sym___attribute__] = ACTIONS(1184),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1186),
    [anon_sym___declspec] = ACTIONS(1184),
    [anon_sym___cdecl] = ACTIONS(1184),
    [anon_sym___clrcall] = ACTIONS(1184),
    [anon_sym___stdcall] = ACTIONS(1184),
    [anon_sym___fastcall] = ACTIONS(1184),
    [anon_sym___thiscall] = ACTIONS(1184),
    [anon_sym___vectorcall] = ACTIONS(1184),
    [anon_sym_LBRACE] = ACTIONS(1186),
    [anon_sym_RBRACE] = ACTIONS(1186),
    [anon_sym_signed] = ACTIONS(1184),
    [anon_sym_unsigned] = ACTIONS(1184),
    [anon_sym_long] = ACTIONS(1184),
    [anon_sym_short] = ACTIONS(1184),
    [anon_sym_static] = ACTIONS(1184),
    [anon_sym_auto] = ACTIONS(1184),
    [anon_sym_register] = ACTIONS(1184),
    [anon_sym_inline] = ACTIONS(1184),
    [anon_sym___inline] = ACTIONS(1184),
    [anon_sym___inline__] = ACTIONS(1184),
    [anon_sym___forceinline] = ACTIONS(1184),
    [anon_sym_thread_local] = ACTIONS(1184),
    [anon_sym___thread] = ACTIONS(1184),
    [anon_sym_const] = ACTIONS(1184),
    [anon_sym_constexpr] = ACTIONS(1184),
    [anon_sym_volatile] = ACTIONS(1184),
    [anon_sym_restrict] = ACTIONS(1184),
    [anon_sym___restrict__] = ACTIONS(1184),
    [anon_sym__Atomic] = ACTIONS(1184),
    [anon_sym__Noreturn] = ACTIONS(1184),
    [anon_sym_noreturn] = ACTIONS(1184),
    [sym_primitive_type] = ACTIONS(1184),
    [anon_sym_enum] = ACTIONS(1184),
    [anon_sym_struct] = ACTIONS(1184),
    [anon_sym_union] = ACTIONS(1184),
    [anon_sym_if] = ACTIONS(1184),
    [anon_sym_else] = ACTIONS(1184),
    [anon_sym_switch] = ACTIONS(1184),
    [anon_sym_case] = ACTIONS(1184),
    [anon_sym_default] = ACTIONS(1184),
    [anon_sym_while] = ACTIONS(1184),
    [anon_sym_do] = ACTIONS(1184),
    [anon_sym_for] = ACTIONS(1184),
    [anon_sym_return] = ACTIONS(1184),
    [anon_sym_break] = ACTIONS(1184),
    [anon_sym_continue] = ACTIONS(1184),
    [anon_sym_goto] = ACTIONS(1184),
    [anon_sym___try] = ACTIONS(1184),
    [anon_sym___leave] = ACTIONS(1184),
    [anon_sym_DASH_DASH] = ACTIONS(1186),
    [anon_sym_PLUS_PLUS] = ACTIONS(1186),
    [anon_sym_sizeof] = ACTIONS(1184),
    [anon_sym___alignof__] = ACTIONS(1184),
    [anon_sym___alignof] = ACTIONS(1184),
    [anon_sym__alignof] = ACTIONS(1184),
    [anon_sym_alignof] = ACTIONS(1184),
    [anon_sym__Alignof] = ACTIONS(1184),
    [anon_sym_offsetof] = ACTIONS(1184),
    [anon_sym__Generic] = ACTIONS(1184),
    [anon_sym_asm] = ACTIONS(1184),
    [anon_sym___asm__] = ACTIONS(1184),
    [sym_number_literal] = ACTIONS(1186),
    [anon_sym_L_SQUOTE] = ACTIONS(1186),
    [anon_sym_u_SQUOTE] = ACTIONS(1186),
    [anon_sym_U_SQUOTE] = ACTIONS(1186),
    [anon_sym_u8_SQUOTE] = ACTIONS(1186),
    [anon_sym_SQUOTE] = ACTIONS(1186),
    [anon_sym_L_DQUOTE] = ACTIONS(1186),
    [anon_sym_u_DQUOTE] = ACTIONS(1186),
    [anon_sym_U_DQUOTE] = ACTIONS(1186),
    [anon_sym_u8_DQUOTE] = ACTIONS(1186),
    [anon_sym_DQUOTE] = ACTIONS(1186),
    [sym_true] = ACTIONS(1184),
    [sym_false] = ACTIONS(1184),
    [anon_sym_NULL] = ACTIONS(1184),
    [anon_sym_nullptr] = ACTIONS(1184),
    [sym_comment] = ACTIONS(3),
  },
  [261] = {
    [ts_builtin_sym_end] = ACTIONS(1142),
    [sym_identifier] = ACTIONS(1140),
    [aux_sym_preproc_include_token1] = ACTIONS(1140),
    [aux_sym_preproc_def_token1] = ACTIONS(1140),
    [aux_sym_preproc_if_token1] = ACTIONS(1140),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1140),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1140),
    [sym_preproc_directive] = ACTIONS(1140),
    [anon_sym_LPAREN2] = ACTIONS(1142),
    [anon_sym_BANG] = ACTIONS(1142),
    [anon_sym_TILDE] = ACTIONS(1142),
    [anon_sym_DASH] = ACTIONS(1140),
    [anon_sym_PLUS] = ACTIONS(1140),
    [anon_sym_STAR] = ACTIONS(1142),
    [anon_sym_AMP] = ACTIONS(1142),
    [anon_sym_SEMI] = ACTIONS(1142),
    [anon_sym___extension__] = ACTIONS(1140),
    [anon_sym_typedef] = ACTIONS(1140),
    [anon_sym_extern] = ACTIONS(1140),
    [anon_sym___attribute__] = ACTIONS(1140),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1142),
    [anon_sym___declspec] = ACTIONS(1140),
    [anon_sym___cdecl] = ACTIONS(1140),
    [anon_sym___clrcall] = ACTIONS(1140),
    [anon_sym___stdcall] = ACTIONS(1140),
    [anon_sym___fastcall] = ACTIONS(1140),
    [anon_sym___thiscall] = ACTIONS(1140),
    [anon_sym___vectorcall] = ACTIONS(1140),
    [anon_sym_LBRACE] = ACTIONS(1142),
    [anon_sym_signed] = ACTIONS(1140),
    [anon_sym_unsigned] = ACTIONS(1140),
    [anon_sym_long] = ACTIONS(1140),
    [anon_sym_short] = ACTIONS(1140),
    [anon_sym_static] = ACTIONS(1140),
    [anon_sym_auto] = ACTIONS(1140),
    [anon_sym_register] = ACTIONS(1140),
    [anon_sym_inline] = ACTIONS(1140),
    [anon_sym___inline] = ACTIONS(1140),
    [anon_sym___inline__] = ACTIONS(1140),
    [anon_sym___forceinline] = ACTIONS(1140),
    [anon_sym_thread_local] = ACTIONS(1140),
    [anon_sym___thread] = ACTIONS(1140),
    [anon_sym_const] = ACTIONS(1140),
    [anon_sym_constexpr] = ACTIONS(1140),
    [anon_sym_volatile] = ACTIONS(1140),
    [anon_sym_restrict] = ACTIONS(1140),
    [anon_sym___restrict__] = ACTIONS(1140),
    [anon_sym__Atomic] = ACTIONS(1140),
    [anon_sym__Noreturn] = ACTIONS(1140),
    [anon_sym_noreturn] = ACTIONS(1140),
    [sym_primitive_type] = ACTIONS(1140),
    [anon_sym_enum] = ACTIONS(1140),
    [anon_sym_struct] = ACTIONS(1140),
    [anon_sym_union] = ACTIONS(1140),
    [anon_sym_if] = ACTIONS(1140),
    [anon_sym_else] = ACTIONS(1140),
    [anon_sym_switch] = ACTIONS(1140),
    [anon_sym_case] = ACTIONS(1140),
    [anon_sym_default] = ACTIONS(1140),
    [anon_sym_while] = ACTIONS(1140),
    [anon_sym_do] = ACTIONS(1140),
    [anon_sym_for] = ACTIONS(1140),
    [anon_sym_return] = ACTIONS(1140),
    [anon_sym_break] = ACTIONS(1140),
    [anon_sym_continue] = ACTIONS(1140),
    [anon_sym_goto] = ACTIONS(1140),
    [anon_sym___try] = ACTIONS(1140),
    [anon_sym___leave] = ACTIONS(1140),
    [anon_sym_DASH_DASH] = ACTIONS(1142),
    [anon_sym_PLUS_PLUS] = ACTIONS(1142),
    [anon_sym_sizeof] = ACTIONS(1140),
    [anon_sym___alignof__] = ACTIONS(1140),
    [anon_sym___alignof] = ACTIONS(1140),
    [anon_sym__alignof] = ACTIONS(1140),
    [anon_sym_alignof] = ACTIONS(1140),
    [anon_sym__Alignof] = ACTIONS(1140),
    [anon_sym_offsetof] = ACTIONS(1140),
    [anon_sym__Generic] = ACTIONS(1140),
    [anon_sym_asm] = ACTIONS(1140),
    [anon_sym___asm__] = ACTIONS(1140),
    [sym_number_literal] = ACTIONS(1142),
    [anon_sym_L_SQUOTE] = ACTIONS(1142),
    [anon_sym_u_SQUOTE] = ACTIONS(1142),
    [anon_sym_U_SQUOTE] = ACTIONS(1142),
    [anon_sym_u8_SQUOTE] = ACTIONS(1142),
    [anon_sym_SQUOTE] = ACTIONS(1142),
    [anon_sym_L_DQUOTE] = ACTIONS(1142),
    [anon_sym_u_DQUOTE] = ACTIONS(1142),
    [anon_sym_U_DQUOTE] = ACTIONS(1142),
    [anon_sym_u8_DQUOTE] = ACTIONS(1142),
    [anon_sym_DQUOTE] = ACTIONS(1142),
    [sym_true] = ACTIONS(1140),
    [sym_false] = ACTIONS(1140),
    [anon_sym_NULL] = ACTIONS(1140),
    [anon_sym_nullptr] = ACTIONS(1140),
    [sym_comment] = ACTIONS(3),
  },
  [262] = {
    [ts_builtin_sym_end] = ACTIONS(1138),
    [sym_identifier] = ACTIONS(1136),
    [aux_sym_preproc_include_token1] = ACTIONS(1136),
    [aux_sym_preproc_def_token1] = ACTIONS(1136),
    [aux_sym_preproc_if_token1] = ACTIONS(1136),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1136),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1136),
    [sym_preproc_directive] = ACTIONS(1136),
    [anon_sym_LPAREN2] = ACTIONS(1138),
    [anon_sym_BANG] = ACTIONS(1138),
    [anon_sym_TILDE] = ACTIONS(1138),
    [anon_sym_DASH] = ACTIONS(1136),
    [anon_sym_PLUS] = ACTIONS(1136),
    [anon_sym_STAR] = ACTIONS(1138),
    [anon_sym_AMP] = ACTIONS(1138),
    [anon_sym_SEMI] = ACTIONS(1138),
    [anon_sym___extension__] = ACTIONS(1136),
    [anon_sym_typedef] = ACTIONS(1136),
    [anon_sym_extern] = ACTIONS(1136),
    [anon_sym___attribute__] = ACTIONS(1136),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1138),
    [anon_sym___declspec] = ACTIONS(1136),
    [anon_sym___cdecl] = ACTIONS(1136),
    [anon_sym___clrcall] = ACTIONS(1136),
    [anon_sym___stdcall] = ACTIONS(1136),
    [anon_sym___fastcall] = ACTIONS(1136),
    [anon_sym___thiscall] = ACTIONS(1136),
    [anon_sym___vectorcall] = ACTIONS(1136),
    [anon_sym_LBRACE] = ACTIONS(1138),
    [anon_sym_signed] = ACTIONS(1136),
    [anon_sym_unsigned] = ACTIONS(1136),
    [anon_sym_long] = ACTIONS(1136),
    [anon_sym_short] = ACTIONS(1136),
    [anon_sym_static] = ACTIONS(1136),
    [anon_sym_auto] = ACTIONS(1136),
    [anon_sym_register] = ACTIONS(1136),
    [anon_sym_inline] = ACTIONS(1136),
    [anon_sym___inline] = ACTIONS(1136),
    [anon_sym___inline__] = ACTIONS(1136),
    [anon_sym___forceinline] = ACTIONS(1136),
    [anon_sym_thread_local] = ACTIONS(1136),
    [anon_sym___thread] = ACTIONS(1136),
    [anon_sym_const] = ACTIONS(1136),
    [anon_sym_constexpr] = ACTIONS(1136),
    [anon_sym_volatile] = ACTIONS(1136),
    [anon_sym_restrict] = ACTIONS(1136),
    [anon_sym___restrict__] = ACTIONS(1136),
    [anon_sym__Atomic] = ACTIONS(1136),
    [anon_sym__Noreturn] = ACTIONS(1136),
    [anon_sym_noreturn] = ACTIONS(1136),
    [sym_primitive_type] = ACTIONS(1136),
    [anon_sym_enum] = ACTIONS(1136),
    [anon_sym_struct] = ACTIONS(1136),
    [anon_sym_union] = ACTIONS(1136),
    [anon_sym_if] = ACTIONS(1136),
    [anon_sym_else] = ACTIONS(1136),
    [anon_sym_switch] = ACTIONS(1136),
    [anon_sym_case] = ACTIONS(1136),
    [anon_sym_default] = ACTIONS(1136),
    [anon_sym_while] = ACTIONS(1136),
    [anon_sym_do] = ACTIONS(1136),
    [anon_sym_for] = ACTIONS(1136),
    [anon_sym_return] = ACTIONS(1136),
    [anon_sym_break] = ACTIONS(1136),
    [anon_sym_continue] = ACTIONS(1136),
    [anon_sym_goto] = ACTIONS(1136),
    [anon_sym___try] = ACTIONS(1136),
    [anon_sym___leave] = ACTIONS(1136),
    [anon_sym_DASH_DASH] = ACTIONS(1138),
    [anon_sym_PLUS_PLUS] = ACTIONS(1138),
    [anon_sym_sizeof] = ACTIONS(1136),
    [anon_sym___alignof__] = ACTIONS(1136),
    [anon_sym___alignof] = ACTIONS(1136),
    [anon_sym__alignof] = ACTIONS(1136),
    [anon_sym_alignof] = ACTIONS(1136),
    [anon_sym__Alignof] = ACTIONS(1136),
    [anon_sym_offsetof] = ACTIONS(1136),
    [anon_sym__Generic] = ACTIONS(1136),
    [anon_sym_asm] = ACTIONS(1136),
    [anon_sym___asm__] = ACTIONS(1136),
    [sym_number_literal] = ACTIONS(1138),
    [anon_sym_L_SQUOTE] = ACTIONS(1138),
    [anon_sym_u_SQUOTE] = ACTIONS(1138),
    [anon_sym_U_SQUOTE] = ACTIONS(1138),
    [anon_sym_u8_SQUOTE] = ACTIONS(1138),
    [anon_sym_SQUOTE] = ACTIONS(1138),
    [anon_sym_L_DQUOTE] = ACTIONS(1138),
    [anon_sym_u_DQUOTE] = ACTIONS(1138),
    [anon_sym_U_DQUOTE] = ACTIONS(1138),
    [anon_sym_u8_DQUOTE] = ACTIONS(1138),
    [anon_sym_DQUOTE] = ACTIONS(1138),
    [sym_true] = ACTIONS(1136),
    [sym_false] = ACTIONS(1136),
    [anon_sym_NULL] = ACTIONS(1136),
    [anon_sym_nullptr] = ACTIONS(1136),
    [sym_comment] = ACTIONS(3),
  },
  [263] = {
    [sym_identifier] = ACTIONS(1180),
    [aux_sym_preproc_include_token1] = ACTIONS(1180),
    [aux_sym_preproc_def_token1] = ACTIONS(1180),
    [aux_sym_preproc_if_token1] = ACTIONS(1180),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1180),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1180),
    [sym_preproc_directive] = ACTIONS(1180),
    [anon_sym_LPAREN2] = ACTIONS(1182),
    [anon_sym_BANG] = ACTIONS(1182),
    [anon_sym_TILDE] = ACTIONS(1182),
    [anon_sym_DASH] = ACTIONS(1180),
    [anon_sym_PLUS] = ACTIONS(1180),
    [anon_sym_STAR] = ACTIONS(1182),
    [anon_sym_AMP] = ACTIONS(1182),
    [anon_sym_SEMI] = ACTIONS(1182),
    [anon_sym___extension__] = ACTIONS(1180),
    [anon_sym_typedef] = ACTIONS(1180),
    [anon_sym_extern] = ACTIONS(1180),
    [anon_sym___attribute__] = ACTIONS(1180),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1182),
    [anon_sym___declspec] = ACTIONS(1180),
    [anon_sym___cdecl] = ACTIONS(1180),
    [anon_sym___clrcall] = ACTIONS(1180),
    [anon_sym___stdcall] = ACTIONS(1180),
    [anon_sym___fastcall] = ACTIONS(1180),
    [anon_sym___thiscall] = ACTIONS(1180),
    [anon_sym___vectorcall] = ACTIONS(1180),
    [anon_sym_LBRACE] = ACTIONS(1182),
    [anon_sym_RBRACE] = ACTIONS(1182),
    [anon_sym_signed] = ACTIONS(1180),
    [anon_sym_unsigned] = ACTIONS(1180),
    [anon_sym_long] = ACTIONS(1180),
    [anon_sym_short] = ACTIONS(1180),
    [anon_sym_static] = ACTIONS(1180),
    [anon_sym_auto] = ACTIONS(1180),
    [anon_sym_register] = ACTIONS(1180),
    [anon_sym_inline] = ACTIONS(1180),
    [anon_sym___inline] = ACTIONS(1180),
    [anon_sym___inline__] = ACTIONS(1180),
    [anon_sym___forceinline] = ACTIONS(1180),
    [anon_sym_thread_local] = ACTIONS(1180),
    [anon_sym___thread] = ACTIONS(1180),
    [anon_sym_const] = ACTIONS(1180),
    [anon_sym_constexpr] = ACTIONS(1180),
    [anon_sym_volatile] = ACTIONS(1180),
    [anon_sym_restrict] = ACTIONS(1180),
    [anon_sym___restrict__] = ACTIONS(1180),
    [anon_sym__Atomic] = ACTIONS(1180),
    [anon_sym__Noreturn] = ACTIONS(1180),
    [anon_sym_noreturn] = ACTIONS(1180),
    [sym_primitive_type] = ACTIONS(1180),
    [anon_sym_enum] = ACTIONS(1180),
    [anon_sym_struct] = ACTIONS(1180),
    [anon_sym_union] = ACTIONS(1180),
    [anon_sym_if] = ACTIONS(1180),
    [anon_sym_else] = ACTIONS(1180),
    [anon_sym_switch] = ACTIONS(1180),
    [anon_sym_case] = ACTIONS(1180),
    [anon_sym_default] = ACTIONS(1180),
    [anon_sym_while] = ACTIONS(1180),
    [anon_sym_do] = ACTIONS(1180),
    [anon_sym_for] = ACTIONS(1180),
    [anon_sym_return] = ACTIONS(1180),
    [anon_sym_break] = ACTIONS(1180),
    [anon_sym_continue] = ACTIONS(1180),
    [anon_sym_goto] = ACTIONS(1180),
    [anon_sym___try] = ACTIONS(1180),
    [anon_sym___leave] = ACTIONS(1180),
    [anon_sym_DASH_DASH] = ACTIONS(1182),
    [anon_sym_PLUS_PLUS] = ACTIONS(1182),
    [anon_sym_sizeof] = ACTIONS(1180),
    [anon_sym___alignof__] = ACTIONS(1180),
    [anon_sym___alignof] = ACTIONS(1180),
    [anon_sym__alignof] = ACTIONS(1180),
    [anon_sym_alignof] = ACTIONS(1180),
    [anon_sym__Alignof] = ACTIONS(1180),
    [anon_sym_offsetof] = ACTIONS(1180),
    [anon_sym__Generic] = ACTIONS(1180),
    [anon_sym_asm] = ACTIONS(1180),
    [anon_sym___asm__] = ACTIONS(1180),
    [sym_number_literal] = ACTIONS(1182),
    [anon_sym_L_SQUOTE] = ACTIONS(1182),
    [anon_sym_u_SQUOTE] = ACTIONS(1182),
    [anon_sym_U_SQUOTE] = ACTIONS(1182),
    [anon_sym_u8_SQUOTE] = ACTIONS(1182),
    [anon_sym_SQUOTE] = ACTIONS(1182),
    [anon_sym_L_DQUOTE] = ACTIONS(1182),
    [anon_sym_u_DQUOTE] = ACTIONS(1182),
    [anon_sym_U_DQUOTE] = ACTIONS(1182),
    [anon_sym_u8_DQUOTE] = ACTIONS(1182),
    [anon_sym_DQUOTE] = ACTIONS(1182),
    [sym_true] = ACTIONS(1180),
    [sym_false] = ACTIONS(1180),
    [anon_sym_NULL] = ACTIONS(1180),
    [anon_sym_nullptr] = ACTIONS(1180),
    [sym_comment] = ACTIONS(3),
  },
  [264] = {
    [sym_identifier] = ACTIONS(1120),
    [aux_sym_preproc_include_token1] = ACTIONS(1120),
    [aux_sym_preproc_def_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1120),
    [sym_preproc_directive] = ACTIONS(1120),
    [anon_sym_LPAREN2] = ACTIONS(1122),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_DASH] = ACTIONS(1120),
    [anon_sym_PLUS] = ACTIONS(1120),
    [anon_sym_STAR] = ACTIONS(1122),
    [anon_sym_AMP] = ACTIONS(1122),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym___extension__] = ACTIONS(1120),
    [anon_sym_typedef] = ACTIONS(1120),
    [anon_sym_extern] = ACTIONS(1120),
    [anon_sym___attribute__] = ACTIONS(1120),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1122),
    [anon_sym___declspec] = ACTIONS(1120),
    [anon_sym___cdecl] = ACTIONS(1120),
    [anon_sym___clrcall] = ACTIONS(1120),
    [anon_sym___stdcall] = ACTIONS(1120),
    [anon_sym___fastcall] = ACTIONS(1120),
    [anon_sym___thiscall] = ACTIONS(1120),
    [anon_sym___vectorcall] = ACTIONS(1120),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_RBRACE] = ACTIONS(1122),
    [anon_sym_signed] = ACTIONS(1120),
    [anon_sym_unsigned] = ACTIONS(1120),
    [anon_sym_long] = ACTIONS(1120),
    [anon_sym_short] = ACTIONS(1120),
    [anon_sym_static] = ACTIONS(1120),
    [anon_sym_auto] = ACTIONS(1120),
    [anon_sym_register] = ACTIONS(1120),
    [anon_sym_inline] = ACTIONS(1120),
    [anon_sym___inline] = ACTIONS(1120),
    [anon_sym___inline__] = ACTIONS(1120),
    [anon_sym___forceinline] = ACTIONS(1120),
    [anon_sym_thread_local] = ACTIONS(1120),
    [anon_sym___thread] = ACTIONS(1120),
    [anon_sym_const] = ACTIONS(1120),
    [anon_sym_constexpr] = ACTIONS(1120),
    [anon_sym_volatile] = ACTIONS(1120),
    [anon_sym_restrict] = ACTIONS(1120),
    [anon_sym___restrict__] = ACTIONS(1120),
    [anon_sym__Atomic] = ACTIONS(1120),
    [anon_sym__Noreturn] = ACTIONS(1120),
    [anon_sym_noreturn] = ACTIONS(1120),
    [sym_primitive_type] = ACTIONS(1120),
    [anon_sym_enum] = ACTIONS(1120),
    [anon_sym_struct] = ACTIONS(1120),
    [anon_sym_union] = ACTIONS(1120),
    [anon_sym_if] = ACTIONS(1120),
    [anon_sym_else] = ACTIONS(1120),
    [anon_sym_switch] = ACTIONS(1120),
    [anon_sym_case] = ACTIONS(1120),
    [anon_sym_default] = ACTIONS(1120),
    [anon_sym_while] = ACTIONS(1120),
    [anon_sym_do] = ACTIONS(1120),
    [anon_sym_for] = ACTIONS(1120),
    [anon_sym_return] = ACTIONS(1120),
    [anon_sym_break] = ACTIONS(1120),
    [anon_sym_continue] = ACTIONS(1120),
    [anon_sym_goto] = ACTIONS(1120),
    [anon_sym___try] = ACTIONS(1120),
    [anon_sym___leave] = ACTIONS(1120),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_sizeof] = ACTIONS(1120),
    [anon_sym___alignof__] = ACTIONS(1120),
    [anon_sym___alignof] = ACTIONS(1120),
    [anon_sym__alignof] = ACTIONS(1120),
    [anon_sym_alignof] = ACTIONS(1120),
    [anon_sym__Alignof] = ACTIONS(1120),
    [anon_sym_offsetof] = ACTIONS(1120),
    [anon_sym__Generic] = ACTIONS(1120),
    [anon_sym_asm] = ACTIONS(1120),
    [anon_sym___asm__] = ACTIONS(1120),
    [sym_number_literal] = ACTIONS(1122),
    [anon_sym_L_SQUOTE] = ACTIONS(1122),
    [anon_sym_u_SQUOTE] = ACTIONS(1122),
    [anon_sym_U_SQUOTE] = ACTIONS(1122),
    [anon_sym_u8_SQUOTE] = ACTIONS(1122),
    [anon_sym_SQUOTE] = ACTIONS(1122),
    [anon_sym_L_DQUOTE] = ACTIONS(1122),
    [anon_sym_u_DQUOTE] = ACTIONS(1122),
    [anon_sym_U_DQUOTE] = ACTIONS(1122),
    [anon_sym_u8_DQUOTE] = ACTIONS(1122),
    [anon_sym_DQUOTE] = ACTIONS(1122),
    [sym_true] = ACTIONS(1120),
    [sym_false] = ACTIONS(1120),
    [anon_sym_NULL] = ACTIONS(1120),
    [anon_sym_nullptr] = ACTIONS(1120),
    [sym_comment] = ACTIONS(3),
  },
  [265] = {
    [ts_builtin_sym_end] = ACTIONS(1134),
    [sym_identifier] = ACTIONS(1132),
    [aux_sym_preproc_include_token1] = ACTIONS(1132),
    [aux_sym_preproc_def_token1] = ACTIONS(1132),
    [aux_sym_preproc_if_token1] = ACTIONS(1132),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1132),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1132),
    [sym_preproc_directive] = ACTIONS(1132),
    [anon_sym_LPAREN2] = ACTIONS(1134),
    [anon_sym_BANG] = ACTIONS(1134),
    [anon_sym_TILDE] = ACTIONS(1134),
    [anon_sym_DASH] = ACTIONS(1132),
    [anon_sym_PLUS] = ACTIONS(1132),
    [anon_sym_STAR] = ACTIONS(1134),
    [anon_sym_AMP] = ACTIONS(1134),
    [anon_sym_SEMI] = ACTIONS(1134),
    [anon_sym___extension__] = ACTIONS(1132),
    [anon_sym_typedef] = ACTIONS(1132),
    [anon_sym_extern] = ACTIONS(1132),
    [anon_sym___attribute__] = ACTIONS(1132),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1134),
    [anon_sym___declspec] = ACTIONS(1132),
    [anon_sym___cdecl] = ACTIONS(1132),
    [anon_sym___clrcall] = ACTIONS(1132),
    [anon_sym___stdcall] = ACTIONS(1132),
    [anon_sym___fastcall] = ACTIONS(1132),
    [anon_sym___thiscall] = ACTIONS(1132),
    [anon_sym___vectorcall] = ACTIONS(1132),
    [anon_sym_LBRACE] = ACTIONS(1134),
    [anon_sym_signed] = ACTIONS(1132),
    [anon_sym_unsigned] = ACTIONS(1132),
    [anon_sym_long] = ACTIONS(1132),
    [anon_sym_short] = ACTIONS(1132),
    [anon_sym_static] = ACTIONS(1132),
    [anon_sym_auto] = ACTIONS(1132),
    [anon_sym_register] = ACTIONS(1132),
    [anon_sym_inline] = ACTIONS(1132),
    [anon_sym___inline] = ACTIONS(1132),
    [anon_sym___inline__] = ACTIONS(1132),
    [anon_sym___forceinline] = ACTIONS(1132),
    [anon_sym_thread_local] = ACTIONS(1132),
    [anon_sym___thread] = ACTIONS(1132),
    [anon_sym_const] = ACTIONS(1132),
    [anon_sym_constexpr] = ACTIONS(1132),
    [anon_sym_volatile] = ACTIONS(1132),
    [anon_sym_restrict] = ACTIONS(1132),
    [anon_sym___restrict__] = ACTIONS(1132),
    [anon_sym__Atomic] = ACTIONS(1132),
    [anon_sym__Noreturn] = ACTIONS(1132),
    [anon_sym_noreturn] = ACTIONS(1132),
    [sym_primitive_type] = ACTIONS(1132),
    [anon_sym_enum] = ACTIONS(1132),
    [anon_sym_struct] = ACTIONS(1132),
    [anon_sym_union] = ACTIONS(1132),
    [anon_sym_if] = ACTIONS(1132),
    [anon_sym_else] = ACTIONS(1132),
    [anon_sym_switch] = ACTIONS(1132),
    [anon_sym_case] = ACTIONS(1132),
    [anon_sym_default] = ACTIONS(1132),
    [anon_sym_while] = ACTIONS(1132),
    [anon_sym_do] = ACTIONS(1132),
    [anon_sym_for] = ACTIONS(1132),
    [anon_sym_return] = ACTIONS(1132),
    [anon_sym_break] = ACTIONS(1132),
    [anon_sym_continue] = ACTIONS(1132),
    [anon_sym_goto] = ACTIONS(1132),
    [anon_sym___try] = ACTIONS(1132),
    [anon_sym___leave] = ACTIONS(1132),
    [anon_sym_DASH_DASH] = ACTIONS(1134),
    [anon_sym_PLUS_PLUS] = ACTIONS(1134),
    [anon_sym_sizeof] = ACTIONS(1132),
    [anon_sym___alignof__] = ACTIONS(1132),
    [anon_sym___alignof] = ACTIONS(1132),
    [anon_sym__alignof] = ACTIONS(1132),
    [anon_sym_alignof] = ACTIONS(1132),
    [anon_sym__Alignof] = ACTIONS(1132),
    [anon_sym_offsetof] = ACTIONS(1132),
    [anon_sym__Generic] = ACTIONS(1132),
    [anon_sym_asm] = ACTIONS(1132),
    [anon_sym___asm__] = ACTIONS(1132),
    [sym_number_literal] = ACTIONS(1134),
    [anon_sym_L_SQUOTE] = ACTIONS(1134),
    [anon_sym_u_SQUOTE] = ACTIONS(1134),
    [anon_sym_U_SQUOTE] = ACTIONS(1134),
    [anon_sym_u8_SQUOTE] = ACTIONS(1134),
    [anon_sym_SQUOTE] = ACTIONS(1134),
    [anon_sym_L_DQUOTE] = ACTIONS(1134),
    [anon_sym_u_DQUOTE] = ACTIONS(1134),
    [anon_sym_U_DQUOTE] = ACTIONS(1134),
    [anon_sym_u8_DQUOTE] = ACTIONS(1134),
    [anon_sym_DQUOTE] = ACTIONS(1134),
    [sym_true] = ACTIONS(1132),
    [sym_false] = ACTIONS(1132),
    [anon_sym_NULL] = ACTIONS(1132),
    [anon_sym_nullptr] = ACTIONS(1132),
    [sym_comment] = ACTIONS(3),
  },
  [266] = {
    [sym_identifier] = ACTIONS(1120),
    [aux_sym_preproc_include_token1] = ACTIONS(1120),
    [aux_sym_preproc_def_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1120),
    [sym_preproc_directive] = ACTIONS(1120),
    [anon_sym_LPAREN2] = ACTIONS(1122),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_DASH] = ACTIONS(1120),
    [anon_sym_PLUS] = ACTIONS(1120),
    [anon_sym_STAR] = ACTIONS(1122),
    [anon_sym_AMP] = ACTIONS(1122),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym___extension__] = ACTIONS(1120),
    [anon_sym_typedef] = ACTIONS(1120),
    [anon_sym_extern] = ACTIONS(1120),
    [anon_sym___attribute__] = ACTIONS(1120),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1122),
    [anon_sym___declspec] = ACTIONS(1120),
    [anon_sym___cdecl] = ACTIONS(1120),
    [anon_sym___clrcall] = ACTIONS(1120),
    [anon_sym___stdcall] = ACTIONS(1120),
    [anon_sym___fastcall] = ACTIONS(1120),
    [anon_sym___thiscall] = ACTIONS(1120),
    [anon_sym___vectorcall] = ACTIONS(1120),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_RBRACE] = ACTIONS(1122),
    [anon_sym_signed] = ACTIONS(1120),
    [anon_sym_unsigned] = ACTIONS(1120),
    [anon_sym_long] = ACTIONS(1120),
    [anon_sym_short] = ACTIONS(1120),
    [anon_sym_static] = ACTIONS(1120),
    [anon_sym_auto] = ACTIONS(1120),
    [anon_sym_register] = ACTIONS(1120),
    [anon_sym_inline] = ACTIONS(1120),
    [anon_sym___inline] = ACTIONS(1120),
    [anon_sym___inline__] = ACTIONS(1120),
    [anon_sym___forceinline] = ACTIONS(1120),
    [anon_sym_thread_local] = ACTIONS(1120),
    [anon_sym___thread] = ACTIONS(1120),
    [anon_sym_const] = ACTIONS(1120),
    [anon_sym_constexpr] = ACTIONS(1120),
    [anon_sym_volatile] = ACTIONS(1120),
    [anon_sym_restrict] = ACTIONS(1120),
    [anon_sym___restrict__] = ACTIONS(1120),
    [anon_sym__Atomic] = ACTIONS(1120),
    [anon_sym__Noreturn] = ACTIONS(1120),
    [anon_sym_noreturn] = ACTIONS(1120),
    [sym_primitive_type] = ACTIONS(1120),
    [anon_sym_enum] = ACTIONS(1120),
    [anon_sym_struct] = ACTIONS(1120),
    [anon_sym_union] = ACTIONS(1120),
    [anon_sym_if] = ACTIONS(1120),
    [anon_sym_else] = ACTIONS(1120),
    [anon_sym_switch] = ACTIONS(1120),
    [anon_sym_case] = ACTIONS(1120),
    [anon_sym_default] = ACTIONS(1120),
    [anon_sym_while] = ACTIONS(1120),
    [anon_sym_do] = ACTIONS(1120),
    [anon_sym_for] = ACTIONS(1120),
    [anon_sym_return] = ACTIONS(1120),
    [anon_sym_break] = ACTIONS(1120),
    [anon_sym_continue] = ACTIONS(1120),
    [anon_sym_goto] = ACTIONS(1120),
    [anon_sym___try] = ACTIONS(1120),
    [anon_sym___leave] = ACTIONS(1120),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_sizeof] = ACTIONS(1120),
    [anon_sym___alignof__] = ACTIONS(1120),
    [anon_sym___alignof] = ACTIONS(1120),
    [anon_sym__alignof] = ACTIONS(1120),
    [anon_sym_alignof] = ACTIONS(1120),
    [anon_sym__Alignof] = ACTIONS(1120),
    [anon_sym_offsetof] = ACTIONS(1120),
    [anon_sym__Generic] = ACTIONS(1120),
    [anon_sym_asm] = ACTIONS(1120),
    [anon_sym___asm__] = ACTIONS(1120),
    [sym_number_literal] = ACTIONS(1122),
    [anon_sym_L_SQUOTE] = ACTIONS(1122),
    [anon_sym_u_SQUOTE] = ACTIONS(1122),
    [anon_sym_U_SQUOTE] = ACTIONS(1122),
    [anon_sym_u8_SQUOTE] = ACTIONS(1122),
    [anon_sym_SQUOTE] = ACTIONS(1122),
    [anon_sym_L_DQUOTE] = ACTIONS(1122),
    [anon_sym_u_DQUOTE] = ACTIONS(1122),
    [anon_sym_U_DQUOTE] = ACTIONS(1122),
    [anon_sym_u8_DQUOTE] = ACTIONS(1122),
    [anon_sym_DQUOTE] = ACTIONS(1122),
    [sym_true] = ACTIONS(1120),
    [sym_false] = ACTIONS(1120),
    [anon_sym_NULL] = ACTIONS(1120),
    [anon_sym_nullptr] = ACTIONS(1120),
    [sym_comment] = ACTIONS(3),
  },
  [267] = {
    [sym_identifier] = ACTIONS(1144),
    [aux_sym_preproc_include_token1] = ACTIONS(1144),
    [aux_sym_preproc_def_token1] = ACTIONS(1144),
    [aux_sym_preproc_if_token1] = ACTIONS(1144),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1144),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1144),
    [sym_preproc_directive] = ACTIONS(1144),
    [anon_sym_LPAREN2] = ACTIONS(1146),
    [anon_sym_BANG] = ACTIONS(1146),
    [anon_sym_TILDE] = ACTIONS(1146),
    [anon_sym_DASH] = ACTIONS(1144),
    [anon_sym_PLUS] = ACTIONS(1144),
    [anon_sym_STAR] = ACTIONS(1146),
    [anon_sym_AMP] = ACTIONS(1146),
    [anon_sym_SEMI] = ACTIONS(1146),
    [anon_sym___extension__] = ACTIONS(1144),
    [anon_sym_typedef] = ACTIONS(1144),
    [anon_sym_extern] = ACTIONS(1144),
    [anon_sym___attribute__] = ACTIONS(1144),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1146),
    [anon_sym___declspec] = ACTIONS(1144),
    [anon_sym___cdecl] = ACTIONS(1144),
    [anon_sym___clrcall] = ACTIONS(1144),
    [anon_sym___stdcall] = ACTIONS(1144),
    [anon_sym___fastcall] = ACTIONS(1144),
    [anon_sym___thiscall] = ACTIONS(1144),
    [anon_sym___vectorcall] = ACTIONS(1144),
    [anon_sym_LBRACE] = ACTIONS(1146),
    [anon_sym_RBRACE] = ACTIONS(1146),
    [anon_sym_signed] = ACTIONS(1144),
    [anon_sym_unsigned] = ACTIONS(1144),
    [anon_sym_long] = ACTIONS(1144),
    [anon_sym_short] = ACTIONS(1144),
    [anon_sym_static] = ACTIONS(1144),
    [anon_sym_auto] = ACTIONS(1144),
    [anon_sym_register] = ACTIONS(1144),
    [anon_sym_inline] = ACTIONS(1144),
    [anon_sym___inline] = ACTIONS(1144),
    [anon_sym___inline__] = ACTIONS(1144),
    [anon_sym___forceinline] = ACTIONS(1144),
    [anon_sym_thread_local] = ACTIONS(1144),
    [anon_sym___thread] = ACTIONS(1144),
    [anon_sym_const] = ACTIONS(1144),
    [anon_sym_constexpr] = ACTIONS(1144),
    [anon_sym_volatile] = ACTIONS(1144),
    [anon_sym_restrict] = ACTIONS(1144),
    [anon_sym___restrict__] = ACTIONS(1144),
    [anon_sym__Atomic] = ACTIONS(1144),
    [anon_sym__Noreturn] = ACTIONS(1144),
    [anon_sym_noreturn] = ACTIONS(1144),
    [sym_primitive_type] = ACTIONS(1144),
    [anon_sym_enum] = ACTIONS(1144),
    [anon_sym_struct] = ACTIONS(1144),
    [anon_sym_union] = ACTIONS(1144),
    [anon_sym_if] = ACTIONS(1144),
    [anon_sym_else] = ACTIONS(1144),
    [anon_sym_switch] = ACTIONS(1144),
    [anon_sym_case] = ACTIONS(1144),
    [anon_sym_default] = ACTIONS(1144),
    [anon_sym_while] = ACTIONS(1144),
    [anon_sym_do] = ACTIONS(1144),
    [anon_sym_for] = ACTIONS(1144),
    [anon_sym_return] = ACTIONS(1144),
    [anon_sym_break] = ACTIONS(1144),
    [anon_sym_continue] = ACTIONS(1144),
    [anon_sym_goto] = ACTIONS(1144),
    [anon_sym___try] = ACTIONS(1144),
    [anon_sym___leave] = ACTIONS(1144),
    [anon_sym_DASH_DASH] = ACTIONS(1146),
    [anon_sym_PLUS_PLUS] = ACTIONS(1146),
    [anon_sym_sizeof] = ACTIONS(1144),
    [anon_sym___alignof__] = ACTIONS(1144),
    [anon_sym___alignof] = ACTIONS(1144),
    [anon_sym__alignof] = ACTIONS(1144),
    [anon_sym_alignof] = ACTIONS(1144),
    [anon_sym__Alignof] = ACTIONS(1144),
    [anon_sym_offsetof] = ACTIONS(1144),
    [anon_sym__Generic] = ACTIONS(1144),
    [anon_sym_asm] = ACTIONS(1144),
    [anon_sym___asm__] = ACTIONS(1144),
    [sym_number_literal] = ACTIONS(1146),
    [anon_sym_L_SQUOTE] = ACTIONS(1146),
    [anon_sym_u_SQUOTE] = ACTIONS(1146),
    [anon_sym_U_SQUOTE] = ACTIONS(1146),
    [anon_sym_u8_SQUOTE] = ACTIONS(1146),
    [anon_sym_SQUOTE] = ACTIONS(1146),
    [anon_sym_L_DQUOTE] = ACTIONS(1146),
    [anon_sym_u_DQUOTE] = ACTIONS(1146),
    [anon_sym_U_DQUOTE] = ACTIONS(1146),
    [anon_sym_u8_DQUOTE] = ACTIONS(1146),
    [anon_sym_DQUOTE] = ACTIONS(1146),
    [sym_true] = ACTIONS(1144),
    [sym_false] = ACTIONS(1144),
    [anon_sym_NULL] = ACTIONS(1144),
    [anon_sym_nullptr] = ACTIONS(1144),
    [sym_comment] = ACTIONS(3),
  },
  [268] = {
    [ts_builtin_sym_end] = ACTIONS(1154),
    [sym_identifier] = ACTIONS(1152),
    [aux_sym_preproc_include_token1] = ACTIONS(1152),
    [aux_sym_preproc_def_token1] = ACTIONS(1152),
    [aux_sym_preproc_if_token1] = ACTIONS(1152),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1152),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1152),
    [sym_preproc_directive] = ACTIONS(1152),
    [anon_sym_LPAREN2] = ACTIONS(1154),
    [anon_sym_BANG] = ACTIONS(1154),
    [anon_sym_TILDE] = ACTIONS(1154),
    [anon_sym_DASH] = ACTIONS(1152),
    [anon_sym_PLUS] = ACTIONS(1152),
    [anon_sym_STAR] = ACTIONS(1154),
    [anon_sym_AMP] = ACTIONS(1154),
    [anon_sym_SEMI] = ACTIONS(1154),
    [anon_sym___extension__] = ACTIONS(1152),
    [anon_sym_typedef] = ACTIONS(1152),
    [anon_sym_extern] = ACTIONS(1152),
    [anon_sym___attribute__] = ACTIONS(1152),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1154),
    [anon_sym___declspec] = ACTIONS(1152),
    [anon_sym___cdecl] = ACTIONS(1152),
    [anon_sym___clrcall] = ACTIONS(1152),
    [anon_sym___stdcall] = ACTIONS(1152),
    [anon_sym___fastcall] = ACTIONS(1152),
    [anon_sym___thiscall] = ACTIONS(1152),
    [anon_sym___vectorcall] = ACTIONS(1152),
    [anon_sym_LBRACE] = ACTIONS(1154),
    [anon_sym_signed] = ACTIONS(1152),
    [anon_sym_unsigned] = ACTIONS(1152),
    [anon_sym_long] = ACTIONS(1152),
    [anon_sym_short] = ACTIONS(1152),
    [anon_sym_static] = ACTIONS(1152),
    [anon_sym_auto] = ACTIONS(1152),
    [anon_sym_register] = ACTIONS(1152),
    [anon_sym_inline] = ACTIONS(1152),
    [anon_sym___inline] = ACTIONS(1152),
    [anon_sym___inline__] = ACTIONS(1152),
    [anon_sym___forceinline] = ACTIONS(1152),
    [anon_sym_thread_local] = ACTIONS(1152),
    [anon_sym___thread] = ACTIONS(1152),
    [anon_sym_const] = ACTIONS(1152),
    [anon_sym_constexpr] = ACTIONS(1152),
    [anon_sym_volatile] = ACTIONS(1152),
    [anon_sym_restrict] = ACTIONS(1152),
    [anon_sym___restrict__] = ACTIONS(1152),
    [anon_sym__Atomic] = ACTIONS(1152),
    [anon_sym__Noreturn] = ACTIONS(1152),
    [anon_sym_noreturn] = ACTIONS(1152),
    [sym_primitive_type] = ACTIONS(1152),
    [anon_sym_enum] = ACTIONS(1152),
    [anon_sym_struct] = ACTIONS(1152),
    [anon_sym_union] = ACTIONS(1152),
    [anon_sym_if] = ACTIONS(1152),
    [anon_sym_else] = ACTIONS(1152),
    [anon_sym_switch] = ACTIONS(1152),
    [anon_sym_case] = ACTIONS(1152),
    [anon_sym_default] = ACTIONS(1152),
    [anon_sym_while] = ACTIONS(1152),
    [anon_sym_do] = ACTIONS(1152),
    [anon_sym_for] = ACTIONS(1152),
    [anon_sym_return] = ACTIONS(1152),
    [anon_sym_break] = ACTIONS(1152),
    [anon_sym_continue] = ACTIONS(1152),
    [anon_sym_goto] = ACTIONS(1152),
    [anon_sym___try] = ACTIONS(1152),
    [anon_sym___leave] = ACTIONS(1152),
    [anon_sym_DASH_DASH] = ACTIONS(1154),
    [anon_sym_PLUS_PLUS] = ACTIONS(1154),
    [anon_sym_sizeof] = ACTIONS(1152),
    [anon_sym___alignof__] = ACTIONS(1152),
    [anon_sym___alignof] = ACTIONS(1152),
    [anon_sym__alignof] = ACTIONS(1152),
    [anon_sym_alignof] = ACTIONS(1152),
    [anon_sym__Alignof] = ACTIONS(1152),
    [anon_sym_offsetof] = ACTIONS(1152),
    [anon_sym__Generic] = ACTIONS(1152),
    [anon_sym_asm] = ACTIONS(1152),
    [anon_sym___asm__] = ACTIONS(1152),
    [sym_number_literal] = ACTIONS(1154),
    [anon_sym_L_SQUOTE] = ACTIONS(1154),
    [anon_sym_u_SQUOTE] = ACTIONS(1154),
    [anon_sym_U_SQUOTE] = ACTIONS(1154),
    [anon_sym_u8_SQUOTE] = ACTIONS(1154),
    [anon_sym_SQUOTE] = ACTIONS(1154),
    [anon_sym_L_DQUOTE] = ACTIONS(1154),
    [anon_sym_u_DQUOTE] = ACTIONS(1154),
    [anon_sym_U_DQUOTE] = ACTIONS(1154),
    [anon_sym_u8_DQUOTE] = ACTIONS(1154),
    [anon_sym_DQUOTE] = ACTIONS(1154),
    [sym_true] = ACTIONS(1152),
    [sym_false] = ACTIONS(1152),
    [anon_sym_NULL] = ACTIONS(1152),
    [anon_sym_nullptr] = ACTIONS(1152),
    [sym_comment] = ACTIONS(3),
  },
  [269] = {
    [sym_identifier] = ACTIONS(1156),
    [aux_sym_preproc_include_token1] = ACTIONS(1156),
    [aux_sym_preproc_def_token1] = ACTIONS(1156),
    [aux_sym_preproc_if_token1] = ACTIONS(1156),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1156),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1156),
    [sym_preproc_directive] = ACTIONS(1156),
    [anon_sym_LPAREN2] = ACTIONS(1158),
    [anon_sym_BANG] = ACTIONS(1158),
    [anon_sym_TILDE] = ACTIONS(1158),
    [anon_sym_DASH] = ACTIONS(1156),
    [anon_sym_PLUS] = ACTIONS(1156),
    [anon_sym_STAR] = ACTIONS(1158),
    [anon_sym_AMP] = ACTIONS(1158),
    [anon_sym_SEMI] = ACTIONS(1158),
    [anon_sym___extension__] = ACTIONS(1156),
    [anon_sym_typedef] = ACTIONS(1156),
    [anon_sym_extern] = ACTIONS(1156),
    [anon_sym___attribute__] = ACTIONS(1156),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1158),
    [anon_sym___declspec] = ACTIONS(1156),
    [anon_sym___cdecl] = ACTIONS(1156),
    [anon_sym___clrcall] = ACTIONS(1156),
    [anon_sym___stdcall] = ACTIONS(1156),
    [anon_sym___fastcall] = ACTIONS(1156),
    [anon_sym___thiscall] = ACTIONS(1156),
    [anon_sym___vectorcall] = ACTIONS(1156),
    [anon_sym_LBRACE] = ACTIONS(1158),
    [anon_sym_RBRACE] = ACTIONS(1158),
    [anon_sym_signed] = ACTIONS(1156),
    [anon_sym_unsigned] = ACTIONS(1156),
    [anon_sym_long] = ACTIONS(1156),
    [anon_sym_short] = ACTIONS(1156),
    [anon_sym_static] = ACTIONS(1156),
    [anon_sym_auto] = ACTIONS(1156),
    [anon_sym_register] = ACTIONS(1156),
    [anon_sym_inline] = ACTIONS(1156),
    [anon_sym___inline] = ACTIONS(1156),
    [anon_sym___inline__] = ACTIONS(1156),
    [anon_sym___forceinline] = ACTIONS(1156),
    [anon_sym_thread_local] = ACTIONS(1156),
    [anon_sym___thread] = ACTIONS(1156),
    [anon_sym_const] = ACTIONS(1156),
    [anon_sym_constexpr] = ACTIONS(1156),
    [anon_sym_volatile] = ACTIONS(1156),
    [anon_sym_restrict] = ACTIONS(1156),
    [anon_sym___restrict__] = ACTIONS(1156),
    [anon_sym__Atomic] = ACTIONS(1156),
    [anon_sym__Noreturn] = ACTIONS(1156),
    [anon_sym_noreturn] = ACTIONS(1156),
    [sym_primitive_type] = ACTIONS(1156),
    [anon_sym_enum] = ACTIONS(1156),
    [anon_sym_struct] = ACTIONS(1156),
    [anon_sym_union] = ACTIONS(1156),
    [anon_sym_if] = ACTIONS(1156),
    [anon_sym_else] = ACTIONS(1156),
    [anon_sym_switch] = ACTIONS(1156),
    [anon_sym_case] = ACTIONS(1156),
    [anon_sym_default] = ACTIONS(1156),
    [anon_sym_while] = ACTIONS(1156),
    [anon_sym_do] = ACTIONS(1156),
    [anon_sym_for] = ACTIONS(1156),
    [anon_sym_return] = ACTIONS(1156),
    [anon_sym_break] = ACTIONS(1156),
    [anon_sym_continue] = ACTIONS(1156),
    [anon_sym_goto] = ACTIONS(1156),
    [anon_sym___try] = ACTIONS(1156),
    [anon_sym___leave] = ACTIONS(1156),
    [anon_sym_DASH_DASH] = ACTIONS(1158),
    [anon_sym_PLUS_PLUS] = ACTIONS(1158),
    [anon_sym_sizeof] = ACTIONS(1156),
    [anon_sym___alignof__] = ACTIONS(1156),
    [anon_sym___alignof] = ACTIONS(1156),
    [anon_sym__alignof] = ACTIONS(1156),
    [anon_sym_alignof] = ACTIONS(1156),
    [anon_sym__Alignof] = ACTIONS(1156),
    [anon_sym_offsetof] = ACTIONS(1156),
    [anon_sym__Generic] = ACTIONS(1156),
    [anon_sym_asm] = ACTIONS(1156),
    [anon_sym___asm__] = ACTIONS(1156),
    [sym_number_literal] = ACTIONS(1158),
    [anon_sym_L_SQUOTE] = ACTIONS(1158),
    [anon_sym_u_SQUOTE] = ACTIONS(1158),
    [anon_sym_U_SQUOTE] = ACTIONS(1158),
    [anon_sym_u8_SQUOTE] = ACTIONS(1158),
    [anon_sym_SQUOTE] = ACTIONS(1158),
    [anon_sym_L_DQUOTE] = ACTIONS(1158),
    [anon_sym_u_DQUOTE] = ACTIONS(1158),
    [anon_sym_U_DQUOTE] = ACTIONS(1158),
    [anon_sym_u8_DQUOTE] = ACTIONS(1158),
    [anon_sym_DQUOTE] = ACTIONS(1158),
    [sym_true] = ACTIONS(1156),
    [sym_false] = ACTIONS(1156),
    [anon_sym_NULL] = ACTIONS(1156),
    [anon_sym_nullptr] = ACTIONS(1156),
    [sym_comment] = ACTIONS(3),
  },
  [270] = {
    [sym_identifier] = ACTIONS(1176),
    [aux_sym_preproc_include_token1] = ACTIONS(1176),
    [aux_sym_preproc_def_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1176),
    [sym_preproc_directive] = ACTIONS(1176),
    [anon_sym_LPAREN2] = ACTIONS(1178),
    [anon_sym_BANG] = ACTIONS(1178),
    [anon_sym_TILDE] = ACTIONS(1178),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_STAR] = ACTIONS(1178),
    [anon_sym_AMP] = ACTIONS(1178),
    [anon_sym_SEMI] = ACTIONS(1178),
    [anon_sym___extension__] = ACTIONS(1176),
    [anon_sym_typedef] = ACTIONS(1176),
    [anon_sym_extern] = ACTIONS(1176),
    [anon_sym___attribute__] = ACTIONS(1176),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1178),
    [anon_sym___declspec] = ACTIONS(1176),
    [anon_sym___cdecl] = ACTIONS(1176),
    [anon_sym___clrcall] = ACTIONS(1176),
    [anon_sym___stdcall] = ACTIONS(1176),
    [anon_sym___fastcall] = ACTIONS(1176),
    [anon_sym___thiscall] = ACTIONS(1176),
    [anon_sym___vectorcall] = ACTIONS(1176),
    [anon_sym_LBRACE] = ACTIONS(1178),
    [anon_sym_RBRACE] = ACTIONS(1178),
    [anon_sym_signed] = ACTIONS(1176),
    [anon_sym_unsigned] = ACTIONS(1176),
    [anon_sym_long] = ACTIONS(1176),
    [anon_sym_short] = ACTIONS(1176),
    [anon_sym_static] = ACTIONS(1176),
    [anon_sym_auto] = ACTIONS(1176),
    [anon_sym_register] = ACTIONS(1176),
    [anon_sym_inline] = ACTIONS(1176),
    [anon_sym___inline] = ACTIONS(1176),
    [anon_sym___inline__] = ACTIONS(1176),
    [anon_sym___forceinline] = ACTIONS(1176),
    [anon_sym_thread_local] = ACTIONS(1176),
    [anon_sym___thread] = ACTIONS(1176),
    [anon_sym_const] = ACTIONS(1176),
    [anon_sym_constexpr] = ACTIONS(1176),
    [anon_sym_volatile] = ACTIONS(1176),
    [anon_sym_restrict] = ACTIONS(1176),
    [anon_sym___restrict__] = ACTIONS(1176),
    [anon_sym__Atomic] = ACTIONS(1176),
    [anon_sym__Noreturn] = ACTIONS(1176),
    [anon_sym_noreturn] = ACTIONS(1176),
    [sym_primitive_type] = ACTIONS(1176),
    [anon_sym_enum] = ACTIONS(1176),
    [anon_sym_struct] = ACTIONS(1176),
    [anon_sym_union] = ACTIONS(1176),
    [anon_sym_if] = ACTIONS(1176),
    [anon_sym_else] = ACTIONS(1176),
    [anon_sym_switch] = ACTIONS(1176),
    [anon_sym_case] = ACTIONS(1176),
    [anon_sym_default] = ACTIONS(1176),
    [anon_sym_while] = ACTIONS(1176),
    [anon_sym_do] = ACTIONS(1176),
    [anon_sym_for] = ACTIONS(1176),
    [anon_sym_return] = ACTIONS(1176),
    [anon_sym_break] = ACTIONS(1176),
    [anon_sym_continue] = ACTIONS(1176),
    [anon_sym_goto] = ACTIONS(1176),
    [anon_sym___try] = ACTIONS(1176),
    [anon_sym___leave] = ACTIONS(1176),
    [anon_sym_DASH_DASH] = ACTIONS(1178),
    [anon_sym_PLUS_PLUS] = ACTIONS(1178),
    [anon_sym_sizeof] = ACTIONS(1176),
    [anon_sym___alignof__] = ACTIONS(1176),
    [anon_sym___alignof] = ACTIONS(1176),
    [anon_sym__alignof] = ACTIONS(1176),
    [anon_sym_alignof] = ACTIONS(1176),
    [anon_sym__Alignof] = ACTIONS(1176),
    [anon_sym_offsetof] = ACTIONS(1176),
    [anon_sym__Generic] = ACTIONS(1176),
    [anon_sym_asm] = ACTIONS(1176),
    [anon_sym___asm__] = ACTIONS(1176),
    [sym_number_literal] = ACTIONS(1178),
    [anon_sym_L_SQUOTE] = ACTIONS(1178),
    [anon_sym_u_SQUOTE] = ACTIONS(1178),
    [anon_sym_U_SQUOTE] = ACTIONS(1178),
    [anon_sym_u8_SQUOTE] = ACTIONS(1178),
    [anon_sym_SQUOTE] = ACTIONS(1178),
    [anon_sym_L_DQUOTE] = ACTIONS(1178),
    [anon_sym_u_DQUOTE] = ACTIONS(1178),
    [anon_sym_U_DQUOTE] = ACTIONS(1178),
    [anon_sym_u8_DQUOTE] = ACTIONS(1178),
    [anon_sym_DQUOTE] = ACTIONS(1178),
    [sym_true] = ACTIONS(1176),
    [sym_false] = ACTIONS(1176),
    [anon_sym_NULL] = ACTIONS(1176),
    [anon_sym_nullptr] = ACTIONS(1176),
    [sym_comment] = ACTIONS(3),
  },
  [271] = {
    [sym_identifier] = ACTIONS(1176),
    [aux_sym_preproc_include_token1] = ACTIONS(1176),
    [aux_sym_preproc_def_token1] = ACTIONS(1176),
    [aux_sym_preproc_if_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1176),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1176),
    [sym_preproc_directive] = ACTIONS(1176),
    [anon_sym_LPAREN2] = ACTIONS(1178),
    [anon_sym_BANG] = ACTIONS(1178),
    [anon_sym_TILDE] = ACTIONS(1178),
    [anon_sym_DASH] = ACTIONS(1176),
    [anon_sym_PLUS] = ACTIONS(1176),
    [anon_sym_STAR] = ACTIONS(1178),
    [anon_sym_AMP] = ACTIONS(1178),
    [anon_sym_SEMI] = ACTIONS(1178),
    [anon_sym___extension__] = ACTIONS(1176),
    [anon_sym_typedef] = ACTIONS(1176),
    [anon_sym_extern] = ACTIONS(1176),
    [anon_sym___attribute__] = ACTIONS(1176),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1178),
    [anon_sym___declspec] = ACTIONS(1176),
    [anon_sym___cdecl] = ACTIONS(1176),
    [anon_sym___clrcall] = ACTIONS(1176),
    [anon_sym___stdcall] = ACTIONS(1176),
    [anon_sym___fastcall] = ACTIONS(1176),
    [anon_sym___thiscall] = ACTIONS(1176),
    [anon_sym___vectorcall] = ACTIONS(1176),
    [anon_sym_LBRACE] = ACTIONS(1178),
    [anon_sym_RBRACE] = ACTIONS(1178),
    [anon_sym_signed] = ACTIONS(1176),
    [anon_sym_unsigned] = ACTIONS(1176),
    [anon_sym_long] = ACTIONS(1176),
    [anon_sym_short] = ACTIONS(1176),
    [anon_sym_static] = ACTIONS(1176),
    [anon_sym_auto] = ACTIONS(1176),
    [anon_sym_register] = ACTIONS(1176),
    [anon_sym_inline] = ACTIONS(1176),
    [anon_sym___inline] = ACTIONS(1176),
    [anon_sym___inline__] = ACTIONS(1176),
    [anon_sym___forceinline] = ACTIONS(1176),
    [anon_sym_thread_local] = ACTIONS(1176),
    [anon_sym___thread] = ACTIONS(1176),
    [anon_sym_const] = ACTIONS(1176),
    [anon_sym_constexpr] = ACTIONS(1176),
    [anon_sym_volatile] = ACTIONS(1176),
    [anon_sym_restrict] = ACTIONS(1176),
    [anon_sym___restrict__] = ACTIONS(1176),
    [anon_sym__Atomic] = ACTIONS(1176),
    [anon_sym__Noreturn] = ACTIONS(1176),
    [anon_sym_noreturn] = ACTIONS(1176),
    [sym_primitive_type] = ACTIONS(1176),
    [anon_sym_enum] = ACTIONS(1176),
    [anon_sym_struct] = ACTIONS(1176),
    [anon_sym_union] = ACTIONS(1176),
    [anon_sym_if] = ACTIONS(1176),
    [anon_sym_else] = ACTIONS(1176),
    [anon_sym_switch] = ACTIONS(1176),
    [anon_sym_case] = ACTIONS(1176),
    [anon_sym_default] = ACTIONS(1176),
    [anon_sym_while] = ACTIONS(1176),
    [anon_sym_do] = ACTIONS(1176),
    [anon_sym_for] = ACTIONS(1176),
    [anon_sym_return] = ACTIONS(1176),
    [anon_sym_break] = ACTIONS(1176),
    [anon_sym_continue] = ACTIONS(1176),
    [anon_sym_goto] = ACTIONS(1176),
    [anon_sym___try] = ACTIONS(1176),
    [anon_sym___leave] = ACTIONS(1176),
    [anon_sym_DASH_DASH] = ACTIONS(1178),
    [anon_sym_PLUS_PLUS] = ACTIONS(1178),
    [anon_sym_sizeof] = ACTIONS(1176),
    [anon_sym___alignof__] = ACTIONS(1176),
    [anon_sym___alignof] = ACTIONS(1176),
    [anon_sym__alignof] = ACTIONS(1176),
    [anon_sym_alignof] = ACTIONS(1176),
    [anon_sym__Alignof] = ACTIONS(1176),
    [anon_sym_offsetof] = ACTIONS(1176),
    [anon_sym__Generic] = ACTIONS(1176),
    [anon_sym_asm] = ACTIONS(1176),
    [anon_sym___asm__] = ACTIONS(1176),
    [sym_number_literal] = ACTIONS(1178),
    [anon_sym_L_SQUOTE] = ACTIONS(1178),
    [anon_sym_u_SQUOTE] = ACTIONS(1178),
    [anon_sym_U_SQUOTE] = ACTIONS(1178),
    [anon_sym_u8_SQUOTE] = ACTIONS(1178),
    [anon_sym_SQUOTE] = ACTIONS(1178),
    [anon_sym_L_DQUOTE] = ACTIONS(1178),
    [anon_sym_u_DQUOTE] = ACTIONS(1178),
    [anon_sym_U_DQUOTE] = ACTIONS(1178),
    [anon_sym_u8_DQUOTE] = ACTIONS(1178),
    [anon_sym_DQUOTE] = ACTIONS(1178),
    [sym_true] = ACTIONS(1176),
    [sym_false] = ACTIONS(1176),
    [anon_sym_NULL] = ACTIONS(1176),
    [anon_sym_nullptr] = ACTIONS(1176),
    [sym_comment] = ACTIONS(3),
  },
  [272] = {
    [sym_identifier] = ACTIONS(1200),
    [aux_sym_preproc_include_token1] = ACTIONS(1200),
    [aux_sym_preproc_def_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1200),
    [sym_preproc_directive] = ACTIONS(1200),
    [anon_sym_LPAREN2] = ACTIONS(1202),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_DASH] = ACTIONS(1200),
    [anon_sym_PLUS] = ACTIONS(1200),
    [anon_sym_STAR] = ACTIONS(1202),
    [anon_sym_AMP] = ACTIONS(1202),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym___extension__] = ACTIONS(1200),
    [anon_sym_typedef] = ACTIONS(1200),
    [anon_sym_extern] = ACTIONS(1200),
    [anon_sym___attribute__] = ACTIONS(1200),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1202),
    [anon_sym___declspec] = ACTIONS(1200),
    [anon_sym___cdecl] = ACTIONS(1200),
    [anon_sym___clrcall] = ACTIONS(1200),
    [anon_sym___stdcall] = ACTIONS(1200),
    [anon_sym___fastcall] = ACTIONS(1200),
    [anon_sym___thiscall] = ACTIONS(1200),
    [anon_sym___vectorcall] = ACTIONS(1200),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_RBRACE] = ACTIONS(1202),
    [anon_sym_signed] = ACTIONS(1200),
    [anon_sym_unsigned] = ACTIONS(1200),
    [anon_sym_long] = ACTIONS(1200),
    [anon_sym_short] = ACTIONS(1200),
    [anon_sym_static] = ACTIONS(1200),
    [anon_sym_auto] = ACTIONS(1200),
    [anon_sym_register] = ACTIONS(1200),
    [anon_sym_inline] = ACTIONS(1200),
    [anon_sym___inline] = ACTIONS(1200),
    [anon_sym___inline__] = ACTIONS(1200),
    [anon_sym___forceinline] = ACTIONS(1200),
    [anon_sym_thread_local] = ACTIONS(1200),
    [anon_sym___thread] = ACTIONS(1200),
    [anon_sym_const] = ACTIONS(1200),
    [anon_sym_constexpr] = ACTIONS(1200),
    [anon_sym_volatile] = ACTIONS(1200),
    [anon_sym_restrict] = ACTIONS(1200),
    [anon_sym___restrict__] = ACTIONS(1200),
    [anon_sym__Atomic] = ACTIONS(1200),
    [anon_sym__Noreturn] = ACTIONS(1200),
    [anon_sym_noreturn] = ACTIONS(1200),
    [sym_primitive_type] = ACTIONS(1200),
    [anon_sym_enum] = ACTIONS(1200),
    [anon_sym_struct] = ACTIONS(1200),
    [anon_sym_union] = ACTIONS(1200),
    [anon_sym_if] = ACTIONS(1200),
    [anon_sym_else] = ACTIONS(1200),
    [anon_sym_switch] = ACTIONS(1200),
    [anon_sym_case] = ACTIONS(1200),
    [anon_sym_default] = ACTIONS(1200),
    [anon_sym_while] = ACTIONS(1200),
    [anon_sym_do] = ACTIONS(1200),
    [anon_sym_for] = ACTIONS(1200),
    [anon_sym_return] = ACTIONS(1200),
    [anon_sym_break] = ACTIONS(1200),
    [anon_sym_continue] = ACTIONS(1200),
    [anon_sym_goto] = ACTIONS(1200),
    [anon_sym___try] = ACTIONS(1200),
    [anon_sym___leave] = ACTIONS(1200),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_sizeof] = ACTIONS(1200),
    [anon_sym___alignof__] = ACTIONS(1200),
    [anon_sym___alignof] = ACTIONS(1200),
    [anon_sym__alignof] = ACTIONS(1200),
    [anon_sym_alignof] = ACTIONS(1200),
    [anon_sym__Alignof] = ACTIONS(1200),
    [anon_sym_offsetof] = ACTIONS(1200),
    [anon_sym__Generic] = ACTIONS(1200),
    [anon_sym_asm] = ACTIONS(1200),
    [anon_sym___asm__] = ACTIONS(1200),
    [sym_number_literal] = ACTIONS(1202),
    [anon_sym_L_SQUOTE] = ACTIONS(1202),
    [anon_sym_u_SQUOTE] = ACTIONS(1202),
    [anon_sym_U_SQUOTE] = ACTIONS(1202),
    [anon_sym_u8_SQUOTE] = ACTIONS(1202),
    [anon_sym_SQUOTE] = ACTIONS(1202),
    [anon_sym_L_DQUOTE] = ACTIONS(1202),
    [anon_sym_u_DQUOTE] = ACTIONS(1202),
    [anon_sym_U_DQUOTE] = ACTIONS(1202),
    [anon_sym_u8_DQUOTE] = ACTIONS(1202),
    [anon_sym_DQUOTE] = ACTIONS(1202),
    [sym_true] = ACTIONS(1200),
    [sym_false] = ACTIONS(1200),
    [anon_sym_NULL] = ACTIONS(1200),
    [anon_sym_nullptr] = ACTIONS(1200),
    [sym_comment] = ACTIONS(3),
  },
  [273] = {
    [sym_identifier] = ACTIONS(1200),
    [aux_sym_preproc_include_token1] = ACTIONS(1200),
    [aux_sym_preproc_def_token1] = ACTIONS(1200),
    [aux_sym_preproc_if_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1200),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1200),
    [sym_preproc_directive] = ACTIONS(1200),
    [anon_sym_LPAREN2] = ACTIONS(1202),
    [anon_sym_BANG] = ACTIONS(1202),
    [anon_sym_TILDE] = ACTIONS(1202),
    [anon_sym_DASH] = ACTIONS(1200),
    [anon_sym_PLUS] = ACTIONS(1200),
    [anon_sym_STAR] = ACTIONS(1202),
    [anon_sym_AMP] = ACTIONS(1202),
    [anon_sym_SEMI] = ACTIONS(1202),
    [anon_sym___extension__] = ACTIONS(1200),
    [anon_sym_typedef] = ACTIONS(1200),
    [anon_sym_extern] = ACTIONS(1200),
    [anon_sym___attribute__] = ACTIONS(1200),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1202),
    [anon_sym___declspec] = ACTIONS(1200),
    [anon_sym___cdecl] = ACTIONS(1200),
    [anon_sym___clrcall] = ACTIONS(1200),
    [anon_sym___stdcall] = ACTIONS(1200),
    [anon_sym___fastcall] = ACTIONS(1200),
    [anon_sym___thiscall] = ACTIONS(1200),
    [anon_sym___vectorcall] = ACTIONS(1200),
    [anon_sym_LBRACE] = ACTIONS(1202),
    [anon_sym_RBRACE] = ACTIONS(1202),
    [anon_sym_signed] = ACTIONS(1200),
    [anon_sym_unsigned] = ACTIONS(1200),
    [anon_sym_long] = ACTIONS(1200),
    [anon_sym_short] = ACTIONS(1200),
    [anon_sym_static] = ACTIONS(1200),
    [anon_sym_auto] = ACTIONS(1200),
    [anon_sym_register] = ACTIONS(1200),
    [anon_sym_inline] = ACTIONS(1200),
    [anon_sym___inline] = ACTIONS(1200),
    [anon_sym___inline__] = ACTIONS(1200),
    [anon_sym___forceinline] = ACTIONS(1200),
    [anon_sym_thread_local] = ACTIONS(1200),
    [anon_sym___thread] = ACTIONS(1200),
    [anon_sym_const] = ACTIONS(1200),
    [anon_sym_constexpr] = ACTIONS(1200),
    [anon_sym_volatile] = ACTIONS(1200),
    [anon_sym_restrict] = ACTIONS(1200),
    [anon_sym___restrict__] = ACTIONS(1200),
    [anon_sym__Atomic] = ACTIONS(1200),
    [anon_sym__Noreturn] = ACTIONS(1200),
    [anon_sym_noreturn] = ACTIONS(1200),
    [sym_primitive_type] = ACTIONS(1200),
    [anon_sym_enum] = ACTIONS(1200),
    [anon_sym_struct] = ACTIONS(1200),
    [anon_sym_union] = ACTIONS(1200),
    [anon_sym_if] = ACTIONS(1200),
    [anon_sym_else] = ACTIONS(1200),
    [anon_sym_switch] = ACTIONS(1200),
    [anon_sym_case] = ACTIONS(1200),
    [anon_sym_default] = ACTIONS(1200),
    [anon_sym_while] = ACTIONS(1200),
    [anon_sym_do] = ACTIONS(1200),
    [anon_sym_for] = ACTIONS(1200),
    [anon_sym_return] = ACTIONS(1200),
    [anon_sym_break] = ACTIONS(1200),
    [anon_sym_continue] = ACTIONS(1200),
    [anon_sym_goto] = ACTIONS(1200),
    [anon_sym___try] = ACTIONS(1200),
    [anon_sym___leave] = ACTIONS(1200),
    [anon_sym_DASH_DASH] = ACTIONS(1202),
    [anon_sym_PLUS_PLUS] = ACTIONS(1202),
    [anon_sym_sizeof] = ACTIONS(1200),
    [anon_sym___alignof__] = ACTIONS(1200),
    [anon_sym___alignof] = ACTIONS(1200),
    [anon_sym__alignof] = ACTIONS(1200),
    [anon_sym_alignof] = ACTIONS(1200),
    [anon_sym__Alignof] = ACTIONS(1200),
    [anon_sym_offsetof] = ACTIONS(1200),
    [anon_sym__Generic] = ACTIONS(1200),
    [anon_sym_asm] = ACTIONS(1200),
    [anon_sym___asm__] = ACTIONS(1200),
    [sym_number_literal] = ACTIONS(1202),
    [anon_sym_L_SQUOTE] = ACTIONS(1202),
    [anon_sym_u_SQUOTE] = ACTIONS(1202),
    [anon_sym_U_SQUOTE] = ACTIONS(1202),
    [anon_sym_u8_SQUOTE] = ACTIONS(1202),
    [anon_sym_SQUOTE] = ACTIONS(1202),
    [anon_sym_L_DQUOTE] = ACTIONS(1202),
    [anon_sym_u_DQUOTE] = ACTIONS(1202),
    [anon_sym_U_DQUOTE] = ACTIONS(1202),
    [anon_sym_u8_DQUOTE] = ACTIONS(1202),
    [anon_sym_DQUOTE] = ACTIONS(1202),
    [sym_true] = ACTIONS(1200),
    [sym_false] = ACTIONS(1200),
    [anon_sym_NULL] = ACTIONS(1200),
    [anon_sym_nullptr] = ACTIONS(1200),
    [sym_comment] = ACTIONS(3),
  },
  [274] = {
    [sym_identifier] = ACTIONS(1216),
    [aux_sym_preproc_include_token1] = ACTIONS(1216),
    [aux_sym_preproc_def_token1] = ACTIONS(1216),
    [aux_sym_preproc_if_token1] = ACTIONS(1216),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1216),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1216),
    [sym_preproc_directive] = ACTIONS(1216),
    [anon_sym_LPAREN2] = ACTIONS(1218),
    [anon_sym_BANG] = ACTIONS(1218),
    [anon_sym_TILDE] = ACTIONS(1218),
    [anon_sym_DASH] = ACTIONS(1216),
    [anon_sym_PLUS] = ACTIONS(1216),
    [anon_sym_STAR] = ACTIONS(1218),
    [anon_sym_AMP] = ACTIONS(1218),
    [anon_sym_SEMI] = ACTIONS(1218),
    [anon_sym___extension__] = ACTIONS(1216),
    [anon_sym_typedef] = ACTIONS(1216),
    [anon_sym_extern] = ACTIONS(1216),
    [anon_sym___attribute__] = ACTIONS(1216),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1218),
    [anon_sym___declspec] = ACTIONS(1216),
    [anon_sym___cdecl] = ACTIONS(1216),
    [anon_sym___clrcall] = ACTIONS(1216),
    [anon_sym___stdcall] = ACTIONS(1216),
    [anon_sym___fastcall] = ACTIONS(1216),
    [anon_sym___thiscall] = ACTIONS(1216),
    [anon_sym___vectorcall] = ACTIONS(1216),
    [anon_sym_LBRACE] = ACTIONS(1218),
    [anon_sym_RBRACE] = ACTIONS(1218),
    [anon_sym_signed] = ACTIONS(1216),
    [anon_sym_unsigned] = ACTIONS(1216),
    [anon_sym_long] = ACTIONS(1216),
    [anon_sym_short] = ACTIONS(1216),
    [anon_sym_static] = ACTIONS(1216),
    [anon_sym_auto] = ACTIONS(1216),
    [anon_sym_register] = ACTIONS(1216),
    [anon_sym_inline] = ACTIONS(1216),
    [anon_sym___inline] = ACTIONS(1216),
    [anon_sym___inline__] = ACTIONS(1216),
    [anon_sym___forceinline] = ACTIONS(1216),
    [anon_sym_thread_local] = ACTIONS(1216),
    [anon_sym___thread] = ACTIONS(1216),
    [anon_sym_const] = ACTIONS(1216),
    [anon_sym_constexpr] = ACTIONS(1216),
    [anon_sym_volatile] = ACTIONS(1216),
    [anon_sym_restrict] = ACTIONS(1216),
    [anon_sym___restrict__] = ACTIONS(1216),
    [anon_sym__Atomic] = ACTIONS(1216),
    [anon_sym__Noreturn] = ACTIONS(1216),
    [anon_sym_noreturn] = ACTIONS(1216),
    [sym_primitive_type] = ACTIONS(1216),
    [anon_sym_enum] = ACTIONS(1216),
    [anon_sym_struct] = ACTIONS(1216),
    [anon_sym_union] = ACTIONS(1216),
    [anon_sym_if] = ACTIONS(1216),
    [anon_sym_else] = ACTIONS(1216),
    [anon_sym_switch] = ACTIONS(1216),
    [anon_sym_case] = ACTIONS(1216),
    [anon_sym_default] = ACTIONS(1216),
    [anon_sym_while] = ACTIONS(1216),
    [anon_sym_do] = ACTIONS(1216),
    [anon_sym_for] = ACTIONS(1216),
    [anon_sym_return] = ACTIONS(1216),
    [anon_sym_break] = ACTIONS(1216),
    [anon_sym_continue] = ACTIONS(1216),
    [anon_sym_goto] = ACTIONS(1216),
    [anon_sym___try] = ACTIONS(1216),
    [anon_sym___leave] = ACTIONS(1216),
    [anon_sym_DASH_DASH] = ACTIONS(1218),
    [anon_sym_PLUS_PLUS] = ACTIONS(1218),
    [anon_sym_sizeof] = ACTIONS(1216),
    [anon_sym___alignof__] = ACTIONS(1216),
    [anon_sym___alignof] = ACTIONS(1216),
    [anon_sym__alignof] = ACTIONS(1216),
    [anon_sym_alignof] = ACTIONS(1216),
    [anon_sym__Alignof] = ACTIONS(1216),
    [anon_sym_offsetof] = ACTIONS(1216),
    [anon_sym__Generic] = ACTIONS(1216),
    [anon_sym_asm] = ACTIONS(1216),
    [anon_sym___asm__] = ACTIONS(1216),
    [sym_number_literal] = ACTIONS(1218),
    [anon_sym_L_SQUOTE] = ACTIONS(1218),
    [anon_sym_u_SQUOTE] = ACTIONS(1218),
    [anon_sym_U_SQUOTE] = ACTIONS(1218),
    [anon_sym_u8_SQUOTE] = ACTIONS(1218),
    [anon_sym_SQUOTE] = ACTIONS(1218),
    [anon_sym_L_DQUOTE] = ACTIONS(1218),
    [anon_sym_u_DQUOTE] = ACTIONS(1218),
    [anon_sym_U_DQUOTE] = ACTIONS(1218),
    [anon_sym_u8_DQUOTE] = ACTIONS(1218),
    [anon_sym_DQUOTE] = ACTIONS(1218),
    [sym_true] = ACTIONS(1216),
    [sym_false] = ACTIONS(1216),
    [anon_sym_NULL] = ACTIONS(1216),
    [anon_sym_nullptr] = ACTIONS(1216),
    [sym_comment] = ACTIONS(3),
  },
  [275] = {
    [sym_identifier] = ACTIONS(1220),
    [aux_sym_preproc_include_token1] = ACTIONS(1220),
    [aux_sym_preproc_def_token1] = ACTIONS(1220),
    [aux_sym_preproc_if_token1] = ACTIONS(1220),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1220),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1220),
    [sym_preproc_directive] = ACTIONS(1220),
    [anon_sym_LPAREN2] = ACTIONS(1222),
    [anon_sym_BANG] = ACTIONS(1222),
    [anon_sym_TILDE] = ACTIONS(1222),
    [anon_sym_DASH] = ACTIONS(1220),
    [anon_sym_PLUS] = ACTIONS(1220),
    [anon_sym_STAR] = ACTIONS(1222),
    [anon_sym_AMP] = ACTIONS(1222),
    [anon_sym_SEMI] = ACTIONS(1222),
    [anon_sym___extension__] = ACTIONS(1220),
    [anon_sym_typedef] = ACTIONS(1220),
    [anon_sym_extern] = ACTIONS(1220),
    [anon_sym___attribute__] = ACTIONS(1220),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1222),
    [anon_sym___declspec] = ACTIONS(1220),
    [anon_sym___cdecl] = ACTIONS(1220),
    [anon_sym___clrcall] = ACTIONS(1220),
    [anon_sym___stdcall] = ACTIONS(1220),
    [anon_sym___fastcall] = ACTIONS(1220),
    [anon_sym___thiscall] = ACTIONS(1220),
    [anon_sym___vectorcall] = ACTIONS(1220),
    [anon_sym_LBRACE] = ACTIONS(1222),
    [anon_sym_RBRACE] = ACTIONS(1222),
    [anon_sym_signed] = ACTIONS(1220),
    [anon_sym_unsigned] = ACTIONS(1220),
    [anon_sym_long] = ACTIONS(1220),
    [anon_sym_short] = ACTIONS(1220),
    [anon_sym_static] = ACTIONS(1220),
    [anon_sym_auto] = ACTIONS(1220),
    [anon_sym_register] = ACTIONS(1220),
    [anon_sym_inline] = ACTIONS(1220),
    [anon_sym___inline] = ACTIONS(1220),
    [anon_sym___inline__] = ACTIONS(1220),
    [anon_sym___forceinline] = ACTIONS(1220),
    [anon_sym_thread_local] = ACTIONS(1220),
    [anon_sym___thread] = ACTIONS(1220),
    [anon_sym_const] = ACTIONS(1220),
    [anon_sym_constexpr] = ACTIONS(1220),
    [anon_sym_volatile] = ACTIONS(1220),
    [anon_sym_restrict] = ACTIONS(1220),
    [anon_sym___restrict__] = ACTIONS(1220),
    [anon_sym__Atomic] = ACTIONS(1220),
    [anon_sym__Noreturn] = ACTIONS(1220),
    [anon_sym_noreturn] = ACTIONS(1220),
    [sym_primitive_type] = ACTIONS(1220),
    [anon_sym_enum] = ACTIONS(1220),
    [anon_sym_struct] = ACTIONS(1220),
    [anon_sym_union] = ACTIONS(1220),
    [anon_sym_if] = ACTIONS(1220),
    [anon_sym_else] = ACTIONS(1220),
    [anon_sym_switch] = ACTIONS(1220),
    [anon_sym_case] = ACTIONS(1220),
    [anon_sym_default] = ACTIONS(1220),
    [anon_sym_while] = ACTIONS(1220),
    [anon_sym_do] = ACTIONS(1220),
    [anon_sym_for] = ACTIONS(1220),
    [anon_sym_return] = ACTIONS(1220),
    [anon_sym_break] = ACTIONS(1220),
    [anon_sym_continue] = ACTIONS(1220),
    [anon_sym_goto] = ACTIONS(1220),
    [anon_sym___try] = ACTIONS(1220),
    [anon_sym___leave] = ACTIONS(1220),
    [anon_sym_DASH_DASH] = ACTIONS(1222),
    [anon_sym_PLUS_PLUS] = ACTIONS(1222),
    [anon_sym_sizeof] = ACTIONS(1220),
    [anon_sym___alignof__] = ACTIONS(1220),
    [anon_sym___alignof] = ACTIONS(1220),
    [anon_sym__alignof] = ACTIONS(1220),
    [anon_sym_alignof] = ACTIONS(1220),
    [anon_sym__Alignof] = ACTIONS(1220),
    [anon_sym_offsetof] = ACTIONS(1220),
    [anon_sym__Generic] = ACTIONS(1220),
    [anon_sym_asm] = ACTIONS(1220),
    [anon_sym___asm__] = ACTIONS(1220),
    [sym_number_literal] = ACTIONS(1222),
    [anon_sym_L_SQUOTE] = ACTIONS(1222),
    [anon_sym_u_SQUOTE] = ACTIONS(1222),
    [anon_sym_U_SQUOTE] = ACTIONS(1222),
    [anon_sym_u8_SQUOTE] = ACTIONS(1222),
    [anon_sym_SQUOTE] = ACTIONS(1222),
    [anon_sym_L_DQUOTE] = ACTIONS(1222),
    [anon_sym_u_DQUOTE] = ACTIONS(1222),
    [anon_sym_U_DQUOTE] = ACTIONS(1222),
    [anon_sym_u8_DQUOTE] = ACTIONS(1222),
    [anon_sym_DQUOTE] = ACTIONS(1222),
    [sym_true] = ACTIONS(1220),
    [sym_false] = ACTIONS(1220),
    [anon_sym_NULL] = ACTIONS(1220),
    [anon_sym_nullptr] = ACTIONS(1220),
    [sym_comment] = ACTIONS(3),
  },
  [276] = {
    [ts_builtin_sym_end] = ACTIONS(1114),
    [sym_identifier] = ACTIONS(1112),
    [aux_sym_preproc_include_token1] = ACTIONS(1112),
    [aux_sym_preproc_def_token1] = ACTIONS(1112),
    [aux_sym_preproc_if_token1] = ACTIONS(1112),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1112),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1112),
    [sym_preproc_directive] = ACTIONS(1112),
    [anon_sym_LPAREN2] = ACTIONS(1114),
    [anon_sym_BANG] = ACTIONS(1114),
    [anon_sym_TILDE] = ACTIONS(1114),
    [anon_sym_DASH] = ACTIONS(1112),
    [anon_sym_PLUS] = ACTIONS(1112),
    [anon_sym_STAR] = ACTIONS(1114),
    [anon_sym_AMP] = ACTIONS(1114),
    [anon_sym_SEMI] = ACTIONS(1114),
    [anon_sym___extension__] = ACTIONS(1112),
    [anon_sym_typedef] = ACTIONS(1112),
    [anon_sym_extern] = ACTIONS(1112),
    [anon_sym___attribute__] = ACTIONS(1112),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1114),
    [anon_sym___declspec] = ACTIONS(1112),
    [anon_sym___cdecl] = ACTIONS(1112),
    [anon_sym___clrcall] = ACTIONS(1112),
    [anon_sym___stdcall] = ACTIONS(1112),
    [anon_sym___fastcall] = ACTIONS(1112),
    [anon_sym___thiscall] = ACTIONS(1112),
    [anon_sym___vectorcall] = ACTIONS(1112),
    [anon_sym_LBRACE] = ACTIONS(1114),
    [anon_sym_signed] = ACTIONS(1112),
    [anon_sym_unsigned] = ACTIONS(1112),
    [anon_sym_long] = ACTIONS(1112),
    [anon_sym_short] = ACTIONS(1112),
    [anon_sym_static] = ACTIONS(1112),
    [anon_sym_auto] = ACTIONS(1112),
    [anon_sym_register] = ACTIONS(1112),
    [anon_sym_inline] = ACTIONS(1112),
    [anon_sym___inline] = ACTIONS(1112),
    [anon_sym___inline__] = ACTIONS(1112),
    [anon_sym___forceinline] = ACTIONS(1112),
    [anon_sym_thread_local] = ACTIONS(1112),
    [anon_sym___thread] = ACTIONS(1112),
    [anon_sym_const] = ACTIONS(1112),
    [anon_sym_constexpr] = ACTIONS(1112),
    [anon_sym_volatile] = ACTIONS(1112),
    [anon_sym_restrict] = ACTIONS(1112),
    [anon_sym___restrict__] = ACTIONS(1112),
    [anon_sym__Atomic] = ACTIONS(1112),
    [anon_sym__Noreturn] = ACTIONS(1112),
    [anon_sym_noreturn] = ACTIONS(1112),
    [sym_primitive_type] = ACTIONS(1112),
    [anon_sym_enum] = ACTIONS(1112),
    [anon_sym_struct] = ACTIONS(1112),
    [anon_sym_union] = ACTIONS(1112),
    [anon_sym_if] = ACTIONS(1112),
    [anon_sym_else] = ACTIONS(1112),
    [anon_sym_switch] = ACTIONS(1112),
    [anon_sym_case] = ACTIONS(1112),
    [anon_sym_default] = ACTIONS(1112),
    [anon_sym_while] = ACTIONS(1112),
    [anon_sym_do] = ACTIONS(1112),
    [anon_sym_for] = ACTIONS(1112),
    [anon_sym_return] = ACTIONS(1112),
    [anon_sym_break] = ACTIONS(1112),
    [anon_sym_continue] = ACTIONS(1112),
    [anon_sym_goto] = ACTIONS(1112),
    [anon_sym___try] = ACTIONS(1112),
    [anon_sym___leave] = ACTIONS(1112),
    [anon_sym_DASH_DASH] = ACTIONS(1114),
    [anon_sym_PLUS_PLUS] = ACTIONS(1114),
    [anon_sym_sizeof] = ACTIONS(1112),
    [anon_sym___alignof__] = ACTIONS(1112),
    [anon_sym___alignof] = ACTIONS(1112),
    [anon_sym__alignof] = ACTIONS(1112),
    [anon_sym_alignof] = ACTIONS(1112),
    [anon_sym__Alignof] = ACTIONS(1112),
    [anon_sym_offsetof] = ACTIONS(1112),
    [anon_sym__Generic] = ACTIONS(1112),
    [anon_sym_asm] = ACTIONS(1112),
    [anon_sym___asm__] = ACTIONS(1112),
    [sym_number_literal] = ACTIONS(1114),
    [anon_sym_L_SQUOTE] = ACTIONS(1114),
    [anon_sym_u_SQUOTE] = ACTIONS(1114),
    [anon_sym_U_SQUOTE] = ACTIONS(1114),
    [anon_sym_u8_SQUOTE] = ACTIONS(1114),
    [anon_sym_SQUOTE] = ACTIONS(1114),
    [anon_sym_L_DQUOTE] = ACTIONS(1114),
    [anon_sym_u_DQUOTE] = ACTIONS(1114),
    [anon_sym_U_DQUOTE] = ACTIONS(1114),
    [anon_sym_u8_DQUOTE] = ACTIONS(1114),
    [anon_sym_DQUOTE] = ACTIONS(1114),
    [sym_true] = ACTIONS(1112),
    [sym_false] = ACTIONS(1112),
    [anon_sym_NULL] = ACTIONS(1112),
    [anon_sym_nullptr] = ACTIONS(1112),
    [sym_comment] = ACTIONS(3),
  },
  [277] = {
    [sym_identifier] = ACTIONS(1228),
    [aux_sym_preproc_include_token1] = ACTIONS(1228),
    [aux_sym_preproc_def_token1] = ACTIONS(1228),
    [aux_sym_preproc_if_token1] = ACTIONS(1228),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1228),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1228),
    [sym_preproc_directive] = ACTIONS(1228),
    [anon_sym_LPAREN2] = ACTIONS(1230),
    [anon_sym_BANG] = ACTIONS(1230),
    [anon_sym_TILDE] = ACTIONS(1230),
    [anon_sym_DASH] = ACTIONS(1228),
    [anon_sym_PLUS] = ACTIONS(1228),
    [anon_sym_STAR] = ACTIONS(1230),
    [anon_sym_AMP] = ACTIONS(1230),
    [anon_sym_SEMI] = ACTIONS(1230),
    [anon_sym___extension__] = ACTIONS(1228),
    [anon_sym_typedef] = ACTIONS(1228),
    [anon_sym_extern] = ACTIONS(1228),
    [anon_sym___attribute__] = ACTIONS(1228),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1230),
    [anon_sym___declspec] = ACTIONS(1228),
    [anon_sym___cdecl] = ACTIONS(1228),
    [anon_sym___clrcall] = ACTIONS(1228),
    [anon_sym___stdcall] = ACTIONS(1228),
    [anon_sym___fastcall] = ACTIONS(1228),
    [anon_sym___thiscall] = ACTIONS(1228),
    [anon_sym___vectorcall] = ACTIONS(1228),
    [anon_sym_LBRACE] = ACTIONS(1230),
    [anon_sym_RBRACE] = ACTIONS(1230),
    [anon_sym_signed] = ACTIONS(1228),
    [anon_sym_unsigned] = ACTIONS(1228),
    [anon_sym_long] = ACTIONS(1228),
    [anon_sym_short] = ACTIONS(1228),
    [anon_sym_static] = ACTIONS(1228),
    [anon_sym_auto] = ACTIONS(1228),
    [anon_sym_register] = ACTIONS(1228),
    [anon_sym_inline] = ACTIONS(1228),
    [anon_sym___inline] = ACTIONS(1228),
    [anon_sym___inline__] = ACTIONS(1228),
    [anon_sym___forceinline] = ACTIONS(1228),
    [anon_sym_thread_local] = ACTIONS(1228),
    [anon_sym___thread] = ACTIONS(1228),
    [anon_sym_const] = ACTIONS(1228),
    [anon_sym_constexpr] = ACTIONS(1228),
    [anon_sym_volatile] = ACTIONS(1228),
    [anon_sym_restrict] = ACTIONS(1228),
    [anon_sym___restrict__] = ACTIONS(1228),
    [anon_sym__Atomic] = ACTIONS(1228),
    [anon_sym__Noreturn] = ACTIONS(1228),
    [anon_sym_noreturn] = ACTIONS(1228),
    [sym_primitive_type] = ACTIONS(1228),
    [anon_sym_enum] = ACTIONS(1228),
    [anon_sym_struct] = ACTIONS(1228),
    [anon_sym_union] = ACTIONS(1228),
    [anon_sym_if] = ACTIONS(1228),
    [anon_sym_else] = ACTIONS(1228),
    [anon_sym_switch] = ACTIONS(1228),
    [anon_sym_case] = ACTIONS(1228),
    [anon_sym_default] = ACTIONS(1228),
    [anon_sym_while] = ACTIONS(1228),
    [anon_sym_do] = ACTIONS(1228),
    [anon_sym_for] = ACTIONS(1228),
    [anon_sym_return] = ACTIONS(1228),
    [anon_sym_break] = ACTIONS(1228),
    [anon_sym_continue] = ACTIONS(1228),
    [anon_sym_goto] = ACTIONS(1228),
    [anon_sym___try] = ACTIONS(1228),
    [anon_sym___leave] = ACTIONS(1228),
    [anon_sym_DASH_DASH] = ACTIONS(1230),
    [anon_sym_PLUS_PLUS] = ACTIONS(1230),
    [anon_sym_sizeof] = ACTIONS(1228),
    [anon_sym___alignof__] = ACTIONS(1228),
    [anon_sym___alignof] = ACTIONS(1228),
    [anon_sym__alignof] = ACTIONS(1228),
    [anon_sym_alignof] = ACTIONS(1228),
    [anon_sym__Alignof] = ACTIONS(1228),
    [anon_sym_offsetof] = ACTIONS(1228),
    [anon_sym__Generic] = ACTIONS(1228),
    [anon_sym_asm] = ACTIONS(1228),
    [anon_sym___asm__] = ACTIONS(1228),
    [sym_number_literal] = ACTIONS(1230),
    [anon_sym_L_SQUOTE] = ACTIONS(1230),
    [anon_sym_u_SQUOTE] = ACTIONS(1230),
    [anon_sym_U_SQUOTE] = ACTIONS(1230),
    [anon_sym_u8_SQUOTE] = ACTIONS(1230),
    [anon_sym_SQUOTE] = ACTIONS(1230),
    [anon_sym_L_DQUOTE] = ACTIONS(1230),
    [anon_sym_u_DQUOTE] = ACTIONS(1230),
    [anon_sym_U_DQUOTE] = ACTIONS(1230),
    [anon_sym_u8_DQUOTE] = ACTIONS(1230),
    [anon_sym_DQUOTE] = ACTIONS(1230),
    [sym_true] = ACTIONS(1228),
    [sym_false] = ACTIONS(1228),
    [anon_sym_NULL] = ACTIONS(1228),
    [anon_sym_nullptr] = ACTIONS(1228),
    [sym_comment] = ACTIONS(3),
  },
  [278] = {
    [sym_identifier] = ACTIONS(1232),
    [aux_sym_preproc_include_token1] = ACTIONS(1232),
    [aux_sym_preproc_def_token1] = ACTIONS(1232),
    [aux_sym_preproc_if_token1] = ACTIONS(1232),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1232),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1232),
    [sym_preproc_directive] = ACTIONS(1232),
    [anon_sym_LPAREN2] = ACTIONS(1234),
    [anon_sym_BANG] = ACTIONS(1234),
    [anon_sym_TILDE] = ACTIONS(1234),
    [anon_sym_DASH] = ACTIONS(1232),
    [anon_sym_PLUS] = ACTIONS(1232),
    [anon_sym_STAR] = ACTIONS(1234),
    [anon_sym_AMP] = ACTIONS(1234),
    [anon_sym_SEMI] = ACTIONS(1234),
    [anon_sym___extension__] = ACTIONS(1232),
    [anon_sym_typedef] = ACTIONS(1232),
    [anon_sym_extern] = ACTIONS(1232),
    [anon_sym___attribute__] = ACTIONS(1232),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1234),
    [anon_sym___declspec] = ACTIONS(1232),
    [anon_sym___cdecl] = ACTIONS(1232),
    [anon_sym___clrcall] = ACTIONS(1232),
    [anon_sym___stdcall] = ACTIONS(1232),
    [anon_sym___fastcall] = ACTIONS(1232),
    [anon_sym___thiscall] = ACTIONS(1232),
    [anon_sym___vectorcall] = ACTIONS(1232),
    [anon_sym_LBRACE] = ACTIONS(1234),
    [anon_sym_RBRACE] = ACTIONS(1234),
    [anon_sym_signed] = ACTIONS(1232),
    [anon_sym_unsigned] = ACTIONS(1232),
    [anon_sym_long] = ACTIONS(1232),
    [anon_sym_short] = ACTIONS(1232),
    [anon_sym_static] = ACTIONS(1232),
    [anon_sym_auto] = ACTIONS(1232),
    [anon_sym_register] = ACTIONS(1232),
    [anon_sym_inline] = ACTIONS(1232),
    [anon_sym___inline] = ACTIONS(1232),
    [anon_sym___inline__] = ACTIONS(1232),
    [anon_sym___forceinline] = ACTIONS(1232),
    [anon_sym_thread_local] = ACTIONS(1232),
    [anon_sym___thread] = ACTIONS(1232),
    [anon_sym_const] = ACTIONS(1232),
    [anon_sym_constexpr] = ACTIONS(1232),
    [anon_sym_volatile] = ACTIONS(1232),
    [anon_sym_restrict] = ACTIONS(1232),
    [anon_sym___restrict__] = ACTIONS(1232),
    [anon_sym__Atomic] = ACTIONS(1232),
    [anon_sym__Noreturn] = ACTIONS(1232),
    [anon_sym_noreturn] = ACTIONS(1232),
    [sym_primitive_type] = ACTIONS(1232),
    [anon_sym_enum] = ACTIONS(1232),
    [anon_sym_struct] = ACTIONS(1232),
    [anon_sym_union] = ACTIONS(1232),
    [anon_sym_if] = ACTIONS(1232),
    [anon_sym_else] = ACTIONS(1232),
    [anon_sym_switch] = ACTIONS(1232),
    [anon_sym_case] = ACTIONS(1232),
    [anon_sym_default] = ACTIONS(1232),
    [anon_sym_while] = ACTIONS(1232),
    [anon_sym_do] = ACTIONS(1232),
    [anon_sym_for] = ACTIONS(1232),
    [anon_sym_return] = ACTIONS(1232),
    [anon_sym_break] = ACTIONS(1232),
    [anon_sym_continue] = ACTIONS(1232),
    [anon_sym_goto] = ACTIONS(1232),
    [anon_sym___try] = ACTIONS(1232),
    [anon_sym___leave] = ACTIONS(1232),
    [anon_sym_DASH_DASH] = ACTIONS(1234),
    [anon_sym_PLUS_PLUS] = ACTIONS(1234),
    [anon_sym_sizeof] = ACTIONS(1232),
    [anon_sym___alignof__] = ACTIONS(1232),
    [anon_sym___alignof] = ACTIONS(1232),
    [anon_sym__alignof] = ACTIONS(1232),
    [anon_sym_alignof] = ACTIONS(1232),
    [anon_sym__Alignof] = ACTIONS(1232),
    [anon_sym_offsetof] = ACTIONS(1232),
    [anon_sym__Generic] = ACTIONS(1232),
    [anon_sym_asm] = ACTIONS(1232),
    [anon_sym___asm__] = ACTIONS(1232),
    [sym_number_literal] = ACTIONS(1234),
    [anon_sym_L_SQUOTE] = ACTIONS(1234),
    [anon_sym_u_SQUOTE] = ACTIONS(1234),
    [anon_sym_U_SQUOTE] = ACTIONS(1234),
    [anon_sym_u8_SQUOTE] = ACTIONS(1234),
    [anon_sym_SQUOTE] = ACTIONS(1234),
    [anon_sym_L_DQUOTE] = ACTIONS(1234),
    [anon_sym_u_DQUOTE] = ACTIONS(1234),
    [anon_sym_U_DQUOTE] = ACTIONS(1234),
    [anon_sym_u8_DQUOTE] = ACTIONS(1234),
    [anon_sym_DQUOTE] = ACTIONS(1234),
    [sym_true] = ACTIONS(1232),
    [sym_false] = ACTIONS(1232),
    [anon_sym_NULL] = ACTIONS(1232),
    [anon_sym_nullptr] = ACTIONS(1232),
    [sym_comment] = ACTIONS(3),
  },
  [279] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [280] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [281] = {
    [ts_builtin_sym_end] = ACTIONS(1198),
    [sym_identifier] = ACTIONS(1196),
    [aux_sym_preproc_include_token1] = ACTIONS(1196),
    [aux_sym_preproc_def_token1] = ACTIONS(1196),
    [aux_sym_preproc_if_token1] = ACTIONS(1196),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1196),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1196),
    [sym_preproc_directive] = ACTIONS(1196),
    [anon_sym_LPAREN2] = ACTIONS(1198),
    [anon_sym_BANG] = ACTIONS(1198),
    [anon_sym_TILDE] = ACTIONS(1198),
    [anon_sym_DASH] = ACTIONS(1196),
    [anon_sym_PLUS] = ACTIONS(1196),
    [anon_sym_STAR] = ACTIONS(1198),
    [anon_sym_AMP] = ACTIONS(1198),
    [anon_sym_SEMI] = ACTIONS(1198),
    [anon_sym___extension__] = ACTIONS(1196),
    [anon_sym_typedef] = ACTIONS(1196),
    [anon_sym_extern] = ACTIONS(1196),
    [anon_sym___attribute__] = ACTIONS(1196),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1198),
    [anon_sym___declspec] = ACTIONS(1196),
    [anon_sym___cdecl] = ACTIONS(1196),
    [anon_sym___clrcall] = ACTIONS(1196),
    [anon_sym___stdcall] = ACTIONS(1196),
    [anon_sym___fastcall] = ACTIONS(1196),
    [anon_sym___thiscall] = ACTIONS(1196),
    [anon_sym___vectorcall] = ACTIONS(1196),
    [anon_sym_LBRACE] = ACTIONS(1198),
    [anon_sym_signed] = ACTIONS(1196),
    [anon_sym_unsigned] = ACTIONS(1196),
    [anon_sym_long] = ACTIONS(1196),
    [anon_sym_short] = ACTIONS(1196),
    [anon_sym_static] = ACTIONS(1196),
    [anon_sym_auto] = ACTIONS(1196),
    [anon_sym_register] = ACTIONS(1196),
    [anon_sym_inline] = ACTIONS(1196),
    [anon_sym___inline] = ACTIONS(1196),
    [anon_sym___inline__] = ACTIONS(1196),
    [anon_sym___forceinline] = ACTIONS(1196),
    [anon_sym_thread_local] = ACTIONS(1196),
    [anon_sym___thread] = ACTIONS(1196),
    [anon_sym_const] = ACTIONS(1196),
    [anon_sym_constexpr] = ACTIONS(1196),
    [anon_sym_volatile] = ACTIONS(1196),
    [anon_sym_restrict] = ACTIONS(1196),
    [anon_sym___restrict__] = ACTIONS(1196),
    [anon_sym__Atomic] = ACTIONS(1196),
    [anon_sym__Noreturn] = ACTIONS(1196),
    [anon_sym_noreturn] = ACTIONS(1196),
    [sym_primitive_type] = ACTIONS(1196),
    [anon_sym_enum] = ACTIONS(1196),
    [anon_sym_struct] = ACTIONS(1196),
    [anon_sym_union] = ACTIONS(1196),
    [anon_sym_if] = ACTIONS(1196),
    [anon_sym_else] = ACTIONS(1196),
    [anon_sym_switch] = ACTIONS(1196),
    [anon_sym_case] = ACTIONS(1196),
    [anon_sym_default] = ACTIONS(1196),
    [anon_sym_while] = ACTIONS(1196),
    [anon_sym_do] = ACTIONS(1196),
    [anon_sym_for] = ACTIONS(1196),
    [anon_sym_return] = ACTIONS(1196),
    [anon_sym_break] = ACTIONS(1196),
    [anon_sym_continue] = ACTIONS(1196),
    [anon_sym_goto] = ACTIONS(1196),
    [anon_sym___try] = ACTIONS(1196),
    [anon_sym___leave] = ACTIONS(1196),
    [anon_sym_DASH_DASH] = ACTIONS(1198),
    [anon_sym_PLUS_PLUS] = ACTIONS(1198),
    [anon_sym_sizeof] = ACTIONS(1196),
    [anon_sym___alignof__] = ACTIONS(1196),
    [anon_sym___alignof] = ACTIONS(1196),
    [anon_sym__alignof] = ACTIONS(1196),
    [anon_sym_alignof] = ACTIONS(1196),
    [anon_sym__Alignof] = ACTIONS(1196),
    [anon_sym_offsetof] = ACTIONS(1196),
    [anon_sym__Generic] = ACTIONS(1196),
    [anon_sym_asm] = ACTIONS(1196),
    [anon_sym___asm__] = ACTIONS(1196),
    [sym_number_literal] = ACTIONS(1198),
    [anon_sym_L_SQUOTE] = ACTIONS(1198),
    [anon_sym_u_SQUOTE] = ACTIONS(1198),
    [anon_sym_U_SQUOTE] = ACTIONS(1198),
    [anon_sym_u8_SQUOTE] = ACTIONS(1198),
    [anon_sym_SQUOTE] = ACTIONS(1198),
    [anon_sym_L_DQUOTE] = ACTIONS(1198),
    [anon_sym_u_DQUOTE] = ACTIONS(1198),
    [anon_sym_U_DQUOTE] = ACTIONS(1198),
    [anon_sym_u8_DQUOTE] = ACTIONS(1198),
    [anon_sym_DQUOTE] = ACTIONS(1198),
    [sym_true] = ACTIONS(1196),
    [sym_false] = ACTIONS(1196),
    [anon_sym_NULL] = ACTIONS(1196),
    [anon_sym_nullptr] = ACTIONS(1196),
    [sym_comment] = ACTIONS(3),
  },
  [282] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [283] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [284] = {
    [sym_identifier] = ACTIONS(1124),
    [aux_sym_preproc_include_token1] = ACTIONS(1124),
    [aux_sym_preproc_def_token1] = ACTIONS(1124),
    [aux_sym_preproc_if_token1] = ACTIONS(1124),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1124),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1124),
    [sym_preproc_directive] = ACTIONS(1124),
    [anon_sym_LPAREN2] = ACTIONS(1126),
    [anon_sym_BANG] = ACTIONS(1126),
    [anon_sym_TILDE] = ACTIONS(1126),
    [anon_sym_DASH] = ACTIONS(1124),
    [anon_sym_PLUS] = ACTIONS(1124),
    [anon_sym_STAR] = ACTIONS(1126),
    [anon_sym_AMP] = ACTIONS(1126),
    [anon_sym_SEMI] = ACTIONS(1126),
    [anon_sym___extension__] = ACTIONS(1124),
    [anon_sym_typedef] = ACTIONS(1124),
    [anon_sym_extern] = ACTIONS(1124),
    [anon_sym___attribute__] = ACTIONS(1124),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1126),
    [anon_sym___declspec] = ACTIONS(1124),
    [anon_sym___cdecl] = ACTIONS(1124),
    [anon_sym___clrcall] = ACTIONS(1124),
    [anon_sym___stdcall] = ACTIONS(1124),
    [anon_sym___fastcall] = ACTIONS(1124),
    [anon_sym___thiscall] = ACTIONS(1124),
    [anon_sym___vectorcall] = ACTIONS(1124),
    [anon_sym_LBRACE] = ACTIONS(1126),
    [anon_sym_RBRACE] = ACTIONS(1126),
    [anon_sym_signed] = ACTIONS(1124),
    [anon_sym_unsigned] = ACTIONS(1124),
    [anon_sym_long] = ACTIONS(1124),
    [anon_sym_short] = ACTIONS(1124),
    [anon_sym_static] = ACTIONS(1124),
    [anon_sym_auto] = ACTIONS(1124),
    [anon_sym_register] = ACTIONS(1124),
    [anon_sym_inline] = ACTIONS(1124),
    [anon_sym___inline] = ACTIONS(1124),
    [anon_sym___inline__] = ACTIONS(1124),
    [anon_sym___forceinline] = ACTIONS(1124),
    [anon_sym_thread_local] = ACTIONS(1124),
    [anon_sym___thread] = ACTIONS(1124),
    [anon_sym_const] = ACTIONS(1124),
    [anon_sym_constexpr] = ACTIONS(1124),
    [anon_sym_volatile] = ACTIONS(1124),
    [anon_sym_restrict] = ACTIONS(1124),
    [anon_sym___restrict__] = ACTIONS(1124),
    [anon_sym__Atomic] = ACTIONS(1124),
    [anon_sym__Noreturn] = ACTIONS(1124),
    [anon_sym_noreturn] = ACTIONS(1124),
    [sym_primitive_type] = ACTIONS(1124),
    [anon_sym_enum] = ACTIONS(1124),
    [anon_sym_struct] = ACTIONS(1124),
    [anon_sym_union] = ACTIONS(1124),
    [anon_sym_if] = ACTIONS(1124),
    [anon_sym_else] = ACTIONS(1124),
    [anon_sym_switch] = ACTIONS(1124),
    [anon_sym_case] = ACTIONS(1124),
    [anon_sym_default] = ACTIONS(1124),
    [anon_sym_while] = ACTIONS(1124),
    [anon_sym_do] = ACTIONS(1124),
    [anon_sym_for] = ACTIONS(1124),
    [anon_sym_return] = ACTIONS(1124),
    [anon_sym_break] = ACTIONS(1124),
    [anon_sym_continue] = ACTIONS(1124),
    [anon_sym_goto] = ACTIONS(1124),
    [anon_sym___try] = ACTIONS(1124),
    [anon_sym___leave] = ACTIONS(1124),
    [anon_sym_DASH_DASH] = ACTIONS(1126),
    [anon_sym_PLUS_PLUS] = ACTIONS(1126),
    [anon_sym_sizeof] = ACTIONS(1124),
    [anon_sym___alignof__] = ACTIONS(1124),
    [anon_sym___alignof] = ACTIONS(1124),
    [anon_sym__alignof] = ACTIONS(1124),
    [anon_sym_alignof] = ACTIONS(1124),
    [anon_sym__Alignof] = ACTIONS(1124),
    [anon_sym_offsetof] = ACTIONS(1124),
    [anon_sym__Generic] = ACTIONS(1124),
    [anon_sym_asm] = ACTIONS(1124),
    [anon_sym___asm__] = ACTIONS(1124),
    [sym_number_literal] = ACTIONS(1126),
    [anon_sym_L_SQUOTE] = ACTIONS(1126),
    [anon_sym_u_SQUOTE] = ACTIONS(1126),
    [anon_sym_U_SQUOTE] = ACTIONS(1126),
    [anon_sym_u8_SQUOTE] = ACTIONS(1126),
    [anon_sym_SQUOTE] = ACTIONS(1126),
    [anon_sym_L_DQUOTE] = ACTIONS(1126),
    [anon_sym_u_DQUOTE] = ACTIONS(1126),
    [anon_sym_U_DQUOTE] = ACTIONS(1126),
    [anon_sym_u8_DQUOTE] = ACTIONS(1126),
    [anon_sym_DQUOTE] = ACTIONS(1126),
    [sym_true] = ACTIONS(1124),
    [sym_false] = ACTIONS(1124),
    [anon_sym_NULL] = ACTIONS(1124),
    [anon_sym_nullptr] = ACTIONS(1124),
    [sym_comment] = ACTIONS(3),
  },
  [285] = {
    [ts_builtin_sym_end] = ACTIONS(1194),
    [sym_identifier] = ACTIONS(1192),
    [aux_sym_preproc_include_token1] = ACTIONS(1192),
    [aux_sym_preproc_def_token1] = ACTIONS(1192),
    [aux_sym_preproc_if_token1] = ACTIONS(1192),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1192),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1192),
    [sym_preproc_directive] = ACTIONS(1192),
    [anon_sym_LPAREN2] = ACTIONS(1194),
    [anon_sym_BANG] = ACTIONS(1194),
    [anon_sym_TILDE] = ACTIONS(1194),
    [anon_sym_DASH] = ACTIONS(1192),
    [anon_sym_PLUS] = ACTIONS(1192),
    [anon_sym_STAR] = ACTIONS(1194),
    [anon_sym_AMP] = ACTIONS(1194),
    [anon_sym_SEMI] = ACTIONS(1194),
    [anon_sym___extension__] = ACTIONS(1192),
    [anon_sym_typedef] = ACTIONS(1192),
    [anon_sym_extern] = ACTIONS(1192),
    [anon_sym___attribute__] = ACTIONS(1192),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1194),
    [anon_sym___declspec] = ACTIONS(1192),
    [anon_sym___cdecl] = ACTIONS(1192),
    [anon_sym___clrcall] = ACTIONS(1192),
    [anon_sym___stdcall] = ACTIONS(1192),
    [anon_sym___fastcall] = ACTIONS(1192),
    [anon_sym___thiscall] = ACTIONS(1192),
    [anon_sym___vectorcall] = ACTIONS(1192),
    [anon_sym_LBRACE] = ACTIONS(1194),
    [anon_sym_signed] = ACTIONS(1192),
    [anon_sym_unsigned] = ACTIONS(1192),
    [anon_sym_long] = ACTIONS(1192),
    [anon_sym_short] = ACTIONS(1192),
    [anon_sym_static] = ACTIONS(1192),
    [anon_sym_auto] = ACTIONS(1192),
    [anon_sym_register] = ACTIONS(1192),
    [anon_sym_inline] = ACTIONS(1192),
    [anon_sym___inline] = ACTIONS(1192),
    [anon_sym___inline__] = ACTIONS(1192),
    [anon_sym___forceinline] = ACTIONS(1192),
    [anon_sym_thread_local] = ACTIONS(1192),
    [anon_sym___thread] = ACTIONS(1192),
    [anon_sym_const] = ACTIONS(1192),
    [anon_sym_constexpr] = ACTIONS(1192),
    [anon_sym_volatile] = ACTIONS(1192),
    [anon_sym_restrict] = ACTIONS(1192),
    [anon_sym___restrict__] = ACTIONS(1192),
    [anon_sym__Atomic] = ACTIONS(1192),
    [anon_sym__Noreturn] = ACTIONS(1192),
    [anon_sym_noreturn] = ACTIONS(1192),
    [sym_primitive_type] = ACTIONS(1192),
    [anon_sym_enum] = ACTIONS(1192),
    [anon_sym_struct] = ACTIONS(1192),
    [anon_sym_union] = ACTIONS(1192),
    [anon_sym_if] = ACTIONS(1192),
    [anon_sym_else] = ACTIONS(1192),
    [anon_sym_switch] = ACTIONS(1192),
    [anon_sym_case] = ACTIONS(1192),
    [anon_sym_default] = ACTIONS(1192),
    [anon_sym_while] = ACTIONS(1192),
    [anon_sym_do] = ACTIONS(1192),
    [anon_sym_for] = ACTIONS(1192),
    [anon_sym_return] = ACTIONS(1192),
    [anon_sym_break] = ACTIONS(1192),
    [anon_sym_continue] = ACTIONS(1192),
    [anon_sym_goto] = ACTIONS(1192),
    [anon_sym___try] = ACTIONS(1192),
    [anon_sym___leave] = ACTIONS(1192),
    [anon_sym_DASH_DASH] = ACTIONS(1194),
    [anon_sym_PLUS_PLUS] = ACTIONS(1194),
    [anon_sym_sizeof] = ACTIONS(1192),
    [anon_sym___alignof__] = ACTIONS(1192),
    [anon_sym___alignof] = ACTIONS(1192),
    [anon_sym__alignof] = ACTIONS(1192),
    [anon_sym_alignof] = ACTIONS(1192),
    [anon_sym__Alignof] = ACTIONS(1192),
    [anon_sym_offsetof] = ACTIONS(1192),
    [anon_sym__Generic] = ACTIONS(1192),
    [anon_sym_asm] = ACTIONS(1192),
    [anon_sym___asm__] = ACTIONS(1192),
    [sym_number_literal] = ACTIONS(1194),
    [anon_sym_L_SQUOTE] = ACTIONS(1194),
    [anon_sym_u_SQUOTE] = ACTIONS(1194),
    [anon_sym_U_SQUOTE] = ACTIONS(1194),
    [anon_sym_u8_SQUOTE] = ACTIONS(1194),
    [anon_sym_SQUOTE] = ACTIONS(1194),
    [anon_sym_L_DQUOTE] = ACTIONS(1194),
    [anon_sym_u_DQUOTE] = ACTIONS(1194),
    [anon_sym_U_DQUOTE] = ACTIONS(1194),
    [anon_sym_u8_DQUOTE] = ACTIONS(1194),
    [anon_sym_DQUOTE] = ACTIONS(1194),
    [sym_true] = ACTIONS(1192),
    [sym_false] = ACTIONS(1192),
    [anon_sym_NULL] = ACTIONS(1192),
    [anon_sym_nullptr] = ACTIONS(1192),
    [sym_comment] = ACTIONS(3),
  },
  [286] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [287] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [288] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [289] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [290] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [291] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [292] = {
    [ts_builtin_sym_end] = ACTIONS(1190),
    [sym_identifier] = ACTIONS(1188),
    [aux_sym_preproc_include_token1] = ACTIONS(1188),
    [aux_sym_preproc_def_token1] = ACTIONS(1188),
    [aux_sym_preproc_if_token1] = ACTIONS(1188),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1188),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1188),
    [sym_preproc_directive] = ACTIONS(1188),
    [anon_sym_LPAREN2] = ACTIONS(1190),
    [anon_sym_BANG] = ACTIONS(1190),
    [anon_sym_TILDE] = ACTIONS(1190),
    [anon_sym_DASH] = ACTIONS(1188),
    [anon_sym_PLUS] = ACTIONS(1188),
    [anon_sym_STAR] = ACTIONS(1190),
    [anon_sym_AMP] = ACTIONS(1190),
    [anon_sym_SEMI] = ACTIONS(1190),
    [anon_sym___extension__] = ACTIONS(1188),
    [anon_sym_typedef] = ACTIONS(1188),
    [anon_sym_extern] = ACTIONS(1188),
    [anon_sym___attribute__] = ACTIONS(1188),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1190),
    [anon_sym___declspec] = ACTIONS(1188),
    [anon_sym___cdecl] = ACTIONS(1188),
    [anon_sym___clrcall] = ACTIONS(1188),
    [anon_sym___stdcall] = ACTIONS(1188),
    [anon_sym___fastcall] = ACTIONS(1188),
    [anon_sym___thiscall] = ACTIONS(1188),
    [anon_sym___vectorcall] = ACTIONS(1188),
    [anon_sym_LBRACE] = ACTIONS(1190),
    [anon_sym_signed] = ACTIONS(1188),
    [anon_sym_unsigned] = ACTIONS(1188),
    [anon_sym_long] = ACTIONS(1188),
    [anon_sym_short] = ACTIONS(1188),
    [anon_sym_static] = ACTIONS(1188),
    [anon_sym_auto] = ACTIONS(1188),
    [anon_sym_register] = ACTIONS(1188),
    [anon_sym_inline] = ACTIONS(1188),
    [anon_sym___inline] = ACTIONS(1188),
    [anon_sym___inline__] = ACTIONS(1188),
    [anon_sym___forceinline] = ACTIONS(1188),
    [anon_sym_thread_local] = ACTIONS(1188),
    [anon_sym___thread] = ACTIONS(1188),
    [anon_sym_const] = ACTIONS(1188),
    [anon_sym_constexpr] = ACTIONS(1188),
    [anon_sym_volatile] = ACTIONS(1188),
    [anon_sym_restrict] = ACTIONS(1188),
    [anon_sym___restrict__] = ACTIONS(1188),
    [anon_sym__Atomic] = ACTIONS(1188),
    [anon_sym__Noreturn] = ACTIONS(1188),
    [anon_sym_noreturn] = ACTIONS(1188),
    [sym_primitive_type] = ACTIONS(1188),
    [anon_sym_enum] = ACTIONS(1188),
    [anon_sym_struct] = ACTIONS(1188),
    [anon_sym_union] = ACTIONS(1188),
    [anon_sym_if] = ACTIONS(1188),
    [anon_sym_else] = ACTIONS(1188),
    [anon_sym_switch] = ACTIONS(1188),
    [anon_sym_case] = ACTIONS(1188),
    [anon_sym_default] = ACTIONS(1188),
    [anon_sym_while] = ACTIONS(1188),
    [anon_sym_do] = ACTIONS(1188),
    [anon_sym_for] = ACTIONS(1188),
    [anon_sym_return] = ACTIONS(1188),
    [anon_sym_break] = ACTIONS(1188),
    [anon_sym_continue] = ACTIONS(1188),
    [anon_sym_goto] = ACTIONS(1188),
    [anon_sym___try] = ACTIONS(1188),
    [anon_sym___leave] = ACTIONS(1188),
    [anon_sym_DASH_DASH] = ACTIONS(1190),
    [anon_sym_PLUS_PLUS] = ACTIONS(1190),
    [anon_sym_sizeof] = ACTIONS(1188),
    [anon_sym___alignof__] = ACTIONS(1188),
    [anon_sym___alignof] = ACTIONS(1188),
    [anon_sym__alignof] = ACTIONS(1188),
    [anon_sym_alignof] = ACTIONS(1188),
    [anon_sym__Alignof] = ACTIONS(1188),
    [anon_sym_offsetof] = ACTIONS(1188),
    [anon_sym__Generic] = ACTIONS(1188),
    [anon_sym_asm] = ACTIONS(1188),
    [anon_sym___asm__] = ACTIONS(1188),
    [sym_number_literal] = ACTIONS(1190),
    [anon_sym_L_SQUOTE] = ACTIONS(1190),
    [anon_sym_u_SQUOTE] = ACTIONS(1190),
    [anon_sym_U_SQUOTE] = ACTIONS(1190),
    [anon_sym_u8_SQUOTE] = ACTIONS(1190),
    [anon_sym_SQUOTE] = ACTIONS(1190),
    [anon_sym_L_DQUOTE] = ACTIONS(1190),
    [anon_sym_u_DQUOTE] = ACTIONS(1190),
    [anon_sym_U_DQUOTE] = ACTIONS(1190),
    [anon_sym_u8_DQUOTE] = ACTIONS(1190),
    [anon_sym_DQUOTE] = ACTIONS(1190),
    [sym_true] = ACTIONS(1188),
    [sym_false] = ACTIONS(1188),
    [anon_sym_NULL] = ACTIONS(1188),
    [anon_sym_nullptr] = ACTIONS(1188),
    [sym_comment] = ACTIONS(3),
  },
  [293] = {
    [ts_builtin_sym_end] = ACTIONS(1186),
    [sym_identifier] = ACTIONS(1184),
    [aux_sym_preproc_include_token1] = ACTIONS(1184),
    [aux_sym_preproc_def_token1] = ACTIONS(1184),
    [aux_sym_preproc_if_token1] = ACTIONS(1184),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1184),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1184),
    [sym_preproc_directive] = ACTIONS(1184),
    [anon_sym_LPAREN2] = ACTIONS(1186),
    [anon_sym_BANG] = ACTIONS(1186),
    [anon_sym_TILDE] = ACTIONS(1186),
    [anon_sym_DASH] = ACTIONS(1184),
    [anon_sym_PLUS] = ACTIONS(1184),
    [anon_sym_STAR] = ACTIONS(1186),
    [anon_sym_AMP] = ACTIONS(1186),
    [anon_sym_SEMI] = ACTIONS(1186),
    [anon_sym___extension__] = ACTIONS(1184),
    [anon_sym_typedef] = ACTIONS(1184),
    [anon_sym_extern] = ACTIONS(1184),
    [anon_sym___attribute__] = ACTIONS(1184),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1186),
    [anon_sym___declspec] = ACTIONS(1184),
    [anon_sym___cdecl] = ACTIONS(1184),
    [anon_sym___clrcall] = ACTIONS(1184),
    [anon_sym___stdcall] = ACTIONS(1184),
    [anon_sym___fastcall] = ACTIONS(1184),
    [anon_sym___thiscall] = ACTIONS(1184),
    [anon_sym___vectorcall] = ACTIONS(1184),
    [anon_sym_LBRACE] = ACTIONS(1186),
    [anon_sym_signed] = ACTIONS(1184),
    [anon_sym_unsigned] = ACTIONS(1184),
    [anon_sym_long] = ACTIONS(1184),
    [anon_sym_short] = ACTIONS(1184),
    [anon_sym_static] = ACTIONS(1184),
    [anon_sym_auto] = ACTIONS(1184),
    [anon_sym_register] = ACTIONS(1184),
    [anon_sym_inline] = ACTIONS(1184),
    [anon_sym___inline] = ACTIONS(1184),
    [anon_sym___inline__] = ACTIONS(1184),
    [anon_sym___forceinline] = ACTIONS(1184),
    [anon_sym_thread_local] = ACTIONS(1184),
    [anon_sym___thread] = ACTIONS(1184),
    [anon_sym_const] = ACTIONS(1184),
    [anon_sym_constexpr] = ACTIONS(1184),
    [anon_sym_volatile] = ACTIONS(1184),
    [anon_sym_restrict] = ACTIONS(1184),
    [anon_sym___restrict__] = ACTIONS(1184),
    [anon_sym__Atomic] = ACTIONS(1184),
    [anon_sym__Noreturn] = ACTIONS(1184),
    [anon_sym_noreturn] = ACTIONS(1184),
    [sym_primitive_type] = ACTIONS(1184),
    [anon_sym_enum] = ACTIONS(1184),
    [anon_sym_struct] = ACTIONS(1184),
    [anon_sym_union] = ACTIONS(1184),
    [anon_sym_if] = ACTIONS(1184),
    [anon_sym_else] = ACTIONS(1184),
    [anon_sym_switch] = ACTIONS(1184),
    [anon_sym_case] = ACTIONS(1184),
    [anon_sym_default] = ACTIONS(1184),
    [anon_sym_while] = ACTIONS(1184),
    [anon_sym_do] = ACTIONS(1184),
    [anon_sym_for] = ACTIONS(1184),
    [anon_sym_return] = ACTIONS(1184),
    [anon_sym_break] = ACTIONS(1184),
    [anon_sym_continue] = ACTIONS(1184),
    [anon_sym_goto] = ACTIONS(1184),
    [anon_sym___try] = ACTIONS(1184),
    [anon_sym___leave] = ACTIONS(1184),
    [anon_sym_DASH_DASH] = ACTIONS(1186),
    [anon_sym_PLUS_PLUS] = ACTIONS(1186),
    [anon_sym_sizeof] = ACTIONS(1184),
    [anon_sym___alignof__] = ACTIONS(1184),
    [anon_sym___alignof] = ACTIONS(1184),
    [anon_sym__alignof] = ACTIONS(1184),
    [anon_sym_alignof] = ACTIONS(1184),
    [anon_sym__Alignof] = ACTIONS(1184),
    [anon_sym_offsetof] = ACTIONS(1184),
    [anon_sym__Generic] = ACTIONS(1184),
    [anon_sym_asm] = ACTIONS(1184),
    [anon_sym___asm__] = ACTIONS(1184),
    [sym_number_literal] = ACTIONS(1186),
    [anon_sym_L_SQUOTE] = ACTIONS(1186),
    [anon_sym_u_SQUOTE] = ACTIONS(1186),
    [anon_sym_U_SQUOTE] = ACTIONS(1186),
    [anon_sym_u8_SQUOTE] = ACTIONS(1186),
    [anon_sym_SQUOTE] = ACTIONS(1186),
    [anon_sym_L_DQUOTE] = ACTIONS(1186),
    [anon_sym_u_DQUOTE] = ACTIONS(1186),
    [anon_sym_U_DQUOTE] = ACTIONS(1186),
    [anon_sym_u8_DQUOTE] = ACTIONS(1186),
    [anon_sym_DQUOTE] = ACTIONS(1186),
    [sym_true] = ACTIONS(1184),
    [sym_false] = ACTIONS(1184),
    [anon_sym_NULL] = ACTIONS(1184),
    [anon_sym_nullptr] = ACTIONS(1184),
    [sym_comment] = ACTIONS(3),
  },
  [294] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [295] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [296] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token2] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [297] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [298] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [299] = {
    [sym_identifier] = ACTIONS(1116),
    [aux_sym_preproc_include_token1] = ACTIONS(1116),
    [aux_sym_preproc_def_token1] = ACTIONS(1116),
    [aux_sym_preproc_if_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1116),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1116),
    [sym_preproc_directive] = ACTIONS(1116),
    [anon_sym_LPAREN2] = ACTIONS(1118),
    [anon_sym_BANG] = ACTIONS(1118),
    [anon_sym_TILDE] = ACTIONS(1118),
    [anon_sym_DASH] = ACTIONS(1116),
    [anon_sym_PLUS] = ACTIONS(1116),
    [anon_sym_STAR] = ACTIONS(1118),
    [anon_sym_AMP] = ACTIONS(1118),
    [anon_sym_SEMI] = ACTIONS(1118),
    [anon_sym___extension__] = ACTIONS(1116),
    [anon_sym_typedef] = ACTIONS(1116),
    [anon_sym_extern] = ACTIONS(1116),
    [anon_sym___attribute__] = ACTIONS(1116),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1118),
    [anon_sym___declspec] = ACTIONS(1116),
    [anon_sym___cdecl] = ACTIONS(1116),
    [anon_sym___clrcall] = ACTIONS(1116),
    [anon_sym___stdcall] = ACTIONS(1116),
    [anon_sym___fastcall] = ACTIONS(1116),
    [anon_sym___thiscall] = ACTIONS(1116),
    [anon_sym___vectorcall] = ACTIONS(1116),
    [anon_sym_LBRACE] = ACTIONS(1118),
    [anon_sym_RBRACE] = ACTIONS(1118),
    [anon_sym_signed] = ACTIONS(1116),
    [anon_sym_unsigned] = ACTIONS(1116),
    [anon_sym_long] = ACTIONS(1116),
    [anon_sym_short] = ACTIONS(1116),
    [anon_sym_static] = ACTIONS(1116),
    [anon_sym_auto] = ACTIONS(1116),
    [anon_sym_register] = ACTIONS(1116),
    [anon_sym_inline] = ACTIONS(1116),
    [anon_sym___inline] = ACTIONS(1116),
    [anon_sym___inline__] = ACTIONS(1116),
    [anon_sym___forceinline] = ACTIONS(1116),
    [anon_sym_thread_local] = ACTIONS(1116),
    [anon_sym___thread] = ACTIONS(1116),
    [anon_sym_const] = ACTIONS(1116),
    [anon_sym_constexpr] = ACTIONS(1116),
    [anon_sym_volatile] = ACTIONS(1116),
    [anon_sym_restrict] = ACTIONS(1116),
    [anon_sym___restrict__] = ACTIONS(1116),
    [anon_sym__Atomic] = ACTIONS(1116),
    [anon_sym__Noreturn] = ACTIONS(1116),
    [anon_sym_noreturn] = ACTIONS(1116),
    [sym_primitive_type] = ACTIONS(1116),
    [anon_sym_enum] = ACTIONS(1116),
    [anon_sym_struct] = ACTIONS(1116),
    [anon_sym_union] = ACTIONS(1116),
    [anon_sym_if] = ACTIONS(1116),
    [anon_sym_else] = ACTIONS(1116),
    [anon_sym_switch] = ACTIONS(1116),
    [anon_sym_case] = ACTIONS(1116),
    [anon_sym_default] = ACTIONS(1116),
    [anon_sym_while] = ACTIONS(1116),
    [anon_sym_do] = ACTIONS(1116),
    [anon_sym_for] = ACTIONS(1116),
    [anon_sym_return] = ACTIONS(1116),
    [anon_sym_break] = ACTIONS(1116),
    [anon_sym_continue] = ACTIONS(1116),
    [anon_sym_goto] = ACTIONS(1116),
    [anon_sym___try] = ACTIONS(1116),
    [anon_sym___leave] = ACTIONS(1116),
    [anon_sym_DASH_DASH] = ACTIONS(1118),
    [anon_sym_PLUS_PLUS] = ACTIONS(1118),
    [anon_sym_sizeof] = ACTIONS(1116),
    [anon_sym___alignof__] = ACTIONS(1116),
    [anon_sym___alignof] = ACTIONS(1116),
    [anon_sym__alignof] = ACTIONS(1116),
    [anon_sym_alignof] = ACTIONS(1116),
    [anon_sym__Alignof] = ACTIONS(1116),
    [anon_sym_offsetof] = ACTIONS(1116),
    [anon_sym__Generic] = ACTIONS(1116),
    [anon_sym_asm] = ACTIONS(1116),
    [anon_sym___asm__] = ACTIONS(1116),
    [sym_number_literal] = ACTIONS(1118),
    [anon_sym_L_SQUOTE] = ACTIONS(1118),
    [anon_sym_u_SQUOTE] = ACTIONS(1118),
    [anon_sym_U_SQUOTE] = ACTIONS(1118),
    [anon_sym_u8_SQUOTE] = ACTIONS(1118),
    [anon_sym_SQUOTE] = ACTIONS(1118),
    [anon_sym_L_DQUOTE] = ACTIONS(1118),
    [anon_sym_u_DQUOTE] = ACTIONS(1118),
    [anon_sym_U_DQUOTE] = ACTIONS(1118),
    [anon_sym_u8_DQUOTE] = ACTIONS(1118),
    [anon_sym_DQUOTE] = ACTIONS(1118),
    [sym_true] = ACTIONS(1116),
    [sym_false] = ACTIONS(1116),
    [anon_sym_NULL] = ACTIONS(1116),
    [anon_sym_nullptr] = ACTIONS(1116),
    [sym_comment] = ACTIONS(3),
  },
  [300] = {
    [sym_identifier] = ACTIONS(1172),
    [aux_sym_preproc_include_token1] = ACTIONS(1172),
    [aux_sym_preproc_def_token1] = ACTIONS(1172),
    [aux_sym_preproc_if_token1] = ACTIONS(1172),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1172),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1172),
    [sym_preproc_directive] = ACTIONS(1172),
    [anon_sym_LPAREN2] = ACTIONS(1174),
    [anon_sym_BANG] = ACTIONS(1174),
    [anon_sym_TILDE] = ACTIONS(1174),
    [anon_sym_DASH] = ACTIONS(1172),
    [anon_sym_PLUS] = ACTIONS(1172),
    [anon_sym_STAR] = ACTIONS(1174),
    [anon_sym_AMP] = ACTIONS(1174),
    [anon_sym_SEMI] = ACTIONS(1174),
    [anon_sym___extension__] = ACTIONS(1172),
    [anon_sym_typedef] = ACTIONS(1172),
    [anon_sym_extern] = ACTIONS(1172),
    [anon_sym___attribute__] = ACTIONS(1172),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1174),
    [anon_sym___declspec] = ACTIONS(1172),
    [anon_sym___cdecl] = ACTIONS(1172),
    [anon_sym___clrcall] = ACTIONS(1172),
    [anon_sym___stdcall] = ACTIONS(1172),
    [anon_sym___fastcall] = ACTIONS(1172),
    [anon_sym___thiscall] = ACTIONS(1172),
    [anon_sym___vectorcall] = ACTIONS(1172),
    [anon_sym_LBRACE] = ACTIONS(1174),
    [anon_sym_RBRACE] = ACTIONS(1174),
    [anon_sym_signed] = ACTIONS(1172),
    [anon_sym_unsigned] = ACTIONS(1172),
    [anon_sym_long] = ACTIONS(1172),
    [anon_sym_short] = ACTIONS(1172),
    [anon_sym_static] = ACTIONS(1172),
    [anon_sym_auto] = ACTIONS(1172),
    [anon_sym_register] = ACTIONS(1172),
    [anon_sym_inline] = ACTIONS(1172),
    [anon_sym___inline] = ACTIONS(1172),
    [anon_sym___inline__] = ACTIONS(1172),
    [anon_sym___forceinline] = ACTIONS(1172),
    [anon_sym_thread_local] = ACTIONS(1172),
    [anon_sym___thread] = ACTIONS(1172),
    [anon_sym_const] = ACTIONS(1172),
    [anon_sym_constexpr] = ACTIONS(1172),
    [anon_sym_volatile] = ACTIONS(1172),
    [anon_sym_restrict] = ACTIONS(1172),
    [anon_sym___restrict__] = ACTIONS(1172),
    [anon_sym__Atomic] = ACTIONS(1172),
    [anon_sym__Noreturn] = ACTIONS(1172),
    [anon_sym_noreturn] = ACTIONS(1172),
    [sym_primitive_type] = ACTIONS(1172),
    [anon_sym_enum] = ACTIONS(1172),
    [anon_sym_struct] = ACTIONS(1172),
    [anon_sym_union] = ACTIONS(1172),
    [anon_sym_if] = ACTIONS(1172),
    [anon_sym_else] = ACTIONS(1172),
    [anon_sym_switch] = ACTIONS(1172),
    [anon_sym_case] = ACTIONS(1172),
    [anon_sym_default] = ACTIONS(1172),
    [anon_sym_while] = ACTIONS(1172),
    [anon_sym_do] = ACTIONS(1172),
    [anon_sym_for] = ACTIONS(1172),
    [anon_sym_return] = ACTIONS(1172),
    [anon_sym_break] = ACTIONS(1172),
    [anon_sym_continue] = ACTIONS(1172),
    [anon_sym_goto] = ACTIONS(1172),
    [anon_sym___try] = ACTIONS(1172),
    [anon_sym___leave] = ACTIONS(1172),
    [anon_sym_DASH_DASH] = ACTIONS(1174),
    [anon_sym_PLUS_PLUS] = ACTIONS(1174),
    [anon_sym_sizeof] = ACTIONS(1172),
    [anon_sym___alignof__] = ACTIONS(1172),
    [anon_sym___alignof] = ACTIONS(1172),
    [anon_sym__alignof] = ACTIONS(1172),
    [anon_sym_alignof] = ACTIONS(1172),
    [anon_sym__Alignof] = ACTIONS(1172),
    [anon_sym_offsetof] = ACTIONS(1172),
    [anon_sym__Generic] = ACTIONS(1172),
    [anon_sym_asm] = ACTIONS(1172),
    [anon_sym___asm__] = ACTIONS(1172),
    [sym_number_literal] = ACTIONS(1174),
    [anon_sym_L_SQUOTE] = ACTIONS(1174),
    [anon_sym_u_SQUOTE] = ACTIONS(1174),
    [anon_sym_U_SQUOTE] = ACTIONS(1174),
    [anon_sym_u8_SQUOTE] = ACTIONS(1174),
    [anon_sym_SQUOTE] = ACTIONS(1174),
    [anon_sym_L_DQUOTE] = ACTIONS(1174),
    [anon_sym_u_DQUOTE] = ACTIONS(1174),
    [anon_sym_U_DQUOTE] = ACTIONS(1174),
    [anon_sym_u8_DQUOTE] = ACTIONS(1174),
    [anon_sym_DQUOTE] = ACTIONS(1174),
    [sym_true] = ACTIONS(1172),
    [sym_false] = ACTIONS(1172),
    [anon_sym_NULL] = ACTIONS(1172),
    [anon_sym_nullptr] = ACTIONS(1172),
    [sym_comment] = ACTIONS(3),
  },
  [301] = {
    [sym_identifier] = ACTIONS(1168),
    [aux_sym_preproc_include_token1] = ACTIONS(1168),
    [aux_sym_preproc_def_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1168),
    [sym_preproc_directive] = ACTIONS(1168),
    [anon_sym_LPAREN2] = ACTIONS(1170),
    [anon_sym_BANG] = ACTIONS(1170),
    [anon_sym_TILDE] = ACTIONS(1170),
    [anon_sym_DASH] = ACTIONS(1168),
    [anon_sym_PLUS] = ACTIONS(1168),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1170),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym___extension__] = ACTIONS(1168),
    [anon_sym_typedef] = ACTIONS(1168),
    [anon_sym_extern] = ACTIONS(1168),
    [anon_sym___attribute__] = ACTIONS(1168),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1170),
    [anon_sym___declspec] = ACTIONS(1168),
    [anon_sym___cdecl] = ACTIONS(1168),
    [anon_sym___clrcall] = ACTIONS(1168),
    [anon_sym___stdcall] = ACTIONS(1168),
    [anon_sym___fastcall] = ACTIONS(1168),
    [anon_sym___thiscall] = ACTIONS(1168),
    [anon_sym___vectorcall] = ACTIONS(1168),
    [anon_sym_LBRACE] = ACTIONS(1170),
    [anon_sym_RBRACE] = ACTIONS(1170),
    [anon_sym_signed] = ACTIONS(1168),
    [anon_sym_unsigned] = ACTIONS(1168),
    [anon_sym_long] = ACTIONS(1168),
    [anon_sym_short] = ACTIONS(1168),
    [anon_sym_static] = ACTIONS(1168),
    [anon_sym_auto] = ACTIONS(1168),
    [anon_sym_register] = ACTIONS(1168),
    [anon_sym_inline] = ACTIONS(1168),
    [anon_sym___inline] = ACTIONS(1168),
    [anon_sym___inline__] = ACTIONS(1168),
    [anon_sym___forceinline] = ACTIONS(1168),
    [anon_sym_thread_local] = ACTIONS(1168),
    [anon_sym___thread] = ACTIONS(1168),
    [anon_sym_const] = ACTIONS(1168),
    [anon_sym_constexpr] = ACTIONS(1168),
    [anon_sym_volatile] = ACTIONS(1168),
    [anon_sym_restrict] = ACTIONS(1168),
    [anon_sym___restrict__] = ACTIONS(1168),
    [anon_sym__Atomic] = ACTIONS(1168),
    [anon_sym__Noreturn] = ACTIONS(1168),
    [anon_sym_noreturn] = ACTIONS(1168),
    [sym_primitive_type] = ACTIONS(1168),
    [anon_sym_enum] = ACTIONS(1168),
    [anon_sym_struct] = ACTIONS(1168),
    [anon_sym_union] = ACTIONS(1168),
    [anon_sym_if] = ACTIONS(1168),
    [anon_sym_else] = ACTIONS(1168),
    [anon_sym_switch] = ACTIONS(1168),
    [anon_sym_case] = ACTIONS(1168),
    [anon_sym_default] = ACTIONS(1168),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_do] = ACTIONS(1168),
    [anon_sym_for] = ACTIONS(1168),
    [anon_sym_return] = ACTIONS(1168),
    [anon_sym_break] = ACTIONS(1168),
    [anon_sym_continue] = ACTIONS(1168),
    [anon_sym_goto] = ACTIONS(1168),
    [anon_sym___try] = ACTIONS(1168),
    [anon_sym___leave] = ACTIONS(1168),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_sizeof] = ACTIONS(1168),
    [anon_sym___alignof__] = ACTIONS(1168),
    [anon_sym___alignof] = ACTIONS(1168),
    [anon_sym__alignof] = ACTIONS(1168),
    [anon_sym_alignof] = ACTIONS(1168),
    [anon_sym__Alignof] = ACTIONS(1168),
    [anon_sym_offsetof] = ACTIONS(1168),
    [anon_sym__Generic] = ACTIONS(1168),
    [anon_sym_asm] = ACTIONS(1168),
    [anon_sym___asm__] = ACTIONS(1168),
    [sym_number_literal] = ACTIONS(1170),
    [anon_sym_L_SQUOTE] = ACTIONS(1170),
    [anon_sym_u_SQUOTE] = ACTIONS(1170),
    [anon_sym_U_SQUOTE] = ACTIONS(1170),
    [anon_sym_u8_SQUOTE] = ACTIONS(1170),
    [anon_sym_SQUOTE] = ACTIONS(1170),
    [anon_sym_L_DQUOTE] = ACTIONS(1170),
    [anon_sym_u_DQUOTE] = ACTIONS(1170),
    [anon_sym_U_DQUOTE] = ACTIONS(1170),
    [anon_sym_u8_DQUOTE] = ACTIONS(1170),
    [anon_sym_DQUOTE] = ACTIONS(1170),
    [sym_true] = ACTIONS(1168),
    [sym_false] = ACTIONS(1168),
    [anon_sym_NULL] = ACTIONS(1168),
    [anon_sym_nullptr] = ACTIONS(1168),
    [sym_comment] = ACTIONS(3),
  },
  [302] = {
    [sym_identifier] = ACTIONS(1168),
    [aux_sym_preproc_include_token1] = ACTIONS(1168),
    [aux_sym_preproc_def_token1] = ACTIONS(1168),
    [aux_sym_preproc_if_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1168),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1168),
    [sym_preproc_directive] = ACTIONS(1168),
    [anon_sym_LPAREN2] = ACTIONS(1170),
    [anon_sym_BANG] = ACTIONS(1170),
    [anon_sym_TILDE] = ACTIONS(1170),
    [anon_sym_DASH] = ACTIONS(1168),
    [anon_sym_PLUS] = ACTIONS(1168),
    [anon_sym_STAR] = ACTIONS(1170),
    [anon_sym_AMP] = ACTIONS(1170),
    [anon_sym_SEMI] = ACTIONS(1170),
    [anon_sym___extension__] = ACTIONS(1168),
    [anon_sym_typedef] = ACTIONS(1168),
    [anon_sym_extern] = ACTIONS(1168),
    [anon_sym___attribute__] = ACTIONS(1168),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1170),
    [anon_sym___declspec] = ACTIONS(1168),
    [anon_sym___cdecl] = ACTIONS(1168),
    [anon_sym___clrcall] = ACTIONS(1168),
    [anon_sym___stdcall] = ACTIONS(1168),
    [anon_sym___fastcall] = ACTIONS(1168),
    [anon_sym___thiscall] = ACTIONS(1168),
    [anon_sym___vectorcall] = ACTIONS(1168),
    [anon_sym_LBRACE] = ACTIONS(1170),
    [anon_sym_RBRACE] = ACTIONS(1170),
    [anon_sym_signed] = ACTIONS(1168),
    [anon_sym_unsigned] = ACTIONS(1168),
    [anon_sym_long] = ACTIONS(1168),
    [anon_sym_short] = ACTIONS(1168),
    [anon_sym_static] = ACTIONS(1168),
    [anon_sym_auto] = ACTIONS(1168),
    [anon_sym_register] = ACTIONS(1168),
    [anon_sym_inline] = ACTIONS(1168),
    [anon_sym___inline] = ACTIONS(1168),
    [anon_sym___inline__] = ACTIONS(1168),
    [anon_sym___forceinline] = ACTIONS(1168),
    [anon_sym_thread_local] = ACTIONS(1168),
    [anon_sym___thread] = ACTIONS(1168),
    [anon_sym_const] = ACTIONS(1168),
    [anon_sym_constexpr] = ACTIONS(1168),
    [anon_sym_volatile] = ACTIONS(1168),
    [anon_sym_restrict] = ACTIONS(1168),
    [anon_sym___restrict__] = ACTIONS(1168),
    [anon_sym__Atomic] = ACTIONS(1168),
    [anon_sym__Noreturn] = ACTIONS(1168),
    [anon_sym_noreturn] = ACTIONS(1168),
    [sym_primitive_type] = ACTIONS(1168),
    [anon_sym_enum] = ACTIONS(1168),
    [anon_sym_struct] = ACTIONS(1168),
    [anon_sym_union] = ACTIONS(1168),
    [anon_sym_if] = ACTIONS(1168),
    [anon_sym_else] = ACTIONS(1168),
    [anon_sym_switch] = ACTIONS(1168),
    [anon_sym_case] = ACTIONS(1168),
    [anon_sym_default] = ACTIONS(1168),
    [anon_sym_while] = ACTIONS(1168),
    [anon_sym_do] = ACTIONS(1168),
    [anon_sym_for] = ACTIONS(1168),
    [anon_sym_return] = ACTIONS(1168),
    [anon_sym_break] = ACTIONS(1168),
    [anon_sym_continue] = ACTIONS(1168),
    [anon_sym_goto] = ACTIONS(1168),
    [anon_sym___try] = ACTIONS(1168),
    [anon_sym___leave] = ACTIONS(1168),
    [anon_sym_DASH_DASH] = ACTIONS(1170),
    [anon_sym_PLUS_PLUS] = ACTIONS(1170),
    [anon_sym_sizeof] = ACTIONS(1168),
    [anon_sym___alignof__] = ACTIONS(1168),
    [anon_sym___alignof] = ACTIONS(1168),
    [anon_sym__alignof] = ACTIONS(1168),
    [anon_sym_alignof] = ACTIONS(1168),
    [anon_sym__Alignof] = ACTIONS(1168),
    [anon_sym_offsetof] = ACTIONS(1168),
    [anon_sym__Generic] = ACTIONS(1168),
    [anon_sym_asm] = ACTIONS(1168),
    [anon_sym___asm__] = ACTIONS(1168),
    [sym_number_literal] = ACTIONS(1170),
    [anon_sym_L_SQUOTE] = ACTIONS(1170),
    [anon_sym_u_SQUOTE] = ACTIONS(1170),
    [anon_sym_U_SQUOTE] = ACTIONS(1170),
    [anon_sym_u8_SQUOTE] = ACTIONS(1170),
    [anon_sym_SQUOTE] = ACTIONS(1170),
    [anon_sym_L_DQUOTE] = ACTIONS(1170),
    [anon_sym_u_DQUOTE] = ACTIONS(1170),
    [anon_sym_U_DQUOTE] = ACTIONS(1170),
    [anon_sym_u8_DQUOTE] = ACTIONS(1170),
    [anon_sym_DQUOTE] = ACTIONS(1170),
    [sym_true] = ACTIONS(1168),
    [sym_false] = ACTIONS(1168),
    [anon_sym_NULL] = ACTIONS(1168),
    [anon_sym_nullptr] = ACTIONS(1168),
    [sym_comment] = ACTIONS(3),
  },
  [303] = {
    [sym_identifier] = ACTIONS(1164),
    [aux_sym_preproc_include_token1] = ACTIONS(1164),
    [aux_sym_preproc_def_token1] = ACTIONS(1164),
    [aux_sym_preproc_if_token1] = ACTIONS(1164),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1164),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1164),
    [sym_preproc_directive] = ACTIONS(1164),
    [anon_sym_LPAREN2] = ACTIONS(1166),
    [anon_sym_BANG] = ACTIONS(1166),
    [anon_sym_TILDE] = ACTIONS(1166),
    [anon_sym_DASH] = ACTIONS(1164),
    [anon_sym_PLUS] = ACTIONS(1164),
    [anon_sym_STAR] = ACTIONS(1166),
    [anon_sym_AMP] = ACTIONS(1166),
    [anon_sym_SEMI] = ACTIONS(1166),
    [anon_sym___extension__] = ACTIONS(1164),
    [anon_sym_typedef] = ACTIONS(1164),
    [anon_sym_extern] = ACTIONS(1164),
    [anon_sym___attribute__] = ACTIONS(1164),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1166),
    [anon_sym___declspec] = ACTIONS(1164),
    [anon_sym___cdecl] = ACTIONS(1164),
    [anon_sym___clrcall] = ACTIONS(1164),
    [anon_sym___stdcall] = ACTIONS(1164),
    [anon_sym___fastcall] = ACTIONS(1164),
    [anon_sym___thiscall] = ACTIONS(1164),
    [anon_sym___vectorcall] = ACTIONS(1164),
    [anon_sym_LBRACE] = ACTIONS(1166),
    [anon_sym_RBRACE] = ACTIONS(1166),
    [anon_sym_signed] = ACTIONS(1164),
    [anon_sym_unsigned] = ACTIONS(1164),
    [anon_sym_long] = ACTIONS(1164),
    [anon_sym_short] = ACTIONS(1164),
    [anon_sym_static] = ACTIONS(1164),
    [anon_sym_auto] = ACTIONS(1164),
    [anon_sym_register] = ACTIONS(1164),
    [anon_sym_inline] = ACTIONS(1164),
    [anon_sym___inline] = ACTIONS(1164),
    [anon_sym___inline__] = ACTIONS(1164),
    [anon_sym___forceinline] = ACTIONS(1164),
    [anon_sym_thread_local] = ACTIONS(1164),
    [anon_sym___thread] = ACTIONS(1164),
    [anon_sym_const] = ACTIONS(1164),
    [anon_sym_constexpr] = ACTIONS(1164),
    [anon_sym_volatile] = ACTIONS(1164),
    [anon_sym_restrict] = ACTIONS(1164),
    [anon_sym___restrict__] = ACTIONS(1164),
    [anon_sym__Atomic] = ACTIONS(1164),
    [anon_sym__Noreturn] = ACTIONS(1164),
    [anon_sym_noreturn] = ACTIONS(1164),
    [sym_primitive_type] = ACTIONS(1164),
    [anon_sym_enum] = ACTIONS(1164),
    [anon_sym_struct] = ACTIONS(1164),
    [anon_sym_union] = ACTIONS(1164),
    [anon_sym_if] = ACTIONS(1164),
    [anon_sym_else] = ACTIONS(1164),
    [anon_sym_switch] = ACTIONS(1164),
    [anon_sym_case] = ACTIONS(1164),
    [anon_sym_default] = ACTIONS(1164),
    [anon_sym_while] = ACTIONS(1164),
    [anon_sym_do] = ACTIONS(1164),
    [anon_sym_for] = ACTIONS(1164),
    [anon_sym_return] = ACTIONS(1164),
    [anon_sym_break] = ACTIONS(1164),
    [anon_sym_continue] = ACTIONS(1164),
    [anon_sym_goto] = ACTIONS(1164),
    [anon_sym___try] = ACTIONS(1164),
    [anon_sym___leave] = ACTIONS(1164),
    [anon_sym_DASH_DASH] = ACTIONS(1166),
    [anon_sym_PLUS_PLUS] = ACTIONS(1166),
    [anon_sym_sizeof] = ACTIONS(1164),
    [anon_sym___alignof__] = ACTIONS(1164),
    [anon_sym___alignof] = ACTIONS(1164),
    [anon_sym__alignof] = ACTIONS(1164),
    [anon_sym_alignof] = ACTIONS(1164),
    [anon_sym__Alignof] = ACTIONS(1164),
    [anon_sym_offsetof] = ACTIONS(1164),
    [anon_sym__Generic] = ACTIONS(1164),
    [anon_sym_asm] = ACTIONS(1164),
    [anon_sym___asm__] = ACTIONS(1164),
    [sym_number_literal] = ACTIONS(1166),
    [anon_sym_L_SQUOTE] = ACTIONS(1166),
    [anon_sym_u_SQUOTE] = ACTIONS(1166),
    [anon_sym_U_SQUOTE] = ACTIONS(1166),
    [anon_sym_u8_SQUOTE] = ACTIONS(1166),
    [anon_sym_SQUOTE] = ACTIONS(1166),
    [anon_sym_L_DQUOTE] = ACTIONS(1166),
    [anon_sym_u_DQUOTE] = ACTIONS(1166),
    [anon_sym_U_DQUOTE] = ACTIONS(1166),
    [anon_sym_u8_DQUOTE] = ACTIONS(1166),
    [anon_sym_DQUOTE] = ACTIONS(1166),
    [sym_true] = ACTIONS(1164),
    [sym_false] = ACTIONS(1164),
    [anon_sym_NULL] = ACTIONS(1164),
    [anon_sym_nullptr] = ACTIONS(1164),
    [sym_comment] = ACTIONS(3),
  },
  [304] = {
    [ts_builtin_sym_end] = ACTIONS(1162),
    [sym_identifier] = ACTIONS(1160),
    [aux_sym_preproc_include_token1] = ACTIONS(1160),
    [aux_sym_preproc_def_token1] = ACTIONS(1160),
    [aux_sym_preproc_if_token1] = ACTIONS(1160),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1160),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1160),
    [sym_preproc_directive] = ACTIONS(1160),
    [anon_sym_LPAREN2] = ACTIONS(1162),
    [anon_sym_BANG] = ACTIONS(1162),
    [anon_sym_TILDE] = ACTIONS(1162),
    [anon_sym_DASH] = ACTIONS(1160),
    [anon_sym_PLUS] = ACTIONS(1160),
    [anon_sym_STAR] = ACTIONS(1162),
    [anon_sym_AMP] = ACTIONS(1162),
    [anon_sym_SEMI] = ACTIONS(1162),
    [anon_sym___extension__] = ACTIONS(1160),
    [anon_sym_typedef] = ACTIONS(1160),
    [anon_sym_extern] = ACTIONS(1160),
    [anon_sym___attribute__] = ACTIONS(1160),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1162),
    [anon_sym___declspec] = ACTIONS(1160),
    [anon_sym___cdecl] = ACTIONS(1160),
    [anon_sym___clrcall] = ACTIONS(1160),
    [anon_sym___stdcall] = ACTIONS(1160),
    [anon_sym___fastcall] = ACTIONS(1160),
    [anon_sym___thiscall] = ACTIONS(1160),
    [anon_sym___vectorcall] = ACTIONS(1160),
    [anon_sym_LBRACE] = ACTIONS(1162),
    [anon_sym_signed] = ACTIONS(1160),
    [anon_sym_unsigned] = ACTIONS(1160),
    [anon_sym_long] = ACTIONS(1160),
    [anon_sym_short] = ACTIONS(1160),
    [anon_sym_static] = ACTIONS(1160),
    [anon_sym_auto] = ACTIONS(1160),
    [anon_sym_register] = ACTIONS(1160),
    [anon_sym_inline] = ACTIONS(1160),
    [anon_sym___inline] = ACTIONS(1160),
    [anon_sym___inline__] = ACTIONS(1160),
    [anon_sym___forceinline] = ACTIONS(1160),
    [anon_sym_thread_local] = ACTIONS(1160),
    [anon_sym___thread] = ACTIONS(1160),
    [anon_sym_const] = ACTIONS(1160),
    [anon_sym_constexpr] = ACTIONS(1160),
    [anon_sym_volatile] = ACTIONS(1160),
    [anon_sym_restrict] = ACTIONS(1160),
    [anon_sym___restrict__] = ACTIONS(1160),
    [anon_sym__Atomic] = ACTIONS(1160),
    [anon_sym__Noreturn] = ACTIONS(1160),
    [anon_sym_noreturn] = ACTIONS(1160),
    [sym_primitive_type] = ACTIONS(1160),
    [anon_sym_enum] = ACTIONS(1160),
    [anon_sym_struct] = ACTIONS(1160),
    [anon_sym_union] = ACTIONS(1160),
    [anon_sym_if] = ACTIONS(1160),
    [anon_sym_else] = ACTIONS(1160),
    [anon_sym_switch] = ACTIONS(1160),
    [anon_sym_case] = ACTIONS(1160),
    [anon_sym_default] = ACTIONS(1160),
    [anon_sym_while] = ACTIONS(1160),
    [anon_sym_do] = ACTIONS(1160),
    [anon_sym_for] = ACTIONS(1160),
    [anon_sym_return] = ACTIONS(1160),
    [anon_sym_break] = ACTIONS(1160),
    [anon_sym_continue] = ACTIONS(1160),
    [anon_sym_goto] = ACTIONS(1160),
    [anon_sym___try] = ACTIONS(1160),
    [anon_sym___leave] = ACTIONS(1160),
    [anon_sym_DASH_DASH] = ACTIONS(1162),
    [anon_sym_PLUS_PLUS] = ACTIONS(1162),
    [anon_sym_sizeof] = ACTIONS(1160),
    [anon_sym___alignof__] = ACTIONS(1160),
    [anon_sym___alignof] = ACTIONS(1160),
    [anon_sym__alignof] = ACTIONS(1160),
    [anon_sym_alignof] = ACTIONS(1160),
    [anon_sym__Alignof] = ACTIONS(1160),
    [anon_sym_offsetof] = ACTIONS(1160),
    [anon_sym__Generic] = ACTIONS(1160),
    [anon_sym_asm] = ACTIONS(1160),
    [anon_sym___asm__] = ACTIONS(1160),
    [sym_number_literal] = ACTIONS(1162),
    [anon_sym_L_SQUOTE] = ACTIONS(1162),
    [anon_sym_u_SQUOTE] = ACTIONS(1162),
    [anon_sym_U_SQUOTE] = ACTIONS(1162),
    [anon_sym_u8_SQUOTE] = ACTIONS(1162),
    [anon_sym_SQUOTE] = ACTIONS(1162),
    [anon_sym_L_DQUOTE] = ACTIONS(1162),
    [anon_sym_u_DQUOTE] = ACTIONS(1162),
    [anon_sym_U_DQUOTE] = ACTIONS(1162),
    [anon_sym_u8_DQUOTE] = ACTIONS(1162),
    [anon_sym_DQUOTE] = ACTIONS(1162),
    [sym_true] = ACTIONS(1160),
    [sym_false] = ACTIONS(1160),
    [anon_sym_NULL] = ACTIONS(1160),
    [anon_sym_nullptr] = ACTIONS(1160),
    [sym_comment] = ACTIONS(3),
  },
  [305] = {
    [sym_identifier] = ACTIONS(1148),
    [aux_sym_preproc_include_token1] = ACTIONS(1148),
    [aux_sym_preproc_def_token1] = ACTIONS(1148),
    [aux_sym_preproc_if_token1] = ACTIONS(1148),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1148),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1148),
    [sym_preproc_directive] = ACTIONS(1148),
    [anon_sym_LPAREN2] = ACTIONS(1150),
    [anon_sym_BANG] = ACTIONS(1150),
    [anon_sym_TILDE] = ACTIONS(1150),
    [anon_sym_DASH] = ACTIONS(1148),
    [anon_sym_PLUS] = ACTIONS(1148),
    [anon_sym_STAR] = ACTIONS(1150),
    [anon_sym_AMP] = ACTIONS(1150),
    [anon_sym_SEMI] = ACTIONS(1150),
    [anon_sym___extension__] = ACTIONS(1148),
    [anon_sym_typedef] = ACTIONS(1148),
    [anon_sym_extern] = ACTIONS(1148),
    [anon_sym___attribute__] = ACTIONS(1148),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1150),
    [anon_sym___declspec] = ACTIONS(1148),
    [anon_sym___cdecl] = ACTIONS(1148),
    [anon_sym___clrcall] = ACTIONS(1148),
    [anon_sym___stdcall] = ACTIONS(1148),
    [anon_sym___fastcall] = ACTIONS(1148),
    [anon_sym___thiscall] = ACTIONS(1148),
    [anon_sym___vectorcall] = ACTIONS(1148),
    [anon_sym_LBRACE] = ACTIONS(1150),
    [anon_sym_RBRACE] = ACTIONS(1150),
    [anon_sym_signed] = ACTIONS(1148),
    [anon_sym_unsigned] = ACTIONS(1148),
    [anon_sym_long] = ACTIONS(1148),
    [anon_sym_short] = ACTIONS(1148),
    [anon_sym_static] = ACTIONS(1148),
    [anon_sym_auto] = ACTIONS(1148),
    [anon_sym_register] = ACTIONS(1148),
    [anon_sym_inline] = ACTIONS(1148),
    [anon_sym___inline] = ACTIONS(1148),
    [anon_sym___inline__] = ACTIONS(1148),
    [anon_sym___forceinline] = ACTIONS(1148),
    [anon_sym_thread_local] = ACTIONS(1148),
    [anon_sym___thread] = ACTIONS(1148),
    [anon_sym_const] = ACTIONS(1148),
    [anon_sym_constexpr] = ACTIONS(1148),
    [anon_sym_volatile] = ACTIONS(1148),
    [anon_sym_restrict] = ACTIONS(1148),
    [anon_sym___restrict__] = ACTIONS(1148),
    [anon_sym__Atomic] = ACTIONS(1148),
    [anon_sym__Noreturn] = ACTIONS(1148),
    [anon_sym_noreturn] = ACTIONS(1148),
    [sym_primitive_type] = ACTIONS(1148),
    [anon_sym_enum] = ACTIONS(1148),
    [anon_sym_struct] = ACTIONS(1148),
    [anon_sym_union] = ACTIONS(1148),
    [anon_sym_if] = ACTIONS(1148),
    [anon_sym_else] = ACTIONS(1148),
    [anon_sym_switch] = ACTIONS(1148),
    [anon_sym_case] = ACTIONS(1148),
    [anon_sym_default] = ACTIONS(1148),
    [anon_sym_while] = ACTIONS(1148),
    [anon_sym_do] = ACTIONS(1148),
    [anon_sym_for] = ACTIONS(1148),
    [anon_sym_return] = ACTIONS(1148),
    [anon_sym_break] = ACTIONS(1148),
    [anon_sym_continue] = ACTIONS(1148),
    [anon_sym_goto] = ACTIONS(1148),
    [anon_sym___try] = ACTIONS(1148),
    [anon_sym___leave] = ACTIONS(1148),
    [anon_sym_DASH_DASH] = ACTIONS(1150),
    [anon_sym_PLUS_PLUS] = ACTIONS(1150),
    [anon_sym_sizeof] = ACTIONS(1148),
    [anon_sym___alignof__] = ACTIONS(1148),
    [anon_sym___alignof] = ACTIONS(1148),
    [anon_sym__alignof] = ACTIONS(1148),
    [anon_sym_alignof] = ACTIONS(1148),
    [anon_sym__Alignof] = ACTIONS(1148),
    [anon_sym_offsetof] = ACTIONS(1148),
    [anon_sym__Generic] = ACTIONS(1148),
    [anon_sym_asm] = ACTIONS(1148),
    [anon_sym___asm__] = ACTIONS(1148),
    [sym_number_literal] = ACTIONS(1150),
    [anon_sym_L_SQUOTE] = ACTIONS(1150),
    [anon_sym_u_SQUOTE] = ACTIONS(1150),
    [anon_sym_U_SQUOTE] = ACTIONS(1150),
    [anon_sym_u8_SQUOTE] = ACTIONS(1150),
    [anon_sym_SQUOTE] = ACTIONS(1150),
    [anon_sym_L_DQUOTE] = ACTIONS(1150),
    [anon_sym_u_DQUOTE] = ACTIONS(1150),
    [anon_sym_U_DQUOTE] = ACTIONS(1150),
    [anon_sym_u8_DQUOTE] = ACTIONS(1150),
    [anon_sym_DQUOTE] = ACTIONS(1150),
    [sym_true] = ACTIONS(1148),
    [sym_false] = ACTIONS(1148),
    [anon_sym_NULL] = ACTIONS(1148),
    [anon_sym_nullptr] = ACTIONS(1148),
    [sym_comment] = ACTIONS(3),
  },
  [306] = {
    [ts_builtin_sym_end] = ACTIONS(1122),
    [sym_identifier] = ACTIONS(1120),
    [aux_sym_preproc_include_token1] = ACTIONS(1120),
    [aux_sym_preproc_def_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1120),
    [sym_preproc_directive] = ACTIONS(1120),
    [anon_sym_LPAREN2] = ACTIONS(1122),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_DASH] = ACTIONS(1120),
    [anon_sym_PLUS] = ACTIONS(1120),
    [anon_sym_STAR] = ACTIONS(1122),
    [anon_sym_AMP] = ACTIONS(1122),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym___extension__] = ACTIONS(1120),
    [anon_sym_typedef] = ACTIONS(1120),
    [anon_sym_extern] = ACTIONS(1120),
    [anon_sym___attribute__] = ACTIONS(1120),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1122),
    [anon_sym___declspec] = ACTIONS(1120),
    [anon_sym___cdecl] = ACTIONS(1120),
    [anon_sym___clrcall] = ACTIONS(1120),
    [anon_sym___stdcall] = ACTIONS(1120),
    [anon_sym___fastcall] = ACTIONS(1120),
    [anon_sym___thiscall] = ACTIONS(1120),
    [anon_sym___vectorcall] = ACTIONS(1120),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_signed] = ACTIONS(1120),
    [anon_sym_unsigned] = ACTIONS(1120),
    [anon_sym_long] = ACTIONS(1120),
    [anon_sym_short] = ACTIONS(1120),
    [anon_sym_static] = ACTIONS(1120),
    [anon_sym_auto] = ACTIONS(1120),
    [anon_sym_register] = ACTIONS(1120),
    [anon_sym_inline] = ACTIONS(1120),
    [anon_sym___inline] = ACTIONS(1120),
    [anon_sym___inline__] = ACTIONS(1120),
    [anon_sym___forceinline] = ACTIONS(1120),
    [anon_sym_thread_local] = ACTIONS(1120),
    [anon_sym___thread] = ACTIONS(1120),
    [anon_sym_const] = ACTIONS(1120),
    [anon_sym_constexpr] = ACTIONS(1120),
    [anon_sym_volatile] = ACTIONS(1120),
    [anon_sym_restrict] = ACTIONS(1120),
    [anon_sym___restrict__] = ACTIONS(1120),
    [anon_sym__Atomic] = ACTIONS(1120),
    [anon_sym__Noreturn] = ACTIONS(1120),
    [anon_sym_noreturn] = ACTIONS(1120),
    [sym_primitive_type] = ACTIONS(1120),
    [anon_sym_enum] = ACTIONS(1120),
    [anon_sym_struct] = ACTIONS(1120),
    [anon_sym_union] = ACTIONS(1120),
    [anon_sym_if] = ACTIONS(1120),
    [anon_sym_else] = ACTIONS(1120),
    [anon_sym_switch] = ACTIONS(1120),
    [anon_sym_case] = ACTIONS(1120),
    [anon_sym_default] = ACTIONS(1120),
    [anon_sym_while] = ACTIONS(1120),
    [anon_sym_do] = ACTIONS(1120),
    [anon_sym_for] = ACTIONS(1120),
    [anon_sym_return] = ACTIONS(1120),
    [anon_sym_break] = ACTIONS(1120),
    [anon_sym_continue] = ACTIONS(1120),
    [anon_sym_goto] = ACTIONS(1120),
    [anon_sym___try] = ACTIONS(1120),
    [anon_sym___leave] = ACTIONS(1120),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_sizeof] = ACTIONS(1120),
    [anon_sym___alignof__] = ACTIONS(1120),
    [anon_sym___alignof] = ACTIONS(1120),
    [anon_sym__alignof] = ACTIONS(1120),
    [anon_sym_alignof] = ACTIONS(1120),
    [anon_sym__Alignof] = ACTIONS(1120),
    [anon_sym_offsetof] = ACTIONS(1120),
    [anon_sym__Generic] = ACTIONS(1120),
    [anon_sym_asm] = ACTIONS(1120),
    [anon_sym___asm__] = ACTIONS(1120),
    [sym_number_literal] = ACTIONS(1122),
    [anon_sym_L_SQUOTE] = ACTIONS(1122),
    [anon_sym_u_SQUOTE] = ACTIONS(1122),
    [anon_sym_U_SQUOTE] = ACTIONS(1122),
    [anon_sym_u8_SQUOTE] = ACTIONS(1122),
    [anon_sym_SQUOTE] = ACTIONS(1122),
    [anon_sym_L_DQUOTE] = ACTIONS(1122),
    [anon_sym_u_DQUOTE] = ACTIONS(1122),
    [anon_sym_U_DQUOTE] = ACTIONS(1122),
    [anon_sym_u8_DQUOTE] = ACTIONS(1122),
    [anon_sym_DQUOTE] = ACTIONS(1122),
    [sym_true] = ACTIONS(1120),
    [sym_false] = ACTIONS(1120),
    [anon_sym_NULL] = ACTIONS(1120),
    [anon_sym_nullptr] = ACTIONS(1120),
    [sym_comment] = ACTIONS(3),
  },
  [307] = {
    [ts_builtin_sym_end] = ACTIONS(1122),
    [sym_identifier] = ACTIONS(1120),
    [aux_sym_preproc_include_token1] = ACTIONS(1120),
    [aux_sym_preproc_def_token1] = ACTIONS(1120),
    [aux_sym_preproc_if_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1120),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1120),
    [sym_preproc_directive] = ACTIONS(1120),
    [anon_sym_LPAREN2] = ACTIONS(1122),
    [anon_sym_BANG] = ACTIONS(1122),
    [anon_sym_TILDE] = ACTIONS(1122),
    [anon_sym_DASH] = ACTIONS(1120),
    [anon_sym_PLUS] = ACTIONS(1120),
    [anon_sym_STAR] = ACTIONS(1122),
    [anon_sym_AMP] = ACTIONS(1122),
    [anon_sym_SEMI] = ACTIONS(1122),
    [anon_sym___extension__] = ACTIONS(1120),
    [anon_sym_typedef] = ACTIONS(1120),
    [anon_sym_extern] = ACTIONS(1120),
    [anon_sym___attribute__] = ACTIONS(1120),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1122),
    [anon_sym___declspec] = ACTIONS(1120),
    [anon_sym___cdecl] = ACTIONS(1120),
    [anon_sym___clrcall] = ACTIONS(1120),
    [anon_sym___stdcall] = ACTIONS(1120),
    [anon_sym___fastcall] = ACTIONS(1120),
    [anon_sym___thiscall] = ACTIONS(1120),
    [anon_sym___vectorcall] = ACTIONS(1120),
    [anon_sym_LBRACE] = ACTIONS(1122),
    [anon_sym_signed] = ACTIONS(1120),
    [anon_sym_unsigned] = ACTIONS(1120),
    [anon_sym_long] = ACTIONS(1120),
    [anon_sym_short] = ACTIONS(1120),
    [anon_sym_static] = ACTIONS(1120),
    [anon_sym_auto] = ACTIONS(1120),
    [anon_sym_register] = ACTIONS(1120),
    [anon_sym_inline] = ACTIONS(1120),
    [anon_sym___inline] = ACTIONS(1120),
    [anon_sym___inline__] = ACTIONS(1120),
    [anon_sym___forceinline] = ACTIONS(1120),
    [anon_sym_thread_local] = ACTIONS(1120),
    [anon_sym___thread] = ACTIONS(1120),
    [anon_sym_const] = ACTIONS(1120),
    [anon_sym_constexpr] = ACTIONS(1120),
    [anon_sym_volatile] = ACTIONS(1120),
    [anon_sym_restrict] = ACTIONS(1120),
    [anon_sym___restrict__] = ACTIONS(1120),
    [anon_sym__Atomic] = ACTIONS(1120),
    [anon_sym__Noreturn] = ACTIONS(1120),
    [anon_sym_noreturn] = ACTIONS(1120),
    [sym_primitive_type] = ACTIONS(1120),
    [anon_sym_enum] = ACTIONS(1120),
    [anon_sym_struct] = ACTIONS(1120),
    [anon_sym_union] = ACTIONS(1120),
    [anon_sym_if] = ACTIONS(1120),
    [anon_sym_else] = ACTIONS(1120),
    [anon_sym_switch] = ACTIONS(1120),
    [anon_sym_case] = ACTIONS(1120),
    [anon_sym_default] = ACTIONS(1120),
    [anon_sym_while] = ACTIONS(1120),
    [anon_sym_do] = ACTIONS(1120),
    [anon_sym_for] = ACTIONS(1120),
    [anon_sym_return] = ACTIONS(1120),
    [anon_sym_break] = ACTIONS(1120),
    [anon_sym_continue] = ACTIONS(1120),
    [anon_sym_goto] = ACTIONS(1120),
    [anon_sym___try] = ACTIONS(1120),
    [anon_sym___leave] = ACTIONS(1120),
    [anon_sym_DASH_DASH] = ACTIONS(1122),
    [anon_sym_PLUS_PLUS] = ACTIONS(1122),
    [anon_sym_sizeof] = ACTIONS(1120),
    [anon_sym___alignof__] = ACTIONS(1120),
    [anon_sym___alignof] = ACTIONS(1120),
    [anon_sym__alignof] = ACTIONS(1120),
    [anon_sym_alignof] = ACTIONS(1120),
    [anon_sym__Alignof] = ACTIONS(1120),
    [anon_sym_offsetof] = ACTIONS(1120),
    [anon_sym__Generic] = ACTIONS(1120),
    [anon_sym_asm] = ACTIONS(1120),
    [anon_sym___asm__] = ACTIONS(1120),
    [sym_number_literal] = ACTIONS(1122),
    [anon_sym_L_SQUOTE] = ACTIONS(1122),
    [anon_sym_u_SQUOTE] = ACTIONS(1122),
    [anon_sym_U_SQUOTE] = ACTIONS(1122),
    [anon_sym_u8_SQUOTE] = ACTIONS(1122),
    [anon_sym_SQUOTE] = ACTIONS(1122),
    [anon_sym_L_DQUOTE] = ACTIONS(1122),
    [anon_sym_u_DQUOTE] = ACTIONS(1122),
    [anon_sym_U_DQUOTE] = ACTIONS(1122),
    [anon_sym_u8_DQUOTE] = ACTIONS(1122),
    [anon_sym_DQUOTE] = ACTIONS(1122),
    [sym_true] = ACTIONS(1120),
    [sym_false] = ACTIONS(1120),
    [anon_sym_NULL] = ACTIONS(1120),
    [anon_sym_nullptr] = ACTIONS(1120),
    [sym_comment] = ACTIONS(3),
  },
  [308] = {
    [ts_builtin_sym_end] = ACTIONS(1182),
    [sym_identifier] = ACTIONS(1180),
    [aux_sym_preproc_include_token1] = ACTIONS(1180),
    [aux_sym_preproc_def_token1] = ACTIONS(1180),
    [aux_sym_preproc_if_token1] = ACTIONS(1180),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1180),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1180),
    [sym_preproc_directive] = ACTIONS(1180),
    [anon_sym_LPAREN2] = ACTIONS(1182),
    [anon_sym_BANG] = ACTIONS(1182),
    [anon_sym_TILDE] = ACTIONS(1182),
    [anon_sym_DASH] = ACTIONS(1180),
    [anon_sym_PLUS] = ACTIONS(1180),
    [anon_sym_STAR] = ACTIONS(1182),
    [anon_sym_AMP] = ACTIONS(1182),
    [anon_sym_SEMI] = ACTIONS(1182),
    [anon_sym___extension__] = ACTIONS(1180),
    [anon_sym_typedef] = ACTIONS(1180),
    [anon_sym_extern] = ACTIONS(1180),
    [anon_sym___attribute__] = ACTIONS(1180),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1182),
    [anon_sym___declspec] = ACTIONS(1180),
    [anon_sym___cdecl] = ACTIONS(1180),
    [anon_sym___clrcall] = ACTIONS(1180),
    [anon_sym___stdcall] = ACTIONS(1180),
    [anon_sym___fastcall] = ACTIONS(1180),
    [anon_sym___thiscall] = ACTIONS(1180),
    [anon_sym___vectorcall] = ACTIONS(1180),
    [anon_sym_LBRACE] = ACTIONS(1182),
    [anon_sym_signed] = ACTIONS(1180),
    [anon_sym_unsigned] = ACTIONS(1180),
    [anon_sym_long] = ACTIONS(1180),
    [anon_sym_short] = ACTIONS(1180),
    [anon_sym_static] = ACTIONS(1180),
    [anon_sym_auto] = ACTIONS(1180),
    [anon_sym_register] = ACTIONS(1180),
    [anon_sym_inline] = ACTIONS(1180),
    [anon_sym___inline] = ACTIONS(1180),
    [anon_sym___inline__] = ACTIONS(1180),
    [anon_sym___forceinline] = ACTIONS(1180),
    [anon_sym_thread_local] = ACTIONS(1180),
    [anon_sym___thread] = ACTIONS(1180),
    [anon_sym_const] = ACTIONS(1180),
    [anon_sym_constexpr] = ACTIONS(1180),
    [anon_sym_volatile] = ACTIONS(1180),
    [anon_sym_restrict] = ACTIONS(1180),
    [anon_sym___restrict__] = ACTIONS(1180),
    [anon_sym__Atomic] = ACTIONS(1180),
    [anon_sym__Noreturn] = ACTIONS(1180),
    [anon_sym_noreturn] = ACTIONS(1180),
    [sym_primitive_type] = ACTIONS(1180),
    [anon_sym_enum] = ACTIONS(1180),
    [anon_sym_struct] = ACTIONS(1180),
    [anon_sym_union] = ACTIONS(1180),
    [anon_sym_if] = ACTIONS(1180),
    [anon_sym_else] = ACTIONS(1180),
    [anon_sym_switch] = ACTIONS(1180),
    [anon_sym_case] = ACTIONS(1180),
    [anon_sym_default] = ACTIONS(1180),
    [anon_sym_while] = ACTIONS(1180),
    [anon_sym_do] = ACTIONS(1180),
    [anon_sym_for] = ACTIONS(1180),
    [anon_sym_return] = ACTIONS(1180),
    [anon_sym_break] = ACTIONS(1180),
    [anon_sym_continue] = ACTIONS(1180),
    [anon_sym_goto] = ACTIONS(1180),
    [anon_sym___try] = ACTIONS(1180),
    [anon_sym___leave] = ACTIONS(1180),
    [anon_sym_DASH_DASH] = ACTIONS(1182),
    [anon_sym_PLUS_PLUS] = ACTIONS(1182),
    [anon_sym_sizeof] = ACTIONS(1180),
    [anon_sym___alignof__] = ACTIONS(1180),
    [anon_sym___alignof] = ACTIONS(1180),
    [anon_sym__alignof] = ACTIONS(1180),
    [anon_sym_alignof] = ACTIONS(1180),
    [anon_sym__Alignof] = ACTIONS(1180),
    [anon_sym_offsetof] = ACTIONS(1180),
    [anon_sym__Generic] = ACTIONS(1180),
    [anon_sym_asm] = ACTIONS(1180),
    [anon_sym___asm__] = ACTIONS(1180),
    [sym_number_literal] = ACTIONS(1182),
    [anon_sym_L_SQUOTE] = ACTIONS(1182),
    [anon_sym_u_SQUOTE] = ACTIONS(1182),
    [anon_sym_U_SQUOTE] = ACTIONS(1182),
    [anon_sym_u8_SQUOTE] = ACTIONS(1182),
    [anon_sym_SQUOTE] = ACTIONS(1182),
    [anon_sym_L_DQUOTE] = ACTIONS(1182),
    [anon_sym_u_DQUOTE] = ACTIONS(1182),
    [anon_sym_U_DQUOTE] = ACTIONS(1182),
    [anon_sym_u8_DQUOTE] = ACTIONS(1182),
    [anon_sym_DQUOTE] = ACTIONS(1182),
    [sym_true] = ACTIONS(1180),
    [sym_false] = ACTIONS(1180),
    [anon_sym_NULL] = ACTIONS(1180),
    [anon_sym_nullptr] = ACTIONS(1180),
    [sym_comment] = ACTIONS(3),
  },
  [309] = {
    [sym_identifier] = ACTIONS(1304),
    [aux_sym_preproc_include_token1] = ACTIONS(1304),
    [aux_sym_preproc_def_token1] = ACTIONS(1304),
    [aux_sym_preproc_if_token1] = ACTIONS(1304),
    [aux_sym_preproc_if_token2] = ACTIONS(1304),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1304),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1304),
    [sym_preproc_directive] = ACTIONS(1304),
    [anon_sym_LPAREN2] = ACTIONS(1306),
    [anon_sym_BANG] = ACTIONS(1306),
    [anon_sym_TILDE] = ACTIONS(1306),
    [anon_sym_DASH] = ACTIONS(1304),
    [anon_sym_PLUS] = ACTIONS(1304),
    [anon_sym_STAR] = ACTIONS(1306),
    [anon_sym_AMP] = ACTIONS(1306),
    [anon_sym_SEMI] = ACTIONS(1306),
    [anon_sym___extension__] = ACTIONS(1304),
    [anon_sym_typedef] = ACTIONS(1304),
    [anon_sym_extern] = ACTIONS(1304),
    [anon_sym___attribute__] = ACTIONS(1304),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1306),
    [anon_sym___declspec] = ACTIONS(1304),
    [anon_sym___cdecl] = ACTIONS(1304),
    [anon_sym___clrcall] = ACTIONS(1304),
    [anon_sym___stdcall] = ACTIONS(1304),
    [anon_sym___fastcall] = ACTIONS(1304),
    [anon_sym___thiscall] = ACTIONS(1304),
    [anon_sym___vectorcall] = ACTIONS(1304),
    [anon_sym_LBRACE] = ACTIONS(1306),
    [anon_sym_signed] = ACTIONS(1304),
    [anon_sym_unsigned] = ACTIONS(1304),
    [anon_sym_long] = ACTIONS(1304),
    [anon_sym_short] = ACTIONS(1304),
    [anon_sym_static] = ACTIONS(1304),
    [anon_sym_auto] = ACTIONS(1304),
    [anon_sym_register] = ACTIONS(1304),
    [anon_sym_inline] = ACTIONS(1304),
    [anon_sym___inline] = ACTIONS(1304),
    [anon_sym___inline__] = ACTIONS(1304),
    [anon_sym___forceinline] = ACTIONS(1304),
    [anon_sym_thread_local] = ACTIONS(1304),
    [anon_sym___thread] = ACTIONS(1304),
    [anon_sym_const] = ACTIONS(1304),
    [anon_sym_constexpr] = ACTIONS(1304),
    [anon_sym_volatile] = ACTIONS(1304),
    [anon_sym_restrict] = ACTIONS(1304),
    [anon_sym___restrict__] = ACTIONS(1304),
    [anon_sym__Atomic] = ACTIONS(1304),
    [anon_sym__Noreturn] = ACTIONS(1304),
    [anon_sym_noreturn] = ACTIONS(1304),
    [sym_primitive_type] = ACTIONS(1304),
    [anon_sym_enum] = ACTIONS(1304),
    [anon_sym_struct] = ACTIONS(1304),
    [anon_sym_union] = ACTIONS(1304),
    [anon_sym_if] = ACTIONS(1304),
    [anon_sym_switch] = ACTIONS(1304),
    [anon_sym_case] = ACTIONS(1304),
    [anon_sym_default] = ACTIONS(1304),
    [anon_sym_while] = ACTIONS(1304),
    [anon_sym_do] = ACTIONS(1304),
    [anon_sym_for] = ACTIONS(1304),
    [anon_sym_return] = ACTIONS(1304),
    [anon_sym_break] = ACTIONS(1304),
    [anon_sym_continue] = ACTIONS(1304),
    [anon_sym_goto] = ACTIONS(1304),
    [anon_sym___try] = ACTIONS(1304),
    [anon_sym___leave] = ACTIONS(1304),
    [anon_sym_DASH_DASH] = ACTIONS(1306),
    [anon_sym_PLUS_PLUS] = ACTIONS(1306),
    [anon_sym_sizeof] = ACTIONS(1304),
    [anon_sym___alignof__] = ACTIONS(1304),
    [anon_sym___alignof] = ACTIONS(1304),
    [anon_sym__alignof] = ACTIONS(1304),
    [anon_sym_alignof] = ACTIONS(1304),
    [anon_sym__Alignof] = ACTIONS(1304),
    [anon_sym_offsetof] = ACTIONS(1304),
    [anon_sym__Generic] = ACTIONS(1304),
    [anon_sym_asm] = ACTIONS(1304),
    [anon_sym___asm__] = ACTIONS(1304),
    [sym_number_literal] = ACTIONS(1306),
    [anon_sym_L_SQUOTE] = ACTIONS(1306),
    [anon_sym_u_SQUOTE] = ACTIONS(1306),
    [anon_sym_U_SQUOTE] = ACTIONS(1306),
    [anon_sym_u8_SQUOTE] = ACTIONS(1306),
    [anon_sym_SQUOTE] = ACTIONS(1306),
    [anon_sym_L_DQUOTE] = ACTIONS(1306),
    [anon_sym_u_DQUOTE] = ACTIONS(1306),
    [anon_sym_U_DQUOTE] = ACTIONS(1306),
    [anon_sym_u8_DQUOTE] = ACTIONS(1306),
    [anon_sym_DQUOTE] = ACTIONS(1306),
    [sym_true] = ACTIONS(1304),
    [sym_false] = ACTIONS(1304),
    [anon_sym_NULL] = ACTIONS(1304),
    [anon_sym_nullptr] = ACTIONS(1304),
    [sym_comment] = ACTIONS(3),
  },
  [310] = {
    [sym_identifier] = ACTIONS(1288),
    [aux_sym_preproc_include_token1] = ACTIONS(1288),
    [aux_sym_preproc_def_token1] = ACTIONS(1288),
    [aux_sym_preproc_if_token1] = ACTIONS(1288),
    [aux_sym_preproc_if_token2] = ACTIONS(1288),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1288),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1288),
    [sym_preproc_directive] = ACTIONS(1288),
    [anon_sym_LPAREN2] = ACTIONS(1290),
    [anon_sym_BANG] = ACTIONS(1290),
    [anon_sym_TILDE] = ACTIONS(1290),
    [anon_sym_DASH] = ACTIONS(1288),
    [anon_sym_PLUS] = ACTIONS(1288),
    [anon_sym_STAR] = ACTIONS(1290),
    [anon_sym_AMP] = ACTIONS(1290),
    [anon_sym_SEMI] = ACTIONS(1290),
    [anon_sym___extension__] = ACTIONS(1288),
    [anon_sym_typedef] = ACTIONS(1288),
    [anon_sym_extern] = ACTIONS(1288),
    [anon_sym___attribute__] = ACTIONS(1288),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1290),
    [anon_sym___declspec] = ACTIONS(1288),
    [anon_sym___cdecl] = ACTIONS(1288),
    [anon_sym___clrcall] = ACTIONS(1288),
    [anon_sym___stdcall] = ACTIONS(1288),
    [anon_sym___fastcall] = ACTIONS(1288),
    [anon_sym___thiscall] = ACTIONS(1288),
    [anon_sym___vectorcall] = ACTIONS(1288),
    [anon_sym_LBRACE] = ACTIONS(1290),
    [anon_sym_signed] = ACTIONS(1288),
    [anon_sym_unsigned] = ACTIONS(1288),
    [anon_sym_long] = ACTIONS(1288),
    [anon_sym_short] = ACTIONS(1288),
    [anon_sym_static] = ACTIONS(1288),
    [anon_sym_auto] = ACTIONS(1288),
    [anon_sym_register] = ACTIONS(1288),
    [anon_sym_inline] = ACTIONS(1288),
    [anon_sym___inline] = ACTIONS(1288),
    [anon_sym___inline__] = ACTIONS(1288),
    [anon_sym___forceinline] = ACTIONS(1288),
    [anon_sym_thread_local] = ACTIONS(1288),
    [anon_sym___thread] = ACTIONS(1288),
    [anon_sym_const] = ACTIONS(1288),
    [anon_sym_constexpr] = ACTIONS(1288),
    [anon_sym_volatile] = ACTIONS(1288),
    [anon_sym_restrict] = ACTIONS(1288),
    [anon_sym___restrict__] = ACTIONS(1288),
    [anon_sym__Atomic] = ACTIONS(1288),
    [anon_sym__Noreturn] = ACTIONS(1288),
    [anon_sym_noreturn] = ACTIONS(1288),
    [sym_primitive_type] = ACTIONS(1288),
    [anon_sym_enum] = ACTIONS(1288),
    [anon_sym_struct] = ACTIONS(1288),
    [anon_sym_union] = ACTIONS(1288),
    [anon_sym_if] = ACTIONS(1288),
    [anon_sym_switch] = ACTIONS(1288),
    [anon_sym_case] = ACTIONS(1288),
    [anon_sym_default] = ACTIONS(1288),
    [anon_sym_while] = ACTIONS(1288),
    [anon_sym_do] = ACTIONS(1288),
    [anon_sym_for] = ACTIONS(1288),
    [anon_sym_return] = ACTIONS(1288),
    [anon_sym_break] = ACTIONS(1288),
    [anon_sym_continue] = ACTIONS(1288),
    [anon_sym_goto] = ACTIONS(1288),
    [anon_sym___try] = ACTIONS(1288),
    [anon_sym___leave] = ACTIONS(1288),
    [anon_sym_DASH_DASH] = ACTIONS(1290),
    [anon_sym_PLUS_PLUS] = ACTIONS(1290),
    [anon_sym_sizeof] = ACTIONS(1288),
    [anon_sym___alignof__] = ACTIONS(1288),
    [anon_sym___alignof] = ACTIONS(1288),
    [anon_sym__alignof] = ACTIONS(1288),
    [anon_sym_alignof] = ACTIONS(1288),
    [anon_sym__Alignof] = ACTIONS(1288),
    [anon_sym_offsetof] = ACTIONS(1288),
    [anon_sym__Generic] = ACTIONS(1288),
    [anon_sym_asm] = ACTIONS(1288),
    [anon_sym___asm__] = ACTIONS(1288),
    [sym_number_literal] = ACTIONS(1290),
    [anon_sym_L_SQUOTE] = ACTIONS(1290),
    [anon_sym_u_SQUOTE] = ACTIONS(1290),
    [anon_sym_U_SQUOTE] = ACTIONS(1290),
    [anon_sym_u8_SQUOTE] = ACTIONS(1290),
    [anon_sym_SQUOTE] = ACTIONS(1290),
    [anon_sym_L_DQUOTE] = ACTIONS(1290),
    [anon_sym_u_DQUOTE] = ACTIONS(1290),
    [anon_sym_U_DQUOTE] = ACTIONS(1290),
    [anon_sym_u8_DQUOTE] = ACTIONS(1290),
    [anon_sym_DQUOTE] = ACTIONS(1290),
    [sym_true] = ACTIONS(1288),
    [sym_false] = ACTIONS(1288),
    [anon_sym_NULL] = ACTIONS(1288),
    [anon_sym_nullptr] = ACTIONS(1288),
    [sym_comment] = ACTIONS(3),
  },
  [311] = {
    [sym_attribute_declaration] = STATE(339),
    [sym_compound_statement] = STATE(156),
    [sym_attributed_statement] = STATE(156),
    [sym_labeled_statement] = STATE(156),
    [sym_expression_statement] = STATE(156),
    [sym_if_statement] = STATE(156),
    [sym_switch_statement] = STATE(156),
    [sym_case_statement] = STATE(156),
    [sym_while_statement] = STATE(156),
    [sym_do_statement] = STATE(156),
    [sym_for_statement] = STATE(156),
    [sym_return_statement] = STATE(156),
    [sym_break_statement] = STATE(156),
    [sym_continue_statement] = STATE(156),
    [sym_goto_statement] = STATE(156),
    [sym_seh_try_statement] = STATE(156),
    [sym_seh_leave_statement] = STATE(156),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [sym_identifier] = ACTIONS(1374),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(429),
    [anon_sym_default] = ACTIONS(431),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [312] = {
    [sym_attribute_declaration] = STATE(339),
    [sym_compound_statement] = STATE(203),
    [sym_attributed_statement] = STATE(203),
    [sym_labeled_statement] = STATE(203),
    [sym_expression_statement] = STATE(203),
    [sym_if_statement] = STATE(203),
    [sym_switch_statement] = STATE(203),
    [sym_case_statement] = STATE(203),
    [sym_while_statement] = STATE(203),
    [sym_do_statement] = STATE(203),
    [sym_for_statement] = STATE(203),
    [sym_return_statement] = STATE(203),
    [sym_break_statement] = STATE(203),
    [sym_continue_statement] = STATE(203),
    [sym_goto_statement] = STATE(203),
    [sym_seh_try_statement] = STATE(203),
    [sym_seh_leave_statement] = STATE(203),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [sym_identifier] = ACTIONS(1374),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(429),
    [anon_sym_default] = ACTIONS(431),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [313] = {
    [sym_attribute_declaration] = STATE(339),
    [sym_compound_statement] = STATE(200),
    [sym_attributed_statement] = STATE(200),
    [sym_labeled_statement] = STATE(200),
    [sym_expression_statement] = STATE(200),
    [sym_if_statement] = STATE(200),
    [sym_switch_statement] = STATE(200),
    [sym_case_statement] = STATE(200),
    [sym_while_statement] = STATE(200),
    [sym_do_statement] = STATE(200),
    [sym_for_statement] = STATE(200),
    [sym_return_statement] = STATE(200),
    [sym_break_statement] = STATE(200),
    [sym_continue_statement] = STATE(200),
    [sym_goto_statement] = STATE(200),
    [sym_seh_try_statement] = STATE(200),
    [sym_seh_leave_statement] = STATE(200),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [sym_identifier] = ACTIONS(1374),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(429),
    [anon_sym_default] = ACTIONS(431),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [314] = {
    [sym_attribute_declaration] = STATE(323),
    [sym_compound_statement] = STATE(179),
    [sym_attributed_statement] = STATE(179),
    [sym_labeled_statement] = STATE(179),
    [sym_expression_statement] = STATE(179),
    [sym_if_statement] = STATE(179),
    [sym_switch_statement] = STATE(179),
    [sym_case_statement] = STATE(179),
    [sym_while_statement] = STATE(179),
    [sym_do_statement] = STATE(179),
    [sym_for_statement] = STATE(179),
    [sym_return_statement] = STATE(179),
    [sym_break_statement] = STATE(179),
    [sym_continue_statement] = STATE(179),
    [sym_goto_statement] = STATE(179),
    [sym_seh_try_statement] = STATE(179),
    [sym_seh_leave_statement] = STATE(179),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(323),
    [sym_identifier] = ACTIONS(1378),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(61),
    [anon_sym_default] = ACTIONS(63),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [315] = {
    [sym_attribute_declaration] = STATE(358),
    [sym_compound_statement] = STATE(2002),
    [sym_attributed_statement] = STATE(2002),
    [sym_labeled_statement] = STATE(2002),
    [sym_expression_statement] = STATE(2002),
    [sym_if_statement] = STATE(2002),
    [sym_switch_statement] = STATE(2002),
    [sym_case_statement] = STATE(2002),
    [sym_while_statement] = STATE(2002),
    [sym_do_statement] = STATE(2002),
    [sym_for_statement] = STATE(2002),
    [sym_return_statement] = STATE(2002),
    [sym_break_statement] = STATE(2002),
    [sym_continue_statement] = STATE(2002),
    [sym_goto_statement] = STATE(2002),
    [sym_seh_try_statement] = STATE(2002),
    [sym_seh_leave_statement] = STATE(2002),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [316] = {
    [sym_attribute_declaration] = STATE(316),
    [sym_compound_statement] = STATE(221),
    [sym_attributed_statement] = STATE(221),
    [sym_labeled_statement] = STATE(221),
    [sym_expression_statement] = STATE(221),
    [sym_if_statement] = STATE(221),
    [sym_switch_statement] = STATE(221),
    [sym_case_statement] = STATE(221),
    [sym_while_statement] = STATE(221),
    [sym_do_statement] = STATE(221),
    [sym_for_statement] = STATE(221),
    [sym_return_statement] = STATE(221),
    [sym_break_statement] = STATE(221),
    [sym_continue_statement] = STATE(221),
    [sym_goto_statement] = STATE(221),
    [sym_seh_try_statement] = STATE(221),
    [sym_seh_leave_statement] = STATE(221),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(316),
    [sym_identifier] = ACTIONS(1386),
    [anon_sym_LPAREN2] = ACTIONS(1389),
    [anon_sym_BANG] = ACTIONS(1392),
    [anon_sym_TILDE] = ACTIONS(1392),
    [anon_sym_DASH] = ACTIONS(1395),
    [anon_sym_PLUS] = ACTIONS(1395),
    [anon_sym_STAR] = ACTIONS(1398),
    [anon_sym_AMP] = ACTIONS(1398),
    [anon_sym_SEMI] = ACTIONS(1401),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1404),
    [anon_sym_LBRACE] = ACTIONS(1407),
    [anon_sym_if] = ACTIONS(1410),
    [anon_sym_switch] = ACTIONS(1413),
    [anon_sym_case] = ACTIONS(1416),
    [anon_sym_default] = ACTIONS(1419),
    [anon_sym_while] = ACTIONS(1422),
    [anon_sym_do] = ACTIONS(1425),
    [anon_sym_for] = ACTIONS(1428),
    [anon_sym_return] = ACTIONS(1431),
    [anon_sym_break] = ACTIONS(1434),
    [anon_sym_continue] = ACTIONS(1437),
    [anon_sym_goto] = ACTIONS(1440),
    [anon_sym___try] = ACTIONS(1443),
    [anon_sym___leave] = ACTIONS(1446),
    [anon_sym_DASH_DASH] = ACTIONS(1449),
    [anon_sym_PLUS_PLUS] = ACTIONS(1449),
    [anon_sym_sizeof] = ACTIONS(1452),
    [anon_sym___alignof__] = ACTIONS(1455),
    [anon_sym___alignof] = ACTIONS(1455),
    [anon_sym__alignof] = ACTIONS(1455),
    [anon_sym_alignof] = ACTIONS(1455),
    [anon_sym__Alignof] = ACTIONS(1455),
    [anon_sym_offsetof] = ACTIONS(1458),
    [anon_sym__Generic] = ACTIONS(1461),
    [anon_sym_asm] = ACTIONS(1464),
    [anon_sym___asm__] = ACTIONS(1464),
    [sym_number_literal] = ACTIONS(1467),
    [anon_sym_L_SQUOTE] = ACTIONS(1470),
    [anon_sym_u_SQUOTE] = ACTIONS(1470),
    [anon_sym_U_SQUOTE] = ACTIONS(1470),
    [anon_sym_u8_SQUOTE] = ACTIONS(1470),
    [anon_sym_SQUOTE] = ACTIONS(1470),
    [anon_sym_L_DQUOTE] = ACTIONS(1473),
    [anon_sym_u_DQUOTE] = ACTIONS(1473),
    [anon_sym_U_DQUOTE] = ACTIONS(1473),
    [anon_sym_u8_DQUOTE] = ACTIONS(1473),
    [anon_sym_DQUOTE] = ACTIONS(1473),
    [sym_true] = ACTIONS(1476),
    [sym_false] = ACTIONS(1476),
    [anon_sym_NULL] = ACTIONS(1479),
    [anon_sym_nullptr] = ACTIONS(1479),
    [sym_comment] = ACTIONS(3),
  },
  [317] = {
    [sym_attribute_declaration] = STATE(358),
    [sym_compound_statement] = STATE(1989),
    [sym_attributed_statement] = STATE(1989),
    [sym_labeled_statement] = STATE(1989),
    [sym_expression_statement] = STATE(1989),
    [sym_if_statement] = STATE(1989),
    [sym_switch_statement] = STATE(1989),
    [sym_case_statement] = STATE(1989),
    [sym_while_statement] = STATE(1989),
    [sym_do_statement] = STATE(1989),
    [sym_for_statement] = STATE(1989),
    [sym_return_statement] = STATE(1989),
    [sym_break_statement] = STATE(1989),
    [sym_continue_statement] = STATE(1989),
    [sym_goto_statement] = STATE(1989),
    [sym_seh_try_statement] = STATE(1989),
    [sym_seh_leave_statement] = STATE(1989),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [318] = {
    [sym_attribute_declaration] = STATE(339),
    [sym_compound_statement] = STATE(169),
    [sym_attributed_statement] = STATE(169),
    [sym_labeled_statement] = STATE(169),
    [sym_expression_statement] = STATE(169),
    [sym_if_statement] = STATE(169),
    [sym_switch_statement] = STATE(169),
    [sym_case_statement] = STATE(169),
    [sym_while_statement] = STATE(169),
    [sym_do_statement] = STATE(169),
    [sym_for_statement] = STATE(169),
    [sym_return_statement] = STATE(169),
    [sym_break_statement] = STATE(169),
    [sym_continue_statement] = STATE(169),
    [sym_goto_statement] = STATE(169),
    [sym_seh_try_statement] = STATE(169),
    [sym_seh_leave_statement] = STATE(169),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [sym_identifier] = ACTIONS(1374),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(429),
    [anon_sym_default] = ACTIONS(431),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [319] = {
    [sym_identifier] = ACTIONS(1256),
    [aux_sym_preproc_include_token1] = ACTIONS(1256),
    [aux_sym_preproc_def_token1] = ACTIONS(1256),
    [aux_sym_preproc_if_token1] = ACTIONS(1256),
    [aux_sym_preproc_if_token2] = ACTIONS(1256),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1256),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1256),
    [sym_preproc_directive] = ACTIONS(1256),
    [anon_sym_LPAREN2] = ACTIONS(1258),
    [anon_sym_BANG] = ACTIONS(1258),
    [anon_sym_TILDE] = ACTIONS(1258),
    [anon_sym_DASH] = ACTIONS(1256),
    [anon_sym_PLUS] = ACTIONS(1256),
    [anon_sym_STAR] = ACTIONS(1258),
    [anon_sym_AMP] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1258),
    [anon_sym___extension__] = ACTIONS(1256),
    [anon_sym_typedef] = ACTIONS(1256),
    [anon_sym_extern] = ACTIONS(1256),
    [anon_sym___attribute__] = ACTIONS(1256),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1258),
    [anon_sym___declspec] = ACTIONS(1256),
    [anon_sym___cdecl] = ACTIONS(1256),
    [anon_sym___clrcall] = ACTIONS(1256),
    [anon_sym___stdcall] = ACTIONS(1256),
    [anon_sym___fastcall] = ACTIONS(1256),
    [anon_sym___thiscall] = ACTIONS(1256),
    [anon_sym___vectorcall] = ACTIONS(1256),
    [anon_sym_LBRACE] = ACTIONS(1258),
    [anon_sym_signed] = ACTIONS(1256),
    [anon_sym_unsigned] = ACTIONS(1256),
    [anon_sym_long] = ACTIONS(1256),
    [anon_sym_short] = ACTIONS(1256),
    [anon_sym_static] = ACTIONS(1256),
    [anon_sym_auto] = ACTIONS(1256),
    [anon_sym_register] = ACTIONS(1256),
    [anon_sym_inline] = ACTIONS(1256),
    [anon_sym___inline] = ACTIONS(1256),
    [anon_sym___inline__] = ACTIONS(1256),
    [anon_sym___forceinline] = ACTIONS(1256),
    [anon_sym_thread_local] = ACTIONS(1256),
    [anon_sym___thread] = ACTIONS(1256),
    [anon_sym_const] = ACTIONS(1256),
    [anon_sym_constexpr] = ACTIONS(1256),
    [anon_sym_volatile] = ACTIONS(1256),
    [anon_sym_restrict] = ACTIONS(1256),
    [anon_sym___restrict__] = ACTIONS(1256),
    [anon_sym__Atomic] = ACTIONS(1256),
    [anon_sym__Noreturn] = ACTIONS(1256),
    [anon_sym_noreturn] = ACTIONS(1256),
    [sym_primitive_type] = ACTIONS(1256),
    [anon_sym_enum] = ACTIONS(1256),
    [anon_sym_struct] = ACTIONS(1256),
    [anon_sym_union] = ACTIONS(1256),
    [anon_sym_if] = ACTIONS(1256),
    [anon_sym_switch] = ACTIONS(1256),
    [anon_sym_case] = ACTIONS(1256),
    [anon_sym_default] = ACTIONS(1256),
    [anon_sym_while] = ACTIONS(1256),
    [anon_sym_do] = ACTIONS(1256),
    [anon_sym_for] = ACTIONS(1256),
    [anon_sym_return] = ACTIONS(1256),
    [anon_sym_break] = ACTIONS(1256),
    [anon_sym_continue] = ACTIONS(1256),
    [anon_sym_goto] = ACTIONS(1256),
    [anon_sym___try] = ACTIONS(1256),
    [anon_sym___leave] = ACTIONS(1256),
    [anon_sym_DASH_DASH] = ACTIONS(1258),
    [anon_sym_PLUS_PLUS] = ACTIONS(1258),
    [anon_sym_sizeof] = ACTIONS(1256),
    [anon_sym___alignof__] = ACTIONS(1256),
    [anon_sym___alignof] = ACTIONS(1256),
    [anon_sym__alignof] = ACTIONS(1256),
    [anon_sym_alignof] = ACTIONS(1256),
    [anon_sym__Alignof] = ACTIONS(1256),
    [anon_sym_offsetof] = ACTIONS(1256),
    [anon_sym__Generic] = ACTIONS(1256),
    [anon_sym_asm] = ACTIONS(1256),
    [anon_sym___asm__] = ACTIONS(1256),
    [sym_number_literal] = ACTIONS(1258),
    [anon_sym_L_SQUOTE] = ACTIONS(1258),
    [anon_sym_u_SQUOTE] = ACTIONS(1258),
    [anon_sym_U_SQUOTE] = ACTIONS(1258),
    [anon_sym_u8_SQUOTE] = ACTIONS(1258),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_L_DQUOTE] = ACTIONS(1258),
    [anon_sym_u_DQUOTE] = ACTIONS(1258),
    [anon_sym_U_DQUOTE] = ACTIONS(1258),
    [anon_sym_u8_DQUOTE] = ACTIONS(1258),
    [anon_sym_DQUOTE] = ACTIONS(1258),
    [sym_true] = ACTIONS(1256),
    [sym_false] = ACTIONS(1256),
    [anon_sym_NULL] = ACTIONS(1256),
    [anon_sym_nullptr] = ACTIONS(1256),
    [sym_comment] = ACTIONS(3),
  },
  [320] = {
    [sym_attribute_declaration] = STATE(339),
    [sym_compound_statement] = STATE(167),
    [sym_attributed_statement] = STATE(166),
    [sym_labeled_statement] = STATE(165),
    [sym_expression_statement] = STATE(164),
    [sym_if_statement] = STATE(163),
    [sym_switch_statement] = STATE(296),
    [sym_case_statement] = STATE(188),
    [sym_while_statement] = STATE(176),
    [sym_do_statement] = STATE(204),
    [sym_for_statement] = STATE(207),
    [sym_return_statement] = STATE(209),
    [sym_break_statement] = STATE(211),
    [sym_continue_statement] = STATE(227),
    [sym_goto_statement] = STATE(230),
    [sym_seh_try_statement] = STATE(231),
    [sym_seh_leave_statement] = STATE(232),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(339),
    [sym_identifier] = ACTIONS(1374),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(429),
    [anon_sym_default] = ACTIONS(431),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [321] = {
    [sym_identifier] = ACTIONS(1260),
    [aux_sym_preproc_include_token1] = ACTIONS(1260),
    [aux_sym_preproc_def_token1] = ACTIONS(1260),
    [aux_sym_preproc_if_token1] = ACTIONS(1260),
    [aux_sym_preproc_if_token2] = ACTIONS(1260),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1260),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1260),
    [sym_preproc_directive] = ACTIONS(1260),
    [anon_sym_LPAREN2] = ACTIONS(1262),
    [anon_sym_BANG] = ACTIONS(1262),
    [anon_sym_TILDE] = ACTIONS(1262),
    [anon_sym_DASH] = ACTIONS(1260),
    [anon_sym_PLUS] = ACTIONS(1260),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1262),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym___extension__] = ACTIONS(1260),
    [anon_sym_typedef] = ACTIONS(1260),
    [anon_sym_extern] = ACTIONS(1260),
    [anon_sym___attribute__] = ACTIONS(1260),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1262),
    [anon_sym___declspec] = ACTIONS(1260),
    [anon_sym___cdecl] = ACTIONS(1260),
    [anon_sym___clrcall] = ACTIONS(1260),
    [anon_sym___stdcall] = ACTIONS(1260),
    [anon_sym___fastcall] = ACTIONS(1260),
    [anon_sym___thiscall] = ACTIONS(1260),
    [anon_sym___vectorcall] = ACTIONS(1260),
    [anon_sym_LBRACE] = ACTIONS(1262),
    [anon_sym_signed] = ACTIONS(1260),
    [anon_sym_unsigned] = ACTIONS(1260),
    [anon_sym_long] = ACTIONS(1260),
    [anon_sym_short] = ACTIONS(1260),
    [anon_sym_static] = ACTIONS(1260),
    [anon_sym_auto] = ACTIONS(1260),
    [anon_sym_register] = ACTIONS(1260),
    [anon_sym_inline] = ACTIONS(1260),
    [anon_sym___inline] = ACTIONS(1260),
    [anon_sym___inline__] = ACTIONS(1260),
    [anon_sym___forceinline] = ACTIONS(1260),
    [anon_sym_thread_local] = ACTIONS(1260),
    [anon_sym___thread] = ACTIONS(1260),
    [anon_sym_const] = ACTIONS(1260),
    [anon_sym_constexpr] = ACTIONS(1260),
    [anon_sym_volatile] = ACTIONS(1260),
    [anon_sym_restrict] = ACTIONS(1260),
    [anon_sym___restrict__] = ACTIONS(1260),
    [anon_sym__Atomic] = ACTIONS(1260),
    [anon_sym__Noreturn] = ACTIONS(1260),
    [anon_sym_noreturn] = ACTIONS(1260),
    [sym_primitive_type] = ACTIONS(1260),
    [anon_sym_enum] = ACTIONS(1260),
    [anon_sym_struct] = ACTIONS(1260),
    [anon_sym_union] = ACTIONS(1260),
    [anon_sym_if] = ACTIONS(1260),
    [anon_sym_switch] = ACTIONS(1260),
    [anon_sym_case] = ACTIONS(1260),
    [anon_sym_default] = ACTIONS(1260),
    [anon_sym_while] = ACTIONS(1260),
    [anon_sym_do] = ACTIONS(1260),
    [anon_sym_for] = ACTIONS(1260),
    [anon_sym_return] = ACTIONS(1260),
    [anon_sym_break] = ACTIONS(1260),
    [anon_sym_continue] = ACTIONS(1260),
    [anon_sym_goto] = ACTIONS(1260),
    [anon_sym___try] = ACTIONS(1260),
    [anon_sym___leave] = ACTIONS(1260),
    [anon_sym_DASH_DASH] = ACTIONS(1262),
    [anon_sym_PLUS_PLUS] = ACTIONS(1262),
    [anon_sym_sizeof] = ACTIONS(1260),
    [anon_sym___alignof__] = ACTIONS(1260),
    [anon_sym___alignof] = ACTIONS(1260),
    [anon_sym__alignof] = ACTIONS(1260),
    [anon_sym_alignof] = ACTIONS(1260),
    [anon_sym__Alignof] = ACTIONS(1260),
    [anon_sym_offsetof] = ACTIONS(1260),
    [anon_sym__Generic] = ACTIONS(1260),
    [anon_sym_asm] = ACTIONS(1260),
    [anon_sym___asm__] = ACTIONS(1260),
    [sym_number_literal] = ACTIONS(1262),
    [anon_sym_L_SQUOTE] = ACTIONS(1262),
    [anon_sym_u_SQUOTE] = ACTIONS(1262),
    [anon_sym_U_SQUOTE] = ACTIONS(1262),
    [anon_sym_u8_SQUOTE] = ACTIONS(1262),
    [anon_sym_SQUOTE] = ACTIONS(1262),
    [anon_sym_L_DQUOTE] = ACTIONS(1262),
    [anon_sym_u_DQUOTE] = ACTIONS(1262),
    [anon_sym_U_DQUOTE] = ACTIONS(1262),
    [anon_sym_u8_DQUOTE] = ACTIONS(1262),
    [anon_sym_DQUOTE] = ACTIONS(1262),
    [sym_true] = ACTIONS(1260),
    [sym_false] = ACTIONS(1260),
    [anon_sym_NULL] = ACTIONS(1260),
    [anon_sym_nullptr] = ACTIONS(1260),
    [sym_comment] = ACTIONS(3),
  },
  [322] = {
    [sym_identifier] = ACTIONS(1264),
    [aux_sym_preproc_include_token1] = ACTIONS(1264),
    [aux_sym_preproc_def_token1] = ACTIONS(1264),
    [aux_sym_preproc_if_token1] = ACTIONS(1264),
    [aux_sym_preproc_if_token2] = ACTIONS(1264),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1264),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1264),
    [sym_preproc_directive] = ACTIONS(1264),
    [anon_sym_LPAREN2] = ACTIONS(1266),
    [anon_sym_BANG] = ACTIONS(1266),
    [anon_sym_TILDE] = ACTIONS(1266),
    [anon_sym_DASH] = ACTIONS(1264),
    [anon_sym_PLUS] = ACTIONS(1264),
    [anon_sym_STAR] = ACTIONS(1266),
    [anon_sym_AMP] = ACTIONS(1266),
    [anon_sym_SEMI] = ACTIONS(1266),
    [anon_sym___extension__] = ACTIONS(1264),
    [anon_sym_typedef] = ACTIONS(1264),
    [anon_sym_extern] = ACTIONS(1264),
    [anon_sym___attribute__] = ACTIONS(1264),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1266),
    [anon_sym___declspec] = ACTIONS(1264),
    [anon_sym___cdecl] = ACTIONS(1264),
    [anon_sym___clrcall] = ACTIONS(1264),
    [anon_sym___stdcall] = ACTIONS(1264),
    [anon_sym___fastcall] = ACTIONS(1264),
    [anon_sym___thiscall] = ACTIONS(1264),
    [anon_sym___vectorcall] = ACTIONS(1264),
    [anon_sym_LBRACE] = ACTIONS(1266),
    [anon_sym_signed] = ACTIONS(1264),
    [anon_sym_unsigned] = ACTIONS(1264),
    [anon_sym_long] = ACTIONS(1264),
    [anon_sym_short] = ACTIONS(1264),
    [anon_sym_static] = ACTIONS(1264),
    [anon_sym_auto] = ACTIONS(1264),
    [anon_sym_register] = ACTIONS(1264),
    [anon_sym_inline] = ACTIONS(1264),
    [anon_sym___inline] = ACTIONS(1264),
    [anon_sym___inline__] = ACTIONS(1264),
    [anon_sym___forceinline] = ACTIONS(1264),
    [anon_sym_thread_local] = ACTIONS(1264),
    [anon_sym___thread] = ACTIONS(1264),
    [anon_sym_const] = ACTIONS(1264),
    [anon_sym_constexpr] = ACTIONS(1264),
    [anon_sym_volatile] = ACTIONS(1264),
    [anon_sym_restrict] = ACTIONS(1264),
    [anon_sym___restrict__] = ACTIONS(1264),
    [anon_sym__Atomic] = ACTIONS(1264),
    [anon_sym__Noreturn] = ACTIONS(1264),
    [anon_sym_noreturn] = ACTIONS(1264),
    [sym_primitive_type] = ACTIONS(1264),
    [anon_sym_enum] = ACTIONS(1264),
    [anon_sym_struct] = ACTIONS(1264),
    [anon_sym_union] = ACTIONS(1264),
    [anon_sym_if] = ACTIONS(1264),
    [anon_sym_switch] = ACTIONS(1264),
    [anon_sym_case] = ACTIONS(1264),
    [anon_sym_default] = ACTIONS(1264),
    [anon_sym_while] = ACTIONS(1264),
    [anon_sym_do] = ACTIONS(1264),
    [anon_sym_for] = ACTIONS(1264),
    [anon_sym_return] = ACTIONS(1264),
    [anon_sym_break] = ACTIONS(1264),
    [anon_sym_continue] = ACTIONS(1264),
    [anon_sym_goto] = ACTIONS(1264),
    [anon_sym___try] = ACTIONS(1264),
    [anon_sym___leave] = ACTIONS(1264),
    [anon_sym_DASH_DASH] = ACTIONS(1266),
    [anon_sym_PLUS_PLUS] = ACTIONS(1266),
    [anon_sym_sizeof] = ACTIONS(1264),
    [anon_sym___alignof__] = ACTIONS(1264),
    [anon_sym___alignof] = ACTIONS(1264),
    [anon_sym__alignof] = ACTIONS(1264),
    [anon_sym_alignof] = ACTIONS(1264),
    [anon_sym__Alignof] = ACTIONS(1264),
    [anon_sym_offsetof] = ACTIONS(1264),
    [anon_sym__Generic] = ACTIONS(1264),
    [anon_sym_asm] = ACTIONS(1264),
    [anon_sym___asm__] = ACTIONS(1264),
    [sym_number_literal] = ACTIONS(1266),
    [anon_sym_L_SQUOTE] = ACTIONS(1266),
    [anon_sym_u_SQUOTE] = ACTIONS(1266),
    [anon_sym_U_SQUOTE] = ACTIONS(1266),
    [anon_sym_u8_SQUOTE] = ACTIONS(1266),
    [anon_sym_SQUOTE] = ACTIONS(1266),
    [anon_sym_L_DQUOTE] = ACTIONS(1266),
    [anon_sym_u_DQUOTE] = ACTIONS(1266),
    [anon_sym_U_DQUOTE] = ACTIONS(1266),
    [anon_sym_u8_DQUOTE] = ACTIONS(1266),
    [anon_sym_DQUOTE] = ACTIONS(1266),
    [sym_true] = ACTIONS(1264),
    [sym_false] = ACTIONS(1264),
    [anon_sym_NULL] = ACTIONS(1264),
    [anon_sym_nullptr] = ACTIONS(1264),
    [sym_comment] = ACTIONS(3),
  },
  [323] = {
    [sym_attribute_declaration] = STATE(323),
    [sym_compound_statement] = STATE(179),
    [sym_attributed_statement] = STATE(179),
    [sym_labeled_statement] = STATE(179),
    [sym_expression_statement] = STATE(179),
    [sym_if_statement] = STATE(179),
    [sym_switch_statement] = STATE(179),
    [sym_case_statement] = STATE(179),
    [sym_while_statement] = STATE(179),
    [sym_do_statement] = STATE(179),
    [sym_for_statement] = STATE(179),
    [sym_return_statement] = STATE(179),
    [sym_break_statement] = STATE(179),
    [sym_continue_statement] = STATE(179),
    [sym_goto_statement] = STATE(179),
    [sym_seh_try_statement] = STATE(179),
    [sym_seh_leave_statement] = STATE(179),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(323),
    [sym_identifier] = ACTIONS(1482),
    [anon_sym_LPAREN2] = ACTIONS(1389),
    [anon_sym_BANG] = ACTIONS(1392),
    [anon_sym_TILDE] = ACTIONS(1392),
    [anon_sym_DASH] = ACTIONS(1395),
    [anon_sym_PLUS] = ACTIONS(1395),
    [anon_sym_STAR] = ACTIONS(1398),
    [anon_sym_AMP] = ACTIONS(1398),
    [anon_sym_SEMI] = ACTIONS(1485),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1404),
    [anon_sym_LBRACE] = ACTIONS(1488),
    [anon_sym_if] = ACTIONS(1491),
    [anon_sym_switch] = ACTIONS(1494),
    [anon_sym_case] = ACTIONS(1497),
    [anon_sym_default] = ACTIONS(1500),
    [anon_sym_while] = ACTIONS(1503),
    [anon_sym_do] = ACTIONS(1506),
    [anon_sym_for] = ACTIONS(1509),
    [anon_sym_return] = ACTIONS(1512),
    [anon_sym_break] = ACTIONS(1515),
    [anon_sym_continue] = ACTIONS(1518),
    [anon_sym_goto] = ACTIONS(1521),
    [anon_sym___try] = ACTIONS(1524),
    [anon_sym___leave] = ACTIONS(1527),
    [anon_sym_DASH_DASH] = ACTIONS(1449),
    [anon_sym_PLUS_PLUS] = ACTIONS(1449),
    [anon_sym_sizeof] = ACTIONS(1452),
    [anon_sym___alignof__] = ACTIONS(1455),
    [anon_sym___alignof] = ACTIONS(1455),
    [anon_sym__alignof] = ACTIONS(1455),
    [anon_sym_alignof] = ACTIONS(1455),
    [anon_sym__Alignof] = ACTIONS(1455),
    [anon_sym_offsetof] = ACTIONS(1458),
    [anon_sym__Generic] = ACTIONS(1461),
    [anon_sym_asm] = ACTIONS(1464),
    [anon_sym___asm__] = ACTIONS(1464),
    [sym_number_literal] = ACTIONS(1467),
    [anon_sym_L_SQUOTE] = ACTIONS(1470),
    [anon_sym_u_SQUOTE] = ACTIONS(1470),
    [anon_sym_U_SQUOTE] = ACTIONS(1470),
    [anon_sym_u8_SQUOTE] = ACTIONS(1470),
    [anon_sym_SQUOTE] = ACTIONS(1470),
    [anon_sym_L_DQUOTE] = ACTIONS(1473),
    [anon_sym_u_DQUOTE] = ACTIONS(1473),
    [anon_sym_U_DQUOTE] = ACTIONS(1473),
    [anon_sym_u8_DQUOTE] = ACTIONS(1473),
    [anon_sym_DQUOTE] = ACTIONS(1473),
    [sym_true] = ACTIONS(1476),
    [sym_false] = ACTIONS(1476),
    [anon_sym_NULL] = ACTIONS(1479),
    [anon_sym_nullptr] = ACTIONS(1479),
    [sym_comment] = ACTIONS(3),
  },
  [324] = {
    [sym_identifier] = ACTIONS(1268),
    [aux_sym_preproc_include_token1] = ACTIONS(1268),
    [aux_sym_preproc_def_token1] = ACTIONS(1268),
    [aux_sym_preproc_if_token1] = ACTIONS(1268),
    [aux_sym_preproc_if_token2] = ACTIONS(1268),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1268),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1268),
    [sym_preproc_directive] = ACTIONS(1268),
    [anon_sym_LPAREN2] = ACTIONS(1270),
    [anon_sym_BANG] = ACTIONS(1270),
    [anon_sym_TILDE] = ACTIONS(1270),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1270),
    [anon_sym_AMP] = ACTIONS(1270),
    [anon_sym_SEMI] = ACTIONS(1270),
    [anon_sym___extension__] = ACTIONS(1268),
    [anon_sym_typedef] = ACTIONS(1268),
    [anon_sym_extern] = ACTIONS(1268),
    [anon_sym___attribute__] = ACTIONS(1268),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1270),
    [anon_sym___declspec] = ACTIONS(1268),
    [anon_sym___cdecl] = ACTIONS(1268),
    [anon_sym___clrcall] = ACTIONS(1268),
    [anon_sym___stdcall] = ACTIONS(1268),
    [anon_sym___fastcall] = ACTIONS(1268),
    [anon_sym___thiscall] = ACTIONS(1268),
    [anon_sym___vectorcall] = ACTIONS(1268),
    [anon_sym_LBRACE] = ACTIONS(1270),
    [anon_sym_signed] = ACTIONS(1268),
    [anon_sym_unsigned] = ACTIONS(1268),
    [anon_sym_long] = ACTIONS(1268),
    [anon_sym_short] = ACTIONS(1268),
    [anon_sym_static] = ACTIONS(1268),
    [anon_sym_auto] = ACTIONS(1268),
    [anon_sym_register] = ACTIONS(1268),
    [anon_sym_inline] = ACTIONS(1268),
    [anon_sym___inline] = ACTIONS(1268),
    [anon_sym___inline__] = ACTIONS(1268),
    [anon_sym___forceinline] = ACTIONS(1268),
    [anon_sym_thread_local] = ACTIONS(1268),
    [anon_sym___thread] = ACTIONS(1268),
    [anon_sym_const] = ACTIONS(1268),
    [anon_sym_constexpr] = ACTIONS(1268),
    [anon_sym_volatile] = ACTIONS(1268),
    [anon_sym_restrict] = ACTIONS(1268),
    [anon_sym___restrict__] = ACTIONS(1268),
    [anon_sym__Atomic] = ACTIONS(1268),
    [anon_sym__Noreturn] = ACTIONS(1268),
    [anon_sym_noreturn] = ACTIONS(1268),
    [sym_primitive_type] = ACTIONS(1268),
    [anon_sym_enum] = ACTIONS(1268),
    [anon_sym_struct] = ACTIONS(1268),
    [anon_sym_union] = ACTIONS(1268),
    [anon_sym_if] = ACTIONS(1268),
    [anon_sym_switch] = ACTIONS(1268),
    [anon_sym_case] = ACTIONS(1268),
    [anon_sym_default] = ACTIONS(1268),
    [anon_sym_while] = ACTIONS(1268),
    [anon_sym_do] = ACTIONS(1268),
    [anon_sym_for] = ACTIONS(1268),
    [anon_sym_return] = ACTIONS(1268),
    [anon_sym_break] = ACTIONS(1268),
    [anon_sym_continue] = ACTIONS(1268),
    [anon_sym_goto] = ACTIONS(1268),
    [anon_sym___try] = ACTIONS(1268),
    [anon_sym___leave] = ACTIONS(1268),
    [anon_sym_DASH_DASH] = ACTIONS(1270),
    [anon_sym_PLUS_PLUS] = ACTIONS(1270),
    [anon_sym_sizeof] = ACTIONS(1268),
    [anon_sym___alignof__] = ACTIONS(1268),
    [anon_sym___alignof] = ACTIONS(1268),
    [anon_sym__alignof] = ACTIONS(1268),
    [anon_sym_alignof] = ACTIONS(1268),
    [anon_sym__Alignof] = ACTIONS(1268),
    [anon_sym_offsetof] = ACTIONS(1268),
    [anon_sym__Generic] = ACTIONS(1268),
    [anon_sym_asm] = ACTIONS(1268),
    [anon_sym___asm__] = ACTIONS(1268),
    [sym_number_literal] = ACTIONS(1270),
    [anon_sym_L_SQUOTE] = ACTIONS(1270),
    [anon_sym_u_SQUOTE] = ACTIONS(1270),
    [anon_sym_U_SQUOTE] = ACTIONS(1270),
    [anon_sym_u8_SQUOTE] = ACTIONS(1270),
    [anon_sym_SQUOTE] = ACTIONS(1270),
    [anon_sym_L_DQUOTE] = ACTIONS(1270),
    [anon_sym_u_DQUOTE] = ACTIONS(1270),
    [anon_sym_U_DQUOTE] = ACTIONS(1270),
    [anon_sym_u8_DQUOTE] = ACTIONS(1270),
    [anon_sym_DQUOTE] = ACTIONS(1270),
    [sym_true] = ACTIONS(1268),
    [sym_false] = ACTIONS(1268),
    [anon_sym_NULL] = ACTIONS(1268),
    [anon_sym_nullptr] = ACTIONS(1268),
    [sym_comment] = ACTIONS(3),
  },
  [325] = {
    [sym_identifier] = ACTIONS(1276),
    [aux_sym_preproc_include_token1] = ACTIONS(1276),
    [aux_sym_preproc_def_token1] = ACTIONS(1276),
    [aux_sym_preproc_if_token1] = ACTIONS(1276),
    [aux_sym_preproc_if_token2] = ACTIONS(1276),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1276),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1276),
    [sym_preproc_directive] = ACTIONS(1276),
    [anon_sym_LPAREN2] = ACTIONS(1278),
    [anon_sym_BANG] = ACTIONS(1278),
    [anon_sym_TILDE] = ACTIONS(1278),
    [anon_sym_DASH] = ACTIONS(1276),
    [anon_sym_PLUS] = ACTIONS(1276),
    [anon_sym_STAR] = ACTIONS(1278),
    [anon_sym_AMP] = ACTIONS(1278),
    [anon_sym_SEMI] = ACTIONS(1278),
    [anon_sym___extension__] = ACTIONS(1276),
    [anon_sym_typedef] = ACTIONS(1276),
    [anon_sym_extern] = ACTIONS(1276),
    [anon_sym___attribute__] = ACTIONS(1276),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1278),
    [anon_sym___declspec] = ACTIONS(1276),
    [anon_sym___cdecl] = ACTIONS(1276),
    [anon_sym___clrcall] = ACTIONS(1276),
    [anon_sym___stdcall] = ACTIONS(1276),
    [anon_sym___fastcall] = ACTIONS(1276),
    [anon_sym___thiscall] = ACTIONS(1276),
    [anon_sym___vectorcall] = ACTIONS(1276),
    [anon_sym_LBRACE] = ACTIONS(1278),
    [anon_sym_signed] = ACTIONS(1276),
    [anon_sym_unsigned] = ACTIONS(1276),
    [anon_sym_long] = ACTIONS(1276),
    [anon_sym_short] = ACTIONS(1276),
    [anon_sym_static] = ACTIONS(1276),
    [anon_sym_auto] = ACTIONS(1276),
    [anon_sym_register] = ACTIONS(1276),
    [anon_sym_inline] = ACTIONS(1276),
    [anon_sym___inline] = ACTIONS(1276),
    [anon_sym___inline__] = ACTIONS(1276),
    [anon_sym___forceinline] = ACTIONS(1276),
    [anon_sym_thread_local] = ACTIONS(1276),
    [anon_sym___thread] = ACTIONS(1276),
    [anon_sym_const] = ACTIONS(1276),
    [anon_sym_constexpr] = ACTIONS(1276),
    [anon_sym_volatile] = ACTIONS(1276),
    [anon_sym_restrict] = ACTIONS(1276),
    [anon_sym___restrict__] = ACTIONS(1276),
    [anon_sym__Atomic] = ACTIONS(1276),
    [anon_sym__Noreturn] = ACTIONS(1276),
    [anon_sym_noreturn] = ACTIONS(1276),
    [sym_primitive_type] = ACTIONS(1276),
    [anon_sym_enum] = ACTIONS(1276),
    [anon_sym_struct] = ACTIONS(1276),
    [anon_sym_union] = ACTIONS(1276),
    [anon_sym_if] = ACTIONS(1276),
    [anon_sym_switch] = ACTIONS(1276),
    [anon_sym_case] = ACTIONS(1276),
    [anon_sym_default] = ACTIONS(1276),
    [anon_sym_while] = ACTIONS(1276),
    [anon_sym_do] = ACTIONS(1276),
    [anon_sym_for] = ACTIONS(1276),
    [anon_sym_return] = ACTIONS(1276),
    [anon_sym_break] = ACTIONS(1276),
    [anon_sym_continue] = ACTIONS(1276),
    [anon_sym_goto] = ACTIONS(1276),
    [anon_sym___try] = ACTIONS(1276),
    [anon_sym___leave] = ACTIONS(1276),
    [anon_sym_DASH_DASH] = ACTIONS(1278),
    [anon_sym_PLUS_PLUS] = ACTIONS(1278),
    [anon_sym_sizeof] = ACTIONS(1276),
    [anon_sym___alignof__] = ACTIONS(1276),
    [anon_sym___alignof] = ACTIONS(1276),
    [anon_sym__alignof] = ACTIONS(1276),
    [anon_sym_alignof] = ACTIONS(1276),
    [anon_sym__Alignof] = ACTIONS(1276),
    [anon_sym_offsetof] = ACTIONS(1276),
    [anon_sym__Generic] = ACTIONS(1276),
    [anon_sym_asm] = ACTIONS(1276),
    [anon_sym___asm__] = ACTIONS(1276),
    [sym_number_literal] = ACTIONS(1278),
    [anon_sym_L_SQUOTE] = ACTIONS(1278),
    [anon_sym_u_SQUOTE] = ACTIONS(1278),
    [anon_sym_U_SQUOTE] = ACTIONS(1278),
    [anon_sym_u8_SQUOTE] = ACTIONS(1278),
    [anon_sym_SQUOTE] = ACTIONS(1278),
    [anon_sym_L_DQUOTE] = ACTIONS(1278),
    [anon_sym_u_DQUOTE] = ACTIONS(1278),
    [anon_sym_U_DQUOTE] = ACTIONS(1278),
    [anon_sym_u8_DQUOTE] = ACTIONS(1278),
    [anon_sym_DQUOTE] = ACTIONS(1278),
    [sym_true] = ACTIONS(1276),
    [sym_false] = ACTIONS(1276),
    [anon_sym_NULL] = ACTIONS(1276),
    [anon_sym_nullptr] = ACTIONS(1276),
    [sym_comment] = ACTIONS(3),
  },
  [326] = {
    [sym_identifier] = ACTIONS(1280),
    [aux_sym_preproc_include_token1] = ACTIONS(1280),
    [aux_sym_preproc_def_token1] = ACTIONS(1280),
    [aux_sym_preproc_if_token1] = ACTIONS(1280),
    [aux_sym_preproc_if_token2] = ACTIONS(1280),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1280),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1280),
    [sym_preproc_directive] = ACTIONS(1280),
    [anon_sym_LPAREN2] = ACTIONS(1282),
    [anon_sym_BANG] = ACTIONS(1282),
    [anon_sym_TILDE] = ACTIONS(1282),
    [anon_sym_DASH] = ACTIONS(1280),
    [anon_sym_PLUS] = ACTIONS(1280),
    [anon_sym_STAR] = ACTIONS(1282),
    [anon_sym_AMP] = ACTIONS(1282),
    [anon_sym_SEMI] = ACTIONS(1282),
    [anon_sym___extension__] = ACTIONS(1280),
    [anon_sym_typedef] = ACTIONS(1280),
    [anon_sym_extern] = ACTIONS(1280),
    [anon_sym___attribute__] = ACTIONS(1280),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1282),
    [anon_sym___declspec] = ACTIONS(1280),
    [anon_sym___cdecl] = ACTIONS(1280),
    [anon_sym___clrcall] = ACTIONS(1280),
    [anon_sym___stdcall] = ACTIONS(1280),
    [anon_sym___fastcall] = ACTIONS(1280),
    [anon_sym___thiscall] = ACTIONS(1280),
    [anon_sym___vectorcall] = ACTIONS(1280),
    [anon_sym_LBRACE] = ACTIONS(1282),
    [anon_sym_signed] = ACTIONS(1280),
    [anon_sym_unsigned] = ACTIONS(1280),
    [anon_sym_long] = ACTIONS(1280),
    [anon_sym_short] = ACTIONS(1280),
    [anon_sym_static] = ACTIONS(1280),
    [anon_sym_auto] = ACTIONS(1280),
    [anon_sym_register] = ACTIONS(1280),
    [anon_sym_inline] = ACTIONS(1280),
    [anon_sym___inline] = ACTIONS(1280),
    [anon_sym___inline__] = ACTIONS(1280),
    [anon_sym___forceinline] = ACTIONS(1280),
    [anon_sym_thread_local] = ACTIONS(1280),
    [anon_sym___thread] = ACTIONS(1280),
    [anon_sym_const] = ACTIONS(1280),
    [anon_sym_constexpr] = ACTIONS(1280),
    [anon_sym_volatile] = ACTIONS(1280),
    [anon_sym_restrict] = ACTIONS(1280),
    [anon_sym___restrict__] = ACTIONS(1280),
    [anon_sym__Atomic] = ACTIONS(1280),
    [anon_sym__Noreturn] = ACTIONS(1280),
    [anon_sym_noreturn] = ACTIONS(1280),
    [sym_primitive_type] = ACTIONS(1280),
    [anon_sym_enum] = ACTIONS(1280),
    [anon_sym_struct] = ACTIONS(1280),
    [anon_sym_union] = ACTIONS(1280),
    [anon_sym_if] = ACTIONS(1280),
    [anon_sym_switch] = ACTIONS(1280),
    [anon_sym_case] = ACTIONS(1280),
    [anon_sym_default] = ACTIONS(1280),
    [anon_sym_while] = ACTIONS(1280),
    [anon_sym_do] = ACTIONS(1280),
    [anon_sym_for] = ACTIONS(1280),
    [anon_sym_return] = ACTIONS(1280),
    [anon_sym_break] = ACTIONS(1280),
    [anon_sym_continue] = ACTIONS(1280),
    [anon_sym_goto] = ACTIONS(1280),
    [anon_sym___try] = ACTIONS(1280),
    [anon_sym___leave] = ACTIONS(1280),
    [anon_sym_DASH_DASH] = ACTIONS(1282),
    [anon_sym_PLUS_PLUS] = ACTIONS(1282),
    [anon_sym_sizeof] = ACTIONS(1280),
    [anon_sym___alignof__] = ACTIONS(1280),
    [anon_sym___alignof] = ACTIONS(1280),
    [anon_sym__alignof] = ACTIONS(1280),
    [anon_sym_alignof] = ACTIONS(1280),
    [anon_sym__Alignof] = ACTIONS(1280),
    [anon_sym_offsetof] = ACTIONS(1280),
    [anon_sym__Generic] = ACTIONS(1280),
    [anon_sym_asm] = ACTIONS(1280),
    [anon_sym___asm__] = ACTIONS(1280),
    [sym_number_literal] = ACTIONS(1282),
    [anon_sym_L_SQUOTE] = ACTIONS(1282),
    [anon_sym_u_SQUOTE] = ACTIONS(1282),
    [anon_sym_U_SQUOTE] = ACTIONS(1282),
    [anon_sym_u8_SQUOTE] = ACTIONS(1282),
    [anon_sym_SQUOTE] = ACTIONS(1282),
    [anon_sym_L_DQUOTE] = ACTIONS(1282),
    [anon_sym_u_DQUOTE] = ACTIONS(1282),
    [anon_sym_U_DQUOTE] = ACTIONS(1282),
    [anon_sym_u8_DQUOTE] = ACTIONS(1282),
    [anon_sym_DQUOTE] = ACTIONS(1282),
    [sym_true] = ACTIONS(1280),
    [sym_false] = ACTIONS(1280),
    [anon_sym_NULL] = ACTIONS(1280),
    [anon_sym_nullptr] = ACTIONS(1280),
    [sym_comment] = ACTIONS(3),
  },
  [327] = {
    [sym_identifier] = ACTIONS(1292),
    [aux_sym_preproc_include_token1] = ACTIONS(1292),
    [aux_sym_preproc_def_token1] = ACTIONS(1292),
    [aux_sym_preproc_if_token1] = ACTIONS(1292),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1292),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1292),
    [sym_preproc_directive] = ACTIONS(1292),
    [anon_sym_LPAREN2] = ACTIONS(1294),
    [anon_sym_BANG] = ACTIONS(1294),
    [anon_sym_TILDE] = ACTIONS(1294),
    [anon_sym_DASH] = ACTIONS(1292),
    [anon_sym_PLUS] = ACTIONS(1292),
    [anon_sym_STAR] = ACTIONS(1294),
    [anon_sym_AMP] = ACTIONS(1294),
    [anon_sym_SEMI] = ACTIONS(1294),
    [anon_sym___extension__] = ACTIONS(1292),
    [anon_sym_typedef] = ACTIONS(1292),
    [anon_sym_extern] = ACTIONS(1292),
    [anon_sym___attribute__] = ACTIONS(1292),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1294),
    [anon_sym___declspec] = ACTIONS(1292),
    [anon_sym___cdecl] = ACTIONS(1292),
    [anon_sym___clrcall] = ACTIONS(1292),
    [anon_sym___stdcall] = ACTIONS(1292),
    [anon_sym___fastcall] = ACTIONS(1292),
    [anon_sym___thiscall] = ACTIONS(1292),
    [anon_sym___vectorcall] = ACTIONS(1292),
    [anon_sym_LBRACE] = ACTIONS(1294),
    [anon_sym_RBRACE] = ACTIONS(1294),
    [anon_sym_signed] = ACTIONS(1292),
    [anon_sym_unsigned] = ACTIONS(1292),
    [anon_sym_long] = ACTIONS(1292),
    [anon_sym_short] = ACTIONS(1292),
    [anon_sym_static] = ACTIONS(1292),
    [anon_sym_auto] = ACTIONS(1292),
    [anon_sym_register] = ACTIONS(1292),
    [anon_sym_inline] = ACTIONS(1292),
    [anon_sym___inline] = ACTIONS(1292),
    [anon_sym___inline__] = ACTIONS(1292),
    [anon_sym___forceinline] = ACTIONS(1292),
    [anon_sym_thread_local] = ACTIONS(1292),
    [anon_sym___thread] = ACTIONS(1292),
    [anon_sym_const] = ACTIONS(1292),
    [anon_sym_constexpr] = ACTIONS(1292),
    [anon_sym_volatile] = ACTIONS(1292),
    [anon_sym_restrict] = ACTIONS(1292),
    [anon_sym___restrict__] = ACTIONS(1292),
    [anon_sym__Atomic] = ACTIONS(1292),
    [anon_sym__Noreturn] = ACTIONS(1292),
    [anon_sym_noreturn] = ACTIONS(1292),
    [sym_primitive_type] = ACTIONS(1292),
    [anon_sym_enum] = ACTIONS(1292),
    [anon_sym_struct] = ACTIONS(1292),
    [anon_sym_union] = ACTIONS(1292),
    [anon_sym_if] = ACTIONS(1292),
    [anon_sym_switch] = ACTIONS(1292),
    [anon_sym_case] = ACTIONS(1292),
    [anon_sym_default] = ACTIONS(1292),
    [anon_sym_while] = ACTIONS(1292),
    [anon_sym_do] = ACTIONS(1292),
    [anon_sym_for] = ACTIONS(1292),
    [anon_sym_return] = ACTIONS(1292),
    [anon_sym_break] = ACTIONS(1292),
    [anon_sym_continue] = ACTIONS(1292),
    [anon_sym_goto] = ACTIONS(1292),
    [anon_sym___try] = ACTIONS(1292),
    [anon_sym___leave] = ACTIONS(1292),
    [anon_sym_DASH_DASH] = ACTIONS(1294),
    [anon_sym_PLUS_PLUS] = ACTIONS(1294),
    [anon_sym_sizeof] = ACTIONS(1292),
    [anon_sym___alignof__] = ACTIONS(1292),
    [anon_sym___alignof] = ACTIONS(1292),
    [anon_sym__alignof] = ACTIONS(1292),
    [anon_sym_alignof] = ACTIONS(1292),
    [anon_sym__Alignof] = ACTIONS(1292),
    [anon_sym_offsetof] = ACTIONS(1292),
    [anon_sym__Generic] = ACTIONS(1292),
    [anon_sym_asm] = ACTIONS(1292),
    [anon_sym___asm__] = ACTIONS(1292),
    [sym_number_literal] = ACTIONS(1294),
    [anon_sym_L_SQUOTE] = ACTIONS(1294),
    [anon_sym_u_SQUOTE] = ACTIONS(1294),
    [anon_sym_U_SQUOTE] = ACTIONS(1294),
    [anon_sym_u8_SQUOTE] = ACTIONS(1294),
    [anon_sym_SQUOTE] = ACTIONS(1294),
    [anon_sym_L_DQUOTE] = ACTIONS(1294),
    [anon_sym_u_DQUOTE] = ACTIONS(1294),
    [anon_sym_U_DQUOTE] = ACTIONS(1294),
    [anon_sym_u8_DQUOTE] = ACTIONS(1294),
    [anon_sym_DQUOTE] = ACTIONS(1294),
    [sym_true] = ACTIONS(1292),
    [sym_false] = ACTIONS(1292),
    [anon_sym_NULL] = ACTIONS(1292),
    [anon_sym_nullptr] = ACTIONS(1292),
    [sym_comment] = ACTIONS(3),
  },
  [328] = {
    [sym_identifier] = ACTIONS(1236),
    [aux_sym_preproc_include_token1] = ACTIONS(1236),
    [aux_sym_preproc_def_token1] = ACTIONS(1236),
    [aux_sym_preproc_if_token1] = ACTIONS(1236),
    [aux_sym_preproc_if_token2] = ACTIONS(1236),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1236),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1236),
    [sym_preproc_directive] = ACTIONS(1236),
    [anon_sym_LPAREN2] = ACTIONS(1238),
    [anon_sym_BANG] = ACTIONS(1238),
    [anon_sym_TILDE] = ACTIONS(1238),
    [anon_sym_DASH] = ACTIONS(1236),
    [anon_sym_PLUS] = ACTIONS(1236),
    [anon_sym_STAR] = ACTIONS(1238),
    [anon_sym_AMP] = ACTIONS(1238),
    [anon_sym_SEMI] = ACTIONS(1238),
    [anon_sym___extension__] = ACTIONS(1236),
    [anon_sym_typedef] = ACTIONS(1236),
    [anon_sym_extern] = ACTIONS(1236),
    [anon_sym___attribute__] = ACTIONS(1236),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1238),
    [anon_sym___declspec] = ACTIONS(1236),
    [anon_sym___cdecl] = ACTIONS(1236),
    [anon_sym___clrcall] = ACTIONS(1236),
    [anon_sym___stdcall] = ACTIONS(1236),
    [anon_sym___fastcall] = ACTIONS(1236),
    [anon_sym___thiscall] = ACTIONS(1236),
    [anon_sym___vectorcall] = ACTIONS(1236),
    [anon_sym_LBRACE] = ACTIONS(1238),
    [anon_sym_signed] = ACTIONS(1236),
    [anon_sym_unsigned] = ACTIONS(1236),
    [anon_sym_long] = ACTIONS(1236),
    [anon_sym_short] = ACTIONS(1236),
    [anon_sym_static] = ACTIONS(1236),
    [anon_sym_auto] = ACTIONS(1236),
    [anon_sym_register] = ACTIONS(1236),
    [anon_sym_inline] = ACTIONS(1236),
    [anon_sym___inline] = ACTIONS(1236),
    [anon_sym___inline__] = ACTIONS(1236),
    [anon_sym___forceinline] = ACTIONS(1236),
    [anon_sym_thread_local] = ACTIONS(1236),
    [anon_sym___thread] = ACTIONS(1236),
    [anon_sym_const] = ACTIONS(1236),
    [anon_sym_constexpr] = ACTIONS(1236),
    [anon_sym_volatile] = ACTIONS(1236),
    [anon_sym_restrict] = ACTIONS(1236),
    [anon_sym___restrict__] = ACTIONS(1236),
    [anon_sym__Atomic] = ACTIONS(1236),
    [anon_sym__Noreturn] = ACTIONS(1236),
    [anon_sym_noreturn] = ACTIONS(1236),
    [sym_primitive_type] = ACTIONS(1236),
    [anon_sym_enum] = ACTIONS(1236),
    [anon_sym_struct] = ACTIONS(1236),
    [anon_sym_union] = ACTIONS(1236),
    [anon_sym_if] = ACTIONS(1236),
    [anon_sym_switch] = ACTIONS(1236),
    [anon_sym_case] = ACTIONS(1236),
    [anon_sym_default] = ACTIONS(1236),
    [anon_sym_while] = ACTIONS(1236),
    [anon_sym_do] = ACTIONS(1236),
    [anon_sym_for] = ACTIONS(1236),
    [anon_sym_return] = ACTIONS(1236),
    [anon_sym_break] = ACTIONS(1236),
    [anon_sym_continue] = ACTIONS(1236),
    [anon_sym_goto] = ACTIONS(1236),
    [anon_sym___try] = ACTIONS(1236),
    [anon_sym___leave] = ACTIONS(1236),
    [anon_sym_DASH_DASH] = ACTIONS(1238),
    [anon_sym_PLUS_PLUS] = ACTIONS(1238),
    [anon_sym_sizeof] = ACTIONS(1236),
    [anon_sym___alignof__] = ACTIONS(1236),
    [anon_sym___alignof] = ACTIONS(1236),
    [anon_sym__alignof] = ACTIONS(1236),
    [anon_sym_alignof] = ACTIONS(1236),
    [anon_sym__Alignof] = ACTIONS(1236),
    [anon_sym_offsetof] = ACTIONS(1236),
    [anon_sym__Generic] = ACTIONS(1236),
    [anon_sym_asm] = ACTIONS(1236),
    [anon_sym___asm__] = ACTIONS(1236),
    [sym_number_literal] = ACTIONS(1238),
    [anon_sym_L_SQUOTE] = ACTIONS(1238),
    [anon_sym_u_SQUOTE] = ACTIONS(1238),
    [anon_sym_U_SQUOTE] = ACTIONS(1238),
    [anon_sym_u8_SQUOTE] = ACTIONS(1238),
    [anon_sym_SQUOTE] = ACTIONS(1238),
    [anon_sym_L_DQUOTE] = ACTIONS(1238),
    [anon_sym_u_DQUOTE] = ACTIONS(1238),
    [anon_sym_U_DQUOTE] = ACTIONS(1238),
    [anon_sym_u8_DQUOTE] = ACTIONS(1238),
    [anon_sym_DQUOTE] = ACTIONS(1238),
    [sym_true] = ACTIONS(1236),
    [sym_false] = ACTIONS(1236),
    [anon_sym_NULL] = ACTIONS(1236),
    [anon_sym_nullptr] = ACTIONS(1236),
    [sym_comment] = ACTIONS(3),
  },
  [329] = {
    [sym_identifier] = ACTIONS(1328),
    [aux_sym_preproc_include_token1] = ACTIONS(1328),
    [aux_sym_preproc_def_token1] = ACTIONS(1328),
    [aux_sym_preproc_if_token1] = ACTIONS(1328),
    [aux_sym_preproc_if_token2] = ACTIONS(1328),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1328),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1328),
    [sym_preproc_directive] = ACTIONS(1328),
    [anon_sym_LPAREN2] = ACTIONS(1330),
    [anon_sym_BANG] = ACTIONS(1330),
    [anon_sym_TILDE] = ACTIONS(1330),
    [anon_sym_DASH] = ACTIONS(1328),
    [anon_sym_PLUS] = ACTIONS(1328),
    [anon_sym_STAR] = ACTIONS(1330),
    [anon_sym_AMP] = ACTIONS(1330),
    [anon_sym_SEMI] = ACTIONS(1330),
    [anon_sym___extension__] = ACTIONS(1328),
    [anon_sym_typedef] = ACTIONS(1328),
    [anon_sym_extern] = ACTIONS(1328),
    [anon_sym___attribute__] = ACTIONS(1328),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1330),
    [anon_sym___declspec] = ACTIONS(1328),
    [anon_sym___cdecl] = ACTIONS(1328),
    [anon_sym___clrcall] = ACTIONS(1328),
    [anon_sym___stdcall] = ACTIONS(1328),
    [anon_sym___fastcall] = ACTIONS(1328),
    [anon_sym___thiscall] = ACTIONS(1328),
    [anon_sym___vectorcall] = ACTIONS(1328),
    [anon_sym_LBRACE] = ACTIONS(1330),
    [anon_sym_signed] = ACTIONS(1328),
    [anon_sym_unsigned] = ACTIONS(1328),
    [anon_sym_long] = ACTIONS(1328),
    [anon_sym_short] = ACTIONS(1328),
    [anon_sym_static] = ACTIONS(1328),
    [anon_sym_auto] = ACTIONS(1328),
    [anon_sym_register] = ACTIONS(1328),
    [anon_sym_inline] = ACTIONS(1328),
    [anon_sym___inline] = ACTIONS(1328),
    [anon_sym___inline__] = ACTIONS(1328),
    [anon_sym___forceinline] = ACTIONS(1328),
    [anon_sym_thread_local] = ACTIONS(1328),
    [anon_sym___thread] = ACTIONS(1328),
    [anon_sym_const] = ACTIONS(1328),
    [anon_sym_constexpr] = ACTIONS(1328),
    [anon_sym_volatile] = ACTIONS(1328),
    [anon_sym_restrict] = ACTIONS(1328),
    [anon_sym___restrict__] = ACTIONS(1328),
    [anon_sym__Atomic] = ACTIONS(1328),
    [anon_sym__Noreturn] = ACTIONS(1328),
    [anon_sym_noreturn] = ACTIONS(1328),
    [sym_primitive_type] = ACTIONS(1328),
    [anon_sym_enum] = ACTIONS(1328),
    [anon_sym_struct] = ACTIONS(1328),
    [anon_sym_union] = ACTIONS(1328),
    [anon_sym_if] = ACTIONS(1328),
    [anon_sym_switch] = ACTIONS(1328),
    [anon_sym_case] = ACTIONS(1328),
    [anon_sym_default] = ACTIONS(1328),
    [anon_sym_while] = ACTIONS(1328),
    [anon_sym_do] = ACTIONS(1328),
    [anon_sym_for] = ACTIONS(1328),
    [anon_sym_return] = ACTIONS(1328),
    [anon_sym_break] = ACTIONS(1328),
    [anon_sym_continue] = ACTIONS(1328),
    [anon_sym_goto] = ACTIONS(1328),
    [anon_sym___try] = ACTIONS(1328),
    [anon_sym___leave] = ACTIONS(1328),
    [anon_sym_DASH_DASH] = ACTIONS(1330),
    [anon_sym_PLUS_PLUS] = ACTIONS(1330),
    [anon_sym_sizeof] = ACTIONS(1328),
    [anon_sym___alignof__] = ACTIONS(1328),
    [anon_sym___alignof] = ACTIONS(1328),
    [anon_sym__alignof] = ACTIONS(1328),
    [anon_sym_alignof] = ACTIONS(1328),
    [anon_sym__Alignof] = ACTIONS(1328),
    [anon_sym_offsetof] = ACTIONS(1328),
    [anon_sym__Generic] = ACTIONS(1328),
    [anon_sym_asm] = ACTIONS(1328),
    [anon_sym___asm__] = ACTIONS(1328),
    [sym_number_literal] = ACTIONS(1330),
    [anon_sym_L_SQUOTE] = ACTIONS(1330),
    [anon_sym_u_SQUOTE] = ACTIONS(1330),
    [anon_sym_U_SQUOTE] = ACTIONS(1330),
    [anon_sym_u8_SQUOTE] = ACTIONS(1330),
    [anon_sym_SQUOTE] = ACTIONS(1330),
    [anon_sym_L_DQUOTE] = ACTIONS(1330),
    [anon_sym_u_DQUOTE] = ACTIONS(1330),
    [anon_sym_U_DQUOTE] = ACTIONS(1330),
    [anon_sym_u8_DQUOTE] = ACTIONS(1330),
    [anon_sym_DQUOTE] = ACTIONS(1330),
    [sym_true] = ACTIONS(1328),
    [sym_false] = ACTIONS(1328),
    [anon_sym_NULL] = ACTIONS(1328),
    [anon_sym_nullptr] = ACTIONS(1328),
    [sym_comment] = ACTIONS(3),
  },
  [330] = {
    [sym_identifier] = ACTIONS(1332),
    [aux_sym_preproc_include_token1] = ACTIONS(1332),
    [aux_sym_preproc_def_token1] = ACTIONS(1332),
    [aux_sym_preproc_if_token1] = ACTIONS(1332),
    [aux_sym_preproc_if_token2] = ACTIONS(1332),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1332),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1332),
    [sym_preproc_directive] = ACTIONS(1332),
    [anon_sym_LPAREN2] = ACTIONS(1334),
    [anon_sym_BANG] = ACTIONS(1334),
    [anon_sym_TILDE] = ACTIONS(1334),
    [anon_sym_DASH] = ACTIONS(1332),
    [anon_sym_PLUS] = ACTIONS(1332),
    [anon_sym_STAR] = ACTIONS(1334),
    [anon_sym_AMP] = ACTIONS(1334),
    [anon_sym_SEMI] = ACTIONS(1334),
    [anon_sym___extension__] = ACTIONS(1332),
    [anon_sym_typedef] = ACTIONS(1332),
    [anon_sym_extern] = ACTIONS(1332),
    [anon_sym___attribute__] = ACTIONS(1332),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1334),
    [anon_sym___declspec] = ACTIONS(1332),
    [anon_sym___cdecl] = ACTIONS(1332),
    [anon_sym___clrcall] = ACTIONS(1332),
    [anon_sym___stdcall] = ACTIONS(1332),
    [anon_sym___fastcall] = ACTIONS(1332),
    [anon_sym___thiscall] = ACTIONS(1332),
    [anon_sym___vectorcall] = ACTIONS(1332),
    [anon_sym_LBRACE] = ACTIONS(1334),
    [anon_sym_signed] = ACTIONS(1332),
    [anon_sym_unsigned] = ACTIONS(1332),
    [anon_sym_long] = ACTIONS(1332),
    [anon_sym_short] = ACTIONS(1332),
    [anon_sym_static] = ACTIONS(1332),
    [anon_sym_auto] = ACTIONS(1332),
    [anon_sym_register] = ACTIONS(1332),
    [anon_sym_inline] = ACTIONS(1332),
    [anon_sym___inline] = ACTIONS(1332),
    [anon_sym___inline__] = ACTIONS(1332),
    [anon_sym___forceinline] = ACTIONS(1332),
    [anon_sym_thread_local] = ACTIONS(1332),
    [anon_sym___thread] = ACTIONS(1332),
    [anon_sym_const] = ACTIONS(1332),
    [anon_sym_constexpr] = ACTIONS(1332),
    [anon_sym_volatile] = ACTIONS(1332),
    [anon_sym_restrict] = ACTIONS(1332),
    [anon_sym___restrict__] = ACTIONS(1332),
    [anon_sym__Atomic] = ACTIONS(1332),
    [anon_sym__Noreturn] = ACTIONS(1332),
    [anon_sym_noreturn] = ACTIONS(1332),
    [sym_primitive_type] = ACTIONS(1332),
    [anon_sym_enum] = ACTIONS(1332),
    [anon_sym_struct] = ACTIONS(1332),
    [anon_sym_union] = ACTIONS(1332),
    [anon_sym_if] = ACTIONS(1332),
    [anon_sym_switch] = ACTIONS(1332),
    [anon_sym_case] = ACTIONS(1332),
    [anon_sym_default] = ACTIONS(1332),
    [anon_sym_while] = ACTIONS(1332),
    [anon_sym_do] = ACTIONS(1332),
    [anon_sym_for] = ACTIONS(1332),
    [anon_sym_return] = ACTIONS(1332),
    [anon_sym_break] = ACTIONS(1332),
    [anon_sym_continue] = ACTIONS(1332),
    [anon_sym_goto] = ACTIONS(1332),
    [anon_sym___try] = ACTIONS(1332),
    [anon_sym___leave] = ACTIONS(1332),
    [anon_sym_DASH_DASH] = ACTIONS(1334),
    [anon_sym_PLUS_PLUS] = ACTIONS(1334),
    [anon_sym_sizeof] = ACTIONS(1332),
    [anon_sym___alignof__] = ACTIONS(1332),
    [anon_sym___alignof] = ACTIONS(1332),
    [anon_sym__alignof] = ACTIONS(1332),
    [anon_sym_alignof] = ACTIONS(1332),
    [anon_sym__Alignof] = ACTIONS(1332),
    [anon_sym_offsetof] = ACTIONS(1332),
    [anon_sym__Generic] = ACTIONS(1332),
    [anon_sym_asm] = ACTIONS(1332),
    [anon_sym___asm__] = ACTIONS(1332),
    [sym_number_literal] = ACTIONS(1334),
    [anon_sym_L_SQUOTE] = ACTIONS(1334),
    [anon_sym_u_SQUOTE] = ACTIONS(1334),
    [anon_sym_U_SQUOTE] = ACTIONS(1334),
    [anon_sym_u8_SQUOTE] = ACTIONS(1334),
    [anon_sym_SQUOTE] = ACTIONS(1334),
    [anon_sym_L_DQUOTE] = ACTIONS(1334),
    [anon_sym_u_DQUOTE] = ACTIONS(1334),
    [anon_sym_U_DQUOTE] = ACTIONS(1334),
    [anon_sym_u8_DQUOTE] = ACTIONS(1334),
    [anon_sym_DQUOTE] = ACTIONS(1334),
    [sym_true] = ACTIONS(1332),
    [sym_false] = ACTIONS(1332),
    [anon_sym_NULL] = ACTIONS(1332),
    [anon_sym_nullptr] = ACTIONS(1332),
    [sym_comment] = ACTIONS(3),
  },
  [331] = {
    [sym_identifier] = ACTIONS(1336),
    [aux_sym_preproc_include_token1] = ACTIONS(1336),
    [aux_sym_preproc_def_token1] = ACTIONS(1336),
    [aux_sym_preproc_if_token1] = ACTIONS(1336),
    [aux_sym_preproc_if_token2] = ACTIONS(1336),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1336),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1336),
    [sym_preproc_directive] = ACTIONS(1336),
    [anon_sym_LPAREN2] = ACTIONS(1338),
    [anon_sym_BANG] = ACTIONS(1338),
    [anon_sym_TILDE] = ACTIONS(1338),
    [anon_sym_DASH] = ACTIONS(1336),
    [anon_sym_PLUS] = ACTIONS(1336),
    [anon_sym_STAR] = ACTIONS(1338),
    [anon_sym_AMP] = ACTIONS(1338),
    [anon_sym_SEMI] = ACTIONS(1338),
    [anon_sym___extension__] = ACTIONS(1336),
    [anon_sym_typedef] = ACTIONS(1336),
    [anon_sym_extern] = ACTIONS(1336),
    [anon_sym___attribute__] = ACTIONS(1336),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1338),
    [anon_sym___declspec] = ACTIONS(1336),
    [anon_sym___cdecl] = ACTIONS(1336),
    [anon_sym___clrcall] = ACTIONS(1336),
    [anon_sym___stdcall] = ACTIONS(1336),
    [anon_sym___fastcall] = ACTIONS(1336),
    [anon_sym___thiscall] = ACTIONS(1336),
    [anon_sym___vectorcall] = ACTIONS(1336),
    [anon_sym_LBRACE] = ACTIONS(1338),
    [anon_sym_signed] = ACTIONS(1336),
    [anon_sym_unsigned] = ACTIONS(1336),
    [anon_sym_long] = ACTIONS(1336),
    [anon_sym_short] = ACTIONS(1336),
    [anon_sym_static] = ACTIONS(1336),
    [anon_sym_auto] = ACTIONS(1336),
    [anon_sym_register] = ACTIONS(1336),
    [anon_sym_inline] = ACTIONS(1336),
    [anon_sym___inline] = ACTIONS(1336),
    [anon_sym___inline__] = ACTIONS(1336),
    [anon_sym___forceinline] = ACTIONS(1336),
    [anon_sym_thread_local] = ACTIONS(1336),
    [anon_sym___thread] = ACTIONS(1336),
    [anon_sym_const] = ACTIONS(1336),
    [anon_sym_constexpr] = ACTIONS(1336),
    [anon_sym_volatile] = ACTIONS(1336),
    [anon_sym_restrict] = ACTIONS(1336),
    [anon_sym___restrict__] = ACTIONS(1336),
    [anon_sym__Atomic] = ACTIONS(1336),
    [anon_sym__Noreturn] = ACTIONS(1336),
    [anon_sym_noreturn] = ACTIONS(1336),
    [sym_primitive_type] = ACTIONS(1336),
    [anon_sym_enum] = ACTIONS(1336),
    [anon_sym_struct] = ACTIONS(1336),
    [anon_sym_union] = ACTIONS(1336),
    [anon_sym_if] = ACTIONS(1336),
    [anon_sym_switch] = ACTIONS(1336),
    [anon_sym_case] = ACTIONS(1336),
    [anon_sym_default] = ACTIONS(1336),
    [anon_sym_while] = ACTIONS(1336),
    [anon_sym_do] = ACTIONS(1336),
    [anon_sym_for] = ACTIONS(1336),
    [anon_sym_return] = ACTIONS(1336),
    [anon_sym_break] = ACTIONS(1336),
    [anon_sym_continue] = ACTIONS(1336),
    [anon_sym_goto] = ACTIONS(1336),
    [anon_sym___try] = ACTIONS(1336),
    [anon_sym___leave] = ACTIONS(1336),
    [anon_sym_DASH_DASH] = ACTIONS(1338),
    [anon_sym_PLUS_PLUS] = ACTIONS(1338),
    [anon_sym_sizeof] = ACTIONS(1336),
    [anon_sym___alignof__] = ACTIONS(1336),
    [anon_sym___alignof] = ACTIONS(1336),
    [anon_sym__alignof] = ACTIONS(1336),
    [anon_sym_alignof] = ACTIONS(1336),
    [anon_sym__Alignof] = ACTIONS(1336),
    [anon_sym_offsetof] = ACTIONS(1336),
    [anon_sym__Generic] = ACTIONS(1336),
    [anon_sym_asm] = ACTIONS(1336),
    [anon_sym___asm__] = ACTIONS(1336),
    [sym_number_literal] = ACTIONS(1338),
    [anon_sym_L_SQUOTE] = ACTIONS(1338),
    [anon_sym_u_SQUOTE] = ACTIONS(1338),
    [anon_sym_U_SQUOTE] = ACTIONS(1338),
    [anon_sym_u8_SQUOTE] = ACTIONS(1338),
    [anon_sym_SQUOTE] = ACTIONS(1338),
    [anon_sym_L_DQUOTE] = ACTIONS(1338),
    [anon_sym_u_DQUOTE] = ACTIONS(1338),
    [anon_sym_U_DQUOTE] = ACTIONS(1338),
    [anon_sym_u8_DQUOTE] = ACTIONS(1338),
    [anon_sym_DQUOTE] = ACTIONS(1338),
    [sym_true] = ACTIONS(1336),
    [sym_false] = ACTIONS(1336),
    [anon_sym_NULL] = ACTIONS(1336),
    [anon_sym_nullptr] = ACTIONS(1336),
    [sym_comment] = ACTIONS(3),
  },
  [332] = {
    [sym_identifier] = ACTIONS(1344),
    [aux_sym_preproc_include_token1] = ACTIONS(1344),
    [aux_sym_preproc_def_token1] = ACTIONS(1344),
    [aux_sym_preproc_if_token1] = ACTIONS(1344),
    [aux_sym_preproc_if_token2] = ACTIONS(1344),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1344),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1344),
    [sym_preproc_directive] = ACTIONS(1344),
    [anon_sym_LPAREN2] = ACTIONS(1346),
    [anon_sym_BANG] = ACTIONS(1346),
    [anon_sym_TILDE] = ACTIONS(1346),
    [anon_sym_DASH] = ACTIONS(1344),
    [anon_sym_PLUS] = ACTIONS(1344),
    [anon_sym_STAR] = ACTIONS(1346),
    [anon_sym_AMP] = ACTIONS(1346),
    [anon_sym_SEMI] = ACTIONS(1346),
    [anon_sym___extension__] = ACTIONS(1344),
    [anon_sym_typedef] = ACTIONS(1344),
    [anon_sym_extern] = ACTIONS(1344),
    [anon_sym___attribute__] = ACTIONS(1344),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1346),
    [anon_sym___declspec] = ACTIONS(1344),
    [anon_sym___cdecl] = ACTIONS(1344),
    [anon_sym___clrcall] = ACTIONS(1344),
    [anon_sym___stdcall] = ACTIONS(1344),
    [anon_sym___fastcall] = ACTIONS(1344),
    [anon_sym___thiscall] = ACTIONS(1344),
    [anon_sym___vectorcall] = ACTIONS(1344),
    [anon_sym_LBRACE] = ACTIONS(1346),
    [anon_sym_signed] = ACTIONS(1344),
    [anon_sym_unsigned] = ACTIONS(1344),
    [anon_sym_long] = ACTIONS(1344),
    [anon_sym_short] = ACTIONS(1344),
    [anon_sym_static] = ACTIONS(1344),
    [anon_sym_auto] = ACTIONS(1344),
    [anon_sym_register] = ACTIONS(1344),
    [anon_sym_inline] = ACTIONS(1344),
    [anon_sym___inline] = ACTIONS(1344),
    [anon_sym___inline__] = ACTIONS(1344),
    [anon_sym___forceinline] = ACTIONS(1344),
    [anon_sym_thread_local] = ACTIONS(1344),
    [anon_sym___thread] = ACTIONS(1344),
    [anon_sym_const] = ACTIONS(1344),
    [anon_sym_constexpr] = ACTIONS(1344),
    [anon_sym_volatile] = ACTIONS(1344),
    [anon_sym_restrict] = ACTIONS(1344),
    [anon_sym___restrict__] = ACTIONS(1344),
    [anon_sym__Atomic] = ACTIONS(1344),
    [anon_sym__Noreturn] = ACTIONS(1344),
    [anon_sym_noreturn] = ACTIONS(1344),
    [sym_primitive_type] = ACTIONS(1344),
    [anon_sym_enum] = ACTIONS(1344),
    [anon_sym_struct] = ACTIONS(1344),
    [anon_sym_union] = ACTIONS(1344),
    [anon_sym_if] = ACTIONS(1344),
    [anon_sym_switch] = ACTIONS(1344),
    [anon_sym_case] = ACTIONS(1344),
    [anon_sym_default] = ACTIONS(1344),
    [anon_sym_while] = ACTIONS(1344),
    [anon_sym_do] = ACTIONS(1344),
    [anon_sym_for] = ACTIONS(1344),
    [anon_sym_return] = ACTIONS(1344),
    [anon_sym_break] = ACTIONS(1344),
    [anon_sym_continue] = ACTIONS(1344),
    [anon_sym_goto] = ACTIONS(1344),
    [anon_sym___try] = ACTIONS(1344),
    [anon_sym___leave] = ACTIONS(1344),
    [anon_sym_DASH_DASH] = ACTIONS(1346),
    [anon_sym_PLUS_PLUS] = ACTIONS(1346),
    [anon_sym_sizeof] = ACTIONS(1344),
    [anon_sym___alignof__] = ACTIONS(1344),
    [anon_sym___alignof] = ACTIONS(1344),
    [anon_sym__alignof] = ACTIONS(1344),
    [anon_sym_alignof] = ACTIONS(1344),
    [anon_sym__Alignof] = ACTIONS(1344),
    [anon_sym_offsetof] = ACTIONS(1344),
    [anon_sym__Generic] = ACTIONS(1344),
    [anon_sym_asm] = ACTIONS(1344),
    [anon_sym___asm__] = ACTIONS(1344),
    [sym_number_literal] = ACTIONS(1346),
    [anon_sym_L_SQUOTE] = ACTIONS(1346),
    [anon_sym_u_SQUOTE] = ACTIONS(1346),
    [anon_sym_U_SQUOTE] = ACTIONS(1346),
    [anon_sym_u8_SQUOTE] = ACTIONS(1346),
    [anon_sym_SQUOTE] = ACTIONS(1346),
    [anon_sym_L_DQUOTE] = ACTIONS(1346),
    [anon_sym_u_DQUOTE] = ACTIONS(1346),
    [anon_sym_U_DQUOTE] = ACTIONS(1346),
    [anon_sym_u8_DQUOTE] = ACTIONS(1346),
    [anon_sym_DQUOTE] = ACTIONS(1346),
    [sym_true] = ACTIONS(1344),
    [sym_false] = ACTIONS(1344),
    [anon_sym_NULL] = ACTIONS(1344),
    [anon_sym_nullptr] = ACTIONS(1344),
    [sym_comment] = ACTIONS(3),
  },
  [333] = {
    [sym_identifier] = ACTIONS(1284),
    [aux_sym_preproc_include_token1] = ACTIONS(1284),
    [aux_sym_preproc_def_token1] = ACTIONS(1284),
    [aux_sym_preproc_if_token1] = ACTIONS(1284),
    [aux_sym_preproc_if_token2] = ACTIONS(1284),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1284),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1284),
    [sym_preproc_directive] = ACTIONS(1284),
    [anon_sym_LPAREN2] = ACTIONS(1286),
    [anon_sym_BANG] = ACTIONS(1286),
    [anon_sym_TILDE] = ACTIONS(1286),
    [anon_sym_DASH] = ACTIONS(1284),
    [anon_sym_PLUS] = ACTIONS(1284),
    [anon_sym_STAR] = ACTIONS(1286),
    [anon_sym_AMP] = ACTIONS(1286),
    [anon_sym_SEMI] = ACTIONS(1286),
    [anon_sym___extension__] = ACTIONS(1284),
    [anon_sym_typedef] = ACTIONS(1284),
    [anon_sym_extern] = ACTIONS(1284),
    [anon_sym___attribute__] = ACTIONS(1284),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1286),
    [anon_sym___declspec] = ACTIONS(1284),
    [anon_sym___cdecl] = ACTIONS(1284),
    [anon_sym___clrcall] = ACTIONS(1284),
    [anon_sym___stdcall] = ACTIONS(1284),
    [anon_sym___fastcall] = ACTIONS(1284),
    [anon_sym___thiscall] = ACTIONS(1284),
    [anon_sym___vectorcall] = ACTIONS(1284),
    [anon_sym_LBRACE] = ACTIONS(1286),
    [anon_sym_signed] = ACTIONS(1284),
    [anon_sym_unsigned] = ACTIONS(1284),
    [anon_sym_long] = ACTIONS(1284),
    [anon_sym_short] = ACTIONS(1284),
    [anon_sym_static] = ACTIONS(1284),
    [anon_sym_auto] = ACTIONS(1284),
    [anon_sym_register] = ACTIONS(1284),
    [anon_sym_inline] = ACTIONS(1284),
    [anon_sym___inline] = ACTIONS(1284),
    [anon_sym___inline__] = ACTIONS(1284),
    [anon_sym___forceinline] = ACTIONS(1284),
    [anon_sym_thread_local] = ACTIONS(1284),
    [anon_sym___thread] = ACTIONS(1284),
    [anon_sym_const] = ACTIONS(1284),
    [anon_sym_constexpr] = ACTIONS(1284),
    [anon_sym_volatile] = ACTIONS(1284),
    [anon_sym_restrict] = ACTIONS(1284),
    [anon_sym___restrict__] = ACTIONS(1284),
    [anon_sym__Atomic] = ACTIONS(1284),
    [anon_sym__Noreturn] = ACTIONS(1284),
    [anon_sym_noreturn] = ACTIONS(1284),
    [sym_primitive_type] = ACTIONS(1284),
    [anon_sym_enum] = ACTIONS(1284),
    [anon_sym_struct] = ACTIONS(1284),
    [anon_sym_union] = ACTIONS(1284),
    [anon_sym_if] = ACTIONS(1284),
    [anon_sym_switch] = ACTIONS(1284),
    [anon_sym_case] = ACTIONS(1284),
    [anon_sym_default] = ACTIONS(1284),
    [anon_sym_while] = ACTIONS(1284),
    [anon_sym_do] = ACTIONS(1284),
    [anon_sym_for] = ACTIONS(1284),
    [anon_sym_return] = ACTIONS(1284),
    [anon_sym_break] = ACTIONS(1284),
    [anon_sym_continue] = ACTIONS(1284),
    [anon_sym_goto] = ACTIONS(1284),
    [anon_sym___try] = ACTIONS(1284),
    [anon_sym___leave] = ACTIONS(1284),
    [anon_sym_DASH_DASH] = ACTIONS(1286),
    [anon_sym_PLUS_PLUS] = ACTIONS(1286),
    [anon_sym_sizeof] = ACTIONS(1284),
    [anon_sym___alignof__] = ACTIONS(1284),
    [anon_sym___alignof] = ACTIONS(1284),
    [anon_sym__alignof] = ACTIONS(1284),
    [anon_sym_alignof] = ACTIONS(1284),
    [anon_sym__Alignof] = ACTIONS(1284),
    [anon_sym_offsetof] = ACTIONS(1284),
    [anon_sym__Generic] = ACTIONS(1284),
    [anon_sym_asm] = ACTIONS(1284),
    [anon_sym___asm__] = ACTIONS(1284),
    [sym_number_literal] = ACTIONS(1286),
    [anon_sym_L_SQUOTE] = ACTIONS(1286),
    [anon_sym_u_SQUOTE] = ACTIONS(1286),
    [anon_sym_U_SQUOTE] = ACTIONS(1286),
    [anon_sym_u8_SQUOTE] = ACTIONS(1286),
    [anon_sym_SQUOTE] = ACTIONS(1286),
    [anon_sym_L_DQUOTE] = ACTIONS(1286),
    [anon_sym_u_DQUOTE] = ACTIONS(1286),
    [anon_sym_U_DQUOTE] = ACTIONS(1286),
    [anon_sym_u8_DQUOTE] = ACTIONS(1286),
    [anon_sym_DQUOTE] = ACTIONS(1286),
    [sym_true] = ACTIONS(1284),
    [sym_false] = ACTIONS(1284),
    [anon_sym_NULL] = ACTIONS(1284),
    [anon_sym_nullptr] = ACTIONS(1284),
    [sym_comment] = ACTIONS(3),
  },
  [334] = {
    [sym_identifier] = ACTIONS(1332),
    [aux_sym_preproc_include_token1] = ACTIONS(1332),
    [aux_sym_preproc_def_token1] = ACTIONS(1332),
    [aux_sym_preproc_if_token1] = ACTIONS(1332),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1332),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1332),
    [sym_preproc_directive] = ACTIONS(1332),
    [anon_sym_LPAREN2] = ACTIONS(1334),
    [anon_sym_BANG] = ACTIONS(1334),
    [anon_sym_TILDE] = ACTIONS(1334),
    [anon_sym_DASH] = ACTIONS(1332),
    [anon_sym_PLUS] = ACTIONS(1332),
    [anon_sym_STAR] = ACTIONS(1334),
    [anon_sym_AMP] = ACTIONS(1334),
    [anon_sym_SEMI] = ACTIONS(1334),
    [anon_sym___extension__] = ACTIONS(1332),
    [anon_sym_typedef] = ACTIONS(1332),
    [anon_sym_extern] = ACTIONS(1332),
    [anon_sym___attribute__] = ACTIONS(1332),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1334),
    [anon_sym___declspec] = ACTIONS(1332),
    [anon_sym___cdecl] = ACTIONS(1332),
    [anon_sym___clrcall] = ACTIONS(1332),
    [anon_sym___stdcall] = ACTIONS(1332),
    [anon_sym___fastcall] = ACTIONS(1332),
    [anon_sym___thiscall] = ACTIONS(1332),
    [anon_sym___vectorcall] = ACTIONS(1332),
    [anon_sym_LBRACE] = ACTIONS(1334),
    [anon_sym_RBRACE] = ACTIONS(1334),
    [anon_sym_signed] = ACTIONS(1332),
    [anon_sym_unsigned] = ACTIONS(1332),
    [anon_sym_long] = ACTIONS(1332),
    [anon_sym_short] = ACTIONS(1332),
    [anon_sym_static] = ACTIONS(1332),
    [anon_sym_auto] = ACTIONS(1332),
    [anon_sym_register] = ACTIONS(1332),
    [anon_sym_inline] = ACTIONS(1332),
    [anon_sym___inline] = ACTIONS(1332),
    [anon_sym___inline__] = ACTIONS(1332),
    [anon_sym___forceinline] = ACTIONS(1332),
    [anon_sym_thread_local] = ACTIONS(1332),
    [anon_sym___thread] = ACTIONS(1332),
    [anon_sym_const] = ACTIONS(1332),
    [anon_sym_constexpr] = ACTIONS(1332),
    [anon_sym_volatile] = ACTIONS(1332),
    [anon_sym_restrict] = ACTIONS(1332),
    [anon_sym___restrict__] = ACTIONS(1332),
    [anon_sym__Atomic] = ACTIONS(1332),
    [anon_sym__Noreturn] = ACTIONS(1332),
    [anon_sym_noreturn] = ACTIONS(1332),
    [sym_primitive_type] = ACTIONS(1332),
    [anon_sym_enum] = ACTIONS(1332),
    [anon_sym_struct] = ACTIONS(1332),
    [anon_sym_union] = ACTIONS(1332),
    [anon_sym_if] = ACTIONS(1332),
    [anon_sym_switch] = ACTIONS(1332),
    [anon_sym_case] = ACTIONS(1332),
    [anon_sym_default] = ACTIONS(1332),
    [anon_sym_while] = ACTIONS(1332),
    [anon_sym_do] = ACTIONS(1332),
    [anon_sym_for] = ACTIONS(1332),
    [anon_sym_return] = ACTIONS(1332),
    [anon_sym_break] = ACTIONS(1332),
    [anon_sym_continue] = ACTIONS(1332),
    [anon_sym_goto] = ACTIONS(1332),
    [anon_sym___try] = ACTIONS(1332),
    [anon_sym___leave] = ACTIONS(1332),
    [anon_sym_DASH_DASH] = ACTIONS(1334),
    [anon_sym_PLUS_PLUS] = ACTIONS(1334),
    [anon_sym_sizeof] = ACTIONS(1332),
    [anon_sym___alignof__] = ACTIONS(1332),
    [anon_sym___alignof] = ACTIONS(1332),
    [anon_sym__alignof] = ACTIONS(1332),
    [anon_sym_alignof] = ACTIONS(1332),
    [anon_sym__Alignof] = ACTIONS(1332),
    [anon_sym_offsetof] = ACTIONS(1332),
    [anon_sym__Generic] = ACTIONS(1332),
    [anon_sym_asm] = ACTIONS(1332),
    [anon_sym___asm__] = ACTIONS(1332),
    [sym_number_literal] = ACTIONS(1334),
    [anon_sym_L_SQUOTE] = ACTIONS(1334),
    [anon_sym_u_SQUOTE] = ACTIONS(1334),
    [anon_sym_U_SQUOTE] = ACTIONS(1334),
    [anon_sym_u8_SQUOTE] = ACTIONS(1334),
    [anon_sym_SQUOTE] = ACTIONS(1334),
    [anon_sym_L_DQUOTE] = ACTIONS(1334),
    [anon_sym_u_DQUOTE] = ACTIONS(1334),
    [anon_sym_U_DQUOTE] = ACTIONS(1334),
    [anon_sym_u8_DQUOTE] = ACTIONS(1334),
    [anon_sym_DQUOTE] = ACTIONS(1334),
    [sym_true] = ACTIONS(1332),
    [sym_false] = ACTIONS(1332),
    [anon_sym_NULL] = ACTIONS(1332),
    [anon_sym_nullptr] = ACTIONS(1332),
    [sym_comment] = ACTIONS(3),
  },
  [335] = {
    [sym_attribute_declaration] = STATE(373),
    [sym_compound_statement] = STATE(112),
    [sym_attributed_statement] = STATE(112),
    [sym_labeled_statement] = STATE(112),
    [sym_expression_statement] = STATE(112),
    [sym_if_statement] = STATE(112),
    [sym_switch_statement] = STATE(112),
    [sym_case_statement] = STATE(112),
    [sym_while_statement] = STATE(112),
    [sym_do_statement] = STATE(112),
    [sym_for_statement] = STATE(112),
    [sym_return_statement] = STATE(112),
    [sym_break_statement] = STATE(112),
    [sym_continue_statement] = STATE(112),
    [sym_goto_statement] = STATE(112),
    [sym_seh_try_statement] = STATE(112),
    [sym_seh_leave_statement] = STATE(112),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(373),
    [sym_identifier] = ACTIONS(1530),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [336] = {
    [sym_identifier] = ACTIONS(1320),
    [aux_sym_preproc_include_token1] = ACTIONS(1320),
    [aux_sym_preproc_def_token1] = ACTIONS(1320),
    [aux_sym_preproc_if_token1] = ACTIONS(1320),
    [aux_sym_preproc_if_token2] = ACTIONS(1320),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1320),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1320),
    [sym_preproc_directive] = ACTIONS(1320),
    [anon_sym_LPAREN2] = ACTIONS(1322),
    [anon_sym_BANG] = ACTIONS(1322),
    [anon_sym_TILDE] = ACTIONS(1322),
    [anon_sym_DASH] = ACTIONS(1320),
    [anon_sym_PLUS] = ACTIONS(1320),
    [anon_sym_STAR] = ACTIONS(1322),
    [anon_sym_AMP] = ACTIONS(1322),
    [anon_sym_SEMI] = ACTIONS(1322),
    [anon_sym___extension__] = ACTIONS(1320),
    [anon_sym_typedef] = ACTIONS(1320),
    [anon_sym_extern] = ACTIONS(1320),
    [anon_sym___attribute__] = ACTIONS(1320),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1322),
    [anon_sym___declspec] = ACTIONS(1320),
    [anon_sym___cdecl] = ACTIONS(1320),
    [anon_sym___clrcall] = ACTIONS(1320),
    [anon_sym___stdcall] = ACTIONS(1320),
    [anon_sym___fastcall] = ACTIONS(1320),
    [anon_sym___thiscall] = ACTIONS(1320),
    [anon_sym___vectorcall] = ACTIONS(1320),
    [anon_sym_LBRACE] = ACTIONS(1322),
    [anon_sym_signed] = ACTIONS(1320),
    [anon_sym_unsigned] = ACTIONS(1320),
    [anon_sym_long] = ACTIONS(1320),
    [anon_sym_short] = ACTIONS(1320),
    [anon_sym_static] = ACTIONS(1320),
    [anon_sym_auto] = ACTIONS(1320),
    [anon_sym_register] = ACTIONS(1320),
    [anon_sym_inline] = ACTIONS(1320),
    [anon_sym___inline] = ACTIONS(1320),
    [anon_sym___inline__] = ACTIONS(1320),
    [anon_sym___forceinline] = ACTIONS(1320),
    [anon_sym_thread_local] = ACTIONS(1320),
    [anon_sym___thread] = ACTIONS(1320),
    [anon_sym_const] = ACTIONS(1320),
    [anon_sym_constexpr] = ACTIONS(1320),
    [anon_sym_volatile] = ACTIONS(1320),
    [anon_sym_restrict] = ACTIONS(1320),
    [anon_sym___restrict__] = ACTIONS(1320),
    [anon_sym__Atomic] = ACTIONS(1320),
    [anon_sym__Noreturn] = ACTIONS(1320),
    [anon_sym_noreturn] = ACTIONS(1320),
    [sym_primitive_type] = ACTIONS(1320),
    [anon_sym_enum] = ACTIONS(1320),
    [anon_sym_struct] = ACTIONS(1320),
    [anon_sym_union] = ACTIONS(1320),
    [anon_sym_if] = ACTIONS(1320),
    [anon_sym_switch] = ACTIONS(1320),
    [anon_sym_case] = ACTIONS(1320),
    [anon_sym_default] = ACTIONS(1320),
    [anon_sym_while] = ACTIONS(1320),
    [anon_sym_do] = ACTIONS(1320),
    [anon_sym_for] = ACTIONS(1320),
    [anon_sym_return] = ACTIONS(1320),
    [anon_sym_break] = ACTIONS(1320),
    [anon_sym_continue] = ACTIONS(1320),
    [anon_sym_goto] = ACTIONS(1320),
    [anon_sym___try] = ACTIONS(1320),
    [anon_sym___leave] = ACTIONS(1320),
    [anon_sym_DASH_DASH] = ACTIONS(1322),
    [anon_sym_PLUS_PLUS] = ACTIONS(1322),
    [anon_sym_sizeof] = ACTIONS(1320),
    [anon_sym___alignof__] = ACTIONS(1320),
    [anon_sym___alignof] = ACTIONS(1320),
    [anon_sym__alignof] = ACTIONS(1320),
    [anon_sym_alignof] = ACTIONS(1320),
    [anon_sym__Alignof] = ACTIONS(1320),
    [anon_sym_offsetof] = ACTIONS(1320),
    [anon_sym__Generic] = ACTIONS(1320),
    [anon_sym_asm] = ACTIONS(1320),
    [anon_sym___asm__] = ACTIONS(1320),
    [sym_number_literal] = ACTIONS(1322),
    [anon_sym_L_SQUOTE] = ACTIONS(1322),
    [anon_sym_u_SQUOTE] = ACTIONS(1322),
    [anon_sym_U_SQUOTE] = ACTIONS(1322),
    [anon_sym_u8_SQUOTE] = ACTIONS(1322),
    [anon_sym_SQUOTE] = ACTIONS(1322),
    [anon_sym_L_DQUOTE] = ACTIONS(1322),
    [anon_sym_u_DQUOTE] = ACTIONS(1322),
    [anon_sym_U_DQUOTE] = ACTIONS(1322),
    [anon_sym_u8_DQUOTE] = ACTIONS(1322),
    [anon_sym_DQUOTE] = ACTIONS(1322),
    [sym_true] = ACTIONS(1320),
    [sym_false] = ACTIONS(1320),
    [anon_sym_NULL] = ACTIONS(1320),
    [anon_sym_nullptr] = ACTIONS(1320),
    [sym_comment] = ACTIONS(3),
  },
  [337] = {
    [sym_identifier] = ACTIONS(1324),
    [aux_sym_preproc_include_token1] = ACTIONS(1324),
    [aux_sym_preproc_def_token1] = ACTIONS(1324),
    [aux_sym_preproc_if_token1] = ACTIONS(1324),
    [aux_sym_preproc_if_token2] = ACTIONS(1324),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1324),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1324),
    [sym_preproc_directive] = ACTIONS(1324),
    [anon_sym_LPAREN2] = ACTIONS(1326),
    [anon_sym_BANG] = ACTIONS(1326),
    [anon_sym_TILDE] = ACTIONS(1326),
    [anon_sym_DASH] = ACTIONS(1324),
    [anon_sym_PLUS] = ACTIONS(1324),
    [anon_sym_STAR] = ACTIONS(1326),
    [anon_sym_AMP] = ACTIONS(1326),
    [anon_sym_SEMI] = ACTIONS(1326),
    [anon_sym___extension__] = ACTIONS(1324),
    [anon_sym_typedef] = ACTIONS(1324),
    [anon_sym_extern] = ACTIONS(1324),
    [anon_sym___attribute__] = ACTIONS(1324),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1326),
    [anon_sym___declspec] = ACTIONS(1324),
    [anon_sym___cdecl] = ACTIONS(1324),
    [anon_sym___clrcall] = ACTIONS(1324),
    [anon_sym___stdcall] = ACTIONS(1324),
    [anon_sym___fastcall] = ACTIONS(1324),
    [anon_sym___thiscall] = ACTIONS(1324),
    [anon_sym___vectorcall] = ACTIONS(1324),
    [anon_sym_LBRACE] = ACTIONS(1326),
    [anon_sym_signed] = ACTIONS(1324),
    [anon_sym_unsigned] = ACTIONS(1324),
    [anon_sym_long] = ACTIONS(1324),
    [anon_sym_short] = ACTIONS(1324),
    [anon_sym_static] = ACTIONS(1324),
    [anon_sym_auto] = ACTIONS(1324),
    [anon_sym_register] = ACTIONS(1324),
    [anon_sym_inline] = ACTIONS(1324),
    [anon_sym___inline] = ACTIONS(1324),
    [anon_sym___inline__] = ACTIONS(1324),
    [anon_sym___forceinline] = ACTIONS(1324),
    [anon_sym_thread_local] = ACTIONS(1324),
    [anon_sym___thread] = ACTIONS(1324),
    [anon_sym_const] = ACTIONS(1324),
    [anon_sym_constexpr] = ACTIONS(1324),
    [anon_sym_volatile] = ACTIONS(1324),
    [anon_sym_restrict] = ACTIONS(1324),
    [anon_sym___restrict__] = ACTIONS(1324),
    [anon_sym__Atomic] = ACTIONS(1324),
    [anon_sym__Noreturn] = ACTIONS(1324),
    [anon_sym_noreturn] = ACTIONS(1324),
    [sym_primitive_type] = ACTIONS(1324),
    [anon_sym_enum] = ACTIONS(1324),
    [anon_sym_struct] = ACTIONS(1324),
    [anon_sym_union] = ACTIONS(1324),
    [anon_sym_if] = ACTIONS(1324),
    [anon_sym_switch] = ACTIONS(1324),
    [anon_sym_case] = ACTIONS(1324),
    [anon_sym_default] = ACTIONS(1324),
    [anon_sym_while] = ACTIONS(1324),
    [anon_sym_do] = ACTIONS(1324),
    [anon_sym_for] = ACTIONS(1324),
    [anon_sym_return] = ACTIONS(1324),
    [anon_sym_break] = ACTIONS(1324),
    [anon_sym_continue] = ACTIONS(1324),
    [anon_sym_goto] = ACTIONS(1324),
    [anon_sym___try] = ACTIONS(1324),
    [anon_sym___leave] = ACTIONS(1324),
    [anon_sym_DASH_DASH] = ACTIONS(1326),
    [anon_sym_PLUS_PLUS] = ACTIONS(1326),
    [anon_sym_sizeof] = ACTIONS(1324),
    [anon_sym___alignof__] = ACTIONS(1324),
    [anon_sym___alignof] = ACTIONS(1324),
    [anon_sym__alignof] = ACTIONS(1324),
    [anon_sym_alignof] = ACTIONS(1324),
    [anon_sym__Alignof] = ACTIONS(1324),
    [anon_sym_offsetof] = ACTIONS(1324),
    [anon_sym__Generic] = ACTIONS(1324),
    [anon_sym_asm] = ACTIONS(1324),
    [anon_sym___asm__] = ACTIONS(1324),
    [sym_number_literal] = ACTIONS(1326),
    [anon_sym_L_SQUOTE] = ACTIONS(1326),
    [anon_sym_u_SQUOTE] = ACTIONS(1326),
    [anon_sym_U_SQUOTE] = ACTIONS(1326),
    [anon_sym_u8_SQUOTE] = ACTIONS(1326),
    [anon_sym_SQUOTE] = ACTIONS(1326),
    [anon_sym_L_DQUOTE] = ACTIONS(1326),
    [anon_sym_u_DQUOTE] = ACTIONS(1326),
    [anon_sym_U_DQUOTE] = ACTIONS(1326),
    [anon_sym_u8_DQUOTE] = ACTIONS(1326),
    [anon_sym_DQUOTE] = ACTIONS(1326),
    [sym_true] = ACTIONS(1324),
    [sym_false] = ACTIONS(1324),
    [anon_sym_NULL] = ACTIONS(1324),
    [anon_sym_nullptr] = ACTIONS(1324),
    [sym_comment] = ACTIONS(3),
  },
  [338] = {
    [sym_attribute_declaration] = STATE(358),
    [sym_compound_statement] = STATE(1894),
    [sym_attributed_statement] = STATE(1894),
    [sym_labeled_statement] = STATE(1894),
    [sym_expression_statement] = STATE(1894),
    [sym_if_statement] = STATE(1894),
    [sym_switch_statement] = STATE(1894),
    [sym_case_statement] = STATE(1894),
    [sym_while_statement] = STATE(1894),
    [sym_do_statement] = STATE(1894),
    [sym_for_statement] = STATE(1894),
    [sym_return_statement] = STATE(1894),
    [sym_break_statement] = STATE(1894),
    [sym_continue_statement] = STATE(1894),
    [sym_goto_statement] = STATE(1894),
    [sym_seh_try_statement] = STATE(1894),
    [sym_seh_leave_statement] = STATE(1894),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [339] = {
    [sym_attribute_declaration] = STATE(316),
    [sym_compound_statement] = STATE(221),
    [sym_attributed_statement] = STATE(221),
    [sym_labeled_statement] = STATE(221),
    [sym_expression_statement] = STATE(221),
    [sym_if_statement] = STATE(221),
    [sym_switch_statement] = STATE(221),
    [sym_case_statement] = STATE(221),
    [sym_while_statement] = STATE(221),
    [sym_do_statement] = STATE(221),
    [sym_for_statement] = STATE(221),
    [sym_return_statement] = STATE(221),
    [sym_break_statement] = STATE(221),
    [sym_continue_statement] = STATE(221),
    [sym_goto_statement] = STATE(221),
    [sym_seh_try_statement] = STATE(221),
    [sym_seh_leave_statement] = STATE(221),
    [sym__expression] = STATE(1065),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1805),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(316),
    [sym_identifier] = ACTIONS(1374),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(415),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(423),
    [anon_sym_if] = ACTIONS(425),
    [anon_sym_switch] = ACTIONS(427),
    [anon_sym_case] = ACTIONS(429),
    [anon_sym_default] = ACTIONS(431),
    [anon_sym_while] = ACTIONS(433),
    [anon_sym_do] = ACTIONS(435),
    [anon_sym_for] = ACTIONS(437),
    [anon_sym_return] = ACTIONS(439),
    [anon_sym_break] = ACTIONS(441),
    [anon_sym_continue] = ACTIONS(443),
    [anon_sym_goto] = ACTIONS(445),
    [anon_sym___try] = ACTIONS(447),
    [anon_sym___leave] = ACTIONS(449),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [340] = {
    [sym_identifier] = ACTIONS(1272),
    [aux_sym_preproc_include_token1] = ACTIONS(1272),
    [aux_sym_preproc_def_token1] = ACTIONS(1272),
    [aux_sym_preproc_if_token1] = ACTIONS(1272),
    [aux_sym_preproc_if_token2] = ACTIONS(1272),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1272),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1272),
    [sym_preproc_directive] = ACTIONS(1272),
    [anon_sym_LPAREN2] = ACTIONS(1274),
    [anon_sym_BANG] = ACTIONS(1274),
    [anon_sym_TILDE] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1272),
    [anon_sym_STAR] = ACTIONS(1274),
    [anon_sym_AMP] = ACTIONS(1274),
    [anon_sym_SEMI] = ACTIONS(1274),
    [anon_sym___extension__] = ACTIONS(1272),
    [anon_sym_typedef] = ACTIONS(1272),
    [anon_sym_extern] = ACTIONS(1272),
    [anon_sym___attribute__] = ACTIONS(1272),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1274),
    [anon_sym___declspec] = ACTIONS(1272),
    [anon_sym___cdecl] = ACTIONS(1272),
    [anon_sym___clrcall] = ACTIONS(1272),
    [anon_sym___stdcall] = ACTIONS(1272),
    [anon_sym___fastcall] = ACTIONS(1272),
    [anon_sym___thiscall] = ACTIONS(1272),
    [anon_sym___vectorcall] = ACTIONS(1272),
    [anon_sym_LBRACE] = ACTIONS(1274),
    [anon_sym_signed] = ACTIONS(1272),
    [anon_sym_unsigned] = ACTIONS(1272),
    [anon_sym_long] = ACTIONS(1272),
    [anon_sym_short] = ACTIONS(1272),
    [anon_sym_static] = ACTIONS(1272),
    [anon_sym_auto] = ACTIONS(1272),
    [anon_sym_register] = ACTIONS(1272),
    [anon_sym_inline] = ACTIONS(1272),
    [anon_sym___inline] = ACTIONS(1272),
    [anon_sym___inline__] = ACTIONS(1272),
    [anon_sym___forceinline] = ACTIONS(1272),
    [anon_sym_thread_local] = ACTIONS(1272),
    [anon_sym___thread] = ACTIONS(1272),
    [anon_sym_const] = ACTIONS(1272),
    [anon_sym_constexpr] = ACTIONS(1272),
    [anon_sym_volatile] = ACTIONS(1272),
    [anon_sym_restrict] = ACTIONS(1272),
    [anon_sym___restrict__] = ACTIONS(1272),
    [anon_sym__Atomic] = ACTIONS(1272),
    [anon_sym__Noreturn] = ACTIONS(1272),
    [anon_sym_noreturn] = ACTIONS(1272),
    [sym_primitive_type] = ACTIONS(1272),
    [anon_sym_enum] = ACTIONS(1272),
    [anon_sym_struct] = ACTIONS(1272),
    [anon_sym_union] = ACTIONS(1272),
    [anon_sym_if] = ACTIONS(1272),
    [anon_sym_switch] = ACTIONS(1272),
    [anon_sym_case] = ACTIONS(1272),
    [anon_sym_default] = ACTIONS(1272),
    [anon_sym_while] = ACTIONS(1272),
    [anon_sym_do] = ACTIONS(1272),
    [anon_sym_for] = ACTIONS(1272),
    [anon_sym_return] = ACTIONS(1272),
    [anon_sym_break] = ACTIONS(1272),
    [anon_sym_continue] = ACTIONS(1272),
    [anon_sym_goto] = ACTIONS(1272),
    [anon_sym___try] = ACTIONS(1272),
    [anon_sym___leave] = ACTIONS(1272),
    [anon_sym_DASH_DASH] = ACTIONS(1274),
    [anon_sym_PLUS_PLUS] = ACTIONS(1274),
    [anon_sym_sizeof] = ACTIONS(1272),
    [anon_sym___alignof__] = ACTIONS(1272),
    [anon_sym___alignof] = ACTIONS(1272),
    [anon_sym__alignof] = ACTIONS(1272),
    [anon_sym_alignof] = ACTIONS(1272),
    [anon_sym__Alignof] = ACTIONS(1272),
    [anon_sym_offsetof] = ACTIONS(1272),
    [anon_sym__Generic] = ACTIONS(1272),
    [anon_sym_asm] = ACTIONS(1272),
    [anon_sym___asm__] = ACTIONS(1272),
    [sym_number_literal] = ACTIONS(1274),
    [anon_sym_L_SQUOTE] = ACTIONS(1274),
    [anon_sym_u_SQUOTE] = ACTIONS(1274),
    [anon_sym_U_SQUOTE] = ACTIONS(1274),
    [anon_sym_u8_SQUOTE] = ACTIONS(1274),
    [anon_sym_SQUOTE] = ACTIONS(1274),
    [anon_sym_L_DQUOTE] = ACTIONS(1274),
    [anon_sym_u_DQUOTE] = ACTIONS(1274),
    [anon_sym_U_DQUOTE] = ACTIONS(1274),
    [anon_sym_u8_DQUOTE] = ACTIONS(1274),
    [anon_sym_DQUOTE] = ACTIONS(1274),
    [sym_true] = ACTIONS(1272),
    [sym_false] = ACTIONS(1272),
    [anon_sym_NULL] = ACTIONS(1272),
    [anon_sym_nullptr] = ACTIONS(1272),
    [sym_comment] = ACTIONS(3),
  },
  [341] = {
    [sym_identifier] = ACTIONS(1252),
    [aux_sym_preproc_include_token1] = ACTIONS(1252),
    [aux_sym_preproc_def_token1] = ACTIONS(1252),
    [aux_sym_preproc_if_token1] = ACTIONS(1252),
    [aux_sym_preproc_if_token2] = ACTIONS(1252),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1252),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1252),
    [sym_preproc_directive] = ACTIONS(1252),
    [anon_sym_LPAREN2] = ACTIONS(1254),
    [anon_sym_BANG] = ACTIONS(1254),
    [anon_sym_TILDE] = ACTIONS(1254),
    [anon_sym_DASH] = ACTIONS(1252),
    [anon_sym_PLUS] = ACTIONS(1252),
    [anon_sym_STAR] = ACTIONS(1254),
    [anon_sym_AMP] = ACTIONS(1254),
    [anon_sym_SEMI] = ACTIONS(1254),
    [anon_sym___extension__] = ACTIONS(1252),
    [anon_sym_typedef] = ACTIONS(1252),
    [anon_sym_extern] = ACTIONS(1252),
    [anon_sym___attribute__] = ACTIONS(1252),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1254),
    [anon_sym___declspec] = ACTIONS(1252),
    [anon_sym___cdecl] = ACTIONS(1252),
    [anon_sym___clrcall] = ACTIONS(1252),
    [anon_sym___stdcall] = ACTIONS(1252),
    [anon_sym___fastcall] = ACTIONS(1252),
    [anon_sym___thiscall] = ACTIONS(1252),
    [anon_sym___vectorcall] = ACTIONS(1252),
    [anon_sym_LBRACE] = ACTIONS(1254),
    [anon_sym_signed] = ACTIONS(1252),
    [anon_sym_unsigned] = ACTIONS(1252),
    [anon_sym_long] = ACTIONS(1252),
    [anon_sym_short] = ACTIONS(1252),
    [anon_sym_static] = ACTIONS(1252),
    [anon_sym_auto] = ACTIONS(1252),
    [anon_sym_register] = ACTIONS(1252),
    [anon_sym_inline] = ACTIONS(1252),
    [anon_sym___inline] = ACTIONS(1252),
    [anon_sym___inline__] = ACTIONS(1252),
    [anon_sym___forceinline] = ACTIONS(1252),
    [anon_sym_thread_local] = ACTIONS(1252),
    [anon_sym___thread] = ACTIONS(1252),
    [anon_sym_const] = ACTIONS(1252),
    [anon_sym_constexpr] = ACTIONS(1252),
    [anon_sym_volatile] = ACTIONS(1252),
    [anon_sym_restrict] = ACTIONS(1252),
    [anon_sym___restrict__] = ACTIONS(1252),
    [anon_sym__Atomic] = ACTIONS(1252),
    [anon_sym__Noreturn] = ACTIONS(1252),
    [anon_sym_noreturn] = ACTIONS(1252),
    [sym_primitive_type] = ACTIONS(1252),
    [anon_sym_enum] = ACTIONS(1252),
    [anon_sym_struct] = ACTIONS(1252),
    [anon_sym_union] = ACTIONS(1252),
    [anon_sym_if] = ACTIONS(1252),
    [anon_sym_switch] = ACTIONS(1252),
    [anon_sym_case] = ACTIONS(1252),
    [anon_sym_default] = ACTIONS(1252),
    [anon_sym_while] = ACTIONS(1252),
    [anon_sym_do] = ACTIONS(1252),
    [anon_sym_for] = ACTIONS(1252),
    [anon_sym_return] = ACTIONS(1252),
    [anon_sym_break] = ACTIONS(1252),
    [anon_sym_continue] = ACTIONS(1252),
    [anon_sym_goto] = ACTIONS(1252),
    [anon_sym___try] = ACTIONS(1252),
    [anon_sym___leave] = ACTIONS(1252),
    [anon_sym_DASH_DASH] = ACTIONS(1254),
    [anon_sym_PLUS_PLUS] = ACTIONS(1254),
    [anon_sym_sizeof] = ACTIONS(1252),
    [anon_sym___alignof__] = ACTIONS(1252),
    [anon_sym___alignof] = ACTIONS(1252),
    [anon_sym__alignof] = ACTIONS(1252),
    [anon_sym_alignof] = ACTIONS(1252),
    [anon_sym__Alignof] = ACTIONS(1252),
    [anon_sym_offsetof] = ACTIONS(1252),
    [anon_sym__Generic] = ACTIONS(1252),
    [anon_sym_asm] = ACTIONS(1252),
    [anon_sym___asm__] = ACTIONS(1252),
    [sym_number_literal] = ACTIONS(1254),
    [anon_sym_L_SQUOTE] = ACTIONS(1254),
    [anon_sym_u_SQUOTE] = ACTIONS(1254),
    [anon_sym_U_SQUOTE] = ACTIONS(1254),
    [anon_sym_u8_SQUOTE] = ACTIONS(1254),
    [anon_sym_SQUOTE] = ACTIONS(1254),
    [anon_sym_L_DQUOTE] = ACTIONS(1254),
    [anon_sym_u_DQUOTE] = ACTIONS(1254),
    [anon_sym_U_DQUOTE] = ACTIONS(1254),
    [anon_sym_u8_DQUOTE] = ACTIONS(1254),
    [anon_sym_DQUOTE] = ACTIONS(1254),
    [sym_true] = ACTIONS(1252),
    [sym_false] = ACTIONS(1252),
    [anon_sym_NULL] = ACTIONS(1252),
    [anon_sym_nullptr] = ACTIONS(1252),
    [sym_comment] = ACTIONS(3),
  },
  [342] = {
    [sym_identifier] = ACTIONS(1328),
    [aux_sym_preproc_include_token1] = ACTIONS(1328),
    [aux_sym_preproc_def_token1] = ACTIONS(1328),
    [aux_sym_preproc_if_token1] = ACTIONS(1328),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1328),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1328),
    [sym_preproc_directive] = ACTIONS(1328),
    [anon_sym_LPAREN2] = ACTIONS(1330),
    [anon_sym_BANG] = ACTIONS(1330),
    [anon_sym_TILDE] = ACTIONS(1330),
    [anon_sym_DASH] = ACTIONS(1328),
    [anon_sym_PLUS] = ACTIONS(1328),
    [anon_sym_STAR] = ACTIONS(1330),
    [anon_sym_AMP] = ACTIONS(1330),
    [anon_sym_SEMI] = ACTIONS(1330),
    [anon_sym___extension__] = ACTIONS(1328),
    [anon_sym_typedef] = ACTIONS(1328),
    [anon_sym_extern] = ACTIONS(1328),
    [anon_sym___attribute__] = ACTIONS(1328),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1330),
    [anon_sym___declspec] = ACTIONS(1328),
    [anon_sym___cdecl] = ACTIONS(1328),
    [anon_sym___clrcall] = ACTIONS(1328),
    [anon_sym___stdcall] = ACTIONS(1328),
    [anon_sym___fastcall] = ACTIONS(1328),
    [anon_sym___thiscall] = ACTIONS(1328),
    [anon_sym___vectorcall] = ACTIONS(1328),
    [anon_sym_LBRACE] = ACTIONS(1330),
    [anon_sym_RBRACE] = ACTIONS(1330),
    [anon_sym_signed] = ACTIONS(1328),
    [anon_sym_unsigned] = ACTIONS(1328),
    [anon_sym_long] = ACTIONS(1328),
    [anon_sym_short] = ACTIONS(1328),
    [anon_sym_static] = ACTIONS(1328),
    [anon_sym_auto] = ACTIONS(1328),
    [anon_sym_register] = ACTIONS(1328),
    [anon_sym_inline] = ACTIONS(1328),
    [anon_sym___inline] = ACTIONS(1328),
    [anon_sym___inline__] = ACTIONS(1328),
    [anon_sym___forceinline] = ACTIONS(1328),
    [anon_sym_thread_local] = ACTIONS(1328),
    [anon_sym___thread] = ACTIONS(1328),
    [anon_sym_const] = ACTIONS(1328),
    [anon_sym_constexpr] = ACTIONS(1328),
    [anon_sym_volatile] = ACTIONS(1328),
    [anon_sym_restrict] = ACTIONS(1328),
    [anon_sym___restrict__] = ACTIONS(1328),
    [anon_sym__Atomic] = ACTIONS(1328),
    [anon_sym__Noreturn] = ACTIONS(1328),
    [anon_sym_noreturn] = ACTIONS(1328),
    [sym_primitive_type] = ACTIONS(1328),
    [anon_sym_enum] = ACTIONS(1328),
    [anon_sym_struct] = ACTIONS(1328),
    [anon_sym_union] = ACTIONS(1328),
    [anon_sym_if] = ACTIONS(1328),
    [anon_sym_switch] = ACTIONS(1328),
    [anon_sym_case] = ACTIONS(1328),
    [anon_sym_default] = ACTIONS(1328),
    [anon_sym_while] = ACTIONS(1328),
    [anon_sym_do] = ACTIONS(1328),
    [anon_sym_for] = ACTIONS(1328),
    [anon_sym_return] = ACTIONS(1328),
    [anon_sym_break] = ACTIONS(1328),
    [anon_sym_continue] = ACTIONS(1328),
    [anon_sym_goto] = ACTIONS(1328),
    [anon_sym___try] = ACTIONS(1328),
    [anon_sym___leave] = ACTIONS(1328),
    [anon_sym_DASH_DASH] = ACTIONS(1330),
    [anon_sym_PLUS_PLUS] = ACTIONS(1330),
    [anon_sym_sizeof] = ACTIONS(1328),
    [anon_sym___alignof__] = ACTIONS(1328),
    [anon_sym___alignof] = ACTIONS(1328),
    [anon_sym__alignof] = ACTIONS(1328),
    [anon_sym_alignof] = ACTIONS(1328),
    [anon_sym__Alignof] = ACTIONS(1328),
    [anon_sym_offsetof] = ACTIONS(1328),
    [anon_sym__Generic] = ACTIONS(1328),
    [anon_sym_asm] = ACTIONS(1328),
    [anon_sym___asm__] = ACTIONS(1328),
    [sym_number_literal] = ACTIONS(1330),
    [anon_sym_L_SQUOTE] = ACTIONS(1330),
    [anon_sym_u_SQUOTE] = ACTIONS(1330),
    [anon_sym_U_SQUOTE] = ACTIONS(1330),
    [anon_sym_u8_SQUOTE] = ACTIONS(1330),
    [anon_sym_SQUOTE] = ACTIONS(1330),
    [anon_sym_L_DQUOTE] = ACTIONS(1330),
    [anon_sym_u_DQUOTE] = ACTIONS(1330),
    [anon_sym_U_DQUOTE] = ACTIONS(1330),
    [anon_sym_u8_DQUOTE] = ACTIONS(1330),
    [anon_sym_DQUOTE] = ACTIONS(1330),
    [sym_true] = ACTIONS(1328),
    [sym_false] = ACTIONS(1328),
    [anon_sym_NULL] = ACTIONS(1328),
    [anon_sym_nullptr] = ACTIONS(1328),
    [sym_comment] = ACTIONS(3),
  },
  [343] = {
    [sym_identifier] = ACTIONS(1336),
    [aux_sym_preproc_include_token1] = ACTIONS(1336),
    [aux_sym_preproc_def_token1] = ACTIONS(1336),
    [aux_sym_preproc_if_token1] = ACTIONS(1336),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1336),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1336),
    [sym_preproc_directive] = ACTIONS(1336),
    [anon_sym_LPAREN2] = ACTIONS(1338),
    [anon_sym_BANG] = ACTIONS(1338),
    [anon_sym_TILDE] = ACTIONS(1338),
    [anon_sym_DASH] = ACTIONS(1336),
    [anon_sym_PLUS] = ACTIONS(1336),
    [anon_sym_STAR] = ACTIONS(1338),
    [anon_sym_AMP] = ACTIONS(1338),
    [anon_sym_SEMI] = ACTIONS(1338),
    [anon_sym___extension__] = ACTIONS(1336),
    [anon_sym_typedef] = ACTIONS(1336),
    [anon_sym_extern] = ACTIONS(1336),
    [anon_sym___attribute__] = ACTIONS(1336),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1338),
    [anon_sym___declspec] = ACTIONS(1336),
    [anon_sym___cdecl] = ACTIONS(1336),
    [anon_sym___clrcall] = ACTIONS(1336),
    [anon_sym___stdcall] = ACTIONS(1336),
    [anon_sym___fastcall] = ACTIONS(1336),
    [anon_sym___thiscall] = ACTIONS(1336),
    [anon_sym___vectorcall] = ACTIONS(1336),
    [anon_sym_LBRACE] = ACTIONS(1338),
    [anon_sym_RBRACE] = ACTIONS(1338),
    [anon_sym_signed] = ACTIONS(1336),
    [anon_sym_unsigned] = ACTIONS(1336),
    [anon_sym_long] = ACTIONS(1336),
    [anon_sym_short] = ACTIONS(1336),
    [anon_sym_static] = ACTIONS(1336),
    [anon_sym_auto] = ACTIONS(1336),
    [anon_sym_register] = ACTIONS(1336),
    [anon_sym_inline] = ACTIONS(1336),
    [anon_sym___inline] = ACTIONS(1336),
    [anon_sym___inline__] = ACTIONS(1336),
    [anon_sym___forceinline] = ACTIONS(1336),
    [anon_sym_thread_local] = ACTIONS(1336),
    [anon_sym___thread] = ACTIONS(1336),
    [anon_sym_const] = ACTIONS(1336),
    [anon_sym_constexpr] = ACTIONS(1336),
    [anon_sym_volatile] = ACTIONS(1336),
    [anon_sym_restrict] = ACTIONS(1336),
    [anon_sym___restrict__] = ACTIONS(1336),
    [anon_sym__Atomic] = ACTIONS(1336),
    [anon_sym__Noreturn] = ACTIONS(1336),
    [anon_sym_noreturn] = ACTIONS(1336),
    [sym_primitive_type] = ACTIONS(1336),
    [anon_sym_enum] = ACTIONS(1336),
    [anon_sym_struct] = ACTIONS(1336),
    [anon_sym_union] = ACTIONS(1336),
    [anon_sym_if] = ACTIONS(1336),
    [anon_sym_switch] = ACTIONS(1336),
    [anon_sym_case] = ACTIONS(1336),
    [anon_sym_default] = ACTIONS(1336),
    [anon_sym_while] = ACTIONS(1336),
    [anon_sym_do] = ACTIONS(1336),
    [anon_sym_for] = ACTIONS(1336),
    [anon_sym_return] = ACTIONS(1336),
    [anon_sym_break] = ACTIONS(1336),
    [anon_sym_continue] = ACTIONS(1336),
    [anon_sym_goto] = ACTIONS(1336),
    [anon_sym___try] = ACTIONS(1336),
    [anon_sym___leave] = ACTIONS(1336),
    [anon_sym_DASH_DASH] = ACTIONS(1338),
    [anon_sym_PLUS_PLUS] = ACTIONS(1338),
    [anon_sym_sizeof] = ACTIONS(1336),
    [anon_sym___alignof__] = ACTIONS(1336),
    [anon_sym___alignof] = ACTIONS(1336),
    [anon_sym__alignof] = ACTIONS(1336),
    [anon_sym_alignof] = ACTIONS(1336),
    [anon_sym__Alignof] = ACTIONS(1336),
    [anon_sym_offsetof] = ACTIONS(1336),
    [anon_sym__Generic] = ACTIONS(1336),
    [anon_sym_asm] = ACTIONS(1336),
    [anon_sym___asm__] = ACTIONS(1336),
    [sym_number_literal] = ACTIONS(1338),
    [anon_sym_L_SQUOTE] = ACTIONS(1338),
    [anon_sym_u_SQUOTE] = ACTIONS(1338),
    [anon_sym_U_SQUOTE] = ACTIONS(1338),
    [anon_sym_u8_SQUOTE] = ACTIONS(1338),
    [anon_sym_SQUOTE] = ACTIONS(1338),
    [anon_sym_L_DQUOTE] = ACTIONS(1338),
    [anon_sym_u_DQUOTE] = ACTIONS(1338),
    [anon_sym_U_DQUOTE] = ACTIONS(1338),
    [anon_sym_u8_DQUOTE] = ACTIONS(1338),
    [anon_sym_DQUOTE] = ACTIONS(1338),
    [sym_true] = ACTIONS(1336),
    [sym_false] = ACTIONS(1336),
    [anon_sym_NULL] = ACTIONS(1336),
    [anon_sym_nullptr] = ACTIONS(1336),
    [sym_comment] = ACTIONS(3),
  },
  [344] = {
    [sym_attribute_declaration] = STATE(358),
    [sym_compound_statement] = STATE(181),
    [sym_attributed_statement] = STATE(181),
    [sym_labeled_statement] = STATE(181),
    [sym_expression_statement] = STATE(181),
    [sym_if_statement] = STATE(181),
    [sym_switch_statement] = STATE(181),
    [sym_case_statement] = STATE(181),
    [sym_while_statement] = STATE(181),
    [sym_do_statement] = STATE(181),
    [sym_for_statement] = STATE(181),
    [sym_return_statement] = STATE(181),
    [sym_break_statement] = STATE(181),
    [sym_continue_statement] = STATE(181),
    [sym_goto_statement] = STATE(181),
    [sym_seh_try_statement] = STATE(181),
    [sym_seh_leave_statement] = STATE(181),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [345] = {
    [sym_identifier] = ACTIONS(1316),
    [aux_sym_preproc_include_token1] = ACTIONS(1316),
    [aux_sym_preproc_def_token1] = ACTIONS(1316),
    [aux_sym_preproc_if_token1] = ACTIONS(1316),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1316),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1316),
    [sym_preproc_directive] = ACTIONS(1316),
    [anon_sym_LPAREN2] = ACTIONS(1318),
    [anon_sym_BANG] = ACTIONS(1318),
    [anon_sym_TILDE] = ACTIONS(1318),
    [anon_sym_DASH] = ACTIONS(1316),
    [anon_sym_PLUS] = ACTIONS(1316),
    [anon_sym_STAR] = ACTIONS(1318),
    [anon_sym_AMP] = ACTIONS(1318),
    [anon_sym_SEMI] = ACTIONS(1318),
    [anon_sym___extension__] = ACTIONS(1316),
    [anon_sym_typedef] = ACTIONS(1316),
    [anon_sym_extern] = ACTIONS(1316),
    [anon_sym___attribute__] = ACTIONS(1316),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1318),
    [anon_sym___declspec] = ACTIONS(1316),
    [anon_sym___cdecl] = ACTIONS(1316),
    [anon_sym___clrcall] = ACTIONS(1316),
    [anon_sym___stdcall] = ACTIONS(1316),
    [anon_sym___fastcall] = ACTIONS(1316),
    [anon_sym___thiscall] = ACTIONS(1316),
    [anon_sym___vectorcall] = ACTIONS(1316),
    [anon_sym_LBRACE] = ACTIONS(1318),
    [anon_sym_RBRACE] = ACTIONS(1318),
    [anon_sym_signed] = ACTIONS(1316),
    [anon_sym_unsigned] = ACTIONS(1316),
    [anon_sym_long] = ACTIONS(1316),
    [anon_sym_short] = ACTIONS(1316),
    [anon_sym_static] = ACTIONS(1316),
    [anon_sym_auto] = ACTIONS(1316),
    [anon_sym_register] = ACTIONS(1316),
    [anon_sym_inline] = ACTIONS(1316),
    [anon_sym___inline] = ACTIONS(1316),
    [anon_sym___inline__] = ACTIONS(1316),
    [anon_sym___forceinline] = ACTIONS(1316),
    [anon_sym_thread_local] = ACTIONS(1316),
    [anon_sym___thread] = ACTIONS(1316),
    [anon_sym_const] = ACTIONS(1316),
    [anon_sym_constexpr] = ACTIONS(1316),
    [anon_sym_volatile] = ACTIONS(1316),
    [anon_sym_restrict] = ACTIONS(1316),
    [anon_sym___restrict__] = ACTIONS(1316),
    [anon_sym__Atomic] = ACTIONS(1316),
    [anon_sym__Noreturn] = ACTIONS(1316),
    [anon_sym_noreturn] = ACTIONS(1316),
    [sym_primitive_type] = ACTIONS(1316),
    [anon_sym_enum] = ACTIONS(1316),
    [anon_sym_struct] = ACTIONS(1316),
    [anon_sym_union] = ACTIONS(1316),
    [anon_sym_if] = ACTIONS(1316),
    [anon_sym_switch] = ACTIONS(1316),
    [anon_sym_case] = ACTIONS(1316),
    [anon_sym_default] = ACTIONS(1316),
    [anon_sym_while] = ACTIONS(1316),
    [anon_sym_do] = ACTIONS(1316),
    [anon_sym_for] = ACTIONS(1316),
    [anon_sym_return] = ACTIONS(1316),
    [anon_sym_break] = ACTIONS(1316),
    [anon_sym_continue] = ACTIONS(1316),
    [anon_sym_goto] = ACTIONS(1316),
    [anon_sym___try] = ACTIONS(1316),
    [anon_sym___leave] = ACTIONS(1316),
    [anon_sym_DASH_DASH] = ACTIONS(1318),
    [anon_sym_PLUS_PLUS] = ACTIONS(1318),
    [anon_sym_sizeof] = ACTIONS(1316),
    [anon_sym___alignof__] = ACTIONS(1316),
    [anon_sym___alignof] = ACTIONS(1316),
    [anon_sym__alignof] = ACTIONS(1316),
    [anon_sym_alignof] = ACTIONS(1316),
    [anon_sym__Alignof] = ACTIONS(1316),
    [anon_sym_offsetof] = ACTIONS(1316),
    [anon_sym__Generic] = ACTIONS(1316),
    [anon_sym_asm] = ACTIONS(1316),
    [anon_sym___asm__] = ACTIONS(1316),
    [sym_number_literal] = ACTIONS(1318),
    [anon_sym_L_SQUOTE] = ACTIONS(1318),
    [anon_sym_u_SQUOTE] = ACTIONS(1318),
    [anon_sym_U_SQUOTE] = ACTIONS(1318),
    [anon_sym_u8_SQUOTE] = ACTIONS(1318),
    [anon_sym_SQUOTE] = ACTIONS(1318),
    [anon_sym_L_DQUOTE] = ACTIONS(1318),
    [anon_sym_u_DQUOTE] = ACTIONS(1318),
    [anon_sym_U_DQUOTE] = ACTIONS(1318),
    [anon_sym_u8_DQUOTE] = ACTIONS(1318),
    [anon_sym_DQUOTE] = ACTIONS(1318),
    [sym_true] = ACTIONS(1316),
    [sym_false] = ACTIONS(1316),
    [anon_sym_NULL] = ACTIONS(1316),
    [anon_sym_nullptr] = ACTIONS(1316),
    [sym_comment] = ACTIONS(3),
  },
  [346] = {
    [sym_identifier] = ACTIONS(1312),
    [aux_sym_preproc_include_token1] = ACTIONS(1312),
    [aux_sym_preproc_def_token1] = ACTIONS(1312),
    [aux_sym_preproc_if_token1] = ACTIONS(1312),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1312),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1312),
    [sym_preproc_directive] = ACTIONS(1312),
    [anon_sym_LPAREN2] = ACTIONS(1314),
    [anon_sym_BANG] = ACTIONS(1314),
    [anon_sym_TILDE] = ACTIONS(1314),
    [anon_sym_DASH] = ACTIONS(1312),
    [anon_sym_PLUS] = ACTIONS(1312),
    [anon_sym_STAR] = ACTIONS(1314),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_SEMI] = ACTIONS(1314),
    [anon_sym___extension__] = ACTIONS(1312),
    [anon_sym_typedef] = ACTIONS(1312),
    [anon_sym_extern] = ACTIONS(1312),
    [anon_sym___attribute__] = ACTIONS(1312),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1314),
    [anon_sym___declspec] = ACTIONS(1312),
    [anon_sym___cdecl] = ACTIONS(1312),
    [anon_sym___clrcall] = ACTIONS(1312),
    [anon_sym___stdcall] = ACTIONS(1312),
    [anon_sym___fastcall] = ACTIONS(1312),
    [anon_sym___thiscall] = ACTIONS(1312),
    [anon_sym___vectorcall] = ACTIONS(1312),
    [anon_sym_LBRACE] = ACTIONS(1314),
    [anon_sym_RBRACE] = ACTIONS(1314),
    [anon_sym_signed] = ACTIONS(1312),
    [anon_sym_unsigned] = ACTIONS(1312),
    [anon_sym_long] = ACTIONS(1312),
    [anon_sym_short] = ACTIONS(1312),
    [anon_sym_static] = ACTIONS(1312),
    [anon_sym_auto] = ACTIONS(1312),
    [anon_sym_register] = ACTIONS(1312),
    [anon_sym_inline] = ACTIONS(1312),
    [anon_sym___inline] = ACTIONS(1312),
    [anon_sym___inline__] = ACTIONS(1312),
    [anon_sym___forceinline] = ACTIONS(1312),
    [anon_sym_thread_local] = ACTIONS(1312),
    [anon_sym___thread] = ACTIONS(1312),
    [anon_sym_const] = ACTIONS(1312),
    [anon_sym_constexpr] = ACTIONS(1312),
    [anon_sym_volatile] = ACTIONS(1312),
    [anon_sym_restrict] = ACTIONS(1312),
    [anon_sym___restrict__] = ACTIONS(1312),
    [anon_sym__Atomic] = ACTIONS(1312),
    [anon_sym__Noreturn] = ACTIONS(1312),
    [anon_sym_noreturn] = ACTIONS(1312),
    [sym_primitive_type] = ACTIONS(1312),
    [anon_sym_enum] = ACTIONS(1312),
    [anon_sym_struct] = ACTIONS(1312),
    [anon_sym_union] = ACTIONS(1312),
    [anon_sym_if] = ACTIONS(1312),
    [anon_sym_switch] = ACTIONS(1312),
    [anon_sym_case] = ACTIONS(1312),
    [anon_sym_default] = ACTIONS(1312),
    [anon_sym_while] = ACTIONS(1312),
    [anon_sym_do] = ACTIONS(1312),
    [anon_sym_for] = ACTIONS(1312),
    [anon_sym_return] = ACTIONS(1312),
    [anon_sym_break] = ACTIONS(1312),
    [anon_sym_continue] = ACTIONS(1312),
    [anon_sym_goto] = ACTIONS(1312),
    [anon_sym___try] = ACTIONS(1312),
    [anon_sym___leave] = ACTIONS(1312),
    [anon_sym_DASH_DASH] = ACTIONS(1314),
    [anon_sym_PLUS_PLUS] = ACTIONS(1314),
    [anon_sym_sizeof] = ACTIONS(1312),
    [anon_sym___alignof__] = ACTIONS(1312),
    [anon_sym___alignof] = ACTIONS(1312),
    [anon_sym__alignof] = ACTIONS(1312),
    [anon_sym_alignof] = ACTIONS(1312),
    [anon_sym__Alignof] = ACTIONS(1312),
    [anon_sym_offsetof] = ACTIONS(1312),
    [anon_sym__Generic] = ACTIONS(1312),
    [anon_sym_asm] = ACTIONS(1312),
    [anon_sym___asm__] = ACTIONS(1312),
    [sym_number_literal] = ACTIONS(1314),
    [anon_sym_L_SQUOTE] = ACTIONS(1314),
    [anon_sym_u_SQUOTE] = ACTIONS(1314),
    [anon_sym_U_SQUOTE] = ACTIONS(1314),
    [anon_sym_u8_SQUOTE] = ACTIONS(1314),
    [anon_sym_SQUOTE] = ACTIONS(1314),
    [anon_sym_L_DQUOTE] = ACTIONS(1314),
    [anon_sym_u_DQUOTE] = ACTIONS(1314),
    [anon_sym_U_DQUOTE] = ACTIONS(1314),
    [anon_sym_u8_DQUOTE] = ACTIONS(1314),
    [anon_sym_DQUOTE] = ACTIONS(1314),
    [sym_true] = ACTIONS(1312),
    [sym_false] = ACTIONS(1312),
    [anon_sym_NULL] = ACTIONS(1312),
    [anon_sym_nullptr] = ACTIONS(1312),
    [sym_comment] = ACTIONS(3),
  },
  [347] = {
    [sym_identifier] = ACTIONS(1304),
    [aux_sym_preproc_include_token1] = ACTIONS(1304),
    [aux_sym_preproc_def_token1] = ACTIONS(1304),
    [aux_sym_preproc_if_token1] = ACTIONS(1304),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1304),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1304),
    [sym_preproc_directive] = ACTIONS(1304),
    [anon_sym_LPAREN2] = ACTIONS(1306),
    [anon_sym_BANG] = ACTIONS(1306),
    [anon_sym_TILDE] = ACTIONS(1306),
    [anon_sym_DASH] = ACTIONS(1304),
    [anon_sym_PLUS] = ACTIONS(1304),
    [anon_sym_STAR] = ACTIONS(1306),
    [anon_sym_AMP] = ACTIONS(1306),
    [anon_sym_SEMI] = ACTIONS(1306),
    [anon_sym___extension__] = ACTIONS(1304),
    [anon_sym_typedef] = ACTIONS(1304),
    [anon_sym_extern] = ACTIONS(1304),
    [anon_sym___attribute__] = ACTIONS(1304),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1306),
    [anon_sym___declspec] = ACTIONS(1304),
    [anon_sym___cdecl] = ACTIONS(1304),
    [anon_sym___clrcall] = ACTIONS(1304),
    [anon_sym___stdcall] = ACTIONS(1304),
    [anon_sym___fastcall] = ACTIONS(1304),
    [anon_sym___thiscall] = ACTIONS(1304),
    [anon_sym___vectorcall] = ACTIONS(1304),
    [anon_sym_LBRACE] = ACTIONS(1306),
    [anon_sym_RBRACE] = ACTIONS(1306),
    [anon_sym_signed] = ACTIONS(1304),
    [anon_sym_unsigned] = ACTIONS(1304),
    [anon_sym_long] = ACTIONS(1304),
    [anon_sym_short] = ACTIONS(1304),
    [anon_sym_static] = ACTIONS(1304),
    [anon_sym_auto] = ACTIONS(1304),
    [anon_sym_register] = ACTIONS(1304),
    [anon_sym_inline] = ACTIONS(1304),
    [anon_sym___inline] = ACTIONS(1304),
    [anon_sym___inline__] = ACTIONS(1304),
    [anon_sym___forceinline] = ACTIONS(1304),
    [anon_sym_thread_local] = ACTIONS(1304),
    [anon_sym___thread] = ACTIONS(1304),
    [anon_sym_const] = ACTIONS(1304),
    [anon_sym_constexpr] = ACTIONS(1304),
    [anon_sym_volatile] = ACTIONS(1304),
    [anon_sym_restrict] = ACTIONS(1304),
    [anon_sym___restrict__] = ACTIONS(1304),
    [anon_sym__Atomic] = ACTIONS(1304),
    [anon_sym__Noreturn] = ACTIONS(1304),
    [anon_sym_noreturn] = ACTIONS(1304),
    [sym_primitive_type] = ACTIONS(1304),
    [anon_sym_enum] = ACTIONS(1304),
    [anon_sym_struct] = ACTIONS(1304),
    [anon_sym_union] = ACTIONS(1304),
    [anon_sym_if] = ACTIONS(1304),
    [anon_sym_switch] = ACTIONS(1304),
    [anon_sym_case] = ACTIONS(1304),
    [anon_sym_default] = ACTIONS(1304),
    [anon_sym_while] = ACTIONS(1304),
    [anon_sym_do] = ACTIONS(1304),
    [anon_sym_for] = ACTIONS(1304),
    [anon_sym_return] = ACTIONS(1304),
    [anon_sym_break] = ACTIONS(1304),
    [anon_sym_continue] = ACTIONS(1304),
    [anon_sym_goto] = ACTIONS(1304),
    [anon_sym___try] = ACTIONS(1304),
    [anon_sym___leave] = ACTIONS(1304),
    [anon_sym_DASH_DASH] = ACTIONS(1306),
    [anon_sym_PLUS_PLUS] = ACTIONS(1306),
    [anon_sym_sizeof] = ACTIONS(1304),
    [anon_sym___alignof__] = ACTIONS(1304),
    [anon_sym___alignof] = ACTIONS(1304),
    [anon_sym__alignof] = ACTIONS(1304),
    [anon_sym_alignof] = ACTIONS(1304),
    [anon_sym__Alignof] = ACTIONS(1304),
    [anon_sym_offsetof] = ACTIONS(1304),
    [anon_sym__Generic] = ACTIONS(1304),
    [anon_sym_asm] = ACTIONS(1304),
    [anon_sym___asm__] = ACTIONS(1304),
    [sym_number_literal] = ACTIONS(1306),
    [anon_sym_L_SQUOTE] = ACTIONS(1306),
    [anon_sym_u_SQUOTE] = ACTIONS(1306),
    [anon_sym_U_SQUOTE] = ACTIONS(1306),
    [anon_sym_u8_SQUOTE] = ACTIONS(1306),
    [anon_sym_SQUOTE] = ACTIONS(1306),
    [anon_sym_L_DQUOTE] = ACTIONS(1306),
    [anon_sym_u_DQUOTE] = ACTIONS(1306),
    [anon_sym_U_DQUOTE] = ACTIONS(1306),
    [anon_sym_u8_DQUOTE] = ACTIONS(1306),
    [anon_sym_DQUOTE] = ACTIONS(1306),
    [sym_true] = ACTIONS(1304),
    [sym_false] = ACTIONS(1304),
    [anon_sym_NULL] = ACTIONS(1304),
    [anon_sym_nullptr] = ACTIONS(1304),
    [sym_comment] = ACTIONS(3),
  },
  [348] = {
    [sym_identifier] = ACTIONS(1244),
    [aux_sym_preproc_include_token1] = ACTIONS(1244),
    [aux_sym_preproc_def_token1] = ACTIONS(1244),
    [aux_sym_preproc_if_token1] = ACTIONS(1244),
    [aux_sym_preproc_if_token2] = ACTIONS(1244),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1244),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1244),
    [sym_preproc_directive] = ACTIONS(1244),
    [anon_sym_LPAREN2] = ACTIONS(1246),
    [anon_sym_BANG] = ACTIONS(1246),
    [anon_sym_TILDE] = ACTIONS(1246),
    [anon_sym_DASH] = ACTIONS(1244),
    [anon_sym_PLUS] = ACTIONS(1244),
    [anon_sym_STAR] = ACTIONS(1246),
    [anon_sym_AMP] = ACTIONS(1246),
    [anon_sym_SEMI] = ACTIONS(1246),
    [anon_sym___extension__] = ACTIONS(1244),
    [anon_sym_typedef] = ACTIONS(1244),
    [anon_sym_extern] = ACTIONS(1244),
    [anon_sym___attribute__] = ACTIONS(1244),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1246),
    [anon_sym___declspec] = ACTIONS(1244),
    [anon_sym___cdecl] = ACTIONS(1244),
    [anon_sym___clrcall] = ACTIONS(1244),
    [anon_sym___stdcall] = ACTIONS(1244),
    [anon_sym___fastcall] = ACTIONS(1244),
    [anon_sym___thiscall] = ACTIONS(1244),
    [anon_sym___vectorcall] = ACTIONS(1244),
    [anon_sym_LBRACE] = ACTIONS(1246),
    [anon_sym_signed] = ACTIONS(1244),
    [anon_sym_unsigned] = ACTIONS(1244),
    [anon_sym_long] = ACTIONS(1244),
    [anon_sym_short] = ACTIONS(1244),
    [anon_sym_static] = ACTIONS(1244),
    [anon_sym_auto] = ACTIONS(1244),
    [anon_sym_register] = ACTIONS(1244),
    [anon_sym_inline] = ACTIONS(1244),
    [anon_sym___inline] = ACTIONS(1244),
    [anon_sym___inline__] = ACTIONS(1244),
    [anon_sym___forceinline] = ACTIONS(1244),
    [anon_sym_thread_local] = ACTIONS(1244),
    [anon_sym___thread] = ACTIONS(1244),
    [anon_sym_const] = ACTIONS(1244),
    [anon_sym_constexpr] = ACTIONS(1244),
    [anon_sym_volatile] = ACTIONS(1244),
    [anon_sym_restrict] = ACTIONS(1244),
    [anon_sym___restrict__] = ACTIONS(1244),
    [anon_sym__Atomic] = ACTIONS(1244),
    [anon_sym__Noreturn] = ACTIONS(1244),
    [anon_sym_noreturn] = ACTIONS(1244),
    [sym_primitive_type] = ACTIONS(1244),
    [anon_sym_enum] = ACTIONS(1244),
    [anon_sym_struct] = ACTIONS(1244),
    [anon_sym_union] = ACTIONS(1244),
    [anon_sym_if] = ACTIONS(1244),
    [anon_sym_switch] = ACTIONS(1244),
    [anon_sym_case] = ACTIONS(1244),
    [anon_sym_default] = ACTIONS(1244),
    [anon_sym_while] = ACTIONS(1244),
    [anon_sym_do] = ACTIONS(1244),
    [anon_sym_for] = ACTIONS(1244),
    [anon_sym_return] = ACTIONS(1244),
    [anon_sym_break] = ACTIONS(1244),
    [anon_sym_continue] = ACTIONS(1244),
    [anon_sym_goto] = ACTIONS(1244),
    [anon_sym___try] = ACTIONS(1244),
    [anon_sym___leave] = ACTIONS(1244),
    [anon_sym_DASH_DASH] = ACTIONS(1246),
    [anon_sym_PLUS_PLUS] = ACTIONS(1246),
    [anon_sym_sizeof] = ACTIONS(1244),
    [anon_sym___alignof__] = ACTIONS(1244),
    [anon_sym___alignof] = ACTIONS(1244),
    [anon_sym__alignof] = ACTIONS(1244),
    [anon_sym_alignof] = ACTIONS(1244),
    [anon_sym__Alignof] = ACTIONS(1244),
    [anon_sym_offsetof] = ACTIONS(1244),
    [anon_sym__Generic] = ACTIONS(1244),
    [anon_sym_asm] = ACTIONS(1244),
    [anon_sym___asm__] = ACTIONS(1244),
    [sym_number_literal] = ACTIONS(1246),
    [anon_sym_L_SQUOTE] = ACTIONS(1246),
    [anon_sym_u_SQUOTE] = ACTIONS(1246),
    [anon_sym_U_SQUOTE] = ACTIONS(1246),
    [anon_sym_u8_SQUOTE] = ACTIONS(1246),
    [anon_sym_SQUOTE] = ACTIONS(1246),
    [anon_sym_L_DQUOTE] = ACTIONS(1246),
    [anon_sym_u_DQUOTE] = ACTIONS(1246),
    [anon_sym_U_DQUOTE] = ACTIONS(1246),
    [anon_sym_u8_DQUOTE] = ACTIONS(1246),
    [anon_sym_DQUOTE] = ACTIONS(1246),
    [sym_true] = ACTIONS(1244),
    [sym_false] = ACTIONS(1244),
    [anon_sym_NULL] = ACTIONS(1244),
    [anon_sym_nullptr] = ACTIONS(1244),
    [sym_comment] = ACTIONS(3),
  },
  [349] = {
    [sym_attribute_declaration] = STATE(349),
    [sym_compound_statement] = STATE(179),
    [sym_attributed_statement] = STATE(179),
    [sym_labeled_statement] = STATE(179),
    [sym_expression_statement] = STATE(179),
    [sym_if_statement] = STATE(179),
    [sym_switch_statement] = STATE(179),
    [sym_case_statement] = STATE(179),
    [sym_while_statement] = STATE(179),
    [sym_do_statement] = STATE(179),
    [sym_for_statement] = STATE(179),
    [sym_return_statement] = STATE(179),
    [sym_break_statement] = STATE(179),
    [sym_continue_statement] = STATE(179),
    [sym_goto_statement] = STATE(179),
    [sym_seh_try_statement] = STATE(179),
    [sym_seh_leave_statement] = STATE(179),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(349),
    [sym_identifier] = ACTIONS(1532),
    [anon_sym_LPAREN2] = ACTIONS(1389),
    [anon_sym_BANG] = ACTIONS(1392),
    [anon_sym_TILDE] = ACTIONS(1392),
    [anon_sym_DASH] = ACTIONS(1395),
    [anon_sym_PLUS] = ACTIONS(1395),
    [anon_sym_STAR] = ACTIONS(1398),
    [anon_sym_AMP] = ACTIONS(1398),
    [anon_sym_SEMI] = ACTIONS(1535),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1404),
    [anon_sym_LBRACE] = ACTIONS(1488),
    [anon_sym_if] = ACTIONS(1538),
    [anon_sym_switch] = ACTIONS(1494),
    [anon_sym_case] = ACTIONS(1541),
    [anon_sym_default] = ACTIONS(1544),
    [anon_sym_while] = ACTIONS(1547),
    [anon_sym_do] = ACTIONS(1506),
    [anon_sym_for] = ACTIONS(1550),
    [anon_sym_return] = ACTIONS(1512),
    [anon_sym_break] = ACTIONS(1515),
    [anon_sym_continue] = ACTIONS(1518),
    [anon_sym_goto] = ACTIONS(1521),
    [anon_sym___try] = ACTIONS(1553),
    [anon_sym___leave] = ACTIONS(1556),
    [anon_sym_DASH_DASH] = ACTIONS(1449),
    [anon_sym_PLUS_PLUS] = ACTIONS(1449),
    [anon_sym_sizeof] = ACTIONS(1452),
    [anon_sym___alignof__] = ACTIONS(1455),
    [anon_sym___alignof] = ACTIONS(1455),
    [anon_sym__alignof] = ACTIONS(1455),
    [anon_sym_alignof] = ACTIONS(1455),
    [anon_sym__Alignof] = ACTIONS(1455),
    [anon_sym_offsetof] = ACTIONS(1458),
    [anon_sym__Generic] = ACTIONS(1461),
    [anon_sym_asm] = ACTIONS(1464),
    [anon_sym___asm__] = ACTIONS(1464),
    [sym_number_literal] = ACTIONS(1467),
    [anon_sym_L_SQUOTE] = ACTIONS(1470),
    [anon_sym_u_SQUOTE] = ACTIONS(1470),
    [anon_sym_U_SQUOTE] = ACTIONS(1470),
    [anon_sym_u8_SQUOTE] = ACTIONS(1470),
    [anon_sym_SQUOTE] = ACTIONS(1470),
    [anon_sym_L_DQUOTE] = ACTIONS(1473),
    [anon_sym_u_DQUOTE] = ACTIONS(1473),
    [anon_sym_U_DQUOTE] = ACTIONS(1473),
    [anon_sym_u8_DQUOTE] = ACTIONS(1473),
    [anon_sym_DQUOTE] = ACTIONS(1473),
    [sym_true] = ACTIONS(1476),
    [sym_false] = ACTIONS(1476),
    [anon_sym_NULL] = ACTIONS(1479),
    [anon_sym_nullptr] = ACTIONS(1479),
    [sym_comment] = ACTIONS(3),
  },
  [350] = {
    [sym_attribute_declaration] = STATE(358),
    [sym_compound_statement] = STATE(308),
    [sym_attributed_statement] = STATE(308),
    [sym_labeled_statement] = STATE(308),
    [sym_expression_statement] = STATE(308),
    [sym_if_statement] = STATE(308),
    [sym_switch_statement] = STATE(308),
    [sym_case_statement] = STATE(308),
    [sym_while_statement] = STATE(308),
    [sym_do_statement] = STATE(308),
    [sym_for_statement] = STATE(308),
    [sym_return_statement] = STATE(308),
    [sym_break_statement] = STATE(308),
    [sym_continue_statement] = STATE(308),
    [sym_goto_statement] = STATE(308),
    [sym_seh_try_statement] = STATE(308),
    [sym_seh_leave_statement] = STATE(308),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [351] = {
    [sym_attribute_declaration] = STATE(358),
    [sym_compound_statement] = STATE(285),
    [sym_attributed_statement] = STATE(285),
    [sym_labeled_statement] = STATE(285),
    [sym_expression_statement] = STATE(285),
    [sym_if_statement] = STATE(285),
    [sym_switch_statement] = STATE(285),
    [sym_case_statement] = STATE(285),
    [sym_while_statement] = STATE(285),
    [sym_do_statement] = STATE(285),
    [sym_for_statement] = STATE(285),
    [sym_return_statement] = STATE(285),
    [sym_break_statement] = STATE(285),
    [sym_continue_statement] = STATE(285),
    [sym_goto_statement] = STATE(285),
    [sym_seh_try_statement] = STATE(285),
    [sym_seh_leave_statement] = STATE(285),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [352] = {
    [sym_attribute_declaration] = STATE(358),
    [sym_compound_statement] = STATE(451),
    [sym_attributed_statement] = STATE(451),
    [sym_labeled_statement] = STATE(451),
    [sym_expression_statement] = STATE(451),
    [sym_if_statement] = STATE(451),
    [sym_switch_statement] = STATE(451),
    [sym_case_statement] = STATE(451),
    [sym_while_statement] = STATE(451),
    [sym_do_statement] = STATE(451),
    [sym_for_statement] = STATE(451),
    [sym_return_statement] = STATE(451),
    [sym_break_statement] = STATE(451),
    [sym_continue_statement] = STATE(451),
    [sym_goto_statement] = STATE(451),
    [sym_seh_try_statement] = STATE(451),
    [sym_seh_leave_statement] = STATE(451),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [353] = {
    [sym_identifier] = ACTIONS(1308),
    [aux_sym_preproc_include_token1] = ACTIONS(1308),
    [aux_sym_preproc_def_token1] = ACTIONS(1308),
    [aux_sym_preproc_if_token1] = ACTIONS(1308),
    [aux_sym_preproc_if_token2] = ACTIONS(1308),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1308),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1308),
    [sym_preproc_directive] = ACTIONS(1308),
    [anon_sym_LPAREN2] = ACTIONS(1310),
    [anon_sym_BANG] = ACTIONS(1310),
    [anon_sym_TILDE] = ACTIONS(1310),
    [anon_sym_DASH] = ACTIONS(1308),
    [anon_sym_PLUS] = ACTIONS(1308),
    [anon_sym_STAR] = ACTIONS(1310),
    [anon_sym_AMP] = ACTIONS(1310),
    [anon_sym_SEMI] = ACTIONS(1310),
    [anon_sym___extension__] = ACTIONS(1308),
    [anon_sym_typedef] = ACTIONS(1308),
    [anon_sym_extern] = ACTIONS(1308),
    [anon_sym___attribute__] = ACTIONS(1308),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1310),
    [anon_sym___declspec] = ACTIONS(1308),
    [anon_sym___cdecl] = ACTIONS(1308),
    [anon_sym___clrcall] = ACTIONS(1308),
    [anon_sym___stdcall] = ACTIONS(1308),
    [anon_sym___fastcall] = ACTIONS(1308),
    [anon_sym___thiscall] = ACTIONS(1308),
    [anon_sym___vectorcall] = ACTIONS(1308),
    [anon_sym_LBRACE] = ACTIONS(1310),
    [anon_sym_signed] = ACTIONS(1308),
    [anon_sym_unsigned] = ACTIONS(1308),
    [anon_sym_long] = ACTIONS(1308),
    [anon_sym_short] = ACTIONS(1308),
    [anon_sym_static] = ACTIONS(1308),
    [anon_sym_auto] = ACTIONS(1308),
    [anon_sym_register] = ACTIONS(1308),
    [anon_sym_inline] = ACTIONS(1308),
    [anon_sym___inline] = ACTIONS(1308),
    [anon_sym___inline__] = ACTIONS(1308),
    [anon_sym___forceinline] = ACTIONS(1308),
    [anon_sym_thread_local] = ACTIONS(1308),
    [anon_sym___thread] = ACTIONS(1308),
    [anon_sym_const] = ACTIONS(1308),
    [anon_sym_constexpr] = ACTIONS(1308),
    [anon_sym_volatile] = ACTIONS(1308),
    [anon_sym_restrict] = ACTIONS(1308),
    [anon_sym___restrict__] = ACTIONS(1308),
    [anon_sym__Atomic] = ACTIONS(1308),
    [anon_sym__Noreturn] = ACTIONS(1308),
    [anon_sym_noreturn] = ACTIONS(1308),
    [sym_primitive_type] = ACTIONS(1308),
    [anon_sym_enum] = ACTIONS(1308),
    [anon_sym_struct] = ACTIONS(1308),
    [anon_sym_union] = ACTIONS(1308),
    [anon_sym_if] = ACTIONS(1308),
    [anon_sym_switch] = ACTIONS(1308),
    [anon_sym_case] = ACTIONS(1308),
    [anon_sym_default] = ACTIONS(1308),
    [anon_sym_while] = ACTIONS(1308),
    [anon_sym_do] = ACTIONS(1308),
    [anon_sym_for] = ACTIONS(1308),
    [anon_sym_return] = ACTIONS(1308),
    [anon_sym_break] = ACTIONS(1308),
    [anon_sym_continue] = ACTIONS(1308),
    [anon_sym_goto] = ACTIONS(1308),
    [anon_sym___try] = ACTIONS(1308),
    [anon_sym___leave] = ACTIONS(1308),
    [anon_sym_DASH_DASH] = ACTIONS(1310),
    [anon_sym_PLUS_PLUS] = ACTIONS(1310),
    [anon_sym_sizeof] = ACTIONS(1308),
    [anon_sym___alignof__] = ACTIONS(1308),
    [anon_sym___alignof] = ACTIONS(1308),
    [anon_sym__alignof] = ACTIONS(1308),
    [anon_sym_alignof] = ACTIONS(1308),
    [anon_sym__Alignof] = ACTIONS(1308),
    [anon_sym_offsetof] = ACTIONS(1308),
    [anon_sym__Generic] = ACTIONS(1308),
    [anon_sym_asm] = ACTIONS(1308),
    [anon_sym___asm__] = ACTIONS(1308),
    [sym_number_literal] = ACTIONS(1310),
    [anon_sym_L_SQUOTE] = ACTIONS(1310),
    [anon_sym_u_SQUOTE] = ACTIONS(1310),
    [anon_sym_U_SQUOTE] = ACTIONS(1310),
    [anon_sym_u8_SQUOTE] = ACTIONS(1310),
    [anon_sym_SQUOTE] = ACTIONS(1310),
    [anon_sym_L_DQUOTE] = ACTIONS(1310),
    [anon_sym_u_DQUOTE] = ACTIONS(1310),
    [anon_sym_U_DQUOTE] = ACTIONS(1310),
    [anon_sym_u8_DQUOTE] = ACTIONS(1310),
    [anon_sym_DQUOTE] = ACTIONS(1310),
    [sym_true] = ACTIONS(1308),
    [sym_false] = ACTIONS(1308),
    [anon_sym_NULL] = ACTIONS(1308),
    [anon_sym_nullptr] = ACTIONS(1308),
    [sym_comment] = ACTIONS(3),
  },
  [354] = {
    [sym_attribute_declaration] = STATE(314),
    [sym_compound_statement] = STATE(183),
    [sym_attributed_statement] = STATE(184),
    [sym_labeled_statement] = STATE(185),
    [sym_expression_statement] = STATE(186),
    [sym_if_statement] = STATE(187),
    [sym_switch_statement] = STATE(192),
    [sym_case_statement] = STATE(195),
    [sym_while_statement] = STATE(196),
    [sym_do_statement] = STATE(197),
    [sym_for_statement] = STATE(198),
    [sym_return_statement] = STATE(199),
    [sym_break_statement] = STATE(205),
    [sym_continue_statement] = STATE(206),
    [sym_goto_statement] = STATE(212),
    [sym_seh_try_statement] = STATE(213),
    [sym_seh_leave_statement] = STATE(215),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [sym_identifier] = ACTIONS(1378),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(61),
    [anon_sym_default] = ACTIONS(63),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [355] = {
    [sym_attribute_declaration] = STATE(314),
    [sym_compound_statement] = STATE(181),
    [sym_attributed_statement] = STATE(181),
    [sym_labeled_statement] = STATE(181),
    [sym_expression_statement] = STATE(181),
    [sym_if_statement] = STATE(181),
    [sym_switch_statement] = STATE(181),
    [sym_case_statement] = STATE(181),
    [sym_while_statement] = STATE(181),
    [sym_do_statement] = STATE(181),
    [sym_for_statement] = STATE(181),
    [sym_return_statement] = STATE(181),
    [sym_break_statement] = STATE(181),
    [sym_continue_statement] = STATE(181),
    [sym_goto_statement] = STATE(181),
    [sym_seh_try_statement] = STATE(181),
    [sym_seh_leave_statement] = STATE(181),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [sym_identifier] = ACTIONS(1378),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(61),
    [anon_sym_default] = ACTIONS(63),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [356] = {
    [sym_identifier] = ACTIONS(1240),
    [aux_sym_preproc_include_token1] = ACTIONS(1240),
    [aux_sym_preproc_def_token1] = ACTIONS(1240),
    [aux_sym_preproc_if_token1] = ACTIONS(1240),
    [aux_sym_preproc_if_token2] = ACTIONS(1240),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1240),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1240),
    [sym_preproc_directive] = ACTIONS(1240),
    [anon_sym_LPAREN2] = ACTIONS(1242),
    [anon_sym_BANG] = ACTIONS(1242),
    [anon_sym_TILDE] = ACTIONS(1242),
    [anon_sym_DASH] = ACTIONS(1240),
    [anon_sym_PLUS] = ACTIONS(1240),
    [anon_sym_STAR] = ACTIONS(1242),
    [anon_sym_AMP] = ACTIONS(1242),
    [anon_sym_SEMI] = ACTIONS(1242),
    [anon_sym___extension__] = ACTIONS(1240),
    [anon_sym_typedef] = ACTIONS(1240),
    [anon_sym_extern] = ACTIONS(1240),
    [anon_sym___attribute__] = ACTIONS(1240),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1242),
    [anon_sym___declspec] = ACTIONS(1240),
    [anon_sym___cdecl] = ACTIONS(1240),
    [anon_sym___clrcall] = ACTIONS(1240),
    [anon_sym___stdcall] = ACTIONS(1240),
    [anon_sym___fastcall] = ACTIONS(1240),
    [anon_sym___thiscall] = ACTIONS(1240),
    [anon_sym___vectorcall] = ACTIONS(1240),
    [anon_sym_LBRACE] = ACTIONS(1242),
    [anon_sym_signed] = ACTIONS(1240),
    [anon_sym_unsigned] = ACTIONS(1240),
    [anon_sym_long] = ACTIONS(1240),
    [anon_sym_short] = ACTIONS(1240),
    [anon_sym_static] = ACTIONS(1240),
    [anon_sym_auto] = ACTIONS(1240),
    [anon_sym_register] = ACTIONS(1240),
    [anon_sym_inline] = ACTIONS(1240),
    [anon_sym___inline] = ACTIONS(1240),
    [anon_sym___inline__] = ACTIONS(1240),
    [anon_sym___forceinline] = ACTIONS(1240),
    [anon_sym_thread_local] = ACTIONS(1240),
    [anon_sym___thread] = ACTIONS(1240),
    [anon_sym_const] = ACTIONS(1240),
    [anon_sym_constexpr] = ACTIONS(1240),
    [anon_sym_volatile] = ACTIONS(1240),
    [anon_sym_restrict] = ACTIONS(1240),
    [anon_sym___restrict__] = ACTIONS(1240),
    [anon_sym__Atomic] = ACTIONS(1240),
    [anon_sym__Noreturn] = ACTIONS(1240),
    [anon_sym_noreturn] = ACTIONS(1240),
    [sym_primitive_type] = ACTIONS(1240),
    [anon_sym_enum] = ACTIONS(1240),
    [anon_sym_struct] = ACTIONS(1240),
    [anon_sym_union] = ACTIONS(1240),
    [anon_sym_if] = ACTIONS(1240),
    [anon_sym_switch] = ACTIONS(1240),
    [anon_sym_case] = ACTIONS(1240),
    [anon_sym_default] = ACTIONS(1240),
    [anon_sym_while] = ACTIONS(1240),
    [anon_sym_do] = ACTIONS(1240),
    [anon_sym_for] = ACTIONS(1240),
    [anon_sym_return] = ACTIONS(1240),
    [anon_sym_break] = ACTIONS(1240),
    [anon_sym_continue] = ACTIONS(1240),
    [anon_sym_goto] = ACTIONS(1240),
    [anon_sym___try] = ACTIONS(1240),
    [anon_sym___leave] = ACTIONS(1240),
    [anon_sym_DASH_DASH] = ACTIONS(1242),
    [anon_sym_PLUS_PLUS] = ACTIONS(1242),
    [anon_sym_sizeof] = ACTIONS(1240),
    [anon_sym___alignof__] = ACTIONS(1240),
    [anon_sym___alignof] = ACTIONS(1240),
    [anon_sym__alignof] = ACTIONS(1240),
    [anon_sym_alignof] = ACTIONS(1240),
    [anon_sym__Alignof] = ACTIONS(1240),
    [anon_sym_offsetof] = ACTIONS(1240),
    [anon_sym__Generic] = ACTIONS(1240),
    [anon_sym_asm] = ACTIONS(1240),
    [anon_sym___asm__] = ACTIONS(1240),
    [sym_number_literal] = ACTIONS(1242),
    [anon_sym_L_SQUOTE] = ACTIONS(1242),
    [anon_sym_u_SQUOTE] = ACTIONS(1242),
    [anon_sym_U_SQUOTE] = ACTIONS(1242),
    [anon_sym_u8_SQUOTE] = ACTIONS(1242),
    [anon_sym_SQUOTE] = ACTIONS(1242),
    [anon_sym_L_DQUOTE] = ACTIONS(1242),
    [anon_sym_u_DQUOTE] = ACTIONS(1242),
    [anon_sym_U_DQUOTE] = ACTIONS(1242),
    [anon_sym_u8_DQUOTE] = ACTIONS(1242),
    [anon_sym_DQUOTE] = ACTIONS(1242),
    [sym_true] = ACTIONS(1240),
    [sym_false] = ACTIONS(1240),
    [anon_sym_NULL] = ACTIONS(1240),
    [anon_sym_nullptr] = ACTIONS(1240),
    [sym_comment] = ACTIONS(3),
  },
  [357] = {
    [sym_attribute_declaration] = STATE(358),
    [sym_compound_statement] = STATE(183),
    [sym_attributed_statement] = STATE(184),
    [sym_labeled_statement] = STATE(185),
    [sym_expression_statement] = STATE(186),
    [sym_if_statement] = STATE(187),
    [sym_switch_statement] = STATE(192),
    [sym_case_statement] = STATE(195),
    [sym_while_statement] = STATE(196),
    [sym_do_statement] = STATE(197),
    [sym_for_statement] = STATE(198),
    [sym_return_statement] = STATE(199),
    [sym_break_statement] = STATE(205),
    [sym_continue_statement] = STATE(206),
    [sym_goto_statement] = STATE(212),
    [sym_seh_try_statement] = STATE(213),
    [sym_seh_leave_statement] = STATE(215),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [358] = {
    [sym_attribute_declaration] = STATE(349),
    [sym_compound_statement] = STATE(179),
    [sym_attributed_statement] = STATE(179),
    [sym_labeled_statement] = STATE(179),
    [sym_expression_statement] = STATE(179),
    [sym_if_statement] = STATE(179),
    [sym_switch_statement] = STATE(179),
    [sym_case_statement] = STATE(179),
    [sym_while_statement] = STATE(179),
    [sym_do_statement] = STATE(179),
    [sym_for_statement] = STATE(179),
    [sym_return_statement] = STATE(179),
    [sym_break_statement] = STATE(179),
    [sym_continue_statement] = STATE(179),
    [sym_goto_statement] = STATE(179),
    [sym_seh_try_statement] = STATE(179),
    [sym_seh_leave_statement] = STATE(179),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(349),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [359] = {
    [sym_identifier] = ACTIONS(1248),
    [aux_sym_preproc_include_token1] = ACTIONS(1248),
    [aux_sym_preproc_def_token1] = ACTIONS(1248),
    [aux_sym_preproc_if_token1] = ACTIONS(1248),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1248),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1248),
    [sym_preproc_directive] = ACTIONS(1248),
    [anon_sym_LPAREN2] = ACTIONS(1250),
    [anon_sym_BANG] = ACTIONS(1250),
    [anon_sym_TILDE] = ACTIONS(1250),
    [anon_sym_DASH] = ACTIONS(1248),
    [anon_sym_PLUS] = ACTIONS(1248),
    [anon_sym_STAR] = ACTIONS(1250),
    [anon_sym_AMP] = ACTIONS(1250),
    [anon_sym_SEMI] = ACTIONS(1250),
    [anon_sym___extension__] = ACTIONS(1248),
    [anon_sym_typedef] = ACTIONS(1248),
    [anon_sym_extern] = ACTIONS(1248),
    [anon_sym___attribute__] = ACTIONS(1248),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1250),
    [anon_sym___declspec] = ACTIONS(1248),
    [anon_sym___cdecl] = ACTIONS(1248),
    [anon_sym___clrcall] = ACTIONS(1248),
    [anon_sym___stdcall] = ACTIONS(1248),
    [anon_sym___fastcall] = ACTIONS(1248),
    [anon_sym___thiscall] = ACTIONS(1248),
    [anon_sym___vectorcall] = ACTIONS(1248),
    [anon_sym_LBRACE] = ACTIONS(1250),
    [anon_sym_RBRACE] = ACTIONS(1250),
    [anon_sym_signed] = ACTIONS(1248),
    [anon_sym_unsigned] = ACTIONS(1248),
    [anon_sym_long] = ACTIONS(1248),
    [anon_sym_short] = ACTIONS(1248),
    [anon_sym_static] = ACTIONS(1248),
    [anon_sym_auto] = ACTIONS(1248),
    [anon_sym_register] = ACTIONS(1248),
    [anon_sym_inline] = ACTIONS(1248),
    [anon_sym___inline] = ACTIONS(1248),
    [anon_sym___inline__] = ACTIONS(1248),
    [anon_sym___forceinline] = ACTIONS(1248),
    [anon_sym_thread_local] = ACTIONS(1248),
    [anon_sym___thread] = ACTIONS(1248),
    [anon_sym_const] = ACTIONS(1248),
    [anon_sym_constexpr] = ACTIONS(1248),
    [anon_sym_volatile] = ACTIONS(1248),
    [anon_sym_restrict] = ACTIONS(1248),
    [anon_sym___restrict__] = ACTIONS(1248),
    [anon_sym__Atomic] = ACTIONS(1248),
    [anon_sym__Noreturn] = ACTIONS(1248),
    [anon_sym_noreturn] = ACTIONS(1248),
    [sym_primitive_type] = ACTIONS(1248),
    [anon_sym_enum] = ACTIONS(1248),
    [anon_sym_struct] = ACTIONS(1248),
    [anon_sym_union] = ACTIONS(1248),
    [anon_sym_if] = ACTIONS(1248),
    [anon_sym_switch] = ACTIONS(1248),
    [anon_sym_case] = ACTIONS(1248),
    [anon_sym_default] = ACTIONS(1248),
    [anon_sym_while] = ACTIONS(1248),
    [anon_sym_do] = ACTIONS(1248),
    [anon_sym_for] = ACTIONS(1248),
    [anon_sym_return] = ACTIONS(1248),
    [anon_sym_break] = ACTIONS(1248),
    [anon_sym_continue] = ACTIONS(1248),
    [anon_sym_goto] = ACTIONS(1248),
    [anon_sym___try] = ACTIONS(1248),
    [anon_sym___leave] = ACTIONS(1248),
    [anon_sym_DASH_DASH] = ACTIONS(1250),
    [anon_sym_PLUS_PLUS] = ACTIONS(1250),
    [anon_sym_sizeof] = ACTIONS(1248),
    [anon_sym___alignof__] = ACTIONS(1248),
    [anon_sym___alignof] = ACTIONS(1248),
    [anon_sym__alignof] = ACTIONS(1248),
    [anon_sym_alignof] = ACTIONS(1248),
    [anon_sym__Alignof] = ACTIONS(1248),
    [anon_sym_offsetof] = ACTIONS(1248),
    [anon_sym__Generic] = ACTIONS(1248),
    [anon_sym_asm] = ACTIONS(1248),
    [anon_sym___asm__] = ACTIONS(1248),
    [sym_number_literal] = ACTIONS(1250),
    [anon_sym_L_SQUOTE] = ACTIONS(1250),
    [anon_sym_u_SQUOTE] = ACTIONS(1250),
    [anon_sym_U_SQUOTE] = ACTIONS(1250),
    [anon_sym_u8_SQUOTE] = ACTIONS(1250),
    [anon_sym_SQUOTE] = ACTIONS(1250),
    [anon_sym_L_DQUOTE] = ACTIONS(1250),
    [anon_sym_u_DQUOTE] = ACTIONS(1250),
    [anon_sym_U_DQUOTE] = ACTIONS(1250),
    [anon_sym_u8_DQUOTE] = ACTIONS(1250),
    [anon_sym_DQUOTE] = ACTIONS(1250),
    [sym_true] = ACTIONS(1248),
    [sym_false] = ACTIONS(1248),
    [anon_sym_NULL] = ACTIONS(1248),
    [anon_sym_nullptr] = ACTIONS(1248),
    [sym_comment] = ACTIONS(3),
  },
  [360] = {
    [sym_identifier] = ACTIONS(1316),
    [aux_sym_preproc_include_token1] = ACTIONS(1316),
    [aux_sym_preproc_def_token1] = ACTIONS(1316),
    [aux_sym_preproc_if_token1] = ACTIONS(1316),
    [aux_sym_preproc_if_token2] = ACTIONS(1316),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1316),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1316),
    [sym_preproc_directive] = ACTIONS(1316),
    [anon_sym_LPAREN2] = ACTIONS(1318),
    [anon_sym_BANG] = ACTIONS(1318),
    [anon_sym_TILDE] = ACTIONS(1318),
    [anon_sym_DASH] = ACTIONS(1316),
    [anon_sym_PLUS] = ACTIONS(1316),
    [anon_sym_STAR] = ACTIONS(1318),
    [anon_sym_AMP] = ACTIONS(1318),
    [anon_sym_SEMI] = ACTIONS(1318),
    [anon_sym___extension__] = ACTIONS(1316),
    [anon_sym_typedef] = ACTIONS(1316),
    [anon_sym_extern] = ACTIONS(1316),
    [anon_sym___attribute__] = ACTIONS(1316),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1318),
    [anon_sym___declspec] = ACTIONS(1316),
    [anon_sym___cdecl] = ACTIONS(1316),
    [anon_sym___clrcall] = ACTIONS(1316),
    [anon_sym___stdcall] = ACTIONS(1316),
    [anon_sym___fastcall] = ACTIONS(1316),
    [anon_sym___thiscall] = ACTIONS(1316),
    [anon_sym___vectorcall] = ACTIONS(1316),
    [anon_sym_LBRACE] = ACTIONS(1318),
    [anon_sym_signed] = ACTIONS(1316),
    [anon_sym_unsigned] = ACTIONS(1316),
    [anon_sym_long] = ACTIONS(1316),
    [anon_sym_short] = ACTIONS(1316),
    [anon_sym_static] = ACTIONS(1316),
    [anon_sym_auto] = ACTIONS(1316),
    [anon_sym_register] = ACTIONS(1316),
    [anon_sym_inline] = ACTIONS(1316),
    [anon_sym___inline] = ACTIONS(1316),
    [anon_sym___inline__] = ACTIONS(1316),
    [anon_sym___forceinline] = ACTIONS(1316),
    [anon_sym_thread_local] = ACTIONS(1316),
    [anon_sym___thread] = ACTIONS(1316),
    [anon_sym_const] = ACTIONS(1316),
    [anon_sym_constexpr] = ACTIONS(1316),
    [anon_sym_volatile] = ACTIONS(1316),
    [anon_sym_restrict] = ACTIONS(1316),
    [anon_sym___restrict__] = ACTIONS(1316),
    [anon_sym__Atomic] = ACTIONS(1316),
    [anon_sym__Noreturn] = ACTIONS(1316),
    [anon_sym_noreturn] = ACTIONS(1316),
    [sym_primitive_type] = ACTIONS(1316),
    [anon_sym_enum] = ACTIONS(1316),
    [anon_sym_struct] = ACTIONS(1316),
    [anon_sym_union] = ACTIONS(1316),
    [anon_sym_if] = ACTIONS(1316),
    [anon_sym_switch] = ACTIONS(1316),
    [anon_sym_case] = ACTIONS(1316),
    [anon_sym_default] = ACTIONS(1316),
    [anon_sym_while] = ACTIONS(1316),
    [anon_sym_do] = ACTIONS(1316),
    [anon_sym_for] = ACTIONS(1316),
    [anon_sym_return] = ACTIONS(1316),
    [anon_sym_break] = ACTIONS(1316),
    [anon_sym_continue] = ACTIONS(1316),
    [anon_sym_goto] = ACTIONS(1316),
    [anon_sym___try] = ACTIONS(1316),
    [anon_sym___leave] = ACTIONS(1316),
    [anon_sym_DASH_DASH] = ACTIONS(1318),
    [anon_sym_PLUS_PLUS] = ACTIONS(1318),
    [anon_sym_sizeof] = ACTIONS(1316),
    [anon_sym___alignof__] = ACTIONS(1316),
    [anon_sym___alignof] = ACTIONS(1316),
    [anon_sym__alignof] = ACTIONS(1316),
    [anon_sym_alignof] = ACTIONS(1316),
    [anon_sym__Alignof] = ACTIONS(1316),
    [anon_sym_offsetof] = ACTIONS(1316),
    [anon_sym__Generic] = ACTIONS(1316),
    [anon_sym_asm] = ACTIONS(1316),
    [anon_sym___asm__] = ACTIONS(1316),
    [sym_number_literal] = ACTIONS(1318),
    [anon_sym_L_SQUOTE] = ACTIONS(1318),
    [anon_sym_u_SQUOTE] = ACTIONS(1318),
    [anon_sym_U_SQUOTE] = ACTIONS(1318),
    [anon_sym_u8_SQUOTE] = ACTIONS(1318),
    [anon_sym_SQUOTE] = ACTIONS(1318),
    [anon_sym_L_DQUOTE] = ACTIONS(1318),
    [anon_sym_u_DQUOTE] = ACTIONS(1318),
    [anon_sym_U_DQUOTE] = ACTIONS(1318),
    [anon_sym_u8_DQUOTE] = ACTIONS(1318),
    [anon_sym_DQUOTE] = ACTIONS(1318),
    [sym_true] = ACTIONS(1316),
    [sym_false] = ACTIONS(1316),
    [anon_sym_NULL] = ACTIONS(1316),
    [anon_sym_nullptr] = ACTIONS(1316),
    [sym_comment] = ACTIONS(3),
  },
  [361] = {
    [sym_attribute_declaration] = STATE(400),
    [sym_compound_statement] = STATE(160),
    [sym_attributed_statement] = STATE(160),
    [sym_labeled_statement] = STATE(160),
    [sym_expression_statement] = STATE(160),
    [sym_if_statement] = STATE(160),
    [sym_switch_statement] = STATE(160),
    [sym_case_statement] = STATE(160),
    [sym_while_statement] = STATE(160),
    [sym_do_statement] = STATE(160),
    [sym_for_statement] = STATE(160),
    [sym_return_statement] = STATE(160),
    [sym_break_statement] = STATE(160),
    [sym_continue_statement] = STATE(160),
    [sym_goto_statement] = STATE(160),
    [sym_seh_try_statement] = STATE(160),
    [sym_seh_leave_statement] = STATE(160),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [sym_identifier] = ACTIONS(1559),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [362] = {
    [sym_attribute_declaration] = STATE(400),
    [sym_compound_statement] = STATE(250),
    [sym_attributed_statement] = STATE(250),
    [sym_labeled_statement] = STATE(250),
    [sym_expression_statement] = STATE(250),
    [sym_if_statement] = STATE(250),
    [sym_switch_statement] = STATE(250),
    [sym_case_statement] = STATE(250),
    [sym_while_statement] = STATE(250),
    [sym_do_statement] = STATE(250),
    [sym_for_statement] = STATE(250),
    [sym_return_statement] = STATE(250),
    [sym_break_statement] = STATE(250),
    [sym_continue_statement] = STATE(250),
    [sym_goto_statement] = STATE(250),
    [sym_seh_try_statement] = STATE(250),
    [sym_seh_leave_statement] = STATE(250),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [sym_identifier] = ACTIONS(1559),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [363] = {
    [sym_attribute_declaration] = STATE(335),
    [sym_compound_statement] = STATE(121),
    [sym_attributed_statement] = STATE(121),
    [sym_labeled_statement] = STATE(121),
    [sym_expression_statement] = STATE(121),
    [sym_if_statement] = STATE(121),
    [sym_switch_statement] = STATE(121),
    [sym_case_statement] = STATE(121),
    [sym_while_statement] = STATE(121),
    [sym_do_statement] = STATE(121),
    [sym_for_statement] = STATE(121),
    [sym_return_statement] = STATE(121),
    [sym_break_statement] = STATE(121),
    [sym_continue_statement] = STATE(121),
    [sym_goto_statement] = STATE(121),
    [sym_seh_try_statement] = STATE(121),
    [sym_seh_leave_statement] = STATE(121),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [sym_identifier] = ACTIONS(1530),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [364] = {
    [sym_identifier] = ACTIONS(1292),
    [aux_sym_preproc_include_token1] = ACTIONS(1292),
    [aux_sym_preproc_def_token1] = ACTIONS(1292),
    [aux_sym_preproc_if_token1] = ACTIONS(1292),
    [aux_sym_preproc_if_token2] = ACTIONS(1292),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1292),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1292),
    [sym_preproc_directive] = ACTIONS(1292),
    [anon_sym_LPAREN2] = ACTIONS(1294),
    [anon_sym_BANG] = ACTIONS(1294),
    [anon_sym_TILDE] = ACTIONS(1294),
    [anon_sym_DASH] = ACTIONS(1292),
    [anon_sym_PLUS] = ACTIONS(1292),
    [anon_sym_STAR] = ACTIONS(1294),
    [anon_sym_AMP] = ACTIONS(1294),
    [anon_sym_SEMI] = ACTIONS(1294),
    [anon_sym___extension__] = ACTIONS(1292),
    [anon_sym_typedef] = ACTIONS(1292),
    [anon_sym_extern] = ACTIONS(1292),
    [anon_sym___attribute__] = ACTIONS(1292),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1294),
    [anon_sym___declspec] = ACTIONS(1292),
    [anon_sym___cdecl] = ACTIONS(1292),
    [anon_sym___clrcall] = ACTIONS(1292),
    [anon_sym___stdcall] = ACTIONS(1292),
    [anon_sym___fastcall] = ACTIONS(1292),
    [anon_sym___thiscall] = ACTIONS(1292),
    [anon_sym___vectorcall] = ACTIONS(1292),
    [anon_sym_LBRACE] = ACTIONS(1294),
    [anon_sym_signed] = ACTIONS(1292),
    [anon_sym_unsigned] = ACTIONS(1292),
    [anon_sym_long] = ACTIONS(1292),
    [anon_sym_short] = ACTIONS(1292),
    [anon_sym_static] = ACTIONS(1292),
    [anon_sym_auto] = ACTIONS(1292),
    [anon_sym_register] = ACTIONS(1292),
    [anon_sym_inline] = ACTIONS(1292),
    [anon_sym___inline] = ACTIONS(1292),
    [anon_sym___inline__] = ACTIONS(1292),
    [anon_sym___forceinline] = ACTIONS(1292),
    [anon_sym_thread_local] = ACTIONS(1292),
    [anon_sym___thread] = ACTIONS(1292),
    [anon_sym_const] = ACTIONS(1292),
    [anon_sym_constexpr] = ACTIONS(1292),
    [anon_sym_volatile] = ACTIONS(1292),
    [anon_sym_restrict] = ACTIONS(1292),
    [anon_sym___restrict__] = ACTIONS(1292),
    [anon_sym__Atomic] = ACTIONS(1292),
    [anon_sym__Noreturn] = ACTIONS(1292),
    [anon_sym_noreturn] = ACTIONS(1292),
    [sym_primitive_type] = ACTIONS(1292),
    [anon_sym_enum] = ACTIONS(1292),
    [anon_sym_struct] = ACTIONS(1292),
    [anon_sym_union] = ACTIONS(1292),
    [anon_sym_if] = ACTIONS(1292),
    [anon_sym_switch] = ACTIONS(1292),
    [anon_sym_case] = ACTIONS(1292),
    [anon_sym_default] = ACTIONS(1292),
    [anon_sym_while] = ACTIONS(1292),
    [anon_sym_do] = ACTIONS(1292),
    [anon_sym_for] = ACTIONS(1292),
    [anon_sym_return] = ACTIONS(1292),
    [anon_sym_break] = ACTIONS(1292),
    [anon_sym_continue] = ACTIONS(1292),
    [anon_sym_goto] = ACTIONS(1292),
    [anon_sym___try] = ACTIONS(1292),
    [anon_sym___leave] = ACTIONS(1292),
    [anon_sym_DASH_DASH] = ACTIONS(1294),
    [anon_sym_PLUS_PLUS] = ACTIONS(1294),
    [anon_sym_sizeof] = ACTIONS(1292),
    [anon_sym___alignof__] = ACTIONS(1292),
    [anon_sym___alignof] = ACTIONS(1292),
    [anon_sym__alignof] = ACTIONS(1292),
    [anon_sym_alignof] = ACTIONS(1292),
    [anon_sym__Alignof] = ACTIONS(1292),
    [anon_sym_offsetof] = ACTIONS(1292),
    [anon_sym__Generic] = ACTIONS(1292),
    [anon_sym_asm] = ACTIONS(1292),
    [anon_sym___asm__] = ACTIONS(1292),
    [sym_number_literal] = ACTIONS(1294),
    [anon_sym_L_SQUOTE] = ACTIONS(1294),
    [anon_sym_u_SQUOTE] = ACTIONS(1294),
    [anon_sym_U_SQUOTE] = ACTIONS(1294),
    [anon_sym_u8_SQUOTE] = ACTIONS(1294),
    [anon_sym_SQUOTE] = ACTIONS(1294),
    [anon_sym_L_DQUOTE] = ACTIONS(1294),
    [anon_sym_u_DQUOTE] = ACTIONS(1294),
    [anon_sym_U_DQUOTE] = ACTIONS(1294),
    [anon_sym_u8_DQUOTE] = ACTIONS(1294),
    [anon_sym_DQUOTE] = ACTIONS(1294),
    [sym_true] = ACTIONS(1292),
    [sym_false] = ACTIONS(1292),
    [anon_sym_NULL] = ACTIONS(1292),
    [anon_sym_nullptr] = ACTIONS(1292),
    [sym_comment] = ACTIONS(3),
  },
  [365] = {
    [sym_identifier] = ACTIONS(1300),
    [aux_sym_preproc_include_token1] = ACTIONS(1300),
    [aux_sym_preproc_def_token1] = ACTIONS(1300),
    [aux_sym_preproc_if_token1] = ACTIONS(1300),
    [aux_sym_preproc_if_token2] = ACTIONS(1300),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1300),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1300),
    [sym_preproc_directive] = ACTIONS(1300),
    [anon_sym_LPAREN2] = ACTIONS(1302),
    [anon_sym_BANG] = ACTIONS(1302),
    [anon_sym_TILDE] = ACTIONS(1302),
    [anon_sym_DASH] = ACTIONS(1300),
    [anon_sym_PLUS] = ACTIONS(1300),
    [anon_sym_STAR] = ACTIONS(1302),
    [anon_sym_AMP] = ACTIONS(1302),
    [anon_sym_SEMI] = ACTIONS(1302),
    [anon_sym___extension__] = ACTIONS(1300),
    [anon_sym_typedef] = ACTIONS(1300),
    [anon_sym_extern] = ACTIONS(1300),
    [anon_sym___attribute__] = ACTIONS(1300),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1302),
    [anon_sym___declspec] = ACTIONS(1300),
    [anon_sym___cdecl] = ACTIONS(1300),
    [anon_sym___clrcall] = ACTIONS(1300),
    [anon_sym___stdcall] = ACTIONS(1300),
    [anon_sym___fastcall] = ACTIONS(1300),
    [anon_sym___thiscall] = ACTIONS(1300),
    [anon_sym___vectorcall] = ACTIONS(1300),
    [anon_sym_LBRACE] = ACTIONS(1302),
    [anon_sym_signed] = ACTIONS(1300),
    [anon_sym_unsigned] = ACTIONS(1300),
    [anon_sym_long] = ACTIONS(1300),
    [anon_sym_short] = ACTIONS(1300),
    [anon_sym_static] = ACTIONS(1300),
    [anon_sym_auto] = ACTIONS(1300),
    [anon_sym_register] = ACTIONS(1300),
    [anon_sym_inline] = ACTIONS(1300),
    [anon_sym___inline] = ACTIONS(1300),
    [anon_sym___inline__] = ACTIONS(1300),
    [anon_sym___forceinline] = ACTIONS(1300),
    [anon_sym_thread_local] = ACTIONS(1300),
    [anon_sym___thread] = ACTIONS(1300),
    [anon_sym_const] = ACTIONS(1300),
    [anon_sym_constexpr] = ACTIONS(1300),
    [anon_sym_volatile] = ACTIONS(1300),
    [anon_sym_restrict] = ACTIONS(1300),
    [anon_sym___restrict__] = ACTIONS(1300),
    [anon_sym__Atomic] = ACTIONS(1300),
    [anon_sym__Noreturn] = ACTIONS(1300),
    [anon_sym_noreturn] = ACTIONS(1300),
    [sym_primitive_type] = ACTIONS(1300),
    [anon_sym_enum] = ACTIONS(1300),
    [anon_sym_struct] = ACTIONS(1300),
    [anon_sym_union] = ACTIONS(1300),
    [anon_sym_if] = ACTIONS(1300),
    [anon_sym_switch] = ACTIONS(1300),
    [anon_sym_case] = ACTIONS(1300),
    [anon_sym_default] = ACTIONS(1300),
    [anon_sym_while] = ACTIONS(1300),
    [anon_sym_do] = ACTIONS(1300),
    [anon_sym_for] = ACTIONS(1300),
    [anon_sym_return] = ACTIONS(1300),
    [anon_sym_break] = ACTIONS(1300),
    [anon_sym_continue] = ACTIONS(1300),
    [anon_sym_goto] = ACTIONS(1300),
    [anon_sym___try] = ACTIONS(1300),
    [anon_sym___leave] = ACTIONS(1300),
    [anon_sym_DASH_DASH] = ACTIONS(1302),
    [anon_sym_PLUS_PLUS] = ACTIONS(1302),
    [anon_sym_sizeof] = ACTIONS(1300),
    [anon_sym___alignof__] = ACTIONS(1300),
    [anon_sym___alignof] = ACTIONS(1300),
    [anon_sym__alignof] = ACTIONS(1300),
    [anon_sym_alignof] = ACTIONS(1300),
    [anon_sym__Alignof] = ACTIONS(1300),
    [anon_sym_offsetof] = ACTIONS(1300),
    [anon_sym__Generic] = ACTIONS(1300),
    [anon_sym_asm] = ACTIONS(1300),
    [anon_sym___asm__] = ACTIONS(1300),
    [sym_number_literal] = ACTIONS(1302),
    [anon_sym_L_SQUOTE] = ACTIONS(1302),
    [anon_sym_u_SQUOTE] = ACTIONS(1302),
    [anon_sym_U_SQUOTE] = ACTIONS(1302),
    [anon_sym_u8_SQUOTE] = ACTIONS(1302),
    [anon_sym_SQUOTE] = ACTIONS(1302),
    [anon_sym_L_DQUOTE] = ACTIONS(1302),
    [anon_sym_u_DQUOTE] = ACTIONS(1302),
    [anon_sym_U_DQUOTE] = ACTIONS(1302),
    [anon_sym_u8_DQUOTE] = ACTIONS(1302),
    [anon_sym_DQUOTE] = ACTIONS(1302),
    [sym_true] = ACTIONS(1300),
    [sym_false] = ACTIONS(1300),
    [anon_sym_NULL] = ACTIONS(1300),
    [anon_sym_nullptr] = ACTIONS(1300),
    [sym_comment] = ACTIONS(3),
  },
  [366] = {
    [sym_attribute_declaration] = STATE(400),
    [sym_compound_statement] = STATE(263),
    [sym_attributed_statement] = STATE(263),
    [sym_labeled_statement] = STATE(263),
    [sym_expression_statement] = STATE(263),
    [sym_if_statement] = STATE(263),
    [sym_switch_statement] = STATE(263),
    [sym_case_statement] = STATE(263),
    [sym_while_statement] = STATE(263),
    [sym_do_statement] = STATE(263),
    [sym_for_statement] = STATE(263),
    [sym_return_statement] = STATE(263),
    [sym_break_statement] = STATE(263),
    [sym_continue_statement] = STATE(263),
    [sym_goto_statement] = STATE(263),
    [sym_seh_try_statement] = STATE(263),
    [sym_seh_leave_statement] = STATE(263),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [sym_identifier] = ACTIONS(1559),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [367] = {
    [sym_identifier] = ACTIONS(1324),
    [aux_sym_preproc_include_token1] = ACTIONS(1324),
    [aux_sym_preproc_def_token1] = ACTIONS(1324),
    [aux_sym_preproc_if_token1] = ACTIONS(1324),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1324),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1324),
    [sym_preproc_directive] = ACTIONS(1324),
    [anon_sym_LPAREN2] = ACTIONS(1326),
    [anon_sym_BANG] = ACTIONS(1326),
    [anon_sym_TILDE] = ACTIONS(1326),
    [anon_sym_DASH] = ACTIONS(1324),
    [anon_sym_PLUS] = ACTIONS(1324),
    [anon_sym_STAR] = ACTIONS(1326),
    [anon_sym_AMP] = ACTIONS(1326),
    [anon_sym_SEMI] = ACTIONS(1326),
    [anon_sym___extension__] = ACTIONS(1324),
    [anon_sym_typedef] = ACTIONS(1324),
    [anon_sym_extern] = ACTIONS(1324),
    [anon_sym___attribute__] = ACTIONS(1324),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1326),
    [anon_sym___declspec] = ACTIONS(1324),
    [anon_sym___cdecl] = ACTIONS(1324),
    [anon_sym___clrcall] = ACTIONS(1324),
    [anon_sym___stdcall] = ACTIONS(1324),
    [anon_sym___fastcall] = ACTIONS(1324),
    [anon_sym___thiscall] = ACTIONS(1324),
    [anon_sym___vectorcall] = ACTIONS(1324),
    [anon_sym_LBRACE] = ACTIONS(1326),
    [anon_sym_RBRACE] = ACTIONS(1326),
    [anon_sym_signed] = ACTIONS(1324),
    [anon_sym_unsigned] = ACTIONS(1324),
    [anon_sym_long] = ACTIONS(1324),
    [anon_sym_short] = ACTIONS(1324),
    [anon_sym_static] = ACTIONS(1324),
    [anon_sym_auto] = ACTIONS(1324),
    [anon_sym_register] = ACTIONS(1324),
    [anon_sym_inline] = ACTIONS(1324),
    [anon_sym___inline] = ACTIONS(1324),
    [anon_sym___inline__] = ACTIONS(1324),
    [anon_sym___forceinline] = ACTIONS(1324),
    [anon_sym_thread_local] = ACTIONS(1324),
    [anon_sym___thread] = ACTIONS(1324),
    [anon_sym_const] = ACTIONS(1324),
    [anon_sym_constexpr] = ACTIONS(1324),
    [anon_sym_volatile] = ACTIONS(1324),
    [anon_sym_restrict] = ACTIONS(1324),
    [anon_sym___restrict__] = ACTIONS(1324),
    [anon_sym__Atomic] = ACTIONS(1324),
    [anon_sym__Noreturn] = ACTIONS(1324),
    [anon_sym_noreturn] = ACTIONS(1324),
    [sym_primitive_type] = ACTIONS(1324),
    [anon_sym_enum] = ACTIONS(1324),
    [anon_sym_struct] = ACTIONS(1324),
    [anon_sym_union] = ACTIONS(1324),
    [anon_sym_if] = ACTIONS(1324),
    [anon_sym_switch] = ACTIONS(1324),
    [anon_sym_case] = ACTIONS(1324),
    [anon_sym_default] = ACTIONS(1324),
    [anon_sym_while] = ACTIONS(1324),
    [anon_sym_do] = ACTIONS(1324),
    [anon_sym_for] = ACTIONS(1324),
    [anon_sym_return] = ACTIONS(1324),
    [anon_sym_break] = ACTIONS(1324),
    [anon_sym_continue] = ACTIONS(1324),
    [anon_sym_goto] = ACTIONS(1324),
    [anon_sym___try] = ACTIONS(1324),
    [anon_sym___leave] = ACTIONS(1324),
    [anon_sym_DASH_DASH] = ACTIONS(1326),
    [anon_sym_PLUS_PLUS] = ACTIONS(1326),
    [anon_sym_sizeof] = ACTIONS(1324),
    [anon_sym___alignof__] = ACTIONS(1324),
    [anon_sym___alignof] = ACTIONS(1324),
    [anon_sym__alignof] = ACTIONS(1324),
    [anon_sym_alignof] = ACTIONS(1324),
    [anon_sym__Alignof] = ACTIONS(1324),
    [anon_sym_offsetof] = ACTIONS(1324),
    [anon_sym__Generic] = ACTIONS(1324),
    [anon_sym_asm] = ACTIONS(1324),
    [anon_sym___asm__] = ACTIONS(1324),
    [sym_number_literal] = ACTIONS(1326),
    [anon_sym_L_SQUOTE] = ACTIONS(1326),
    [anon_sym_u_SQUOTE] = ACTIONS(1326),
    [anon_sym_U_SQUOTE] = ACTIONS(1326),
    [anon_sym_u8_SQUOTE] = ACTIONS(1326),
    [anon_sym_SQUOTE] = ACTIONS(1326),
    [anon_sym_L_DQUOTE] = ACTIONS(1326),
    [anon_sym_u_DQUOTE] = ACTIONS(1326),
    [anon_sym_U_DQUOTE] = ACTIONS(1326),
    [anon_sym_u8_DQUOTE] = ACTIONS(1326),
    [anon_sym_DQUOTE] = ACTIONS(1326),
    [sym_true] = ACTIONS(1324),
    [sym_false] = ACTIONS(1324),
    [anon_sym_NULL] = ACTIONS(1324),
    [anon_sym_nullptr] = ACTIONS(1324),
    [sym_comment] = ACTIONS(3),
  },
  [368] = {
    [sym_identifier] = ACTIONS(1240),
    [aux_sym_preproc_include_token1] = ACTIONS(1240),
    [aux_sym_preproc_def_token1] = ACTIONS(1240),
    [aux_sym_preproc_if_token1] = ACTIONS(1240),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1240),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1240),
    [sym_preproc_directive] = ACTIONS(1240),
    [anon_sym_LPAREN2] = ACTIONS(1242),
    [anon_sym_BANG] = ACTIONS(1242),
    [anon_sym_TILDE] = ACTIONS(1242),
    [anon_sym_DASH] = ACTIONS(1240),
    [anon_sym_PLUS] = ACTIONS(1240),
    [anon_sym_STAR] = ACTIONS(1242),
    [anon_sym_AMP] = ACTIONS(1242),
    [anon_sym_SEMI] = ACTIONS(1242),
    [anon_sym___extension__] = ACTIONS(1240),
    [anon_sym_typedef] = ACTIONS(1240),
    [anon_sym_extern] = ACTIONS(1240),
    [anon_sym___attribute__] = ACTIONS(1240),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1242),
    [anon_sym___declspec] = ACTIONS(1240),
    [anon_sym___cdecl] = ACTIONS(1240),
    [anon_sym___clrcall] = ACTIONS(1240),
    [anon_sym___stdcall] = ACTIONS(1240),
    [anon_sym___fastcall] = ACTIONS(1240),
    [anon_sym___thiscall] = ACTIONS(1240),
    [anon_sym___vectorcall] = ACTIONS(1240),
    [anon_sym_LBRACE] = ACTIONS(1242),
    [anon_sym_RBRACE] = ACTIONS(1242),
    [anon_sym_signed] = ACTIONS(1240),
    [anon_sym_unsigned] = ACTIONS(1240),
    [anon_sym_long] = ACTIONS(1240),
    [anon_sym_short] = ACTIONS(1240),
    [anon_sym_static] = ACTIONS(1240),
    [anon_sym_auto] = ACTIONS(1240),
    [anon_sym_register] = ACTIONS(1240),
    [anon_sym_inline] = ACTIONS(1240),
    [anon_sym___inline] = ACTIONS(1240),
    [anon_sym___inline__] = ACTIONS(1240),
    [anon_sym___forceinline] = ACTIONS(1240),
    [anon_sym_thread_local] = ACTIONS(1240),
    [anon_sym___thread] = ACTIONS(1240),
    [anon_sym_const] = ACTIONS(1240),
    [anon_sym_constexpr] = ACTIONS(1240),
    [anon_sym_volatile] = ACTIONS(1240),
    [anon_sym_restrict] = ACTIONS(1240),
    [anon_sym___restrict__] = ACTIONS(1240),
    [anon_sym__Atomic] = ACTIONS(1240),
    [anon_sym__Noreturn] = ACTIONS(1240),
    [anon_sym_noreturn] = ACTIONS(1240),
    [sym_primitive_type] = ACTIONS(1240),
    [anon_sym_enum] = ACTIONS(1240),
    [anon_sym_struct] = ACTIONS(1240),
    [anon_sym_union] = ACTIONS(1240),
    [anon_sym_if] = ACTIONS(1240),
    [anon_sym_switch] = ACTIONS(1240),
    [anon_sym_case] = ACTIONS(1240),
    [anon_sym_default] = ACTIONS(1240),
    [anon_sym_while] = ACTIONS(1240),
    [anon_sym_do] = ACTIONS(1240),
    [anon_sym_for] = ACTIONS(1240),
    [anon_sym_return] = ACTIONS(1240),
    [anon_sym_break] = ACTIONS(1240),
    [anon_sym_continue] = ACTIONS(1240),
    [anon_sym_goto] = ACTIONS(1240),
    [anon_sym___try] = ACTIONS(1240),
    [anon_sym___leave] = ACTIONS(1240),
    [anon_sym_DASH_DASH] = ACTIONS(1242),
    [anon_sym_PLUS_PLUS] = ACTIONS(1242),
    [anon_sym_sizeof] = ACTIONS(1240),
    [anon_sym___alignof__] = ACTIONS(1240),
    [anon_sym___alignof] = ACTIONS(1240),
    [anon_sym__alignof] = ACTIONS(1240),
    [anon_sym_alignof] = ACTIONS(1240),
    [anon_sym__Alignof] = ACTIONS(1240),
    [anon_sym_offsetof] = ACTIONS(1240),
    [anon_sym__Generic] = ACTIONS(1240),
    [anon_sym_asm] = ACTIONS(1240),
    [anon_sym___asm__] = ACTIONS(1240),
    [sym_number_literal] = ACTIONS(1242),
    [anon_sym_L_SQUOTE] = ACTIONS(1242),
    [anon_sym_u_SQUOTE] = ACTIONS(1242),
    [anon_sym_U_SQUOTE] = ACTIONS(1242),
    [anon_sym_u8_SQUOTE] = ACTIONS(1242),
    [anon_sym_SQUOTE] = ACTIONS(1242),
    [anon_sym_L_DQUOTE] = ACTIONS(1242),
    [anon_sym_u_DQUOTE] = ACTIONS(1242),
    [anon_sym_U_DQUOTE] = ACTIONS(1242),
    [anon_sym_u8_DQUOTE] = ACTIONS(1242),
    [anon_sym_DQUOTE] = ACTIONS(1242),
    [sym_true] = ACTIONS(1240),
    [sym_false] = ACTIONS(1240),
    [anon_sym_NULL] = ACTIONS(1240),
    [anon_sym_nullptr] = ACTIONS(1240),
    [sym_comment] = ACTIONS(3),
  },
  [369] = {
    [sym_identifier] = ACTIONS(1248),
    [aux_sym_preproc_include_token1] = ACTIONS(1248),
    [aux_sym_preproc_def_token1] = ACTIONS(1248),
    [aux_sym_preproc_if_token1] = ACTIONS(1248),
    [aux_sym_preproc_if_token2] = ACTIONS(1248),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1248),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1248),
    [sym_preproc_directive] = ACTIONS(1248),
    [anon_sym_LPAREN2] = ACTIONS(1250),
    [anon_sym_BANG] = ACTIONS(1250),
    [anon_sym_TILDE] = ACTIONS(1250),
    [anon_sym_DASH] = ACTIONS(1248),
    [anon_sym_PLUS] = ACTIONS(1248),
    [anon_sym_STAR] = ACTIONS(1250),
    [anon_sym_AMP] = ACTIONS(1250),
    [anon_sym_SEMI] = ACTIONS(1250),
    [anon_sym___extension__] = ACTIONS(1248),
    [anon_sym_typedef] = ACTIONS(1248),
    [anon_sym_extern] = ACTIONS(1248),
    [anon_sym___attribute__] = ACTIONS(1248),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1250),
    [anon_sym___declspec] = ACTIONS(1248),
    [anon_sym___cdecl] = ACTIONS(1248),
    [anon_sym___clrcall] = ACTIONS(1248),
    [anon_sym___stdcall] = ACTIONS(1248),
    [anon_sym___fastcall] = ACTIONS(1248),
    [anon_sym___thiscall] = ACTIONS(1248),
    [anon_sym___vectorcall] = ACTIONS(1248),
    [anon_sym_LBRACE] = ACTIONS(1250),
    [anon_sym_signed] = ACTIONS(1248),
    [anon_sym_unsigned] = ACTIONS(1248),
    [anon_sym_long] = ACTIONS(1248),
    [anon_sym_short] = ACTIONS(1248),
    [anon_sym_static] = ACTIONS(1248),
    [anon_sym_auto] = ACTIONS(1248),
    [anon_sym_register] = ACTIONS(1248),
    [anon_sym_inline] = ACTIONS(1248),
    [anon_sym___inline] = ACTIONS(1248),
    [anon_sym___inline__] = ACTIONS(1248),
    [anon_sym___forceinline] = ACTIONS(1248),
    [anon_sym_thread_local] = ACTIONS(1248),
    [anon_sym___thread] = ACTIONS(1248),
    [anon_sym_const] = ACTIONS(1248),
    [anon_sym_constexpr] = ACTIONS(1248),
    [anon_sym_volatile] = ACTIONS(1248),
    [anon_sym_restrict] = ACTIONS(1248),
    [anon_sym___restrict__] = ACTIONS(1248),
    [anon_sym__Atomic] = ACTIONS(1248),
    [anon_sym__Noreturn] = ACTIONS(1248),
    [anon_sym_noreturn] = ACTIONS(1248),
    [sym_primitive_type] = ACTIONS(1248),
    [anon_sym_enum] = ACTIONS(1248),
    [anon_sym_struct] = ACTIONS(1248),
    [anon_sym_union] = ACTIONS(1248),
    [anon_sym_if] = ACTIONS(1248),
    [anon_sym_switch] = ACTIONS(1248),
    [anon_sym_case] = ACTIONS(1248),
    [anon_sym_default] = ACTIONS(1248),
    [anon_sym_while] = ACTIONS(1248),
    [anon_sym_do] = ACTIONS(1248),
    [anon_sym_for] = ACTIONS(1248),
    [anon_sym_return] = ACTIONS(1248),
    [anon_sym_break] = ACTIONS(1248),
    [anon_sym_continue] = ACTIONS(1248),
    [anon_sym_goto] = ACTIONS(1248),
    [anon_sym___try] = ACTIONS(1248),
    [anon_sym___leave] = ACTIONS(1248),
    [anon_sym_DASH_DASH] = ACTIONS(1250),
    [anon_sym_PLUS_PLUS] = ACTIONS(1250),
    [anon_sym_sizeof] = ACTIONS(1248),
    [anon_sym___alignof__] = ACTIONS(1248),
    [anon_sym___alignof] = ACTIONS(1248),
    [anon_sym__alignof] = ACTIONS(1248),
    [anon_sym_alignof] = ACTIONS(1248),
    [anon_sym__Alignof] = ACTIONS(1248),
    [anon_sym_offsetof] = ACTIONS(1248),
    [anon_sym__Generic] = ACTIONS(1248),
    [anon_sym_asm] = ACTIONS(1248),
    [anon_sym___asm__] = ACTIONS(1248),
    [sym_number_literal] = ACTIONS(1250),
    [anon_sym_L_SQUOTE] = ACTIONS(1250),
    [anon_sym_u_SQUOTE] = ACTIONS(1250),
    [anon_sym_U_SQUOTE] = ACTIONS(1250),
    [anon_sym_u8_SQUOTE] = ACTIONS(1250),
    [anon_sym_SQUOTE] = ACTIONS(1250),
    [anon_sym_L_DQUOTE] = ACTIONS(1250),
    [anon_sym_u_DQUOTE] = ACTIONS(1250),
    [anon_sym_U_DQUOTE] = ACTIONS(1250),
    [anon_sym_u8_DQUOTE] = ACTIONS(1250),
    [anon_sym_DQUOTE] = ACTIONS(1250),
    [sym_true] = ACTIONS(1248),
    [sym_false] = ACTIONS(1248),
    [anon_sym_NULL] = ACTIONS(1248),
    [anon_sym_nullptr] = ACTIONS(1248),
    [sym_comment] = ACTIONS(3),
  },
  [370] = {
    [sym_identifier] = ACTIONS(1296),
    [aux_sym_preproc_include_token1] = ACTIONS(1296),
    [aux_sym_preproc_def_token1] = ACTIONS(1296),
    [aux_sym_preproc_if_token1] = ACTIONS(1296),
    [aux_sym_preproc_if_token2] = ACTIONS(1296),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1296),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1296),
    [sym_preproc_directive] = ACTIONS(1296),
    [anon_sym_LPAREN2] = ACTIONS(1298),
    [anon_sym_BANG] = ACTIONS(1298),
    [anon_sym_TILDE] = ACTIONS(1298),
    [anon_sym_DASH] = ACTIONS(1296),
    [anon_sym_PLUS] = ACTIONS(1296),
    [anon_sym_STAR] = ACTIONS(1298),
    [anon_sym_AMP] = ACTIONS(1298),
    [anon_sym_SEMI] = ACTIONS(1298),
    [anon_sym___extension__] = ACTIONS(1296),
    [anon_sym_typedef] = ACTIONS(1296),
    [anon_sym_extern] = ACTIONS(1296),
    [anon_sym___attribute__] = ACTIONS(1296),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1298),
    [anon_sym___declspec] = ACTIONS(1296),
    [anon_sym___cdecl] = ACTIONS(1296),
    [anon_sym___clrcall] = ACTIONS(1296),
    [anon_sym___stdcall] = ACTIONS(1296),
    [anon_sym___fastcall] = ACTIONS(1296),
    [anon_sym___thiscall] = ACTIONS(1296),
    [anon_sym___vectorcall] = ACTIONS(1296),
    [anon_sym_LBRACE] = ACTIONS(1298),
    [anon_sym_signed] = ACTIONS(1296),
    [anon_sym_unsigned] = ACTIONS(1296),
    [anon_sym_long] = ACTIONS(1296),
    [anon_sym_short] = ACTIONS(1296),
    [anon_sym_static] = ACTIONS(1296),
    [anon_sym_auto] = ACTIONS(1296),
    [anon_sym_register] = ACTIONS(1296),
    [anon_sym_inline] = ACTIONS(1296),
    [anon_sym___inline] = ACTIONS(1296),
    [anon_sym___inline__] = ACTIONS(1296),
    [anon_sym___forceinline] = ACTIONS(1296),
    [anon_sym_thread_local] = ACTIONS(1296),
    [anon_sym___thread] = ACTIONS(1296),
    [anon_sym_const] = ACTIONS(1296),
    [anon_sym_constexpr] = ACTIONS(1296),
    [anon_sym_volatile] = ACTIONS(1296),
    [anon_sym_restrict] = ACTIONS(1296),
    [anon_sym___restrict__] = ACTIONS(1296),
    [anon_sym__Atomic] = ACTIONS(1296),
    [anon_sym__Noreturn] = ACTIONS(1296),
    [anon_sym_noreturn] = ACTIONS(1296),
    [sym_primitive_type] = ACTIONS(1296),
    [anon_sym_enum] = ACTIONS(1296),
    [anon_sym_struct] = ACTIONS(1296),
    [anon_sym_union] = ACTIONS(1296),
    [anon_sym_if] = ACTIONS(1296),
    [anon_sym_switch] = ACTIONS(1296),
    [anon_sym_case] = ACTIONS(1296),
    [anon_sym_default] = ACTIONS(1296),
    [anon_sym_while] = ACTIONS(1296),
    [anon_sym_do] = ACTIONS(1296),
    [anon_sym_for] = ACTIONS(1296),
    [anon_sym_return] = ACTIONS(1296),
    [anon_sym_break] = ACTIONS(1296),
    [anon_sym_continue] = ACTIONS(1296),
    [anon_sym_goto] = ACTIONS(1296),
    [anon_sym___try] = ACTIONS(1296),
    [anon_sym___leave] = ACTIONS(1296),
    [anon_sym_DASH_DASH] = ACTIONS(1298),
    [anon_sym_PLUS_PLUS] = ACTIONS(1298),
    [anon_sym_sizeof] = ACTIONS(1296),
    [anon_sym___alignof__] = ACTIONS(1296),
    [anon_sym___alignof] = ACTIONS(1296),
    [anon_sym__alignof] = ACTIONS(1296),
    [anon_sym_alignof] = ACTIONS(1296),
    [anon_sym__Alignof] = ACTIONS(1296),
    [anon_sym_offsetof] = ACTIONS(1296),
    [anon_sym__Generic] = ACTIONS(1296),
    [anon_sym_asm] = ACTIONS(1296),
    [anon_sym___asm__] = ACTIONS(1296),
    [sym_number_literal] = ACTIONS(1298),
    [anon_sym_L_SQUOTE] = ACTIONS(1298),
    [anon_sym_u_SQUOTE] = ACTIONS(1298),
    [anon_sym_U_SQUOTE] = ACTIONS(1298),
    [anon_sym_u8_SQUOTE] = ACTIONS(1298),
    [anon_sym_SQUOTE] = ACTIONS(1298),
    [anon_sym_L_DQUOTE] = ACTIONS(1298),
    [anon_sym_u_DQUOTE] = ACTIONS(1298),
    [anon_sym_U_DQUOTE] = ACTIONS(1298),
    [anon_sym_u8_DQUOTE] = ACTIONS(1298),
    [anon_sym_DQUOTE] = ACTIONS(1298),
    [sym_true] = ACTIONS(1296),
    [sym_false] = ACTIONS(1296),
    [anon_sym_NULL] = ACTIONS(1296),
    [anon_sym_nullptr] = ACTIONS(1296),
    [sym_comment] = ACTIONS(3),
  },
  [371] = {
    [sym_attribute_declaration] = STATE(371),
    [sym_compound_statement] = STATE(241),
    [sym_attributed_statement] = STATE(241),
    [sym_labeled_statement] = STATE(241),
    [sym_expression_statement] = STATE(241),
    [sym_if_statement] = STATE(241),
    [sym_switch_statement] = STATE(241),
    [sym_case_statement] = STATE(241),
    [sym_while_statement] = STATE(241),
    [sym_do_statement] = STATE(241),
    [sym_for_statement] = STATE(241),
    [sym_return_statement] = STATE(241),
    [sym_break_statement] = STATE(241),
    [sym_continue_statement] = STATE(241),
    [sym_goto_statement] = STATE(241),
    [sym_seh_try_statement] = STATE(241),
    [sym_seh_leave_statement] = STATE(241),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(371),
    [sym_identifier] = ACTIONS(1561),
    [anon_sym_LPAREN2] = ACTIONS(1389),
    [anon_sym_BANG] = ACTIONS(1392),
    [anon_sym_TILDE] = ACTIONS(1392),
    [anon_sym_DASH] = ACTIONS(1395),
    [anon_sym_PLUS] = ACTIONS(1395),
    [anon_sym_STAR] = ACTIONS(1398),
    [anon_sym_AMP] = ACTIONS(1398),
    [anon_sym_SEMI] = ACTIONS(1535),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1404),
    [anon_sym_LBRACE] = ACTIONS(1564),
    [anon_sym_if] = ACTIONS(1567),
    [anon_sym_switch] = ACTIONS(1570),
    [anon_sym_case] = ACTIONS(1573),
    [anon_sym_default] = ACTIONS(1576),
    [anon_sym_while] = ACTIONS(1579),
    [anon_sym_do] = ACTIONS(1582),
    [anon_sym_for] = ACTIONS(1585),
    [anon_sym_return] = ACTIONS(1588),
    [anon_sym_break] = ACTIONS(1591),
    [anon_sym_continue] = ACTIONS(1594),
    [anon_sym_goto] = ACTIONS(1597),
    [anon_sym___try] = ACTIONS(1600),
    [anon_sym___leave] = ACTIONS(1556),
    [anon_sym_DASH_DASH] = ACTIONS(1449),
    [anon_sym_PLUS_PLUS] = ACTIONS(1449),
    [anon_sym_sizeof] = ACTIONS(1452),
    [anon_sym___alignof__] = ACTIONS(1455),
    [anon_sym___alignof] = ACTIONS(1455),
    [anon_sym__alignof] = ACTIONS(1455),
    [anon_sym_alignof] = ACTIONS(1455),
    [anon_sym__Alignof] = ACTIONS(1455),
    [anon_sym_offsetof] = ACTIONS(1458),
    [anon_sym__Generic] = ACTIONS(1461),
    [anon_sym_asm] = ACTIONS(1464),
    [anon_sym___asm__] = ACTIONS(1464),
    [sym_number_literal] = ACTIONS(1467),
    [anon_sym_L_SQUOTE] = ACTIONS(1470),
    [anon_sym_u_SQUOTE] = ACTIONS(1470),
    [anon_sym_U_SQUOTE] = ACTIONS(1470),
    [anon_sym_u8_SQUOTE] = ACTIONS(1470),
    [anon_sym_SQUOTE] = ACTIONS(1470),
    [anon_sym_L_DQUOTE] = ACTIONS(1473),
    [anon_sym_u_DQUOTE] = ACTIONS(1473),
    [anon_sym_U_DQUOTE] = ACTIONS(1473),
    [anon_sym_u8_DQUOTE] = ACTIONS(1473),
    [anon_sym_DQUOTE] = ACTIONS(1473),
    [sym_true] = ACTIONS(1476),
    [sym_false] = ACTIONS(1476),
    [anon_sym_NULL] = ACTIONS(1479),
    [anon_sym_nullptr] = ACTIONS(1479),
    [sym_comment] = ACTIONS(3),
  },
  [372] = {
    [sym_identifier] = ACTIONS(1308),
    [aux_sym_preproc_include_token1] = ACTIONS(1308),
    [aux_sym_preproc_def_token1] = ACTIONS(1308),
    [aux_sym_preproc_if_token1] = ACTIONS(1308),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1308),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1308),
    [sym_preproc_directive] = ACTIONS(1308),
    [anon_sym_LPAREN2] = ACTIONS(1310),
    [anon_sym_BANG] = ACTIONS(1310),
    [anon_sym_TILDE] = ACTIONS(1310),
    [anon_sym_DASH] = ACTIONS(1308),
    [anon_sym_PLUS] = ACTIONS(1308),
    [anon_sym_STAR] = ACTIONS(1310),
    [anon_sym_AMP] = ACTIONS(1310),
    [anon_sym_SEMI] = ACTIONS(1310),
    [anon_sym___extension__] = ACTIONS(1308),
    [anon_sym_typedef] = ACTIONS(1308),
    [anon_sym_extern] = ACTIONS(1308),
    [anon_sym___attribute__] = ACTIONS(1308),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1310),
    [anon_sym___declspec] = ACTIONS(1308),
    [anon_sym___cdecl] = ACTIONS(1308),
    [anon_sym___clrcall] = ACTIONS(1308),
    [anon_sym___stdcall] = ACTIONS(1308),
    [anon_sym___fastcall] = ACTIONS(1308),
    [anon_sym___thiscall] = ACTIONS(1308),
    [anon_sym___vectorcall] = ACTIONS(1308),
    [anon_sym_LBRACE] = ACTIONS(1310),
    [anon_sym_RBRACE] = ACTIONS(1310),
    [anon_sym_signed] = ACTIONS(1308),
    [anon_sym_unsigned] = ACTIONS(1308),
    [anon_sym_long] = ACTIONS(1308),
    [anon_sym_short] = ACTIONS(1308),
    [anon_sym_static] = ACTIONS(1308),
    [anon_sym_auto] = ACTIONS(1308),
    [anon_sym_register] = ACTIONS(1308),
    [anon_sym_inline] = ACTIONS(1308),
    [anon_sym___inline] = ACTIONS(1308),
    [anon_sym___inline__] = ACTIONS(1308),
    [anon_sym___forceinline] = ACTIONS(1308),
    [anon_sym_thread_local] = ACTIONS(1308),
    [anon_sym___thread] = ACTIONS(1308),
    [anon_sym_const] = ACTIONS(1308),
    [anon_sym_constexpr] = ACTIONS(1308),
    [anon_sym_volatile] = ACTIONS(1308),
    [anon_sym_restrict] = ACTIONS(1308),
    [anon_sym___restrict__] = ACTIONS(1308),
    [anon_sym__Atomic] = ACTIONS(1308),
    [anon_sym__Noreturn] = ACTIONS(1308),
    [anon_sym_noreturn] = ACTIONS(1308),
    [sym_primitive_type] = ACTIONS(1308),
    [anon_sym_enum] = ACTIONS(1308),
    [anon_sym_struct] = ACTIONS(1308),
    [anon_sym_union] = ACTIONS(1308),
    [anon_sym_if] = ACTIONS(1308),
    [anon_sym_switch] = ACTIONS(1308),
    [anon_sym_case] = ACTIONS(1308),
    [anon_sym_default] = ACTIONS(1308),
    [anon_sym_while] = ACTIONS(1308),
    [anon_sym_do] = ACTIONS(1308),
    [anon_sym_for] = ACTIONS(1308),
    [anon_sym_return] = ACTIONS(1308),
    [anon_sym_break] = ACTIONS(1308),
    [anon_sym_continue] = ACTIONS(1308),
    [anon_sym_goto] = ACTIONS(1308),
    [anon_sym___try] = ACTIONS(1308),
    [anon_sym___leave] = ACTIONS(1308),
    [anon_sym_DASH_DASH] = ACTIONS(1310),
    [anon_sym_PLUS_PLUS] = ACTIONS(1310),
    [anon_sym_sizeof] = ACTIONS(1308),
    [anon_sym___alignof__] = ACTIONS(1308),
    [anon_sym___alignof] = ACTIONS(1308),
    [anon_sym__alignof] = ACTIONS(1308),
    [anon_sym_alignof] = ACTIONS(1308),
    [anon_sym__Alignof] = ACTIONS(1308),
    [anon_sym_offsetof] = ACTIONS(1308),
    [anon_sym__Generic] = ACTIONS(1308),
    [anon_sym_asm] = ACTIONS(1308),
    [anon_sym___asm__] = ACTIONS(1308),
    [sym_number_literal] = ACTIONS(1310),
    [anon_sym_L_SQUOTE] = ACTIONS(1310),
    [anon_sym_u_SQUOTE] = ACTIONS(1310),
    [anon_sym_U_SQUOTE] = ACTIONS(1310),
    [anon_sym_u8_SQUOTE] = ACTIONS(1310),
    [anon_sym_SQUOTE] = ACTIONS(1310),
    [anon_sym_L_DQUOTE] = ACTIONS(1310),
    [anon_sym_u_DQUOTE] = ACTIONS(1310),
    [anon_sym_U_DQUOTE] = ACTIONS(1310),
    [anon_sym_u8_DQUOTE] = ACTIONS(1310),
    [anon_sym_DQUOTE] = ACTIONS(1310),
    [sym_true] = ACTIONS(1308),
    [sym_false] = ACTIONS(1308),
    [anon_sym_NULL] = ACTIONS(1308),
    [anon_sym_nullptr] = ACTIONS(1308),
    [sym_comment] = ACTIONS(3),
  },
  [373] = {
    [sym_attribute_declaration] = STATE(373),
    [sym_compound_statement] = STATE(112),
    [sym_attributed_statement] = STATE(112),
    [sym_labeled_statement] = STATE(112),
    [sym_expression_statement] = STATE(112),
    [sym_if_statement] = STATE(112),
    [sym_switch_statement] = STATE(112),
    [sym_case_statement] = STATE(112),
    [sym_while_statement] = STATE(112),
    [sym_do_statement] = STATE(112),
    [sym_for_statement] = STATE(112),
    [sym_return_statement] = STATE(112),
    [sym_break_statement] = STATE(112),
    [sym_continue_statement] = STATE(112),
    [sym_goto_statement] = STATE(112),
    [sym_seh_try_statement] = STATE(112),
    [sym_seh_leave_statement] = STATE(112),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(373),
    [sym_identifier] = ACTIONS(1603),
    [anon_sym_LPAREN2] = ACTIONS(1389),
    [anon_sym_BANG] = ACTIONS(1392),
    [anon_sym_TILDE] = ACTIONS(1392),
    [anon_sym_DASH] = ACTIONS(1395),
    [anon_sym_PLUS] = ACTIONS(1395),
    [anon_sym_STAR] = ACTIONS(1398),
    [anon_sym_AMP] = ACTIONS(1398),
    [anon_sym_SEMI] = ACTIONS(1606),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1404),
    [anon_sym_LBRACE] = ACTIONS(1609),
    [anon_sym_if] = ACTIONS(1612),
    [anon_sym_switch] = ACTIONS(1615),
    [anon_sym_case] = ACTIONS(1618),
    [anon_sym_default] = ACTIONS(1621),
    [anon_sym_while] = ACTIONS(1624),
    [anon_sym_do] = ACTIONS(1627),
    [anon_sym_for] = ACTIONS(1630),
    [anon_sym_return] = ACTIONS(1633),
    [anon_sym_break] = ACTIONS(1636),
    [anon_sym_continue] = ACTIONS(1639),
    [anon_sym_goto] = ACTIONS(1642),
    [anon_sym___try] = ACTIONS(1645),
    [anon_sym___leave] = ACTIONS(1648),
    [anon_sym_DASH_DASH] = ACTIONS(1449),
    [anon_sym_PLUS_PLUS] = ACTIONS(1449),
    [anon_sym_sizeof] = ACTIONS(1452),
    [anon_sym___alignof__] = ACTIONS(1455),
    [anon_sym___alignof] = ACTIONS(1455),
    [anon_sym__alignof] = ACTIONS(1455),
    [anon_sym_alignof] = ACTIONS(1455),
    [anon_sym__Alignof] = ACTIONS(1455),
    [anon_sym_offsetof] = ACTIONS(1458),
    [anon_sym__Generic] = ACTIONS(1461),
    [anon_sym_asm] = ACTIONS(1464),
    [anon_sym___asm__] = ACTIONS(1464),
    [sym_number_literal] = ACTIONS(1467),
    [anon_sym_L_SQUOTE] = ACTIONS(1470),
    [anon_sym_u_SQUOTE] = ACTIONS(1470),
    [anon_sym_U_SQUOTE] = ACTIONS(1470),
    [anon_sym_u8_SQUOTE] = ACTIONS(1470),
    [anon_sym_SQUOTE] = ACTIONS(1470),
    [anon_sym_L_DQUOTE] = ACTIONS(1473),
    [anon_sym_u_DQUOTE] = ACTIONS(1473),
    [anon_sym_U_DQUOTE] = ACTIONS(1473),
    [anon_sym_u8_DQUOTE] = ACTIONS(1473),
    [anon_sym_DQUOTE] = ACTIONS(1473),
    [sym_true] = ACTIONS(1476),
    [sym_false] = ACTIONS(1476),
    [anon_sym_NULL] = ACTIONS(1479),
    [anon_sym_nullptr] = ACTIONS(1479),
    [sym_comment] = ACTIONS(3),
  },
  [374] = {
    [sym_identifier] = ACTIONS(1256),
    [aux_sym_preproc_include_token1] = ACTIONS(1256),
    [aux_sym_preproc_def_token1] = ACTIONS(1256),
    [aux_sym_preproc_if_token1] = ACTIONS(1256),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1256),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1256),
    [sym_preproc_directive] = ACTIONS(1256),
    [anon_sym_LPAREN2] = ACTIONS(1258),
    [anon_sym_BANG] = ACTIONS(1258),
    [anon_sym_TILDE] = ACTIONS(1258),
    [anon_sym_DASH] = ACTIONS(1256),
    [anon_sym_PLUS] = ACTIONS(1256),
    [anon_sym_STAR] = ACTIONS(1258),
    [anon_sym_AMP] = ACTIONS(1258),
    [anon_sym_SEMI] = ACTIONS(1258),
    [anon_sym___extension__] = ACTIONS(1256),
    [anon_sym_typedef] = ACTIONS(1256),
    [anon_sym_extern] = ACTIONS(1256),
    [anon_sym___attribute__] = ACTIONS(1256),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1258),
    [anon_sym___declspec] = ACTIONS(1256),
    [anon_sym___cdecl] = ACTIONS(1256),
    [anon_sym___clrcall] = ACTIONS(1256),
    [anon_sym___stdcall] = ACTIONS(1256),
    [anon_sym___fastcall] = ACTIONS(1256),
    [anon_sym___thiscall] = ACTIONS(1256),
    [anon_sym___vectorcall] = ACTIONS(1256),
    [anon_sym_LBRACE] = ACTIONS(1258),
    [anon_sym_RBRACE] = ACTIONS(1258),
    [anon_sym_signed] = ACTIONS(1256),
    [anon_sym_unsigned] = ACTIONS(1256),
    [anon_sym_long] = ACTIONS(1256),
    [anon_sym_short] = ACTIONS(1256),
    [anon_sym_static] = ACTIONS(1256),
    [anon_sym_auto] = ACTIONS(1256),
    [anon_sym_register] = ACTIONS(1256),
    [anon_sym_inline] = ACTIONS(1256),
    [anon_sym___inline] = ACTIONS(1256),
    [anon_sym___inline__] = ACTIONS(1256),
    [anon_sym___forceinline] = ACTIONS(1256),
    [anon_sym_thread_local] = ACTIONS(1256),
    [anon_sym___thread] = ACTIONS(1256),
    [anon_sym_const] = ACTIONS(1256),
    [anon_sym_constexpr] = ACTIONS(1256),
    [anon_sym_volatile] = ACTIONS(1256),
    [anon_sym_restrict] = ACTIONS(1256),
    [anon_sym___restrict__] = ACTIONS(1256),
    [anon_sym__Atomic] = ACTIONS(1256),
    [anon_sym__Noreturn] = ACTIONS(1256),
    [anon_sym_noreturn] = ACTIONS(1256),
    [sym_primitive_type] = ACTIONS(1256),
    [anon_sym_enum] = ACTIONS(1256),
    [anon_sym_struct] = ACTIONS(1256),
    [anon_sym_union] = ACTIONS(1256),
    [anon_sym_if] = ACTIONS(1256),
    [anon_sym_switch] = ACTIONS(1256),
    [anon_sym_case] = ACTIONS(1256),
    [anon_sym_default] = ACTIONS(1256),
    [anon_sym_while] = ACTIONS(1256),
    [anon_sym_do] = ACTIONS(1256),
    [anon_sym_for] = ACTIONS(1256),
    [anon_sym_return] = ACTIONS(1256),
    [anon_sym_break] = ACTIONS(1256),
    [anon_sym_continue] = ACTIONS(1256),
    [anon_sym_goto] = ACTIONS(1256),
    [anon_sym___try] = ACTIONS(1256),
    [anon_sym___leave] = ACTIONS(1256),
    [anon_sym_DASH_DASH] = ACTIONS(1258),
    [anon_sym_PLUS_PLUS] = ACTIONS(1258),
    [anon_sym_sizeof] = ACTIONS(1256),
    [anon_sym___alignof__] = ACTIONS(1256),
    [anon_sym___alignof] = ACTIONS(1256),
    [anon_sym__alignof] = ACTIONS(1256),
    [anon_sym_alignof] = ACTIONS(1256),
    [anon_sym__Alignof] = ACTIONS(1256),
    [anon_sym_offsetof] = ACTIONS(1256),
    [anon_sym__Generic] = ACTIONS(1256),
    [anon_sym_asm] = ACTIONS(1256),
    [anon_sym___asm__] = ACTIONS(1256),
    [sym_number_literal] = ACTIONS(1258),
    [anon_sym_L_SQUOTE] = ACTIONS(1258),
    [anon_sym_u_SQUOTE] = ACTIONS(1258),
    [anon_sym_U_SQUOTE] = ACTIONS(1258),
    [anon_sym_u8_SQUOTE] = ACTIONS(1258),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_L_DQUOTE] = ACTIONS(1258),
    [anon_sym_u_DQUOTE] = ACTIONS(1258),
    [anon_sym_U_DQUOTE] = ACTIONS(1258),
    [anon_sym_u8_DQUOTE] = ACTIONS(1258),
    [anon_sym_DQUOTE] = ACTIONS(1258),
    [sym_true] = ACTIONS(1256),
    [sym_false] = ACTIONS(1256),
    [anon_sym_NULL] = ACTIONS(1256),
    [anon_sym_nullptr] = ACTIONS(1256),
    [sym_comment] = ACTIONS(3),
  },
  [375] = {
    [sym_identifier] = ACTIONS(1300),
    [aux_sym_preproc_include_token1] = ACTIONS(1300),
    [aux_sym_preproc_def_token1] = ACTIONS(1300),
    [aux_sym_preproc_if_token1] = ACTIONS(1300),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1300),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1300),
    [sym_preproc_directive] = ACTIONS(1300),
    [anon_sym_LPAREN2] = ACTIONS(1302),
    [anon_sym_BANG] = ACTIONS(1302),
    [anon_sym_TILDE] = ACTIONS(1302),
    [anon_sym_DASH] = ACTIONS(1300),
    [anon_sym_PLUS] = ACTIONS(1300),
    [anon_sym_STAR] = ACTIONS(1302),
    [anon_sym_AMP] = ACTIONS(1302),
    [anon_sym_SEMI] = ACTIONS(1302),
    [anon_sym___extension__] = ACTIONS(1300),
    [anon_sym_typedef] = ACTIONS(1300),
    [anon_sym_extern] = ACTIONS(1300),
    [anon_sym___attribute__] = ACTIONS(1300),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1302),
    [anon_sym___declspec] = ACTIONS(1300),
    [anon_sym___cdecl] = ACTIONS(1300),
    [anon_sym___clrcall] = ACTIONS(1300),
    [anon_sym___stdcall] = ACTIONS(1300),
    [anon_sym___fastcall] = ACTIONS(1300),
    [anon_sym___thiscall] = ACTIONS(1300),
    [anon_sym___vectorcall] = ACTIONS(1300),
    [anon_sym_LBRACE] = ACTIONS(1302),
    [anon_sym_RBRACE] = ACTIONS(1302),
    [anon_sym_signed] = ACTIONS(1300),
    [anon_sym_unsigned] = ACTIONS(1300),
    [anon_sym_long] = ACTIONS(1300),
    [anon_sym_short] = ACTIONS(1300),
    [anon_sym_static] = ACTIONS(1300),
    [anon_sym_auto] = ACTIONS(1300),
    [anon_sym_register] = ACTIONS(1300),
    [anon_sym_inline] = ACTIONS(1300),
    [anon_sym___inline] = ACTIONS(1300),
    [anon_sym___inline__] = ACTIONS(1300),
    [anon_sym___forceinline] = ACTIONS(1300),
    [anon_sym_thread_local] = ACTIONS(1300),
    [anon_sym___thread] = ACTIONS(1300),
    [anon_sym_const] = ACTIONS(1300),
    [anon_sym_constexpr] = ACTIONS(1300),
    [anon_sym_volatile] = ACTIONS(1300),
    [anon_sym_restrict] = ACTIONS(1300),
    [anon_sym___restrict__] = ACTIONS(1300),
    [anon_sym__Atomic] = ACTIONS(1300),
    [anon_sym__Noreturn] = ACTIONS(1300),
    [anon_sym_noreturn] = ACTIONS(1300),
    [sym_primitive_type] = ACTIONS(1300),
    [anon_sym_enum] = ACTIONS(1300),
    [anon_sym_struct] = ACTIONS(1300),
    [anon_sym_union] = ACTIONS(1300),
    [anon_sym_if] = ACTIONS(1300),
    [anon_sym_switch] = ACTIONS(1300),
    [anon_sym_case] = ACTIONS(1300),
    [anon_sym_default] = ACTIONS(1300),
    [anon_sym_while] = ACTIONS(1300),
    [anon_sym_do] = ACTIONS(1300),
    [anon_sym_for] = ACTIONS(1300),
    [anon_sym_return] = ACTIONS(1300),
    [anon_sym_break] = ACTIONS(1300),
    [anon_sym_continue] = ACTIONS(1300),
    [anon_sym_goto] = ACTIONS(1300),
    [anon_sym___try] = ACTIONS(1300),
    [anon_sym___leave] = ACTIONS(1300),
    [anon_sym_DASH_DASH] = ACTIONS(1302),
    [anon_sym_PLUS_PLUS] = ACTIONS(1302),
    [anon_sym_sizeof] = ACTIONS(1300),
    [anon_sym___alignof__] = ACTIONS(1300),
    [anon_sym___alignof] = ACTIONS(1300),
    [anon_sym__alignof] = ACTIONS(1300),
    [anon_sym_alignof] = ACTIONS(1300),
    [anon_sym__Alignof] = ACTIONS(1300),
    [anon_sym_offsetof] = ACTIONS(1300),
    [anon_sym__Generic] = ACTIONS(1300),
    [anon_sym_asm] = ACTIONS(1300),
    [anon_sym___asm__] = ACTIONS(1300),
    [sym_number_literal] = ACTIONS(1302),
    [anon_sym_L_SQUOTE] = ACTIONS(1302),
    [anon_sym_u_SQUOTE] = ACTIONS(1302),
    [anon_sym_U_SQUOTE] = ACTIONS(1302),
    [anon_sym_u8_SQUOTE] = ACTIONS(1302),
    [anon_sym_SQUOTE] = ACTIONS(1302),
    [anon_sym_L_DQUOTE] = ACTIONS(1302),
    [anon_sym_u_DQUOTE] = ACTIONS(1302),
    [anon_sym_U_DQUOTE] = ACTIONS(1302),
    [anon_sym_u8_DQUOTE] = ACTIONS(1302),
    [anon_sym_DQUOTE] = ACTIONS(1302),
    [sym_true] = ACTIONS(1300),
    [sym_false] = ACTIONS(1300),
    [anon_sym_NULL] = ACTIONS(1300),
    [anon_sym_nullptr] = ACTIONS(1300),
    [sym_comment] = ACTIONS(3),
  },
  [376] = {
    [sym_identifier] = ACTIONS(1244),
    [aux_sym_preproc_include_token1] = ACTIONS(1244),
    [aux_sym_preproc_def_token1] = ACTIONS(1244),
    [aux_sym_preproc_if_token1] = ACTIONS(1244),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1244),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1244),
    [sym_preproc_directive] = ACTIONS(1244),
    [anon_sym_LPAREN2] = ACTIONS(1246),
    [anon_sym_BANG] = ACTIONS(1246),
    [anon_sym_TILDE] = ACTIONS(1246),
    [anon_sym_DASH] = ACTIONS(1244),
    [anon_sym_PLUS] = ACTIONS(1244),
    [anon_sym_STAR] = ACTIONS(1246),
    [anon_sym_AMP] = ACTIONS(1246),
    [anon_sym_SEMI] = ACTIONS(1246),
    [anon_sym___extension__] = ACTIONS(1244),
    [anon_sym_typedef] = ACTIONS(1244),
    [anon_sym_extern] = ACTIONS(1244),
    [anon_sym___attribute__] = ACTIONS(1244),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1246),
    [anon_sym___declspec] = ACTIONS(1244),
    [anon_sym___cdecl] = ACTIONS(1244),
    [anon_sym___clrcall] = ACTIONS(1244),
    [anon_sym___stdcall] = ACTIONS(1244),
    [anon_sym___fastcall] = ACTIONS(1244),
    [anon_sym___thiscall] = ACTIONS(1244),
    [anon_sym___vectorcall] = ACTIONS(1244),
    [anon_sym_LBRACE] = ACTIONS(1246),
    [anon_sym_RBRACE] = ACTIONS(1246),
    [anon_sym_signed] = ACTIONS(1244),
    [anon_sym_unsigned] = ACTIONS(1244),
    [anon_sym_long] = ACTIONS(1244),
    [anon_sym_short] = ACTIONS(1244),
    [anon_sym_static] = ACTIONS(1244),
    [anon_sym_auto] = ACTIONS(1244),
    [anon_sym_register] = ACTIONS(1244),
    [anon_sym_inline] = ACTIONS(1244),
    [anon_sym___inline] = ACTIONS(1244),
    [anon_sym___inline__] = ACTIONS(1244),
    [anon_sym___forceinline] = ACTIONS(1244),
    [anon_sym_thread_local] = ACTIONS(1244),
    [anon_sym___thread] = ACTIONS(1244),
    [anon_sym_const] = ACTIONS(1244),
    [anon_sym_constexpr] = ACTIONS(1244),
    [anon_sym_volatile] = ACTIONS(1244),
    [anon_sym_restrict] = ACTIONS(1244),
    [anon_sym___restrict__] = ACTIONS(1244),
    [anon_sym__Atomic] = ACTIONS(1244),
    [anon_sym__Noreturn] = ACTIONS(1244),
    [anon_sym_noreturn] = ACTIONS(1244),
    [sym_primitive_type] = ACTIONS(1244),
    [anon_sym_enum] = ACTIONS(1244),
    [anon_sym_struct] = ACTIONS(1244),
    [anon_sym_union] = ACTIONS(1244),
    [anon_sym_if] = ACTIONS(1244),
    [anon_sym_switch] = ACTIONS(1244),
    [anon_sym_case] = ACTIONS(1244),
    [anon_sym_default] = ACTIONS(1244),
    [anon_sym_while] = ACTIONS(1244),
    [anon_sym_do] = ACTIONS(1244),
    [anon_sym_for] = ACTIONS(1244),
    [anon_sym_return] = ACTIONS(1244),
    [anon_sym_break] = ACTIONS(1244),
    [anon_sym_continue] = ACTIONS(1244),
    [anon_sym_goto] = ACTIONS(1244),
    [anon_sym___try] = ACTIONS(1244),
    [anon_sym___leave] = ACTIONS(1244),
    [anon_sym_DASH_DASH] = ACTIONS(1246),
    [anon_sym_PLUS_PLUS] = ACTIONS(1246),
    [anon_sym_sizeof] = ACTIONS(1244),
    [anon_sym___alignof__] = ACTIONS(1244),
    [anon_sym___alignof] = ACTIONS(1244),
    [anon_sym__alignof] = ACTIONS(1244),
    [anon_sym_alignof] = ACTIONS(1244),
    [anon_sym__Alignof] = ACTIONS(1244),
    [anon_sym_offsetof] = ACTIONS(1244),
    [anon_sym__Generic] = ACTIONS(1244),
    [anon_sym_asm] = ACTIONS(1244),
    [anon_sym___asm__] = ACTIONS(1244),
    [sym_number_literal] = ACTIONS(1246),
    [anon_sym_L_SQUOTE] = ACTIONS(1246),
    [anon_sym_u_SQUOTE] = ACTIONS(1246),
    [anon_sym_U_SQUOTE] = ACTIONS(1246),
    [anon_sym_u8_SQUOTE] = ACTIONS(1246),
    [anon_sym_SQUOTE] = ACTIONS(1246),
    [anon_sym_L_DQUOTE] = ACTIONS(1246),
    [anon_sym_u_DQUOTE] = ACTIONS(1246),
    [anon_sym_U_DQUOTE] = ACTIONS(1246),
    [anon_sym_u8_DQUOTE] = ACTIONS(1246),
    [anon_sym_DQUOTE] = ACTIONS(1246),
    [sym_true] = ACTIONS(1244),
    [sym_false] = ACTIONS(1244),
    [anon_sym_NULL] = ACTIONS(1244),
    [anon_sym_nullptr] = ACTIONS(1244),
    [sym_comment] = ACTIONS(3),
  },
  [377] = {
    [sym_identifier] = ACTIONS(1260),
    [aux_sym_preproc_include_token1] = ACTIONS(1260),
    [aux_sym_preproc_def_token1] = ACTIONS(1260),
    [aux_sym_preproc_if_token1] = ACTIONS(1260),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1260),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1260),
    [sym_preproc_directive] = ACTIONS(1260),
    [anon_sym_LPAREN2] = ACTIONS(1262),
    [anon_sym_BANG] = ACTIONS(1262),
    [anon_sym_TILDE] = ACTIONS(1262),
    [anon_sym_DASH] = ACTIONS(1260),
    [anon_sym_PLUS] = ACTIONS(1260),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1262),
    [anon_sym_SEMI] = ACTIONS(1262),
    [anon_sym___extension__] = ACTIONS(1260),
    [anon_sym_typedef] = ACTIONS(1260),
    [anon_sym_extern] = ACTIONS(1260),
    [anon_sym___attribute__] = ACTIONS(1260),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1262),
    [anon_sym___declspec] = ACTIONS(1260),
    [anon_sym___cdecl] = ACTIONS(1260),
    [anon_sym___clrcall] = ACTIONS(1260),
    [anon_sym___stdcall] = ACTIONS(1260),
    [anon_sym___fastcall] = ACTIONS(1260),
    [anon_sym___thiscall] = ACTIONS(1260),
    [anon_sym___vectorcall] = ACTIONS(1260),
    [anon_sym_LBRACE] = ACTIONS(1262),
    [anon_sym_RBRACE] = ACTIONS(1262),
    [anon_sym_signed] = ACTIONS(1260),
    [anon_sym_unsigned] = ACTIONS(1260),
    [anon_sym_long] = ACTIONS(1260),
    [anon_sym_short] = ACTIONS(1260),
    [anon_sym_static] = ACTIONS(1260),
    [anon_sym_auto] = ACTIONS(1260),
    [anon_sym_register] = ACTIONS(1260),
    [anon_sym_inline] = ACTIONS(1260),
    [anon_sym___inline] = ACTIONS(1260),
    [anon_sym___inline__] = ACTIONS(1260),
    [anon_sym___forceinline] = ACTIONS(1260),
    [anon_sym_thread_local] = ACTIONS(1260),
    [anon_sym___thread] = ACTIONS(1260),
    [anon_sym_const] = ACTIONS(1260),
    [anon_sym_constexpr] = ACTIONS(1260),
    [anon_sym_volatile] = ACTIONS(1260),
    [anon_sym_restrict] = ACTIONS(1260),
    [anon_sym___restrict__] = ACTIONS(1260),
    [anon_sym__Atomic] = ACTIONS(1260),
    [anon_sym__Noreturn] = ACTIONS(1260),
    [anon_sym_noreturn] = ACTIONS(1260),
    [sym_primitive_type] = ACTIONS(1260),
    [anon_sym_enum] = ACTIONS(1260),
    [anon_sym_struct] = ACTIONS(1260),
    [anon_sym_union] = ACTIONS(1260),
    [anon_sym_if] = ACTIONS(1260),
    [anon_sym_switch] = ACTIONS(1260),
    [anon_sym_case] = ACTIONS(1260),
    [anon_sym_default] = ACTIONS(1260),
    [anon_sym_while] = ACTIONS(1260),
    [anon_sym_do] = ACTIONS(1260),
    [anon_sym_for] = ACTIONS(1260),
    [anon_sym_return] = ACTIONS(1260),
    [anon_sym_break] = ACTIONS(1260),
    [anon_sym_continue] = ACTIONS(1260),
    [anon_sym_goto] = ACTIONS(1260),
    [anon_sym___try] = ACTIONS(1260),
    [anon_sym___leave] = ACTIONS(1260),
    [anon_sym_DASH_DASH] = ACTIONS(1262),
    [anon_sym_PLUS_PLUS] = ACTIONS(1262),
    [anon_sym_sizeof] = ACTIONS(1260),
    [anon_sym___alignof__] = ACTIONS(1260),
    [anon_sym___alignof] = ACTIONS(1260),
    [anon_sym__alignof] = ACTIONS(1260),
    [anon_sym_alignof] = ACTIONS(1260),
    [anon_sym__Alignof] = ACTIONS(1260),
    [anon_sym_offsetof] = ACTIONS(1260),
    [anon_sym__Generic] = ACTIONS(1260),
    [anon_sym_asm] = ACTIONS(1260),
    [anon_sym___asm__] = ACTIONS(1260),
    [sym_number_literal] = ACTIONS(1262),
    [anon_sym_L_SQUOTE] = ACTIONS(1262),
    [anon_sym_u_SQUOTE] = ACTIONS(1262),
    [anon_sym_U_SQUOTE] = ACTIONS(1262),
    [anon_sym_u8_SQUOTE] = ACTIONS(1262),
    [anon_sym_SQUOTE] = ACTIONS(1262),
    [anon_sym_L_DQUOTE] = ACTIONS(1262),
    [anon_sym_u_DQUOTE] = ACTIONS(1262),
    [anon_sym_U_DQUOTE] = ACTIONS(1262),
    [anon_sym_u8_DQUOTE] = ACTIONS(1262),
    [anon_sym_DQUOTE] = ACTIONS(1262),
    [sym_true] = ACTIONS(1260),
    [sym_false] = ACTIONS(1260),
    [anon_sym_NULL] = ACTIONS(1260),
    [anon_sym_nullptr] = ACTIONS(1260),
    [sym_comment] = ACTIONS(3),
  },
  [378] = {
    [sym_identifier] = ACTIONS(1264),
    [aux_sym_preproc_include_token1] = ACTIONS(1264),
    [aux_sym_preproc_def_token1] = ACTIONS(1264),
    [aux_sym_preproc_if_token1] = ACTIONS(1264),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1264),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1264),
    [sym_preproc_directive] = ACTIONS(1264),
    [anon_sym_LPAREN2] = ACTIONS(1266),
    [anon_sym_BANG] = ACTIONS(1266),
    [anon_sym_TILDE] = ACTIONS(1266),
    [anon_sym_DASH] = ACTIONS(1264),
    [anon_sym_PLUS] = ACTIONS(1264),
    [anon_sym_STAR] = ACTIONS(1266),
    [anon_sym_AMP] = ACTIONS(1266),
    [anon_sym_SEMI] = ACTIONS(1266),
    [anon_sym___extension__] = ACTIONS(1264),
    [anon_sym_typedef] = ACTIONS(1264),
    [anon_sym_extern] = ACTIONS(1264),
    [anon_sym___attribute__] = ACTIONS(1264),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1266),
    [anon_sym___declspec] = ACTIONS(1264),
    [anon_sym___cdecl] = ACTIONS(1264),
    [anon_sym___clrcall] = ACTIONS(1264),
    [anon_sym___stdcall] = ACTIONS(1264),
    [anon_sym___fastcall] = ACTIONS(1264),
    [anon_sym___thiscall] = ACTIONS(1264),
    [anon_sym___vectorcall] = ACTIONS(1264),
    [anon_sym_LBRACE] = ACTIONS(1266),
    [anon_sym_RBRACE] = ACTIONS(1266),
    [anon_sym_signed] = ACTIONS(1264),
    [anon_sym_unsigned] = ACTIONS(1264),
    [anon_sym_long] = ACTIONS(1264),
    [anon_sym_short] = ACTIONS(1264),
    [anon_sym_static] = ACTIONS(1264),
    [anon_sym_auto] = ACTIONS(1264),
    [anon_sym_register] = ACTIONS(1264),
    [anon_sym_inline] = ACTIONS(1264),
    [anon_sym___inline] = ACTIONS(1264),
    [anon_sym___inline__] = ACTIONS(1264),
    [anon_sym___forceinline] = ACTIONS(1264),
    [anon_sym_thread_local] = ACTIONS(1264),
    [anon_sym___thread] = ACTIONS(1264),
    [anon_sym_const] = ACTIONS(1264),
    [anon_sym_constexpr] = ACTIONS(1264),
    [anon_sym_volatile] = ACTIONS(1264),
    [anon_sym_restrict] = ACTIONS(1264),
    [anon_sym___restrict__] = ACTIONS(1264),
    [anon_sym__Atomic] = ACTIONS(1264),
    [anon_sym__Noreturn] = ACTIONS(1264),
    [anon_sym_noreturn] = ACTIONS(1264),
    [sym_primitive_type] = ACTIONS(1264),
    [anon_sym_enum] = ACTIONS(1264),
    [anon_sym_struct] = ACTIONS(1264),
    [anon_sym_union] = ACTIONS(1264),
    [anon_sym_if] = ACTIONS(1264),
    [anon_sym_switch] = ACTIONS(1264),
    [anon_sym_case] = ACTIONS(1264),
    [anon_sym_default] = ACTIONS(1264),
    [anon_sym_while] = ACTIONS(1264),
    [anon_sym_do] = ACTIONS(1264),
    [anon_sym_for] = ACTIONS(1264),
    [anon_sym_return] = ACTIONS(1264),
    [anon_sym_break] = ACTIONS(1264),
    [anon_sym_continue] = ACTIONS(1264),
    [anon_sym_goto] = ACTIONS(1264),
    [anon_sym___try] = ACTIONS(1264),
    [anon_sym___leave] = ACTIONS(1264),
    [anon_sym_DASH_DASH] = ACTIONS(1266),
    [anon_sym_PLUS_PLUS] = ACTIONS(1266),
    [anon_sym_sizeof] = ACTIONS(1264),
    [anon_sym___alignof__] = ACTIONS(1264),
    [anon_sym___alignof] = ACTIONS(1264),
    [anon_sym__alignof] = ACTIONS(1264),
    [anon_sym_alignof] = ACTIONS(1264),
    [anon_sym__Alignof] = ACTIONS(1264),
    [anon_sym_offsetof] = ACTIONS(1264),
    [anon_sym__Generic] = ACTIONS(1264),
    [anon_sym_asm] = ACTIONS(1264),
    [anon_sym___asm__] = ACTIONS(1264),
    [sym_number_literal] = ACTIONS(1266),
    [anon_sym_L_SQUOTE] = ACTIONS(1266),
    [anon_sym_u_SQUOTE] = ACTIONS(1266),
    [anon_sym_U_SQUOTE] = ACTIONS(1266),
    [anon_sym_u8_SQUOTE] = ACTIONS(1266),
    [anon_sym_SQUOTE] = ACTIONS(1266),
    [anon_sym_L_DQUOTE] = ACTIONS(1266),
    [anon_sym_u_DQUOTE] = ACTIONS(1266),
    [anon_sym_U_DQUOTE] = ACTIONS(1266),
    [anon_sym_u8_DQUOTE] = ACTIONS(1266),
    [anon_sym_DQUOTE] = ACTIONS(1266),
    [sym_true] = ACTIONS(1264),
    [sym_false] = ACTIONS(1264),
    [anon_sym_NULL] = ACTIONS(1264),
    [anon_sym_nullptr] = ACTIONS(1264),
    [sym_comment] = ACTIONS(3),
  },
  [379] = {
    [sym_identifier] = ACTIONS(1268),
    [aux_sym_preproc_include_token1] = ACTIONS(1268),
    [aux_sym_preproc_def_token1] = ACTIONS(1268),
    [aux_sym_preproc_if_token1] = ACTIONS(1268),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1268),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1268),
    [sym_preproc_directive] = ACTIONS(1268),
    [anon_sym_LPAREN2] = ACTIONS(1270),
    [anon_sym_BANG] = ACTIONS(1270),
    [anon_sym_TILDE] = ACTIONS(1270),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1270),
    [anon_sym_AMP] = ACTIONS(1270),
    [anon_sym_SEMI] = ACTIONS(1270),
    [anon_sym___extension__] = ACTIONS(1268),
    [anon_sym_typedef] = ACTIONS(1268),
    [anon_sym_extern] = ACTIONS(1268),
    [anon_sym___attribute__] = ACTIONS(1268),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1270),
    [anon_sym___declspec] = ACTIONS(1268),
    [anon_sym___cdecl] = ACTIONS(1268),
    [anon_sym___clrcall] = ACTIONS(1268),
    [anon_sym___stdcall] = ACTIONS(1268),
    [anon_sym___fastcall] = ACTIONS(1268),
    [anon_sym___thiscall] = ACTIONS(1268),
    [anon_sym___vectorcall] = ACTIONS(1268),
    [anon_sym_LBRACE] = ACTIONS(1270),
    [anon_sym_RBRACE] = ACTIONS(1270),
    [anon_sym_signed] = ACTIONS(1268),
    [anon_sym_unsigned] = ACTIONS(1268),
    [anon_sym_long] = ACTIONS(1268),
    [anon_sym_short] = ACTIONS(1268),
    [anon_sym_static] = ACTIONS(1268),
    [anon_sym_auto] = ACTIONS(1268),
    [anon_sym_register] = ACTIONS(1268),
    [anon_sym_inline] = ACTIONS(1268),
    [anon_sym___inline] = ACTIONS(1268),
    [anon_sym___inline__] = ACTIONS(1268),
    [anon_sym___forceinline] = ACTIONS(1268),
    [anon_sym_thread_local] = ACTIONS(1268),
    [anon_sym___thread] = ACTIONS(1268),
    [anon_sym_const] = ACTIONS(1268),
    [anon_sym_constexpr] = ACTIONS(1268),
    [anon_sym_volatile] = ACTIONS(1268),
    [anon_sym_restrict] = ACTIONS(1268),
    [anon_sym___restrict__] = ACTIONS(1268),
    [anon_sym__Atomic] = ACTIONS(1268),
    [anon_sym__Noreturn] = ACTIONS(1268),
    [anon_sym_noreturn] = ACTIONS(1268),
    [sym_primitive_type] = ACTIONS(1268),
    [anon_sym_enum] = ACTIONS(1268),
    [anon_sym_struct] = ACTIONS(1268),
    [anon_sym_union] = ACTIONS(1268),
    [anon_sym_if] = ACTIONS(1268),
    [anon_sym_switch] = ACTIONS(1268),
    [anon_sym_case] = ACTIONS(1268),
    [anon_sym_default] = ACTIONS(1268),
    [anon_sym_while] = ACTIONS(1268),
    [anon_sym_do] = ACTIONS(1268),
    [anon_sym_for] = ACTIONS(1268),
    [anon_sym_return] = ACTIONS(1268),
    [anon_sym_break] = ACTIONS(1268),
    [anon_sym_continue] = ACTIONS(1268),
    [anon_sym_goto] = ACTIONS(1268),
    [anon_sym___try] = ACTIONS(1268),
    [anon_sym___leave] = ACTIONS(1268),
    [anon_sym_DASH_DASH] = ACTIONS(1270),
    [anon_sym_PLUS_PLUS] = ACTIONS(1270),
    [anon_sym_sizeof] = ACTIONS(1268),
    [anon_sym___alignof__] = ACTIONS(1268),
    [anon_sym___alignof] = ACTIONS(1268),
    [anon_sym__alignof] = ACTIONS(1268),
    [anon_sym_alignof] = ACTIONS(1268),
    [anon_sym__Alignof] = ACTIONS(1268),
    [anon_sym_offsetof] = ACTIONS(1268),
    [anon_sym__Generic] = ACTIONS(1268),
    [anon_sym_asm] = ACTIONS(1268),
    [anon_sym___asm__] = ACTIONS(1268),
    [sym_number_literal] = ACTIONS(1270),
    [anon_sym_L_SQUOTE] = ACTIONS(1270),
    [anon_sym_u_SQUOTE] = ACTIONS(1270),
    [anon_sym_U_SQUOTE] = ACTIONS(1270),
    [anon_sym_u8_SQUOTE] = ACTIONS(1270),
    [anon_sym_SQUOTE] = ACTIONS(1270),
    [anon_sym_L_DQUOTE] = ACTIONS(1270),
    [anon_sym_u_DQUOTE] = ACTIONS(1270),
    [anon_sym_U_DQUOTE] = ACTIONS(1270),
    [anon_sym_u8_DQUOTE] = ACTIONS(1270),
    [anon_sym_DQUOTE] = ACTIONS(1270),
    [sym_true] = ACTIONS(1268),
    [sym_false] = ACTIONS(1268),
    [anon_sym_NULL] = ACTIONS(1268),
    [anon_sym_nullptr] = ACTIONS(1268),
    [sym_comment] = ACTIONS(3),
  },
  [380] = {
    [sym_identifier] = ACTIONS(1272),
    [aux_sym_preproc_include_token1] = ACTIONS(1272),
    [aux_sym_preproc_def_token1] = ACTIONS(1272),
    [aux_sym_preproc_if_token1] = ACTIONS(1272),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1272),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1272),
    [sym_preproc_directive] = ACTIONS(1272),
    [anon_sym_LPAREN2] = ACTIONS(1274),
    [anon_sym_BANG] = ACTIONS(1274),
    [anon_sym_TILDE] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1272),
    [anon_sym_STAR] = ACTIONS(1274),
    [anon_sym_AMP] = ACTIONS(1274),
    [anon_sym_SEMI] = ACTIONS(1274),
    [anon_sym___extension__] = ACTIONS(1272),
    [anon_sym_typedef] = ACTIONS(1272),
    [anon_sym_extern] = ACTIONS(1272),
    [anon_sym___attribute__] = ACTIONS(1272),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1274),
    [anon_sym___declspec] = ACTIONS(1272),
    [anon_sym___cdecl] = ACTIONS(1272),
    [anon_sym___clrcall] = ACTIONS(1272),
    [anon_sym___stdcall] = ACTIONS(1272),
    [anon_sym___fastcall] = ACTIONS(1272),
    [anon_sym___thiscall] = ACTIONS(1272),
    [anon_sym___vectorcall] = ACTIONS(1272),
    [anon_sym_LBRACE] = ACTIONS(1274),
    [anon_sym_RBRACE] = ACTIONS(1274),
    [anon_sym_signed] = ACTIONS(1272),
    [anon_sym_unsigned] = ACTIONS(1272),
    [anon_sym_long] = ACTIONS(1272),
    [anon_sym_short] = ACTIONS(1272),
    [anon_sym_static] = ACTIONS(1272),
    [anon_sym_auto] = ACTIONS(1272),
    [anon_sym_register] = ACTIONS(1272),
    [anon_sym_inline] = ACTIONS(1272),
    [anon_sym___inline] = ACTIONS(1272),
    [anon_sym___inline__] = ACTIONS(1272),
    [anon_sym___forceinline] = ACTIONS(1272),
    [anon_sym_thread_local] = ACTIONS(1272),
    [anon_sym___thread] = ACTIONS(1272),
    [anon_sym_const] = ACTIONS(1272),
    [anon_sym_constexpr] = ACTIONS(1272),
    [anon_sym_volatile] = ACTIONS(1272),
    [anon_sym_restrict] = ACTIONS(1272),
    [anon_sym___restrict__] = ACTIONS(1272),
    [anon_sym__Atomic] = ACTIONS(1272),
    [anon_sym__Noreturn] = ACTIONS(1272),
    [anon_sym_noreturn] = ACTIONS(1272),
    [sym_primitive_type] = ACTIONS(1272),
    [anon_sym_enum] = ACTIONS(1272),
    [anon_sym_struct] = ACTIONS(1272),
    [anon_sym_union] = ACTIONS(1272),
    [anon_sym_if] = ACTIONS(1272),
    [anon_sym_switch] = ACTIONS(1272),
    [anon_sym_case] = ACTIONS(1272),
    [anon_sym_default] = ACTIONS(1272),
    [anon_sym_while] = ACTIONS(1272),
    [anon_sym_do] = ACTIONS(1272),
    [anon_sym_for] = ACTIONS(1272),
    [anon_sym_return] = ACTIONS(1272),
    [anon_sym_break] = ACTIONS(1272),
    [anon_sym_continue] = ACTIONS(1272),
    [anon_sym_goto] = ACTIONS(1272),
    [anon_sym___try] = ACTIONS(1272),
    [anon_sym___leave] = ACTIONS(1272),
    [anon_sym_DASH_DASH] = ACTIONS(1274),
    [anon_sym_PLUS_PLUS] = ACTIONS(1274),
    [anon_sym_sizeof] = ACTIONS(1272),
    [anon_sym___alignof__] = ACTIONS(1272),
    [anon_sym___alignof] = ACTIONS(1272),
    [anon_sym__alignof] = ACTIONS(1272),
    [anon_sym_alignof] = ACTIONS(1272),
    [anon_sym__Alignof] = ACTIONS(1272),
    [anon_sym_offsetof] = ACTIONS(1272),
    [anon_sym__Generic] = ACTIONS(1272),
    [anon_sym_asm] = ACTIONS(1272),
    [anon_sym___asm__] = ACTIONS(1272),
    [sym_number_literal] = ACTIONS(1274),
    [anon_sym_L_SQUOTE] = ACTIONS(1274),
    [anon_sym_u_SQUOTE] = ACTIONS(1274),
    [anon_sym_U_SQUOTE] = ACTIONS(1274),
    [anon_sym_u8_SQUOTE] = ACTIONS(1274),
    [anon_sym_SQUOTE] = ACTIONS(1274),
    [anon_sym_L_DQUOTE] = ACTIONS(1274),
    [anon_sym_u_DQUOTE] = ACTIONS(1274),
    [anon_sym_U_DQUOTE] = ACTIONS(1274),
    [anon_sym_u8_DQUOTE] = ACTIONS(1274),
    [anon_sym_DQUOTE] = ACTIONS(1274),
    [sym_true] = ACTIONS(1272),
    [sym_false] = ACTIONS(1272),
    [anon_sym_NULL] = ACTIONS(1272),
    [anon_sym_nullptr] = ACTIONS(1272),
    [sym_comment] = ACTIONS(3),
  },
  [381] = {
    [sym_identifier] = ACTIONS(1276),
    [aux_sym_preproc_include_token1] = ACTIONS(1276),
    [aux_sym_preproc_def_token1] = ACTIONS(1276),
    [aux_sym_preproc_if_token1] = ACTIONS(1276),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1276),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1276),
    [sym_preproc_directive] = ACTIONS(1276),
    [anon_sym_LPAREN2] = ACTIONS(1278),
    [anon_sym_BANG] = ACTIONS(1278),
    [anon_sym_TILDE] = ACTIONS(1278),
    [anon_sym_DASH] = ACTIONS(1276),
    [anon_sym_PLUS] = ACTIONS(1276),
    [anon_sym_STAR] = ACTIONS(1278),
    [anon_sym_AMP] = ACTIONS(1278),
    [anon_sym_SEMI] = ACTIONS(1278),
    [anon_sym___extension__] = ACTIONS(1276),
    [anon_sym_typedef] = ACTIONS(1276),
    [anon_sym_extern] = ACTIONS(1276),
    [anon_sym___attribute__] = ACTIONS(1276),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1278),
    [anon_sym___declspec] = ACTIONS(1276),
    [anon_sym___cdecl] = ACTIONS(1276),
    [anon_sym___clrcall] = ACTIONS(1276),
    [anon_sym___stdcall] = ACTIONS(1276),
    [anon_sym___fastcall] = ACTIONS(1276),
    [anon_sym___thiscall] = ACTIONS(1276),
    [anon_sym___vectorcall] = ACTIONS(1276),
    [anon_sym_LBRACE] = ACTIONS(1278),
    [anon_sym_RBRACE] = ACTIONS(1278),
    [anon_sym_signed] = ACTIONS(1276),
    [anon_sym_unsigned] = ACTIONS(1276),
    [anon_sym_long] = ACTIONS(1276),
    [anon_sym_short] = ACTIONS(1276),
    [anon_sym_static] = ACTIONS(1276),
    [anon_sym_auto] = ACTIONS(1276),
    [anon_sym_register] = ACTIONS(1276),
    [anon_sym_inline] = ACTIONS(1276),
    [anon_sym___inline] = ACTIONS(1276),
    [anon_sym___inline__] = ACTIONS(1276),
    [anon_sym___forceinline] = ACTIONS(1276),
    [anon_sym_thread_local] = ACTIONS(1276),
    [anon_sym___thread] = ACTIONS(1276),
    [anon_sym_const] = ACTIONS(1276),
    [anon_sym_constexpr] = ACTIONS(1276),
    [anon_sym_volatile] = ACTIONS(1276),
    [anon_sym_restrict] = ACTIONS(1276),
    [anon_sym___restrict__] = ACTIONS(1276),
    [anon_sym__Atomic] = ACTIONS(1276),
    [anon_sym__Noreturn] = ACTIONS(1276),
    [anon_sym_noreturn] = ACTIONS(1276),
    [sym_primitive_type] = ACTIONS(1276),
    [anon_sym_enum] = ACTIONS(1276),
    [anon_sym_struct] = ACTIONS(1276),
    [anon_sym_union] = ACTIONS(1276),
    [anon_sym_if] = ACTIONS(1276),
    [anon_sym_switch] = ACTIONS(1276),
    [anon_sym_case] = ACTIONS(1276),
    [anon_sym_default] = ACTIONS(1276),
    [anon_sym_while] = ACTIONS(1276),
    [anon_sym_do] = ACTIONS(1276),
    [anon_sym_for] = ACTIONS(1276),
    [anon_sym_return] = ACTIONS(1276),
    [anon_sym_break] = ACTIONS(1276),
    [anon_sym_continue] = ACTIONS(1276),
    [anon_sym_goto] = ACTIONS(1276),
    [anon_sym___try] = ACTIONS(1276),
    [anon_sym___leave] = ACTIONS(1276),
    [anon_sym_DASH_DASH] = ACTIONS(1278),
    [anon_sym_PLUS_PLUS] = ACTIONS(1278),
    [anon_sym_sizeof] = ACTIONS(1276),
    [anon_sym___alignof__] = ACTIONS(1276),
    [anon_sym___alignof] = ACTIONS(1276),
    [anon_sym__alignof] = ACTIONS(1276),
    [anon_sym_alignof] = ACTIONS(1276),
    [anon_sym__Alignof] = ACTIONS(1276),
    [anon_sym_offsetof] = ACTIONS(1276),
    [anon_sym__Generic] = ACTIONS(1276),
    [anon_sym_asm] = ACTIONS(1276),
    [anon_sym___asm__] = ACTIONS(1276),
    [sym_number_literal] = ACTIONS(1278),
    [anon_sym_L_SQUOTE] = ACTIONS(1278),
    [anon_sym_u_SQUOTE] = ACTIONS(1278),
    [anon_sym_U_SQUOTE] = ACTIONS(1278),
    [anon_sym_u8_SQUOTE] = ACTIONS(1278),
    [anon_sym_SQUOTE] = ACTIONS(1278),
    [anon_sym_L_DQUOTE] = ACTIONS(1278),
    [anon_sym_u_DQUOTE] = ACTIONS(1278),
    [anon_sym_U_DQUOTE] = ACTIONS(1278),
    [anon_sym_u8_DQUOTE] = ACTIONS(1278),
    [anon_sym_DQUOTE] = ACTIONS(1278),
    [sym_true] = ACTIONS(1276),
    [sym_false] = ACTIONS(1276),
    [anon_sym_NULL] = ACTIONS(1276),
    [anon_sym_nullptr] = ACTIONS(1276),
    [sym_comment] = ACTIONS(3),
  },
  [382] = {
    [sym_identifier] = ACTIONS(1280),
    [aux_sym_preproc_include_token1] = ACTIONS(1280),
    [aux_sym_preproc_def_token1] = ACTIONS(1280),
    [aux_sym_preproc_if_token1] = ACTIONS(1280),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1280),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1280),
    [sym_preproc_directive] = ACTIONS(1280),
    [anon_sym_LPAREN2] = ACTIONS(1282),
    [anon_sym_BANG] = ACTIONS(1282),
    [anon_sym_TILDE] = ACTIONS(1282),
    [anon_sym_DASH] = ACTIONS(1280),
    [anon_sym_PLUS] = ACTIONS(1280),
    [anon_sym_STAR] = ACTIONS(1282),
    [anon_sym_AMP] = ACTIONS(1282),
    [anon_sym_SEMI] = ACTIONS(1282),
    [anon_sym___extension__] = ACTIONS(1280),
    [anon_sym_typedef] = ACTIONS(1280),
    [anon_sym_extern] = ACTIONS(1280),
    [anon_sym___attribute__] = ACTIONS(1280),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1282),
    [anon_sym___declspec] = ACTIONS(1280),
    [anon_sym___cdecl] = ACTIONS(1280),
    [anon_sym___clrcall] = ACTIONS(1280),
    [anon_sym___stdcall] = ACTIONS(1280),
    [anon_sym___fastcall] = ACTIONS(1280),
    [anon_sym___thiscall] = ACTIONS(1280),
    [anon_sym___vectorcall] = ACTIONS(1280),
    [anon_sym_LBRACE] = ACTIONS(1282),
    [anon_sym_RBRACE] = ACTIONS(1282),
    [anon_sym_signed] = ACTIONS(1280),
    [anon_sym_unsigned] = ACTIONS(1280),
    [anon_sym_long] = ACTIONS(1280),
    [anon_sym_short] = ACTIONS(1280),
    [anon_sym_static] = ACTIONS(1280),
    [anon_sym_auto] = ACTIONS(1280),
    [anon_sym_register] = ACTIONS(1280),
    [anon_sym_inline] = ACTIONS(1280),
    [anon_sym___inline] = ACTIONS(1280),
    [anon_sym___inline__] = ACTIONS(1280),
    [anon_sym___forceinline] = ACTIONS(1280),
    [anon_sym_thread_local] = ACTIONS(1280),
    [anon_sym___thread] = ACTIONS(1280),
    [anon_sym_const] = ACTIONS(1280),
    [anon_sym_constexpr] = ACTIONS(1280),
    [anon_sym_volatile] = ACTIONS(1280),
    [anon_sym_restrict] = ACTIONS(1280),
    [anon_sym___restrict__] = ACTIONS(1280),
    [anon_sym__Atomic] = ACTIONS(1280),
    [anon_sym__Noreturn] = ACTIONS(1280),
    [anon_sym_noreturn] = ACTIONS(1280),
    [sym_primitive_type] = ACTIONS(1280),
    [anon_sym_enum] = ACTIONS(1280),
    [anon_sym_struct] = ACTIONS(1280),
    [anon_sym_union] = ACTIONS(1280),
    [anon_sym_if] = ACTIONS(1280),
    [anon_sym_switch] = ACTIONS(1280),
    [anon_sym_case] = ACTIONS(1280),
    [anon_sym_default] = ACTIONS(1280),
    [anon_sym_while] = ACTIONS(1280),
    [anon_sym_do] = ACTIONS(1280),
    [anon_sym_for] = ACTIONS(1280),
    [anon_sym_return] = ACTIONS(1280),
    [anon_sym_break] = ACTIONS(1280),
    [anon_sym_continue] = ACTIONS(1280),
    [anon_sym_goto] = ACTIONS(1280),
    [anon_sym___try] = ACTIONS(1280),
    [anon_sym___leave] = ACTIONS(1280),
    [anon_sym_DASH_DASH] = ACTIONS(1282),
    [anon_sym_PLUS_PLUS] = ACTIONS(1282),
    [anon_sym_sizeof] = ACTIONS(1280),
    [anon_sym___alignof__] = ACTIONS(1280),
    [anon_sym___alignof] = ACTIONS(1280),
    [anon_sym__alignof] = ACTIONS(1280),
    [anon_sym_alignof] = ACTIONS(1280),
    [anon_sym__Alignof] = ACTIONS(1280),
    [anon_sym_offsetof] = ACTIONS(1280),
    [anon_sym__Generic] = ACTIONS(1280),
    [anon_sym_asm] = ACTIONS(1280),
    [anon_sym___asm__] = ACTIONS(1280),
    [sym_number_literal] = ACTIONS(1282),
    [anon_sym_L_SQUOTE] = ACTIONS(1282),
    [anon_sym_u_SQUOTE] = ACTIONS(1282),
    [anon_sym_U_SQUOTE] = ACTIONS(1282),
    [anon_sym_u8_SQUOTE] = ACTIONS(1282),
    [anon_sym_SQUOTE] = ACTIONS(1282),
    [anon_sym_L_DQUOTE] = ACTIONS(1282),
    [anon_sym_u_DQUOTE] = ACTIONS(1282),
    [anon_sym_U_DQUOTE] = ACTIONS(1282),
    [anon_sym_u8_DQUOTE] = ACTIONS(1282),
    [anon_sym_DQUOTE] = ACTIONS(1282),
    [sym_true] = ACTIONS(1280),
    [sym_false] = ACTIONS(1280),
    [anon_sym_NULL] = ACTIONS(1280),
    [anon_sym_nullptr] = ACTIONS(1280),
    [sym_comment] = ACTIONS(3),
  },
  [383] = {
    [sym_identifier] = ACTIONS(1312),
    [aux_sym_preproc_include_token1] = ACTIONS(1312),
    [aux_sym_preproc_def_token1] = ACTIONS(1312),
    [aux_sym_preproc_if_token1] = ACTIONS(1312),
    [aux_sym_preproc_if_token2] = ACTIONS(1312),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1312),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1312),
    [sym_preproc_directive] = ACTIONS(1312),
    [anon_sym_LPAREN2] = ACTIONS(1314),
    [anon_sym_BANG] = ACTIONS(1314),
    [anon_sym_TILDE] = ACTIONS(1314),
    [anon_sym_DASH] = ACTIONS(1312),
    [anon_sym_PLUS] = ACTIONS(1312),
    [anon_sym_STAR] = ACTIONS(1314),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym_SEMI] = ACTIONS(1314),
    [anon_sym___extension__] = ACTIONS(1312),
    [anon_sym_typedef] = ACTIONS(1312),
    [anon_sym_extern] = ACTIONS(1312),
    [anon_sym___attribute__] = ACTIONS(1312),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1314),
    [anon_sym___declspec] = ACTIONS(1312),
    [anon_sym___cdecl] = ACTIONS(1312),
    [anon_sym___clrcall] = ACTIONS(1312),
    [anon_sym___stdcall] = ACTIONS(1312),
    [anon_sym___fastcall] = ACTIONS(1312),
    [anon_sym___thiscall] = ACTIONS(1312),
    [anon_sym___vectorcall] = ACTIONS(1312),
    [anon_sym_LBRACE] = ACTIONS(1314),
    [anon_sym_signed] = ACTIONS(1312),
    [anon_sym_unsigned] = ACTIONS(1312),
    [anon_sym_long] = ACTIONS(1312),
    [anon_sym_short] = ACTIONS(1312),
    [anon_sym_static] = ACTIONS(1312),
    [anon_sym_auto] = ACTIONS(1312),
    [anon_sym_register] = ACTIONS(1312),
    [anon_sym_inline] = ACTIONS(1312),
    [anon_sym___inline] = ACTIONS(1312),
    [anon_sym___inline__] = ACTIONS(1312),
    [anon_sym___forceinline] = ACTIONS(1312),
    [anon_sym_thread_local] = ACTIONS(1312),
    [anon_sym___thread] = ACTIONS(1312),
    [anon_sym_const] = ACTIONS(1312),
    [anon_sym_constexpr] = ACTIONS(1312),
    [anon_sym_volatile] = ACTIONS(1312),
    [anon_sym_restrict] = ACTIONS(1312),
    [anon_sym___restrict__] = ACTIONS(1312),
    [anon_sym__Atomic] = ACTIONS(1312),
    [anon_sym__Noreturn] = ACTIONS(1312),
    [anon_sym_noreturn] = ACTIONS(1312),
    [sym_primitive_type] = ACTIONS(1312),
    [anon_sym_enum] = ACTIONS(1312),
    [anon_sym_struct] = ACTIONS(1312),
    [anon_sym_union] = ACTIONS(1312),
    [anon_sym_if] = ACTIONS(1312),
    [anon_sym_switch] = ACTIONS(1312),
    [anon_sym_case] = ACTIONS(1312),
    [anon_sym_default] = ACTIONS(1312),
    [anon_sym_while] = ACTIONS(1312),
    [anon_sym_do] = ACTIONS(1312),
    [anon_sym_for] = ACTIONS(1312),
    [anon_sym_return] = ACTIONS(1312),
    [anon_sym_break] = ACTIONS(1312),
    [anon_sym_continue] = ACTIONS(1312),
    [anon_sym_goto] = ACTIONS(1312),
    [anon_sym___try] = ACTIONS(1312),
    [anon_sym___leave] = ACTIONS(1312),
    [anon_sym_DASH_DASH] = ACTIONS(1314),
    [anon_sym_PLUS_PLUS] = ACTIONS(1314),
    [anon_sym_sizeof] = ACTIONS(1312),
    [anon_sym___alignof__] = ACTIONS(1312),
    [anon_sym___alignof] = ACTIONS(1312),
    [anon_sym__alignof] = ACTIONS(1312),
    [anon_sym_alignof] = ACTIONS(1312),
    [anon_sym__Alignof] = ACTIONS(1312),
    [anon_sym_offsetof] = ACTIONS(1312),
    [anon_sym__Generic] = ACTIONS(1312),
    [anon_sym_asm] = ACTIONS(1312),
    [anon_sym___asm__] = ACTIONS(1312),
    [sym_number_literal] = ACTIONS(1314),
    [anon_sym_L_SQUOTE] = ACTIONS(1314),
    [anon_sym_u_SQUOTE] = ACTIONS(1314),
    [anon_sym_U_SQUOTE] = ACTIONS(1314),
    [anon_sym_u8_SQUOTE] = ACTIONS(1314),
    [anon_sym_SQUOTE] = ACTIONS(1314),
    [anon_sym_L_DQUOTE] = ACTIONS(1314),
    [anon_sym_u_DQUOTE] = ACTIONS(1314),
    [anon_sym_U_DQUOTE] = ACTIONS(1314),
    [anon_sym_u8_DQUOTE] = ACTIONS(1314),
    [anon_sym_DQUOTE] = ACTIONS(1314),
    [sym_true] = ACTIONS(1312),
    [sym_false] = ACTIONS(1312),
    [anon_sym_NULL] = ACTIONS(1312),
    [anon_sym_nullptr] = ACTIONS(1312),
    [sym_comment] = ACTIONS(3),
  },
  [384] = {
    [sym_attribute_declaration] = STATE(335),
    [sym_compound_statement] = STATE(96),
    [sym_attributed_statement] = STATE(96),
    [sym_labeled_statement] = STATE(96),
    [sym_expression_statement] = STATE(96),
    [sym_if_statement] = STATE(96),
    [sym_switch_statement] = STATE(96),
    [sym_case_statement] = STATE(96),
    [sym_while_statement] = STATE(96),
    [sym_do_statement] = STATE(96),
    [sym_for_statement] = STATE(96),
    [sym_return_statement] = STATE(96),
    [sym_break_statement] = STATE(96),
    [sym_continue_statement] = STATE(96),
    [sym_goto_statement] = STATE(96),
    [sym_seh_try_statement] = STATE(96),
    [sym_seh_leave_statement] = STATE(96),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [sym_identifier] = ACTIONS(1530),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [385] = {
    [sym_identifier] = ACTIONS(1252),
    [aux_sym_preproc_include_token1] = ACTIONS(1252),
    [aux_sym_preproc_def_token1] = ACTIONS(1252),
    [aux_sym_preproc_if_token1] = ACTIONS(1252),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1252),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1252),
    [sym_preproc_directive] = ACTIONS(1252),
    [anon_sym_LPAREN2] = ACTIONS(1254),
    [anon_sym_BANG] = ACTIONS(1254),
    [anon_sym_TILDE] = ACTIONS(1254),
    [anon_sym_DASH] = ACTIONS(1252),
    [anon_sym_PLUS] = ACTIONS(1252),
    [anon_sym_STAR] = ACTIONS(1254),
    [anon_sym_AMP] = ACTIONS(1254),
    [anon_sym_SEMI] = ACTIONS(1254),
    [anon_sym___extension__] = ACTIONS(1252),
    [anon_sym_typedef] = ACTIONS(1252),
    [anon_sym_extern] = ACTIONS(1252),
    [anon_sym___attribute__] = ACTIONS(1252),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1254),
    [anon_sym___declspec] = ACTIONS(1252),
    [anon_sym___cdecl] = ACTIONS(1252),
    [anon_sym___clrcall] = ACTIONS(1252),
    [anon_sym___stdcall] = ACTIONS(1252),
    [anon_sym___fastcall] = ACTIONS(1252),
    [anon_sym___thiscall] = ACTIONS(1252),
    [anon_sym___vectorcall] = ACTIONS(1252),
    [anon_sym_LBRACE] = ACTIONS(1254),
    [anon_sym_RBRACE] = ACTIONS(1254),
    [anon_sym_signed] = ACTIONS(1252),
    [anon_sym_unsigned] = ACTIONS(1252),
    [anon_sym_long] = ACTIONS(1252),
    [anon_sym_short] = ACTIONS(1252),
    [anon_sym_static] = ACTIONS(1252),
    [anon_sym_auto] = ACTIONS(1252),
    [anon_sym_register] = ACTIONS(1252),
    [anon_sym_inline] = ACTIONS(1252),
    [anon_sym___inline] = ACTIONS(1252),
    [anon_sym___inline__] = ACTIONS(1252),
    [anon_sym___forceinline] = ACTIONS(1252),
    [anon_sym_thread_local] = ACTIONS(1252),
    [anon_sym___thread] = ACTIONS(1252),
    [anon_sym_const] = ACTIONS(1252),
    [anon_sym_constexpr] = ACTIONS(1252),
    [anon_sym_volatile] = ACTIONS(1252),
    [anon_sym_restrict] = ACTIONS(1252),
    [anon_sym___restrict__] = ACTIONS(1252),
    [anon_sym__Atomic] = ACTIONS(1252),
    [anon_sym__Noreturn] = ACTIONS(1252),
    [anon_sym_noreturn] = ACTIONS(1252),
    [sym_primitive_type] = ACTIONS(1252),
    [anon_sym_enum] = ACTIONS(1252),
    [anon_sym_struct] = ACTIONS(1252),
    [anon_sym_union] = ACTIONS(1252),
    [anon_sym_if] = ACTIONS(1252),
    [anon_sym_switch] = ACTIONS(1252),
    [anon_sym_case] = ACTIONS(1252),
    [anon_sym_default] = ACTIONS(1252),
    [anon_sym_while] = ACTIONS(1252),
    [anon_sym_do] = ACTIONS(1252),
    [anon_sym_for] = ACTIONS(1252),
    [anon_sym_return] = ACTIONS(1252),
    [anon_sym_break] = ACTIONS(1252),
    [anon_sym_continue] = ACTIONS(1252),
    [anon_sym_goto] = ACTIONS(1252),
    [anon_sym___try] = ACTIONS(1252),
    [anon_sym___leave] = ACTIONS(1252),
    [anon_sym_DASH_DASH] = ACTIONS(1254),
    [anon_sym_PLUS_PLUS] = ACTIONS(1254),
    [anon_sym_sizeof] = ACTIONS(1252),
    [anon_sym___alignof__] = ACTIONS(1252),
    [anon_sym___alignof] = ACTIONS(1252),
    [anon_sym__alignof] = ACTIONS(1252),
    [anon_sym_alignof] = ACTIONS(1252),
    [anon_sym__Alignof] = ACTIONS(1252),
    [anon_sym_offsetof] = ACTIONS(1252),
    [anon_sym__Generic] = ACTIONS(1252),
    [anon_sym_asm] = ACTIONS(1252),
    [anon_sym___asm__] = ACTIONS(1252),
    [sym_number_literal] = ACTIONS(1254),
    [anon_sym_L_SQUOTE] = ACTIONS(1254),
    [anon_sym_u_SQUOTE] = ACTIONS(1254),
    [anon_sym_U_SQUOTE] = ACTIONS(1254),
    [anon_sym_u8_SQUOTE] = ACTIONS(1254),
    [anon_sym_SQUOTE] = ACTIONS(1254),
    [anon_sym_L_DQUOTE] = ACTIONS(1254),
    [anon_sym_u_DQUOTE] = ACTIONS(1254),
    [anon_sym_U_DQUOTE] = ACTIONS(1254),
    [anon_sym_u8_DQUOTE] = ACTIONS(1254),
    [anon_sym_DQUOTE] = ACTIONS(1254),
    [sym_true] = ACTIONS(1252),
    [sym_false] = ACTIONS(1252),
    [anon_sym_NULL] = ACTIONS(1252),
    [anon_sym_nullptr] = ACTIONS(1252),
    [sym_comment] = ACTIONS(3),
  },
  [386] = {
    [sym_attribute_declaration] = STATE(314),
    [sym_compound_statement] = STATE(308),
    [sym_attributed_statement] = STATE(308),
    [sym_labeled_statement] = STATE(308),
    [sym_expression_statement] = STATE(308),
    [sym_if_statement] = STATE(308),
    [sym_switch_statement] = STATE(308),
    [sym_case_statement] = STATE(308),
    [sym_while_statement] = STATE(308),
    [sym_do_statement] = STATE(308),
    [sym_for_statement] = STATE(308),
    [sym_return_statement] = STATE(308),
    [sym_break_statement] = STATE(308),
    [sym_continue_statement] = STATE(308),
    [sym_goto_statement] = STATE(308),
    [sym_seh_try_statement] = STATE(308),
    [sym_seh_leave_statement] = STATE(308),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [sym_identifier] = ACTIONS(1378),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(61),
    [anon_sym_default] = ACTIONS(63),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [387] = {
    [sym_attribute_declaration] = STATE(335),
    [sym_compound_statement] = STATE(123),
    [sym_attributed_statement] = STATE(124),
    [sym_labeled_statement] = STATE(125),
    [sym_expression_statement] = STATE(127),
    [sym_if_statement] = STATE(126),
    [sym_switch_statement] = STATE(120),
    [sym_case_statement] = STATE(119),
    [sym_while_statement] = STATE(118),
    [sym_do_statement] = STATE(117),
    [sym_for_statement] = STATE(116),
    [sym_return_statement] = STATE(110),
    [sym_break_statement] = STATE(109),
    [sym_continue_statement] = STATE(108),
    [sym_goto_statement] = STATE(107),
    [sym_seh_try_statement] = STATE(77),
    [sym_seh_leave_statement] = STATE(103),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [sym_identifier] = ACTIONS(1530),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [388] = {
    [sym_identifier] = ACTIONS(1288),
    [aux_sym_preproc_include_token1] = ACTIONS(1288),
    [aux_sym_preproc_def_token1] = ACTIONS(1288),
    [aux_sym_preproc_if_token1] = ACTIONS(1288),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1288),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1288),
    [sym_preproc_directive] = ACTIONS(1288),
    [anon_sym_LPAREN2] = ACTIONS(1290),
    [anon_sym_BANG] = ACTIONS(1290),
    [anon_sym_TILDE] = ACTIONS(1290),
    [anon_sym_DASH] = ACTIONS(1288),
    [anon_sym_PLUS] = ACTIONS(1288),
    [anon_sym_STAR] = ACTIONS(1290),
    [anon_sym_AMP] = ACTIONS(1290),
    [anon_sym_SEMI] = ACTIONS(1290),
    [anon_sym___extension__] = ACTIONS(1288),
    [anon_sym_typedef] = ACTIONS(1288),
    [anon_sym_extern] = ACTIONS(1288),
    [anon_sym___attribute__] = ACTIONS(1288),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1290),
    [anon_sym___declspec] = ACTIONS(1288),
    [anon_sym___cdecl] = ACTIONS(1288),
    [anon_sym___clrcall] = ACTIONS(1288),
    [anon_sym___stdcall] = ACTIONS(1288),
    [anon_sym___fastcall] = ACTIONS(1288),
    [anon_sym___thiscall] = ACTIONS(1288),
    [anon_sym___vectorcall] = ACTIONS(1288),
    [anon_sym_LBRACE] = ACTIONS(1290),
    [anon_sym_RBRACE] = ACTIONS(1290),
    [anon_sym_signed] = ACTIONS(1288),
    [anon_sym_unsigned] = ACTIONS(1288),
    [anon_sym_long] = ACTIONS(1288),
    [anon_sym_short] = ACTIONS(1288),
    [anon_sym_static] = ACTIONS(1288),
    [anon_sym_auto] = ACTIONS(1288),
    [anon_sym_register] = ACTIONS(1288),
    [anon_sym_inline] = ACTIONS(1288),
    [anon_sym___inline] = ACTIONS(1288),
    [anon_sym___inline__] = ACTIONS(1288),
    [anon_sym___forceinline] = ACTIONS(1288),
    [anon_sym_thread_local] = ACTIONS(1288),
    [anon_sym___thread] = ACTIONS(1288),
    [anon_sym_const] = ACTIONS(1288),
    [anon_sym_constexpr] = ACTIONS(1288),
    [anon_sym_volatile] = ACTIONS(1288),
    [anon_sym_restrict] = ACTIONS(1288),
    [anon_sym___restrict__] = ACTIONS(1288),
    [anon_sym__Atomic] = ACTIONS(1288),
    [anon_sym__Noreturn] = ACTIONS(1288),
    [anon_sym_noreturn] = ACTIONS(1288),
    [sym_primitive_type] = ACTIONS(1288),
    [anon_sym_enum] = ACTIONS(1288),
    [anon_sym_struct] = ACTIONS(1288),
    [anon_sym_union] = ACTIONS(1288),
    [anon_sym_if] = ACTIONS(1288),
    [anon_sym_switch] = ACTIONS(1288),
    [anon_sym_case] = ACTIONS(1288),
    [anon_sym_default] = ACTIONS(1288),
    [anon_sym_while] = ACTIONS(1288),
    [anon_sym_do] = ACTIONS(1288),
    [anon_sym_for] = ACTIONS(1288),
    [anon_sym_return] = ACTIONS(1288),
    [anon_sym_break] = ACTIONS(1288),
    [anon_sym_continue] = ACTIONS(1288),
    [anon_sym_goto] = ACTIONS(1288),
    [anon_sym___try] = ACTIONS(1288),
    [anon_sym___leave] = ACTIONS(1288),
    [anon_sym_DASH_DASH] = ACTIONS(1290),
    [anon_sym_PLUS_PLUS] = ACTIONS(1290),
    [anon_sym_sizeof] = ACTIONS(1288),
    [anon_sym___alignof__] = ACTIONS(1288),
    [anon_sym___alignof] = ACTIONS(1288),
    [anon_sym__alignof] = ACTIONS(1288),
    [anon_sym_alignof] = ACTIONS(1288),
    [anon_sym__Alignof] = ACTIONS(1288),
    [anon_sym_offsetof] = ACTIONS(1288),
    [anon_sym__Generic] = ACTIONS(1288),
    [anon_sym_asm] = ACTIONS(1288),
    [anon_sym___asm__] = ACTIONS(1288),
    [sym_number_literal] = ACTIONS(1290),
    [anon_sym_L_SQUOTE] = ACTIONS(1290),
    [anon_sym_u_SQUOTE] = ACTIONS(1290),
    [anon_sym_U_SQUOTE] = ACTIONS(1290),
    [anon_sym_u8_SQUOTE] = ACTIONS(1290),
    [anon_sym_SQUOTE] = ACTIONS(1290),
    [anon_sym_L_DQUOTE] = ACTIONS(1290),
    [anon_sym_u_DQUOTE] = ACTIONS(1290),
    [anon_sym_U_DQUOTE] = ACTIONS(1290),
    [anon_sym_u8_DQUOTE] = ACTIONS(1290),
    [anon_sym_DQUOTE] = ACTIONS(1290),
    [sym_true] = ACTIONS(1288),
    [sym_false] = ACTIONS(1288),
    [anon_sym_NULL] = ACTIONS(1288),
    [anon_sym_nullptr] = ACTIONS(1288),
    [sym_comment] = ACTIONS(3),
  },
  [389] = {
    [sym_attribute_declaration] = STATE(335),
    [sym_compound_statement] = STATE(99),
    [sym_attributed_statement] = STATE(99),
    [sym_labeled_statement] = STATE(99),
    [sym_expression_statement] = STATE(99),
    [sym_if_statement] = STATE(99),
    [sym_switch_statement] = STATE(99),
    [sym_case_statement] = STATE(99),
    [sym_while_statement] = STATE(99),
    [sym_do_statement] = STATE(99),
    [sym_for_statement] = STATE(99),
    [sym_return_statement] = STATE(99),
    [sym_break_statement] = STATE(99),
    [sym_continue_statement] = STATE(99),
    [sym_goto_statement] = STATE(99),
    [sym_seh_try_statement] = STATE(99),
    [sym_seh_leave_statement] = STATE(99),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [sym_identifier] = ACTIONS(1530),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [390] = {
    [sym_identifier] = ACTIONS(1344),
    [aux_sym_preproc_include_token1] = ACTIONS(1344),
    [aux_sym_preproc_def_token1] = ACTIONS(1344),
    [aux_sym_preproc_if_token1] = ACTIONS(1344),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1344),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1344),
    [sym_preproc_directive] = ACTIONS(1344),
    [anon_sym_LPAREN2] = ACTIONS(1346),
    [anon_sym_BANG] = ACTIONS(1346),
    [anon_sym_TILDE] = ACTIONS(1346),
    [anon_sym_DASH] = ACTIONS(1344),
    [anon_sym_PLUS] = ACTIONS(1344),
    [anon_sym_STAR] = ACTIONS(1346),
    [anon_sym_AMP] = ACTIONS(1346),
    [anon_sym_SEMI] = ACTIONS(1346),
    [anon_sym___extension__] = ACTIONS(1344),
    [anon_sym_typedef] = ACTIONS(1344),
    [anon_sym_extern] = ACTIONS(1344),
    [anon_sym___attribute__] = ACTIONS(1344),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1346),
    [anon_sym___declspec] = ACTIONS(1344),
    [anon_sym___cdecl] = ACTIONS(1344),
    [anon_sym___clrcall] = ACTIONS(1344),
    [anon_sym___stdcall] = ACTIONS(1344),
    [anon_sym___fastcall] = ACTIONS(1344),
    [anon_sym___thiscall] = ACTIONS(1344),
    [anon_sym___vectorcall] = ACTIONS(1344),
    [anon_sym_LBRACE] = ACTIONS(1346),
    [anon_sym_RBRACE] = ACTIONS(1346),
    [anon_sym_signed] = ACTIONS(1344),
    [anon_sym_unsigned] = ACTIONS(1344),
    [anon_sym_long] = ACTIONS(1344),
    [anon_sym_short] = ACTIONS(1344),
    [anon_sym_static] = ACTIONS(1344),
    [anon_sym_auto] = ACTIONS(1344),
    [anon_sym_register] = ACTIONS(1344),
    [anon_sym_inline] = ACTIONS(1344),
    [anon_sym___inline] = ACTIONS(1344),
    [anon_sym___inline__] = ACTIONS(1344),
    [anon_sym___forceinline] = ACTIONS(1344),
    [anon_sym_thread_local] = ACTIONS(1344),
    [anon_sym___thread] = ACTIONS(1344),
    [anon_sym_const] = ACTIONS(1344),
    [anon_sym_constexpr] = ACTIONS(1344),
    [anon_sym_volatile] = ACTIONS(1344),
    [anon_sym_restrict] = ACTIONS(1344),
    [anon_sym___restrict__] = ACTIONS(1344),
    [anon_sym__Atomic] = ACTIONS(1344),
    [anon_sym__Noreturn] = ACTIONS(1344),
    [anon_sym_noreturn] = ACTIONS(1344),
    [sym_primitive_type] = ACTIONS(1344),
    [anon_sym_enum] = ACTIONS(1344),
    [anon_sym_struct] = ACTIONS(1344),
    [anon_sym_union] = ACTIONS(1344),
    [anon_sym_if] = ACTIONS(1344),
    [anon_sym_switch] = ACTIONS(1344),
    [anon_sym_case] = ACTIONS(1344),
    [anon_sym_default] = ACTIONS(1344),
    [anon_sym_while] = ACTIONS(1344),
    [anon_sym_do] = ACTIONS(1344),
    [anon_sym_for] = ACTIONS(1344),
    [anon_sym_return] = ACTIONS(1344),
    [anon_sym_break] = ACTIONS(1344),
    [anon_sym_continue] = ACTIONS(1344),
    [anon_sym_goto] = ACTIONS(1344),
    [anon_sym___try] = ACTIONS(1344),
    [anon_sym___leave] = ACTIONS(1344),
    [anon_sym_DASH_DASH] = ACTIONS(1346),
    [anon_sym_PLUS_PLUS] = ACTIONS(1346),
    [anon_sym_sizeof] = ACTIONS(1344),
    [anon_sym___alignof__] = ACTIONS(1344),
    [anon_sym___alignof] = ACTIONS(1344),
    [anon_sym__alignof] = ACTIONS(1344),
    [anon_sym_alignof] = ACTIONS(1344),
    [anon_sym__Alignof] = ACTIONS(1344),
    [anon_sym_offsetof] = ACTIONS(1344),
    [anon_sym__Generic] = ACTIONS(1344),
    [anon_sym_asm] = ACTIONS(1344),
    [anon_sym___asm__] = ACTIONS(1344),
    [sym_number_literal] = ACTIONS(1346),
    [anon_sym_L_SQUOTE] = ACTIONS(1346),
    [anon_sym_u_SQUOTE] = ACTIONS(1346),
    [anon_sym_U_SQUOTE] = ACTIONS(1346),
    [anon_sym_u8_SQUOTE] = ACTIONS(1346),
    [anon_sym_SQUOTE] = ACTIONS(1346),
    [anon_sym_L_DQUOTE] = ACTIONS(1346),
    [anon_sym_u_DQUOTE] = ACTIONS(1346),
    [anon_sym_U_DQUOTE] = ACTIONS(1346),
    [anon_sym_u8_DQUOTE] = ACTIONS(1346),
    [anon_sym_DQUOTE] = ACTIONS(1346),
    [sym_true] = ACTIONS(1344),
    [sym_false] = ACTIONS(1344),
    [anon_sym_NULL] = ACTIONS(1344),
    [anon_sym_nullptr] = ACTIONS(1344),
    [sym_comment] = ACTIONS(3),
  },
  [391] = {
    [sym_attribute_declaration] = STATE(314),
    [sym_compound_statement] = STATE(158),
    [sym_attributed_statement] = STATE(158),
    [sym_labeled_statement] = STATE(158),
    [sym_expression_statement] = STATE(158),
    [sym_if_statement] = STATE(158),
    [sym_switch_statement] = STATE(158),
    [sym_case_statement] = STATE(158),
    [sym_while_statement] = STATE(158),
    [sym_do_statement] = STATE(158),
    [sym_for_statement] = STATE(158),
    [sym_return_statement] = STATE(158),
    [sym_break_statement] = STATE(158),
    [sym_continue_statement] = STATE(158),
    [sym_goto_statement] = STATE(158),
    [sym_seh_try_statement] = STATE(158),
    [sym_seh_leave_statement] = STATE(158),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [sym_identifier] = ACTIONS(1378),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(61),
    [anon_sym_default] = ACTIONS(63),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [392] = {
    [sym_identifier] = ACTIONS(1236),
    [aux_sym_preproc_include_token1] = ACTIONS(1236),
    [aux_sym_preproc_def_token1] = ACTIONS(1236),
    [aux_sym_preproc_if_token1] = ACTIONS(1236),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1236),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1236),
    [sym_preproc_directive] = ACTIONS(1236),
    [anon_sym_LPAREN2] = ACTIONS(1238),
    [anon_sym_BANG] = ACTIONS(1238),
    [anon_sym_TILDE] = ACTIONS(1238),
    [anon_sym_DASH] = ACTIONS(1236),
    [anon_sym_PLUS] = ACTIONS(1236),
    [anon_sym_STAR] = ACTIONS(1238),
    [anon_sym_AMP] = ACTIONS(1238),
    [anon_sym_SEMI] = ACTIONS(1238),
    [anon_sym___extension__] = ACTIONS(1236),
    [anon_sym_typedef] = ACTIONS(1236),
    [anon_sym_extern] = ACTIONS(1236),
    [anon_sym___attribute__] = ACTIONS(1236),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1238),
    [anon_sym___declspec] = ACTIONS(1236),
    [anon_sym___cdecl] = ACTIONS(1236),
    [anon_sym___clrcall] = ACTIONS(1236),
    [anon_sym___stdcall] = ACTIONS(1236),
    [anon_sym___fastcall] = ACTIONS(1236),
    [anon_sym___thiscall] = ACTIONS(1236),
    [anon_sym___vectorcall] = ACTIONS(1236),
    [anon_sym_LBRACE] = ACTIONS(1238),
    [anon_sym_RBRACE] = ACTIONS(1238),
    [anon_sym_signed] = ACTIONS(1236),
    [anon_sym_unsigned] = ACTIONS(1236),
    [anon_sym_long] = ACTIONS(1236),
    [anon_sym_short] = ACTIONS(1236),
    [anon_sym_static] = ACTIONS(1236),
    [anon_sym_auto] = ACTIONS(1236),
    [anon_sym_register] = ACTIONS(1236),
    [anon_sym_inline] = ACTIONS(1236),
    [anon_sym___inline] = ACTIONS(1236),
    [anon_sym___inline__] = ACTIONS(1236),
    [anon_sym___forceinline] = ACTIONS(1236),
    [anon_sym_thread_local] = ACTIONS(1236),
    [anon_sym___thread] = ACTIONS(1236),
    [anon_sym_const] = ACTIONS(1236),
    [anon_sym_constexpr] = ACTIONS(1236),
    [anon_sym_volatile] = ACTIONS(1236),
    [anon_sym_restrict] = ACTIONS(1236),
    [anon_sym___restrict__] = ACTIONS(1236),
    [anon_sym__Atomic] = ACTIONS(1236),
    [anon_sym__Noreturn] = ACTIONS(1236),
    [anon_sym_noreturn] = ACTIONS(1236),
    [sym_primitive_type] = ACTIONS(1236),
    [anon_sym_enum] = ACTIONS(1236),
    [anon_sym_struct] = ACTIONS(1236),
    [anon_sym_union] = ACTIONS(1236),
    [anon_sym_if] = ACTIONS(1236),
    [anon_sym_switch] = ACTIONS(1236),
    [anon_sym_case] = ACTIONS(1236),
    [anon_sym_default] = ACTIONS(1236),
    [anon_sym_while] = ACTIONS(1236),
    [anon_sym_do] = ACTIONS(1236),
    [anon_sym_for] = ACTIONS(1236),
    [anon_sym_return] = ACTIONS(1236),
    [anon_sym_break] = ACTIONS(1236),
    [anon_sym_continue] = ACTIONS(1236),
    [anon_sym_goto] = ACTIONS(1236),
    [anon_sym___try] = ACTIONS(1236),
    [anon_sym___leave] = ACTIONS(1236),
    [anon_sym_DASH_DASH] = ACTIONS(1238),
    [anon_sym_PLUS_PLUS] = ACTIONS(1238),
    [anon_sym_sizeof] = ACTIONS(1236),
    [anon_sym___alignof__] = ACTIONS(1236),
    [anon_sym___alignof] = ACTIONS(1236),
    [anon_sym__alignof] = ACTIONS(1236),
    [anon_sym_alignof] = ACTIONS(1236),
    [anon_sym__Alignof] = ACTIONS(1236),
    [anon_sym_offsetof] = ACTIONS(1236),
    [anon_sym__Generic] = ACTIONS(1236),
    [anon_sym_asm] = ACTIONS(1236),
    [anon_sym___asm__] = ACTIONS(1236),
    [sym_number_literal] = ACTIONS(1238),
    [anon_sym_L_SQUOTE] = ACTIONS(1238),
    [anon_sym_u_SQUOTE] = ACTIONS(1238),
    [anon_sym_U_SQUOTE] = ACTIONS(1238),
    [anon_sym_u8_SQUOTE] = ACTIONS(1238),
    [anon_sym_SQUOTE] = ACTIONS(1238),
    [anon_sym_L_DQUOTE] = ACTIONS(1238),
    [anon_sym_u_DQUOTE] = ACTIONS(1238),
    [anon_sym_U_DQUOTE] = ACTIONS(1238),
    [anon_sym_u8_DQUOTE] = ACTIONS(1238),
    [anon_sym_DQUOTE] = ACTIONS(1238),
    [sym_true] = ACTIONS(1236),
    [sym_false] = ACTIONS(1236),
    [anon_sym_NULL] = ACTIONS(1236),
    [anon_sym_nullptr] = ACTIONS(1236),
    [sym_comment] = ACTIONS(3),
  },
  [393] = {
    [sym_identifier] = ACTIONS(1340),
    [aux_sym_preproc_include_token1] = ACTIONS(1340),
    [aux_sym_preproc_def_token1] = ACTIONS(1340),
    [aux_sym_preproc_if_token1] = ACTIONS(1340),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1340),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1340),
    [sym_preproc_directive] = ACTIONS(1340),
    [anon_sym_LPAREN2] = ACTIONS(1342),
    [anon_sym_BANG] = ACTIONS(1342),
    [anon_sym_TILDE] = ACTIONS(1342),
    [anon_sym_DASH] = ACTIONS(1340),
    [anon_sym_PLUS] = ACTIONS(1340),
    [anon_sym_STAR] = ACTIONS(1342),
    [anon_sym_AMP] = ACTIONS(1342),
    [anon_sym_SEMI] = ACTIONS(1342),
    [anon_sym___extension__] = ACTIONS(1340),
    [anon_sym_typedef] = ACTIONS(1340),
    [anon_sym_extern] = ACTIONS(1340),
    [anon_sym___attribute__] = ACTIONS(1340),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1342),
    [anon_sym___declspec] = ACTIONS(1340),
    [anon_sym___cdecl] = ACTIONS(1340),
    [anon_sym___clrcall] = ACTIONS(1340),
    [anon_sym___stdcall] = ACTIONS(1340),
    [anon_sym___fastcall] = ACTIONS(1340),
    [anon_sym___thiscall] = ACTIONS(1340),
    [anon_sym___vectorcall] = ACTIONS(1340),
    [anon_sym_LBRACE] = ACTIONS(1342),
    [anon_sym_RBRACE] = ACTIONS(1342),
    [anon_sym_signed] = ACTIONS(1340),
    [anon_sym_unsigned] = ACTIONS(1340),
    [anon_sym_long] = ACTIONS(1340),
    [anon_sym_short] = ACTIONS(1340),
    [anon_sym_static] = ACTIONS(1340),
    [anon_sym_auto] = ACTIONS(1340),
    [anon_sym_register] = ACTIONS(1340),
    [anon_sym_inline] = ACTIONS(1340),
    [anon_sym___inline] = ACTIONS(1340),
    [anon_sym___inline__] = ACTIONS(1340),
    [anon_sym___forceinline] = ACTIONS(1340),
    [anon_sym_thread_local] = ACTIONS(1340),
    [anon_sym___thread] = ACTIONS(1340),
    [anon_sym_const] = ACTIONS(1340),
    [anon_sym_constexpr] = ACTIONS(1340),
    [anon_sym_volatile] = ACTIONS(1340),
    [anon_sym_restrict] = ACTIONS(1340),
    [anon_sym___restrict__] = ACTIONS(1340),
    [anon_sym__Atomic] = ACTIONS(1340),
    [anon_sym__Noreturn] = ACTIONS(1340),
    [anon_sym_noreturn] = ACTIONS(1340),
    [sym_primitive_type] = ACTIONS(1340),
    [anon_sym_enum] = ACTIONS(1340),
    [anon_sym_struct] = ACTIONS(1340),
    [anon_sym_union] = ACTIONS(1340),
    [anon_sym_if] = ACTIONS(1340),
    [anon_sym_switch] = ACTIONS(1340),
    [anon_sym_case] = ACTIONS(1340),
    [anon_sym_default] = ACTIONS(1340),
    [anon_sym_while] = ACTIONS(1340),
    [anon_sym_do] = ACTIONS(1340),
    [anon_sym_for] = ACTIONS(1340),
    [anon_sym_return] = ACTIONS(1340),
    [anon_sym_break] = ACTIONS(1340),
    [anon_sym_continue] = ACTIONS(1340),
    [anon_sym_goto] = ACTIONS(1340),
    [anon_sym___try] = ACTIONS(1340),
    [anon_sym___leave] = ACTIONS(1340),
    [anon_sym_DASH_DASH] = ACTIONS(1342),
    [anon_sym_PLUS_PLUS] = ACTIONS(1342),
    [anon_sym_sizeof] = ACTIONS(1340),
    [anon_sym___alignof__] = ACTIONS(1340),
    [anon_sym___alignof] = ACTIONS(1340),
    [anon_sym__alignof] = ACTIONS(1340),
    [anon_sym_alignof] = ACTIONS(1340),
    [anon_sym__Alignof] = ACTIONS(1340),
    [anon_sym_offsetof] = ACTIONS(1340),
    [anon_sym__Generic] = ACTIONS(1340),
    [anon_sym_asm] = ACTIONS(1340),
    [anon_sym___asm__] = ACTIONS(1340),
    [sym_number_literal] = ACTIONS(1342),
    [anon_sym_L_SQUOTE] = ACTIONS(1342),
    [anon_sym_u_SQUOTE] = ACTIONS(1342),
    [anon_sym_U_SQUOTE] = ACTIONS(1342),
    [anon_sym_u8_SQUOTE] = ACTIONS(1342),
    [anon_sym_SQUOTE] = ACTIONS(1342),
    [anon_sym_L_DQUOTE] = ACTIONS(1342),
    [anon_sym_u_DQUOTE] = ACTIONS(1342),
    [anon_sym_U_DQUOTE] = ACTIONS(1342),
    [anon_sym_u8_DQUOTE] = ACTIONS(1342),
    [anon_sym_DQUOTE] = ACTIONS(1342),
    [sym_true] = ACTIONS(1340),
    [sym_false] = ACTIONS(1340),
    [anon_sym_NULL] = ACTIONS(1340),
    [anon_sym_nullptr] = ACTIONS(1340),
    [sym_comment] = ACTIONS(3),
  },
  [394] = {
    [sym_attribute_declaration] = STATE(400),
    [sym_compound_statement] = STATE(279),
    [sym_attributed_statement] = STATE(280),
    [sym_labeled_statement] = STATE(282),
    [sym_expression_statement] = STATE(283),
    [sym_if_statement] = STATE(286),
    [sym_switch_statement] = STATE(287),
    [sym_case_statement] = STATE(288),
    [sym_while_statement] = STATE(289),
    [sym_do_statement] = STATE(290),
    [sym_for_statement] = STATE(291),
    [sym_return_statement] = STATE(294),
    [sym_break_statement] = STATE(295),
    [sym_continue_statement] = STATE(162),
    [sym_goto_statement] = STATE(297),
    [sym_seh_try_statement] = STATE(298),
    [sym_seh_leave_statement] = STATE(299),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [sym_identifier] = ACTIONS(1559),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [395] = {
    [sym_identifier] = ACTIONS(1296),
    [aux_sym_preproc_include_token1] = ACTIONS(1296),
    [aux_sym_preproc_def_token1] = ACTIONS(1296),
    [aux_sym_preproc_if_token1] = ACTIONS(1296),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1296),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1296),
    [sym_preproc_directive] = ACTIONS(1296),
    [anon_sym_LPAREN2] = ACTIONS(1298),
    [anon_sym_BANG] = ACTIONS(1298),
    [anon_sym_TILDE] = ACTIONS(1298),
    [anon_sym_DASH] = ACTIONS(1296),
    [anon_sym_PLUS] = ACTIONS(1296),
    [anon_sym_STAR] = ACTIONS(1298),
    [anon_sym_AMP] = ACTIONS(1298),
    [anon_sym_SEMI] = ACTIONS(1298),
    [anon_sym___extension__] = ACTIONS(1296),
    [anon_sym_typedef] = ACTIONS(1296),
    [anon_sym_extern] = ACTIONS(1296),
    [anon_sym___attribute__] = ACTIONS(1296),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1298),
    [anon_sym___declspec] = ACTIONS(1296),
    [anon_sym___cdecl] = ACTIONS(1296),
    [anon_sym___clrcall] = ACTIONS(1296),
    [anon_sym___stdcall] = ACTIONS(1296),
    [anon_sym___fastcall] = ACTIONS(1296),
    [anon_sym___thiscall] = ACTIONS(1296),
    [anon_sym___vectorcall] = ACTIONS(1296),
    [anon_sym_LBRACE] = ACTIONS(1298),
    [anon_sym_RBRACE] = ACTIONS(1298),
    [anon_sym_signed] = ACTIONS(1296),
    [anon_sym_unsigned] = ACTIONS(1296),
    [anon_sym_long] = ACTIONS(1296),
    [anon_sym_short] = ACTIONS(1296),
    [anon_sym_static] = ACTIONS(1296),
    [anon_sym_auto] = ACTIONS(1296),
    [anon_sym_register] = ACTIONS(1296),
    [anon_sym_inline] = ACTIONS(1296),
    [anon_sym___inline] = ACTIONS(1296),
    [anon_sym___inline__] = ACTIONS(1296),
    [anon_sym___forceinline] = ACTIONS(1296),
    [anon_sym_thread_local] = ACTIONS(1296),
    [anon_sym___thread] = ACTIONS(1296),
    [anon_sym_const] = ACTIONS(1296),
    [anon_sym_constexpr] = ACTIONS(1296),
    [anon_sym_volatile] = ACTIONS(1296),
    [anon_sym_restrict] = ACTIONS(1296),
    [anon_sym___restrict__] = ACTIONS(1296),
    [anon_sym__Atomic] = ACTIONS(1296),
    [anon_sym__Noreturn] = ACTIONS(1296),
    [anon_sym_noreturn] = ACTIONS(1296),
    [sym_primitive_type] = ACTIONS(1296),
    [anon_sym_enum] = ACTIONS(1296),
    [anon_sym_struct] = ACTIONS(1296),
    [anon_sym_union] = ACTIONS(1296),
    [anon_sym_if] = ACTIONS(1296),
    [anon_sym_switch] = ACTIONS(1296),
    [anon_sym_case] = ACTIONS(1296),
    [anon_sym_default] = ACTIONS(1296),
    [anon_sym_while] = ACTIONS(1296),
    [anon_sym_do] = ACTIONS(1296),
    [anon_sym_for] = ACTIONS(1296),
    [anon_sym_return] = ACTIONS(1296),
    [anon_sym_break] = ACTIONS(1296),
    [anon_sym_continue] = ACTIONS(1296),
    [anon_sym_goto] = ACTIONS(1296),
    [anon_sym___try] = ACTIONS(1296),
    [anon_sym___leave] = ACTIONS(1296),
    [anon_sym_DASH_DASH] = ACTIONS(1298),
    [anon_sym_PLUS_PLUS] = ACTIONS(1298),
    [anon_sym_sizeof] = ACTIONS(1296),
    [anon_sym___alignof__] = ACTIONS(1296),
    [anon_sym___alignof] = ACTIONS(1296),
    [anon_sym__alignof] = ACTIONS(1296),
    [anon_sym_alignof] = ACTIONS(1296),
    [anon_sym__Alignof] = ACTIONS(1296),
    [anon_sym_offsetof] = ACTIONS(1296),
    [anon_sym__Generic] = ACTIONS(1296),
    [anon_sym_asm] = ACTIONS(1296),
    [anon_sym___asm__] = ACTIONS(1296),
    [sym_number_literal] = ACTIONS(1298),
    [anon_sym_L_SQUOTE] = ACTIONS(1298),
    [anon_sym_u_SQUOTE] = ACTIONS(1298),
    [anon_sym_U_SQUOTE] = ACTIONS(1298),
    [anon_sym_u8_SQUOTE] = ACTIONS(1298),
    [anon_sym_SQUOTE] = ACTIONS(1298),
    [anon_sym_L_DQUOTE] = ACTIONS(1298),
    [anon_sym_u_DQUOTE] = ACTIONS(1298),
    [anon_sym_U_DQUOTE] = ACTIONS(1298),
    [anon_sym_u8_DQUOTE] = ACTIONS(1298),
    [anon_sym_DQUOTE] = ACTIONS(1298),
    [sym_true] = ACTIONS(1296),
    [sym_false] = ACTIONS(1296),
    [anon_sym_NULL] = ACTIONS(1296),
    [anon_sym_nullptr] = ACTIONS(1296),
    [sym_comment] = ACTIONS(3),
  },
  [396] = {
    [sym_attribute_declaration] = STATE(314),
    [sym_compound_statement] = STATE(285),
    [sym_attributed_statement] = STATE(285),
    [sym_labeled_statement] = STATE(285),
    [sym_expression_statement] = STATE(285),
    [sym_if_statement] = STATE(285),
    [sym_switch_statement] = STATE(285),
    [sym_case_statement] = STATE(285),
    [sym_while_statement] = STATE(285),
    [sym_do_statement] = STATE(285),
    [sym_for_statement] = STATE(285),
    [sym_return_statement] = STATE(285),
    [sym_break_statement] = STATE(285),
    [sym_continue_statement] = STATE(285),
    [sym_goto_statement] = STATE(285),
    [sym_seh_try_statement] = STATE(285),
    [sym_seh_leave_statement] = STATE(285),
    [sym__expression] = STATE(1080),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1898),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(314),
    [sym_identifier] = ACTIONS(1378),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(913),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(57),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(61),
    [anon_sym_default] = ACTIONS(63),
    [anon_sym_while] = ACTIONS(65),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(69),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(915),
    [anon_sym___leave] = ACTIONS(917),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [397] = {
    [sym_attribute_declaration] = STATE(335),
    [sym_compound_statement] = STATE(75),
    [sym_attributed_statement] = STATE(75),
    [sym_labeled_statement] = STATE(75),
    [sym_expression_statement] = STATE(75),
    [sym_if_statement] = STATE(75),
    [sym_switch_statement] = STATE(75),
    [sym_case_statement] = STATE(75),
    [sym_while_statement] = STATE(75),
    [sym_do_statement] = STATE(75),
    [sym_for_statement] = STATE(75),
    [sym_return_statement] = STATE(75),
    [sym_break_statement] = STATE(75),
    [sym_continue_statement] = STATE(75),
    [sym_goto_statement] = STATE(75),
    [sym_seh_try_statement] = STATE(75),
    [sym_seh_leave_statement] = STATE(75),
    [sym__expression] = STATE(1085),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1922),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(335),
    [sym_identifier] = ACTIONS(1530),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(121),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(129),
    [anon_sym_if] = ACTIONS(131),
    [anon_sym_switch] = ACTIONS(133),
    [anon_sym_case] = ACTIONS(135),
    [anon_sym_default] = ACTIONS(137),
    [anon_sym_while] = ACTIONS(139),
    [anon_sym_do] = ACTIONS(141),
    [anon_sym_for] = ACTIONS(143),
    [anon_sym_return] = ACTIONS(145),
    [anon_sym_break] = ACTIONS(147),
    [anon_sym_continue] = ACTIONS(149),
    [anon_sym_goto] = ACTIONS(151),
    [anon_sym___try] = ACTIONS(153),
    [anon_sym___leave] = ACTIONS(155),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [398] = {
    [sym_identifier] = ACTIONS(1284),
    [aux_sym_preproc_include_token1] = ACTIONS(1284),
    [aux_sym_preproc_def_token1] = ACTIONS(1284),
    [aux_sym_preproc_if_token1] = ACTIONS(1284),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1284),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1284),
    [sym_preproc_directive] = ACTIONS(1284),
    [anon_sym_LPAREN2] = ACTIONS(1286),
    [anon_sym_BANG] = ACTIONS(1286),
    [anon_sym_TILDE] = ACTIONS(1286),
    [anon_sym_DASH] = ACTIONS(1284),
    [anon_sym_PLUS] = ACTIONS(1284),
    [anon_sym_STAR] = ACTIONS(1286),
    [anon_sym_AMP] = ACTIONS(1286),
    [anon_sym_SEMI] = ACTIONS(1286),
    [anon_sym___extension__] = ACTIONS(1284),
    [anon_sym_typedef] = ACTIONS(1284),
    [anon_sym_extern] = ACTIONS(1284),
    [anon_sym___attribute__] = ACTIONS(1284),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1286),
    [anon_sym___declspec] = ACTIONS(1284),
    [anon_sym___cdecl] = ACTIONS(1284),
    [anon_sym___clrcall] = ACTIONS(1284),
    [anon_sym___stdcall] = ACTIONS(1284),
    [anon_sym___fastcall] = ACTIONS(1284),
    [anon_sym___thiscall] = ACTIONS(1284),
    [anon_sym___vectorcall] = ACTIONS(1284),
    [anon_sym_LBRACE] = ACTIONS(1286),
    [anon_sym_RBRACE] = ACTIONS(1286),
    [anon_sym_signed] = ACTIONS(1284),
    [anon_sym_unsigned] = ACTIONS(1284),
    [anon_sym_long] = ACTIONS(1284),
    [anon_sym_short] = ACTIONS(1284),
    [anon_sym_static] = ACTIONS(1284),
    [anon_sym_auto] = ACTIONS(1284),
    [anon_sym_register] = ACTIONS(1284),
    [anon_sym_inline] = ACTIONS(1284),
    [anon_sym___inline] = ACTIONS(1284),
    [anon_sym___inline__] = ACTIONS(1284),
    [anon_sym___forceinline] = ACTIONS(1284),
    [anon_sym_thread_local] = ACTIONS(1284),
    [anon_sym___thread] = ACTIONS(1284),
    [anon_sym_const] = ACTIONS(1284),
    [anon_sym_constexpr] = ACTIONS(1284),
    [anon_sym_volatile] = ACTIONS(1284),
    [anon_sym_restrict] = ACTIONS(1284),
    [anon_sym___restrict__] = ACTIONS(1284),
    [anon_sym__Atomic] = ACTIONS(1284),
    [anon_sym__Noreturn] = ACTIONS(1284),
    [anon_sym_noreturn] = ACTIONS(1284),
    [sym_primitive_type] = ACTIONS(1284),
    [anon_sym_enum] = ACTIONS(1284),
    [anon_sym_struct] = ACTIONS(1284),
    [anon_sym_union] = ACTIONS(1284),
    [anon_sym_if] = ACTIONS(1284),
    [anon_sym_switch] = ACTIONS(1284),
    [anon_sym_case] = ACTIONS(1284),
    [anon_sym_default] = ACTIONS(1284),
    [anon_sym_while] = ACTIONS(1284),
    [anon_sym_do] = ACTIONS(1284),
    [anon_sym_for] = ACTIONS(1284),
    [anon_sym_return] = ACTIONS(1284),
    [anon_sym_break] = ACTIONS(1284),
    [anon_sym_continue] = ACTIONS(1284),
    [anon_sym_goto] = ACTIONS(1284),
    [anon_sym___try] = ACTIONS(1284),
    [anon_sym___leave] = ACTIONS(1284),
    [anon_sym_DASH_DASH] = ACTIONS(1286),
    [anon_sym_PLUS_PLUS] = ACTIONS(1286),
    [anon_sym_sizeof] = ACTIONS(1284),
    [anon_sym___alignof__] = ACTIONS(1284),
    [anon_sym___alignof] = ACTIONS(1284),
    [anon_sym__alignof] = ACTIONS(1284),
    [anon_sym_alignof] = ACTIONS(1284),
    [anon_sym__Alignof] = ACTIONS(1284),
    [anon_sym_offsetof] = ACTIONS(1284),
    [anon_sym__Generic] = ACTIONS(1284),
    [anon_sym_asm] = ACTIONS(1284),
    [anon_sym___asm__] = ACTIONS(1284),
    [sym_number_literal] = ACTIONS(1286),
    [anon_sym_L_SQUOTE] = ACTIONS(1286),
    [anon_sym_u_SQUOTE] = ACTIONS(1286),
    [anon_sym_U_SQUOTE] = ACTIONS(1286),
    [anon_sym_u8_SQUOTE] = ACTIONS(1286),
    [anon_sym_SQUOTE] = ACTIONS(1286),
    [anon_sym_L_DQUOTE] = ACTIONS(1286),
    [anon_sym_u_DQUOTE] = ACTIONS(1286),
    [anon_sym_U_DQUOTE] = ACTIONS(1286),
    [anon_sym_u8_DQUOTE] = ACTIONS(1286),
    [anon_sym_DQUOTE] = ACTIONS(1286),
    [sym_true] = ACTIONS(1284),
    [sym_false] = ACTIONS(1284),
    [anon_sym_NULL] = ACTIONS(1284),
    [anon_sym_nullptr] = ACTIONS(1284),
    [sym_comment] = ACTIONS(3),
  },
  [399] = {
    [sym_attribute_declaration] = STATE(400),
    [sym_compound_statement] = STATE(277),
    [sym_attributed_statement] = STATE(277),
    [sym_labeled_statement] = STATE(277),
    [sym_expression_statement] = STATE(277),
    [sym_if_statement] = STATE(277),
    [sym_switch_statement] = STATE(277),
    [sym_case_statement] = STATE(277),
    [sym_while_statement] = STATE(277),
    [sym_do_statement] = STATE(277),
    [sym_for_statement] = STATE(277),
    [sym_return_statement] = STATE(277),
    [sym_break_statement] = STATE(277),
    [sym_continue_statement] = STATE(277),
    [sym_goto_statement] = STATE(277),
    [sym_seh_try_statement] = STATE(277),
    [sym_seh_leave_statement] = STATE(277),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(400),
    [sym_identifier] = ACTIONS(1559),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [400] = {
    [sym_attribute_declaration] = STATE(371),
    [sym_compound_statement] = STATE(241),
    [sym_attributed_statement] = STATE(241),
    [sym_labeled_statement] = STATE(241),
    [sym_expression_statement] = STATE(241),
    [sym_if_statement] = STATE(241),
    [sym_switch_statement] = STATE(241),
    [sym_case_statement] = STATE(241),
    [sym_while_statement] = STATE(241),
    [sym_do_statement] = STATE(241),
    [sym_for_statement] = STATE(241),
    [sym_return_statement] = STATE(241),
    [sym_break_statement] = STATE(241),
    [sym_continue_statement] = STATE(241),
    [sym_goto_statement] = STATE(241),
    [sym_seh_try_statement] = STATE(241),
    [sym_seh_leave_statement] = STATE(241),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(371),
    [sym_identifier] = ACTIONS(1559),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(371),
    [anon_sym_if] = ACTIONS(375),
    [anon_sym_switch] = ACTIONS(377),
    [anon_sym_case] = ACTIONS(379),
    [anon_sym_default] = ACTIONS(381),
    [anon_sym_while] = ACTIONS(383),
    [anon_sym_do] = ACTIONS(385),
    [anon_sym_for] = ACTIONS(387),
    [anon_sym_return] = ACTIONS(389),
    [anon_sym_break] = ACTIONS(391),
    [anon_sym_continue] = ACTIONS(393),
    [anon_sym_goto] = ACTIONS(395),
    [anon_sym___try] = ACTIONS(397),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [401] = {
    [sym_identifier] = ACTIONS(1320),
    [aux_sym_preproc_include_token1] = ACTIONS(1320),
    [aux_sym_preproc_def_token1] = ACTIONS(1320),
    [aux_sym_preproc_if_token1] = ACTIONS(1320),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1320),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1320),
    [sym_preproc_directive] = ACTIONS(1320),
    [anon_sym_LPAREN2] = ACTIONS(1322),
    [anon_sym_BANG] = ACTIONS(1322),
    [anon_sym_TILDE] = ACTIONS(1322),
    [anon_sym_DASH] = ACTIONS(1320),
    [anon_sym_PLUS] = ACTIONS(1320),
    [anon_sym_STAR] = ACTIONS(1322),
    [anon_sym_AMP] = ACTIONS(1322),
    [anon_sym_SEMI] = ACTIONS(1322),
    [anon_sym___extension__] = ACTIONS(1320),
    [anon_sym_typedef] = ACTIONS(1320),
    [anon_sym_extern] = ACTIONS(1320),
    [anon_sym___attribute__] = ACTIONS(1320),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1322),
    [anon_sym___declspec] = ACTIONS(1320),
    [anon_sym___cdecl] = ACTIONS(1320),
    [anon_sym___clrcall] = ACTIONS(1320),
    [anon_sym___stdcall] = ACTIONS(1320),
    [anon_sym___fastcall] = ACTIONS(1320),
    [anon_sym___thiscall] = ACTIONS(1320),
    [anon_sym___vectorcall] = ACTIONS(1320),
    [anon_sym_LBRACE] = ACTIONS(1322),
    [anon_sym_RBRACE] = ACTIONS(1322),
    [anon_sym_signed] = ACTIONS(1320),
    [anon_sym_unsigned] = ACTIONS(1320),
    [anon_sym_long] = ACTIONS(1320),
    [anon_sym_short] = ACTIONS(1320),
    [anon_sym_static] = ACTIONS(1320),
    [anon_sym_auto] = ACTIONS(1320),
    [anon_sym_register] = ACTIONS(1320),
    [anon_sym_inline] = ACTIONS(1320),
    [anon_sym___inline] = ACTIONS(1320),
    [anon_sym___inline__] = ACTIONS(1320),
    [anon_sym___forceinline] = ACTIONS(1320),
    [anon_sym_thread_local] = ACTIONS(1320),
    [anon_sym___thread] = ACTIONS(1320),
    [anon_sym_const] = ACTIONS(1320),
    [anon_sym_constexpr] = ACTIONS(1320),
    [anon_sym_volatile] = ACTIONS(1320),
    [anon_sym_restrict] = ACTIONS(1320),
    [anon_sym___restrict__] = ACTIONS(1320),
    [anon_sym__Atomic] = ACTIONS(1320),
    [anon_sym__Noreturn] = ACTIONS(1320),
    [anon_sym_noreturn] = ACTIONS(1320),
    [sym_primitive_type] = ACTIONS(1320),
    [anon_sym_enum] = ACTIONS(1320),
    [anon_sym_struct] = ACTIONS(1320),
    [anon_sym_union] = ACTIONS(1320),
    [anon_sym_if] = ACTIONS(1320),
    [anon_sym_switch] = ACTIONS(1320),
    [anon_sym_case] = ACTIONS(1320),
    [anon_sym_default] = ACTIONS(1320),
    [anon_sym_while] = ACTIONS(1320),
    [anon_sym_do] = ACTIONS(1320),
    [anon_sym_for] = ACTIONS(1320),
    [anon_sym_return] = ACTIONS(1320),
    [anon_sym_break] = ACTIONS(1320),
    [anon_sym_continue] = ACTIONS(1320),
    [anon_sym_goto] = ACTIONS(1320),
    [anon_sym___try] = ACTIONS(1320),
    [anon_sym___leave] = ACTIONS(1320),
    [anon_sym_DASH_DASH] = ACTIONS(1322),
    [anon_sym_PLUS_PLUS] = ACTIONS(1322),
    [anon_sym_sizeof] = ACTIONS(1320),
    [anon_sym___alignof__] = ACTIONS(1320),
    [anon_sym___alignof] = ACTIONS(1320),
    [anon_sym__alignof] = ACTIONS(1320),
    [anon_sym_alignof] = ACTIONS(1320),
    [anon_sym__Alignof] = ACTIONS(1320),
    [anon_sym_offsetof] = ACTIONS(1320),
    [anon_sym__Generic] = ACTIONS(1320),
    [anon_sym_asm] = ACTIONS(1320),
    [anon_sym___asm__] = ACTIONS(1320),
    [sym_number_literal] = ACTIONS(1322),
    [anon_sym_L_SQUOTE] = ACTIONS(1322),
    [anon_sym_u_SQUOTE] = ACTIONS(1322),
    [anon_sym_U_SQUOTE] = ACTIONS(1322),
    [anon_sym_u8_SQUOTE] = ACTIONS(1322),
    [anon_sym_SQUOTE] = ACTIONS(1322),
    [anon_sym_L_DQUOTE] = ACTIONS(1322),
    [anon_sym_u_DQUOTE] = ACTIONS(1322),
    [anon_sym_U_DQUOTE] = ACTIONS(1322),
    [anon_sym_u8_DQUOTE] = ACTIONS(1322),
    [anon_sym_DQUOTE] = ACTIONS(1322),
    [sym_true] = ACTIONS(1320),
    [sym_false] = ACTIONS(1320),
    [anon_sym_NULL] = ACTIONS(1320),
    [anon_sym_nullptr] = ACTIONS(1320),
    [sym_comment] = ACTIONS(3),
  },
  [402] = {
    [sym_identifier] = ACTIONS(1340),
    [aux_sym_preproc_include_token1] = ACTIONS(1340),
    [aux_sym_preproc_def_token1] = ACTIONS(1340),
    [aux_sym_preproc_if_token1] = ACTIONS(1340),
    [aux_sym_preproc_if_token2] = ACTIONS(1340),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1340),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1340),
    [sym_preproc_directive] = ACTIONS(1340),
    [anon_sym_LPAREN2] = ACTIONS(1342),
    [anon_sym_BANG] = ACTIONS(1342),
    [anon_sym_TILDE] = ACTIONS(1342),
    [anon_sym_DASH] = ACTIONS(1340),
    [anon_sym_PLUS] = ACTIONS(1340),
    [anon_sym_STAR] = ACTIONS(1342),
    [anon_sym_AMP] = ACTIONS(1342),
    [anon_sym_SEMI] = ACTIONS(1342),
    [anon_sym___extension__] = ACTIONS(1340),
    [anon_sym_typedef] = ACTIONS(1340),
    [anon_sym_extern] = ACTIONS(1340),
    [anon_sym___attribute__] = ACTIONS(1340),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1342),
    [anon_sym___declspec] = ACTIONS(1340),
    [anon_sym___cdecl] = ACTIONS(1340),
    [anon_sym___clrcall] = ACTIONS(1340),
    [anon_sym___stdcall] = ACTIONS(1340),
    [anon_sym___fastcall] = ACTIONS(1340),
    [anon_sym___thiscall] = ACTIONS(1340),
    [anon_sym___vectorcall] = ACTIONS(1340),
    [anon_sym_LBRACE] = ACTIONS(1342),
    [anon_sym_signed] = ACTIONS(1340),
    [anon_sym_unsigned] = ACTIONS(1340),
    [anon_sym_long] = ACTIONS(1340),
    [anon_sym_short] = ACTIONS(1340),
    [anon_sym_static] = ACTIONS(1340),
    [anon_sym_auto] = ACTIONS(1340),
    [anon_sym_register] = ACTIONS(1340),
    [anon_sym_inline] = ACTIONS(1340),
    [anon_sym___inline] = ACTIONS(1340),
    [anon_sym___inline__] = ACTIONS(1340),
    [anon_sym___forceinline] = ACTIONS(1340),
    [anon_sym_thread_local] = ACTIONS(1340),
    [anon_sym___thread] = ACTIONS(1340),
    [anon_sym_const] = ACTIONS(1340),
    [anon_sym_constexpr] = ACTIONS(1340),
    [anon_sym_volatile] = ACTIONS(1340),
    [anon_sym_restrict] = ACTIONS(1340),
    [anon_sym___restrict__] = ACTIONS(1340),
    [anon_sym__Atomic] = ACTIONS(1340),
    [anon_sym__Noreturn] = ACTIONS(1340),
    [anon_sym_noreturn] = ACTIONS(1340),
    [sym_primitive_type] = ACTIONS(1340),
    [anon_sym_enum] = ACTIONS(1340),
    [anon_sym_struct] = ACTIONS(1340),
    [anon_sym_union] = ACTIONS(1340),
    [anon_sym_if] = ACTIONS(1340),
    [anon_sym_switch] = ACTIONS(1340),
    [anon_sym_case] = ACTIONS(1340),
    [anon_sym_default] = ACTIONS(1340),
    [anon_sym_while] = ACTIONS(1340),
    [anon_sym_do] = ACTIONS(1340),
    [anon_sym_for] = ACTIONS(1340),
    [anon_sym_return] = ACTIONS(1340),
    [anon_sym_break] = ACTIONS(1340),
    [anon_sym_continue] = ACTIONS(1340),
    [anon_sym_goto] = ACTIONS(1340),
    [anon_sym___try] = ACTIONS(1340),
    [anon_sym___leave] = ACTIONS(1340),
    [anon_sym_DASH_DASH] = ACTIONS(1342),
    [anon_sym_PLUS_PLUS] = ACTIONS(1342),
    [anon_sym_sizeof] = ACTIONS(1340),
    [anon_sym___alignof__] = ACTIONS(1340),
    [anon_sym___alignof] = ACTIONS(1340),
    [anon_sym__alignof] = ACTIONS(1340),
    [anon_sym_alignof] = ACTIONS(1340),
    [anon_sym__Alignof] = ACTIONS(1340),
    [anon_sym_offsetof] = ACTIONS(1340),
    [anon_sym__Generic] = ACTIONS(1340),
    [anon_sym_asm] = ACTIONS(1340),
    [anon_sym___asm__] = ACTIONS(1340),
    [sym_number_literal] = ACTIONS(1342),
    [anon_sym_L_SQUOTE] = ACTIONS(1342),
    [anon_sym_u_SQUOTE] = ACTIONS(1342),
    [anon_sym_U_SQUOTE] = ACTIONS(1342),
    [anon_sym_u8_SQUOTE] = ACTIONS(1342),
    [anon_sym_SQUOTE] = ACTIONS(1342),
    [anon_sym_L_DQUOTE] = ACTIONS(1342),
    [anon_sym_u_DQUOTE] = ACTIONS(1342),
    [anon_sym_U_DQUOTE] = ACTIONS(1342),
    [anon_sym_u8_DQUOTE] = ACTIONS(1342),
    [anon_sym_DQUOTE] = ACTIONS(1342),
    [sym_true] = ACTIONS(1340),
    [sym_false] = ACTIONS(1340),
    [anon_sym_NULL] = ACTIONS(1340),
    [anon_sym_nullptr] = ACTIONS(1340),
    [sym_comment] = ACTIONS(3),
  },
  [403] = {
    [sym_attribute_declaration] = STATE(358),
    [sym_compound_statement] = STATE(2010),
    [sym_attributed_statement] = STATE(2010),
    [sym_labeled_statement] = STATE(2010),
    [sym_expression_statement] = STATE(2010),
    [sym_if_statement] = STATE(2010),
    [sym_switch_statement] = STATE(2010),
    [sym_case_statement] = STATE(2010),
    [sym_while_statement] = STATE(2010),
    [sym_do_statement] = STATE(2010),
    [sym_for_statement] = STATE(2010),
    [sym_return_statement] = STATE(2010),
    [sym_break_statement] = STATE(2010),
    [sym_continue_statement] = STATE(2010),
    [sym_goto_statement] = STATE(2010),
    [sym_seh_try_statement] = STATE(2010),
    [sym_seh_leave_statement] = STATE(2010),
    [sym__expression] = STATE(1033),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(2003),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_attributed_declarator_repeat1] = STATE(358),
    [sym_identifier] = ACTIONS(1380),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_SEMI] = ACTIONS(363),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1376),
    [anon_sym_LBRACE] = ACTIONS(41),
    [anon_sym_if] = ACTIONS(1077),
    [anon_sym_switch] = ACTIONS(59),
    [anon_sym_case] = ACTIONS(1382),
    [anon_sym_default] = ACTIONS(1384),
    [anon_sym_while] = ACTIONS(1079),
    [anon_sym_do] = ACTIONS(67),
    [anon_sym_for] = ACTIONS(1081),
    [anon_sym_return] = ACTIONS(71),
    [anon_sym_break] = ACTIONS(73),
    [anon_sym_continue] = ACTIONS(75),
    [anon_sym_goto] = ACTIONS(77),
    [anon_sym___try] = ACTIONS(1083),
    [anon_sym___leave] = ACTIONS(399),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [404] = {
    [ts_builtin_sym_end] = ACTIONS(1250),
    [sym_identifier] = ACTIONS(1248),
    [aux_sym_preproc_include_token1] = ACTIONS(1248),
    [aux_sym_preproc_def_token1] = ACTIONS(1248),
    [anon_sym_COMMA] = ACTIONS(1250),
    [aux_sym_preproc_if_token1] = ACTIONS(1248),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1248),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1248),
    [sym_preproc_directive] = ACTIONS(1248),
    [anon_sym_LPAREN2] = ACTIONS(1250),
    [anon_sym_BANG] = ACTIONS(1250),
    [anon_sym_TILDE] = ACTIONS(1250),
    [anon_sym_DASH] = ACTIONS(1248),
    [anon_sym_PLUS] = ACTIONS(1248),
    [anon_sym_STAR] = ACTIONS(1250),
    [anon_sym_AMP] = ACTIONS(1250),
    [anon_sym___extension__] = ACTIONS(1248),
    [anon_sym_typedef] = ACTIONS(1248),
    [anon_sym_extern] = ACTIONS(1248),
    [anon_sym___attribute__] = ACTIONS(1248),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1250),
    [anon_sym___declspec] = ACTIONS(1248),
    [anon_sym___cdecl] = ACTIONS(1248),
    [anon_sym___clrcall] = ACTIONS(1248),
    [anon_sym___stdcall] = ACTIONS(1248),
    [anon_sym___fastcall] = ACTIONS(1248),
    [anon_sym___thiscall] = ACTIONS(1248),
    [anon_sym___vectorcall] = ACTIONS(1248),
    [anon_sym_LBRACE] = ACTIONS(1250),
    [anon_sym_RBRACE] = ACTIONS(1250),
    [anon_sym_signed] = ACTIONS(1248),
    [anon_sym_unsigned] = ACTIONS(1248),
    [anon_sym_long] = ACTIONS(1248),
    [anon_sym_short] = ACTIONS(1248),
    [anon_sym_static] = ACTIONS(1248),
    [anon_sym_auto] = ACTIONS(1248),
    [anon_sym_register] = ACTIONS(1248),
    [anon_sym_inline] = ACTIONS(1248),
    [anon_sym___inline] = ACTIONS(1248),
    [anon_sym___inline__] = ACTIONS(1248),
    [anon_sym___forceinline] = ACTIONS(1248),
    [anon_sym_thread_local] = ACTIONS(1248),
    [anon_sym___thread] = ACTIONS(1248),
    [anon_sym_const] = ACTIONS(1248),
    [anon_sym_constexpr] = ACTIONS(1248),
    [anon_sym_volatile] = ACTIONS(1248),
    [anon_sym_restrict] = ACTIONS(1248),
    [anon_sym___restrict__] = ACTIONS(1248),
    [anon_sym__Atomic] = ACTIONS(1248),
    [anon_sym__Noreturn] = ACTIONS(1248),
    [anon_sym_noreturn] = ACTIONS(1248),
    [sym_primitive_type] = ACTIONS(1248),
    [anon_sym_enum] = ACTIONS(1248),
    [anon_sym_struct] = ACTIONS(1248),
    [anon_sym_union] = ACTIONS(1248),
    [anon_sym_if] = ACTIONS(1248),
    [anon_sym_switch] = ACTIONS(1248),
    [anon_sym_case] = ACTIONS(1248),
    [anon_sym_default] = ACTIONS(1248),
    [anon_sym_while] = ACTIONS(1248),
    [anon_sym_do] = ACTIONS(1248),
    [anon_sym_for] = ACTIONS(1248),
    [anon_sym_return] = ACTIONS(1248),
    [anon_sym_break] = ACTIONS(1248),
    [anon_sym_continue] = ACTIONS(1248),
    [anon_sym_goto] = ACTIONS(1248),
    [anon_sym_DASH_DASH] = ACTIONS(1250),
    [anon_sym_PLUS_PLUS] = ACTIONS(1250),
    [anon_sym_sizeof] = ACTIONS(1248),
    [anon_sym___alignof__] = ACTIONS(1248),
    [anon_sym___alignof] = ACTIONS(1248),
    [anon_sym__alignof] = ACTIONS(1248),
    [anon_sym_alignof] = ACTIONS(1248),
    [anon_sym__Alignof] = ACTIONS(1248),
    [anon_sym_offsetof] = ACTIONS(1248),
    [anon_sym__Generic] = ACTIONS(1248),
    [anon_sym_asm] = ACTIONS(1248),
    [anon_sym___asm__] = ACTIONS(1248),
    [sym_number_literal] = ACTIONS(1250),
    [anon_sym_L_SQUOTE] = ACTIONS(1250),
    [anon_sym_u_SQUOTE] = ACTIONS(1250),
    [anon_sym_U_SQUOTE] = ACTIONS(1250),
    [anon_sym_u8_SQUOTE] = ACTIONS(1250),
    [anon_sym_SQUOTE] = ACTIONS(1250),
    [anon_sym_L_DQUOTE] = ACTIONS(1250),
    [anon_sym_u_DQUOTE] = ACTIONS(1250),
    [anon_sym_U_DQUOTE] = ACTIONS(1250),
    [anon_sym_u8_DQUOTE] = ACTIONS(1250),
    [anon_sym_DQUOTE] = ACTIONS(1250),
    [sym_true] = ACTIONS(1248),
    [sym_false] = ACTIONS(1248),
    [anon_sym_NULL] = ACTIONS(1248),
    [anon_sym_nullptr] = ACTIONS(1248),
    [sym_comment] = ACTIONS(3),
  },
  [405] = {
    [ts_builtin_sym_end] = ACTIONS(1294),
    [sym_identifier] = ACTIONS(1292),
    [aux_sym_preproc_include_token1] = ACTIONS(1292),
    [aux_sym_preproc_def_token1] = ACTIONS(1292),
    [anon_sym_COMMA] = ACTIONS(1294),
    [aux_sym_preproc_if_token1] = ACTIONS(1292),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1292),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1292),
    [sym_preproc_directive] = ACTIONS(1292),
    [anon_sym_LPAREN2] = ACTIONS(1294),
    [anon_sym_BANG] = ACTIONS(1294),
    [anon_sym_TILDE] = ACTIONS(1294),
    [anon_sym_DASH] = ACTIONS(1292),
    [anon_sym_PLUS] = ACTIONS(1292),
    [anon_sym_STAR] = ACTIONS(1294),
    [anon_sym_AMP] = ACTIONS(1294),
    [anon_sym___extension__] = ACTIONS(1292),
    [anon_sym_typedef] = ACTIONS(1292),
    [anon_sym_extern] = ACTIONS(1292),
    [anon_sym___attribute__] = ACTIONS(1292),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1294),
    [anon_sym___declspec] = ACTIONS(1292),
    [anon_sym___cdecl] = ACTIONS(1292),
    [anon_sym___clrcall] = ACTIONS(1292),
    [anon_sym___stdcall] = ACTIONS(1292),
    [anon_sym___fastcall] = ACTIONS(1292),
    [anon_sym___thiscall] = ACTIONS(1292),
    [anon_sym___vectorcall] = ACTIONS(1292),
    [anon_sym_LBRACE] = ACTIONS(1294),
    [anon_sym_RBRACE] = ACTIONS(1294),
    [anon_sym_signed] = ACTIONS(1292),
    [anon_sym_unsigned] = ACTIONS(1292),
    [anon_sym_long] = ACTIONS(1292),
    [anon_sym_short] = ACTIONS(1292),
    [anon_sym_static] = ACTIONS(1292),
    [anon_sym_auto] = ACTIONS(1292),
    [anon_sym_register] = ACTIONS(1292),
    [anon_sym_inline] = ACTIONS(1292),
    [anon_sym___inline] = ACTIONS(1292),
    [anon_sym___inline__] = ACTIONS(1292),
    [anon_sym___forceinline] = ACTIONS(1292),
    [anon_sym_thread_local] = ACTIONS(1292),
    [anon_sym___thread] = ACTIONS(1292),
    [anon_sym_const] = ACTIONS(1292),
    [anon_sym_constexpr] = ACTIONS(1292),
    [anon_sym_volatile] = ACTIONS(1292),
    [anon_sym_restrict] = ACTIONS(1292),
    [anon_sym___restrict__] = ACTIONS(1292),
    [anon_sym__Atomic] = ACTIONS(1292),
    [anon_sym__Noreturn] = ACTIONS(1292),
    [anon_sym_noreturn] = ACTIONS(1292),
    [sym_primitive_type] = ACTIONS(1292),
    [anon_sym_enum] = ACTIONS(1292),
    [anon_sym_struct] = ACTIONS(1292),
    [anon_sym_union] = ACTIONS(1292),
    [anon_sym_if] = ACTIONS(1292),
    [anon_sym_switch] = ACTIONS(1292),
    [anon_sym_case] = ACTIONS(1292),
    [anon_sym_default] = ACTIONS(1292),
    [anon_sym_while] = ACTIONS(1292),
    [anon_sym_do] = ACTIONS(1292),
    [anon_sym_for] = ACTIONS(1292),
    [anon_sym_return] = ACTIONS(1292),
    [anon_sym_break] = ACTIONS(1292),
    [anon_sym_continue] = ACTIONS(1292),
    [anon_sym_goto] = ACTIONS(1292),
    [anon_sym_DASH_DASH] = ACTIONS(1294),
    [anon_sym_PLUS_PLUS] = ACTIONS(1294),
    [anon_sym_sizeof] = ACTIONS(1292),
    [anon_sym___alignof__] = ACTIONS(1292),
    [anon_sym___alignof] = ACTIONS(1292),
    [anon_sym__alignof] = ACTIONS(1292),
    [anon_sym_alignof] = ACTIONS(1292),
    [anon_sym__Alignof] = ACTIONS(1292),
    [anon_sym_offsetof] = ACTIONS(1292),
    [anon_sym__Generic] = ACTIONS(1292),
    [anon_sym_asm] = ACTIONS(1292),
    [anon_sym___asm__] = ACTIONS(1292),
    [sym_number_literal] = ACTIONS(1294),
    [anon_sym_L_SQUOTE] = ACTIONS(1294),
    [anon_sym_u_SQUOTE] = ACTIONS(1294),
    [anon_sym_U_SQUOTE] = ACTIONS(1294),
    [anon_sym_u8_SQUOTE] = ACTIONS(1294),
    [anon_sym_SQUOTE] = ACTIONS(1294),
    [anon_sym_L_DQUOTE] = ACTIONS(1294),
    [anon_sym_u_DQUOTE] = ACTIONS(1294),
    [anon_sym_U_DQUOTE] = ACTIONS(1294),
    [anon_sym_u8_DQUOTE] = ACTIONS(1294),
    [anon_sym_DQUOTE] = ACTIONS(1294),
    [sym_true] = ACTIONS(1292),
    [sym_false] = ACTIONS(1292),
    [anon_sym_NULL] = ACTIONS(1292),
    [anon_sym_nullptr] = ACTIONS(1292),
    [sym_comment] = ACTIONS(3),
  },
  [406] = {
    [sym__expression] = STATE(869),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(700),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(700),
    [sym_call_expression] = STATE(700),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(700),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(700),
    [sym_initializer_list] = STATE(714),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_identifier] = ACTIONS(1364),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1352),
    [anon_sym_LPAREN2] = ACTIONS(1352),
    [anon_sym_BANG] = ACTIONS(1651),
    [anon_sym_TILDE] = ACTIONS(1653),
    [anon_sym_DASH] = ACTIONS(1350),
    [anon_sym_PLUS] = ACTIONS(1350),
    [anon_sym_STAR] = ACTIONS(1350),
    [anon_sym_SLASH] = ACTIONS(1350),
    [anon_sym_PERCENT] = ACTIONS(1350),
    [anon_sym_PIPE_PIPE] = ACTIONS(1352),
    [anon_sym_AMP_AMP] = ACTIONS(1352),
    [anon_sym_PIPE] = ACTIONS(1350),
    [anon_sym_CARET] = ACTIONS(1350),
    [anon_sym_AMP] = ACTIONS(1350),
    [anon_sym_EQ_EQ] = ACTIONS(1352),
    [anon_sym_BANG_EQ] = ACTIONS(1352),
    [anon_sym_GT] = ACTIONS(1350),
    [anon_sym_GT_EQ] = ACTIONS(1352),
    [anon_sym_LT_EQ] = ACTIONS(1352),
    [anon_sym_LT] = ACTIONS(1350),
    [anon_sym_LT_LT] = ACTIONS(1350),
    [anon_sym_GT_GT] = ACTIONS(1350),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_LBRACK] = ACTIONS(1352),
    [anon_sym_RBRACK] = ACTIONS(1352),
    [anon_sym_EQ] = ACTIONS(1350),
    [anon_sym_QMARK] = ACTIONS(1352),
    [anon_sym_STAR_EQ] = ACTIONS(1352),
    [anon_sym_SLASH_EQ] = ACTIONS(1352),
    [anon_sym_PERCENT_EQ] = ACTIONS(1352),
    [anon_sym_PLUS_EQ] = ACTIONS(1352),
    [anon_sym_DASH_EQ] = ACTIONS(1352),
    [anon_sym_LT_LT_EQ] = ACTIONS(1352),
    [anon_sym_GT_GT_EQ] = ACTIONS(1352),
    [anon_sym_AMP_EQ] = ACTIONS(1352),
    [anon_sym_CARET_EQ] = ACTIONS(1352),
    [anon_sym_PIPE_EQ] = ACTIONS(1352),
    [anon_sym_DASH_DASH] = ACTIONS(1352),
    [anon_sym_PLUS_PLUS] = ACTIONS(1352),
    [anon_sym_sizeof] = ACTIONS(1655),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1350),
    [anon_sym_DASH_GT] = ACTIONS(1352),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [407] = {
    [ts_builtin_sym_end] = ACTIONS(1318),
    [sym_identifier] = ACTIONS(1316),
    [aux_sym_preproc_include_token1] = ACTIONS(1316),
    [aux_sym_preproc_def_token1] = ACTIONS(1316),
    [aux_sym_preproc_if_token1] = ACTIONS(1316),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1316),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1316),
    [sym_preproc_directive] = ACTIONS(1316),
    [anon_sym_LPAREN2] = ACTIONS(1318),
    [anon_sym_BANG] = ACTIONS(1318),
    [anon_sym_TILDE] = ACTIONS(1318),
    [anon_sym_DASH] = ACTIONS(1316),
    [anon_sym_PLUS] = ACTIONS(1316),
    [anon_sym_STAR] = ACTIONS(1318),
    [anon_sym_AMP] = ACTIONS(1318),
    [anon_sym___extension__] = ACTIONS(1316),
    [anon_sym_typedef] = ACTIONS(1316),
    [anon_sym_extern] = ACTIONS(1316),
    [anon_sym___attribute__] = ACTIONS(1316),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1318),
    [anon_sym___declspec] = ACTIONS(1316),
    [anon_sym___cdecl] = ACTIONS(1316),
    [anon_sym___clrcall] = ACTIONS(1316),
    [anon_sym___stdcall] = ACTIONS(1316),
    [anon_sym___fastcall] = ACTIONS(1316),
    [anon_sym___thiscall] = ACTIONS(1316),
    [anon_sym___vectorcall] = ACTIONS(1316),
    [anon_sym_LBRACE] = ACTIONS(1318),
    [anon_sym_signed] = ACTIONS(1316),
    [anon_sym_unsigned] = ACTIONS(1316),
    [anon_sym_long] = ACTIONS(1316),
    [anon_sym_short] = ACTIONS(1316),
    [anon_sym_static] = ACTIONS(1316),
    [anon_sym_auto] = ACTIONS(1316),
    [anon_sym_register] = ACTIONS(1316),
    [anon_sym_inline] = ACTIONS(1316),
    [anon_sym___inline] = ACTIONS(1316),
    [anon_sym___inline__] = ACTIONS(1316),
    [anon_sym___forceinline] = ACTIONS(1316),
    [anon_sym_thread_local] = ACTIONS(1316),
    [anon_sym___thread] = ACTIONS(1316),
    [anon_sym_const] = ACTIONS(1316),
    [anon_sym_constexpr] = ACTIONS(1316),
    [anon_sym_volatile] = ACTIONS(1316),
    [anon_sym_restrict] = ACTIONS(1316),
    [anon_sym___restrict__] = ACTIONS(1316),
    [anon_sym__Atomic] = ACTIONS(1316),
    [anon_sym__Noreturn] = ACTIONS(1316),
    [anon_sym_noreturn] = ACTIONS(1316),
    [sym_primitive_type] = ACTIONS(1316),
    [anon_sym_enum] = ACTIONS(1316),
    [anon_sym_struct] = ACTIONS(1316),
    [anon_sym_union] = ACTIONS(1316),
    [anon_sym_if] = ACTIONS(1316),
    [anon_sym_switch] = ACTIONS(1316),
    [anon_sym_case] = ACTIONS(1316),
    [anon_sym_default] = ACTIONS(1316),
    [anon_sym_while] = ACTIONS(1316),
    [anon_sym_do] = ACTIONS(1316),
    [anon_sym_for] = ACTIONS(1316),
    [anon_sym_return] = ACTIONS(1316),
    [anon_sym_break] = ACTIONS(1316),
    [anon_sym_continue] = ACTIONS(1316),
    [anon_sym_goto] = ACTIONS(1316),
    [anon_sym_DASH_DASH] = ACTIONS(1318),
    [anon_sym_PLUS_PLUS] = ACTIONS(1318),
    [anon_sym_sizeof] = ACTIONS(1316),
    [anon_sym___alignof__] = ACTIONS(1316),
    [anon_sym___alignof] = ACTIONS(1316),
    [anon_sym__alignof] = ACTIONS(1316),
    [anon_sym_alignof] = ACTIONS(1316),
    [anon_sym__Alignof] = ACTIONS(1316),
    [anon_sym_offsetof] = ACTIONS(1316),
    [anon_sym__Generic] = ACTIONS(1316),
    [anon_sym_asm] = ACTIONS(1316),
    [anon_sym___asm__] = ACTIONS(1316),
    [sym_number_literal] = ACTIONS(1318),
    [anon_sym_L_SQUOTE] = ACTIONS(1318),
    [anon_sym_u_SQUOTE] = ACTIONS(1318),
    [anon_sym_U_SQUOTE] = ACTIONS(1318),
    [anon_sym_u8_SQUOTE] = ACTIONS(1318),
    [anon_sym_SQUOTE] = ACTIONS(1318),
    [anon_sym_L_DQUOTE] = ACTIONS(1318),
    [anon_sym_u_DQUOTE] = ACTIONS(1318),
    [anon_sym_U_DQUOTE] = ACTIONS(1318),
    [anon_sym_u8_DQUOTE] = ACTIONS(1318),
    [anon_sym_DQUOTE] = ACTIONS(1318),
    [sym_true] = ACTIONS(1316),
    [sym_false] = ACTIONS(1316),
    [anon_sym_NULL] = ACTIONS(1316),
    [anon_sym_nullptr] = ACTIONS(1316),
    [sym_comment] = ACTIONS(3),
  },
  [408] = {
    [ts_builtin_sym_end] = ACTIONS(1346),
    [sym_identifier] = ACTIONS(1344),
    [aux_sym_preproc_include_token1] = ACTIONS(1344),
    [aux_sym_preproc_def_token1] = ACTIONS(1344),
    [aux_sym_preproc_if_token1] = ACTIONS(1344),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1344),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1344),
    [sym_preproc_directive] = ACTIONS(1344),
    [anon_sym_LPAREN2] = ACTIONS(1346),
    [anon_sym_BANG] = ACTIONS(1346),
    [anon_sym_TILDE] = ACTIONS(1346),
    [anon_sym_DASH] = ACTIONS(1344),
    [anon_sym_PLUS] = ACTIONS(1344),
    [anon_sym_STAR] = ACTIONS(1346),
    [anon_sym_AMP] = ACTIONS(1346),
    [anon_sym___extension__] = ACTIONS(1344),
    [anon_sym_typedef] = ACTIONS(1344),
    [anon_sym_extern] = ACTIONS(1344),
    [anon_sym___attribute__] = ACTIONS(1344),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1346),
    [anon_sym___declspec] = ACTIONS(1344),
    [anon_sym___cdecl] = ACTIONS(1344),
    [anon_sym___clrcall] = ACTIONS(1344),
    [anon_sym___stdcall] = ACTIONS(1344),
    [anon_sym___fastcall] = ACTIONS(1344),
    [anon_sym___thiscall] = ACTIONS(1344),
    [anon_sym___vectorcall] = ACTIONS(1344),
    [anon_sym_LBRACE] = ACTIONS(1346),
    [anon_sym_signed] = ACTIONS(1344),
    [anon_sym_unsigned] = ACTIONS(1344),
    [anon_sym_long] = ACTIONS(1344),
    [anon_sym_short] = ACTIONS(1344),
    [anon_sym_static] = ACTIONS(1344),
    [anon_sym_auto] = ACTIONS(1344),
    [anon_sym_register] = ACTIONS(1344),
    [anon_sym_inline] = ACTIONS(1344),
    [anon_sym___inline] = ACTIONS(1344),
    [anon_sym___inline__] = ACTIONS(1344),
    [anon_sym___forceinline] = ACTIONS(1344),
    [anon_sym_thread_local] = ACTIONS(1344),
    [anon_sym___thread] = ACTIONS(1344),
    [anon_sym_const] = ACTIONS(1344),
    [anon_sym_constexpr] = ACTIONS(1344),
    [anon_sym_volatile] = ACTIONS(1344),
    [anon_sym_restrict] = ACTIONS(1344),
    [anon_sym___restrict__] = ACTIONS(1344),
    [anon_sym__Atomic] = ACTIONS(1344),
    [anon_sym__Noreturn] = ACTIONS(1344),
    [anon_sym_noreturn] = ACTIONS(1344),
    [sym_primitive_type] = ACTIONS(1344),
    [anon_sym_enum] = ACTIONS(1344),
    [anon_sym_struct] = ACTIONS(1344),
    [anon_sym_union] = ACTIONS(1344),
    [anon_sym_if] = ACTIONS(1344),
    [anon_sym_switch] = ACTIONS(1344),
    [anon_sym_case] = ACTIONS(1344),
    [anon_sym_default] = ACTIONS(1344),
    [anon_sym_while] = ACTIONS(1344),
    [anon_sym_do] = ACTIONS(1344),
    [anon_sym_for] = ACTIONS(1344),
    [anon_sym_return] = ACTIONS(1344),
    [anon_sym_break] = ACTIONS(1344),
    [anon_sym_continue] = ACTIONS(1344),
    [anon_sym_goto] = ACTIONS(1344),
    [anon_sym_DASH_DASH] = ACTIONS(1346),
    [anon_sym_PLUS_PLUS] = ACTIONS(1346),
    [anon_sym_sizeof] = ACTIONS(1344),
    [anon_sym___alignof__] = ACTIONS(1344),
    [anon_sym___alignof] = ACTIONS(1344),
    [anon_sym__alignof] = ACTIONS(1344),
    [anon_sym_alignof] = ACTIONS(1344),
    [anon_sym__Alignof] = ACTIONS(1344),
    [anon_sym_offsetof] = ACTIONS(1344),
    [anon_sym__Generic] = ACTIONS(1344),
    [anon_sym_asm] = ACTIONS(1344),
    [anon_sym___asm__] = ACTIONS(1344),
    [sym_number_literal] = ACTIONS(1346),
    [anon_sym_L_SQUOTE] = ACTIONS(1346),
    [anon_sym_u_SQUOTE] = ACTIONS(1346),
    [anon_sym_U_SQUOTE] = ACTIONS(1346),
    [anon_sym_u8_SQUOTE] = ACTIONS(1346),
    [anon_sym_SQUOTE] = ACTIONS(1346),
    [anon_sym_L_DQUOTE] = ACTIONS(1346),
    [anon_sym_u_DQUOTE] = ACTIONS(1346),
    [anon_sym_U_DQUOTE] = ACTIONS(1346),
    [anon_sym_u8_DQUOTE] = ACTIONS(1346),
    [anon_sym_DQUOTE] = ACTIONS(1346),
    [sym_true] = ACTIONS(1344),
    [sym_false] = ACTIONS(1344),
    [anon_sym_NULL] = ACTIONS(1344),
    [anon_sym_nullptr] = ACTIONS(1344),
    [sym_comment] = ACTIONS(3),
  },
  [409] = {
    [ts_builtin_sym_end] = ACTIONS(1262),
    [sym_identifier] = ACTIONS(1260),
    [aux_sym_preproc_include_token1] = ACTIONS(1260),
    [aux_sym_preproc_def_token1] = ACTIONS(1260),
    [aux_sym_preproc_if_token1] = ACTIONS(1260),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1260),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1260),
    [sym_preproc_directive] = ACTIONS(1260),
    [anon_sym_LPAREN2] = ACTIONS(1262),
    [anon_sym_BANG] = ACTIONS(1262),
    [anon_sym_TILDE] = ACTIONS(1262),
    [anon_sym_DASH] = ACTIONS(1260),
    [anon_sym_PLUS] = ACTIONS(1260),
    [anon_sym_STAR] = ACTIONS(1262),
    [anon_sym_AMP] = ACTIONS(1262),
    [anon_sym___extension__] = ACTIONS(1260),
    [anon_sym_typedef] = ACTIONS(1260),
    [anon_sym_extern] = ACTIONS(1260),
    [anon_sym___attribute__] = ACTIONS(1260),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1262),
    [anon_sym___declspec] = ACTIONS(1260),
    [anon_sym___cdecl] = ACTIONS(1260),
    [anon_sym___clrcall] = ACTIONS(1260),
    [anon_sym___stdcall] = ACTIONS(1260),
    [anon_sym___fastcall] = ACTIONS(1260),
    [anon_sym___thiscall] = ACTIONS(1260),
    [anon_sym___vectorcall] = ACTIONS(1260),
    [anon_sym_LBRACE] = ACTIONS(1262),
    [anon_sym_signed] = ACTIONS(1260),
    [anon_sym_unsigned] = ACTIONS(1260),
    [anon_sym_long] = ACTIONS(1260),
    [anon_sym_short] = ACTIONS(1260),
    [anon_sym_static] = ACTIONS(1260),
    [anon_sym_auto] = ACTIONS(1260),
    [anon_sym_register] = ACTIONS(1260),
    [anon_sym_inline] = ACTIONS(1260),
    [anon_sym___inline] = ACTIONS(1260),
    [anon_sym___inline__] = ACTIONS(1260),
    [anon_sym___forceinline] = ACTIONS(1260),
    [anon_sym_thread_local] = ACTIONS(1260),
    [anon_sym___thread] = ACTIONS(1260),
    [anon_sym_const] = ACTIONS(1260),
    [anon_sym_constexpr] = ACTIONS(1260),
    [anon_sym_volatile] = ACTIONS(1260),
    [anon_sym_restrict] = ACTIONS(1260),
    [anon_sym___restrict__] = ACTIONS(1260),
    [anon_sym__Atomic] = ACTIONS(1260),
    [anon_sym__Noreturn] = ACTIONS(1260),
    [anon_sym_noreturn] = ACTIONS(1260),
    [sym_primitive_type] = ACTIONS(1260),
    [anon_sym_enum] = ACTIONS(1260),
    [anon_sym_struct] = ACTIONS(1260),
    [anon_sym_union] = ACTIONS(1260),
    [anon_sym_if] = ACTIONS(1260),
    [anon_sym_switch] = ACTIONS(1260),
    [anon_sym_case] = ACTIONS(1260),
    [anon_sym_default] = ACTIONS(1260),
    [anon_sym_while] = ACTIONS(1260),
    [anon_sym_do] = ACTIONS(1260),
    [anon_sym_for] = ACTIONS(1260),
    [anon_sym_return] = ACTIONS(1260),
    [anon_sym_break] = ACTIONS(1260),
    [anon_sym_continue] = ACTIONS(1260),
    [anon_sym_goto] = ACTIONS(1260),
    [anon_sym_DASH_DASH] = ACTIONS(1262),
    [anon_sym_PLUS_PLUS] = ACTIONS(1262),
    [anon_sym_sizeof] = ACTIONS(1260),
    [anon_sym___alignof__] = ACTIONS(1260),
    [anon_sym___alignof] = ACTIONS(1260),
    [anon_sym__alignof] = ACTIONS(1260),
    [anon_sym_alignof] = ACTIONS(1260),
    [anon_sym__Alignof] = ACTIONS(1260),
    [anon_sym_offsetof] = ACTIONS(1260),
    [anon_sym__Generic] = ACTIONS(1260),
    [anon_sym_asm] = ACTIONS(1260),
    [anon_sym___asm__] = ACTIONS(1260),
    [sym_number_literal] = ACTIONS(1262),
    [anon_sym_L_SQUOTE] = ACTIONS(1262),
    [anon_sym_u_SQUOTE] = ACTIONS(1262),
    [anon_sym_U_SQUOTE] = ACTIONS(1262),
    [anon_sym_u8_SQUOTE] = ACTIONS(1262),
    [anon_sym_SQUOTE] = ACTIONS(1262),
    [anon_sym_L_DQUOTE] = ACTIONS(1262),
    [anon_sym_u_DQUOTE] = ACTIONS(1262),
    [anon_sym_U_DQUOTE] = ACTIONS(1262),
    [anon_sym_u8_DQUOTE] = ACTIONS(1262),
    [anon_sym_DQUOTE] = ACTIONS(1262),
    [sym_true] = ACTIONS(1260),
    [sym_false] = ACTIONS(1260),
    [anon_sym_NULL] = ACTIONS(1260),
    [anon_sym_nullptr] = ACTIONS(1260),
    [sym_comment] = ACTIONS(3),
  },
  [410] = {
    [ts_builtin_sym_end] = ACTIONS(1266),
    [sym_identifier] = ACTIONS(1264),
    [aux_sym_preproc_include_token1] = ACTIONS(1264),
    [aux_sym_preproc_def_token1] = ACTIONS(1264),
    [aux_sym_preproc_if_token1] = ACTIONS(1264),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1264),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1264),
    [sym_preproc_directive] = ACTIONS(1264),
    [anon_sym_LPAREN2] = ACTIONS(1266),
    [anon_sym_BANG] = ACTIONS(1266),
    [anon_sym_TILDE] = ACTIONS(1266),
    [anon_sym_DASH] = ACTIONS(1264),
    [anon_sym_PLUS] = ACTIONS(1264),
    [anon_sym_STAR] = ACTIONS(1266),
    [anon_sym_AMP] = ACTIONS(1266),
    [anon_sym___extension__] = ACTIONS(1264),
    [anon_sym_typedef] = ACTIONS(1264),
    [anon_sym_extern] = ACTIONS(1264),
    [anon_sym___attribute__] = ACTIONS(1264),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1266),
    [anon_sym___declspec] = ACTIONS(1264),
    [anon_sym___cdecl] = ACTIONS(1264),
    [anon_sym___clrcall] = ACTIONS(1264),
    [anon_sym___stdcall] = ACTIONS(1264),
    [anon_sym___fastcall] = ACTIONS(1264),
    [anon_sym___thiscall] = ACTIONS(1264),
    [anon_sym___vectorcall] = ACTIONS(1264),
    [anon_sym_LBRACE] = ACTIONS(1266),
    [anon_sym_signed] = ACTIONS(1264),
    [anon_sym_unsigned] = ACTIONS(1264),
    [anon_sym_long] = ACTIONS(1264),
    [anon_sym_short] = ACTIONS(1264),
    [anon_sym_static] = ACTIONS(1264),
    [anon_sym_auto] = ACTIONS(1264),
    [anon_sym_register] = ACTIONS(1264),
    [anon_sym_inline] = ACTIONS(1264),
    [anon_sym___inline] = ACTIONS(1264),
    [anon_sym___inline__] = ACTIONS(1264),
    [anon_sym___forceinline] = ACTIONS(1264),
    [anon_sym_thread_local] = ACTIONS(1264),
    [anon_sym___thread] = ACTIONS(1264),
    [anon_sym_const] = ACTIONS(1264),
    [anon_sym_constexpr] = ACTIONS(1264),
    [anon_sym_volatile] = ACTIONS(1264),
    [anon_sym_restrict] = ACTIONS(1264),
    [anon_sym___restrict__] = ACTIONS(1264),
    [anon_sym__Atomic] = ACTIONS(1264),
    [anon_sym__Noreturn] = ACTIONS(1264),
    [anon_sym_noreturn] = ACTIONS(1264),
    [sym_primitive_type] = ACTIONS(1264),
    [anon_sym_enum] = ACTIONS(1264),
    [anon_sym_struct] = ACTIONS(1264),
    [anon_sym_union] = ACTIONS(1264),
    [anon_sym_if] = ACTIONS(1264),
    [anon_sym_switch] = ACTIONS(1264),
    [anon_sym_case] = ACTIONS(1264),
    [anon_sym_default] = ACTIONS(1264),
    [anon_sym_while] = ACTIONS(1264),
    [anon_sym_do] = ACTIONS(1264),
    [anon_sym_for] = ACTIONS(1264),
    [anon_sym_return] = ACTIONS(1264),
    [anon_sym_break] = ACTIONS(1264),
    [anon_sym_continue] = ACTIONS(1264),
    [anon_sym_goto] = ACTIONS(1264),
    [anon_sym_DASH_DASH] = ACTIONS(1266),
    [anon_sym_PLUS_PLUS] = ACTIONS(1266),
    [anon_sym_sizeof] = ACTIONS(1264),
    [anon_sym___alignof__] = ACTIONS(1264),
    [anon_sym___alignof] = ACTIONS(1264),
    [anon_sym__alignof] = ACTIONS(1264),
    [anon_sym_alignof] = ACTIONS(1264),
    [anon_sym__Alignof] = ACTIONS(1264),
    [anon_sym_offsetof] = ACTIONS(1264),
    [anon_sym__Generic] = ACTIONS(1264),
    [anon_sym_asm] = ACTIONS(1264),
    [anon_sym___asm__] = ACTIONS(1264),
    [sym_number_literal] = ACTIONS(1266),
    [anon_sym_L_SQUOTE] = ACTIONS(1266),
    [anon_sym_u_SQUOTE] = ACTIONS(1266),
    [anon_sym_U_SQUOTE] = ACTIONS(1266),
    [anon_sym_u8_SQUOTE] = ACTIONS(1266),
    [anon_sym_SQUOTE] = ACTIONS(1266),
    [anon_sym_L_DQUOTE] = ACTIONS(1266),
    [anon_sym_u_DQUOTE] = ACTIONS(1266),
    [anon_sym_U_DQUOTE] = ACTIONS(1266),
    [anon_sym_u8_DQUOTE] = ACTIONS(1266),
    [anon_sym_DQUOTE] = ACTIONS(1266),
    [sym_true] = ACTIONS(1264),
    [sym_false] = ACTIONS(1264),
    [anon_sym_NULL] = ACTIONS(1264),
    [anon_sym_nullptr] = ACTIONS(1264),
    [sym_comment] = ACTIONS(3),
  },
  [411] = {
    [ts_builtin_sym_end] = ACTIONS(1278),
    [sym_identifier] = ACTIONS(1276),
    [aux_sym_preproc_include_token1] = ACTIONS(1276),
    [aux_sym_preproc_def_token1] = ACTIONS(1276),
    [aux_sym_preproc_if_token1] = ACTIONS(1276),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1276),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1276),
    [sym_preproc_directive] = ACTIONS(1276),
    [anon_sym_LPAREN2] = ACTIONS(1278),
    [anon_sym_BANG] = ACTIONS(1278),
    [anon_sym_TILDE] = ACTIONS(1278),
    [anon_sym_DASH] = ACTIONS(1276),
    [anon_sym_PLUS] = ACTIONS(1276),
    [anon_sym_STAR] = ACTIONS(1278),
    [anon_sym_AMP] = ACTIONS(1278),
    [anon_sym___extension__] = ACTIONS(1276),
    [anon_sym_typedef] = ACTIONS(1276),
    [anon_sym_extern] = ACTIONS(1276),
    [anon_sym___attribute__] = ACTIONS(1276),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1278),
    [anon_sym___declspec] = ACTIONS(1276),
    [anon_sym___cdecl] = ACTIONS(1276),
    [anon_sym___clrcall] = ACTIONS(1276),
    [anon_sym___stdcall] = ACTIONS(1276),
    [anon_sym___fastcall] = ACTIONS(1276),
    [anon_sym___thiscall] = ACTIONS(1276),
    [anon_sym___vectorcall] = ACTIONS(1276),
    [anon_sym_LBRACE] = ACTIONS(1278),
    [anon_sym_signed] = ACTIONS(1276),
    [anon_sym_unsigned] = ACTIONS(1276),
    [anon_sym_long] = ACTIONS(1276),
    [anon_sym_short] = ACTIONS(1276),
    [anon_sym_static] = ACTIONS(1276),
    [anon_sym_auto] = ACTIONS(1276),
    [anon_sym_register] = ACTIONS(1276),
    [anon_sym_inline] = ACTIONS(1276),
    [anon_sym___inline] = ACTIONS(1276),
    [anon_sym___inline__] = ACTIONS(1276),
    [anon_sym___forceinline] = ACTIONS(1276),
    [anon_sym_thread_local] = ACTIONS(1276),
    [anon_sym___thread] = ACTIONS(1276),
    [anon_sym_const] = ACTIONS(1276),
    [anon_sym_constexpr] = ACTIONS(1276),
    [anon_sym_volatile] = ACTIONS(1276),
    [anon_sym_restrict] = ACTIONS(1276),
    [anon_sym___restrict__] = ACTIONS(1276),
    [anon_sym__Atomic] = ACTIONS(1276),
    [anon_sym__Noreturn] = ACTIONS(1276),
    [anon_sym_noreturn] = ACTIONS(1276),
    [sym_primitive_type] = ACTIONS(1276),
    [anon_sym_enum] = ACTIONS(1276),
    [anon_sym_struct] = ACTIONS(1276),
    [anon_sym_union] = ACTIONS(1276),
    [anon_sym_if] = ACTIONS(1276),
    [anon_sym_switch] = ACTIONS(1276),
    [anon_sym_case] = ACTIONS(1276),
    [anon_sym_default] = ACTIONS(1276),
    [anon_sym_while] = ACTIONS(1276),
    [anon_sym_do] = ACTIONS(1276),
    [anon_sym_for] = ACTIONS(1276),
    [anon_sym_return] = ACTIONS(1276),
    [anon_sym_break] = ACTIONS(1276),
    [anon_sym_continue] = ACTIONS(1276),
    [anon_sym_goto] = ACTIONS(1276),
    [anon_sym_DASH_DASH] = ACTIONS(1278),
    [anon_sym_PLUS_PLUS] = ACTIONS(1278),
    [anon_sym_sizeof] = ACTIONS(1276),
    [anon_sym___alignof__] = ACTIONS(1276),
    [anon_sym___alignof] = ACTIONS(1276),
    [anon_sym__alignof] = ACTIONS(1276),
    [anon_sym_alignof] = ACTIONS(1276),
    [anon_sym__Alignof] = ACTIONS(1276),
    [anon_sym_offsetof] = ACTIONS(1276),
    [anon_sym__Generic] = ACTIONS(1276),
    [anon_sym_asm] = ACTIONS(1276),
    [anon_sym___asm__] = ACTIONS(1276),
    [sym_number_literal] = ACTIONS(1278),
    [anon_sym_L_SQUOTE] = ACTIONS(1278),
    [anon_sym_u_SQUOTE] = ACTIONS(1278),
    [anon_sym_U_SQUOTE] = ACTIONS(1278),
    [anon_sym_u8_SQUOTE] = ACTIONS(1278),
    [anon_sym_SQUOTE] = ACTIONS(1278),
    [anon_sym_L_DQUOTE] = ACTIONS(1278),
    [anon_sym_u_DQUOTE] = ACTIONS(1278),
    [anon_sym_U_DQUOTE] = ACTIONS(1278),
    [anon_sym_u8_DQUOTE] = ACTIONS(1278),
    [anon_sym_DQUOTE] = ACTIONS(1278),
    [sym_true] = ACTIONS(1276),
    [sym_false] = ACTIONS(1276),
    [anon_sym_NULL] = ACTIONS(1276),
    [anon_sym_nullptr] = ACTIONS(1276),
    [sym_comment] = ACTIONS(3),
  },
  [412] = {
    [ts_builtin_sym_end] = ACTIONS(1254),
    [sym_identifier] = ACTIONS(1252),
    [aux_sym_preproc_include_token1] = ACTIONS(1252),
    [aux_sym_preproc_def_token1] = ACTIONS(1252),
    [aux_sym_preproc_if_token1] = ACTIONS(1252),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1252),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1252),
    [sym_preproc_directive] = ACTIONS(1252),
    [anon_sym_LPAREN2] = ACTIONS(1254),
    [anon_sym_BANG] = ACTIONS(1254),
    [anon_sym_TILDE] = ACTIONS(1254),
    [anon_sym_DASH] = ACTIONS(1252),
    [anon_sym_PLUS] = ACTIONS(1252),
    [anon_sym_STAR] = ACTIONS(1254),
    [anon_sym_AMP] = ACTIONS(1254),
    [anon_sym___extension__] = ACTIONS(1252),
    [anon_sym_typedef] = ACTIONS(1252),
    [anon_sym_extern] = ACTIONS(1252),
    [anon_sym___attribute__] = ACTIONS(1252),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1254),
    [anon_sym___declspec] = ACTIONS(1252),
    [anon_sym___cdecl] = ACTIONS(1252),
    [anon_sym___clrcall] = ACTIONS(1252),
    [anon_sym___stdcall] = ACTIONS(1252),
    [anon_sym___fastcall] = ACTIONS(1252),
    [anon_sym___thiscall] = ACTIONS(1252),
    [anon_sym___vectorcall] = ACTIONS(1252),
    [anon_sym_LBRACE] = ACTIONS(1254),
    [anon_sym_signed] = ACTIONS(1252),
    [anon_sym_unsigned] = ACTIONS(1252),
    [anon_sym_long] = ACTIONS(1252),
    [anon_sym_short] = ACTIONS(1252),
    [anon_sym_static] = ACTIONS(1252),
    [anon_sym_auto] = ACTIONS(1252),
    [anon_sym_register] = ACTIONS(1252),
    [anon_sym_inline] = ACTIONS(1252),
    [anon_sym___inline] = ACTIONS(1252),
    [anon_sym___inline__] = ACTIONS(1252),
    [anon_sym___forceinline] = ACTIONS(1252),
    [anon_sym_thread_local] = ACTIONS(1252),
    [anon_sym___thread] = ACTIONS(1252),
    [anon_sym_const] = ACTIONS(1252),
    [anon_sym_constexpr] = ACTIONS(1252),
    [anon_sym_volatile] = ACTIONS(1252),
    [anon_sym_restrict] = ACTIONS(1252),
    [anon_sym___restrict__] = ACTIONS(1252),
    [anon_sym__Atomic] = ACTIONS(1252),
    [anon_sym__Noreturn] = ACTIONS(1252),
    [anon_sym_noreturn] = ACTIONS(1252),
    [sym_primitive_type] = ACTIONS(1252),
    [anon_sym_enum] = ACTIONS(1252),
    [anon_sym_struct] = ACTIONS(1252),
    [anon_sym_union] = ACTIONS(1252),
    [anon_sym_if] = ACTIONS(1252),
    [anon_sym_switch] = ACTIONS(1252),
    [anon_sym_case] = ACTIONS(1252),
    [anon_sym_default] = ACTIONS(1252),
    [anon_sym_while] = ACTIONS(1252),
    [anon_sym_do] = ACTIONS(1252),
    [anon_sym_for] = ACTIONS(1252),
    [anon_sym_return] = ACTIONS(1252),
    [anon_sym_break] = ACTIONS(1252),
    [anon_sym_continue] = ACTIONS(1252),
    [anon_sym_goto] = ACTIONS(1252),
    [anon_sym_DASH_DASH] = ACTIONS(1254),
    [anon_sym_PLUS_PLUS] = ACTIONS(1254),
    [anon_sym_sizeof] = ACTIONS(1252),
    [anon_sym___alignof__] = ACTIONS(1252),
    [anon_sym___alignof] = ACTIONS(1252),
    [anon_sym__alignof] = ACTIONS(1252),
    [anon_sym_alignof] = ACTIONS(1252),
    [anon_sym__Alignof] = ACTIONS(1252),
    [anon_sym_offsetof] = ACTIONS(1252),
    [anon_sym__Generic] = ACTIONS(1252),
    [anon_sym_asm] = ACTIONS(1252),
    [anon_sym___asm__] = ACTIONS(1252),
    [sym_number_literal] = ACTIONS(1254),
    [anon_sym_L_SQUOTE] = ACTIONS(1254),
    [anon_sym_u_SQUOTE] = ACTIONS(1254),
    [anon_sym_U_SQUOTE] = ACTIONS(1254),
    [anon_sym_u8_SQUOTE] = ACTIONS(1254),
    [anon_sym_SQUOTE] = ACTIONS(1254),
    [anon_sym_L_DQUOTE] = ACTIONS(1254),
    [anon_sym_u_DQUOTE] = ACTIONS(1254),
    [anon_sym_U_DQUOTE] = ACTIONS(1254),
    [anon_sym_u8_DQUOTE] = ACTIONS(1254),
    [anon_sym_DQUOTE] = ACTIONS(1254),
    [sym_true] = ACTIONS(1252),
    [sym_false] = ACTIONS(1252),
    [anon_sym_NULL] = ACTIONS(1252),
    [anon_sym_nullptr] = ACTIONS(1252),
    [sym_comment] = ACTIONS(3),
  },
  [413] = {
    [ts_builtin_sym_end] = ACTIONS(1282),
    [sym_identifier] = ACTIONS(1280),
    [aux_sym_preproc_include_token1] = ACTIONS(1280),
    [aux_sym_preproc_def_token1] = ACTIONS(1280),
    [aux_sym_preproc_if_token1] = ACTIONS(1280),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1280),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1280),
    [sym_preproc_directive] = ACTIONS(1280),
    [anon_sym_LPAREN2] = ACTIONS(1282),
    [anon_sym_BANG] = ACTIONS(1282),
    [anon_sym_TILDE] = ACTIONS(1282),
    [anon_sym_DASH] = ACTIONS(1280),
    [anon_sym_PLUS] = ACTIONS(1280),
    [anon_sym_STAR] = ACTIONS(1282),
    [anon_sym_AMP] = ACTIONS(1282),
    [anon_sym___extension__] = ACTIONS(1280),
    [anon_sym_typedef] = ACTIONS(1280),
    [anon_sym_extern] = ACTIONS(1280),
    [anon_sym___attribute__] = ACTIONS(1280),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1282),
    [anon_sym___declspec] = ACTIONS(1280),
    [anon_sym___cdecl] = ACTIONS(1280),
    [anon_sym___clrcall] = ACTIONS(1280),
    [anon_sym___stdcall] = ACTIONS(1280),
    [anon_sym___fastcall] = ACTIONS(1280),
    [anon_sym___thiscall] = ACTIONS(1280),
    [anon_sym___vectorcall] = ACTIONS(1280),
    [anon_sym_LBRACE] = ACTIONS(1282),
    [anon_sym_signed] = ACTIONS(1280),
    [anon_sym_unsigned] = ACTIONS(1280),
    [anon_sym_long] = ACTIONS(1280),
    [anon_sym_short] = ACTIONS(1280),
    [anon_sym_static] = ACTIONS(1280),
    [anon_sym_auto] = ACTIONS(1280),
    [anon_sym_register] = ACTIONS(1280),
    [anon_sym_inline] = ACTIONS(1280),
    [anon_sym___inline] = ACTIONS(1280),
    [anon_sym___inline__] = ACTIONS(1280),
    [anon_sym___forceinline] = ACTIONS(1280),
    [anon_sym_thread_local] = ACTIONS(1280),
    [anon_sym___thread] = ACTIONS(1280),
    [anon_sym_const] = ACTIONS(1280),
    [anon_sym_constexpr] = ACTIONS(1280),
    [anon_sym_volatile] = ACTIONS(1280),
    [anon_sym_restrict] = ACTIONS(1280),
    [anon_sym___restrict__] = ACTIONS(1280),
    [anon_sym__Atomic] = ACTIONS(1280),
    [anon_sym__Noreturn] = ACTIONS(1280),
    [anon_sym_noreturn] = ACTIONS(1280),
    [sym_primitive_type] = ACTIONS(1280),
    [anon_sym_enum] = ACTIONS(1280),
    [anon_sym_struct] = ACTIONS(1280),
    [anon_sym_union] = ACTIONS(1280),
    [anon_sym_if] = ACTIONS(1280),
    [anon_sym_switch] = ACTIONS(1280),
    [anon_sym_case] = ACTIONS(1280),
    [anon_sym_default] = ACTIONS(1280),
    [anon_sym_while] = ACTIONS(1280),
    [anon_sym_do] = ACTIONS(1280),
    [anon_sym_for] = ACTIONS(1280),
    [anon_sym_return] = ACTIONS(1280),
    [anon_sym_break] = ACTIONS(1280),
    [anon_sym_continue] = ACTIONS(1280),
    [anon_sym_goto] = ACTIONS(1280),
    [anon_sym_DASH_DASH] = ACTIONS(1282),
    [anon_sym_PLUS_PLUS] = ACTIONS(1282),
    [anon_sym_sizeof] = ACTIONS(1280),
    [anon_sym___alignof__] = ACTIONS(1280),
    [anon_sym___alignof] = ACTIONS(1280),
    [anon_sym__alignof] = ACTIONS(1280),
    [anon_sym_alignof] = ACTIONS(1280),
    [anon_sym__Alignof] = ACTIONS(1280),
    [anon_sym_offsetof] = ACTIONS(1280),
    [anon_sym__Generic] = ACTIONS(1280),
    [anon_sym_asm] = ACTIONS(1280),
    [anon_sym___asm__] = ACTIONS(1280),
    [sym_number_literal] = ACTIONS(1282),
    [anon_sym_L_SQUOTE] = ACTIONS(1282),
    [anon_sym_u_SQUOTE] = ACTIONS(1282),
    [anon_sym_U_SQUOTE] = ACTIONS(1282),
    [anon_sym_u8_SQUOTE] = ACTIONS(1282),
    [anon_sym_SQUOTE] = ACTIONS(1282),
    [anon_sym_L_DQUOTE] = ACTIONS(1282),
    [anon_sym_u_DQUOTE] = ACTIONS(1282),
    [anon_sym_U_DQUOTE] = ACTIONS(1282),
    [anon_sym_u8_DQUOTE] = ACTIONS(1282),
    [anon_sym_DQUOTE] = ACTIONS(1282),
    [sym_true] = ACTIONS(1280),
    [sym_false] = ACTIONS(1280),
    [anon_sym_NULL] = ACTIONS(1280),
    [anon_sym_nullptr] = ACTIONS(1280),
    [sym_comment] = ACTIONS(3),
  },
  [414] = {
    [ts_builtin_sym_end] = ACTIONS(1270),
    [sym_identifier] = ACTIONS(1268),
    [aux_sym_preproc_include_token1] = ACTIONS(1268),
    [aux_sym_preproc_def_token1] = ACTIONS(1268),
    [aux_sym_preproc_if_token1] = ACTIONS(1268),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1268),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1268),
    [sym_preproc_directive] = ACTIONS(1268),
    [anon_sym_LPAREN2] = ACTIONS(1270),
    [anon_sym_BANG] = ACTIONS(1270),
    [anon_sym_TILDE] = ACTIONS(1270),
    [anon_sym_DASH] = ACTIONS(1268),
    [anon_sym_PLUS] = ACTIONS(1268),
    [anon_sym_STAR] = ACTIONS(1270),
    [anon_sym_AMP] = ACTIONS(1270),
    [anon_sym___extension__] = ACTIONS(1268),
    [anon_sym_typedef] = ACTIONS(1268),
    [anon_sym_extern] = ACTIONS(1268),
    [anon_sym___attribute__] = ACTIONS(1268),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1270),
    [anon_sym___declspec] = ACTIONS(1268),
    [anon_sym___cdecl] = ACTIONS(1268),
    [anon_sym___clrcall] = ACTIONS(1268),
    [anon_sym___stdcall] = ACTIONS(1268),
    [anon_sym___fastcall] = ACTIONS(1268),
    [anon_sym___thiscall] = ACTIONS(1268),
    [anon_sym___vectorcall] = ACTIONS(1268),
    [anon_sym_LBRACE] = ACTIONS(1270),
    [anon_sym_signed] = ACTIONS(1268),
    [anon_sym_unsigned] = ACTIONS(1268),
    [anon_sym_long] = ACTIONS(1268),
    [anon_sym_short] = ACTIONS(1268),
    [anon_sym_static] = ACTIONS(1268),
    [anon_sym_auto] = ACTIONS(1268),
    [anon_sym_register] = ACTIONS(1268),
    [anon_sym_inline] = ACTIONS(1268),
    [anon_sym___inline] = ACTIONS(1268),
    [anon_sym___inline__] = ACTIONS(1268),
    [anon_sym___forceinline] = ACTIONS(1268),
    [anon_sym_thread_local] = ACTIONS(1268),
    [anon_sym___thread] = ACTIONS(1268),
    [anon_sym_const] = ACTIONS(1268),
    [anon_sym_constexpr] = ACTIONS(1268),
    [anon_sym_volatile] = ACTIONS(1268),
    [anon_sym_restrict] = ACTIONS(1268),
    [anon_sym___restrict__] = ACTIONS(1268),
    [anon_sym__Atomic] = ACTIONS(1268),
    [anon_sym__Noreturn] = ACTIONS(1268),
    [anon_sym_noreturn] = ACTIONS(1268),
    [sym_primitive_type] = ACTIONS(1268),
    [anon_sym_enum] = ACTIONS(1268),
    [anon_sym_struct] = ACTIONS(1268),
    [anon_sym_union] = ACTIONS(1268),
    [anon_sym_if] = ACTIONS(1268),
    [anon_sym_switch] = ACTIONS(1268),
    [anon_sym_case] = ACTIONS(1268),
    [anon_sym_default] = ACTIONS(1268),
    [anon_sym_while] = ACTIONS(1268),
    [anon_sym_do] = ACTIONS(1268),
    [anon_sym_for] = ACTIONS(1268),
    [anon_sym_return] = ACTIONS(1268),
    [anon_sym_break] = ACTIONS(1268),
    [anon_sym_continue] = ACTIONS(1268),
    [anon_sym_goto] = ACTIONS(1268),
    [anon_sym_DASH_DASH] = ACTIONS(1270),
    [anon_sym_PLUS_PLUS] = ACTIONS(1270),
    [anon_sym_sizeof] = ACTIONS(1268),
    [anon_sym___alignof__] = ACTIONS(1268),
    [anon_sym___alignof] = ACTIONS(1268),
    [anon_sym__alignof] = ACTIONS(1268),
    [anon_sym_alignof] = ACTIONS(1268),
    [anon_sym__Alignof] = ACTIONS(1268),
    [anon_sym_offsetof] = ACTIONS(1268),
    [anon_sym__Generic] = ACTIONS(1268),
    [anon_sym_asm] = ACTIONS(1268),
    [anon_sym___asm__] = ACTIONS(1268),
    [sym_number_literal] = ACTIONS(1270),
    [anon_sym_L_SQUOTE] = ACTIONS(1270),
    [anon_sym_u_SQUOTE] = ACTIONS(1270),
    [anon_sym_U_SQUOTE] = ACTIONS(1270),
    [anon_sym_u8_SQUOTE] = ACTIONS(1270),
    [anon_sym_SQUOTE] = ACTIONS(1270),
    [anon_sym_L_DQUOTE] = ACTIONS(1270),
    [anon_sym_u_DQUOTE] = ACTIONS(1270),
    [anon_sym_U_DQUOTE] = ACTIONS(1270),
    [anon_sym_u8_DQUOTE] = ACTIONS(1270),
    [anon_sym_DQUOTE] = ACTIONS(1270),
    [sym_true] = ACTIONS(1268),
    [sym_false] = ACTIONS(1268),
    [anon_sym_NULL] = ACTIONS(1268),
    [anon_sym_nullptr] = ACTIONS(1268),
    [sym_comment] = ACTIONS(3),
  },
  [415] = {
    [ts_builtin_sym_end] = ACTIONS(1657),
    [sym_identifier] = ACTIONS(1659),
    [aux_sym_preproc_include_token1] = ACTIONS(1659),
    [aux_sym_preproc_def_token1] = ACTIONS(1659),
    [aux_sym_preproc_if_token1] = ACTIONS(1659),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1659),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1659),
    [sym_preproc_directive] = ACTIONS(1659),
    [anon_sym_LPAREN2] = ACTIONS(1657),
    [anon_sym_BANG] = ACTIONS(1657),
    [anon_sym_TILDE] = ACTIONS(1657),
    [anon_sym_DASH] = ACTIONS(1659),
    [anon_sym_PLUS] = ACTIONS(1659),
    [anon_sym_STAR] = ACTIONS(1657),
    [anon_sym_AMP] = ACTIONS(1657),
    [anon_sym___extension__] = ACTIONS(1659),
    [anon_sym_typedef] = ACTIONS(1659),
    [anon_sym_extern] = ACTIONS(1659),
    [anon_sym___attribute__] = ACTIONS(1659),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1657),
    [anon_sym___declspec] = ACTIONS(1659),
    [anon_sym___cdecl] = ACTIONS(1659),
    [anon_sym___clrcall] = ACTIONS(1659),
    [anon_sym___stdcall] = ACTIONS(1659),
    [anon_sym___fastcall] = ACTIONS(1659),
    [anon_sym___thiscall] = ACTIONS(1659),
    [anon_sym___vectorcall] = ACTIONS(1659),
    [anon_sym_LBRACE] = ACTIONS(1657),
    [anon_sym_signed] = ACTIONS(1659),
    [anon_sym_unsigned] = ACTIONS(1659),
    [anon_sym_long] = ACTIONS(1659),
    [anon_sym_short] = ACTIONS(1659),
    [anon_sym_static] = ACTIONS(1659),
    [anon_sym_auto] = ACTIONS(1659),
    [anon_sym_register] = ACTIONS(1659),
    [anon_sym_inline] = ACTIONS(1659),
    [anon_sym___inline] = ACTIONS(1659),
    [anon_sym___inline__] = ACTIONS(1659),
    [anon_sym___forceinline] = ACTIONS(1659),
    [anon_sym_thread_local] = ACTIONS(1659),
    [anon_sym___thread] = ACTIONS(1659),
    [anon_sym_const] = ACTIONS(1659),
    [anon_sym_constexpr] = ACTIONS(1659),
    [anon_sym_volatile] = ACTIONS(1659),
    [anon_sym_restrict] = ACTIONS(1659),
    [anon_sym___restrict__] = ACTIONS(1659),
    [anon_sym__Atomic] = ACTIONS(1659),
    [anon_sym__Noreturn] = ACTIONS(1659),
    [anon_sym_noreturn] = ACTIONS(1659),
    [sym_primitive_type] = ACTIONS(1659),
    [anon_sym_enum] = ACTIONS(1659),
    [anon_sym_struct] = ACTIONS(1659),
    [anon_sym_union] = ACTIONS(1659),
    [anon_sym_if] = ACTIONS(1659),
    [anon_sym_switch] = ACTIONS(1659),
    [anon_sym_case] = ACTIONS(1659),
    [anon_sym_default] = ACTIONS(1659),
    [anon_sym_while] = ACTIONS(1659),
    [anon_sym_do] = ACTIONS(1659),
    [anon_sym_for] = ACTIONS(1659),
    [anon_sym_return] = ACTIONS(1659),
    [anon_sym_break] = ACTIONS(1659),
    [anon_sym_continue] = ACTIONS(1659),
    [anon_sym_goto] = ACTIONS(1659),
    [anon_sym_DASH_DASH] = ACTIONS(1657),
    [anon_sym_PLUS_PLUS] = ACTIONS(1657),
    [anon_sym_sizeof] = ACTIONS(1659),
    [anon_sym___alignof__] = ACTIONS(1659),
    [anon_sym___alignof] = ACTIONS(1659),
    [anon_sym__alignof] = ACTIONS(1659),
    [anon_sym_alignof] = ACTIONS(1659),
    [anon_sym__Alignof] = ACTIONS(1659),
    [anon_sym_offsetof] = ACTIONS(1659),
    [anon_sym__Generic] = ACTIONS(1659),
    [anon_sym_asm] = ACTIONS(1659),
    [anon_sym___asm__] = ACTIONS(1659),
    [sym_number_literal] = ACTIONS(1657),
    [anon_sym_L_SQUOTE] = ACTIONS(1657),
    [anon_sym_u_SQUOTE] = ACTIONS(1657),
    [anon_sym_U_SQUOTE] = ACTIONS(1657),
    [anon_sym_u8_SQUOTE] = ACTIONS(1657),
    [anon_sym_SQUOTE] = ACTIONS(1657),
    [anon_sym_L_DQUOTE] = ACTIONS(1657),
    [anon_sym_u_DQUOTE] = ACTIONS(1657),
    [anon_sym_U_DQUOTE] = ACTIONS(1657),
    [anon_sym_u8_DQUOTE] = ACTIONS(1657),
    [anon_sym_DQUOTE] = ACTIONS(1657),
    [sym_true] = ACTIONS(1659),
    [sym_false] = ACTIONS(1659),
    [anon_sym_NULL] = ACTIONS(1659),
    [anon_sym_nullptr] = ACTIONS(1659),
    [sym_comment] = ACTIONS(3),
  },
  [416] = {
    [ts_builtin_sym_end] = ACTIONS(1661),
    [sym_identifier] = ACTIONS(1663),
    [aux_sym_preproc_include_token1] = ACTIONS(1663),
    [aux_sym_preproc_def_token1] = ACTIONS(1663),
    [aux_sym_preproc_if_token1] = ACTIONS(1663),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1663),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1663),
    [sym_preproc_directive] = ACTIONS(1663),
    [anon_sym_LPAREN2] = ACTIONS(1661),
    [anon_sym_BANG] = ACTIONS(1661),
    [anon_sym_TILDE] = ACTIONS(1661),
    [anon_sym_DASH] = ACTIONS(1663),
    [anon_sym_PLUS] = ACTIONS(1663),
    [anon_sym_STAR] = ACTIONS(1661),
    [anon_sym_AMP] = ACTIONS(1661),
    [anon_sym___extension__] = ACTIONS(1663),
    [anon_sym_typedef] = ACTIONS(1663),
    [anon_sym_extern] = ACTIONS(1663),
    [anon_sym___attribute__] = ACTIONS(1663),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1661),
    [anon_sym___declspec] = ACTIONS(1663),
    [anon_sym___cdecl] = ACTIONS(1663),
    [anon_sym___clrcall] = ACTIONS(1663),
    [anon_sym___stdcall] = ACTIONS(1663),
    [anon_sym___fastcall] = ACTIONS(1663),
    [anon_sym___thiscall] = ACTIONS(1663),
    [anon_sym___vectorcall] = ACTIONS(1663),
    [anon_sym_LBRACE] = ACTIONS(1661),
    [anon_sym_signed] = ACTIONS(1663),
    [anon_sym_unsigned] = ACTIONS(1663),
    [anon_sym_long] = ACTIONS(1663),
    [anon_sym_short] = ACTIONS(1663),
    [anon_sym_static] = ACTIONS(1663),
    [anon_sym_auto] = ACTIONS(1663),
    [anon_sym_register] = ACTIONS(1663),
    [anon_sym_inline] = ACTIONS(1663),
    [anon_sym___inline] = ACTIONS(1663),
    [anon_sym___inline__] = ACTIONS(1663),
    [anon_sym___forceinline] = ACTIONS(1663),
    [anon_sym_thread_local] = ACTIONS(1663),
    [anon_sym___thread] = ACTIONS(1663),
    [anon_sym_const] = ACTIONS(1663),
    [anon_sym_constexpr] = ACTIONS(1663),
    [anon_sym_volatile] = ACTIONS(1663),
    [anon_sym_restrict] = ACTIONS(1663),
    [anon_sym___restrict__] = ACTIONS(1663),
    [anon_sym__Atomic] = ACTIONS(1663),
    [anon_sym__Noreturn] = ACTIONS(1663),
    [anon_sym_noreturn] = ACTIONS(1663),
    [sym_primitive_type] = ACTIONS(1663),
    [anon_sym_enum] = ACTIONS(1663),
    [anon_sym_struct] = ACTIONS(1663),
    [anon_sym_union] = ACTIONS(1663),
    [anon_sym_if] = ACTIONS(1663),
    [anon_sym_switch] = ACTIONS(1663),
    [anon_sym_case] = ACTIONS(1663),
    [anon_sym_default] = ACTIONS(1663),
    [anon_sym_while] = ACTIONS(1663),
    [anon_sym_do] = ACTIONS(1663),
    [anon_sym_for] = ACTIONS(1663),
    [anon_sym_return] = ACTIONS(1663),
    [anon_sym_break] = ACTIONS(1663),
    [anon_sym_continue] = ACTIONS(1663),
    [anon_sym_goto] = ACTIONS(1663),
    [anon_sym_DASH_DASH] = ACTIONS(1661),
    [anon_sym_PLUS_PLUS] = ACTIONS(1661),
    [anon_sym_sizeof] = ACTIONS(1663),
    [anon_sym___alignof__] = ACTIONS(1663),
    [anon_sym___alignof] = ACTIONS(1663),
    [anon_sym__alignof] = ACTIONS(1663),
    [anon_sym_alignof] = ACTIONS(1663),
    [anon_sym__Alignof] = ACTIONS(1663),
    [anon_sym_offsetof] = ACTIONS(1663),
    [anon_sym__Generic] = ACTIONS(1663),
    [anon_sym_asm] = ACTIONS(1663),
    [anon_sym___asm__] = ACTIONS(1663),
    [sym_number_literal] = ACTIONS(1661),
    [anon_sym_L_SQUOTE] = ACTIONS(1661),
    [anon_sym_u_SQUOTE] = ACTIONS(1661),
    [anon_sym_U_SQUOTE] = ACTIONS(1661),
    [anon_sym_u8_SQUOTE] = ACTIONS(1661),
    [anon_sym_SQUOTE] = ACTIONS(1661),
    [anon_sym_L_DQUOTE] = ACTIONS(1661),
    [anon_sym_u_DQUOTE] = ACTIONS(1661),
    [anon_sym_U_DQUOTE] = ACTIONS(1661),
    [anon_sym_u8_DQUOTE] = ACTIONS(1661),
    [anon_sym_DQUOTE] = ACTIONS(1661),
    [sym_true] = ACTIONS(1663),
    [sym_false] = ACTIONS(1663),
    [anon_sym_NULL] = ACTIONS(1663),
    [anon_sym_nullptr] = ACTIONS(1663),
    [sym_comment] = ACTIONS(3),
  },
  [417] = {
    [ts_builtin_sym_end] = ACTIONS(1290),
    [sym_identifier] = ACTIONS(1288),
    [aux_sym_preproc_include_token1] = ACTIONS(1288),
    [aux_sym_preproc_def_token1] = ACTIONS(1288),
    [aux_sym_preproc_if_token1] = ACTIONS(1288),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1288),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1288),
    [sym_preproc_directive] = ACTIONS(1288),
    [anon_sym_LPAREN2] = ACTIONS(1290),
    [anon_sym_BANG] = ACTIONS(1290),
    [anon_sym_TILDE] = ACTIONS(1290),
    [anon_sym_DASH] = ACTIONS(1288),
    [anon_sym_PLUS] = ACTIONS(1288),
    [anon_sym_STAR] = ACTIONS(1290),
    [anon_sym_AMP] = ACTIONS(1290),
    [anon_sym___extension__] = ACTIONS(1288),
    [anon_sym_typedef] = ACTIONS(1288),
    [anon_sym_extern] = ACTIONS(1288),
    [anon_sym___attribute__] = ACTIONS(1288),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1290),
    [anon_sym___declspec] = ACTIONS(1288),
    [anon_sym___cdecl] = ACTIONS(1288),
    [anon_sym___clrcall] = ACTIONS(1288),
    [anon_sym___stdcall] = ACTIONS(1288),
    [anon_sym___fastcall] = ACTIONS(1288),
    [anon_sym___thiscall] = ACTIONS(1288),
    [anon_sym___vectorcall] = ACTIONS(1288),
    [anon_sym_LBRACE] = ACTIONS(1290),
    [anon_sym_signed] = ACTIONS(1288),
    [anon_sym_unsigned] = ACTIONS(1288),
    [anon_sym_long] = ACTIONS(1288),
    [anon_sym_short] = ACTIONS(1288),
    [anon_sym_static] = ACTIONS(1288),
    [anon_sym_auto] = ACTIONS(1288),
    [anon_sym_register] = ACTIONS(1288),
    [anon_sym_inline] = ACTIONS(1288),
    [anon_sym___inline] = ACTIONS(1288),
    [anon_sym___inline__] = ACTIONS(1288),
    [anon_sym___forceinline] = ACTIONS(1288),
    [anon_sym_thread_local] = ACTIONS(1288),
    [anon_sym___thread] = ACTIONS(1288),
    [anon_sym_const] = ACTIONS(1288),
    [anon_sym_constexpr] = ACTIONS(1288),
    [anon_sym_volatile] = ACTIONS(1288),
    [anon_sym_restrict] = ACTIONS(1288),
    [anon_sym___restrict__] = ACTIONS(1288),
    [anon_sym__Atomic] = ACTIONS(1288),
    [anon_sym__Noreturn] = ACTIONS(1288),
    [anon_sym_noreturn] = ACTIONS(1288),
    [sym_primitive_type] = ACTIONS(1288),
    [anon_sym_enum] = ACTIONS(1288),
    [anon_sym_struct] = ACTIONS(1288),
    [anon_sym_union] = ACTIONS(1288),
    [anon_sym_if] = ACTIONS(1288),
    [anon_sym_switch] = ACTIONS(1288),
    [anon_sym_case] = ACTIONS(1288),
    [anon_sym_default] = ACTIONS(1288),
    [anon_sym_while] = ACTIONS(1288),
    [anon_sym_do] = ACTIONS(1288),
    [anon_sym_for] = ACTIONS(1288),
    [anon_sym_return] = ACTIONS(1288),
    [anon_sym_break] = ACTIONS(1288),
    [anon_sym_continue] = ACTIONS(1288),
    [anon_sym_goto] = ACTIONS(1288),
    [anon_sym_DASH_DASH] = ACTIONS(1290),
    [anon_sym_PLUS_PLUS] = ACTIONS(1290),
    [anon_sym_sizeof] = ACTIONS(1288),
    [anon_sym___alignof__] = ACTIONS(1288),
    [anon_sym___alignof] = ACTIONS(1288),
    [anon_sym__alignof] = ACTIONS(1288),
    [anon_sym_alignof] = ACTIONS(1288),
    [anon_sym__Alignof] = ACTIONS(1288),
    [anon_sym_offsetof] = ACTIONS(1288),
    [anon_sym__Generic] = ACTIONS(1288),
    [anon_sym_asm] = ACTIONS(1288),
    [anon_sym___asm__] = ACTIONS(1288),
    [sym_number_literal] = ACTIONS(1290),
    [anon_sym_L_SQUOTE] = ACTIONS(1290),
    [anon_sym_u_SQUOTE] = ACTIONS(1290),
    [anon_sym_U_SQUOTE] = ACTIONS(1290),
    [anon_sym_u8_SQUOTE] = ACTIONS(1290),
    [anon_sym_SQUOTE] = ACTIONS(1290),
    [anon_sym_L_DQUOTE] = ACTIONS(1290),
    [anon_sym_u_DQUOTE] = ACTIONS(1290),
    [anon_sym_U_DQUOTE] = ACTIONS(1290),
    [anon_sym_u8_DQUOTE] = ACTIONS(1290),
    [anon_sym_DQUOTE] = ACTIONS(1290),
    [sym_true] = ACTIONS(1288),
    [sym_false] = ACTIONS(1288),
    [anon_sym_NULL] = ACTIONS(1288),
    [anon_sym_nullptr] = ACTIONS(1288),
    [sym_comment] = ACTIONS(3),
  },
  [418] = {
    [ts_builtin_sym_end] = ACTIONS(1274),
    [sym_identifier] = ACTIONS(1272),
    [aux_sym_preproc_include_token1] = ACTIONS(1272),
    [aux_sym_preproc_def_token1] = ACTIONS(1272),
    [aux_sym_preproc_if_token1] = ACTIONS(1272),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1272),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1272),
    [sym_preproc_directive] = ACTIONS(1272),
    [anon_sym_LPAREN2] = ACTIONS(1274),
    [anon_sym_BANG] = ACTIONS(1274),
    [anon_sym_TILDE] = ACTIONS(1274),
    [anon_sym_DASH] = ACTIONS(1272),
    [anon_sym_PLUS] = ACTIONS(1272),
    [anon_sym_STAR] = ACTIONS(1274),
    [anon_sym_AMP] = ACTIONS(1274),
    [anon_sym___extension__] = ACTIONS(1272),
    [anon_sym_typedef] = ACTIONS(1272),
    [anon_sym_extern] = ACTIONS(1272),
    [anon_sym___attribute__] = ACTIONS(1272),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1274),
    [anon_sym___declspec] = ACTIONS(1272),
    [anon_sym___cdecl] = ACTIONS(1272),
    [anon_sym___clrcall] = ACTIONS(1272),
    [anon_sym___stdcall] = ACTIONS(1272),
    [anon_sym___fastcall] = ACTIONS(1272),
    [anon_sym___thiscall] = ACTIONS(1272),
    [anon_sym___vectorcall] = ACTIONS(1272),
    [anon_sym_LBRACE] = ACTIONS(1274),
    [anon_sym_signed] = ACTIONS(1272),
    [anon_sym_unsigned] = ACTIONS(1272),
    [anon_sym_long] = ACTIONS(1272),
    [anon_sym_short] = ACTIONS(1272),
    [anon_sym_static] = ACTIONS(1272),
    [anon_sym_auto] = ACTIONS(1272),
    [anon_sym_register] = ACTIONS(1272),
    [anon_sym_inline] = ACTIONS(1272),
    [anon_sym___inline] = ACTIONS(1272),
    [anon_sym___inline__] = ACTIONS(1272),
    [anon_sym___forceinline] = ACTIONS(1272),
    [anon_sym_thread_local] = ACTIONS(1272),
    [anon_sym___thread] = ACTIONS(1272),
    [anon_sym_const] = ACTIONS(1272),
    [anon_sym_constexpr] = ACTIONS(1272),
    [anon_sym_volatile] = ACTIONS(1272),
    [anon_sym_restrict] = ACTIONS(1272),
    [anon_sym___restrict__] = ACTIONS(1272),
    [anon_sym__Atomic] = ACTIONS(1272),
    [anon_sym__Noreturn] = ACTIONS(1272),
    [anon_sym_noreturn] = ACTIONS(1272),
    [sym_primitive_type] = ACTIONS(1272),
    [anon_sym_enum] = ACTIONS(1272),
    [anon_sym_struct] = ACTIONS(1272),
    [anon_sym_union] = ACTIONS(1272),
    [anon_sym_if] = ACTIONS(1272),
    [anon_sym_switch] = ACTIONS(1272),
    [anon_sym_case] = ACTIONS(1272),
    [anon_sym_default] = ACTIONS(1272),
    [anon_sym_while] = ACTIONS(1272),
    [anon_sym_do] = ACTIONS(1272),
    [anon_sym_for] = ACTIONS(1272),
    [anon_sym_return] = ACTIONS(1272),
    [anon_sym_break] = ACTIONS(1272),
    [anon_sym_continue] = ACTIONS(1272),
    [anon_sym_goto] = ACTIONS(1272),
    [anon_sym_DASH_DASH] = ACTIONS(1274),
    [anon_sym_PLUS_PLUS] = ACTIONS(1274),
    [anon_sym_sizeof] = ACTIONS(1272),
    [anon_sym___alignof__] = ACTIONS(1272),
    [anon_sym___alignof] = ACTIONS(1272),
    [anon_sym__alignof] = ACTIONS(1272),
    [anon_sym_alignof] = ACTIONS(1272),
    [anon_sym__Alignof] = ACTIONS(1272),
    [anon_sym_offsetof] = ACTIONS(1272),
    [anon_sym__Generic] = ACTIONS(1272),
    [anon_sym_asm] = ACTIONS(1272),
    [anon_sym___asm__] = ACTIONS(1272),
    [sym_number_literal] = ACTIONS(1274),
    [anon_sym_L_SQUOTE] = ACTIONS(1274),
    [anon_sym_u_SQUOTE] = ACTIONS(1274),
    [anon_sym_U_SQUOTE] = ACTIONS(1274),
    [anon_sym_u8_SQUOTE] = ACTIONS(1274),
    [anon_sym_SQUOTE] = ACTIONS(1274),
    [anon_sym_L_DQUOTE] = ACTIONS(1274),
    [anon_sym_u_DQUOTE] = ACTIONS(1274),
    [anon_sym_U_DQUOTE] = ACTIONS(1274),
    [anon_sym_u8_DQUOTE] = ACTIONS(1274),
    [anon_sym_DQUOTE] = ACTIONS(1274),
    [sym_true] = ACTIONS(1272),
    [sym_false] = ACTIONS(1272),
    [anon_sym_NULL] = ACTIONS(1272),
    [anon_sym_nullptr] = ACTIONS(1272),
    [sym_comment] = ACTIONS(3),
  },
  [419] = {
    [ts_builtin_sym_end] = ACTIONS(1322),
    [sym_identifier] = ACTIONS(1320),
    [aux_sym_preproc_include_token1] = ACTIONS(1320),
    [aux_sym_preproc_def_token1] = ACTIONS(1320),
    [aux_sym_preproc_if_token1] = ACTIONS(1320),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1320),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1320),
    [sym_preproc_directive] = ACTIONS(1320),
    [anon_sym_LPAREN2] = ACTIONS(1322),
    [anon_sym_BANG] = ACTIONS(1322),
    [anon_sym_TILDE] = ACTIONS(1322),
    [anon_sym_DASH] = ACTIONS(1320),
    [anon_sym_PLUS] = ACTIONS(1320),
    [anon_sym_STAR] = ACTIONS(1322),
    [anon_sym_AMP] = ACTIONS(1322),
    [anon_sym___extension__] = ACTIONS(1320),
    [anon_sym_typedef] = ACTIONS(1320),
    [anon_sym_extern] = ACTIONS(1320),
    [anon_sym___attribute__] = ACTIONS(1320),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1322),
    [anon_sym___declspec] = ACTIONS(1320),
    [anon_sym___cdecl] = ACTIONS(1320),
    [anon_sym___clrcall] = ACTIONS(1320),
    [anon_sym___stdcall] = ACTIONS(1320),
    [anon_sym___fastcall] = ACTIONS(1320),
    [anon_sym___thiscall] = ACTIONS(1320),
    [anon_sym___vectorcall] = ACTIONS(1320),
    [anon_sym_LBRACE] = ACTIONS(1322),
    [anon_sym_signed] = ACTIONS(1320),
    [anon_sym_unsigned] = ACTIONS(1320),
    [anon_sym_long] = ACTIONS(1320),
    [anon_sym_short] = ACTIONS(1320),
    [anon_sym_static] = ACTIONS(1320),
    [anon_sym_auto] = ACTIONS(1320),
    [anon_sym_register] = ACTIONS(1320),
    [anon_sym_inline] = ACTIONS(1320),
    [anon_sym___inline] = ACTIONS(1320),
    [anon_sym___inline__] = ACTIONS(1320),
    [anon_sym___forceinline] = ACTIONS(1320),
    [anon_sym_thread_local] = ACTIONS(1320),
    [anon_sym___thread] = ACTIONS(1320),
    [anon_sym_const] = ACTIONS(1320),
    [anon_sym_constexpr] = ACTIONS(1320),
    [anon_sym_volatile] = ACTIONS(1320),
    [anon_sym_restrict] = ACTIONS(1320),
    [anon_sym___restrict__] = ACTIONS(1320),
    [anon_sym__Atomic] = ACTIONS(1320),
    [anon_sym__Noreturn] = ACTIONS(1320),
    [anon_sym_noreturn] = ACTIONS(1320),
    [sym_primitive_type] = ACTIONS(1320),
    [anon_sym_enum] = ACTIONS(1320),
    [anon_sym_struct] = ACTIONS(1320),
    [anon_sym_union] = ACTIONS(1320),
    [anon_sym_if] = ACTIONS(1320),
    [anon_sym_switch] = ACTIONS(1320),
    [anon_sym_case] = ACTIONS(1320),
    [anon_sym_default] = ACTIONS(1320),
    [anon_sym_while] = ACTIONS(1320),
    [anon_sym_do] = ACTIONS(1320),
    [anon_sym_for] = ACTIONS(1320),
    [anon_sym_return] = ACTIONS(1320),
    [anon_sym_break] = ACTIONS(1320),
    [anon_sym_continue] = ACTIONS(1320),
    [anon_sym_goto] = ACTIONS(1320),
    [anon_sym_DASH_DASH] = ACTIONS(1322),
    [anon_sym_PLUS_PLUS] = ACTIONS(1322),
    [anon_sym_sizeof] = ACTIONS(1320),
    [anon_sym___alignof__] = ACTIONS(1320),
    [anon_sym___alignof] = ACTIONS(1320),
    [anon_sym__alignof] = ACTIONS(1320),
    [anon_sym_alignof] = ACTIONS(1320),
    [anon_sym__Alignof] = ACTIONS(1320),
    [anon_sym_offsetof] = ACTIONS(1320),
    [anon_sym__Generic] = ACTIONS(1320),
    [anon_sym_asm] = ACTIONS(1320),
    [anon_sym___asm__] = ACTIONS(1320),
    [sym_number_literal] = ACTIONS(1322),
    [anon_sym_L_SQUOTE] = ACTIONS(1322),
    [anon_sym_u_SQUOTE] = ACTIONS(1322),
    [anon_sym_U_SQUOTE] = ACTIONS(1322),
    [anon_sym_u8_SQUOTE] = ACTIONS(1322),
    [anon_sym_SQUOTE] = ACTIONS(1322),
    [anon_sym_L_DQUOTE] = ACTIONS(1322),
    [anon_sym_u_DQUOTE] = ACTIONS(1322),
    [anon_sym_U_DQUOTE] = ACTIONS(1322),
    [anon_sym_u8_DQUOTE] = ACTIONS(1322),
    [anon_sym_DQUOTE] = ACTIONS(1322),
    [sym_true] = ACTIONS(1320),
    [sym_false] = ACTIONS(1320),
    [anon_sym_NULL] = ACTIONS(1320),
    [anon_sym_nullptr] = ACTIONS(1320),
    [sym_comment] = ACTIONS(3),
  },
  [420] = {
    [ts_builtin_sym_end] = ACTIONS(1310),
    [sym_identifier] = ACTIONS(1308),
    [aux_sym_preproc_include_token1] = ACTIONS(1308),
    [aux_sym_preproc_def_token1] = ACTIONS(1308),
    [aux_sym_preproc_if_token1] = ACTIONS(1308),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1308),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1308),
    [sym_preproc_directive] = ACTIONS(1308),
    [anon_sym_LPAREN2] = ACTIONS(1310),
    [anon_sym_BANG] = ACTIONS(1310),
    [anon_sym_TILDE] = ACTIONS(1310),
    [anon_sym_DASH] = ACTIONS(1308),
    [anon_sym_PLUS] = ACTIONS(1308),
    [anon_sym_STAR] = ACTIONS(1310),
    [anon_sym_AMP] = ACTIONS(1310),
    [anon_sym___extension__] = ACTIONS(1308),
    [anon_sym_typedef] = ACTIONS(1308),
    [anon_sym_extern] = ACTIONS(1308),
    [anon_sym___attribute__] = ACTIONS(1308),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1310),
    [anon_sym___declspec] = ACTIONS(1308),
    [anon_sym___cdecl] = ACTIONS(1308),
    [anon_sym___clrcall] = ACTIONS(1308),
    [anon_sym___stdcall] = ACTIONS(1308),
    [anon_sym___fastcall] = ACTIONS(1308),
    [anon_sym___thiscall] = ACTIONS(1308),
    [anon_sym___vectorcall] = ACTIONS(1308),
    [anon_sym_LBRACE] = ACTIONS(1310),
    [anon_sym_signed] = ACTIONS(1308),
    [anon_sym_unsigned] = ACTIONS(1308),
    [anon_sym_long] = ACTIONS(1308),
    [anon_sym_short] = ACTIONS(1308),
    [anon_sym_static] = ACTIONS(1308),
    [anon_sym_auto] = ACTIONS(1308),
    [anon_sym_register] = ACTIONS(1308),
    [anon_sym_inline] = ACTIONS(1308),
    [anon_sym___inline] = ACTIONS(1308),
    [anon_sym___inline__] = ACTIONS(1308),
    [anon_sym___forceinline] = ACTIONS(1308),
    [anon_sym_thread_local] = ACTIONS(1308),
    [anon_sym___thread] = ACTIONS(1308),
    [anon_sym_const] = ACTIONS(1308),
    [anon_sym_constexpr] = ACTIONS(1308),
    [anon_sym_volatile] = ACTIONS(1308),
    [anon_sym_restrict] = ACTIONS(1308),
    [anon_sym___restrict__] = ACTIONS(1308),
    [anon_sym__Atomic] = ACTIONS(1308),
    [anon_sym__Noreturn] = ACTIONS(1308),
    [anon_sym_noreturn] = ACTIONS(1308),
    [sym_primitive_type] = ACTIONS(1308),
    [anon_sym_enum] = ACTIONS(1308),
    [anon_sym_struct] = ACTIONS(1308),
    [anon_sym_union] = ACTIONS(1308),
    [anon_sym_if] = ACTIONS(1308),
    [anon_sym_switch] = ACTIONS(1308),
    [anon_sym_case] = ACTIONS(1308),
    [anon_sym_default] = ACTIONS(1308),
    [anon_sym_while] = ACTIONS(1308),
    [anon_sym_do] = ACTIONS(1308),
    [anon_sym_for] = ACTIONS(1308),
    [anon_sym_return] = ACTIONS(1308),
    [anon_sym_break] = ACTIONS(1308),
    [anon_sym_continue] = ACTIONS(1308),
    [anon_sym_goto] = ACTIONS(1308),
    [anon_sym_DASH_DASH] = ACTIONS(1310),
    [anon_sym_PLUS_PLUS] = ACTIONS(1310),
    [anon_sym_sizeof] = ACTIONS(1308),
    [anon_sym___alignof__] = ACTIONS(1308),
    [anon_sym___alignof] = ACTIONS(1308),
    [anon_sym__alignof] = ACTIONS(1308),
    [anon_sym_alignof] = ACTIONS(1308),
    [anon_sym__Alignof] = ACTIONS(1308),
    [anon_sym_offsetof] = ACTIONS(1308),
    [anon_sym__Generic] = ACTIONS(1308),
    [anon_sym_asm] = ACTIONS(1308),
    [anon_sym___asm__] = ACTIONS(1308),
    [sym_number_literal] = ACTIONS(1310),
    [anon_sym_L_SQUOTE] = ACTIONS(1310),
    [anon_sym_u_SQUOTE] = ACTIONS(1310),
    [anon_sym_U_SQUOTE] = ACTIONS(1310),
    [anon_sym_u8_SQUOTE] = ACTIONS(1310),
    [anon_sym_SQUOTE] = ACTIONS(1310),
    [anon_sym_L_DQUOTE] = ACTIONS(1310),
    [anon_sym_u_DQUOTE] = ACTIONS(1310),
    [anon_sym_U_DQUOTE] = ACTIONS(1310),
    [anon_sym_u8_DQUOTE] = ACTIONS(1310),
    [anon_sym_DQUOTE] = ACTIONS(1310),
    [sym_true] = ACTIONS(1308),
    [sym_false] = ACTIONS(1308),
    [anon_sym_NULL] = ACTIONS(1308),
    [anon_sym_nullptr] = ACTIONS(1308),
    [sym_comment] = ACTIONS(3),
  },
  [421] = {
    [ts_builtin_sym_end] = ACTIONS(1334),
    [sym_identifier] = ACTIONS(1332),
    [aux_sym_preproc_include_token1] = ACTIONS(1332),
    [aux_sym_preproc_def_token1] = ACTIONS(1332),
    [aux_sym_preproc_if_token1] = ACTIONS(1332),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1332),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1332),
    [sym_preproc_directive] = ACTIONS(1332),
    [anon_sym_LPAREN2] = ACTIONS(1334),
    [anon_sym_BANG] = ACTIONS(1334),
    [anon_sym_TILDE] = ACTIONS(1334),
    [anon_sym_DASH] = ACTIONS(1332),
    [anon_sym_PLUS] = ACTIONS(1332),
    [anon_sym_STAR] = ACTIONS(1334),
    [anon_sym_AMP] = ACTIONS(1334),
    [anon_sym___extension__] = ACTIONS(1332),
    [anon_sym_typedef] = ACTIONS(1332),
    [anon_sym_extern] = ACTIONS(1332),
    [anon_sym___attribute__] = ACTIONS(1332),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1334),
    [anon_sym___declspec] = ACTIONS(1332),
    [anon_sym___cdecl] = ACTIONS(1332),
    [anon_sym___clrcall] = ACTIONS(1332),
    [anon_sym___stdcall] = ACTIONS(1332),
    [anon_sym___fastcall] = ACTIONS(1332),
    [anon_sym___thiscall] = ACTIONS(1332),
    [anon_sym___vectorcall] = ACTIONS(1332),
    [anon_sym_LBRACE] = ACTIONS(1334),
    [anon_sym_signed] = ACTIONS(1332),
    [anon_sym_unsigned] = ACTIONS(1332),
    [anon_sym_long] = ACTIONS(1332),
    [anon_sym_short] = ACTIONS(1332),
    [anon_sym_static] = ACTIONS(1332),
    [anon_sym_auto] = ACTIONS(1332),
    [anon_sym_register] = ACTIONS(1332),
    [anon_sym_inline] = ACTIONS(1332),
    [anon_sym___inline] = ACTIONS(1332),
    [anon_sym___inline__] = ACTIONS(1332),
    [anon_sym___forceinline] = ACTIONS(1332),
    [anon_sym_thread_local] = ACTIONS(1332),
    [anon_sym___thread] = ACTIONS(1332),
    [anon_sym_const] = ACTIONS(1332),
    [anon_sym_constexpr] = ACTIONS(1332),
    [anon_sym_volatile] = ACTIONS(1332),
    [anon_sym_restrict] = ACTIONS(1332),
    [anon_sym___restrict__] = ACTIONS(1332),
    [anon_sym__Atomic] = ACTIONS(1332),
    [anon_sym__Noreturn] = ACTIONS(1332),
    [anon_sym_noreturn] = ACTIONS(1332),
    [sym_primitive_type] = ACTIONS(1332),
    [anon_sym_enum] = ACTIONS(1332),
    [anon_sym_struct] = ACTIONS(1332),
    [anon_sym_union] = ACTIONS(1332),
    [anon_sym_if] = ACTIONS(1332),
    [anon_sym_switch] = ACTIONS(1332),
    [anon_sym_case] = ACTIONS(1332),
    [anon_sym_default] = ACTIONS(1332),
    [anon_sym_while] = ACTIONS(1332),
    [anon_sym_do] = ACTIONS(1332),
    [anon_sym_for] = ACTIONS(1332),
    [anon_sym_return] = ACTIONS(1332),
    [anon_sym_break] = ACTIONS(1332),
    [anon_sym_continue] = ACTIONS(1332),
    [anon_sym_goto] = ACTIONS(1332),
    [anon_sym_DASH_DASH] = ACTIONS(1334),
    [anon_sym_PLUS_PLUS] = ACTIONS(1334),
    [anon_sym_sizeof] = ACTIONS(1332),
    [anon_sym___alignof__] = ACTIONS(1332),
    [anon_sym___alignof] = ACTIONS(1332),
    [anon_sym__alignof] = ACTIONS(1332),
    [anon_sym_alignof] = ACTIONS(1332),
    [anon_sym__Alignof] = ACTIONS(1332),
    [anon_sym_offsetof] = ACTIONS(1332),
    [anon_sym__Generic] = ACTIONS(1332),
    [anon_sym_asm] = ACTIONS(1332),
    [anon_sym___asm__] = ACTIONS(1332),
    [sym_number_literal] = ACTIONS(1334),
    [anon_sym_L_SQUOTE] = ACTIONS(1334),
    [anon_sym_u_SQUOTE] = ACTIONS(1334),
    [anon_sym_U_SQUOTE] = ACTIONS(1334),
    [anon_sym_u8_SQUOTE] = ACTIONS(1334),
    [anon_sym_SQUOTE] = ACTIONS(1334),
    [anon_sym_L_DQUOTE] = ACTIONS(1334),
    [anon_sym_u_DQUOTE] = ACTIONS(1334),
    [anon_sym_U_DQUOTE] = ACTIONS(1334),
    [anon_sym_u8_DQUOTE] = ACTIONS(1334),
    [anon_sym_DQUOTE] = ACTIONS(1334),
    [sym_true] = ACTIONS(1332),
    [sym_false] = ACTIONS(1332),
    [anon_sym_NULL] = ACTIONS(1332),
    [anon_sym_nullptr] = ACTIONS(1332),
    [sym_comment] = ACTIONS(3),
  },
  [422] = {
    [ts_builtin_sym_end] = ACTIONS(1298),
    [sym_identifier] = ACTIONS(1296),
    [aux_sym_preproc_include_token1] = ACTIONS(1296),
    [aux_sym_preproc_def_token1] = ACTIONS(1296),
    [aux_sym_preproc_if_token1] = ACTIONS(1296),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1296),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1296),
    [sym_preproc_directive] = ACTIONS(1296),
    [anon_sym_LPAREN2] = ACTIONS(1298),
    [anon_sym_BANG] = ACTIONS(1298),
    [anon_sym_TILDE] = ACTIONS(1298),
    [anon_sym_DASH] = ACTIONS(1296),
    [anon_sym_PLUS] = ACTIONS(1296),
    [anon_sym_STAR] = ACTIONS(1298),
    [anon_sym_AMP] = ACTIONS(1298),
    [anon_sym___extension__] = ACTIONS(1296),
    [anon_sym_typedef] = ACTIONS(1296),
    [anon_sym_extern] = ACTIONS(1296),
    [anon_sym___attribute__] = ACTIONS(1296),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1298),
    [anon_sym___declspec] = ACTIONS(1296),
    [anon_sym___cdecl] = ACTIONS(1296),
    [anon_sym___clrcall] = ACTIONS(1296),
    [anon_sym___stdcall] = ACTIONS(1296),
    [anon_sym___fastcall] = ACTIONS(1296),
    [anon_sym___thiscall] = ACTIONS(1296),
    [anon_sym___vectorcall] = ACTIONS(1296),
    [anon_sym_LBRACE] = ACTIONS(1298),
    [anon_sym_signed] = ACTIONS(1296),
    [anon_sym_unsigned] = ACTIONS(1296),
    [anon_sym_long] = ACTIONS(1296),
    [anon_sym_short] = ACTIONS(1296),
    [anon_sym_static] = ACTIONS(1296),
    [anon_sym_auto] = ACTIONS(1296),
    [anon_sym_register] = ACTIONS(1296),
    [anon_sym_inline] = ACTIONS(1296),
    [anon_sym___inline] = ACTIONS(1296),
    [anon_sym___inline__] = ACTIONS(1296),
    [anon_sym___forceinline] = ACTIONS(1296),
    [anon_sym_thread_local] = ACTIONS(1296),
    [anon_sym___thread] = ACTIONS(1296),
    [anon_sym_const] = ACTIONS(1296),
    [anon_sym_constexpr] = ACTIONS(1296),
    [anon_sym_volatile] = ACTIONS(1296),
    [anon_sym_restrict] = ACTIONS(1296),
    [anon_sym___restrict__] = ACTIONS(1296),
    [anon_sym__Atomic] = ACTIONS(1296),
    [anon_sym__Noreturn] = ACTIONS(1296),
    [anon_sym_noreturn] = ACTIONS(1296),
    [sym_primitive_type] = ACTIONS(1296),
    [anon_sym_enum] = ACTIONS(1296),
    [anon_sym_struct] = ACTIONS(1296),
    [anon_sym_union] = ACTIONS(1296),
    [anon_sym_if] = ACTIONS(1296),
    [anon_sym_switch] = ACTIONS(1296),
    [anon_sym_case] = ACTIONS(1296),
    [anon_sym_default] = ACTIONS(1296),
    [anon_sym_while] = ACTIONS(1296),
    [anon_sym_do] = ACTIONS(1296),
    [anon_sym_for] = ACTIONS(1296),
    [anon_sym_return] = ACTIONS(1296),
    [anon_sym_break] = ACTIONS(1296),
    [anon_sym_continue] = ACTIONS(1296),
    [anon_sym_goto] = ACTIONS(1296),
    [anon_sym_DASH_DASH] = ACTIONS(1298),
    [anon_sym_PLUS_PLUS] = ACTIONS(1298),
    [anon_sym_sizeof] = ACTIONS(1296),
    [anon_sym___alignof__] = ACTIONS(1296),
    [anon_sym___alignof] = ACTIONS(1296),
    [anon_sym__alignof] = ACTIONS(1296),
    [anon_sym_alignof] = ACTIONS(1296),
    [anon_sym__Alignof] = ACTIONS(1296),
    [anon_sym_offsetof] = ACTIONS(1296),
    [anon_sym__Generic] = ACTIONS(1296),
    [anon_sym_asm] = ACTIONS(1296),
    [anon_sym___asm__] = ACTIONS(1296),
    [sym_number_literal] = ACTIONS(1298),
    [anon_sym_L_SQUOTE] = ACTIONS(1298),
    [anon_sym_u_SQUOTE] = ACTIONS(1298),
    [anon_sym_U_SQUOTE] = ACTIONS(1298),
    [anon_sym_u8_SQUOTE] = ACTIONS(1298),
    [anon_sym_SQUOTE] = ACTIONS(1298),
    [anon_sym_L_DQUOTE] = ACTIONS(1298),
    [anon_sym_u_DQUOTE] = ACTIONS(1298),
    [anon_sym_U_DQUOTE] = ACTIONS(1298),
    [anon_sym_u8_DQUOTE] = ACTIONS(1298),
    [anon_sym_DQUOTE] = ACTIONS(1298),
    [sym_true] = ACTIONS(1296),
    [sym_false] = ACTIONS(1296),
    [anon_sym_NULL] = ACTIONS(1296),
    [anon_sym_nullptr] = ACTIONS(1296),
    [sym_comment] = ACTIONS(3),
  },
  [423] = {
    [ts_builtin_sym_end] = ACTIONS(1258),
    [sym_identifier] = ACTIONS(1256),
    [aux_sym_preproc_include_token1] = ACTIONS(1256),
    [aux_sym_preproc_def_token1] = ACTIONS(1256),
    [aux_sym_preproc_if_token1] = ACTIONS(1256),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1256),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1256),
    [sym_preproc_directive] = ACTIONS(1256),
    [anon_sym_LPAREN2] = ACTIONS(1258),
    [anon_sym_BANG] = ACTIONS(1258),
    [anon_sym_TILDE] = ACTIONS(1258),
    [anon_sym_DASH] = ACTIONS(1256),
    [anon_sym_PLUS] = ACTIONS(1256),
    [anon_sym_STAR] = ACTIONS(1258),
    [anon_sym_AMP] = ACTIONS(1258),
    [anon_sym___extension__] = ACTIONS(1256),
    [anon_sym_typedef] = ACTIONS(1256),
    [anon_sym_extern] = ACTIONS(1256),
    [anon_sym___attribute__] = ACTIONS(1256),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1258),
    [anon_sym___declspec] = ACTIONS(1256),
    [anon_sym___cdecl] = ACTIONS(1256),
    [anon_sym___clrcall] = ACTIONS(1256),
    [anon_sym___stdcall] = ACTIONS(1256),
    [anon_sym___fastcall] = ACTIONS(1256),
    [anon_sym___thiscall] = ACTIONS(1256),
    [anon_sym___vectorcall] = ACTIONS(1256),
    [anon_sym_LBRACE] = ACTIONS(1258),
    [anon_sym_signed] = ACTIONS(1256),
    [anon_sym_unsigned] = ACTIONS(1256),
    [anon_sym_long] = ACTIONS(1256),
    [anon_sym_short] = ACTIONS(1256),
    [anon_sym_static] = ACTIONS(1256),
    [anon_sym_auto] = ACTIONS(1256),
    [anon_sym_register] = ACTIONS(1256),
    [anon_sym_inline] = ACTIONS(1256),
    [anon_sym___inline] = ACTIONS(1256),
    [anon_sym___inline__] = ACTIONS(1256),
    [anon_sym___forceinline] = ACTIONS(1256),
    [anon_sym_thread_local] = ACTIONS(1256),
    [anon_sym___thread] = ACTIONS(1256),
    [anon_sym_const] = ACTIONS(1256),
    [anon_sym_constexpr] = ACTIONS(1256),
    [anon_sym_volatile] = ACTIONS(1256),
    [anon_sym_restrict] = ACTIONS(1256),
    [anon_sym___restrict__] = ACTIONS(1256),
    [anon_sym__Atomic] = ACTIONS(1256),
    [anon_sym__Noreturn] = ACTIONS(1256),
    [anon_sym_noreturn] = ACTIONS(1256),
    [sym_primitive_type] = ACTIONS(1256),
    [anon_sym_enum] = ACTIONS(1256),
    [anon_sym_struct] = ACTIONS(1256),
    [anon_sym_union] = ACTIONS(1256),
    [anon_sym_if] = ACTIONS(1256),
    [anon_sym_switch] = ACTIONS(1256),
    [anon_sym_case] = ACTIONS(1256),
    [anon_sym_default] = ACTIONS(1256),
    [anon_sym_while] = ACTIONS(1256),
    [anon_sym_do] = ACTIONS(1256),
    [anon_sym_for] = ACTIONS(1256),
    [anon_sym_return] = ACTIONS(1256),
    [anon_sym_break] = ACTIONS(1256),
    [anon_sym_continue] = ACTIONS(1256),
    [anon_sym_goto] = ACTIONS(1256),
    [anon_sym_DASH_DASH] = ACTIONS(1258),
    [anon_sym_PLUS_PLUS] = ACTIONS(1258),
    [anon_sym_sizeof] = ACTIONS(1256),
    [anon_sym___alignof__] = ACTIONS(1256),
    [anon_sym___alignof] = ACTIONS(1256),
    [anon_sym__alignof] = ACTIONS(1256),
    [anon_sym_alignof] = ACTIONS(1256),
    [anon_sym__Alignof] = ACTIONS(1256),
    [anon_sym_offsetof] = ACTIONS(1256),
    [anon_sym__Generic] = ACTIONS(1256),
    [anon_sym_asm] = ACTIONS(1256),
    [anon_sym___asm__] = ACTIONS(1256),
    [sym_number_literal] = ACTIONS(1258),
    [anon_sym_L_SQUOTE] = ACTIONS(1258),
    [anon_sym_u_SQUOTE] = ACTIONS(1258),
    [anon_sym_U_SQUOTE] = ACTIONS(1258),
    [anon_sym_u8_SQUOTE] = ACTIONS(1258),
    [anon_sym_SQUOTE] = ACTIONS(1258),
    [anon_sym_L_DQUOTE] = ACTIONS(1258),
    [anon_sym_u_DQUOTE] = ACTIONS(1258),
    [anon_sym_U_DQUOTE] = ACTIONS(1258),
    [anon_sym_u8_DQUOTE] = ACTIONS(1258),
    [anon_sym_DQUOTE] = ACTIONS(1258),
    [sym_true] = ACTIONS(1256),
    [sym_false] = ACTIONS(1256),
    [anon_sym_NULL] = ACTIONS(1256),
    [anon_sym_nullptr] = ACTIONS(1256),
    [sym_comment] = ACTIONS(3),
  },
  [424] = {
    [ts_builtin_sym_end] = ACTIONS(1342),
    [sym_identifier] = ACTIONS(1340),
    [aux_sym_preproc_include_token1] = ACTIONS(1340),
    [aux_sym_preproc_def_token1] = ACTIONS(1340),
    [aux_sym_preproc_if_token1] = ACTIONS(1340),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1340),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1340),
    [sym_preproc_directive] = ACTIONS(1340),
    [anon_sym_LPAREN2] = ACTIONS(1342),
    [anon_sym_BANG] = ACTIONS(1342),
    [anon_sym_TILDE] = ACTIONS(1342),
    [anon_sym_DASH] = ACTIONS(1340),
    [anon_sym_PLUS] = ACTIONS(1340),
    [anon_sym_STAR] = ACTIONS(1342),
    [anon_sym_AMP] = ACTIONS(1342),
    [anon_sym___extension__] = ACTIONS(1340),
    [anon_sym_typedef] = ACTIONS(1340),
    [anon_sym_extern] = ACTIONS(1340),
    [anon_sym___attribute__] = ACTIONS(1340),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1342),
    [anon_sym___declspec] = ACTIONS(1340),
    [anon_sym___cdecl] = ACTIONS(1340),
    [anon_sym___clrcall] = ACTIONS(1340),
    [anon_sym___stdcall] = ACTIONS(1340),
    [anon_sym___fastcall] = ACTIONS(1340),
    [anon_sym___thiscall] = ACTIONS(1340),
    [anon_sym___vectorcall] = ACTIONS(1340),
    [anon_sym_LBRACE] = ACTIONS(1342),
    [anon_sym_signed] = ACTIONS(1340),
    [anon_sym_unsigned] = ACTIONS(1340),
    [anon_sym_long] = ACTIONS(1340),
    [anon_sym_short] = ACTIONS(1340),
    [anon_sym_static] = ACTIONS(1340),
    [anon_sym_auto] = ACTIONS(1340),
    [anon_sym_register] = ACTIONS(1340),
    [anon_sym_inline] = ACTIONS(1340),
    [anon_sym___inline] = ACTIONS(1340),
    [anon_sym___inline__] = ACTIONS(1340),
    [anon_sym___forceinline] = ACTIONS(1340),
    [anon_sym_thread_local] = ACTIONS(1340),
    [anon_sym___thread] = ACTIONS(1340),
    [anon_sym_const] = ACTIONS(1340),
    [anon_sym_constexpr] = ACTIONS(1340),
    [anon_sym_volatile] = ACTIONS(1340),
    [anon_sym_restrict] = ACTIONS(1340),
    [anon_sym___restrict__] = ACTIONS(1340),
    [anon_sym__Atomic] = ACTIONS(1340),
    [anon_sym__Noreturn] = ACTIONS(1340),
    [anon_sym_noreturn] = ACTIONS(1340),
    [sym_primitive_type] = ACTIONS(1340),
    [anon_sym_enum] = ACTIONS(1340),
    [anon_sym_struct] = ACTIONS(1340),
    [anon_sym_union] = ACTIONS(1340),
    [anon_sym_if] = ACTIONS(1340),
    [anon_sym_switch] = ACTIONS(1340),
    [anon_sym_case] = ACTIONS(1340),
    [anon_sym_default] = ACTIONS(1340),
    [anon_sym_while] = ACTIONS(1340),
    [anon_sym_do] = ACTIONS(1340),
    [anon_sym_for] = ACTIONS(1340),
    [anon_sym_return] = ACTIONS(1340),
    [anon_sym_break] = ACTIONS(1340),
    [anon_sym_continue] = ACTIONS(1340),
    [anon_sym_goto] = ACTIONS(1340),
    [anon_sym_DASH_DASH] = ACTIONS(1342),
    [anon_sym_PLUS_PLUS] = ACTIONS(1342),
    [anon_sym_sizeof] = ACTIONS(1340),
    [anon_sym___alignof__] = ACTIONS(1340),
    [anon_sym___alignof] = ACTIONS(1340),
    [anon_sym__alignof] = ACTIONS(1340),
    [anon_sym_alignof] = ACTIONS(1340),
    [anon_sym__Alignof] = ACTIONS(1340),
    [anon_sym_offsetof] = ACTIONS(1340),
    [anon_sym__Generic] = ACTIONS(1340),
    [anon_sym_asm] = ACTIONS(1340),
    [anon_sym___asm__] = ACTIONS(1340),
    [sym_number_literal] = ACTIONS(1342),
    [anon_sym_L_SQUOTE] = ACTIONS(1342),
    [anon_sym_u_SQUOTE] = ACTIONS(1342),
    [anon_sym_U_SQUOTE] = ACTIONS(1342),
    [anon_sym_u8_SQUOTE] = ACTIONS(1342),
    [anon_sym_SQUOTE] = ACTIONS(1342),
    [anon_sym_L_DQUOTE] = ACTIONS(1342),
    [anon_sym_u_DQUOTE] = ACTIONS(1342),
    [anon_sym_U_DQUOTE] = ACTIONS(1342),
    [anon_sym_u8_DQUOTE] = ACTIONS(1342),
    [anon_sym_DQUOTE] = ACTIONS(1342),
    [sym_true] = ACTIONS(1340),
    [sym_false] = ACTIONS(1340),
    [anon_sym_NULL] = ACTIONS(1340),
    [anon_sym_nullptr] = ACTIONS(1340),
    [sym_comment] = ACTIONS(3),
  },
  [425] = {
    [ts_builtin_sym_end] = ACTIONS(1302),
    [sym_identifier] = ACTIONS(1300),
    [aux_sym_preproc_include_token1] = ACTIONS(1300),
    [aux_sym_preproc_def_token1] = ACTIONS(1300),
    [aux_sym_preproc_if_token1] = ACTIONS(1300),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1300),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1300),
    [sym_preproc_directive] = ACTIONS(1300),
    [anon_sym_LPAREN2] = ACTIONS(1302),
    [anon_sym_BANG] = ACTIONS(1302),
    [anon_sym_TILDE] = ACTIONS(1302),
    [anon_sym_DASH] = ACTIONS(1300),
    [anon_sym_PLUS] = ACTIONS(1300),
    [anon_sym_STAR] = ACTIONS(1302),
    [anon_sym_AMP] = ACTIONS(1302),
    [anon_sym___extension__] = ACTIONS(1300),
    [anon_sym_typedef] = ACTIONS(1300),
    [anon_sym_extern] = ACTIONS(1300),
    [anon_sym___attribute__] = ACTIONS(1300),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1302),
    [anon_sym___declspec] = ACTIONS(1300),
    [anon_sym___cdecl] = ACTIONS(1300),
    [anon_sym___clrcall] = ACTIONS(1300),
    [anon_sym___stdcall] = ACTIONS(1300),
    [anon_sym___fastcall] = ACTIONS(1300),
    [anon_sym___thiscall] = ACTIONS(1300),
    [anon_sym___vectorcall] = ACTIONS(1300),
    [anon_sym_LBRACE] = ACTIONS(1302),
    [anon_sym_signed] = ACTIONS(1300),
    [anon_sym_unsigned] = ACTIONS(1300),
    [anon_sym_long] = ACTIONS(1300),
    [anon_sym_short] = ACTIONS(1300),
    [anon_sym_static] = ACTIONS(1300),
    [anon_sym_auto] = ACTIONS(1300),
    [anon_sym_register] = ACTIONS(1300),
    [anon_sym_inline] = ACTIONS(1300),
    [anon_sym___inline] = ACTIONS(1300),
    [anon_sym___inline__] = ACTIONS(1300),
    [anon_sym___forceinline] = ACTIONS(1300),
    [anon_sym_thread_local] = ACTIONS(1300),
    [anon_sym___thread] = ACTIONS(1300),
    [anon_sym_const] = ACTIONS(1300),
    [anon_sym_constexpr] = ACTIONS(1300),
    [anon_sym_volatile] = ACTIONS(1300),
    [anon_sym_restrict] = ACTIONS(1300),
    [anon_sym___restrict__] = ACTIONS(1300),
    [anon_sym__Atomic] = ACTIONS(1300),
    [anon_sym__Noreturn] = ACTIONS(1300),
    [anon_sym_noreturn] = ACTIONS(1300),
    [sym_primitive_type] = ACTIONS(1300),
    [anon_sym_enum] = ACTIONS(1300),
    [anon_sym_struct] = ACTIONS(1300),
    [anon_sym_union] = ACTIONS(1300),
    [anon_sym_if] = ACTIONS(1300),
    [anon_sym_switch] = ACTIONS(1300),
    [anon_sym_case] = ACTIONS(1300),
    [anon_sym_default] = ACTIONS(1300),
    [anon_sym_while] = ACTIONS(1300),
    [anon_sym_do] = ACTIONS(1300),
    [anon_sym_for] = ACTIONS(1300),
    [anon_sym_return] = ACTIONS(1300),
    [anon_sym_break] = ACTIONS(1300),
    [anon_sym_continue] = ACTIONS(1300),
    [anon_sym_goto] = ACTIONS(1300),
    [anon_sym_DASH_DASH] = ACTIONS(1302),
    [anon_sym_PLUS_PLUS] = ACTIONS(1302),
    [anon_sym_sizeof] = ACTIONS(1300),
    [anon_sym___alignof__] = ACTIONS(1300),
    [anon_sym___alignof] = ACTIONS(1300),
    [anon_sym__alignof] = ACTIONS(1300),
    [anon_sym_alignof] = ACTIONS(1300),
    [anon_sym__Alignof] = ACTIONS(1300),
    [anon_sym_offsetof] = ACTIONS(1300),
    [anon_sym__Generic] = ACTIONS(1300),
    [anon_sym_asm] = ACTIONS(1300),
    [anon_sym___asm__] = ACTIONS(1300),
    [sym_number_literal] = ACTIONS(1302),
    [anon_sym_L_SQUOTE] = ACTIONS(1302),
    [anon_sym_u_SQUOTE] = ACTIONS(1302),
    [anon_sym_U_SQUOTE] = ACTIONS(1302),
    [anon_sym_u8_SQUOTE] = ACTIONS(1302),
    [anon_sym_SQUOTE] = ACTIONS(1302),
    [anon_sym_L_DQUOTE] = ACTIONS(1302),
    [anon_sym_u_DQUOTE] = ACTIONS(1302),
    [anon_sym_U_DQUOTE] = ACTIONS(1302),
    [anon_sym_u8_DQUOTE] = ACTIONS(1302),
    [anon_sym_DQUOTE] = ACTIONS(1302),
    [sym_true] = ACTIONS(1300),
    [sym_false] = ACTIONS(1300),
    [anon_sym_NULL] = ACTIONS(1300),
    [anon_sym_nullptr] = ACTIONS(1300),
    [sym_comment] = ACTIONS(3),
  },
  [426] = {
    [ts_builtin_sym_end] = ACTIONS(1314),
    [sym_identifier] = ACTIONS(1312),
    [aux_sym_preproc_include_token1] = ACTIONS(1312),
    [aux_sym_preproc_def_token1] = ACTIONS(1312),
    [aux_sym_preproc_if_token1] = ACTIONS(1312),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1312),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1312),
    [sym_preproc_directive] = ACTIONS(1312),
    [anon_sym_LPAREN2] = ACTIONS(1314),
    [anon_sym_BANG] = ACTIONS(1314),
    [anon_sym_TILDE] = ACTIONS(1314),
    [anon_sym_DASH] = ACTIONS(1312),
    [anon_sym_PLUS] = ACTIONS(1312),
    [anon_sym_STAR] = ACTIONS(1314),
    [anon_sym_AMP] = ACTIONS(1314),
    [anon_sym___extension__] = ACTIONS(1312),
    [anon_sym_typedef] = ACTIONS(1312),
    [anon_sym_extern] = ACTIONS(1312),
    [anon_sym___attribute__] = ACTIONS(1312),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1314),
    [anon_sym___declspec] = ACTIONS(1312),
    [anon_sym___cdecl] = ACTIONS(1312),
    [anon_sym___clrcall] = ACTIONS(1312),
    [anon_sym___stdcall] = ACTIONS(1312),
    [anon_sym___fastcall] = ACTIONS(1312),
    [anon_sym___thiscall] = ACTIONS(1312),
    [anon_sym___vectorcall] = ACTIONS(1312),
    [anon_sym_LBRACE] = ACTIONS(1314),
    [anon_sym_signed] = ACTIONS(1312),
    [anon_sym_unsigned] = ACTIONS(1312),
    [anon_sym_long] = ACTIONS(1312),
    [anon_sym_short] = ACTIONS(1312),
    [anon_sym_static] = ACTIONS(1312),
    [anon_sym_auto] = ACTIONS(1312),
    [anon_sym_register] = ACTIONS(1312),
    [anon_sym_inline] = ACTIONS(1312),
    [anon_sym___inline] = ACTIONS(1312),
    [anon_sym___inline__] = ACTIONS(1312),
    [anon_sym___forceinline] = ACTIONS(1312),
    [anon_sym_thread_local] = ACTIONS(1312),
    [anon_sym___thread] = ACTIONS(1312),
    [anon_sym_const] = ACTIONS(1312),
    [anon_sym_constexpr] = ACTIONS(1312),
    [anon_sym_volatile] = ACTIONS(1312),
    [anon_sym_restrict] = ACTIONS(1312),
    [anon_sym___restrict__] = ACTIONS(1312),
    [anon_sym__Atomic] = ACTIONS(1312),
    [anon_sym__Noreturn] = ACTIONS(1312),
    [anon_sym_noreturn] = ACTIONS(1312),
    [sym_primitive_type] = ACTIONS(1312),
    [anon_sym_enum] = ACTIONS(1312),
    [anon_sym_struct] = ACTIONS(1312),
    [anon_sym_union] = ACTIONS(1312),
    [anon_sym_if] = ACTIONS(1312),
    [anon_sym_switch] = ACTIONS(1312),
    [anon_sym_case] = ACTIONS(1312),
    [anon_sym_default] = ACTIONS(1312),
    [anon_sym_while] = ACTIONS(1312),
    [anon_sym_do] = ACTIONS(1312),
    [anon_sym_for] = ACTIONS(1312),
    [anon_sym_return] = ACTIONS(1312),
    [anon_sym_break] = ACTIONS(1312),
    [anon_sym_continue] = ACTIONS(1312),
    [anon_sym_goto] = ACTIONS(1312),
    [anon_sym_DASH_DASH] = ACTIONS(1314),
    [anon_sym_PLUS_PLUS] = ACTIONS(1314),
    [anon_sym_sizeof] = ACTIONS(1312),
    [anon_sym___alignof__] = ACTIONS(1312),
    [anon_sym___alignof] = ACTIONS(1312),
    [anon_sym__alignof] = ACTIONS(1312),
    [anon_sym_alignof] = ACTIONS(1312),
    [anon_sym__Alignof] = ACTIONS(1312),
    [anon_sym_offsetof] = ACTIONS(1312),
    [anon_sym__Generic] = ACTIONS(1312),
    [anon_sym_asm] = ACTIONS(1312),
    [anon_sym___asm__] = ACTIONS(1312),
    [sym_number_literal] = ACTIONS(1314),
    [anon_sym_L_SQUOTE] = ACTIONS(1314),
    [anon_sym_u_SQUOTE] = ACTIONS(1314),
    [anon_sym_U_SQUOTE] = ACTIONS(1314),
    [anon_sym_u8_SQUOTE] = ACTIONS(1314),
    [anon_sym_SQUOTE] = ACTIONS(1314),
    [anon_sym_L_DQUOTE] = ACTIONS(1314),
    [anon_sym_u_DQUOTE] = ACTIONS(1314),
    [anon_sym_U_DQUOTE] = ACTIONS(1314),
    [anon_sym_u8_DQUOTE] = ACTIONS(1314),
    [anon_sym_DQUOTE] = ACTIONS(1314),
    [sym_true] = ACTIONS(1312),
    [sym_false] = ACTIONS(1312),
    [anon_sym_NULL] = ACTIONS(1312),
    [anon_sym_nullptr] = ACTIONS(1312),
    [sym_comment] = ACTIONS(3),
  },
  [427] = {
    [ts_builtin_sym_end] = ACTIONS(1306),
    [sym_identifier] = ACTIONS(1304),
    [aux_sym_preproc_include_token1] = ACTIONS(1304),
    [aux_sym_preproc_def_token1] = ACTIONS(1304),
    [aux_sym_preproc_if_token1] = ACTIONS(1304),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1304),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1304),
    [sym_preproc_directive] = ACTIONS(1304),
    [anon_sym_LPAREN2] = ACTIONS(1306),
    [anon_sym_BANG] = ACTIONS(1306),
    [anon_sym_TILDE] = ACTIONS(1306),
    [anon_sym_DASH] = ACTIONS(1304),
    [anon_sym_PLUS] = ACTIONS(1304),
    [anon_sym_STAR] = ACTIONS(1306),
    [anon_sym_AMP] = ACTIONS(1306),
    [anon_sym___extension__] = ACTIONS(1304),
    [anon_sym_typedef] = ACTIONS(1304),
    [anon_sym_extern] = ACTIONS(1304),
    [anon_sym___attribute__] = ACTIONS(1304),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1306),
    [anon_sym___declspec] = ACTIONS(1304),
    [anon_sym___cdecl] = ACTIONS(1304),
    [anon_sym___clrcall] = ACTIONS(1304),
    [anon_sym___stdcall] = ACTIONS(1304),
    [anon_sym___fastcall] = ACTIONS(1304),
    [anon_sym___thiscall] = ACTIONS(1304),
    [anon_sym___vectorcall] = ACTIONS(1304),
    [anon_sym_LBRACE] = ACTIONS(1306),
    [anon_sym_signed] = ACTIONS(1304),
    [anon_sym_unsigned] = ACTIONS(1304),
    [anon_sym_long] = ACTIONS(1304),
    [anon_sym_short] = ACTIONS(1304),
    [anon_sym_static] = ACTIONS(1304),
    [anon_sym_auto] = ACTIONS(1304),
    [anon_sym_register] = ACTIONS(1304),
    [anon_sym_inline] = ACTIONS(1304),
    [anon_sym___inline] = ACTIONS(1304),
    [anon_sym___inline__] = ACTIONS(1304),
    [anon_sym___forceinline] = ACTIONS(1304),
    [anon_sym_thread_local] = ACTIONS(1304),
    [anon_sym___thread] = ACTIONS(1304),
    [anon_sym_const] = ACTIONS(1304),
    [anon_sym_constexpr] = ACTIONS(1304),
    [anon_sym_volatile] = ACTIONS(1304),
    [anon_sym_restrict] = ACTIONS(1304),
    [anon_sym___restrict__] = ACTIONS(1304),
    [anon_sym__Atomic] = ACTIONS(1304),
    [anon_sym__Noreturn] = ACTIONS(1304),
    [anon_sym_noreturn] = ACTIONS(1304),
    [sym_primitive_type] = ACTIONS(1304),
    [anon_sym_enum] = ACTIONS(1304),
    [anon_sym_struct] = ACTIONS(1304),
    [anon_sym_union] = ACTIONS(1304),
    [anon_sym_if] = ACTIONS(1304),
    [anon_sym_switch] = ACTIONS(1304),
    [anon_sym_case] = ACTIONS(1304),
    [anon_sym_default] = ACTIONS(1304),
    [anon_sym_while] = ACTIONS(1304),
    [anon_sym_do] = ACTIONS(1304),
    [anon_sym_for] = ACTIONS(1304),
    [anon_sym_return] = ACTIONS(1304),
    [anon_sym_break] = ACTIONS(1304),
    [anon_sym_continue] = ACTIONS(1304),
    [anon_sym_goto] = ACTIONS(1304),
    [anon_sym_DASH_DASH] = ACTIONS(1306),
    [anon_sym_PLUS_PLUS] = ACTIONS(1306),
    [anon_sym_sizeof] = ACTIONS(1304),
    [anon_sym___alignof__] = ACTIONS(1304),
    [anon_sym___alignof] = ACTIONS(1304),
    [anon_sym__alignof] = ACTIONS(1304),
    [anon_sym_alignof] = ACTIONS(1304),
    [anon_sym__Alignof] = ACTIONS(1304),
    [anon_sym_offsetof] = ACTIONS(1304),
    [anon_sym__Generic] = ACTIONS(1304),
    [anon_sym_asm] = ACTIONS(1304),
    [anon_sym___asm__] = ACTIONS(1304),
    [sym_number_literal] = ACTIONS(1306),
    [anon_sym_L_SQUOTE] = ACTIONS(1306),
    [anon_sym_u_SQUOTE] = ACTIONS(1306),
    [anon_sym_U_SQUOTE] = ACTIONS(1306),
    [anon_sym_u8_SQUOTE] = ACTIONS(1306),
    [anon_sym_SQUOTE] = ACTIONS(1306),
    [anon_sym_L_DQUOTE] = ACTIONS(1306),
    [anon_sym_u_DQUOTE] = ACTIONS(1306),
    [anon_sym_U_DQUOTE] = ACTIONS(1306),
    [anon_sym_u8_DQUOTE] = ACTIONS(1306),
    [anon_sym_DQUOTE] = ACTIONS(1306),
    [sym_true] = ACTIONS(1304),
    [sym_false] = ACTIONS(1304),
    [anon_sym_NULL] = ACTIONS(1304),
    [anon_sym_nullptr] = ACTIONS(1304),
    [sym_comment] = ACTIONS(3),
  },
  [428] = {
    [ts_builtin_sym_end] = ACTIONS(1330),
    [sym_identifier] = ACTIONS(1328),
    [aux_sym_preproc_include_token1] = ACTIONS(1328),
    [aux_sym_preproc_def_token1] = ACTIONS(1328),
    [aux_sym_preproc_if_token1] = ACTIONS(1328),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1328),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1328),
    [sym_preproc_directive] = ACTIONS(1328),
    [anon_sym_LPAREN2] = ACTIONS(1330),
    [anon_sym_BANG] = ACTIONS(1330),
    [anon_sym_TILDE] = ACTIONS(1330),
    [anon_sym_DASH] = ACTIONS(1328),
    [anon_sym_PLUS] = ACTIONS(1328),
    [anon_sym_STAR] = ACTIONS(1330),
    [anon_sym_AMP] = ACTIONS(1330),
    [anon_sym___extension__] = ACTIONS(1328),
    [anon_sym_typedef] = ACTIONS(1328),
    [anon_sym_extern] = ACTIONS(1328),
    [anon_sym___attribute__] = ACTIONS(1328),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1330),
    [anon_sym___declspec] = ACTIONS(1328),
    [anon_sym___cdecl] = ACTIONS(1328),
    [anon_sym___clrcall] = ACTIONS(1328),
    [anon_sym___stdcall] = ACTIONS(1328),
    [anon_sym___fastcall] = ACTIONS(1328),
    [anon_sym___thiscall] = ACTIONS(1328),
    [anon_sym___vectorcall] = ACTIONS(1328),
    [anon_sym_LBRACE] = ACTIONS(1330),
    [anon_sym_signed] = ACTIONS(1328),
    [anon_sym_unsigned] = ACTIONS(1328),
    [anon_sym_long] = ACTIONS(1328),
    [anon_sym_short] = ACTIONS(1328),
    [anon_sym_static] = ACTIONS(1328),
    [anon_sym_auto] = ACTIONS(1328),
    [anon_sym_register] = ACTIONS(1328),
    [anon_sym_inline] = ACTIONS(1328),
    [anon_sym___inline] = ACTIONS(1328),
    [anon_sym___inline__] = ACTIONS(1328),
    [anon_sym___forceinline] = ACTIONS(1328),
    [anon_sym_thread_local] = ACTIONS(1328),
    [anon_sym___thread] = ACTIONS(1328),
    [anon_sym_const] = ACTIONS(1328),
    [anon_sym_constexpr] = ACTIONS(1328),
    [anon_sym_volatile] = ACTIONS(1328),
    [anon_sym_restrict] = ACTIONS(1328),
    [anon_sym___restrict__] = ACTIONS(1328),
    [anon_sym__Atomic] = ACTIONS(1328),
    [anon_sym__Noreturn] = ACTIONS(1328),
    [anon_sym_noreturn] = ACTIONS(1328),
    [sym_primitive_type] = ACTIONS(1328),
    [anon_sym_enum] = ACTIONS(1328),
    [anon_sym_struct] = ACTIONS(1328),
    [anon_sym_union] = ACTIONS(1328),
    [anon_sym_if] = ACTIONS(1328),
    [anon_sym_switch] = ACTIONS(1328),
    [anon_sym_case] = ACTIONS(1328),
    [anon_sym_default] = ACTIONS(1328),
    [anon_sym_while] = ACTIONS(1328),
    [anon_sym_do] = ACTIONS(1328),
    [anon_sym_for] = ACTIONS(1328),
    [anon_sym_return] = ACTIONS(1328),
    [anon_sym_break] = ACTIONS(1328),
    [anon_sym_continue] = ACTIONS(1328),
    [anon_sym_goto] = ACTIONS(1328),
    [anon_sym_DASH_DASH] = ACTIONS(1330),
    [anon_sym_PLUS_PLUS] = ACTIONS(1330),
    [anon_sym_sizeof] = ACTIONS(1328),
    [anon_sym___alignof__] = ACTIONS(1328),
    [anon_sym___alignof] = ACTIONS(1328),
    [anon_sym__alignof] = ACTIONS(1328),
    [anon_sym_alignof] = ACTIONS(1328),
    [anon_sym__Alignof] = ACTIONS(1328),
    [anon_sym_offsetof] = ACTIONS(1328),
    [anon_sym__Generic] = ACTIONS(1328),
    [anon_sym_asm] = ACTIONS(1328),
    [anon_sym___asm__] = ACTIONS(1328),
    [sym_number_literal] = ACTIONS(1330),
    [anon_sym_L_SQUOTE] = ACTIONS(1330),
    [anon_sym_u_SQUOTE] = ACTIONS(1330),
    [anon_sym_U_SQUOTE] = ACTIONS(1330),
    [anon_sym_u8_SQUOTE] = ACTIONS(1330),
    [anon_sym_SQUOTE] = ACTIONS(1330),
    [anon_sym_L_DQUOTE] = ACTIONS(1330),
    [anon_sym_u_DQUOTE] = ACTIONS(1330),
    [anon_sym_U_DQUOTE] = ACTIONS(1330),
    [anon_sym_u8_DQUOTE] = ACTIONS(1330),
    [anon_sym_DQUOTE] = ACTIONS(1330),
    [sym_true] = ACTIONS(1328),
    [sym_false] = ACTIONS(1328),
    [anon_sym_NULL] = ACTIONS(1328),
    [anon_sym_nullptr] = ACTIONS(1328),
    [sym_comment] = ACTIONS(3),
  },
  [429] = {
    [ts_builtin_sym_end] = ACTIONS(1246),
    [sym_identifier] = ACTIONS(1244),
    [aux_sym_preproc_include_token1] = ACTIONS(1244),
    [aux_sym_preproc_def_token1] = ACTIONS(1244),
    [aux_sym_preproc_if_token1] = ACTIONS(1244),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1244),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1244),
    [sym_preproc_directive] = ACTIONS(1244),
    [anon_sym_LPAREN2] = ACTIONS(1246),
    [anon_sym_BANG] = ACTIONS(1246),
    [anon_sym_TILDE] = ACTIONS(1246),
    [anon_sym_DASH] = ACTIONS(1244),
    [anon_sym_PLUS] = ACTIONS(1244),
    [anon_sym_STAR] = ACTIONS(1246),
    [anon_sym_AMP] = ACTIONS(1246),
    [anon_sym___extension__] = ACTIONS(1244),
    [anon_sym_typedef] = ACTIONS(1244),
    [anon_sym_extern] = ACTIONS(1244),
    [anon_sym___attribute__] = ACTIONS(1244),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1246),
    [anon_sym___declspec] = ACTIONS(1244),
    [anon_sym___cdecl] = ACTIONS(1244),
    [anon_sym___clrcall] = ACTIONS(1244),
    [anon_sym___stdcall] = ACTIONS(1244),
    [anon_sym___fastcall] = ACTIONS(1244),
    [anon_sym___thiscall] = ACTIONS(1244),
    [anon_sym___vectorcall] = ACTIONS(1244),
    [anon_sym_LBRACE] = ACTIONS(1246),
    [anon_sym_signed] = ACTIONS(1244),
    [anon_sym_unsigned] = ACTIONS(1244),
    [anon_sym_long] = ACTIONS(1244),
    [anon_sym_short] = ACTIONS(1244),
    [anon_sym_static] = ACTIONS(1244),
    [anon_sym_auto] = ACTIONS(1244),
    [anon_sym_register] = ACTIONS(1244),
    [anon_sym_inline] = ACTIONS(1244),
    [anon_sym___inline] = ACTIONS(1244),
    [anon_sym___inline__] = ACTIONS(1244),
    [anon_sym___forceinline] = ACTIONS(1244),
    [anon_sym_thread_local] = ACTIONS(1244),
    [anon_sym___thread] = ACTIONS(1244),
    [anon_sym_const] = ACTIONS(1244),
    [anon_sym_constexpr] = ACTIONS(1244),
    [anon_sym_volatile] = ACTIONS(1244),
    [anon_sym_restrict] = ACTIONS(1244),
    [anon_sym___restrict__] = ACTIONS(1244),
    [anon_sym__Atomic] = ACTIONS(1244),
    [anon_sym__Noreturn] = ACTIONS(1244),
    [anon_sym_noreturn] = ACTIONS(1244),
    [sym_primitive_type] = ACTIONS(1244),
    [anon_sym_enum] = ACTIONS(1244),
    [anon_sym_struct] = ACTIONS(1244),
    [anon_sym_union] = ACTIONS(1244),
    [anon_sym_if] = ACTIONS(1244),
    [anon_sym_switch] = ACTIONS(1244),
    [anon_sym_case] = ACTIONS(1244),
    [anon_sym_default] = ACTIONS(1244),
    [anon_sym_while] = ACTIONS(1244),
    [anon_sym_do] = ACTIONS(1244),
    [anon_sym_for] = ACTIONS(1244),
    [anon_sym_return] = ACTIONS(1244),
    [anon_sym_break] = ACTIONS(1244),
    [anon_sym_continue] = ACTIONS(1244),
    [anon_sym_goto] = ACTIONS(1244),
    [anon_sym_DASH_DASH] = ACTIONS(1246),
    [anon_sym_PLUS_PLUS] = ACTIONS(1246),
    [anon_sym_sizeof] = ACTIONS(1244),
    [anon_sym___alignof__] = ACTIONS(1244),
    [anon_sym___alignof] = ACTIONS(1244),
    [anon_sym__alignof] = ACTIONS(1244),
    [anon_sym_alignof] = ACTIONS(1244),
    [anon_sym__Alignof] = ACTIONS(1244),
    [anon_sym_offsetof] = ACTIONS(1244),
    [anon_sym__Generic] = ACTIONS(1244),
    [anon_sym_asm] = ACTIONS(1244),
    [anon_sym___asm__] = ACTIONS(1244),
    [sym_number_literal] = ACTIONS(1246),
    [anon_sym_L_SQUOTE] = ACTIONS(1246),
    [anon_sym_u_SQUOTE] = ACTIONS(1246),
    [anon_sym_U_SQUOTE] = ACTIONS(1246),
    [anon_sym_u8_SQUOTE] = ACTIONS(1246),
    [anon_sym_SQUOTE] = ACTIONS(1246),
    [anon_sym_L_DQUOTE] = ACTIONS(1246),
    [anon_sym_u_DQUOTE] = ACTIONS(1246),
    [anon_sym_U_DQUOTE] = ACTIONS(1246),
    [anon_sym_u8_DQUOTE] = ACTIONS(1246),
    [anon_sym_DQUOTE] = ACTIONS(1246),
    [sym_true] = ACTIONS(1244),
    [sym_false] = ACTIONS(1244),
    [anon_sym_NULL] = ACTIONS(1244),
    [anon_sym_nullptr] = ACTIONS(1244),
    [sym_comment] = ACTIONS(3),
  },
  [430] = {
    [ts_builtin_sym_end] = ACTIONS(1242),
    [sym_identifier] = ACTIONS(1240),
    [aux_sym_preproc_include_token1] = ACTIONS(1240),
    [aux_sym_preproc_def_token1] = ACTIONS(1240),
    [aux_sym_preproc_if_token1] = ACTIONS(1240),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1240),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1240),
    [sym_preproc_directive] = ACTIONS(1240),
    [anon_sym_LPAREN2] = ACTIONS(1242),
    [anon_sym_BANG] = ACTIONS(1242),
    [anon_sym_TILDE] = ACTIONS(1242),
    [anon_sym_DASH] = ACTIONS(1240),
    [anon_sym_PLUS] = ACTIONS(1240),
    [anon_sym_STAR] = ACTIONS(1242),
    [anon_sym_AMP] = ACTIONS(1242),
    [anon_sym___extension__] = ACTIONS(1240),
    [anon_sym_typedef] = ACTIONS(1240),
    [anon_sym_extern] = ACTIONS(1240),
    [anon_sym___attribute__] = ACTIONS(1240),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1242),
    [anon_sym___declspec] = ACTIONS(1240),
    [anon_sym___cdecl] = ACTIONS(1240),
    [anon_sym___clrcall] = ACTIONS(1240),
    [anon_sym___stdcall] = ACTIONS(1240),
    [anon_sym___fastcall] = ACTIONS(1240),
    [anon_sym___thiscall] = ACTIONS(1240),
    [anon_sym___vectorcall] = ACTIONS(1240),
    [anon_sym_LBRACE] = ACTIONS(1242),
    [anon_sym_signed] = ACTIONS(1240),
    [anon_sym_unsigned] = ACTIONS(1240),
    [anon_sym_long] = ACTIONS(1240),
    [anon_sym_short] = ACTIONS(1240),
    [anon_sym_static] = ACTIONS(1240),
    [anon_sym_auto] = ACTIONS(1240),
    [anon_sym_register] = ACTIONS(1240),
    [anon_sym_inline] = ACTIONS(1240),
    [anon_sym___inline] = ACTIONS(1240),
    [anon_sym___inline__] = ACTIONS(1240),
    [anon_sym___forceinline] = ACTIONS(1240),
    [anon_sym_thread_local] = ACTIONS(1240),
    [anon_sym___thread] = ACTIONS(1240),
    [anon_sym_const] = ACTIONS(1240),
    [anon_sym_constexpr] = ACTIONS(1240),
    [anon_sym_volatile] = ACTIONS(1240),
    [anon_sym_restrict] = ACTIONS(1240),
    [anon_sym___restrict__] = ACTIONS(1240),
    [anon_sym__Atomic] = ACTIONS(1240),
    [anon_sym__Noreturn] = ACTIONS(1240),
    [anon_sym_noreturn] = ACTIONS(1240),
    [sym_primitive_type] = ACTIONS(1240),
    [anon_sym_enum] = ACTIONS(1240),
    [anon_sym_struct] = ACTIONS(1240),
    [anon_sym_union] = ACTIONS(1240),
    [anon_sym_if] = ACTIONS(1240),
    [anon_sym_switch] = ACTIONS(1240),
    [anon_sym_case] = ACTIONS(1240),
    [anon_sym_default] = ACTIONS(1240),
    [anon_sym_while] = ACTIONS(1240),
    [anon_sym_do] = ACTIONS(1240),
    [anon_sym_for] = ACTIONS(1240),
    [anon_sym_return] = ACTIONS(1240),
    [anon_sym_break] = ACTIONS(1240),
    [anon_sym_continue] = ACTIONS(1240),
    [anon_sym_goto] = ACTIONS(1240),
    [anon_sym_DASH_DASH] = ACTIONS(1242),
    [anon_sym_PLUS_PLUS] = ACTIONS(1242),
    [anon_sym_sizeof] = ACTIONS(1240),
    [anon_sym___alignof__] = ACTIONS(1240),
    [anon_sym___alignof] = ACTIONS(1240),
    [anon_sym__alignof] = ACTIONS(1240),
    [anon_sym_alignof] = ACTIONS(1240),
    [anon_sym__Alignof] = ACTIONS(1240),
    [anon_sym_offsetof] = ACTIONS(1240),
    [anon_sym__Generic] = ACTIONS(1240),
    [anon_sym_asm] = ACTIONS(1240),
    [anon_sym___asm__] = ACTIONS(1240),
    [sym_number_literal] = ACTIONS(1242),
    [anon_sym_L_SQUOTE] = ACTIONS(1242),
    [anon_sym_u_SQUOTE] = ACTIONS(1242),
    [anon_sym_U_SQUOTE] = ACTIONS(1242),
    [anon_sym_u8_SQUOTE] = ACTIONS(1242),
    [anon_sym_SQUOTE] = ACTIONS(1242),
    [anon_sym_L_DQUOTE] = ACTIONS(1242),
    [anon_sym_u_DQUOTE] = ACTIONS(1242),
    [anon_sym_U_DQUOTE] = ACTIONS(1242),
    [anon_sym_u8_DQUOTE] = ACTIONS(1242),
    [anon_sym_DQUOTE] = ACTIONS(1242),
    [sym_true] = ACTIONS(1240),
    [sym_false] = ACTIONS(1240),
    [anon_sym_NULL] = ACTIONS(1240),
    [anon_sym_nullptr] = ACTIONS(1240),
    [sym_comment] = ACTIONS(3),
  },
  [431] = {
    [ts_builtin_sym_end] = ACTIONS(1286),
    [sym_identifier] = ACTIONS(1284),
    [aux_sym_preproc_include_token1] = ACTIONS(1284),
    [aux_sym_preproc_def_token1] = ACTIONS(1284),
    [aux_sym_preproc_if_token1] = ACTIONS(1284),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1284),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1284),
    [sym_preproc_directive] = ACTIONS(1284),
    [anon_sym_LPAREN2] = ACTIONS(1286),
    [anon_sym_BANG] = ACTIONS(1286),
    [anon_sym_TILDE] = ACTIONS(1286),
    [anon_sym_DASH] = ACTIONS(1284),
    [anon_sym_PLUS] = ACTIONS(1284),
    [anon_sym_STAR] = ACTIONS(1286),
    [anon_sym_AMP] = ACTIONS(1286),
    [anon_sym___extension__] = ACTIONS(1284),
    [anon_sym_typedef] = ACTIONS(1284),
    [anon_sym_extern] = ACTIONS(1284),
    [anon_sym___attribute__] = ACTIONS(1284),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1286),
    [anon_sym___declspec] = ACTIONS(1284),
    [anon_sym___cdecl] = ACTIONS(1284),
    [anon_sym___clrcall] = ACTIONS(1284),
    [anon_sym___stdcall] = ACTIONS(1284),
    [anon_sym___fastcall] = ACTIONS(1284),
    [anon_sym___thiscall] = ACTIONS(1284),
    [anon_sym___vectorcall] = ACTIONS(1284),
    [anon_sym_LBRACE] = ACTIONS(1286),
    [anon_sym_signed] = ACTIONS(1284),
    [anon_sym_unsigned] = ACTIONS(1284),
    [anon_sym_long] = ACTIONS(1284),
    [anon_sym_short] = ACTIONS(1284),
    [anon_sym_static] = ACTIONS(1284),
    [anon_sym_auto] = ACTIONS(1284),
    [anon_sym_register] = ACTIONS(1284),
    [anon_sym_inline] = ACTIONS(1284),
    [anon_sym___inline] = ACTIONS(1284),
    [anon_sym___inline__] = ACTIONS(1284),
    [anon_sym___forceinline] = ACTIONS(1284),
    [anon_sym_thread_local] = ACTIONS(1284),
    [anon_sym___thread] = ACTIONS(1284),
    [anon_sym_const] = ACTIONS(1284),
    [anon_sym_constexpr] = ACTIONS(1284),
    [anon_sym_volatile] = ACTIONS(1284),
    [anon_sym_restrict] = ACTIONS(1284),
    [anon_sym___restrict__] = ACTIONS(1284),
    [anon_sym__Atomic] = ACTIONS(1284),
    [anon_sym__Noreturn] = ACTIONS(1284),
    [anon_sym_noreturn] = ACTIONS(1284),
    [sym_primitive_type] = ACTIONS(1284),
    [anon_sym_enum] = ACTIONS(1284),
    [anon_sym_struct] = ACTIONS(1284),
    [anon_sym_union] = ACTIONS(1284),
    [anon_sym_if] = ACTIONS(1284),
    [anon_sym_switch] = ACTIONS(1284),
    [anon_sym_case] = ACTIONS(1284),
    [anon_sym_default] = ACTIONS(1284),
    [anon_sym_while] = ACTIONS(1284),
    [anon_sym_do] = ACTIONS(1284),
    [anon_sym_for] = ACTIONS(1284),
    [anon_sym_return] = ACTIONS(1284),
    [anon_sym_break] = ACTIONS(1284),
    [anon_sym_continue] = ACTIONS(1284),
    [anon_sym_goto] = ACTIONS(1284),
    [anon_sym_DASH_DASH] = ACTIONS(1286),
    [anon_sym_PLUS_PLUS] = ACTIONS(1286),
    [anon_sym_sizeof] = ACTIONS(1284),
    [anon_sym___alignof__] = ACTIONS(1284),
    [anon_sym___alignof] = ACTIONS(1284),
    [anon_sym__alignof] = ACTIONS(1284),
    [anon_sym_alignof] = ACTIONS(1284),
    [anon_sym__Alignof] = ACTIONS(1284),
    [anon_sym_offsetof] = ACTIONS(1284),
    [anon_sym__Generic] = ACTIONS(1284),
    [anon_sym_asm] = ACTIONS(1284),
    [anon_sym___asm__] = ACTIONS(1284),
    [sym_number_literal] = ACTIONS(1286),
    [anon_sym_L_SQUOTE] = ACTIONS(1286),
    [anon_sym_u_SQUOTE] = ACTIONS(1286),
    [anon_sym_U_SQUOTE] = ACTIONS(1286),
    [anon_sym_u8_SQUOTE] = ACTIONS(1286),
    [anon_sym_SQUOTE] = ACTIONS(1286),
    [anon_sym_L_DQUOTE] = ACTIONS(1286),
    [anon_sym_u_DQUOTE] = ACTIONS(1286),
    [anon_sym_U_DQUOTE] = ACTIONS(1286),
    [anon_sym_u8_DQUOTE] = ACTIONS(1286),
    [anon_sym_DQUOTE] = ACTIONS(1286),
    [sym_true] = ACTIONS(1284),
    [sym_false] = ACTIONS(1284),
    [anon_sym_NULL] = ACTIONS(1284),
    [anon_sym_nullptr] = ACTIONS(1284),
    [sym_comment] = ACTIONS(3),
  },
  [432] = {
    [ts_builtin_sym_end] = ACTIONS(1338),
    [sym_identifier] = ACTIONS(1336),
    [aux_sym_preproc_include_token1] = ACTIONS(1336),
    [aux_sym_preproc_def_token1] = ACTIONS(1336),
    [aux_sym_preproc_if_token1] = ACTIONS(1336),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1336),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1336),
    [sym_preproc_directive] = ACTIONS(1336),
    [anon_sym_LPAREN2] = ACTIONS(1338),
    [anon_sym_BANG] = ACTIONS(1338),
    [anon_sym_TILDE] = ACTIONS(1338),
    [anon_sym_DASH] = ACTIONS(1336),
    [anon_sym_PLUS] = ACTIONS(1336),
    [anon_sym_STAR] = ACTIONS(1338),
    [anon_sym_AMP] = ACTIONS(1338),
    [anon_sym___extension__] = ACTIONS(1336),
    [anon_sym_typedef] = ACTIONS(1336),
    [anon_sym_extern] = ACTIONS(1336),
    [anon_sym___attribute__] = ACTIONS(1336),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1338),
    [anon_sym___declspec] = ACTIONS(1336),
    [anon_sym___cdecl] = ACTIONS(1336),
    [anon_sym___clrcall] = ACTIONS(1336),
    [anon_sym___stdcall] = ACTIONS(1336),
    [anon_sym___fastcall] = ACTIONS(1336),
    [anon_sym___thiscall] = ACTIONS(1336),
    [anon_sym___vectorcall] = ACTIONS(1336),
    [anon_sym_LBRACE] = ACTIONS(1338),
    [anon_sym_signed] = ACTIONS(1336),
    [anon_sym_unsigned] = ACTIONS(1336),
    [anon_sym_long] = ACTIONS(1336),
    [anon_sym_short] = ACTIONS(1336),
    [anon_sym_static] = ACTIONS(1336),
    [anon_sym_auto] = ACTIONS(1336),
    [anon_sym_register] = ACTIONS(1336),
    [anon_sym_inline] = ACTIONS(1336),
    [anon_sym___inline] = ACTIONS(1336),
    [anon_sym___inline__] = ACTIONS(1336),
    [anon_sym___forceinline] = ACTIONS(1336),
    [anon_sym_thread_local] = ACTIONS(1336),
    [anon_sym___thread] = ACTIONS(1336),
    [anon_sym_const] = ACTIONS(1336),
    [anon_sym_constexpr] = ACTIONS(1336),
    [anon_sym_volatile] = ACTIONS(1336),
    [anon_sym_restrict] = ACTIONS(1336),
    [anon_sym___restrict__] = ACTIONS(1336),
    [anon_sym__Atomic] = ACTIONS(1336),
    [anon_sym__Noreturn] = ACTIONS(1336),
    [anon_sym_noreturn] = ACTIONS(1336),
    [sym_primitive_type] = ACTIONS(1336),
    [anon_sym_enum] = ACTIONS(1336),
    [anon_sym_struct] = ACTIONS(1336),
    [anon_sym_union] = ACTIONS(1336),
    [anon_sym_if] = ACTIONS(1336),
    [anon_sym_switch] = ACTIONS(1336),
    [anon_sym_case] = ACTIONS(1336),
    [anon_sym_default] = ACTIONS(1336),
    [anon_sym_while] = ACTIONS(1336),
    [anon_sym_do] = ACTIONS(1336),
    [anon_sym_for] = ACTIONS(1336),
    [anon_sym_return] = ACTIONS(1336),
    [anon_sym_break] = ACTIONS(1336),
    [anon_sym_continue] = ACTIONS(1336),
    [anon_sym_goto] = ACTIONS(1336),
    [anon_sym_DASH_DASH] = ACTIONS(1338),
    [anon_sym_PLUS_PLUS] = ACTIONS(1338),
    [anon_sym_sizeof] = ACTIONS(1336),
    [anon_sym___alignof__] = ACTIONS(1336),
    [anon_sym___alignof] = ACTIONS(1336),
    [anon_sym__alignof] = ACTIONS(1336),
    [anon_sym_alignof] = ACTIONS(1336),
    [anon_sym__Alignof] = ACTIONS(1336),
    [anon_sym_offsetof] = ACTIONS(1336),
    [anon_sym__Generic] = ACTIONS(1336),
    [anon_sym_asm] = ACTIONS(1336),
    [anon_sym___asm__] = ACTIONS(1336),
    [sym_number_literal] = ACTIONS(1338),
    [anon_sym_L_SQUOTE] = ACTIONS(1338),
    [anon_sym_u_SQUOTE] = ACTIONS(1338),
    [anon_sym_U_SQUOTE] = ACTIONS(1338),
    [anon_sym_u8_SQUOTE] = ACTIONS(1338),
    [anon_sym_SQUOTE] = ACTIONS(1338),
    [anon_sym_L_DQUOTE] = ACTIONS(1338),
    [anon_sym_u_DQUOTE] = ACTIONS(1338),
    [anon_sym_U_DQUOTE] = ACTIONS(1338),
    [anon_sym_u8_DQUOTE] = ACTIONS(1338),
    [anon_sym_DQUOTE] = ACTIONS(1338),
    [sym_true] = ACTIONS(1336),
    [sym_false] = ACTIONS(1336),
    [anon_sym_NULL] = ACTIONS(1336),
    [anon_sym_nullptr] = ACTIONS(1336),
    [sym_comment] = ACTIONS(3),
  },
  [433] = {
    [ts_builtin_sym_end] = ACTIONS(1238),
    [sym_identifier] = ACTIONS(1236),
    [aux_sym_preproc_include_token1] = ACTIONS(1236),
    [aux_sym_preproc_def_token1] = ACTIONS(1236),
    [aux_sym_preproc_if_token1] = ACTIONS(1236),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1236),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1236),
    [sym_preproc_directive] = ACTIONS(1236),
    [anon_sym_LPAREN2] = ACTIONS(1238),
    [anon_sym_BANG] = ACTIONS(1238),
    [anon_sym_TILDE] = ACTIONS(1238),
    [anon_sym_DASH] = ACTIONS(1236),
    [anon_sym_PLUS] = ACTIONS(1236),
    [anon_sym_STAR] = ACTIONS(1238),
    [anon_sym_AMP] = ACTIONS(1238),
    [anon_sym___extension__] = ACTIONS(1236),
    [anon_sym_typedef] = ACTIONS(1236),
    [anon_sym_extern] = ACTIONS(1236),
    [anon_sym___attribute__] = ACTIONS(1236),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1238),
    [anon_sym___declspec] = ACTIONS(1236),
    [anon_sym___cdecl] = ACTIONS(1236),
    [anon_sym___clrcall] = ACTIONS(1236),
    [anon_sym___stdcall] = ACTIONS(1236),
    [anon_sym___fastcall] = ACTIONS(1236),
    [anon_sym___thiscall] = ACTIONS(1236),
    [anon_sym___vectorcall] = ACTIONS(1236),
    [anon_sym_LBRACE] = ACTIONS(1238),
    [anon_sym_signed] = ACTIONS(1236),
    [anon_sym_unsigned] = ACTIONS(1236),
    [anon_sym_long] = ACTIONS(1236),
    [anon_sym_short] = ACTIONS(1236),
    [anon_sym_static] = ACTIONS(1236),
    [anon_sym_auto] = ACTIONS(1236),
    [anon_sym_register] = ACTIONS(1236),
    [anon_sym_inline] = ACTIONS(1236),
    [anon_sym___inline] = ACTIONS(1236),
    [anon_sym___inline__] = ACTIONS(1236),
    [anon_sym___forceinline] = ACTIONS(1236),
    [anon_sym_thread_local] = ACTIONS(1236),
    [anon_sym___thread] = ACTIONS(1236),
    [anon_sym_const] = ACTIONS(1236),
    [anon_sym_constexpr] = ACTIONS(1236),
    [anon_sym_volatile] = ACTIONS(1236),
    [anon_sym_restrict] = ACTIONS(1236),
    [anon_sym___restrict__] = ACTIONS(1236),
    [anon_sym__Atomic] = ACTIONS(1236),
    [anon_sym__Noreturn] = ACTIONS(1236),
    [anon_sym_noreturn] = ACTIONS(1236),
    [sym_primitive_type] = ACTIONS(1236),
    [anon_sym_enum] = ACTIONS(1236),
    [anon_sym_struct] = ACTIONS(1236),
    [anon_sym_union] = ACTIONS(1236),
    [anon_sym_if] = ACTIONS(1236),
    [anon_sym_switch] = ACTIONS(1236),
    [anon_sym_case] = ACTIONS(1236),
    [anon_sym_default] = ACTIONS(1236),
    [anon_sym_while] = ACTIONS(1236),
    [anon_sym_do] = ACTIONS(1236),
    [anon_sym_for] = ACTIONS(1236),
    [anon_sym_return] = ACTIONS(1236),
    [anon_sym_break] = ACTIONS(1236),
    [anon_sym_continue] = ACTIONS(1236),
    [anon_sym_goto] = ACTIONS(1236),
    [anon_sym_DASH_DASH] = ACTIONS(1238),
    [anon_sym_PLUS_PLUS] = ACTIONS(1238),
    [anon_sym_sizeof] = ACTIONS(1236),
    [anon_sym___alignof__] = ACTIONS(1236),
    [anon_sym___alignof] = ACTIONS(1236),
    [anon_sym__alignof] = ACTIONS(1236),
    [anon_sym_alignof] = ACTIONS(1236),
    [anon_sym__Alignof] = ACTIONS(1236),
    [anon_sym_offsetof] = ACTIONS(1236),
    [anon_sym__Generic] = ACTIONS(1236),
    [anon_sym_asm] = ACTIONS(1236),
    [anon_sym___asm__] = ACTIONS(1236),
    [sym_number_literal] = ACTIONS(1238),
    [anon_sym_L_SQUOTE] = ACTIONS(1238),
    [anon_sym_u_SQUOTE] = ACTIONS(1238),
    [anon_sym_U_SQUOTE] = ACTIONS(1238),
    [anon_sym_u8_SQUOTE] = ACTIONS(1238),
    [anon_sym_SQUOTE] = ACTIONS(1238),
    [anon_sym_L_DQUOTE] = ACTIONS(1238),
    [anon_sym_u_DQUOTE] = ACTIONS(1238),
    [anon_sym_U_DQUOTE] = ACTIONS(1238),
    [anon_sym_u8_DQUOTE] = ACTIONS(1238),
    [anon_sym_DQUOTE] = ACTIONS(1238),
    [sym_true] = ACTIONS(1236),
    [sym_false] = ACTIONS(1236),
    [anon_sym_NULL] = ACTIONS(1236),
    [anon_sym_nullptr] = ACTIONS(1236),
    [sym_comment] = ACTIONS(3),
  },
  [434] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1885),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [435] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1977),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [436] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1802),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [437] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1782),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [438] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1809),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [439] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1880),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [440] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1956),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [441] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1925),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [442] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1985),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [443] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1853),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [444] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1887),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [445] = {
    [sym_type_qualifier] = STATE(1102),
    [sym__type_specifier] = STATE(1123),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__expression] = STATE(1068),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_comma_expression] = STATE(1945),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_type_descriptor] = STATE(1946),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__type_definition_type_repeat1] = STATE(1102),
    [aux_sym_sized_type_specifier_repeat1] = STATE(1139),
    [sym_identifier] = ACTIONS(1665),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_signed] = ACTIONS(1667),
    [anon_sym_unsigned] = ACTIONS(1667),
    [anon_sym_long] = ACTIONS(1667),
    [anon_sym_short] = ACTIONS(1667),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(1669),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [446] = {
    [sym_identifier] = ACTIONS(1671),
    [anon_sym_COMMA] = ACTIONS(1673),
    [anon_sym_RPAREN] = ACTIONS(1673),
    [anon_sym_LPAREN2] = ACTIONS(1673),
    [anon_sym_BANG] = ACTIONS(1673),
    [anon_sym_TILDE] = ACTIONS(1673),
    [anon_sym_DASH] = ACTIONS(1671),
    [anon_sym_PLUS] = ACTIONS(1671),
    [anon_sym_STAR] = ACTIONS(1673),
    [anon_sym_AMP] = ACTIONS(1673),
    [anon_sym_SEMI] = ACTIONS(1673),
    [anon_sym___extension__] = ACTIONS(1671),
    [anon_sym_extern] = ACTIONS(1671),
    [anon_sym___attribute__] = ACTIONS(1671),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1673),
    [anon_sym___declspec] = ACTIONS(1671),
    [anon_sym_LBRACE] = ACTIONS(1673),
    [anon_sym_signed] = ACTIONS(1671),
    [anon_sym_unsigned] = ACTIONS(1671),
    [anon_sym_long] = ACTIONS(1671),
    [anon_sym_short] = ACTIONS(1671),
    [anon_sym_LBRACK] = ACTIONS(1671),
    [anon_sym_EQ] = ACTIONS(1673),
    [anon_sym_static] = ACTIONS(1671),
    [anon_sym_auto] = ACTIONS(1671),
    [anon_sym_register] = ACTIONS(1671),
    [anon_sym_inline] = ACTIONS(1671),
    [anon_sym___inline] = ACTIONS(1671),
    [anon_sym___inline__] = ACTIONS(1671),
    [anon_sym___forceinline] = ACTIONS(1671),
    [anon_sym_thread_local] = ACTIONS(1671),
    [anon_sym___thread] = ACTIONS(1671),
    [anon_sym_const] = ACTIONS(1671),
    [anon_sym_constexpr] = ACTIONS(1671),
    [anon_sym_volatile] = ACTIONS(1671),
    [anon_sym_restrict] = ACTIONS(1671),
    [anon_sym___restrict__] = ACTIONS(1671),
    [anon_sym__Atomic] = ACTIONS(1671),
    [anon_sym__Noreturn] = ACTIONS(1671),
    [anon_sym_noreturn] = ACTIONS(1671),
    [sym_primitive_type] = ACTIONS(1671),
    [anon_sym_enum] = ACTIONS(1671),
    [anon_sym_COLON] = ACTIONS(1673),
    [anon_sym_struct] = ACTIONS(1671),
    [anon_sym_union] = ACTIONS(1671),
    [anon_sym_if] = ACTIONS(1671),
    [anon_sym_switch] = ACTIONS(1671),
    [anon_sym_case] = ACTIONS(1671),
    [anon_sym_default] = ACTIONS(1671),
    [anon_sym_while] = ACTIONS(1671),
    [anon_sym_do] = ACTIONS(1671),
    [anon_sym_for] = ACTIONS(1671),
    [anon_sym_return] = ACTIONS(1671),
    [anon_sym_break] = ACTIONS(1671),
    [anon_sym_continue] = ACTIONS(1671),
    [anon_sym_goto] = ACTIONS(1671),
    [anon_sym___try] = ACTIONS(1671),
    [anon_sym___leave] = ACTIONS(1671),
    [anon_sym_DASH_DASH] = ACTIONS(1673),
    [anon_sym_PLUS_PLUS] = ACTIONS(1673),
    [anon_sym_sizeof] = ACTIONS(1671),
    [anon_sym___alignof__] = ACTIONS(1671),
    [anon_sym___alignof] = ACTIONS(1671),
    [anon_sym__alignof] = ACTIONS(1671),
    [anon_sym_alignof] = ACTIONS(1671),
    [anon_sym__Alignof] = ACTIONS(1671),
    [anon_sym_offsetof] = ACTIONS(1671),
    [anon_sym__Generic] = ACTIONS(1671),
    [anon_sym_asm] = ACTIONS(1671),
    [anon_sym___asm__] = ACTIONS(1671),
    [sym_number_literal] = ACTIONS(1673),
    [anon_sym_L_SQUOTE] = ACTIONS(1673),
    [anon_sym_u_SQUOTE] = ACTIONS(1673),
    [anon_sym_U_SQUOTE] = ACTIONS(1673),
    [anon_sym_u8_SQUOTE] = ACTIONS(1673),
    [anon_sym_SQUOTE] = ACTIONS(1673),
    [anon_sym_L_DQUOTE] = ACTIONS(1673),
    [anon_sym_u_DQUOTE] = ACTIONS(1673),
    [anon_sym_U_DQUOTE] = ACTIONS(1673),
    [anon_sym_u8_DQUOTE] = ACTIONS(1673),
    [anon_sym_DQUOTE] = ACTIONS(1673),
    [sym_true] = ACTIONS(1671),
    [sym_false] = ACTIONS(1671),
    [anon_sym_NULL] = ACTIONS(1671),
    [anon_sym_nullptr] = ACTIONS(1671),
    [sym_comment] = ACTIONS(3),
  },
  [447] = {
    [sym_identifier] = ACTIONS(1675),
    [anon_sym_COMMA] = ACTIONS(1677),
    [anon_sym_RPAREN] = ACTIONS(1677),
    [anon_sym_LPAREN2] = ACTIONS(1677),
    [anon_sym_BANG] = ACTIONS(1677),
    [anon_sym_TILDE] = ACTIONS(1677),
    [anon_sym_DASH] = ACTIONS(1675),
    [anon_sym_PLUS] = ACTIONS(1675),
    [anon_sym_STAR] = ACTIONS(1677),
    [anon_sym_AMP] = ACTIONS(1677),
    [anon_sym_SEMI] = ACTIONS(1677),
    [anon_sym___extension__] = ACTIONS(1675),
    [anon_sym_extern] = ACTIONS(1675),
    [anon_sym___attribute__] = ACTIONS(1675),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1677),
    [anon_sym___declspec] = ACTIONS(1675),
    [anon_sym_LBRACE] = ACTIONS(1677),
    [anon_sym_signed] = ACTIONS(1675),
    [anon_sym_unsigned] = ACTIONS(1675),
    [anon_sym_long] = ACTIONS(1675),
    [anon_sym_short] = ACTIONS(1675),
    [anon_sym_LBRACK] = ACTIONS(1675),
    [anon_sym_EQ] = ACTIONS(1677),
    [anon_sym_static] = ACTIONS(1675),
    [anon_sym_auto] = ACTIONS(1675),
    [anon_sym_register] = ACTIONS(1675),
    [anon_sym_inline] = ACTIONS(1675),
    [anon_sym___inline] = ACTIONS(1675),
    [anon_sym___inline__] = ACTIONS(1675),
    [anon_sym___forceinline] = ACTIONS(1675),
    [anon_sym_thread_local] = ACTIONS(1675),
    [anon_sym___thread] = ACTIONS(1675),
    [anon_sym_const] = ACTIONS(1675),
    [anon_sym_constexpr] = ACTIONS(1675),
    [anon_sym_volatile] = ACTIONS(1675),
    [anon_sym_restrict] = ACTIONS(1675),
    [anon_sym___restrict__] = ACTIONS(1675),
    [anon_sym__Atomic] = ACTIONS(1675),
    [anon_sym__Noreturn] = ACTIONS(1675),
    [anon_sym_noreturn] = ACTIONS(1675),
    [sym_primitive_type] = ACTIONS(1675),
    [anon_sym_enum] = ACTIONS(1675),
    [anon_sym_COLON] = ACTIONS(1677),
    [anon_sym_struct] = ACTIONS(1675),
    [anon_sym_union] = ACTIONS(1675),
    [anon_sym_if] = ACTIONS(1675),
    [anon_sym_switch] = ACTIONS(1675),
    [anon_sym_case] = ACTIONS(1675),
    [anon_sym_default] = ACTIONS(1675),
    [anon_sym_while] = ACTIONS(1675),
    [anon_sym_do] = ACTIONS(1675),
    [anon_sym_for] = ACTIONS(1675),
    [anon_sym_return] = ACTIONS(1675),
    [anon_sym_break] = ACTIONS(1675),
    [anon_sym_continue] = ACTIONS(1675),
    [anon_sym_goto] = ACTIONS(1675),
    [anon_sym___try] = ACTIONS(1675),
    [anon_sym___leave] = ACTIONS(1675),
    [anon_sym_DASH_DASH] = ACTIONS(1677),
    [anon_sym_PLUS_PLUS] = ACTIONS(1677),
    [anon_sym_sizeof] = ACTIONS(1675),
    [anon_sym___alignof__] = ACTIONS(1675),
    [anon_sym___alignof] = ACTIONS(1675),
    [anon_sym__alignof] = ACTIONS(1675),
    [anon_sym_alignof] = ACTIONS(1675),
    [anon_sym__Alignof] = ACTIONS(1675),
    [anon_sym_offsetof] = ACTIONS(1675),
    [anon_sym__Generic] = ACTIONS(1675),
    [anon_sym_asm] = ACTIONS(1675),
    [anon_sym___asm__] = ACTIONS(1675),
    [sym_number_literal] = ACTIONS(1677),
    [anon_sym_L_SQUOTE] = ACTIONS(1677),
    [anon_sym_u_SQUOTE] = ACTIONS(1677),
    [anon_sym_U_SQUOTE] = ACTIONS(1677),
    [anon_sym_u8_SQUOTE] = ACTIONS(1677),
    [anon_sym_SQUOTE] = ACTIONS(1677),
    [anon_sym_L_DQUOTE] = ACTIONS(1677),
    [anon_sym_u_DQUOTE] = ACTIONS(1677),
    [anon_sym_U_DQUOTE] = ACTIONS(1677),
    [anon_sym_u8_DQUOTE] = ACTIONS(1677),
    [anon_sym_DQUOTE] = ACTIONS(1677),
    [sym_true] = ACTIONS(1675),
    [sym_false] = ACTIONS(1675),
    [anon_sym_NULL] = ACTIONS(1675),
    [anon_sym_nullptr] = ACTIONS(1675),
    [sym_comment] = ACTIONS(3),
  },
  [448] = {
    [sym__expression] = STATE(722),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_initializer_list] = STATE(714),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_identifier] = ACTIONS(1679),
    [anon_sym_COMMA] = ACTIONS(1352),
    [anon_sym_RPAREN] = ACTIONS(1352),
    [anon_sym_LPAREN2] = ACTIONS(1352),
    [anon_sym_BANG] = ACTIONS(23),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(1350),
    [anon_sym_PLUS] = ACTIONS(1350),
    [anon_sym_STAR] = ACTIONS(1352),
    [anon_sym_SLASH] = ACTIONS(1350),
    [anon_sym_PERCENT] = ACTIONS(1352),
    [anon_sym_PIPE_PIPE] = ACTIONS(1352),
    [anon_sym_AMP_AMP] = ACTIONS(1352),
    [anon_sym_PIPE] = ACTIONS(1350),
    [anon_sym_CARET] = ACTIONS(1352),
    [anon_sym_AMP] = ACTIONS(1350),
    [anon_sym_EQ_EQ] = ACTIONS(1352),
    [anon_sym_BANG_EQ] = ACTIONS(1352),
    [anon_sym_GT] = ACTIONS(1350),
    [anon_sym_GT_EQ] = ACTIONS(1352),
    [anon_sym_LT_EQ] = ACTIONS(1352),
    [anon_sym_LT] = ACTIONS(1350),
    [anon_sym_LT_LT] = ACTIONS(1352),
    [anon_sym_GT_GT] = ACTIONS(1352),
    [anon_sym_SEMI] = ACTIONS(1352),
    [anon_sym___attribute__] = ACTIONS(1350),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_RBRACE] = ACTIONS(1352),
    [anon_sym_LBRACK] = ACTIONS(1352),
    [anon_sym_COLON] = ACTIONS(1352),
    [anon_sym_QMARK] = ACTIONS(1352),
    [anon_sym_DASH_DASH] = ACTIONS(1352),
    [anon_sym_PLUS_PLUS] = ACTIONS(1352),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1350),
    [anon_sym_DASH_GT] = ACTIONS(1352),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [449] = {
    [sym__expression] = STATE(722),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(799),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(799),
    [sym_call_expression] = STATE(799),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(799),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(799),
    [sym_initializer_list] = STATE(714),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(735),
    [sym_null] = STATE(700),
    [sym_identifier] = ACTIONS(1350),
    [anon_sym_COMMA] = ACTIONS(1352),
    [aux_sym_preproc_if_token2] = ACTIONS(1352),
    [aux_sym_preproc_else_token1] = ACTIONS(1352),
    [aux_sym_preproc_elif_token1] = ACTIONS(1350),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1352),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1352),
    [anon_sym_LPAREN2] = ACTIONS(1352),
    [anon_sym_BANG] = ACTIONS(1681),
    [anon_sym_TILDE] = ACTIONS(1683),
    [anon_sym_DASH] = ACTIONS(1350),
    [anon_sym_PLUS] = ACTIONS(1350),
    [anon_sym_STAR] = ACTIONS(1352),
    [anon_sym_SLASH] = ACTIONS(1350),
    [anon_sym_PERCENT] = ACTIONS(1352),
    [anon_sym_PIPE_PIPE] = ACTIONS(1352),
    [anon_sym_AMP_AMP] = ACTIONS(1352),
    [anon_sym_PIPE] = ACTIONS(1350),
    [anon_sym_CARET] = ACTIONS(1352),
    [anon_sym_AMP] = ACTIONS(1350),
    [anon_sym_EQ_EQ] = ACTIONS(1352),
    [anon_sym_BANG_EQ] = ACTIONS(1352),
    [anon_sym_GT] = ACTIONS(1350),
    [anon_sym_GT_EQ] = ACTIONS(1352),
    [anon_sym_LT_EQ] = ACTIONS(1352),
    [anon_sym_LT] = ACTIONS(1350),
    [anon_sym_LT_LT] = ACTIONS(1352),
    [anon_sym_GT_GT] = ACTIONS(1352),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_LBRACK] = ACTIONS(1352),
    [anon_sym_QMARK] = ACTIONS(1352),
    [anon_sym_DASH_DASH] = ACTIONS(1352),
    [anon_sym_PLUS_PLUS] = ACTIONS(1352),
    [anon_sym_sizeof] = ACTIONS(1685),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1350),
    [anon_sym_DASH_GT] = ACTIONS(1352),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [450] = {
    [sym__expression] = STATE(869),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_initializer_list] = STATE(714),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1352),
    [anon_sym_LPAREN2] = ACTIONS(1352),
    [anon_sym_BANG] = ACTIONS(1689),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1350),
    [anon_sym_PLUS] = ACTIONS(1350),
    [anon_sym_STAR] = ACTIONS(1352),
    [anon_sym_SLASH] = ACTIONS(1350),
    [anon_sym_PERCENT] = ACTIONS(1352),
    [anon_sym_PIPE_PIPE] = ACTIONS(1352),
    [anon_sym_AMP_AMP] = ACTIONS(1352),
    [anon_sym_PIPE] = ACTIONS(1350),
    [anon_sym_CARET] = ACTIONS(1352),
    [anon_sym_AMP] = ACTIONS(1350),
    [anon_sym_EQ_EQ] = ACTIONS(1352),
    [anon_sym_BANG_EQ] = ACTIONS(1352),
    [anon_sym_GT] = ACTIONS(1350),
    [anon_sym_GT_EQ] = ACTIONS(1352),
    [anon_sym_LT_EQ] = ACTIONS(1352),
    [anon_sym_LT] = ACTIONS(1350),
    [anon_sym_LT_LT] = ACTIONS(1352),
    [anon_sym_GT_GT] = ACTIONS(1352),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_LBRACK] = ACTIONS(1352),
    [anon_sym_RBRACK] = ACTIONS(1352),
    [anon_sym_QMARK] = ACTIONS(1352),
    [anon_sym_DASH_DASH] = ACTIONS(1352),
    [anon_sym_PLUS_PLUS] = ACTIONS(1352),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1350),
    [anon_sym_DASH_GT] = ACTIONS(1352),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [451] = {
    [sym_else_clause] = STATE(220),
    [sym_identifier] = ACTIONS(1106),
    [anon_sym_LPAREN2] = ACTIONS(1108),
    [anon_sym_BANG] = ACTIONS(1108),
    [anon_sym_TILDE] = ACTIONS(1108),
    [anon_sym_DASH] = ACTIONS(1106),
    [anon_sym_PLUS] = ACTIONS(1106),
    [anon_sym_STAR] = ACTIONS(1108),
    [anon_sym_AMP] = ACTIONS(1108),
    [anon_sym_SEMI] = ACTIONS(1108),
    [anon_sym___extension__] = ACTIONS(1106),
    [anon_sym_typedef] = ACTIONS(1106),
    [anon_sym_extern] = ACTIONS(1106),
    [anon_sym___attribute__] = ACTIONS(1106),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1108),
    [anon_sym___declspec] = ACTIONS(1106),
    [anon_sym_LBRACE] = ACTIONS(1108),
    [anon_sym_signed] = ACTIONS(1106),
    [anon_sym_unsigned] = ACTIONS(1106),
    [anon_sym_long] = ACTIONS(1106),
    [anon_sym_short] = ACTIONS(1106),
    [anon_sym_static] = ACTIONS(1106),
    [anon_sym_auto] = ACTIONS(1106),
    [anon_sym_register] = ACTIONS(1106),
    [anon_sym_inline] = ACTIONS(1106),
    [anon_sym___inline] = ACTIONS(1106),
    [anon_sym___inline__] = ACTIONS(1106),
    [anon_sym___forceinline] = ACTIONS(1106),
    [anon_sym_thread_local] = ACTIONS(1106),
    [anon_sym___thread] = ACTIONS(1106),
    [anon_sym_const] = ACTIONS(1106),
    [anon_sym_constexpr] = ACTIONS(1106),
    [anon_sym_volatile] = ACTIONS(1106),
    [anon_sym_restrict] = ACTIONS(1106),
    [anon_sym___restrict__] = ACTIONS(1106),
    [anon_sym__Atomic] = ACTIONS(1106),
    [anon_sym__Noreturn] = ACTIONS(1106),
    [anon_sym_noreturn] = ACTIONS(1106),
    [sym_primitive_type] = ACTIONS(1106),
    [anon_sym_enum] = ACTIONS(1106),
    [anon_sym_struct] = ACTIONS(1106),
    [anon_sym_union] = ACTIONS(1106),
    [anon_sym_if] = ACTIONS(1106),
    [anon_sym_else] = ACTIONS(1695),
    [anon_sym_switch] = ACTIONS(1106),
    [anon_sym_while] = ACTIONS(1106),
    [anon_sym_do] = ACTIONS(1106),
    [anon_sym_for] = ACTIONS(1106),
    [anon_sym_return] = ACTIONS(1106),
    [anon_sym_break] = ACTIONS(1106),
    [anon_sym_continue] = ACTIONS(1106),
    [anon_sym_goto] = ACTIONS(1106),
    [anon_sym___try] = ACTIONS(1106),
    [anon_sym___leave] = ACTIONS(1106),
    [anon_sym_DASH_DASH] = ACTIONS(1108),
    [anon_sym_PLUS_PLUS] = ACTIONS(1108),
    [anon_sym_sizeof] = ACTIONS(1106),
    [anon_sym___alignof__] = ACTIONS(1106),
    [anon_sym___alignof] = ACTIONS(1106),
    [anon_sym__alignof] = ACTIONS(1106),
    [anon_sym_alignof] = ACTIONS(1106),
    [anon_sym__Alignof] = ACTIONS(1106),
    [anon_sym_offsetof] = ACTIONS(1106),
    [anon_sym__Generic] = ACTIONS(1106),
    [anon_sym_asm] = ACTIONS(1106),
    [anon_sym___asm__] = ACTIONS(1106),
    [sym_number_literal] = ACTIONS(1108),
    [anon_sym_L_SQUOTE] = ACTIONS(1108),
    [anon_sym_u_SQUOTE] = ACTIONS(1108),
    [anon_sym_U_SQUOTE] = ACTIONS(1108),
    [anon_sym_u8_SQUOTE] = ACTIONS(1108),
    [anon_sym_SQUOTE] = ACTIONS(1108),
    [anon_sym_L_DQUOTE] = ACTIONS(1108),
    [anon_sym_u_DQUOTE] = ACTIONS(1108),
    [anon_sym_U_DQUOTE] = ACTIONS(1108),
    [anon_sym_u8_DQUOTE] = ACTIONS(1108),
    [anon_sym_DQUOTE] = ACTIONS(1108),
    [sym_true] = ACTIONS(1106),
    [sym_false] = ACTIONS(1106),
    [anon_sym_NULL] = ACTIONS(1106),
    [anon_sym_nullptr] = ACTIONS(1106),
    [sym_comment] = ACTIONS(3),
  },
  [452] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_COMMA] = ACTIONS(1699),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1699),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_COLON] = ACTIONS(1716),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [453] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_COMMA] = ACTIONS(1699),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1720),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_COLON] = ACTIONS(1723),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [454] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_COMMA] = ACTIONS(1699),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1720),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_COLON] = ACTIONS(1716),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [455] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_COMMA] = ACTIONS(1699),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1699),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_COLON] = ACTIONS(1725),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [456] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_COMMA] = ACTIONS(1699),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1720),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_COLON] = ACTIONS(1727),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [457] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_COMMA] = ACTIONS(1699),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1699),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_COLON] = ACTIONS(1727),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [458] = {
    [sym_identifier] = ACTIONS(1729),
    [anon_sym_LPAREN2] = ACTIONS(1732),
    [anon_sym_BANG] = ACTIONS(1732),
    [anon_sym_TILDE] = ACTIONS(1732),
    [anon_sym_DASH] = ACTIONS(1734),
    [anon_sym_PLUS] = ACTIONS(1734),
    [anon_sym_STAR] = ACTIONS(1732),
    [anon_sym_AMP] = ACTIONS(1732),
    [anon_sym_SEMI] = ACTIONS(1732),
    [anon_sym___extension__] = ACTIONS(1736),
    [anon_sym_extern] = ACTIONS(1736),
    [anon_sym___attribute__] = ACTIONS(1736),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1738),
    [anon_sym___declspec] = ACTIONS(1736),
    [anon_sym_LBRACE] = ACTIONS(1732),
    [anon_sym_signed] = ACTIONS(1736),
    [anon_sym_unsigned] = ACTIONS(1736),
    [anon_sym_long] = ACTIONS(1736),
    [anon_sym_short] = ACTIONS(1736),
    [anon_sym_static] = ACTIONS(1736),
    [anon_sym_auto] = ACTIONS(1736),
    [anon_sym_register] = ACTIONS(1736),
    [anon_sym_inline] = ACTIONS(1736),
    [anon_sym___inline] = ACTIONS(1736),
    [anon_sym___inline__] = ACTIONS(1736),
    [anon_sym___forceinline] = ACTIONS(1736),
    [anon_sym_thread_local] = ACTIONS(1736),
    [anon_sym___thread] = ACTIONS(1736),
    [anon_sym_const] = ACTIONS(1736),
    [anon_sym_constexpr] = ACTIONS(1736),
    [anon_sym_volatile] = ACTIONS(1736),
    [anon_sym_restrict] = ACTIONS(1736),
    [anon_sym___restrict__] = ACTIONS(1736),
    [anon_sym__Atomic] = ACTIONS(1736),
    [anon_sym__Noreturn] = ACTIONS(1736),
    [anon_sym_noreturn] = ACTIONS(1736),
    [sym_primitive_type] = ACTIONS(1736),
    [anon_sym_enum] = ACTIONS(1736),
    [anon_sym_struct] = ACTIONS(1736),
    [anon_sym_union] = ACTIONS(1736),
    [anon_sym_if] = ACTIONS(1734),
    [anon_sym_switch] = ACTIONS(1734),
    [anon_sym_case] = ACTIONS(1734),
    [anon_sym_default] = ACTIONS(1734),
    [anon_sym_while] = ACTIONS(1734),
    [anon_sym_do] = ACTIONS(1734),
    [anon_sym_for] = ACTIONS(1734),
    [anon_sym_return] = ACTIONS(1734),
    [anon_sym_break] = ACTIONS(1734),
    [anon_sym_continue] = ACTIONS(1734),
    [anon_sym_goto] = ACTIONS(1734),
    [anon_sym___try] = ACTIONS(1734),
    [anon_sym___leave] = ACTIONS(1734),
    [anon_sym_DASH_DASH] = ACTIONS(1732),
    [anon_sym_PLUS_PLUS] = ACTIONS(1732),
    [anon_sym_sizeof] = ACTIONS(1734),
    [anon_sym___alignof__] = ACTIONS(1734),
    [anon_sym___alignof] = ACTIONS(1734),
    [anon_sym__alignof] = ACTIONS(1734),
    [anon_sym_alignof] = ACTIONS(1734),
    [anon_sym__Alignof] = ACTIONS(1734),
    [anon_sym_offsetof] = ACTIONS(1734),
    [anon_sym__Generic] = ACTIONS(1734),
    [anon_sym_asm] = ACTIONS(1734),
    [anon_sym___asm__] = ACTIONS(1734),
    [sym_number_literal] = ACTIONS(1732),
    [anon_sym_L_SQUOTE] = ACTIONS(1732),
    [anon_sym_u_SQUOTE] = ACTIONS(1732),
    [anon_sym_U_SQUOTE] = ACTIONS(1732),
    [anon_sym_u8_SQUOTE] = ACTIONS(1732),
    [anon_sym_SQUOTE] = ACTIONS(1732),
    [anon_sym_L_DQUOTE] = ACTIONS(1732),
    [anon_sym_u_DQUOTE] = ACTIONS(1732),
    [anon_sym_U_DQUOTE] = ACTIONS(1732),
    [anon_sym_u8_DQUOTE] = ACTIONS(1732),
    [anon_sym_DQUOTE] = ACTIONS(1732),
    [sym_true] = ACTIONS(1734),
    [sym_false] = ACTIONS(1734),
    [anon_sym_NULL] = ACTIONS(1734),
    [anon_sym_nullptr] = ACTIONS(1734),
    [sym_comment] = ACTIONS(3),
  },
  [459] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_COMMA] = ACTIONS(1699),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1699),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_COLON] = ACTIONS(1723),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [460] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_COMMA] = ACTIONS(1699),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1699),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_COLON] = ACTIONS(1741),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [461] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_COMMA] = ACTIONS(1699),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1699),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [462] = {
    [sym_string_literal] = STATE(662),
    [aux_sym_sized_type_specifier_repeat1] = STATE(811),
    [sym_identifier] = ACTIONS(1697),
    [anon_sym_LPAREN2] = ACTIONS(1701),
    [anon_sym_DASH] = ACTIONS(1705),
    [anon_sym_PLUS] = ACTIONS(1705),
    [anon_sym_STAR] = ACTIONS(1707),
    [anon_sym_SLASH] = ACTIONS(1705),
    [anon_sym_PERCENT] = ACTIONS(1705),
    [anon_sym_PIPE_PIPE] = ACTIONS(1699),
    [anon_sym_AMP_AMP] = ACTIONS(1699),
    [anon_sym_PIPE] = ACTIONS(1705),
    [anon_sym_CARET] = ACTIONS(1705),
    [anon_sym_AMP] = ACTIONS(1705),
    [anon_sym_EQ_EQ] = ACTIONS(1699),
    [anon_sym_BANG_EQ] = ACTIONS(1699),
    [anon_sym_GT] = ACTIONS(1705),
    [anon_sym_GT_EQ] = ACTIONS(1699),
    [anon_sym_LT_EQ] = ACTIONS(1699),
    [anon_sym_LT] = ACTIONS(1705),
    [anon_sym_LT_LT] = ACTIONS(1705),
    [anon_sym_GT_GT] = ACTIONS(1705),
    [anon_sym_SEMI] = ACTIONS(1720),
    [anon_sym___extension__] = ACTIONS(1697),
    [anon_sym_extern] = ACTIONS(1697),
    [anon_sym___attribute__] = ACTIONS(1697),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1710),
    [anon_sym___declspec] = ACTIONS(1697),
    [anon_sym___based] = ACTIONS(1697),
    [anon_sym___cdecl] = ACTIONS(1697),
    [anon_sym___clrcall] = ACTIONS(1697),
    [anon_sym___stdcall] = ACTIONS(1697),
    [anon_sym___fastcall] = ACTIONS(1697),
    [anon_sym___thiscall] = ACTIONS(1697),
    [anon_sym___vectorcall] = ACTIONS(1697),
    [anon_sym_signed] = ACTIONS(1712),
    [anon_sym_unsigned] = ACTIONS(1712),
    [anon_sym_long] = ACTIONS(1712),
    [anon_sym_short] = ACTIONS(1712),
    [anon_sym_LBRACK] = ACTIONS(1705),
    [anon_sym_EQ] = ACTIONS(1714),
    [anon_sym_static] = ACTIONS(1697),
    [anon_sym_auto] = ACTIONS(1697),
    [anon_sym_register] = ACTIONS(1697),
    [anon_sym_inline] = ACTIONS(1697),
    [anon_sym___inline] = ACTIONS(1697),
    [anon_sym___inline__] = ACTIONS(1697),
    [anon_sym___forceinline] = ACTIONS(1697),
    [anon_sym_thread_local] = ACTIONS(1697),
    [anon_sym___thread] = ACTIONS(1697),
    [anon_sym_const] = ACTIONS(1697),
    [anon_sym_constexpr] = ACTIONS(1697),
    [anon_sym_volatile] = ACTIONS(1697),
    [anon_sym_restrict] = ACTIONS(1697),
    [anon_sym___restrict__] = ACTIONS(1697),
    [anon_sym__Atomic] = ACTIONS(1697),
    [anon_sym__Noreturn] = ACTIONS(1697),
    [anon_sym_noreturn] = ACTIONS(1697),
    [anon_sym_COLON] = ACTIONS(1725),
    [anon_sym_QMARK] = ACTIONS(1699),
    [anon_sym_STAR_EQ] = ACTIONS(1718),
    [anon_sym_SLASH_EQ] = ACTIONS(1718),
    [anon_sym_PERCENT_EQ] = ACTIONS(1718),
    [anon_sym_PLUS_EQ] = ACTIONS(1718),
    [anon_sym_DASH_EQ] = ACTIONS(1718),
    [anon_sym_LT_LT_EQ] = ACTIONS(1718),
    [anon_sym_GT_GT_EQ] = ACTIONS(1718),
    [anon_sym_AMP_EQ] = ACTIONS(1718),
    [anon_sym_CARET_EQ] = ACTIONS(1718),
    [anon_sym_PIPE_EQ] = ACTIONS(1718),
    [anon_sym_DASH_DASH] = ACTIONS(1699),
    [anon_sym_PLUS_PLUS] = ACTIONS(1699),
    [anon_sym_DOT] = ACTIONS(1699),
    [anon_sym_DASH_GT] = ACTIONS(1699),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_comment] = ACTIONS(3),
  },
  [463] = {
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1184),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_based_modifier] = STATE(1803),
    [sym_ms_call_modifier] = STATE(1275),
    [sym__declarator] = STATE(1421),
    [sym__abstract_declarator] = STATE(1523),
    [sym_parenthesized_declarator] = STATE(1354),
    [sym_abstract_parenthesized_declarator] = STATE(1496),
    [sym_attributed_declarator] = STATE(1354),
    [sym_pointer_declarator] = STATE(1354),
    [sym_abstract_pointer_declarator] = STATE(1496),
    [sym_function_declarator] = STATE(1354),
    [sym_abstract_function_declarator] = STATE(1496),
    [sym_array_declarator] = STATE(1354),
    [sym_abstract_array_declarator] = STATE(1496),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_variadic_parameter] = STATE(1670),
    [sym_parameter_list] = STATE(1486),
    [sym_parameter_declaration] = STATE(1670),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1743),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1745),
    [anon_sym_RPAREN] = ACTIONS(1747),
    [anon_sym_LPAREN2] = ACTIONS(1749),
    [anon_sym_STAR] = ACTIONS(1751),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___based] = ACTIONS(1753),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_LBRACK] = ACTIONS(1755),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [464] = {
    [sym_type_qualifier] = STATE(739),
    [sym__expression] = STATE(1095),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(739),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1759),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1765),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [465] = {
    [sym_type_qualifier] = STATE(464),
    [sym__expression] = STATE(1104),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(464),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1769),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1771),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [466] = {
    [sym_type_qualifier] = STATE(739),
    [sym__expression] = STATE(1114),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(739),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1773),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1775),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [467] = {
    [sym_type_qualifier] = STATE(739),
    [sym__expression] = STATE(1100),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(739),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1777),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1779),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [468] = {
    [sym_type_qualifier] = STATE(466),
    [sym__expression] = STATE(1091),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(466),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1781),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1783),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [469] = {
    [sym_type_qualifier] = STATE(739),
    [sym__expression] = STATE(1092),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(739),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1785),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1787),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [470] = {
    [sym_type_qualifier] = STATE(467),
    [sym__expression] = STATE(1093),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(467),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1789),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1791),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [471] = {
    [sym_type_qualifier] = STATE(469),
    [sym__expression] = STATE(1088),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(469),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1793),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1795),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [472] = {
    [sym_type_qualifier] = STATE(473),
    [sym__expression] = STATE(1094),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(473),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1797),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1799),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [473] = {
    [sym_type_qualifier] = STATE(739),
    [sym__expression] = STATE(1090),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(895),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(895),
    [sym_call_expression] = STATE(895),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(895),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(895),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym__type_definition_type_repeat1] = STATE(739),
    [sym_identifier] = ACTIONS(1687),
    [anon_sym_LPAREN2] = ACTIONS(1757),
    [anon_sym_BANG] = ACTIONS(1691),
    [anon_sym_TILDE] = ACTIONS(1691),
    [anon_sym_DASH] = ACTIONS(1689),
    [anon_sym_PLUS] = ACTIONS(1689),
    [anon_sym_STAR] = ACTIONS(1801),
    [anon_sym_AMP] = ACTIONS(1761),
    [anon_sym___extension__] = ACTIONS(1763),
    [anon_sym_RBRACK] = ACTIONS(1803),
    [anon_sym_const] = ACTIONS(1763),
    [anon_sym_constexpr] = ACTIONS(1763),
    [anon_sym_volatile] = ACTIONS(1763),
    [anon_sym_restrict] = ACTIONS(1763),
    [anon_sym___restrict__] = ACTIONS(1763),
    [anon_sym__Atomic] = ACTIONS(1763),
    [anon_sym__Noreturn] = ACTIONS(1763),
    [anon_sym_noreturn] = ACTIONS(1763),
    [anon_sym_DASH_DASH] = ACTIONS(1767),
    [anon_sym_PLUS_PLUS] = ACTIONS(1767),
    [anon_sym_sizeof] = ACTIONS(1693),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [474] = {
    [sym__expression] = STATE(1024),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_initializer_list] = STATE(1586),
    [sym_initializer_pair] = STATE(1586),
    [sym_subscript_designator] = STATE(1430),
    [sym_subscript_range_designator] = STATE(1430),
    [sym_field_designator] = STATE(1430),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_initializer_pair_repeat1] = STATE(1430),
    [sym_identifier] = ACTIONS(1805),
    [anon_sym_COMMA] = ACTIONS(1807),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_RBRACE] = ACTIONS(1809),
    [anon_sym_LBRACK] = ACTIONS(1811),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1813),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [475] = {
    [sym__expression] = STATE(1076),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_initializer_list] = STATE(1751),
    [sym_initializer_pair] = STATE(1751),
    [sym_subscript_designator] = STATE(1430),
    [sym_subscript_range_designator] = STATE(1430),
    [sym_field_designator] = STATE(1430),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_initializer_pair_repeat1] = STATE(1430),
    [sym_identifier] = ACTIONS(1805),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_RBRACE] = ACTIONS(1815),
    [anon_sym_LBRACK] = ACTIONS(1811),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1813),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [476] = {
    [sym__expression] = STATE(1076),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_initializer_list] = STATE(1751),
    [sym_initializer_pair] = STATE(1751),
    [sym_subscript_designator] = STATE(1430),
    [sym_subscript_range_designator] = STATE(1430),
    [sym_field_designator] = STATE(1430),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_initializer_pair_repeat1] = STATE(1430),
    [sym_identifier] = ACTIONS(1805),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_RBRACE] = ACTIONS(1817),
    [anon_sym_LBRACK] = ACTIONS(1811),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1813),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [477] = {
    [sym__expression] = STATE(1076),
    [sym__expression_not_binary] = STATE(700),
    [sym__string] = STATE(700),
    [sym_conditional_expression] = STATE(700),
    [sym_assignment_expression] = STATE(700),
    [sym_pointer_expression] = STATE(840),
    [sym_unary_expression] = STATE(700),
    [sym_binary_expression] = STATE(700),
    [sym_update_expression] = STATE(700),
    [sym_cast_expression] = STATE(700),
    [sym_sizeof_expression] = STATE(700),
    [sym_alignof_expression] = STATE(700),
    [sym_offsetof_expression] = STATE(700),
    [sym_generic_expression] = STATE(700),
    [sym_subscript_expression] = STATE(840),
    [sym_call_expression] = STATE(840),
    [sym_gnu_asm_expression] = STATE(700),
    [sym_field_expression] = STATE(840),
    [sym_compound_literal_expression] = STATE(700),
    [sym_parenthesized_expression] = STATE(840),
    [sym_initializer_list] = STATE(1751),
    [sym_initializer_pair] = STATE(1751),
    [sym_subscript_designator] = STATE(1430),
    [sym_subscript_range_designator] = STATE(1430),
    [sym_field_designator] = STATE(1430),
    [sym_char_literal] = STATE(700),
    [sym_concatenated_string] = STATE(700),
    [sym_string_literal] = STATE(683),
    [sym_null] = STATE(700),
    [aux_sym_initializer_pair_repeat1] = STATE(1430),
    [sym_identifier] = ACTIONS(1805),
    [anon_sym_LPAREN2] = ACTIONS(19),
    [anon_sym_BANG] = ACTIONS(21),
    [anon_sym_TILDE] = ACTIONS(21),
    [anon_sym_DASH] = ACTIONS(23),
    [anon_sym_PLUS] = ACTIONS(23),
    [anon_sym_STAR] = ACTIONS(25),
    [anon_sym_AMP] = ACTIONS(25),
    [anon_sym_LBRACE] = ACTIONS(1358),
    [anon_sym_LBRACK] = ACTIONS(1811),
    [anon_sym_DASH_DASH] = ACTIONS(79),
    [anon_sym_PLUS_PLUS] = ACTIONS(79),
    [anon_sym_sizeof] = ACTIONS(81),
    [anon_sym___alignof__] = ACTIONS(83),
    [anon_sym___alignof] = ACTIONS(83),
    [anon_sym__alignof] = ACTIONS(83),
    [anon_sym_alignof] = ACTIONS(83),
    [anon_sym__Alignof] = ACTIONS(83),
    [anon_sym_offsetof] = ACTIONS(85),
    [anon_sym__Generic] = ACTIONS(87),
    [anon_sym_asm] = ACTIONS(89),
    [anon_sym___asm__] = ACTIONS(89),
    [anon_sym_DOT] = ACTIONS(1813),
    [sym_number_literal] = ACTIONS(157),
    [anon_sym_L_SQUOTE] = ACTIONS(93),
    [anon_sym_u_SQUOTE] = ACTIONS(93),
    [anon_sym_U_SQUOTE] = ACTIONS(93),
    [anon_sym_u8_SQUOTE] = ACTIONS(93),
    [anon_sym_SQUOTE] = ACTIONS(93),
    [anon_sym_L_DQUOTE] = ACTIONS(95),
    [anon_sym_u_DQUOTE] = ACTIONS(95),
    [anon_sym_U_DQUOTE] = ACTIONS(95),
    [anon_sym_u8_DQUOTE] = ACTIONS(95),
    [anon_sym_DQUOTE] = ACTIONS(95),
    [sym_true] = ACTIONS(159),
    [sym_false] = ACTIONS(159),
    [anon_sym_NULL] = ACTIONS(99),
    [anon_sym_nullptr] = ACTIONS(99),
    [sym_comment] = ACTIONS(3),
  },
  [478] = {
    [sym_preproc_def] = STATE(486),
    [sym_preproc_function_def] = STATE(486),
    [sym_preproc_call] = STATE(486),
    [sym_preproc_if_in_field_declaration_list] = STATE(486),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(486),
    [sym_preproc_else_in_field_declaration_list] = STATE(1978),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1978),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1978),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(486),
    [sym_field_declaration] = STATE(486),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(486),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1825),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [479] = {
    [sym_preproc_def] = STATE(495),
    [sym_preproc_function_def] = STATE(495),
    [sym_preproc_call] = STATE(495),
    [sym_preproc_if_in_field_declaration_list] = STATE(495),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(495),
    [sym_preproc_else_in_field_declaration_list] = STATE(1866),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1866),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1866),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(495),
    [sym_field_declaration] = STATE(495),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(495),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1837),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [480] = {
    [sym_preproc_def] = STATE(491),
    [sym_preproc_function_def] = STATE(491),
    [sym_preproc_call] = STATE(491),
    [sym_preproc_if_in_field_declaration_list] = STATE(491),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(491),
    [sym_preproc_else_in_field_declaration_list] = STATE(1840),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1840),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1840),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(491),
    [sym_field_declaration] = STATE(491),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(491),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1839),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [481] = {
    [sym_preproc_def] = STATE(495),
    [sym_preproc_function_def] = STATE(495),
    [sym_preproc_call] = STATE(495),
    [sym_preproc_if_in_field_declaration_list] = STATE(495),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(495),
    [sym_preproc_else_in_field_declaration_list] = STATE(1929),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1929),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1929),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(495),
    [sym_field_declaration] = STATE(495),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(495),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1841),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [482] = {
    [sym_preproc_def] = STATE(479),
    [sym_preproc_function_def] = STATE(479),
    [sym_preproc_call] = STATE(479),
    [sym_preproc_if_in_field_declaration_list] = STATE(479),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(479),
    [sym_preproc_else_in_field_declaration_list] = STATE(1798),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1798),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1798),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(479),
    [sym_field_declaration] = STATE(479),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(479),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1843),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [483] = {
    [sym_preproc_def] = STATE(481),
    [sym_preproc_function_def] = STATE(481),
    [sym_preproc_call] = STATE(481),
    [sym_preproc_if_in_field_declaration_list] = STATE(481),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(481),
    [sym_preproc_else_in_field_declaration_list] = STATE(1831),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1831),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1831),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(481),
    [sym_field_declaration] = STATE(481),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(481),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1845),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [484] = {
    [sym_preproc_def] = STATE(485),
    [sym_preproc_function_def] = STATE(485),
    [sym_preproc_call] = STATE(485),
    [sym_preproc_if_in_field_declaration_list] = STATE(485),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(485),
    [sym_preproc_else_in_field_declaration_list] = STATE(1873),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1873),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1873),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(485),
    [sym_field_declaration] = STATE(485),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(485),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1847),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [485] = {
    [sym_preproc_def] = STATE(495),
    [sym_preproc_function_def] = STATE(495),
    [sym_preproc_call] = STATE(495),
    [sym_preproc_if_in_field_declaration_list] = STATE(495),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(495),
    [sym_preproc_else_in_field_declaration_list] = STATE(1981),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1981),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1981),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(495),
    [sym_field_declaration] = STATE(495),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(495),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1849),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [486] = {
    [sym_preproc_def] = STATE(495),
    [sym_preproc_function_def] = STATE(495),
    [sym_preproc_call] = STATE(495),
    [sym_preproc_if_in_field_declaration_list] = STATE(495),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(495),
    [sym_preproc_else_in_field_declaration_list] = STATE(1843),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1843),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1843),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(495),
    [sym_field_declaration] = STATE(495),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(495),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1851),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [487] = {
    [sym_preproc_def] = STATE(488),
    [sym_preproc_function_def] = STATE(488),
    [sym_preproc_call] = STATE(488),
    [sym_preproc_if_in_field_declaration_list] = STATE(488),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(488),
    [sym_preproc_else_in_field_declaration_list] = STATE(1987),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1987),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1987),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(488),
    [sym_field_declaration] = STATE(488),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(488),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1853),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [488] = {
    [sym_preproc_def] = STATE(495),
    [sym_preproc_function_def] = STATE(495),
    [sym_preproc_call] = STATE(495),
    [sym_preproc_if_in_field_declaration_list] = STATE(495),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(495),
    [sym_preproc_else_in_field_declaration_list] = STATE(1983),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1983),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1983),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(495),
    [sym_field_declaration] = STATE(495),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(495),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1855),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [489] = {
    [sym_preproc_def] = STATE(493),
    [sym_preproc_function_def] = STATE(493),
    [sym_preproc_call] = STATE(493),
    [sym_preproc_if_in_field_declaration_list] = STATE(493),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(493),
    [sym_preproc_else_in_field_declaration_list] = STATE(1830),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1830),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1830),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(493),
    [sym_field_declaration] = STATE(493),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(493),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1857),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [490] = {
    [sym_preproc_def] = STATE(492),
    [sym_preproc_function_def] = STATE(492),
    [sym_preproc_call] = STATE(492),
    [sym_preproc_if_in_field_declaration_list] = STATE(492),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(492),
    [sym_preproc_else_in_field_declaration_list] = STATE(1984),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1984),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1984),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(492),
    [sym_field_declaration] = STATE(492),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(492),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1859),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [491] = {
    [sym_preproc_def] = STATE(495),
    [sym_preproc_function_def] = STATE(495),
    [sym_preproc_call] = STATE(495),
    [sym_preproc_if_in_field_declaration_list] = STATE(495),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(495),
    [sym_preproc_else_in_field_declaration_list] = STATE(1829),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1829),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1829),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(495),
    [sym_field_declaration] = STATE(495),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(495),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1861),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [492] = {
    [sym_preproc_def] = STATE(495),
    [sym_preproc_function_def] = STATE(495),
    [sym_preproc_call] = STATE(495),
    [sym_preproc_if_in_field_declaration_list] = STATE(495),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(495),
    [sym_preproc_else_in_field_declaration_list] = STATE(1982),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1982),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1982),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(495),
    [sym_field_declaration] = STATE(495),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(495),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1863),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [493] = {
    [sym_preproc_def] = STATE(495),
    [sym_preproc_function_def] = STATE(495),
    [sym_preproc_call] = STATE(495),
    [sym_preproc_if_in_field_declaration_list] = STATE(495),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(495),
    [sym_preproc_else_in_field_declaration_list] = STATE(1828),
    [sym_preproc_elif_in_field_declaration_list] = STATE(1828),
    [sym_preproc_elifdef_in_field_declaration_list] = STATE(1828),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(495),
    [sym_field_declaration] = STATE(495),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(495),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [aux_sym_preproc_def_token1] = ACTIONS(1821),
    [aux_sym_preproc_if_token1] = ACTIONS(1823),
    [aux_sym_preproc_if_token2] = ACTIONS(1865),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1827),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1827),
    [aux_sym_preproc_else_token1] = ACTIONS(1829),
    [aux_sym_preproc_elif_token1] = ACTIONS(1831),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1833),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1833),
    [sym_preproc_directive] = ACTIONS(1835),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [494] = {
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1184),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_ms_call_modifier] = STATE(1390),
    [sym__abstract_declarator] = STATE(1523),
    [sym_abstract_parenthesized_declarator] = STATE(1496),
    [sym_abstract_pointer_declarator] = STATE(1496),
    [sym_abstract_function_declarator] = STATE(1496),
    [sym_abstract_array_declarator] = STATE(1496),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym_variadic_parameter] = STATE(1670),
    [sym_parameter_list] = STATE(1486),
    [sym_parameter_declaration] = STATE(1670),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1819),
    [anon_sym_DOT_DOT_DOT] = ACTIONS(1745),
    [anon_sym_RPAREN] = ACTIONS(1747),
    [anon_sym_LPAREN2] = ACTIONS(1867),
    [anon_sym_STAR] = ACTIONS(1869),
    [anon_sym___extension__] = ACTIONS(47),
    [anon_sym_extern] = ACTIONS(45),
    [anon_sym___attribute__] = ACTIONS(33),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1104),
    [anon_sym___declspec] = ACTIONS(37),
    [anon_sym___cdecl] = ACTIONS(39),
    [anon_sym___clrcall] = ACTIONS(39),
    [anon_sym___stdcall] = ACTIONS(39),
    [anon_sym___fastcall] = ACTIONS(39),
    [anon_sym___thiscall] = ACTIONS(39),
    [anon_sym___vectorcall] = ACTIONS(39),
    [anon_sym_signed] = ACTIONS(43),
    [anon_sym_unsigned] = ACTIONS(43),
    [anon_sym_long] = ACTIONS(43),
    [anon_sym_short] = ACTIONS(43),
    [anon_sym_LBRACK] = ACTIONS(1755),
    [anon_sym_static] = ACTIONS(45),
    [anon_sym_auto] = ACTIONS(45),
    [anon_sym_register] = ACTIONS(45),
    [anon_sym_inline] = ACTIONS(45),
    [anon_sym___inline] = ACTIONS(45),
    [anon_sym___inline__] = ACTIONS(45),
    [anon_sym___forceinline] = ACTIONS(45),
    [anon_sym_thread_local] = ACTIONS(45),
    [anon_sym___thread] = ACTIONS(45),
    [anon_sym_const] = ACTIONS(47),
    [anon_sym_constexpr] = ACTIONS(47),
    [anon_sym_volatile] = ACTIONS(47),
    [anon_sym_restrict] = ACTIONS(47),
    [anon_sym___restrict__] = ACTIONS(47),
    [anon_sym__Atomic] = ACTIONS(47),
    [anon_sym__Noreturn] = ACTIONS(47),
    [anon_sym_noreturn] = ACTIONS(47),
    [sym_primitive_type] = ACTIONS(49),
    [anon_sym_enum] = ACTIONS(51),
    [anon_sym_struct] = ACTIONS(53),
    [anon_sym_union] = ACTIONS(55),
    [sym_comment] = ACTIONS(3),
  },
  [495] = {
    [sym_preproc_def] = STATE(495),
    [sym_preproc_function_def] = STATE(495),
    [sym_preproc_call] = STATE(495),
    [sym_preproc_if_in_field_declaration_list] = STATE(495),
    [sym_preproc_ifdef_in_field_declaration_list] = STATE(495),
    [sym__declaration_modifiers] = STATE(749),
    [sym__declaration_specifiers] = STATE(1289),
    [sym_attribute_specifier] = STATE(749),
    [sym_attribute_declaration] = STATE(749),
    [sym_ms_declspec_modifier] = STATE(749),
    [sym_storage_class_specifier] = STATE(749),
    [sym_type_qualifier] = STATE(749),
    [sym__type_specifier] = STATE(795),
    [sym_sized_type_specifier] = STATE(821),
    [sym_enum_specifier] = STATE(821),
    [sym_struct_specifier] = STATE(821),
    [sym_union_specifier] = STATE(821),
    [sym__field_declaration_list_item] = STATE(495),
    [sym_field_declaration] = STATE(495),
    [sym_macro_type_specifier] = STATE(821),
    [aux_sym_preproc_if_in_field_declaration_list_repeat1] = STATE(495),
    [aux_sym__declaration_specifiers_repeat1] = STATE(749),
    [aux_sym_sized_type_specifier_repeat1] = STATE(843),
    [sym_identifier] = ACTIONS(1871),
    [aux_sym_preproc_def_token1] = ACTIONS(1874),
    [aux_sym_preproc_if_token1] = ACTIONS(1877),
    [aux_sym_preproc_if_token2] = ACTIONS(1880),
    [aux_sym_preproc_ifdef_token1] = ACTIONS(1882),
    [aux_sym_preproc_ifdef_token2] = ACTIONS(1882),
    [aux_sym_preproc_else_token1] = ACTIONS(1880),
    [aux_sym_preproc_elif_token1] = ACTIONS(1880),
    [aux_sym_preproc_elifdef_token1] = ACTIONS(1880),
    [aux_sym_preproc_elifdef_token2] = ACTIONS(1880),
    [sym_preproc_directive] = ACTIONS(1885),
    [anon_sym___extension__] = ACTIONS(1888),
    [anon_sym_extern] = ACTIONS(1891),
    [anon_sym___attribute__] = ACTIONS(1894),
    [anon_sym_LBRACK_LBRACK] = ACTIONS(1897),
    [anon_sym___declspec] = ACTIONS(1900),
    [anon_sym_signed] = ACTIONS(1903),
    [anon_sym_unsigned] = ACTIONS(1903),
    [anon_sym_long] = ACTIONS(1903),
    [anon_sym_short] = ACTIONS(1903),
    [anon_sym_static] = ACTIONS(1891),
    [anon_sym_auto] = ACTIONS(1891),
    [anon_sym_register] = ACTIONS(1891),
    [anon_sym_inline] = ACTIONS(1891),
    [anon_sym___inline] = ACTIONS(1891),
    [anon_sym___inline__] = ACTIONS(1891),
    [anon_sym___forceinline] = ACTIONS(1891),
    [anon_sym_thread_local] = ACTIONS(1891),
    [anon_sym___thread] = ACTIONS(1891),
    [anon_sym_const] = ACTIONS(1888),
    [anon_sym_constexpr] = ACTIONS(1888),
    [anon_sym_volatile] = ACTIONS(1888),
    [anon_sym_restrict] = ACTIONS(1888),
    [anon_sym___restrict__] = ACTIONS(1888),
    [anon_sym__Atomic] = ACTIONS(1888),
    [anon_sym__Noreturn] = ACTIONS(1888),
    [anon_sym_noreturn] = ACTIONS(1888),
    [sym_primitive_type] = ACTIONS(1906),
    [anon_sym_enum] = ACTIONS(1909),
    [anon_sym_struct] = ACTIONS(1912),
    [anon_sym_union] = ACTIONS(1915),
    [sym_comment] = ACTIONS(3),
  },
};

static const uint16_t ts_small_parse_table[] = {
  [0] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1918), 1,
      anon_sym_RPAREN,
    ACTIONS(1920), 1,
      anon_sym___extension__,
    STATE(683), 1,
      sym_string_literal,
    STATE(1023), 1,
      sym__expression,
    STATE(1640), 1,
      sym_compound_statement,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [115] = 25,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1922), 1,
      anon_sym_RPAREN,
    ACTIONS(1924), 1,
      anon_sym___extension__,
    STATE(683), 1,
      sym_string_literal,
    STATE(1022), 1,
      sym__expression,
    STATE(1659), 1,
      sym_compound_statement,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [230] = 24,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1926), 1,
      anon_sym___extension__,
    STATE(683), 1,
      sym_string_literal,
    STATE(1049), 1,
      sym__expression,
    STATE(1704), 1,
      sym_compound_statement,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [342] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1358), 1,
      anon_sym_LBRACE,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1044), 1,
      sym__expression,
    STATE(1693), 1,
      sym_initializer_list,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [451] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1928), 1,
      anon_sym_SEMI,
    STATE(683), 1,
      sym_string_literal,
    STATE(1069), 1,
      sym__expression,
    STATE(1920), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [560] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1358), 1,
      anon_sym_LBRACE,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(714), 1,
      sym_initializer_list,
    STATE(869), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [667] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1934), 1,
      anon_sym_RPAREN,
    STATE(683), 1,
      sym_string_literal,
    STATE(1064), 1,
      sym__expression,
    STATE(1790), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [776] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1936), 1,
      anon_sym_COLON,
    STATE(683), 1,
      sym_string_literal,
    STATE(1031), 1,
      sym__expression,
    STATE(2016), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [885] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1938), 1,
      anon_sym_SEMI,
    STATE(683), 1,
      sym_string_literal,
    STATE(1081), 1,
      sym__expression,
    STATE(1861), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [994] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1940), 1,
      anon_sym_SEMI,
    STATE(683), 1,
      sym_string_literal,
    STATE(1056), 1,
      sym__expression,
    STATE(1863), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [1103] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1942), 1,
      anon_sym_SEMI,
    STATE(683), 1,
      sym_string_literal,
    STATE(1034), 1,
      sym__expression,
    STATE(1801), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [1212] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1944), 1,
      anon_sym_COLON,
    STATE(683), 1,
      sym_string_literal,
    STATE(1075), 1,
      sym__expression,
    STATE(1892), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [1321] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1358), 1,
      anon_sym_LBRACE,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(714), 1,
      sym_initializer_list,
    STATE(722), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [1428] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1358), 1,
      anon_sym_LBRACE,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(714), 1,
      sym_initializer_list,
    STATE(722), 1,
      sym__expression,
    STATE(735), 1,
      sym_string_literal,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [1535] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1956), 1,
      anon_sym_COLON,
    STATE(683), 1,
      sym_string_literal,
    STATE(1046), 1,
      sym__expression,
    STATE(1860), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [1644] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1358), 1,
      anon_sym_LBRACE,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(714), 1,
      sym_initializer_list,
    STATE(722), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [1753] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1958), 1,
      anon_sym_SEMI,
    STATE(683), 1,
      sym_string_literal,
    STATE(1082), 1,
      sym__expression,
    STATE(1786), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [1862] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1960), 1,
      anon_sym_COLON,
    STATE(683), 1,
      sym_string_literal,
    STATE(1054), 1,
      sym__expression,
    STATE(1996), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [1971] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1358), 1,
      anon_sym_LBRACE,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(714), 1,
      sym_initializer_list,
    STATE(869), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [2080] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1962), 1,
      anon_sym_RPAREN,
    STATE(683), 1,
      sym_string_literal,
    STATE(1067), 1,
      sym__expression,
    STATE(1906), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [2189] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1358), 1,
      anon_sym_LBRACE,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1055), 1,
      sym__expression,
    STATE(1748), 1,
      sym_initializer_list,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [2298] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1964), 1,
      anon_sym_RPAREN,
    STATE(683), 1,
      sym_string_literal,
    STATE(1032), 1,
      sym__expression,
    STATE(1804), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [2407] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1358), 1,
      anon_sym_LBRACE,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1084), 1,
      sym__expression,
    STATE(1753), 1,
      sym_initializer_list,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [2516] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1358), 1,
      anon_sym_LBRACE,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(714), 1,
      sym_initializer_list,
    STATE(722), 1,
      sym__expression,
    STATE(735), 1,
      sym_string_literal,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [2625] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1972), 1,
      anon_sym_SEMI,
    STATE(683), 1,
      sym_string_literal,
    STATE(1026), 1,
      sym__expression,
    STATE(1808), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [2734] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1974), 1,
      anon_sym_SEMI,
    STATE(683), 1,
      sym_string_literal,
    STATE(1041), 1,
      sym__expression,
    STATE(1911), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [2843] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1976), 1,
      anon_sym_COLON,
    STATE(683), 1,
      sym_string_literal,
    STATE(1042), 1,
      sym__expression,
    STATE(1997), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [2952] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1978), 1,
      anon_sym_RPAREN,
    STATE(683), 1,
      sym_string_literal,
    STATE(1047), 1,
      sym__expression,
    STATE(1797), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [3061] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1980), 1,
      anon_sym_COLON,
    STATE(683), 1,
      sym_string_literal,
    STATE(1078), 1,
      sym__expression,
    STATE(1914), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [3170] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1982), 1,
      anon_sym_RPAREN,
    STATE(683), 1,
      sym_string_literal,
    STATE(1058), 1,
      sym__expression,
    STATE(1844), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [3279] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(1984), 1,
      anon_sym_RPAREN,
    STATE(683), 1,
      sym_string_literal,
    STATE(1057), 1,
      sym__expression,
    STATE(1827), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [3388] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(1986), 1,
      aux_sym_preproc_def_token1,
    ACTIONS(1988), 1,
      aux_sym_preproc_if_token1,
    ACTIONS(1990), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(1994), 1,
      sym_preproc_directive,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1295), 1,
      sym__declaration_specifiers,
    ACTIONS(1992), 2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    STATE(539), 8,
      sym_preproc_def,
      sym_preproc_function_def,
      sym_preproc_call,
      sym_preproc_if_in_field_declaration_list,
      sym_preproc_ifdef_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_in_field_declaration_list_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [3496] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1871), 1,
      sym_identifier,
    ACTIONS(1880), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(1894), 1,
      anon_sym___attribute__,
    ACTIONS(1897), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1900), 1,
      anon_sym___declspec,
    ACTIONS(1906), 1,
      sym_primitive_type,
    ACTIONS(1909), 1,
      anon_sym_enum,
    ACTIONS(1912), 1,
      anon_sym_struct,
    ACTIONS(1915), 1,
      anon_sym_union,
    ACTIONS(1996), 1,
      aux_sym_preproc_def_token1,
    ACTIONS(1999), 1,
      aux_sym_preproc_if_token1,
    ACTIONS(2005), 1,
      sym_preproc_directive,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1295), 1,
      sym__declaration_specifiers,
    ACTIONS(2002), 2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
    ACTIONS(1903), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    STATE(528), 8,
      sym_preproc_def,
      sym_preproc_function_def,
      sym_preproc_call,
      sym_preproc_if_in_field_declaration_list,
      sym_preproc_ifdef_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_in_field_declaration_list_repeat1,
    ACTIONS(1888), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(1891), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [3604] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2008), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [3708] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2010), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [3812] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2012), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [3916] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1068), 1,
      sym__expression,
    STATE(1945), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [4022] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1871), 1,
      sym_identifier,
    ACTIONS(1894), 1,
      anon_sym___attribute__,
    ACTIONS(1897), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1900), 1,
      anon_sym___declspec,
    ACTIONS(1906), 1,
      sym_primitive_type,
    ACTIONS(1909), 1,
      anon_sym_enum,
    ACTIONS(1912), 1,
      anon_sym_struct,
    ACTIONS(1915), 1,
      anon_sym_union,
    ACTIONS(2014), 1,
      aux_sym_preproc_def_token1,
    ACTIONS(2017), 1,
      aux_sym_preproc_if_token1,
    ACTIONS(2023), 1,
      sym_preproc_directive,
    ACTIONS(2026), 1,
      anon_sym_RBRACE,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1299), 1,
      sym__declaration_specifiers,
    ACTIONS(2020), 2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
    ACTIONS(1903), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    STATE(533), 8,
      sym_preproc_def,
      sym_preproc_function_def,
      sym_preproc_call,
      sym_preproc_if_in_field_declaration_list,
      sym_preproc_ifdef_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_in_field_declaration_list_repeat1,
    ACTIONS(1888), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(1891), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [4130] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(2028), 1,
      aux_sym_preproc_def_token1,
    ACTIONS(2030), 1,
      aux_sym_preproc_if_token1,
    ACTIONS(2034), 1,
      sym_preproc_directive,
    ACTIONS(2036), 1,
      anon_sym_RBRACE,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1299), 1,
      sym__declaration_specifiers,
    ACTIONS(2032), 2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    STATE(533), 8,
      sym_preproc_def,
      sym_preproc_function_def,
      sym_preproc_call,
      sym_preproc_if_in_field_declaration_list,
      sym_preproc_ifdef_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_in_field_declaration_list_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [4238] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2038), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [4342] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2040), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [4446] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2042), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [4550] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2044), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [4654] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(1986), 1,
      aux_sym_preproc_def_token1,
    ACTIONS(1988), 1,
      aux_sym_preproc_if_token1,
    ACTIONS(1994), 1,
      sym_preproc_directive,
    ACTIONS(2046), 1,
      aux_sym_preproc_if_token2,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1295), 1,
      sym__declaration_specifiers,
    ACTIONS(1992), 2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    STATE(528), 8,
      sym_preproc_def,
      sym_preproc_function_def,
      sym_preproc_call,
      sym_preproc_if_in_field_declaration_list,
      sym_preproc_ifdef_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_in_field_declaration_list_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [4762] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2048), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [4866] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1083), 1,
      sym__expression,
    STATE(1883), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [4972] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2050), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [5076] = 23,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(2028), 1,
      aux_sym_preproc_def_token1,
    ACTIONS(2030), 1,
      aux_sym_preproc_if_token1,
    ACTIONS(2034), 1,
      sym_preproc_directive,
    ACTIONS(2052), 1,
      anon_sym_RBRACE,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1299), 1,
      sym__declaration_specifiers,
    ACTIONS(2032), 2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    STATE(534), 8,
      sym_preproc_def,
      sym_preproc_function_def,
      sym_preproc_call,
      sym_preproc_if_in_field_declaration_list,
      sym_preproc_ifdef_in_field_declaration_list,
      sym__field_declaration_list_item,
      sym_field_declaration,
      aux_sym_preproc_if_in_field_declaration_list_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [5184] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    ACTIONS(2054), 1,
      anon_sym_RBRACK,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [5288] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1018), 1,
      sym__expression,
    STATE(1685), 1,
      sym_comma_expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [5394] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(963), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [5497] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(769), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [5598] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(961), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [5701] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(727), 1,
      sym__expression,
    STATE(735), 1,
      sym_string_literal,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [5804] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(964), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [5907] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(959), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6010] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(966), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6113] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(967), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6216] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(960), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6319] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(968), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6422] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(969), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6525] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(970), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6628] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(971), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6731] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1101), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6834] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1000), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [6937] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(992), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7040] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(2056), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(736), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7141] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(737), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7244] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(793), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7345] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(733), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7446] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(723), 1,
      sym__expression,
    STATE(735), 1,
      sym_string_literal,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7547] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(737), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7648] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1086), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7751] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(723), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7852] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1025), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [7955] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1106), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8058] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(962), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8161] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(998), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8264] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(980), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8367] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(965), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8470] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(727), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8573] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(977), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8676] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(2058), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(885), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8779] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(886), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8882] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(976), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [8985] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(887), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [9086] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(890), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [9189] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(974), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [9292] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(985), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [9395] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(973), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [9498] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1087), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [9601] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(763), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [9702] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1079), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [9805] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1096), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [9908] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(868), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10009] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(770), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10110] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(768), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10211] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(767), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10312] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(766), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10413] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(765), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10514] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1097), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10617] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(771), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10718] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(772), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10819] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1019), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [10922] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(990), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11025] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(972), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11128] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(2060), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(885), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11229] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(886), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11330] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(987), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11433] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(890), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11534] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(881), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11635] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(882), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11736] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(870), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11837] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(871), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [11938] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(873), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12039] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(874), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12140] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(875), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12241] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(876), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12342] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(877), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12443] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(878), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12544] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1655), 1,
      anon_sym_sizeof,
    ACTIONS(1930), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(879), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1651), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1653), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1932), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12645] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1059), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12748] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1112), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12851] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(774), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [12952] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(777), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13053] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1050), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13156] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1021), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13259] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1063), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13362] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1027), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13465] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1103), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13568] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1051), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13671] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(797), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13772] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1048), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13875] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1045), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [13978] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1036), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [14081] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1030), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [14184] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1040), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [14287] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1053), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [14390] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(868), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [14493] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1061), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [14596] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1687), 1,
      sym_identifier,
    ACTIONS(1693), 1,
      anon_sym_sizeof,
    ACTIONS(1757), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(1066), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1689), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1691), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1761), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1767), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(895), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [14699] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(737), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [14802] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(1968), 1,
      anon_sym_LPAREN2,
    STATE(733), 1,
      sym__expression,
    STATE(735), 1,
      sym_string_literal,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [14905] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1685), 1,
      anon_sym_sizeof,
    ACTIONS(1966), 1,
      sym_identifier,
    ACTIONS(2062), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(736), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1681), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1683), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1970), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(799), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15008] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1074), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15111] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(789), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15212] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1020), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15315] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(727), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15416] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(790), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15517] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(792), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15618] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(796), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15719] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(780), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15820] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(798), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [15921] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(805), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16022] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(803), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16123] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1370), 1,
      anon_sym_sizeof,
    ACTIONS(1946), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(801), 1,
      sym__expression,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1366), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1368), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1948), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16224] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1070), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16327] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(727), 1,
      sym__expression,
    STATE(735), 1,
      sym_string_literal,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16428] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(737), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16529] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1098), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16632] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1111), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16735] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(1950), 1,
      anon_sym_LPAREN2,
    STATE(733), 1,
      sym__expression,
    STATE(735), 1,
      sym_string_literal,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16836] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1360), 1,
      anon_sym_sizeof,
    ACTIONS(1364), 1,
      sym_identifier,
    ACTIONS(2064), 1,
      anon_sym_LPAREN2,
    STATE(735), 1,
      sym_string_literal,
    STATE(736), 1,
      sym__expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(1354), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(1356), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(1952), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(1954), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(700), 22,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_pointer_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_gnu_asm_expression,
      sym_field_expression,
      sym_compound_literal_expression,
      sym_parenthesized_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [16937] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    ACTIONS(2066), 1,
      anon_sym_LPAREN2,
    STATE(683), 1,
      sym_string_literal,
    STATE(736), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [17040] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(19), 1,
      anon_sym_LPAREN2,
    ACTIONS(81), 1,
      anon_sym_sizeof,
    ACTIONS(85), 1,
      anon_sym_offsetof,
    ACTIONS(87), 1,
      anon_sym__Generic,
    ACTIONS(157), 1,
      sym_number_literal,
    ACTIONS(1679), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(733), 1,
      sym__expression,
    ACTIONS(21), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(23), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(25), 2,
      anon_sym_STAR,
      anon_sym_AMP,
    ACTIONS(79), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(99), 2,
      anon_sym_NULL,
      anon_sym_nullptr,
    ACTIONS(159), 2,
      sym_true,
      sym_false,
    ACTIONS(83), 5,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
    ACTIONS(93), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    STATE(840), 5,
      sym_pointer_expression,
      sym_subscript_expression,
      sym_call_expression,
      sym_field_expression,
      sym_parenthesized_expression,
    STATE(700), 17,
      sym__expression_not_binary,
      sym__string,
      sym_conditional_expression,
      sym_assignment_expression,
      sym_unary_expression,
      sym_binary_expression,
      sym_update_expression,
      sym_cast_expression,
      sym_sizeof_expression,
      sym_alignof_expression,
      sym_offsetof_expression,
      sym_generic_expression,
      sym_gnu_asm_expression,
      sym_compound_literal_expression,
      sym_char_literal,
      sym_concatenated_string,
      sym_null,
  [17143] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1697), 1,
      anon_sym_const,
    ACTIONS(1701), 1,
      anon_sym_LPAREN2,
    ACTIONS(1707), 1,
      anon_sym_STAR,
    ACTIONS(1714), 1,
      anon_sym_EQ,
    STATE(662), 1,
      sym_string_literal,
    STATE(811), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(1720), 2,
      anon_sym_RPAREN,
      anon_sym_LBRACK,
    ACTIONS(2068), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1710), 8,
      anon_sym___extension__,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(1718), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 11,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 12,
      anon_sym_COMMA,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [17231] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2070), 1,
      sym_identifier,
    STATE(665), 2,
      sym_string_literal,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2074), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2072), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [17302] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(2076), 1,
      anon_sym_LBRACE,
    STATE(742), 1,
      sym_ms_call_modifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1143), 1,
      sym__declaration_specifiers,
    STATE(132), 3,
      sym_function_definition,
      sym_declaration,
      sym_declaration_list,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [17403] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(2078), 1,
      anon_sym_LBRACE,
    STATE(745), 1,
      sym_ms_call_modifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1154), 1,
      sym__declaration_specifiers,
    STATE(341), 3,
      sym_function_definition,
      sym_declaration,
      sym_declaration_list,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [17504] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2080), 1,
      sym_identifier,
    STATE(666), 2,
      sym_string_literal,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2084), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2082), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [17575] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2086), 1,
      sym_identifier,
    STATE(666), 2,
      sym_string_literal,
      aux_sym_concatenated_string_repeat1,
    ACTIONS(2093), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2091), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2089), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [17646] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(2096), 1,
      anon_sym_LBRACE,
    STATE(743), 1,
      sym_ms_call_modifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1155), 1,
      sym__declaration_specifiers,
    STATE(385), 3,
      sym_function_definition,
      sym_declaration,
      sym_declaration_list,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [17747] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(2098), 1,
      anon_sym_LBRACE,
    STATE(741), 1,
      sym_ms_call_modifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1156), 1,
      sym__declaration_specifiers,
    STATE(412), 3,
      sym_function_definition,
      sym_declaration,
      sym_declaration_list,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [17848] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1705), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(1699), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [17916] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2100), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2102), 38,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [17979] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2104), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2106), 38,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [18042] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2108), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2110), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18104] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2112), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2114), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18166] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2116), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2118), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18228] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2120), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2122), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18290] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2124), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2126), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18352] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2128), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2130), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18414] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2132), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2134), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18476] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2136), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2138), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18538] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2140), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2142), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18600] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2144), 20,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2146), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18662] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2158), 1,
      anon_sym___attribute__,
    ACTIONS(2161), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2164), 1,
      anon_sym___declspec,
    ACTIONS(2150), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
    STATE(682), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(2152), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(2155), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
    ACTIONS(2148), 17,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [18733] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2167), 1,
      sym_identifier,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2171), 15,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2169), 29,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18798] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2173), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2175), 34,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [18857] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2175), 21,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2173), 30,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_if,
      anon_sym_switch,
      anon_sym_case,
      anon_sym_default,
      anon_sym_while,
      anon_sym_do,
      anon_sym_for,
      anon_sym_return,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_goto,
      anon_sym___try,
      anon_sym___leave,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
  [18916] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1677), 21,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1675), 30,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_if,
      anon_sym_switch,
      anon_sym_case,
      anon_sym_default,
      anon_sym_while,
      anon_sym_do,
      anon_sym_for,
      anon_sym_return,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_goto,
      anon_sym___try,
      anon_sym___leave,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
  [18975] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1673), 21,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1671), 30,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_if,
      anon_sym_switch,
      anon_sym_case,
      anon_sym_default,
      anon_sym_while,
      anon_sym_do,
      anon_sym_for,
      anon_sym_return,
      anon_sym_break,
      anon_sym_continue,
      anon_sym_goto,
      anon_sym___try,
      anon_sym___leave,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
  [19034] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(141), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(734), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [19126] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1745), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1747), 1,
      anon_sym_RPAREN,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1184), 1,
      sym__declaration_specifiers,
    STATE(1670), 2,
      sym_variadic_parameter,
      sym_parameter_declaration,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [19218] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(431), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(710), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [19310] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(346), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(734), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [19402] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2177), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2179), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [19460] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(398), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(694), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [19552] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(388), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(734), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [19644] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(325), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(712), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [19736] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2181), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2183), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [19794] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2185), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2187), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [19852] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(310), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(734), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [19944] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(140), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(688), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [20036] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2189), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2191), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [20094] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(426), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(734), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [20186] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2193), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2195), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [20244] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(138), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(708), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [20336] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2197), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2199), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [20394] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2201), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2203), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [20452] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1745), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(2205), 1,
      sym_identifier,
    ACTIONS(2207), 1,
      anon_sym_RPAREN,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1184), 1,
      sym__declaration_specifiers,
    STATE(1506), 1,
      sym_variadic_parameter,
    STATE(1670), 1,
      sym_parameter_declaration,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [20546] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(411), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(701), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [20638] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(147), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(734), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [20730] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(381), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(691), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [20822] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(417), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(734), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [20914] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2209), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2211), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [20972] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(383), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(734), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [21064] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(333), 1,
      sym_compound_statement,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(698), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [21156] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2213), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2215), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [21214] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2217), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2219), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [21272] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2221), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2223), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [21330] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2225), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2227), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [21388] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2229), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2231), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [21446] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2233), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2235), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [21504] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2237), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2239), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [21562] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2241), 17,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      anon_sym_DOT,
      sym_identifier,
    ACTIONS(2243), 33,
      anon_sym_DOT_DOT_DOT,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [21620] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2245), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2247), 26,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [21687] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2257), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2259), 26,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [21754] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2261), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2263), 33,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACK_RBRACK,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [21811] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1745), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1184), 1,
      sym__declaration_specifiers,
    STATE(1715), 2,
      sym_variadic_parameter,
      sym_parameter_declaration,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [21900] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2265), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2267), 33,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACK_RBRACK,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [21957] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2269), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2271), 26,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [22024] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2273), 1,
      anon_sym_EQ,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2275), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 14,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      sym_identifier,
    ACTIONS(1699), 18,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [22089] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2277), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2279), 33,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACK_RBRACK,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [22146] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1745), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(2281), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1184), 1,
      sym__declaration_specifiers,
    STATE(1715), 1,
      sym_parameter_declaration,
    STATE(1773), 1,
      sym_variadic_parameter,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [22237] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2285), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_COLON,
    ACTIONS(2283), 40,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      anon_sym_asm,
      anon_sym___asm__,
      sym_identifier,
  [22294] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2287), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2289), 33,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACK_RBRACK,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [22351] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2291), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2293), 26,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [22418] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2295), 1,
      sym_identifier,
    ACTIONS(2304), 1,
      anon_sym___attribute__,
    ACTIONS(2307), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2310), 1,
      anon_sym___declspec,
    ACTIONS(2313), 1,
      anon_sym_LBRACE,
    ACTIONS(2318), 1,
      sym_primitive_type,
    ACTIONS(2321), 1,
      anon_sym_enum,
    ACTIONS(2324), 1,
      anon_sym_struct,
    ACTIONS(2327), 1,
      anon_sym_union,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1142), 1,
      sym__declaration_specifiers,
    STATE(734), 2,
      sym_declaration,
      aux_sym__old_style_function_definition_repeat1,
    ACTIONS(2315), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(2298), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(2301), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [22507] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2171), 15,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2169), 28,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [22568] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2330), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2332), 26,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [22635] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2334), 16,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym___attribute__,
      anon_sym_EQ,
      sym_identifier,
    ACTIONS(2336), 28,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
  [22700] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1714), 1,
      anon_sym_EQ,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1718), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 12,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [22764] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(739), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2342), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(2338), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(2340), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_RBRACK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [22823] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1245), 1,
      sym__declaration_specifiers,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [22905] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1272), 1,
      sym__declaration_specifiers,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [22987] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1271), 1,
      sym__declaration_specifiers,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [23069] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1273), 1,
      sym__declaration_specifiers,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [23151] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1228), 1,
      sym__declaration_specifiers,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [23233] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1270), 1,
      sym__declaration_specifiers,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [23315] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1226), 1,
      sym__declaration_specifiers,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [23397] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(795), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1229), 1,
      sym__declaration_specifiers,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(749), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [23479] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1714), 1,
      anon_sym_EQ,
    ACTIONS(1723), 1,
      anon_sym_COLON,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1718), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 12,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 15,
      anon_sym_COMMA,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [23542] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(51), 1,
      anon_sym_enum,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(787), 1,
      sym__type_specifier,
    STATE(843), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(43), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    STATE(682), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [23621] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2347), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_RBRACK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2345), 26,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
  [23674] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    STATE(779), 1,
      sym_field_declaration_list,
    STATE(830), 1,
      sym_attribute_specifier,
    ACTIONS(2351), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2349), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [23735] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    STATE(784), 1,
      sym_field_declaration_list,
    STATE(807), 1,
      sym_attribute_specifier,
    ACTIONS(2357), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2355), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [23796] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1714), 1,
      anon_sym_EQ,
    ACTIONS(1716), 1,
      anon_sym_COLON,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1718), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 12,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 15,
      anon_sym_COMMA,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [23859] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2361), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(2359), 38,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [23912] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2347), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2345), 38,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [23965] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    STATE(786), 1,
      sym_field_declaration_list,
    STATE(828), 1,
      sym_attribute_specifier,
    ACTIONS(2365), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2363), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [24026] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    STATE(804), 1,
      sym_field_declaration_list,
    STATE(838), 1,
      sym_attribute_specifier,
    ACTIONS(2369), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2367), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [24087] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1714), 1,
      anon_sym_EQ,
    ACTIONS(1727), 1,
      anon_sym_COLON,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1718), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 12,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 15,
      anon_sym_COMMA,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [24150] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1714), 1,
      anon_sym_EQ,
    ACTIONS(2371), 1,
      anon_sym_COLON,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1718), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 12,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 15,
      anon_sym_COMMA,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [24213] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1714), 1,
      anon_sym_EQ,
    ACTIONS(1725), 1,
      anon_sym_COLON,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1718), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 12,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 15,
      anon_sym_COMMA,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [24276] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1714), 1,
      anon_sym_EQ,
    ACTIONS(1741), 1,
      anon_sym_COLON,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(1718), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 12,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 15,
      anon_sym_COMMA,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [24339] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    STATE(802), 1,
      sym_field_declaration_list,
    STATE(817), 1,
      sym_attribute_specifier,
    ACTIONS(2375), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2373), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [24400] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 11,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2271), 23,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [24464] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2379), 1,
      anon_sym_EQ,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2381), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 13,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DOT,
    ACTIONS(1699), 14,
      anon_sym_DOT_DOT_DOT,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [24524] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2385), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2387), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 5,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ,
    ACTIONS(2271), 21,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [24596] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2385), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2387), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2391), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 5,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ,
    ACTIONS(2271), 19,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [24670] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2393), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2385), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2387), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2391), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 4,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_EQ,
    ACTIONS(2271), 19,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [24746] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2393), 1,
      anon_sym_AMP,
    ACTIONS(2395), 1,
      anon_sym_CARET,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2385), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2387), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2391), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2269), 3,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_EQ,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 19,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [24824] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2393), 1,
      anon_sym_AMP,
    ACTIONS(2395), 1,
      anon_sym_CARET,
    ACTIONS(2397), 1,
      anon_sym_PIPE,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2269), 2,
      aux_sym_preproc_elif_token1,
      anon_sym_EQ,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2385), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2387), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2391), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 19,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [24904] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2393), 1,
      anon_sym_AMP,
    ACTIONS(2395), 1,
      anon_sym_CARET,
    ACTIONS(2397), 1,
      anon_sym_PIPE,
    ACTIONS(2399), 1,
      anon_sym_AMP_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2269), 2,
      aux_sym_preproc_elif_token1,
      anon_sym_EQ,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2385), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2387), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2391), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 18,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [24986] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 7,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_EQ,
    ACTIONS(2271), 23,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [25054] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 9,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2271), 23,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [25120] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1673), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1671), 38,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [25172] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2393), 1,
      anon_sym_AMP,
    ACTIONS(2395), 1,
      anon_sym_CARET,
    ACTIONS(2397), 1,
      anon_sym_PIPE,
    ACTIONS(2399), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2405), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2407), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2385), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2387), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2391), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2403), 2,
      aux_sym_preproc_elif_token1,
      anon_sym_EQ,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2401), 16,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [25258] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1677), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1675), 38,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [25310] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2411), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2409), 38,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [25362] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2393), 1,
      anon_sym_AMP,
    ACTIONS(2395), 1,
      anon_sym_CARET,
    ACTIONS(2397), 1,
      anon_sym_PIPE,
    ACTIONS(2399), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2405), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2407), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2385), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2387), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2391), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2415), 2,
      aux_sym_preproc_elif_token1,
      anon_sym_EQ,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2413), 16,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [25448] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2393), 1,
      anon_sym_AMP,
    ACTIONS(2395), 1,
      anon_sym_CARET,
    ACTIONS(2397), 1,
      anon_sym_PIPE,
    ACTIONS(2399), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2405), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2407), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2383), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2385), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2387), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2389), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2391), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2419), 2,
      aux_sym_preproc_elif_token1,
      anon_sym_EQ,
    ACTIONS(2377), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2417), 16,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      sym_identifier,
  [25534] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(814), 1,
      sym_attribute_specifier,
    ACTIONS(2423), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2421), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [25589] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2429), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2431), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2433), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2435), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2269), 3,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_EQ,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [25664] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2441), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
    STATE(682), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(2439), 9,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      sym_identifier,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [25727] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(832), 1,
      sym_attribute_specifier,
    ACTIONS(2445), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2443), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [25782] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(810), 1,
      sym_attribute_specifier,
    ACTIONS(2449), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2447), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [25837] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(820), 1,
      sym_attribute_specifier,
    ACTIONS(2453), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2451), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [25892] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2459), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2457), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2455), 31,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [25947] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(815), 1,
      sym_attribute_specifier,
    ACTIONS(2464), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2462), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [26002] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2468), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
    STATE(781), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(2466), 9,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      sym_identifier,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [26065] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(819), 1,
      sym_attribute_specifier,
    ACTIONS(2472), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2470), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [26120] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 10,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2271), 23,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [26183] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2269), 1,
      anon_sym_EQ,
    ACTIONS(2429), 1,
      anon_sym_AMP,
    ACTIONS(2474), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2476), 1,
      anon_sym_PIPE,
    ACTIONS(2478), 1,
      anon_sym_CARET,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2431), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2433), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2435), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 18,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [26264] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2482), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
    STATE(682), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(2480), 9,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      sym_identifier,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [26327] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2269), 1,
      anon_sym_EQ,
    ACTIONS(2429), 1,
      anon_sym_AMP,
    ACTIONS(2476), 1,
      anon_sym_PIPE,
    ACTIONS(2478), 1,
      anon_sym_CARET,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2431), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2433), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2435), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [26406] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2415), 1,
      anon_sym_EQ,
    ACTIONS(2429), 1,
      anon_sym_AMP,
    ACTIONS(2474), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2476), 1,
      anon_sym_PIPE,
    ACTIONS(2478), 1,
      anon_sym_CARET,
    ACTIONS(2484), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2486), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2431), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2433), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2435), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2413), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [26491] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(826), 1,
      sym_attribute_specifier,
    ACTIONS(2490), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2488), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [26546] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2494), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
    STATE(791), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(2492), 9,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      sym_identifier,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [26609] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2429), 1,
      anon_sym_AMP,
    ACTIONS(2478), 1,
      anon_sym_CARET,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2269), 2,
      anon_sym_PIPE,
      anon_sym_EQ,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2431), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2433), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2435), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [26686] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2403), 1,
      anon_sym_EQ,
    ACTIONS(2429), 1,
      anon_sym_AMP,
    ACTIONS(2474), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2476), 1,
      anon_sym_PIPE,
    ACTIONS(2478), 1,
      anon_sym_CARET,
    ACTIONS(2484), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2486), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2431), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2433), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2435), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2401), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [26771] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2431), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2433), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2435), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 4,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ,
    ACTIONS(2271), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [26844] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2273), 1,
      anon_sym_EQ,
    ACTIONS(2275), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 13,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 19,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
      sym_identifier,
  [26899] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2419), 1,
      anon_sym_EQ,
    ACTIONS(2429), 1,
      anon_sym_AMP,
    ACTIONS(2474), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2476), 1,
      anon_sym_PIPE,
    ACTIONS(2478), 1,
      anon_sym_CARET,
    ACTIONS(2484), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2486), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2431), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2433), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2435), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2417), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [26984] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 8,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2271), 23,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [27049] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(825), 1,
      sym_attribute_specifier,
    ACTIONS(2498), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2496), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27104] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 6,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_EQ,
    ACTIONS(2271), 23,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [27171] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(829), 1,
      sym_attribute_specifier,
    ACTIONS(2502), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2500), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27226] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2425), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2433), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2435), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2437), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2427), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 4,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ,
    ACTIONS(2271), 21,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [27297] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(836), 1,
      sym_attribute_specifier,
    ACTIONS(2506), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2504), 34,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27352] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2510), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2508), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27402] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2514), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2512), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27452] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2518), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2516), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27502] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2522), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2520), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27552] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2528), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2526), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2524), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [27606] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2528), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2532), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2530), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [27660] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2528), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2536), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2534), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [27714] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2540), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2538), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27764] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2544), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2542), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27814] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(813), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2550), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2548), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2546), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [27868] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2554), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2552), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [27918] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(831), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2560), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2558), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2556), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [27972] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2564), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2562), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28022] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2568), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2566), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28072] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2558), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2556), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28122] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2572), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2570), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28172] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2576), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2574), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28222] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2580), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2578), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28272] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2584), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2582), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28322] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2588), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2586), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28372] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2592), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2590), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28422] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2596), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2594), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28472] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2600), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2598), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28522] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2604), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2602), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28572] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2528), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2608), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2606), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [28626] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2612), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2610), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28676] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2616), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2614), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28726] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(812), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2622), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2620), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2618), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [28780] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2455), 1,
      sym_primitive_type,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2459), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2627), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2624), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [28836] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2632), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2630), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28886] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2636), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2634), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28936] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2640), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2638), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [28986] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2644), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2642), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [29036] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1714), 1,
      anon_sym_EQ,
    ACTIONS(1718), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 12,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(1699), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_LBRACK,
      anon_sym_COLON,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [29090] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2646), 1,
      anon_sym_LPAREN2,
    STATE(811), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(1712), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(1710), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(1697), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [29146] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2651), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2649), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [29196] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2653), 1,
      sym_identifier,
    ACTIONS(2662), 1,
      sym_primitive_type,
    STATE(835), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2660), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2656), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2658), 29,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [29254] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2666), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
    ACTIONS(2664), 35,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [29304] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2670), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2668), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29353] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1302), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1300), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29402] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2674), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2672), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29451] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2678), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2676), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29500] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1274), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1272), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29549] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1342), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1340), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29598] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1298), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1296), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29647] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2682), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2680), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29696] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2686), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2684), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29745] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2690), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2688), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29794] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2694), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2692), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29843] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2698), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2696), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29892] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1250), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1248), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29941] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1294), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1292), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [29990] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2702), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2700), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [30039] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2706), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2704), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [30088] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2710), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2708), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [30137] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2714), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2712), 40,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elif_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [30186] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2716), 1,
      anon_sym_SEMI,
    ACTIONS(2494), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    STATE(791), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(2492), 8,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      sym_identifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [30248] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2722), 1,
      anon_sym___attribute__,
    ACTIONS(2725), 1,
      anon_sym_LBRACE,
    ACTIONS(2727), 1,
      anon_sym_COLON,
    STATE(808), 1,
      sym_attribute_specifier,
    STATE(909), 1,
      sym_enumerator_list,
    ACTIONS(2720), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2718), 29,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [30306] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2729), 1,
      anon_sym_SEMI,
    ACTIONS(2494), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    STATE(791), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(2492), 8,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      sym_identifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [30368] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2731), 1,
      anon_sym_SEMI,
    ACTIONS(2494), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    STATE(791), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(2492), 8,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      sym_identifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [30430] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(1104), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2733), 1,
      anon_sym_SEMI,
    ACTIONS(2494), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    STATE(791), 7,
      sym__declaration_modifiers,
      sym_attribute_specifier,
      sym_attribute_declaration,
      sym_ms_declspec_modifier,
      sym_storage_class_specifier,
      sym_type_qualifier,
      aux_sym__declaration_specifiers_repeat1,
    ACTIONS(2492), 8,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      sym_identifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(45), 10,
      anon_sym_extern,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
  [30492] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2269), 13,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2271), 19,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [30551] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2245), 13,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2247), 19,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [30610] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 8,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2271), 19,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [30673] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 6,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_EQ,
    ACTIONS(2271), 19,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [30738] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1749), 1,
      anon_sym_LPAREN2,
    ACTIONS(1751), 1,
      anon_sym_STAR,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1303), 1,
      sym__declarator,
    STATE(1474), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1803), 1,
      sym_ms_based_modifier,
    ACTIONS(2747), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(884), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1002), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [30817] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2755), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2757), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 4,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ,
    ACTIONS(2271), 17,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [30886] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2755), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2757), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2759), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 4,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ,
    ACTIONS(2271), 15,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [30957] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(2761), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2755), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2757), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2759), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2269), 3,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_EQ,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 15,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31030] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(2761), 1,
      anon_sym_AMP,
    ACTIONS(2763), 1,
      anon_sym_CARET,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2269), 2,
      anon_sym_PIPE,
      anon_sym_EQ,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2755), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2757), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2759), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 15,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31105] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2269), 1,
      anon_sym_EQ,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(2761), 1,
      anon_sym_AMP,
    ACTIONS(2763), 1,
      anon_sym_CARET,
    ACTIONS(2765), 1,
      anon_sym_PIPE,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2755), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2757), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2759), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 15,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31182] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2269), 1,
      anon_sym_EQ,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(2761), 1,
      anon_sym_AMP,
    ACTIONS(2763), 1,
      anon_sym_CARET,
    ACTIONS(2765), 1,
      anon_sym_PIPE,
    ACTIONS(2767), 1,
      anon_sym_AMP_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2755), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2757), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2759), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2271), 14,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31261] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2269), 10,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2271), 19,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31322] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2419), 1,
      anon_sym_EQ,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(2761), 1,
      anon_sym_AMP,
    ACTIONS(2763), 1,
      anon_sym_CARET,
    ACTIONS(2765), 1,
      anon_sym_PIPE,
    ACTIONS(2767), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2769), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2771), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2755), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2757), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2759), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2417), 12,
      anon_sym_DOT_DOT_DOT,
      anon_sym_RBRACK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31405] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2415), 1,
      anon_sym_EQ,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(2761), 1,
      anon_sym_AMP,
    ACTIONS(2763), 1,
      anon_sym_CARET,
    ACTIONS(2765), 1,
      anon_sym_PIPE,
    ACTIONS(2767), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2769), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2771), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2755), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2757), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2759), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2413), 12,
      anon_sym_DOT_DOT_DOT,
      anon_sym_RBRACK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31488] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2403), 1,
      anon_sym_EQ,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(2761), 1,
      anon_sym_AMP,
    ACTIONS(2763), 1,
      anon_sym_CARET,
    ACTIONS(2765), 1,
      anon_sym_PIPE,
    ACTIONS(2767), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2769), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2771), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2739), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2743), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2755), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2757), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2759), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2741), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(2401), 12,
      anon_sym_DOT_DOT_DOT,
      anon_sym_RBRACK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31571] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2725), 1,
      anon_sym_LBRACE,
    ACTIONS(2777), 1,
      anon_sym___attribute__,
    STATE(839), 1,
      sym_attribute_specifier,
    STATE(928), 1,
      sym_enumerator_list,
    ACTIONS(2775), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2773), 29,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [31626] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1749), 1,
      anon_sym_LPAREN2,
    ACTIONS(1751), 1,
      anon_sym_STAR,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1321), 1,
      sym__declarator,
    STATE(1475), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1803), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    ACTIONS(2780), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1001), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    STATE(1016), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [31705] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2330), 13,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2332), 19,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31764] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2291), 13,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2293), 19,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31823] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2257), 13,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2259), 19,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
  [31882] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2782), 2,
      anon_sym___attribute__,
      sym_identifier,
    ACTIONS(2789), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(2792), 3,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(2785), 4,
      anon_sym_COMMA,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_EQ,
    ACTIONS(2787), 28,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
  [31935] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2794), 2,
      anon_sym___attribute__,
      sym_identifier,
    ACTIONS(2801), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(2804), 3,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(2797), 4,
      anon_sym_COMMA,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_EQ,
    ACTIONS(2799), 28,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
  [31988] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2334), 13,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
    ACTIONS(2336), 21,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
  [32045] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2287), 14,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2289), 24,
      anon_sym_DOT_DOT_DOT,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [32091] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2106), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(2104), 36,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32137] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2102), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(2100), 36,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32183] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2265), 14,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2267), 24,
      anon_sym_DOT_DOT_DOT,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [32229] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2379), 1,
      anon_sym_EQ,
    ACTIONS(2381), 10,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
    ACTIONS(1705), 13,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_DOT,
    ACTIONS(1699), 14,
      anon_sym_DOT_DOT_DOT,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [32279] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2261), 14,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2263), 24,
      anon_sym_DOT_DOT_DOT,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [32325] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2277), 14,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_EQ,
      anon_sym_DOT,
    ACTIONS(2279), 24,
      anon_sym_DOT_DOT_DOT,
      anon_sym_LPAREN2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LBRACK,
      anon_sym_RBRACK,
      anon_sym_QMARK,
      anon_sym_STAR_EQ,
      anon_sym_SLASH_EQ,
      anon_sym_PERCENT_EQ,
      anon_sym_PLUS_EQ,
      anon_sym_DASH_EQ,
      anon_sym_LT_LT_EQ,
      anon_sym_GT_GT_EQ,
      anon_sym_AMP_EQ,
      anon_sym_CARET_EQ,
      anon_sym_PIPE_EQ,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DASH_GT,
  [32371] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2710), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2708), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32416] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2714), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2712), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32461] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1342), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1340), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32506] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2694), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2692), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32551] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2411), 1,
      anon_sym_LBRACK_LBRACK,
    STATE(667), 1,
      sym_string_literal,
    ACTIONS(2806), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2409), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32600] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2706), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2704), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32645] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2670), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2668), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32690] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1274), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1272), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32735] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2678), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2676), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32780] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2682), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2680), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32825] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2694), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2692), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32870] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2812), 1,
      anon_sym___attribute__,
    STATE(844), 1,
      sym_attribute_specifier,
    ACTIONS(2810), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2808), 29,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [32919] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2686), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2684), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [32964] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2674), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2672), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33009] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2702), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2700), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33054] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2714), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2712), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33099] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2690), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2688), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33144] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1298), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1296), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33189] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2674), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2672), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33234] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2670), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2668), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33279] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2710), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2708), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33324] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1294), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1292), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33369] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2682), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2680), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33414] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2702), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2700), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33459] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2819), 1,
      anon_sym___attribute__,
    STATE(824), 1,
      sym_attribute_specifier,
    ACTIONS(2817), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2815), 29,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [33508] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2686), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2684), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33553] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2698), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2696), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33598] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2678), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2676), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33643] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2690), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2688), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33688] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1294), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(1292), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33733] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2826), 1,
      anon_sym___attribute__,
    STATE(822), 1,
      sym_attribute_specifier,
    ACTIONS(2824), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2822), 29,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [33782] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2411), 1,
      anon_sym_LBRACK_LBRACK,
    STATE(663), 1,
      sym_string_literal,
    ACTIONS(2806), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2409), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33831] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2411), 1,
      anon_sym_LBRACK_LBRACK,
    STATE(664), 1,
      sym_string_literal,
    ACTIONS(2806), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2409), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33880] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2698), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2696), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33925] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2411), 1,
      anon_sym_LBRACK_LBRACK,
    STATE(668), 1,
      sym_string_literal,
    ACTIONS(2806), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
    ACTIONS(2409), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [33974] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1302), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1300), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [34019] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1250), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1248), 36,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [34064] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1342), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(1340), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [34109] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1274), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(1272), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [34154] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1298), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(1296), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [34199] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1250), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(1248), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [34244] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2706), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(2704), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [34289] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1302), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_RBRACE,
    ACTIONS(1300), 35,
      aux_sym_preproc_def_token1,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [34334] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1172), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1174), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34378] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1200), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1202), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34422] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1200), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1202), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34466] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1120), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1122), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34510] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1176), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1178), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34554] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1120), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1122), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34598] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1148), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1150), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34642] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1168), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1170), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34686] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1168), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1170), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34730] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1176), 17,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_sizeof,
      anon_sym___alignof__,
      anon_sym___alignof,
      anon_sym__alignof,
      anon_sym_alignof,
      anon_sym__Alignof,
      anon_sym_offsetof,
      anon_sym__Generic,
      anon_sym_asm,
      anon_sym___asm__,
      sym_true,
      sym_false,
      anon_sym_NULL,
      anon_sym_nullptr,
      sym_identifier,
    ACTIONS(1178), 19,
      anon_sym_LPAREN2,
      anon_sym_BANG,
      anon_sym_TILDE,
      anon_sym_STAR,
      anon_sym_AMP,
      anon_sym_SEMI,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      sym_number_literal,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [34774] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1366), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(954), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1110), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [34843] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2646), 1,
      anon_sym_LPAREN2,
    ACTIONS(2839), 1,
      anon_sym_COMMA,
    ACTIONS(2842), 1,
      anon_sym_RPAREN,
    STATE(811), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1658), 1,
      aux_sym__old_style_parameter_list_repeat1,
    ACTIONS(1710), 2,
      anon_sym_STAR,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1712), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(1697), 24,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [34898] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1355), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(956), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1089), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [34967] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1355), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1016), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1089), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [35036] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2847), 3,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2845), 32,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [35079] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1360), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1016), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1099), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [35148] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2852), 1,
      anon_sym_LPAREN2,
    ACTIONS(2856), 1,
      anon_sym_LBRACK,
    STATE(811), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(1710), 2,
      anon_sym_COMMA,
      anon_sym_STAR,
    ACTIONS(2849), 2,
      anon_sym_RPAREN,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(1712), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(1697), 23,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [35200] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2646), 1,
      anon_sym_LPAREN2,
    STATE(811), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(1710), 2,
      anon_sym_STAR,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2859), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    ACTIONS(1712), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(1697), 24,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym___based,
      anon_sym_LBRACK,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_identifier,
  [35250] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2269), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    ACTIONS(2868), 1,
      anon_sym_PIPE,
    ACTIONS(2870), 1,
      anon_sym_CARET,
    ACTIONS(2872), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2874), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 9,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_QMARK,
      sym_identifier,
  [35321] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2419), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    ACTIONS(2868), 1,
      anon_sym_PIPE,
    ACTIONS(2870), 1,
      anon_sym_CARET,
    ACTIONS(2872), 1,
      anon_sym_AMP,
    ACTIONS(2882), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2884), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2886), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2874), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2417), 6,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_identifier,
  [35398] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2269), 7,
      aux_sym_preproc_elif_token1,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2271), 16,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_QMARK,
      sym_identifier,
  [35453] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2403), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    ACTIONS(2868), 1,
      anon_sym_PIPE,
    ACTIONS(2870), 1,
      anon_sym_CARET,
    ACTIONS(2872), 1,
      anon_sym_AMP,
    ACTIONS(2882), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2884), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2886), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2874), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2401), 6,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_identifier,
  [35530] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    ACTIONS(2868), 1,
      anon_sym_PIPE,
    ACTIONS(2870), 1,
      anon_sym_CARET,
    ACTIONS(2872), 1,
      anon_sym_AMP,
    ACTIONS(2882), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2884), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2886), 1,
      anon_sym_QMARK,
    ACTIONS(2890), 1,
      aux_sym_preproc_elif_token1,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2874), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2888), 6,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_identifier,
  [35607] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2269), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    ACTIONS(2868), 1,
      anon_sym_PIPE,
    ACTIONS(2870), 1,
      anon_sym_CARET,
    ACTIONS(2872), 1,
      anon_sym_AMP,
    ACTIONS(2884), 1,
      anon_sym_AMP_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2874), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 8,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_QMARK,
      sym_identifier,
  [35680] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2415), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    ACTIONS(2868), 1,
      anon_sym_PIPE,
    ACTIONS(2870), 1,
      anon_sym_CARET,
    ACTIONS(2872), 1,
      anon_sym_AMP,
    ACTIONS(2882), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2884), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2886), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2874), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2413), 6,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_identifier,
  [35757] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    ACTIONS(2870), 1,
      anon_sym_CARET,
    ACTIONS(2872), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2269), 2,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2874), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 9,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_QMARK,
      sym_identifier,
  [35826] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    ACTIONS(2872), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2269), 2,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2874), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 10,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_QMARK,
      sym_identifier,
  [35893] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2874), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2269), 3,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(2271), 10,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_QMARK,
      sym_identifier,
  [35958] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2876), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2878), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2269), 3,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(2271), 12,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_QMARK,
      sym_identifier,
  [36021] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2880), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2269), 5,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2271), 14,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_QMARK,
      sym_identifier,
  [36080] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2866), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2862), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2864), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2269), 5,
      aux_sym_preproc_elif_token1,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2271), 16,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_QMARK,
      sym_identifier,
  [36137] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2269), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2271), 14,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
  [36195] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2269), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2271), 10,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
  [36259] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2269), 1,
      anon_sym_PIPE,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2271), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
  [36327] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2347), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2910), 1,
      anon_sym_typedef,
    ACTIONS(2345), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [36369] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2271), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
  [36437] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2271), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
  [36507] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2916), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(2787), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [36547] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2347), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2918), 1,
      anon_sym_typedef,
    ACTIONS(2345), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [36589] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2269), 6,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2271), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
  [36643] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2347), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2920), 1,
      anon_sym_typedef,
    ACTIONS(2345), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [36685] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1202), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1200), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [36725] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1202), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1200), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [36765] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1174), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1172), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [36805] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2269), 1,
      anon_sym_PIPE,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2271), 10,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
  [36871] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1178), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1176), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [36911] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2269), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2271), 16,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
  [36967] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1178), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1176), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [37007] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2924), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(2922), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [37047] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2269), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2271), 12,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
      anon_sym_QMARK,
  [37109] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1122), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1120), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [37149] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2413), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
  [37223] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1170), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1168), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [37263] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1122), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1120), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [37303] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2932), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(2930), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [37343] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1170), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1168), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [37383] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1150), 2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
    ACTIONS(1148), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [37423] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2417), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
  [37497] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2347), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(2934), 1,
      anon_sym_typedef,
    ACTIONS(2345), 30,
      anon_sym___extension__,
      anon_sym_extern,
      anon_sym___attribute__,
      anon_sym___declspec,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_static,
      anon_sym_auto,
      anon_sym_register,
      anon_sym_inline,
      anon_sym___inline,
      anon_sym___inline__,
      anon_sym___forceinline,
      anon_sym_thread_local,
      anon_sym___thread,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [37539] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2401), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_RBRACE,
      anon_sym_COLON,
  [37613] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1749), 1,
      anon_sym_LPAREN2,
    ACTIONS(1751), 1,
      anon_sym_STAR,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1312), 1,
      sym__declarator,
    STATE(1446), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1803), 1,
      sym_ms_based_modifier,
    ACTIONS(2936), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [37676] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1749), 1,
      anon_sym_LPAREN2,
    ACTIONS(1751), 1,
      anon_sym_STAR,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1321), 1,
      sym__declarator,
    STATE(1475), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1803), 1,
      sym_ms_based_modifier,
    ACTIONS(2780), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [37739] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(47), 1,
      anon_sym_const,
    ACTIONS(1867), 1,
      anon_sym_LPAREN2,
    ACTIONS(1869), 1,
      anon_sym_STAR,
    ACTIONS(2749), 1,
      sym_ms_restrict_modifier,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1474), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    ACTIONS(2940), 2,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    ACTIONS(2942), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1004), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1125), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2747), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    ACTIONS(2938), 8,
      anon_sym___extension__,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [37804] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(47), 1,
      anon_sym_const,
    ACTIONS(1867), 1,
      anon_sym_LPAREN2,
    ACTIONS(1869), 1,
      anon_sym_STAR,
    ACTIONS(2749), 1,
      sym_ms_restrict_modifier,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1475), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    ACTIONS(2940), 2,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    ACTIONS(2942), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1016), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1124), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2780), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    ACTIONS(2938), 8,
      anon_sym___extension__,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [37869] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1312), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1016), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1137), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [37929] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1303), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1007), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1141), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [37989] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1321), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1016), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1136), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [38049] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2948), 11,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      sym_identifier,
    ACTIONS(2950), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
  [38087] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1344), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1015), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1130), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [38147] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1344), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1016), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1130), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [38207] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1321), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1005), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1136), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [38267] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2958), 11,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      sym_identifier,
    ACTIONS(2960), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
  [38305] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1335), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1010), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1131), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [38365] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2962), 11,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      sym_identifier,
    ACTIONS(2964), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
  [38403] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    STATE(1334), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    ACTIONS(2751), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1016), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    STATE(1138), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2749), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [38463] = 7,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1109), 1,
      sym_ms_unaligned_ptr_modifier,
    ACTIONS(2973), 2,
      anon_sym__unaligned,
      anon_sym___unaligned,
    STATE(1016), 2,
      sym_ms_pointer_modifier,
      aux_sym_pointer_declarator_repeat1,
    ACTIONS(2970), 3,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
    ACTIONS(2968), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(2966), 16,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [38509] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2976), 11,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
      sym_identifier,
    ACTIONS(2978), 19,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
  [38547] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2982), 3,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_COLON,
  [38621] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(2986), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    STATE(1672), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [38696] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(2988), 3,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [38767] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(2990), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    STATE(1567), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [38842] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(2992), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    STATE(1679), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [38917] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(2994), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    STATE(1583), 1,
      aux_sym_argument_list_repeat1,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [38992] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2996), 1,
      anon_sym_COMMA,
    ACTIONS(2998), 1,
      anon_sym_RBRACE,
    STATE(720), 1,
      sym_argument_list,
    STATE(1616), 1,
      aux_sym_initializer_list_repeat1,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [39067] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(3000), 1,
      anon_sym_COMMA,
    ACTIONS(3002), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    STATE(1677), 1,
      aux_sym_generic_expression_repeat1,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [39142] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3004), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [39214] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3006), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [39284] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(3008), 1,
      anon_sym_enum,
    STATE(1123), 1,
      sym__type_specifier,
    STATE(1139), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1868), 1,
      sym_type_descriptor,
    STATE(1107), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(1667), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [39340] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(3010), 1,
      sym_identifier,
    ACTIONS(3014), 1,
      sym_primitive_type,
    STATE(1176), 1,
      sym__type_specifier,
    STATE(1223), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1284), 1,
      sym__type_definition_type,
    STATE(1105), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3012), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [39396] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2269), 1,
      anon_sym_PIPE,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 5,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_RBRACK,
      anon_sym_QMARK,
  [39462] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3034), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [39534] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3036), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [39606] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3038), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [39678] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3040), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [39750] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(3010), 1,
      sym_identifier,
    ACTIONS(3014), 1,
      sym_primitive_type,
    STATE(1176), 1,
      sym__type_specifier,
    STATE(1223), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1280), 1,
      sym__type_definition_type,
    STATE(1105), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3012), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [39806] = 17,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2269), 1,
      anon_sym_PIPE,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 6,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_RBRACK,
      anon_sym_QMARK,
  [39870] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(1123), 1,
      sym__type_specifier,
    STATE(1139), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1960), 1,
      sym_type_descriptor,
    STATE(1102), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(1667), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [39926] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(1123), 1,
      sym__type_specifier,
    STATE(1139), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1968), 1,
      sym_type_descriptor,
    STATE(1102), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(1667), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [39982] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3042), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [40054] = 18,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 5,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_RBRACK,
      anon_sym_QMARK,
  [40120] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3046), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [40192] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3048), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [40264] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(3010), 1,
      sym_identifier,
    ACTIONS(3014), 1,
      sym_primitive_type,
    STATE(1176), 1,
      sym__type_specifier,
    STATE(1223), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1277), 1,
      sym__type_definition_type,
    STATE(1105), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3012), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [40320] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3050), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [40390] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2269), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 6,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_RBRACK,
      anon_sym_QMARK,
  [40452] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3052), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [40524] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3054), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [40596] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2269), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 8,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
  [40656] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3056), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [40726] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2401), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_RBRACK,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [40798] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2269), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2271), 10,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_RBRACK,
      anon_sym_QMARK,
  [40854] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(3010), 1,
      sym_identifier,
    ACTIONS(3014), 1,
      sym_primitive_type,
    STATE(1176), 1,
      sym__type_specifier,
    STATE(1223), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1276), 1,
      sym__type_definition_type,
    STATE(1105), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3012), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [40910] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2271), 4,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_RBRACK,
      anon_sym_QMARK,
  [40978] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3064), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [41050] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3066), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [41120] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3068), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [41192] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3070), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [41264] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3072), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [41336] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2413), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_RBRACK,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [41408] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(3010), 1,
      sym_identifier,
    ACTIONS(3014), 1,
      sym_primitive_type,
    STATE(1176), 1,
      sym__type_specifier,
    STATE(1223), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1282), 1,
      sym__type_definition_type,
    STATE(1105), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3012), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [41464] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2269), 6,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2271), 12,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
  [41516] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(1123), 1,
      sym__type_specifier,
    STATE(1139), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1957), 1,
      sym_type_descriptor,
    STATE(1102), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(1667), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [41572] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2269), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2271), 12,
      anon_sym_DOT_DOT_DOT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_RBRACK,
      anon_sym_QMARK,
  [41626] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3074), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [41698] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3076), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [41770] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2417), 2,
      anon_sym_DOT_DOT_DOT,
      anon_sym_RBRACK,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [41842] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3078), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [41914] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3080), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [41986] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3082), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [42058] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3084), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [42128] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(3010), 1,
      sym_identifier,
    ACTIONS(3014), 1,
      sym_primitive_type,
    STATE(1176), 1,
      sym__type_specifier,
    STATE(1223), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1279), 1,
      sym__type_definition_type,
    STATE(1105), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3012), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [42184] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(3008), 1,
      anon_sym_enum,
    STATE(1123), 1,
      sym__type_specifier,
    STATE(1139), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1815), 1,
      sym_type_descriptor,
    STATE(1107), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(1667), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [42240] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(3010), 1,
      sym_identifier,
    ACTIONS(3014), 1,
      sym_primitive_type,
    STATE(1176), 1,
      sym__type_specifier,
    STATE(1223), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1283), 1,
      sym__type_definition_type,
    STATE(1105), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3012), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [42296] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2888), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [42366] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3086), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [42438] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3088), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [42508] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(3010), 1,
      sym_identifier,
    ACTIONS(3014), 1,
      sym_primitive_type,
    STATE(1176), 1,
      sym__type_specifier,
    STATE(1223), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1278), 1,
      sym__type_definition_type,
    STATE(1105), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3012), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [42564] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3090), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [42636] = 22,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2255), 1,
      anon_sym_DASH_GT,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(2737), 1,
      anon_sym_DOT,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    ACTIONS(3092), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(3094), 1,
      anon_sym_RBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [42710] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3096), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [42782] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3098), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [42854] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3100), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [42926] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3102), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [42998] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3104), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [43068] = 21,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(2980), 1,
      anon_sym_COMMA,
    ACTIONS(3106), 1,
      anon_sym_SEMI,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [43140] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(3108), 1,
      anon_sym_COMMA,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [43209] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(3110), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [43278] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2008), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [43347] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1360), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [43400] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2044), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [43469] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2010), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [43538] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2038), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [43607] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2040), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [43676] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2042), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [43745] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2048), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [43814] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(3112), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [43883] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(3114), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [43952] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(3116), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [44021] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1361), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [44074] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2054), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [44143] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    ACTIONS(3118), 1,
      anon_sym_RBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [44212] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(1819), 1,
      sym_identifier,
    STATE(1122), 1,
      sym__type_specifier,
    STATE(1139), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(1667), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [44265] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(3120), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [44334] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2012), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [44403] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1669), 1,
      anon_sym_enum,
    ACTIONS(3010), 1,
      sym_identifier,
    ACTIONS(3014), 1,
      sym_primitive_type,
    STATE(1198), 1,
      sym__type_specifier,
    STATE(1223), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3012), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [44456] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(3122), 1,
      anon_sym_RPAREN,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [44525] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(49), 1,
      sym_primitive_type,
    ACTIONS(53), 1,
      anon_sym_struct,
    ACTIONS(55), 1,
      anon_sym_union,
    ACTIONS(1819), 1,
      sym_identifier,
    ACTIONS(3008), 1,
      anon_sym_enum,
    STATE(1122), 1,
      sym__type_specifier,
    STATE(1139), 1,
      aux_sym_sized_type_specifier_repeat1,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(1667), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(821), 5,
      sym_sized_type_specifier,
      sym_enum_specifier,
      sym_struct_specifier,
      sym_union_specifier,
      sym_macro_type_specifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [44578] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3126), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(3124), 21,
      anon_sym___extension__,
      anon_sym___based,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
      anon_sym__unaligned,
      anon_sym___unaligned,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [44613] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3130), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(3128), 21,
      anon_sym___extension__,
      anon_sym___based,
      sym_ms_restrict_modifier,
      sym_ms_unsigned_ptr_modifier,
      sym_ms_signed_ptr_modifier,
      anon_sym__unaligned,
      anon_sym___unaligned,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [44648] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1355), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [44701] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    ACTIONS(3132), 1,
      anon_sym_COLON,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [44770] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    ACTIONS(3134), 1,
      anon_sym_RBRACK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [44839] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2340), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(3136), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
    ACTIONS(2338), 10,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      sym_primitive_type,
      anon_sym_enum,
      anon_sym_struct,
      anon_sym_union,
      sym_identifier,
  [44878] = 20,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2050), 1,
      anon_sym_RBRACK,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2735), 1,
      anon_sym_LPAREN2,
    ACTIONS(3020), 1,
      anon_sym_SLASH,
    ACTIONS(3022), 1,
      anon_sym_CARET,
    ACTIONS(3024), 1,
      anon_sym_AMP,
    ACTIONS(3044), 1,
      anon_sym_PIPE,
    ACTIONS(3058), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3060), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3062), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(3016), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3018), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3026), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3028), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3030), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3032), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [44947] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3139), 1,
      anon_sym_SEMI,
    ACTIONS(2189), 7,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2191), 18,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
      anon_sym_LBRACK,
      anon_sym_QMARK,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
      anon_sym_DOT,
      anon_sym_DASH_GT,
  [44983] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(2725), 1,
      anon_sym_LBRACE,
    ACTIONS(3141), 1,
      anon_sym_COLON,
    STATE(808), 1,
      sym_attribute_specifier,
    STATE(1119), 1,
      sym_enumerator_list,
    ACTIONS(2720), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
    ACTIONS(2718), 16,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [45027] = 19,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(2251), 1,
      anon_sym_LBRACK,
    ACTIONS(2896), 1,
      anon_sym_SLASH,
    ACTIONS(2906), 1,
      anon_sym_CARET,
    ACTIONS(2908), 1,
      anon_sym_AMP,
    ACTIONS(2912), 1,
      anon_sym_PIPE,
    ACTIONS(2914), 1,
      anon_sym_AMP_AMP,
    ACTIONS(2926), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(2928), 1,
      anon_sym_QMARK,
    STATE(720), 1,
      sym_argument_list,
    ACTIONS(2253), 2,
      anon_sym_DASH_DASH,
      anon_sym_PLUS_PLUS,
    ACTIONS(2255), 2,
      anon_sym_DOT,
      anon_sym_DASH_GT,
    ACTIONS(2892), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(2894), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(2898), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(2900), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(2902), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2904), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [45093] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(2725), 1,
      anon_sym_LBRACE,
    STATE(839), 1,
      sym_attribute_specifier,
    STATE(1121), 1,
      sym_enumerator_list,
    ACTIONS(2775), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(2773), 16,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [45135] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(844), 1,
      sym_attribute_specifier,
    ACTIONS(2810), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(2808), 16,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [45171] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(824), 1,
      sym_attribute_specifier,
    ACTIONS(2817), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(2815), 16,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [45207] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    STATE(822), 1,
      sym_attribute_specifier,
    ACTIONS(2824), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(2822), 16,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [45243] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(47), 1,
      anon_sym_const,
    ACTIONS(1867), 1,
      anon_sym_LPAREN2,
    ACTIONS(1869), 1,
      anon_sym_STAR,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1469), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1126), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3143), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    ACTIONS(2938), 8,
      anon_sym___extension__,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45290] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(47), 1,
      anon_sym_const,
    ACTIONS(1867), 1,
      anon_sym_LPAREN2,
    ACTIONS(1869), 1,
      anon_sym_STAR,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1447), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1127), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3145), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    ACTIONS(2938), 8,
      anon_sym___extension__,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45337] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(47), 1,
      anon_sym_const,
    ACTIONS(1867), 1,
      anon_sym_LPAREN2,
    ACTIONS(1869), 1,
      anon_sym_STAR,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1446), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2936), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    ACTIONS(2938), 8,
      anon_sym___extension__,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45384] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(47), 1,
      anon_sym_const,
    ACTIONS(1867), 1,
      anon_sym_LPAREN2,
    ACTIONS(1869), 1,
      anon_sym_STAR,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1475), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(2780), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    ACTIONS(2938), 8,
      anon_sym___extension__,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45431] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(47), 1,
      anon_sym_const,
    ACTIONS(1867), 1,
      anon_sym_LPAREN2,
    ACTIONS(1869), 1,
      anon_sym_STAR,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1473), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3147), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    ACTIONS(2938), 8,
      anon_sym___extension__,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45478] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(47), 1,
      anon_sym_const,
    ACTIONS(1867), 1,
      anon_sym_LPAREN2,
    ACTIONS(1869), 1,
      anon_sym_STAR,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1449), 1,
      sym__abstract_declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3149), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    ACTIONS(2938), 8,
      anon_sym___extension__,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45525] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1288), 1,
      sym_ms_call_modifier,
    STATE(1437), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [45574] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(690), 1,
      sym__old_style_function_declarator,
    STATE(1309), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1362), 1,
      sym__declarator,
    STATE(1466), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1626), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [45628] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1334), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45672] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1344), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45716] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(699), 1,
      sym__old_style_function_declarator,
    STATE(1311), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1357), 1,
      sym__declarator,
    STATE(1471), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1631), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [45770] = 6,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2455), 2,
      sym_primitive_type,
      sym_identifier,
    ACTIONS(2459), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2627), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(2624), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45806] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(693), 1,
      sym__old_style_function_declarator,
    STATE(1319), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1358), 1,
      sym__declarator,
    STATE(1450), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1607), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [45860] = 15,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(713), 1,
      sym__old_style_function_declarator,
    STATE(1315), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1369), 1,
      sym__declarator,
    STATE(1444), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1622), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [45914] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1312), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [45958] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1307), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [46002] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1350), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [46046] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2662), 1,
      sym_primitive_type,
    ACTIONS(3153), 1,
      sym_identifier,
    STATE(1133), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(3155), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2656), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym_LBRACK,
      anon_sym_COLON,
    ACTIONS(2658), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [46084] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3159), 1,
      anon_sym_LPAREN2,
    STATE(1012), 1,
      sym_preproc_argument_list,
    ACTIONS(3161), 5,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3157), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [46118] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1321), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [46162] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1305), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1439), 1,
      sym__declarator,
    STATE(1480), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1596), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46213] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1311), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1379), 1,
      sym__declarator,
    STATE(1471), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1631), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46264] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1308), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1439), 1,
      sym__declarator,
    STATE(1450), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1607), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46315] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1313), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1439), 1,
      sym__declarator,
    STATE(1466), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1626), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46366] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3165), 1,
      anon_sym_RPAREN,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3175), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1148), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [46409] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1320), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1439), 1,
      sym__declarator,
    STATE(1472), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1594), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46460] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3179), 1,
      anon_sym_COMMA,
    ACTIONS(3181), 1,
      anon_sym_RPAREN,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3189), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3193), 1,
      anon_sym_PIPE,
    ACTIONS(3195), 1,
      anon_sym_CARET,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    STATE(1684), 1,
      aux_sym_preproc_argument_list_repeat1,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [46515] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1306), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1439), 1,
      sym__declarator,
    STATE(1499), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1712), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46566] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3179), 1,
      anon_sym_COMMA,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3189), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3193), 1,
      anon_sym_PIPE,
    ACTIONS(3195), 1,
      anon_sym_CARET,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    ACTIONS(3207), 1,
      anon_sym_RPAREN,
    STATE(1673), 1,
      aux_sym_preproc_argument_list_repeat1,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [46621] = 5,
    ACTIONS(3157), 1,
      anon_sym_LF,
    ACTIONS(3209), 1,
      anon_sym_LPAREN2,
    ACTIONS(3211), 1,
      sym_comment,
    STATE(1251), 1,
      sym_preproc_argument_list,
    ACTIONS(3161), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [46654] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3213), 1,
      anon_sym_RPAREN,
    ACTIONS(3215), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1150), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [46697] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1314), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1439), 1,
      sym__declarator,
    STATE(1471), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1631), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46748] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1315), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1375), 1,
      sym__declarator,
    STATE(1444), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1622), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46799] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1319), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1377), 1,
      sym__declarator,
    STATE(1450), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1607), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46850] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1309), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1373), 1,
      sym__declarator,
    STATE(1466), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1626), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [46901] = 16,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3179), 1,
      anon_sym_COMMA,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3189), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3193), 1,
      anon_sym_PIPE,
    ACTIONS(3195), 1,
      anon_sym_CARET,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    ACTIONS(3217), 1,
      anon_sym_RPAREN,
    STATE(1595), 1,
      aux_sym_preproc_argument_list_repeat1,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [46956] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1304), 1,
      sym_ms_call_modifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1439), 1,
      sym__declarator,
    STATE(1444), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1622), 1,
      sym_init_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [47007] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3219), 1,
      anon_sym_RPAREN,
    ACTIONS(3221), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1157), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47050] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3189), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3193), 1,
      anon_sym_PIPE,
    ACTIONS(3195), 1,
      anon_sym_CARET,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3223), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [47100] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3235), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1255), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47140] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3241), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3239), 11,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [47174] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3241), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3239), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
  [47210] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3241), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3239), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
  [47250] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3241), 2,
      anon_sym_PIPE,
      anon_sym_AMP,
    ACTIONS(3239), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
  [47292] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    ACTIONS(3241), 1,
      anon_sym_PIPE,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3239), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
  [47336] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3243), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1242), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47376] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3195), 1,
      anon_sym_CARET,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    ACTIONS(3241), 1,
      anon_sym_PIPE,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3239), 4,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [47422] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3193), 1,
      anon_sym_PIPE,
    ACTIONS(3195), 1,
      anon_sym_CARET,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3239), 4,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
  [47468] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3193), 1,
      anon_sym_PIPE,
    ACTIONS(3195), 1,
      anon_sym_CARET,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3239), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_PIPE_PIPE,
  [47516] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3245), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1227), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47556] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3247), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1266), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47596] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3249), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1248), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47636] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3241), 5,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3239), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [47664] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3241), 4,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3239), 13,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [47696] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3253), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    STATE(1196), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3251), 7,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      sym_primitive_type,
      sym_identifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [47728] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3255), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1252), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47768] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3257), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1221), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47808] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3259), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1254), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47848] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3263), 5,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3261), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [47876] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3265), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1257), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47916] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3267), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1264), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47956] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3269), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1247), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [47996] = 13,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1749), 1,
      anon_sym_LPAREN2,
    ACTIONS(1751), 1,
      anon_sym_STAR,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1394), 1,
      sym__declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1491), 1,
      sym__abstract_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    ACTIONS(3271), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
  [48044] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3273), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1244), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48084] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3275), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1238), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48124] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3277), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1231), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48164] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3279), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1256), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48204] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3281), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1230), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48244] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3283), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1170), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48284] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3285), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1207), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48324] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3289), 5,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3287), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [48352] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3291), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1225), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48392] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3293), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1237), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48432] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3295), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1246), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48472] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3299), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3297), 7,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      sym_primitive_type,
      sym_identifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [48504] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2225), 5,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(2227), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [48532] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3303), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    STATE(1218), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3301), 7,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      sym_primitive_type,
      sym_identifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [48564] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3305), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1243), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48604] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3307), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1250), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48644] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3309), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1240), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48684] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3311), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1234), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48724] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3313), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1174), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48764] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3315), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1175), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48804] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3317), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1160), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48844] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3319), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1232), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48884] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3323), 5,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3321), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [48912] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3325), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1162), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48952] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3327), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1163), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [48992] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3329), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1164), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [49032] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3333), 5,
      anon_sym_SLASH,
      anon_sym_PIPE,
      anon_sym_AMP,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3331), 15,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_CARET,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [49060] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3335), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1165), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [49100] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3337), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1166), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [49140] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3339), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1168), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [49180] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3341), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1267), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [49220] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3343), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1233), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [49260] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3163), 1,
      sym_identifier,
    ACTIONS(3167), 1,
      anon_sym_LPAREN2,
    ACTIONS(3169), 1,
      anon_sym_defined,
    ACTIONS(3345), 1,
      sym_number_literal,
    ACTIONS(3171), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3173), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3177), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1169), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [49300] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3349), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    STATE(1113), 2,
      sym_type_qualifier,
      aux_sym__type_definition_type_repeat1,
    ACTIONS(3347), 7,
      anon_sym___based,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
      sym_primitive_type,
      sym_identifier,
    ACTIONS(47), 9,
      anon_sym___extension__,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [49332] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3225), 1,
      sym_identifier,
    ACTIONS(3227), 1,
      anon_sym_LPAREN2,
    ACTIONS(3229), 1,
      anon_sym_defined,
    ACTIONS(3351), 1,
      sym_number_literal,
    ACTIONS(3231), 2,
      anon_sym_BANG,
      anon_sym_TILDE,
    ACTIONS(3233), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3237), 5,
      anon_sym_L_SQUOTE,
      anon_sym_u_SQUOTE,
      anon_sym_U_SQUOTE,
      anon_sym_u8_SQUOTE,
      anon_sym_SQUOTE,
    STATE(1262), 7,
      sym__preproc_expression,
      sym_preproc_parenthesized_expression,
      sym_preproc_defined,
      sym_preproc_unary_expression,
      sym_preproc_call_expression,
      sym_preproc_binary_expression,
      sym_char_literal,
  [49372] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3331), 1,
      anon_sym_LF,
    ACTIONS(3333), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [49399] = 10,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3241), 3,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [49440] = 3,
    ACTIONS(2227), 1,
      anon_sym_LF,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(2225), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [49467] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3367), 1,
      sym_identifier,
    ACTIONS(3373), 1,
      sym_primitive_type,
    STATE(1261), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2656), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    ACTIONS(3370), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2658), 10,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [49502] = 3,
    ACTIONS(2950), 1,
      anon_sym_LF,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(2948), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [49529] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3376), 1,
      anon_sym_LF,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [49574] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(709), 1,
      sym__old_style_function_declarator,
    STATE(1327), 1,
      sym_ms_call_modifier,
    STATE(1383), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [49617] = 4,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3241), 15,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [49646] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(695), 1,
      sym__old_style_function_declarator,
    STATE(1341), 1,
      sym_ms_call_modifier,
    STATE(1376), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [49689] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(707), 1,
      sym__old_style_function_declarator,
    STATE(1353), 1,
      sym_ms_call_modifier,
    STATE(1388), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [49732] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3384), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [49777] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3386), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [49822] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3388), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [49867] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3390), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [49912] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3392), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [49957] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1710), 1,
      anon_sym_STAR,
    ACTIONS(2646), 1,
      anon_sym_LPAREN2,
    STATE(1249), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(3394), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(1697), 12,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [49990] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3261), 1,
      anon_sym_LF,
    ACTIONS(3263), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [50017] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3189), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3193), 1,
      anon_sym_PIPE,
    ACTIONS(3195), 1,
      anon_sym_CARET,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    ACTIONS(3397), 1,
      anon_sym_RPAREN,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [50066] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3241), 13,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [50097] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2608), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    ACTIONS(3399), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2606), 12,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [50128] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3402), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50173] = 3,
    ACTIONS(2978), 1,
      anon_sym_LF,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(2976), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [50200] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3404), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50245] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3406), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50290] = 6,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3241), 11,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50323] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(703), 1,
      sym__old_style_function_declarator,
    STATE(1347), 1,
      sym_ms_call_modifier,
    STATE(1378), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [50366] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3321), 1,
      anon_sym_LF,
    ACTIONS(3323), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [50393] = 7,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
    ACTIONS(3241), 7,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
  [50428] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3241), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50473] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2526), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    ACTIONS(3408), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2524), 12,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [50504] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3411), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50549] = 3,
    ACTIONS(2960), 1,
      anon_sym_LF,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(2958), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [50576] = 11,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3241), 2,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50619] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2532), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    ACTIONS(3413), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2530), 12,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [50650] = 14,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3187), 1,
      anon_sym_SLASH,
    ACTIONS(3189), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3191), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3193), 1,
      anon_sym_PIPE,
    ACTIONS(3195), 1,
      anon_sym_CARET,
    ACTIONS(3197), 1,
      anon_sym_AMP,
    ACTIONS(3416), 1,
      anon_sym_RPAREN,
    ACTIONS(3183), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3185), 2,
      anon_sym_STAR,
      anon_sym_PERCENT,
    ACTIONS(3199), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3201), 2,
      anon_sym_GT,
      anon_sym_LT,
    ACTIONS(3203), 2,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
    ACTIONS(3205), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [50699] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3418), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50744] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3420), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50789] = 9,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3241), 4,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50828] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2536), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    ACTIONS(3422), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2534), 12,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [50859] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3287), 1,
      anon_sym_LF,
    ACTIONS(3289), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [50886] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1253), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2620), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    ACTIONS(3425), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2618), 12,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [50917] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(785), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2627), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    ACTIONS(3428), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2624), 12,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [50948] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3432), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [50993] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1258), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2548), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    ACTIONS(3434), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2546), 12,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [51024] = 8,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
    ACTIONS(3241), 5,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
  [51061] = 5,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1239), 1,
      aux_sym_sized_type_specifier_repeat1,
    ACTIONS(2558), 2,
      anon_sym_LPAREN2,
      anon_sym_STAR,
    ACTIONS(3437), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    ACTIONS(2556), 12,
      anon_sym___extension__,
      anon_sym___based,
      anon_sym_const,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
      sym_primitive_type,
      sym_identifier,
  [51092] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3239), 1,
      anon_sym_LF,
    ACTIONS(3241), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [51119] = 12,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3357), 1,
      anon_sym_CARET,
    ACTIONS(3359), 1,
      anon_sym_AMP,
    ACTIONS(3378), 1,
      anon_sym_PIPE_PIPE,
    ACTIONS(3380), 1,
      anon_sym_AMP_AMP,
    ACTIONS(3382), 1,
      anon_sym_PIPE,
    ACTIONS(3440), 1,
      anon_sym_LF,
    ACTIONS(3353), 2,
      anon_sym_DASH,
      anon_sym_PLUS,
    ACTIONS(3361), 2,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
    ACTIONS(3365), 2,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
    ACTIONS(3355), 3,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
    ACTIONS(3363), 4,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
  [51164] = 3,
    ACTIONS(2964), 1,
      anon_sym_LF,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(2962), 18,
      anon_sym_DASH,
      anon_sym_PLUS,
      anon_sym_STAR,
      anon_sym_SLASH,
      anon_sym_PERCENT,
      anon_sym_PIPE_PIPE,
      anon_sym_AMP_AMP,
      anon_sym_PIPE,
      anon_sym_CARET,
      anon_sym_AMP,
      anon_sym_EQ_EQ,
      anon_sym_BANG_EQ,
      anon_sym_GT,
      anon_sym_GT_EQ,
      anon_sym_LT_EQ,
      anon_sym_LT,
      anon_sym_LT_LT,
      anon_sym_GT_GT,
  [51191] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1332), 1,
      sym_ms_call_modifier,
    STATE(1421), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [51231] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1341), 1,
      sym_ms_call_modifier,
    STATE(1405), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [51271] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1347), 1,
      sym_ms_call_modifier,
    STATE(1401), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [51311] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1353), 1,
      sym_ms_call_modifier,
    STATE(1411), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [51351] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1327), 1,
      sym_ms_call_modifier,
    STATE(1397), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [51391] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1333), 1,
      sym_ms_call_modifier,
    STATE(1427), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
    ACTIONS(39), 6,
      anon_sym___cdecl,
      anon_sym___clrcall,
      anon_sym___stdcall,
      anon_sym___fastcall,
      anon_sym___thiscall,
      anon_sym___vectorcall,
  [51431] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1749), 1,
      anon_sym_LPAREN2,
    ACTIONS(1751), 1,
      anon_sym_STAR,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1425), 1,
      sym__declarator,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1534), 1,
      sym__abstract_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
  [51475] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1359), 1,
      sym__type_declarator,
    STATE(1542), 1,
      sym__type_definition_declarators,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51516] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1359), 1,
      sym__type_declarator,
    STATE(1531), 1,
      sym__type_definition_declarators,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51557] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1359), 1,
      sym__type_declarator,
    STATE(1547), 1,
      sym__type_definition_declarators,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51598] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1359), 1,
      sym__type_declarator,
    STATE(1550), 1,
      sym__type_definition_declarators,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51639] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1359), 1,
      sym__type_declarator,
    STATE(1508), 1,
      sym__type_definition_declarators,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51680] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2718), 1,
      anon_sym_const,
    ACTIONS(2725), 1,
      anon_sym_LBRACE,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(3444), 1,
      anon_sym_COLON,
    STATE(808), 1,
      sym_attribute_specifier,
    STATE(1119), 1,
      sym_enumerator_list,
    ACTIONS(2720), 11,
      anon_sym_LPAREN2,
      anon_sym_STAR,
      anon_sym___extension__,
      anon_sym_LBRACK,
      anon_sym_constexpr,
      anon_sym_volatile,
      anon_sym_restrict,
      anon_sym___restrict__,
      anon_sym__Atomic,
      anon_sym__Noreturn,
      anon_sym_noreturn,
  [51715] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1359), 1,
      sym__type_declarator,
    STATE(1555), 1,
      sym__type_definition_declarators,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51756] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1359), 1,
      sym__type_declarator,
    STATE(1510), 1,
      sym__type_definition_declarators,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51797] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1359), 1,
      sym__type_declarator,
    STATE(1505), 1,
      sym__type_definition_declarators,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51838] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(3447), 1,
      sym_identifier,
    ACTIONS(3453), 1,
      anon_sym_LBRACK,
    STATE(1291), 1,
      sym_gnu_asm_expression,
    STATE(1340), 1,
      sym_attribute_specifier,
    STATE(1424), 1,
      aux_sym_type_definition_repeat1,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(3449), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
    STATE(1297), 2,
      sym_preproc_call_expression,
      aux_sym_function_declarator_repeat1,
    ACTIONS(3451), 4,
      anon_sym_LPAREN2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
  [51878] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(3447), 1,
      sym_identifier,
    ACTIONS(3453), 1,
      anon_sym_LBRACK,
    STATE(1296), 1,
      sym_gnu_asm_expression,
    ACTIONS(89), 2,
      anon_sym_asm,
      anon_sym___asm__,
    STATE(1297), 3,
      sym_preproc_call_expression,
      sym_attribute_specifier,
      aux_sym_function_declarator_repeat1,
    ACTIONS(3451), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
  [51912] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1385), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51950] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2829), 1,
      sym_identifier,
    ACTIONS(2831), 1,
      anon_sym_LPAREN2,
    ACTIONS(2833), 1,
      anon_sym_STAR,
    ACTIONS(2837), 1,
      sym_primitive_type,
    STATE(1431), 1,
      sym__type_declarator,
    STATE(1913), 1,
      sym_ms_based_modifier,
    ACTIONS(2835), 4,
      anon_sym_signed,
      anon_sym_unsigned,
      anon_sym_long,
      anon_sym_short,
    STATE(1423), 5,
      sym_parenthesized_type_declarator,
      sym_attributed_type_declarator,
      sym_pointer_type_declarator,
      sym_function_type_declarator,
      sym_array_type_declarator,
  [51988] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    ACTIONS(3455), 1,
      anon_sym_SEMI,
    STATE(1323), 1,
      sym__field_declarator,
    STATE(1613), 1,
      sym__field_declaration_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    STATE(2001), 1,
      sym_attribute_specifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
  [52029] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3459), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(3461), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3463), 1,
      aux_sym_preproc_elif_token1,
    STATE(1348), 1,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1351), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1420), 1,
      sym_enumerator,
    ACTIONS(3465), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1776), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
    STATE(1780), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
  [52068] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(3447), 1,
      sym_identifier,
    ACTIONS(3471), 1,
      anon_sym_LBRACK,
    STATE(1340), 1,
      sym_attribute_specifier,
    STATE(1436), 1,
      aux_sym_type_definition_repeat1,
    ACTIONS(3467), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(3473), 2,
      anon_sym_asm,
      anon_sym___asm__,
    STATE(1293), 2,
      sym_preproc_call_expression,
      aux_sym_function_declarator_repeat1,
    ACTIONS(3469), 4,
      anon_sym_LPAREN2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
  [52105] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3475), 1,
      sym_identifier,
    ACTIONS(3480), 1,
      anon_sym___attribute__,
    ACTIONS(3483), 3,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
    STATE(1292), 3,
      sym_preproc_call_expression,
      sym_attribute_specifier,
      aux_sym_function_declarator_repeat1,
    ACTIONS(3478), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
  [52134] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(3447), 1,
      sym_identifier,
    ACTIONS(3487), 3,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
    STATE(1292), 3,
      sym_preproc_call_expression,
      sym_attribute_specifier,
      aux_sym_function_declarator_repeat1,
    ACTIONS(3485), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
  [52163] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3461), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3463), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3489), 1,
      aux_sym_preproc_if_token2,
    STATE(1336), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1338), 1,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1420), 1,
      sym_enumerator,
    ACTIONS(3465), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1944), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
    STATE(1947), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
  [52202] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    ACTIONS(3491), 1,
      anon_sym_SEMI,
    STATE(1323), 1,
      sym__field_declarator,
    STATE(1678), 1,
      sym__field_declaration_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    STATE(1850), 1,
      sym_attribute_specifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
  [52243] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(3447), 1,
      sym_identifier,
    ACTIONS(3471), 3,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
    STATE(1293), 3,
      sym_preproc_call_expression,
      sym_attribute_specifier,
      aux_sym_function_declarator_repeat1,
    ACTIONS(3469), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
  [52272] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(3447), 1,
      sym_identifier,
    ACTIONS(3471), 3,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
    STATE(1292), 3,
      sym_preproc_call_expression,
      sym_attribute_specifier,
      aux_sym_function_declarator_repeat1,
    ACTIONS(3469), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
  [52301] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3461), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3463), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3493), 1,
      aux_sym_preproc_if_token2,
    STATE(1339), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1346), 1,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1420), 1,
      sym_enumerator,
    ACTIONS(3465), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1899), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
    STATE(1900), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [52340] = 12,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    ACTIONS(3495), 1,
      anon_sym_SEMI,
    STATE(1323), 1,
      sym__field_declarator,
    STATE(1664), 1,
      sym__field_declaration_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    STATE(1904), 1,
      sym_attribute_specifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
  [52381] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3461), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3463), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3497), 1,
      aux_sym_preproc_if_token2,
    STATE(1325), 1,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1349), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1420), 1,
      sym_enumerator,
    ACTIONS(3465), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1907), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
    STATE(1919), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [52420] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3501), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3504), 1,
      anon_sym_LBRACK,
    STATE(1301), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3499), 10,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_COLON,
      anon_sym_asm,
      anon_sym___asm__,
  [52446] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2804), 5,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
      sym_identifier,
    ACTIONS(2797), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_COLON,
  [52467] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3506), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [52496] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1451), 1,
      sym__declarator,
    STATE(1459), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [52533] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1451), 1,
      sym__declarator,
    STATE(1461), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [52570] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1451), 1,
      sym__declarator,
    STATE(1494), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [52607] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3512), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [52636] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1451), 1,
      sym__declarator,
    STATE(1470), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [52673] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1406), 1,
      sym__declarator,
    STATE(1460), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [52710] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(17), 1,
      sym_preproc_directive,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3516), 1,
      aux_sym_preproc_if_token1,
    ACTIONS(3520), 1,
      anon_sym_RBRACE,
    ACTIONS(3518), 2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
    STATE(1726), 2,
      sym_preproc_call,
      sym_enumerator,
    STATE(1905), 2,
      sym_preproc_if_in_enumerator_list_no_comma,
      sym_preproc_ifdef_in_enumerator_list_no_comma,
    STATE(1328), 3,
      sym_preproc_if_in_enumerator_list,
      sym_preproc_ifdef_in_enumerator_list,
      aux_sym_enumerator_list_repeat1,
  [52743] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1414), 1,
      sym__declarator,
    STATE(1468), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [52780] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3522), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [52809] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1451), 1,
      sym__declarator,
    STATE(1460), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [52846] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1451), 1,
      sym__declarator,
    STATE(1468), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [52883] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1417), 1,
      sym__declarator,
    STATE(1459), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [52920] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3159), 1,
      anon_sym_LPAREN2,
    STATE(1012), 1,
      sym_preproc_argument_list,
    ACTIONS(3524), 5,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
      sym_identifier,
    ACTIONS(3526), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
  [52945] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(17), 1,
      sym_preproc_directive,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3516), 1,
      aux_sym_preproc_if_token1,
    ACTIONS(3528), 1,
      anon_sym_RBRACE,
    ACTIONS(3518), 2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
    STATE(1739), 2,
      sym_preproc_call,
      sym_enumerator,
    STATE(1928), 2,
      sym_preproc_if_in_enumerator_list_no_comma,
      sym_preproc_ifdef_in_enumerator_list_no_comma,
    STATE(1310), 3,
      sym_preproc_if_in_enumerator_list,
      sym_preproc_ifdef_in_enumerator_list,
      aux_sym_enumerator_list_repeat1,
  [52978] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3530), 5,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
      sym_identifier,
    ACTIONS(3532), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_COLON,
  [52999] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1402), 1,
      sym__declarator,
    STATE(1470), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [53036] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    ACTIONS(3151), 1,
      sym_identifier,
    STATE(1354), 1,
      sym_function_declarator,
    STATE(1451), 1,
      sym__declarator,
    STATE(1477), 1,
      sym__declaration_declarator,
    STATE(1501), 1,
      sym__function_declaration_declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1374), 4,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_array_declarator,
  [53073] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3534), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [53102] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2792), 5,
      anon_sym___attribute__,
      anon_sym_LBRACK,
      anon_sym_asm,
      anon_sym___asm__,
      sym_identifier,
    ACTIONS(2785), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_COLON,
  [53123] = 11,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3536), 1,
      anon_sym_COMMA,
    ACTIONS(3540), 1,
      anon_sym_LBRACK,
    ACTIONS(3542), 1,
      anon_sym_COLON,
    STATE(1393), 1,
      sym_parameter_list,
    STATE(1536), 1,
      aux_sym__field_declaration_declarator_repeat1,
    STATE(1537), 1,
      sym_bitfield_clause,
    ACTIONS(3538), 2,
      anon_sym_SEMI,
      anon_sym___attribute__,
    STATE(1368), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [53159] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3546), 1,
      anon_sym_LBRACK,
    STATE(1301), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3544), 8,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [53183] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3548), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(3550), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3552), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3554), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1410), 2,
      sym_enumerator,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1785), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
  [53212] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3550), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3552), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3556), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(3554), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1348), 2,
      sym_enumerator,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1780), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
  [53241] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1407), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
  [53270] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3558), 1,
      sym_identifier,
    ACTIONS(3561), 1,
      aux_sym_preproc_if_token1,
    ACTIONS(3567), 1,
      sym_preproc_directive,
    ACTIONS(3570), 1,
      anon_sym_RBRACE,
    ACTIONS(3564), 2,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
    STATE(1889), 2,
      sym_preproc_call,
      sym_enumerator,
    STATE(1328), 3,
      sym_preproc_if_in_enumerator_list,
      sym_preproc_ifdef_in_enumerator_list,
      aux_sym_enumerator_list_repeat1,
  [53299] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3497), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3572), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_elif_token1,
    STATE(1349), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
    ACTIONS(3576), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1919), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [53330] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3550), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3552), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3578), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(3554), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1325), 2,
      sym_enumerator,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1907), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
  [53359] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3572), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3580), 1,
      aux_sym_preproc_if_token2,
    STATE(1336), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
    ACTIONS(3576), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1944), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [53390] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1425), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
  [53419] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1429), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
  [53448] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3540), 1,
      anon_sym_LBRACK,
    STATE(1393), 1,
      sym_parameter_list,
    STATE(1368), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3582), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_COLON,
  [53475] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3540), 1,
      anon_sym_LBRACK,
    STATE(1393), 1,
      sym_parameter_list,
    STATE(1368), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3584), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_COLON,
  [53502] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3572), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3586), 1,
      aux_sym_preproc_if_token2,
    STATE(1409), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
    ACTIONS(3576), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1939), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [53533] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2952), 1,
      sym_identifier,
    ACTIONS(2954), 1,
      anon_sym_LPAREN2,
    ACTIONS(2956), 1,
      anon_sym_STAR,
    STATE(1342), 1,
      sym__field_declarator,
    STATE(1817), 1,
      sym_ms_based_modifier,
    STATE(1399), 5,
      sym_parenthesized_field_declarator,
      sym_attributed_field_declarator,
      sym_pointer_field_declarator,
      sym_function_field_declarator,
      sym_array_field_declarator,
  [53562] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3550), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3552), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3588), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(3554), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1410), 2,
      sym_enumerator,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1932), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
  [53591] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3572), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3590), 1,
      aux_sym_preproc_if_token2,
    STATE(1409), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
    ACTIONS(3576), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1962), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [53622] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3594), 1,
      anon_sym___attribute__,
    ACTIONS(3524), 2,
      anon_sym_LBRACK,
      sym_identifier,
    ACTIONS(3592), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
    ACTIONS(3597), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(3526), 4,
      anon_sym_LPAREN2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
  [53647] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1396), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
  [53676] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3540), 1,
      anon_sym_LBRACK,
    ACTIONS(3542), 1,
      anon_sym_COLON,
    STATE(1393), 1,
      sym_parameter_list,
    STATE(1641), 1,
      sym_bitfield_clause,
    STATE(1368), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3599), 3,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [53707] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3601), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    ACTIONS(3603), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
    STATE(1509), 2,
      sym__string,
      sym_concatenated_string,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [53732] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3540), 1,
      anon_sym_LBRACK,
    STATE(1393), 1,
      sym_parameter_list,
    STATE(1368), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3605), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_COLON,
  [53759] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3459), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3572), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_elif_token1,
    STATE(1351), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
    ACTIONS(3576), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1776), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [53790] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3550), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3552), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3607), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(3554), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1410), 2,
      sym_enumerator,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1966), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
  [53819] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1404), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
  [53848] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3550), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3552), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3609), 1,
      aux_sym_preproc_if_token2,
    ACTIONS(3554), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1410), 2,
      sym_enumerator,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1865), 3,
      sym_preproc_else_in_enumerator_list_no_comma,
      sym_preproc_elif_in_enumerator_list_no_comma,
      sym_preproc_elifdef_in_enumerator_list_no_comma,
  [53877] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3572), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3611), 1,
      aux_sym_preproc_if_token2,
    STATE(1409), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
    ACTIONS(3576), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1779), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [53908] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3540), 1,
      anon_sym_LBRACK,
    STATE(1393), 1,
      sym_parameter_list,
    STATE(1368), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3613), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_COLON,
  [53935] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3572), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3615), 1,
      aux_sym_preproc_if_token2,
    STATE(1409), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
    ACTIONS(3576), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1864), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [53966] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3572), 1,
      aux_sym_preproc_else_token1,
    ACTIONS(3574), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3617), 1,
      aux_sym_preproc_if_token2,
    STATE(1339), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
    ACTIONS(3576), 2,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
    STATE(1900), 3,
      sym_preproc_else_in_enumerator_list,
      sym_preproc_elif_in_enumerator_list,
      sym_preproc_elifdef_in_enumerator_list,
  [53997] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1753), 1,
      anon_sym___based,
    ACTIONS(2745), 1,
      sym_identifier,
    ACTIONS(2944), 1,
      anon_sym_LPAREN2,
    ACTIONS(2946), 1,
      anon_sym_STAR,
    STATE(1403), 1,
      sym__declarator,
    STATE(1803), 1,
      sym_ms_based_modifier,
    STATE(1354), 5,
      sym_parenthesized_declarator,
      sym_attributed_declarator,
      sym_pointer_declarator,
      sym_function_declarator,
      sym_array_declarator,
  [54026] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3621), 1,
      anon_sym_LBRACK,
    ACTIONS(3619), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [54044] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3625), 1,
      anon_sym_LBRACK,
    STATE(1428), 1,
      sym_parameter_list,
    STATE(1386), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3623), 4,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [54070] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3629), 1,
      anon_sym_LBRACK,
    ACTIONS(3627), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [54088] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3631), 1,
      anon_sym_LPAREN2,
    ACTIONS(3633), 1,
      anon_sym_EQ,
    STATE(149), 1,
      sym_compound_statement,
    STATE(989), 1,
      sym__old_style_parameter_list,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54120] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3631), 1,
      anon_sym_LPAREN2,
    ACTIONS(3633), 1,
      anon_sym_EQ,
    STATE(401), 1,
      sym_compound_statement,
    STATE(989), 1,
      sym__old_style_parameter_list,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54152] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3625), 1,
      anon_sym_LBRACK,
    ACTIONS(3635), 1,
      anon_sym_COMMA,
    STATE(1428), 1,
      sym_parameter_list,
    STATE(1543), 1,
      aux_sym__type_definition_declarators_repeat1,
    ACTIONS(3637), 2,
      anon_sym_SEMI,
      anon_sym___attribute__,
    STATE(1386), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54182] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3625), 1,
      anon_sym_LBRACK,
    STATE(1428), 1,
      sym_parameter_list,
    STATE(1386), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3639), 4,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [54208] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3625), 1,
      anon_sym_LBRACK,
    STATE(1428), 1,
      sym_parameter_list,
    STATE(1386), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3641), 4,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [54234] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3631), 1,
      anon_sym_LPAREN2,
    ACTIONS(3633), 1,
      anon_sym_EQ,
    STATE(419), 1,
      sym_compound_statement,
    STATE(989), 1,
      sym__old_style_parameter_list,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54266] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3645), 1,
      anon_sym_LBRACK,
    ACTIONS(3643), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [54284] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3649), 1,
      anon_sym_LBRACK,
    ACTIONS(3647), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [54302] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_LBRACK,
    STATE(1521), 1,
      sym_gnu_asm_output_operand,
    STATE(1826), 1,
      sym_string_literal,
    ACTIONS(3651), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [54326] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3625), 1,
      anon_sym_LBRACK,
    STATE(1428), 1,
      sym_parameter_list,
    STATE(1386), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3655), 4,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [54352] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3659), 1,
      anon_sym_LBRACK,
    ACTIONS(3657), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [54370] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3663), 1,
      anon_sym_LBRACK,
    STATE(1301), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3661), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_COLON,
  [54392] = 10,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3631), 1,
      anon_sym_LPAREN2,
    ACTIONS(3633), 1,
      anon_sym_EQ,
    STATE(336), 1,
      sym_compound_statement,
    STATE(989), 1,
      sym__old_style_parameter_list,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54424] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3667), 1,
      anon_sym_LBRACK,
    STATE(1529), 1,
      sym_gnu_asm_input_operand,
    STATE(1990), 1,
      sym_string_literal,
    ACTIONS(3665), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [54448] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3671), 1,
      anon_sym_LBRACK,
    ACTIONS(3669), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [54466] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3675), 1,
      anon_sym_LBRACK,
    ACTIONS(3673), 9,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
      anon_sym_asm,
      anon_sym___asm__,
  [54484] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3633), 1,
      anon_sym_EQ,
    STATE(419), 1,
      sym_compound_statement,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54513] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3621), 1,
      anon_sym_LBRACK,
    ACTIONS(3619), 4,
      anon_sym_LPAREN2,
      anon_sym_LBRACK_LBRACK,
      anon_sym_LBRACE,
      anon_sym_EQ,
    ACTIONS(3677), 4,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_asm,
      anon_sym___asm__,
  [54532] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3633), 1,
      anon_sym_EQ,
    STATE(336), 1,
      sym_compound_statement,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54561] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3631), 1,
      anon_sym_LPAREN2,
    STATE(326), 1,
      sym_compound_statement,
    STATE(989), 1,
      sym__old_style_parameter_list,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54590] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3633), 1,
      anon_sym_EQ,
    STATE(401), 1,
      sym_compound_statement,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54619] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3631), 1,
      anon_sym_LPAREN2,
    STATE(139), 1,
      sym_compound_statement,
    STATE(989), 1,
      sym__old_style_parameter_list,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54648] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3633), 1,
      anon_sym_EQ,
    STATE(149), 1,
      sym_compound_statement,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54677] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3601), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1602), 2,
      sym__string,
      sym_concatenated_string,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [54698] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3679), 1,
      sym_identifier,
    ACTIONS(3683), 1,
      sym_system_lib_string,
    STATE(1908), 2,
      sym_preproc_call_expression,
      sym_string_literal,
    ACTIONS(3681), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [54719] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3601), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1649), 2,
      sym__string,
      sym_concatenated_string,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [54740] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3631), 1,
      anon_sym_LPAREN2,
    STATE(382), 1,
      sym_compound_statement,
    STATE(989), 1,
      sym__old_style_parameter_list,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54769] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3685), 1,
      sym_identifier,
    ACTIONS(3687), 1,
      sym_system_lib_string,
    STATE(1931), 2,
      sym_preproc_call_expression,
      sym_string_literal,
    ACTIONS(3681), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [54790] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3625), 1,
      anon_sym_LBRACK,
    STATE(1428), 1,
      sym_parameter_list,
    STATE(1386), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3689), 3,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [54815] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3693), 1,
      anon_sym_LBRACK,
    STATE(1301), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
    ACTIONS(3691), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [54836] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3601), 1,
      sym_identifier,
    STATE(683), 1,
      sym_string_literal,
    STATE(1582), 2,
      sym__string,
      sym_concatenated_string,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [54857] = 9,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3631), 1,
      anon_sym_LPAREN2,
    STATE(413), 1,
      sym_compound_statement,
    STATE(989), 1,
      sym__old_style_parameter_list,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [54886] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3695), 1,
      sym_identifier,
    ACTIONS(3697), 1,
      sym_system_lib_string,
    STATE(1875), 2,
      sym_preproc_call_expression,
      sym_string_literal,
    ACTIONS(3681), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [54907] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1867), 1,
      anon_sym_LPAREN2,
    ACTIONS(1869), 1,
      anon_sym_STAR,
    ACTIONS(2753), 1,
      anon_sym_LBRACK,
    STATE(1486), 1,
      sym_parameter_list,
    STATE(1534), 1,
      sym__abstract_declarator,
    STATE(1496), 4,
      sym_abstract_parenthesized_declarator,
      sym_abstract_pointer_declarator,
      sym_abstract_function_declarator,
      sym_abstract_array_declarator,
  [54932] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3699), 1,
      sym_identifier,
    ACTIONS(3701), 1,
      sym_system_lib_string,
    STATE(1812), 2,
      sym_preproc_call_expression,
      sym_string_literal,
    ACTIONS(3681), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [54953] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3705), 1,
      anon_sym_LBRACK,
    ACTIONS(3703), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
  [54969] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3709), 1,
      anon_sym_LBRACK,
    ACTIONS(3707), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
  [54985] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(1286), 1,
      sym_parameter_list,
    ACTIONS(3711), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55009] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3715), 1,
      anon_sym_LBRACK,
    ACTIONS(3713), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
  [55025] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(309), 1,
      sym_compound_statement,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55051] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(382), 1,
      sym_compound_statement,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55077] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3719), 1,
      anon_sym_LBRACK,
    ACTIONS(3717), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
  [55093] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3723), 1,
      anon_sym_LBRACK,
    ACTIONS(3721), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
  [55109] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3727), 1,
      anon_sym_LBRACK,
    ACTIONS(3725), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
  [55125] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(139), 1,
      sym_compound_statement,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55151] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(392), 1,
      sym_compound_statement,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55177] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(427), 1,
      sym_compound_statement,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55203] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(145), 1,
      sym_compound_statement,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55229] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(326), 1,
      sym_compound_statement,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55255] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(433), 1,
      sym_compound_statement,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55281] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(347), 1,
      sym_compound_statement,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55307] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3667), 1,
      anon_sym_LBRACK,
    STATE(1675), 1,
      sym_gnu_asm_input_operand,
    STATE(1990), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [55327] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3729), 1,
      sym_identifier,
    ACTIONS(3734), 1,
      aux_sym_preproc_elif_token1,
    STATE(1409), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
    ACTIONS(3732), 4,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
  [55349] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3736), 1,
      sym_identifier,
    ACTIONS(3741), 1,
      aux_sym_preproc_elif_token1,
    STATE(1410), 2,
      sym_enumerator,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    ACTIONS(3739), 4,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
  [55369] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(413), 1,
      sym_compound_statement,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55395] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3745), 1,
      anon_sym_LBRACK,
    ACTIONS(3743), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
  [55411] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3749), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3751), 1,
      anon_sym_EQ,
    ACTIONS(3747), 6,
      anon_sym_COMMA,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_identifier,
  [55429] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(128), 1,
      sym_compound_statement,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55455] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3755), 1,
      anon_sym_LBRACK,
    ACTIONS(3753), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
  [55471] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3759), 1,
      anon_sym_LBRACK,
    ACTIONS(3757), 7,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
      anon_sym_COLON,
  [55487] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(328), 1,
      sym_compound_statement,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55513] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3653), 1,
      anon_sym_LBRACK,
    STATE(1637), 1,
      sym_gnu_asm_output_operand,
    STATE(1826), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [55533] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3763), 1,
      anon_sym_LBRACK,
    ACTIONS(3761), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55548] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3767), 1,
      anon_sym_COMMA,
    ACTIONS(3769), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3765), 5,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_identifier,
  [55565] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3771), 1,
      anon_sym_RPAREN,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55588] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3775), 1,
      anon_sym_LBRACK,
    ACTIONS(3773), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55603] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3779), 1,
      anon_sym_LBRACK,
    ACTIONS(3777), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55618] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
    ACTIONS(3467), 4,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_asm,
      anon_sym___asm__,
  [55635] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3781), 1,
      anon_sym_RPAREN,
    STATE(1286), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55658] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3785), 1,
      anon_sym_LBRACK,
    ACTIONS(3783), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55673] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3540), 1,
      anon_sym_LBRACK,
    ACTIONS(3787), 1,
      anon_sym_RPAREN,
    STATE(1393), 1,
      sym_parameter_list,
    STATE(1368), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55696] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3791), 1,
      anon_sym_LBRACK,
    ACTIONS(3789), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55711] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3540), 1,
      anon_sym_LBRACK,
    ACTIONS(3793), 1,
      anon_sym_RPAREN,
    STATE(1393), 1,
      sym_parameter_list,
    STATE(1368), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55734] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1811), 1,
      anon_sym_LBRACK,
    ACTIONS(3795), 1,
      anon_sym_EQ,
    ACTIONS(3797), 1,
      anon_sym_DOT,
    STATE(1440), 4,
      sym_subscript_designator,
      sym_subscript_range_designator,
      sym_field_designator,
      aux_sym_initializer_pair_repeat1,
  [55753] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3625), 1,
      anon_sym_LBRACK,
    ACTIONS(3799), 1,
      anon_sym_RPAREN,
    STATE(1428), 1,
      sym_parameter_list,
    STATE(1386), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55776] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3803), 1,
      anon_sym_LBRACK,
    ACTIONS(3801), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55791] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3807), 1,
      anon_sym___attribute__,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
    ACTIONS(3805), 4,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_asm,
      anon_sym___asm__,
  [55808] = 8,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(33), 1,
      anon_sym___attribute__,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    ACTIONS(3810), 1,
      sym_identifier,
    STATE(788), 1,
      sym_field_declaration_list,
    STATE(1483), 1,
      sym_attribute_specifier,
    STATE(1588), 1,
      sym_ms_declspec_modifier,
  [55833] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3814), 1,
      anon_sym_LBRACK,
    ACTIONS(3812), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55848] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
    ACTIONS(3816), 4,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_asm,
      anon_sym___asm__,
  [55865] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3625), 1,
      anon_sym_LBRACK,
    ACTIONS(3818), 1,
      anon_sym_RPAREN,
    STATE(1428), 1,
      sym_parameter_list,
    STATE(1386), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55888] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3822), 1,
      anon_sym_LBRACK,
    ACTIONS(3820), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55903] = 7,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    ACTIONS(3633), 1,
      anon_sym_EQ,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [55926] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3824), 1,
      anon_sym_LBRACK,
    ACTIONS(3827), 1,
      anon_sym_EQ,
    ACTIONS(3829), 1,
      anon_sym_DOT,
    STATE(1440), 4,
      sym_subscript_designator,
      sym_subscript_range_designator,
      sym_field_designator,
      aux_sym_initializer_pair_repeat1,
  [55945] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3834), 1,
      anon_sym_LBRACK,
    ACTIONS(3832), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55960] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3838), 1,
      anon_sym_LBRACK,
    ACTIONS(3836), 6,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_SEMI,
      anon_sym___attribute__,
      anon_sym_LBRACK_LBRACK,
  [55975] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3840), 1,
      anon_sym_LPAREN2,
    STATE(1445), 2,
      sym_gnu_asm_qualifier,
      aux_sym_gnu_asm_expression_repeat1,
    ACTIONS(3842), 3,
      anon_sym_inline,
      anon_sym_volatile,
      anon_sym_goto,
  [55991] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3846), 1,
      anon_sym_SEMI,
    STATE(1566), 1,
      aux_sym_declaration_repeat1,
    STATE(1625), 1,
      sym_gnu_asm_expression,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56011] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3850), 1,
      anon_sym_LPAREN2,
    STATE(1445), 2,
      sym_gnu_asm_qualifier,
      aux_sym_gnu_asm_expression_repeat1,
    ACTIONS(3852), 3,
      anon_sym_inline,
      anon_sym_volatile,
      anon_sym_goto,
  [56027] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    STATE(1490), 1,
      sym_parameter_list,
    ACTIONS(3855), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [56045] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    STATE(1490), 1,
      sym_parameter_list,
    ACTIONS(3859), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [56063] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3861), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3863), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56077] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    STATE(1490), 1,
      sym_parameter_list,
    ACTIONS(3865), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [56095] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3867), 1,
      anon_sym_SEMI,
    STATE(1575), 1,
      aux_sym_declaration_repeat1,
    STATE(1576), 1,
      sym_gnu_asm_expression,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56115] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(35), 1,
      anon_sym_LBRACK_LBRACK,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3510), 1,
      anon_sym_LBRACK,
    STATE(1285), 1,
      sym_parameter_list,
    STATE(1324), 2,
      sym_attribute_declaration,
      aux_sym_attributed_declarator_repeat1,
  [56135] = 3,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1871), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [56149] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3869), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3871), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56163] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3873), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3875), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56177] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3877), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3879), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56191] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3570), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3881), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56205] = 3,
    ACTIONS(3), 1,
      sym_comment,
    STATE(662), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [56219] = 3,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1849), 1,
      sym_string_literal,
    ACTIONS(95), 5,
      anon_sym_L_DQUOTE,
      anon_sym_u_DQUOTE,
      anon_sym_U_DQUOTE,
      anon_sym_u8_DQUOTE,
      anon_sym_DQUOTE,
  [56233] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3883), 1,
      anon_sym_SEMI,
    STATE(1639), 1,
      sym_gnu_asm_expression,
    STATE(1662), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56253] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3885), 1,
      anon_sym_SEMI,
    STATE(1592), 1,
      sym_gnu_asm_expression,
    STATE(1600), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56273] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3887), 1,
      anon_sym_SEMI,
    STATE(1608), 1,
      sym_gnu_asm_expression,
    STATE(1609), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56293] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3889), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3891), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56307] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3893), 1,
      anon_sym_LPAREN2,
    STATE(1443), 2,
      sym_gnu_asm_qualifier,
      aux_sym_gnu_asm_expression_repeat1,
    ACTIONS(3842), 3,
      anon_sym_inline,
      anon_sym_volatile,
      anon_sym_goto,
  [56323] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3895), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3897), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56337] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3734), 1,
      aux_sym_preproc_elif_token1,
    ACTIONS(3732), 5,
      aux_sym_preproc_if_token2,
      aux_sym_preproc_else_token1,
      aux_sym_preproc_elifdef_token1,
      aux_sym_preproc_elifdef_token2,
      sym_identifier,
  [56351] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3899), 1,
      anon_sym_SEMI,
    STATE(1571), 1,
      aux_sym_declaration_repeat1,
    STATE(1572), 1,
      sym_gnu_asm_expression,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56371] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3901), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3903), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56385] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3905), 1,
      anon_sym_SEMI,
    STATE(1681), 1,
      sym_gnu_asm_expression,
    STATE(1682), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56405] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    STATE(1490), 1,
      sym_parameter_list,
    ACTIONS(3907), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [56423] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3909), 1,
      anon_sym_SEMI,
    STATE(1628), 1,
      sym_gnu_asm_expression,
    STATE(1636), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56443] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3911), 1,
      anon_sym_SEMI,
    STATE(1652), 1,
      sym_gnu_asm_expression,
    STATE(1653), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56463] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3913), 1,
      anon_sym_SEMI,
    STATE(1585), 1,
      aux_sym_declaration_repeat1,
    STATE(1587), 1,
      sym_gnu_asm_expression,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56483] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    STATE(1490), 1,
      sym_parameter_list,
    ACTIONS(3915), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [56501] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    STATE(1490), 1,
      sym_parameter_list,
    ACTIONS(3917), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [56519] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    STATE(1490), 1,
      sym_parameter_list,
    ACTIONS(3919), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [56537] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3921), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3923), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56551] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3925), 1,
      anon_sym_SEMI,
    STATE(1577), 1,
      aux_sym_declaration_repeat1,
    STATE(1578), 1,
      sym_gnu_asm_expression,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56571] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3877), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3879), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56585] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3861), 2,
      anon_sym_RBRACE,
      sym_identifier,
    ACTIONS(3863), 4,
      aux_sym_preproc_if_token1,
      aux_sym_preproc_ifdef_token1,
      aux_sym_preproc_ifdef_token2,
      sym_preproc_directive,
  [56599] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(3927), 1,
      anon_sym_SEMI,
    STATE(1601), 1,
      sym_gnu_asm_expression,
    STATE(1605), 1,
      aux_sym_declaration_repeat1,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
  [56619] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3929), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56630] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3931), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56641] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    ACTIONS(3933), 1,
      sym_identifier,
    STATE(783), 1,
      sym_field_declaration_list,
    STATE(1599), 1,
      sym_ms_declspec_modifier,
  [56660] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3935), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56671] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    ACTIONS(3939), 1,
      anon_sym_COLON_COLON,
    STATE(1728), 1,
      sym_argument_list,
    ACTIONS(3937), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK_RBRACK,
  [56688] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3941), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56699] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3943), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56710] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3945), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56721] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3947), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56732] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3949), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56743] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    STATE(1490), 1,
      sym_parameter_list,
    ACTIONS(3711), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [56760] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3951), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56771] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(3953), 1,
      aux_sym_preproc_if_token2,
    STATE(1420), 1,
      sym_enumerator,
    STATE(1553), 1,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
    STATE(1554), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
  [56790] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1766), 1,
      sym_gnu_asm_expression,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(3955), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [56805] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3957), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56816] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3959), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56827] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3961), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56838] = 6,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(37), 1,
      anon_sym___declspec,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    ACTIONS(3963), 1,
      sym_identifier,
    STATE(782), 1,
      sym_field_declaration_list,
    STATE(1603), 1,
      sym_ms_declspec_modifier,
  [56857] = 4,
    ACTIONS(3), 1,
      sym_comment,
    STATE(1698), 1,
      sym_gnu_asm_expression,
    ACTIONS(3848), 2,
      anon_sym_asm,
      anon_sym___asm__,
    ACTIONS(3965), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [56872] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3967), 5,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_LPAREN2,
      anon_sym_LBRACK,
      anon_sym_COLON,
  [56883] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3969), 4,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym_asm,
      anon_sym___asm__,
  [56893] = 4,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(3971), 1,
      anon_sym_SQUOTE,
    STATE(1560), 1,
      aux_sym_char_literal_repeat1,
    ACTIONS(3973), 2,
      aux_sym_char_literal_token1,
      sym_escape_sequence,
  [56907] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(3975), 1,
      anon_sym_SEMI,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [56921] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(3977), 1,
      anon_sym_SEMI,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [56935] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(3979), 1,
      anon_sym_SEMI,
    STATE(1503), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [56949] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3981), 1,
      anon_sym_COMMA,
    ACTIONS(3983), 1,
      anon_sym_RPAREN,
    STATE(1569), 1,
      aux_sym_parameter_list_repeat1,
    STATE(1658), 1,
      aux_sym__old_style_parameter_list_repeat1,
  [56965] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3985), 1,
      anon_sym___except,
    ACTIONS(3987), 1,
      anon_sym___finally,
    STATE(191), 2,
      sym_seh_except_clause,
      sym_seh_finally_clause,
  [56979] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(3989), 1,
      anon_sym_SEMI,
    STATE(1504), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [56993] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3991), 1,
      anon_sym_COMMA,
    STATE(1533), 1,
      aux_sym_gnu_asm_clobber_list_repeat1,
    ACTIONS(3993), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [57007] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(3995), 1,
      anon_sym_SEMI,
    STATE(1535), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57021] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(3997), 1,
      anon_sym_SEMI,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57035] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3999), 1,
      anon_sym_COMMA,
    STATE(1512), 1,
      aux_sym__field_declaration_declarator_repeat1,
    ACTIONS(4002), 2,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [57049] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4004), 1,
      anon_sym_COMMA,
    STATE(1532), 1,
      aux_sym_gnu_asm_input_operand_list_repeat1,
    ACTIONS(4006), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [57063] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4008), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4010), 1,
      anon_sym_LPAREN,
    ACTIONS(4012), 1,
      sym_preproc_arg,
    STATE(1746), 1,
      sym_preproc_params,
  [57079] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4014), 1,
      anon_sym_DQUOTE,
    ACTIONS(4016), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4018), 1,
      sym_escape_sequence,
    STATE(1557), 1,
      aux_sym_string_literal_repeat1,
  [57095] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3536), 1,
      anon_sym_COMMA,
    STATE(1512), 1,
      aux_sym__field_declaration_declarator_repeat1,
    ACTIONS(4020), 2,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [57109] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4010), 1,
      anon_sym_LPAREN,
    ACTIONS(4022), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4024), 1,
      sym_preproc_arg,
    STATE(1755), 1,
      sym_preproc_params,
  [57125] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4010), 1,
      anon_sym_LPAREN,
    ACTIONS(4026), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4028), 1,
      sym_preproc_arg,
    STATE(1727), 1,
      sym_preproc_params,
  [57141] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4030), 1,
      anon_sym_COMMA,
    STATE(1519), 1,
      aux_sym_gnu_asm_output_operand_list_repeat1,
    ACTIONS(4033), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [57155] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4035), 1,
      anon_sym_COMMA,
    STATE(1519), 1,
      aux_sym_gnu_asm_output_operand_list_repeat1,
    ACTIONS(4037), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [57169] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4035), 1,
      anon_sym_COMMA,
    STATE(1520), 1,
      aux_sym_gnu_asm_output_operand_list_repeat1,
    ACTIONS(4039), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [57183] = 4,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4041), 1,
      anon_sym_SQUOTE,
    STATE(1560), 1,
      aux_sym_char_literal_repeat1,
    ACTIONS(3973), 2,
      aux_sym_char_literal_token1,
      sym_escape_sequence,
  [57197] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    ACTIONS(4043), 1,
      anon_sym_RPAREN,
    STATE(1490), 1,
      sym_parameter_list,
  [57213] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4045), 4,
      anon_sym_LPAREN2,
      anon_sym_inline,
      anon_sym_volatile,
      anon_sym_goto,
  [57223] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4047), 1,
      anon_sym_DQUOTE,
    ACTIONS(4049), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4051), 1,
      sym_escape_sequence,
    STATE(1515), 1,
      aux_sym_string_literal_repeat1,
  [57239] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4010), 1,
      anon_sym_LPAREN,
    ACTIONS(4053), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4055), 1,
      sym_preproc_arg,
    STATE(1699), 1,
      sym_preproc_params,
  [57255] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4057), 1,
      anon_sym_SEMI,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57269] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4059), 1,
      anon_sym_DQUOTE,
    ACTIONS(4061), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4063), 1,
      sym_escape_sequence,
    STATE(1564), 1,
      aux_sym_string_literal_repeat1,
  [57285] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4004), 1,
      anon_sym_COMMA,
    STATE(1513), 1,
      aux_sym_gnu_asm_input_operand_list_repeat1,
    ACTIONS(4065), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [57299] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4067), 1,
      anon_sym_SEMI,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57313] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4069), 1,
      anon_sym_SEMI,
    STATE(1527), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57327] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4071), 1,
      anon_sym_COMMA,
    STATE(1532), 1,
      aux_sym_gnu_asm_input_operand_list_repeat1,
    ACTIONS(4074), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [57341] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3991), 1,
      anon_sym_COMMA,
    STATE(1539), 1,
      aux_sym_gnu_asm_clobber_list_repeat1,
    ACTIONS(4076), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [57355] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3508), 1,
      anon_sym_LPAREN2,
    ACTIONS(3857), 1,
      anon_sym_LBRACK,
    ACTIONS(4078), 1,
      anon_sym_RPAREN,
    STATE(1490), 1,
      sym_parameter_list,
  [57371] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4080), 1,
      anon_sym_SEMI,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57385] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3536), 1,
      anon_sym_COMMA,
    STATE(1512), 1,
      aux_sym__field_declaration_declarator_repeat1,
    ACTIONS(4082), 2,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [57399] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3536), 1,
      anon_sym_COMMA,
    STATE(1516), 1,
      aux_sym__field_declaration_declarator_repeat1,
    ACTIONS(4084), 2,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [57413] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4010), 1,
      anon_sym_LPAREN,
    ACTIONS(4086), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4088), 1,
      sym_preproc_arg,
    STATE(1763), 1,
      sym_preproc_params,
  [57429] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4090), 1,
      anon_sym_COMMA,
    STATE(1539), 1,
      aux_sym_gnu_asm_clobber_list_repeat1,
    ACTIONS(4093), 2,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [57443] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4095), 1,
      anon_sym___except,
    ACTIONS(4097), 1,
      anon_sym___finally,
    STATE(242), 2,
      sym_seh_except_clause,
      sym_seh_finally_clause,
  [57457] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4099), 1,
      anon_sym___except,
    ACTIONS(4101), 1,
      anon_sym___finally,
    STATE(86), 2,
      sym_seh_except_clause,
      sym_seh_finally_clause,
  [57471] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4103), 1,
      anon_sym_SEMI,
    STATE(1530), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57485] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3635), 1,
      anon_sym_COMMA,
    STATE(1552), 1,
      aux_sym__type_definition_declarators_repeat1,
    ACTIONS(4105), 2,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [57499] = 4,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4107), 1,
      anon_sym_SQUOTE,
    STATE(1560), 1,
      aux_sym_char_literal_repeat1,
    ACTIONS(3973), 2,
      aux_sym_char_literal_token1,
      sym_escape_sequence,
  [57513] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4109), 1,
      anon_sym___except,
    ACTIONS(4111), 1,
      anon_sym___finally,
    STATE(242), 2,
      sym_seh_except_clause,
      sym_seh_finally_clause,
  [57527] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(3953), 1,
      aux_sym_preproc_if_token2,
    STATE(1554), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
  [57543] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4113), 1,
      anon_sym_SEMI,
    STATE(1511), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57557] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4010), 1,
      anon_sym_LPAREN,
    ACTIONS(4115), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4117), 1,
      sym_preproc_arg,
    STATE(1762), 1,
      sym_preproc_params,
  [57573] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4010), 1,
      anon_sym_LPAREN,
    ACTIONS(4119), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4121), 1,
      sym_preproc_arg,
    STATE(1718), 1,
      sym_preproc_params,
  [57589] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4123), 1,
      anon_sym_SEMI,
    STATE(1559), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57603] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4125), 1,
      anon_sym_SEMI,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57617] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4127), 1,
      anon_sym_COMMA,
    STATE(1552), 1,
      aux_sym__type_definition_declarators_repeat1,
    ACTIONS(4130), 2,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [57631] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(4132), 1,
      aux_sym_preproc_if_token2,
    STATE(1410), 2,
      sym_enumerator,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
  [57645] = 5,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3514), 1,
      sym_identifier,
    ACTIONS(4134), 1,
      aux_sym_preproc_if_token2,
    STATE(1409), 1,
      aux_sym_preproc_if_in_enumerator_list_repeat1,
    STATE(1963), 1,
      sym_enumerator,
  [57661] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4136), 1,
      anon_sym_SEMI,
    STATE(1551), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57675] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4016), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4018), 1,
      sym_escape_sequence,
    ACTIONS(4138), 1,
      anon_sym_DQUOTE,
    STATE(1557), 1,
      aux_sym_string_literal_repeat1,
  [57691] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4140), 1,
      anon_sym_DQUOTE,
    ACTIONS(4142), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4145), 1,
      sym_escape_sequence,
    STATE(1557), 1,
      aux_sym_string_literal_repeat1,
  [57707] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4148), 1,
      anon_sym_DQUOTE,
    ACTIONS(4150), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4152), 1,
      sym_escape_sequence,
    STATE(1556), 1,
      aux_sym_string_literal_repeat1,
  [57723] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4154), 1,
      anon_sym_SEMI,
    STATE(1433), 2,
      sym_attribute_specifier,
      aux_sym_type_definition_repeat1,
  [57737] = 4,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4156), 1,
      anon_sym_SQUOTE,
    STATE(1560), 1,
      aux_sym_char_literal_repeat1,
    ACTIONS(4158), 2,
      aux_sym_char_literal_token1,
      sym_escape_sequence,
  [57751] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4161), 1,
      anon_sym___except,
    ACTIONS(4163), 1,
      anon_sym___finally,
    STATE(268), 2,
      sym_seh_except_clause,
      sym_seh_finally_clause,
  [57765] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3457), 1,
      sym_identifier,
    ACTIONS(4165), 1,
      aux_sym_preproc_if_token2,
    STATE(1553), 2,
      sym_enumerator,
      aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1,
  [57779] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    STATE(1725), 1,
      sym_argument_list,
    ACTIONS(4167), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK_RBRACK,
  [57793] = 5,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4016), 1,
      aux_sym_string_literal_token1,
    ACTIONS(4018), 1,
      sym_escape_sequence,
    ACTIONS(4169), 1,
      anon_sym_DQUOTE,
    STATE(1557), 1,
      aux_sym_string_literal_repeat1,
  [57809] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4171), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [57822] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4173), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [57835] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(4175), 1,
      anon_sym_RPAREN,
    STATE(1666), 1,
      aux_sym_argument_list_repeat1,
  [57848] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4177), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [57861] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4179), 1,
      anon_sym_COMMA,
    ACTIONS(4181), 1,
      anon_sym_RPAREN,
    STATE(1633), 1,
      aux_sym_parameter_list_repeat1,
  [57874] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4183), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [57887] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4185), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [57900] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4187), 1,
      anon_sym_SEMI,
    STATE(1570), 1,
      aux_sym_declaration_repeat1,
  [57913] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4189), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [57926] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4191), 1,
      anon_sym_COMMA,
    ACTIONS(4194), 1,
      anon_sym_RPAREN,
    STATE(1574), 1,
      aux_sym__old_style_parameter_list_repeat1,
  [57939] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4196), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [57952] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4198), 1,
      anon_sym_SEMI,
    STATE(1661), 1,
      aux_sym_declaration_repeat1,
  [57965] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4200), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [57978] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4202), 1,
      anon_sym_SEMI,
    STATE(1590), 1,
      aux_sym_declaration_repeat1,
  [57991] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4204), 1,
      anon_sym_COMMA,
    ACTIONS(4207), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58004] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2725), 1,
      anon_sym_LBRACE,
    ACTIONS(4209), 1,
      sym_identifier,
    STATE(922), 1,
      sym_enumerator_list,
  [58017] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4211), 1,
      anon_sym_COMMA,
    ACTIONS(4213), 1,
      anon_sym_RBRACK_RBRACK,
    STATE(1593), 1,
      aux_sym_attribute_declaration_repeat1,
  [58030] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4215), 1,
      anon_sym_RPAREN,
    ACTIONS(4217), 1,
      anon_sym_COLON,
    STATE(1645), 1,
      sym_gnu_asm_output_operand_list,
  [58043] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(2986), 1,
      anon_sym_RPAREN,
    STATE(1666), 1,
      aux_sym_argument_list_repeat1,
  [58056] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4219), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58069] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4221), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58082] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2996), 1,
      anon_sym_COMMA,
    ACTIONS(2998), 1,
      anon_sym_RBRACE,
    STATE(1616), 1,
      aux_sym_initializer_list_repeat1,
  [58095] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4223), 1,
      anon_sym_SEMI,
    STATE(1573), 1,
      aux_sym_declaration_repeat1,
  [58108] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    ACTIONS(4225), 1,
      sym_identifier,
    STATE(783), 1,
      sym_field_declaration_list,
  [58121] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4227), 1,
      anon_sym_COMMA,
    ACTIONS(4229), 1,
      anon_sym_RPAREN,
    STATE(1621), 1,
      aux_sym_preproc_params_repeat1,
  [58134] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4231), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58147] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4233), 3,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_DOT,
  [58156] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4235), 1,
      anon_sym_SEMI,
    STATE(1646), 1,
      aux_sym_declaration_repeat1,
  [58169] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4237), 1,
      anon_sym_COMMA,
    ACTIONS(4240), 1,
      anon_sym_RBRACK_RBRACK,
    STATE(1593), 1,
      aux_sym_attribute_declaration_repeat1,
  [58182] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4242), 1,
      anon_sym_SEMI,
    STATE(1584), 1,
      aux_sym_declaration_repeat1,
  [58195] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3179), 1,
      anon_sym_COMMA,
    ACTIONS(4244), 1,
      anon_sym_RPAREN,
    STATE(1663), 1,
      aux_sym_preproc_argument_list_repeat1,
  [58208] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4246), 1,
      anon_sym_SEMI,
    STATE(1606), 1,
      aux_sym_declaration_repeat1,
  [58221] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3088), 1,
      anon_sym_RBRACE,
    ACTIONS(4248), 1,
      anon_sym_COMMA,
    STATE(1597), 1,
      aux_sym_initializer_list_repeat1,
  [58234] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4251), 1,
      anon_sym_RPAREN,
    ACTIONS(4253), 1,
      anon_sym_COLON,
    STATE(1858), 1,
      sym_gnu_asm_goto_list,
  [58247] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    ACTIONS(4255), 1,
      sym_identifier,
    STATE(794), 1,
      sym_field_declaration_list,
  [58260] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4257), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58273] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4259), 1,
      anon_sym_SEMI,
    STATE(1619), 1,
      aux_sym_declaration_repeat1,
  [58286] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4217), 1,
      anon_sym_COLON,
    ACTIONS(4261), 1,
      anon_sym_RPAREN,
    STATE(1630), 1,
      sym_gnu_asm_output_operand_list,
  [58299] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2353), 1,
      anon_sym_LBRACE,
    ACTIONS(4263), 1,
      sym_identifier,
    STATE(806), 1,
      sym_field_declaration_list,
  [58312] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2725), 1,
      anon_sym_LBRACE,
    ACTIONS(4265), 1,
      sym_identifier,
    STATE(1120), 1,
      sym_enumerator_list,
  [58325] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4267), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58338] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4269), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58351] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4271), 1,
      anon_sym_SEMI,
    STATE(1568), 1,
      aux_sym_declaration_repeat1,
  [58364] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4273), 1,
      anon_sym_SEMI,
    STATE(1620), 1,
      aux_sym_declaration_repeat1,
  [58377] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4275), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58390] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4277), 1,
      anon_sym_EQ,
    ACTIONS(3747), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [58401] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    STATE(1544), 1,
      aux_sym_char_literal_repeat1,
    ACTIONS(4279), 2,
      aux_sym_char_literal_token1,
      sym_escape_sequence,
  [58412] = 4,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4281), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4283), 1,
      anon_sym_LPAREN2,
    STATE(1878), 1,
      sym_preproc_argument_list,
  [58425] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4285), 1,
      anon_sym_SEMI,
    STATE(1986), 1,
      sym_attribute_specifier,
  [58438] = 4,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4283), 1,
      anon_sym_LPAREN2,
    ACTIONS(4287), 1,
      aux_sym_preproc_include_token2,
    STATE(1878), 1,
      sym_preproc_argument_list,
  [58451] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4211), 1,
      anon_sym_COMMA,
    ACTIONS(4289), 1,
      anon_sym_RBRACK_RBRACK,
    STATE(1593), 1,
      aux_sym_attribute_declaration_repeat1,
  [58464] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1815), 1,
      anon_sym_RBRACE,
    ACTIONS(4291), 1,
      anon_sym_COMMA,
    STATE(1597), 1,
      aux_sym_initializer_list_repeat1,
  [58477] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4211), 1,
      anon_sym_COMMA,
    ACTIONS(4293), 1,
      anon_sym_RBRACK_RBRACK,
    STATE(1643), 1,
      aux_sym_attribute_declaration_repeat1,
  [58490] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4295), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58503] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4297), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58516] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4299), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58529] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4227), 1,
      anon_sym_COMMA,
    ACTIONS(4301), 1,
      anon_sym_RPAREN,
    STATE(1660), 1,
      aux_sym_preproc_params_repeat1,
  [58542] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4303), 1,
      anon_sym_SEMI,
    STATE(1627), 1,
      aux_sym_declaration_repeat1,
  [58555] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4305), 3,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_DOT,
  [58564] = 4,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4283), 1,
      anon_sym_LPAREN2,
    ACTIONS(4307), 1,
      aux_sym_preproc_include_token2,
    STATE(1878), 1,
      sym_preproc_argument_list,
  [58577] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4309), 1,
      anon_sym_SEMI,
    STATE(1565), 1,
      aux_sym_declaration_repeat1,
  [58590] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4311), 1,
      anon_sym_SEMI,
    STATE(1644), 1,
      aux_sym_declaration_repeat1,
  [58603] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4313), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58616] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4315), 1,
      anon_sym_SEMI,
    STATE(1618), 1,
      aux_sym_declaration_repeat1,
  [58629] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4317), 1,
      anon_sym_RPAREN,
    ACTIONS(4319), 1,
      anon_sym_COLON,
    STATE(1674), 1,
      sym_gnu_asm_clobber_list,
  [58642] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4321), 1,
      anon_sym_RPAREN,
    ACTIONS(4323), 1,
      anon_sym_COLON,
    STATE(1671), 1,
      sym_gnu_asm_input_operand_list,
  [58655] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4325), 1,
      anon_sym_SEMI,
    STATE(1655), 1,
      aux_sym_declaration_repeat1,
  [58668] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4327), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [58677] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4329), 1,
      anon_sym_COMMA,
    ACTIONS(4332), 1,
      anon_sym_RPAREN,
    STATE(1633), 1,
      aux_sym_parameter_list_repeat1,
  [58690] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4334), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [58699] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4336), 1,
      anon_sym_COMMA,
    ACTIONS(4339), 1,
      anon_sym_RPAREN,
    STATE(1635), 1,
      aux_sym_gnu_asm_goto_list_repeat1,
  [58712] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4341), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58725] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4343), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [58734] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4347), 1,
      anon_sym_RPAREN,
    ACTIONS(4345), 2,
      anon_sym_DOT_DOT_DOT,
      sym_identifier,
  [58745] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4349), 1,
      anon_sym_SEMI,
    STATE(1667), 1,
      aux_sym_declaration_repeat1,
  [58758] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(2994), 1,
      anon_sym_RPAREN,
    STATE(1583), 1,
      aux_sym_argument_list_repeat1,
  [58771] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4351), 3,
      anon_sym_COMMA,
      anon_sym_SEMI,
      anon_sym___attribute__,
  [58780] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4211), 1,
      anon_sym_COMMA,
    ACTIONS(4353), 1,
      anon_sym_RBRACK_RBRACK,
    STATE(1581), 1,
      aux_sym_attribute_declaration_repeat1,
  [58793] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4211), 1,
      anon_sym_COMMA,
    ACTIONS(4355), 1,
      anon_sym_RBRACK_RBRACK,
    STATE(1593), 1,
      aux_sym_attribute_declaration_repeat1,
  [58806] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4357), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58819] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4323), 1,
      anon_sym_COLON,
    ACTIONS(4359), 1,
      anon_sym_RPAREN,
    STATE(1629), 1,
      sym_gnu_asm_input_operand_list,
  [58832] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4361), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58845] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4363), 1,
      anon_sym_COMMA,
    ACTIONS(4365), 1,
      anon_sym_RPAREN,
    STATE(1635), 1,
      aux_sym_gnu_asm_goto_list_repeat1,
  [58858] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(1745), 1,
      anon_sym_DOT_DOT_DOT,
    ACTIONS(4367), 1,
      sym_identifier,
    STATE(1713), 1,
      sym_variadic_parameter,
  [58871] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4369), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [58880] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    STATE(1522), 1,
      aux_sym_char_literal_repeat1,
    ACTIONS(4371), 2,
      aux_sym_char_literal_token1,
      sym_escape_sequence,
  [58891] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4211), 1,
      anon_sym_COMMA,
    ACTIONS(4373), 1,
      anon_sym_RBRACK_RBRACK,
    STATE(1615), 1,
      aux_sym_attribute_declaration_repeat1,
  [58904] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4375), 1,
      anon_sym_SEMI,
    STATE(1683), 1,
      aux_sym_declaration_repeat1,
  [58917] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4377), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58930] = 4,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4283), 1,
      anon_sym_LPAREN2,
    ACTIONS(4379), 1,
      aux_sym_preproc_include_token2,
    STATE(1878), 1,
      sym_preproc_argument_list,
  [58943] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4381), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [58956] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4383), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [58965] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4385), 1,
      anon_sym_COMMA,
    ACTIONS(4388), 1,
      anon_sym_RPAREN,
    STATE(1657), 1,
      aux_sym_generic_expression_repeat1,
  [58978] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4390), 1,
      anon_sym_COMMA,
    ACTIONS(4392), 1,
      anon_sym_RPAREN,
    STATE(1574), 1,
      aux_sym__old_style_parameter_list_repeat1,
  [58991] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(2992), 1,
      anon_sym_RPAREN,
    STATE(1679), 1,
      aux_sym_argument_list_repeat1,
  [59004] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4394), 1,
      anon_sym_COMMA,
    ACTIONS(4397), 1,
      anon_sym_RPAREN,
    STATE(1660), 1,
      aux_sym_preproc_params_repeat1,
  [59017] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4399), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [59030] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4401), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [59043] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3223), 1,
      anon_sym_RPAREN,
    ACTIONS(4403), 1,
      anon_sym_COMMA,
    STATE(1663), 1,
      aux_sym_preproc_argument_list_repeat1,
  [59056] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4406), 1,
      anon_sym_SEMI,
    STATE(1855), 1,
      sym_attribute_specifier,
  [59069] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4408), 3,
      anon_sym_LBRACK,
      anon_sym_EQ,
      anon_sym_DOT,
  [59078] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3056), 1,
      anon_sym_RPAREN,
    ACTIONS(4410), 1,
      anon_sym_COMMA,
    STATE(1666), 1,
      aux_sym_argument_list_repeat1,
  [59091] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4413), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [59104] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4415), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [59117] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4363), 1,
      anon_sym_COMMA,
    ACTIONS(4417), 1,
      anon_sym_RPAREN,
    STATE(1647), 1,
      aux_sym_gnu_asm_goto_list_repeat1,
  [59130] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4179), 1,
      anon_sym_COMMA,
    ACTIONS(4419), 1,
      anon_sym_RPAREN,
    STATE(1569), 1,
      aux_sym_parameter_list_repeat1,
  [59143] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4319), 1,
      anon_sym_COLON,
    ACTIONS(4421), 1,
      anon_sym_RPAREN,
    STATE(1598), 1,
      sym_gnu_asm_clobber_list,
  [59156] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(4423), 1,
      anon_sym_RPAREN,
    STATE(1666), 1,
      aux_sym_argument_list_repeat1,
  [59169] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3179), 1,
      anon_sym_COMMA,
    ACTIONS(4425), 1,
      anon_sym_RPAREN,
    STATE(1663), 1,
      aux_sym_preproc_argument_list_repeat1,
  [59182] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4253), 1,
      anon_sym_COLON,
    ACTIONS(4427), 1,
      anon_sym_RPAREN,
    STATE(1788), 1,
      sym_gnu_asm_goto_list,
  [59195] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4429), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [59204] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4431), 3,
      anon_sym_COMMA,
      anon_sym_RPAREN,
      anon_sym_COLON,
  [59213] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3000), 1,
      anon_sym_COMMA,
    ACTIONS(4433), 1,
      anon_sym_RPAREN,
    STATE(1657), 1,
      aux_sym_generic_expression_repeat1,
  [59226] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3442), 1,
      anon_sym___attribute__,
    ACTIONS(4435), 1,
      anon_sym_SEMI,
    STATE(1834), 1,
      sym_attribute_specifier,
  [59239] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2984), 1,
      anon_sym_COMMA,
    ACTIONS(2990), 1,
      anon_sym_RPAREN,
    STATE(1666), 1,
      aux_sym_argument_list_repeat1,
  [59252] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    STATE(1502), 1,
      aux_sym_char_literal_repeat1,
    ACTIONS(4437), 2,
      aux_sym_char_literal_token1,
      sym_escape_sequence,
  [59263] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4439), 1,
      anon_sym_SEMI,
    STATE(1668), 1,
      aux_sym_declaration_repeat1,
  [59276] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4441), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [59289] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3844), 1,
      anon_sym_COMMA,
    ACTIONS(4443), 1,
      anon_sym_SEMI,
    STATE(1579), 1,
      aux_sym_declaration_repeat1,
  [59302] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3179), 1,
      anon_sym_COMMA,
    ACTIONS(4445), 1,
      anon_sym_RPAREN,
    STATE(1663), 1,
      aux_sym_preproc_argument_list_repeat1,
  [59315] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2982), 3,
      anon_sym_RPAREN,
      anon_sym_SEMI,
      anon_sym_COLON,
  [59324] = 4,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2725), 1,
      anon_sym_LBRACE,
    ACTIONS(4447), 1,
      sym_identifier,
    STATE(1120), 1,
      sym_enumerator_list,
  [59337] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1729), 1,
      sym_parenthesized_expression,
  [59347] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(1540), 1,
      sym_compound_statement,
  [59357] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(397), 1,
      sym_parenthesized_expression,
  [59367] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(362), 1,
      sym_parenthesized_expression,
  [59377] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4453), 1,
      sym_identifier,
    STATE(1642), 1,
      sym_attribute,
  [59387] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(389), 1,
      sym_parenthesized_expression,
  [59397] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3050), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [59405] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1719), 1,
      sym_parenthesized_expression,
  [59415] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(361), 1,
      sym_parenthesized_expression,
  [59425] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(304), 1,
      sym_compound_statement,
  [59435] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(276), 1,
      sym_compound_statement,
  [59445] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3955), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [59453] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4455), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4457), 1,
      sym_preproc_arg,
  [59463] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    STATE(1915), 1,
      sym_argument_list,
  [59473] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(223), 1,
      sym_compound_statement,
  [59483] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4459), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4461), 1,
      sym_preproc_arg,
  [59493] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(1561), 1,
      sym_compound_statement,
  [59503] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3056), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [59511] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2249), 1,
      anon_sym_LPAREN2,
    STATE(1979), 1,
      sym_argument_list,
  [59521] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1720), 1,
      sym_parenthesized_expression,
  [59531] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4397), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [59539] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4463), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4465), 1,
      sym_preproc_arg,
  [59549] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(391), 1,
      sym_parenthesized_expression,
  [59559] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4467), 1,
      sym_identifier,
    ACTIONS(4469), 1,
      anon_sym_LPAREN2,
  [59569] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4471), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4473), 1,
      sym_preproc_arg,
  [59579] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3965), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [59587] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4194), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [59595] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4475), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [59603] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4332), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [59611] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1998), 1,
      sym_parenthesized_expression,
  [59621] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1701), 1,
      sym_parenthesized_expression,
  [59631] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4477), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4479), 1,
      sym_preproc_arg,
  [59641] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    STATE(246), 1,
      sym_compound_statement,
  [59651] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(281), 1,
      sym_compound_statement,
  [59661] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    STATE(223), 1,
      sym_compound_statement,
  [59671] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4481), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4483), 1,
      sym_preproc_arg,
  [59681] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(1545), 1,
      sym_compound_statement,
  [59691] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4485), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [59699] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4487), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK_RBRACK,
  [59707] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4489), 1,
      anon_sym_COMMA,
    ACTIONS(4491), 1,
      anon_sym_RBRACE,
  [59717] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4493), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4495), 1,
      sym_preproc_arg,
  [59727] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4497), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK_RBRACK,
  [59735] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    STATE(101), 1,
      sym_compound_statement,
  [59745] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4499), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4501), 1,
      sym_preproc_arg,
  [59755] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4453), 1,
      sym_identifier,
    STATE(1758), 1,
      sym_attribute,
  [59765] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    STATE(238), 1,
      sym_compound_statement,
  [59775] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4453), 1,
      sym_identifier,
    STATE(1617), 1,
      sym_attribute,
  [59785] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(311), 1,
      sym_parenthesized_expression,
  [59795] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(312), 1,
      sym_parenthesized_expression,
  [59805] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4503), 1,
      sym_identifier,
    ACTIONS(4505), 1,
      anon_sym_LPAREN2,
  [59815] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(1541), 1,
      sym_compound_statement,
  [59825] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    STATE(170), 1,
      sym_compound_statement,
  [59835] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3520), 1,
      anon_sym_RBRACE,
    ACTIONS(4489), 1,
      anon_sym_COMMA,
  [59845] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(351), 1,
      sym_parenthesized_expression,
  [59855] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(352), 1,
      sym_parenthesized_expression,
  [59865] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1721), 1,
      sym_parenthesized_expression,
  [59875] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4507), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4509), 1,
      sym_preproc_arg,
  [59885] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(423), 1,
      anon_sym_LBRACE,
    STATE(208), 1,
      sym_compound_statement,
  [59895] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1847), 1,
      sym_parenthesized_expression,
  [59905] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4511), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4513), 1,
      sym_preproc_arg,
  [59915] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4515), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4517), 1,
      sym_preproc_arg,
  [59925] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3066), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [59933] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4519), 2,
      anon_sym_DOT_DOT_DOT,
      sym_identifier,
  [59941] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(171), 1,
      sym_compound_statement,
  [59951] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3088), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [59959] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4521), 1,
      sym_identifier,
    ACTIONS(4523), 1,
      anon_sym_RPAREN,
  [59969] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3104), 2,
      anon_sym_COMMA,
      anon_sym_RBRACE,
  [59977] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4451), 1,
      anon_sym_LPAREN2,
    STATE(396), 1,
      sym_parenthesized_expression,
  [59987] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4525), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4527), 1,
      sym_preproc_arg,
  [59997] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1696), 1,
      sym_parenthesized_expression,
  [60007] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4529), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4531), 1,
      sym_preproc_arg,
  [60017] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4240), 2,
      anon_sym_COMMA,
      anon_sym_RBRACK_RBRACK,
  [60025] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1923), 1,
      sym_parenthesized_expression,
  [60035] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    STATE(88), 1,
      sym_compound_statement,
  [60045] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4533), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4535), 1,
      sym_preproc_arg,
  [60055] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4537), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4539), 1,
      sym_preproc_arg,
  [60065] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4541), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4543), 1,
      sym_preproc_arg,
  [60075] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1744), 1,
      sym_parenthesized_expression,
  [60085] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(41), 1,
      anon_sym_LBRACE,
    STATE(1507), 1,
      sym_compound_statement,
  [60095] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4545), 2,
      anon_sym_COMMA,
      anon_sym_SEMI,
  [60103] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4453), 1,
      sym_identifier,
    STATE(1651), 1,
      sym_attribute,
  [60113] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1781), 1,
      sym_parenthesized_expression,
  [60123] = 3,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4547), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(4549), 1,
      sym_preproc_arg,
  [60133] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(129), 1,
      anon_sym_LBRACE,
    STATE(76), 1,
      sym_compound_statement,
  [60143] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1760), 1,
      sym_parenthesized_expression,
  [60153] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4449), 1,
      anon_sym_LPAREN2,
    STATE(1732), 1,
      sym_parenthesized_expression,
  [60163] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4551), 2,
      anon_sym_COMMA,
      anon_sym_RPAREN,
  [60171] = 3,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(371), 1,
      anon_sym_LBRACE,
    STATE(171), 1,
      sym_compound_statement,
  [60181] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4554), 1,
      anon_sym_SEMI,
  [60188] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4556), 1,
      aux_sym_preproc_if_token2,
  [60195] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4558), 1,
      anon_sym_RBRACE,
  [60202] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4560), 1,
      anon_sym_LPAREN2,
  [60209] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4562), 1,
      aux_sym_preproc_if_token2,
  [60216] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4564), 1,
      aux_sym_preproc_if_token2,
  [60223] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4566), 1,
      anon_sym_SEMI,
  [60230] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4568), 1,
      anon_sym_RPAREN,
  [60237] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4570), 1,
      aux_sym_preproc_include_token2,
  [60244] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4572), 1,
      aux_sym_preproc_if_token2,
  [60251] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4574), 1,
      aux_sym_preproc_if_token2,
  [60258] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3100), 1,
      anon_sym_SEMI,
  [60265] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4576), 1,
      aux_sym_preproc_if_token2,
  [60272] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4578), 1,
      anon_sym_RPAREN,
  [60279] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4580), 1,
      aux_sym_preproc_if_token2,
  [60286] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3074), 1,
      anon_sym_RPAREN,
  [60293] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4582), 1,
      aux_sym_preproc_if_token2,
  [60300] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4584), 1,
      sym_identifier,
  [60307] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4586), 1,
      anon_sym_COLON,
  [60314] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4588), 1,
      aux_sym_preproc_if_token2,
  [60321] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4590), 1,
      aux_sym_preproc_include_token2,
  [60328] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4592), 1,
      aux_sym_preproc_include_token2,
  [60335] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3054), 1,
      anon_sym_RPAREN,
  [60342] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4594), 1,
      aux_sym_preproc_if_token2,
  [60349] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4596), 1,
      anon_sym_SEMI,
  [60356] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4598), 1,
      aux_sym_preproc_include_token2,
  [60363] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3040), 1,
      anon_sym_SEMI,
  [60370] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4600), 1,
      anon_sym_RPAREN,
  [60377] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4602), 1,
      anon_sym_STAR,
  [60384] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3036), 1,
      anon_sym_RPAREN,
  [60391] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3076), 1,
      anon_sym_SEMI,
  [60398] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4604), 1,
      anon_sym_SEMI,
  [60405] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4606), 1,
      aux_sym_preproc_include_token2,
  [60412] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3004), 1,
      anon_sym_SEMI,
  [60419] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4608), 1,
      anon_sym_RPAREN,
  [60426] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4610), 1,
      aux_sym_preproc_include_token2,
  [60433] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4612), 1,
      anon_sym_LPAREN2,
  [60440] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4287), 1,
      aux_sym_preproc_include_token2,
  [60447] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4614), 1,
      anon_sym_LPAREN2,
  [60454] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4616), 1,
      anon_sym_RPAREN,
  [60461] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4618), 1,
      anon_sym_COLON,
  [60468] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4620), 1,
      aux_sym_preproc_include_token2,
  [60475] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4622), 1,
      anon_sym_STAR,
  [60482] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4624), 1,
      sym_identifier,
  [60489] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4626), 1,
      aux_sym_preproc_include_token2,
  [60496] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4628), 1,
      anon_sym_SEMI,
  [60503] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4630), 1,
      anon_sym_RBRACK,
  [60510] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4632), 1,
      anon_sym_SEMI,
  [60517] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4634), 1,
      anon_sym_LPAREN2,
  [60524] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4636), 1,
      anon_sym_COLON,
  [60531] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4638), 1,
      aux_sym_preproc_include_token2,
  [60538] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4640), 1,
      anon_sym_LPAREN2,
  [60545] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3070), 1,
      anon_sym_RPAREN,
  [60552] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4642), 1,
      aux_sym_preproc_if_token2,
  [60559] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4644), 1,
      aux_sym_preproc_if_token2,
  [60566] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4646), 1,
      aux_sym_preproc_if_token2,
  [60573] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4648), 1,
      aux_sym_preproc_if_token2,
  [60580] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4650), 1,
      aux_sym_preproc_include_token2,
  [60587] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4652), 1,
      aux_sym_preproc_include_token2,
  [60594] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4654), 1,
      anon_sym_SEMI,
  [60601] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4656), 1,
      aux_sym_preproc_include_token2,
  [60608] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4658), 1,
      ts_builtin_sym_end,
  [60615] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4660), 1,
      sym_identifier,
  [60622] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4662), 1,
      sym_identifier,
  [60629] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4664), 1,
      anon_sym_COLON,
  [60636] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4666), 1,
      aux_sym_preproc_if_token2,
  [60643] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4668), 1,
      aux_sym_preproc_if_token2,
  [60650] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4670), 1,
      anon_sym_COLON,
  [60657] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4672), 1,
      aux_sym_preproc_if_token2,
  [60664] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3072), 1,
      anon_sym_RPAREN,
  [60671] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4674), 1,
      anon_sym_SEMI,
  [60678] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4676), 1,
      sym_identifier,
  [60685] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4678), 1,
      anon_sym_SEMI,
  [60692] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4680), 1,
      anon_sym_SEMI,
  [60699] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4682), 1,
      anon_sym_LPAREN2,
  [60706] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4684), 1,
      anon_sym_SEMI,
  [60713] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4686), 1,
      anon_sym_SEMI,
  [60720] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4688), 1,
      sym_identifier,
  [60727] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4690), 1,
      anon_sym_RPAREN,
  [60734] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4692), 1,
      aux_sym_preproc_if_token2,
  [60741] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4694), 1,
      anon_sym_SEMI,
  [60748] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4696), 1,
      aux_sym_preproc_if_token2,
  [60755] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4698), 1,
      aux_sym_preproc_include_token2,
  [60762] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4700), 1,
      anon_sym_RPAREN,
  [60769] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4702), 1,
      anon_sym_RBRACE,
  [60776] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3052), 1,
      anon_sym_COLON,
  [60783] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3098), 1,
      anon_sym_SEMI,
  [60790] = 2,
    ACTIONS(2950), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(3211), 1,
      sym_comment,
  [60797] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3068), 1,
      anon_sym_SEMI,
  [60804] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4704), 1,
      aux_sym_preproc_if_token2,
  [60811] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4706), 1,
      aux_sym_preproc_if_token2,
  [60818] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4708), 1,
      aux_sym_preproc_if_token2,
  [60825] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4710), 1,
      sym_identifier,
  [60832] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4712), 1,
      anon_sym_COLON,
  [60839] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4714), 1,
      aux_sym_preproc_if_token2,
  [60846] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4716), 1,
      sym_identifier,
  [60853] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4718), 1,
      anon_sym_LPAREN2,
  [60860] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4720), 1,
      aux_sym_preproc_if_token2,
  [60867] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4722), 1,
      aux_sym_preproc_if_token2,
  [60874] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4724), 1,
      sym_identifier,
  [60881] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4379), 1,
      aux_sym_preproc_include_token2,
  [60888] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4726), 1,
      aux_sym_preproc_include_token2,
  [60895] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4728), 1,
      sym_identifier,
  [60902] = 2,
    ACTIONS(2960), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(3211), 1,
      sym_comment,
  [60909] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4730), 1,
      sym_identifier,
  [60916] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4732), 1,
      anon_sym_RPAREN,
  [60923] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4734), 1,
      anon_sym_RPAREN,
  [60930] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4736), 1,
      anon_sym_RPAREN,
  [60937] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3102), 1,
      anon_sym_RPAREN,
  [60944] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4738), 1,
      sym_identifier,
  [60951] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4740), 1,
      anon_sym_RPAREN,
  [60958] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4742), 1,
      anon_sym_RPAREN,
  [60965] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4744), 1,
      anon_sym_RPAREN,
  [60972] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4746), 1,
      anon_sym_RPAREN,
  [60979] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4489), 1,
      anon_sym_COMMA,
  [60986] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4748), 1,
      sym_identifier,
  [60993] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4750), 1,
      aux_sym_preproc_include_token2,
  [61000] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3086), 1,
      anon_sym_COLON,
  [61007] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4752), 1,
      anon_sym_RPAREN,
  [61014] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4754), 1,
      anon_sym_while,
  [61021] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4756), 1,
      sym_identifier,
  [61028] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4758), 1,
      anon_sym_SEMI,
  [61035] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4760), 1,
      aux_sym_preproc_include_token2,
  [61042] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3096), 1,
      anon_sym_SEMI,
  [61049] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4762), 1,
      aux_sym_preproc_if_token2,
  [61056] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4764), 1,
      aux_sym_preproc_if_token2,
  [61063] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4766), 1,
      sym_identifier,
  [61070] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4768), 1,
      sym_identifier,
  [61077] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4770), 1,
      aux_sym_preproc_if_token2,
  [61084] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4772), 1,
      anon_sym_SEMI,
  [61091] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4491), 1,
      anon_sym_RBRACE,
  [61098] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3078), 1,
      anon_sym_RPAREN,
  [61105] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4774), 1,
      aux_sym_preproc_if_token2,
  [61112] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4307), 1,
      aux_sym_preproc_include_token2,
  [61119] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4776), 1,
      anon_sym_SEMI,
  [61126] = 2,
    ACTIONS(2102), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(3211), 1,
      sym_comment,
  [61133] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3046), 1,
      anon_sym_SEMI,
  [61140] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4778), 1,
      aux_sym_preproc_if_token2,
  [61147] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4780), 1,
      anon_sym_STAR,
  [61154] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3090), 1,
      anon_sym_COLON,
  [61161] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4782), 1,
      anon_sym_RPAREN,
  [61168] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4784), 1,
      sym_identifier,
  [61175] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4786), 1,
      sym_identifier,
  [61182] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4788), 1,
      sym_identifier,
  [61189] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4790), 1,
      aux_sym_preproc_if_token2,
  [61196] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3082), 1,
      anon_sym_SEMI,
  [61203] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4792), 1,
      anon_sym_SEMI,
  [61210] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3106), 1,
      anon_sym_SEMI,
  [61217] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4794), 1,
      anon_sym_SEMI,
  [61224] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4796), 1,
      anon_sym_SEMI,
  [61231] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4798), 1,
      anon_sym_RPAREN,
  [61238] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4800), 1,
      sym_identifier,
  [61245] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4802), 1,
      aux_sym_preproc_include_token2,
  [61252] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3520), 1,
      anon_sym_RBRACE,
  [61259] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4804), 1,
      aux_sym_preproc_if_token2,
  [61266] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4806), 1,
      sym_identifier,
  [61273] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4281), 1,
      aux_sym_preproc_include_token2,
  [61280] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4808), 1,
      aux_sym_preproc_if_token2,
  [61287] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4810), 1,
      anon_sym_RBRACE,
  [61294] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4812), 1,
      anon_sym_SEMI,
  [61301] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4814), 1,
      anon_sym_SEMI,
  [61308] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4816), 1,
      sym_primitive_type,
  [61315] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4818), 1,
      anon_sym_COLON,
  [61322] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4820), 1,
      sym_identifier,
  [61329] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4822), 1,
      aux_sym_preproc_if_token2,
  [61336] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4824), 1,
      anon_sym_RPAREN,
  [61343] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4826), 1,
      sym_identifier,
  [61350] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4828), 1,
      aux_sym_preproc_include_token2,
  [61357] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4830), 1,
      anon_sym_RBRACE,
  [61364] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4832), 1,
      aux_sym_preproc_if_token2,
  [61371] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3080), 1,
      anon_sym_RPAREN,
  [61378] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4834), 1,
      anon_sym_RPAREN,
  [61385] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4836), 1,
      aux_sym_preproc_if_token2,
  [61392] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4838), 1,
      sym_identifier,
  [61399] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4840), 1,
      sym_identifier,
  [61406] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4842), 1,
      anon_sym_LPAREN2,
  [61413] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4844), 1,
      anon_sym_RPAREN,
  [61420] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4846), 1,
      aux_sym_preproc_include_token2,
  [61427] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4848), 1,
      aux_sym_preproc_if_token2,
  [61434] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4850), 1,
      sym_identifier,
  [61441] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3042), 1,
      anon_sym_SEMI,
  [61448] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4852), 1,
      anon_sym_RPAREN,
  [61455] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4854), 1,
      anon_sym_RPAREN,
  [61462] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4856), 1,
      anon_sym_RBRACE,
  [61469] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4858), 1,
      aux_sym_preproc_if_token2,
  [61476] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4860), 1,
      anon_sym_COMMA,
  [61483] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4862), 1,
      sym_identifier,
  [61490] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4864), 1,
      aux_sym_preproc_if_token2,
  [61497] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3767), 1,
      anon_sym_COMMA,
  [61504] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4866), 1,
      anon_sym_RBRACE,
  [61511] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4868), 1,
      sym_identifier,
  [61518] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4870), 1,
      aux_sym_preproc_if_token2,
  [61525] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4872), 1,
      sym_primitive_type,
  [61532] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4874), 1,
      anon_sym_RPAREN,
  [61539] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4876), 1,
      sym_identifier,
  [61546] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4878), 1,
      sym_identifier,
  [61553] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4880), 1,
      anon_sym_LPAREN2,
  [61560] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4882), 1,
      anon_sym_RPAREN,
  [61567] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4884), 1,
      sym_identifier,
  [61574] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4886), 1,
      sym_identifier,
  [61581] = 2,
    ACTIONS(2106), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(3211), 1,
      sym_comment,
  [61588] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(2998), 1,
      anon_sym_RBRACE,
  [61595] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4888), 1,
      anon_sym_RPAREN,
  [61602] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4890), 1,
      aux_sym_preproc_if_token2,
  [61609] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4892), 1,
      anon_sym_STAR,
  [61616] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4894), 1,
      anon_sym_LPAREN2,
  [61623] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4896), 1,
      aux_sym_preproc_if_token2,
  [61630] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4898), 1,
      aux_sym_preproc_if_token2,
  [61637] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4900), 1,
      aux_sym_preproc_if_token2,
  [61644] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4902), 1,
      aux_sym_preproc_if_token2,
  [61651] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4904), 1,
      anon_sym_RPAREN,
  [61658] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4906), 1,
      anon_sym_SEMI,
  [61665] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4908), 1,
      aux_sym_preproc_if_token2,
  [61672] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4910), 1,
      aux_sym_preproc_if_token2,
  [61679] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4912), 1,
      anon_sym_while,
  [61686] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4914), 1,
      anon_sym_LPAREN2,
  [61693] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4916), 1,
      sym_identifier,
  [61700] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4918), 1,
      aux_sym_preproc_if_token2,
  [61707] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4920), 1,
      aux_sym_preproc_if_token2,
  [61714] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4922), 1,
      sym_identifier,
  [61721] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4924), 1,
      sym_identifier,
  [61728] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3064), 1,
      anon_sym_COLON,
  [61735] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3048), 1,
      anon_sym_COLON,
  [61742] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4926), 1,
      anon_sym_SEMI,
  [61749] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4928), 1,
      anon_sym_LPAREN2,
  [61756] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4930), 1,
      anon_sym_SEMI,
  [61763] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4932), 1,
      anon_sym_SEMI,
  [61770] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4934), 1,
      anon_sym_while,
  [61777] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3038), 1,
      anon_sym_SEMI,
  [61784] = 2,
    ACTIONS(2978), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(3211), 1,
      sym_comment,
  [61791] = 2,
    ACTIONS(2964), 1,
      aux_sym_preproc_include_token2,
    ACTIONS(3211), 1,
      sym_comment,
  [61798] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4936), 1,
      anon_sym_RBRACK,
  [61805] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4938), 1,
      anon_sym_SEMI,
  [61812] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4940), 1,
      aux_sym_preproc_if_token2,
  [61819] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4942), 1,
      aux_sym_preproc_include_token2,
  [61826] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4944), 1,
      anon_sym_while,
  [61833] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4946), 1,
      aux_sym_preproc_if_token2,
  [61840] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4948), 1,
      anon_sym_RPAREN,
  [61847] = 2,
    ACTIONS(3211), 1,
      sym_comment,
    ACTIONS(4950), 1,
      aux_sym_preproc_include_token2,
  [61854] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4952), 1,
      sym_identifier,
  [61861] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4954), 1,
      anon_sym_LPAREN2,
  [61868] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(3034), 1,
      anon_sym_COLON,
  [61875] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4956), 1,
      anon_sym_RPAREN,
  [61882] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4958), 1,
      anon_sym_LPAREN2,
  [61889] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4960), 1,
      aux_sym_preproc_if_token2,
  [61896] = 2,
    ACTIONS(3), 1,
      sym_comment,
    ACTIONS(4962), 1,
      sym_identifier,
};

static const uint32_t ts_small_parse_table_map[] = {
  [SMALL_STATE(496)] = 0,
  [SMALL_STATE(497)] = 115,
  [SMALL_STATE(498)] = 230,
  [SMALL_STATE(499)] = 342,
  [SMALL_STATE(500)] = 451,
  [SMALL_STATE(501)] = 560,
  [SMALL_STATE(502)] = 667,
  [SMALL_STATE(503)] = 776,
  [SMALL_STATE(504)] = 885,
  [SMALL_STATE(505)] = 994,
  [SMALL_STATE(506)] = 1103,
  [SMALL_STATE(507)] = 1212,
  [SMALL_STATE(508)] = 1321,
  [SMALL_STATE(509)] = 1428,
  [SMALL_STATE(510)] = 1535,
  [SMALL_STATE(511)] = 1644,
  [SMALL_STATE(512)] = 1753,
  [SMALL_STATE(513)] = 1862,
  [SMALL_STATE(514)] = 1971,
  [SMALL_STATE(515)] = 2080,
  [SMALL_STATE(516)] = 2189,
  [SMALL_STATE(517)] = 2298,
  [SMALL_STATE(518)] = 2407,
  [SMALL_STATE(519)] = 2516,
  [SMALL_STATE(520)] = 2625,
  [SMALL_STATE(521)] = 2734,
  [SMALL_STATE(522)] = 2843,
  [SMALL_STATE(523)] = 2952,
  [SMALL_STATE(524)] = 3061,
  [SMALL_STATE(525)] = 3170,
  [SMALL_STATE(526)] = 3279,
  [SMALL_STATE(527)] = 3388,
  [SMALL_STATE(528)] = 3496,
  [SMALL_STATE(529)] = 3604,
  [SMALL_STATE(530)] = 3708,
  [SMALL_STATE(531)] = 3812,
  [SMALL_STATE(532)] = 3916,
  [SMALL_STATE(533)] = 4022,
  [SMALL_STATE(534)] = 4130,
  [SMALL_STATE(535)] = 4238,
  [SMALL_STATE(536)] = 4342,
  [SMALL_STATE(537)] = 4446,
  [SMALL_STATE(538)] = 4550,
  [SMALL_STATE(539)] = 4654,
  [SMALL_STATE(540)] = 4762,
  [SMALL_STATE(541)] = 4866,
  [SMALL_STATE(542)] = 4972,
  [SMALL_STATE(543)] = 5076,
  [SMALL_STATE(544)] = 5184,
  [SMALL_STATE(545)] = 5288,
  [SMALL_STATE(546)] = 5394,
  [SMALL_STATE(547)] = 5497,
  [SMALL_STATE(548)] = 5598,
  [SMALL_STATE(549)] = 5701,
  [SMALL_STATE(550)] = 5804,
  [SMALL_STATE(551)] = 5907,
  [SMALL_STATE(552)] = 6010,
  [SMALL_STATE(553)] = 6113,
  [SMALL_STATE(554)] = 6216,
  [SMALL_STATE(555)] = 6319,
  [SMALL_STATE(556)] = 6422,
  [SMALL_STATE(557)] = 6525,
  [SMALL_STATE(558)] = 6628,
  [SMALL_STATE(559)] = 6731,
  [SMALL_STATE(560)] = 6834,
  [SMALL_STATE(561)] = 6937,
  [SMALL_STATE(562)] = 7040,
  [SMALL_STATE(563)] = 7141,
  [SMALL_STATE(564)] = 7244,
  [SMALL_STATE(565)] = 7345,
  [SMALL_STATE(566)] = 7446,
  [SMALL_STATE(567)] = 7547,
  [SMALL_STATE(568)] = 7648,
  [SMALL_STATE(569)] = 7751,
  [SMALL_STATE(570)] = 7852,
  [SMALL_STATE(571)] = 7955,
  [SMALL_STATE(572)] = 8058,
  [SMALL_STATE(573)] = 8161,
  [SMALL_STATE(574)] = 8264,
  [SMALL_STATE(575)] = 8367,
  [SMALL_STATE(576)] = 8470,
  [SMALL_STATE(577)] = 8573,
  [SMALL_STATE(578)] = 8676,
  [SMALL_STATE(579)] = 8779,
  [SMALL_STATE(580)] = 8882,
  [SMALL_STATE(581)] = 8985,
  [SMALL_STATE(582)] = 9086,
  [SMALL_STATE(583)] = 9189,
  [SMALL_STATE(584)] = 9292,
  [SMALL_STATE(585)] = 9395,
  [SMALL_STATE(586)] = 9498,
  [SMALL_STATE(587)] = 9601,
  [SMALL_STATE(588)] = 9702,
  [SMALL_STATE(589)] = 9805,
  [SMALL_STATE(590)] = 9908,
  [SMALL_STATE(591)] = 10009,
  [SMALL_STATE(592)] = 10110,
  [SMALL_STATE(593)] = 10211,
  [SMALL_STATE(594)] = 10312,
  [SMALL_STATE(595)] = 10413,
  [SMALL_STATE(596)] = 10514,
  [SMALL_STATE(597)] = 10617,
  [SMALL_STATE(598)] = 10718,
  [SMALL_STATE(599)] = 10819,
  [SMALL_STATE(600)] = 10922,
  [SMALL_STATE(601)] = 11025,
  [SMALL_STATE(602)] = 11128,
  [SMALL_STATE(603)] = 11229,
  [SMALL_STATE(604)] = 11330,
  [SMALL_STATE(605)] = 11433,
  [SMALL_STATE(606)] = 11534,
  [SMALL_STATE(607)] = 11635,
  [SMALL_STATE(608)] = 11736,
  [SMALL_STATE(609)] = 11837,
  [SMALL_STATE(610)] = 11938,
  [SMALL_STATE(611)] = 12039,
  [SMALL_STATE(612)] = 12140,
  [SMALL_STATE(613)] = 12241,
  [SMALL_STATE(614)] = 12342,
  [SMALL_STATE(615)] = 12443,
  [SMALL_STATE(616)] = 12544,
  [SMALL_STATE(617)] = 12645,
  [SMALL_STATE(618)] = 12748,
  [SMALL_STATE(619)] = 12851,
  [SMALL_STATE(620)] = 12952,
  [SMALL_STATE(621)] = 13053,
  [SMALL_STATE(622)] = 13156,
  [SMALL_STATE(623)] = 13259,
  [SMALL_STATE(624)] = 13362,
  [SMALL_STATE(625)] = 13465,
  [SMALL_STATE(626)] = 13568,
  [SMALL_STATE(627)] = 13671,
  [SMALL_STATE(628)] = 13772,
  [SMALL_STATE(629)] = 13875,
  [SMALL_STATE(630)] = 13978,
  [SMALL_STATE(631)] = 14081,
  [SMALL_STATE(632)] = 14184,
  [SMALL_STATE(633)] = 14287,
  [SMALL_STATE(634)] = 14390,
  [SMALL_STATE(635)] = 14493,
  [SMALL_STATE(636)] = 14596,
  [SMALL_STATE(637)] = 14699,
  [SMALL_STATE(638)] = 14802,
  [SMALL_STATE(639)] = 14905,
  [SMALL_STATE(640)] = 15008,
  [SMALL_STATE(641)] = 15111,
  [SMALL_STATE(642)] = 15212,
  [SMALL_STATE(643)] = 15315,
  [SMALL_STATE(644)] = 15416,
  [SMALL_STATE(645)] = 15517,
  [SMALL_STATE(646)] = 15618,
  [SMALL_STATE(647)] = 15719,
  [SMALL_STATE(648)] = 15820,
  [SMALL_STATE(649)] = 15921,
  [SMALL_STATE(650)] = 16022,
  [SMALL_STATE(651)] = 16123,
  [SMALL_STATE(652)] = 16224,
  [SMALL_STATE(653)] = 16327,
  [SMALL_STATE(654)] = 16428,
  [SMALL_STATE(655)] = 16529,
  [SMALL_STATE(656)] = 16632,
  [SMALL_STATE(657)] = 16735,
  [SMALL_STATE(658)] = 16836,
  [SMALL_STATE(659)] = 16937,
  [SMALL_STATE(660)] = 17040,
  [SMALL_STATE(661)] = 17143,
  [SMALL_STATE(662)] = 17231,
  [SMALL_STATE(663)] = 17302,
  [SMALL_STATE(664)] = 17403,
  [SMALL_STATE(665)] = 17504,
  [SMALL_STATE(666)] = 17575,
  [SMALL_STATE(667)] = 17646,
  [SMALL_STATE(668)] = 17747,
  [SMALL_STATE(669)] = 17848,
  [SMALL_STATE(670)] = 17916,
  [SMALL_STATE(671)] = 17979,
  [SMALL_STATE(672)] = 18042,
  [SMALL_STATE(673)] = 18104,
  [SMALL_STATE(674)] = 18166,
  [SMALL_STATE(675)] = 18228,
  [SMALL_STATE(676)] = 18290,
  [SMALL_STATE(677)] = 18352,
  [SMALL_STATE(678)] = 18414,
  [SMALL_STATE(679)] = 18476,
  [SMALL_STATE(680)] = 18538,
  [SMALL_STATE(681)] = 18600,
  [SMALL_STATE(682)] = 18662,
  [SMALL_STATE(683)] = 18733,
  [SMALL_STATE(684)] = 18798,
  [SMALL_STATE(685)] = 18857,
  [SMALL_STATE(686)] = 18916,
  [SMALL_STATE(687)] = 18975,
  [SMALL_STATE(688)] = 19034,
  [SMALL_STATE(689)] = 19126,
  [SMALL_STATE(690)] = 19218,
  [SMALL_STATE(691)] = 19310,
  [SMALL_STATE(692)] = 19402,
  [SMALL_STATE(693)] = 19460,
  [SMALL_STATE(694)] = 19552,
  [SMALL_STATE(695)] = 19644,
  [SMALL_STATE(696)] = 19736,
  [SMALL_STATE(697)] = 19794,
  [SMALL_STATE(698)] = 19852,
  [SMALL_STATE(699)] = 19944,
  [SMALL_STATE(700)] = 20036,
  [SMALL_STATE(701)] = 20094,
  [SMALL_STATE(702)] = 20186,
  [SMALL_STATE(703)] = 20244,
  [SMALL_STATE(704)] = 20336,
  [SMALL_STATE(705)] = 20394,
  [SMALL_STATE(706)] = 20452,
  [SMALL_STATE(707)] = 20546,
  [SMALL_STATE(708)] = 20638,
  [SMALL_STATE(709)] = 20730,
  [SMALL_STATE(710)] = 20822,
  [SMALL_STATE(711)] = 20914,
  [SMALL_STATE(712)] = 20972,
  [SMALL_STATE(713)] = 21064,
  [SMALL_STATE(714)] = 21156,
  [SMALL_STATE(715)] = 21214,
  [SMALL_STATE(716)] = 21272,
  [SMALL_STATE(717)] = 21330,
  [SMALL_STATE(718)] = 21388,
  [SMALL_STATE(719)] = 21446,
  [SMALL_STATE(720)] = 21504,
  [SMALL_STATE(721)] = 21562,
  [SMALL_STATE(722)] = 21620,
  [SMALL_STATE(723)] = 21687,
  [SMALL_STATE(724)] = 21754,
  [SMALL_STATE(725)] = 21811,
  [SMALL_STATE(726)] = 21900,
  [SMALL_STATE(727)] = 21957,
  [SMALL_STATE(728)] = 22024,
  [SMALL_STATE(729)] = 22089,
  [SMALL_STATE(730)] = 22146,
  [SMALL_STATE(731)] = 22237,
  [SMALL_STATE(732)] = 22294,
  [SMALL_STATE(733)] = 22351,
  [SMALL_STATE(734)] = 22418,
  [SMALL_STATE(735)] = 22507,
  [SMALL_STATE(736)] = 22568,
  [SMALL_STATE(737)] = 22635,
  [SMALL_STATE(738)] = 22700,
  [SMALL_STATE(739)] = 22764,
  [SMALL_STATE(740)] = 22823,
  [SMALL_STATE(741)] = 22905,
  [SMALL_STATE(742)] = 22987,
  [SMALL_STATE(743)] = 23069,
  [SMALL_STATE(744)] = 23151,
  [SMALL_STATE(745)] = 23233,
  [SMALL_STATE(746)] = 23315,
  [SMALL_STATE(747)] = 23397,
  [SMALL_STATE(748)] = 23479,
  [SMALL_STATE(749)] = 23542,
  [SMALL_STATE(750)] = 23621,
  [SMALL_STATE(751)] = 23674,
  [SMALL_STATE(752)] = 23735,
  [SMALL_STATE(753)] = 23796,
  [SMALL_STATE(754)] = 23859,
  [SMALL_STATE(755)] = 23912,
  [SMALL_STATE(756)] = 23965,
  [SMALL_STATE(757)] = 24026,
  [SMALL_STATE(758)] = 24087,
  [SMALL_STATE(759)] = 24150,
  [SMALL_STATE(760)] = 24213,
  [SMALL_STATE(761)] = 24276,
  [SMALL_STATE(762)] = 24339,
  [SMALL_STATE(763)] = 24400,
  [SMALL_STATE(764)] = 24464,
  [SMALL_STATE(765)] = 24524,
  [SMALL_STATE(766)] = 24596,
  [SMALL_STATE(767)] = 24670,
  [SMALL_STATE(768)] = 24746,
  [SMALL_STATE(769)] = 24824,
  [SMALL_STATE(770)] = 24904,
  [SMALL_STATE(771)] = 24986,
  [SMALL_STATE(772)] = 25054,
  [SMALL_STATE(773)] = 25120,
  [SMALL_STATE(774)] = 25172,
  [SMALL_STATE(775)] = 25258,
  [SMALL_STATE(776)] = 25310,
  [SMALL_STATE(777)] = 25362,
  [SMALL_STATE(778)] = 25448,
  [SMALL_STATE(779)] = 25534,
  [SMALL_STATE(780)] = 25589,
  [SMALL_STATE(781)] = 25664,
  [SMALL_STATE(782)] = 25727,
  [SMALL_STATE(783)] = 25782,
  [SMALL_STATE(784)] = 25837,
  [SMALL_STATE(785)] = 25892,
  [SMALL_STATE(786)] = 25947,
  [SMALL_STATE(787)] = 26002,
  [SMALL_STATE(788)] = 26065,
  [SMALL_STATE(789)] = 26120,
  [SMALL_STATE(790)] = 26183,
  [SMALL_STATE(791)] = 26264,
  [SMALL_STATE(792)] = 26327,
  [SMALL_STATE(793)] = 26406,
  [SMALL_STATE(794)] = 26491,
  [SMALL_STATE(795)] = 26546,
  [SMALL_STATE(796)] = 26609,
  [SMALL_STATE(797)] = 26686,
  [SMALL_STATE(798)] = 26771,
  [SMALL_STATE(799)] = 26844,
  [SMALL_STATE(800)] = 26899,
  [SMALL_STATE(801)] = 26984,
  [SMALL_STATE(802)] = 27049,
  [SMALL_STATE(803)] = 27104,
  [SMALL_STATE(804)] = 27171,
  [SMALL_STATE(805)] = 27226,
  [SMALL_STATE(806)] = 27297,
  [SMALL_STATE(807)] = 27352,
  [SMALL_STATE(808)] = 27402,
  [SMALL_STATE(809)] = 27452,
  [SMALL_STATE(810)] = 27502,
  [SMALL_STATE(811)] = 27552,
  [SMALL_STATE(812)] = 27606,
  [SMALL_STATE(813)] = 27660,
  [SMALL_STATE(814)] = 27714,
  [SMALL_STATE(815)] = 27764,
  [SMALL_STATE(816)] = 27814,
  [SMALL_STATE(817)] = 27868,
  [SMALL_STATE(818)] = 27918,
  [SMALL_STATE(819)] = 27972,
  [SMALL_STATE(820)] = 28022,
  [SMALL_STATE(821)] = 28072,
  [SMALL_STATE(822)] = 28122,
  [SMALL_STATE(823)] = 28172,
  [SMALL_STATE(824)] = 28222,
  [SMALL_STATE(825)] = 28272,
  [SMALL_STATE(826)] = 28322,
  [SMALL_STATE(827)] = 28372,
  [SMALL_STATE(828)] = 28422,
  [SMALL_STATE(829)] = 28472,
  [SMALL_STATE(830)] = 28522,
  [SMALL_STATE(831)] = 28572,
  [SMALL_STATE(832)] = 28626,
  [SMALL_STATE(833)] = 28676,
  [SMALL_STATE(834)] = 28726,
  [SMALL_STATE(835)] = 28780,
  [SMALL_STATE(836)] = 28836,
  [SMALL_STATE(837)] = 28886,
  [SMALL_STATE(838)] = 28936,
  [SMALL_STATE(839)] = 28986,
  [SMALL_STATE(840)] = 29036,
  [SMALL_STATE(841)] = 29090,
  [SMALL_STATE(842)] = 29146,
  [SMALL_STATE(843)] = 29196,
  [SMALL_STATE(844)] = 29254,
  [SMALL_STATE(845)] = 29304,
  [SMALL_STATE(846)] = 29353,
  [SMALL_STATE(847)] = 29402,
  [SMALL_STATE(848)] = 29451,
  [SMALL_STATE(849)] = 29500,
  [SMALL_STATE(850)] = 29549,
  [SMALL_STATE(851)] = 29598,
  [SMALL_STATE(852)] = 29647,
  [SMALL_STATE(853)] = 29696,
  [SMALL_STATE(854)] = 29745,
  [SMALL_STATE(855)] = 29794,
  [SMALL_STATE(856)] = 29843,
  [SMALL_STATE(857)] = 29892,
  [SMALL_STATE(858)] = 29941,
  [SMALL_STATE(859)] = 29990,
  [SMALL_STATE(860)] = 30039,
  [SMALL_STATE(861)] = 30088,
  [SMALL_STATE(862)] = 30137,
  [SMALL_STATE(863)] = 30186,
  [SMALL_STATE(864)] = 30248,
  [SMALL_STATE(865)] = 30306,
  [SMALL_STATE(866)] = 30368,
  [SMALL_STATE(867)] = 30430,
  [SMALL_STATE(868)] = 30492,
  [SMALL_STATE(869)] = 30551,
  [SMALL_STATE(870)] = 30610,
  [SMALL_STATE(871)] = 30673,
  [SMALL_STATE(872)] = 30738,
  [SMALL_STATE(873)] = 30817,
  [SMALL_STATE(874)] = 30886,
  [SMALL_STATE(875)] = 30957,
  [SMALL_STATE(876)] = 31030,
  [SMALL_STATE(877)] = 31105,
  [SMALL_STATE(878)] = 31182,
  [SMALL_STATE(879)] = 31261,
  [SMALL_STATE(880)] = 31322,
  [SMALL_STATE(881)] = 31405,
  [SMALL_STATE(882)] = 31488,
  [SMALL_STATE(883)] = 31571,
  [SMALL_STATE(884)] = 31626,
  [SMALL_STATE(885)] = 31705,
  [SMALL_STATE(886)] = 31764,
  [SMALL_STATE(887)] = 31823,
  [SMALL_STATE(888)] = 31882,
  [SMALL_STATE(889)] = 31935,
  [SMALL_STATE(890)] = 31988,
  [SMALL_STATE(891)] = 32045,
  [SMALL_STATE(892)] = 32091,
  [SMALL_STATE(893)] = 32137,
  [SMALL_STATE(894)] = 32183,
  [SMALL_STATE(895)] = 32229,
  [SMALL_STATE(896)] = 32279,
  [SMALL_STATE(897)] = 32325,
  [SMALL_STATE(898)] = 32371,
  [SMALL_STATE(899)] = 32416,
  [SMALL_STATE(900)] = 32461,
  [SMALL_STATE(901)] = 32506,
  [SMALL_STATE(902)] = 32551,
  [SMALL_STATE(903)] = 32600,
  [SMALL_STATE(904)] = 32645,
  [SMALL_STATE(905)] = 32690,
  [SMALL_STATE(906)] = 32735,
  [SMALL_STATE(907)] = 32780,
  [SMALL_STATE(908)] = 32825,
  [SMALL_STATE(909)] = 32870,
  [SMALL_STATE(910)] = 32919,
  [SMALL_STATE(911)] = 32964,
  [SMALL_STATE(912)] = 33009,
  [SMALL_STATE(913)] = 33054,
  [SMALL_STATE(914)] = 33099,
  [SMALL_STATE(915)] = 33144,
  [SMALL_STATE(916)] = 33189,
  [SMALL_STATE(917)] = 33234,
  [SMALL_STATE(918)] = 33279,
  [SMALL_STATE(919)] = 33324,
  [SMALL_STATE(920)] = 33369,
  [SMALL_STATE(921)] = 33414,
  [SMALL_STATE(922)] = 33459,
  [SMALL_STATE(923)] = 33508,
  [SMALL_STATE(924)] = 33553,
  [SMALL_STATE(925)] = 33598,
  [SMALL_STATE(926)] = 33643,
  [SMALL_STATE(927)] = 33688,
  [SMALL_STATE(928)] = 33733,
  [SMALL_STATE(929)] = 33782,
  [SMALL_STATE(930)] = 33831,
  [SMALL_STATE(931)] = 33880,
  [SMALL_STATE(932)] = 33925,
  [SMALL_STATE(933)] = 33974,
  [SMALL_STATE(934)] = 34019,
  [SMALL_STATE(935)] = 34064,
  [SMALL_STATE(936)] = 34109,
  [SMALL_STATE(937)] = 34154,
  [SMALL_STATE(938)] = 34199,
  [SMALL_STATE(939)] = 34244,
  [SMALL_STATE(940)] = 34289,
  [SMALL_STATE(941)] = 34334,
  [SMALL_STATE(942)] = 34378,
  [SMALL_STATE(943)] = 34422,
  [SMALL_STATE(944)] = 34466,
  [SMALL_STATE(945)] = 34510,
  [SMALL_STATE(946)] = 34554,
  [SMALL_STATE(947)] = 34598,
  [SMALL_STATE(948)] = 34642,
  [SMALL_STATE(949)] = 34686,
  [SMALL_STATE(950)] = 34730,
  [SMALL_STATE(951)] = 34774,
  [SMALL_STATE(952)] = 34843,
  [SMALL_STATE(953)] = 34898,
  [SMALL_STATE(954)] = 34967,
  [SMALL_STATE(955)] = 35036,
  [SMALL_STATE(956)] = 35079,
  [SMALL_STATE(957)] = 35148,
  [SMALL_STATE(958)] = 35200,
  [SMALL_STATE(959)] = 35250,
  [SMALL_STATE(960)] = 35321,
  [SMALL_STATE(961)] = 35398,
  [SMALL_STATE(962)] = 35453,
  [SMALL_STATE(963)] = 35530,
  [SMALL_STATE(964)] = 35607,
  [SMALL_STATE(965)] = 35680,
  [SMALL_STATE(966)] = 35757,
  [SMALL_STATE(967)] = 35826,
  [SMALL_STATE(968)] = 35893,
  [SMALL_STATE(969)] = 35958,
  [SMALL_STATE(970)] = 36021,
  [SMALL_STATE(971)] = 36080,
  [SMALL_STATE(972)] = 36137,
  [SMALL_STATE(973)] = 36195,
  [SMALL_STATE(974)] = 36259,
  [SMALL_STATE(975)] = 36327,
  [SMALL_STATE(976)] = 36369,
  [SMALL_STATE(977)] = 36437,
  [SMALL_STATE(978)] = 36507,
  [SMALL_STATE(979)] = 36547,
  [SMALL_STATE(980)] = 36589,
  [SMALL_STATE(981)] = 36643,
  [SMALL_STATE(982)] = 36685,
  [SMALL_STATE(983)] = 36725,
  [SMALL_STATE(984)] = 36765,
  [SMALL_STATE(985)] = 36805,
  [SMALL_STATE(986)] = 36871,
  [SMALL_STATE(987)] = 36911,
  [SMALL_STATE(988)] = 36967,
  [SMALL_STATE(989)] = 37007,
  [SMALL_STATE(990)] = 37047,
  [SMALL_STATE(991)] = 37109,
  [SMALL_STATE(992)] = 37149,
  [SMALL_STATE(993)] = 37223,
  [SMALL_STATE(994)] = 37263,
  [SMALL_STATE(995)] = 37303,
  [SMALL_STATE(996)] = 37343,
  [SMALL_STATE(997)] = 37383,
  [SMALL_STATE(998)] = 37423,
  [SMALL_STATE(999)] = 37497,
  [SMALL_STATE(1000)] = 37539,
  [SMALL_STATE(1001)] = 37613,
  [SMALL_STATE(1002)] = 37676,
  [SMALL_STATE(1003)] = 37739,
  [SMALL_STATE(1004)] = 37804,
  [SMALL_STATE(1005)] = 37869,
  [SMALL_STATE(1006)] = 37929,
  [SMALL_STATE(1007)] = 37989,
  [SMALL_STATE(1008)] = 38049,
  [SMALL_STATE(1009)] = 38087,
  [SMALL_STATE(1010)] = 38147,
  [SMALL_STATE(1011)] = 38207,
  [SMALL_STATE(1012)] = 38267,
  [SMALL_STATE(1013)] = 38305,
  [SMALL_STATE(1014)] = 38365,
  [SMALL_STATE(1015)] = 38403,
  [SMALL_STATE(1016)] = 38463,
  [SMALL_STATE(1017)] = 38509,
  [SMALL_STATE(1018)] = 38547,
  [SMALL_STATE(1019)] = 38621,
  [SMALL_STATE(1020)] = 38696,
  [SMALL_STATE(1021)] = 38767,
  [SMALL_STATE(1022)] = 38842,
  [SMALL_STATE(1023)] = 38917,
  [SMALL_STATE(1024)] = 38992,
  [SMALL_STATE(1025)] = 39067,
  [SMALL_STATE(1026)] = 39142,
  [SMALL_STATE(1027)] = 39214,
  [SMALL_STATE(1028)] = 39284,
  [SMALL_STATE(1029)] = 39340,
  [SMALL_STATE(1030)] = 39396,
  [SMALL_STATE(1031)] = 39462,
  [SMALL_STATE(1032)] = 39534,
  [SMALL_STATE(1033)] = 39606,
  [SMALL_STATE(1034)] = 39678,
  [SMALL_STATE(1035)] = 39750,
  [SMALL_STATE(1036)] = 39806,
  [SMALL_STATE(1037)] = 39870,
  [SMALL_STATE(1038)] = 39926,
  [SMALL_STATE(1039)] = 39982,
  [SMALL_STATE(1040)] = 40054,
  [SMALL_STATE(1041)] = 40120,
  [SMALL_STATE(1042)] = 40192,
  [SMALL_STATE(1043)] = 40264,
  [SMALL_STATE(1044)] = 40320,
  [SMALL_STATE(1045)] = 40390,
  [SMALL_STATE(1046)] = 40452,
  [SMALL_STATE(1047)] = 40524,
  [SMALL_STATE(1048)] = 40596,
  [SMALL_STATE(1049)] = 40656,
  [SMALL_STATE(1050)] = 40726,
  [SMALL_STATE(1051)] = 40798,
  [SMALL_STATE(1052)] = 40854,
  [SMALL_STATE(1053)] = 40910,
  [SMALL_STATE(1054)] = 40978,
  [SMALL_STATE(1055)] = 41050,
  [SMALL_STATE(1056)] = 41120,
  [SMALL_STATE(1057)] = 41192,
  [SMALL_STATE(1058)] = 41264,
  [SMALL_STATE(1059)] = 41336,
  [SMALL_STATE(1060)] = 41408,
  [SMALL_STATE(1061)] = 41464,
  [SMALL_STATE(1062)] = 41516,
  [SMALL_STATE(1063)] = 41572,
  [SMALL_STATE(1064)] = 41626,
  [SMALL_STATE(1065)] = 41698,
  [SMALL_STATE(1066)] = 41770,
  [SMALL_STATE(1067)] = 41842,
  [SMALL_STATE(1068)] = 41914,
  [SMALL_STATE(1069)] = 41986,
  [SMALL_STATE(1070)] = 42058,
  [SMALL_STATE(1071)] = 42128,
  [SMALL_STATE(1072)] = 42184,
  [SMALL_STATE(1073)] = 42240,
  [SMALL_STATE(1074)] = 42296,
  [SMALL_STATE(1075)] = 42366,
  [SMALL_STATE(1076)] = 42438,
  [SMALL_STATE(1077)] = 42508,
  [SMALL_STATE(1078)] = 42564,
  [SMALL_STATE(1079)] = 42636,
  [SMALL_STATE(1080)] = 42710,
  [SMALL_STATE(1081)] = 42782,
  [SMALL_STATE(1082)] = 42854,
  [SMALL_STATE(1083)] = 42926,
  [SMALL_STATE(1084)] = 42998,
  [SMALL_STATE(1085)] = 43068,
  [SMALL_STATE(1086)] = 43140,
  [SMALL_STATE(1087)] = 43209,
  [SMALL_STATE(1088)] = 43278,
  [SMALL_STATE(1089)] = 43347,
  [SMALL_STATE(1090)] = 43400,
  [SMALL_STATE(1091)] = 43469,
  [SMALL_STATE(1092)] = 43538,
  [SMALL_STATE(1093)] = 43607,
  [SMALL_STATE(1094)] = 43676,
  [SMALL_STATE(1095)] = 43745,
  [SMALL_STATE(1096)] = 43814,
  [SMALL_STATE(1097)] = 43883,
  [SMALL_STATE(1098)] = 43952,
  [SMALL_STATE(1099)] = 44021,
  [SMALL_STATE(1100)] = 44074,
  [SMALL_STATE(1101)] = 44143,
  [SMALL_STATE(1102)] = 44212,
  [SMALL_STATE(1103)] = 44265,
  [SMALL_STATE(1104)] = 44334,
  [SMALL_STATE(1105)] = 44403,
  [SMALL_STATE(1106)] = 44456,
  [SMALL_STATE(1107)] = 44525,
  [SMALL_STATE(1108)] = 44578,
  [SMALL_STATE(1109)] = 44613,
  [SMALL_STATE(1110)] = 44648,
  [SMALL_STATE(1111)] = 44701,
  [SMALL_STATE(1112)] = 44770,
  [SMALL_STATE(1113)] = 44839,
  [SMALL_STATE(1114)] = 44878,
  [SMALL_STATE(1115)] = 44947,
  [SMALL_STATE(1116)] = 44983,
  [SMALL_STATE(1117)] = 45027,
  [SMALL_STATE(1118)] = 45093,
  [SMALL_STATE(1119)] = 45135,
  [SMALL_STATE(1120)] = 45171,
  [SMALL_STATE(1121)] = 45207,
  [SMALL_STATE(1122)] = 45243,
  [SMALL_STATE(1123)] = 45290,
  [SMALL_STATE(1124)] = 45337,
  [SMALL_STATE(1125)] = 45384,
  [SMALL_STATE(1126)] = 45431,
  [SMALL_STATE(1127)] = 45478,
  [SMALL_STATE(1128)] = 45525,
  [SMALL_STATE(1129)] = 45574,
  [SMALL_STATE(1130)] = 45628,
  [SMALL_STATE(1131)] = 45672,
  [SMALL_STATE(1132)] = 45716,
  [SMALL_STATE(1133)] = 45770,
  [SMALL_STATE(1134)] = 45806,
  [SMALL_STATE(1135)] = 45860,
  [SMALL_STATE(1136)] = 45914,
  [SMALL_STATE(1137)] = 45958,
  [SMALL_STATE(1138)] = 46002,
  [SMALL_STATE(1139)] = 46046,
  [SMALL_STATE(1140)] = 46084,
  [SMALL_STATE(1141)] = 46118,
  [SMALL_STATE(1142)] = 46162,
  [SMALL_STATE(1143)] = 46213,
  [SMALL_STATE(1144)] = 46264,
  [SMALL_STATE(1145)] = 46315,
  [SMALL_STATE(1146)] = 46366,
  [SMALL_STATE(1147)] = 46409,
  [SMALL_STATE(1148)] = 46460,
  [SMALL_STATE(1149)] = 46515,
  [SMALL_STATE(1150)] = 46566,
  [SMALL_STATE(1151)] = 46621,
  [SMALL_STATE(1152)] = 46654,
  [SMALL_STATE(1153)] = 46697,
  [SMALL_STATE(1154)] = 46748,
  [SMALL_STATE(1155)] = 46799,
  [SMALL_STATE(1156)] = 46850,
  [SMALL_STATE(1157)] = 46901,
  [SMALL_STATE(1158)] = 46956,
  [SMALL_STATE(1159)] = 47007,
  [SMALL_STATE(1160)] = 47050,
  [SMALL_STATE(1161)] = 47100,
  [SMALL_STATE(1162)] = 47140,
  [SMALL_STATE(1163)] = 47174,
  [SMALL_STATE(1164)] = 47210,
  [SMALL_STATE(1165)] = 47250,
  [SMALL_STATE(1166)] = 47292,
  [SMALL_STATE(1167)] = 47336,
  [SMALL_STATE(1168)] = 47376,
  [SMALL_STATE(1169)] = 47422,
  [SMALL_STATE(1170)] = 47468,
  [SMALL_STATE(1171)] = 47516,
  [SMALL_STATE(1172)] = 47556,
  [SMALL_STATE(1173)] = 47596,
  [SMALL_STATE(1174)] = 47636,
  [SMALL_STATE(1175)] = 47664,
  [SMALL_STATE(1176)] = 47696,
  [SMALL_STATE(1177)] = 47728,
  [SMALL_STATE(1178)] = 47768,
  [SMALL_STATE(1179)] = 47808,
  [SMALL_STATE(1180)] = 47848,
  [SMALL_STATE(1181)] = 47876,
  [SMALL_STATE(1182)] = 47916,
  [SMALL_STATE(1183)] = 47956,
  [SMALL_STATE(1184)] = 47996,
  [SMALL_STATE(1185)] = 48044,
  [SMALL_STATE(1186)] = 48084,
  [SMALL_STATE(1187)] = 48124,
  [SMALL_STATE(1188)] = 48164,
  [SMALL_STATE(1189)] = 48204,
  [SMALL_STATE(1190)] = 48244,
  [SMALL_STATE(1191)] = 48284,
  [SMALL_STATE(1192)] = 48324,
  [SMALL_STATE(1193)] = 48352,
  [SMALL_STATE(1194)] = 48392,
  [SMALL_STATE(1195)] = 48432,
  [SMALL_STATE(1196)] = 48472,
  [SMALL_STATE(1197)] = 48504,
  [SMALL_STATE(1198)] = 48532,
  [SMALL_STATE(1199)] = 48564,
  [SMALL_STATE(1200)] = 48604,
  [SMALL_STATE(1201)] = 48644,
  [SMALL_STATE(1202)] = 48684,
  [SMALL_STATE(1203)] = 48724,
  [SMALL_STATE(1204)] = 48764,
  [SMALL_STATE(1205)] = 48804,
  [SMALL_STATE(1206)] = 48844,
  [SMALL_STATE(1207)] = 48884,
  [SMALL_STATE(1208)] = 48912,
  [SMALL_STATE(1209)] = 48952,
  [SMALL_STATE(1210)] = 48992,
  [SMALL_STATE(1211)] = 49032,
  [SMALL_STATE(1212)] = 49060,
  [SMALL_STATE(1213)] = 49100,
  [SMALL_STATE(1214)] = 49140,
  [SMALL_STATE(1215)] = 49180,
  [SMALL_STATE(1216)] = 49220,
  [SMALL_STATE(1217)] = 49260,
  [SMALL_STATE(1218)] = 49300,
  [SMALL_STATE(1219)] = 49332,
  [SMALL_STATE(1220)] = 49372,
  [SMALL_STATE(1221)] = 49399,
  [SMALL_STATE(1222)] = 49440,
  [SMALL_STATE(1223)] = 49467,
  [SMALL_STATE(1224)] = 49502,
  [SMALL_STATE(1225)] = 49529,
  [SMALL_STATE(1226)] = 49574,
  [SMALL_STATE(1227)] = 49617,
  [SMALL_STATE(1228)] = 49646,
  [SMALL_STATE(1229)] = 49689,
  [SMALL_STATE(1230)] = 49732,
  [SMALL_STATE(1231)] = 49777,
  [SMALL_STATE(1232)] = 49822,
  [SMALL_STATE(1233)] = 49867,
  [SMALL_STATE(1234)] = 49912,
  [SMALL_STATE(1235)] = 49957,
  [SMALL_STATE(1236)] = 49990,
  [SMALL_STATE(1237)] = 50017,
  [SMALL_STATE(1238)] = 50066,
  [SMALL_STATE(1239)] = 50097,
  [SMALL_STATE(1240)] = 50128,
  [SMALL_STATE(1241)] = 50173,
  [SMALL_STATE(1242)] = 50200,
  [SMALL_STATE(1243)] = 50245,
  [SMALL_STATE(1244)] = 50290,
  [SMALL_STATE(1245)] = 50323,
  [SMALL_STATE(1246)] = 50366,
  [SMALL_STATE(1247)] = 50393,
  [SMALL_STATE(1248)] = 50428,
  [SMALL_STATE(1249)] = 50473,
  [SMALL_STATE(1250)] = 50504,
  [SMALL_STATE(1251)] = 50549,
  [SMALL_STATE(1252)] = 50576,
  [SMALL_STATE(1253)] = 50619,
  [SMALL_STATE(1254)] = 50650,
  [SMALL_STATE(1255)] = 50699,
  [SMALL_STATE(1256)] = 50744,
  [SMALL_STATE(1257)] = 50789,
  [SMALL_STATE(1258)] = 50828,
  [SMALL_STATE(1259)] = 50859,
  [SMALL_STATE(1260)] = 50886,
  [SMALL_STATE(1261)] = 50917,
  [SMALL_STATE(1262)] = 50948,
  [SMALL_STATE(1263)] = 50993,
  [SMALL_STATE(1264)] = 51024,
  [SMALL_STATE(1265)] = 51061,
  [SMALL_STATE(1266)] = 51092,
  [SMALL_STATE(1267)] = 51119,
  [SMALL_STATE(1268)] = 51164,
  [SMALL_STATE(1269)] = 51191,
  [SMALL_STATE(1270)] = 51231,
  [SMALL_STATE(1271)] = 51271,
  [SMALL_STATE(1272)] = 51311,
  [SMALL_STATE(1273)] = 51351,
  [SMALL_STATE(1274)] = 51391,
  [SMALL_STATE(1275)] = 51431,
  [SMALL_STATE(1276)] = 51475,
  [SMALL_STATE(1277)] = 51516,
  [SMALL_STATE(1278)] = 51557,
  [SMALL_STATE(1279)] = 51598,
  [SMALL_STATE(1280)] = 51639,
  [SMALL_STATE(1281)] = 51680,
  [SMALL_STATE(1282)] = 51715,
  [SMALL_STATE(1283)] = 51756,
  [SMALL_STATE(1284)] = 51797,
  [SMALL_STATE(1285)] = 51838,
  [SMALL_STATE(1286)] = 51878,
  [SMALL_STATE(1287)] = 51912,
  [SMALL_STATE(1288)] = 51950,
  [SMALL_STATE(1289)] = 51988,
  [SMALL_STATE(1290)] = 52029,
  [SMALL_STATE(1291)] = 52068,
  [SMALL_STATE(1292)] = 52105,
  [SMALL_STATE(1293)] = 52134,
  [SMALL_STATE(1294)] = 52163,
  [SMALL_STATE(1295)] = 52202,
  [SMALL_STATE(1296)] = 52243,
  [SMALL_STATE(1297)] = 52272,
  [SMALL_STATE(1298)] = 52301,
  [SMALL_STATE(1299)] = 52340,
  [SMALL_STATE(1300)] = 52381,
  [SMALL_STATE(1301)] = 52420,
  [SMALL_STATE(1302)] = 52446,
  [SMALL_STATE(1303)] = 52467,
  [SMALL_STATE(1304)] = 52496,
  [SMALL_STATE(1305)] = 52533,
  [SMALL_STATE(1306)] = 52570,
  [SMALL_STATE(1307)] = 52607,
  [SMALL_STATE(1308)] = 52636,
  [SMALL_STATE(1309)] = 52673,
  [SMALL_STATE(1310)] = 52710,
  [SMALL_STATE(1311)] = 52743,
  [SMALL_STATE(1312)] = 52780,
  [SMALL_STATE(1313)] = 52809,
  [SMALL_STATE(1314)] = 52846,
  [SMALL_STATE(1315)] = 52883,
  [SMALL_STATE(1316)] = 52920,
  [SMALL_STATE(1317)] = 52945,
  [SMALL_STATE(1318)] = 52978,
  [SMALL_STATE(1319)] = 52999,
  [SMALL_STATE(1320)] = 53036,
  [SMALL_STATE(1321)] = 53073,
  [SMALL_STATE(1322)] = 53102,
  [SMALL_STATE(1323)] = 53123,
  [SMALL_STATE(1324)] = 53159,
  [SMALL_STATE(1325)] = 53183,
  [SMALL_STATE(1326)] = 53212,
  [SMALL_STATE(1327)] = 53241,
  [SMALL_STATE(1328)] = 53270,
  [SMALL_STATE(1329)] = 53299,
  [SMALL_STATE(1330)] = 53330,
  [SMALL_STATE(1331)] = 53359,
  [SMALL_STATE(1332)] = 53390,
  [SMALL_STATE(1333)] = 53419,
  [SMALL_STATE(1334)] = 53448,
  [SMALL_STATE(1335)] = 53475,
  [SMALL_STATE(1336)] = 53502,
  [SMALL_STATE(1337)] = 53533,
  [SMALL_STATE(1338)] = 53562,
  [SMALL_STATE(1339)] = 53591,
  [SMALL_STATE(1340)] = 53622,
  [SMALL_STATE(1341)] = 53647,
  [SMALL_STATE(1342)] = 53676,
  [SMALL_STATE(1343)] = 53707,
  [SMALL_STATE(1344)] = 53732,
  [SMALL_STATE(1345)] = 53759,
  [SMALL_STATE(1346)] = 53790,
  [SMALL_STATE(1347)] = 53819,
  [SMALL_STATE(1348)] = 53848,
  [SMALL_STATE(1349)] = 53877,
  [SMALL_STATE(1350)] = 53908,
  [SMALL_STATE(1351)] = 53935,
  [SMALL_STATE(1352)] = 53966,
  [SMALL_STATE(1353)] = 53997,
  [SMALL_STATE(1354)] = 54026,
  [SMALL_STATE(1355)] = 54044,
  [SMALL_STATE(1356)] = 54070,
  [SMALL_STATE(1357)] = 54088,
  [SMALL_STATE(1358)] = 54120,
  [SMALL_STATE(1359)] = 54152,
  [SMALL_STATE(1360)] = 54182,
  [SMALL_STATE(1361)] = 54208,
  [SMALL_STATE(1362)] = 54234,
  [SMALL_STATE(1363)] = 54266,
  [SMALL_STATE(1364)] = 54284,
  [SMALL_STATE(1365)] = 54302,
  [SMALL_STATE(1366)] = 54326,
  [SMALL_STATE(1367)] = 54352,
  [SMALL_STATE(1368)] = 54370,
  [SMALL_STATE(1369)] = 54392,
  [SMALL_STATE(1370)] = 54424,
  [SMALL_STATE(1371)] = 54448,
  [SMALL_STATE(1372)] = 54466,
  [SMALL_STATE(1373)] = 54484,
  [SMALL_STATE(1374)] = 54513,
  [SMALL_STATE(1375)] = 54532,
  [SMALL_STATE(1376)] = 54561,
  [SMALL_STATE(1377)] = 54590,
  [SMALL_STATE(1378)] = 54619,
  [SMALL_STATE(1379)] = 54648,
  [SMALL_STATE(1380)] = 54677,
  [SMALL_STATE(1381)] = 54698,
  [SMALL_STATE(1382)] = 54719,
  [SMALL_STATE(1383)] = 54740,
  [SMALL_STATE(1384)] = 54769,
  [SMALL_STATE(1385)] = 54790,
  [SMALL_STATE(1386)] = 54815,
  [SMALL_STATE(1387)] = 54836,
  [SMALL_STATE(1388)] = 54857,
  [SMALL_STATE(1389)] = 54886,
  [SMALL_STATE(1390)] = 54907,
  [SMALL_STATE(1391)] = 54932,
  [SMALL_STATE(1392)] = 54953,
  [SMALL_STATE(1393)] = 54969,
  [SMALL_STATE(1394)] = 54985,
  [SMALL_STATE(1395)] = 55009,
  [SMALL_STATE(1396)] = 55025,
  [SMALL_STATE(1397)] = 55051,
  [SMALL_STATE(1398)] = 55077,
  [SMALL_STATE(1399)] = 55093,
  [SMALL_STATE(1400)] = 55109,
  [SMALL_STATE(1401)] = 55125,
  [SMALL_STATE(1402)] = 55151,
  [SMALL_STATE(1403)] = 55177,
  [SMALL_STATE(1404)] = 55203,
  [SMALL_STATE(1405)] = 55229,
  [SMALL_STATE(1406)] = 55255,
  [SMALL_STATE(1407)] = 55281,
  [SMALL_STATE(1408)] = 55307,
  [SMALL_STATE(1409)] = 55327,
  [SMALL_STATE(1410)] = 55349,
  [SMALL_STATE(1411)] = 55369,
  [SMALL_STATE(1412)] = 55395,
  [SMALL_STATE(1413)] = 55411,
  [SMALL_STATE(1414)] = 55429,
  [SMALL_STATE(1415)] = 55455,
  [SMALL_STATE(1416)] = 55471,
  [SMALL_STATE(1417)] = 55487,
  [SMALL_STATE(1418)] = 55513,
  [SMALL_STATE(1419)] = 55533,
  [SMALL_STATE(1420)] = 55548,
  [SMALL_STATE(1421)] = 55565,
  [SMALL_STATE(1422)] = 55588,
  [SMALL_STATE(1423)] = 55603,
  [SMALL_STATE(1424)] = 55618,
  [SMALL_STATE(1425)] = 55635,
  [SMALL_STATE(1426)] = 55658,
  [SMALL_STATE(1427)] = 55673,
  [SMALL_STATE(1428)] = 55696,
  [SMALL_STATE(1429)] = 55711,
  [SMALL_STATE(1430)] = 55734,
  [SMALL_STATE(1431)] = 55753,
  [SMALL_STATE(1432)] = 55776,
  [SMALL_STATE(1433)] = 55791,
  [SMALL_STATE(1434)] = 55808,
  [SMALL_STATE(1435)] = 55833,
  [SMALL_STATE(1436)] = 55848,
  [SMALL_STATE(1437)] = 55865,
  [SMALL_STATE(1438)] = 55888,
  [SMALL_STATE(1439)] = 55903,
  [SMALL_STATE(1440)] = 55926,
  [SMALL_STATE(1441)] = 55945,
  [SMALL_STATE(1442)] = 55960,
  [SMALL_STATE(1443)] = 55975,
  [SMALL_STATE(1444)] = 55991,
  [SMALL_STATE(1445)] = 56011,
  [SMALL_STATE(1446)] = 56027,
  [SMALL_STATE(1447)] = 56045,
  [SMALL_STATE(1448)] = 56063,
  [SMALL_STATE(1449)] = 56077,
  [SMALL_STATE(1450)] = 56095,
  [SMALL_STATE(1451)] = 56115,
  [SMALL_STATE(1452)] = 56135,
  [SMALL_STATE(1453)] = 56149,
  [SMALL_STATE(1454)] = 56163,
  [SMALL_STATE(1455)] = 56177,
  [SMALL_STATE(1456)] = 56191,
  [SMALL_STATE(1457)] = 56205,
  [SMALL_STATE(1458)] = 56219,
  [SMALL_STATE(1459)] = 56233,
  [SMALL_STATE(1460)] = 56253,
  [SMALL_STATE(1461)] = 56273,
  [SMALL_STATE(1462)] = 56293,
  [SMALL_STATE(1463)] = 56307,
  [SMALL_STATE(1464)] = 56323,
  [SMALL_STATE(1465)] = 56337,
  [SMALL_STATE(1466)] = 56351,
  [SMALL_STATE(1467)] = 56371,
  [SMALL_STATE(1468)] = 56385,
  [SMALL_STATE(1469)] = 56405,
  [SMALL_STATE(1470)] = 56423,
  [SMALL_STATE(1471)] = 56443,
  [SMALL_STATE(1472)] = 56463,
  [SMALL_STATE(1473)] = 56483,
  [SMALL_STATE(1474)] = 56501,
  [SMALL_STATE(1475)] = 56519,
  [SMALL_STATE(1476)] = 56537,
  [SMALL_STATE(1477)] = 56551,
  [SMALL_STATE(1478)] = 56571,
  [SMALL_STATE(1479)] = 56585,
  [SMALL_STATE(1480)] = 56599,
  [SMALL_STATE(1481)] = 56619,
  [SMALL_STATE(1482)] = 56630,
  [SMALL_STATE(1483)] = 56641,
  [SMALL_STATE(1484)] = 56660,
  [SMALL_STATE(1485)] = 56671,
  [SMALL_STATE(1486)] = 56688,
  [SMALL_STATE(1487)] = 56699,
  [SMALL_STATE(1488)] = 56710,
  [SMALL_STATE(1489)] = 56721,
  [SMALL_STATE(1490)] = 56732,
  [SMALL_STATE(1491)] = 56743,
  [SMALL_STATE(1492)] = 56760,
  [SMALL_STATE(1493)] = 56771,
  [SMALL_STATE(1494)] = 56790,
  [SMALL_STATE(1495)] = 56805,
  [SMALL_STATE(1496)] = 56816,
  [SMALL_STATE(1497)] = 56827,
  [SMALL_STATE(1498)] = 56838,
  [SMALL_STATE(1499)] = 56857,
  [SMALL_STATE(1500)] = 56872,
  [SMALL_STATE(1501)] = 56883,
  [SMALL_STATE(1502)] = 56893,
  [SMALL_STATE(1503)] = 56907,
  [SMALL_STATE(1504)] = 56921,
  [SMALL_STATE(1505)] = 56935,
  [SMALL_STATE(1506)] = 56949,
  [SMALL_STATE(1507)] = 56965,
  [SMALL_STATE(1508)] = 56979,
  [SMALL_STATE(1509)] = 56993,
  [SMALL_STATE(1510)] = 57007,
  [SMALL_STATE(1511)] = 57021,
  [SMALL_STATE(1512)] = 57035,
  [SMALL_STATE(1513)] = 57049,
  [SMALL_STATE(1514)] = 57063,
  [SMALL_STATE(1515)] = 57079,
  [SMALL_STATE(1516)] = 57095,
  [SMALL_STATE(1517)] = 57109,
  [SMALL_STATE(1518)] = 57125,
  [SMALL_STATE(1519)] = 57141,
  [SMALL_STATE(1520)] = 57155,
  [SMALL_STATE(1521)] = 57169,
  [SMALL_STATE(1522)] = 57183,
  [SMALL_STATE(1523)] = 57197,
  [SMALL_STATE(1524)] = 57213,
  [SMALL_STATE(1525)] = 57223,
  [SMALL_STATE(1526)] = 57239,
  [SMALL_STATE(1527)] = 57255,
  [SMALL_STATE(1528)] = 57269,
  [SMALL_STATE(1529)] = 57285,
  [SMALL_STATE(1530)] = 57299,
  [SMALL_STATE(1531)] = 57313,
  [SMALL_STATE(1532)] = 57327,
  [SMALL_STATE(1533)] = 57341,
  [SMALL_STATE(1534)] = 57355,
  [SMALL_STATE(1535)] = 57371,
  [SMALL_STATE(1536)] = 57385,
  [SMALL_STATE(1537)] = 57399,
  [SMALL_STATE(1538)] = 57413,
  [SMALL_STATE(1539)] = 57429,
  [SMALL_STATE(1540)] = 57443,
  [SMALL_STATE(1541)] = 57457,
  [SMALL_STATE(1542)] = 57471,
  [SMALL_STATE(1543)] = 57485,
  [SMALL_STATE(1544)] = 57499,
  [SMALL_STATE(1545)] = 57513,
  [SMALL_STATE(1546)] = 57527,
  [SMALL_STATE(1547)] = 57543,
  [SMALL_STATE(1548)] = 57557,
  [SMALL_STATE(1549)] = 57573,
  [SMALL_STATE(1550)] = 57589,
  [SMALL_STATE(1551)] = 57603,
  [SMALL_STATE(1552)] = 57617,
  [SMALL_STATE(1553)] = 57631,
  [SMALL_STATE(1554)] = 57645,
  [SMALL_STATE(1555)] = 57661,
  [SMALL_STATE(1556)] = 57675,
  [SMALL_STATE(1557)] = 57691,
  [SMALL_STATE(1558)] = 57707,
  [SMALL_STATE(1559)] = 57723,
  [SMALL_STATE(1560)] = 57737,
  [SMALL_STATE(1561)] = 57751,
  [SMALL_STATE(1562)] = 57765,
  [SMALL_STATE(1563)] = 57779,
  [SMALL_STATE(1564)] = 57793,
  [SMALL_STATE(1565)] = 57809,
  [SMALL_STATE(1566)] = 57822,
  [SMALL_STATE(1567)] = 57835,
  [SMALL_STATE(1568)] = 57848,
  [SMALL_STATE(1569)] = 57861,
  [SMALL_STATE(1570)] = 57874,
  [SMALL_STATE(1571)] = 57887,
  [SMALL_STATE(1572)] = 57900,
  [SMALL_STATE(1573)] = 57913,
  [SMALL_STATE(1574)] = 57926,
  [SMALL_STATE(1575)] = 57939,
  [SMALL_STATE(1576)] = 57952,
  [SMALL_STATE(1577)] = 57965,
  [SMALL_STATE(1578)] = 57978,
  [SMALL_STATE(1579)] = 57991,
  [SMALL_STATE(1580)] = 58004,
  [SMALL_STATE(1581)] = 58017,
  [SMALL_STATE(1582)] = 58030,
  [SMALL_STATE(1583)] = 58043,
  [SMALL_STATE(1584)] = 58056,
  [SMALL_STATE(1585)] = 58069,
  [SMALL_STATE(1586)] = 58082,
  [SMALL_STATE(1587)] = 58095,
  [SMALL_STATE(1588)] = 58108,
  [SMALL_STATE(1589)] = 58121,
  [SMALL_STATE(1590)] = 58134,
  [SMALL_STATE(1591)] = 58147,
  [SMALL_STATE(1592)] = 58156,
  [SMALL_STATE(1593)] = 58169,
  [SMALL_STATE(1594)] = 58182,
  [SMALL_STATE(1595)] = 58195,
  [SMALL_STATE(1596)] = 58208,
  [SMALL_STATE(1597)] = 58221,
  [SMALL_STATE(1598)] = 58234,
  [SMALL_STATE(1599)] = 58247,
  [SMALL_STATE(1600)] = 58260,
  [SMALL_STATE(1601)] = 58273,
  [SMALL_STATE(1602)] = 58286,
  [SMALL_STATE(1603)] = 58299,
  [SMALL_STATE(1604)] = 58312,
  [SMALL_STATE(1605)] = 58325,
  [SMALL_STATE(1606)] = 58338,
  [SMALL_STATE(1607)] = 58351,
  [SMALL_STATE(1608)] = 58364,
  [SMALL_STATE(1609)] = 58377,
  [SMALL_STATE(1610)] = 58390,
  [SMALL_STATE(1611)] = 58401,
  [SMALL_STATE(1612)] = 58412,
  [SMALL_STATE(1613)] = 58425,
  [SMALL_STATE(1614)] = 58438,
  [SMALL_STATE(1615)] = 58451,
  [SMALL_STATE(1616)] = 58464,
  [SMALL_STATE(1617)] = 58477,
  [SMALL_STATE(1618)] = 58490,
  [SMALL_STATE(1619)] = 58503,
  [SMALL_STATE(1620)] = 58516,
  [SMALL_STATE(1621)] = 58529,
  [SMALL_STATE(1622)] = 58542,
  [SMALL_STATE(1623)] = 58555,
  [SMALL_STATE(1624)] = 58564,
  [SMALL_STATE(1625)] = 58577,
  [SMALL_STATE(1626)] = 58590,
  [SMALL_STATE(1627)] = 58603,
  [SMALL_STATE(1628)] = 58616,
  [SMALL_STATE(1629)] = 58629,
  [SMALL_STATE(1630)] = 58642,
  [SMALL_STATE(1631)] = 58655,
  [SMALL_STATE(1632)] = 58668,
  [SMALL_STATE(1633)] = 58677,
  [SMALL_STATE(1634)] = 58690,
  [SMALL_STATE(1635)] = 58699,
  [SMALL_STATE(1636)] = 58712,
  [SMALL_STATE(1637)] = 58725,
  [SMALL_STATE(1638)] = 58734,
  [SMALL_STATE(1639)] = 58745,
  [SMALL_STATE(1640)] = 58758,
  [SMALL_STATE(1641)] = 58771,
  [SMALL_STATE(1642)] = 58780,
  [SMALL_STATE(1643)] = 58793,
  [SMALL_STATE(1644)] = 58806,
  [SMALL_STATE(1645)] = 58819,
  [SMALL_STATE(1646)] = 58832,
  [SMALL_STATE(1647)] = 58845,
  [SMALL_STATE(1648)] = 58858,
  [SMALL_STATE(1649)] = 58871,
  [SMALL_STATE(1650)] = 58880,
  [SMALL_STATE(1651)] = 58891,
  [SMALL_STATE(1652)] = 58904,
  [SMALL_STATE(1653)] = 58917,
  [SMALL_STATE(1654)] = 58930,
  [SMALL_STATE(1655)] = 58943,
  [SMALL_STATE(1656)] = 58956,
  [SMALL_STATE(1657)] = 58965,
  [SMALL_STATE(1658)] = 58978,
  [SMALL_STATE(1659)] = 58991,
  [SMALL_STATE(1660)] = 59004,
  [SMALL_STATE(1661)] = 59017,
  [SMALL_STATE(1662)] = 59030,
  [SMALL_STATE(1663)] = 59043,
  [SMALL_STATE(1664)] = 59056,
  [SMALL_STATE(1665)] = 59069,
  [SMALL_STATE(1666)] = 59078,
  [SMALL_STATE(1667)] = 59091,
  [SMALL_STATE(1668)] = 59104,
  [SMALL_STATE(1669)] = 59117,
  [SMALL_STATE(1670)] = 59130,
  [SMALL_STATE(1671)] = 59143,
  [SMALL_STATE(1672)] = 59156,
  [SMALL_STATE(1673)] = 59169,
  [SMALL_STATE(1674)] = 59182,
  [SMALL_STATE(1675)] = 59195,
  [SMALL_STATE(1676)] = 59204,
  [SMALL_STATE(1677)] = 59213,
  [SMALL_STATE(1678)] = 59226,
  [SMALL_STATE(1679)] = 59239,
  [SMALL_STATE(1680)] = 59252,
  [SMALL_STATE(1681)] = 59263,
  [SMALL_STATE(1682)] = 59276,
  [SMALL_STATE(1683)] = 59289,
  [SMALL_STATE(1684)] = 59302,
  [SMALL_STATE(1685)] = 59315,
  [SMALL_STATE(1686)] = 59324,
  [SMALL_STATE(1687)] = 59337,
  [SMALL_STATE(1688)] = 59347,
  [SMALL_STATE(1689)] = 59357,
  [SMALL_STATE(1690)] = 59367,
  [SMALL_STATE(1691)] = 59377,
  [SMALL_STATE(1692)] = 59387,
  [SMALL_STATE(1693)] = 59397,
  [SMALL_STATE(1694)] = 59405,
  [SMALL_STATE(1695)] = 59415,
  [SMALL_STATE(1696)] = 59425,
  [SMALL_STATE(1697)] = 59435,
  [SMALL_STATE(1698)] = 59445,
  [SMALL_STATE(1699)] = 59453,
  [SMALL_STATE(1700)] = 59463,
  [SMALL_STATE(1701)] = 59473,
  [SMALL_STATE(1702)] = 59483,
  [SMALL_STATE(1703)] = 59493,
  [SMALL_STATE(1704)] = 59503,
  [SMALL_STATE(1705)] = 59511,
  [SMALL_STATE(1706)] = 59521,
  [SMALL_STATE(1707)] = 59531,
  [SMALL_STATE(1708)] = 59539,
  [SMALL_STATE(1709)] = 59549,
  [SMALL_STATE(1710)] = 59559,
  [SMALL_STATE(1711)] = 59569,
  [SMALL_STATE(1712)] = 59579,
  [SMALL_STATE(1713)] = 59587,
  [SMALL_STATE(1714)] = 59595,
  [SMALL_STATE(1715)] = 59603,
  [SMALL_STATE(1716)] = 59611,
  [SMALL_STATE(1717)] = 59621,
  [SMALL_STATE(1718)] = 59631,
  [SMALL_STATE(1719)] = 59641,
  [SMALL_STATE(1720)] = 59651,
  [SMALL_STATE(1721)] = 59661,
  [SMALL_STATE(1722)] = 59671,
  [SMALL_STATE(1723)] = 59681,
  [SMALL_STATE(1724)] = 59691,
  [SMALL_STATE(1725)] = 59699,
  [SMALL_STATE(1726)] = 59707,
  [SMALL_STATE(1727)] = 59717,
  [SMALL_STATE(1728)] = 59727,
  [SMALL_STATE(1729)] = 59735,
  [SMALL_STATE(1730)] = 59745,
  [SMALL_STATE(1731)] = 59755,
  [SMALL_STATE(1732)] = 59765,
  [SMALL_STATE(1733)] = 59775,
  [SMALL_STATE(1734)] = 59785,
  [SMALL_STATE(1735)] = 59795,
  [SMALL_STATE(1736)] = 59805,
  [SMALL_STATE(1737)] = 59815,
  [SMALL_STATE(1738)] = 59825,
  [SMALL_STATE(1739)] = 59835,
  [SMALL_STATE(1740)] = 59845,
  [SMALL_STATE(1741)] = 59855,
  [SMALL_STATE(1742)] = 59865,
  [SMALL_STATE(1743)] = 59875,
  [SMALL_STATE(1744)] = 59885,
  [SMALL_STATE(1745)] = 59895,
  [SMALL_STATE(1746)] = 59905,
  [SMALL_STATE(1747)] = 59915,
  [SMALL_STATE(1748)] = 59925,
  [SMALL_STATE(1749)] = 59933,
  [SMALL_STATE(1750)] = 59941,
  [SMALL_STATE(1751)] = 59951,
  [SMALL_STATE(1752)] = 59959,
  [SMALL_STATE(1753)] = 59969,
  [SMALL_STATE(1754)] = 59977,
  [SMALL_STATE(1755)] = 59987,
  [SMALL_STATE(1756)] = 59997,
  [SMALL_STATE(1757)] = 60007,
  [SMALL_STATE(1758)] = 60017,
  [SMALL_STATE(1759)] = 60025,
  [SMALL_STATE(1760)] = 60035,
  [SMALL_STATE(1761)] = 60045,
  [SMALL_STATE(1762)] = 60055,
  [SMALL_STATE(1763)] = 60065,
  [SMALL_STATE(1764)] = 60075,
  [SMALL_STATE(1765)] = 60085,
  [SMALL_STATE(1766)] = 60095,
  [SMALL_STATE(1767)] = 60103,
  [SMALL_STATE(1768)] = 60113,
  [SMALL_STATE(1769)] = 60123,
  [SMALL_STATE(1770)] = 60133,
  [SMALL_STATE(1771)] = 60143,
  [SMALL_STATE(1772)] = 60153,
  [SMALL_STATE(1773)] = 60163,
  [SMALL_STATE(1774)] = 60171,
  [SMALL_STATE(1775)] = 60181,
  [SMALL_STATE(1776)] = 60188,
  [SMALL_STATE(1777)] = 60195,
  [SMALL_STATE(1778)] = 60202,
  [SMALL_STATE(1779)] = 60209,
  [SMALL_STATE(1780)] = 60216,
  [SMALL_STATE(1781)] = 60223,
  [SMALL_STATE(1782)] = 60230,
  [SMALL_STATE(1783)] = 60237,
  [SMALL_STATE(1784)] = 60244,
  [SMALL_STATE(1785)] = 60251,
  [SMALL_STATE(1786)] = 60258,
  [SMALL_STATE(1787)] = 60265,
  [SMALL_STATE(1788)] = 60272,
  [SMALL_STATE(1789)] = 60279,
  [SMALL_STATE(1790)] = 60286,
  [SMALL_STATE(1791)] = 60293,
  [SMALL_STATE(1792)] = 60300,
  [SMALL_STATE(1793)] = 60307,
  [SMALL_STATE(1794)] = 60314,
  [SMALL_STATE(1795)] = 60321,
  [SMALL_STATE(1796)] = 60328,
  [SMALL_STATE(1797)] = 60335,
  [SMALL_STATE(1798)] = 60342,
  [SMALL_STATE(1799)] = 60349,
  [SMALL_STATE(1800)] = 60356,
  [SMALL_STATE(1801)] = 60363,
  [SMALL_STATE(1802)] = 60370,
  [SMALL_STATE(1803)] = 60377,
  [SMALL_STATE(1804)] = 60384,
  [SMALL_STATE(1805)] = 60391,
  [SMALL_STATE(1806)] = 60398,
  [SMALL_STATE(1807)] = 60405,
  [SMALL_STATE(1808)] = 60412,
  [SMALL_STATE(1809)] = 60419,
  [SMALL_STATE(1810)] = 60426,
  [SMALL_STATE(1811)] = 60433,
  [SMALL_STATE(1812)] = 60440,
  [SMALL_STATE(1813)] = 60447,
  [SMALL_STATE(1814)] = 60454,
  [SMALL_STATE(1815)] = 60461,
  [SMALL_STATE(1816)] = 60468,
  [SMALL_STATE(1817)] = 60475,
  [SMALL_STATE(1818)] = 60482,
  [SMALL_STATE(1819)] = 60489,
  [SMALL_STATE(1820)] = 60496,
  [SMALL_STATE(1821)] = 60503,
  [SMALL_STATE(1822)] = 60510,
  [SMALL_STATE(1823)] = 60517,
  [SMALL_STATE(1824)] = 60524,
  [SMALL_STATE(1825)] = 60531,
  [SMALL_STATE(1826)] = 60538,
  [SMALL_STATE(1827)] = 60545,
  [SMALL_STATE(1828)] = 60552,
  [SMALL_STATE(1829)] = 60559,
  [SMALL_STATE(1830)] = 60566,
  [SMALL_STATE(1831)] = 60573,
  [SMALL_STATE(1832)] = 60580,
  [SMALL_STATE(1833)] = 60587,
  [SMALL_STATE(1834)] = 60594,
  [SMALL_STATE(1835)] = 60601,
  [SMALL_STATE(1836)] = 60608,
  [SMALL_STATE(1837)] = 60615,
  [SMALL_STATE(1838)] = 60622,
  [SMALL_STATE(1839)] = 60629,
  [SMALL_STATE(1840)] = 60636,
  [SMALL_STATE(1841)] = 60643,
  [SMALL_STATE(1842)] = 60650,
  [SMALL_STATE(1843)] = 60657,
  [SMALL_STATE(1844)] = 60664,
  [SMALL_STATE(1845)] = 60671,
  [SMALL_STATE(1846)] = 60678,
  [SMALL_STATE(1847)] = 60685,
  [SMALL_STATE(1848)] = 60692,
  [SMALL_STATE(1849)] = 60699,
  [SMALL_STATE(1850)] = 60706,
  [SMALL_STATE(1851)] = 60713,
  [SMALL_STATE(1852)] = 60720,
  [SMALL_STATE(1853)] = 60727,
  [SMALL_STATE(1854)] = 60734,
  [SMALL_STATE(1855)] = 60741,
  [SMALL_STATE(1856)] = 60748,
  [SMALL_STATE(1857)] = 60755,
  [SMALL_STATE(1858)] = 60762,
  [SMALL_STATE(1859)] = 60769,
  [SMALL_STATE(1860)] = 60776,
  [SMALL_STATE(1861)] = 60783,
  [SMALL_STATE(1862)] = 60790,
  [SMALL_STATE(1863)] = 60797,
  [SMALL_STATE(1864)] = 60804,
  [SMALL_STATE(1865)] = 60811,
  [SMALL_STATE(1866)] = 60818,
  [SMALL_STATE(1867)] = 60825,
  [SMALL_STATE(1868)] = 60832,
  [SMALL_STATE(1869)] = 60839,
  [SMALL_STATE(1870)] = 60846,
  [SMALL_STATE(1871)] = 60853,
  [SMALL_STATE(1872)] = 60860,
  [SMALL_STATE(1873)] = 60867,
  [SMALL_STATE(1874)] = 60874,
  [SMALL_STATE(1875)] = 60881,
  [SMALL_STATE(1876)] = 60888,
  [SMALL_STATE(1877)] = 60895,
  [SMALL_STATE(1878)] = 60902,
  [SMALL_STATE(1879)] = 60909,
  [SMALL_STATE(1880)] = 60916,
  [SMALL_STATE(1881)] = 60923,
  [SMALL_STATE(1882)] = 60930,
  [SMALL_STATE(1883)] = 60937,
  [SMALL_STATE(1884)] = 60944,
  [SMALL_STATE(1885)] = 60951,
  [SMALL_STATE(1886)] = 60958,
  [SMALL_STATE(1887)] = 60965,
  [SMALL_STATE(1888)] = 60972,
  [SMALL_STATE(1889)] = 60979,
  [SMALL_STATE(1890)] = 60986,
  [SMALL_STATE(1891)] = 60993,
  [SMALL_STATE(1892)] = 61000,
  [SMALL_STATE(1893)] = 61007,
  [SMALL_STATE(1894)] = 61014,
  [SMALL_STATE(1895)] = 61021,
  [SMALL_STATE(1896)] = 61028,
  [SMALL_STATE(1897)] = 61035,
  [SMALL_STATE(1898)] = 61042,
  [SMALL_STATE(1899)] = 61049,
  [SMALL_STATE(1900)] = 61056,
  [SMALL_STATE(1901)] = 61063,
  [SMALL_STATE(1902)] = 61070,
  [SMALL_STATE(1903)] = 61077,
  [SMALL_STATE(1904)] = 61084,
  [SMALL_STATE(1905)] = 61091,
  [SMALL_STATE(1906)] = 61098,
  [SMALL_STATE(1907)] = 61105,
  [SMALL_STATE(1908)] = 61112,
  [SMALL_STATE(1909)] = 61119,
  [SMALL_STATE(1910)] = 61126,
  [SMALL_STATE(1911)] = 61133,
  [SMALL_STATE(1912)] = 61140,
  [SMALL_STATE(1913)] = 61147,
  [SMALL_STATE(1914)] = 61154,
  [SMALL_STATE(1915)] = 61161,
  [SMALL_STATE(1916)] = 61168,
  [SMALL_STATE(1917)] = 61175,
  [SMALL_STATE(1918)] = 61182,
  [SMALL_STATE(1919)] = 61189,
  [SMALL_STATE(1920)] = 61196,
  [SMALL_STATE(1921)] = 61203,
  [SMALL_STATE(1922)] = 61210,
  [SMALL_STATE(1923)] = 61217,
  [SMALL_STATE(1924)] = 61224,
  [SMALL_STATE(1925)] = 61231,
  [SMALL_STATE(1926)] = 61238,
  [SMALL_STATE(1927)] = 61245,
  [SMALL_STATE(1928)] = 61252,
  [SMALL_STATE(1929)] = 61259,
  [SMALL_STATE(1930)] = 61266,
  [SMALL_STATE(1931)] = 61273,
  [SMALL_STATE(1932)] = 61280,
  [SMALL_STATE(1933)] = 61287,
  [SMALL_STATE(1934)] = 61294,
  [SMALL_STATE(1935)] = 61301,
  [SMALL_STATE(1936)] = 61308,
  [SMALL_STATE(1937)] = 61315,
  [SMALL_STATE(1938)] = 61322,
  [SMALL_STATE(1939)] = 61329,
  [SMALL_STATE(1940)] = 61336,
  [SMALL_STATE(1941)] = 61343,
  [SMALL_STATE(1942)] = 61350,
  [SMALL_STATE(1943)] = 61357,
  [SMALL_STATE(1944)] = 61364,
  [SMALL_STATE(1945)] = 61371,
  [SMALL_STATE(1946)] = 61378,
  [SMALL_STATE(1947)] = 61385,
  [SMALL_STATE(1948)] = 61392,
  [SMALL_STATE(1949)] = 61399,
  [SMALL_STATE(1950)] = 61406,
  [SMALL_STATE(1951)] = 61413,
  [SMALL_STATE(1952)] = 61420,
  [SMALL_STATE(1953)] = 61427,
  [SMALL_STATE(1954)] = 61434,
  [SMALL_STATE(1955)] = 61441,
  [SMALL_STATE(1956)] = 61448,
  [SMALL_STATE(1957)] = 61455,
  [SMALL_STATE(1958)] = 61462,
  [SMALL_STATE(1959)] = 61469,
  [SMALL_STATE(1960)] = 61476,
  [SMALL_STATE(1961)] = 61483,
  [SMALL_STATE(1962)] = 61490,
  [SMALL_STATE(1963)] = 61497,
  [SMALL_STATE(1964)] = 61504,
  [SMALL_STATE(1965)] = 61511,
  [SMALL_STATE(1966)] = 61518,
  [SMALL_STATE(1967)] = 61525,
  [SMALL_STATE(1968)] = 61532,
  [SMALL_STATE(1969)] = 61539,
  [SMALL_STATE(1970)] = 61546,
  [SMALL_STATE(1971)] = 61553,
  [SMALL_STATE(1972)] = 61560,
  [SMALL_STATE(1973)] = 61567,
  [SMALL_STATE(1974)] = 61574,
  [SMALL_STATE(1975)] = 61581,
  [SMALL_STATE(1976)] = 61588,
  [SMALL_STATE(1977)] = 61595,
  [SMALL_STATE(1978)] = 61602,
  [SMALL_STATE(1979)] = 61609,
  [SMALL_STATE(1980)] = 61616,
  [SMALL_STATE(1981)] = 61623,
  [SMALL_STATE(1982)] = 61630,
  [SMALL_STATE(1983)] = 61637,
  [SMALL_STATE(1984)] = 61644,
  [SMALL_STATE(1985)] = 61651,
  [SMALL_STATE(1986)] = 61658,
  [SMALL_STATE(1987)] = 61665,
  [SMALL_STATE(1988)] = 61672,
  [SMALL_STATE(1989)] = 61679,
  [SMALL_STATE(1990)] = 61686,
  [SMALL_STATE(1991)] = 61693,
  [SMALL_STATE(1992)] = 61700,
  [SMALL_STATE(1993)] = 61707,
  [SMALL_STATE(1994)] = 61714,
  [SMALL_STATE(1995)] = 61721,
  [SMALL_STATE(1996)] = 61728,
  [SMALL_STATE(1997)] = 61735,
  [SMALL_STATE(1998)] = 61742,
  [SMALL_STATE(1999)] = 61749,
  [SMALL_STATE(2000)] = 61756,
  [SMALL_STATE(2001)] = 61763,
  [SMALL_STATE(2002)] = 61770,
  [SMALL_STATE(2003)] = 61777,
  [SMALL_STATE(2004)] = 61784,
  [SMALL_STATE(2005)] = 61791,
  [SMALL_STATE(2006)] = 61798,
  [SMALL_STATE(2007)] = 61805,
  [SMALL_STATE(2008)] = 61812,
  [SMALL_STATE(2009)] = 61819,
  [SMALL_STATE(2010)] = 61826,
  [SMALL_STATE(2011)] = 61833,
  [SMALL_STATE(2012)] = 61840,
  [SMALL_STATE(2013)] = 61847,
  [SMALL_STATE(2014)] = 61854,
  [SMALL_STATE(2015)] = 61861,
  [SMALL_STATE(2016)] = 61868,
  [SMALL_STATE(2017)] = 61875,
  [SMALL_STATE(2018)] = 61882,
  [SMALL_STATE(2019)] = 61889,
  [SMALL_STATE(2020)] = 61896,
};

static const TSParseActionEntry ts_parse_actions[] = {
  [0] = {.entry = {.count = 0, .reusable = false}},
  [1] = {.entry = {.count = 1, .reusable = false}}, RECOVER(),
  [3] = {.entry = {.count = 1, .reusable = true}}, SHIFT_EXTRA(),
  [5] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_translation_unit, 0, 0, 0),
  [7] = {.entry = {.count = 1, .reusable = false}}, SHIFT(462),
  [9] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1381),
  [11] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2020),
  [13] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1219),
  [15] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2014),
  [17] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1769),
  [19] = {.entry = {.count = 1, .reusable = true}}, SHIFT(445),
  [21] = {.entry = {.count = 1, .reusable = true}}, SHIFT(563),
  [23] = {.entry = {.count = 1, .reusable = false}}, SHIFT(563),
  [25] = {.entry = {.count = 1, .reusable = true}}, SHIFT(569),
  [27] = {.entry = {.count = 1, .reusable = false}}, SHIFT(975),
  [29] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1071),
  [31] = {.entry = {.count = 1, .reusable = false}}, SHIFT(932),
  [33] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1971),
  [35] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1767),
  [37] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1950),
  [39] = {.entry = {.count = 1, .reusable = false}}, SHIFT(955),
  [41] = {.entry = {.count = 1, .reusable = true}}, SHIFT(33),
  [43] = {.entry = {.count = 1, .reusable = false}}, SHIFT(843),
  [45] = {.entry = {.count = 1, .reusable = false}}, SHIFT(776),
  [47] = {.entry = {.count = 1, .reusable = false}}, SHIFT(755),
  [49] = {.entry = {.count = 1, .reusable = false}}, SHIFT(818),
  [51] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1580),
  [53] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1434),
  [55] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1498),
  [57] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1709),
  [59] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1706),
  [61] = {.entry = {.count = 1, .reusable = false}}, SHIFT(655),
  [63] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1839),
  [65] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1754),
  [67] = {.entry = {.count = 1, .reusable = false}}, SHIFT(338),
  [69] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1778),
  [71] = {.entry = {.count = 1, .reusable = false}}, SHIFT(504),
  [73] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1896),
  [75] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1775),
  [77] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1792),
  [79] = {.entry = {.count = 1, .reusable = true}}, SHIFT(660),
  [81] = {.entry = {.count = 1, .reusable = false}}, SHIFT(659),
  [83] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1811),
  [85] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1813),
  [87] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1823),
  [89] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1463),
  [91] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1115),
  [93] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1650),
  [95] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1525),
  [97] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1115),
  [99] = {.entry = {.count = 1, .reusable = false}}, SHIFT(702),
  [101] = {.entry = {.count = 1, .reusable = false}}, SHIFT(456),
  [103] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1384),
  [105] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1917),
  [107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1200),
  [109] = {.entry = {.count = 1, .reusable = false}}, SHIFT(322),
  [111] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1918),
  [113] = {.entry = {.count = 1, .reusable = false}}, SHIFT(27),
  [115] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1187),
  [117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1902),
  [119] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1743),
  [121] = {.entry = {.count = 1, .reusable = true}}, SHIFT(79),
  [123] = {.entry = {.count = 1, .reusable = false}}, SHIFT(979),
  [125] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1052),
  [127] = {.entry = {.count = 1, .reusable = false}}, SHIFT(929),
  [129] = {.entry = {.count = 1, .reusable = true}}, SHIFT(41),
  [131] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1689),
  [133] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1687),
  [135] = {.entry = {.count = 1, .reusable = false}}, SHIFT(589),
  [137] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1824),
  [139] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1692),
  [141] = {.entry = {.count = 1, .reusable = false}}, SHIFT(315),
  [143] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2015),
  [145] = {.entry = {.count = 1, .reusable = false}}, SHIFT(521),
  [147] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1935),
  [149] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1934),
  [151] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1926),
  [153] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1737),
  [155] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1924),
  [157] = {.entry = {.count = 1, .reusable = true}}, SHIFT(700),
  [159] = {.entry = {.count = 1, .reusable = false}}, SHIFT(700),
  [161] = {.entry = {.count = 1, .reusable = false}}, SHIFT(330),
  [163] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_elif, 4, 0, 40),
  [165] = {.entry = {.count = 1, .reusable = false}}, SHIFT(429),
  [167] = {.entry = {.count = 1, .reusable = false}}, SHIFT(376),
  [169] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_elifdef, 3, 0, 17),
  [171] = {.entry = {.count = 1, .reusable = false}}, SHIFT(130),
  [173] = {.entry = {.count = 1, .reusable = false}}, SHIFT(348),
  [175] = {.entry = {.count = 1, .reusable = false}}, SHIFT(332),
  [177] = {.entry = {.count = 1, .reusable = false}}, SHIFT(421),
  [179] = {.entry = {.count = 1, .reusable = false}}, SHIFT(390),
  [181] = {.entry = {.count = 1, .reusable = false}}, SHIFT(334),
  [183] = {.entry = {.count = 1, .reusable = false}}, SHIFT(378),
  [185] = {.entry = {.count = 1, .reusable = false}}, SHIFT(155),
  [187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(152),
  [189] = {.entry = {.count = 1, .reusable = false}}, SHIFT(410),
  [191] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_elif, 3, 0, 40),
  [193] = {.entry = {.count = 1, .reusable = false}}, SHIFT(135),
  [195] = {.entry = {.count = 1, .reusable = false}}, SHIFT(408),
  [197] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_elifdef, 2, 0, 17),
  [199] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(456),
  [202] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1384),
  [205] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1917),
  [208] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1200),
  [211] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0),
  [213] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1918),
  [216] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1743),
  [219] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(445),
  [222] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(563),
  [225] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(563),
  [228] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(569),
  [231] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(79),
  [234] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(979),
  [237] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1052),
  [240] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(929),
  [243] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1971),
  [246] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1767),
  [249] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1950),
  [252] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(955),
  [255] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(41),
  [258] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(843),
  [261] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(776),
  [264] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(755),
  [267] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(818),
  [270] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1580),
  [273] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1434),
  [276] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1498),
  [279] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1689),
  [282] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1687),
  [285] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(589),
  [288] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1824),
  [291] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1692),
  [294] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(315),
  [297] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(2015),
  [300] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(521),
  [303] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1935),
  [306] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1934),
  [309] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1926),
  [312] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1737),
  [315] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1924),
  [318] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(660),
  [321] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(659),
  [324] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1811),
  [327] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1813),
  [330] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1823),
  [333] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1463),
  [336] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(700),
  [339] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1650),
  [342] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1525),
  [345] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(700),
  [348] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(702),
  [351] = {.entry = {.count = 1, .reusable = false}}, SHIFT(453),
  [353] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1389),
  [355] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1837),
  [357] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1193),
  [359] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1838),
  [361] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1711),
  [363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(284),
  [365] = {.entry = {.count = 1, .reusable = false}}, SHIFT(999),
  [367] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1077),
  [369] = {.entry = {.count = 1, .reusable = false}}, SHIFT(902),
  [371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(40),
  [373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(329),
  [375] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1695),
  [377] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1694),
  [379] = {.entry = {.count = 1, .reusable = false}}, SHIFT(656),
  [381] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1842),
  [383] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1690),
  [385] = {.entry = {.count = 1, .reusable = false}}, SHIFT(317),
  [387] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1980),
  [389] = {.entry = {.count = 1, .reusable = false}}, SHIFT(500),
  [391] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1845),
  [393] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1848),
  [395] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1846),
  [397] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1723),
  [399] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2000),
  [401] = {.entry = {.count = 1, .reusable = false}}, SHIFT(454),
  [403] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1391),
  [405] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1969),
  [407] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1202),
  [409] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_else, 2, 0, 0),
  [411] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1949),
  [413] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1757),
  [415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(229),
  [417] = {.entry = {.count = 1, .reusable = false}}, SHIFT(981),
  [419] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1035),
  [421] = {.entry = {.count = 1, .reusable = false}}, SHIFT(930),
  [423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(38),
  [425] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1734),
  [427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1764),
  [429] = {.entry = {.count = 1, .reusable = false}}, SHIFT(586),
  [431] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1793),
  [433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1735),
  [435] = {.entry = {.count = 1, .reusable = false}}, SHIFT(403),
  [437] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2018),
  [439] = {.entry = {.count = 1, .reusable = false}}, SHIFT(506),
  [441] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1822),
  [443] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1820),
  [445] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1954),
  [447] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1765),
  [449] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1806),
  [451] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(454),
  [454] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1391),
  [457] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1969),
  [460] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1202),
  [463] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1949),
  [466] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1757),
  [469] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(229),
  [472] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(981),
  [475] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1035),
  [478] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(930),
  [481] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(38),
  [484] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1734),
  [487] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1764),
  [490] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(586),
  [493] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1793),
  [496] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1735),
  [499] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(403),
  [502] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(2018),
  [505] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(506),
  [508] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1822),
  [511] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1820),
  [514] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1954),
  [517] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1765),
  [520] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1806),
  [523] = {.entry = {.count = 1, .reusable = true}}, SHIFT(342),
  [525] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_else, 1, 0, 0),
  [527] = {.entry = {.count = 1, .reusable = true}}, SHIFT(319),
  [529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(111),
  [531] = {.entry = {.count = 1, .reusable = true}}, SHIFT(133),
  [533] = {.entry = {.count = 1, .reusable = true}}, SHIFT(151),
  [535] = {.entry = {.count = 1, .reusable = true}}, SHIFT(423),
  [537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(92),
  [539] = {.entry = {.count = 1, .reusable = true}}, SHIFT(374),
  [541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(210),
  [543] = {.entry = {.count = 1, .reusable = true}}, SHIFT(104),
  [545] = {.entry = {.count = 1, .reusable = true}}, SHIFT(428),
  [547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(228),
  [549] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(453),
  [552] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1389),
  [555] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1837),
  [558] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1193),
  [561] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1838),
  [564] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1711),
  [567] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(284),
  [570] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(999),
  [573] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1077),
  [576] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(902),
  [579] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(40),
  [582] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0),
  [584] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1695),
  [587] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1694),
  [590] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(656),
  [593] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1842),
  [596] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1690),
  [599] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(317),
  [602] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1980),
  [605] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(500),
  [608] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1845),
  [611] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1848),
  [614] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1846),
  [617] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(1723),
  [620] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 2, 0, 0), SHIFT_REPEAT(2000),
  [623] = {.entry = {.count = 1, .reusable = true}}, SHIFT(235),
  [625] = {.entry = {.count = 1, .reusable = true}}, SHIFT(80),
  [627] = {.entry = {.count = 1, .reusable = true}}, SHIFT(245),
  [629] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_translation_unit, 1, 0, 0),
  [631] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0),
  [633] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(462),
  [636] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1381),
  [639] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(2020),
  [642] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1219),
  [645] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(2014),
  [648] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1769),
  [651] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(445),
  [654] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(563),
  [657] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(563),
  [660] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(569),
  [663] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(975),
  [666] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1071),
  [669] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(932),
  [672] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1971),
  [675] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1767),
  [678] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1950),
  [681] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(955),
  [684] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(33),
  [687] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(843),
  [690] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(776),
  [693] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(755),
  [696] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(818),
  [699] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1580),
  [702] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1434),
  [705] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1498),
  [708] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1709),
  [711] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1706),
  [714] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(655),
  [717] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1839),
  [720] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1754),
  [723] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(338),
  [726] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1778),
  [729] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(504),
  [732] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1896),
  [735] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1775),
  [738] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1792),
  [741] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(660),
  [744] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(659),
  [747] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1811),
  [750] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1813),
  [753] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1823),
  [756] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1463),
  [759] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1115),
  [762] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1650),
  [765] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1525),
  [768] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(1115),
  [771] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 2, 0, 0), SHIFT_REPEAT(702),
  [774] = {.entry = {.count = 1, .reusable = false}}, SHIFT(457),
  [776] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 3, 0, 9),
  [778] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 4, 0, 9),
  [780] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(457),
  [783] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0),
  [785] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(445),
  [788] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(563),
  [791] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(563),
  [794] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(569),
  [797] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(79),
  [800] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(979),
  [803] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1052),
  [806] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(776),
  [809] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1971),
  [812] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1767),
  [815] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1950),
  [818] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(41),
  [821] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(843),
  [824] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(755),
  [827] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(818),
  [830] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1580),
  [833] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1434),
  [836] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1498),
  [839] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1689),
  [842] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1687),
  [845] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1692),
  [848] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(315),
  [851] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2015),
  [854] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(521),
  [857] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1935),
  [860] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1934),
  [863] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1926),
  [866] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1737),
  [869] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1924),
  [872] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(660),
  [875] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(659),
  [878] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1811),
  [881] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1813),
  [884] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1823),
  [887] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1463),
  [890] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(700),
  [893] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1650),
  [896] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1525),
  [899] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(700),
  [902] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(702),
  [905] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 3, 0, 0),
  [907] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_case_statement, 2, 0, 0),
  [909] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_statement, 3, 0, 0),
  [911] = {.entry = {.count = 1, .reusable = false}}, SHIFT(455),
  [913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(234),
  [915] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1703),
  [917] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2007),
  [919] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(459),
  [922] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(284),
  [925] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(999),
  [928] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1077),
  [931] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(40),
  [934] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0),
  [936] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1695),
  [939] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1694),
  [942] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1690),
  [945] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(317),
  [948] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1980),
  [951] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(500),
  [954] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1845),
  [957] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1848),
  [960] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1846),
  [963] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1723),
  [966] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2000),
  [969] = {.entry = {.count = 1, .reusable = false}}, SHIFT(459),
  [971] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_statement, 4, 0, 9),
  [973] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(452),
  [976] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(229),
  [979] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(981),
  [982] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1035),
  [985] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(38),
  [988] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1734),
  [991] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1764),
  [994] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1735),
  [997] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(403),
  [1000] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2018),
  [1003] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(506),
  [1006] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1822),
  [1009] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1820),
  [1012] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1954),
  [1015] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1765),
  [1018] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1806),
  [1021] = {.entry = {.count = 1, .reusable = false}}, SHIFT(452),
  [1023] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_statement, 3, 0, 9),
  [1025] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_case_statement, 2, 0, 0),
  [1027] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(455),
  [1030] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(234),
  [1033] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(975),
  [1036] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1071),
  [1039] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(33),
  [1042] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1709),
  [1045] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1706),
  [1048] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1754),
  [1051] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(338),
  [1054] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1778),
  [1057] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(504),
  [1060] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1896),
  [1063] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1775),
  [1066] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1792),
  [1069] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1703),
  [1072] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(2007),
  [1075] = {.entry = {.count = 1, .reusable = false}}, SHIFT(460),
  [1077] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1741),
  [1079] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1740),
  [1081] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1999),
  [1083] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1688),
  [1085] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(460),
  [1088] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1741),
  [1091] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1740),
  [1094] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1999),
  [1097] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_case_statement_repeat1, 2, 0, 0), SHIFT_REPEAT(1688),
  [1100] = {.entry = {.count = 1, .reusable = false}}, SHIFT(461),
  [1102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(512),
  [1104] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1733),
  [1106] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 3, 0, 27),
  [1108] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 3, 0, 27),
  [1110] = {.entry = {.count = 1, .reusable = false}}, SHIFT(363),
  [1112] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_seh_finally_clause, 2, 0, 8),
  [1114] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_seh_finally_clause, 2, 0, 8),
  [1116] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_for_statement, 5, 0, 85),
  [1118] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_for_statement, 5, 0, 85),
  [1120] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 3, 0, 35),
  [1122] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 3, 0, 35),
  [1124] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_statement, 1, 0, 0),
  [1126] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 1, 0, 0),
  [1128] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_compound_statement, 2, 0, 0),
  [1130] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compound_statement, 2, 0, 0),
  [1132] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_statement, 2, 0, 0),
  [1134] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_statement, 2, 0, 0),
  [1136] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_break_statement, 2, 0, 0),
  [1138] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_break_statement, 2, 0, 0),
  [1140] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_continue_statement, 2, 0, 0),
  [1142] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_continue_statement, 2, 0, 0),
  [1144] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_definition, 4, 0, 47),
  [1146] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 4, 0, 47),
  [1148] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 6, 0, 111),
  [1150] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 6, 0, 111),
  [1152] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_seh_try_statement, 3, 0, 8),
  [1154] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_seh_try_statement, 3, 0, 8),
  [1156] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_if_statement, 4, 0, 55),
  [1158] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_if_statement, 4, 0, 55),
  [1160] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_seh_except_clause, 3, 0, 100),
  [1162] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_seh_except_clause, 3, 0, 100),
  [1164] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_definition, 6, 0, 76),
  [1166] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 6, 0, 76),
  [1168] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 5, 0, 91),
  [1170] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 5, 0, 91),
  [1172] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 5, 0, 90),
  [1174] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 5, 0, 90),
  [1176] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 4, 0, 62),
  [1178] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 4, 0, 62),
  [1180] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_labeled_statement, 3, 0, 31),
  [1182] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_labeled_statement, 3, 0, 31),
  [1184] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_goto_statement, 3, 0, 29),
  [1186] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_goto_statement, 3, 0, 29),
  [1188] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_return_statement, 3, 0, 0),
  [1190] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_return_statement, 3, 0, 0),
  [1192] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_while_statement, 3, 0, 28),
  [1194] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_while_statement, 3, 0, 28),
  [1196] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_switch_statement, 3, 0, 28),
  [1198] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_switch_statement, 3, 0, 28),
  [1200] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration, 4, 0, 64),
  [1202] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration, 4, 0, 64),
  [1204] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_compound_statement, 3, 0, 0),
  [1206] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compound_statement, 3, 0, 0),
  [1208] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_expression_statement, 2, 0, 0),
  [1210] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_expression_statement, 2, 0, 0),
  [1212] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attributed_statement, 2, 0, 0),
  [1214] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attributed_statement, 2, 0, 0),
  [1216] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_definition, 5, 0, 76),
  [1218] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 5, 0, 76),
  [1220] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_definition, 5, 0, 47),
  [1222] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_definition, 5, 0, 47),
  [1224] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_seh_leave_statement, 2, 0, 0),
  [1226] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_seh_leave_statement, 2, 0, 0),
  [1228] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_else_clause, 2, 0, 0),
  [1230] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_else_clause, 2, 0, 0),
  [1232] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_do_statement, 5, 0, 81),
  [1234] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_do_statement, 5, 0, 81),
  [1236] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 4, 0, 61),
  [1238] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 4, 0, 61),
  [1240] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__empty_declaration, 2, 0, 0),
  [1242] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__empty_declaration, 2, 0, 0),
  [1244] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef, 3, 0, 17),
  [1246] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef, 3, 0, 17),
  [1248] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_call, 3, 0, 18),
  [1250] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_call, 3, 0, 18),
  [1252] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_linkage_specification, 3, 0, 23),
  [1254] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_linkage_specification, 3, 0, 23),
  [1256] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration_list, 3, 0, 0),
  [1258] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration_list, 3, 0, 0),
  [1260] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef, 5, 0, 73),
  [1262] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef, 5, 0, 73),
  [1264] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if, 5, 0, 40),
  [1266] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if, 5, 0, 40),
  [1268] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if, 5, 0, 72),
  [1270] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if, 5, 0, 72),
  [1272] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_function_def, 5, 0, 71),
  [1274] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_function_def, 5, 0, 71),
  [1276] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__old_style_function_definition, 4, 0, 68),
  [1278] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__old_style_function_definition, 4, 0, 68),
  [1280] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 4, 0, 67),
  [1282] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 4, 0, 67),
  [1284] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__old_style_function_definition, 3, 0, 36),
  [1286] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__old_style_function_definition, 3, 0, 36),
  [1288] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__old_style_function_definition, 4, 0, 66),
  [1290] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__old_style_function_definition, 4, 0, 66),
  [1292] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_call, 2, 0, 4),
  [1294] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_call, 2, 0, 4),
  [1296] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_def, 4, 0, 38),
  [1298] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_def, 4, 0, 38),
  [1300] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_def, 3, 0, 17),
  [1302] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_def, 3, 0, 17),
  [1304] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 5, 0, 94),
  [1306] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 5, 0, 94),
  [1308] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_include, 3, 0, 16),
  [1310] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_include, 3, 0, 16),
  [1312] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__old_style_function_definition, 5, 0, 95),
  [1314] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__old_style_function_definition, 5, 0, 95),
  [1316] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if, 6, 0, 97),
  [1318] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if, 6, 0, 97),
  [1320] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_definition, 3, 0, 33),
  [1322] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_definition, 3, 0, 33),
  [1324] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_repeat1, 1, 0, 2),
  [1326] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_repeat1, 1, 0, 2),
  [1328] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_declaration_list, 2, 0, 0),
  [1330] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_declaration_list, 2, 0, 0),
  [1332] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef, 4, 0, 17),
  [1334] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef, 4, 0, 17),
  [1336] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef, 4, 0, 41),
  [1338] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef, 4, 0, 41),
  [1340] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_function_def, 4, 0, 39),
  [1342] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_function_def, 4, 0, 39),
  [1344] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if, 4, 0, 40),
  [1346] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if, 4, 0, 40),
  [1348] = {.entry = {.count = 1, .reusable = false}}, SHIFT(318),
  [1350] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sizeof_expression, 4, 0, 57),
  [1352] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sizeof_expression, 4, 0, 57),
  [1354] = {.entry = {.count = 1, .reusable = false}}, SHIFT(654),
  [1356] = {.entry = {.count = 1, .reusable = true}}, SHIFT(654),
  [1358] = {.entry = {.count = 1, .reusable = true}}, SHIFT(474),
  [1360] = {.entry = {.count = 1, .reusable = false}}, SHIFT(658),
  [1362] = {.entry = {.count = 1, .reusable = false}}, SHIFT(355),
  [1364] = {.entry = {.count = 1, .reusable = false}}, SHIFT(669),
  [1366] = {.entry = {.count = 1, .reusable = false}}, SHIFT(567),
  [1368] = {.entry = {.count = 1, .reusable = true}}, SHIFT(567),
  [1370] = {.entry = {.count = 1, .reusable = false}}, SHIFT(562),
  [1372] = {.entry = {.count = 1, .reusable = false}}, SHIFT(399),
  [1374] = {.entry = {.count = 1, .reusable = false}}, SHIFT(753),
  [1376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1691),
  [1378] = {.entry = {.count = 1, .reusable = false}}, SHIFT(760),
  [1380] = {.entry = {.count = 1, .reusable = false}}, SHIFT(761),
  [1382] = {.entry = {.count = 1, .reusable = false}}, SHIFT(596),
  [1384] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1937),
  [1386] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(753),
  [1389] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(445),
  [1392] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(563),
  [1395] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(563),
  [1398] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(569),
  [1401] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(229),
  [1404] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1691),
  [1407] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(38),
  [1410] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1734),
  [1413] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1764),
  [1416] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(586),
  [1419] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1793),
  [1422] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1735),
  [1425] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(403),
  [1428] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(2018),
  [1431] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(506),
  [1434] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1822),
  [1437] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1820),
  [1440] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1954),
  [1443] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1765),
  [1446] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1806),
  [1449] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(660),
  [1452] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(659),
  [1455] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1811),
  [1458] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1813),
  [1461] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1823),
  [1464] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1463),
  [1467] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(700),
  [1470] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1650),
  [1473] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1525),
  [1476] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(700),
  [1479] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(702),
  [1482] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(760),
  [1485] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(234),
  [1488] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(33),
  [1491] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1709),
  [1494] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1706),
  [1497] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(655),
  [1500] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1839),
  [1503] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1754),
  [1506] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(338),
  [1509] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1778),
  [1512] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(504),
  [1515] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1896),
  [1518] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1775),
  [1521] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1792),
  [1524] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1703),
  [1527] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(2007),
  [1530] = {.entry = {.count = 1, .reusable = false}}, SHIFT(758),
  [1532] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(761),
  [1535] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(284),
  [1538] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1741),
  [1541] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(596),
  [1544] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1937),
  [1547] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1740),
  [1550] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1999),
  [1553] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1688),
  [1556] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(2000),
  [1559] = {.entry = {.count = 1, .reusable = false}}, SHIFT(748),
  [1561] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(748),
  [1564] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(40),
  [1567] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1695),
  [1570] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1694),
  [1573] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(656),
  [1576] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1842),
  [1579] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1690),
  [1582] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(317),
  [1585] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1980),
  [1588] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(500),
  [1591] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1845),
  [1594] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1848),
  [1597] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1846),
  [1600] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1723),
  [1603] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(758),
  [1606] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(79),
  [1609] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(41),
  [1612] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1689),
  [1615] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1687),
  [1618] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(589),
  [1621] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1824),
  [1624] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1692),
  [1627] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(315),
  [1630] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(2015),
  [1633] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(521),
  [1636] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1935),
  [1639] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1934),
  [1642] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1926),
  [1645] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1737),
  [1648] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT(1924),
  [1651] = {.entry = {.count = 1, .reusable = false}}, SHIFT(605),
  [1653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(605),
  [1655] = {.entry = {.count = 1, .reusable = false}}, SHIFT(602),
  [1657] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_translation_unit_repeat1, 1, 0, 2),
  [1659] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_translation_unit_repeat1, 1, 0, 2),
  [1661] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__top_level_expression_statement, 2, 0, 0),
  [1663] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__top_level_expression_statement, 2, 0, 0),
  [1665] = {.entry = {.count = 1, .reusable = false}}, SHIFT(661),
  [1667] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1139),
  [1669] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1686),
  [1671] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attribute_declaration, 4, 0, 0),
  [1673] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute_declaration, 4, 0, 0),
  [1675] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attribute_declaration, 3, 0, 0),
  [1677] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute_declaration, 3, 0, 0),
  [1679] = {.entry = {.count = 1, .reusable = false}}, SHIFT(738),
  [1681] = {.entry = {.count = 1, .reusable = false}}, SHIFT(637),
  [1683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(637),
  [1685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(639),
  [1687] = {.entry = {.count = 1, .reusable = false}}, SHIFT(764),
  [1689] = {.entry = {.count = 1, .reusable = false}}, SHIFT(582),
  [1691] = {.entry = {.count = 1, .reusable = true}}, SHIFT(582),
  [1693] = {.entry = {.count = 1, .reusable = false}}, SHIFT(578),
  [1695] = {.entry = {.count = 1, .reusable = false}}, SHIFT(344),
  [1697] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_specifier, 1, 0, 1),
  [1699] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression_not_binary, 1, 0, 0),
  [1701] = {.entry = {.count = 3, .reusable = true}}, REDUCE(sym__type_specifier, 1, 0, 1), REDUCE(sym__expression_not_binary, 1, 0, 0), SHIFT(1038),
  [1705] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression_not_binary, 1, 0, 0),
  [1707] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__type_specifier, 1, 0, 1), REDUCE(sym__expression_not_binary, 1, 0, 0),
  [1710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_specifier, 1, 0, 1),
  [1712] = {.entry = {.count = 1, .reusable = false}}, SHIFT(811),
  [1714] = {.entry = {.count = 1, .reusable = false}}, SHIFT(573),
  [1716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(313),
  [1718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(573),
  [1720] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type_specifier, 1, 0, 1), REDUCE(sym__expression_not_binary, 1, 0, 0),
  [1723] = {.entry = {.count = 1, .reusable = true}}, SHIFT(366),
  [1725] = {.entry = {.count = 1, .reusable = true}}, SHIFT(386),
  [1727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(384),
  [1729] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__declaration_modifiers, 1, 0, 0), REDUCE(aux_sym_attributed_declarator_repeat1, 1, 0, 0),
  [1732] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 1, 0, 0),
  [1734] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 1, 0, 0),
  [1736] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_modifiers, 1, 0, 0),
  [1738] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__declaration_modifiers, 1, 0, 0), REDUCE(aux_sym_attributed_declarator_repeat1, 1, 0, 0),
  [1741] = {.entry = {.count = 1, .reusable = true}}, SHIFT(350),
  [1743] = {.entry = {.count = 1, .reusable = false}}, SHIFT(957),
  [1745] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1724),
  [1747] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1302),
  [1749] = {.entry = {.count = 1, .reusable = true}}, SHIFT(463),
  [1751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(872),
  [1753] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1705),
  [1755] = {.entry = {.count = 1, .reusable = false}}, SHIFT(472),
  [1757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(442),
  [1759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(540),
  [1761] = {.entry = {.count = 1, .reusable = true}}, SHIFT(581),
  [1763] = {.entry = {.count = 1, .reusable = false}}, SHIFT(750),
  [1765] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1363),
  [1767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(579),
  [1769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(531),
  [1771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1356),
  [1773] = {.entry = {.count = 1, .reusable = true}}, SHIFT(542),
  [1775] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1400),
  [1777] = {.entry = {.count = 1, .reusable = true}}, SHIFT(544),
  [1779] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1419),
  [1781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(530),
  [1783] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1395),
  [1785] = {.entry = {.count = 1, .reusable = true}}, SHIFT(535),
  [1787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1500),
  [1789] = {.entry = {.count = 1, .reusable = true}}, SHIFT(536),
  [1791] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1438),
  [1793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(529),
  [1795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1484),
  [1797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(537),
  [1799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1481),
  [1801] = {.entry = {.count = 1, .reusable = true}}, SHIFT(538),
  [1803] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1492),
  [1805] = {.entry = {.count = 1, .reusable = false}}, SHIFT(759),
  [1807] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1976),
  [1809] = {.entry = {.count = 1, .reusable = true}}, SHIFT(696),
  [1811] = {.entry = {.count = 1, .reusable = true}}, SHIFT(588),
  [1813] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1970),
  [1815] = {.entry = {.count = 1, .reusable = true}}, SHIFT(711),
  [1817] = {.entry = {.count = 1, .reusable = true}}, SHIFT(704),
  [1819] = {.entry = {.count = 1, .reusable = false}}, SHIFT(841),
  [1821] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1973),
  [1823] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1161),
  [1825] = {.entry = {.count = 1, .reusable = false}}, SHIFT(923),
  [1827] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1884),
  [1829] = {.entry = {.count = 1, .reusable = false}}, SHIFT(527),
  [1831] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1189),
  [1833] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1874),
  [1835] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1730),
  [1837] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_elif_in_field_declaration_list, 4, 0, 40),
  [1839] = {.entry = {.count = 1, .reusable = false}}, SHIFT(906),
  [1841] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_elifdef_in_field_declaration_list, 3, 0, 17),
  [1843] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_elif_in_field_declaration_list, 3, 0, 40),
  [1845] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_elifdef_in_field_declaration_list, 2, 0, 17),
  [1847] = {.entry = {.count = 1, .reusable = false}}, SHIFT(925),
  [1849] = {.entry = {.count = 1, .reusable = false}}, SHIFT(899),
  [1851] = {.entry = {.count = 1, .reusable = false}}, SHIFT(911),
  [1853] = {.entry = {.count = 1, .reusable = false}}, SHIFT(848),
  [1855] = {.entry = {.count = 1, .reusable = false}}, SHIFT(862),
  [1857] = {.entry = {.count = 1, .reusable = false}}, SHIFT(910),
  [1859] = {.entry = {.count = 1, .reusable = false}}, SHIFT(853),
  [1861] = {.entry = {.count = 1, .reusable = false}}, SHIFT(913),
  [1863] = {.entry = {.count = 1, .reusable = false}}, SHIFT(847),
  [1865] = {.entry = {.count = 1, .reusable = false}}, SHIFT(916),
  [1867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(494),
  [1869] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1003),
  [1871] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(841),
  [1874] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1973),
  [1877] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1161),
  [1880] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0),
  [1882] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1884),
  [1885] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1730),
  [1888] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(755),
  [1891] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(776),
  [1894] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1971),
  [1897] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1733),
  [1900] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1950),
  [1903] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(843),
  [1906] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(818),
  [1909] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1580),
  [1912] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1434),
  [1915] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1498),
  [1918] = {.entry = {.count = 1, .reusable = true}}, SHIFT(732),
  [1920] = {.entry = {.count = 1, .reusable = false}}, SHIFT(599),
  [1922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(891),
  [1924] = {.entry = {.count = 1, .reusable = false}}, SHIFT(622),
  [1926] = {.entry = {.count = 1, .reusable = false}}, SHIFT(652),
  [1928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(237),
  [1930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(441),
  [1932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(603),
  [1934] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 3, 0, 56),
  [1936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(627),
  [1938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(265),
  [1940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(523),
  [1942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(226),
  [1944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(572),
  [1946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(434),
  [1948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(565),
  [1950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(438),
  [1952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(566),
  [1954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(657),
  [1956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(619),
  [1958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(525),
  [1960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(621),
  [1962] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 3, 0, 84),
  [1964] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 3, 0, 40),
  [1966] = {.entry = {.count = 1, .reusable = false}}, SHIFT(728),
  [1968] = {.entry = {.count = 1, .reusable = true}}, SHIFT(443),
  [1970] = {.entry = {.count = 1, .reusable = true}}, SHIFT(638),
  [1972] = {.entry = {.count = 1, .reusable = true}}, SHIFT(502),
  [1974] = {.entry = {.count = 1, .reusable = true}}, SHIFT(81),
  [1976] = {.entry = {.count = 1, .reusable = true}}, SHIFT(560),
  [1978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 2, 0, 56),
  [1980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(607),
  [1982] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 2, 0, 0),
  [1984] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 4, 0, 105),
  [1986] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1974),
  [1988] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1201),
  [1990] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_else_in_field_declaration_list, 1, 0, 0),
  [1992] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1938),
  [1994] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1702),
  [1996] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1974),
  [1999] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1201),
  [2002] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1938),
  [2005] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1702),
  [2008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1487),
  [2010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1398),
  [2012] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1364),
  [2014] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1948),
  [2017] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1206),
  [2020] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1941),
  [2023] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1722),
  [2026] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_in_field_declaration_list_repeat1, 2, 0, 0),
  [2028] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1948),
  [2030] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1206),
  [2032] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1941),
  [2034] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1722),
  [2036] = {.entry = {.count = 1, .reusable = true}}, SHIFT(842),
  [2038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1488),
  [2040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1435),
  [2042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1482),
  [2044] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1489),
  [2046] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_else_in_field_declaration_list, 2, 0, 0),
  [2048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1372),
  [2050] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1412),
  [2052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(823),
  [2054] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1442),
  [2056] = {.entry = {.count = 1, .reusable = true}}, SHIFT(435),
  [2058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(444),
  [2060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(439),
  [2062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(437),
  [2064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(436),
  [2066] = {.entry = {.count = 1, .reusable = true}}, SHIFT(440),
  [2068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(811),
  [2070] = {.entry = {.count = 1, .reusable = false}}, SHIFT(665),
  [2072] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concatenated_string, 2, 0, 0),
  [2074] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concatenated_string, 2, 0, 0),
  [2076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(31),
  [2078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(23),
  [2080] = {.entry = {.count = 1, .reusable = false}}, SHIFT(666),
  [2082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_concatenated_string, 3, 0, 0),
  [2084] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_concatenated_string, 3, 0, 0),
  [2086] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_concatenated_string_repeat1, 2, 0, 0), SHIFT_REPEAT(666),
  [2089] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_concatenated_string_repeat1, 2, 0, 0),
  [2091] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_concatenated_string_repeat1, 2, 0, 0),
  [2093] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_concatenated_string_repeat1, 2, 0, 0), SHIFT_REPEAT(1525),
  [2096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(26),
  [2098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(37),
  [2100] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_literal, 2, 0, 0),
  [2102] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_literal, 2, 0, 0),
  [2104] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_string_literal, 3, 0, 0),
  [2106] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_string_literal, 3, 0, 0),
  [2108] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 9, 0, 129),
  [2110] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 9, 0, 129),
  [2112] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 7, 0, 120),
  [2114] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 7, 0, 120),
  [2116] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 7, 0, 119),
  [2118] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 7, 0, 119),
  [2120] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 6, 0, 109),
  [2122] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 6, 0, 109),
  [2124] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 4, 0, 58),
  [2126] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 4, 0, 58),
  [2128] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 8, 0, 125),
  [2130] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 8, 0, 125),
  [2132] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 8, 0, 124),
  [2134] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 8, 0, 124),
  [2136] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 6, 0, 108),
  [2138] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 6, 0, 108),
  [2140] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 5, 0, 88),
  [2142] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 5, 0, 88),
  [2144] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_gnu_asm_expression, 5, 0, 87),
  [2146] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_expression, 5, 0, 87),
  [2148] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0),
  [2150] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0),
  [2152] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(755),
  [2155] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(776),
  [2158] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(1971),
  [2161] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(1733),
  [2164] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__declaration_specifiers_repeat1, 2, 0, 0), SHIFT_REPEAT(1950),
  [2167] = {.entry = {.count = 1, .reusable = false}}, SHIFT(662),
  [2169] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__string, 1, 0, 0),
  [2171] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__string, 1, 0, 0),
  [2173] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [2175] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_expression, 3, 0, 0),
  [2177] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_subscript_expression, 4, 0, 69),
  [2179] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subscript_expression, 4, 0, 69),
  [2181] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_initializer_list, 2, 0, 0),
  [2183] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_initializer_list, 2, 0, 0),
  [2185] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_initializer_list, 3, 0, 0),
  [2187] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_initializer_list, 3, 0, 0),
  [2189] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__expression, 1, 0, 0),
  [2191] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__expression, 1, 0, 0),
  [2193] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_null, 1, 0, 0),
  [2195] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_null, 1, 0, 0),
  [2197] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_initializer_list, 5, 0, 0),
  [2199] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_initializer_list, 5, 0, 0),
  [2201] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_expression, 3, 0, 37),
  [2203] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_expression, 3, 0, 37),
  [2205] = {.entry = {.count = 1, .reusable = false}}, SHIFT(952),
  [2207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(889),
  [2209] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_initializer_list, 4, 0, 0),
  [2211] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_initializer_list, 4, 0, 0),
  [2213] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_compound_literal_expression, 4, 0, 45),
  [2215] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_compound_literal_expression, 4, 0, 45),
  [2217] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_alignof_expression, 4, 0, 57),
  [2219] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_alignof_expression, 4, 0, 57),
  [2221] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_expression, 9, 0, 0),
  [2223] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_expression, 9, 0, 0),
  [2225] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_char_literal, 3, 0, 0),
  [2227] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_char_literal, 3, 0, 0),
  [2229] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_generic_expression, 8, 0, 0),
  [2231] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_generic_expression, 8, 0, 0),
  [2233] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_update_expression, 2, 0, 12),
  [2235] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_update_expression, 2, 0, 12),
  [2237] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_call_expression, 2, 0, 13),
  [2239] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_call_expression, 2, 0, 13),
  [2241] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_offsetof_expression, 6, 0, 106),
  [2243] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_offsetof_expression, 6, 0, 106),
  [2245] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_cast_expression, 4, 0, 45),
  [2247] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_cast_expression, 4, 0, 45),
  [2249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(496),
  [2251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(618),
  [2253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(719),
  [2255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1867),
  [2257] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_pointer_expression, 2, 0, 5),
  [2259] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_expression, 2, 0, 5),
  [2261] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 3, 0, 0),
  [2263] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 3, 0, 0),
  [2265] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 5, 0, 0),
  [2267] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 5, 0, 0),
  [2269] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_binary_expression, 3, 0, 30),
  [2271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_binary_expression, 3, 0, 30),
  [2273] = {.entry = {.count = 1, .reusable = false}}, SHIFT(554),
  [2275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(554),
  [2277] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 4, 0, 0),
  [2279] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 4, 0, 0),
  [2281] = {.entry = {.count = 1, .reusable = false}}, SHIFT(958),
  [2283] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attribute_specifier, 4, 0, 0),
  [2285] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute_specifier, 4, 0, 0),
  [2287] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_argument_list, 2, 0, 0),
  [2289] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_argument_list, 2, 0, 0),
  [2291] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_update_expression, 2, 0, 5),
  [2293] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_update_expression, 2, 0, 5),
  [2295] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(841),
  [2298] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(755),
  [2301] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(776),
  [2304] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1971),
  [2307] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1733),
  [2310] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1950),
  [2313] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0),
  [2315] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(843),
  [2318] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(818),
  [2321] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1580),
  [2324] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1434),
  [2327] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__old_style_function_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1498),
  [2330] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sizeof_expression, 2, 0, 9),
  [2332] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sizeof_expression, 2, 0, 9),
  [2334] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_unary_expression, 2, 0, 5),
  [2336] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_unary_expression, 2, 0, 5),
  [2338] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym__type_definition_type_repeat1, 2, 0, 0),
  [2340] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__type_definition_type_repeat1, 2, 0, 0),
  [2342] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__type_definition_type_repeat1, 2, 0, 0), SHIFT_REPEAT(750),
  [2345] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_type_qualifier, 1, 0, 0),
  [2347] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_qualifier, 1, 0, 0),
  [2349] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 3, 0, 25),
  [2351] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 3, 0, 25),
  [2353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(543),
  [2355] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 2, 0, 7),
  [2357] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 2, 0, 7),
  [2359] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_ms_declspec_modifier, 4, 0, 0),
  [2361] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ms_declspec_modifier, 4, 0, 0),
  [2363] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 4, 0, 53),
  [2365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 4, 0, 53),
  [2367] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 3, 0, 25),
  [2369] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 3, 0, 25),
  [2371] = {.entry = {.count = 1, .reusable = true}}, SHIFT(518),
  [2373] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 2, 0, 7),
  [2375] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 2, 0, 7),
  [2377] = {.entry = {.count = 1, .reusable = false}}, SHIFT(653),
  [2379] = {.entry = {.count = 1, .reusable = false}}, SHIFT(636),
  [2381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(636),
  [2383] = {.entry = {.count = 1, .reusable = false}}, SHIFT(587),
  [2385] = {.entry = {.count = 1, .reusable = false}}, SHIFT(597),
  [2387] = {.entry = {.count = 1, .reusable = true}}, SHIFT(597),
  [2389] = {.entry = {.count = 1, .reusable = false}}, SHIFT(598),
  [2391] = {.entry = {.count = 1, .reusable = true}}, SHIFT(595),
  [2393] = {.entry = {.count = 1, .reusable = false}}, SHIFT(594),
  [2395] = {.entry = {.count = 1, .reusable = false}}, SHIFT(593),
  [2397] = {.entry = {.count = 1, .reusable = false}}, SHIFT(592),
  [2399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(547),
  [2401] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_conditional_expression, 4, 0, 70),
  [2403] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_conditional_expression, 4, 0, 70),
  [2405] = {.entry = {.count = 1, .reusable = true}}, SHIFT(591),
  [2407] = {.entry = {.count = 1, .reusable = true}}, SHIFT(510),
  [2409] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_storage_class_specifier, 1, 0, 0),
  [2411] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_storage_class_specifier, 1, 0, 0),
  [2413] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_conditional_expression, 5, 0, 96),
  [2415] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_conditional_expression, 5, 0, 96),
  [2417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_assignment_expression, 3, 0, 30),
  [2419] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_assignment_expression, 3, 0, 30),
  [2421] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 4, 0, 52),
  [2423] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 4, 0, 52),
  [2425] = {.entry = {.count = 1, .reusable = false}}, SHIFT(641),
  [2427] = {.entry = {.count = 1, .reusable = false}}, SHIFT(643),
  [2429] = {.entry = {.count = 1, .reusable = false}}, SHIFT(648),
  [2431] = {.entry = {.count = 1, .reusable = true}}, SHIFT(649),
  [2433] = {.entry = {.count = 1, .reusable = false}}, SHIFT(650),
  [2435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(650),
  [2437] = {.entry = {.count = 1, .reusable = false}}, SHIFT(651),
  [2439] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_specifiers, 3, 0, 14),
  [2441] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_specifiers, 3, 0, 14),
  [2443] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 2, 0, 8),
  [2445] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 2, 0, 8),
  [2447] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 3, 0, 26),
  [2449] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 3, 0, 26),
  [2451] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 3, 0, 24),
  [2453] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 3, 0, 24),
  [2455] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_sized_type_specifier_repeat1, 2, 0, 0),
  [2457] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_sized_type_specifier_repeat1, 2, 0, 0),
  [2459] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_sized_type_specifier_repeat1, 2, 0, 0), SHIFT_REPEAT(785),
  [2462] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 5, 0, 80),
  [2464] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 5, 0, 80),
  [2466] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_specifiers, 2, 0, 14),
  [2468] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_specifiers, 2, 0, 14),
  [2470] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 2, 0, 8),
  [2472] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 2, 0, 8),
  [2474] = {.entry = {.count = 1, .reusable = true}}, SHIFT(645),
  [2476] = {.entry = {.count = 1, .reusable = false}}, SHIFT(646),
  [2478] = {.entry = {.count = 1, .reusable = false}}, SHIFT(647),
  [2480] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_specifiers, 2, 0, 3),
  [2482] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_specifiers, 2, 0, 3),
  [2484] = {.entry = {.count = 1, .reusable = true}}, SHIFT(644),
  [2486] = {.entry = {.count = 1, .reusable = true}}, SHIFT(503),
  [2488] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 4, 0, 54),
  [2490] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 4, 0, 54),
  [2492] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declaration_specifiers, 1, 0, 3),
  [2494] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_specifiers, 1, 0, 3),
  [2496] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 3, 0, 24),
  [2498] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 3, 0, 24),
  [2500] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 4, 0, 52),
  [2502] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 4, 0, 52),
  [2504] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 3, 0, 26),
  [2506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 3, 0, 26),
  [2508] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 3, 0, 7),
  [2510] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 3, 0, 7),
  [2512] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 3, 0, 7),
  [2514] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 3, 0, 7),
  [2516] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_macro_type_specifier, 4, -1, 59),
  [2518] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_macro_type_specifier, 4, -1, 59),
  [2520] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 4, 0, 26),
  [2522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 4, 0, 26),
  [2524] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, -1, 10),
  [2526] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sized_type_specifier, 2, -1, 10),
  [2528] = {.entry = {.count = 1, .reusable = false}}, SHIFT(785),
  [2530] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sized_type_specifier, 3, -1, 15),
  [2532] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sized_type_specifier, 3, -1, 15),
  [2534] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sized_type_specifier, 3, 0, 14),
  [2536] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sized_type_specifier, 3, 0, 14),
  [2538] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 5, 0, 52),
  [2540] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 5, 0, 52),
  [2542] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 6, 0, 80),
  [2544] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 6, 0, 80),
  [2546] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, 0, 14),
  [2548] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sized_type_specifier, 2, 0, 14),
  [2550] = {.entry = {.count = 1, .reusable = false}}, SHIFT(813),
  [2552] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 3, 0, 7),
  [2554] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 3, 0, 7),
  [2556] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_specifier, 1, 0, 0),
  [2558] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_specifier, 1, 0, 0),
  [2560] = {.entry = {.count = 1, .reusable = false}}, SHIFT(831),
  [2562] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 3, 0, 8),
  [2564] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 3, 0, 8),
  [2566] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 4, 0, 24),
  [2568] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 4, 0, 24),
  [2570] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 6, 0, 78),
  [2572] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 6, 0, 78),
  [2574] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 2, 0, 0),
  [2576] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 2, 0, 0),
  [2578] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 3, 0, 8),
  [2580] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 3, 0, 8),
  [2582] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 4, 0, 24),
  [2584] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 4, 0, 24),
  [2586] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 5, 0, 54),
  [2588] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 5, 0, 54),
  [2590] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator_list, 2, 0, 0),
  [2592] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator_list, 2, 0, 0),
  [2594] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 5, 0, 53),
  [2596] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 5, 0, 53),
  [2598] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 5, 0, 52),
  [2600] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 5, 0, 52),
  [2602] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 4, 0, 25),
  [2604] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 4, 0, 25),
  [2606] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, 0, 3),
  [2608] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sized_type_specifier, 2, 0, 3),
  [2610] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 3, 0, 8),
  [2612] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 3, 0, 8),
  [2614] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator_list, 3, 0, 0),
  [2616] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator_list, 3, 0, 0),
  [2618] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, -1, 15),
  [2620] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sized_type_specifier, 2, -1, 15),
  [2622] = {.entry = {.count = 1, .reusable = false}}, SHIFT(812),
  [2624] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, 0, 0), REDUCE(aux_sym_sized_type_specifier_repeat1, 2, 0, 0),
  [2627] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_sized_type_specifier, 2, 0, 0), REDUCE(aux_sym_sized_type_specifier_repeat1, 2, 0, 0),
  [2630] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_union_specifier, 4, 0, 26),
  [2632] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_union_specifier, 4, 0, 26),
  [2634] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator_list, 4, 0, 0),
  [2636] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator_list, 4, 0, 0),
  [2638] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_struct_specifier, 4, 0, 25),
  [2640] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_struct_specifier, 4, 0, 25),
  [2642] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 5, 0, 50),
  [2644] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 5, 0, 50),
  [2646] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type_specifier, 1, 0, 1), SHIFT(1038),
  [2649] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration_list, 3, 0, 0),
  [2651] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration_list, 3, 0, 0),
  [2653] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 1, 0, 0), SHIFT(834),
  [2656] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_sized_type_specifier, 1, 0, 0),
  [2658] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_sized_type_specifier, 1, 0, 0),
  [2660] = {.entry = {.count = 1, .reusable = false}}, SHIFT(835),
  [2662] = {.entry = {.count = 1, .reusable = false}}, SHIFT(816),
  [2664] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 4, 0, 24),
  [2666] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 4, 0, 24),
  [2668] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef_in_field_declaration_list, 5, 0, 73),
  [2670] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_field_declaration_list, 5, 0, 73),
  [2672] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_in_field_declaration_list, 5, 0, 40),
  [2674] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_field_declaration_list, 5, 0, 40),
  [2676] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef_in_field_declaration_list, 3, 0, 17),
  [2678] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_field_declaration_list, 3, 0, 17),
  [2680] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 3, 0, 42),
  [2682] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 3, 0, 42),
  [2684] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_in_field_declaration_list, 4, 0, 40),
  [2686] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_field_declaration_list, 4, 0, 40),
  [2688] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 4, 0, 79),
  [2690] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 4, 0, 79),
  [2692] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 3, 0, 79),
  [2694] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 3, 0, 79),
  [2696] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_in_field_declaration_list, 6, 0, 97),
  [2698] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_field_declaration_list, 6, 0, 97),
  [2700] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef_in_field_declaration_list, 4, 0, 41),
  [2702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_field_declaration_list, 4, 0, 41),
  [2704] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_field_declaration, 2, 0, 42),
  [2706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_declaration, 2, 0, 42),
  [2708] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_in_field_declaration_list, 5, 0, 72),
  [2710] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_field_declaration_list, 5, 0, 72),
  [2712] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef_in_field_declaration_list, 4, 0, 17),
  [2714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_field_declaration_list, 4, 0, 17),
  [2716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(356),
  [2718] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 2, 0, 7),
  [2720] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 2, 0, 7),
  [2722] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_enum_specifier, 2, 0, 7), SHIFT(1971),
  [2725] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1317),
  [2727] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1936),
  [2729] = {.entry = {.count = 1, .reusable = true}}, SHIFT(129),
  [2731] = {.entry = {.count = 1, .reusable = true}}, SHIFT(430),
  [2733] = {.entry = {.count = 1, .reusable = true}}, SHIFT(368),
  [2735] = {.entry = {.count = 1, .reusable = true}}, SHIFT(497),
  [2737] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1867),
  [2739] = {.entry = {.count = 1, .reusable = false}}, SHIFT(616),
  [2741] = {.entry = {.count = 1, .reusable = false}}, SHIFT(590),
  [2743] = {.entry = {.count = 1, .reusable = false}}, SHIFT(608),
  [2745] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1354),
  [2747] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_pointer_declarator, 1, 1, 0),
  [2749] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1109),
  [2751] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1108),
  [2753] = {.entry = {.count = 1, .reusable = true}}, SHIFT(472),
  [2755] = {.entry = {.count = 1, .reusable = false}}, SHIFT(609),
  [2757] = {.entry = {.count = 1, .reusable = true}}, SHIFT(609),
  [2759] = {.entry = {.count = 1, .reusable = true}}, SHIFT(610),
  [2761] = {.entry = {.count = 1, .reusable = false}}, SHIFT(611),
  [2763] = {.entry = {.count = 1, .reusable = false}}, SHIFT(612),
  [2765] = {.entry = {.count = 1, .reusable = false}}, SHIFT(613),
  [2767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(614),
  [2769] = {.entry = {.count = 1, .reusable = true}}, SHIFT(615),
  [2771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(524),
  [2773] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 4, 0, 50),
  [2775] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 4, 0, 50),
  [2777] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_enum_specifier, 4, 0, 50), SHIFT(1971),
  [2780] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_pointer_declarator, 2, 1, 0),
  [2782] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_parameter_list, 3, 0, 0), REDUCE(sym__old_style_parameter_list, 3, 0, 0),
  [2785] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 3, 0, 0),
  [2787] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__old_style_parameter_list, 3, 0, 0),
  [2789] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_parameter_list, 3, 0, 0), REDUCE(sym__old_style_parameter_list, 3, 0, 0),
  [2792] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 3, 0, 0),
  [2794] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_parameter_list, 2, 0, 0), REDUCE(sym__old_style_parameter_list, 2, 0, 0),
  [2797] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 2, 0, 0),
  [2799] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__old_style_parameter_list, 2, 0, 0),
  [2801] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_parameter_list, 2, 0, 0), REDUCE(sym__old_style_parameter_list, 2, 0, 0),
  [2804] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 2, 0, 0),
  [2806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1558),
  [2808] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 3, 0, 24),
  [2810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 3, 0, 24),
  [2812] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_enum_specifier, 3, 0, 24), SHIFT(1971),
  [2815] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 2, 0, 8),
  [2817] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 2, 0, 8),
  [2819] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_enum_specifier, 2, 0, 8), SHIFT(1971),
  [2822] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enum_specifier, 5, 0, 78),
  [2824] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enum_specifier, 5, 0, 78),
  [2826] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_enum_specifier, 5, 0, 78), SHIFT(1971),
  [2829] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1426),
  [2831] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1128),
  [2833] = {.entry = {.count = 1, .reusable = true}}, SHIFT(951),
  [2835] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1422),
  [2837] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1423),
  [2839] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type_specifier, 1, 0, 1), SHIFT(1648),
  [2842] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type_specifier, 1, 0, 1), SHIFT(978),
  [2845] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_ms_call_modifier, 1, 0, 0),
  [2847] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ms_call_modifier, 1, 0, 0),
  [2849] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__declarator, 1, 0, 0), REDUCE(sym__type_specifier, 1, 0, 1),
  [2852] = {.entry = {.count = 3, .reusable = true}}, REDUCE(sym__declarator, 1, 0, 0), REDUCE(sym__type_specifier, 1, 0, 1), SHIFT(1038),
  [2856] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__declarator, 1, 0, 0), REDUCE(sym__type_specifier, 1, 0, 1),
  [2859] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym__type_specifier, 1, 0, 1), REDUCE(aux_sym__old_style_parameter_list_repeat1, 2, 0, 0),
  [2862] = {.entry = {.count = 1, .reusable = false}}, SHIFT(548),
  [2864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(549),
  [2866] = {.entry = {.count = 1, .reusable = false}}, SHIFT(549),
  [2868] = {.entry = {.count = 1, .reusable = false}}, SHIFT(552),
  [2870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(553),
  [2872] = {.entry = {.count = 1, .reusable = false}}, SHIFT(555),
  [2874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(556),
  [2876] = {.entry = {.count = 1, .reusable = false}}, SHIFT(557),
  [2878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(557),
  [2880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(558),
  [2882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(550),
  [2884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(551),
  [2886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(507),
  [2888] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator, 3, 0, 77),
  [2890] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator, 3, 0, 77),
  [2892] = {.entry = {.count = 1, .reusable = false}}, SHIFT(574),
  [2894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(576),
  [2896] = {.entry = {.count = 1, .reusable = false}}, SHIFT(576),
  [2898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(604),
  [2900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(600),
  [2902] = {.entry = {.count = 1, .reusable = false}}, SHIFT(601),
  [2904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(601),
  [2906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(584),
  [2908] = {.entry = {.count = 1, .reusable = false}}, SHIFT(585),
  [2910] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1060),
  [2912] = {.entry = {.count = 1, .reusable = false}}, SHIFT(583),
  [2914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(580),
  [2916] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__old_style_parameter_list, 3, 0, 0),
  [2918] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1043),
  [2920] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1029),
  [2922] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__old_style_function_declarator, 2, 0, 34),
  [2924] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__old_style_function_declarator, 2, 0, 34),
  [2926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(577),
  [2928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(522),
  [2930] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__old_style_parameter_list, 4, 0, 0),
  [2932] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__old_style_parameter_list, 4, 0, 0),
  [2934] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1073),
  [2936] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_pointer_declarator, 3, 1, 0),
  [2938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(755),
  [2940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1109),
  [2942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1108),
  [2944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1269),
  [2946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1006),
  [2948] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_argument_list, 4, 0, 0),
  [2950] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_argument_list, 4, 0, 0),
  [2952] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1416),
  [2954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1274),
  [2956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1013),
  [2958] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_call_expression, 2, 0, 13),
  [2960] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_call_expression, 2, 0, 13),
  [2962] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_argument_list, 2, 0, 0),
  [2964] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_argument_list, 2, 0, 0),
  [2966] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_pointer_declarator_repeat1, 2, 0, 0),
  [2968] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_pointer_declarator_repeat1, 2, 0, 0),
  [2970] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_pointer_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1109),
  [2973] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_pointer_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1108),
  [2976] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_argument_list, 3, 0, 0),
  [2978] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_argument_list, 3, 0, 0),
  [2980] = {.entry = {.count = 1, .reusable = true}}, SHIFT(545),
  [2982] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_comma_expression, 3, 0, 44),
  [2984] = {.entry = {.count = 1, .reusable = true}}, SHIFT(498),
  [2986] = {.entry = {.count = 1, .reusable = true}}, SHIFT(729),
  [2988] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_bitfield_clause, 2, 0, 0),
  [2990] = {.entry = {.count = 1, .reusable = true}}, SHIFT(897),
  [2992] = {.entry = {.count = 1, .reusable = true}}, SHIFT(896),
  [2994] = {.entry = {.count = 1, .reusable = true}}, SHIFT(724),
  [2996] = {.entry = {.count = 1, .reusable = true}}, SHIFT(475),
  [2998] = {.entry = {.count = 1, .reusable = true}}, SHIFT(697),
  [3000] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1028),
  [3002] = {.entry = {.count = 1, .reusable = true}}, SHIFT(718),
  [3004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(526),
  [3006] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_generic_expression_repeat1, 4, 0, 0),
  [3008] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1604),
  [3010] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1235),
  [3012] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1223),
  [3014] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1265),
  [3016] = {.entry = {.count = 1, .reusable = false}}, SHIFT(635),
  [3018] = {.entry = {.count = 1, .reusable = true}}, SHIFT(634),
  [3020] = {.entry = {.count = 1, .reusable = false}}, SHIFT(634),
  [3022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(630),
  [3024] = {.entry = {.count = 1, .reusable = false}}, SHIFT(629),
  [3026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(628),
  [3028] = {.entry = {.count = 1, .reusable = false}}, SHIFT(626),
  [3030] = {.entry = {.count = 1, .reusable = true}}, SHIFT(626),
  [3032] = {.entry = {.count = 1, .reusable = true}}, SHIFT(623),
  [3034] = {.entry = {.count = 1, .reusable = true}}, SHIFT(564),
  [3036] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 4, 0, 102),
  [3038] = {.entry = {.count = 1, .reusable = true}}, SHIFT(249),
  [3040] = {.entry = {.count = 1, .reusable = true}}, SHIFT(202),
  [3042] = {.entry = {.count = 1, .reusable = true}}, SHIFT(520),
  [3044] = {.entry = {.count = 1, .reusable = false}}, SHIFT(631),
  [3046] = {.entry = {.count = 1, .reusable = true}}, SHIFT(98),
  [3048] = {.entry = {.count = 1, .reusable = true}}, SHIFT(561),
  [3050] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_init_declarator, 3, 0, 63),
  [3052] = {.entry = {.count = 1, .reusable = true}}, SHIFT(620),
  [3054] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 3, 0, 83),
  [3056] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2, 0, 0),
  [3058] = {.entry = {.count = 1, .reusable = true}}, SHIFT(633),
  [3060] = {.entry = {.count = 1, .reusable = true}}, SHIFT(632),
  [3062] = {.entry = {.count = 1, .reusable = true}}, SHIFT(513),
  [3064] = {.entry = {.count = 1, .reusable = true}}, SHIFT(617),
  [3066] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_initializer_pair, 3, 0, 115),
  [3068] = {.entry = {.count = 1, .reusable = true}}, SHIFT(515),
  [3070] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 5, 0, 116),
  [3072] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 3, 0, 82),
  [3074] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 4, 0, 104),
  [3076] = {.entry = {.count = 1, .reusable = true}}, SHIFT(214),
  [3078] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__for_statement_body, 4, 0, 103),
  [3080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(684),
  [3082] = {.entry = {.count = 1, .reusable = true}}, SHIFT(259),
  [3084] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 3, 0, 0),
  [3086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(575),
  [3088] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0),
  [3090] = {.entry = {.count = 1, .reusable = true}}, SHIFT(606),
  [3092] = {.entry = {.count = 1, .reusable = true}}, SHIFT(559),
  [3094] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1591),
  [3096] = {.entry = {.count = 1, .reusable = true}}, SHIFT(244),
  [3098] = {.entry = {.count = 1, .reusable = true}}, SHIFT(292),
  [3100] = {.entry = {.count = 1, .reusable = true}}, SHIFT(517),
  [3102] = {.entry = {.count = 1, .reusable = true}}, SHIFT(685),
  [3104] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_initializer_pair, 3, 0, 114),
  [3106] = {.entry = {.count = 1, .reusable = true}}, SHIFT(105),
  [3108] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1072),
  [3110] = {.entry = {.count = 1, .reusable = true}}, SHIFT(55),
  [3112] = {.entry = {.count = 1, .reusable = true}}, SHIFT(45),
  [3114] = {.entry = {.count = 1, .reusable = true}}, SHIFT(68),
  [3116] = {.entry = {.count = 1, .reusable = true}}, SHIFT(61),
  [3118] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1665),
  [3120] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1632),
  [3122] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1656),
  [3124] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_ms_unaligned_ptr_modifier, 1, 0, 0),
  [3126] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ms_unaligned_ptr_modifier, 1, 0, 0),
  [3128] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_ms_pointer_modifier, 1, 0, 0),
  [3130] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ms_pointer_modifier, 1, 0, 0),
  [3132] = {.entry = {.count = 1, .reusable = true}}, SHIFT(56),
  [3134] = {.entry = {.count = 1, .reusable = true}}, SHIFT(692),
  [3136] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym__type_definition_type_repeat1, 2, 0, 0), SHIFT_REPEAT(755),
  [3139] = {.entry = {.count = 1, .reusable = true}}, SHIFT(416),
  [3141] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1967),
  [3143] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_descriptor, 2, 0, 14),
  [3145] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_descriptor, 1, 0, 3),
  [3147] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_descriptor, 3, 0, 14),
  [3149] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_descriptor, 2, 0, 3),
  [3151] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1374),
  [3153] = {.entry = {.count = 1, .reusable = false}}, SHIFT(834),
  [3155] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1133),
  [3157] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__preproc_expression, 1, 0, 0),
  [3159] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1146),
  [3161] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__preproc_expression, 1, 0, 0),
  [3163] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1140),
  [3165] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1014),
  [3167] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1179),
  [3169] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1710),
  [3171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1191),
  [3173] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1191),
  [3175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1148),
  [3177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1611),
  [3179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1205),
  [3181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1017),
  [3183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1204),
  [3185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1203),
  [3187] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1203),
  [3189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1190),
  [3191] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1217),
  [3193] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1214),
  [3195] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1213),
  [3197] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1212),
  [3199] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1210),
  [3201] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1209),
  [3203] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1209),
  [3205] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1208),
  [3207] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1241),
  [3209] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1152),
  [3211] = {.entry = {.count = 1, .reusable = false}}, SHIFT_EXTRA(),
  [3213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1268),
  [3215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1150),
  [3217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2004),
  [3219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2005),
  [3221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1157),
  [3223] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_argument_list_repeat1, 2, 0, 0),
  [3225] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1151),
  [3227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1194),
  [3229] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1736),
  [3231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1195),
  [3233] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1195),
  [3235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1255),
  [3237] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1680),
  [3239] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_binary_expression, 3, 0, 30),
  [3241] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_binary_expression, 3, 0, 30),
  [3243] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1242),
  [3245] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1227),
  [3247] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1266),
  [3249] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1248),
  [3251] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_definition_type, 1, 0, 3),
  [3253] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_definition_type, 1, 0, 3),
  [3255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1252),
  [3257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1221),
  [3259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1254),
  [3261] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_parenthesized_expression, 3, 0, 0),
  [3263] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_parenthesized_expression, 3, 0, 0),
  [3265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1257),
  [3267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1264),
  [3269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1247),
  [3271] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_declaration, 1, 0, 42),
  [3273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1244),
  [3275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1238),
  [3277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1231),
  [3279] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1256),
  [3281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1230),
  [3283] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1170),
  [3285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1207),
  [3287] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_defined, 4, 0, 0),
  [3289] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_defined, 4, 0, 0),
  [3291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1225),
  [3293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1237),
  [3295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1246),
  [3297] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_definition_type, 2, 0, 3),
  [3299] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_definition_type, 2, 0, 3),
  [3301] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_definition_type, 2, 0, 14),
  [3303] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_definition_type, 2, 0, 14),
  [3305] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1243),
  [3307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1250),
  [3309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1240),
  [3311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1234),
  [3313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1174),
  [3315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1175),
  [3317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1160),
  [3319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1232),
  [3321] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_unary_expression, 2, 0, 5),
  [3323] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_unary_expression, 2, 0, 5),
  [3325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1162),
  [3327] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1163),
  [3329] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1164),
  [3331] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_defined, 2, 0, 0),
  [3333] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_defined, 2, 0, 0),
  [3335] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1165),
  [3337] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1166),
  [3339] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1168),
  [3341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1267),
  [3343] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1233),
  [3345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1169),
  [3347] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_definition_type, 3, 0, 14),
  [3349] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_definition_type, 3, 0, 14),
  [3351] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1262),
  [3353] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1171),
  [3355] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1172),
  [3357] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1181),
  [3359] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1182),
  [3361] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1183),
  [3363] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1185),
  [3365] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1186),
  [3367] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 1, 0, 0), SHIFT(1260),
  [3370] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 1, 0, 0), SHIFT(1261),
  [3373] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 1, 0, 0), SHIFT(1263),
  [3376] = {.entry = {.count = 1, .reusable = true}}, SHIFT(12),
  [3378] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1173),
  [3380] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1177),
  [3382] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1178),
  [3384] = {.entry = {.count = 1, .reusable = true}}, SHIFT(482),
  [3386] = {.entry = {.count = 1, .reusable = true}}, SHIFT(18),
  [3388] = {.entry = {.count = 1, .reusable = true}}, SHIFT(478),
  [3390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1326),
  [3392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(10),
  [3394] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__type_specifier, 1, 0, 1), SHIFT(1249),
  [3397] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1236),
  [3399] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, 0, 3), SHIFT(785),
  [3402] = {.entry = {.count = 1, .reusable = true}}, SHIFT(489),
  [3404] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1331),
  [3406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1345),
  [3408] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, -1, 10), SHIFT(785),
  [3411] = {.entry = {.count = 1, .reusable = true}}, SHIFT(15),
  [3413] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 3, -1, 15), SHIFT(785),
  [3416] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1180),
  [3418] = {.entry = {.count = 1, .reusable = true}}, SHIFT(490),
  [3420] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1290),
  [3422] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 3, 0, 14), SHIFT(785),
  [3425] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, -1, 15), SHIFT(1253),
  [3428] = {.entry = {.count = 3, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, 0, 0), REDUCE(aux_sym_sized_type_specifier_repeat1, 2, 0, 0), SHIFT(785),
  [3432] = {.entry = {.count = 1, .reusable = true}}, SHIFT(20),
  [3434] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym_sized_type_specifier, 2, 0, 14), SHIFT(1258),
  [3437] = {.entry = {.count = 2, .reusable = false}}, REDUCE(sym__type_specifier, 1, 0, 0), SHIFT(1239),
  [3440] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1294),
  [3442] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1971),
  [3444] = {.entry = {.count = 2, .reusable = true}}, REDUCE(sym_enum_specifier, 2, 0, 7), SHIFT(1967),
  [3447] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1316),
  [3449] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_declaration_declarator, 2, 0, 34),
  [3451] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declarator, 2, 0, 34),
  [3453] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declarator, 2, 0, 34),
  [3455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(860),
  [3457] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1413),
  [3459] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_enumerator_list, 3, 0, 40),
  [3461] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1493),
  [3463] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1188),
  [3465] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1901),
  [3467] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_declaration_declarator, 3, 0, 34),
  [3469] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declarator, 3, 0, 34),
  [3471] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declarator, 3, 0, 34),
  [3473] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__function_declaration_declarator, 3, 0, 34),
  [3475] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_function_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1316),
  [3478] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_function_declarator_repeat1, 2, 0, 0),
  [3480] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_function_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1971),
  [3483] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_function_declarator_repeat1, 2, 0, 0),
  [3485] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_declarator, 4, 0, 34),
  [3487] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_declarator, 4, 0, 34),
  [3489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1455),
  [3491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(903),
  [3493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1479),
  [3495] = {.entry = {.count = 1, .reusable = true}}, SHIFT(939),
  [3497] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_enumerator_list, 2, 0, 17),
  [3499] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0),
  [3501] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0), SHIFT_REPEAT(1767),
  [3504] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_attributed_declarator_repeat1, 2, 0, 0),
  [3506] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_declarator, 2, 1, 32),
  [3508] = {.entry = {.count = 1, .reusable = true}}, SHIFT(689),
  [3510] = {.entry = {.count = 1, .reusable = false}}, SHIFT(465),
  [3512] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_declarator, 5, 1, 110),
  [3514] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1610),
  [3516] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1215),
  [3518] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1930),
  [3520] = {.entry = {.count = 1, .reusable = true}}, SHIFT(833),
  [3522] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_declarator, 4, 1, 89),
  [3524] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_function_declarator_repeat1, 1, 0, 0),
  [3526] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_function_declarator_repeat1, 1, 0, 0),
  [3528] = {.entry = {.count = 1, .reusable = true}}, SHIFT(827),
  [3530] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parameter_list, 4, 0, 0),
  [3532] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_list, 4, 0, 0),
  [3534] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_declarator, 3, 1, 60),
  [3536] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1337),
  [3538] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declaration_declarator, 1, 0, 22),
  [3540] = {.entry = {.count = 1, .reusable = false}}, SHIFT(468),
  [3542] = {.entry = {.count = 1, .reusable = true}}, SHIFT(642),
  [3544] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attributed_declarator, 2, 0, 0),
  [3546] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attributed_declarator, 2, 0, 0),
  [3548] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_enumerator_list_no_comma, 3, 0, 17),
  [3550] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1562),
  [3552] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1216),
  [3554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1965),
  [3556] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_enumerator_list_no_comma, 3, 0, 40),
  [3558] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_enumerator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1610),
  [3561] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enumerator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1167),
  [3564] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enumerator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1890),
  [3567] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_enumerator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1769),
  [3570] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_enumerator_list_repeat1, 2, 0, 0),
  [3572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1546),
  [3574] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1199),
  [3576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1961),
  [3578] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_enumerator_list_no_comma, 2, 0, 17),
  [3580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1478),
  [3582] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_field_declarator, 4, 1, 89),
  [3584] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_field_declarator, 2, 1, 32),
  [3586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1453),
  [3588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1933),
  [3590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1467),
  [3592] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_definition_repeat1, 1, 0, 0),
  [3594] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_type_definition_repeat1, 1, 0, 0), REDUCE(aux_sym_function_declarator_repeat1, 1, 0, 0),
  [3597] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_type_definition_repeat1, 1, 0, 0),
  [3599] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__field_declaration_declarator_repeat1, 2, 0, 32),
  [3601] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1457),
  [3603] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_clobber_list, 1, 0, 0),
  [3605] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_field_declarator, 3, 1, 60),
  [3607] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1964),
  [3609] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_enumerator_list_no_comma, 4, 0, 40),
  [3611] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_enumerator_list, 3, 0, 17),
  [3613] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_field_declarator, 5, 1, 110),
  [3615] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_enumerator_list, 4, 0, 40),
  [3617] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1448),
  [3619] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declarator, 1, 0, 0),
  [3621] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__declarator, 1, 0, 0),
  [3623] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_type_declarator, 3, 1, 60),
  [3625] = {.entry = {.count = 1, .reusable = false}}, SHIFT(470),
  [3627] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_declarator, 3, 0, 22),
  [3629] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_declarator, 3, 0, 22),
  [3631] = {.entry = {.count = 1, .reusable = true}}, SHIFT(706),
  [3633] = {.entry = {.count = 1, .reusable = true}}, SHIFT(499),
  [3635] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1287),
  [3637] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_definition_declarators, 1, 0, 22),
  [3639] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_type_declarator, 4, 1, 89),
  [3641] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_type_declarator, 5, 1, 110),
  [3643] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_declarator, 4, 0, 22),
  [3645] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_declarator, 4, 0, 22),
  [3647] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_declarator, 4, 0, 92),
  [3649] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_declarator, 4, 0, 92),
  [3651] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_output_operand_list, 1, 0, 0),
  [3653] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1818),
  [3655] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_pointer_type_declarator, 2, 1, 32),
  [3657] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_declarator, 3, -10, 0),
  [3659] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_declarator, 3, -10, 0),
  [3661] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attributed_field_declarator, 2, 0, 0),
  [3663] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attributed_field_declarator, 2, 0, 0),
  [3665] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_input_operand_list, 1, 0, 0),
  [3667] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1991),
  [3669] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_declarator, 4, -10, 0),
  [3671] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_declarator, 4, -10, 0),
  [3673] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_declarator, 5, 0, 112),
  [3675] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_declarator, 5, 0, 112),
  [3677] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_declarator, 1, 0, 0),
  [3679] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1624),
  [3681] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1528),
  [3683] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1908),
  [3685] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1612),
  [3687] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1931),
  [3689] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__type_definition_declarators_repeat1, 2, 0, 32),
  [3691] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attributed_type_declarator, 2, 0, 0),
  [3693] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_attributed_type_declarator, 2, 0, 0),
  [3695] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1654),
  [3697] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1875),
  [3699] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1614),
  [3701] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1812),
  [3703] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_field_declarator, 4, -10, 0),
  [3705] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_field_declarator, 4, -10, 0),
  [3707] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_field_declarator, 2, 0, 34),
  [3709] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_field_declarator, 2, 0, 34),
  [3711] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parameter_declaration, 2, 0, 35),
  [3713] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_field_declarator, 3, 0, 22),
  [3715] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_field_declarator, 3, 0, 22),
  [3717] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_field_declarator, 4, 0, 92),
  [3719] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_field_declarator, 4, 0, 92),
  [3721] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declarator, 1, 0, 0),
  [3723] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__field_declarator, 1, 0, 0),
  [3725] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_field_declarator, 4, 0, 22),
  [3727] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_field_declarator, 4, 0, 22),
  [3729] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_in_enumerator_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1610),
  [3732] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_in_enumerator_list_repeat1, 2, 0, 0),
  [3734] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_enumerator_list_repeat1, 2, 0, 0),
  [3736] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1, 2, 0, 0), SHIFT_REPEAT(1413),
  [3739] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1, 2, 0, 0),
  [3741] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1, 2, 0, 0),
  [3743] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_field_declarator, 5, 0, 112),
  [3745] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_field_declarator, 5, 0, 112),
  [3747] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_enumerator, 1, 0, 6),
  [3749] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_enumerator, 1, 0, 6),
  [3751] = {.entry = {.count = 1, .reusable = true}}, SHIFT(546),
  [3753] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_field_declarator, 3, -10, 0),
  [3755] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_field_declarator, 3, -10, 0),
  [3757] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declarator, 1, 0, 51),
  [3759] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__field_declarator, 1, 0, 51),
  [3761] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_type_declarator, 4, 0, 22),
  [3763] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_type_declarator, 4, 0, 22),
  [3765] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1, 1, 0, 0),
  [3767] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1465),
  [3769] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_preproc_if_in_enumerator_list_no_comma_repeat1, 1, 0, 0),
  [3771] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1367),
  [3773] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_declarator, 1, 0, 21),
  [3775] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_declarator, 1, 0, 21),
  [3777] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_declarator, 1, 0, 0),
  [3779] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_declarator, 1, 0, 0),
  [3781] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1371),
  [3783] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_declarator, 1, 0, 1),
  [3785] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym__type_declarator, 1, 0, 1),
  [3787] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1415),
  [3789] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_function_type_declarator, 2, 0, 34),
  [3791] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_function_type_declarator, 2, 0, 34),
  [3793] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1392),
  [3795] = {.entry = {.count = 1, .reusable = true}}, SHIFT(516),
  [3797] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1970),
  [3799] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1441),
  [3801] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_type_declarator, 3, -10, 0),
  [3803] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_type_declarator, 3, -10, 0),
  [3805] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_type_definition_repeat1, 2, 0, 0),
  [3807] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_type_definition_repeat1, 2, 0, 0), SHIFT_REPEAT(1971),
  [3810] = {.entry = {.count = 1, .reusable = false}}, SHIFT(752),
  [3812] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_type_declarator, 4, 0, 92),
  [3814] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_type_declarator, 4, 0, 92),
  [3816] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__function_declaration_declarator, 4, 0, 34),
  [3818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1432),
  [3820] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_type_declarator, 3, 0, 22),
  [3822] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_type_declarator, 3, 0, 22),
  [3824] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_pair_repeat1, 2, 0, 0), SHIFT_REPEAT(588),
  [3827] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_initializer_pair_repeat1, 2, 0, 0),
  [3829] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_pair_repeat1, 2, 0, 0), SHIFT_REPEAT(1970),
  [3832] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_parenthesized_type_declarator, 4, -10, 0),
  [3834] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_parenthesized_type_declarator, 4, -10, 0),
  [3836] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_array_type_declarator, 5, 0, 112),
  [3838] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_array_type_declarator, 5, 0, 112),
  [3840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1380),
  [3842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1524),
  [3844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1149),
  [3846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(247),
  [3848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1463),
  [3850] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_gnu_asm_expression_repeat1, 2, 0, 0),
  [3852] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_gnu_asm_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1524),
  [3855] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_pointer_declarator, 4, 1, 89),
  [3857] = {.entry = {.count = 1, .reusable = true}}, SHIFT(471),
  [3859] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_descriptor, 2, 0, 19),
  [3861] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_enumerator_list, 3, 0, 17),
  [3863] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef_in_enumerator_list, 3, 0, 17),
  [3865] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_descriptor, 3, 0, 43),
  [3867] = {.entry = {.count = 1, .reusable = true}}, SHIFT(264),
  [3869] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_enumerator_list, 5, 0, 40),
  [3871] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_in_enumerator_list, 5, 0, 40),
  [3873] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_enumerator_list, 5, 0, 72),
  [3875] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_in_enumerator_list, 5, 0, 72),
  [3877] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_enumerator_list, 4, 0, 40),
  [3879] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_in_enumerator_list, 4, 0, 40),
  [3881] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_enumerator_list_repeat1, 2, 0, 0),
  [3883] = {.entry = {.count = 1, .reusable = true}}, SHIFT(251),
  [3885] = {.entry = {.count = 1, .reusable = true}}, SHIFT(194),
  [3887] = {.entry = {.count = 1, .reusable = true}}, SHIFT(988),
  [3889] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_enumerator_list, 5, 0, 73),
  [3891] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef_in_enumerator_list, 5, 0, 73),
  [3893] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1387),
  [3895] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_enumerator_list, 4, 0, 41),
  [3897] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef_in_enumerator_list, 4, 0, 41),
  [3899] = {.entry = {.count = 1, .reusable = true}}, SHIFT(307),
  [3901] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_enumerator_list, 4, 0, 17),
  [3903] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_ifdef_in_enumerator_list, 4, 0, 17),
  [3905] = {.entry = {.count = 1, .reusable = true}}, SHIFT(95),
  [3907] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_descriptor, 3, 0, 46),
  [3909] = {.entry = {.count = 1, .reusable = true}}, SHIFT(270),
  [3911] = {.entry = {.count = 1, .reusable = true}}, SHIFT(89),
  [3913] = {.entry = {.count = 1, .reusable = true}}, SHIFT(944),
  [3915] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_type_descriptor, 4, 0, 75),
  [3917] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_pointer_declarator, 2, 1, 32),
  [3919] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_pointer_declarator, 3, 1, 60),
  [3921] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_enumerator_list, 6, 0, 97),
  [3923] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_if_in_enumerator_list, 6, 0, 97),
  [3925] = {.entry = {.count = 1, .reusable = true}}, SHIFT(945),
  [3927] = {.entry = {.count = 1, .reusable = true}}, SHIFT(994),
  [3929] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 2, 0, 0),
  [3931] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 3, 0, 74),
  [3933] = {.entry = {.count = 1, .reusable = false}}, SHIFT(757),
  [3935] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 3, 0, 22),
  [3937] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 1, 0, 6),
  [3939] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1916),
  [3941] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_function_declarator, 1, 0, 20),
  [3943] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 4, 0, 92),
  [3945] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 5, 0, 112),
  [3947] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 4, 0, 98),
  [3949] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_function_declarator, 2, 0, 34),
  [3951] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 3, 0, 0),
  [3953] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_enumerator_list, 1, 0, 0),
  [3955] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 3, 0, 93),
  [3957] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_parenthesized_declarator, 3, 0, 0),
  [3959] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__abstract_declarator, 1, 0, 0),
  [3961] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_parenthesized_declarator, 4, 0, 0),
  [3963] = {.entry = {.count = 1, .reusable = false}}, SHIFT(762),
  [3965] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 2, 0, 32),
  [3967] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_abstract_array_declarator, 4, 0, 22),
  [3969] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__declaration_declarator, 1, 0, 11),
  [3971] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1222),
  [3973] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1560),
  [3975] = {.entry = {.count = 1, .reusable = true}}, SHIFT(236),
  [3977] = {.entry = {.count = 1, .reusable = true}}, SHIFT(173),
  [3979] = {.entry = {.count = 1, .reusable = true}}, SHIFT(174),
  [3981] = {.entry = {.count = 1, .reusable = true}}, SHIFT(730),
  [3983] = {.entry = {.count = 1, .reusable = true}}, SHIFT(888),
  [3985] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1772),
  [3987] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1738),
  [3989] = {.entry = {.count = 1, .reusable = true}}, SHIFT(193),
  [3991] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1382),
  [3993] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_clobber_list, 2, 0, 118),
  [3995] = {.entry = {.count = 1, .reusable = true}}, SHIFT(274),
  [3997] = {.entry = {.count = 1, .reusable = true}}, SHIFT(275),
  [3999] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__field_declaration_declarator_repeat1, 2, 0, 65), SHIFT_REPEAT(1337),
  [4002] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__field_declaration_declarator_repeat1, 2, 0, 65),
  [4004] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1408),
  [4006] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_input_operand_list, 3, 0, 107),
  [4008] = {.entry = {.count = 1, .reusable = true}}, SHIFT(365),
  [4010] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1638),
  [4012] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1800),
  [4014] = {.entry = {.count = 1, .reusable = false}}, SHIFT(671),
  [4016] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1557),
  [4018] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1557),
  [4020] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declaration_declarator, 3, 0, 101),
  [4022] = {.entry = {.count = 1, .reusable = true}}, SHIFT(940),
  [4024] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1795),
  [4026] = {.entry = {.count = 1, .reusable = true}}, SHIFT(846),
  [4028] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1819),
  [4030] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_gnu_asm_output_operand_list_repeat1, 2, 0, 117), SHIFT_REPEAT(1418),
  [4033] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_gnu_asm_output_operand_list_repeat1, 2, 0, 117),
  [4035] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1418),
  [4037] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_output_operand_list, 3, 0, 107),
  [4039] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_output_operand_list, 2, 0, 86),
  [4041] = {.entry = {.count = 1, .reusable = false}}, SHIFT(717),
  [4043] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1495),
  [4045] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_qualifier, 1, 0, 0),
  [4047] = {.entry = {.count = 1, .reusable = false}}, SHIFT(670),
  [4049] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1515),
  [4051] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1515),
  [4053] = {.entry = {.count = 1, .reusable = true}}, SHIFT(933),
  [4055] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1833),
  [4057] = {.entry = {.count = 1, .reusable = true}}, SHIFT(90),
  [4059] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1910),
  [4061] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1564),
  [4063] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1564),
  [4065] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_input_operand_list, 2, 0, 86),
  [4067] = {.entry = {.count = 1, .reusable = true}}, SHIFT(114),
  [4069] = {.entry = {.count = 1, .reusable = true}}, SHIFT(113),
  [4071] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_gnu_asm_input_operand_list_repeat1, 2, 0, 117), SHIFT_REPEAT(1408),
  [4074] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_gnu_asm_input_operand_list_repeat1, 2, 0, 117),
  [4076] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_clobber_list, 3, 0, 122),
  [4078] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1497),
  [4080] = {.entry = {.count = 1, .reusable = true}}, SHIFT(303),
  [4082] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declaration_declarator, 2, 0, 48),
  [4084] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__field_declaration_declarator, 2, 0, 22),
  [4086] = {.entry = {.count = 1, .reusable = true}}, SHIFT(375),
  [4088] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1952),
  [4090] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_gnu_asm_clobber_list_repeat1, 2, 0, 127), SHIFT_REPEAT(1382),
  [4093] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_gnu_asm_clobber_list_repeat1, 2, 0, 127),
  [4095] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1717),
  [4097] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1750),
  [4099] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1771),
  [4101] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1770),
  [4103] = {.entry = {.count = 1, .reusable = true}}, SHIFT(84),
  [4105] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym__type_definition_declarators, 2, 0, 48),
  [4107] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1197),
  [4109] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1742),
  [4111] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1774),
  [4113] = {.entry = {.count = 1, .reusable = true}}, SHIFT(267),
  [4115] = {.entry = {.count = 1, .reusable = true}}, SHIFT(425),
  [4117] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1891),
  [4119] = {.entry = {.count = 1, .reusable = true}}, SHIFT(144),
  [4121] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1897),
  [4123] = {.entry = {.count = 1, .reusable = true}}, SHIFT(256),
  [4125] = {.entry = {.count = 1, .reusable = true}}, SHIFT(222),
  [4127] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__type_definition_declarators_repeat1, 2, 0, 65), SHIFT_REPEAT(1287),
  [4130] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__type_definition_declarators_repeat1, 2, 0, 65),
  [4132] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_enumerator_list_no_comma, 2, 0, 0),
  [4134] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_enumerator_list, 2, 0, 0),
  [4136] = {.entry = {.count = 1, .reusable = true}}, SHIFT(180),
  [4138] = {.entry = {.count = 1, .reusable = false}}, SHIFT(892),
  [4140] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0),
  [4142] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0), SHIFT_REPEAT(1557),
  [4145] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_string_literal_repeat1, 2, 0, 0), SHIFT_REPEAT(1557),
  [4148] = {.entry = {.count = 1, .reusable = false}}, SHIFT(893),
  [4150] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1556),
  [4152] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1556),
  [4154] = {.entry = {.count = 1, .reusable = true}}, SHIFT(175),
  [4156] = {.entry = {.count = 1, .reusable = false}}, REDUCE(aux_sym_char_literal_repeat1, 2, 0, 0),
  [4158] = {.entry = {.count = 2, .reusable = false}}, REDUCE(aux_sym_char_literal_repeat1, 2, 0, 0), SHIFT_REPEAT(1560),
  [4161] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1756),
  [4163] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1697),
  [4165] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_else_in_enumerator_list_no_comma, 1, 0, 0),
  [4167] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 3, 0, 49),
  [4169] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1975),
  [4171] = {.entry = {.count = 1, .reusable = true}}, SHIFT(257),
  [4173] = {.entry = {.count = 1, .reusable = true}}, SHIFT(253),
  [4175] = {.entry = {.count = 1, .reusable = true}}, SHIFT(894),
  [4177] = {.entry = {.count = 1, .reusable = true}}, SHIFT(273),
  [4179] = {.entry = {.count = 1, .reusable = true}}, SHIFT(725),
  [4181] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1318),
  [4183] = {.entry = {.count = 1, .reusable = true}}, SHIFT(219),
  [4185] = {.entry = {.count = 1, .reusable = true}}, SHIFT(177),
  [4187] = {.entry = {.count = 1, .reusable = true}}, SHIFT(178),
  [4189] = {.entry = {.count = 1, .reusable = true}}, SHIFT(948),
  [4191] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym__old_style_parameter_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1648),
  [4194] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__old_style_parameter_list_repeat1, 2, 0, 0),
  [4196] = {.entry = {.count = 1, .reusable = true}}, SHIFT(272),
  [4198] = {.entry = {.count = 1, .reusable = true}}, SHIFT(271),
  [4200] = {.entry = {.count = 1, .reusable = true}}, SHIFT(949),
  [4202] = {.entry = {.count = 1, .reusable = true}}, SHIFT(941),
  [4204] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 2, 0, 65), SHIFT_REPEAT(1149),
  [4207] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 2, 0, 65),
  [4209] = {.entry = {.count = 1, .reusable = true}}, SHIFT(864),
  [4211] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1731),
  [4213] = {.entry = {.count = 1, .reusable = true}}, SHIFT(687),
  [4215] = {.entry = {.count = 1, .reusable = true}}, SHIFT(676),
  [4217] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1365),
  [4219] = {.entry = {.count = 1, .reusable = true}}, SHIFT(943),
  [4221] = {.entry = {.count = 1, .reusable = true}}, SHIFT(942),
  [4223] = {.entry = {.count = 1, .reusable = true}}, SHIFT(950),
  [4225] = {.entry = {.count = 1, .reusable = true}}, SHIFT(757),
  [4227] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1749),
  [4229] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1747),
  [4231] = {.entry = {.count = 1, .reusable = true}}, SHIFT(947),
  [4233] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subscript_designator, 3, 0, 0),
  [4235] = {.entry = {.count = 1, .reusable = true}}, SHIFT(217),
  [4237] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_attribute_declaration_repeat1, 2, 0, 0), SHIFT_REPEAT(1731),
  [4240] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_attribute_declaration_repeat1, 2, 0, 0),
  [4242] = {.entry = {.count = 1, .reusable = true}}, SHIFT(946),
  [4244] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1862),
  [4246] = {.entry = {.count = 1, .reusable = true}}, SHIFT(991),
  [4248] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_initializer_list_repeat1, 2, 0, 0), SHIFT_REPEAT(477),
  [4251] = {.entry = {.count = 1, .reusable = true}}, SHIFT(677),
  [4253] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1752),
  [4255] = {.entry = {.count = 1, .reusable = true}}, SHIFT(756),
  [4257] = {.entry = {.count = 1, .reusable = true}}, SHIFT(218),
  [4259] = {.entry = {.count = 1, .reusable = true}}, SHIFT(986),
  [4261] = {.entry = {.count = 1, .reusable = true}}, SHIFT(680),
  [4263] = {.entry = {.count = 1, .reusable = true}}, SHIFT(751),
  [4265] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1281),
  [4267] = {.entry = {.count = 1, .reusable = true}}, SHIFT(983),
  [4269] = {.entry = {.count = 1, .reusable = true}}, SHIFT(982),
  [4271] = {.entry = {.count = 1, .reusable = true}}, SHIFT(266),
  [4273] = {.entry = {.count = 1, .reusable = true}}, SHIFT(984),
  [4275] = {.entry = {.count = 1, .reusable = true}}, SHIFT(993),
  [4277] = {.entry = {.count = 1, .reusable = true}}, SHIFT(640),
  [4279] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1544),
  [4281] = {.entry = {.count = 1, .reusable = true}}, SHIFT(146),
  [4283] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1159),
  [4285] = {.entry = {.count = 1, .reusable = true}}, SHIFT(855),
  [4287] = {.entry = {.count = 1, .reusable = true}}, SHIFT(353),
  [4289] = {.entry = {.count = 1, .reusable = true}}, SHIFT(446),
  [4291] = {.entry = {.count = 1, .reusable = true}}, SHIFT(476),
  [4293] = {.entry = {.count = 1, .reusable = true}}, SHIFT(775),
  [4295] = {.entry = {.count = 1, .reusable = true}}, SHIFT(305),
  [4297] = {.entry = {.count = 1, .reusable = true}}, SHIFT(996),
  [4299] = {.entry = {.count = 1, .reusable = true}}, SHIFT(997),
  [4301] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1708),
  [4303] = {.entry = {.count = 1, .reusable = true}}, SHIFT(161),
  [4305] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_field_designator, 2, 0, 99),
  [4307] = {.entry = {.count = 1, .reusable = true}}, SHIFT(420),
  [4309] = {.entry = {.count = 1, .reusable = true}}, SHIFT(252),
  [4311] = {.entry = {.count = 1, .reusable = true}}, SHIFT(306),
  [4313] = {.entry = {.count = 1, .reusable = true}}, SHIFT(254),
  [4315] = {.entry = {.count = 1, .reusable = true}}, SHIFT(300),
  [4317] = {.entry = {.count = 1, .reusable = true}}, SHIFT(679),
  [4319] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1343),
  [4321] = {.entry = {.count = 1, .reusable = true}}, SHIFT(675),
  [4323] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1370),
  [4325] = {.entry = {.count = 1, .reusable = true}}, SHIFT(78),
  [4327] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_input_operand, 7, 0, 131),
  [4329] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameter_list_repeat1, 2, 0, 0), SHIFT_REPEAT(725),
  [4332] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_parameter_list_repeat1, 2, 0, 0),
  [4334] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_output_operand, 7, 0, 131),
  [4336] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_gnu_asm_goto_list_repeat1, 2, 0, 130), SHIFT_REPEAT(1877),
  [4339] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_gnu_asm_goto_list_repeat1, 2, 0, 130),
  [4341] = {.entry = {.count = 1, .reusable = true}}, SHIFT(301),
  [4343] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_gnu_asm_output_operand_list_repeat1, 2, 0, 86),
  [4345] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1589),
  [4347] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1761),
  [4349] = {.entry = {.count = 1, .reusable = true}}, SHIFT(255),
  [4351] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym__field_declaration_declarator_repeat1, 3, 0, 32),
  [4353] = {.entry = {.count = 1, .reusable = true}}, SHIFT(686),
  [4355] = {.entry = {.count = 1, .reusable = true}}, SHIFT(773),
  [4357] = {.entry = {.count = 1, .reusable = true}}, SHIFT(172),
  [4359] = {.entry = {.count = 1, .reusable = true}}, SHIFT(681),
  [4361] = {.entry = {.count = 1, .reusable = true}}, SHIFT(233),
  [4363] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1877),
  [4365] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_goto_list, 3, 0, 128),
  [4367] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1713),
  [4369] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_gnu_asm_clobber_list_repeat1, 2, 0, 118),
  [4371] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1522),
  [4373] = {.entry = {.count = 1, .reusable = true}}, SHIFT(447),
  [4375] = {.entry = {.count = 1, .reusable = true}}, SHIFT(100),
  [4377] = {.entry = {.count = 1, .reusable = true}}, SHIFT(102),
  [4379] = {.entry = {.count = 1, .reusable = true}}, SHIFT(372),
  [4381] = {.entry = {.count = 1, .reusable = true}}, SHIFT(106),
  [4383] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_input_operand, 4, 0, 121),
  [4385] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_generic_expression_repeat1, 2, 0, 0), SHIFT_REPEAT(1028),
  [4388] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_generic_expression_repeat1, 2, 0, 0),
  [4390] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1648),
  [4392] = {.entry = {.count = 1, .reusable = true}}, SHIFT(995),
  [4394] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_params_repeat1, 2, 0, 0), SHIFT_REPEAT(1749),
  [4397] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_preproc_params_repeat1, 2, 0, 0),
  [4399] = {.entry = {.count = 1, .reusable = true}}, SHIFT(302),
  [4401] = {.entry = {.count = 1, .reusable = true}}, SHIFT(189),
  [4403] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_preproc_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(1205),
  [4406] = {.entry = {.count = 1, .reusable = true}}, SHIFT(901),
  [4408] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_subscript_range_designator, 5, 0, 126),
  [4410] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_argument_list_repeat1, 2, 0, 0), SHIFT_REPEAT(498),
  [4413] = {.entry = {.count = 1, .reusable = true}}, SHIFT(258),
  [4415] = {.entry = {.count = 1, .reusable = true}}, SHIFT(85),
  [4417] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_goto_list, 2, 0, 123),
  [4419] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1322),
  [4421] = {.entry = {.count = 1, .reusable = true}}, SHIFT(673),
  [4423] = {.entry = {.count = 1, .reusable = true}}, SHIFT(726),
  [4425] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1224),
  [4427] = {.entry = {.count = 1, .reusable = true}}, SHIFT(674),
  [4429] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_gnu_asm_input_operand_list_repeat1, 2, 0, 86),
  [4431] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_output_operand, 4, 0, 121),
  [4433] = {.entry = {.count = 1, .reusable = true}}, SHIFT(716),
  [4435] = {.entry = {.count = 1, .reusable = true}}, SHIFT(908),
  [4437] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1502),
  [4439] = {.entry = {.count = 1, .reusable = true}}, SHIFT(94),
  [4441] = {.entry = {.count = 1, .reusable = true}}, SHIFT(93),
  [4443] = {.entry = {.count = 1, .reusable = true}}, SHIFT(91),
  [4445] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1008),
  [4447] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1116),
  [4449] = {.entry = {.count = 1, .reusable = true}}, SHIFT(532),
  [4451] = {.entry = {.count = 1, .reusable = true}}, SHIFT(541),
  [4453] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1485),
  [4455] = {.entry = {.count = 1, .reusable = true}}, SHIFT(900),
  [4457] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1835),
  [4459] = {.entry = {.count = 1, .reusable = true}}, SHIFT(919),
  [4461] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1832),
  [4463] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_params, 4, 0, 0),
  [4465] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_params, 4, 0, 0),
  [4467] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1211),
  [4469] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1879),
  [4471] = {.entry = {.count = 1, .reusable = true}}, SHIFT(327),
  [4473] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1876),
  [4475] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_gnu_asm_goto_list_repeat1, 2, 0, 123),
  [4477] = {.entry = {.count = 1, .reusable = true}}, SHIFT(154),
  [4479] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1857),
  [4481] = {.entry = {.count = 1, .reusable = true}}, SHIFT(927),
  [4483] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1810),
  [4485] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_variadic_parameter, 1, 0, 0),
  [4487] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 4, 0, 49),
  [4489] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1456),
  [4491] = {.entry = {.count = 1, .reusable = true}}, SHIFT(837),
  [4493] = {.entry = {.count = 1, .reusable = true}}, SHIFT(850),
  [4495] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1825),
  [4497] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_attribute, 2, 0, 6),
  [4499] = {.entry = {.count = 1, .reusable = true}}, SHIFT(858),
  [4501] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1816),
  [4503] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1220),
  [4505] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1895),
  [4507] = {.entry = {.count = 1, .reusable = true}}, SHIFT(142),
  [4509] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1927),
  [4511] = {.entry = {.count = 1, .reusable = true}}, SHIFT(402),
  [4513] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1807),
  [4515] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_params, 3, 0, 0),
  [4517] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_params, 3, 0, 0),
  [4519] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1707),
  [4521] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1669),
  [4523] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_gnu_asm_goto_list, 1, 0, 0),
  [4525] = {.entry = {.count = 1, .reusable = true}}, SHIFT(935),
  [4527] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1783),
  [4529] = {.entry = {.count = 1, .reusable = true}}, SHIFT(364),
  [4531] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1796),
  [4533] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_params, 2, 0, 0),
  [4535] = {.entry = {.count = 1, .reusable = false}}, REDUCE(sym_preproc_params, 2, 0, 0),
  [4537] = {.entry = {.count = 1, .reusable = true}}, SHIFT(424),
  [4539] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2009),
  [4541] = {.entry = {.count = 1, .reusable = true}}, SHIFT(393),
  [4543] = {.entry = {.count = 1, .reusable = false}}, SHIFT(2013),
  [4545] = {.entry = {.count = 1, .reusable = true}}, REDUCE(aux_sym_declaration_repeat1, 4, 0, 113),
  [4547] = {.entry = {.count = 1, .reusable = true}}, SHIFT(405),
  [4549] = {.entry = {.count = 1, .reusable = false}}, SHIFT(1942),
  [4551] = {.entry = {.count = 2, .reusable = true}}, REDUCE(aux_sym_parameter_list_repeat1, 2, 0, 0), REDUCE(aux_sym__old_style_parameter_list_repeat1, 2, 0, 0),
  [4554] = {.entry = {.count = 1, .reusable = true}}, SHIFT(261),
  [4556] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_enumerator_list, 4, 0, 72),
  [4558] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_enumerator_list_no_comma, 6, 0, 97),
  [4560] = {.entry = {.count = 1, .reusable = true}}, SHIFT(72),
  [4562] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_enumerator_list, 4, 0, 73),
  [4564] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_enumerator_list_no_comma, 4, 0, 72),
  [4566] = {.entry = {.count = 1, .reusable = true}}, SHIFT(168),
  [4568] = {.entry = {.count = 1, .reusable = true}}, SHIFT(449),
  [4570] = {.entry = {.count = 1, .reusable = true}}, SHIFT(936),
  [4572] = {.entry = {.count = 1, .reusable = true}}, SHIFT(324),
  [4574] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_enumerator_list_no_comma, 4, 0, 73),
  [4576] = {.entry = {.count = 1, .reusable = true}}, SHIFT(360),
  [4578] = {.entry = {.count = 1, .reusable = true}}, SHIFT(678),
  [4580] = {.entry = {.count = 1, .reusable = true}}, SHIFT(321),
  [4582] = {.entry = {.count = 1, .reusable = true}}, SHIFT(153),
  [4584] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1851),
  [4586] = {.entry = {.count = 1, .reusable = true}}, SHIFT(63),
  [4588] = {.entry = {.count = 1, .reusable = true}}, SHIFT(331),
  [4590] = {.entry = {.count = 1, .reusable = true}}, SHIFT(937),
  [4592] = {.entry = {.count = 1, .reusable = true}}, SHIFT(369),
  [4594] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_field_declaration_list, 4, 0, 72),
  [4596] = {.entry = {.count = 1, .reusable = true}}, SHIFT(201),
  [4598] = {.entry = {.count = 1, .reusable = true}}, SHIFT(370),
  [4600] = {.entry = {.count = 1, .reusable = true}}, SHIFT(157),
  [4602] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1011),
  [4604] = {.entry = {.count = 1, .reusable = true}}, SHIFT(216),
  [4606] = {.entry = {.count = 1, .reusable = true}}, SHIFT(340),
  [4608] = {.entry = {.count = 1, .reusable = true}}, SHIFT(509),
  [4610] = {.entry = {.count = 1, .reusable = true}}, SHIFT(938),
  [4612] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1062),
  [4614] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1037),
  [4616] = {.entry = {.count = 1, .reusable = true}}, SHIFT(721),
  [4618] = {.entry = {.count = 1, .reusable = true}}, SHIFT(570),
  [4620] = {.entry = {.count = 1, .reusable = true}}, SHIFT(857),
  [4622] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1009),
  [4624] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2006),
  [4626] = {.entry = {.count = 1, .reusable = true}}, SHIFT(851),
  [4628] = {.entry = {.count = 1, .reusable = true}}, SHIFT(224),
  [4630] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1452),
  [4632] = {.entry = {.count = 1, .reusable = true}}, SHIFT(225),
  [4634] = {.entry = {.count = 1, .reusable = true}}, SHIFT(568),
  [4636] = {.entry = {.count = 1, .reusable = true}}, SHIFT(49),
  [4638] = {.entry = {.count = 1, .reusable = true}}, SHIFT(849),
  [4640] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1994),
  [4642] = {.entry = {.count = 1, .reusable = true}}, SHIFT(924),
  [4644] = {.entry = {.count = 1, .reusable = true}}, SHIFT(917),
  [4646] = {.entry = {.count = 1, .reusable = true}}, SHIFT(898),
  [4648] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_field_declaration_list, 3, 0, 41),
  [4650] = {.entry = {.count = 1, .reusable = true}}, SHIFT(934),
  [4652] = {.entry = {.count = 1, .reusable = true}}, SHIFT(915),
  [4654] = {.entry = {.count = 1, .reusable = true}}, SHIFT(914),
  [4656] = {.entry = {.count = 1, .reusable = true}}, SHIFT(905),
  [4658] = {.entry = {.count = 1, .reusable = true}},  ACCEPT_INPUT(),
  [4660] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1538),
  [4662] = {.entry = {.count = 1, .reusable = true}}, SHIFT(6),
  [4664] = {.entry = {.count = 1, .reusable = true}}, SHIFT(64),
  [4666] = {.entry = {.count = 1, .reusable = true}}, SHIFT(912),
  [4668] = {.entry = {.count = 1, .reusable = true}}, SHIFT(148),
  [4670] = {.entry = {.count = 1, .reusable = true}}, SHIFT(59),
  [4672] = {.entry = {.count = 1, .reusable = true}}, SHIFT(931),
  [4674] = {.entry = {.count = 1, .reusable = true}}, SHIFT(239),
  [4676] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1921),
  [4678] = {.entry = {.count = 1, .reusable = true}}, SHIFT(122),
  [4680] = {.entry = {.count = 1, .reusable = true}}, SHIFT(240),
  [4682] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1870),
  [4684] = {.entry = {.count = 1, .reusable = true}}, SHIFT(907),
  [4686] = {.entry = {.count = 1, .reusable = true}}, SHIFT(293),
  [4688] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1814),
  [4690] = {.entry = {.count = 1, .reusable = true}}, SHIFT(519),
  [4692] = {.entry = {.count = 1, .reusable = true}}, SHIFT(134),
  [4694] = {.entry = {.count = 1, .reusable = true}}, SHIFT(926),
  [4696] = {.entry = {.count = 1, .reusable = true}}, SHIFT(136),
  [4698] = {.entry = {.count = 1, .reusable = true}}, SHIFT(137),
  [4700] = {.entry = {.count = 1, .reusable = true}}, SHIFT(672),
  [4702] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_enumerator_list_no_comma, 5, 0, 73),
  [4704] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_enumerator_list, 5, 0, 97),
  [4706] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_enumerator_list_no_comma, 5, 0, 97),
  [4708] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif_in_field_declaration_list, 5, 0, 97),
  [4710] = {.entry = {.count = 1, .reusable = true}}, SHIFT(705),
  [4712] = {.entry = {.count = 1, .reusable = true}}, SHIFT(624),
  [4714] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif, 4, 0, 72),
  [4716] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1881),
  [4718] = {.entry = {.count = 1, .reusable = true}}, SHIFT(625),
  [4720] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef, 4, 0, 73),
  [4722] = {.entry = {.count = 1, .reusable = true}}, SHIFT(921),
  [4724] = {.entry = {.count = 1, .reusable = true}}, SHIFT(483),
  [4726] = {.entry = {.count = 1, .reusable = true}}, SHIFT(359),
  [4728] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1714),
  [4730] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2012),
  [4732] = {.entry = {.count = 1, .reusable = true}}, SHIFT(406),
  [4734] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1634),
  [4736] = {.entry = {.count = 1, .reusable = true}}, SHIFT(357),
  [4738] = {.entry = {.count = 1, .reusable = true}}, SHIFT(487),
  [4740] = {.entry = {.count = 1, .reusable = true}}, SHIFT(508),
  [4742] = {.entry = {.count = 1, .reusable = true}}, SHIFT(387),
  [4744] = {.entry = {.count = 1, .reusable = true}}, SHIFT(450),
  [4746] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1676),
  [4748] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1352),
  [4750] = {.entry = {.count = 1, .reusable = true}}, SHIFT(422),
  [4752] = {.entry = {.count = 1, .reusable = true}}, SHIFT(754),
  [4754] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1759),
  [4756] = {.entry = {.count = 1, .reusable = true}}, SHIFT(2017),
  [4758] = {.entry = {.count = 1, .reusable = true}}, SHIFT(262),
  [4760] = {.entry = {.count = 1, .reusable = true}}, SHIFT(143),
  [4762] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1958),
  [4764] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1464),
  [4766] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1300),
  [4768] = {.entry = {.count = 1, .reusable = true}}, SHIFT(21),
  [4770] = {.entry = {.count = 1, .reusable = true}}, SHIFT(432),
  [4772] = {.entry = {.count = 1, .reusable = true}}, SHIFT(920),
  [4774] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_enumerator_list_no_comma, 3, 0, 41),
  [4776] = {.entry = {.count = 1, .reusable = true}}, SHIFT(97),
  [4778] = {.entry = {.count = 1, .reusable = true}}, SHIFT(409),
  [4780] = {.entry = {.count = 1, .reusable = true}}, SHIFT(953),
  [4782] = {.entry = {.count = 1, .reusable = true}}, SHIFT(731),
  [4784] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1563),
  [4786] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1549),
  [4788] = {.entry = {.count = 1, .reusable = true}}, SHIFT(8),
  [4790] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_enumerator_list, 3, 0, 41),
  [4792] = {.entry = {.count = 1, .reusable = true}}, SHIFT(260),
  [4794] = {.entry = {.count = 1, .reusable = true}}, SHIFT(182),
  [4796] = {.entry = {.count = 1, .reusable = true}}, SHIFT(115),
  [4798] = {.entry = {.count = 1, .reusable = true}}, SHIFT(501),
  [4800] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1909),
  [4802] = {.entry = {.count = 1, .reusable = true}}, SHIFT(131),
  [4804] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef_in_field_declaration_list, 4, 0, 73),
  [4806] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1298),
  [4808] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1777),
  [4810] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_enumerator_list_no_comma, 5, 0, 40),
  [4812] = {.entry = {.count = 1, .reusable = true}}, SHIFT(83),
  [4814] = {.entry = {.count = 1, .reusable = true}}, SHIFT(82),
  [4816] = {.entry = {.count = 1, .reusable = true}}, SHIFT(883),
  [4818] = {.entry = {.count = 1, .reusable = true}}, SHIFT(69),
  [4820] = {.entry = {.count = 1, .reusable = true}}, SHIFT(480),
  [4822] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1476),
  [4824] = {.entry = {.count = 1, .reusable = true}}, SHIFT(320),
  [4826] = {.entry = {.count = 1, .reusable = true}}, SHIFT(484),
  [4828] = {.entry = {.count = 1, .reusable = true}}, SHIFT(404),
  [4830] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_if_in_enumerator_list_no_comma, 5, 0, 72),
  [4832] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1454),
  [4834] = {.entry = {.count = 1, .reusable = true}}, SHIFT(511),
  [4836] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1943),
  [4838] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1517),
  [4840] = {.entry = {.count = 1, .reusable = true}}, SHIFT(9),
  [4842] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1995),
  [4844] = {.entry = {.count = 1, .reusable = true}}, SHIFT(354),
  [4846] = {.entry = {.count = 1, .reusable = true}}, SHIFT(395),
  [4848] = {.entry = {.count = 1, .reusable = true}}, SHIFT(343),
  [4850] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1799),
  [4852] = {.entry = {.count = 1, .reusable = true}}, SHIFT(448),
  [4854] = {.entry = {.count = 1, .reusable = true}}, SHIFT(715),
  [4856] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_enumerator_list_no_comma, 4, 0, 41),
  [4858] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elif, 5, 0, 97),
  [4860] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1852),
  [4862] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1329),
  [4864] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1462),
  [4866] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_ifdef_in_enumerator_list_no_comma, 4, 0, 17),
  [4868] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1330),
  [4870] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1859),
  [4872] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1118),
  [4874] = {.entry = {.count = 1, .reusable = true}}, SHIFT(809),
  [4876] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1514),
  [4878] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1623),
  [4880] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1700),
  [4882] = {.entry = {.count = 1, .reusable = true}}, SHIFT(394),
  [4884] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1518),
  [4886] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1526),
  [4888] = {.entry = {.count = 1, .reusable = true}}, SHIFT(159),
  [4890] = {.entry = {.count = 1, .reusable = true}}, SHIFT(918),
  [4892] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_ms_based_modifier, 2, 0, 0),
  [4894] = {.entry = {.count = 1, .reusable = true}}, SHIFT(73),
  [4896] = {.entry = {.count = 1, .reusable = true}}, SHIFT(904),
  [4898] = {.entry = {.count = 1, .reusable = true}}, SHIFT(856),
  [4900] = {.entry = {.count = 1, .reusable = true}}, SHIFT(845),
  [4902] = {.entry = {.count = 1, .reusable = true}}, SHIFT(861),
  [4904] = {.entry = {.count = 1, .reusable = true}}, SHIFT(514),
  [4906] = {.entry = {.count = 1, .reusable = true}}, SHIFT(854),
  [4908] = {.entry = {.count = 1, .reusable = true}}, SHIFT(859),
  [4910] = {.entry = {.count = 1, .reusable = true}}, SHIFT(345),
  [4912] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1716),
  [4914] = {.entry = {.count = 1, .reusable = true}}, SHIFT(571),
  [4916] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1821),
  [4918] = {.entry = {.count = 1, .reusable = true}}, REDUCE(sym_preproc_elifdef, 3, 0, 41),
  [4920] = {.entry = {.count = 1, .reusable = true}}, SHIFT(407),
  [4922] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1888),
  [4924] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1893),
  [4926] = {.entry = {.count = 1, .reusable = true}}, SHIFT(278),
  [4928] = {.entry = {.count = 1, .reusable = true}}, SHIFT(74),
  [4930] = {.entry = {.count = 1, .reusable = true}}, SHIFT(248),
  [4932] = {.entry = {.count = 1, .reusable = true}}, SHIFT(852),
  [4934] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1745),
  [4936] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1458),
  [4938] = {.entry = {.count = 1, .reusable = true}}, SHIFT(243),
  [4940] = {.entry = {.count = 1, .reusable = true}}, SHIFT(377),
  [4942] = {.entry = {.count = 1, .reusable = true}}, SHIFT(418),
  [4944] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1768),
  [4946] = {.entry = {.count = 1, .reusable = true}}, SHIFT(379),
  [4948] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1192),
  [4950] = {.entry = {.count = 1, .reusable = true}}, SHIFT(380),
  [4952] = {.entry = {.count = 1, .reusable = true}}, SHIFT(5),
  [4954] = {.entry = {.count = 1, .reusable = true}}, SHIFT(70),
  [4956] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1259),
  [4958] = {.entry = {.count = 1, .reusable = true}}, SHIFT(71),
  [4960] = {.entry = {.count = 1, .reusable = true}}, SHIFT(414),
  [4962] = {.entry = {.count = 1, .reusable = true}}, SHIFT(1548),
};

#ifdef __cplusplus
extern "C" {
#endif
#ifdef TREE_SITTER_HIDE_SYMBOLS
#define TS_PUBLIC
#elif defined(_WIN32)
#define TS_PUBLIC __declspec(dllexport)
#else
#define TS_PUBLIC __attribute__((visibility("default")))
#endif

TS_PUBLIC const TSLanguage *tree_sitter_c(void) {
  static const TSLanguage language = {
    .version = LANGUAGE_VERSION,
    .symbol_count = SYMBOL_COUNT,
    .alias_count = ALIAS_COUNT,
    .token_count = TOKEN_COUNT,
    .external_token_count = EXTERNAL_TOKEN_COUNT,
    .state_count = STATE_COUNT,
    .large_state_count = LARGE_STATE_COUNT,
    .production_id_count = PRODUCTION_ID_COUNT,
    .field_count = FIELD_COUNT,
    .max_alias_sequence_length = MAX_ALIAS_SEQUENCE_LENGTH,
    .parse_table = &ts_parse_table[0][0],
    .small_parse_table = ts_small_parse_table,
    .small_parse_table_map = ts_small_parse_table_map,
    .parse_actions = ts_parse_actions,
    .symbol_names = ts_symbol_names,
    .field_names = ts_field_names,
    .field_map_slices = ts_field_map_slices,
    .field_map_entries = ts_field_map_entries,
    .symbol_metadata = ts_symbol_metadata,
    .public_symbol_map = ts_symbol_map,
    .alias_map = ts_non_terminal_alias_map,
    .alias_sequences = &ts_alias_sequences[0][0],
    .lex_modes = ts_lex_modes,
    .lex_fn = ts_lex,
    .keyword_lex_fn = ts_lex_keywords,
    .keyword_capture_token = sym_identifier,
    .primary_state_ids = ts_primary_state_ids,
  };
  return &language;
}
#ifdef __cplusplus
}
#endif
